// +build !minimal

package gui

//#include <stdint.h>
//#include <stdlib.h>
//#include <string.h>
//#include "gui.h"
import "C"
import (
	"github.com/therecipe/qt"
	"github.com/therecipe/qt/core"
	_ "github.com/therecipe/qt/internal/binding/runtime"
	"strings"
	"unsafe"
)

func cGoFreePacked(ptr unsafe.Pointer) { core.NewQByteArrayFromPointer(ptr).DestroyQByteArray() }
func cGoUnpackString(s C.struct_QtGui_PackedString) string {
	defer cGoFreePacked(s.ptr)
	if int(s.len) == -1 {
		return C.GoString(s.data)
	}
	return C.GoStringN(s.data, C.int(s.len))
}
func cGoUnpackBytes(s C.struct_QtGui_PackedString) []byte {
	defer cGoFreePacked(s.ptr)
	if int(s.len) == -1 {
		gs := C.GoString(s.data)
		return []byte(gs)
	}
	return C.GoBytes(unsafe.Pointer(s.data), C.int(s.len))
}
func unpackStringList(s string) []string {
	if len(s) == 0 {
		return make([]string, 0)
	}
	return strings.Split(s, "¡¦!")
}

type QAbstractOpenGLFunctions struct {
	ptr unsafe.Pointer
}

type QAbstractOpenGLFunctions_ITF interface {
	QAbstractOpenGLFunctions_PTR() *QAbstractOpenGLFunctions
}

func (ptr *QAbstractOpenGLFunctions) QAbstractOpenGLFunctions_PTR() *QAbstractOpenGLFunctions {
	return ptr
}

func (ptr *QAbstractOpenGLFunctions) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAbstractOpenGLFunctions) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAbstractOpenGLFunctions(ptr QAbstractOpenGLFunctions_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func NewQAbstractOpenGLFunctionsFromPointer(ptr unsafe.Pointer) (n *QAbstractOpenGLFunctions) {
	n = new(QAbstractOpenGLFunctions)
	n.SetPointer(ptr)
	return
}

type QAbstractTextDocumentLayout struct {
	core.QObject
}

type QAbstractTextDocumentLayout_ITF interface {
	core.QObject_ITF
	QAbstractTextDocumentLayout_PTR() *QAbstractTextDocumentLayout
}

func (ptr *QAbstractTextDocumentLayout) QAbstractTextDocumentLayout_PTR() *QAbstractTextDocumentLayout {
	return ptr
}

func (ptr *QAbstractTextDocumentLayout) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QAbstractTextDocumentLayout) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQAbstractTextDocumentLayout(ptr QAbstractTextDocumentLayout_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractTextDocumentLayout_PTR().Pointer()
	}
	return nil
}

func NewQAbstractTextDocumentLayoutFromPointer(ptr unsafe.Pointer) (n *QAbstractTextDocumentLayout) {
	n = new(QAbstractTextDocumentLayout)
	n.SetPointer(ptr)
	return
}
func (ptr *QAbstractTextDocumentLayout) AnchorAt(position core.QPointF_ITF) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAbstractTextDocumentLayout_AnchorAt(ptr.Pointer(), core.PointerFromQPointF(position)))
	}
	return ""
}

//export callbackQAbstractTextDocumentLayout_BlockBoundingRect
func callbackQAbstractTextDocumentLayout_BlockBoundingRect(ptr unsafe.Pointer, block unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "blockBoundingRect"); signal != nil {
		return core.PointerFromQRectF((*(*func(*QTextBlock) *core.QRectF)(signal))(NewQTextBlockFromPointer(block)))
	}

	return core.PointerFromQRectF(core.NewQRectF())
}

func (ptr *QAbstractTextDocumentLayout) ConnectBlockBoundingRect(f func(block *QTextBlock) *core.QRectF) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "blockBoundingRect"); signal != nil {
			f := func(block *QTextBlock) *core.QRectF {
				(*(*func(*QTextBlock) *core.QRectF)(signal))(block)
				return f(block)
			}
			qt.ConnectSignal(ptr.Pointer(), "blockBoundingRect", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "blockBoundingRect", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectBlockBoundingRect() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "blockBoundingRect")
	}
}

func (ptr *QAbstractTextDocumentLayout) BlockBoundingRect(block QTextBlock_ITF) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QAbstractTextDocumentLayout_BlockBoundingRect(ptr.Pointer(), PointerFromQTextBlock(block)))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QAbstractTextDocumentLayout) Document() *QTextDocument {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextDocumentFromPointer(C.QAbstractTextDocumentLayout_Document(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQAbstractTextDocumentLayout_DocumentChanged
func callbackQAbstractTextDocumentLayout_DocumentChanged(ptr unsafe.Pointer, position C.int, charsRemoved C.int, charsAdded C.int) {
	if signal := qt.GetSignal(ptr, "documentChanged"); signal != nil {
		(*(*func(int, int, int))(signal))(int(int32(position)), int(int32(charsRemoved)), int(int32(charsAdded)))
	}

}

func (ptr *QAbstractTextDocumentLayout) ConnectDocumentChanged(f func(position int, charsRemoved int, charsAdded int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "documentChanged"); signal != nil {
			f := func(position int, charsRemoved int, charsAdded int) {
				(*(*func(int, int, int))(signal))(position, charsRemoved, charsAdded)
				f(position, charsRemoved, charsAdded)
			}
			qt.ConnectSignal(ptr.Pointer(), "documentChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "documentChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectDocumentChanged() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "documentChanged")
	}
}

func (ptr *QAbstractTextDocumentLayout) DocumentChanged(position int, charsRemoved int, charsAdded int) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_DocumentChanged(ptr.Pointer(), C.int(int32(position)), C.int(int32(charsRemoved)), C.int(int32(charsAdded)))
	}
}

//export callbackQAbstractTextDocumentLayout_DocumentSize
func callbackQAbstractTextDocumentLayout_DocumentSize(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "documentSize"); signal != nil {
		return core.PointerFromQSizeF((*(*func() *core.QSizeF)(signal))())
	}

	return core.PointerFromQSizeF(core.NewQSizeF())
}

func (ptr *QAbstractTextDocumentLayout) ConnectDocumentSize(f func() *core.QSizeF) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "documentSize"); signal != nil {
			f := func() *core.QSizeF {
				(*(*func() *core.QSizeF)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "documentSize", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "documentSize", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectDocumentSize() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "documentSize")
	}
}

func (ptr *QAbstractTextDocumentLayout) DocumentSize() *core.QSizeF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFFromPointer(C.QAbstractTextDocumentLayout_DocumentSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

//export callbackQAbstractTextDocumentLayout_DocumentSizeChanged
func callbackQAbstractTextDocumentLayout_DocumentSizeChanged(ptr unsafe.Pointer, newSize unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "documentSizeChanged"); signal != nil {
		(*(*func(*core.QSizeF))(signal))(core.NewQSizeFFromPointer(newSize))
	}

}

func (ptr *QAbstractTextDocumentLayout) ConnectDocumentSizeChanged(f func(newSize *core.QSizeF)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "documentSizeChanged") {
			C.QAbstractTextDocumentLayout_ConnectDocumentSizeChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "documentSizeChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "documentSizeChanged"); signal != nil {
			f := func(newSize *core.QSizeF) {
				(*(*func(*core.QSizeF))(signal))(newSize)
				f(newSize)
			}
			qt.ConnectSignal(ptr.Pointer(), "documentSizeChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "documentSizeChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectDocumentSizeChanged() {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_DisconnectDocumentSizeChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "documentSizeChanged")
	}
}

func (ptr *QAbstractTextDocumentLayout) DocumentSizeChanged(newSize core.QSizeF_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_DocumentSizeChanged(ptr.Pointer(), core.PointerFromQSizeF(newSize))
	}
}

//export callbackQAbstractTextDocumentLayout_DrawInlineObject
func callbackQAbstractTextDocumentLayout_DrawInlineObject(ptr unsafe.Pointer, painter unsafe.Pointer, rect unsafe.Pointer, object unsafe.Pointer, posInDocument C.int, format unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "drawInlineObject"); signal != nil {
		(*(*func(*QPainter, *core.QRectF, *QTextInlineObject, int, *QTextFormat))(signal))(NewQPainterFromPointer(painter), core.NewQRectFFromPointer(rect), func() *QTextInlineObject {
			tmpValue := NewQTextInlineObjectFromPointer(object)
			qt.SetFinalizer(tmpValue, (*QTextInlineObject).DestroyQTextInlineObject)
			return tmpValue
		}(), int(int32(posInDocument)), NewQTextFormatFromPointer(format))
	} else {
		NewQAbstractTextDocumentLayoutFromPointer(ptr).DrawInlineObjectDefault(NewQPainterFromPointer(painter), core.NewQRectFFromPointer(rect), func() *QTextInlineObject {
			tmpValue := NewQTextInlineObjectFromPointer(object)
			qt.SetFinalizer(tmpValue, (*QTextInlineObject).DestroyQTextInlineObject)
			return tmpValue
		}(), int(int32(posInDocument)), NewQTextFormatFromPointer(format))
	}
}

func (ptr *QAbstractTextDocumentLayout) ConnectDrawInlineObject(f func(painter *QPainter, rect *core.QRectF, object *QTextInlineObject, posInDocument int, format *QTextFormat)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawInlineObject"); signal != nil {
			f := func(painter *QPainter, rect *core.QRectF, object *QTextInlineObject, posInDocument int, format *QTextFormat) {
				(*(*func(*QPainter, *core.QRectF, *QTextInlineObject, int, *QTextFormat))(signal))(painter, rect, object, posInDocument, format)
				f(painter, rect, object, posInDocument, format)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawInlineObject", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawInlineObject", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectDrawInlineObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawInlineObject")
	}
}

func (ptr *QAbstractTextDocumentLayout) DrawInlineObject(painter QPainter_ITF, rect core.QRectF_ITF, object QTextInlineObject_ITF, posInDocument int, format QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_DrawInlineObject(ptr.Pointer(), PointerFromQPainter(painter), core.PointerFromQRectF(rect), PointerFromQTextInlineObject(object), C.int(int32(posInDocument)), PointerFromQTextFormat(format))
	}
}

func (ptr *QAbstractTextDocumentLayout) DrawInlineObjectDefault(painter QPainter_ITF, rect core.QRectF_ITF, object QTextInlineObject_ITF, posInDocument int, format QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_DrawInlineObjectDefault(ptr.Pointer(), PointerFromQPainter(painter), core.PointerFromQRectF(rect), PointerFromQTextInlineObject(object), C.int(int32(posInDocument)), PointerFromQTextFormat(format))
	}
}

func (ptr *QAbstractTextDocumentLayout) Format(position int) *QTextCharFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCharFormatFromPointer(C.QAbstractTextDocumentLayout_Format(ptr.Pointer(), C.int(int32(position))))
		qt.SetFinalizer(tmpValue, (*QTextCharFormat).DestroyQTextCharFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QAbstractTextDocumentLayout) FormatAt(pos core.QPointF_ITF) *QTextFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFormatFromPointer(C.QAbstractTextDocumentLayout_FormatAt(ptr.Pointer(), core.PointerFromQPointF(pos)))
		qt.SetFinalizer(tmpValue, (*QTextFormat).DestroyQTextFormat)
		return tmpValue
	}
	return nil
}

//export callbackQAbstractTextDocumentLayout_FrameBoundingRect
func callbackQAbstractTextDocumentLayout_FrameBoundingRect(ptr unsafe.Pointer, frame unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "frameBoundingRect"); signal != nil {
		return core.PointerFromQRectF((*(*func(*QTextFrame) *core.QRectF)(signal))(NewQTextFrameFromPointer(frame)))
	}

	return core.PointerFromQRectF(core.NewQRectF())
}

func (ptr *QAbstractTextDocumentLayout) ConnectFrameBoundingRect(f func(frame *QTextFrame) *core.QRectF) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "frameBoundingRect"); signal != nil {
			f := func(frame *QTextFrame) *core.QRectF {
				(*(*func(*QTextFrame) *core.QRectF)(signal))(frame)
				return f(frame)
			}
			qt.ConnectSignal(ptr.Pointer(), "frameBoundingRect", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "frameBoundingRect", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectFrameBoundingRect() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "frameBoundingRect")
	}
}

func (ptr *QAbstractTextDocumentLayout) FrameBoundingRect(frame QTextFrame_ITF) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QAbstractTextDocumentLayout_FrameBoundingRect(ptr.Pointer(), PointerFromQTextFrame(frame)))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QAbstractTextDocumentLayout) HandlerForObject(objectType int) *QTextObjectInterface {
	if ptr.Pointer() != nil {
		return NewQTextObjectInterfaceFromPointer(C.QAbstractTextDocumentLayout_HandlerForObject(ptr.Pointer(), C.int(int32(objectType))))
	}
	return nil
}

//export callbackQAbstractTextDocumentLayout_HitTest
func callbackQAbstractTextDocumentLayout_HitTest(ptr unsafe.Pointer, point unsafe.Pointer, accuracy C.longlong) C.int {
	if signal := qt.GetSignal(ptr, "hitTest"); signal != nil {
		return C.int(int32((*(*func(*core.QPointF, core.Qt__HitTestAccuracy) int)(signal))(core.NewQPointFFromPointer(point), core.Qt__HitTestAccuracy(accuracy))))
	}

	return C.int(int32(0))
}

func (ptr *QAbstractTextDocumentLayout) ConnectHitTest(f func(point *core.QPointF, accuracy core.Qt__HitTestAccuracy) int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "hitTest"); signal != nil {
			f := func(point *core.QPointF, accuracy core.Qt__HitTestAccuracy) int {
				(*(*func(*core.QPointF, core.Qt__HitTestAccuracy) int)(signal))(point, accuracy)
				return f(point, accuracy)
			}
			qt.ConnectSignal(ptr.Pointer(), "hitTest", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "hitTest", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectHitTest() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "hitTest")
	}
}

func (ptr *QAbstractTextDocumentLayout) HitTest(point core.QPointF_ITF, accuracy core.Qt__HitTestAccuracy) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAbstractTextDocumentLayout_HitTest(ptr.Pointer(), core.PointerFromQPointF(point), C.longlong(accuracy))))
	}
	return 0
}

func (ptr *QAbstractTextDocumentLayout) ImageAt(pos core.QPointF_ITF) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAbstractTextDocumentLayout_ImageAt(ptr.Pointer(), core.PointerFromQPointF(pos)))
	}
	return ""
}

//export callbackQAbstractTextDocumentLayout_PageCount
func callbackQAbstractTextDocumentLayout_PageCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "pageCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAbstractTextDocumentLayout) ConnectPageCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "pageCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "pageCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "pageCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectPageCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "pageCount")
	}
}

func (ptr *QAbstractTextDocumentLayout) PageCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAbstractTextDocumentLayout_PageCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQAbstractTextDocumentLayout_PageCountChanged
func callbackQAbstractTextDocumentLayout_PageCountChanged(ptr unsafe.Pointer, newPages C.int) {
	if signal := qt.GetSignal(ptr, "pageCountChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(newPages)))
	}

}

func (ptr *QAbstractTextDocumentLayout) ConnectPageCountChanged(f func(newPages int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "pageCountChanged") {
			C.QAbstractTextDocumentLayout_ConnectPageCountChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "pageCountChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "pageCountChanged"); signal != nil {
			f := func(newPages int) {
				(*(*func(int))(signal))(newPages)
				f(newPages)
			}
			qt.ConnectSignal(ptr.Pointer(), "pageCountChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "pageCountChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectPageCountChanged() {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_DisconnectPageCountChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "pageCountChanged")
	}
}

func (ptr *QAbstractTextDocumentLayout) PageCountChanged(newPages int) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_PageCountChanged(ptr.Pointer(), C.int(int32(newPages)))
	}
}

func (ptr *QAbstractTextDocumentLayout) PaintDevice() *QPaintDevice {
	if ptr.Pointer() != nil {
		return NewQPaintDeviceFromPointer(C.QAbstractTextDocumentLayout_PaintDevice(ptr.Pointer()))
	}
	return nil
}

//export callbackQAbstractTextDocumentLayout_PositionInlineObject
func callbackQAbstractTextDocumentLayout_PositionInlineObject(ptr unsafe.Pointer, item unsafe.Pointer, posInDocument C.int, format unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "positionInlineObject"); signal != nil {
		(*(*func(*QTextInlineObject, int, *QTextFormat))(signal))(func() *QTextInlineObject {
			tmpValue := NewQTextInlineObjectFromPointer(item)
			qt.SetFinalizer(tmpValue, (*QTextInlineObject).DestroyQTextInlineObject)
			return tmpValue
		}(), int(int32(posInDocument)), NewQTextFormatFromPointer(format))
	} else {
		NewQAbstractTextDocumentLayoutFromPointer(ptr).PositionInlineObjectDefault(func() *QTextInlineObject {
			tmpValue := NewQTextInlineObjectFromPointer(item)
			qt.SetFinalizer(tmpValue, (*QTextInlineObject).DestroyQTextInlineObject)
			return tmpValue
		}(), int(int32(posInDocument)), NewQTextFormatFromPointer(format))
	}
}

func (ptr *QAbstractTextDocumentLayout) ConnectPositionInlineObject(f func(item *QTextInlineObject, posInDocument int, format *QTextFormat)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "positionInlineObject"); signal != nil {
			f := func(item *QTextInlineObject, posInDocument int, format *QTextFormat) {
				(*(*func(*QTextInlineObject, int, *QTextFormat))(signal))(item, posInDocument, format)
				f(item, posInDocument, format)
			}
			qt.ConnectSignal(ptr.Pointer(), "positionInlineObject", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "positionInlineObject", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectPositionInlineObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "positionInlineObject")
	}
}

func (ptr *QAbstractTextDocumentLayout) PositionInlineObject(item QTextInlineObject_ITF, posInDocument int, format QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_PositionInlineObject(ptr.Pointer(), PointerFromQTextInlineObject(item), C.int(int32(posInDocument)), PointerFromQTextFormat(format))
	}
}

func (ptr *QAbstractTextDocumentLayout) PositionInlineObjectDefault(item QTextInlineObject_ITF, posInDocument int, format QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_PositionInlineObjectDefault(ptr.Pointer(), PointerFromQTextInlineObject(item), C.int(int32(posInDocument)), PointerFromQTextFormat(format))
	}
}

func (ptr *QAbstractTextDocumentLayout) RegisterHandler(objectType int, component core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_RegisterHandler(ptr.Pointer(), C.int(int32(objectType)), core.PointerFromQObject(component))
	}
}

//export callbackQAbstractTextDocumentLayout_ResizeInlineObject
func callbackQAbstractTextDocumentLayout_ResizeInlineObject(ptr unsafe.Pointer, item unsafe.Pointer, posInDocument C.int, format unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "resizeInlineObject"); signal != nil {
		(*(*func(*QTextInlineObject, int, *QTextFormat))(signal))(func() *QTextInlineObject {
			tmpValue := NewQTextInlineObjectFromPointer(item)
			qt.SetFinalizer(tmpValue, (*QTextInlineObject).DestroyQTextInlineObject)
			return tmpValue
		}(), int(int32(posInDocument)), NewQTextFormatFromPointer(format))
	} else {
		NewQAbstractTextDocumentLayoutFromPointer(ptr).ResizeInlineObjectDefault(func() *QTextInlineObject {
			tmpValue := NewQTextInlineObjectFromPointer(item)
			qt.SetFinalizer(tmpValue, (*QTextInlineObject).DestroyQTextInlineObject)
			return tmpValue
		}(), int(int32(posInDocument)), NewQTextFormatFromPointer(format))
	}
}

func (ptr *QAbstractTextDocumentLayout) ConnectResizeInlineObject(f func(item *QTextInlineObject, posInDocument int, format *QTextFormat)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "resizeInlineObject"); signal != nil {
			f := func(item *QTextInlineObject, posInDocument int, format *QTextFormat) {
				(*(*func(*QTextInlineObject, int, *QTextFormat))(signal))(item, posInDocument, format)
				f(item, posInDocument, format)
			}
			qt.ConnectSignal(ptr.Pointer(), "resizeInlineObject", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "resizeInlineObject", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectResizeInlineObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "resizeInlineObject")
	}
}

func (ptr *QAbstractTextDocumentLayout) ResizeInlineObject(item QTextInlineObject_ITF, posInDocument int, format QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_ResizeInlineObject(ptr.Pointer(), PointerFromQTextInlineObject(item), C.int(int32(posInDocument)), PointerFromQTextFormat(format))
	}
}

func (ptr *QAbstractTextDocumentLayout) ResizeInlineObjectDefault(item QTextInlineObject_ITF, posInDocument int, format QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_ResizeInlineObjectDefault(ptr.Pointer(), PointerFromQTextInlineObject(item), C.int(int32(posInDocument)), PointerFromQTextFormat(format))
	}
}

func (ptr *QAbstractTextDocumentLayout) SetPaintDevice(device QPaintDevice_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_SetPaintDevice(ptr.Pointer(), PointerFromQPaintDevice(device))
	}
}

func (ptr *QAbstractTextDocumentLayout) UnregisterHandler(objectType int, component core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_UnregisterHandler(ptr.Pointer(), C.int(int32(objectType)), core.PointerFromQObject(component))
	}
}

//export callbackQAbstractTextDocumentLayout_Update
func callbackQAbstractTextDocumentLayout_Update(ptr unsafe.Pointer, rect unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "update"); signal != nil {
		(*(*func(*core.QRectF))(signal))(core.NewQRectFFromPointer(rect))
	}

}

func (ptr *QAbstractTextDocumentLayout) ConnectUpdate(f func(rect *core.QRectF)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "update") {
			C.QAbstractTextDocumentLayout_ConnectUpdate(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "update")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "update"); signal != nil {
			f := func(rect *core.QRectF) {
				(*(*func(*core.QRectF))(signal))(rect)
				f(rect)
			}
			qt.ConnectSignal(ptr.Pointer(), "update", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "update", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectUpdate() {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_DisconnectUpdate(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "update")
	}
}

func (ptr *QAbstractTextDocumentLayout) Update(rect core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_Update(ptr.Pointer(), core.PointerFromQRectF(rect))
	}
}

//export callbackQAbstractTextDocumentLayout_UpdateBlock
func callbackQAbstractTextDocumentLayout_UpdateBlock(ptr unsafe.Pointer, block unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "updateBlock"); signal != nil {
		(*(*func(*QTextBlock))(signal))(NewQTextBlockFromPointer(block))
	}

}

func (ptr *QAbstractTextDocumentLayout) ConnectUpdateBlock(f func(block *QTextBlock)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "updateBlock") {
			C.QAbstractTextDocumentLayout_ConnectUpdateBlock(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "updateBlock")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "updateBlock"); signal != nil {
			f := func(block *QTextBlock) {
				(*(*func(*QTextBlock))(signal))(block)
				f(block)
			}
			qt.ConnectSignal(ptr.Pointer(), "updateBlock", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "updateBlock", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectUpdateBlock() {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_DisconnectUpdateBlock(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "updateBlock")
	}
}

func (ptr *QAbstractTextDocumentLayout) UpdateBlock(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_UpdateBlock(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

func (ptr *QAbstractTextDocumentLayout) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QAbstractTextDocumentLayout___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QAbstractTextDocumentLayout) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QAbstractTextDocumentLayout) __children_newList() unsafe.Pointer {
	return C.QAbstractTextDocumentLayout___children_newList(ptr.Pointer())
}

func (ptr *QAbstractTextDocumentLayout) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QAbstractTextDocumentLayout___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QAbstractTextDocumentLayout) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QAbstractTextDocumentLayout) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QAbstractTextDocumentLayout___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QAbstractTextDocumentLayout) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QAbstractTextDocumentLayout___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QAbstractTextDocumentLayout) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QAbstractTextDocumentLayout) __findChildren_newList() unsafe.Pointer {
	return C.QAbstractTextDocumentLayout___findChildren_newList(ptr.Pointer())
}

func (ptr *QAbstractTextDocumentLayout) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QAbstractTextDocumentLayout___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QAbstractTextDocumentLayout) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QAbstractTextDocumentLayout) __findChildren_newList3() unsafe.Pointer {
	return C.QAbstractTextDocumentLayout___findChildren_newList3(ptr.Pointer())
}

//export callbackQAbstractTextDocumentLayout_ChildEvent
func callbackQAbstractTextDocumentLayout_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQAbstractTextDocumentLayoutFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QAbstractTextDocumentLayout) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQAbstractTextDocumentLayout_ConnectNotify
func callbackQAbstractTextDocumentLayout_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQAbstractTextDocumentLayoutFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QAbstractTextDocumentLayout) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQAbstractTextDocumentLayout_CustomEvent
func callbackQAbstractTextDocumentLayout_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQAbstractTextDocumentLayoutFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QAbstractTextDocumentLayout) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQAbstractTextDocumentLayout_DeleteLater
func callbackQAbstractTextDocumentLayout_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAbstractTextDocumentLayoutFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QAbstractTextDocumentLayout) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAbstractTextDocumentLayout_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQAbstractTextDocumentLayout_Destroyed
func callbackQAbstractTextDocumentLayout_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQAbstractTextDocumentLayout_DisconnectNotify
func callbackQAbstractTextDocumentLayout_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQAbstractTextDocumentLayoutFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QAbstractTextDocumentLayout) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQAbstractTextDocumentLayout_Event
func callbackQAbstractTextDocumentLayout_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQAbstractTextDocumentLayoutFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QAbstractTextDocumentLayout) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QAbstractTextDocumentLayout_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQAbstractTextDocumentLayout_EventFilter
func callbackQAbstractTextDocumentLayout_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQAbstractTextDocumentLayoutFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QAbstractTextDocumentLayout) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QAbstractTextDocumentLayout_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQAbstractTextDocumentLayout_MetaObject
func callbackQAbstractTextDocumentLayout_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQAbstractTextDocumentLayoutFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QAbstractTextDocumentLayout) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QAbstractTextDocumentLayout_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQAbstractTextDocumentLayout_ObjectNameChanged
func callbackQAbstractTextDocumentLayout_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQAbstractTextDocumentLayout_TimerEvent
func callbackQAbstractTextDocumentLayout_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQAbstractTextDocumentLayoutFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QAbstractTextDocumentLayout) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QAbstractTextDocumentLayout_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QAbstractUndoItem struct {
	ptr unsafe.Pointer
}

type QAbstractUndoItem_ITF interface {
	QAbstractUndoItem_PTR() *QAbstractUndoItem
}

func (ptr *QAbstractUndoItem) QAbstractUndoItem_PTR() *QAbstractUndoItem {
	return ptr
}

func (ptr *QAbstractUndoItem) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAbstractUndoItem) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAbstractUndoItem(ptr QAbstractUndoItem_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractUndoItem_PTR().Pointer()
	}
	return nil
}

func NewQAbstractUndoItemFromPointer(ptr unsafe.Pointer) (n *QAbstractUndoItem) {
	n = new(QAbstractUndoItem)
	n.SetPointer(ptr)
	return
}
func (ptr *QAbstractUndoItem) DestroyQAbstractUndoItem() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QAccessible struct {
	ptr unsafe.Pointer
}

type QAccessible_ITF interface {
	QAccessible_PTR() *QAccessible
}

func (ptr *QAccessible) QAccessible_PTR() *QAccessible {
	return ptr
}

func (ptr *QAccessible) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAccessible) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAccessible(ptr QAccessible_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessible_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleFromPointer(ptr unsafe.Pointer) (n *QAccessible) {
	n = new(QAccessible)
	n.SetPointer(ptr)
	return
}
func (ptr *QAccessible) DestroyQAccessible() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QAccessible__Event
//QAccessible::Event
type QAccessible__Event int64

const (
	QAccessible__SoundPlayed                     QAccessible__Event = QAccessible__Event(0x0001)
	QAccessible__Alert                           QAccessible__Event = QAccessible__Event(0x0002)
	QAccessible__ForegroundChanged               QAccessible__Event = QAccessible__Event(0x0003)
	QAccessible__MenuStart                       QAccessible__Event = QAccessible__Event(0x0004)
	QAccessible__MenuEnd                         QAccessible__Event = QAccessible__Event(0x0005)
	QAccessible__PopupMenuStart                  QAccessible__Event = QAccessible__Event(0x0006)
	QAccessible__PopupMenuEnd                    QAccessible__Event = QAccessible__Event(0x0007)
	QAccessible__ContextHelpStart                QAccessible__Event = QAccessible__Event(0x000C)
	QAccessible__ContextHelpEnd                  QAccessible__Event = QAccessible__Event(0x000D)
	QAccessible__DragDropStart                   QAccessible__Event = QAccessible__Event(0x000E)
	QAccessible__DragDropEnd                     QAccessible__Event = QAccessible__Event(0x000F)
	QAccessible__DialogStart                     QAccessible__Event = QAccessible__Event(0x0010)
	QAccessible__DialogEnd                       QAccessible__Event = QAccessible__Event(0x0011)
	QAccessible__ScrollingStart                  QAccessible__Event = QAccessible__Event(0x0012)
	QAccessible__ScrollingEnd                    QAccessible__Event = QAccessible__Event(0x0013)
	QAccessible__MenuCommand                     QAccessible__Event = QAccessible__Event(0x0018)
	QAccessible__ActionChanged                   QAccessible__Event = QAccessible__Event(0x0101)
	QAccessible__ActiveDescendantChanged         QAccessible__Event = QAccessible__Event(0x0102)
	QAccessible__AttributeChanged                QAccessible__Event = QAccessible__Event(0x0103)
	QAccessible__DocumentContentChanged          QAccessible__Event = QAccessible__Event(0x0104)
	QAccessible__DocumentLoadComplete            QAccessible__Event = QAccessible__Event(0x0105)
	QAccessible__DocumentLoadStopped             QAccessible__Event = QAccessible__Event(0x0106)
	QAccessible__DocumentReload                  QAccessible__Event = QAccessible__Event(0x0107)
	QAccessible__HyperlinkEndIndexChanged        QAccessible__Event = QAccessible__Event(0x0108)
	QAccessible__HyperlinkNumberOfAnchorsChanged QAccessible__Event = QAccessible__Event(0x0109)
	QAccessible__HyperlinkSelectedLinkChanged    QAccessible__Event = QAccessible__Event(0x010A)
	QAccessible__HypertextLinkActivated          QAccessible__Event = QAccessible__Event(0x010B)
	QAccessible__HypertextLinkSelected           QAccessible__Event = QAccessible__Event(0x010C)
	QAccessible__HyperlinkStartIndexChanged      QAccessible__Event = QAccessible__Event(0x010D)
	QAccessible__HypertextChanged                QAccessible__Event = QAccessible__Event(0x010E)
	QAccessible__HypertextNLinksChanged          QAccessible__Event = QAccessible__Event(0x010F)
	QAccessible__ObjectAttributeChanged          QAccessible__Event = QAccessible__Event(0x0110)
	QAccessible__PageChanged                     QAccessible__Event = QAccessible__Event(0x0111)
	QAccessible__SectionChanged                  QAccessible__Event = QAccessible__Event(0x0112)
	QAccessible__TableCaptionChanged             QAccessible__Event = QAccessible__Event(0x0113)
	QAccessible__TableColumnDescriptionChanged   QAccessible__Event = QAccessible__Event(0x0114)
	QAccessible__TableColumnHeaderChanged        QAccessible__Event = QAccessible__Event(0x0115)
	QAccessible__TableModelChanged               QAccessible__Event = QAccessible__Event(0x0116)
	QAccessible__TableRowDescriptionChanged      QAccessible__Event = QAccessible__Event(0x0117)
	QAccessible__TableRowHeaderChanged           QAccessible__Event = QAccessible__Event(0x0118)
	QAccessible__TableSummaryChanged             QAccessible__Event = QAccessible__Event(0x0119)
	QAccessible__TextAttributeChanged            QAccessible__Event = QAccessible__Event(0x011A)
	QAccessible__TextCaretMoved                  QAccessible__Event = QAccessible__Event(0x011B)
	QAccessible__TextColumnChanged               QAccessible__Event = QAccessible__Event(0x011D)
	QAccessible__TextInserted                    QAccessible__Event = QAccessible__Event(0x011E)
	QAccessible__TextRemoved                     QAccessible__Event = QAccessible__Event(0x011F)
	QAccessible__TextUpdated                     QAccessible__Event = QAccessible__Event(0x0120)
	QAccessible__TextSelectionChanged            QAccessible__Event = QAccessible__Event(0x0121)
	QAccessible__VisibleDataChanged              QAccessible__Event = QAccessible__Event(0x0122)
	QAccessible__ObjectCreated                   QAccessible__Event = QAccessible__Event(0x8000)
	QAccessible__ObjectDestroyed                 QAccessible__Event = QAccessible__Event(0x8001)
	QAccessible__ObjectShow                      QAccessible__Event = QAccessible__Event(0x8002)
	QAccessible__ObjectHide                      QAccessible__Event = QAccessible__Event(0x8003)
	QAccessible__ObjectReorder                   QAccessible__Event = QAccessible__Event(0x8004)
	QAccessible__Focus                           QAccessible__Event = QAccessible__Event(0x8005)
	QAccessible__Selection                       QAccessible__Event = QAccessible__Event(0x8006)
	QAccessible__SelectionAdd                    QAccessible__Event = QAccessible__Event(0x8007)
	QAccessible__SelectionRemove                 QAccessible__Event = QAccessible__Event(0x8008)
	QAccessible__SelectionWithin                 QAccessible__Event = QAccessible__Event(0x8009)
	QAccessible__StateChanged                    QAccessible__Event = QAccessible__Event(0x800A)
	QAccessible__LocationChanged                 QAccessible__Event = QAccessible__Event(0x800B)
	QAccessible__NameChanged                     QAccessible__Event = QAccessible__Event(0x800C)
	QAccessible__DescriptionChanged              QAccessible__Event = QAccessible__Event(0x800D)
	QAccessible__ValueChanged                    QAccessible__Event = QAccessible__Event(0x800E)
	QAccessible__ParentChanged                   QAccessible__Event = QAccessible__Event(0x800F)
	QAccessible__HelpChanged                     QAccessible__Event = QAccessible__Event(0x80A0)
	QAccessible__DefaultActionChanged            QAccessible__Event = QAccessible__Event(0x80B0)
	QAccessible__AcceleratorChanged              QAccessible__Event = QAccessible__Event(0x80C0)
	QAccessible__InvalidEvent                    QAccessible__Event = QAccessible__Event(0x80c1)
)

//go:generate stringer -type=QAccessible__Role
//QAccessible::Role
type QAccessible__Role int64

const (
	QAccessible__NoRole               QAccessible__Role = QAccessible__Role(0x00000000)
	QAccessible__TitleBar             QAccessible__Role = QAccessible__Role(0x00000001)
	QAccessible__MenuBar              QAccessible__Role = QAccessible__Role(0x00000002)
	QAccessible__ScrollBar            QAccessible__Role = QAccessible__Role(0x00000003)
	QAccessible__Grip                 QAccessible__Role = QAccessible__Role(0x00000004)
	QAccessible__Sound                QAccessible__Role = QAccessible__Role(0x00000005)
	QAccessible__Cursor               QAccessible__Role = QAccessible__Role(0x00000006)
	QAccessible__Caret                QAccessible__Role = QAccessible__Role(0x00000007)
	QAccessible__AlertMessage         QAccessible__Role = QAccessible__Role(0x00000008)
	QAccessible__Window               QAccessible__Role = QAccessible__Role(0x00000009)
	QAccessible__Client               QAccessible__Role = QAccessible__Role(0x0000000A)
	QAccessible__PopupMenu            QAccessible__Role = QAccessible__Role(0x0000000B)
	QAccessible__MenuItem             QAccessible__Role = QAccessible__Role(0x0000000C)
	QAccessible__ToolTip              QAccessible__Role = QAccessible__Role(0x0000000D)
	QAccessible__Application          QAccessible__Role = QAccessible__Role(0x0000000E)
	QAccessible__Document             QAccessible__Role = QAccessible__Role(0x0000000F)
	QAccessible__Pane                 QAccessible__Role = QAccessible__Role(0x00000010)
	QAccessible__Chart                QAccessible__Role = QAccessible__Role(0x00000011)
	QAccessible__Dialog               QAccessible__Role = QAccessible__Role(0x00000012)
	QAccessible__Border               QAccessible__Role = QAccessible__Role(0x00000013)
	QAccessible__Grouping             QAccessible__Role = QAccessible__Role(0x00000014)
	QAccessible__Separator            QAccessible__Role = QAccessible__Role(0x00000015)
	QAccessible__ToolBar              QAccessible__Role = QAccessible__Role(0x00000016)
	QAccessible__StatusBar            QAccessible__Role = QAccessible__Role(0x00000017)
	QAccessible__Table                QAccessible__Role = QAccessible__Role(0x00000018)
	QAccessible__ColumnHeader         QAccessible__Role = QAccessible__Role(0x00000019)
	QAccessible__RowHeader            QAccessible__Role = QAccessible__Role(0x0000001A)
	QAccessible__Column               QAccessible__Role = QAccessible__Role(0x0000001B)
	QAccessible__Row                  QAccessible__Role = QAccessible__Role(0x0000001C)
	QAccessible__Cell                 QAccessible__Role = QAccessible__Role(0x0000001D)
	QAccessible__Link                 QAccessible__Role = QAccessible__Role(0x0000001E)
	QAccessible__HelpBalloon          QAccessible__Role = QAccessible__Role(0x0000001F)
	QAccessible__Assistant            QAccessible__Role = QAccessible__Role(0x00000020)
	QAccessible__List                 QAccessible__Role = QAccessible__Role(0x00000021)
	QAccessible__ListItem             QAccessible__Role = QAccessible__Role(0x00000022)
	QAccessible__Tree                 QAccessible__Role = QAccessible__Role(0x00000023)
	QAccessible__TreeItem             QAccessible__Role = QAccessible__Role(0x00000024)
	QAccessible__PageTab              QAccessible__Role = QAccessible__Role(0x00000025)
	QAccessible__PropertyPage         QAccessible__Role = QAccessible__Role(0x00000026)
	QAccessible__Indicator            QAccessible__Role = QAccessible__Role(0x00000027)
	QAccessible__Graphic              QAccessible__Role = QAccessible__Role(0x00000028)
	QAccessible__StaticText           QAccessible__Role = QAccessible__Role(0x00000029)
	QAccessible__EditableText         QAccessible__Role = QAccessible__Role(0x0000002A)
	QAccessible__Button               QAccessible__Role = QAccessible__Role(0x0000002B)
	QAccessible__CheckBox             QAccessible__Role = QAccessible__Role(0x0000002C)
	QAccessible__RadioButton          QAccessible__Role = QAccessible__Role(0x0000002D)
	QAccessible__ComboBox             QAccessible__Role = QAccessible__Role(0x0000002E)
	QAccessible__ProgressBar          QAccessible__Role = QAccessible__Role(0x00000030)
	QAccessible__Dial                 QAccessible__Role = QAccessible__Role(0x00000031)
	QAccessible__HotkeyField          QAccessible__Role = QAccessible__Role(0x00000032)
	QAccessible__Slider               QAccessible__Role = QAccessible__Role(0x00000033)
	QAccessible__SpinBox              QAccessible__Role = QAccessible__Role(0x00000034)
	QAccessible__Canvas               QAccessible__Role = QAccessible__Role(0x00000035)
	QAccessible__Animation            QAccessible__Role = QAccessible__Role(0x00000036)
	QAccessible__Equation             QAccessible__Role = QAccessible__Role(0x00000037)
	QAccessible__ButtonDropDown       QAccessible__Role = QAccessible__Role(0x00000038)
	QAccessible__ButtonMenu           QAccessible__Role = QAccessible__Role(0x00000039)
	QAccessible__ButtonDropGrid       QAccessible__Role = QAccessible__Role(0x0000003A)
	QAccessible__Whitespace           QAccessible__Role = QAccessible__Role(0x0000003B)
	QAccessible__PageTabList          QAccessible__Role = QAccessible__Role(0x0000003C)
	QAccessible__Clock                QAccessible__Role = QAccessible__Role(0x0000003D)
	QAccessible__Splitter             QAccessible__Role = QAccessible__Role(0x0000003E)
	QAccessible__LayeredPane          QAccessible__Role = QAccessible__Role(0x00000080)
	QAccessible__Terminal             QAccessible__Role = QAccessible__Role(0x00000081)
	QAccessible__Desktop              QAccessible__Role = QAccessible__Role(0x00000082)
	QAccessible__Paragraph            QAccessible__Role = QAccessible__Role(0x00000083)
	QAccessible__WebDocument          QAccessible__Role = QAccessible__Role(0x00000084)
	QAccessible__Section              QAccessible__Role = QAccessible__Role(0x00000085)
	QAccessible__ColorChooser         QAccessible__Role = QAccessible__Role(0x404)
	QAccessible__Footer               QAccessible__Role = QAccessible__Role(0x40E)
	QAccessible__Form                 QAccessible__Role = QAccessible__Role(0x410)
	QAccessible__Heading              QAccessible__Role = QAccessible__Role(0x414)
	QAccessible__Note                 QAccessible__Role = QAccessible__Role(0x41B)
	QAccessible__ComplementaryContent QAccessible__Role = QAccessible__Role(0x42C)
	QAccessible__UserRole             QAccessible__Role = QAccessible__Role(0x0000ffff)
)

//go:generate stringer -type=QAccessible__Text
//QAccessible::Text
type QAccessible__Text int64

const (
	QAccessible__Name             QAccessible__Text = QAccessible__Text(0)
	QAccessible__Description      QAccessible__Text = QAccessible__Text(1)
	QAccessible__Value            QAccessible__Text = QAccessible__Text(2)
	QAccessible__Help             QAccessible__Text = QAccessible__Text(3)
	QAccessible__Accelerator      QAccessible__Text = QAccessible__Text(4)
	QAccessible__DebugDescription QAccessible__Text = QAccessible__Text(5)
	QAccessible__UserText         QAccessible__Text = QAccessible__Text(0x0000ffff)
)

//go:generate stringer -type=QAccessible__RelationFlag
//QAccessible::RelationFlag
type QAccessible__RelationFlag int64

const (
	QAccessible__Label        QAccessible__RelationFlag = QAccessible__RelationFlag(0x00000001)
	QAccessible__Labelled     QAccessible__RelationFlag = QAccessible__RelationFlag(0x00000002)
	QAccessible__Controller   QAccessible__RelationFlag = QAccessible__RelationFlag(0x00000004)
	QAccessible__Controlled   QAccessible__RelationFlag = QAccessible__RelationFlag(0x00000008)
	QAccessible__AllRelations QAccessible__RelationFlag = QAccessible__RelationFlag(0xffffffff)
)

//go:generate stringer -type=QAccessible__InterfaceType
//QAccessible::InterfaceType
type QAccessible__InterfaceType int64

const (
	QAccessible__TextInterface         QAccessible__InterfaceType = QAccessible__InterfaceType(0)
	QAccessible__EditableTextInterface QAccessible__InterfaceType = QAccessible__InterfaceType(1)
	QAccessible__ValueInterface        QAccessible__InterfaceType = QAccessible__InterfaceType(2)
	QAccessible__ActionInterface       QAccessible__InterfaceType = QAccessible__InterfaceType(3)
	QAccessible__ImageInterface        QAccessible__InterfaceType = QAccessible__InterfaceType(4)
	QAccessible__TableInterface        QAccessible__InterfaceType = QAccessible__InterfaceType(5)
	QAccessible__TableCellInterface    QAccessible__InterfaceType = QAccessible__InterfaceType(6)
)

//go:generate stringer -type=QAccessible__TextBoundaryType
//QAccessible::TextBoundaryType
type QAccessible__TextBoundaryType int64

const (
	QAccessible__CharBoundary      QAccessible__TextBoundaryType = QAccessible__TextBoundaryType(0)
	QAccessible__WordBoundary      QAccessible__TextBoundaryType = QAccessible__TextBoundaryType(1)
	QAccessible__SentenceBoundary  QAccessible__TextBoundaryType = QAccessible__TextBoundaryType(2)
	QAccessible__ParagraphBoundary QAccessible__TextBoundaryType = QAccessible__TextBoundaryType(3)
	QAccessible__LineBoundary      QAccessible__TextBoundaryType = QAccessible__TextBoundaryType(4)
	QAccessible__NoBoundary        QAccessible__TextBoundaryType = QAccessible__TextBoundaryType(5)
)

func QAccessible_IsActive() bool {
	return int8(C.QAccessible_QAccessible_IsActive()) != 0
}

func (ptr *QAccessible) IsActive() bool {
	return int8(C.QAccessible_QAccessible_IsActive()) != 0
}

func QAccessible_QueryAccessibleInterface(object core.QObject_ITF) *QAccessibleInterface {
	return NewQAccessibleInterfaceFromPointer(C.QAccessible_QAccessible_QueryAccessibleInterface(core.PointerFromQObject(object)))
}

func (ptr *QAccessible) QueryAccessibleInterface(object core.QObject_ITF) *QAccessibleInterface {
	return NewQAccessibleInterfaceFromPointer(C.QAccessible_QAccessible_QueryAccessibleInterface(core.PointerFromQObject(object)))
}

func QAccessible_SetRootObject(object core.QObject_ITF) {
	C.QAccessible_QAccessible_SetRootObject(core.PointerFromQObject(object))
}

func (ptr *QAccessible) SetRootObject(object core.QObject_ITF) {
	C.QAccessible_QAccessible_SetRootObject(core.PointerFromQObject(object))
}

func QAccessible_UpdateAccessibility2(event QAccessibleEvent_ITF) {
	C.QAccessible_QAccessible_UpdateAccessibility2(PointerFromQAccessibleEvent(event))
}

func (ptr *QAccessible) UpdateAccessibility2(event QAccessibleEvent_ITF) {
	C.QAccessible_QAccessible_UpdateAccessibility2(PointerFromQAccessibleEvent(event))
}

type QAccessibleActionInterface struct {
	ptr unsafe.Pointer
}

type QAccessibleActionInterface_ITF interface {
	QAccessibleActionInterface_PTR() *QAccessibleActionInterface
}

func (ptr *QAccessibleActionInterface) QAccessibleActionInterface_PTR() *QAccessibleActionInterface {
	return ptr
}

func (ptr *QAccessibleActionInterface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAccessibleActionInterface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAccessibleActionInterface(ptr QAccessibleActionInterface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleActionInterface_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleActionInterfaceFromPointer(ptr unsafe.Pointer) (n *QAccessibleActionInterface) {
	n = new(QAccessibleActionInterface)
	n.SetPointer(ptr)
	return
}

//export callbackQAccessibleActionInterface_ActionNames
func callbackQAccessibleActionInterface_ActionNames(ptr unsafe.Pointer) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "actionNames"); signal != nil {
		tempVal := (*(*func() []string)(signal))()
		return C.struct_QtGui_PackedString{data: C.CString(strings.Join(tempVal, "¡¦!")), len: C.longlong(len(strings.Join(tempVal, "¡¦!")))}
	}
	tempVal := make([]string, 0)
	return C.struct_QtGui_PackedString{data: C.CString(strings.Join(tempVal, "¡¦!")), len: C.longlong(len(strings.Join(tempVal, "¡¦!")))}
}

func (ptr *QAccessibleActionInterface) ConnectActionNames(f func() []string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "actionNames"); signal != nil {
			f := func() []string {
				(*(*func() []string)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "actionNames", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "actionNames", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleActionInterface) DisconnectActionNames() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "actionNames")
	}
}

func (ptr *QAccessibleActionInterface) ActionNames() []string {
	if ptr.Pointer() != nil {
		return unpackStringList(cGoUnpackString(C.QAccessibleActionInterface_ActionNames(ptr.Pointer())))
	}
	return make([]string, 0)
}

func QAccessibleActionInterface_DecreaseAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_DecreaseAction())
}

func (ptr *QAccessibleActionInterface) DecreaseAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_DecreaseAction())
}

//export callbackQAccessibleActionInterface_DoAction
func callbackQAccessibleActionInterface_DoAction(ptr unsafe.Pointer, actionName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "doAction"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(actionName))
	}

}

func (ptr *QAccessibleActionInterface) ConnectDoAction(f func(actionName string)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "doAction"); signal != nil {
			f := func(actionName string) {
				(*(*func(string))(signal))(actionName)
				f(actionName)
			}
			qt.ConnectSignal(ptr.Pointer(), "doAction", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "doAction", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleActionInterface) DisconnectDoAction() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "doAction")
	}
}

func (ptr *QAccessibleActionInterface) DoAction(actionName string) {
	if ptr.Pointer() != nil {
		var actionNameC *C.char
		if actionName != "" {
			actionNameC = C.CString(actionName)
			defer C.free(unsafe.Pointer(actionNameC))
		}
		C.QAccessibleActionInterface_DoAction(ptr.Pointer(), C.struct_QtGui_PackedString{data: actionNameC, len: C.longlong(len(actionName))})
	}
}

func QAccessibleActionInterface_IncreaseAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_IncreaseAction())
}

func (ptr *QAccessibleActionInterface) IncreaseAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_IncreaseAction())
}

//export callbackQAccessibleActionInterface_KeyBindingsForAction
func callbackQAccessibleActionInterface_KeyBindingsForAction(ptr unsafe.Pointer, actionName C.struct_QtGui_PackedString) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "keyBindingsForAction"); signal != nil {
		tempVal := (*(*func(string) []string)(signal))(cGoUnpackString(actionName))
		return C.struct_QtGui_PackedString{data: C.CString(strings.Join(tempVal, "¡¦!")), len: C.longlong(len(strings.Join(tempVal, "¡¦!")))}
	}
	tempVal := make([]string, 0)
	return C.struct_QtGui_PackedString{data: C.CString(strings.Join(tempVal, "¡¦!")), len: C.longlong(len(strings.Join(tempVal, "¡¦!")))}
}

func (ptr *QAccessibleActionInterface) ConnectKeyBindingsForAction(f func(actionName string) []string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "keyBindingsForAction"); signal != nil {
			f := func(actionName string) []string {
				(*(*func(string) []string)(signal))(actionName)
				return f(actionName)
			}
			qt.ConnectSignal(ptr.Pointer(), "keyBindingsForAction", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "keyBindingsForAction", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleActionInterface) DisconnectKeyBindingsForAction() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "keyBindingsForAction")
	}
}

func (ptr *QAccessibleActionInterface) KeyBindingsForAction(actionName string) []string {
	if ptr.Pointer() != nil {
		var actionNameC *C.char
		if actionName != "" {
			actionNameC = C.CString(actionName)
			defer C.free(unsafe.Pointer(actionNameC))
		}
		return unpackStringList(cGoUnpackString(C.QAccessibleActionInterface_KeyBindingsForAction(ptr.Pointer(), C.struct_QtGui_PackedString{data: actionNameC, len: C.longlong(len(actionName))})))
	}
	return make([]string, 0)
}

//export callbackQAccessibleActionInterface_LocalizedActionDescription
func callbackQAccessibleActionInterface_LocalizedActionDescription(ptr unsafe.Pointer, actionName C.struct_QtGui_PackedString) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "localizedActionDescription"); signal != nil {
		tempVal := (*(*func(string) string)(signal))(cGoUnpackString(actionName))
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := NewQAccessibleActionInterfaceFromPointer(ptr).LocalizedActionDescriptionDefault(cGoUnpackString(actionName))
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QAccessibleActionInterface) ConnectLocalizedActionDescription(f func(actionName string) string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "localizedActionDescription"); signal != nil {
			f := func(actionName string) string {
				(*(*func(string) string)(signal))(actionName)
				return f(actionName)
			}
			qt.ConnectSignal(ptr.Pointer(), "localizedActionDescription", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "localizedActionDescription", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleActionInterface) DisconnectLocalizedActionDescription() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "localizedActionDescription")
	}
}

func (ptr *QAccessibleActionInterface) LocalizedActionDescription(actionName string) string {
	if ptr.Pointer() != nil {
		var actionNameC *C.char
		if actionName != "" {
			actionNameC = C.CString(actionName)
			defer C.free(unsafe.Pointer(actionNameC))
		}
		return cGoUnpackString(C.QAccessibleActionInterface_LocalizedActionDescription(ptr.Pointer(), C.struct_QtGui_PackedString{data: actionNameC, len: C.longlong(len(actionName))}))
	}
	return ""
}

func (ptr *QAccessibleActionInterface) LocalizedActionDescriptionDefault(actionName string) string {
	if ptr.Pointer() != nil {
		var actionNameC *C.char
		if actionName != "" {
			actionNameC = C.CString(actionName)
			defer C.free(unsafe.Pointer(actionNameC))
		}
		return cGoUnpackString(C.QAccessibleActionInterface_LocalizedActionDescriptionDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: actionNameC, len: C.longlong(len(actionName))}))
	}
	return ""
}

//export callbackQAccessibleActionInterface_LocalizedActionName
func callbackQAccessibleActionInterface_LocalizedActionName(ptr unsafe.Pointer, actionName C.struct_QtGui_PackedString) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "localizedActionName"); signal != nil {
		tempVal := (*(*func(string) string)(signal))(cGoUnpackString(actionName))
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := NewQAccessibleActionInterfaceFromPointer(ptr).LocalizedActionNameDefault(cGoUnpackString(actionName))
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QAccessibleActionInterface) ConnectLocalizedActionName(f func(actionName string) string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "localizedActionName"); signal != nil {
			f := func(actionName string) string {
				(*(*func(string) string)(signal))(actionName)
				return f(actionName)
			}
			qt.ConnectSignal(ptr.Pointer(), "localizedActionName", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "localizedActionName", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleActionInterface) DisconnectLocalizedActionName() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "localizedActionName")
	}
}

func (ptr *QAccessibleActionInterface) LocalizedActionName(actionName string) string {
	if ptr.Pointer() != nil {
		var actionNameC *C.char
		if actionName != "" {
			actionNameC = C.CString(actionName)
			defer C.free(unsafe.Pointer(actionNameC))
		}
		return cGoUnpackString(C.QAccessibleActionInterface_LocalizedActionName(ptr.Pointer(), C.struct_QtGui_PackedString{data: actionNameC, len: C.longlong(len(actionName))}))
	}
	return ""
}

func (ptr *QAccessibleActionInterface) LocalizedActionNameDefault(actionName string) string {
	if ptr.Pointer() != nil {
		var actionNameC *C.char
		if actionName != "" {
			actionNameC = C.CString(actionName)
			defer C.free(unsafe.Pointer(actionNameC))
		}
		return cGoUnpackString(C.QAccessibleActionInterface_LocalizedActionNameDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: actionNameC, len: C.longlong(len(actionName))}))
	}
	return ""
}

func QAccessibleActionInterface_NextPageAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_NextPageAction())
}

func (ptr *QAccessibleActionInterface) NextPageAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_NextPageAction())
}

func QAccessibleActionInterface_PressAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_PressAction())
}

func (ptr *QAccessibleActionInterface) PressAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_PressAction())
}

func QAccessibleActionInterface_PreviousPageAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_PreviousPageAction())
}

func (ptr *QAccessibleActionInterface) PreviousPageAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_PreviousPageAction())
}

func QAccessibleActionInterface_ScrollDownAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ScrollDownAction())
}

func (ptr *QAccessibleActionInterface) ScrollDownAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ScrollDownAction())
}

func QAccessibleActionInterface_ScrollLeftAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ScrollLeftAction())
}

func (ptr *QAccessibleActionInterface) ScrollLeftAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ScrollLeftAction())
}

func QAccessibleActionInterface_ScrollRightAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ScrollRightAction())
}

func (ptr *QAccessibleActionInterface) ScrollRightAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ScrollRightAction())
}

func QAccessibleActionInterface_ScrollUpAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ScrollUpAction())
}

func (ptr *QAccessibleActionInterface) ScrollUpAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ScrollUpAction())
}

func QAccessibleActionInterface_SetFocusAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_SetFocusAction())
}

func (ptr *QAccessibleActionInterface) SetFocusAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_SetFocusAction())
}

func QAccessibleActionInterface_ShowMenuAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ShowMenuAction())
}

func (ptr *QAccessibleActionInterface) ShowMenuAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ShowMenuAction())
}

func QAccessibleActionInterface_ToggleAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ToggleAction())
}

func (ptr *QAccessibleActionInterface) ToggleAction() string {
	return cGoUnpackString(C.QAccessibleActionInterface_QAccessibleActionInterface_ToggleAction())
}

//export callbackQAccessibleActionInterface_DestroyQAccessibleActionInterface
func callbackQAccessibleActionInterface_DestroyQAccessibleActionInterface(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QAccessibleActionInterface"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessibleActionInterfaceFromPointer(ptr).DestroyQAccessibleActionInterfaceDefault()
	}
}

func (ptr *QAccessibleActionInterface) ConnectDestroyQAccessibleActionInterface(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QAccessibleActionInterface"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleActionInterface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleActionInterface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleActionInterface) DisconnectDestroyQAccessibleActionInterface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QAccessibleActionInterface")
	}
}

func (ptr *QAccessibleActionInterface) DestroyQAccessibleActionInterface() {
	if ptr.Pointer() != nil {
		C.QAccessibleActionInterface_DestroyQAccessibleActionInterface(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleActionInterface) DestroyQAccessibleActionInterfaceDefault() {
	if ptr.Pointer() != nil {
		C.QAccessibleActionInterface_DestroyQAccessibleActionInterfaceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QAccessibleEditableTextInterface struct {
	ptr unsafe.Pointer
}

type QAccessibleEditableTextInterface_ITF interface {
	QAccessibleEditableTextInterface_PTR() *QAccessibleEditableTextInterface
}

func (ptr *QAccessibleEditableTextInterface) QAccessibleEditableTextInterface_PTR() *QAccessibleEditableTextInterface {
	return ptr
}

func (ptr *QAccessibleEditableTextInterface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAccessibleEditableTextInterface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAccessibleEditableTextInterface(ptr QAccessibleEditableTextInterface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleEditableTextInterface_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleEditableTextInterfaceFromPointer(ptr unsafe.Pointer) (n *QAccessibleEditableTextInterface) {
	n = new(QAccessibleEditableTextInterface)
	n.SetPointer(ptr)
	return
}

//export callbackQAccessibleEditableTextInterface_DeleteText
func callbackQAccessibleEditableTextInterface_DeleteText(ptr unsafe.Pointer, startOffset C.int, endOffset C.int) {
	if signal := qt.GetSignal(ptr, "deleteText"); signal != nil {
		(*(*func(int, int))(signal))(int(int32(startOffset)), int(int32(endOffset)))
	}

}

func (ptr *QAccessibleEditableTextInterface) ConnectDeleteText(f func(startOffset int, endOffset int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "deleteText"); signal != nil {
			f := func(startOffset int, endOffset int) {
				(*(*func(int, int))(signal))(startOffset, endOffset)
				f(startOffset, endOffset)
			}
			qt.ConnectSignal(ptr.Pointer(), "deleteText", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "deleteText", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleEditableTextInterface) DisconnectDeleteText() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "deleteText")
	}
}

func (ptr *QAccessibleEditableTextInterface) DeleteText(startOffset int, endOffset int) {
	if ptr.Pointer() != nil {
		C.QAccessibleEditableTextInterface_DeleteText(ptr.Pointer(), C.int(int32(startOffset)), C.int(int32(endOffset)))
	}
}

//export callbackQAccessibleEditableTextInterface_InsertText
func callbackQAccessibleEditableTextInterface_InsertText(ptr unsafe.Pointer, offset C.int, text C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "insertText"); signal != nil {
		(*(*func(int, string))(signal))(int(int32(offset)), cGoUnpackString(text))
	}

}

func (ptr *QAccessibleEditableTextInterface) ConnectInsertText(f func(offset int, text string)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "insertText"); signal != nil {
			f := func(offset int, text string) {
				(*(*func(int, string))(signal))(offset, text)
				f(offset, text)
			}
			qt.ConnectSignal(ptr.Pointer(), "insertText", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "insertText", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleEditableTextInterface) DisconnectInsertText() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "insertText")
	}
}

func (ptr *QAccessibleEditableTextInterface) InsertText(offset int, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QAccessibleEditableTextInterface_InsertText(ptr.Pointer(), C.int(int32(offset)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

//export callbackQAccessibleEditableTextInterface_ReplaceText
func callbackQAccessibleEditableTextInterface_ReplaceText(ptr unsafe.Pointer, startOffset C.int, endOffset C.int, text C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "replaceText"); signal != nil {
		(*(*func(int, int, string))(signal))(int(int32(startOffset)), int(int32(endOffset)), cGoUnpackString(text))
	}

}

func (ptr *QAccessibleEditableTextInterface) ConnectReplaceText(f func(startOffset int, endOffset int, text string)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "replaceText"); signal != nil {
			f := func(startOffset int, endOffset int, text string) {
				(*(*func(int, int, string))(signal))(startOffset, endOffset, text)
				f(startOffset, endOffset, text)
			}
			qt.ConnectSignal(ptr.Pointer(), "replaceText", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "replaceText", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleEditableTextInterface) DisconnectReplaceText() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "replaceText")
	}
}

func (ptr *QAccessibleEditableTextInterface) ReplaceText(startOffset int, endOffset int, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QAccessibleEditableTextInterface_ReplaceText(ptr.Pointer(), C.int(int32(startOffset)), C.int(int32(endOffset)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

//export callbackQAccessibleEditableTextInterface_DestroyQAccessibleEditableTextInterface
func callbackQAccessibleEditableTextInterface_DestroyQAccessibleEditableTextInterface(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QAccessibleEditableTextInterface"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessibleEditableTextInterfaceFromPointer(ptr).DestroyQAccessibleEditableTextInterfaceDefault()
	}
}

func (ptr *QAccessibleEditableTextInterface) ConnectDestroyQAccessibleEditableTextInterface(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QAccessibleEditableTextInterface"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleEditableTextInterface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleEditableTextInterface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleEditableTextInterface) DisconnectDestroyQAccessibleEditableTextInterface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QAccessibleEditableTextInterface")
	}
}

func (ptr *QAccessibleEditableTextInterface) DestroyQAccessibleEditableTextInterface() {
	if ptr.Pointer() != nil {
		C.QAccessibleEditableTextInterface_DestroyQAccessibleEditableTextInterface(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleEditableTextInterface) DestroyQAccessibleEditableTextInterfaceDefault() {
	if ptr.Pointer() != nil {
		C.QAccessibleEditableTextInterface_DestroyQAccessibleEditableTextInterfaceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QAccessibleEvent struct {
	ptr unsafe.Pointer
}

type QAccessibleEvent_ITF interface {
	QAccessibleEvent_PTR() *QAccessibleEvent
}

func (ptr *QAccessibleEvent) QAccessibleEvent_PTR() *QAccessibleEvent {
	return ptr
}

func (ptr *QAccessibleEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAccessibleEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAccessibleEvent(ptr QAccessibleEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleEvent_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleEventFromPointer(ptr unsafe.Pointer) (n *QAccessibleEvent) {
	n = new(QAccessibleEvent)
	n.SetPointer(ptr)
	return
}
func NewQAccessibleEvent2(object core.QObject_ITF, ty QAccessible__Event) *QAccessibleEvent {
	return NewQAccessibleEventFromPointer(C.QAccessibleEvent_NewQAccessibleEvent2(core.PointerFromQObject(object), C.longlong(ty)))
}

func NewQAccessibleEvent3(interfa QAccessibleInterface_ITF, ty QAccessible__Event) *QAccessibleEvent {
	return NewQAccessibleEventFromPointer(C.QAccessibleEvent_NewQAccessibleEvent3(PointerFromQAccessibleInterface(interfa), C.longlong(ty)))
}

//export callbackQAccessibleEvent_AccessibleInterface
func callbackQAccessibleEvent_AccessibleInterface(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "accessibleInterface"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func() *QAccessibleInterface)(signal))())
	}

	return PointerFromQAccessibleInterface(NewQAccessibleEventFromPointer(ptr).AccessibleInterfaceDefault())
}

func (ptr *QAccessibleEvent) ConnectAccessibleInterface(f func() *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "accessibleInterface"); signal != nil {
			f := func() *QAccessibleInterface {
				(*(*func() *QAccessibleInterface)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "accessibleInterface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "accessibleInterface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleEvent) DisconnectAccessibleInterface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "accessibleInterface")
	}
}

func (ptr *QAccessibleEvent) AccessibleInterface() *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleEvent_AccessibleInterface(ptr.Pointer()))
	}
	return nil
}

func (ptr *QAccessibleEvent) AccessibleInterfaceDefault() *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleEvent_AccessibleInterfaceDefault(ptr.Pointer()))
	}
	return nil
}

func (ptr *QAccessibleEvent) Child() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleEvent_Child(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleEvent) Object() *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QAccessibleEvent_Object(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QAccessibleEvent) SetChild(child int) {
	if ptr.Pointer() != nil {
		C.QAccessibleEvent_SetChild(ptr.Pointer(), C.int(int32(child)))
	}
}

func (ptr *QAccessibleEvent) Type() QAccessible__Event {
	if ptr.Pointer() != nil {
		return QAccessible__Event(C.QAccessibleEvent_Type(ptr.Pointer()))
	}
	return 0
}

//export callbackQAccessibleEvent_DestroyQAccessibleEvent
func callbackQAccessibleEvent_DestroyQAccessibleEvent(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QAccessibleEvent"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessibleEventFromPointer(ptr).DestroyQAccessibleEventDefault()
	}
}

func (ptr *QAccessibleEvent) ConnectDestroyQAccessibleEvent(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QAccessibleEvent"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleEvent) DisconnectDestroyQAccessibleEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QAccessibleEvent")
	}
}

func (ptr *QAccessibleEvent) DestroyQAccessibleEvent() {
	if ptr.Pointer() != nil {
		C.QAccessibleEvent_DestroyQAccessibleEvent(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleEvent) DestroyQAccessibleEventDefault() {
	if ptr.Pointer() != nil {
		C.QAccessibleEvent_DestroyQAccessibleEventDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QAccessibleInterface struct {
	ptr unsafe.Pointer
}

type QAccessibleInterface_ITF interface {
	QAccessibleInterface_PTR() *QAccessibleInterface
}

func (ptr *QAccessibleInterface) QAccessibleInterface_PTR() *QAccessibleInterface {
	return ptr
}

func (ptr *QAccessibleInterface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAccessibleInterface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAccessibleInterface(ptr QAccessibleInterface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleInterface_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleInterfaceFromPointer(ptr unsafe.Pointer) (n *QAccessibleInterface) {
	n = new(QAccessibleInterface)
	n.SetPointer(ptr)
	return
}
func (ptr *QAccessibleInterface) ActionInterface() *QAccessibleActionInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleActionInterfaceFromPointer(C.QAccessibleInterface_ActionInterface(ptr.Pointer()))
	}
	return nil
}

//export callbackQAccessibleInterface_BackgroundColor
func callbackQAccessibleInterface_BackgroundColor(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "backgroundColor"); signal != nil {
		return PointerFromQColor((*(*func() *QColor)(signal))())
	}

	return PointerFromQColor(NewQAccessibleInterfaceFromPointer(ptr).BackgroundColorDefault())
}

func (ptr *QAccessibleInterface) ConnectBackgroundColor(f func() *QColor) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "backgroundColor"); signal != nil {
			f := func() *QColor {
				(*(*func() *QColor)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "backgroundColor", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "backgroundColor", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectBackgroundColor() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "backgroundColor")
	}
}

func (ptr *QAccessibleInterface) BackgroundColor() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QAccessibleInterface_BackgroundColor(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QAccessibleInterface) BackgroundColorDefault() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QAccessibleInterface_BackgroundColorDefault(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleInterface_Child
func callbackQAccessibleInterface_Child(ptr unsafe.Pointer, index C.int) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "child"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func(int) *QAccessibleInterface)(signal))(int(int32(index))))
	}

	return PointerFromQAccessibleInterface(nil)
}

func (ptr *QAccessibleInterface) ConnectChild(f func(index int) *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "child"); signal != nil {
			f := func(index int) *QAccessibleInterface {
				(*(*func(int) *QAccessibleInterface)(signal))(index)
				return f(index)
			}
			qt.ConnectSignal(ptr.Pointer(), "child", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "child", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectChild() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "child")
	}
}

func (ptr *QAccessibleInterface) Child(index int) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleInterface_Child(ptr.Pointer(), C.int(int32(index))))
	}
	return nil
}

//export callbackQAccessibleInterface_ChildAt
func callbackQAccessibleInterface_ChildAt(ptr unsafe.Pointer, x C.int, y C.int) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "childAt"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func(int, int) *QAccessibleInterface)(signal))(int(int32(x)), int(int32(y))))
	}

	return PointerFromQAccessibleInterface(nil)
}

func (ptr *QAccessibleInterface) ConnectChildAt(f func(x int, y int) *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "childAt"); signal != nil {
			f := func(x int, y int) *QAccessibleInterface {
				(*(*func(int, int) *QAccessibleInterface)(signal))(x, y)
				return f(x, y)
			}
			qt.ConnectSignal(ptr.Pointer(), "childAt", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "childAt", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectChildAt() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "childAt")
	}
}

func (ptr *QAccessibleInterface) ChildAt(x int, y int) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleInterface_ChildAt(ptr.Pointer(), C.int(int32(x)), C.int(int32(y))))
	}
	return nil
}

//export callbackQAccessibleInterface_ChildCount
func callbackQAccessibleInterface_ChildCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "childCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleInterface) ConnectChildCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "childCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "childCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "childCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectChildCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "childCount")
	}
}

func (ptr *QAccessibleInterface) ChildCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleInterface_ChildCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleInterface_FocusChild
func callbackQAccessibleInterface_FocusChild(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "focusChild"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func() *QAccessibleInterface)(signal))())
	}

	return PointerFromQAccessibleInterface(NewQAccessibleInterfaceFromPointer(ptr).FocusChildDefault())
}

func (ptr *QAccessibleInterface) ConnectFocusChild(f func() *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "focusChild"); signal != nil {
			f := func() *QAccessibleInterface {
				(*(*func() *QAccessibleInterface)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "focusChild", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "focusChild", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectFocusChild() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "focusChild")
	}
}

func (ptr *QAccessibleInterface) FocusChild() *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleInterface_FocusChild(ptr.Pointer()))
	}
	return nil
}

func (ptr *QAccessibleInterface) FocusChildDefault() *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleInterface_FocusChildDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQAccessibleInterface_ForegroundColor
func callbackQAccessibleInterface_ForegroundColor(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "foregroundColor"); signal != nil {
		return PointerFromQColor((*(*func() *QColor)(signal))())
	}

	return PointerFromQColor(NewQAccessibleInterfaceFromPointer(ptr).ForegroundColorDefault())
}

func (ptr *QAccessibleInterface) ConnectForegroundColor(f func() *QColor) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "foregroundColor"); signal != nil {
			f := func() *QColor {
				(*(*func() *QColor)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "foregroundColor", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "foregroundColor", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectForegroundColor() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "foregroundColor")
	}
}

func (ptr *QAccessibleInterface) ForegroundColor() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QAccessibleInterface_ForegroundColor(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QAccessibleInterface) ForegroundColorDefault() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QAccessibleInterface_ForegroundColorDefault(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleInterface_IndexOfChild
func callbackQAccessibleInterface_IndexOfChild(ptr unsafe.Pointer, child unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "indexOfChild"); signal != nil {
		return C.int(int32((*(*func(*QAccessibleInterface) int)(signal))(NewQAccessibleInterfaceFromPointer(child))))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleInterface) ConnectIndexOfChild(f func(child *QAccessibleInterface) int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "indexOfChild"); signal != nil {
			f := func(child *QAccessibleInterface) int {
				(*(*func(*QAccessibleInterface) int)(signal))(child)
				return f(child)
			}
			qt.ConnectSignal(ptr.Pointer(), "indexOfChild", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "indexOfChild", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectIndexOfChild() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "indexOfChild")
	}
}

func (ptr *QAccessibleInterface) IndexOfChild(child QAccessibleInterface_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleInterface_IndexOfChild(ptr.Pointer(), PointerFromQAccessibleInterface(child))))
	}
	return 0
}

//export callbackQAccessibleInterface_Interface_cast
func callbackQAccessibleInterface_Interface_cast(ptr unsafe.Pointer, ty C.longlong) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "interface_cast"); signal != nil {
		return (*(*func(QAccessible__InterfaceType) unsafe.Pointer)(signal))(QAccessible__InterfaceType(ty))
	}

	return NewQAccessibleInterfaceFromPointer(ptr).Interface_castDefault(QAccessible__InterfaceType(ty))
}

func (ptr *QAccessibleInterface) ConnectInterface_cast(f func(ty QAccessible__InterfaceType) unsafe.Pointer) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "interface_cast"); signal != nil {
			f := func(ty QAccessible__InterfaceType) unsafe.Pointer {
				(*(*func(QAccessible__InterfaceType) unsafe.Pointer)(signal))(ty)
				return f(ty)
			}
			qt.ConnectSignal(ptr.Pointer(), "interface_cast", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "interface_cast", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectInterface_cast() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "interface_cast")
	}
}

func (ptr *QAccessibleInterface) Interface_cast(ty QAccessible__InterfaceType) unsafe.Pointer {
	if ptr.Pointer() != nil {
		return C.QAccessibleInterface_Interface_cast(ptr.Pointer(), C.longlong(ty))
	}
	return nil
}

func (ptr *QAccessibleInterface) Interface_castDefault(ty QAccessible__InterfaceType) unsafe.Pointer {
	if ptr.Pointer() != nil {
		return C.QAccessibleInterface_Interface_castDefault(ptr.Pointer(), C.longlong(ty))
	}
	return nil
}

//export callbackQAccessibleInterface_IsValid
func callbackQAccessibleInterface_IsValid(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "isValid"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QAccessibleInterface) ConnectIsValid(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "isValid"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "isValid", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "isValid", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectIsValid() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "isValid")
	}
}

func (ptr *QAccessibleInterface) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessibleInterface_IsValid(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQAccessibleInterface_Object
func callbackQAccessibleInterface_Object(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "object"); signal != nil {
		return core.PointerFromQObject((*(*func() *core.QObject)(signal))())
	}

	return core.PointerFromQObject(core.NewQObject(nil))
}

func (ptr *QAccessibleInterface) ConnectObject(f func() *core.QObject) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "object"); signal != nil {
			f := func() *core.QObject {
				(*(*func() *core.QObject)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "object", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "object", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "object")
	}
}

func (ptr *QAccessibleInterface) Object() *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QAccessibleInterface_Object(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleInterface_Parent
func callbackQAccessibleInterface_Parent(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "parent"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func() *QAccessibleInterface)(signal))())
	}

	return PointerFromQAccessibleInterface(nil)
}

func (ptr *QAccessibleInterface) ConnectParent(f func() *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "parent"); signal != nil {
			f := func() *QAccessibleInterface {
				(*(*func() *QAccessibleInterface)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "parent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "parent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectParent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "parent")
	}
}

func (ptr *QAccessibleInterface) Parent() *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleInterface_Parent(ptr.Pointer()))
	}
	return nil
}

//export callbackQAccessibleInterface_Rect
func callbackQAccessibleInterface_Rect(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "rect"); signal != nil {
		return core.PointerFromQRect((*(*func() *core.QRect)(signal))())
	}

	return core.PointerFromQRect(core.NewQRect())
}

func (ptr *QAccessibleInterface) ConnectRect(f func() *core.QRect) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "rect"); signal != nil {
			f := func() *core.QRect {
				(*(*func() *core.QRect)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "rect", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "rect", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectRect() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "rect")
	}
}

func (ptr *QAccessibleInterface) Rect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QAccessibleInterface_Rect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleInterface_Role
func callbackQAccessibleInterface_Role(ptr unsafe.Pointer) C.longlong {
	if signal := qt.GetSignal(ptr, "role"); signal != nil {
		return C.longlong((*(*func() QAccessible__Role)(signal))())
	}

	return C.longlong(0)
}

func (ptr *QAccessibleInterface) ConnectRole(f func() QAccessible__Role) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "role"); signal != nil {
			f := func() QAccessible__Role {
				(*(*func() QAccessible__Role)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "role", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "role", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectRole() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "role")
	}
}

func (ptr *QAccessibleInterface) Role() QAccessible__Role {
	if ptr.Pointer() != nil {
		return QAccessible__Role(C.QAccessibleInterface_Role(ptr.Pointer()))
	}
	return 0
}

//export callbackQAccessibleInterface_SetText
func callbackQAccessibleInterface_SetText(ptr unsafe.Pointer, t C.longlong, text C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "setText"); signal != nil {
		(*(*func(QAccessible__Text, string))(signal))(QAccessible__Text(t), cGoUnpackString(text))
	}

}

func (ptr *QAccessibleInterface) ConnectSetText(f func(t QAccessible__Text, text string)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setText"); signal != nil {
			f := func(t QAccessible__Text, text string) {
				(*(*func(QAccessible__Text, string))(signal))(t, text)
				f(t, text)
			}
			qt.ConnectSignal(ptr.Pointer(), "setText", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setText", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectSetText() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setText")
	}
}

func (ptr *QAccessibleInterface) SetText(t QAccessible__Text, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QAccessibleInterface_SetText(ptr.Pointer(), C.longlong(t), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QAccessibleInterface) TableCellInterface() *QAccessibleTableCellInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleTableCellInterfaceFromPointer(C.QAccessibleInterface_TableCellInterface(ptr.Pointer()))
	}
	return nil
}

func (ptr *QAccessibleInterface) TableInterface() *QAccessibleTableInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleTableInterfaceFromPointer(C.QAccessibleInterface_TableInterface(ptr.Pointer()))
	}
	return nil
}

//export callbackQAccessibleInterface_Text
func callbackQAccessibleInterface_Text(ptr unsafe.Pointer, t C.longlong) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "text"); signal != nil {
		tempVal := (*(*func(QAccessible__Text) string)(signal))(QAccessible__Text(t))
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := ""
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QAccessibleInterface) ConnectText(f func(t QAccessible__Text) string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "text"); signal != nil {
			f := func(t QAccessible__Text) string {
				(*(*func(QAccessible__Text) string)(signal))(t)
				return f(t)
			}
			qt.ConnectSignal(ptr.Pointer(), "text", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "text", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectText() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "text")
	}
}

func (ptr *QAccessibleInterface) Text(t QAccessible__Text) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleInterface_Text(ptr.Pointer(), C.longlong(t)))
	}
	return ""
}

func (ptr *QAccessibleInterface) TextInterface() *QAccessibleTextInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleTextInterfaceFromPointer(C.QAccessibleInterface_TextInterface(ptr.Pointer()))
	}
	return nil
}

func (ptr *QAccessibleInterface) ValueInterface() *QAccessibleValueInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleValueInterfaceFromPointer(C.QAccessibleInterface_ValueInterface(ptr.Pointer()))
	}
	return nil
}

//export callbackQAccessibleInterface_Window
func callbackQAccessibleInterface_Window(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "window"); signal != nil {
		return PointerFromQWindow((*(*func() *QWindow)(signal))())
	}

	return PointerFromQWindow(NewQAccessibleInterfaceFromPointer(ptr).WindowDefault())
}

func (ptr *QAccessibleInterface) ConnectWindow(f func() *QWindow) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "window"); signal != nil {
			f := func() *QWindow {
				(*(*func() *QWindow)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "window", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "window", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectWindow() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "window")
	}
}

func (ptr *QAccessibleInterface) Window() *QWindow {
	if ptr.Pointer() != nil {
		tmpValue := NewQWindowFromPointer(C.QAccessibleInterface_Window(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QAccessibleInterface) WindowDefault() *QWindow {
	if ptr.Pointer() != nil {
		tmpValue := NewQWindowFromPointer(C.QAccessibleInterface_WindowDefault(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleInterface_DestroyQAccessibleInterface
func callbackQAccessibleInterface_DestroyQAccessibleInterface(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QAccessibleInterface"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessibleInterfaceFromPointer(ptr).DestroyQAccessibleInterfaceDefault()
	}
}

func (ptr *QAccessibleInterface) ConnectDestroyQAccessibleInterface(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QAccessibleInterface"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleInterface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleInterface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleInterface) DisconnectDestroyQAccessibleInterface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QAccessibleInterface")
	}
}

func (ptr *QAccessibleInterface) DestroyQAccessibleInterface() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessibleInterface_DestroyQAccessibleInterface(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleInterface) DestroyQAccessibleInterfaceDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessibleInterface_DestroyQAccessibleInterfaceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QAccessibleObject struct {
	QAccessibleInterface
}

type QAccessibleObject_ITF interface {
	QAccessibleInterface_ITF
	QAccessibleObject_PTR() *QAccessibleObject
}

func (ptr *QAccessibleObject) QAccessibleObject_PTR() *QAccessibleObject {
	return ptr
}

func (ptr *QAccessibleObject) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleInterface_PTR().Pointer()
	}
	return nil
}

func (ptr *QAccessibleObject) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAccessibleInterface_PTR().SetPointer(p)
	}
}

func PointerFromQAccessibleObject(ptr QAccessibleObject_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleObject_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleObjectFromPointer(ptr unsafe.Pointer) (n *QAccessibleObject) {
	n = new(QAccessibleObject)
	n.SetPointer(ptr)
	return
}

//export callbackQAccessibleObject_ChildAt
func callbackQAccessibleObject_ChildAt(ptr unsafe.Pointer, x C.int, y C.int) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "childAt"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func(int, int) *QAccessibleInterface)(signal))(int(int32(x)), int(int32(y))))
	}

	return PointerFromQAccessibleInterface(NewQAccessibleObjectFromPointer(ptr).ChildAtDefault(int(int32(x)), int(int32(y))))
}

func (ptr *QAccessibleObject) ConnectChildAt(f func(x int, y int) *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "childAt"); signal != nil {
			f := func(x int, y int) *QAccessibleInterface {
				(*(*func(int, int) *QAccessibleInterface)(signal))(x, y)
				return f(x, y)
			}
			qt.ConnectSignal(ptr.Pointer(), "childAt", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "childAt", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleObject) DisconnectChildAt() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "childAt")
	}
}

func (ptr *QAccessibleObject) ChildAt(x int, y int) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleObject_ChildAt(ptr.Pointer(), C.int(int32(x)), C.int(int32(y))))
	}
	return nil
}

func (ptr *QAccessibleObject) ChildAtDefault(x int, y int) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleObject_ChildAtDefault(ptr.Pointer(), C.int(int32(x)), C.int(int32(y))))
	}
	return nil
}

//export callbackQAccessibleObject_IsValid
func callbackQAccessibleObject_IsValid(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "isValid"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(NewQAccessibleObjectFromPointer(ptr).IsValidDefault())))
}

func (ptr *QAccessibleObject) ConnectIsValid(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "isValid"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "isValid", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "isValid", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleObject) DisconnectIsValid() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "isValid")
	}
}

func (ptr *QAccessibleObject) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessibleObject_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QAccessibleObject) IsValidDefault() bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessibleObject_IsValidDefault(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQAccessibleObject_Object
func callbackQAccessibleObject_Object(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "object"); signal != nil {
		return core.PointerFromQObject((*(*func() *core.QObject)(signal))())
	}

	return core.PointerFromQObject(NewQAccessibleObjectFromPointer(ptr).ObjectDefault())
}

func (ptr *QAccessibleObject) ConnectObject(f func() *core.QObject) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "object"); signal != nil {
			f := func() *core.QObject {
				(*(*func() *core.QObject)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "object", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "object", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleObject) DisconnectObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "object")
	}
}

func (ptr *QAccessibleObject) Object() *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QAccessibleObject_Object(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QAccessibleObject) ObjectDefault() *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QAccessibleObject_ObjectDefault(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleObject_Rect
func callbackQAccessibleObject_Rect(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "rect"); signal != nil {
		return core.PointerFromQRect((*(*func() *core.QRect)(signal))())
	}

	return core.PointerFromQRect(NewQAccessibleObjectFromPointer(ptr).RectDefault())
}

func (ptr *QAccessibleObject) ConnectRect(f func() *core.QRect) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "rect"); signal != nil {
			f := func() *core.QRect {
				(*(*func() *core.QRect)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "rect", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "rect", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleObject) DisconnectRect() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "rect")
	}
}

func (ptr *QAccessibleObject) Rect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QAccessibleObject_Rect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QAccessibleObject) RectDefault() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QAccessibleObject_RectDefault(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleObject_SetText
func callbackQAccessibleObject_SetText(ptr unsafe.Pointer, t C.longlong, text C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "setText"); signal != nil {
		(*(*func(QAccessible__Text, string))(signal))(QAccessible__Text(t), cGoUnpackString(text))
	} else {
		NewQAccessibleObjectFromPointer(ptr).SetTextDefault(QAccessible__Text(t), cGoUnpackString(text))
	}
}

func (ptr *QAccessibleObject) ConnectSetText(f func(t QAccessible__Text, text string)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setText"); signal != nil {
			f := func(t QAccessible__Text, text string) {
				(*(*func(QAccessible__Text, string))(signal))(t, text)
				f(t, text)
			}
			qt.ConnectSignal(ptr.Pointer(), "setText", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setText", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleObject) DisconnectSetText() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setText")
	}
}

func (ptr *QAccessibleObject) SetText(t QAccessible__Text, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QAccessibleObject_SetText(ptr.Pointer(), C.longlong(t), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QAccessibleObject) SetTextDefault(t QAccessible__Text, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QAccessibleObject_SetTextDefault(ptr.Pointer(), C.longlong(t), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

//export callbackQAccessibleObject_DestroyQAccessibleObject
func callbackQAccessibleObject_DestroyQAccessibleObject(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QAccessibleObject"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessibleObjectFromPointer(ptr).DestroyQAccessibleObjectDefault()
	}
}

func (ptr *QAccessibleObject) ConnectDestroyQAccessibleObject(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QAccessibleObject"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleObject", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleObject", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleObject) DisconnectDestroyQAccessibleObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QAccessibleObject")
	}
}

func (ptr *QAccessibleObject) DestroyQAccessibleObject() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessibleObject_DestroyQAccessibleObject(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleObject) DestroyQAccessibleObjectDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessibleObject_DestroyQAccessibleObjectDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//export callbackQAccessibleObject_Child
func callbackQAccessibleObject_Child(ptr unsafe.Pointer, index C.int) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "child"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func(int) *QAccessibleInterface)(signal))(int(int32(index))))
	}

	return PointerFromQAccessibleInterface(NewQAccessibleObjectFromPointer(ptr).ChildDefault(int(int32(index))))
}

func (ptr *QAccessibleObject) Child(index int) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleObject_Child(ptr.Pointer(), C.int(int32(index))))
	}
	return nil
}

func (ptr *QAccessibleObject) ChildDefault(index int) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleObject_ChildDefault(ptr.Pointer(), C.int(int32(index))))
	}
	return nil
}

//export callbackQAccessibleObject_ChildCount
func callbackQAccessibleObject_ChildCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "childCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(NewQAccessibleObjectFromPointer(ptr).ChildCountDefault()))
}

func (ptr *QAccessibleObject) ChildCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleObject_ChildCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleObject) ChildCountDefault() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleObject_ChildCountDefault(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleObject_IndexOfChild
func callbackQAccessibleObject_IndexOfChild(ptr unsafe.Pointer, child unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "indexOfChild"); signal != nil {
		return C.int(int32((*(*func(*QAccessibleInterface) int)(signal))(NewQAccessibleInterfaceFromPointer(child))))
	}

	return C.int(int32(NewQAccessibleObjectFromPointer(ptr).IndexOfChildDefault(NewQAccessibleInterfaceFromPointer(child))))
}

func (ptr *QAccessibleObject) IndexOfChild(child QAccessibleInterface_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleObject_IndexOfChild(ptr.Pointer(), PointerFromQAccessibleInterface(child))))
	}
	return 0
}

func (ptr *QAccessibleObject) IndexOfChildDefault(child QAccessibleInterface_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleObject_IndexOfChildDefault(ptr.Pointer(), PointerFromQAccessibleInterface(child))))
	}
	return 0
}

//export callbackQAccessibleObject_Parent
func callbackQAccessibleObject_Parent(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "parent"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func() *QAccessibleInterface)(signal))())
	}

	return PointerFromQAccessibleInterface(NewQAccessibleObjectFromPointer(ptr).ParentDefault())
}

func (ptr *QAccessibleObject) Parent() *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleObject_Parent(ptr.Pointer()))
	}
	return nil
}

func (ptr *QAccessibleObject) ParentDefault() *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleObject_ParentDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQAccessibleObject_Role
func callbackQAccessibleObject_Role(ptr unsafe.Pointer) C.longlong {
	if signal := qt.GetSignal(ptr, "role"); signal != nil {
		return C.longlong((*(*func() QAccessible__Role)(signal))())
	}

	return C.longlong(NewQAccessibleObjectFromPointer(ptr).RoleDefault())
}

func (ptr *QAccessibleObject) Role() QAccessible__Role {
	if ptr.Pointer() != nil {
		return QAccessible__Role(C.QAccessibleObject_Role(ptr.Pointer()))
	}
	return 0
}

func (ptr *QAccessibleObject) RoleDefault() QAccessible__Role {
	if ptr.Pointer() != nil {
		return QAccessible__Role(C.QAccessibleObject_RoleDefault(ptr.Pointer()))
	}
	return 0
}

//export callbackQAccessibleObject_Text
func callbackQAccessibleObject_Text(ptr unsafe.Pointer, t C.longlong) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "text"); signal != nil {
		tempVal := (*(*func(QAccessible__Text) string)(signal))(QAccessible__Text(t))
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := NewQAccessibleObjectFromPointer(ptr).TextDefault(QAccessible__Text(t))
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QAccessibleObject) Text(t QAccessible__Text) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleObject_Text(ptr.Pointer(), C.longlong(t)))
	}
	return ""
}

func (ptr *QAccessibleObject) TextDefault(t QAccessible__Text) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleObject_TextDefault(ptr.Pointer(), C.longlong(t)))
	}
	return ""
}

type QAccessiblePlugin struct {
	core.QObject
}

type QAccessiblePlugin_ITF interface {
	core.QObject_ITF
	QAccessiblePlugin_PTR() *QAccessiblePlugin
}

func (ptr *QAccessiblePlugin) QAccessiblePlugin_PTR() *QAccessiblePlugin {
	return ptr
}

func (ptr *QAccessiblePlugin) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QAccessiblePlugin) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQAccessiblePlugin(ptr QAccessiblePlugin_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessiblePlugin_PTR().Pointer()
	}
	return nil
}

func NewQAccessiblePluginFromPointer(ptr unsafe.Pointer) (n *QAccessiblePlugin) {
	n = new(QAccessiblePlugin)
	n.SetPointer(ptr)
	return
}
func NewQAccessiblePlugin(parent core.QObject_ITF) *QAccessiblePlugin {
	tmpValue := NewQAccessiblePluginFromPointer(C.QAccessiblePlugin_NewQAccessiblePlugin(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQAccessiblePlugin_Create
func callbackQAccessiblePlugin_Create(ptr unsafe.Pointer, key C.struct_QtGui_PackedString, object unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "create"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func(string, *core.QObject) *QAccessibleInterface)(signal))(cGoUnpackString(key), core.NewQObjectFromPointer(object)))
	}

	return PointerFromQAccessibleInterface(nil)
}

func (ptr *QAccessiblePlugin) ConnectCreate(f func(key string, object *core.QObject) *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "create"); signal != nil {
			f := func(key string, object *core.QObject) *QAccessibleInterface {
				(*(*func(string, *core.QObject) *QAccessibleInterface)(signal))(key, object)
				return f(key, object)
			}
			qt.ConnectSignal(ptr.Pointer(), "create", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "create", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessiblePlugin) DisconnectCreate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "create")
	}
}

func (ptr *QAccessiblePlugin) Create(key string, object core.QObject_ITF) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		var keyC *C.char
		if key != "" {
			keyC = C.CString(key)
			defer C.free(unsafe.Pointer(keyC))
		}
		return NewQAccessibleInterfaceFromPointer(C.QAccessiblePlugin_Create(ptr.Pointer(), C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, core.PointerFromQObject(object)))
	}
	return nil
}

//export callbackQAccessiblePlugin_DestroyQAccessiblePlugin
func callbackQAccessiblePlugin_DestroyQAccessiblePlugin(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QAccessiblePlugin"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessiblePluginFromPointer(ptr).DestroyQAccessiblePluginDefault()
	}
}

func (ptr *QAccessiblePlugin) ConnectDestroyQAccessiblePlugin(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QAccessiblePlugin"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QAccessiblePlugin", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QAccessiblePlugin", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessiblePlugin) DisconnectDestroyQAccessiblePlugin() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QAccessiblePlugin")
	}
}

func (ptr *QAccessiblePlugin) DestroyQAccessiblePlugin() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessiblePlugin_DestroyQAccessiblePlugin(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessiblePlugin) DestroyQAccessiblePluginDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessiblePlugin_DestroyQAccessiblePluginDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessiblePlugin) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QAccessiblePlugin___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QAccessiblePlugin) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessiblePlugin___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QAccessiblePlugin) __children_newList() unsafe.Pointer {
	return C.QAccessiblePlugin___children_newList(ptr.Pointer())
}

func (ptr *QAccessiblePlugin) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QAccessiblePlugin___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QAccessiblePlugin) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessiblePlugin___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QAccessiblePlugin) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QAccessiblePlugin___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QAccessiblePlugin) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QAccessiblePlugin___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QAccessiblePlugin) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessiblePlugin___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QAccessiblePlugin) __findChildren_newList() unsafe.Pointer {
	return C.QAccessiblePlugin___findChildren_newList(ptr.Pointer())
}

func (ptr *QAccessiblePlugin) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QAccessiblePlugin___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QAccessiblePlugin) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessiblePlugin___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QAccessiblePlugin) __findChildren_newList3() unsafe.Pointer {
	return C.QAccessiblePlugin___findChildren_newList3(ptr.Pointer())
}

//export callbackQAccessiblePlugin_ChildEvent
func callbackQAccessiblePlugin_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQAccessiblePluginFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QAccessiblePlugin) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessiblePlugin_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQAccessiblePlugin_ConnectNotify
func callbackQAccessiblePlugin_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQAccessiblePluginFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QAccessiblePlugin) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessiblePlugin_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQAccessiblePlugin_CustomEvent
func callbackQAccessiblePlugin_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQAccessiblePluginFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QAccessiblePlugin) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessiblePlugin_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQAccessiblePlugin_DeleteLater
func callbackQAccessiblePlugin_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessiblePluginFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QAccessiblePlugin) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessiblePlugin_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQAccessiblePlugin_Destroyed
func callbackQAccessiblePlugin_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQAccessiblePlugin_DisconnectNotify
func callbackQAccessiblePlugin_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQAccessiblePluginFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QAccessiblePlugin) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessiblePlugin_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQAccessiblePlugin_Event
func callbackQAccessiblePlugin_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQAccessiblePluginFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QAccessiblePlugin) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessiblePlugin_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQAccessiblePlugin_EventFilter
func callbackQAccessiblePlugin_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQAccessiblePluginFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QAccessiblePlugin) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessiblePlugin_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQAccessiblePlugin_MetaObject
func callbackQAccessiblePlugin_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQAccessiblePluginFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QAccessiblePlugin) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QAccessiblePlugin_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQAccessiblePlugin_ObjectNameChanged
func callbackQAccessiblePlugin_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQAccessiblePlugin_TimerEvent
func callbackQAccessiblePlugin_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQAccessiblePluginFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QAccessiblePlugin) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessiblePlugin_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QAccessibleStateChangeEvent struct {
	QAccessibleEvent
}

type QAccessibleStateChangeEvent_ITF interface {
	QAccessibleEvent_ITF
	QAccessibleStateChangeEvent_PTR() *QAccessibleStateChangeEvent
}

func (ptr *QAccessibleStateChangeEvent) QAccessibleStateChangeEvent_PTR() *QAccessibleStateChangeEvent {
	return ptr
}

func (ptr *QAccessibleStateChangeEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QAccessibleStateChangeEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAccessibleEvent_PTR().SetPointer(p)
	}
}

func PointerFromQAccessibleStateChangeEvent(ptr QAccessibleStateChangeEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleStateChangeEvent_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleStateChangeEventFromPointer(ptr unsafe.Pointer) (n *QAccessibleStateChangeEvent) {
	n = new(QAccessibleStateChangeEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QAccessibleStateChangeEvent) DestroyQAccessibleStateChangeEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QAccessibleTableCellInterface struct {
	ptr unsafe.Pointer
}

type QAccessibleTableCellInterface_ITF interface {
	QAccessibleTableCellInterface_PTR() *QAccessibleTableCellInterface
}

func (ptr *QAccessibleTableCellInterface) QAccessibleTableCellInterface_PTR() *QAccessibleTableCellInterface {
	return ptr
}

func (ptr *QAccessibleTableCellInterface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAccessibleTableCellInterface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAccessibleTableCellInterface(ptr QAccessibleTableCellInterface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTableCellInterface_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleTableCellInterfaceFromPointer(ptr unsafe.Pointer) (n *QAccessibleTableCellInterface) {
	n = new(QAccessibleTableCellInterface)
	n.SetPointer(ptr)
	return
}

//export callbackQAccessibleTableCellInterface_ColumnExtent
func callbackQAccessibleTableCellInterface_ColumnExtent(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "columnExtent"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTableCellInterface) ConnectColumnExtent(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "columnExtent"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "columnExtent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "columnExtent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableCellInterface) DisconnectColumnExtent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "columnExtent")
	}
}

func (ptr *QAccessibleTableCellInterface) ColumnExtent() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableCellInterface_ColumnExtent(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTableCellInterface_ColumnHeaderCells
func callbackQAccessibleTableCellInterface_ColumnHeaderCells(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "columnHeaderCells"); signal != nil {
		return func() unsafe.Pointer {
			tmpList := NewQAccessibleTableCellInterfaceFromPointer(NewQAccessibleTableCellInterfaceFromPointer(nil).__columnHeaderCells_newList())
			for _, v := range (*(*func() []*QAccessibleInterface)(signal))() {
				tmpList.__columnHeaderCells_setList(v)
			}
			return tmpList.Pointer()
		}()
	}

	return func() unsafe.Pointer {
		tmpList := NewQAccessibleTableCellInterfaceFromPointer(NewQAccessibleTableCellInterfaceFromPointer(nil).__columnHeaderCells_newList())
		for _, v := range make([]*QAccessibleInterface, 0) {
			tmpList.__columnHeaderCells_setList(v)
		}
		return tmpList.Pointer()
	}()
}

func (ptr *QAccessibleTableCellInterface) ConnectColumnHeaderCells(f func() []*QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "columnHeaderCells"); signal != nil {
			f := func() []*QAccessibleInterface {
				(*(*func() []*QAccessibleInterface)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "columnHeaderCells", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "columnHeaderCells", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableCellInterface) DisconnectColumnHeaderCells() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "columnHeaderCells")
	}
}

func (ptr *QAccessibleTableCellInterface) ColumnHeaderCells() []*QAccessibleInterface {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QAccessibleInterface {
			out := make([]*QAccessibleInterface, int(l.len))
			tmpList := NewQAccessibleTableCellInterfaceFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__columnHeaderCells_atList(i)
			}
			return out
		}(C.QAccessibleTableCellInterface_ColumnHeaderCells(ptr.Pointer()))
	}
	return make([]*QAccessibleInterface, 0)
}

//export callbackQAccessibleTableCellInterface_ColumnIndex
func callbackQAccessibleTableCellInterface_ColumnIndex(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "columnIndex"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTableCellInterface) ConnectColumnIndex(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "columnIndex"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "columnIndex", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "columnIndex", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableCellInterface) DisconnectColumnIndex() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "columnIndex")
	}
}

func (ptr *QAccessibleTableCellInterface) ColumnIndex() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableCellInterface_ColumnIndex(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTableCellInterface_IsSelected
func callbackQAccessibleTableCellInterface_IsSelected(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "isSelected"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QAccessibleTableCellInterface) ConnectIsSelected(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "isSelected"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "isSelected", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "isSelected", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableCellInterface) DisconnectIsSelected() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "isSelected")
	}
}

func (ptr *QAccessibleTableCellInterface) IsSelected() bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessibleTableCellInterface_IsSelected(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQAccessibleTableCellInterface_RowExtent
func callbackQAccessibleTableCellInterface_RowExtent(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "rowExtent"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTableCellInterface) ConnectRowExtent(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "rowExtent"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "rowExtent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "rowExtent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableCellInterface) DisconnectRowExtent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "rowExtent")
	}
}

func (ptr *QAccessibleTableCellInterface) RowExtent() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableCellInterface_RowExtent(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTableCellInterface_RowHeaderCells
func callbackQAccessibleTableCellInterface_RowHeaderCells(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "rowHeaderCells"); signal != nil {
		return func() unsafe.Pointer {
			tmpList := NewQAccessibleTableCellInterfaceFromPointer(NewQAccessibleTableCellInterfaceFromPointer(nil).__rowHeaderCells_newList())
			for _, v := range (*(*func() []*QAccessibleInterface)(signal))() {
				tmpList.__rowHeaderCells_setList(v)
			}
			return tmpList.Pointer()
		}()
	}

	return func() unsafe.Pointer {
		tmpList := NewQAccessibleTableCellInterfaceFromPointer(NewQAccessibleTableCellInterfaceFromPointer(nil).__rowHeaderCells_newList())
		for _, v := range make([]*QAccessibleInterface, 0) {
			tmpList.__rowHeaderCells_setList(v)
		}
		return tmpList.Pointer()
	}()
}

func (ptr *QAccessibleTableCellInterface) ConnectRowHeaderCells(f func() []*QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "rowHeaderCells"); signal != nil {
			f := func() []*QAccessibleInterface {
				(*(*func() []*QAccessibleInterface)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "rowHeaderCells", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "rowHeaderCells", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableCellInterface) DisconnectRowHeaderCells() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "rowHeaderCells")
	}
}

func (ptr *QAccessibleTableCellInterface) RowHeaderCells() []*QAccessibleInterface {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QAccessibleInterface {
			out := make([]*QAccessibleInterface, int(l.len))
			tmpList := NewQAccessibleTableCellInterfaceFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__rowHeaderCells_atList(i)
			}
			return out
		}(C.QAccessibleTableCellInterface_RowHeaderCells(ptr.Pointer()))
	}
	return make([]*QAccessibleInterface, 0)
}

//export callbackQAccessibleTableCellInterface_RowIndex
func callbackQAccessibleTableCellInterface_RowIndex(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "rowIndex"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTableCellInterface) ConnectRowIndex(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "rowIndex"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "rowIndex", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "rowIndex", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableCellInterface) DisconnectRowIndex() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "rowIndex")
	}
}

func (ptr *QAccessibleTableCellInterface) RowIndex() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableCellInterface_RowIndex(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTableCellInterface_Table
func callbackQAccessibleTableCellInterface_Table(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "table"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func() *QAccessibleInterface)(signal))())
	}

	return PointerFromQAccessibleInterface(nil)
}

func (ptr *QAccessibleTableCellInterface) ConnectTable(f func() *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "table"); signal != nil {
			f := func() *QAccessibleInterface {
				(*(*func() *QAccessibleInterface)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "table", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "table", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableCellInterface) DisconnectTable() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "table")
	}
}

func (ptr *QAccessibleTableCellInterface) Table() *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleTableCellInterface_Table(ptr.Pointer()))
	}
	return nil
}

//export callbackQAccessibleTableCellInterface_DestroyQAccessibleTableCellInterface
func callbackQAccessibleTableCellInterface_DestroyQAccessibleTableCellInterface(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QAccessibleTableCellInterface"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessibleTableCellInterfaceFromPointer(ptr).DestroyQAccessibleTableCellInterfaceDefault()
	}
}

func (ptr *QAccessibleTableCellInterface) ConnectDestroyQAccessibleTableCellInterface(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QAccessibleTableCellInterface"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleTableCellInterface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleTableCellInterface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableCellInterface) DisconnectDestroyQAccessibleTableCellInterface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QAccessibleTableCellInterface")
	}
}

func (ptr *QAccessibleTableCellInterface) DestroyQAccessibleTableCellInterface() {
	if ptr.Pointer() != nil {
		C.QAccessibleTableCellInterface_DestroyQAccessibleTableCellInterface(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleTableCellInterface) DestroyQAccessibleTableCellInterfaceDefault() {
	if ptr.Pointer() != nil {
		C.QAccessibleTableCellInterface_DestroyQAccessibleTableCellInterfaceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleTableCellInterface) __columnHeaderCells_atList(i int) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleTableCellInterface___columnHeaderCells_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QAccessibleTableCellInterface) __columnHeaderCells_setList(i QAccessibleInterface_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableCellInterface___columnHeaderCells_setList(ptr.Pointer(), PointerFromQAccessibleInterface(i))
	}
}

func (ptr *QAccessibleTableCellInterface) __columnHeaderCells_newList() unsafe.Pointer {
	return C.QAccessibleTableCellInterface___columnHeaderCells_newList(ptr.Pointer())
}

func (ptr *QAccessibleTableCellInterface) __rowHeaderCells_atList(i int) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleTableCellInterface___rowHeaderCells_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QAccessibleTableCellInterface) __rowHeaderCells_setList(i QAccessibleInterface_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableCellInterface___rowHeaderCells_setList(ptr.Pointer(), PointerFromQAccessibleInterface(i))
	}
}

func (ptr *QAccessibleTableCellInterface) __rowHeaderCells_newList() unsafe.Pointer {
	return C.QAccessibleTableCellInterface___rowHeaderCells_newList(ptr.Pointer())
}

type QAccessibleTableInterface struct {
	ptr unsafe.Pointer
}

type QAccessibleTableInterface_ITF interface {
	QAccessibleTableInterface_PTR() *QAccessibleTableInterface
}

func (ptr *QAccessibleTableInterface) QAccessibleTableInterface_PTR() *QAccessibleTableInterface {
	return ptr
}

func (ptr *QAccessibleTableInterface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAccessibleTableInterface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAccessibleTableInterface(ptr QAccessibleTableInterface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTableInterface_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleTableInterfaceFromPointer(ptr unsafe.Pointer) (n *QAccessibleTableInterface) {
	n = new(QAccessibleTableInterface)
	n.SetPointer(ptr)
	return
}

//export callbackQAccessibleTableInterface_Caption
func callbackQAccessibleTableInterface_Caption(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "caption"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func() *QAccessibleInterface)(signal))())
	}

	return PointerFromQAccessibleInterface(nil)
}

func (ptr *QAccessibleTableInterface) ConnectCaption(f func() *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "caption"); signal != nil {
			f := func() *QAccessibleInterface {
				(*(*func() *QAccessibleInterface)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "caption", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "caption", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectCaption() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "caption")
	}
}

func (ptr *QAccessibleTableInterface) Caption() *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleTableInterface_Caption(ptr.Pointer()))
	}
	return nil
}

//export callbackQAccessibleTableInterface_CellAt
func callbackQAccessibleTableInterface_CellAt(ptr unsafe.Pointer, row C.int, column C.int) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "cellAt"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func(int, int) *QAccessibleInterface)(signal))(int(int32(row)), int(int32(column))))
	}

	return PointerFromQAccessibleInterface(nil)
}

func (ptr *QAccessibleTableInterface) ConnectCellAt(f func(row int, column int) *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "cellAt"); signal != nil {
			f := func(row int, column int) *QAccessibleInterface {
				(*(*func(int, int) *QAccessibleInterface)(signal))(row, column)
				return f(row, column)
			}
			qt.ConnectSignal(ptr.Pointer(), "cellAt", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "cellAt", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectCellAt() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "cellAt")
	}
}

func (ptr *QAccessibleTableInterface) CellAt(row int, column int) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleTableInterface_CellAt(ptr.Pointer(), C.int(int32(row)), C.int(int32(column))))
	}
	return nil
}

//export callbackQAccessibleTableInterface_ColumnCount
func callbackQAccessibleTableInterface_ColumnCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "columnCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTableInterface) ConnectColumnCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "columnCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "columnCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "columnCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectColumnCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "columnCount")
	}
}

func (ptr *QAccessibleTableInterface) ColumnCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableInterface_ColumnCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTableInterface_ColumnDescription
func callbackQAccessibleTableInterface_ColumnDescription(ptr unsafe.Pointer, column C.int) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "columnDescription"); signal != nil {
		tempVal := (*(*func(int) string)(signal))(int(int32(column)))
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := ""
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QAccessibleTableInterface) ConnectColumnDescription(f func(column int) string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "columnDescription"); signal != nil {
			f := func(column int) string {
				(*(*func(int) string)(signal))(column)
				return f(column)
			}
			qt.ConnectSignal(ptr.Pointer(), "columnDescription", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "columnDescription", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectColumnDescription() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "columnDescription")
	}
}

func (ptr *QAccessibleTableInterface) ColumnDescription(column int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTableInterface_ColumnDescription(ptr.Pointer(), C.int(int32(column))))
	}
	return ""
}

//export callbackQAccessibleTableInterface_IsColumnSelected
func callbackQAccessibleTableInterface_IsColumnSelected(ptr unsafe.Pointer, column C.int) C.char {
	if signal := qt.GetSignal(ptr, "isColumnSelected"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int) bool)(signal))(int(int32(column))))))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QAccessibleTableInterface) ConnectIsColumnSelected(f func(column int) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "isColumnSelected"); signal != nil {
			f := func(column int) bool {
				(*(*func(int) bool)(signal))(column)
				return f(column)
			}
			qt.ConnectSignal(ptr.Pointer(), "isColumnSelected", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "isColumnSelected", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectIsColumnSelected() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "isColumnSelected")
	}
}

func (ptr *QAccessibleTableInterface) IsColumnSelected(column int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessibleTableInterface_IsColumnSelected(ptr.Pointer(), C.int(int32(column)))) != 0
	}
	return false
}

//export callbackQAccessibleTableInterface_IsRowSelected
func callbackQAccessibleTableInterface_IsRowSelected(ptr unsafe.Pointer, row C.int) C.char {
	if signal := qt.GetSignal(ptr, "isRowSelected"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int) bool)(signal))(int(int32(row))))))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QAccessibleTableInterface) ConnectIsRowSelected(f func(row int) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "isRowSelected"); signal != nil {
			f := func(row int) bool {
				(*(*func(int) bool)(signal))(row)
				return f(row)
			}
			qt.ConnectSignal(ptr.Pointer(), "isRowSelected", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "isRowSelected", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectIsRowSelected() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "isRowSelected")
	}
}

func (ptr *QAccessibleTableInterface) IsRowSelected(row int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessibleTableInterface_IsRowSelected(ptr.Pointer(), C.int(int32(row)))) != 0
	}
	return false
}

//export callbackQAccessibleTableInterface_ModelChange
func callbackQAccessibleTableInterface_ModelChange(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "modelChange"); signal != nil {
		(*(*func(*QAccessibleTableModelChangeEvent))(signal))(NewQAccessibleTableModelChangeEventFromPointer(event))
	}

}

func (ptr *QAccessibleTableInterface) ConnectModelChange(f func(event *QAccessibleTableModelChangeEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "modelChange"); signal != nil {
			f := func(event *QAccessibleTableModelChangeEvent) {
				(*(*func(*QAccessibleTableModelChangeEvent))(signal))(event)
				f(event)
			}
			qt.ConnectSignal(ptr.Pointer(), "modelChange", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "modelChange", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectModelChange() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "modelChange")
	}
}

func (ptr *QAccessibleTableInterface) ModelChange(event QAccessibleTableModelChangeEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableInterface_ModelChange(ptr.Pointer(), PointerFromQAccessibleTableModelChangeEvent(event))
	}
}

//export callbackQAccessibleTableInterface_RowCount
func callbackQAccessibleTableInterface_RowCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "rowCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTableInterface) ConnectRowCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "rowCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "rowCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "rowCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectRowCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "rowCount")
	}
}

func (ptr *QAccessibleTableInterface) RowCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableInterface_RowCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTableInterface_RowDescription
func callbackQAccessibleTableInterface_RowDescription(ptr unsafe.Pointer, row C.int) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "rowDescription"); signal != nil {
		tempVal := (*(*func(int) string)(signal))(int(int32(row)))
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := ""
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QAccessibleTableInterface) ConnectRowDescription(f func(row int) string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "rowDescription"); signal != nil {
			f := func(row int) string {
				(*(*func(int) string)(signal))(row)
				return f(row)
			}
			qt.ConnectSignal(ptr.Pointer(), "rowDescription", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "rowDescription", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectRowDescription() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "rowDescription")
	}
}

func (ptr *QAccessibleTableInterface) RowDescription(row int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTableInterface_RowDescription(ptr.Pointer(), C.int(int32(row))))
	}
	return ""
}

//export callbackQAccessibleTableInterface_SelectColumn
func callbackQAccessibleTableInterface_SelectColumn(ptr unsafe.Pointer, column C.int) C.char {
	if signal := qt.GetSignal(ptr, "selectColumn"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int) bool)(signal))(int(int32(column))))))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QAccessibleTableInterface) ConnectSelectColumn(f func(column int) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "selectColumn"); signal != nil {
			f := func(column int) bool {
				(*(*func(int) bool)(signal))(column)
				return f(column)
			}
			qt.ConnectSignal(ptr.Pointer(), "selectColumn", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selectColumn", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectSelectColumn() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "selectColumn")
	}
}

func (ptr *QAccessibleTableInterface) SelectColumn(column int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessibleTableInterface_SelectColumn(ptr.Pointer(), C.int(int32(column)))) != 0
	}
	return false
}

//export callbackQAccessibleTableInterface_SelectRow
func callbackQAccessibleTableInterface_SelectRow(ptr unsafe.Pointer, row C.int) C.char {
	if signal := qt.GetSignal(ptr, "selectRow"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int) bool)(signal))(int(int32(row))))))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QAccessibleTableInterface) ConnectSelectRow(f func(row int) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "selectRow"); signal != nil {
			f := func(row int) bool {
				(*(*func(int) bool)(signal))(row)
				return f(row)
			}
			qt.ConnectSignal(ptr.Pointer(), "selectRow", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selectRow", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectSelectRow() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "selectRow")
	}
}

func (ptr *QAccessibleTableInterface) SelectRow(row int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessibleTableInterface_SelectRow(ptr.Pointer(), C.int(int32(row)))) != 0
	}
	return false
}

//export callbackQAccessibleTableInterface_SelectedCellCount
func callbackQAccessibleTableInterface_SelectedCellCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "selectedCellCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTableInterface) ConnectSelectedCellCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "selectedCellCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "selectedCellCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selectedCellCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectSelectedCellCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "selectedCellCount")
	}
}

func (ptr *QAccessibleTableInterface) SelectedCellCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableInterface_SelectedCellCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTableInterface_SelectedCells
func callbackQAccessibleTableInterface_SelectedCells(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "selectedCells"); signal != nil {
		return func() unsafe.Pointer {
			tmpList := NewQAccessibleTableInterfaceFromPointer(NewQAccessibleTableInterfaceFromPointer(nil).__selectedCells_newList())
			for _, v := range (*(*func() []*QAccessibleInterface)(signal))() {
				tmpList.__selectedCells_setList(v)
			}
			return tmpList.Pointer()
		}()
	}

	return func() unsafe.Pointer {
		tmpList := NewQAccessibleTableInterfaceFromPointer(NewQAccessibleTableInterfaceFromPointer(nil).__selectedCells_newList())
		for _, v := range make([]*QAccessibleInterface, 0) {
			tmpList.__selectedCells_setList(v)
		}
		return tmpList.Pointer()
	}()
}

func (ptr *QAccessibleTableInterface) ConnectSelectedCells(f func() []*QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "selectedCells"); signal != nil {
			f := func() []*QAccessibleInterface {
				(*(*func() []*QAccessibleInterface)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "selectedCells", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selectedCells", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectSelectedCells() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "selectedCells")
	}
}

func (ptr *QAccessibleTableInterface) SelectedCells() []*QAccessibleInterface {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QAccessibleInterface {
			out := make([]*QAccessibleInterface, int(l.len))
			tmpList := NewQAccessibleTableInterfaceFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__selectedCells_atList(i)
			}
			return out
		}(C.QAccessibleTableInterface_SelectedCells(ptr.Pointer()))
	}
	return make([]*QAccessibleInterface, 0)
}

//export callbackQAccessibleTableInterface_SelectedColumnCount
func callbackQAccessibleTableInterface_SelectedColumnCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "selectedColumnCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTableInterface) ConnectSelectedColumnCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "selectedColumnCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "selectedColumnCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selectedColumnCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectSelectedColumnCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "selectedColumnCount")
	}
}

func (ptr *QAccessibleTableInterface) SelectedColumnCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableInterface_SelectedColumnCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTableInterface_SelectedColumns
func callbackQAccessibleTableInterface_SelectedColumns(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "selectedColumns"); signal != nil {
		return func() unsafe.Pointer {
			tmpList := NewQAccessibleTableInterfaceFromPointer(NewQAccessibleTableInterfaceFromPointer(nil).__selectedColumns_newList())
			for _, v := range (*(*func() []int)(signal))() {
				tmpList.__selectedColumns_setList(v)
			}
			return tmpList.Pointer()
		}()
	}

	return func() unsafe.Pointer {
		tmpList := NewQAccessibleTableInterfaceFromPointer(NewQAccessibleTableInterfaceFromPointer(nil).__selectedColumns_newList())
		for _, v := range make([]int, 0) {
			tmpList.__selectedColumns_setList(v)
		}
		return tmpList.Pointer()
	}()
}

func (ptr *QAccessibleTableInterface) ConnectSelectedColumns(f func() []int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "selectedColumns"); signal != nil {
			f := func() []int {
				(*(*func() []int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "selectedColumns", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selectedColumns", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectSelectedColumns() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "selectedColumns")
	}
}

func (ptr *QAccessibleTableInterface) SelectedColumns() []int {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []int {
			out := make([]int, int(l.len))
			tmpList := NewQAccessibleTableInterfaceFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__selectedColumns_atList(i)
			}
			return out
		}(C.QAccessibleTableInterface_SelectedColumns(ptr.Pointer()))
	}
	return make([]int, 0)
}

//export callbackQAccessibleTableInterface_SelectedRowCount
func callbackQAccessibleTableInterface_SelectedRowCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "selectedRowCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTableInterface) ConnectSelectedRowCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "selectedRowCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "selectedRowCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selectedRowCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectSelectedRowCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "selectedRowCount")
	}
}

func (ptr *QAccessibleTableInterface) SelectedRowCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableInterface_SelectedRowCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTableInterface_SelectedRows
func callbackQAccessibleTableInterface_SelectedRows(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "selectedRows"); signal != nil {
		return func() unsafe.Pointer {
			tmpList := NewQAccessibleTableInterfaceFromPointer(NewQAccessibleTableInterfaceFromPointer(nil).__selectedRows_newList())
			for _, v := range (*(*func() []int)(signal))() {
				tmpList.__selectedRows_setList(v)
			}
			return tmpList.Pointer()
		}()
	}

	return func() unsafe.Pointer {
		tmpList := NewQAccessibleTableInterfaceFromPointer(NewQAccessibleTableInterfaceFromPointer(nil).__selectedRows_newList())
		for _, v := range make([]int, 0) {
			tmpList.__selectedRows_setList(v)
		}
		return tmpList.Pointer()
	}()
}

func (ptr *QAccessibleTableInterface) ConnectSelectedRows(f func() []int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "selectedRows"); signal != nil {
			f := func() []int {
				(*(*func() []int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "selectedRows", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selectedRows", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectSelectedRows() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "selectedRows")
	}
}

func (ptr *QAccessibleTableInterface) SelectedRows() []int {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []int {
			out := make([]int, int(l.len))
			tmpList := NewQAccessibleTableInterfaceFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__selectedRows_atList(i)
			}
			return out
		}(C.QAccessibleTableInterface_SelectedRows(ptr.Pointer()))
	}
	return make([]int, 0)
}

//export callbackQAccessibleTableInterface_Summary
func callbackQAccessibleTableInterface_Summary(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "summary"); signal != nil {
		return PointerFromQAccessibleInterface((*(*func() *QAccessibleInterface)(signal))())
	}

	return PointerFromQAccessibleInterface(nil)
}

func (ptr *QAccessibleTableInterface) ConnectSummary(f func() *QAccessibleInterface) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "summary"); signal != nil {
			f := func() *QAccessibleInterface {
				(*(*func() *QAccessibleInterface)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "summary", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "summary", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectSummary() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "summary")
	}
}

func (ptr *QAccessibleTableInterface) Summary() *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleTableInterface_Summary(ptr.Pointer()))
	}
	return nil
}

//export callbackQAccessibleTableInterface_UnselectColumn
func callbackQAccessibleTableInterface_UnselectColumn(ptr unsafe.Pointer, column C.int) C.char {
	if signal := qt.GetSignal(ptr, "unselectColumn"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int) bool)(signal))(int(int32(column))))))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QAccessibleTableInterface) ConnectUnselectColumn(f func(column int) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "unselectColumn"); signal != nil {
			f := func(column int) bool {
				(*(*func(int) bool)(signal))(column)
				return f(column)
			}
			qt.ConnectSignal(ptr.Pointer(), "unselectColumn", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "unselectColumn", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectUnselectColumn() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "unselectColumn")
	}
}

func (ptr *QAccessibleTableInterface) UnselectColumn(column int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessibleTableInterface_UnselectColumn(ptr.Pointer(), C.int(int32(column)))) != 0
	}
	return false
}

//export callbackQAccessibleTableInterface_UnselectRow
func callbackQAccessibleTableInterface_UnselectRow(ptr unsafe.Pointer, row C.int) C.char {
	if signal := qt.GetSignal(ptr, "unselectRow"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int) bool)(signal))(int(int32(row))))))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QAccessibleTableInterface) ConnectUnselectRow(f func(row int) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "unselectRow"); signal != nil {
			f := func(row int) bool {
				(*(*func(int) bool)(signal))(row)
				return f(row)
			}
			qt.ConnectSignal(ptr.Pointer(), "unselectRow", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "unselectRow", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectUnselectRow() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "unselectRow")
	}
}

func (ptr *QAccessibleTableInterface) UnselectRow(row int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QAccessibleTableInterface_UnselectRow(ptr.Pointer(), C.int(int32(row)))) != 0
	}
	return false
}

//export callbackQAccessibleTableInterface_DestroyQAccessibleTableInterface
func callbackQAccessibleTableInterface_DestroyQAccessibleTableInterface(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QAccessibleTableInterface"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessibleTableInterfaceFromPointer(ptr).DestroyQAccessibleTableInterfaceDefault()
	}
}

func (ptr *QAccessibleTableInterface) ConnectDestroyQAccessibleTableInterface(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QAccessibleTableInterface"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleTableInterface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleTableInterface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTableInterface) DisconnectDestroyQAccessibleTableInterface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QAccessibleTableInterface")
	}
}

func (ptr *QAccessibleTableInterface) DestroyQAccessibleTableInterface() {
	if ptr.Pointer() != nil {
		C.QAccessibleTableInterface_DestroyQAccessibleTableInterface(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleTableInterface) DestroyQAccessibleTableInterfaceDefault() {
	if ptr.Pointer() != nil {
		C.QAccessibleTableInterface_DestroyQAccessibleTableInterfaceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleTableInterface) __selectedCells_atList(i int) *QAccessibleInterface {
	if ptr.Pointer() != nil {
		return NewQAccessibleInterfaceFromPointer(C.QAccessibleTableInterface___selectedCells_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QAccessibleTableInterface) __selectedCells_setList(i QAccessibleInterface_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableInterface___selectedCells_setList(ptr.Pointer(), PointerFromQAccessibleInterface(i))
	}
}

func (ptr *QAccessibleTableInterface) __selectedCells_newList() unsafe.Pointer {
	return C.QAccessibleTableInterface___selectedCells_newList(ptr.Pointer())
}

func (ptr *QAccessibleTableInterface) __selectedColumns_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableInterface___selectedColumns_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QAccessibleTableInterface) __selectedColumns_setList(i int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableInterface___selectedColumns_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QAccessibleTableInterface) __selectedColumns_newList() unsafe.Pointer {
	return C.QAccessibleTableInterface___selectedColumns_newList(ptr.Pointer())
}

func (ptr *QAccessibleTableInterface) __selectedRows_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableInterface___selectedRows_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QAccessibleTableInterface) __selectedRows_setList(i int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableInterface___selectedRows_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QAccessibleTableInterface) __selectedRows_newList() unsafe.Pointer {
	return C.QAccessibleTableInterface___selectedRows_newList(ptr.Pointer())
}

type QAccessibleTableModelChangeEvent struct {
	QAccessibleEvent
}

type QAccessibleTableModelChangeEvent_ITF interface {
	QAccessibleEvent_ITF
	QAccessibleTableModelChangeEvent_PTR() *QAccessibleTableModelChangeEvent
}

func (ptr *QAccessibleTableModelChangeEvent) QAccessibleTableModelChangeEvent_PTR() *QAccessibleTableModelChangeEvent {
	return ptr
}

func (ptr *QAccessibleTableModelChangeEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QAccessibleTableModelChangeEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAccessibleEvent_PTR().SetPointer(p)
	}
}

func PointerFromQAccessibleTableModelChangeEvent(ptr QAccessibleTableModelChangeEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTableModelChangeEvent_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleTableModelChangeEventFromPointer(ptr unsafe.Pointer) (n *QAccessibleTableModelChangeEvent) {
	n = new(QAccessibleTableModelChangeEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QAccessibleTableModelChangeEvent) DestroyQAccessibleTableModelChangeEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QAccessibleTableModelChangeEvent__ModelChangeType
//QAccessibleTableModelChangeEvent::ModelChangeType
type QAccessibleTableModelChangeEvent__ModelChangeType int64

const (
	QAccessibleTableModelChangeEvent__ModelReset      QAccessibleTableModelChangeEvent__ModelChangeType = QAccessibleTableModelChangeEvent__ModelChangeType(0)
	QAccessibleTableModelChangeEvent__DataChanged     QAccessibleTableModelChangeEvent__ModelChangeType = QAccessibleTableModelChangeEvent__ModelChangeType(1)
	QAccessibleTableModelChangeEvent__RowsInserted    QAccessibleTableModelChangeEvent__ModelChangeType = QAccessibleTableModelChangeEvent__ModelChangeType(2)
	QAccessibleTableModelChangeEvent__ColumnsInserted QAccessibleTableModelChangeEvent__ModelChangeType = QAccessibleTableModelChangeEvent__ModelChangeType(3)
	QAccessibleTableModelChangeEvent__RowsRemoved     QAccessibleTableModelChangeEvent__ModelChangeType = QAccessibleTableModelChangeEvent__ModelChangeType(4)
	QAccessibleTableModelChangeEvent__ColumnsRemoved  QAccessibleTableModelChangeEvent__ModelChangeType = QAccessibleTableModelChangeEvent__ModelChangeType(5)
)

func NewQAccessibleTableModelChangeEvent(object core.QObject_ITF, changeType QAccessibleTableModelChangeEvent__ModelChangeType) *QAccessibleTableModelChangeEvent {
	return NewQAccessibleTableModelChangeEventFromPointer(C.QAccessibleTableModelChangeEvent_NewQAccessibleTableModelChangeEvent(core.PointerFromQObject(object), C.longlong(changeType)))
}

func NewQAccessibleTableModelChangeEvent2(iface QAccessibleInterface_ITF, changeType QAccessibleTableModelChangeEvent__ModelChangeType) *QAccessibleTableModelChangeEvent {
	return NewQAccessibleTableModelChangeEventFromPointer(C.QAccessibleTableModelChangeEvent_NewQAccessibleTableModelChangeEvent2(PointerFromQAccessibleInterface(iface), C.longlong(changeType)))
}

func (ptr *QAccessibleTableModelChangeEvent) FirstColumn() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableModelChangeEvent_FirstColumn(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleTableModelChangeEvent) FirstRow() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableModelChangeEvent_FirstRow(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleTableModelChangeEvent) LastColumn() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableModelChangeEvent_LastColumn(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleTableModelChangeEvent) LastRow() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTableModelChangeEvent_LastRow(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleTableModelChangeEvent) ModelChangeType() QAccessibleTableModelChangeEvent__ModelChangeType {
	if ptr.Pointer() != nil {
		return QAccessibleTableModelChangeEvent__ModelChangeType(C.QAccessibleTableModelChangeEvent_ModelChangeType(ptr.Pointer()))
	}
	return 0
}

func (ptr *QAccessibleTableModelChangeEvent) SetFirstColumn(column int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableModelChangeEvent_SetFirstColumn(ptr.Pointer(), C.int(int32(column)))
	}
}

func (ptr *QAccessibleTableModelChangeEvent) SetFirstRow(row int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableModelChangeEvent_SetFirstRow(ptr.Pointer(), C.int(int32(row)))
	}
}

func (ptr *QAccessibleTableModelChangeEvent) SetLastColumn(column int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableModelChangeEvent_SetLastColumn(ptr.Pointer(), C.int(int32(column)))
	}
}

func (ptr *QAccessibleTableModelChangeEvent) SetLastRow(row int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableModelChangeEvent_SetLastRow(ptr.Pointer(), C.int(int32(row)))
	}
}

func (ptr *QAccessibleTableModelChangeEvent) SetModelChangeType(changeType QAccessibleTableModelChangeEvent__ModelChangeType) {
	if ptr.Pointer() != nil {
		C.QAccessibleTableModelChangeEvent_SetModelChangeType(ptr.Pointer(), C.longlong(changeType))
	}
}

type QAccessibleTextCursorEvent struct {
	QAccessibleEvent
}

type QAccessibleTextCursorEvent_ITF interface {
	QAccessibleEvent_ITF
	QAccessibleTextCursorEvent_PTR() *QAccessibleTextCursorEvent
}

func (ptr *QAccessibleTextCursorEvent) QAccessibleTextCursorEvent_PTR() *QAccessibleTextCursorEvent {
	return ptr
}

func (ptr *QAccessibleTextCursorEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QAccessibleTextCursorEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAccessibleEvent_PTR().SetPointer(p)
	}
}

func PointerFromQAccessibleTextCursorEvent(ptr QAccessibleTextCursorEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTextCursorEvent_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleTextCursorEventFromPointer(ptr unsafe.Pointer) (n *QAccessibleTextCursorEvent) {
	n = new(QAccessibleTextCursorEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QAccessibleTextCursorEvent) DestroyQAccessibleTextCursorEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQAccessibleTextCursorEvent(object core.QObject_ITF, cursorPos int) *QAccessibleTextCursorEvent {
	return NewQAccessibleTextCursorEventFromPointer(C.QAccessibleTextCursorEvent_NewQAccessibleTextCursorEvent(core.PointerFromQObject(object), C.int(int32(cursorPos))))
}

func NewQAccessibleTextCursorEvent2(iface QAccessibleInterface_ITF, cursorPos int) *QAccessibleTextCursorEvent {
	return NewQAccessibleTextCursorEventFromPointer(C.QAccessibleTextCursorEvent_NewQAccessibleTextCursorEvent2(PointerFromQAccessibleInterface(iface), C.int(int32(cursorPos))))
}

func (ptr *QAccessibleTextCursorEvent) CursorPosition() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTextCursorEvent_CursorPosition(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleTextCursorEvent) SetCursorPosition(position int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTextCursorEvent_SetCursorPosition(ptr.Pointer(), C.int(int32(position)))
	}
}

type QAccessibleTextInsertEvent struct {
	QAccessibleTextCursorEvent
}

type QAccessibleTextInsertEvent_ITF interface {
	QAccessibleTextCursorEvent_ITF
	QAccessibleTextInsertEvent_PTR() *QAccessibleTextInsertEvent
}

func (ptr *QAccessibleTextInsertEvent) QAccessibleTextInsertEvent_PTR() *QAccessibleTextInsertEvent {
	return ptr
}

func (ptr *QAccessibleTextInsertEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTextCursorEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QAccessibleTextInsertEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAccessibleTextCursorEvent_PTR().SetPointer(p)
	}
}

func PointerFromQAccessibleTextInsertEvent(ptr QAccessibleTextInsertEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTextInsertEvent_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleTextInsertEventFromPointer(ptr unsafe.Pointer) (n *QAccessibleTextInsertEvent) {
	n = new(QAccessibleTextInsertEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QAccessibleTextInsertEvent) DestroyQAccessibleTextInsertEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQAccessibleTextInsertEvent(object core.QObject_ITF, position int, text string) *QAccessibleTextInsertEvent {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	return NewQAccessibleTextInsertEventFromPointer(C.QAccessibleTextInsertEvent_NewQAccessibleTextInsertEvent(core.PointerFromQObject(object), C.int(int32(position)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
}

func NewQAccessibleTextInsertEvent2(iface QAccessibleInterface_ITF, position int, text string) *QAccessibleTextInsertEvent {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	return NewQAccessibleTextInsertEventFromPointer(C.QAccessibleTextInsertEvent_NewQAccessibleTextInsertEvent2(PointerFromQAccessibleInterface(iface), C.int(int32(position)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
}

func (ptr *QAccessibleTextInsertEvent) ChangePosition() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTextInsertEvent_ChangePosition(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleTextInsertEvent) TextInserted() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextInsertEvent_TextInserted(ptr.Pointer()))
	}
	return ""
}

type QAccessibleTextInterface struct {
	ptr unsafe.Pointer
}

type QAccessibleTextInterface_ITF interface {
	QAccessibleTextInterface_PTR() *QAccessibleTextInterface
}

func (ptr *QAccessibleTextInterface) QAccessibleTextInterface_PTR() *QAccessibleTextInterface {
	return ptr
}

func (ptr *QAccessibleTextInterface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAccessibleTextInterface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAccessibleTextInterface(ptr QAccessibleTextInterface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTextInterface_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleTextInterfaceFromPointer(ptr unsafe.Pointer) (n *QAccessibleTextInterface) {
	n = new(QAccessibleTextInterface)
	n.SetPointer(ptr)
	return
}

//export callbackQAccessibleTextInterface_AddSelection
func callbackQAccessibleTextInterface_AddSelection(ptr unsafe.Pointer, startOffset C.int, endOffset C.int) {
	if signal := qt.GetSignal(ptr, "addSelection"); signal != nil {
		(*(*func(int, int))(signal))(int(int32(startOffset)), int(int32(endOffset)))
	}

}

func (ptr *QAccessibleTextInterface) ConnectAddSelection(f func(startOffset int, endOffset int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "addSelection"); signal != nil {
			f := func(startOffset int, endOffset int) {
				(*(*func(int, int))(signal))(startOffset, endOffset)
				f(startOffset, endOffset)
			}
			qt.ConnectSignal(ptr.Pointer(), "addSelection", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "addSelection", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectAddSelection() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "addSelection")
	}
}

func (ptr *QAccessibleTextInterface) AddSelection(startOffset int, endOffset int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTextInterface_AddSelection(ptr.Pointer(), C.int(int32(startOffset)), C.int(int32(endOffset)))
	}
}

//export callbackQAccessibleTextInterface_CharacterCount
func callbackQAccessibleTextInterface_CharacterCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "characterCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTextInterface) ConnectCharacterCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "characterCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "characterCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "characterCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectCharacterCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "characterCount")
	}
}

func (ptr *QAccessibleTextInterface) CharacterCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTextInterface_CharacterCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTextInterface_CharacterRect
func callbackQAccessibleTextInterface_CharacterRect(ptr unsafe.Pointer, offset C.int) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "characterRect"); signal != nil {
		return core.PointerFromQRect((*(*func(int) *core.QRect)(signal))(int(int32(offset))))
	}

	return core.PointerFromQRect(core.NewQRect())
}

func (ptr *QAccessibleTextInterface) ConnectCharacterRect(f func(offset int) *core.QRect) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "characterRect"); signal != nil {
			f := func(offset int) *core.QRect {
				(*(*func(int) *core.QRect)(signal))(offset)
				return f(offset)
			}
			qt.ConnectSignal(ptr.Pointer(), "characterRect", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "characterRect", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectCharacterRect() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "characterRect")
	}
}

func (ptr *QAccessibleTextInterface) CharacterRect(offset int) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QAccessibleTextInterface_CharacterRect(ptr.Pointer(), C.int(int32(offset))))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleTextInterface_CursorPosition
func callbackQAccessibleTextInterface_CursorPosition(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "cursorPosition"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTextInterface) ConnectCursorPosition(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "cursorPosition"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "cursorPosition", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "cursorPosition", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectCursorPosition() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "cursorPosition")
	}
}

func (ptr *QAccessibleTextInterface) CursorPosition() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTextInterface_CursorPosition(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTextInterface_OffsetAtPoint
func callbackQAccessibleTextInterface_OffsetAtPoint(ptr unsafe.Pointer, point unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "offsetAtPoint"); signal != nil {
		return C.int(int32((*(*func(*core.QPoint) int)(signal))(core.NewQPointFromPointer(point))))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTextInterface) ConnectOffsetAtPoint(f func(point *core.QPoint) int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "offsetAtPoint"); signal != nil {
			f := func(point *core.QPoint) int {
				(*(*func(*core.QPoint) int)(signal))(point)
				return f(point)
			}
			qt.ConnectSignal(ptr.Pointer(), "offsetAtPoint", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "offsetAtPoint", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectOffsetAtPoint() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "offsetAtPoint")
	}
}

func (ptr *QAccessibleTextInterface) OffsetAtPoint(point core.QPoint_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTextInterface_OffsetAtPoint(ptr.Pointer(), core.PointerFromQPoint(point))))
	}
	return 0
}

//export callbackQAccessibleTextInterface_RemoveSelection
func callbackQAccessibleTextInterface_RemoveSelection(ptr unsafe.Pointer, selectionIndex C.int) {
	if signal := qt.GetSignal(ptr, "removeSelection"); signal != nil {
		(*(*func(int))(signal))(int(int32(selectionIndex)))
	}

}

func (ptr *QAccessibleTextInterface) ConnectRemoveSelection(f func(selectionIndex int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "removeSelection"); signal != nil {
			f := func(selectionIndex int) {
				(*(*func(int))(signal))(selectionIndex)
				f(selectionIndex)
			}
			qt.ConnectSignal(ptr.Pointer(), "removeSelection", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "removeSelection", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectRemoveSelection() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "removeSelection")
	}
}

func (ptr *QAccessibleTextInterface) RemoveSelection(selectionIndex int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTextInterface_RemoveSelection(ptr.Pointer(), C.int(int32(selectionIndex)))
	}
}

//export callbackQAccessibleTextInterface_ScrollToSubstring
func callbackQAccessibleTextInterface_ScrollToSubstring(ptr unsafe.Pointer, startIndex C.int, endIndex C.int) {
	if signal := qt.GetSignal(ptr, "scrollToSubstring"); signal != nil {
		(*(*func(int, int))(signal))(int(int32(startIndex)), int(int32(endIndex)))
	}

}

func (ptr *QAccessibleTextInterface) ConnectScrollToSubstring(f func(startIndex int, endIndex int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "scrollToSubstring"); signal != nil {
			f := func(startIndex int, endIndex int) {
				(*(*func(int, int))(signal))(startIndex, endIndex)
				f(startIndex, endIndex)
			}
			qt.ConnectSignal(ptr.Pointer(), "scrollToSubstring", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "scrollToSubstring", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectScrollToSubstring() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "scrollToSubstring")
	}
}

func (ptr *QAccessibleTextInterface) ScrollToSubstring(startIndex int, endIndex int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTextInterface_ScrollToSubstring(ptr.Pointer(), C.int(int32(startIndex)), C.int(int32(endIndex)))
	}
}

//export callbackQAccessibleTextInterface_Selection
func callbackQAccessibleTextInterface_Selection(ptr unsafe.Pointer, selectionIndex C.int, startOffset C.int, endOffset C.int) {
	if signal := qt.GetSignal(ptr, "selection"); signal != nil {
		(*(*func(int, int, int))(signal))(int(int32(selectionIndex)), int(int32(startOffset)), int(int32(endOffset)))
	}

}

func (ptr *QAccessibleTextInterface) ConnectSelection(f func(selectionIndex int, startOffset int, endOffset int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "selection"); signal != nil {
			f := func(selectionIndex int, startOffset int, endOffset int) {
				(*(*func(int, int, int))(signal))(selectionIndex, startOffset, endOffset)
				f(selectionIndex, startOffset, endOffset)
			}
			qt.ConnectSignal(ptr.Pointer(), "selection", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selection", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectSelection() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "selection")
	}
}

func (ptr *QAccessibleTextInterface) Selection(selectionIndex int, startOffset int, endOffset int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTextInterface_Selection(ptr.Pointer(), C.int(int32(selectionIndex)), C.int(int32(startOffset)), C.int(int32(endOffset)))
	}
}

//export callbackQAccessibleTextInterface_SelectionCount
func callbackQAccessibleTextInterface_SelectionCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "selectionCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(0))
}

func (ptr *QAccessibleTextInterface) ConnectSelectionCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "selectionCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "selectionCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selectionCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectSelectionCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "selectionCount")
	}
}

func (ptr *QAccessibleTextInterface) SelectionCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTextInterface_SelectionCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQAccessibleTextInterface_SetCursorPosition
func callbackQAccessibleTextInterface_SetCursorPosition(ptr unsafe.Pointer, position C.int) {
	if signal := qt.GetSignal(ptr, "setCursorPosition"); signal != nil {
		(*(*func(int))(signal))(int(int32(position)))
	}

}

func (ptr *QAccessibleTextInterface) ConnectSetCursorPosition(f func(position int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setCursorPosition"); signal != nil {
			f := func(position int) {
				(*(*func(int))(signal))(position)
				f(position)
			}
			qt.ConnectSignal(ptr.Pointer(), "setCursorPosition", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setCursorPosition", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectSetCursorPosition() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setCursorPosition")
	}
}

func (ptr *QAccessibleTextInterface) SetCursorPosition(position int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTextInterface_SetCursorPosition(ptr.Pointer(), C.int(int32(position)))
	}
}

//export callbackQAccessibleTextInterface_SetSelection
func callbackQAccessibleTextInterface_SetSelection(ptr unsafe.Pointer, selectionIndex C.int, startOffset C.int, endOffset C.int) {
	if signal := qt.GetSignal(ptr, "setSelection"); signal != nil {
		(*(*func(int, int, int))(signal))(int(int32(selectionIndex)), int(int32(startOffset)), int(int32(endOffset)))
	}

}

func (ptr *QAccessibleTextInterface) ConnectSetSelection(f func(selectionIndex int, startOffset int, endOffset int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setSelection"); signal != nil {
			f := func(selectionIndex int, startOffset int, endOffset int) {
				(*(*func(int, int, int))(signal))(selectionIndex, startOffset, endOffset)
				f(selectionIndex, startOffset, endOffset)
			}
			qt.ConnectSignal(ptr.Pointer(), "setSelection", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setSelection", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectSetSelection() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setSelection")
	}
}

func (ptr *QAccessibleTextInterface) SetSelection(selectionIndex int, startOffset int, endOffset int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTextInterface_SetSelection(ptr.Pointer(), C.int(int32(selectionIndex)), C.int(int32(startOffset)), C.int(int32(endOffset)))
	}
}

//export callbackQAccessibleTextInterface_Text
func callbackQAccessibleTextInterface_Text(ptr unsafe.Pointer, startOffset C.int, endOffset C.int) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "text"); signal != nil {
		tempVal := (*(*func(int, int) string)(signal))(int(int32(startOffset)), int(int32(endOffset)))
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := ""
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QAccessibleTextInterface) ConnectText(f func(startOffset int, endOffset int) string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "text"); signal != nil {
			f := func(startOffset int, endOffset int) string {
				(*(*func(int, int) string)(signal))(startOffset, endOffset)
				return f(startOffset, endOffset)
			}
			qt.ConnectSignal(ptr.Pointer(), "text", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "text", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectText() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "text")
	}
}

func (ptr *QAccessibleTextInterface) Text(startOffset int, endOffset int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextInterface_Text(ptr.Pointer(), C.int(int32(startOffset)), C.int(int32(endOffset))))
	}
	return ""
}

//export callbackQAccessibleTextInterface_TextAfterOffset
func callbackQAccessibleTextInterface_TextAfterOffset(ptr unsafe.Pointer, offset C.int, boundaryType C.longlong, startOffset C.int, endOffset C.int) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "textAfterOffset"); signal != nil {
		tempVal := (*(*func(int, QAccessible__TextBoundaryType, int, int) string)(signal))(int(int32(offset)), QAccessible__TextBoundaryType(boundaryType), int(int32(startOffset)), int(int32(endOffset)))
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := NewQAccessibleTextInterfaceFromPointer(ptr).TextAfterOffsetDefault(int(int32(offset)), QAccessible__TextBoundaryType(boundaryType), int(int32(startOffset)), int(int32(endOffset)))
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QAccessibleTextInterface) ConnectTextAfterOffset(f func(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "textAfterOffset"); signal != nil {
			f := func(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string {
				(*(*func(int, QAccessible__TextBoundaryType, int, int) string)(signal))(offset, boundaryType, startOffset, endOffset)
				return f(offset, boundaryType, startOffset, endOffset)
			}
			qt.ConnectSignal(ptr.Pointer(), "textAfterOffset", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "textAfterOffset", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectTextAfterOffset() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "textAfterOffset")
	}
}

func (ptr *QAccessibleTextInterface) TextAfterOffset(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextInterface_TextAfterOffset(ptr.Pointer(), C.int(int32(offset)), C.longlong(boundaryType), C.int(int32(startOffset)), C.int(int32(endOffset))))
	}
	return ""
}

func (ptr *QAccessibleTextInterface) TextAfterOffsetDefault(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextInterface_TextAfterOffsetDefault(ptr.Pointer(), C.int(int32(offset)), C.longlong(boundaryType), C.int(int32(startOffset)), C.int(int32(endOffset))))
	}
	return ""
}

//export callbackQAccessibleTextInterface_TextAtOffset
func callbackQAccessibleTextInterface_TextAtOffset(ptr unsafe.Pointer, offset C.int, boundaryType C.longlong, startOffset C.int, endOffset C.int) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "textAtOffset"); signal != nil {
		tempVal := (*(*func(int, QAccessible__TextBoundaryType, int, int) string)(signal))(int(int32(offset)), QAccessible__TextBoundaryType(boundaryType), int(int32(startOffset)), int(int32(endOffset)))
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := NewQAccessibleTextInterfaceFromPointer(ptr).TextAtOffsetDefault(int(int32(offset)), QAccessible__TextBoundaryType(boundaryType), int(int32(startOffset)), int(int32(endOffset)))
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QAccessibleTextInterface) ConnectTextAtOffset(f func(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "textAtOffset"); signal != nil {
			f := func(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string {
				(*(*func(int, QAccessible__TextBoundaryType, int, int) string)(signal))(offset, boundaryType, startOffset, endOffset)
				return f(offset, boundaryType, startOffset, endOffset)
			}
			qt.ConnectSignal(ptr.Pointer(), "textAtOffset", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "textAtOffset", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectTextAtOffset() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "textAtOffset")
	}
}

func (ptr *QAccessibleTextInterface) TextAtOffset(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextInterface_TextAtOffset(ptr.Pointer(), C.int(int32(offset)), C.longlong(boundaryType), C.int(int32(startOffset)), C.int(int32(endOffset))))
	}
	return ""
}

func (ptr *QAccessibleTextInterface) TextAtOffsetDefault(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextInterface_TextAtOffsetDefault(ptr.Pointer(), C.int(int32(offset)), C.longlong(boundaryType), C.int(int32(startOffset)), C.int(int32(endOffset))))
	}
	return ""
}

//export callbackQAccessibleTextInterface_TextBeforeOffset
func callbackQAccessibleTextInterface_TextBeforeOffset(ptr unsafe.Pointer, offset C.int, boundaryType C.longlong, startOffset C.int, endOffset C.int) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "textBeforeOffset"); signal != nil {
		tempVal := (*(*func(int, QAccessible__TextBoundaryType, int, int) string)(signal))(int(int32(offset)), QAccessible__TextBoundaryType(boundaryType), int(int32(startOffset)), int(int32(endOffset)))
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := NewQAccessibleTextInterfaceFromPointer(ptr).TextBeforeOffsetDefault(int(int32(offset)), QAccessible__TextBoundaryType(boundaryType), int(int32(startOffset)), int(int32(endOffset)))
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QAccessibleTextInterface) ConnectTextBeforeOffset(f func(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "textBeforeOffset"); signal != nil {
			f := func(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string {
				(*(*func(int, QAccessible__TextBoundaryType, int, int) string)(signal))(offset, boundaryType, startOffset, endOffset)
				return f(offset, boundaryType, startOffset, endOffset)
			}
			qt.ConnectSignal(ptr.Pointer(), "textBeforeOffset", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "textBeforeOffset", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectTextBeforeOffset() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "textBeforeOffset")
	}
}

func (ptr *QAccessibleTextInterface) TextBeforeOffset(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextInterface_TextBeforeOffset(ptr.Pointer(), C.int(int32(offset)), C.longlong(boundaryType), C.int(int32(startOffset)), C.int(int32(endOffset))))
	}
	return ""
}

func (ptr *QAccessibleTextInterface) TextBeforeOffsetDefault(offset int, boundaryType QAccessible__TextBoundaryType, startOffset int, endOffset int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextInterface_TextBeforeOffsetDefault(ptr.Pointer(), C.int(int32(offset)), C.longlong(boundaryType), C.int(int32(startOffset)), C.int(int32(endOffset))))
	}
	return ""
}

//export callbackQAccessibleTextInterface_DestroyQAccessibleTextInterface
func callbackQAccessibleTextInterface_DestroyQAccessibleTextInterface(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QAccessibleTextInterface"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessibleTextInterfaceFromPointer(ptr).DestroyQAccessibleTextInterfaceDefault()
	}
}

func (ptr *QAccessibleTextInterface) ConnectDestroyQAccessibleTextInterface(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QAccessibleTextInterface"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleTextInterface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleTextInterface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleTextInterface) DisconnectDestroyQAccessibleTextInterface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QAccessibleTextInterface")
	}
}

func (ptr *QAccessibleTextInterface) DestroyQAccessibleTextInterface() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessibleTextInterface_DestroyQAccessibleTextInterface(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleTextInterface) DestroyQAccessibleTextInterfaceDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessibleTextInterface_DestroyQAccessibleTextInterfaceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QAccessibleTextRemoveEvent struct {
	QAccessibleTextCursorEvent
}

type QAccessibleTextRemoveEvent_ITF interface {
	QAccessibleTextCursorEvent_ITF
	QAccessibleTextRemoveEvent_PTR() *QAccessibleTextRemoveEvent
}

func (ptr *QAccessibleTextRemoveEvent) QAccessibleTextRemoveEvent_PTR() *QAccessibleTextRemoveEvent {
	return ptr
}

func (ptr *QAccessibleTextRemoveEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTextCursorEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QAccessibleTextRemoveEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAccessibleTextCursorEvent_PTR().SetPointer(p)
	}
}

func PointerFromQAccessibleTextRemoveEvent(ptr QAccessibleTextRemoveEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTextRemoveEvent_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleTextRemoveEventFromPointer(ptr unsafe.Pointer) (n *QAccessibleTextRemoveEvent) {
	n = new(QAccessibleTextRemoveEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QAccessibleTextRemoveEvent) DestroyQAccessibleTextRemoveEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQAccessibleTextRemoveEvent(object core.QObject_ITF, position int, text string) *QAccessibleTextRemoveEvent {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	return NewQAccessibleTextRemoveEventFromPointer(C.QAccessibleTextRemoveEvent_NewQAccessibleTextRemoveEvent(core.PointerFromQObject(object), C.int(int32(position)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
}

func NewQAccessibleTextRemoveEvent2(iface QAccessibleInterface_ITF, position int, text string) *QAccessibleTextRemoveEvent {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	return NewQAccessibleTextRemoveEventFromPointer(C.QAccessibleTextRemoveEvent_NewQAccessibleTextRemoveEvent2(PointerFromQAccessibleInterface(iface), C.int(int32(position)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
}

func (ptr *QAccessibleTextRemoveEvent) ChangePosition() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTextRemoveEvent_ChangePosition(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleTextRemoveEvent) TextRemoved() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextRemoveEvent_TextRemoved(ptr.Pointer()))
	}
	return ""
}

type QAccessibleTextSelectionEvent struct {
	QAccessibleTextCursorEvent
}

type QAccessibleTextSelectionEvent_ITF interface {
	QAccessibleTextCursorEvent_ITF
	QAccessibleTextSelectionEvent_PTR() *QAccessibleTextSelectionEvent
}

func (ptr *QAccessibleTextSelectionEvent) QAccessibleTextSelectionEvent_PTR() *QAccessibleTextSelectionEvent {
	return ptr
}

func (ptr *QAccessibleTextSelectionEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTextCursorEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QAccessibleTextSelectionEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAccessibleTextCursorEvent_PTR().SetPointer(p)
	}
}

func PointerFromQAccessibleTextSelectionEvent(ptr QAccessibleTextSelectionEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTextSelectionEvent_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleTextSelectionEventFromPointer(ptr unsafe.Pointer) (n *QAccessibleTextSelectionEvent) {
	n = new(QAccessibleTextSelectionEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QAccessibleTextSelectionEvent) DestroyQAccessibleTextSelectionEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQAccessibleTextSelectionEvent(object core.QObject_ITF, start int, end int) *QAccessibleTextSelectionEvent {
	return NewQAccessibleTextSelectionEventFromPointer(C.QAccessibleTextSelectionEvent_NewQAccessibleTextSelectionEvent(core.PointerFromQObject(object), C.int(int32(start)), C.int(int32(end))))
}

func NewQAccessibleTextSelectionEvent2(iface QAccessibleInterface_ITF, start int, end int) *QAccessibleTextSelectionEvent {
	return NewQAccessibleTextSelectionEventFromPointer(C.QAccessibleTextSelectionEvent_NewQAccessibleTextSelectionEvent2(PointerFromQAccessibleInterface(iface), C.int(int32(start)), C.int(int32(end))))
}

func (ptr *QAccessibleTextSelectionEvent) SelectionEnd() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTextSelectionEvent_SelectionEnd(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleTextSelectionEvent) SelectionStart() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTextSelectionEvent_SelectionStart(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleTextSelectionEvent) SetSelection(start int, end int) {
	if ptr.Pointer() != nil {
		C.QAccessibleTextSelectionEvent_SetSelection(ptr.Pointer(), C.int(int32(start)), C.int(int32(end)))
	}
}

type QAccessibleTextUpdateEvent struct {
	QAccessibleTextCursorEvent
}

type QAccessibleTextUpdateEvent_ITF interface {
	QAccessibleTextCursorEvent_ITF
	QAccessibleTextUpdateEvent_PTR() *QAccessibleTextUpdateEvent
}

func (ptr *QAccessibleTextUpdateEvent) QAccessibleTextUpdateEvent_PTR() *QAccessibleTextUpdateEvent {
	return ptr
}

func (ptr *QAccessibleTextUpdateEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTextCursorEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QAccessibleTextUpdateEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAccessibleTextCursorEvent_PTR().SetPointer(p)
	}
}

func PointerFromQAccessibleTextUpdateEvent(ptr QAccessibleTextUpdateEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleTextUpdateEvent_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleTextUpdateEventFromPointer(ptr unsafe.Pointer) (n *QAccessibleTextUpdateEvent) {
	n = new(QAccessibleTextUpdateEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QAccessibleTextUpdateEvent) DestroyQAccessibleTextUpdateEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQAccessibleTextUpdateEvent(object core.QObject_ITF, position int, oldText string, text string) *QAccessibleTextUpdateEvent {
	var oldTextC *C.char
	if oldText != "" {
		oldTextC = C.CString(oldText)
		defer C.free(unsafe.Pointer(oldTextC))
	}
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	return NewQAccessibleTextUpdateEventFromPointer(C.QAccessibleTextUpdateEvent_NewQAccessibleTextUpdateEvent(core.PointerFromQObject(object), C.int(int32(position)), C.struct_QtGui_PackedString{data: oldTextC, len: C.longlong(len(oldText))}, C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
}

func NewQAccessibleTextUpdateEvent2(iface QAccessibleInterface_ITF, position int, oldText string, text string) *QAccessibleTextUpdateEvent {
	var oldTextC *C.char
	if oldText != "" {
		oldTextC = C.CString(oldText)
		defer C.free(unsafe.Pointer(oldTextC))
	}
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	return NewQAccessibleTextUpdateEventFromPointer(C.QAccessibleTextUpdateEvent_NewQAccessibleTextUpdateEvent2(PointerFromQAccessibleInterface(iface), C.int(int32(position)), C.struct_QtGui_PackedString{data: oldTextC, len: C.longlong(len(oldText))}, C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
}

func (ptr *QAccessibleTextUpdateEvent) ChangePosition() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QAccessibleTextUpdateEvent_ChangePosition(ptr.Pointer())))
	}
	return 0
}

func (ptr *QAccessibleTextUpdateEvent) TextInserted() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextUpdateEvent_TextInserted(ptr.Pointer()))
	}
	return ""
}

func (ptr *QAccessibleTextUpdateEvent) TextRemoved() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QAccessibleTextUpdateEvent_TextRemoved(ptr.Pointer()))
	}
	return ""
}

type QAccessibleValueChangeEvent struct {
	QAccessibleEvent
}

type QAccessibleValueChangeEvent_ITF interface {
	QAccessibleEvent_ITF
	QAccessibleValueChangeEvent_PTR() *QAccessibleValueChangeEvent
}

func (ptr *QAccessibleValueChangeEvent) QAccessibleValueChangeEvent_PTR() *QAccessibleValueChangeEvent {
	return ptr
}

func (ptr *QAccessibleValueChangeEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QAccessibleValueChangeEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAccessibleEvent_PTR().SetPointer(p)
	}
}

func PointerFromQAccessibleValueChangeEvent(ptr QAccessibleValueChangeEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleValueChangeEvent_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleValueChangeEventFromPointer(ptr unsafe.Pointer) (n *QAccessibleValueChangeEvent) {
	n = new(QAccessibleValueChangeEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QAccessibleValueChangeEvent) DestroyQAccessibleValueChangeEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQAccessibleValueChangeEvent(object core.QObject_ITF, value core.QVariant_ITF) *QAccessibleValueChangeEvent {
	return NewQAccessibleValueChangeEventFromPointer(C.QAccessibleValueChangeEvent_NewQAccessibleValueChangeEvent(core.PointerFromQObject(object), core.PointerFromQVariant(value)))
}

func NewQAccessibleValueChangeEvent2(iface QAccessibleInterface_ITF, val core.QVariant_ITF) *QAccessibleValueChangeEvent {
	return NewQAccessibleValueChangeEventFromPointer(C.QAccessibleValueChangeEvent_NewQAccessibleValueChangeEvent2(PointerFromQAccessibleInterface(iface), core.PointerFromQVariant(val)))
}

func (ptr *QAccessibleValueChangeEvent) SetValue(value core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessibleValueChangeEvent_SetValue(ptr.Pointer(), core.PointerFromQVariant(value))
	}
}

func (ptr *QAccessibleValueChangeEvent) Value() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QAccessibleValueChangeEvent_Value(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

type QAccessibleValueInterface struct {
	ptr unsafe.Pointer
}

type QAccessibleValueInterface_ITF interface {
	QAccessibleValueInterface_PTR() *QAccessibleValueInterface
}

func (ptr *QAccessibleValueInterface) QAccessibleValueInterface_PTR() *QAccessibleValueInterface {
	return ptr
}

func (ptr *QAccessibleValueInterface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QAccessibleValueInterface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQAccessibleValueInterface(ptr QAccessibleValueInterface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QAccessibleValueInterface_PTR().Pointer()
	}
	return nil
}

func NewQAccessibleValueInterfaceFromPointer(ptr unsafe.Pointer) (n *QAccessibleValueInterface) {
	n = new(QAccessibleValueInterface)
	n.SetPointer(ptr)
	return
}

//export callbackQAccessibleValueInterface_CurrentValue
func callbackQAccessibleValueInterface_CurrentValue(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "currentValue"); signal != nil {
		return core.PointerFromQVariant((*(*func() *core.QVariant)(signal))())
	}

	return core.PointerFromQVariant(core.NewQVariant())
}

func (ptr *QAccessibleValueInterface) ConnectCurrentValue(f func() *core.QVariant) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "currentValue"); signal != nil {
			f := func() *core.QVariant {
				(*(*func() *core.QVariant)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "currentValue", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "currentValue", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleValueInterface) DisconnectCurrentValue() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "currentValue")
	}
}

func (ptr *QAccessibleValueInterface) CurrentValue() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QAccessibleValueInterface_CurrentValue(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleValueInterface_MaximumValue
func callbackQAccessibleValueInterface_MaximumValue(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "maximumValue"); signal != nil {
		return core.PointerFromQVariant((*(*func() *core.QVariant)(signal))())
	}

	return core.PointerFromQVariant(core.NewQVariant())
}

func (ptr *QAccessibleValueInterface) ConnectMaximumValue(f func() *core.QVariant) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "maximumValue"); signal != nil {
			f := func() *core.QVariant {
				(*(*func() *core.QVariant)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "maximumValue", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "maximumValue", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleValueInterface) DisconnectMaximumValue() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "maximumValue")
	}
}

func (ptr *QAccessibleValueInterface) MaximumValue() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QAccessibleValueInterface_MaximumValue(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleValueInterface_MinimumStepSize
func callbackQAccessibleValueInterface_MinimumStepSize(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "minimumStepSize"); signal != nil {
		return core.PointerFromQVariant((*(*func() *core.QVariant)(signal))())
	}

	return core.PointerFromQVariant(core.NewQVariant())
}

func (ptr *QAccessibleValueInterface) ConnectMinimumStepSize(f func() *core.QVariant) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "minimumStepSize"); signal != nil {
			f := func() *core.QVariant {
				(*(*func() *core.QVariant)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "minimumStepSize", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "minimumStepSize", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleValueInterface) DisconnectMinimumStepSize() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "minimumStepSize")
	}
}

func (ptr *QAccessibleValueInterface) MinimumStepSize() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QAccessibleValueInterface_MinimumStepSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleValueInterface_MinimumValue
func callbackQAccessibleValueInterface_MinimumValue(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "minimumValue"); signal != nil {
		return core.PointerFromQVariant((*(*func() *core.QVariant)(signal))())
	}

	return core.PointerFromQVariant(core.NewQVariant())
}

func (ptr *QAccessibleValueInterface) ConnectMinimumValue(f func() *core.QVariant) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "minimumValue"); signal != nil {
			f := func() *core.QVariant {
				(*(*func() *core.QVariant)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "minimumValue", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "minimumValue", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleValueInterface) DisconnectMinimumValue() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "minimumValue")
	}
}

func (ptr *QAccessibleValueInterface) MinimumValue() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QAccessibleValueInterface_MinimumValue(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

//export callbackQAccessibleValueInterface_SetCurrentValue
func callbackQAccessibleValueInterface_SetCurrentValue(ptr unsafe.Pointer, value unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "setCurrentValue"); signal != nil {
		(*(*func(*core.QVariant))(signal))(core.NewQVariantFromPointer(value))
	}

}

func (ptr *QAccessibleValueInterface) ConnectSetCurrentValue(f func(value *core.QVariant)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setCurrentValue"); signal != nil {
			f := func(value *core.QVariant) {
				(*(*func(*core.QVariant))(signal))(value)
				f(value)
			}
			qt.ConnectSignal(ptr.Pointer(), "setCurrentValue", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setCurrentValue", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleValueInterface) DisconnectSetCurrentValue() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setCurrentValue")
	}
}

func (ptr *QAccessibleValueInterface) SetCurrentValue(value core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QAccessibleValueInterface_SetCurrentValue(ptr.Pointer(), core.PointerFromQVariant(value))
	}
}

//export callbackQAccessibleValueInterface_DestroyQAccessibleValueInterface
func callbackQAccessibleValueInterface_DestroyQAccessibleValueInterface(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QAccessibleValueInterface"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQAccessibleValueInterfaceFromPointer(ptr).DestroyQAccessibleValueInterfaceDefault()
	}
}

func (ptr *QAccessibleValueInterface) ConnectDestroyQAccessibleValueInterface(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QAccessibleValueInterface"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleValueInterface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QAccessibleValueInterface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QAccessibleValueInterface) DisconnectDestroyQAccessibleValueInterface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QAccessibleValueInterface")
	}
}

func (ptr *QAccessibleValueInterface) DestroyQAccessibleValueInterface() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessibleValueInterface_DestroyQAccessibleValueInterface(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QAccessibleValueInterface) DestroyQAccessibleValueInterfaceDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QAccessibleValueInterface_DestroyQAccessibleValueInterfaceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QActionEvent struct {
	core.QEvent
}

type QActionEvent_ITF interface {
	core.QEvent_ITF
	QActionEvent_PTR() *QActionEvent
}

func (ptr *QActionEvent) QActionEvent_PTR() *QActionEvent {
	return ptr
}

func (ptr *QActionEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QActionEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQActionEvent(ptr QActionEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QActionEvent_PTR().Pointer()
	}
	return nil
}

func NewQActionEventFromPointer(ptr unsafe.Pointer) (n *QActionEvent) {
	n = new(QActionEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QActionEvent) DestroyQActionEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQActionEvent(ty int, action unsafe.Pointer, before unsafe.Pointer) *QActionEvent {
	tmpValue := NewQActionEventFromPointer(C.QActionEvent_NewQActionEvent(C.int(int32(ty)), action, before))
	qt.SetFinalizer(tmpValue, (*QActionEvent).DestroyQActionEvent)
	return tmpValue
}

func (ptr *QActionEvent) Action() unsafe.Pointer {
	if ptr.Pointer() != nil {
		return unsafe.Pointer(C.QActionEvent_Action(ptr.Pointer()))
	}
	return nil
}

func (ptr *QActionEvent) Before() unsafe.Pointer {
	if ptr.Pointer() != nil {
		return unsafe.Pointer(C.QActionEvent_Before(ptr.Pointer()))
	}
	return nil
}

type QApplicationStateChangeEvent struct {
	core.QEvent
}

type QApplicationStateChangeEvent_ITF interface {
	core.QEvent_ITF
	QApplicationStateChangeEvent_PTR() *QApplicationStateChangeEvent
}

func (ptr *QApplicationStateChangeEvent) QApplicationStateChangeEvent_PTR() *QApplicationStateChangeEvent {
	return ptr
}

func (ptr *QApplicationStateChangeEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QApplicationStateChangeEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQApplicationStateChangeEvent(ptr QApplicationStateChangeEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QApplicationStateChangeEvent_PTR().Pointer()
	}
	return nil
}

func NewQApplicationStateChangeEventFromPointer(ptr unsafe.Pointer) (n *QApplicationStateChangeEvent) {
	n = new(QApplicationStateChangeEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QApplicationStateChangeEvent) DestroyQApplicationStateChangeEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQApplicationStateChangeEvent(applicationState core.Qt__ApplicationState) *QApplicationStateChangeEvent {
	tmpValue := NewQApplicationStateChangeEventFromPointer(C.QApplicationStateChangeEvent_NewQApplicationStateChangeEvent(C.longlong(applicationState)))
	qt.SetFinalizer(tmpValue, (*QApplicationStateChangeEvent).DestroyQApplicationStateChangeEvent)
	return tmpValue
}

func (ptr *QApplicationStateChangeEvent) ApplicationState() core.Qt__ApplicationState {
	if ptr.Pointer() != nil {
		return core.Qt__ApplicationState(C.QApplicationStateChangeEvent_ApplicationState(ptr.Pointer()))
	}
	return 0
}

type QBackingStore struct {
	ptr unsafe.Pointer
}

type QBackingStore_ITF interface {
	QBackingStore_PTR() *QBackingStore
}

func (ptr *QBackingStore) QBackingStore_PTR() *QBackingStore {
	return ptr
}

func (ptr *QBackingStore) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QBackingStore) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQBackingStore(ptr QBackingStore_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QBackingStore_PTR().Pointer()
	}
	return nil
}

func NewQBackingStoreFromPointer(ptr unsafe.Pointer) (n *QBackingStore) {
	n = new(QBackingStore)
	n.SetPointer(ptr)
	return
}
func NewQBackingStore(window QWindow_ITF) *QBackingStore {
	tmpValue := NewQBackingStoreFromPointer(C.QBackingStore_NewQBackingStore(PointerFromQWindow(window)))
	qt.SetFinalizer(tmpValue, (*QBackingStore).DestroyQBackingStore)
	return tmpValue
}

func (ptr *QBackingStore) BeginPaint(region QRegion_ITF) {
	if ptr.Pointer() != nil {
		C.QBackingStore_BeginPaint(ptr.Pointer(), PointerFromQRegion(region))
	}
}

func (ptr *QBackingStore) EndPaint() {
	if ptr.Pointer() != nil {
		C.QBackingStore_EndPaint(ptr.Pointer())
	}
}

func (ptr *QBackingStore) Flush(region QRegion_ITF, window QWindow_ITF, offset core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QBackingStore_Flush(ptr.Pointer(), PointerFromQRegion(region), PointerFromQWindow(window), core.PointerFromQPoint(offset))
	}
}

func (ptr *QBackingStore) HasStaticContents() bool {
	if ptr.Pointer() != nil {
		return int8(C.QBackingStore_HasStaticContents(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QBackingStore) PaintDevice() *QPaintDevice {
	if ptr.Pointer() != nil {
		return NewQPaintDeviceFromPointer(C.QBackingStore_PaintDevice(ptr.Pointer()))
	}
	return nil
}

func (ptr *QBackingStore) Resize(size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QBackingStore_Resize(ptr.Pointer(), core.PointerFromQSize(size))
	}
}

func (ptr *QBackingStore) Scroll(area QRegion_ITF, dx int, dy int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QBackingStore_Scroll(ptr.Pointer(), PointerFromQRegion(area), C.int(int32(dx)), C.int(int32(dy)))) != 0
	}
	return false
}

func (ptr *QBackingStore) SetStaticContents(region QRegion_ITF) {
	if ptr.Pointer() != nil {
		C.QBackingStore_SetStaticContents(ptr.Pointer(), PointerFromQRegion(region))
	}
}

func (ptr *QBackingStore) Size() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QBackingStore_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QBackingStore) StaticContents() *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QBackingStore_StaticContents(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QBackingStore) Window() *QWindow {
	if ptr.Pointer() != nil {
		tmpValue := NewQWindowFromPointer(C.QBackingStore_Window(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QBackingStore) DestroyQBackingStore() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QBackingStore_DestroyQBackingStore(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QBitmap struct {
	QPixmap
}

type QBitmap_ITF interface {
	QPixmap_ITF
	QBitmap_PTR() *QBitmap
}

func (ptr *QBitmap) QBitmap_PTR() *QBitmap {
	return ptr
}

func (ptr *QBitmap) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QPixmap_PTR().Pointer()
	}
	return nil
}

func (ptr *QBitmap) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QPixmap_PTR().SetPointer(p)
	}
}

func PointerFromQBitmap(ptr QBitmap_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QBitmap_PTR().Pointer()
	}
	return nil
}

func NewQBitmapFromPointer(ptr unsafe.Pointer) (n *QBitmap) {
	n = new(QBitmap)
	n.SetPointer(ptr)
	return
}
func NewQBitmap() *QBitmap {
	return NewQBitmapFromPointer(C.QBitmap_NewQBitmap())
}

func NewQBitmap2(pixmap QPixmap_ITF) *QBitmap {
	return NewQBitmapFromPointer(C.QBitmap_NewQBitmap2(PointerFromQPixmap(pixmap)))
}

func NewQBitmap3(width int, height int) *QBitmap {
	return NewQBitmapFromPointer(C.QBitmap_NewQBitmap3(C.int(int32(width)), C.int(int32(height))))
}

func NewQBitmap4(size core.QSize_ITF) *QBitmap {
	return NewQBitmapFromPointer(C.QBitmap_NewQBitmap4(core.PointerFromQSize(size)))
}

func NewQBitmap5(fileName string, format string) *QBitmap {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	var formatC *C.char
	if format != "" {
		formatC = C.CString(format)
		defer C.free(unsafe.Pointer(formatC))
	}
	return NewQBitmapFromPointer(C.QBitmap_NewQBitmap5(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, formatC))
}

func (ptr *QBitmap) Clear() {
	if ptr.Pointer() != nil {
		C.QBitmap_Clear(ptr.Pointer())
	}
}

func QBitmap_FromData(size core.QSize_ITF, bits string, monoFormat QImage__Format) *QBitmap {
	var bitsC *C.char
	if bits != "" {
		bitsC = C.CString(bits)
		defer C.free(unsafe.Pointer(bitsC))
	}
	tmpValue := NewQBitmapFromPointer(C.QBitmap_QBitmap_FromData(core.PointerFromQSize(size), bitsC, C.longlong(monoFormat)))
	qt.SetFinalizer(tmpValue, (*QBitmap).DestroyQBitmap)
	return tmpValue
}

func (ptr *QBitmap) FromData(size core.QSize_ITF, bits string, monoFormat QImage__Format) *QBitmap {
	var bitsC *C.char
	if bits != "" {
		bitsC = C.CString(bits)
		defer C.free(unsafe.Pointer(bitsC))
	}
	tmpValue := NewQBitmapFromPointer(C.QBitmap_QBitmap_FromData(core.PointerFromQSize(size), bitsC, C.longlong(monoFormat)))
	qt.SetFinalizer(tmpValue, (*QBitmap).DestroyQBitmap)
	return tmpValue
}

func QBitmap_FromImage(image QImage_ITF, flags core.Qt__ImageConversionFlag) *QBitmap {
	tmpValue := NewQBitmapFromPointer(C.QBitmap_QBitmap_FromImage(PointerFromQImage(image), C.longlong(flags)))
	qt.SetFinalizer(tmpValue, (*QBitmap).DestroyQBitmap)
	return tmpValue
}

func (ptr *QBitmap) FromImage(image QImage_ITF, flags core.Qt__ImageConversionFlag) *QBitmap {
	tmpValue := NewQBitmapFromPointer(C.QBitmap_QBitmap_FromImage(PointerFromQImage(image), C.longlong(flags)))
	qt.SetFinalizer(tmpValue, (*QBitmap).DestroyQBitmap)
	return tmpValue
}

func QBitmap_FromImage2(image QImage_ITF, flags core.Qt__ImageConversionFlag) *QBitmap {
	tmpValue := NewQBitmapFromPointer(C.QBitmap_QBitmap_FromImage2(PointerFromQImage(image), C.longlong(flags)))
	qt.SetFinalizer(tmpValue, (*QBitmap).DestroyQBitmap)
	return tmpValue
}

func (ptr *QBitmap) FromImage2(image QImage_ITF, flags core.Qt__ImageConversionFlag) *QBitmap {
	tmpValue := NewQBitmapFromPointer(C.QBitmap_QBitmap_FromImage2(PointerFromQImage(image), C.longlong(flags)))
	qt.SetFinalizer(tmpValue, (*QBitmap).DestroyQBitmap)
	return tmpValue
}

func (ptr *QBitmap) Swap(other QBitmap_ITF) {
	if ptr.Pointer() != nil {
		C.QBitmap_Swap(ptr.Pointer(), PointerFromQBitmap(other))
	}
}

func (ptr *QBitmap) Transformed(matrix QTransform_ITF) *QBitmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQBitmapFromPointer(C.QBitmap_Transformed(ptr.Pointer(), PointerFromQTransform(matrix)))
		qt.SetFinalizer(tmpValue, (*QBitmap).DestroyQBitmap)
		return tmpValue
	}
	return nil
}

//export callbackQBitmap_DestroyQBitmap
func callbackQBitmap_DestroyQBitmap(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QBitmap"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQBitmapFromPointer(ptr).DestroyQBitmapDefault()
	}
}

func (ptr *QBitmap) ConnectDestroyQBitmap(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QBitmap"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QBitmap", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QBitmap", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QBitmap) DisconnectDestroyQBitmap() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QBitmap")
	}
}

func (ptr *QBitmap) DestroyQBitmap() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QBitmap_DestroyQBitmap(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QBitmap) DestroyQBitmapDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QBitmap_DestroyQBitmapDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QBrush struct {
	ptr unsafe.Pointer
}

type QBrush_ITF interface {
	QBrush_PTR() *QBrush
}

func (ptr *QBrush) QBrush_PTR() *QBrush {
	return ptr
}

func (ptr *QBrush) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QBrush) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQBrush(ptr QBrush_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QBrush_PTR().Pointer()
	}
	return nil
}

func NewQBrushFromPointer(ptr unsafe.Pointer) (n *QBrush) {
	n = new(QBrush)
	n.SetPointer(ptr)
	return
}
func NewQBrush() *QBrush {
	tmpValue := NewQBrushFromPointer(C.QBrush_NewQBrush())
	qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
	return tmpValue
}

func NewQBrush2(style core.Qt__BrushStyle) *QBrush {
	tmpValue := NewQBrushFromPointer(C.QBrush_NewQBrush2(C.longlong(style)))
	qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
	return tmpValue
}

func NewQBrush3(color QColor_ITF, style core.Qt__BrushStyle) *QBrush {
	tmpValue := NewQBrushFromPointer(C.QBrush_NewQBrush3(PointerFromQColor(color), C.longlong(style)))
	qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
	return tmpValue
}

func NewQBrush4(color core.Qt__GlobalColor, style core.Qt__BrushStyle) *QBrush {
	tmpValue := NewQBrushFromPointer(C.QBrush_NewQBrush4(C.longlong(color), C.longlong(style)))
	qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
	return tmpValue
}

func NewQBrush5(color QColor_ITF, pixmap QPixmap_ITF) *QBrush {
	tmpValue := NewQBrushFromPointer(C.QBrush_NewQBrush5(PointerFromQColor(color), PointerFromQPixmap(pixmap)))
	qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
	return tmpValue
}

func NewQBrush6(color core.Qt__GlobalColor, pixmap QPixmap_ITF) *QBrush {
	tmpValue := NewQBrushFromPointer(C.QBrush_NewQBrush6(C.longlong(color), PointerFromQPixmap(pixmap)))
	qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
	return tmpValue
}

func NewQBrush7(pixmap QPixmap_ITF) *QBrush {
	tmpValue := NewQBrushFromPointer(C.QBrush_NewQBrush7(PointerFromQPixmap(pixmap)))
	qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
	return tmpValue
}

func NewQBrush8(image QImage_ITF) *QBrush {
	tmpValue := NewQBrushFromPointer(C.QBrush_NewQBrush8(PointerFromQImage(image)))
	qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
	return tmpValue
}

func NewQBrush9(other QBrush_ITF) *QBrush {
	tmpValue := NewQBrushFromPointer(C.QBrush_NewQBrush9(PointerFromQBrush(other)))
	qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
	return tmpValue
}

func NewQBrush10(gradient QGradient_ITF) *QBrush {
	tmpValue := NewQBrushFromPointer(C.QBrush_NewQBrush10(PointerFromQGradient(gradient)))
	qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
	return tmpValue
}

func (ptr *QBrush) Color() *QColor {
	if ptr.Pointer() != nil {
		return NewQColorFromPointer(C.QBrush_Color(ptr.Pointer()))
	}
	return nil
}

func (ptr *QBrush) Gradient() *QGradient {
	if ptr.Pointer() != nil {
		return NewQGradientFromPointer(C.QBrush_Gradient(ptr.Pointer()))
	}
	return nil
}

func (ptr *QBrush) IsOpaque() bool {
	if ptr.Pointer() != nil {
		return int8(C.QBrush_IsOpaque(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QBrush) Matrix() *QMatrix {
	if ptr.Pointer() != nil {
		return NewQMatrixFromPointer(C.QBrush_Matrix(ptr.Pointer()))
	}
	return nil
}

func (ptr *QBrush) SetColor(color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QBrush_SetColor(ptr.Pointer(), PointerFromQColor(color))
	}
}

func (ptr *QBrush) SetColor2(color core.Qt__GlobalColor) {
	if ptr.Pointer() != nil {
		C.QBrush_SetColor2(ptr.Pointer(), C.longlong(color))
	}
}

func (ptr *QBrush) SetMatrix(matrix QMatrix_ITF) {
	if ptr.Pointer() != nil {
		C.QBrush_SetMatrix(ptr.Pointer(), PointerFromQMatrix(matrix))
	}
}

func (ptr *QBrush) SetStyle(style core.Qt__BrushStyle) {
	if ptr.Pointer() != nil {
		C.QBrush_SetStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QBrush) SetTexture(pixmap QPixmap_ITF) {
	if ptr.Pointer() != nil {
		C.QBrush_SetTexture(ptr.Pointer(), PointerFromQPixmap(pixmap))
	}
}

func (ptr *QBrush) SetTextureImage(image QImage_ITF) {
	if ptr.Pointer() != nil {
		C.QBrush_SetTextureImage(ptr.Pointer(), PointerFromQImage(image))
	}
}

func (ptr *QBrush) SetTransform(matrix QTransform_ITF) {
	if ptr.Pointer() != nil {
		C.QBrush_SetTransform(ptr.Pointer(), PointerFromQTransform(matrix))
	}
}

func (ptr *QBrush) Style() core.Qt__BrushStyle {
	if ptr.Pointer() != nil {
		return core.Qt__BrushStyle(C.QBrush_Style(ptr.Pointer()))
	}
	return 0
}

func (ptr *QBrush) Swap(other QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QBrush_Swap(ptr.Pointer(), PointerFromQBrush(other))
	}
}

func (ptr *QBrush) Texture() *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QBrush_Texture(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QBrush) TextureImage() *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QBrush_TextureImage(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QBrush) Transform() *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QBrush_Transform(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QBrush) DestroyQBrush() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QBrush_DestroyQBrush(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QBrush) ToVariant() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QBrush_ToVariant(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

type QClipboard struct {
	core.QObject
}

type QClipboard_ITF interface {
	core.QObject_ITF
	QClipboard_PTR() *QClipboard
}

func (ptr *QClipboard) QClipboard_PTR() *QClipboard {
	return ptr
}

func (ptr *QClipboard) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QClipboard) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQClipboard(ptr QClipboard_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QClipboard_PTR().Pointer()
	}
	return nil
}

func NewQClipboardFromPointer(ptr unsafe.Pointer) (n *QClipboard) {
	n = new(QClipboard)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QClipboard__Mode
//QClipboard::Mode
type QClipboard__Mode int64

const (
	QClipboard__Clipboard  QClipboard__Mode = QClipboard__Mode(0)
	QClipboard__Selection  QClipboard__Mode = QClipboard__Mode(1)
	QClipboard__FindBuffer QClipboard__Mode = QClipboard__Mode(2)
	QClipboard__LastMode   QClipboard__Mode = QClipboard__Mode(QClipboard__FindBuffer)
)

//export callbackQClipboard_Changed
func callbackQClipboard_Changed(ptr unsafe.Pointer, mode C.longlong) {
	if signal := qt.GetSignal(ptr, "changed"); signal != nil {
		(*(*func(QClipboard__Mode))(signal))(QClipboard__Mode(mode))
	}

}

func (ptr *QClipboard) ConnectChanged(f func(mode QClipboard__Mode)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "changed") {
			C.QClipboard_ConnectChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "changed")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "changed"); signal != nil {
			f := func(mode QClipboard__Mode) {
				(*(*func(QClipboard__Mode))(signal))(mode)
				f(mode)
			}
			qt.ConnectSignal(ptr.Pointer(), "changed", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "changed", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QClipboard) DisconnectChanged() {
	if ptr.Pointer() != nil {
		C.QClipboard_DisconnectChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "changed")
	}
}

func (ptr *QClipboard) Changed(mode QClipboard__Mode) {
	if ptr.Pointer() != nil {
		C.QClipboard_Changed(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QClipboard) Clear(mode QClipboard__Mode) {
	if ptr.Pointer() != nil {
		C.QClipboard_Clear(ptr.Pointer(), C.longlong(mode))
	}
}

//export callbackQClipboard_DataChanged
func callbackQClipboard_DataChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "dataChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QClipboard) ConnectDataChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "dataChanged") {
			C.QClipboard_ConnectDataChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "dataChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "dataChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "dataChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "dataChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QClipboard) DisconnectDataChanged() {
	if ptr.Pointer() != nil {
		C.QClipboard_DisconnectDataChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "dataChanged")
	}
}

func (ptr *QClipboard) DataChanged() {
	if ptr.Pointer() != nil {
		C.QClipboard_DataChanged(ptr.Pointer())
	}
}

//export callbackQClipboard_FindBufferChanged
func callbackQClipboard_FindBufferChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "findBufferChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QClipboard) ConnectFindBufferChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "findBufferChanged") {
			C.QClipboard_ConnectFindBufferChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "findBufferChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "findBufferChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "findBufferChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "findBufferChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QClipboard) DisconnectFindBufferChanged() {
	if ptr.Pointer() != nil {
		C.QClipboard_DisconnectFindBufferChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "findBufferChanged")
	}
}

func (ptr *QClipboard) FindBufferChanged() {
	if ptr.Pointer() != nil {
		C.QClipboard_FindBufferChanged(ptr.Pointer())
	}
}

func (ptr *QClipboard) Image(mode QClipboard__Mode) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QClipboard_Image(ptr.Pointer(), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QClipboard) MimeData(mode QClipboard__Mode) *core.QMimeData {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMimeDataFromPointer(C.QClipboard_MimeData(ptr.Pointer(), C.longlong(mode)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QClipboard) OwnsClipboard() bool {
	if ptr.Pointer() != nil {
		return int8(C.QClipboard_OwnsClipboard(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QClipboard) OwnsFindBuffer() bool {
	if ptr.Pointer() != nil {
		return int8(C.QClipboard_OwnsFindBuffer(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QClipboard) OwnsSelection() bool {
	if ptr.Pointer() != nil {
		return int8(C.QClipboard_OwnsSelection(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QClipboard) Pixmap(mode QClipboard__Mode) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QClipboard_Pixmap(ptr.Pointer(), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

//export callbackQClipboard_SelectionChanged
func callbackQClipboard_SelectionChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "selectionChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QClipboard) ConnectSelectionChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "selectionChanged") {
			C.QClipboard_ConnectSelectionChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "selectionChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "selectionChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "selectionChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "selectionChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QClipboard) DisconnectSelectionChanged() {
	if ptr.Pointer() != nil {
		C.QClipboard_DisconnectSelectionChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "selectionChanged")
	}
}

func (ptr *QClipboard) SelectionChanged() {
	if ptr.Pointer() != nil {
		C.QClipboard_SelectionChanged(ptr.Pointer())
	}
}

func (ptr *QClipboard) SetImage(image QImage_ITF, mode QClipboard__Mode) {
	if ptr.Pointer() != nil {
		C.QClipboard_SetImage(ptr.Pointer(), PointerFromQImage(image), C.longlong(mode))
	}
}

func (ptr *QClipboard) SetMimeData(src core.QMimeData_ITF, mode QClipboard__Mode) {
	if ptr.Pointer() != nil {
		C.QClipboard_SetMimeData(ptr.Pointer(), core.PointerFromQMimeData(src), C.longlong(mode))
	}
}

func (ptr *QClipboard) SetPixmap(pixmap QPixmap_ITF, mode QClipboard__Mode) {
	if ptr.Pointer() != nil {
		C.QClipboard_SetPixmap(ptr.Pointer(), PointerFromQPixmap(pixmap), C.longlong(mode))
	}
}

func (ptr *QClipboard) SetText(text string, mode QClipboard__Mode) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QClipboard_SetText(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.longlong(mode))
	}
}

func (ptr *QClipboard) SupportsFindBuffer() bool {
	if ptr.Pointer() != nil {
		return int8(C.QClipboard_SupportsFindBuffer(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QClipboard) SupportsSelection() bool {
	if ptr.Pointer() != nil {
		return int8(C.QClipboard_SupportsSelection(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QClipboard) Text(mode QClipboard__Mode) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QClipboard_Text(ptr.Pointer(), C.longlong(mode)))
	}
	return ""
}

func (ptr *QClipboard) Text2(subtype string, mode QClipboard__Mode) string {
	if ptr.Pointer() != nil {
		var subtypeC *C.char
		if subtype != "" {
			subtypeC = C.CString(subtype)
			defer C.free(unsafe.Pointer(subtypeC))
		}
		return cGoUnpackString(C.QClipboard_Text2(ptr.Pointer(), C.struct_QtGui_PackedString{data: subtypeC, len: C.longlong(len(subtype))}, C.longlong(mode)))
	}
	return ""
}

func (ptr *QClipboard) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QClipboard___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QClipboard) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QClipboard___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QClipboard) __children_newList() unsafe.Pointer {
	return C.QClipboard___children_newList(ptr.Pointer())
}

func (ptr *QClipboard) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QClipboard___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QClipboard) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QClipboard___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QClipboard) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QClipboard___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QClipboard) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QClipboard___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QClipboard) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QClipboard___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QClipboard) __findChildren_newList() unsafe.Pointer {
	return C.QClipboard___findChildren_newList(ptr.Pointer())
}

func (ptr *QClipboard) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QClipboard___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QClipboard) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QClipboard___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QClipboard) __findChildren_newList3() unsafe.Pointer {
	return C.QClipboard___findChildren_newList3(ptr.Pointer())
}

//export callbackQClipboard_ChildEvent
func callbackQClipboard_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQClipboardFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QClipboard) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QClipboard_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQClipboard_ConnectNotify
func callbackQClipboard_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQClipboardFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QClipboard) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QClipboard_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQClipboard_CustomEvent
func callbackQClipboard_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQClipboardFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QClipboard) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QClipboard_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQClipboard_DeleteLater
func callbackQClipboard_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQClipboardFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QClipboard) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QClipboard_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQClipboard_Destroyed
func callbackQClipboard_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQClipboard_DisconnectNotify
func callbackQClipboard_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQClipboardFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QClipboard) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QClipboard_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQClipboard_Event
func callbackQClipboard_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQClipboardFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QClipboard) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QClipboard_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQClipboard_EventFilter
func callbackQClipboard_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQClipboardFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QClipboard) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QClipboard_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQClipboard_MetaObject
func callbackQClipboard_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQClipboardFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QClipboard) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QClipboard_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQClipboard_ObjectNameChanged
func callbackQClipboard_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQClipboard_TimerEvent
func callbackQClipboard_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQClipboardFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QClipboard) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QClipboard_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QCloseEvent struct {
	core.QEvent
}

type QCloseEvent_ITF interface {
	core.QEvent_ITF
	QCloseEvent_PTR() *QCloseEvent
}

func (ptr *QCloseEvent) QCloseEvent_PTR() *QCloseEvent {
	return ptr
}

func (ptr *QCloseEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QCloseEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQCloseEvent(ptr QCloseEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QCloseEvent_PTR().Pointer()
	}
	return nil
}

func NewQCloseEventFromPointer(ptr unsafe.Pointer) (n *QCloseEvent) {
	n = new(QCloseEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QCloseEvent) DestroyQCloseEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQCloseEvent() *QCloseEvent {
	tmpValue := NewQCloseEventFromPointer(C.QCloseEvent_NewQCloseEvent())
	qt.SetFinalizer(tmpValue, (*QCloseEvent).DestroyQCloseEvent)
	return tmpValue
}

type QColor struct {
	ptr unsafe.Pointer
}

type QColor_ITF interface {
	QColor_PTR() *QColor
}

func (ptr *QColor) QColor_PTR() *QColor {
	return ptr
}

func (ptr *QColor) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QColor) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQColor(ptr QColor_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QColor_PTR().Pointer()
	}
	return nil
}

func NewQColorFromPointer(ptr unsafe.Pointer) (n *QColor) {
	n = new(QColor)
	n.SetPointer(ptr)
	return
}
func (ptr *QColor) DestroyQColor() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QColor__Spec
//QColor::Spec
type QColor__Spec int64

const (
	QColor__Invalid QColor__Spec = QColor__Spec(0)
	QColor__Rgb     QColor__Spec = QColor__Spec(1)
	QColor__Hsv     QColor__Spec = QColor__Spec(2)
	QColor__Cmyk    QColor__Spec = QColor__Spec(3)
	QColor__Hsl     QColor__Spec = QColor__Spec(4)
)

//go:generate stringer -type=QColor__NameFormat
//QColor::NameFormat
type QColor__NameFormat int64

const (
	QColor__HexRgb  QColor__NameFormat = QColor__NameFormat(0)
	QColor__HexArgb QColor__NameFormat = QColor__NameFormat(1)
)

func NewQColor() *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_NewQColor())
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func NewQColor2(color core.Qt__GlobalColor) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_NewQColor2(C.longlong(color)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func NewQColor3(r int, g int, b int, a int) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_NewQColor3(C.int(int32(r)), C.int(int32(g)), C.int(int32(b)), C.int(int32(a))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func NewQColor4(color uint) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_NewQColor4(C.uint(uint32(color))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func NewQColor5(rgba64 QRgba64_ITF) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_NewQColor5(PointerFromQRgba64(rgba64)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func NewQColor6(name string) *QColor {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	tmpValue := NewQColorFromPointer(C.QColor_NewQColor6(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func NewQColor8(name string) *QColor {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	tmpValue := NewQColorFromPointer(C.QColor_NewQColor8(nameC))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func NewQColor9(name core.QLatin1String_ITF) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_NewQColor9(core.PointerFromQLatin1String(name)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) Alpha() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Alpha(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) AlphaF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_AlphaF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) Black() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Black(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) BlackF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_BlackF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) Blue() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Blue(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) BlueF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_BlueF(ptr.Pointer()))
	}
	return 0
}

func QColor_ColorNames() []string {
	return unpackStringList(cGoUnpackString(C.QColor_QColor_ColorNames()))
}

func (ptr *QColor) ColorNames() []string {
	return unpackStringList(cGoUnpackString(C.QColor_QColor_ColorNames()))
}

func (ptr *QColor) ConvertTo(colorSpec QColor__Spec) *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QColor_ConvertTo(ptr.Pointer(), C.longlong(colorSpec)))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QColor) Cyan() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Cyan(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) CyanF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_CyanF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) Darker(factor int) *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QColor_Darker(ptr.Pointer(), C.int(int32(factor))))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func QColor_FromCmyk(c int, m int, y int, k int, a int) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromCmyk(C.int(int32(c)), C.int(int32(m)), C.int(int32(y)), C.int(int32(k)), C.int(int32(a))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromCmyk(c int, m int, y int, k int, a int) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromCmyk(C.int(int32(c)), C.int(int32(m)), C.int(int32(y)), C.int(int32(k)), C.int(int32(a))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromCmykF(c float64, m float64, y float64, k float64, a float64) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromCmykF(C.double(c), C.double(m), C.double(y), C.double(k), C.double(a)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromCmykF(c float64, m float64, y float64, k float64, a float64) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromCmykF(C.double(c), C.double(m), C.double(y), C.double(k), C.double(a)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromHsl(h int, s int, l int, a int) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromHsl(C.int(int32(h)), C.int(int32(s)), C.int(int32(l)), C.int(int32(a))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromHsl(h int, s int, l int, a int) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromHsl(C.int(int32(h)), C.int(int32(s)), C.int(int32(l)), C.int(int32(a))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromHslF(h float64, s float64, l float64, a float64) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromHslF(C.double(h), C.double(s), C.double(l), C.double(a)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromHslF(h float64, s float64, l float64, a float64) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromHslF(C.double(h), C.double(s), C.double(l), C.double(a)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromHsv(h int, s int, v int, a int) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromHsv(C.int(int32(h)), C.int(int32(s)), C.int(int32(v)), C.int(int32(a))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromHsv(h int, s int, v int, a int) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromHsv(C.int(int32(h)), C.int(int32(s)), C.int(int32(v)), C.int(int32(a))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromHsvF(h float64, s float64, v float64, a float64) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromHsvF(C.double(h), C.double(s), C.double(v), C.double(a)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromHsvF(h float64, s float64, v float64, a float64) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromHsvF(C.double(h), C.double(s), C.double(v), C.double(a)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromRgb(rgb uint) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgb(C.uint(uint32(rgb))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromRgb(rgb uint) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgb(C.uint(uint32(rgb))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromRgb2(r int, g int, b int, a int) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgb2(C.int(int32(r)), C.int(int32(g)), C.int(int32(b)), C.int(int32(a))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromRgb2(r int, g int, b int, a int) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgb2(C.int(int32(r)), C.int(int32(g)), C.int(int32(b)), C.int(int32(a))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromRgbF(r float64, g float64, b float64, a float64) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgbF(C.double(r), C.double(g), C.double(b), C.double(a)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromRgbF(r float64, g float64, b float64, a float64) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgbF(C.double(r), C.double(g), C.double(b), C.double(a)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromRgba(rgba uint) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgba(C.uint(uint32(rgba))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromRgba(rgba uint) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgba(C.uint(uint32(rgba))))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromRgba64(r uint16, g uint16, b uint16, a uint16) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgba64(C.ushort(r), C.ushort(g), C.ushort(b), C.ushort(a)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromRgba64(r uint16, g uint16, b uint16, a uint16) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgba64(C.ushort(r), C.ushort(g), C.ushort(b), C.ushort(a)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func QColor_FromRgba642(rgba64 QRgba64_ITF) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgba642(PointerFromQRgba64(rgba64)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) FromRgba642(rgba64 QRgba64_ITF) *QColor {
	tmpValue := NewQColorFromPointer(C.QColor_QColor_FromRgba642(PointerFromQRgba64(rgba64)))
	qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
	return tmpValue
}

func (ptr *QColor) GetCmyk2(c int, m int, y int, k int, a int) {
	if ptr.Pointer() != nil {
		C.QColor_GetCmyk2(ptr.Pointer(), C.int(int32(c)), C.int(int32(m)), C.int(int32(y)), C.int(int32(k)), C.int(int32(a)))
	}
}

func (ptr *QColor) GetCmykF2(c float64, m float64, y float64, k float64, a float64) {
	if ptr.Pointer() != nil {
		C.QColor_GetCmykF2(ptr.Pointer(), C.double(c), C.double(m), C.double(y), C.double(k), C.double(a))
	}
}

func (ptr *QColor) GetHsl(h int, s int, l int, a int) {
	if ptr.Pointer() != nil {
		C.QColor_GetHsl(ptr.Pointer(), C.int(int32(h)), C.int(int32(s)), C.int(int32(l)), C.int(int32(a)))
	}
}

func (ptr *QColor) GetHslF(h float64, s float64, l float64, a float64) {
	if ptr.Pointer() != nil {
		C.QColor_GetHslF(ptr.Pointer(), C.double(h), C.double(s), C.double(l), C.double(a))
	}
}

func (ptr *QColor) GetHsv(h int, s int, v int, a int) {
	if ptr.Pointer() != nil {
		C.QColor_GetHsv(ptr.Pointer(), C.int(int32(h)), C.int(int32(s)), C.int(int32(v)), C.int(int32(a)))
	}
}

func (ptr *QColor) GetHsvF(h float64, s float64, v float64, a float64) {
	if ptr.Pointer() != nil {
		C.QColor_GetHsvF(ptr.Pointer(), C.double(h), C.double(s), C.double(v), C.double(a))
	}
}

func (ptr *QColor) GetRgb(r int, g int, b int, a int) {
	if ptr.Pointer() != nil {
		C.QColor_GetRgb(ptr.Pointer(), C.int(int32(r)), C.int(int32(g)), C.int(int32(b)), C.int(int32(a)))
	}
}

func (ptr *QColor) GetRgbF(r float64, g float64, b float64, a float64) {
	if ptr.Pointer() != nil {
		C.QColor_GetRgbF(ptr.Pointer(), C.double(r), C.double(g), C.double(b), C.double(a))
	}
}

func (ptr *QColor) Green() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Green(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) GreenF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_GreenF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) HslHue() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_HslHue(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) HslHueF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_HslHueF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) HslSaturation() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_HslSaturation(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) HslSaturationF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_HslSaturationF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) HsvHue() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_HsvHue(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) HsvHueF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_HsvHueF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) HsvSaturation() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_HsvSaturation(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) HsvSaturationF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_HsvSaturationF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) Hue() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Hue(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) HueF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_HueF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QColor_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func QColor_IsValidColor(name string) bool {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	return int8(C.QColor_QColor_IsValidColor(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})) != 0
}

func (ptr *QColor) IsValidColor(name string) bool {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	return int8(C.QColor_QColor_IsValidColor(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})) != 0
}

func QColor_IsValidColor2(name core.QStringView_ITF) bool {
	return int8(C.QColor_QColor_IsValidColor2(core.PointerFromQStringView(name))) != 0
}

func (ptr *QColor) IsValidColor2(name core.QStringView_ITF) bool {
	return int8(C.QColor_QColor_IsValidColor2(core.PointerFromQStringView(name))) != 0
}

func QColor_IsValidColor3(name core.QLatin1String_ITF) bool {
	return int8(C.QColor_QColor_IsValidColor3(core.PointerFromQLatin1String(name))) != 0
}

func (ptr *QColor) IsValidColor3(name core.QLatin1String_ITF) bool {
	return int8(C.QColor_QColor_IsValidColor3(core.PointerFromQLatin1String(name))) != 0
}

func (ptr *QColor) Lighter(factor int) *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QColor_Lighter(ptr.Pointer(), C.int(int32(factor))))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QColor) Lightness() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Lightness(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) LightnessF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_LightnessF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) Magenta() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Magenta(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) MagentaF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_MagentaF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) Name() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QColor_Name(ptr.Pointer()))
	}
	return ""
}

func (ptr *QColor) Name2(format QColor__NameFormat) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QColor_Name2(ptr.Pointer(), C.longlong(format)))
	}
	return ""
}

func (ptr *QColor) Red() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Red(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) RedF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_RedF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) Rgb() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QColor_Rgb(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) Rgba() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QColor_Rgba(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) Saturation() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Saturation(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) SaturationF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_SaturationF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) SetAlpha(alpha int) {
	if ptr.Pointer() != nil {
		C.QColor_SetAlpha(ptr.Pointer(), C.int(int32(alpha)))
	}
}

func (ptr *QColor) SetAlphaF(alpha float64) {
	if ptr.Pointer() != nil {
		C.QColor_SetAlphaF(ptr.Pointer(), C.double(alpha))
	}
}

func (ptr *QColor) SetBlue(blue int) {
	if ptr.Pointer() != nil {
		C.QColor_SetBlue(ptr.Pointer(), C.int(int32(blue)))
	}
}

func (ptr *QColor) SetBlueF(blue float64) {
	if ptr.Pointer() != nil {
		C.QColor_SetBlueF(ptr.Pointer(), C.double(blue))
	}
}

func (ptr *QColor) SetCmyk(c int, m int, y int, k int, a int) {
	if ptr.Pointer() != nil {
		C.QColor_SetCmyk(ptr.Pointer(), C.int(int32(c)), C.int(int32(m)), C.int(int32(y)), C.int(int32(k)), C.int(int32(a)))
	}
}

func (ptr *QColor) SetCmykF(c float64, m float64, y float64, k float64, a float64) {
	if ptr.Pointer() != nil {
		C.QColor_SetCmykF(ptr.Pointer(), C.double(c), C.double(m), C.double(y), C.double(k), C.double(a))
	}
}

func (ptr *QColor) SetGreen(green int) {
	if ptr.Pointer() != nil {
		C.QColor_SetGreen(ptr.Pointer(), C.int(int32(green)))
	}
}

func (ptr *QColor) SetGreenF(green float64) {
	if ptr.Pointer() != nil {
		C.QColor_SetGreenF(ptr.Pointer(), C.double(green))
	}
}

func (ptr *QColor) SetHsl(h int, s int, l int, a int) {
	if ptr.Pointer() != nil {
		C.QColor_SetHsl(ptr.Pointer(), C.int(int32(h)), C.int(int32(s)), C.int(int32(l)), C.int(int32(a)))
	}
}

func (ptr *QColor) SetHslF(h float64, s float64, l float64, a float64) {
	if ptr.Pointer() != nil {
		C.QColor_SetHslF(ptr.Pointer(), C.double(h), C.double(s), C.double(l), C.double(a))
	}
}

func (ptr *QColor) SetHsv(h int, s int, v int, a int) {
	if ptr.Pointer() != nil {
		C.QColor_SetHsv(ptr.Pointer(), C.int(int32(h)), C.int(int32(s)), C.int(int32(v)), C.int(int32(a)))
	}
}

func (ptr *QColor) SetHsvF(h float64, s float64, v float64, a float64) {
	if ptr.Pointer() != nil {
		C.QColor_SetHsvF(ptr.Pointer(), C.double(h), C.double(s), C.double(v), C.double(a))
	}
}

func (ptr *QColor) SetNamedColor(name string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QColor_SetNamedColor(ptr.Pointer(), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
	}
}

func (ptr *QColor) SetNamedColor2(name core.QStringView_ITF) {
	if ptr.Pointer() != nil {
		C.QColor_SetNamedColor2(ptr.Pointer(), core.PointerFromQStringView(name))
	}
}

func (ptr *QColor) SetNamedColor3(name core.QLatin1String_ITF) {
	if ptr.Pointer() != nil {
		C.QColor_SetNamedColor3(ptr.Pointer(), core.PointerFromQLatin1String(name))
	}
}

func (ptr *QColor) SetRed(red int) {
	if ptr.Pointer() != nil {
		C.QColor_SetRed(ptr.Pointer(), C.int(int32(red)))
	}
}

func (ptr *QColor) SetRedF(red float64) {
	if ptr.Pointer() != nil {
		C.QColor_SetRedF(ptr.Pointer(), C.double(red))
	}
}

func (ptr *QColor) SetRgb(r int, g int, b int, a int) {
	if ptr.Pointer() != nil {
		C.QColor_SetRgb(ptr.Pointer(), C.int(int32(r)), C.int(int32(g)), C.int(int32(b)), C.int(int32(a)))
	}
}

func (ptr *QColor) SetRgb2(rgb uint) {
	if ptr.Pointer() != nil {
		C.QColor_SetRgb2(ptr.Pointer(), C.uint(uint32(rgb)))
	}
}

func (ptr *QColor) SetRgbF(r float64, g float64, b float64, a float64) {
	if ptr.Pointer() != nil {
		C.QColor_SetRgbF(ptr.Pointer(), C.double(r), C.double(g), C.double(b), C.double(a))
	}
}

func (ptr *QColor) SetRgba(rgba uint) {
	if ptr.Pointer() != nil {
		C.QColor_SetRgba(ptr.Pointer(), C.uint(uint32(rgba)))
	}
}

func (ptr *QColor) SetRgba64(rgba QRgba64_ITF) {
	if ptr.Pointer() != nil {
		C.QColor_SetRgba64(ptr.Pointer(), PointerFromQRgba64(rgba))
	}
}

func (ptr *QColor) Spec() QColor__Spec {
	if ptr.Pointer() != nil {
		return QColor__Spec(C.QColor_Spec(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) ToCmyk() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QColor_ToCmyk(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QColor) ToHsl() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QColor_ToHsl(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QColor) ToHsv() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QColor_ToHsv(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QColor) ToRgb() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QColor_ToRgb(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QColor) Value() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Value(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) ValueF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_ValueF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) Yellow() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QColor_Yellow(ptr.Pointer())))
	}
	return 0
}

func (ptr *QColor) YellowF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QColor_YellowF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QColor) ToVariant() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QColor_ToVariant(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

type QColorDialogOptions struct {
	ptr unsafe.Pointer
}

type QColorDialogOptions_ITF interface {
	QColorDialogOptions_PTR() *QColorDialogOptions
}

func (ptr *QColorDialogOptions) QColorDialogOptions_PTR() *QColorDialogOptions {
	return ptr
}

func (ptr *QColorDialogOptions) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QColorDialogOptions) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQColorDialogOptions(ptr QColorDialogOptions_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QColorDialogOptions_PTR().Pointer()
	}
	return nil
}

func NewQColorDialogOptionsFromPointer(ptr unsafe.Pointer) (n *QColorDialogOptions) {
	n = new(QColorDialogOptions)
	n.SetPointer(ptr)
	return
}
func (ptr *QColorDialogOptions) DestroyQColorDialogOptions() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QConicalGradient struct {
	QGradient
}

type QConicalGradient_ITF interface {
	QGradient_ITF
	QConicalGradient_PTR() *QConicalGradient
}

func (ptr *QConicalGradient) QConicalGradient_PTR() *QConicalGradient {
	return ptr
}

func (ptr *QConicalGradient) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QGradient_PTR().Pointer()
	}
	return nil
}

func (ptr *QConicalGradient) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QGradient_PTR().SetPointer(p)
	}
}

func PointerFromQConicalGradient(ptr QConicalGradient_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QConicalGradient_PTR().Pointer()
	}
	return nil
}

func NewQConicalGradientFromPointer(ptr unsafe.Pointer) (n *QConicalGradient) {
	n = new(QConicalGradient)
	n.SetPointer(ptr)
	return
}
func (ptr *QConicalGradient) DestroyQConicalGradient() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQConicalGradient() *QConicalGradient {
	tmpValue := NewQConicalGradientFromPointer(C.QConicalGradient_NewQConicalGradient())
	qt.SetFinalizer(tmpValue, (*QConicalGradient).DestroyQConicalGradient)
	return tmpValue
}

func NewQConicalGradient2(center core.QPointF_ITF, angle float64) *QConicalGradient {
	tmpValue := NewQConicalGradientFromPointer(C.QConicalGradient_NewQConicalGradient2(core.PointerFromQPointF(center), C.double(angle)))
	qt.SetFinalizer(tmpValue, (*QConicalGradient).DestroyQConicalGradient)
	return tmpValue
}

func NewQConicalGradient3(cx float64, cy float64, angle float64) *QConicalGradient {
	tmpValue := NewQConicalGradientFromPointer(C.QConicalGradient_NewQConicalGradient3(C.double(cx), C.double(cy), C.double(angle)))
	qt.SetFinalizer(tmpValue, (*QConicalGradient).DestroyQConicalGradient)
	return tmpValue
}

func (ptr *QConicalGradient) Angle() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QConicalGradient_Angle(ptr.Pointer()))
	}
	return 0
}

func (ptr *QConicalGradient) Center() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QConicalGradient_Center(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QConicalGradient) SetAngle(angle float64) {
	if ptr.Pointer() != nil {
		C.QConicalGradient_SetAngle(ptr.Pointer(), C.double(angle))
	}
}

func (ptr *QConicalGradient) SetCenter(center core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QConicalGradient_SetCenter(ptr.Pointer(), core.PointerFromQPointF(center))
	}
}

func (ptr *QConicalGradient) SetCenter2(x float64, y float64) {
	if ptr.Pointer() != nil {
		C.QConicalGradient_SetCenter2(ptr.Pointer(), C.double(x), C.double(y))
	}
}

type QContextMenuEvent struct {
	QInputEvent
}

type QContextMenuEvent_ITF interface {
	QInputEvent_ITF
	QContextMenuEvent_PTR() *QContextMenuEvent
}

func (ptr *QContextMenuEvent) QContextMenuEvent_PTR() *QContextMenuEvent {
	return ptr
}

func (ptr *QContextMenuEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QContextMenuEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QInputEvent_PTR().SetPointer(p)
	}
}

func PointerFromQContextMenuEvent(ptr QContextMenuEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QContextMenuEvent_PTR().Pointer()
	}
	return nil
}

func NewQContextMenuEventFromPointer(ptr unsafe.Pointer) (n *QContextMenuEvent) {
	n = new(QContextMenuEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QContextMenuEvent) DestroyQContextMenuEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QContextMenuEvent__Reason
//QContextMenuEvent::Reason
type QContextMenuEvent__Reason int64

const (
	QContextMenuEvent__Mouse    QContextMenuEvent__Reason = QContextMenuEvent__Reason(0)
	QContextMenuEvent__Keyboard QContextMenuEvent__Reason = QContextMenuEvent__Reason(1)
	QContextMenuEvent__Other    QContextMenuEvent__Reason = QContextMenuEvent__Reason(2)
)

func NewQContextMenuEvent(reason QContextMenuEvent__Reason, pos core.QPoint_ITF, globalPos core.QPoint_ITF, modifiers core.Qt__KeyboardModifier) *QContextMenuEvent {
	tmpValue := NewQContextMenuEventFromPointer(C.QContextMenuEvent_NewQContextMenuEvent(C.longlong(reason), core.PointerFromQPoint(pos), core.PointerFromQPoint(globalPos), C.longlong(modifiers)))
	qt.SetFinalizer(tmpValue, (*QContextMenuEvent).DestroyQContextMenuEvent)
	return tmpValue
}

func NewQContextMenuEvent2(reason QContextMenuEvent__Reason, pos core.QPoint_ITF, globalPos core.QPoint_ITF) *QContextMenuEvent {
	tmpValue := NewQContextMenuEventFromPointer(C.QContextMenuEvent_NewQContextMenuEvent2(C.longlong(reason), core.PointerFromQPoint(pos), core.PointerFromQPoint(globalPos)))
	qt.SetFinalizer(tmpValue, (*QContextMenuEvent).DestroyQContextMenuEvent)
	return tmpValue
}

func NewQContextMenuEvent3(reason QContextMenuEvent__Reason, pos core.QPoint_ITF) *QContextMenuEvent {
	tmpValue := NewQContextMenuEventFromPointer(C.QContextMenuEvent_NewQContextMenuEvent3(C.longlong(reason), core.PointerFromQPoint(pos)))
	qt.SetFinalizer(tmpValue, (*QContextMenuEvent).DestroyQContextMenuEvent)
	return tmpValue
}

func (ptr *QContextMenuEvent) GlobalPos() *core.QPoint {
	if ptr.Pointer() != nil {
		return core.NewQPointFromPointer(C.QContextMenuEvent_GlobalPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QContextMenuEvent) GlobalX() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QContextMenuEvent_GlobalX(ptr.Pointer())))
	}
	return 0
}

func (ptr *QContextMenuEvent) GlobalY() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QContextMenuEvent_GlobalY(ptr.Pointer())))
	}
	return 0
}

func (ptr *QContextMenuEvent) Pos() *core.QPoint {
	if ptr.Pointer() != nil {
		return core.NewQPointFromPointer(C.QContextMenuEvent_Pos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QContextMenuEvent) Reason() QContextMenuEvent__Reason {
	if ptr.Pointer() != nil {
		return QContextMenuEvent__Reason(C.QContextMenuEvent_Reason(ptr.Pointer()))
	}
	return 0
}

func (ptr *QContextMenuEvent) X() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QContextMenuEvent_X(ptr.Pointer())))
	}
	return 0
}

func (ptr *QContextMenuEvent) Y() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QContextMenuEvent_Y(ptr.Pointer())))
	}
	return 0
}

type QCursor struct {
	ptr unsafe.Pointer
}

type QCursor_ITF interface {
	QCursor_PTR() *QCursor
}

func (ptr *QCursor) QCursor_PTR() *QCursor {
	return ptr
}

func (ptr *QCursor) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QCursor) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQCursor(ptr QCursor_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QCursor_PTR().Pointer()
	}
	return nil
}

func NewQCursorFromPointer(ptr unsafe.Pointer) (n *QCursor) {
	n = new(QCursor)
	n.SetPointer(ptr)
	return
}
func NewQCursor() *QCursor {
	tmpValue := NewQCursorFromPointer(C.QCursor_NewQCursor())
	qt.SetFinalizer(tmpValue, (*QCursor).DestroyQCursor)
	return tmpValue
}

func NewQCursor2(shape core.Qt__CursorShape) *QCursor {
	tmpValue := NewQCursorFromPointer(C.QCursor_NewQCursor2(C.longlong(shape)))
	qt.SetFinalizer(tmpValue, (*QCursor).DestroyQCursor)
	return tmpValue
}

func NewQCursor3(bitmap QBitmap_ITF, mask QBitmap_ITF, hotX int, hotY int) *QCursor {
	tmpValue := NewQCursorFromPointer(C.QCursor_NewQCursor3(PointerFromQBitmap(bitmap), PointerFromQBitmap(mask), C.int(int32(hotX)), C.int(int32(hotY))))
	qt.SetFinalizer(tmpValue, (*QCursor).DestroyQCursor)
	return tmpValue
}

func NewQCursor4(pixmap QPixmap_ITF, hotX int, hotY int) *QCursor {
	tmpValue := NewQCursorFromPointer(C.QCursor_NewQCursor4(PointerFromQPixmap(pixmap), C.int(int32(hotX)), C.int(int32(hotY))))
	qt.SetFinalizer(tmpValue, (*QCursor).DestroyQCursor)
	return tmpValue
}

func NewQCursor5(c QCursor_ITF) *QCursor {
	tmpValue := NewQCursorFromPointer(C.QCursor_NewQCursor5(PointerFromQCursor(c)))
	qt.SetFinalizer(tmpValue, (*QCursor).DestroyQCursor)
	return tmpValue
}

func NewQCursor6(other QCursor_ITF) *QCursor {
	tmpValue := NewQCursorFromPointer(C.QCursor_NewQCursor6(PointerFromQCursor(other)))
	qt.SetFinalizer(tmpValue, (*QCursor).DestroyQCursor)
	return tmpValue
}

func (ptr *QCursor) Bitmap() *QBitmap {
	if ptr.Pointer() != nil {
		return NewQBitmapFromPointer(C.QCursor_Bitmap(ptr.Pointer()))
	}
	return nil
}

func (ptr *QCursor) HotSpot() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QCursor_HotSpot(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QCursor) Mask() *QBitmap {
	if ptr.Pointer() != nil {
		return NewQBitmapFromPointer(C.QCursor_Mask(ptr.Pointer()))
	}
	return nil
}

func (ptr *QCursor) Pixmap() *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QCursor_Pixmap(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func QCursor_Pos() *core.QPoint {
	tmpValue := core.NewQPointFromPointer(C.QCursor_QCursor_Pos())
	qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
	return tmpValue
}

func (ptr *QCursor) Pos() *core.QPoint {
	tmpValue := core.NewQPointFromPointer(C.QCursor_QCursor_Pos())
	qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
	return tmpValue
}

func QCursor_Pos2(screen QScreen_ITF) *core.QPoint {
	tmpValue := core.NewQPointFromPointer(C.QCursor_QCursor_Pos2(PointerFromQScreen(screen)))
	qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
	return tmpValue
}

func (ptr *QCursor) Pos2(screen QScreen_ITF) *core.QPoint {
	tmpValue := core.NewQPointFromPointer(C.QCursor_QCursor_Pos2(PointerFromQScreen(screen)))
	qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
	return tmpValue
}

func QCursor_SetPos(x int, y int) {
	C.QCursor_QCursor_SetPos(C.int(int32(x)), C.int(int32(y)))
}

func (ptr *QCursor) SetPos(x int, y int) {
	C.QCursor_QCursor_SetPos(C.int(int32(x)), C.int(int32(y)))
}

func QCursor_SetPos2(screen QScreen_ITF, x int, y int) {
	C.QCursor_QCursor_SetPos2(PointerFromQScreen(screen), C.int(int32(x)), C.int(int32(y)))
}

func (ptr *QCursor) SetPos2(screen QScreen_ITF, x int, y int) {
	C.QCursor_QCursor_SetPos2(PointerFromQScreen(screen), C.int(int32(x)), C.int(int32(y)))
}

func QCursor_SetPos3(p core.QPoint_ITF) {
	C.QCursor_QCursor_SetPos3(core.PointerFromQPoint(p))
}

func (ptr *QCursor) SetPos3(p core.QPoint_ITF) {
	C.QCursor_QCursor_SetPos3(core.PointerFromQPoint(p))
}

func QCursor_SetPos4(screen QScreen_ITF, p core.QPoint_ITF) {
	C.QCursor_QCursor_SetPos4(PointerFromQScreen(screen), core.PointerFromQPoint(p))
}

func (ptr *QCursor) SetPos4(screen QScreen_ITF, p core.QPoint_ITF) {
	C.QCursor_QCursor_SetPos4(PointerFromQScreen(screen), core.PointerFromQPoint(p))
}

func (ptr *QCursor) SetShape(shape core.Qt__CursorShape) {
	if ptr.Pointer() != nil {
		C.QCursor_SetShape(ptr.Pointer(), C.longlong(shape))
	}
}

func (ptr *QCursor) Shape() core.Qt__CursorShape {
	if ptr.Pointer() != nil {
		return core.Qt__CursorShape(C.QCursor_Shape(ptr.Pointer()))
	}
	return 0
}

func (ptr *QCursor) Swap(other QCursor_ITF) {
	if ptr.Pointer() != nil {
		C.QCursor_Swap(ptr.Pointer(), PointerFromQCursor(other))
	}
}

func (ptr *QCursor) DestroyQCursor() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QCursor_DestroyQCursor(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QDesktopServices struct {
	ptr unsafe.Pointer
}

type QDesktopServices_ITF interface {
	QDesktopServices_PTR() *QDesktopServices
}

func (ptr *QDesktopServices) QDesktopServices_PTR() *QDesktopServices {
	return ptr
}

func (ptr *QDesktopServices) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QDesktopServices) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQDesktopServices(ptr QDesktopServices_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QDesktopServices_PTR().Pointer()
	}
	return nil
}

func NewQDesktopServicesFromPointer(ptr unsafe.Pointer) (n *QDesktopServices) {
	n = new(QDesktopServices)
	n.SetPointer(ptr)
	return
}
func (ptr *QDesktopServices) DestroyQDesktopServices() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func QDesktopServices_OpenUrl(url core.QUrl_ITF) bool {
	return int8(C.QDesktopServices_QDesktopServices_OpenUrl(core.PointerFromQUrl(url))) != 0
}

func (ptr *QDesktopServices) OpenUrl(url core.QUrl_ITF) bool {
	return int8(C.QDesktopServices_QDesktopServices_OpenUrl(core.PointerFromQUrl(url))) != 0
}

func QDesktopServices_SetUrlHandler(scheme string, receiver core.QObject_ITF, method string) {
	var schemeC *C.char
	if scheme != "" {
		schemeC = C.CString(scheme)
		defer C.free(unsafe.Pointer(schemeC))
	}
	var methodC *C.char
	if method != "" {
		methodC = C.CString(method)
		defer C.free(unsafe.Pointer(methodC))
	}
	C.QDesktopServices_QDesktopServices_SetUrlHandler(C.struct_QtGui_PackedString{data: schemeC, len: C.longlong(len(scheme))}, core.PointerFromQObject(receiver), methodC)
}

func (ptr *QDesktopServices) SetUrlHandler(scheme string, receiver core.QObject_ITF, method string) {
	var schemeC *C.char
	if scheme != "" {
		schemeC = C.CString(scheme)
		defer C.free(unsafe.Pointer(schemeC))
	}
	var methodC *C.char
	if method != "" {
		methodC = C.CString(method)
		defer C.free(unsafe.Pointer(methodC))
	}
	C.QDesktopServices_QDesktopServices_SetUrlHandler(C.struct_QtGui_PackedString{data: schemeC, len: C.longlong(len(scheme))}, core.PointerFromQObject(receiver), methodC)
}

func QDesktopServices_UnsetUrlHandler(scheme string) {
	var schemeC *C.char
	if scheme != "" {
		schemeC = C.CString(scheme)
		defer C.free(unsafe.Pointer(schemeC))
	}
	C.QDesktopServices_QDesktopServices_UnsetUrlHandler(C.struct_QtGui_PackedString{data: schemeC, len: C.longlong(len(scheme))})
}

func (ptr *QDesktopServices) UnsetUrlHandler(scheme string) {
	var schemeC *C.char
	if scheme != "" {
		schemeC = C.CString(scheme)
		defer C.free(unsafe.Pointer(schemeC))
	}
	C.QDesktopServices_QDesktopServices_UnsetUrlHandler(C.struct_QtGui_PackedString{data: schemeC, len: C.longlong(len(scheme))})
}

type QDoubleValidator struct {
	QValidator
}

type QDoubleValidator_ITF interface {
	QValidator_ITF
	QDoubleValidator_PTR() *QDoubleValidator
}

func (ptr *QDoubleValidator) QDoubleValidator_PTR() *QDoubleValidator {
	return ptr
}

func (ptr *QDoubleValidator) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QValidator_PTR().Pointer()
	}
	return nil
}

func (ptr *QDoubleValidator) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QValidator_PTR().SetPointer(p)
	}
}

func PointerFromQDoubleValidator(ptr QDoubleValidator_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QDoubleValidator_PTR().Pointer()
	}
	return nil
}

func NewQDoubleValidatorFromPointer(ptr unsafe.Pointer) (n *QDoubleValidator) {
	n = new(QDoubleValidator)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QDoubleValidator__Notation
//QDoubleValidator::Notation
type QDoubleValidator__Notation int64

const (
	QDoubleValidator__StandardNotation   QDoubleValidator__Notation = QDoubleValidator__Notation(0)
	QDoubleValidator__ScientificNotation QDoubleValidator__Notation = QDoubleValidator__Notation(1)
)

func NewQDoubleValidator(parent core.QObject_ITF) *QDoubleValidator {
	tmpValue := NewQDoubleValidatorFromPointer(C.QDoubleValidator_NewQDoubleValidator(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQDoubleValidator2(bottom float64, top float64, decimals int, parent core.QObject_ITF) *QDoubleValidator {
	tmpValue := NewQDoubleValidatorFromPointer(C.QDoubleValidator_NewQDoubleValidator2(C.double(bottom), C.double(top), C.int(int32(decimals)), core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QDoubleValidator) Bottom() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QDoubleValidator_Bottom(ptr.Pointer()))
	}
	return 0
}

//export callbackQDoubleValidator_BottomChanged
func callbackQDoubleValidator_BottomChanged(ptr unsafe.Pointer, bottom C.double) {
	if signal := qt.GetSignal(ptr, "bottomChanged"); signal != nil {
		(*(*func(float64))(signal))(float64(bottom))
	}

}

func (ptr *QDoubleValidator) ConnectBottomChanged(f func(bottom float64)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "bottomChanged") {
			C.QDoubleValidator_ConnectBottomChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "bottomChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "bottomChanged"); signal != nil {
			f := func(bottom float64) {
				(*(*func(float64))(signal))(bottom)
				f(bottom)
			}
			qt.ConnectSignal(ptr.Pointer(), "bottomChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "bottomChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDoubleValidator) DisconnectBottomChanged() {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_DisconnectBottomChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "bottomChanged")
	}
}

func (ptr *QDoubleValidator) BottomChanged(bottom float64) {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_BottomChanged(ptr.Pointer(), C.double(bottom))
	}
}

func (ptr *QDoubleValidator) Decimals() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QDoubleValidator_Decimals(ptr.Pointer())))
	}
	return 0
}

//export callbackQDoubleValidator_DecimalsChanged
func callbackQDoubleValidator_DecimalsChanged(ptr unsafe.Pointer, decimals C.int) {
	if signal := qt.GetSignal(ptr, "decimalsChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(decimals)))
	}

}

func (ptr *QDoubleValidator) ConnectDecimalsChanged(f func(decimals int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "decimalsChanged") {
			C.QDoubleValidator_ConnectDecimalsChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "decimalsChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "decimalsChanged"); signal != nil {
			f := func(decimals int) {
				(*(*func(int))(signal))(decimals)
				f(decimals)
			}
			qt.ConnectSignal(ptr.Pointer(), "decimalsChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "decimalsChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDoubleValidator) DisconnectDecimalsChanged() {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_DisconnectDecimalsChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "decimalsChanged")
	}
}

func (ptr *QDoubleValidator) DecimalsChanged(decimals int) {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_DecimalsChanged(ptr.Pointer(), C.int(int32(decimals)))
	}
}

func (ptr *QDoubleValidator) Notation() QDoubleValidator__Notation {
	if ptr.Pointer() != nil {
		return QDoubleValidator__Notation(C.QDoubleValidator_Notation(ptr.Pointer()))
	}
	return 0
}

//export callbackQDoubleValidator_NotationChanged
func callbackQDoubleValidator_NotationChanged(ptr unsafe.Pointer, notation C.longlong) {
	if signal := qt.GetSignal(ptr, "notationChanged"); signal != nil {
		(*(*func(QDoubleValidator__Notation))(signal))(QDoubleValidator__Notation(notation))
	}

}

func (ptr *QDoubleValidator) ConnectNotationChanged(f func(notation QDoubleValidator__Notation)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "notationChanged") {
			C.QDoubleValidator_ConnectNotationChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "notationChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "notationChanged"); signal != nil {
			f := func(notation QDoubleValidator__Notation) {
				(*(*func(QDoubleValidator__Notation))(signal))(notation)
				f(notation)
			}
			qt.ConnectSignal(ptr.Pointer(), "notationChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "notationChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDoubleValidator) DisconnectNotationChanged() {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_DisconnectNotationChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "notationChanged")
	}
}

func (ptr *QDoubleValidator) NotationChanged(notation QDoubleValidator__Notation) {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_NotationChanged(ptr.Pointer(), C.longlong(notation))
	}
}

func (ptr *QDoubleValidator) SetBottom(vdo float64) {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_SetBottom(ptr.Pointer(), C.double(vdo))
	}
}

func (ptr *QDoubleValidator) SetDecimals(vin int) {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_SetDecimals(ptr.Pointer(), C.int(int32(vin)))
	}
}

func (ptr *QDoubleValidator) SetNotation(vqd QDoubleValidator__Notation) {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_SetNotation(ptr.Pointer(), C.longlong(vqd))
	}
}

//export callbackQDoubleValidator_SetRange
func callbackQDoubleValidator_SetRange(ptr unsafe.Pointer, minimum C.double, maximum C.double, decimals C.int) {
	if signal := qt.GetSignal(ptr, "setRange"); signal != nil {
		(*(*func(float64, float64, int))(signal))(float64(minimum), float64(maximum), int(int32(decimals)))
	} else {
		NewQDoubleValidatorFromPointer(ptr).SetRangeDefault(float64(minimum), float64(maximum), int(int32(decimals)))
	}
}

func (ptr *QDoubleValidator) ConnectSetRange(f func(minimum float64, maximum float64, decimals int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setRange"); signal != nil {
			f := func(minimum float64, maximum float64, decimals int) {
				(*(*func(float64, float64, int))(signal))(minimum, maximum, decimals)
				f(minimum, maximum, decimals)
			}
			qt.ConnectSignal(ptr.Pointer(), "setRange", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setRange", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDoubleValidator) DisconnectSetRange() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setRange")
	}
}

func (ptr *QDoubleValidator) SetRange(minimum float64, maximum float64, decimals int) {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_SetRange(ptr.Pointer(), C.double(minimum), C.double(maximum), C.int(int32(decimals)))
	}
}

func (ptr *QDoubleValidator) SetRangeDefault(minimum float64, maximum float64, decimals int) {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_SetRangeDefault(ptr.Pointer(), C.double(minimum), C.double(maximum), C.int(int32(decimals)))
	}
}

func (ptr *QDoubleValidator) SetTop(vdo float64) {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_SetTop(ptr.Pointer(), C.double(vdo))
	}
}

func (ptr *QDoubleValidator) Top() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QDoubleValidator_Top(ptr.Pointer()))
	}
	return 0
}

//export callbackQDoubleValidator_TopChanged
func callbackQDoubleValidator_TopChanged(ptr unsafe.Pointer, top C.double) {
	if signal := qt.GetSignal(ptr, "topChanged"); signal != nil {
		(*(*func(float64))(signal))(float64(top))
	}

}

func (ptr *QDoubleValidator) ConnectTopChanged(f func(top float64)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "topChanged") {
			C.QDoubleValidator_ConnectTopChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "topChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "topChanged"); signal != nil {
			f := func(top float64) {
				(*(*func(float64))(signal))(top)
				f(top)
			}
			qt.ConnectSignal(ptr.Pointer(), "topChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "topChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDoubleValidator) DisconnectTopChanged() {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_DisconnectTopChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "topChanged")
	}
}

func (ptr *QDoubleValidator) TopChanged(top float64) {
	if ptr.Pointer() != nil {
		C.QDoubleValidator_TopChanged(ptr.Pointer(), C.double(top))
	}
}

//export callbackQDoubleValidator_Validate
func callbackQDoubleValidator_Validate(ptr unsafe.Pointer, input C.struct_QtGui_PackedString, pos C.int) C.longlong {
	if signal := qt.GetSignal(ptr, "validate"); signal != nil {
		return C.longlong((*(*func(string, int) QValidator__State)(signal))(cGoUnpackString(input), int(int32(pos))))
	}

	return C.longlong(NewQDoubleValidatorFromPointer(ptr).ValidateDefault(cGoUnpackString(input), int(int32(pos))))
}

func (ptr *QDoubleValidator) ConnectValidate(f func(input string, pos int) QValidator__State) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "validate"); signal != nil {
			f := func(input string, pos int) QValidator__State {
				(*(*func(string, int) QValidator__State)(signal))(input, pos)
				return f(input, pos)
			}
			qt.ConnectSignal(ptr.Pointer(), "validate", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "validate", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDoubleValidator) DisconnectValidate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "validate")
	}
}

func (ptr *QDoubleValidator) Validate(input string, pos int) QValidator__State {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		return QValidator__State(C.QDoubleValidator_Validate(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))}, C.int(int32(pos))))
	}
	return 0
}

func (ptr *QDoubleValidator) ValidateDefault(input string, pos int) QValidator__State {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		return QValidator__State(C.QDoubleValidator_ValidateDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))}, C.int(int32(pos))))
	}
	return 0
}

//export callbackQDoubleValidator_DestroyQDoubleValidator
func callbackQDoubleValidator_DestroyQDoubleValidator(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QDoubleValidator"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQDoubleValidatorFromPointer(ptr).DestroyQDoubleValidatorDefault()
	}
}

func (ptr *QDoubleValidator) ConnectDestroyQDoubleValidator(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QDoubleValidator"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QDoubleValidator", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QDoubleValidator", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDoubleValidator) DisconnectDestroyQDoubleValidator() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QDoubleValidator")
	}
}

func (ptr *QDoubleValidator) DestroyQDoubleValidator() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QDoubleValidator_DestroyQDoubleValidator(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QDoubleValidator) DestroyQDoubleValidatorDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QDoubleValidator_DestroyQDoubleValidatorDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QDrag struct {
	core.QObject
}

type QDrag_ITF interface {
	core.QObject_ITF
	QDrag_PTR() *QDrag
}

func (ptr *QDrag) QDrag_PTR() *QDrag {
	return ptr
}

func (ptr *QDrag) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QDrag) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQDrag(ptr QDrag_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QDrag_PTR().Pointer()
	}
	return nil
}

func NewQDragFromPointer(ptr unsafe.Pointer) (n *QDrag) {
	n = new(QDrag)
	n.SetPointer(ptr)
	return
}
func NewQDrag(dragSource core.QObject_ITF) *QDrag {
	tmpValue := NewQDragFromPointer(C.QDrag_NewQDrag(core.PointerFromQObject(dragSource)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQDrag_ActionChanged
func callbackQDrag_ActionChanged(ptr unsafe.Pointer, action C.longlong) {
	if signal := qt.GetSignal(ptr, "actionChanged"); signal != nil {
		(*(*func(core.Qt__DropAction))(signal))(core.Qt__DropAction(action))
	}

}

func (ptr *QDrag) ConnectActionChanged(f func(action core.Qt__DropAction)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "actionChanged") {
			C.QDrag_ConnectActionChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "actionChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "actionChanged"); signal != nil {
			f := func(action core.Qt__DropAction) {
				(*(*func(core.Qt__DropAction))(signal))(action)
				f(action)
			}
			qt.ConnectSignal(ptr.Pointer(), "actionChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "actionChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDrag) DisconnectActionChanged() {
	if ptr.Pointer() != nil {
		C.QDrag_DisconnectActionChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "actionChanged")
	}
}

func (ptr *QDrag) ActionChanged(action core.Qt__DropAction) {
	if ptr.Pointer() != nil {
		C.QDrag_ActionChanged(ptr.Pointer(), C.longlong(action))
	}
}

func QDrag_Cancel() {
	C.QDrag_QDrag_Cancel()
}

func (ptr *QDrag) Cancel() {
	C.QDrag_QDrag_Cancel()
}

func (ptr *QDrag) DefaultAction() core.Qt__DropAction {
	if ptr.Pointer() != nil {
		return core.Qt__DropAction(C.QDrag_DefaultAction(ptr.Pointer()))
	}
	return 0
}

func (ptr *QDrag) DragCursor(action core.Qt__DropAction) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QDrag_DragCursor(ptr.Pointer(), C.longlong(action)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QDrag) Exec(supportedActions core.Qt__DropAction) core.Qt__DropAction {
	if ptr.Pointer() != nil {
		return core.Qt__DropAction(C.QDrag_Exec(ptr.Pointer(), C.longlong(supportedActions)))
	}
	return 0
}

func (ptr *QDrag) Exec2(supportedActions core.Qt__DropAction, defaultDropAction core.Qt__DropAction) core.Qt__DropAction {
	if ptr.Pointer() != nil {
		return core.Qt__DropAction(C.QDrag_Exec2(ptr.Pointer(), C.longlong(supportedActions), C.longlong(defaultDropAction)))
	}
	return 0
}

func (ptr *QDrag) HotSpot() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QDrag_HotSpot(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QDrag) MimeData() *core.QMimeData {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMimeDataFromPointer(C.QDrag_MimeData(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QDrag) Pixmap() *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QDrag_Pixmap(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QDrag) SetDragCursor(cursor QPixmap_ITF, action core.Qt__DropAction) {
	if ptr.Pointer() != nil {
		C.QDrag_SetDragCursor(ptr.Pointer(), PointerFromQPixmap(cursor), C.longlong(action))
	}
}

func (ptr *QDrag) SetHotSpot(hotspot core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag_SetHotSpot(ptr.Pointer(), core.PointerFromQPoint(hotspot))
	}
}

func (ptr *QDrag) SetMimeData(data core.QMimeData_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag_SetMimeData(ptr.Pointer(), core.PointerFromQMimeData(data))
	}
}

func (ptr *QDrag) SetPixmap(pixmap QPixmap_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag_SetPixmap(ptr.Pointer(), PointerFromQPixmap(pixmap))
	}
}

func (ptr *QDrag) Source() *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QDrag_Source(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QDrag) SupportedActions() core.Qt__DropAction {
	if ptr.Pointer() != nil {
		return core.Qt__DropAction(C.QDrag_SupportedActions(ptr.Pointer()))
	}
	return 0
}

func (ptr *QDrag) Target() *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QDrag_Target(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQDrag_TargetChanged
func callbackQDrag_TargetChanged(ptr unsafe.Pointer, newTarget unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "targetChanged"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(newTarget))
	}

}

func (ptr *QDrag) ConnectTargetChanged(f func(newTarget *core.QObject)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "targetChanged") {
			C.QDrag_ConnectTargetChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "targetChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "targetChanged"); signal != nil {
			f := func(newTarget *core.QObject) {
				(*(*func(*core.QObject))(signal))(newTarget)
				f(newTarget)
			}
			qt.ConnectSignal(ptr.Pointer(), "targetChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "targetChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDrag) DisconnectTargetChanged() {
	if ptr.Pointer() != nil {
		C.QDrag_DisconnectTargetChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "targetChanged")
	}
}

func (ptr *QDrag) TargetChanged(newTarget core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag_TargetChanged(ptr.Pointer(), core.PointerFromQObject(newTarget))
	}
}

//export callbackQDrag_DestroyQDrag
func callbackQDrag_DestroyQDrag(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QDrag"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQDragFromPointer(ptr).DestroyQDragDefault()
	}
}

func (ptr *QDrag) ConnectDestroyQDrag(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QDrag"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QDrag", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QDrag", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDrag) DisconnectDestroyQDrag() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QDrag")
	}
}

func (ptr *QDrag) DestroyQDrag() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QDrag_DestroyQDrag(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QDrag) DestroyQDragDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QDrag_DestroyQDragDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QDrag) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QDrag___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QDrag) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QDrag) __children_newList() unsafe.Pointer {
	return C.QDrag___children_newList(ptr.Pointer())
}

func (ptr *QDrag) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QDrag___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QDrag) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QDrag) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QDrag___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QDrag) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QDrag___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QDrag) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QDrag) __findChildren_newList() unsafe.Pointer {
	return C.QDrag___findChildren_newList(ptr.Pointer())
}

func (ptr *QDrag) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QDrag___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QDrag) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QDrag) __findChildren_newList3() unsafe.Pointer {
	return C.QDrag___findChildren_newList3(ptr.Pointer())
}

//export callbackQDrag_ChildEvent
func callbackQDrag_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQDragFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QDrag) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQDrag_ConnectNotify
func callbackQDrag_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQDragFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QDrag) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQDrag_CustomEvent
func callbackQDrag_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQDragFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QDrag) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQDrag_DeleteLater
func callbackQDrag_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQDragFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QDrag) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QDrag_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQDrag_Destroyed
func callbackQDrag_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQDrag_DisconnectNotify
func callbackQDrag_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQDragFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QDrag) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQDrag_Event
func callbackQDrag_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQDragFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QDrag) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QDrag_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQDrag_EventFilter
func callbackQDrag_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQDragFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QDrag) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QDrag_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQDrag_MetaObject
func callbackQDrag_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQDragFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QDrag) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QDrag_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQDrag_ObjectNameChanged
func callbackQDrag_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQDrag_TimerEvent
func callbackQDrag_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQDragFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QDrag) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QDrag_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QDragEnterEvent struct {
	QDragMoveEvent
}

type QDragEnterEvent_ITF interface {
	QDragMoveEvent_ITF
	QDragEnterEvent_PTR() *QDragEnterEvent
}

func (ptr *QDragEnterEvent) QDragEnterEvent_PTR() *QDragEnterEvent {
	return ptr
}

func (ptr *QDragEnterEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QDragMoveEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QDragEnterEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QDragMoveEvent_PTR().SetPointer(p)
	}
}

func PointerFromQDragEnterEvent(ptr QDragEnterEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QDragEnterEvent_PTR().Pointer()
	}
	return nil
}

func NewQDragEnterEventFromPointer(ptr unsafe.Pointer) (n *QDragEnterEvent) {
	n = new(QDragEnterEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QDragEnterEvent) DestroyQDragEnterEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQDragEnterEvent(point core.QPoint_ITF, actions core.Qt__DropAction, data core.QMimeData_ITF, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier) *QDragEnterEvent {
	tmpValue := NewQDragEnterEventFromPointer(C.QDragEnterEvent_NewQDragEnterEvent(core.PointerFromQPoint(point), C.longlong(actions), core.PointerFromQMimeData(data), C.longlong(buttons), C.longlong(modifiers)))
	qt.SetFinalizer(tmpValue, (*QDragEnterEvent).DestroyQDragEnterEvent)
	return tmpValue
}

type QDragLeaveEvent struct {
	core.QEvent
}

type QDragLeaveEvent_ITF interface {
	core.QEvent_ITF
	QDragLeaveEvent_PTR() *QDragLeaveEvent
}

func (ptr *QDragLeaveEvent) QDragLeaveEvent_PTR() *QDragLeaveEvent {
	return ptr
}

func (ptr *QDragLeaveEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QDragLeaveEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQDragLeaveEvent(ptr QDragLeaveEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QDragLeaveEvent_PTR().Pointer()
	}
	return nil
}

func NewQDragLeaveEventFromPointer(ptr unsafe.Pointer) (n *QDragLeaveEvent) {
	n = new(QDragLeaveEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QDragLeaveEvent) DestroyQDragLeaveEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQDragLeaveEvent() *QDragLeaveEvent {
	tmpValue := NewQDragLeaveEventFromPointer(C.QDragLeaveEvent_NewQDragLeaveEvent())
	qt.SetFinalizer(tmpValue, (*QDragLeaveEvent).DestroyQDragLeaveEvent)
	return tmpValue
}

type QDragMoveEvent struct {
	QDropEvent
}

type QDragMoveEvent_ITF interface {
	QDropEvent_ITF
	QDragMoveEvent_PTR() *QDragMoveEvent
}

func (ptr *QDragMoveEvent) QDragMoveEvent_PTR() *QDragMoveEvent {
	return ptr
}

func (ptr *QDragMoveEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QDropEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QDragMoveEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QDropEvent_PTR().SetPointer(p)
	}
}

func PointerFromQDragMoveEvent(ptr QDragMoveEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QDragMoveEvent_PTR().Pointer()
	}
	return nil
}

func NewQDragMoveEventFromPointer(ptr unsafe.Pointer) (n *QDragMoveEvent) {
	n = new(QDragMoveEvent)
	n.SetPointer(ptr)
	return
}
func NewQDragMoveEvent(pos core.QPoint_ITF, actions core.Qt__DropAction, data core.QMimeData_ITF, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier, ty core.QEvent__Type) *QDragMoveEvent {
	tmpValue := NewQDragMoveEventFromPointer(C.QDragMoveEvent_NewQDragMoveEvent(core.PointerFromQPoint(pos), C.longlong(actions), core.PointerFromQMimeData(data), C.longlong(buttons), C.longlong(modifiers), C.longlong(ty)))
	qt.SetFinalizer(tmpValue, (*QDragMoveEvent).DestroyQDragMoveEvent)
	return tmpValue
}

func (ptr *QDragMoveEvent) Accept(rectangle core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QDragMoveEvent_Accept(ptr.Pointer(), core.PointerFromQRect(rectangle))
	}
}

func (ptr *QDragMoveEvent) AnswerRect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QDragMoveEvent_AnswerRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QDragMoveEvent) Ignore(rectangle core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QDragMoveEvent_Ignore(ptr.Pointer(), core.PointerFromQRect(rectangle))
	}
}

//export callbackQDragMoveEvent_DestroyQDragMoveEvent
func callbackQDragMoveEvent_DestroyQDragMoveEvent(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QDragMoveEvent"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQDragMoveEventFromPointer(ptr).DestroyQDragMoveEventDefault()
	}
}

func (ptr *QDragMoveEvent) ConnectDestroyQDragMoveEvent(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QDragMoveEvent"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QDragMoveEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QDragMoveEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QDragMoveEvent) DisconnectDestroyQDragMoveEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QDragMoveEvent")
	}
}

func (ptr *QDragMoveEvent) DestroyQDragMoveEvent() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QDragMoveEvent_DestroyQDragMoveEvent(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QDragMoveEvent) DestroyQDragMoveEventDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QDragMoveEvent_DestroyQDragMoveEventDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QDropEvent struct {
	core.QEvent
}

type QDropEvent_ITF interface {
	core.QEvent_ITF
	QDropEvent_PTR() *QDropEvent
}

func (ptr *QDropEvent) QDropEvent_PTR() *QDropEvent {
	return ptr
}

func (ptr *QDropEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QDropEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQDropEvent(ptr QDropEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QDropEvent_PTR().Pointer()
	}
	return nil
}

func NewQDropEventFromPointer(ptr unsafe.Pointer) (n *QDropEvent) {
	n = new(QDropEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QDropEvent) DestroyQDropEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQDropEvent(pos core.QPointF_ITF, actions core.Qt__DropAction, data core.QMimeData_ITF, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier, ty core.QEvent__Type) *QDropEvent {
	tmpValue := NewQDropEventFromPointer(C.QDropEvent_NewQDropEvent(core.PointerFromQPointF(pos), C.longlong(actions), core.PointerFromQMimeData(data), C.longlong(buttons), C.longlong(modifiers), C.longlong(ty)))
	qt.SetFinalizer(tmpValue, (*QDropEvent).DestroyQDropEvent)
	return tmpValue
}

func (ptr *QDropEvent) AcceptProposedAction() {
	if ptr.Pointer() != nil {
		C.QDropEvent_AcceptProposedAction(ptr.Pointer())
	}
}

func (ptr *QDropEvent) DropAction() core.Qt__DropAction {
	if ptr.Pointer() != nil {
		return core.Qt__DropAction(C.QDropEvent_DropAction(ptr.Pointer()))
	}
	return 0
}

func (ptr *QDropEvent) KeyboardModifiers() core.Qt__KeyboardModifier {
	if ptr.Pointer() != nil {
		return core.Qt__KeyboardModifier(C.QDropEvent_KeyboardModifiers(ptr.Pointer()))
	}
	return 0
}

func (ptr *QDropEvent) MimeData() *core.QMimeData {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMimeDataFromPointer(C.QDropEvent_MimeData(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QDropEvent) MouseButtons() core.Qt__MouseButton {
	if ptr.Pointer() != nil {
		return core.Qt__MouseButton(C.QDropEvent_MouseButtons(ptr.Pointer()))
	}
	return 0
}

func (ptr *QDropEvent) Pos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QDropEvent_Pos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QDropEvent) PosF() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QDropEvent_PosF(ptr.Pointer()))
	}
	return nil
}

func (ptr *QDropEvent) PossibleActions() core.Qt__DropAction {
	if ptr.Pointer() != nil {
		return core.Qt__DropAction(C.QDropEvent_PossibleActions(ptr.Pointer()))
	}
	return 0
}

func (ptr *QDropEvent) ProposedAction() core.Qt__DropAction {
	if ptr.Pointer() != nil {
		return core.Qt__DropAction(C.QDropEvent_ProposedAction(ptr.Pointer()))
	}
	return 0
}

func (ptr *QDropEvent) SetDropAction(action core.Qt__DropAction) {
	if ptr.Pointer() != nil {
		C.QDropEvent_SetDropAction(ptr.Pointer(), C.longlong(action))
	}
}

func (ptr *QDropEvent) Source() *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QDropEvent_Source(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

type QEnterEvent struct {
	core.QEvent
}

type QEnterEvent_ITF interface {
	core.QEvent_ITF
	QEnterEvent_PTR() *QEnterEvent
}

func (ptr *QEnterEvent) QEnterEvent_PTR() *QEnterEvent {
	return ptr
}

func (ptr *QEnterEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QEnterEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQEnterEvent(ptr QEnterEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QEnterEvent_PTR().Pointer()
	}
	return nil
}

func NewQEnterEventFromPointer(ptr unsafe.Pointer) (n *QEnterEvent) {
	n = new(QEnterEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QEnterEvent) DestroyQEnterEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQEnterEvent(localPos core.QPointF_ITF, windowPos core.QPointF_ITF, screenPos core.QPointF_ITF) *QEnterEvent {
	tmpValue := NewQEnterEventFromPointer(C.QEnterEvent_NewQEnterEvent(core.PointerFromQPointF(localPos), core.PointerFromQPointF(windowPos), core.PointerFromQPointF(screenPos)))
	qt.SetFinalizer(tmpValue, (*QEnterEvent).DestroyQEnterEvent)
	return tmpValue
}

func (ptr *QEnterEvent) GlobalPos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QEnterEvent_GlobalPos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QEnterEvent) GlobalX() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QEnterEvent_GlobalX(ptr.Pointer())))
	}
	return 0
}

func (ptr *QEnterEvent) GlobalY() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QEnterEvent_GlobalY(ptr.Pointer())))
	}
	return 0
}

func (ptr *QEnterEvent) LocalPos() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QEnterEvent_LocalPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QEnterEvent) Pos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QEnterEvent_Pos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QEnterEvent) ScreenPos() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QEnterEvent_ScreenPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QEnterEvent) WindowPos() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QEnterEvent_WindowPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QEnterEvent) X() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QEnterEvent_X(ptr.Pointer())))
	}
	return 0
}

func (ptr *QEnterEvent) Y() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QEnterEvent_Y(ptr.Pointer())))
	}
	return 0
}

type QExposeEvent struct {
	core.QEvent
}

type QExposeEvent_ITF interface {
	core.QEvent_ITF
	QExposeEvent_PTR() *QExposeEvent
}

func (ptr *QExposeEvent) QExposeEvent_PTR() *QExposeEvent {
	return ptr
}

func (ptr *QExposeEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QExposeEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQExposeEvent(ptr QExposeEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QExposeEvent_PTR().Pointer()
	}
	return nil
}

func NewQExposeEventFromPointer(ptr unsafe.Pointer) (n *QExposeEvent) {
	n = new(QExposeEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QExposeEvent) DestroyQExposeEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQExposeEvent(exposeRegion QRegion_ITF) *QExposeEvent {
	tmpValue := NewQExposeEventFromPointer(C.QExposeEvent_NewQExposeEvent(PointerFromQRegion(exposeRegion)))
	qt.SetFinalizer(tmpValue, (*QExposeEvent).DestroyQExposeEvent)
	return tmpValue
}

func (ptr *QExposeEvent) Region() *QRegion {
	if ptr.Pointer() != nil {
		return NewQRegionFromPointer(C.QExposeEvent_Region(ptr.Pointer()))
	}
	return nil
}

type QFileDialogOptions struct {
	ptr unsafe.Pointer
}

type QFileDialogOptions_ITF interface {
	QFileDialogOptions_PTR() *QFileDialogOptions
}

func (ptr *QFileDialogOptions) QFileDialogOptions_PTR() *QFileDialogOptions {
	return ptr
}

func (ptr *QFileDialogOptions) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QFileDialogOptions) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQFileDialogOptions(ptr QFileDialogOptions_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QFileDialogOptions_PTR().Pointer()
	}
	return nil
}

func NewQFileDialogOptionsFromPointer(ptr unsafe.Pointer) (n *QFileDialogOptions) {
	n = new(QFileDialogOptions)
	n.SetPointer(ptr)
	return
}
func (ptr *QFileDialogOptions) DestroyQFileDialogOptions() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QFileOpenEvent struct {
	core.QEvent
}

type QFileOpenEvent_ITF interface {
	core.QEvent_ITF
	QFileOpenEvent_PTR() *QFileOpenEvent
}

func (ptr *QFileOpenEvent) QFileOpenEvent_PTR() *QFileOpenEvent {
	return ptr
}

func (ptr *QFileOpenEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QFileOpenEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQFileOpenEvent(ptr QFileOpenEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QFileOpenEvent_PTR().Pointer()
	}
	return nil
}

func NewQFileOpenEventFromPointer(ptr unsafe.Pointer) (n *QFileOpenEvent) {
	n = new(QFileOpenEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QFileOpenEvent) DestroyQFileOpenEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func (ptr *QFileOpenEvent) File() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QFileOpenEvent_File(ptr.Pointer()))
	}
	return ""
}

func (ptr *QFileOpenEvent) OpenFile(file core.QFile_ITF, flags core.QIODevice__OpenModeFlag) bool {
	if ptr.Pointer() != nil {
		return int8(C.QFileOpenEvent_OpenFile(ptr.Pointer(), core.PointerFromQFile(file), C.longlong(flags))) != 0
	}
	return false
}

func (ptr *QFileOpenEvent) Url() *core.QUrl {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQUrlFromPointer(C.QFileOpenEvent_Url(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QUrl).DestroyQUrl)
		return tmpValue
	}
	return nil
}

type QFocusEvent struct {
	core.QEvent
}

type QFocusEvent_ITF interface {
	core.QEvent_ITF
	QFocusEvent_PTR() *QFocusEvent
}

func (ptr *QFocusEvent) QFocusEvent_PTR() *QFocusEvent {
	return ptr
}

func (ptr *QFocusEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QFocusEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQFocusEvent(ptr QFocusEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QFocusEvent_PTR().Pointer()
	}
	return nil
}

func NewQFocusEventFromPointer(ptr unsafe.Pointer) (n *QFocusEvent) {
	n = new(QFocusEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QFocusEvent) DestroyQFocusEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQFocusEvent(ty core.QEvent__Type, reason core.Qt__FocusReason) *QFocusEvent {
	tmpValue := NewQFocusEventFromPointer(C.QFocusEvent_NewQFocusEvent(C.longlong(ty), C.longlong(reason)))
	qt.SetFinalizer(tmpValue, (*QFocusEvent).DestroyQFocusEvent)
	return tmpValue
}

func (ptr *QFocusEvent) GotFocus() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFocusEvent_GotFocus(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFocusEvent) LostFocus() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFocusEvent_LostFocus(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFocusEvent) Reason() core.Qt__FocusReason {
	if ptr.Pointer() != nil {
		return core.Qt__FocusReason(C.QFocusEvent_Reason(ptr.Pointer()))
	}
	return 0
}

type QFont struct {
	ptr unsafe.Pointer
}

type QFont_ITF interface {
	QFont_PTR() *QFont
}

func (ptr *QFont) QFont_PTR() *QFont {
	return ptr
}

func (ptr *QFont) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QFont) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQFont(ptr QFont_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QFont_PTR().Pointer()
	}
	return nil
}

func NewQFontFromPointer(ptr unsafe.Pointer) (n *QFont) {
	n = new(QFont)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QFont__StyleHint
//QFont::StyleHint
type QFont__StyleHint int64

var (
	QFont__Helvetica  QFont__StyleHint = QFont__StyleHint(0)
	QFont__SansSerif  QFont__StyleHint = QFont__StyleHint(QFont__Helvetica)
	QFont__Times      QFont__StyleHint = QFont__StyleHint(1)
	QFont__Serif      QFont__StyleHint = QFont__StyleHint(QFont__Times)
	QFont__Courier    QFont__StyleHint = QFont__StyleHint(2)
	QFont__TypeWriter QFont__StyleHint = QFont__StyleHint(QFont__Courier)
	QFont__OldEnglish QFont__StyleHint = QFont__StyleHint(3)
	QFont__Decorative QFont__StyleHint = QFont__StyleHint(QFont__OldEnglish)
	QFont__System     QFont__StyleHint = QFont__StyleHint(4)
	QFont__AnyStyle   QFont__StyleHint = QFont__StyleHint(5)
	QFont__Cursive    QFont__StyleHint = QFont__StyleHint(6)
	QFont__Monospace  QFont__StyleHint = QFont__StyleHint(7)
	QFont__Fantasy    QFont__StyleHint = QFont__StyleHint(8)
)

//go:generate stringer -type=QFont__StyleStrategy
//QFont::StyleStrategy
type QFont__StyleStrategy int64

var (
	QFont__PreferDefault       QFont__StyleStrategy = QFont__StyleStrategy(0x0001)
	QFont__PreferBitmap        QFont__StyleStrategy = QFont__StyleStrategy(0x0002)
	QFont__PreferDevice        QFont__StyleStrategy = QFont__StyleStrategy(0x0004)
	QFont__PreferOutline       QFont__StyleStrategy = QFont__StyleStrategy(0x0008)
	QFont__ForceOutline        QFont__StyleStrategy = QFont__StyleStrategy(0x0010)
	QFont__PreferMatch         QFont__StyleStrategy = QFont__StyleStrategy(0x0020)
	QFont__PreferQuality       QFont__StyleStrategy = QFont__StyleStrategy(0x0040)
	QFont__PreferAntialias     QFont__StyleStrategy = QFont__StyleStrategy(0x0080)
	QFont__NoAntialias         QFont__StyleStrategy = QFont__StyleStrategy(0x0100)
	QFont__OpenGLCompatible    QFont__StyleStrategy = QFont__StyleStrategy(0x0200)
	QFont__ForceIntegerMetrics QFont__StyleStrategy = QFont__StyleStrategy(0x0400)
	QFont__NoSubpixelAntialias QFont__StyleStrategy = QFont__StyleStrategy(0x0800)
	QFont__PreferNoShaping     QFont__StyleStrategy = QFont__StyleStrategy(0x1000)
	QFont__NoFontMerging       QFont__StyleStrategy = QFont__StyleStrategy(0x8000)
)

//go:generate stringer -type=QFont__HintingPreference
//QFont::HintingPreference
type QFont__HintingPreference int64

const (
	QFont__PreferDefaultHinting  QFont__HintingPreference = QFont__HintingPreference(0)
	QFont__PreferNoHinting       QFont__HintingPreference = QFont__HintingPreference(1)
	QFont__PreferVerticalHinting QFont__HintingPreference = QFont__HintingPreference(2)
	QFont__PreferFullHinting     QFont__HintingPreference = QFont__HintingPreference(3)
)

//go:generate stringer -type=QFont__Weight
//QFont::Weight
type QFont__Weight int64

const (
	QFont__Thin       QFont__Weight = QFont__Weight(0)
	QFont__ExtraLight QFont__Weight = QFont__Weight(12)
	QFont__Light      QFont__Weight = QFont__Weight(25)
	QFont__Normal     QFont__Weight = QFont__Weight(50)
	QFont__Medium     QFont__Weight = QFont__Weight(57)
	QFont__DemiBold   QFont__Weight = QFont__Weight(63)
	QFont__Bold       QFont__Weight = QFont__Weight(75)
	QFont__ExtraBold  QFont__Weight = QFont__Weight(81)
	QFont__Black      QFont__Weight = QFont__Weight(87)
)

//go:generate stringer -type=QFont__Style
//QFont::Style
type QFont__Style int64

var (
	QFont__StyleNormal  QFont__Style = QFont__Style(0)
	QFont__StyleItalic  QFont__Style = QFont__Style(1)
	QFont__StyleOblique QFont__Style = QFont__Style(2)
)

//go:generate stringer -type=QFont__Stretch
//QFont::Stretch
type QFont__Stretch int64

const (
	QFont__AnyStretch     QFont__Stretch = QFont__Stretch(0)
	QFont__UltraCondensed QFont__Stretch = QFont__Stretch(50)
	QFont__ExtraCondensed QFont__Stretch = QFont__Stretch(62)
	QFont__Condensed      QFont__Stretch = QFont__Stretch(75)
	QFont__SemiCondensed  QFont__Stretch = QFont__Stretch(87)
	QFont__Unstretched    QFont__Stretch = QFont__Stretch(100)
	QFont__SemiExpanded   QFont__Stretch = QFont__Stretch(112)
	QFont__Expanded       QFont__Stretch = QFont__Stretch(125)
	QFont__ExtraExpanded  QFont__Stretch = QFont__Stretch(150)
	QFont__UltraExpanded  QFont__Stretch = QFont__Stretch(200)
)

//go:generate stringer -type=QFont__Capitalization
//QFont::Capitalization
type QFont__Capitalization int64

const (
	QFont__MixedCase    QFont__Capitalization = QFont__Capitalization(0)
	QFont__AllUppercase QFont__Capitalization = QFont__Capitalization(1)
	QFont__AllLowercase QFont__Capitalization = QFont__Capitalization(2)
	QFont__SmallCaps    QFont__Capitalization = QFont__Capitalization(3)
	QFont__Capitalize   QFont__Capitalization = QFont__Capitalization(4)
)

//go:generate stringer -type=QFont__SpacingType
//QFont::SpacingType
type QFont__SpacingType int64

const (
	QFont__PercentageSpacing QFont__SpacingType = QFont__SpacingType(0)
	QFont__AbsoluteSpacing   QFont__SpacingType = QFont__SpacingType(1)
)

func NewQFont() *QFont {
	tmpValue := NewQFontFromPointer(C.QFont_NewQFont())
	qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
	return tmpValue
}

func NewQFont2(family string, pointSize int, weight int, italic bool) *QFont {
	var familyC *C.char
	if family != "" {
		familyC = C.CString(family)
		defer C.free(unsafe.Pointer(familyC))
	}
	tmpValue := NewQFontFromPointer(C.QFont_NewQFont2(C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.int(int32(pointSize)), C.int(int32(weight)), C.char(int8(qt.GoBoolToInt(italic)))))
	qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
	return tmpValue
}

func NewQFont4(font QFont_ITF, pd QPaintDevice_ITF) *QFont {
	tmpValue := NewQFontFromPointer(C.QFont_NewQFont4(PointerFromQFont(font), PointerFromQPaintDevice(pd)))
	qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
	return tmpValue
}

func NewQFont5(font QFont_ITF) *QFont {
	tmpValue := NewQFontFromPointer(C.QFont_NewQFont5(PointerFromQFont(font)))
	qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
	return tmpValue
}

func (ptr *QFont) Bold() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFont_Bold(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFont) Capitalization() QFont__Capitalization {
	if ptr.Pointer() != nil {
		return QFont__Capitalization(C.QFont_Capitalization(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFont) DefaultFamily() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QFont_DefaultFamily(ptr.Pointer()))
	}
	return ""
}

func (ptr *QFont) ExactMatch() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFont_ExactMatch(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFont) Families() []string {
	if ptr.Pointer() != nil {
		return unpackStringList(cGoUnpackString(C.QFont_Families(ptr.Pointer())))
	}
	return make([]string, 0)
}

func (ptr *QFont) Family() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QFont_Family(ptr.Pointer()))
	}
	return ""
}

func (ptr *QFont) FixedPitch() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFont_FixedPitch(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFont) FromString(descrip string) bool {
	if ptr.Pointer() != nil {
		var descripC *C.char
		if descrip != "" {
			descripC = C.CString(descrip)
			defer C.free(unsafe.Pointer(descripC))
		}
		return int8(C.QFont_FromString(ptr.Pointer(), C.struct_QtGui_PackedString{data: descripC, len: C.longlong(len(descrip))})) != 0
	}
	return false
}

func (ptr *QFont) HintingPreference() QFont__HintingPreference {
	if ptr.Pointer() != nil {
		return QFont__HintingPreference(C.QFont_HintingPreference(ptr.Pointer()))
	}
	return 0
}

func QFont_InsertSubstitution(familyName string, substituteName string) {
	var familyNameC *C.char
	if familyName != "" {
		familyNameC = C.CString(familyName)
		defer C.free(unsafe.Pointer(familyNameC))
	}
	var substituteNameC *C.char
	if substituteName != "" {
		substituteNameC = C.CString(substituteName)
		defer C.free(unsafe.Pointer(substituteNameC))
	}
	C.QFont_QFont_InsertSubstitution(C.struct_QtGui_PackedString{data: familyNameC, len: C.longlong(len(familyName))}, C.struct_QtGui_PackedString{data: substituteNameC, len: C.longlong(len(substituteName))})
}

func (ptr *QFont) InsertSubstitution(familyName string, substituteName string) {
	var familyNameC *C.char
	if familyName != "" {
		familyNameC = C.CString(familyName)
		defer C.free(unsafe.Pointer(familyNameC))
	}
	var substituteNameC *C.char
	if substituteName != "" {
		substituteNameC = C.CString(substituteName)
		defer C.free(unsafe.Pointer(substituteNameC))
	}
	C.QFont_QFont_InsertSubstitution(C.struct_QtGui_PackedString{data: familyNameC, len: C.longlong(len(familyName))}, C.struct_QtGui_PackedString{data: substituteNameC, len: C.longlong(len(substituteName))})
}

func QFont_InsertSubstitutions(familyName string, substituteNames []string) {
	var familyNameC *C.char
	if familyName != "" {
		familyNameC = C.CString(familyName)
		defer C.free(unsafe.Pointer(familyNameC))
	}
	substituteNamesC := C.CString(strings.Join(substituteNames, "¡¦!"))
	defer C.free(unsafe.Pointer(substituteNamesC))
	C.QFont_QFont_InsertSubstitutions(C.struct_QtGui_PackedString{data: familyNameC, len: C.longlong(len(familyName))}, C.struct_QtGui_PackedString{data: substituteNamesC, len: C.longlong(len(strings.Join(substituteNames, "¡¦!")))})
}

func (ptr *QFont) InsertSubstitutions(familyName string, substituteNames []string) {
	var familyNameC *C.char
	if familyName != "" {
		familyNameC = C.CString(familyName)
		defer C.free(unsafe.Pointer(familyNameC))
	}
	substituteNamesC := C.CString(strings.Join(substituteNames, "¡¦!"))
	defer C.free(unsafe.Pointer(substituteNamesC))
	C.QFont_QFont_InsertSubstitutions(C.struct_QtGui_PackedString{data: familyNameC, len: C.longlong(len(familyName))}, C.struct_QtGui_PackedString{data: substituteNamesC, len: C.longlong(len(strings.Join(substituteNames, "¡¦!")))})
}

func (ptr *QFont) IsCopyOf(ff QFont_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QFont_IsCopyOf(ptr.Pointer(), PointerFromQFont(ff))) != 0
	}
	return false
}

func (ptr *QFont) Italic() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFont_Italic(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFont) Kerning() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFont_Kerning(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFont) Key() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QFont_Key(ptr.Pointer()))
	}
	return ""
}

func (ptr *QFont) LetterSpacing() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFont_LetterSpacing(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFont) LetterSpacingType() QFont__SpacingType {
	if ptr.Pointer() != nil {
		return QFont__SpacingType(C.QFont_LetterSpacingType(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFont) Overline() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFont_Overline(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFont) PixelSize() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFont_PixelSize(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFont) PointSize() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFont_PointSize(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFont) PointSizeF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFont_PointSizeF(ptr.Pointer()))
	}
	return 0
}

func QFont_RemoveSubstitutions(familyName string) {
	var familyNameC *C.char
	if familyName != "" {
		familyNameC = C.CString(familyName)
		defer C.free(unsafe.Pointer(familyNameC))
	}
	C.QFont_QFont_RemoveSubstitutions(C.struct_QtGui_PackedString{data: familyNameC, len: C.longlong(len(familyName))})
}

func (ptr *QFont) RemoveSubstitutions(familyName string) {
	var familyNameC *C.char
	if familyName != "" {
		familyNameC = C.CString(familyName)
		defer C.free(unsafe.Pointer(familyNameC))
	}
	C.QFont_QFont_RemoveSubstitutions(C.struct_QtGui_PackedString{data: familyNameC, len: C.longlong(len(familyName))})
}

func (ptr *QFont) Resolve(other QFont_ITF) *QFont {
	if ptr.Pointer() != nil {
		tmpValue := NewQFontFromPointer(C.QFont_Resolve(ptr.Pointer(), PointerFromQFont(other)))
		qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
		return tmpValue
	}
	return nil
}

func (ptr *QFont) SetBold(enable bool) {
	if ptr.Pointer() != nil {
		C.QFont_SetBold(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QFont) SetCapitalization(caps QFont__Capitalization) {
	if ptr.Pointer() != nil {
		C.QFont_SetCapitalization(ptr.Pointer(), C.longlong(caps))
	}
}

func (ptr *QFont) SetFamilies(families []string) {
	if ptr.Pointer() != nil {
		familiesC := C.CString(strings.Join(families, "¡¦!"))
		defer C.free(unsafe.Pointer(familiesC))
		C.QFont_SetFamilies(ptr.Pointer(), C.struct_QtGui_PackedString{data: familiesC, len: C.longlong(len(strings.Join(families, "¡¦!")))})
	}
}

func (ptr *QFont) SetFamily(family string) {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		C.QFont_SetFamily(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))})
	}
}

func (ptr *QFont) SetFixedPitch(enable bool) {
	if ptr.Pointer() != nil {
		C.QFont_SetFixedPitch(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QFont) SetHintingPreference(hintingPreference QFont__HintingPreference) {
	if ptr.Pointer() != nil {
		C.QFont_SetHintingPreference(ptr.Pointer(), C.longlong(hintingPreference))
	}
}

func (ptr *QFont) SetItalic(enable bool) {
	if ptr.Pointer() != nil {
		C.QFont_SetItalic(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QFont) SetKerning(enable bool) {
	if ptr.Pointer() != nil {
		C.QFont_SetKerning(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QFont) SetLetterSpacing(ty QFont__SpacingType, spacing float64) {
	if ptr.Pointer() != nil {
		C.QFont_SetLetterSpacing(ptr.Pointer(), C.longlong(ty), C.double(spacing))
	}
}

func (ptr *QFont) SetOverline(enable bool) {
	if ptr.Pointer() != nil {
		C.QFont_SetOverline(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QFont) SetPixelSize(pixelSize int) {
	if ptr.Pointer() != nil {
		C.QFont_SetPixelSize(ptr.Pointer(), C.int(int32(pixelSize)))
	}
}

func (ptr *QFont) SetPointSize(pointSize int) {
	if ptr.Pointer() != nil {
		C.QFont_SetPointSize(ptr.Pointer(), C.int(int32(pointSize)))
	}
}

func (ptr *QFont) SetPointSizeF(pointSize float64) {
	if ptr.Pointer() != nil {
		C.QFont_SetPointSizeF(ptr.Pointer(), C.double(pointSize))
	}
}

func (ptr *QFont) SetStretch(factor int) {
	if ptr.Pointer() != nil {
		C.QFont_SetStretch(ptr.Pointer(), C.int(int32(factor)))
	}
}

func (ptr *QFont) SetStrikeOut(enable bool) {
	if ptr.Pointer() != nil {
		C.QFont_SetStrikeOut(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QFont) SetStyle(style QFont__Style) {
	if ptr.Pointer() != nil {
		C.QFont_SetStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QFont) SetStyleHint(hint QFont__StyleHint, strategy QFont__StyleStrategy) {
	if ptr.Pointer() != nil {
		C.QFont_SetStyleHint(ptr.Pointer(), C.longlong(hint), C.longlong(strategy))
	}
}

func (ptr *QFont) SetStyleName(styleName string) {
	if ptr.Pointer() != nil {
		var styleNameC *C.char
		if styleName != "" {
			styleNameC = C.CString(styleName)
			defer C.free(unsafe.Pointer(styleNameC))
		}
		C.QFont_SetStyleName(ptr.Pointer(), C.struct_QtGui_PackedString{data: styleNameC, len: C.longlong(len(styleName))})
	}
}

func (ptr *QFont) SetStyleStrategy(s QFont__StyleStrategy) {
	if ptr.Pointer() != nil {
		C.QFont_SetStyleStrategy(ptr.Pointer(), C.longlong(s))
	}
}

func (ptr *QFont) SetUnderline(enable bool) {
	if ptr.Pointer() != nil {
		C.QFont_SetUnderline(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QFont) SetWeight(weight int) {
	if ptr.Pointer() != nil {
		C.QFont_SetWeight(ptr.Pointer(), C.int(int32(weight)))
	}
}

func (ptr *QFont) SetWordSpacing(spacing float64) {
	if ptr.Pointer() != nil {
		C.QFont_SetWordSpacing(ptr.Pointer(), C.double(spacing))
	}
}

func (ptr *QFont) Stretch() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFont_Stretch(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFont) StrikeOut() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFont_StrikeOut(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFont) Style() QFont__Style {
	if ptr.Pointer() != nil {
		return QFont__Style(C.QFont_Style(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFont) StyleHint() QFont__StyleHint {
	if ptr.Pointer() != nil {
		return QFont__StyleHint(C.QFont_StyleHint(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFont) StyleName() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QFont_StyleName(ptr.Pointer()))
	}
	return ""
}

func (ptr *QFont) StyleStrategy() QFont__StyleStrategy {
	if ptr.Pointer() != nil {
		return QFont__StyleStrategy(C.QFont_StyleStrategy(ptr.Pointer()))
	}
	return 0
}

func QFont_Substitute(familyName string) string {
	var familyNameC *C.char
	if familyName != "" {
		familyNameC = C.CString(familyName)
		defer C.free(unsafe.Pointer(familyNameC))
	}
	return cGoUnpackString(C.QFont_QFont_Substitute(C.struct_QtGui_PackedString{data: familyNameC, len: C.longlong(len(familyName))}))
}

func (ptr *QFont) Substitute(familyName string) string {
	var familyNameC *C.char
	if familyName != "" {
		familyNameC = C.CString(familyName)
		defer C.free(unsafe.Pointer(familyNameC))
	}
	return cGoUnpackString(C.QFont_QFont_Substitute(C.struct_QtGui_PackedString{data: familyNameC, len: C.longlong(len(familyName))}))
}

func QFont_Substitutes(familyName string) []string {
	var familyNameC *C.char
	if familyName != "" {
		familyNameC = C.CString(familyName)
		defer C.free(unsafe.Pointer(familyNameC))
	}
	return unpackStringList(cGoUnpackString(C.QFont_QFont_Substitutes(C.struct_QtGui_PackedString{data: familyNameC, len: C.longlong(len(familyName))})))
}

func (ptr *QFont) Substitutes(familyName string) []string {
	var familyNameC *C.char
	if familyName != "" {
		familyNameC = C.CString(familyName)
		defer C.free(unsafe.Pointer(familyNameC))
	}
	return unpackStringList(cGoUnpackString(C.QFont_QFont_Substitutes(C.struct_QtGui_PackedString{data: familyNameC, len: C.longlong(len(familyName))})))
}

func QFont_Substitutions() []string {
	return unpackStringList(cGoUnpackString(C.QFont_QFont_Substitutions()))
}

func (ptr *QFont) Substitutions() []string {
	return unpackStringList(cGoUnpackString(C.QFont_QFont_Substitutions()))
}

func (ptr *QFont) Swap(other QFont_ITF) {
	if ptr.Pointer() != nil {
		C.QFont_Swap(ptr.Pointer(), PointerFromQFont(other))
	}
}

func (ptr *QFont) ToString() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QFont_ToString(ptr.Pointer()))
	}
	return ""
}

func (ptr *QFont) Underline() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFont_Underline(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFont) Weight() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFont_Weight(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFont) WordSpacing() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFont_WordSpacing(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFont) DestroyQFont() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QFont_DestroyQFont(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QFont) ToVariant() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QFont_ToVariant(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

type QFontDatabase struct {
	ptr unsafe.Pointer
}

type QFontDatabase_ITF interface {
	QFontDatabase_PTR() *QFontDatabase
}

func (ptr *QFontDatabase) QFontDatabase_PTR() *QFontDatabase {
	return ptr
}

func (ptr *QFontDatabase) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QFontDatabase) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQFontDatabase(ptr QFontDatabase_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QFontDatabase_PTR().Pointer()
	}
	return nil
}

func NewQFontDatabaseFromPointer(ptr unsafe.Pointer) (n *QFontDatabase) {
	n = new(QFontDatabase)
	n.SetPointer(ptr)
	return
}
func (ptr *QFontDatabase) DestroyQFontDatabase() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QFontDatabase__WritingSystem
//QFontDatabase::WritingSystem
type QFontDatabase__WritingSystem int64

const (
	QFontDatabase__Any                 QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(0)
	QFontDatabase__Latin               QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(1)
	QFontDatabase__Greek               QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(2)
	QFontDatabase__Cyrillic            QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(3)
	QFontDatabase__Armenian            QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(4)
	QFontDatabase__Hebrew              QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(5)
	QFontDatabase__Arabic              QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(6)
	QFontDatabase__Syriac              QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(7)
	QFontDatabase__Thaana              QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(8)
	QFontDatabase__Devanagari          QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(9)
	QFontDatabase__Bengali             QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(10)
	QFontDatabase__Gurmukhi            QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(11)
	QFontDatabase__Gujarati            QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(12)
	QFontDatabase__Oriya               QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(13)
	QFontDatabase__Tamil               QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(14)
	QFontDatabase__Telugu              QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(15)
	QFontDatabase__Kannada             QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(16)
	QFontDatabase__Malayalam           QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(17)
	QFontDatabase__Sinhala             QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(18)
	QFontDatabase__Thai                QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(19)
	QFontDatabase__Lao                 QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(20)
	QFontDatabase__Tibetan             QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(21)
	QFontDatabase__Myanmar             QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(22)
	QFontDatabase__Georgian            QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(23)
	QFontDatabase__Khmer               QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(24)
	QFontDatabase__SimplifiedChinese   QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(25)
	QFontDatabase__TraditionalChinese  QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(26)
	QFontDatabase__Japanese            QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(27)
	QFontDatabase__Korean              QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(28)
	QFontDatabase__Vietnamese          QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(29)
	QFontDatabase__Symbol              QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(30)
	QFontDatabase__Other               QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(QFontDatabase__Symbol)
	QFontDatabase__Ogham               QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(31)
	QFontDatabase__Runic               QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(32)
	QFontDatabase__Nko                 QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(33)
	QFontDatabase__WritingSystemsCount QFontDatabase__WritingSystem = QFontDatabase__WritingSystem(34)
)

//go:generate stringer -type=QFontDatabase__SystemFont
//QFontDatabase::SystemFont
type QFontDatabase__SystemFont int64

const (
	QFontDatabase__GeneralFont          QFontDatabase__SystemFont = QFontDatabase__SystemFont(0)
	QFontDatabase__FixedFont            QFontDatabase__SystemFont = QFontDatabase__SystemFont(1)
	QFontDatabase__TitleFont            QFontDatabase__SystemFont = QFontDatabase__SystemFont(2)
	QFontDatabase__SmallestReadableFont QFontDatabase__SystemFont = QFontDatabase__SystemFont(3)
)

func NewQFontDatabase() *QFontDatabase {
	tmpValue := NewQFontDatabaseFromPointer(C.QFontDatabase_NewQFontDatabase())
	qt.SetFinalizer(tmpValue, (*QFontDatabase).DestroyQFontDatabase)
	return tmpValue
}

func QFontDatabase_AddApplicationFont(fileName string) int {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	return int(int32(C.QFontDatabase_QFontDatabase_AddApplicationFont(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))})))
}

func (ptr *QFontDatabase) AddApplicationFont(fileName string) int {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	return int(int32(C.QFontDatabase_QFontDatabase_AddApplicationFont(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))})))
}

func QFontDatabase_AddApplicationFontFromData(fontData core.QByteArray_ITF) int {
	return int(int32(C.QFontDatabase_QFontDatabase_AddApplicationFontFromData(core.PointerFromQByteArray(fontData))))
}

func (ptr *QFontDatabase) AddApplicationFontFromData(fontData core.QByteArray_ITF) int {
	return int(int32(C.QFontDatabase_QFontDatabase_AddApplicationFontFromData(core.PointerFromQByteArray(fontData))))
}

func QFontDatabase_ApplicationFontFamilies(id int) []string {
	return unpackStringList(cGoUnpackString(C.QFontDatabase_QFontDatabase_ApplicationFontFamilies(C.int(int32(id)))))
}

func (ptr *QFontDatabase) ApplicationFontFamilies(id int) []string {
	return unpackStringList(cGoUnpackString(C.QFontDatabase_QFontDatabase_ApplicationFontFamilies(C.int(int32(id)))))
}

func (ptr *QFontDatabase) Bold(family string, style string) bool {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		var styleC *C.char
		if style != "" {
			styleC = C.CString(style)
			defer C.free(unsafe.Pointer(styleC))
		}
		return int8(C.QFontDatabase_Bold(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.struct_QtGui_PackedString{data: styleC, len: C.longlong(len(style))})) != 0
	}
	return false
}

func (ptr *QFontDatabase) Families(writingSystem QFontDatabase__WritingSystem) []string {
	if ptr.Pointer() != nil {
		return unpackStringList(cGoUnpackString(C.QFontDatabase_Families(ptr.Pointer(), C.longlong(writingSystem))))
	}
	return make([]string, 0)
}

func (ptr *QFontDatabase) Font(family string, style string, pointSize int) *QFont {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		var styleC *C.char
		if style != "" {
			styleC = C.CString(style)
			defer C.free(unsafe.Pointer(styleC))
		}
		tmpValue := NewQFontFromPointer(C.QFontDatabase_Font(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.struct_QtGui_PackedString{data: styleC, len: C.longlong(len(style))}, C.int(int32(pointSize))))
		qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
		return tmpValue
	}
	return nil
}

func (ptr *QFontDatabase) IsBitmapScalable(family string, style string) bool {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		var styleC *C.char
		if style != "" {
			styleC = C.CString(style)
			defer C.free(unsafe.Pointer(styleC))
		}
		return int8(C.QFontDatabase_IsBitmapScalable(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.struct_QtGui_PackedString{data: styleC, len: C.longlong(len(style))})) != 0
	}
	return false
}

func (ptr *QFontDatabase) IsFixedPitch(family string, style string) bool {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		var styleC *C.char
		if style != "" {
			styleC = C.CString(style)
			defer C.free(unsafe.Pointer(styleC))
		}
		return int8(C.QFontDatabase_IsFixedPitch(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.struct_QtGui_PackedString{data: styleC, len: C.longlong(len(style))})) != 0
	}
	return false
}

func (ptr *QFontDatabase) IsPrivateFamily(family string) bool {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		return int8(C.QFontDatabase_IsPrivateFamily(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))})) != 0
	}
	return false
}

func (ptr *QFontDatabase) IsScalable(family string, style string) bool {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		var styleC *C.char
		if style != "" {
			styleC = C.CString(style)
			defer C.free(unsafe.Pointer(styleC))
		}
		return int8(C.QFontDatabase_IsScalable(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.struct_QtGui_PackedString{data: styleC, len: C.longlong(len(style))})) != 0
	}
	return false
}

func (ptr *QFontDatabase) IsSmoothlyScalable(family string, style string) bool {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		var styleC *C.char
		if style != "" {
			styleC = C.CString(style)
			defer C.free(unsafe.Pointer(styleC))
		}
		return int8(C.QFontDatabase_IsSmoothlyScalable(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.struct_QtGui_PackedString{data: styleC, len: C.longlong(len(style))})) != 0
	}
	return false
}

func (ptr *QFontDatabase) Italic(family string, style string) bool {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		var styleC *C.char
		if style != "" {
			styleC = C.CString(style)
			defer C.free(unsafe.Pointer(styleC))
		}
		return int8(C.QFontDatabase_Italic(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.struct_QtGui_PackedString{data: styleC, len: C.longlong(len(style))})) != 0
	}
	return false
}

func (ptr *QFontDatabase) PointSizes(family string, styleName string) []int {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		var styleNameC *C.char
		if styleName != "" {
			styleNameC = C.CString(styleName)
			defer C.free(unsafe.Pointer(styleNameC))
		}
		return func(l C.struct_QtGui_PackedList) []int {
			out := make([]int, int(l.len))
			tmpList := NewQFontDatabaseFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__pointSizes_atList(i)
			}
			return out
		}(C.QFontDatabase_PointSizes(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.struct_QtGui_PackedString{data: styleNameC, len: C.longlong(len(styleName))}))
	}
	return make([]int, 0)
}

func QFontDatabase_RemoveAllApplicationFonts() bool {
	return int8(C.QFontDatabase_QFontDatabase_RemoveAllApplicationFonts()) != 0
}

func (ptr *QFontDatabase) RemoveAllApplicationFonts() bool {
	return int8(C.QFontDatabase_QFontDatabase_RemoveAllApplicationFonts()) != 0
}

func QFontDatabase_RemoveApplicationFont(id int) bool {
	return int8(C.QFontDatabase_QFontDatabase_RemoveApplicationFont(C.int(int32(id)))) != 0
}

func (ptr *QFontDatabase) RemoveApplicationFont(id int) bool {
	return int8(C.QFontDatabase_QFontDatabase_RemoveApplicationFont(C.int(int32(id)))) != 0
}

func (ptr *QFontDatabase) SmoothSizes(family string, styleName string) []int {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		var styleNameC *C.char
		if styleName != "" {
			styleNameC = C.CString(styleName)
			defer C.free(unsafe.Pointer(styleNameC))
		}
		return func(l C.struct_QtGui_PackedList) []int {
			out := make([]int, int(l.len))
			tmpList := NewQFontDatabaseFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__smoothSizes_atList(i)
			}
			return out
		}(C.QFontDatabase_SmoothSizes(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.struct_QtGui_PackedString{data: styleNameC, len: C.longlong(len(styleName))}))
	}
	return make([]int, 0)
}

func QFontDatabase_StandardSizes() []int {
	return func(l C.struct_QtGui_PackedList) []int {
		out := make([]int, int(l.len))
		tmpList := NewQFontDatabaseFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__standardSizes_atList(i)
		}
		return out
	}(C.QFontDatabase_QFontDatabase_StandardSizes())
}

func (ptr *QFontDatabase) StandardSizes() []int {
	return func(l C.struct_QtGui_PackedList) []int {
		out := make([]int, int(l.len))
		tmpList := NewQFontDatabaseFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__standardSizes_atList(i)
		}
		return out
	}(C.QFontDatabase_QFontDatabase_StandardSizes())
}

func (ptr *QFontDatabase) StyleString(font QFont_ITF) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QFontDatabase_StyleString(ptr.Pointer(), PointerFromQFont(font)))
	}
	return ""
}

func (ptr *QFontDatabase) StyleString2(fontInfo QFontInfo_ITF) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QFontDatabase_StyleString2(ptr.Pointer(), PointerFromQFontInfo(fontInfo)))
	}
	return ""
}

func (ptr *QFontDatabase) Styles(family string) []string {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		return unpackStringList(cGoUnpackString(C.QFontDatabase_Styles(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))})))
	}
	return make([]string, 0)
}

func QFontDatabase_SystemFont(ty QFontDatabase__SystemFont) *QFont {
	tmpValue := NewQFontFromPointer(C.QFontDatabase_QFontDatabase_SystemFont(C.longlong(ty)))
	qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
	return tmpValue
}

func (ptr *QFontDatabase) SystemFont(ty QFontDatabase__SystemFont) *QFont {
	tmpValue := NewQFontFromPointer(C.QFontDatabase_QFontDatabase_SystemFont(C.longlong(ty)))
	qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
	return tmpValue
}

func (ptr *QFontDatabase) Weight(family string, style string) int {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		var styleC *C.char
		if style != "" {
			styleC = C.CString(style)
			defer C.free(unsafe.Pointer(styleC))
		}
		return int(int32(C.QFontDatabase_Weight(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))}, C.struct_QtGui_PackedString{data: styleC, len: C.longlong(len(style))})))
	}
	return 0
}

func (ptr *QFontDatabase) __pointSizes_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontDatabase___pointSizes_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QFontDatabase) __pointSizes_setList(i int) {
	if ptr.Pointer() != nil {
		C.QFontDatabase___pointSizes_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QFontDatabase) __pointSizes_newList() unsafe.Pointer {
	return C.QFontDatabase___pointSizes_newList(ptr.Pointer())
}

func (ptr *QFontDatabase) __smoothSizes_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontDatabase___smoothSizes_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QFontDatabase) __smoothSizes_setList(i int) {
	if ptr.Pointer() != nil {
		C.QFontDatabase___smoothSizes_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QFontDatabase) __smoothSizes_newList() unsafe.Pointer {
	return C.QFontDatabase___smoothSizes_newList(ptr.Pointer())
}

func (ptr *QFontDatabase) __standardSizes_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontDatabase___standardSizes_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QFontDatabase) __standardSizes_setList(i int) {
	if ptr.Pointer() != nil {
		C.QFontDatabase___standardSizes_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QFontDatabase) __standardSizes_newList() unsafe.Pointer {
	return C.QFontDatabase___standardSizes_newList(ptr.Pointer())
}

type QFontDialogOptions struct {
	ptr unsafe.Pointer
}

type QFontDialogOptions_ITF interface {
	QFontDialogOptions_PTR() *QFontDialogOptions
}

func (ptr *QFontDialogOptions) QFontDialogOptions_PTR() *QFontDialogOptions {
	return ptr
}

func (ptr *QFontDialogOptions) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QFontDialogOptions) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQFontDialogOptions(ptr QFontDialogOptions_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QFontDialogOptions_PTR().Pointer()
	}
	return nil
}

func NewQFontDialogOptionsFromPointer(ptr unsafe.Pointer) (n *QFontDialogOptions) {
	n = new(QFontDialogOptions)
	n.SetPointer(ptr)
	return
}
func (ptr *QFontDialogOptions) DestroyQFontDialogOptions() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QFontInfo struct {
	ptr unsafe.Pointer
}

type QFontInfo_ITF interface {
	QFontInfo_PTR() *QFontInfo
}

func (ptr *QFontInfo) QFontInfo_PTR() *QFontInfo {
	return ptr
}

func (ptr *QFontInfo) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QFontInfo) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQFontInfo(ptr QFontInfo_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QFontInfo_PTR().Pointer()
	}
	return nil
}

func NewQFontInfoFromPointer(ptr unsafe.Pointer) (n *QFontInfo) {
	n = new(QFontInfo)
	n.SetPointer(ptr)
	return
}
func NewQFontInfo(font QFont_ITF) *QFontInfo {
	tmpValue := NewQFontInfoFromPointer(C.QFontInfo_NewQFontInfo(PointerFromQFont(font)))
	qt.SetFinalizer(tmpValue, (*QFontInfo).DestroyQFontInfo)
	return tmpValue
}

func NewQFontInfo2(fi QFontInfo_ITF) *QFontInfo {
	tmpValue := NewQFontInfoFromPointer(C.QFontInfo_NewQFontInfo2(PointerFromQFontInfo(fi)))
	qt.SetFinalizer(tmpValue, (*QFontInfo).DestroyQFontInfo)
	return tmpValue
}

func (ptr *QFontInfo) Bold() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFontInfo_Bold(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFontInfo) ExactMatch() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFontInfo_ExactMatch(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFontInfo) Family() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QFontInfo_Family(ptr.Pointer()))
	}
	return ""
}

func (ptr *QFontInfo) FixedPitch() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFontInfo_FixedPitch(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFontInfo) Italic() bool {
	if ptr.Pointer() != nil {
		return int8(C.QFontInfo_Italic(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QFontInfo) PixelSize() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontInfo_PixelSize(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontInfo) PointSize() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontInfo_PointSize(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontInfo) PointSizeF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontInfo_PointSizeF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontInfo) Style() QFont__Style {
	if ptr.Pointer() != nil {
		return QFont__Style(C.QFontInfo_Style(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontInfo) StyleHint() QFont__StyleHint {
	if ptr.Pointer() != nil {
		return QFont__StyleHint(C.QFontInfo_StyleHint(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontInfo) StyleName() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QFontInfo_StyleName(ptr.Pointer()))
	}
	return ""
}

func (ptr *QFontInfo) Swap(other QFontInfo_ITF) {
	if ptr.Pointer() != nil {
		C.QFontInfo_Swap(ptr.Pointer(), PointerFromQFontInfo(other))
	}
}

func (ptr *QFontInfo) Weight() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontInfo_Weight(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontInfo) DestroyQFontInfo() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QFontInfo_DestroyQFontInfo(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QFontMetrics struct {
	ptr unsafe.Pointer
}

type QFontMetrics_ITF interface {
	QFontMetrics_PTR() *QFontMetrics
}

func (ptr *QFontMetrics) QFontMetrics_PTR() *QFontMetrics {
	return ptr
}

func (ptr *QFontMetrics) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QFontMetrics) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQFontMetrics(ptr QFontMetrics_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QFontMetrics_PTR().Pointer()
	}
	return nil
}

func NewQFontMetricsFromPointer(ptr unsafe.Pointer) (n *QFontMetrics) {
	n = new(QFontMetrics)
	n.SetPointer(ptr)
	return
}
func NewQFontMetrics(font QFont_ITF) *QFontMetrics {
	tmpValue := NewQFontMetricsFromPointer(C.QFontMetrics_NewQFontMetrics(PointerFromQFont(font)))
	qt.SetFinalizer(tmpValue, (*QFontMetrics).DestroyQFontMetrics)
	return tmpValue
}

func NewQFontMetrics3(font QFont_ITF, paintdevice QPaintDevice_ITF) *QFontMetrics {
	tmpValue := NewQFontMetricsFromPointer(C.QFontMetrics_NewQFontMetrics3(PointerFromQFont(font), PointerFromQPaintDevice(paintdevice)))
	qt.SetFinalizer(tmpValue, (*QFontMetrics).DestroyQFontMetrics)
	return tmpValue
}

func NewQFontMetrics4(fm QFontMetrics_ITF) *QFontMetrics {
	tmpValue := NewQFontMetricsFromPointer(C.QFontMetrics_NewQFontMetrics4(PointerFromQFontMetrics(fm)))
	qt.SetFinalizer(tmpValue, (*QFontMetrics).DestroyQFontMetrics)
	return tmpValue
}

func (ptr *QFontMetrics) Ascent() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_Ascent(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) AverageCharWidth() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_AverageCharWidth(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) BoundingRect(ch core.QChar_ITF) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QFontMetrics_BoundingRect(ptr.Pointer(), core.PointerFromQChar(ch)))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetrics) BoundingRect2(text string) *core.QRect {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFromPointer(C.QFontMetrics_BoundingRect2(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetrics) BoundingRect3(rect core.QRect_ITF, flags int, text string, tabStops int, tabArray int) *core.QRect {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFromPointer(C.QFontMetrics_BoundingRect3(ptr.Pointer(), core.PointerFromQRect(rect), C.int(int32(flags)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.int(int32(tabStops)), C.int(int32(tabArray))))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetrics) BoundingRect4(x int, y int, width int, height int, flags int, text string, tabStops int, tabArray int) *core.QRect {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFromPointer(C.QFontMetrics_BoundingRect4(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.int(int32(flags)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.int(int32(tabStops)), C.int(int32(tabArray))))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetrics) CapHeight() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_CapHeight(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) Descent() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_Descent(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) ElidedText(text string, mode core.Qt__TextElideMode, width int, flags int) string {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		return cGoUnpackString(C.QFontMetrics_ElidedText(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.longlong(mode), C.int(int32(width)), C.int(int32(flags))))
	}
	return ""
}

func (ptr *QFontMetrics) Height() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_Height(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) HorizontalAdvance(text string, l int) int {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		return int(int32(C.QFontMetrics_HorizontalAdvance(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.int(int32(l)))))
	}
	return 0
}

func (ptr *QFontMetrics) HorizontalAdvance2(ch core.QChar_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_HorizontalAdvance2(ptr.Pointer(), core.PointerFromQChar(ch))))
	}
	return 0
}

func (ptr *QFontMetrics) InFont(ch core.QChar_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QFontMetrics_InFont(ptr.Pointer(), core.PointerFromQChar(ch))) != 0
	}
	return false
}

func (ptr *QFontMetrics) InFontUcs4(ucs4 uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QFontMetrics_InFontUcs4(ptr.Pointer(), C.uint(uint32(ucs4)))) != 0
	}
	return false
}

func (ptr *QFontMetrics) Leading() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_Leading(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) LeftBearing(ch core.QChar_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_LeftBearing(ptr.Pointer(), core.PointerFromQChar(ch))))
	}
	return 0
}

func (ptr *QFontMetrics) LineSpacing() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_LineSpacing(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) LineWidth() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_LineWidth(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) MaxWidth() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_MaxWidth(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) MinLeftBearing() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_MinLeftBearing(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) MinRightBearing() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_MinRightBearing(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) OverlinePos() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_OverlinePos(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) RightBearing(ch core.QChar_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_RightBearing(ptr.Pointer(), core.PointerFromQChar(ch))))
	}
	return 0
}

func (ptr *QFontMetrics) Size(flags int, text string, tabStops int, tabArray int) *core.QSize {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQSizeFromPointer(C.QFontMetrics_Size(ptr.Pointer(), C.int(int32(flags)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.int(int32(tabStops)), C.int(int32(tabArray))))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetrics) StrikeOutPos() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_StrikeOutPos(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) Swap(other QFontMetrics_ITF) {
	if ptr.Pointer() != nil {
		C.QFontMetrics_Swap(ptr.Pointer(), PointerFromQFontMetrics(other))
	}
}

func (ptr *QFontMetrics) TightBoundingRect(text string) *core.QRect {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFromPointer(C.QFontMetrics_TightBoundingRect(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetrics) UnderlinePos() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_UnderlinePos(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) XHeight() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QFontMetrics_XHeight(ptr.Pointer())))
	}
	return 0
}

func (ptr *QFontMetrics) DestroyQFontMetrics() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QFontMetrics_DestroyQFontMetrics(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QFontMetricsF struct {
	ptr unsafe.Pointer
}

type QFontMetricsF_ITF interface {
	QFontMetricsF_PTR() *QFontMetricsF
}

func (ptr *QFontMetricsF) QFontMetricsF_PTR() *QFontMetricsF {
	return ptr
}

func (ptr *QFontMetricsF) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QFontMetricsF) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQFontMetricsF(ptr QFontMetricsF_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QFontMetricsF_PTR().Pointer()
	}
	return nil
}

func NewQFontMetricsFFromPointer(ptr unsafe.Pointer) (n *QFontMetricsF) {
	n = new(QFontMetricsF)
	n.SetPointer(ptr)
	return
}
func NewQFontMetricsF(font QFont_ITF) *QFontMetricsF {
	tmpValue := NewQFontMetricsFFromPointer(C.QFontMetricsF_NewQFontMetricsF(PointerFromQFont(font)))
	qt.SetFinalizer(tmpValue, (*QFontMetricsF).DestroyQFontMetricsF)
	return tmpValue
}

func NewQFontMetricsF3(font QFont_ITF, paintdevice QPaintDevice_ITF) *QFontMetricsF {
	tmpValue := NewQFontMetricsFFromPointer(C.QFontMetricsF_NewQFontMetricsF3(PointerFromQFont(font), PointerFromQPaintDevice(paintdevice)))
	qt.SetFinalizer(tmpValue, (*QFontMetricsF).DestroyQFontMetricsF)
	return tmpValue
}

func NewQFontMetricsF4(fontMetrics QFontMetrics_ITF) *QFontMetricsF {
	tmpValue := NewQFontMetricsFFromPointer(C.QFontMetricsF_NewQFontMetricsF4(PointerFromQFontMetrics(fontMetrics)))
	qt.SetFinalizer(tmpValue, (*QFontMetricsF).DestroyQFontMetricsF)
	return tmpValue
}

func NewQFontMetricsF5(fm QFontMetricsF_ITF) *QFontMetricsF {
	tmpValue := NewQFontMetricsFFromPointer(C.QFontMetricsF_NewQFontMetricsF5(PointerFromQFontMetricsF(fm)))
	qt.SetFinalizer(tmpValue, (*QFontMetricsF).DestroyQFontMetricsF)
	return tmpValue
}

func (ptr *QFontMetricsF) Ascent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_Ascent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) AverageCharWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_AverageCharWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) BoundingRect(text string) *core.QRectF {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFFromPointer(C.QFontMetricsF_BoundingRect(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetricsF) BoundingRect2(ch core.QChar_ITF) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QFontMetricsF_BoundingRect2(ptr.Pointer(), core.PointerFromQChar(ch)))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetricsF) BoundingRect3(rect core.QRectF_ITF, flags int, text string, tabStops int, tabArray int) *core.QRectF {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFFromPointer(C.QFontMetricsF_BoundingRect3(ptr.Pointer(), core.PointerFromQRectF(rect), C.int(int32(flags)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.int(int32(tabStops)), C.int(int32(tabArray))))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetricsF) CapHeight() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_CapHeight(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) Descent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_Descent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) ElidedText(text string, mode core.Qt__TextElideMode, width float64, flags int) string {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		return cGoUnpackString(C.QFontMetricsF_ElidedText(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.longlong(mode), C.double(width), C.int(int32(flags))))
	}
	return ""
}

func (ptr *QFontMetricsF) Height() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_Height(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) HorizontalAdvance(text string, length int) float64 {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		return float64(C.QFontMetricsF_HorizontalAdvance(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.int(int32(length))))
	}
	return 0
}

func (ptr *QFontMetricsF) HorizontalAdvance2(ch core.QChar_ITF) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_HorizontalAdvance2(ptr.Pointer(), core.PointerFromQChar(ch)))
	}
	return 0
}

func (ptr *QFontMetricsF) InFont(ch core.QChar_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QFontMetricsF_InFont(ptr.Pointer(), core.PointerFromQChar(ch))) != 0
	}
	return false
}

func (ptr *QFontMetricsF) InFontUcs4(ch uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QFontMetricsF_InFontUcs4(ptr.Pointer(), C.uint(uint32(ch)))) != 0
	}
	return false
}

func (ptr *QFontMetricsF) Leading() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_Leading(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) LeftBearing(ch core.QChar_ITF) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_LeftBearing(ptr.Pointer(), core.PointerFromQChar(ch)))
	}
	return 0
}

func (ptr *QFontMetricsF) LineSpacing() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_LineSpacing(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) LineWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_LineWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) MaxWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_MaxWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) MinLeftBearing() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_MinLeftBearing(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) MinRightBearing() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_MinRightBearing(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) OverlinePos() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_OverlinePos(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) RightBearing(ch core.QChar_ITF) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_RightBearing(ptr.Pointer(), core.PointerFromQChar(ch)))
	}
	return 0
}

func (ptr *QFontMetricsF) Size(flags int, text string, tabStops int, tabArray int) *core.QSizeF {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQSizeFFromPointer(C.QFontMetricsF_Size(ptr.Pointer(), C.int(int32(flags)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.int(int32(tabStops)), C.int(int32(tabArray))))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetricsF) StrikeOutPos() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_StrikeOutPos(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) Swap(other QFontMetricsF_ITF) {
	if ptr.Pointer() != nil {
		C.QFontMetricsF_Swap(ptr.Pointer(), PointerFromQFontMetricsF(other))
	}
}

func (ptr *QFontMetricsF) TightBoundingRect(text string) *core.QRectF {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFFromPointer(C.QFontMetricsF_TightBoundingRect(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QFontMetricsF) UnderlinePos() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_UnderlinePos(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) XHeight() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QFontMetricsF_XHeight(ptr.Pointer()))
	}
	return 0
}

func (ptr *QFontMetricsF) DestroyQFontMetricsF() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QFontMetricsF_DestroyQFontMetricsF(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QGenericMatrix struct {
	ptr unsafe.Pointer
}

type QGenericMatrix_ITF interface {
	QGenericMatrix_PTR() *QGenericMatrix
}

func (ptr *QGenericMatrix) QGenericMatrix_PTR() *QGenericMatrix {
	return ptr
}

func (ptr *QGenericMatrix) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QGenericMatrix) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQGenericMatrix(ptr QGenericMatrix_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QGenericMatrix_PTR().Pointer()
	}
	return nil
}

func NewQGenericMatrixFromPointer(ptr unsafe.Pointer) (n *QGenericMatrix) {
	n = new(QGenericMatrix)
	n.SetPointer(ptr)
	return
}
func (ptr *QGenericMatrix) DestroyQGenericMatrix() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QGenericPlugin struct {
	core.QObject
}

type QGenericPlugin_ITF interface {
	core.QObject_ITF
	QGenericPlugin_PTR() *QGenericPlugin
}

func (ptr *QGenericPlugin) QGenericPlugin_PTR() *QGenericPlugin {
	return ptr
}

func (ptr *QGenericPlugin) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QGenericPlugin) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQGenericPlugin(ptr QGenericPlugin_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QGenericPlugin_PTR().Pointer()
	}
	return nil
}

func NewQGenericPluginFromPointer(ptr unsafe.Pointer) (n *QGenericPlugin) {
	n = new(QGenericPlugin)
	n.SetPointer(ptr)
	return
}
func NewQGenericPlugin(parent core.QObject_ITF) *QGenericPlugin {
	tmpValue := NewQGenericPluginFromPointer(C.QGenericPlugin_NewQGenericPlugin(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQGenericPlugin_Create
func callbackQGenericPlugin_Create(ptr unsafe.Pointer, key C.struct_QtGui_PackedString, specification C.struct_QtGui_PackedString) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "create"); signal != nil {
		return core.PointerFromQObject((*(*func(string, string) *core.QObject)(signal))(cGoUnpackString(key), cGoUnpackString(specification)))
	}

	return core.PointerFromQObject(core.NewQObject(nil))
}

func (ptr *QGenericPlugin) ConnectCreate(f func(key string, specification string) *core.QObject) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "create"); signal != nil {
			f := func(key string, specification string) *core.QObject {
				(*(*func(string, string) *core.QObject)(signal))(key, specification)
				return f(key, specification)
			}
			qt.ConnectSignal(ptr.Pointer(), "create", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "create", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGenericPlugin) DisconnectCreate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "create")
	}
}

func (ptr *QGenericPlugin) Create(key string, specification string) *core.QObject {
	if ptr.Pointer() != nil {
		var keyC *C.char
		if key != "" {
			keyC = C.CString(key)
			defer C.free(unsafe.Pointer(keyC))
		}
		var specificationC *C.char
		if specification != "" {
			specificationC = C.CString(specification)
			defer C.free(unsafe.Pointer(specificationC))
		}
		tmpValue := core.NewQObjectFromPointer(C.QGenericPlugin_Create(ptr.Pointer(), C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, C.struct_QtGui_PackedString{data: specificationC, len: C.longlong(len(specification))}))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQGenericPlugin_DestroyQGenericPlugin
func callbackQGenericPlugin_DestroyQGenericPlugin(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QGenericPlugin"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQGenericPluginFromPointer(ptr).DestroyQGenericPluginDefault()
	}
}

func (ptr *QGenericPlugin) ConnectDestroyQGenericPlugin(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QGenericPlugin"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QGenericPlugin", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QGenericPlugin", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGenericPlugin) DisconnectDestroyQGenericPlugin() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QGenericPlugin")
	}
}

func (ptr *QGenericPlugin) DestroyQGenericPlugin() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QGenericPlugin_DestroyQGenericPlugin(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QGenericPlugin) DestroyQGenericPluginDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QGenericPlugin_DestroyQGenericPluginDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QGenericPlugin) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QGenericPlugin___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QGenericPlugin) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QGenericPlugin___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QGenericPlugin) __children_newList() unsafe.Pointer {
	return C.QGenericPlugin___children_newList(ptr.Pointer())
}

func (ptr *QGenericPlugin) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QGenericPlugin___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QGenericPlugin) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QGenericPlugin___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QGenericPlugin) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QGenericPlugin___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QGenericPlugin) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QGenericPlugin___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QGenericPlugin) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QGenericPlugin___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QGenericPlugin) __findChildren_newList() unsafe.Pointer {
	return C.QGenericPlugin___findChildren_newList(ptr.Pointer())
}

func (ptr *QGenericPlugin) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QGenericPlugin___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QGenericPlugin) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QGenericPlugin___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QGenericPlugin) __findChildren_newList3() unsafe.Pointer {
	return C.QGenericPlugin___findChildren_newList3(ptr.Pointer())
}

//export callbackQGenericPlugin_ChildEvent
func callbackQGenericPlugin_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQGenericPluginFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QGenericPlugin) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QGenericPlugin_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQGenericPlugin_ConnectNotify
func callbackQGenericPlugin_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQGenericPluginFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QGenericPlugin) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QGenericPlugin_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQGenericPlugin_CustomEvent
func callbackQGenericPlugin_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQGenericPluginFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QGenericPlugin) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QGenericPlugin_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQGenericPlugin_DeleteLater
func callbackQGenericPlugin_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQGenericPluginFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QGenericPlugin) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QGenericPlugin_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQGenericPlugin_Destroyed
func callbackQGenericPlugin_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQGenericPlugin_DisconnectNotify
func callbackQGenericPlugin_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQGenericPluginFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QGenericPlugin) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QGenericPlugin_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQGenericPlugin_Event
func callbackQGenericPlugin_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQGenericPluginFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QGenericPlugin) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QGenericPlugin_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQGenericPlugin_EventFilter
func callbackQGenericPlugin_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQGenericPluginFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QGenericPlugin) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QGenericPlugin_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQGenericPlugin_MetaObject
func callbackQGenericPlugin_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQGenericPluginFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QGenericPlugin) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QGenericPlugin_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQGenericPlugin_ObjectNameChanged
func callbackQGenericPlugin_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQGenericPlugin_TimerEvent
func callbackQGenericPlugin_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQGenericPluginFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QGenericPlugin) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QGenericPlugin_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QGenericPluginFactory struct {
	ptr unsafe.Pointer
}

type QGenericPluginFactory_ITF interface {
	QGenericPluginFactory_PTR() *QGenericPluginFactory
}

func (ptr *QGenericPluginFactory) QGenericPluginFactory_PTR() *QGenericPluginFactory {
	return ptr
}

func (ptr *QGenericPluginFactory) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QGenericPluginFactory) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQGenericPluginFactory(ptr QGenericPluginFactory_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QGenericPluginFactory_PTR().Pointer()
	}
	return nil
}

func NewQGenericPluginFactoryFromPointer(ptr unsafe.Pointer) (n *QGenericPluginFactory) {
	n = new(QGenericPluginFactory)
	n.SetPointer(ptr)
	return
}
func (ptr *QGenericPluginFactory) DestroyQGenericPluginFactory() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func QGenericPluginFactory_Create(key string, specification string) *core.QObject {
	var keyC *C.char
	if key != "" {
		keyC = C.CString(key)
		defer C.free(unsafe.Pointer(keyC))
	}
	var specificationC *C.char
	if specification != "" {
		specificationC = C.CString(specification)
		defer C.free(unsafe.Pointer(specificationC))
	}
	tmpValue := core.NewQObjectFromPointer(C.QGenericPluginFactory_QGenericPluginFactory_Create(C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, C.struct_QtGui_PackedString{data: specificationC, len: C.longlong(len(specification))}))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QGenericPluginFactory) Create(key string, specification string) *core.QObject {
	var keyC *C.char
	if key != "" {
		keyC = C.CString(key)
		defer C.free(unsafe.Pointer(keyC))
	}
	var specificationC *C.char
	if specification != "" {
		specificationC = C.CString(specification)
		defer C.free(unsafe.Pointer(specificationC))
	}
	tmpValue := core.NewQObjectFromPointer(C.QGenericPluginFactory_QGenericPluginFactory_Create(C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, C.struct_QtGui_PackedString{data: specificationC, len: C.longlong(len(specification))}))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func QGenericPluginFactory_Keys() []string {
	return unpackStringList(cGoUnpackString(C.QGenericPluginFactory_QGenericPluginFactory_Keys()))
}

func (ptr *QGenericPluginFactory) Keys() []string {
	return unpackStringList(cGoUnpackString(C.QGenericPluginFactory_QGenericPluginFactory_Keys()))
}

type QGlyphRun struct {
	ptr unsafe.Pointer
}

type QGlyphRun_ITF interface {
	QGlyphRun_PTR() *QGlyphRun
}

func (ptr *QGlyphRun) QGlyphRun_PTR() *QGlyphRun {
	return ptr
}

func (ptr *QGlyphRun) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QGlyphRun) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQGlyphRun(ptr QGlyphRun_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QGlyphRun_PTR().Pointer()
	}
	return nil
}

func NewQGlyphRunFromPointer(ptr unsafe.Pointer) (n *QGlyphRun) {
	n = new(QGlyphRun)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QGlyphRun__GlyphRunFlag
//QGlyphRun::GlyphRunFlag
type QGlyphRun__GlyphRunFlag int64

const (
	QGlyphRun__Overline      QGlyphRun__GlyphRunFlag = QGlyphRun__GlyphRunFlag(0x01)
	QGlyphRun__Underline     QGlyphRun__GlyphRunFlag = QGlyphRun__GlyphRunFlag(0x02)
	QGlyphRun__StrikeOut     QGlyphRun__GlyphRunFlag = QGlyphRun__GlyphRunFlag(0x04)
	QGlyphRun__RightToLeft   QGlyphRun__GlyphRunFlag = QGlyphRun__GlyphRunFlag(0x08)
	QGlyphRun__SplitLigature QGlyphRun__GlyphRunFlag = QGlyphRun__GlyphRunFlag(0x10)
)

func NewQGlyphRun() *QGlyphRun {
	tmpValue := NewQGlyphRunFromPointer(C.QGlyphRun_NewQGlyphRun())
	qt.SetFinalizer(tmpValue, (*QGlyphRun).DestroyQGlyphRun)
	return tmpValue
}

func NewQGlyphRun2(other QGlyphRun_ITF) *QGlyphRun {
	tmpValue := NewQGlyphRunFromPointer(C.QGlyphRun_NewQGlyphRun2(PointerFromQGlyphRun(other)))
	qt.SetFinalizer(tmpValue, (*QGlyphRun).DestroyQGlyphRun)
	return tmpValue
}

func (ptr *QGlyphRun) BoundingRect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QGlyphRun_BoundingRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QGlyphRun) Clear() {
	if ptr.Pointer() != nil {
		C.QGlyphRun_Clear(ptr.Pointer())
	}
}

func (ptr *QGlyphRun) Flags() QGlyphRun__GlyphRunFlag {
	if ptr.Pointer() != nil {
		return QGlyphRun__GlyphRunFlag(C.QGlyphRun_Flags(ptr.Pointer()))
	}
	return 0
}

func (ptr *QGlyphRun) GlyphIndexes() []uint {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []uint {
			out := make([]uint, int(l.len))
			tmpList := NewQGlyphRunFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__glyphIndexes_atList(i)
			}
			return out
		}(C.QGlyphRun_GlyphIndexes(ptr.Pointer()))
	}
	return make([]uint, 0)
}

func (ptr *QGlyphRun) IsEmpty() bool {
	if ptr.Pointer() != nil {
		return int8(C.QGlyphRun_IsEmpty(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QGlyphRun) IsRightToLeft() bool {
	if ptr.Pointer() != nil {
		return int8(C.QGlyphRun_IsRightToLeft(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QGlyphRun) Overline() bool {
	if ptr.Pointer() != nil {
		return int8(C.QGlyphRun_Overline(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QGlyphRun) Positions() []*core.QPointF {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*core.QPointF {
			out := make([]*core.QPointF, int(l.len))
			tmpList := NewQGlyphRunFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__positions_atList(i)
			}
			return out
		}(C.QGlyphRun_Positions(ptr.Pointer()))
	}
	return make([]*core.QPointF, 0)
}

func (ptr *QGlyphRun) RawFont() *QRawFont {
	if ptr.Pointer() != nil {
		tmpValue := NewQRawFontFromPointer(C.QGlyphRun_RawFont(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QRawFont).DestroyQRawFont)
		return tmpValue
	}
	return nil
}

func (ptr *QGlyphRun) SetBoundingRect(boundingRect core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetBoundingRect(ptr.Pointer(), core.PointerFromQRectF(boundingRect))
	}
}

func (ptr *QGlyphRun) SetFlag(flag QGlyphRun__GlyphRunFlag, enabled bool) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetFlag(ptr.Pointer(), C.longlong(flag), C.char(int8(qt.GoBoolToInt(enabled))))
	}
}

func (ptr *QGlyphRun) SetFlags(flags QGlyphRun__GlyphRunFlag) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetFlags(ptr.Pointer(), C.longlong(flags))
	}
}

func (ptr *QGlyphRun) SetGlyphIndexes(glyphIndexes []uint) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetGlyphIndexes(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQGlyphRunFromPointer(NewQGlyphRunFromPointer(nil).__setGlyphIndexes_glyphIndexes_newList())
			for _, v := range glyphIndexes {
				tmpList.__setGlyphIndexes_glyphIndexes_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QGlyphRun) SetOverline(overline bool) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetOverline(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(overline))))
	}
}

func (ptr *QGlyphRun) SetPositions(positions []*core.QPointF) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetPositions(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQGlyphRunFromPointer(NewQGlyphRunFromPointer(nil).__setPositions_positions_newList())
			for _, v := range positions {
				tmpList.__setPositions_positions_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QGlyphRun) SetRawData(glyphIndexArray uint, glyphPositionArray core.QPointF_ITF, size int) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetRawData(ptr.Pointer(), C.uint(uint32(glyphIndexArray)), core.PointerFromQPointF(glyphPositionArray), C.int(int32(size)))
	}
}

func (ptr *QGlyphRun) SetRawFont(rawFont QRawFont_ITF) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetRawFont(ptr.Pointer(), PointerFromQRawFont(rawFont))
	}
}

func (ptr *QGlyphRun) SetRightToLeft(rightToLeft bool) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetRightToLeft(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(rightToLeft))))
	}
}

func (ptr *QGlyphRun) SetStrikeOut(strikeOut bool) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetStrikeOut(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(strikeOut))))
	}
}

func (ptr *QGlyphRun) SetUnderline(underline bool) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_SetUnderline(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(underline))))
	}
}

func (ptr *QGlyphRun) StrikeOut() bool {
	if ptr.Pointer() != nil {
		return int8(C.QGlyphRun_StrikeOut(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QGlyphRun) Swap(other QGlyphRun_ITF) {
	if ptr.Pointer() != nil {
		C.QGlyphRun_Swap(ptr.Pointer(), PointerFromQGlyphRun(other))
	}
}

func (ptr *QGlyphRun) Underline() bool {
	if ptr.Pointer() != nil {
		return int8(C.QGlyphRun_Underline(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QGlyphRun) DestroyQGlyphRun() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QGlyphRun_DestroyQGlyphRun(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QGlyphRun) __glyphIndexes_atList(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QGlyphRun___glyphIndexes_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QGlyphRun) __glyphIndexes_setList(i uint) {
	if ptr.Pointer() != nil {
		C.QGlyphRun___glyphIndexes_setList(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QGlyphRun) __glyphIndexes_newList() unsafe.Pointer {
	return C.QGlyphRun___glyphIndexes_newList(ptr.Pointer())
}

func (ptr *QGlyphRun) __positions_atList(i int) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QGlyphRun___positions_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QGlyphRun) __positions_setList(i core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QGlyphRun___positions_setList(ptr.Pointer(), core.PointerFromQPointF(i))
	}
}

func (ptr *QGlyphRun) __positions_newList() unsafe.Pointer {
	return C.QGlyphRun___positions_newList(ptr.Pointer())
}

func (ptr *QGlyphRun) __setGlyphIndexes_glyphIndexes_atList(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QGlyphRun___setGlyphIndexes_glyphIndexes_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QGlyphRun) __setGlyphIndexes_glyphIndexes_setList(i uint) {
	if ptr.Pointer() != nil {
		C.QGlyphRun___setGlyphIndexes_glyphIndexes_setList(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QGlyphRun) __setGlyphIndexes_glyphIndexes_newList() unsafe.Pointer {
	return C.QGlyphRun___setGlyphIndexes_glyphIndexes_newList(ptr.Pointer())
}

func (ptr *QGlyphRun) __setPositions_positions_atList(i int) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QGlyphRun___setPositions_positions_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QGlyphRun) __setPositions_positions_setList(i core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QGlyphRun___setPositions_positions_setList(ptr.Pointer(), core.PointerFromQPointF(i))
	}
}

func (ptr *QGlyphRun) __setPositions_positions_newList() unsafe.Pointer {
	return C.QGlyphRun___setPositions_positions_newList(ptr.Pointer())
}

type QGradient struct {
	ptr unsafe.Pointer
}

type QGradient_ITF interface {
	QGradient_PTR() *QGradient
}

func (ptr *QGradient) QGradient_PTR() *QGradient {
	return ptr
}

func (ptr *QGradient) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QGradient) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQGradient(ptr QGradient_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QGradient_PTR().Pointer()
	}
	return nil
}

func NewQGradientFromPointer(ptr unsafe.Pointer) (n *QGradient) {
	n = new(QGradient)
	n.SetPointer(ptr)
	return
}
func (ptr *QGradient) DestroyQGradient() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QGradient__Type
//QGradient::Type
type QGradient__Type int64

const (
	QGradient__LinearGradient  QGradient__Type = QGradient__Type(0)
	QGradient__RadialGradient  QGradient__Type = QGradient__Type(1)
	QGradient__ConicalGradient QGradient__Type = QGradient__Type(2)
	QGradient__NoGradient      QGradient__Type = QGradient__Type(3)
)

//go:generate stringer -type=QGradient__Spread
//QGradient::Spread
type QGradient__Spread int64

const (
	QGradient__PadSpread     QGradient__Spread = QGradient__Spread(0)
	QGradient__ReflectSpread QGradient__Spread = QGradient__Spread(1)
	QGradient__RepeatSpread  QGradient__Spread = QGradient__Spread(2)
)

//go:generate stringer -type=QGradient__CoordinateMode
//QGradient::CoordinateMode
type QGradient__CoordinateMode int64

const (
	QGradient__LogicalMode         QGradient__CoordinateMode = QGradient__CoordinateMode(0)
	QGradient__StretchToDeviceMode QGradient__CoordinateMode = QGradient__CoordinateMode(1)
	QGradient__ObjectBoundingMode  QGradient__CoordinateMode = QGradient__CoordinateMode(2)
	QGradient__ObjectMode          QGradient__CoordinateMode = QGradient__CoordinateMode(3)
)

//go:generate stringer -type=QGradient__Preset
//QGradient::Preset
type QGradient__Preset int64

const (
	QGradient__WarmFlame        QGradient__Preset = QGradient__Preset(1)
	QGradient__NightFade        QGradient__Preset = QGradient__Preset(2)
	QGradient__SpringWarmth     QGradient__Preset = QGradient__Preset(3)
	QGradient__JuicyPeach       QGradient__Preset = QGradient__Preset(4)
	QGradient__YoungPassion     QGradient__Preset = QGradient__Preset(5)
	QGradient__LadyLips         QGradient__Preset = QGradient__Preset(6)
	QGradient__SunnyMorning     QGradient__Preset = QGradient__Preset(7)
	QGradient__RainyAshville    QGradient__Preset = QGradient__Preset(8)
	QGradient__FrozenDreams     QGradient__Preset = QGradient__Preset(9)
	QGradient__WinterNeva       QGradient__Preset = QGradient__Preset(10)
	QGradient__DustyGrass       QGradient__Preset = QGradient__Preset(11)
	QGradient__TemptingAzure    QGradient__Preset = QGradient__Preset(12)
	QGradient__HeavyRain        QGradient__Preset = QGradient__Preset(13)
	QGradient__AmyCrisp         QGradient__Preset = QGradient__Preset(14)
	QGradient__MeanFruit        QGradient__Preset = QGradient__Preset(15)
	QGradient__DeepBlue         QGradient__Preset = QGradient__Preset(16)
	QGradient__RipeMalinka      QGradient__Preset = QGradient__Preset(17)
	QGradient__CloudyKnoxville  QGradient__Preset = QGradient__Preset(18)
	QGradient__MalibuBeach      QGradient__Preset = QGradient__Preset(19)
	QGradient__NewLife          QGradient__Preset = QGradient__Preset(20)
	QGradient__TrueSunset       QGradient__Preset = QGradient__Preset(21)
	QGradient__MorpheusDen      QGradient__Preset = QGradient__Preset(22)
	QGradient__RareWind         QGradient__Preset = QGradient__Preset(23)
	QGradient__NearMoon         QGradient__Preset = QGradient__Preset(24)
	QGradient__WildApple        QGradient__Preset = QGradient__Preset(25)
	QGradient__SaintPetersburg  QGradient__Preset = QGradient__Preset(26)
	QGradient__PlumPlate        QGradient__Preset = QGradient__Preset(28)
	QGradient__EverlastingSky   QGradient__Preset = QGradient__Preset(29)
	QGradient__HappyFisher      QGradient__Preset = QGradient__Preset(30)
	QGradient__Blessing         QGradient__Preset = QGradient__Preset(31)
	QGradient__SharpeyeEagle    QGradient__Preset = QGradient__Preset(32)
	QGradient__LadogaBottom     QGradient__Preset = QGradient__Preset(33)
	QGradient__LemonGate        QGradient__Preset = QGradient__Preset(34)
	QGradient__ItmeoBranding    QGradient__Preset = QGradient__Preset(35)
	QGradient__ZeusMiracle      QGradient__Preset = QGradient__Preset(36)
	QGradient__OldHat           QGradient__Preset = QGradient__Preset(37)
	QGradient__StarWine         QGradient__Preset = QGradient__Preset(38)
	QGradient__HappyAcid        QGradient__Preset = QGradient__Preset(41)
	QGradient__AwesomePine      QGradient__Preset = QGradient__Preset(42)
	QGradient__NewYork          QGradient__Preset = QGradient__Preset(43)
	QGradient__ShyRainbow       QGradient__Preset = QGradient__Preset(44)
	QGradient__MixedHopes       QGradient__Preset = QGradient__Preset(46)
	QGradient__FlyHigh          QGradient__Preset = QGradient__Preset(47)
	QGradient__StrongBliss      QGradient__Preset = QGradient__Preset(48)
	QGradient__FreshMilk        QGradient__Preset = QGradient__Preset(49)
	QGradient__SnowAgain        QGradient__Preset = QGradient__Preset(50)
	QGradient__FebruaryInk      QGradient__Preset = QGradient__Preset(51)
	QGradient__KindSteel        QGradient__Preset = QGradient__Preset(52)
	QGradient__SoftGrass        QGradient__Preset = QGradient__Preset(53)
	QGradient__GrownEarly       QGradient__Preset = QGradient__Preset(54)
	QGradient__SharpBlues       QGradient__Preset = QGradient__Preset(55)
	QGradient__ShadyWater       QGradient__Preset = QGradient__Preset(56)
	QGradient__DirtyBeauty      QGradient__Preset = QGradient__Preset(57)
	QGradient__GreatWhale       QGradient__Preset = QGradient__Preset(58)
	QGradient__TeenNotebook     QGradient__Preset = QGradient__Preset(59)
	QGradient__PoliteRumors     QGradient__Preset = QGradient__Preset(60)
	QGradient__SweetPeriod      QGradient__Preset = QGradient__Preset(61)
	QGradient__WideMatrix       QGradient__Preset = QGradient__Preset(62)
	QGradient__SoftCherish      QGradient__Preset = QGradient__Preset(63)
	QGradient__RedSalvation     QGradient__Preset = QGradient__Preset(64)
	QGradient__BurningSpring    QGradient__Preset = QGradient__Preset(65)
	QGradient__NightParty       QGradient__Preset = QGradient__Preset(66)
	QGradient__SkyGlider        QGradient__Preset = QGradient__Preset(67)
	QGradient__HeavenPeach      QGradient__Preset = QGradient__Preset(68)
	QGradient__PurpleDivision   QGradient__Preset = QGradient__Preset(69)
	QGradient__AquaSplash       QGradient__Preset = QGradient__Preset(70)
	QGradient__SpikyNaga        QGradient__Preset = QGradient__Preset(72)
	QGradient__LoveKiss         QGradient__Preset = QGradient__Preset(73)
	QGradient__CleanMirror      QGradient__Preset = QGradient__Preset(75)
	QGradient__PremiumDark      QGradient__Preset = QGradient__Preset(76)
	QGradient__ColdEvening      QGradient__Preset = QGradient__Preset(77)
	QGradient__CochitiLake      QGradient__Preset = QGradient__Preset(78)
	QGradient__SummerGames      QGradient__Preset = QGradient__Preset(79)
	QGradient__PassionateBed    QGradient__Preset = QGradient__Preset(80)
	QGradient__MountainRock     QGradient__Preset = QGradient__Preset(81)
	QGradient__DesertHump       QGradient__Preset = QGradient__Preset(82)
	QGradient__JungleDay        QGradient__Preset = QGradient__Preset(83)
	QGradient__PhoenixStart     QGradient__Preset = QGradient__Preset(84)
	QGradient__OctoberSilence   QGradient__Preset = QGradient__Preset(85)
	QGradient__FarawayRiver     QGradient__Preset = QGradient__Preset(86)
	QGradient__AlchemistLab     QGradient__Preset = QGradient__Preset(87)
	QGradient__OverSun          QGradient__Preset = QGradient__Preset(88)
	QGradient__PremiumWhite     QGradient__Preset = QGradient__Preset(89)
	QGradient__MarsParty        QGradient__Preset = QGradient__Preset(90)
	QGradient__EternalConstance QGradient__Preset = QGradient__Preset(91)
	QGradient__JapanBlush       QGradient__Preset = QGradient__Preset(92)
	QGradient__SmilingRain      QGradient__Preset = QGradient__Preset(93)
	QGradient__CloudyApple      QGradient__Preset = QGradient__Preset(94)
	QGradient__BigMango         QGradient__Preset = QGradient__Preset(95)
	QGradient__HealthyWater     QGradient__Preset = QGradient__Preset(96)
	QGradient__AmourAmour       QGradient__Preset = QGradient__Preset(97)
	QGradient__RiskyConcrete    QGradient__Preset = QGradient__Preset(98)
	QGradient__StrongStick      QGradient__Preset = QGradient__Preset(99)
	QGradient__ViciousStance    QGradient__Preset = QGradient__Preset(100)
	QGradient__PaloAlto         QGradient__Preset = QGradient__Preset(101)
	QGradient__HappyMemories    QGradient__Preset = QGradient__Preset(102)
	QGradient__MidnightBloom    QGradient__Preset = QGradient__Preset(103)
	QGradient__Crystalline      QGradient__Preset = QGradient__Preset(104)
	QGradient__PartyBliss       QGradient__Preset = QGradient__Preset(106)
	QGradient__ConfidentCloud   QGradient__Preset = QGradient__Preset(107)
	QGradient__LeCocktail       QGradient__Preset = QGradient__Preset(108)
	QGradient__RiverCity        QGradient__Preset = QGradient__Preset(109)
	QGradient__FrozenBerry      QGradient__Preset = QGradient__Preset(110)
	QGradient__ChildCare        QGradient__Preset = QGradient__Preset(112)
	QGradient__FlyingLemon      QGradient__Preset = QGradient__Preset(113)
	QGradient__NewRetrowave     QGradient__Preset = QGradient__Preset(114)
	QGradient__HiddenJaguar     QGradient__Preset = QGradient__Preset(115)
	QGradient__AboveTheSky      QGradient__Preset = QGradient__Preset(116)
	QGradient__Nega             QGradient__Preset = QGradient__Preset(117)
	QGradient__DenseWater       QGradient__Preset = QGradient__Preset(118)
	QGradient__Seashore         QGradient__Preset = QGradient__Preset(120)
	QGradient__MarbleWall       QGradient__Preset = QGradient__Preset(121)
	QGradient__CheerfulCaramel  QGradient__Preset = QGradient__Preset(122)
	QGradient__NightSky         QGradient__Preset = QGradient__Preset(123)
	QGradient__MagicLake        QGradient__Preset = QGradient__Preset(124)
	QGradient__YoungGrass       QGradient__Preset = QGradient__Preset(125)
	QGradient__ColorfulPeach    QGradient__Preset = QGradient__Preset(126)
	QGradient__GentleCare       QGradient__Preset = QGradient__Preset(127)
	QGradient__PlumBath         QGradient__Preset = QGradient__Preset(128)
	QGradient__HappyUnicorn     QGradient__Preset = QGradient__Preset(129)
	QGradient__AfricanField     QGradient__Preset = QGradient__Preset(131)
	QGradient__SolidStone       QGradient__Preset = QGradient__Preset(132)
	QGradient__OrangeJuice      QGradient__Preset = QGradient__Preset(133)
	QGradient__GlassWater       QGradient__Preset = QGradient__Preset(134)
	QGradient__NorthMiracle     QGradient__Preset = QGradient__Preset(136)
	QGradient__FruitBlend       QGradient__Preset = QGradient__Preset(137)
	QGradient__MillenniumPine   QGradient__Preset = QGradient__Preset(138)
	QGradient__HighFlight       QGradient__Preset = QGradient__Preset(139)
	QGradient__MoleHall         QGradient__Preset = QGradient__Preset(140)
	QGradient__SpaceShift       QGradient__Preset = QGradient__Preset(142)
	QGradient__ForestInei       QGradient__Preset = QGradient__Preset(143)
	QGradient__RoyalGarden      QGradient__Preset = QGradient__Preset(144)
	QGradient__RichMetal        QGradient__Preset = QGradient__Preset(145)
	QGradient__JuicyCake        QGradient__Preset = QGradient__Preset(146)
	QGradient__SmartIndigo      QGradient__Preset = QGradient__Preset(147)
	QGradient__SandStrike       QGradient__Preset = QGradient__Preset(148)
	QGradient__NorseBeauty      QGradient__Preset = QGradient__Preset(149)
	QGradient__AquaGuidance     QGradient__Preset = QGradient__Preset(150)
	QGradient__SunVeggie        QGradient__Preset = QGradient__Preset(151)
	QGradient__SeaLord          QGradient__Preset = QGradient__Preset(152)
	QGradient__BlackSea         QGradient__Preset = QGradient__Preset(153)
	QGradient__GrassShampoo     QGradient__Preset = QGradient__Preset(154)
	QGradient__LandingAircraft  QGradient__Preset = QGradient__Preset(155)
	QGradient__WitchDance       QGradient__Preset = QGradient__Preset(156)
	QGradient__SleeplessNight   QGradient__Preset = QGradient__Preset(157)
	QGradient__AngelCare        QGradient__Preset = QGradient__Preset(158)
	QGradient__CrystalRiver     QGradient__Preset = QGradient__Preset(159)
	QGradient__SoftLipstick     QGradient__Preset = QGradient__Preset(160)
	QGradient__SaltMountain     QGradient__Preset = QGradient__Preset(161)
	QGradient__PerfectWhite     QGradient__Preset = QGradient__Preset(162)
	QGradient__FreshOasis       QGradient__Preset = QGradient__Preset(163)
	QGradient__StrictNovember   QGradient__Preset = QGradient__Preset(164)
	QGradient__MorningSalad     QGradient__Preset = QGradient__Preset(165)
	QGradient__DeepRelief       QGradient__Preset = QGradient__Preset(166)
	QGradient__SeaStrike        QGradient__Preset = QGradient__Preset(167)
	QGradient__NightCall        QGradient__Preset = QGradient__Preset(168)
	QGradient__SupremeSky       QGradient__Preset = QGradient__Preset(169)
	QGradient__LightBlue        QGradient__Preset = QGradient__Preset(170)
	QGradient__MindCrawl        QGradient__Preset = QGradient__Preset(171)
	QGradient__LilyMeadow       QGradient__Preset = QGradient__Preset(172)
	QGradient__SugarLollipop    QGradient__Preset = QGradient__Preset(173)
	QGradient__SweetDessert     QGradient__Preset = QGradient__Preset(174)
	QGradient__MagicRay         QGradient__Preset = QGradient__Preset(175)
	QGradient__TeenParty        QGradient__Preset = QGradient__Preset(176)
	QGradient__FrozenHeat       QGradient__Preset = QGradient__Preset(177)
	QGradient__GagarinView      QGradient__Preset = QGradient__Preset(178)
	QGradient__FabledSunset     QGradient__Preset = QGradient__Preset(179)
	QGradient__PerfectBlue      QGradient__Preset = QGradient__Preset(180)
)

func NewQGradient2(preset QGradient__Preset) *QGradient {
	tmpValue := NewQGradientFromPointer(C.QGradient_NewQGradient2(C.longlong(preset)))
	qt.SetFinalizer(tmpValue, (*QGradient).DestroyQGradient)
	return tmpValue
}

func (ptr *QGradient) CoordinateMode() QGradient__CoordinateMode {
	if ptr.Pointer() != nil {
		return QGradient__CoordinateMode(C.QGradient_CoordinateMode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QGradient) SetColorAt(position float64, color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QGradient_SetColorAt(ptr.Pointer(), C.double(position), PointerFromQColor(color))
	}
}

func (ptr *QGradient) SetCoordinateMode(mode QGradient__CoordinateMode) {
	if ptr.Pointer() != nil {
		C.QGradient_SetCoordinateMode(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QGradient) SetSpread(method QGradient__Spread) {
	if ptr.Pointer() != nil {
		C.QGradient_SetSpread(ptr.Pointer(), C.longlong(method))
	}
}

func (ptr *QGradient) Spread() QGradient__Spread {
	if ptr.Pointer() != nil {
		return QGradient__Spread(C.QGradient_Spread(ptr.Pointer()))
	}
	return 0
}

func (ptr *QGradient) Type() QGradient__Type {
	if ptr.Pointer() != nil {
		return QGradient__Type(C.QGradient_Type(ptr.Pointer()))
	}
	return 0
}

type QGuiApplication struct {
	core.QCoreApplication
}

type QGuiApplication_ITF interface {
	core.QCoreApplication_ITF
	QGuiApplication_PTR() *QGuiApplication
}

func (ptr *QGuiApplication) QGuiApplication_PTR() *QGuiApplication {
	return ptr
}

func (ptr *QGuiApplication) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QCoreApplication_PTR().Pointer()
	}
	return nil
}

func (ptr *QGuiApplication) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QCoreApplication_PTR().SetPointer(p)
	}
}

func PointerFromQGuiApplication(ptr QGuiApplication_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QGuiApplication_PTR().Pointer()
	}
	return nil
}

func NewQGuiApplicationFromPointer(ptr unsafe.Pointer) (n *QGuiApplication) {
	n = new(QGuiApplication)
	n.SetPointer(ptr)
	return
}
func NewQGuiApplication(argc int, argv []string) *QGuiApplication {
	argvC := C.CString(strings.Join(argv, "|"))
	defer C.free(unsafe.Pointer(argvC))
	tmpValue := NewQGuiApplicationFromPointer(C.QGuiApplication_NewQGuiApplication(C.int(int32(argc)), argvC))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func QGuiApplication_ApplicationDisplayName() string {
	return cGoUnpackString(C.QGuiApplication_QGuiApplication_ApplicationDisplayName())
}

func (ptr *QGuiApplication) ApplicationDisplayName() string {
	return cGoUnpackString(C.QGuiApplication_QGuiApplication_ApplicationDisplayName())
}

//export callbackQGuiApplication_ApplicationDisplayNameChanged
func callbackQGuiApplication_ApplicationDisplayNameChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "applicationDisplayNameChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QGuiApplication) ConnectApplicationDisplayNameChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "applicationDisplayNameChanged") {
			C.QGuiApplication_ConnectApplicationDisplayNameChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "applicationDisplayNameChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "applicationDisplayNameChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "applicationDisplayNameChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "applicationDisplayNameChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectApplicationDisplayNameChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectApplicationDisplayNameChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "applicationDisplayNameChanged")
	}
}

func (ptr *QGuiApplication) ApplicationDisplayNameChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_ApplicationDisplayNameChanged(ptr.Pointer())
	}
}

func QGuiApplication_ApplicationState() core.Qt__ApplicationState {
	return core.Qt__ApplicationState(C.QGuiApplication_QGuiApplication_ApplicationState())
}

func (ptr *QGuiApplication) ApplicationState() core.Qt__ApplicationState {
	return core.Qt__ApplicationState(C.QGuiApplication_QGuiApplication_ApplicationState())
}

//export callbackQGuiApplication_ApplicationStateChanged
func callbackQGuiApplication_ApplicationStateChanged(ptr unsafe.Pointer, state C.longlong) {
	if signal := qt.GetSignal(ptr, "applicationStateChanged"); signal != nil {
		(*(*func(core.Qt__ApplicationState))(signal))(core.Qt__ApplicationState(state))
	}

}

func (ptr *QGuiApplication) ConnectApplicationStateChanged(f func(state core.Qt__ApplicationState)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "applicationStateChanged") {
			C.QGuiApplication_ConnectApplicationStateChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "applicationStateChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "applicationStateChanged"); signal != nil {
			f := func(state core.Qt__ApplicationState) {
				(*(*func(core.Qt__ApplicationState))(signal))(state)
				f(state)
			}
			qt.ConnectSignal(ptr.Pointer(), "applicationStateChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "applicationStateChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectApplicationStateChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectApplicationStateChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "applicationStateChanged")
	}
}

func (ptr *QGuiApplication) ApplicationStateChanged(state core.Qt__ApplicationState) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_ApplicationStateChanged(ptr.Pointer(), C.longlong(state))
	}
}

func QGuiApplication_ChangeOverrideCursor(cursor QCursor_ITF) {
	C.QGuiApplication_QGuiApplication_ChangeOverrideCursor(PointerFromQCursor(cursor))
}

func (ptr *QGuiApplication) ChangeOverrideCursor(cursor QCursor_ITF) {
	C.QGuiApplication_QGuiApplication_ChangeOverrideCursor(PointerFromQCursor(cursor))
}

func QGuiApplication_Clipboard() *QClipboard {
	tmpValue := NewQClipboardFromPointer(C.QGuiApplication_QGuiApplication_Clipboard())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QGuiApplication) Clipboard() *QClipboard {
	tmpValue := NewQClipboardFromPointer(C.QGuiApplication_QGuiApplication_Clipboard())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQGuiApplication_CommitDataRequest
func callbackQGuiApplication_CommitDataRequest(ptr unsafe.Pointer, manager unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "commitDataRequest"); signal != nil {
		(*(*func(*QSessionManager))(signal))(NewQSessionManagerFromPointer(manager))
	}

}

func (ptr *QGuiApplication) ConnectCommitDataRequest(f func(manager *QSessionManager)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "commitDataRequest") {
			C.QGuiApplication_ConnectCommitDataRequest(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "commitDataRequest")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "commitDataRequest"); signal != nil {
			f := func(manager *QSessionManager) {
				(*(*func(*QSessionManager))(signal))(manager)
				f(manager)
			}
			qt.ConnectSignal(ptr.Pointer(), "commitDataRequest", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "commitDataRequest", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectCommitDataRequest() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectCommitDataRequest(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "commitDataRequest")
	}
}

func (ptr *QGuiApplication) CommitDataRequest(manager QSessionManager_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_CommitDataRequest(ptr.Pointer(), PointerFromQSessionManager(manager))
	}
}

func QGuiApplication_DesktopFileName() string {
	return cGoUnpackString(C.QGuiApplication_QGuiApplication_DesktopFileName())
}

func (ptr *QGuiApplication) DesktopFileName() string {
	return cGoUnpackString(C.QGuiApplication_QGuiApplication_DesktopFileName())
}

func QGuiApplication_DesktopSettingsAware() bool {
	return int8(C.QGuiApplication_QGuiApplication_DesktopSettingsAware()) != 0
}

func (ptr *QGuiApplication) DesktopSettingsAware() bool {
	return int8(C.QGuiApplication_QGuiApplication_DesktopSettingsAware()) != 0
}

func (ptr *QGuiApplication) DevicePixelRatio() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QGuiApplication_DevicePixelRatio(ptr.Pointer()))
	}
	return 0
}

//export callbackQGuiApplication_Event
func callbackQGuiApplication_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQGuiApplicationFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QGuiApplication) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QGuiApplication_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

func QGuiApplication_Exec() int {
	return int(int32(C.QGuiApplication_QGuiApplication_Exec()))
}

func (ptr *QGuiApplication) Exec() int {
	return int(int32(C.QGuiApplication_QGuiApplication_Exec()))
}

func QGuiApplication_FocusObject() *core.QObject {
	tmpValue := core.NewQObjectFromPointer(C.QGuiApplication_QGuiApplication_FocusObject())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QGuiApplication) FocusObject() *core.QObject {
	tmpValue := core.NewQObjectFromPointer(C.QGuiApplication_QGuiApplication_FocusObject())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQGuiApplication_FocusObjectChanged
func callbackQGuiApplication_FocusObjectChanged(ptr unsafe.Pointer, focusObject unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "focusObjectChanged"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(focusObject))
	}

}

func (ptr *QGuiApplication) ConnectFocusObjectChanged(f func(focusObject *core.QObject)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "focusObjectChanged") {
			C.QGuiApplication_ConnectFocusObjectChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "focusObjectChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "focusObjectChanged"); signal != nil {
			f := func(focusObject *core.QObject) {
				(*(*func(*core.QObject))(signal))(focusObject)
				f(focusObject)
			}
			qt.ConnectSignal(ptr.Pointer(), "focusObjectChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "focusObjectChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectFocusObjectChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectFocusObjectChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "focusObjectChanged")
	}
}

func (ptr *QGuiApplication) FocusObjectChanged(focusObject core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_FocusObjectChanged(ptr.Pointer(), core.PointerFromQObject(focusObject))
	}
}

func QGuiApplication_FocusWindow() *QWindow {
	tmpValue := NewQWindowFromPointer(C.QGuiApplication_QGuiApplication_FocusWindow())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QGuiApplication) FocusWindow() *QWindow {
	tmpValue := NewQWindowFromPointer(C.QGuiApplication_QGuiApplication_FocusWindow())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQGuiApplication_FocusWindowChanged
func callbackQGuiApplication_FocusWindowChanged(ptr unsafe.Pointer, focusWindow unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "focusWindowChanged"); signal != nil {
		(*(*func(*QWindow))(signal))(NewQWindowFromPointer(focusWindow))
	}

}

func (ptr *QGuiApplication) ConnectFocusWindowChanged(f func(focusWindow *QWindow)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "focusWindowChanged") {
			C.QGuiApplication_ConnectFocusWindowChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "focusWindowChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "focusWindowChanged"); signal != nil {
			f := func(focusWindow *QWindow) {
				(*(*func(*QWindow))(signal))(focusWindow)
				f(focusWindow)
			}
			qt.ConnectSignal(ptr.Pointer(), "focusWindowChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "focusWindowChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectFocusWindowChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectFocusWindowChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "focusWindowChanged")
	}
}

func (ptr *QGuiApplication) FocusWindowChanged(focusWindow QWindow_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_FocusWindowChanged(ptr.Pointer(), PointerFromQWindow(focusWindow))
	}
}

func QGuiApplication_Font() *QFont {
	tmpValue := NewQFontFromPointer(C.QGuiApplication_QGuiApplication_Font())
	qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
	return tmpValue
}

func (ptr *QGuiApplication) Font() *QFont {
	tmpValue := NewQFontFromPointer(C.QGuiApplication_QGuiApplication_Font())
	qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
	return tmpValue
}

//export callbackQGuiApplication_FontChanged
func callbackQGuiApplication_FontChanged(ptr unsafe.Pointer, font unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "fontChanged"); signal != nil {
		(*(*func(*QFont))(signal))(NewQFontFromPointer(font))
	}

}

func (ptr *QGuiApplication) ConnectFontChanged(f func(font *QFont)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "fontChanged") {
			C.QGuiApplication_ConnectFontChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "fontChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "fontChanged"); signal != nil {
			f := func(font *QFont) {
				(*(*func(*QFont))(signal))(font)
				f(font)
			}
			qt.ConnectSignal(ptr.Pointer(), "fontChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "fontChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectFontChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectFontChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "fontChanged")
	}
}

func (ptr *QGuiApplication) FontChanged(font QFont_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_FontChanged(ptr.Pointer(), PointerFromQFont(font))
	}
}

//export callbackQGuiApplication_FontDatabaseChanged
func callbackQGuiApplication_FontDatabaseChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "fontDatabaseChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QGuiApplication) ConnectFontDatabaseChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "fontDatabaseChanged") {
			C.QGuiApplication_ConnectFontDatabaseChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "fontDatabaseChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "fontDatabaseChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "fontDatabaseChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "fontDatabaseChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectFontDatabaseChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectFontDatabaseChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "fontDatabaseChanged")
	}
}

func (ptr *QGuiApplication) FontDatabaseChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_FontDatabaseChanged(ptr.Pointer())
	}
}

func QGuiApplication_InputMethod() *QInputMethod {
	tmpValue := NewQInputMethodFromPointer(C.QGuiApplication_QGuiApplication_InputMethod())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QGuiApplication) InputMethod() *QInputMethod {
	tmpValue := NewQInputMethodFromPointer(C.QGuiApplication_QGuiApplication_InputMethod())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func QGuiApplication_IsFallbackSessionManagementEnabled() bool {
	return int8(C.QGuiApplication_QGuiApplication_IsFallbackSessionManagementEnabled()) != 0
}

func (ptr *QGuiApplication) IsFallbackSessionManagementEnabled() bool {
	return int8(C.QGuiApplication_QGuiApplication_IsFallbackSessionManagementEnabled()) != 0
}

func QGuiApplication_IsLeftToRight() bool {
	return int8(C.QGuiApplication_QGuiApplication_IsLeftToRight()) != 0
}

func (ptr *QGuiApplication) IsLeftToRight() bool {
	return int8(C.QGuiApplication_QGuiApplication_IsLeftToRight()) != 0
}

func QGuiApplication_IsRightToLeft() bool {
	return int8(C.QGuiApplication_QGuiApplication_IsRightToLeft()) != 0
}

func (ptr *QGuiApplication) IsRightToLeft() bool {
	return int8(C.QGuiApplication_QGuiApplication_IsRightToLeft()) != 0
}

func (ptr *QGuiApplication) IsSavingSession() bool {
	if ptr.Pointer() != nil {
		return int8(C.QGuiApplication_IsSavingSession(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QGuiApplication) IsSessionRestored() bool {
	if ptr.Pointer() != nil {
		return int8(C.QGuiApplication_IsSessionRestored(ptr.Pointer())) != 0
	}
	return false
}

func QGuiApplication_KeyboardModifiers() core.Qt__KeyboardModifier {
	return core.Qt__KeyboardModifier(C.QGuiApplication_QGuiApplication_KeyboardModifiers())
}

func (ptr *QGuiApplication) KeyboardModifiers() core.Qt__KeyboardModifier {
	return core.Qt__KeyboardModifier(C.QGuiApplication_QGuiApplication_KeyboardModifiers())
}

//export callbackQGuiApplication_LastWindowClosed
func callbackQGuiApplication_LastWindowClosed(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "lastWindowClosed"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QGuiApplication) ConnectLastWindowClosed(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "lastWindowClosed") {
			C.QGuiApplication_ConnectLastWindowClosed(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "lastWindowClosed")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "lastWindowClosed"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "lastWindowClosed", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "lastWindowClosed", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectLastWindowClosed() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectLastWindowClosed(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "lastWindowClosed")
	}
}

func (ptr *QGuiApplication) LastWindowClosed() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_LastWindowClosed(ptr.Pointer())
	}
}

func QGuiApplication_LayoutDirection() core.Qt__LayoutDirection {
	return core.Qt__LayoutDirection(C.QGuiApplication_QGuiApplication_LayoutDirection())
}

func (ptr *QGuiApplication) LayoutDirection() core.Qt__LayoutDirection {
	return core.Qt__LayoutDirection(C.QGuiApplication_QGuiApplication_LayoutDirection())
}

//export callbackQGuiApplication_LayoutDirectionChanged
func callbackQGuiApplication_LayoutDirectionChanged(ptr unsafe.Pointer, direction C.longlong) {
	if signal := qt.GetSignal(ptr, "layoutDirectionChanged"); signal != nil {
		(*(*func(core.Qt__LayoutDirection))(signal))(core.Qt__LayoutDirection(direction))
	}

}

func (ptr *QGuiApplication) ConnectLayoutDirectionChanged(f func(direction core.Qt__LayoutDirection)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "layoutDirectionChanged") {
			C.QGuiApplication_ConnectLayoutDirectionChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "layoutDirectionChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "layoutDirectionChanged"); signal != nil {
			f := func(direction core.Qt__LayoutDirection) {
				(*(*func(core.Qt__LayoutDirection))(signal))(direction)
				f(direction)
			}
			qt.ConnectSignal(ptr.Pointer(), "layoutDirectionChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "layoutDirectionChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectLayoutDirectionChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectLayoutDirectionChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "layoutDirectionChanged")
	}
}

func (ptr *QGuiApplication) LayoutDirectionChanged(direction core.Qt__LayoutDirection) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_LayoutDirectionChanged(ptr.Pointer(), C.longlong(direction))
	}
}

func QGuiApplication_ModalWindow() *QWindow {
	tmpValue := NewQWindowFromPointer(C.QGuiApplication_QGuiApplication_ModalWindow())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QGuiApplication) ModalWindow() *QWindow {
	tmpValue := NewQWindowFromPointer(C.QGuiApplication_QGuiApplication_ModalWindow())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func QGuiApplication_MouseButtons() core.Qt__MouseButton {
	return core.Qt__MouseButton(C.QGuiApplication_QGuiApplication_MouseButtons())
}

func (ptr *QGuiApplication) MouseButtons() core.Qt__MouseButton {
	return core.Qt__MouseButton(C.QGuiApplication_QGuiApplication_MouseButtons())
}

func QGuiApplication_OverrideCursor() *QCursor {
	return NewQCursorFromPointer(C.QGuiApplication_QGuiApplication_OverrideCursor())
}

func (ptr *QGuiApplication) OverrideCursor() *QCursor {
	return NewQCursorFromPointer(C.QGuiApplication_QGuiApplication_OverrideCursor())
}

func QGuiApplication_Palette() *QPalette {
	tmpValue := NewQPaletteFromPointer(C.QGuiApplication_QGuiApplication_Palette())
	qt.SetFinalizer(tmpValue, (*QPalette).DestroyQPalette)
	return tmpValue
}

func (ptr *QGuiApplication) Palette() *QPalette {
	tmpValue := NewQPaletteFromPointer(C.QGuiApplication_QGuiApplication_Palette())
	qt.SetFinalizer(tmpValue, (*QPalette).DestroyQPalette)
	return tmpValue
}

//export callbackQGuiApplication_PaletteChanged
func callbackQGuiApplication_PaletteChanged(ptr unsafe.Pointer, palette unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "paletteChanged"); signal != nil {
		(*(*func(*QPalette))(signal))(NewQPaletteFromPointer(palette))
	}

}

func (ptr *QGuiApplication) ConnectPaletteChanged(f func(palette *QPalette)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "paletteChanged") {
			C.QGuiApplication_ConnectPaletteChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "paletteChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "paletteChanged"); signal != nil {
			f := func(palette *QPalette) {
				(*(*func(*QPalette))(signal))(palette)
				f(palette)
			}
			qt.ConnectSignal(ptr.Pointer(), "paletteChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "paletteChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectPaletteChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectPaletteChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "paletteChanged")
	}
}

func (ptr *QGuiApplication) PaletteChanged(palette QPalette_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_PaletteChanged(ptr.Pointer(), PointerFromQPalette(palette))
	}
}

func QGuiApplication_PlatformFunction(function core.QByteArray_ITF) unsafe.Pointer {
	return C.QGuiApplication_QGuiApplication_PlatformFunction(core.PointerFromQByteArray(function))
}

func (ptr *QGuiApplication) PlatformFunction(function core.QByteArray_ITF) unsafe.Pointer {
	return C.QGuiApplication_QGuiApplication_PlatformFunction(core.PointerFromQByteArray(function))
}

func QGuiApplication_PlatformName() string {
	return cGoUnpackString(C.QGuiApplication_QGuiApplication_PlatformName())
}

func (ptr *QGuiApplication) PlatformName() string {
	return cGoUnpackString(C.QGuiApplication_QGuiApplication_PlatformName())
}

func QGuiApplication_PrimaryScreen() *QScreen {
	tmpValue := NewQScreenFromPointer(C.QGuiApplication_QGuiApplication_PrimaryScreen())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QGuiApplication) PrimaryScreen() *QScreen {
	tmpValue := NewQScreenFromPointer(C.QGuiApplication_QGuiApplication_PrimaryScreen())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQGuiApplication_PrimaryScreenChanged
func callbackQGuiApplication_PrimaryScreenChanged(ptr unsafe.Pointer, screen unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "primaryScreenChanged"); signal != nil {
		(*(*func(*QScreen))(signal))(NewQScreenFromPointer(screen))
	}

}

func (ptr *QGuiApplication) ConnectPrimaryScreenChanged(f func(screen *QScreen)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "primaryScreenChanged") {
			C.QGuiApplication_ConnectPrimaryScreenChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "primaryScreenChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "primaryScreenChanged"); signal != nil {
			f := func(screen *QScreen) {
				(*(*func(*QScreen))(signal))(screen)
				f(screen)
			}
			qt.ConnectSignal(ptr.Pointer(), "primaryScreenChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "primaryScreenChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectPrimaryScreenChanged() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectPrimaryScreenChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "primaryScreenChanged")
	}
}

func (ptr *QGuiApplication) PrimaryScreenChanged(screen QScreen_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_PrimaryScreenChanged(ptr.Pointer(), PointerFromQScreen(screen))
	}
}

func QGuiApplication_QueryKeyboardModifiers() core.Qt__KeyboardModifier {
	return core.Qt__KeyboardModifier(C.QGuiApplication_QGuiApplication_QueryKeyboardModifiers())
}

func (ptr *QGuiApplication) QueryKeyboardModifiers() core.Qt__KeyboardModifier {
	return core.Qt__KeyboardModifier(C.QGuiApplication_QGuiApplication_QueryKeyboardModifiers())
}

func QGuiApplication_QuitOnLastWindowClosed() bool {
	return int8(C.QGuiApplication_QGuiApplication_QuitOnLastWindowClosed()) != 0
}

func (ptr *QGuiApplication) QuitOnLastWindowClosed() bool {
	return int8(C.QGuiApplication_QGuiApplication_QuitOnLastWindowClosed()) != 0
}

func QGuiApplication_RestoreOverrideCursor() {
	C.QGuiApplication_QGuiApplication_RestoreOverrideCursor()
}

func (ptr *QGuiApplication) RestoreOverrideCursor() {
	C.QGuiApplication_QGuiApplication_RestoreOverrideCursor()
}

//export callbackQGuiApplication_SaveStateRequest
func callbackQGuiApplication_SaveStateRequest(ptr unsafe.Pointer, manager unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "saveStateRequest"); signal != nil {
		(*(*func(*QSessionManager))(signal))(NewQSessionManagerFromPointer(manager))
	}

}

func (ptr *QGuiApplication) ConnectSaveStateRequest(f func(manager *QSessionManager)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "saveStateRequest") {
			C.QGuiApplication_ConnectSaveStateRequest(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "saveStateRequest")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "saveStateRequest"); signal != nil {
			f := func(manager *QSessionManager) {
				(*(*func(*QSessionManager))(signal))(manager)
				f(manager)
			}
			qt.ConnectSignal(ptr.Pointer(), "saveStateRequest", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "saveStateRequest", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectSaveStateRequest() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectSaveStateRequest(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "saveStateRequest")
	}
}

func (ptr *QGuiApplication) SaveStateRequest(manager QSessionManager_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_SaveStateRequest(ptr.Pointer(), PointerFromQSessionManager(manager))
	}
}

//export callbackQGuiApplication_ScreenAdded
func callbackQGuiApplication_ScreenAdded(ptr unsafe.Pointer, screen unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "screenAdded"); signal != nil {
		(*(*func(*QScreen))(signal))(NewQScreenFromPointer(screen))
	}

}

func (ptr *QGuiApplication) ConnectScreenAdded(f func(screen *QScreen)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "screenAdded") {
			C.QGuiApplication_ConnectScreenAdded(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "screenAdded")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "screenAdded"); signal != nil {
			f := func(screen *QScreen) {
				(*(*func(*QScreen))(signal))(screen)
				f(screen)
			}
			qt.ConnectSignal(ptr.Pointer(), "screenAdded", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "screenAdded", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectScreenAdded() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectScreenAdded(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "screenAdded")
	}
}

func (ptr *QGuiApplication) ScreenAdded(screen QScreen_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_ScreenAdded(ptr.Pointer(), PointerFromQScreen(screen))
	}
}

func QGuiApplication_ScreenAt(point core.QPoint_ITF) *QScreen {
	tmpValue := NewQScreenFromPointer(C.QGuiApplication_QGuiApplication_ScreenAt(core.PointerFromQPoint(point)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QGuiApplication) ScreenAt(point core.QPoint_ITF) *QScreen {
	tmpValue := NewQScreenFromPointer(C.QGuiApplication_QGuiApplication_ScreenAt(core.PointerFromQPoint(point)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQGuiApplication_ScreenRemoved
func callbackQGuiApplication_ScreenRemoved(ptr unsafe.Pointer, screen unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "screenRemoved"); signal != nil {
		(*(*func(*QScreen))(signal))(NewQScreenFromPointer(screen))
	}

}

func (ptr *QGuiApplication) ConnectScreenRemoved(f func(screen *QScreen)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "screenRemoved") {
			C.QGuiApplication_ConnectScreenRemoved(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "screenRemoved")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "screenRemoved"); signal != nil {
			f := func(screen *QScreen) {
				(*(*func(*QScreen))(signal))(screen)
				f(screen)
			}
			qt.ConnectSignal(ptr.Pointer(), "screenRemoved", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "screenRemoved", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectScreenRemoved() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectScreenRemoved(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "screenRemoved")
	}
}

func (ptr *QGuiApplication) ScreenRemoved(screen QScreen_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_ScreenRemoved(ptr.Pointer(), PointerFromQScreen(screen))
	}
}

func QGuiApplication_Screens() []*QScreen {
	return func(l C.struct_QtGui_PackedList) []*QScreen {
		out := make([]*QScreen, int(l.len))
		tmpList := NewQGuiApplicationFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__screens_atList(i)
		}
		return out
	}(C.QGuiApplication_QGuiApplication_Screens())
}

func (ptr *QGuiApplication) Screens() []*QScreen {
	return func(l C.struct_QtGui_PackedList) []*QScreen {
		out := make([]*QScreen, int(l.len))
		tmpList := NewQGuiApplicationFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__screens_atList(i)
		}
		return out
	}(C.QGuiApplication_QGuiApplication_Screens())
}

func (ptr *QGuiApplication) SessionId() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QGuiApplication_SessionId(ptr.Pointer()))
	}
	return ""
}

func (ptr *QGuiApplication) SessionKey() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QGuiApplication_SessionKey(ptr.Pointer()))
	}
	return ""
}

func QGuiApplication_SetApplicationDisplayName(name string) {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	C.QGuiApplication_QGuiApplication_SetApplicationDisplayName(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
}

func (ptr *QGuiApplication) SetApplicationDisplayName(name string) {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	C.QGuiApplication_QGuiApplication_SetApplicationDisplayName(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
}

func QGuiApplication_SetDesktopFileName(name string) {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	C.QGuiApplication_QGuiApplication_SetDesktopFileName(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
}

func (ptr *QGuiApplication) SetDesktopFileName(name string) {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	C.QGuiApplication_QGuiApplication_SetDesktopFileName(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
}

func QGuiApplication_SetDesktopSettingsAware(on bool) {
	C.QGuiApplication_QGuiApplication_SetDesktopSettingsAware(C.char(int8(qt.GoBoolToInt(on))))
}

func (ptr *QGuiApplication) SetDesktopSettingsAware(on bool) {
	C.QGuiApplication_QGuiApplication_SetDesktopSettingsAware(C.char(int8(qt.GoBoolToInt(on))))
}

func QGuiApplication_SetFallbackSessionManagementEnabled(enabled bool) {
	C.QGuiApplication_QGuiApplication_SetFallbackSessionManagementEnabled(C.char(int8(qt.GoBoolToInt(enabled))))
}

func (ptr *QGuiApplication) SetFallbackSessionManagementEnabled(enabled bool) {
	C.QGuiApplication_QGuiApplication_SetFallbackSessionManagementEnabled(C.char(int8(qt.GoBoolToInt(enabled))))
}

func QGuiApplication_SetFont(font QFont_ITF) {
	C.QGuiApplication_QGuiApplication_SetFont(PointerFromQFont(font))
}

func (ptr *QGuiApplication) SetFont(font QFont_ITF) {
	C.QGuiApplication_QGuiApplication_SetFont(PointerFromQFont(font))
}

func QGuiApplication_SetLayoutDirection(direction core.Qt__LayoutDirection) {
	C.QGuiApplication_QGuiApplication_SetLayoutDirection(C.longlong(direction))
}

func (ptr *QGuiApplication) SetLayoutDirection(direction core.Qt__LayoutDirection) {
	C.QGuiApplication_QGuiApplication_SetLayoutDirection(C.longlong(direction))
}

func QGuiApplication_SetOverrideCursor(cursor QCursor_ITF) {
	C.QGuiApplication_QGuiApplication_SetOverrideCursor(PointerFromQCursor(cursor))
}

func (ptr *QGuiApplication) SetOverrideCursor(cursor QCursor_ITF) {
	C.QGuiApplication_QGuiApplication_SetOverrideCursor(PointerFromQCursor(cursor))
}

func QGuiApplication_SetPalette(pal QPalette_ITF) {
	C.QGuiApplication_QGuiApplication_SetPalette(PointerFromQPalette(pal))
}

func (ptr *QGuiApplication) SetPalette(pal QPalette_ITF) {
	C.QGuiApplication_QGuiApplication_SetPalette(PointerFromQPalette(pal))
}

func QGuiApplication_SetQuitOnLastWindowClosed(quit bool) {
	C.QGuiApplication_QGuiApplication_SetQuitOnLastWindowClosed(C.char(int8(qt.GoBoolToInt(quit))))
}

func (ptr *QGuiApplication) SetQuitOnLastWindowClosed(quit bool) {
	C.QGuiApplication_QGuiApplication_SetQuitOnLastWindowClosed(C.char(int8(qt.GoBoolToInt(quit))))
}

func QGuiApplication_SetWindowIcon(icon QIcon_ITF) {
	C.QGuiApplication_QGuiApplication_SetWindowIcon(PointerFromQIcon(icon))
}

func (ptr *QGuiApplication) SetWindowIcon(icon QIcon_ITF) {
	C.QGuiApplication_QGuiApplication_SetWindowIcon(PointerFromQIcon(icon))
}

func QGuiApplication_StyleHints() *QStyleHints {
	tmpValue := NewQStyleHintsFromPointer(C.QGuiApplication_QGuiApplication_StyleHints())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QGuiApplication) StyleHints() *QStyleHints {
	tmpValue := NewQStyleHintsFromPointer(C.QGuiApplication_QGuiApplication_StyleHints())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func QGuiApplication_Sync() {
	C.QGuiApplication_QGuiApplication_Sync()
}

func (ptr *QGuiApplication) Sync() {
	C.QGuiApplication_QGuiApplication_Sync()
}

func QGuiApplication_TopLevelAt(pos core.QPoint_ITF) *QWindow {
	tmpValue := NewQWindowFromPointer(C.QGuiApplication_QGuiApplication_TopLevelAt(core.PointerFromQPoint(pos)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QGuiApplication) TopLevelAt(pos core.QPoint_ITF) *QWindow {
	tmpValue := NewQWindowFromPointer(C.QGuiApplication_QGuiApplication_TopLevelAt(core.PointerFromQPoint(pos)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func QGuiApplication_WindowIcon() *QIcon {
	tmpValue := NewQIconFromPointer(C.QGuiApplication_QGuiApplication_WindowIcon())
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func (ptr *QGuiApplication) WindowIcon() *QIcon {
	tmpValue := NewQIconFromPointer(C.QGuiApplication_QGuiApplication_WindowIcon())
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

//export callbackQGuiApplication_DestroyQGuiApplication
func callbackQGuiApplication_DestroyQGuiApplication(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QGuiApplication"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQGuiApplicationFromPointer(ptr).DestroyQGuiApplicationDefault()
	}
}

func (ptr *QGuiApplication) ConnectDestroyQGuiApplication(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QGuiApplication"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QGuiApplication", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QGuiApplication", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QGuiApplication) DisconnectDestroyQGuiApplication() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QGuiApplication")
	}
}

func (ptr *QGuiApplication) DestroyQGuiApplication() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QGuiApplication_DestroyQGuiApplication(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QGuiApplication) DestroyQGuiApplicationDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QGuiApplication_DestroyQGuiApplicationDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QGuiApplication) __screens_atList(i int) *QScreen {
	if ptr.Pointer() != nil {
		tmpValue := NewQScreenFromPointer(C.QGuiApplication___screens_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QGuiApplication) __screens_setList(i QScreen_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication___screens_setList(ptr.Pointer(), PointerFromQScreen(i))
	}
}

func (ptr *QGuiApplication) __screens_newList() unsafe.Pointer {
	return C.QGuiApplication___screens_newList(ptr.Pointer())
}

func (ptr *QGuiApplication) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QGuiApplication___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QGuiApplication) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QGuiApplication) __children_newList() unsafe.Pointer {
	return C.QGuiApplication___children_newList(ptr.Pointer())
}

func (ptr *QGuiApplication) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QGuiApplication___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QGuiApplication) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QGuiApplication) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QGuiApplication___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QGuiApplication) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QGuiApplication___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QGuiApplication) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QGuiApplication) __findChildren_newList() unsafe.Pointer {
	return C.QGuiApplication___findChildren_newList(ptr.Pointer())
}

func (ptr *QGuiApplication) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QGuiApplication___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QGuiApplication) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QGuiApplication) __findChildren_newList3() unsafe.Pointer {
	return C.QGuiApplication___findChildren_newList3(ptr.Pointer())
}

//export callbackQGuiApplication_AboutToQuit
func callbackQGuiApplication_AboutToQuit(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "aboutToQuit"); signal != nil {
		(*(*func())(signal))()
	}

}

//export callbackQGuiApplication_ApplicationNameChanged
func callbackQGuiApplication_ApplicationNameChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "applicationNameChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

//export callbackQGuiApplication_ApplicationVersionChanged
func callbackQGuiApplication_ApplicationVersionChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "applicationVersionChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

//export callbackQGuiApplication_OrganizationDomainChanged
func callbackQGuiApplication_OrganizationDomainChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "organizationDomainChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

//export callbackQGuiApplication_OrganizationNameChanged
func callbackQGuiApplication_OrganizationNameChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "organizationNameChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

//export callbackQGuiApplication_Quit
func callbackQGuiApplication_Quit(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "quit"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQGuiApplicationFromPointer(ptr).QuitDefault()
	}
}

func (ptr *QGuiApplication) QuitDefault() {
	if ptr.Pointer() != nil {
		C.QGuiApplication_QuitDefault(ptr.Pointer())
	}
}

//export callbackQGuiApplication_ChildEvent
func callbackQGuiApplication_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQGuiApplicationFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QGuiApplication) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQGuiApplication_ConnectNotify
func callbackQGuiApplication_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQGuiApplicationFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QGuiApplication) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQGuiApplication_CustomEvent
func callbackQGuiApplication_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQGuiApplicationFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QGuiApplication) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQGuiApplication_DeleteLater
func callbackQGuiApplication_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQGuiApplicationFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QGuiApplication) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QGuiApplication_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQGuiApplication_Destroyed
func callbackQGuiApplication_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQGuiApplication_DisconnectNotify
func callbackQGuiApplication_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQGuiApplicationFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QGuiApplication) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQGuiApplication_EventFilter
func callbackQGuiApplication_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQGuiApplicationFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QGuiApplication) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QGuiApplication_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQGuiApplication_MetaObject
func callbackQGuiApplication_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQGuiApplicationFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QGuiApplication) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QGuiApplication_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQGuiApplication_ObjectNameChanged
func callbackQGuiApplication_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQGuiApplication_TimerEvent
func callbackQGuiApplication_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQGuiApplicationFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QGuiApplication) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QGuiApplication_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QHelpEvent struct {
	core.QEvent
}

type QHelpEvent_ITF interface {
	core.QEvent_ITF
	QHelpEvent_PTR() *QHelpEvent
}

func (ptr *QHelpEvent) QHelpEvent_PTR() *QHelpEvent {
	return ptr
}

func (ptr *QHelpEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QHelpEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQHelpEvent(ptr QHelpEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QHelpEvent_PTR().Pointer()
	}
	return nil
}

func NewQHelpEventFromPointer(ptr unsafe.Pointer) (n *QHelpEvent) {
	n = new(QHelpEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QHelpEvent) DestroyQHelpEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQHelpEvent(ty core.QEvent__Type, pos core.QPoint_ITF, globalPos core.QPoint_ITF) *QHelpEvent {
	tmpValue := NewQHelpEventFromPointer(C.QHelpEvent_NewQHelpEvent(C.longlong(ty), core.PointerFromQPoint(pos), core.PointerFromQPoint(globalPos)))
	qt.SetFinalizer(tmpValue, (*QHelpEvent).DestroyQHelpEvent)
	return tmpValue
}

func (ptr *QHelpEvent) GlobalPos() *core.QPoint {
	if ptr.Pointer() != nil {
		return core.NewQPointFromPointer(C.QHelpEvent_GlobalPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QHelpEvent) GlobalX() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QHelpEvent_GlobalX(ptr.Pointer())))
	}
	return 0
}

func (ptr *QHelpEvent) GlobalY() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QHelpEvent_GlobalY(ptr.Pointer())))
	}
	return 0
}

func (ptr *QHelpEvent) Pos() *core.QPoint {
	if ptr.Pointer() != nil {
		return core.NewQPointFromPointer(C.QHelpEvent_Pos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QHelpEvent) X() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QHelpEvent_X(ptr.Pointer())))
	}
	return 0
}

func (ptr *QHelpEvent) Y() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QHelpEvent_Y(ptr.Pointer())))
	}
	return 0
}

type QHideEvent struct {
	core.QEvent
}

type QHideEvent_ITF interface {
	core.QEvent_ITF
	QHideEvent_PTR() *QHideEvent
}

func (ptr *QHideEvent) QHideEvent_PTR() *QHideEvent {
	return ptr
}

func (ptr *QHideEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QHideEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQHideEvent(ptr QHideEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QHideEvent_PTR().Pointer()
	}
	return nil
}

func NewQHideEventFromPointer(ptr unsafe.Pointer) (n *QHideEvent) {
	n = new(QHideEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QHideEvent) DestroyQHideEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQHideEvent() *QHideEvent {
	tmpValue := NewQHideEventFromPointer(C.QHideEvent_NewQHideEvent())
	qt.SetFinalizer(tmpValue, (*QHideEvent).DestroyQHideEvent)
	return tmpValue
}

type QHoverEvent struct {
	QInputEvent
}

type QHoverEvent_ITF interface {
	QInputEvent_ITF
	QHoverEvent_PTR() *QHoverEvent
}

func (ptr *QHoverEvent) QHoverEvent_PTR() *QHoverEvent {
	return ptr
}

func (ptr *QHoverEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QHoverEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QInputEvent_PTR().SetPointer(p)
	}
}

func PointerFromQHoverEvent(ptr QHoverEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QHoverEvent_PTR().Pointer()
	}
	return nil
}

func NewQHoverEventFromPointer(ptr unsafe.Pointer) (n *QHoverEvent) {
	n = new(QHoverEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QHoverEvent) DestroyQHoverEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQHoverEvent(ty core.QEvent__Type, pos core.QPointF_ITF, oldPos core.QPointF_ITF, modifiers core.Qt__KeyboardModifier) *QHoverEvent {
	tmpValue := NewQHoverEventFromPointer(C.QHoverEvent_NewQHoverEvent(C.longlong(ty), core.PointerFromQPointF(pos), core.PointerFromQPointF(oldPos), C.longlong(modifiers)))
	qt.SetFinalizer(tmpValue, (*QHoverEvent).DestroyQHoverEvent)
	return tmpValue
}

func (ptr *QHoverEvent) OldPos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QHoverEvent_OldPos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QHoverEvent) OldPosF() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QHoverEvent_OldPosF(ptr.Pointer()))
	}
	return nil
}

func (ptr *QHoverEvent) Pos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QHoverEvent_Pos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QHoverEvent) PosF() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QHoverEvent_PosF(ptr.Pointer()))
	}
	return nil
}

type QIcon struct {
	ptr unsafe.Pointer
}

type QIcon_ITF interface {
	QIcon_PTR() *QIcon
}

func (ptr *QIcon) QIcon_PTR() *QIcon {
	return ptr
}

func (ptr *QIcon) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QIcon) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQIcon(ptr QIcon_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QIcon_PTR().Pointer()
	}
	return nil
}

func NewQIconFromPointer(ptr unsafe.Pointer) (n *QIcon) {
	n = new(QIcon)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QIcon__Mode
//QIcon::Mode
type QIcon__Mode int64

const (
	QIcon__Normal   QIcon__Mode = QIcon__Mode(0)
	QIcon__Disabled QIcon__Mode = QIcon__Mode(1)
	QIcon__Active   QIcon__Mode = QIcon__Mode(2)
	QIcon__Selected QIcon__Mode = QIcon__Mode(3)
)

//go:generate stringer -type=QIcon__State
//QIcon::State
type QIcon__State int64

const (
	QIcon__On  QIcon__State = QIcon__State(0)
	QIcon__Off QIcon__State = QIcon__State(1)
)

func NewQIcon() *QIcon {
	tmpValue := NewQIconFromPointer(C.QIcon_NewQIcon())
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func NewQIcon2(pixmap QPixmap_ITF) *QIcon {
	tmpValue := NewQIconFromPointer(C.QIcon_NewQIcon2(PointerFromQPixmap(pixmap)))
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func NewQIcon3(other QIcon_ITF) *QIcon {
	tmpValue := NewQIconFromPointer(C.QIcon_NewQIcon3(PointerFromQIcon(other)))
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func NewQIcon4(other QIcon_ITF) *QIcon {
	tmpValue := NewQIconFromPointer(C.QIcon_NewQIcon4(PointerFromQIcon(other)))
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func NewQIcon5(fileName string) *QIcon {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	tmpValue := NewQIconFromPointer(C.QIcon_NewQIcon5(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}))
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func NewQIcon6(engine QIconEngine_ITF) *QIcon {
	tmpValue := NewQIconFromPointer(C.QIcon_NewQIcon6(PointerFromQIconEngine(engine)))
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func (ptr *QIcon) ActualSize(size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QIcon_ActualSize(ptr.Pointer(), core.PointerFromQSize(size), C.longlong(mode), C.longlong(state)))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QIcon) ActualSize2(window QWindow_ITF, size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QIcon_ActualSize2(ptr.Pointer(), PointerFromQWindow(window), core.PointerFromQSize(size), C.longlong(mode), C.longlong(state)))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QIcon) AddFile(fileName string, size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		C.QIcon_AddFile(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, core.PointerFromQSize(size), C.longlong(mode), C.longlong(state))
	}
}

func (ptr *QIcon) AddPixmap(pixmap QPixmap_ITF, mode QIcon__Mode, state QIcon__State) {
	if ptr.Pointer() != nil {
		C.QIcon_AddPixmap(ptr.Pointer(), PointerFromQPixmap(pixmap), C.longlong(mode), C.longlong(state))
	}
}

func (ptr *QIcon) AvailableSizes(mode QIcon__Mode, state QIcon__State) []*core.QSize {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*core.QSize {
			out := make([]*core.QSize, int(l.len))
			tmpList := NewQIconFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__availableSizes_atList(i)
			}
			return out
		}(C.QIcon_AvailableSizes(ptr.Pointer(), C.longlong(mode), C.longlong(state)))
	}
	return make([]*core.QSize, 0)
}

func (ptr *QIcon) CacheKey() int64 {
	if ptr.Pointer() != nil {
		return int64(C.QIcon_CacheKey(ptr.Pointer()))
	}
	return 0
}

func QIcon_FallbackSearchPaths() []string {
	return unpackStringList(cGoUnpackString(C.QIcon_QIcon_FallbackSearchPaths()))
}

func (ptr *QIcon) FallbackSearchPaths() []string {
	return unpackStringList(cGoUnpackString(C.QIcon_QIcon_FallbackSearchPaths()))
}

func QIcon_FallbackThemeName() string {
	return cGoUnpackString(C.QIcon_QIcon_FallbackThemeName())
}

func (ptr *QIcon) FallbackThemeName() string {
	return cGoUnpackString(C.QIcon_QIcon_FallbackThemeName())
}

func QIcon_FromTheme(name string) *QIcon {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	tmpValue := NewQIconFromPointer(C.QIcon_QIcon_FromTheme(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}))
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func (ptr *QIcon) FromTheme(name string) *QIcon {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	tmpValue := NewQIconFromPointer(C.QIcon_QIcon_FromTheme(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}))
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func QIcon_FromTheme2(name string, fallback QIcon_ITF) *QIcon {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	tmpValue := NewQIconFromPointer(C.QIcon_QIcon_FromTheme2(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}, PointerFromQIcon(fallback)))
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func (ptr *QIcon) FromTheme2(name string, fallback QIcon_ITF) *QIcon {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	tmpValue := NewQIconFromPointer(C.QIcon_QIcon_FromTheme2(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}, PointerFromQIcon(fallback)))
	qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
	return tmpValue
}

func QIcon_HasThemeIcon(name string) bool {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	return int8(C.QIcon_QIcon_HasThemeIcon(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})) != 0
}

func (ptr *QIcon) HasThemeIcon(name string) bool {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	return int8(C.QIcon_QIcon_HasThemeIcon(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})) != 0
}

func (ptr *QIcon) IsMask() bool {
	if ptr.Pointer() != nil {
		return int8(C.QIcon_IsMask(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QIcon) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QIcon_IsNull(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QIcon) Name() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QIcon_Name(ptr.Pointer()))
	}
	return ""
}

func (ptr *QIcon) Paint(painter QPainter_ITF, rect core.QRect_ITF, alignment core.Qt__AlignmentFlag, mode QIcon__Mode, state QIcon__State) {
	if ptr.Pointer() != nil {
		C.QIcon_Paint(ptr.Pointer(), PointerFromQPainter(painter), core.PointerFromQRect(rect), C.longlong(alignment), C.longlong(mode), C.longlong(state))
	}
}

func (ptr *QIcon) Paint2(painter QPainter_ITF, x int, y int, w int, h int, alignment core.Qt__AlignmentFlag, mode QIcon__Mode, state QIcon__State) {
	if ptr.Pointer() != nil {
		C.QIcon_Paint2(ptr.Pointer(), PointerFromQPainter(painter), C.int(int32(x)), C.int(int32(y)), C.int(int32(w)), C.int(int32(h)), C.longlong(alignment), C.longlong(mode), C.longlong(state))
	}
}

func (ptr *QIcon) Pixmap(size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QIcon_Pixmap(ptr.Pointer(), core.PointerFromQSize(size), C.longlong(mode), C.longlong(state)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QIcon) Pixmap2(w int, h int, mode QIcon__Mode, state QIcon__State) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QIcon_Pixmap2(ptr.Pointer(), C.int(int32(w)), C.int(int32(h)), C.longlong(mode), C.longlong(state)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QIcon) Pixmap3(extent int, mode QIcon__Mode, state QIcon__State) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QIcon_Pixmap3(ptr.Pointer(), C.int(int32(extent)), C.longlong(mode), C.longlong(state)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QIcon) Pixmap4(window QWindow_ITF, size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QIcon_Pixmap4(ptr.Pointer(), PointerFromQWindow(window), core.PointerFromQSize(size), C.longlong(mode), C.longlong(state)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func QIcon_SetFallbackSearchPaths(paths []string) {
	pathsC := C.CString(strings.Join(paths, "¡¦!"))
	defer C.free(unsafe.Pointer(pathsC))
	C.QIcon_QIcon_SetFallbackSearchPaths(C.struct_QtGui_PackedString{data: pathsC, len: C.longlong(len(strings.Join(paths, "¡¦!")))})
}

func (ptr *QIcon) SetFallbackSearchPaths(paths []string) {
	pathsC := C.CString(strings.Join(paths, "¡¦!"))
	defer C.free(unsafe.Pointer(pathsC))
	C.QIcon_QIcon_SetFallbackSearchPaths(C.struct_QtGui_PackedString{data: pathsC, len: C.longlong(len(strings.Join(paths, "¡¦!")))})
}

func QIcon_SetFallbackThemeName(name string) {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	C.QIcon_QIcon_SetFallbackThemeName(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
}

func (ptr *QIcon) SetFallbackThemeName(name string) {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	C.QIcon_QIcon_SetFallbackThemeName(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
}

func (ptr *QIcon) SetIsMask(isMask bool) {
	if ptr.Pointer() != nil {
		C.QIcon_SetIsMask(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(isMask))))
	}
}

func QIcon_SetThemeName(name string) {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	C.QIcon_QIcon_SetThemeName(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
}

func (ptr *QIcon) SetThemeName(name string) {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	C.QIcon_QIcon_SetThemeName(C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
}

func QIcon_SetThemeSearchPaths(paths []string) {
	pathsC := C.CString(strings.Join(paths, "¡¦!"))
	defer C.free(unsafe.Pointer(pathsC))
	C.QIcon_QIcon_SetThemeSearchPaths(C.struct_QtGui_PackedString{data: pathsC, len: C.longlong(len(strings.Join(paths, "¡¦!")))})
}

func (ptr *QIcon) SetThemeSearchPaths(paths []string) {
	pathsC := C.CString(strings.Join(paths, "¡¦!"))
	defer C.free(unsafe.Pointer(pathsC))
	C.QIcon_QIcon_SetThemeSearchPaths(C.struct_QtGui_PackedString{data: pathsC, len: C.longlong(len(strings.Join(paths, "¡¦!")))})
}

func (ptr *QIcon) Swap(other QIcon_ITF) {
	if ptr.Pointer() != nil {
		C.QIcon_Swap(ptr.Pointer(), PointerFromQIcon(other))
	}
}

func QIcon_ThemeName() string {
	return cGoUnpackString(C.QIcon_QIcon_ThemeName())
}

func (ptr *QIcon) ThemeName() string {
	return cGoUnpackString(C.QIcon_QIcon_ThemeName())
}

func QIcon_ThemeSearchPaths() []string {
	return unpackStringList(cGoUnpackString(C.QIcon_QIcon_ThemeSearchPaths()))
}

func (ptr *QIcon) ThemeSearchPaths() []string {
	return unpackStringList(cGoUnpackString(C.QIcon_QIcon_ThemeSearchPaths()))
}

func (ptr *QIcon) DestroyQIcon() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QIcon_DestroyQIcon(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QIcon) ToVariant() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QIcon_ToVariant(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QIcon) __availableSizes_atList(i int) *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QIcon___availableSizes_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QIcon) __availableSizes_setList(i core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QIcon___availableSizes_setList(ptr.Pointer(), core.PointerFromQSize(i))
	}
}

func (ptr *QIcon) __availableSizes_newList() unsafe.Pointer {
	return C.QIcon___availableSizes_newList(ptr.Pointer())
}

type QIconDragEvent struct {
	core.QEvent
}

type QIconDragEvent_ITF interface {
	core.QEvent_ITF
	QIconDragEvent_PTR() *QIconDragEvent
}

func (ptr *QIconDragEvent) QIconDragEvent_PTR() *QIconDragEvent {
	return ptr
}

func (ptr *QIconDragEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QIconDragEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQIconDragEvent(ptr QIconDragEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QIconDragEvent_PTR().Pointer()
	}
	return nil
}

func NewQIconDragEventFromPointer(ptr unsafe.Pointer) (n *QIconDragEvent) {
	n = new(QIconDragEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QIconDragEvent) DestroyQIconDragEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQIconDragEvent() *QIconDragEvent {
	tmpValue := NewQIconDragEventFromPointer(C.QIconDragEvent_NewQIconDragEvent())
	qt.SetFinalizer(tmpValue, (*QIconDragEvent).DestroyQIconDragEvent)
	return tmpValue
}

type QIconEngine struct {
	ptr unsafe.Pointer
}

type QIconEngine_ITF interface {
	QIconEngine_PTR() *QIconEngine
}

func (ptr *QIconEngine) QIconEngine_PTR() *QIconEngine {
	return ptr
}

func (ptr *QIconEngine) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QIconEngine) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQIconEngine(ptr QIconEngine_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QIconEngine_PTR().Pointer()
	}
	return nil
}

func NewQIconEngineFromPointer(ptr unsafe.Pointer) (n *QIconEngine) {
	n = new(QIconEngine)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QIconEngine__IconEngineHook
//QIconEngine::IconEngineHook
type QIconEngine__IconEngineHook int64

const (
	QIconEngine__AvailableSizesHook QIconEngine__IconEngineHook = QIconEngine__IconEngineHook(1)
	QIconEngine__IconNameHook       QIconEngine__IconEngineHook = QIconEngine__IconEngineHook(2)
	QIconEngine__IsNullHook         QIconEngine__IconEngineHook = QIconEngine__IconEngineHook(3)
	QIconEngine__ScaledPixmapHook   QIconEngine__IconEngineHook = QIconEngine__IconEngineHook(4)
)

func NewQIconEngine() *QIconEngine {
	return NewQIconEngineFromPointer(C.QIconEngine_NewQIconEngine())
}

//export callbackQIconEngine_ActualSize
func callbackQIconEngine_ActualSize(ptr unsafe.Pointer, size unsafe.Pointer, mode C.longlong, state C.longlong) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "actualSize"); signal != nil {
		return core.PointerFromQSize((*(*func(*core.QSize, QIcon__Mode, QIcon__State) *core.QSize)(signal))(core.NewQSizeFromPointer(size), QIcon__Mode(mode), QIcon__State(state)))
	}

	return core.PointerFromQSize(NewQIconEngineFromPointer(ptr).ActualSizeDefault(core.NewQSizeFromPointer(size), QIcon__Mode(mode), QIcon__State(state)))
}

func (ptr *QIconEngine) ConnectActualSize(f func(size *core.QSize, mode QIcon__Mode, state QIcon__State) *core.QSize) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "actualSize"); signal != nil {
			f := func(size *core.QSize, mode QIcon__Mode, state QIcon__State) *core.QSize {
				(*(*func(*core.QSize, QIcon__Mode, QIcon__State) *core.QSize)(signal))(size, mode, state)
				return f(size, mode, state)
			}
			qt.ConnectSignal(ptr.Pointer(), "actualSize", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "actualSize", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectActualSize() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "actualSize")
	}
}

func (ptr *QIconEngine) ActualSize(size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QIconEngine_ActualSize(ptr.Pointer(), core.PointerFromQSize(size), C.longlong(mode), C.longlong(state)))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QIconEngine) ActualSizeDefault(size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QIconEngine_ActualSizeDefault(ptr.Pointer(), core.PointerFromQSize(size), C.longlong(mode), C.longlong(state)))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

//export callbackQIconEngine_AddFile
func callbackQIconEngine_AddFile(ptr unsafe.Pointer, fileName C.struct_QtGui_PackedString, size unsafe.Pointer, mode C.longlong, state C.longlong) {
	if signal := qt.GetSignal(ptr, "addFile"); signal != nil {
		(*(*func(string, *core.QSize, QIcon__Mode, QIcon__State))(signal))(cGoUnpackString(fileName), core.NewQSizeFromPointer(size), QIcon__Mode(mode), QIcon__State(state))
	} else {
		NewQIconEngineFromPointer(ptr).AddFileDefault(cGoUnpackString(fileName), core.NewQSizeFromPointer(size), QIcon__Mode(mode), QIcon__State(state))
	}
}

func (ptr *QIconEngine) ConnectAddFile(f func(fileName string, size *core.QSize, mode QIcon__Mode, state QIcon__State)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "addFile"); signal != nil {
			f := func(fileName string, size *core.QSize, mode QIcon__Mode, state QIcon__State) {
				(*(*func(string, *core.QSize, QIcon__Mode, QIcon__State))(signal))(fileName, size, mode, state)
				f(fileName, size, mode, state)
			}
			qt.ConnectSignal(ptr.Pointer(), "addFile", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "addFile", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectAddFile() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "addFile")
	}
}

func (ptr *QIconEngine) AddFile(fileName string, size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		C.QIconEngine_AddFile(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, core.PointerFromQSize(size), C.longlong(mode), C.longlong(state))
	}
}

func (ptr *QIconEngine) AddFileDefault(fileName string, size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		C.QIconEngine_AddFileDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, core.PointerFromQSize(size), C.longlong(mode), C.longlong(state))
	}
}

//export callbackQIconEngine_AddPixmap
func callbackQIconEngine_AddPixmap(ptr unsafe.Pointer, pixmap unsafe.Pointer, mode C.longlong, state C.longlong) {
	if signal := qt.GetSignal(ptr, "addPixmap"); signal != nil {
		(*(*func(*QPixmap, QIcon__Mode, QIcon__State))(signal))(NewQPixmapFromPointer(pixmap), QIcon__Mode(mode), QIcon__State(state))
	} else {
		NewQIconEngineFromPointer(ptr).AddPixmapDefault(NewQPixmapFromPointer(pixmap), QIcon__Mode(mode), QIcon__State(state))
	}
}

func (ptr *QIconEngine) ConnectAddPixmap(f func(pixmap *QPixmap, mode QIcon__Mode, state QIcon__State)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "addPixmap"); signal != nil {
			f := func(pixmap *QPixmap, mode QIcon__Mode, state QIcon__State) {
				(*(*func(*QPixmap, QIcon__Mode, QIcon__State))(signal))(pixmap, mode, state)
				f(pixmap, mode, state)
			}
			qt.ConnectSignal(ptr.Pointer(), "addPixmap", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "addPixmap", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectAddPixmap() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "addPixmap")
	}
}

func (ptr *QIconEngine) AddPixmap(pixmap QPixmap_ITF, mode QIcon__Mode, state QIcon__State) {
	if ptr.Pointer() != nil {
		C.QIconEngine_AddPixmap(ptr.Pointer(), PointerFromQPixmap(pixmap), C.longlong(mode), C.longlong(state))
	}
}

func (ptr *QIconEngine) AddPixmapDefault(pixmap QPixmap_ITF, mode QIcon__Mode, state QIcon__State) {
	if ptr.Pointer() != nil {
		C.QIconEngine_AddPixmapDefault(ptr.Pointer(), PointerFromQPixmap(pixmap), C.longlong(mode), C.longlong(state))
	}
}

//export callbackQIconEngine_AvailableSizes
func callbackQIconEngine_AvailableSizes(ptr unsafe.Pointer, mode C.longlong, state C.longlong) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "availableSizes"); signal != nil {
		return func() unsafe.Pointer {
			tmpList := NewQIconEngineFromPointer(NewQIconEngineFromPointer(nil).__availableSizes_newList())
			for _, v := range (*(*func(QIcon__Mode, QIcon__State) []*core.QSize)(signal))(QIcon__Mode(mode), QIcon__State(state)) {
				tmpList.__availableSizes_setList(v)
			}
			return tmpList.Pointer()
		}()
	}

	return func() unsafe.Pointer {
		tmpList := NewQIconEngineFromPointer(NewQIconEngineFromPointer(nil).__availableSizes_newList())
		for _, v := range NewQIconEngineFromPointer(ptr).AvailableSizesDefault(QIcon__Mode(mode), QIcon__State(state)) {
			tmpList.__availableSizes_setList(v)
		}
		return tmpList.Pointer()
	}()
}

func (ptr *QIconEngine) ConnectAvailableSizes(f func(mode QIcon__Mode, state QIcon__State) []*core.QSize) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "availableSizes"); signal != nil {
			f := func(mode QIcon__Mode, state QIcon__State) []*core.QSize {
				(*(*func(QIcon__Mode, QIcon__State) []*core.QSize)(signal))(mode, state)
				return f(mode, state)
			}
			qt.ConnectSignal(ptr.Pointer(), "availableSizes", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "availableSizes", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectAvailableSizes() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "availableSizes")
	}
}

func (ptr *QIconEngine) AvailableSizes(mode QIcon__Mode, state QIcon__State) []*core.QSize {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*core.QSize {
			out := make([]*core.QSize, int(l.len))
			tmpList := NewQIconEngineFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__availableSizes_atList(i)
			}
			return out
		}(C.QIconEngine_AvailableSizes(ptr.Pointer(), C.longlong(mode), C.longlong(state)))
	}
	return make([]*core.QSize, 0)
}

func (ptr *QIconEngine) AvailableSizesDefault(mode QIcon__Mode, state QIcon__State) []*core.QSize {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*core.QSize {
			out := make([]*core.QSize, int(l.len))
			tmpList := NewQIconEngineFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__availableSizes_atList(i)
			}
			return out
		}(C.QIconEngine_AvailableSizesDefault(ptr.Pointer(), C.longlong(mode), C.longlong(state)))
	}
	return make([]*core.QSize, 0)
}

//export callbackQIconEngine_Clone
func callbackQIconEngine_Clone(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "clone"); signal != nil {
		return PointerFromQIconEngine((*(*func() *QIconEngine)(signal))())
	}

	return PointerFromQIconEngine(NewQIconEngine())
}

func (ptr *QIconEngine) ConnectClone(f func() *QIconEngine) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "clone"); signal != nil {
			f := func() *QIconEngine {
				(*(*func() *QIconEngine)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "clone", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "clone", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectClone() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "clone")
	}
}

func (ptr *QIconEngine) Clone() *QIconEngine {
	if ptr.Pointer() != nil {
		return NewQIconEngineFromPointer(C.QIconEngine_Clone(ptr.Pointer()))
	}
	return nil
}

//export callbackQIconEngine_IconName
func callbackQIconEngine_IconName(ptr unsafe.Pointer) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "iconName"); signal != nil {
		tempVal := (*(*func() string)(signal))()
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := NewQIconEngineFromPointer(ptr).IconNameDefault()
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QIconEngine) ConnectIconName(f func() string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "iconName"); signal != nil {
			f := func() string {
				(*(*func() string)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "iconName", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "iconName", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectIconName() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "iconName")
	}
}

func (ptr *QIconEngine) IconName() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QIconEngine_IconName(ptr.Pointer()))
	}
	return ""
}

func (ptr *QIconEngine) IconNameDefault() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QIconEngine_IconNameDefault(ptr.Pointer()))
	}
	return ""
}

func (ptr *QIconEngine) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QIconEngine_IsNull(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQIconEngine_Key
func callbackQIconEngine_Key(ptr unsafe.Pointer) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "key"); signal != nil {
		tempVal := (*(*func() string)(signal))()
		return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
	}
	tempVal := NewQIconEngineFromPointer(ptr).KeyDefault()
	return C.struct_QtGui_PackedString{data: C.CString(tempVal), len: C.longlong(len(tempVal))}
}

func (ptr *QIconEngine) ConnectKey(f func() string) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "key"); signal != nil {
			f := func() string {
				(*(*func() string)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "key", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "key", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectKey() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "key")
	}
}

func (ptr *QIconEngine) Key() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QIconEngine_Key(ptr.Pointer()))
	}
	return ""
}

func (ptr *QIconEngine) KeyDefault() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QIconEngine_KeyDefault(ptr.Pointer()))
	}
	return ""
}

//export callbackQIconEngine_Paint
func callbackQIconEngine_Paint(ptr unsafe.Pointer, painter unsafe.Pointer, rect unsafe.Pointer, mode C.longlong, state C.longlong) {
	if signal := qt.GetSignal(ptr, "paint"); signal != nil {
		(*(*func(*QPainter, *core.QRect, QIcon__Mode, QIcon__State))(signal))(NewQPainterFromPointer(painter), core.NewQRectFromPointer(rect), QIcon__Mode(mode), QIcon__State(state))
	}

}

func (ptr *QIconEngine) ConnectPaint(f func(painter *QPainter, rect *core.QRect, mode QIcon__Mode, state QIcon__State)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "paint"); signal != nil {
			f := func(painter *QPainter, rect *core.QRect, mode QIcon__Mode, state QIcon__State) {
				(*(*func(*QPainter, *core.QRect, QIcon__Mode, QIcon__State))(signal))(painter, rect, mode, state)
				f(painter, rect, mode, state)
			}
			qt.ConnectSignal(ptr.Pointer(), "paint", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "paint", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectPaint() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "paint")
	}
}

func (ptr *QIconEngine) Paint(painter QPainter_ITF, rect core.QRect_ITF, mode QIcon__Mode, state QIcon__State) {
	if ptr.Pointer() != nil {
		C.QIconEngine_Paint(ptr.Pointer(), PointerFromQPainter(painter), core.PointerFromQRect(rect), C.longlong(mode), C.longlong(state))
	}
}

//export callbackQIconEngine_Pixmap
func callbackQIconEngine_Pixmap(ptr unsafe.Pointer, size unsafe.Pointer, mode C.longlong, state C.longlong) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "pixmap"); signal != nil {
		return PointerFromQPixmap((*(*func(*core.QSize, QIcon__Mode, QIcon__State) *QPixmap)(signal))(core.NewQSizeFromPointer(size), QIcon__Mode(mode), QIcon__State(state)))
	}

	return PointerFromQPixmap(NewQIconEngineFromPointer(ptr).PixmapDefault(core.NewQSizeFromPointer(size), QIcon__Mode(mode), QIcon__State(state)))
}

func (ptr *QIconEngine) ConnectPixmap(f func(size *core.QSize, mode QIcon__Mode, state QIcon__State) *QPixmap) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "pixmap"); signal != nil {
			f := func(size *core.QSize, mode QIcon__Mode, state QIcon__State) *QPixmap {
				(*(*func(*core.QSize, QIcon__Mode, QIcon__State) *QPixmap)(signal))(size, mode, state)
				return f(size, mode, state)
			}
			qt.ConnectSignal(ptr.Pointer(), "pixmap", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "pixmap", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectPixmap() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "pixmap")
	}
}

func (ptr *QIconEngine) Pixmap(size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QIconEngine_Pixmap(ptr.Pointer(), core.PointerFromQSize(size), C.longlong(mode), C.longlong(state)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QIconEngine) PixmapDefault(size core.QSize_ITF, mode QIcon__Mode, state QIcon__State) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QIconEngine_PixmapDefault(ptr.Pointer(), core.PointerFromQSize(size), C.longlong(mode), C.longlong(state)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

//export callbackQIconEngine_Read
func callbackQIconEngine_Read(ptr unsafe.Pointer, in unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "read"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QDataStream) bool)(signal))(core.NewQDataStreamFromPointer(in)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQIconEngineFromPointer(ptr).ReadDefault(core.NewQDataStreamFromPointer(in)))))
}

func (ptr *QIconEngine) ConnectRead(f func(in *core.QDataStream) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "read"); signal != nil {
			f := func(in *core.QDataStream) bool {
				(*(*func(*core.QDataStream) bool)(signal))(in)
				return f(in)
			}
			qt.ConnectSignal(ptr.Pointer(), "read", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "read", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectRead() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "read")
	}
}

func (ptr *QIconEngine) Read(in core.QDataStream_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QIconEngine_Read(ptr.Pointer(), core.PointerFromQDataStream(in))) != 0
	}
	return false
}

func (ptr *QIconEngine) ReadDefault(in core.QDataStream_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QIconEngine_ReadDefault(ptr.Pointer(), core.PointerFromQDataStream(in))) != 0
	}
	return false
}

func (ptr *QIconEngine) ScaledPixmap(size core.QSize_ITF, mode QIcon__Mode, state QIcon__State, scale float64) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QIconEngine_ScaledPixmap(ptr.Pointer(), core.PointerFromQSize(size), C.longlong(mode), C.longlong(state), C.double(scale)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

//export callbackQIconEngine_Virtual_hook
func callbackQIconEngine_Virtual_hook(ptr unsafe.Pointer, id C.int, data unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "virtual_hook"); signal != nil {
		(*(*func(int, unsafe.Pointer))(signal))(int(int32(id)), data)
	} else {
		NewQIconEngineFromPointer(ptr).Virtual_hookDefault(int(int32(id)), data)
	}
}

func (ptr *QIconEngine) ConnectVirtual_hook(f func(id int, data unsafe.Pointer)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "virtual_hook"); signal != nil {
			f := func(id int, data unsafe.Pointer) {
				(*(*func(int, unsafe.Pointer))(signal))(id, data)
				f(id, data)
			}
			qt.ConnectSignal(ptr.Pointer(), "virtual_hook", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "virtual_hook", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectVirtual_hook() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "virtual_hook")
	}
}

func (ptr *QIconEngine) Virtual_hook(id int, data unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QIconEngine_Virtual_hook(ptr.Pointer(), C.int(int32(id)), data)
	}
}

func (ptr *QIconEngine) Virtual_hookDefault(id int, data unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QIconEngine_Virtual_hookDefault(ptr.Pointer(), C.int(int32(id)), data)
	}
}

//export callbackQIconEngine_Write
func callbackQIconEngine_Write(ptr unsafe.Pointer, out unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "write"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QDataStream) bool)(signal))(core.NewQDataStreamFromPointer(out)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQIconEngineFromPointer(ptr).WriteDefault(core.NewQDataStreamFromPointer(out)))))
}

func (ptr *QIconEngine) ConnectWrite(f func(out *core.QDataStream) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "write"); signal != nil {
			f := func(out *core.QDataStream) bool {
				(*(*func(*core.QDataStream) bool)(signal))(out)
				return f(out)
			}
			qt.ConnectSignal(ptr.Pointer(), "write", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "write", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectWrite() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "write")
	}
}

func (ptr *QIconEngine) Write(out core.QDataStream_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QIconEngine_Write(ptr.Pointer(), core.PointerFromQDataStream(out))) != 0
	}
	return false
}

func (ptr *QIconEngine) WriteDefault(out core.QDataStream_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QIconEngine_WriteDefault(ptr.Pointer(), core.PointerFromQDataStream(out))) != 0
	}
	return false
}

//export callbackQIconEngine_DestroyQIconEngine
func callbackQIconEngine_DestroyQIconEngine(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QIconEngine"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQIconEngineFromPointer(ptr).DestroyQIconEngineDefault()
	}
}

func (ptr *QIconEngine) ConnectDestroyQIconEngine(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QIconEngine"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QIconEngine", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QIconEngine", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEngine) DisconnectDestroyQIconEngine() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QIconEngine")
	}
}

func (ptr *QIconEngine) DestroyQIconEngine() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QIconEngine_DestroyQIconEngine(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QIconEngine) DestroyQIconEngineDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QIconEngine_DestroyQIconEngineDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QIconEngine) __availableSizes_atList(i int) *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QIconEngine___availableSizes_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QIconEngine) __availableSizes_setList(i core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QIconEngine___availableSizes_setList(ptr.Pointer(), core.PointerFromQSize(i))
	}
}

func (ptr *QIconEngine) __availableSizes_newList() unsafe.Pointer {
	return C.QIconEngine___availableSizes_newList(ptr.Pointer())
}

type QIconEnginePlugin struct {
	core.QObject
}

type QIconEnginePlugin_ITF interface {
	core.QObject_ITF
	QIconEnginePlugin_PTR() *QIconEnginePlugin
}

func (ptr *QIconEnginePlugin) QIconEnginePlugin_PTR() *QIconEnginePlugin {
	return ptr
}

func (ptr *QIconEnginePlugin) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QIconEnginePlugin) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQIconEnginePlugin(ptr QIconEnginePlugin_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QIconEnginePlugin_PTR().Pointer()
	}
	return nil
}

func NewQIconEnginePluginFromPointer(ptr unsafe.Pointer) (n *QIconEnginePlugin) {
	n = new(QIconEnginePlugin)
	n.SetPointer(ptr)
	return
}
func NewQIconEnginePlugin(parent core.QObject_ITF) *QIconEnginePlugin {
	tmpValue := NewQIconEnginePluginFromPointer(C.QIconEnginePlugin_NewQIconEnginePlugin(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQIconEnginePlugin_Create
func callbackQIconEnginePlugin_Create(ptr unsafe.Pointer, filename C.struct_QtGui_PackedString) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "create"); signal != nil {
		return PointerFromQIconEngine((*(*func(string) *QIconEngine)(signal))(cGoUnpackString(filename)))
	}

	return PointerFromQIconEngine(NewQIconEngine())
}

func (ptr *QIconEnginePlugin) ConnectCreate(f func(filename string) *QIconEngine) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "create"); signal != nil {
			f := func(filename string) *QIconEngine {
				(*(*func(string) *QIconEngine)(signal))(filename)
				return f(filename)
			}
			qt.ConnectSignal(ptr.Pointer(), "create", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "create", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEnginePlugin) DisconnectCreate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "create")
	}
}

func (ptr *QIconEnginePlugin) Create(filename string) *QIconEngine {
	if ptr.Pointer() != nil {
		var filenameC *C.char
		if filename != "" {
			filenameC = C.CString(filename)
			defer C.free(unsafe.Pointer(filenameC))
		}
		return NewQIconEngineFromPointer(C.QIconEnginePlugin_Create(ptr.Pointer(), C.struct_QtGui_PackedString{data: filenameC, len: C.longlong(len(filename))}))
	}
	return nil
}

//export callbackQIconEnginePlugin_DestroyQIconEnginePlugin
func callbackQIconEnginePlugin_DestroyQIconEnginePlugin(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QIconEnginePlugin"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQIconEnginePluginFromPointer(ptr).DestroyQIconEnginePluginDefault()
	}
}

func (ptr *QIconEnginePlugin) ConnectDestroyQIconEnginePlugin(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QIconEnginePlugin"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QIconEnginePlugin", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QIconEnginePlugin", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIconEnginePlugin) DisconnectDestroyQIconEnginePlugin() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QIconEnginePlugin")
	}
}

func (ptr *QIconEnginePlugin) DestroyQIconEnginePlugin() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QIconEnginePlugin_DestroyQIconEnginePlugin(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QIconEnginePlugin) DestroyQIconEnginePluginDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QIconEnginePlugin_DestroyQIconEnginePluginDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QIconEnginePlugin) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QIconEnginePlugin___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QIconEnginePlugin) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QIconEnginePlugin___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QIconEnginePlugin) __children_newList() unsafe.Pointer {
	return C.QIconEnginePlugin___children_newList(ptr.Pointer())
}

func (ptr *QIconEnginePlugin) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QIconEnginePlugin___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QIconEnginePlugin) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QIconEnginePlugin___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QIconEnginePlugin) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QIconEnginePlugin___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QIconEnginePlugin) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QIconEnginePlugin___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QIconEnginePlugin) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QIconEnginePlugin___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QIconEnginePlugin) __findChildren_newList() unsafe.Pointer {
	return C.QIconEnginePlugin___findChildren_newList(ptr.Pointer())
}

func (ptr *QIconEnginePlugin) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QIconEnginePlugin___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QIconEnginePlugin) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QIconEnginePlugin___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QIconEnginePlugin) __findChildren_newList3() unsafe.Pointer {
	return C.QIconEnginePlugin___findChildren_newList3(ptr.Pointer())
}

//export callbackQIconEnginePlugin_ChildEvent
func callbackQIconEnginePlugin_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQIconEnginePluginFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QIconEnginePlugin) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QIconEnginePlugin_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQIconEnginePlugin_ConnectNotify
func callbackQIconEnginePlugin_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQIconEnginePluginFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QIconEnginePlugin) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QIconEnginePlugin_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQIconEnginePlugin_CustomEvent
func callbackQIconEnginePlugin_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQIconEnginePluginFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QIconEnginePlugin) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QIconEnginePlugin_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQIconEnginePlugin_DeleteLater
func callbackQIconEnginePlugin_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQIconEnginePluginFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QIconEnginePlugin) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QIconEnginePlugin_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQIconEnginePlugin_Destroyed
func callbackQIconEnginePlugin_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQIconEnginePlugin_DisconnectNotify
func callbackQIconEnginePlugin_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQIconEnginePluginFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QIconEnginePlugin) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QIconEnginePlugin_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQIconEnginePlugin_Event
func callbackQIconEnginePlugin_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQIconEnginePluginFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QIconEnginePlugin) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QIconEnginePlugin_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQIconEnginePlugin_EventFilter
func callbackQIconEnginePlugin_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQIconEnginePluginFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QIconEnginePlugin) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QIconEnginePlugin_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQIconEnginePlugin_MetaObject
func callbackQIconEnginePlugin_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQIconEnginePluginFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QIconEnginePlugin) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QIconEnginePlugin_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQIconEnginePlugin_ObjectNameChanged
func callbackQIconEnginePlugin_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQIconEnginePlugin_TimerEvent
func callbackQIconEnginePlugin_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQIconEnginePluginFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QIconEnginePlugin) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QIconEnginePlugin_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QImage struct {
	QPaintDevice
}

type QImage_ITF interface {
	QPaintDevice_ITF
	QImage_PTR() *QImage
}

func (ptr *QImage) QImage_PTR() *QImage {
	return ptr
}

func (ptr *QImage) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintDevice_PTR().Pointer()
	}
	return nil
}

func (ptr *QImage) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QPaintDevice_PTR().SetPointer(p)
	}
}

func PointerFromQImage(ptr QImage_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QImage_PTR().Pointer()
	}
	return nil
}

func NewQImageFromPointer(ptr unsafe.Pointer) (n *QImage) {
	n = new(QImage)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QImage__InvertMode
//QImage::InvertMode
type QImage__InvertMode int64

const (
	QImage__InvertRgb  QImage__InvertMode = QImage__InvertMode(0)
	QImage__InvertRgba QImage__InvertMode = QImage__InvertMode(1)
)

//go:generate stringer -type=QImage__Format
//QImage::Format
type QImage__Format int64

const (
	QImage__Format_Invalid                QImage__Format = QImage__Format(0)
	QImage__Format_Mono                   QImage__Format = QImage__Format(1)
	QImage__Format_MonoLSB                QImage__Format = QImage__Format(2)
	QImage__Format_Indexed8               QImage__Format = QImage__Format(3)
	QImage__Format_RGB32                  QImage__Format = QImage__Format(4)
	QImage__Format_ARGB32                 QImage__Format = QImage__Format(5)
	QImage__Format_ARGB32_Premultiplied   QImage__Format = QImage__Format(6)
	QImage__Format_RGB16                  QImage__Format = QImage__Format(7)
	QImage__Format_ARGB8565_Premultiplied QImage__Format = QImage__Format(8)
	QImage__Format_RGB666                 QImage__Format = QImage__Format(9)
	QImage__Format_ARGB6666_Premultiplied QImage__Format = QImage__Format(10)
	QImage__Format_RGB555                 QImage__Format = QImage__Format(11)
	QImage__Format_ARGB8555_Premultiplied QImage__Format = QImage__Format(12)
	QImage__Format_RGB888                 QImage__Format = QImage__Format(13)
	QImage__Format_RGB444                 QImage__Format = QImage__Format(14)
	QImage__Format_ARGB4444_Premultiplied QImage__Format = QImage__Format(15)
	QImage__Format_RGBX8888               QImage__Format = QImage__Format(16)
	QImage__Format_RGBA8888               QImage__Format = QImage__Format(17)
	QImage__Format_RGBA8888_Premultiplied QImage__Format = QImage__Format(18)
	QImage__Format_BGR30                  QImage__Format = QImage__Format(19)
	QImage__Format_A2BGR30_Premultiplied  QImage__Format = QImage__Format(20)
	QImage__Format_RGB30                  QImage__Format = QImage__Format(21)
	QImage__Format_A2RGB30_Premultiplied  QImage__Format = QImage__Format(22)
	QImage__Format_Alpha8                 QImage__Format = QImage__Format(23)
	QImage__Format_Grayscale8             QImage__Format = QImage__Format(24)
	QImage__Format_RGBX64                 QImage__Format = QImage__Format(25)
	QImage__Format_RGBA64                 QImage__Format = QImage__Format(26)
	QImage__Format_RGBA64_Premultiplied   QImage__Format = QImage__Format(27)
	QImage__Format_Grayscale16            QImage__Format = QImage__Format(28)
)

func NewQImage() *QImage {
	return NewQImageFromPointer(C.QImage_NewQImage())
}

func NewQImage2(size core.QSize_ITF, format QImage__Format) *QImage {
	return NewQImageFromPointer(C.QImage_NewQImage2(core.PointerFromQSize(size), C.longlong(format)))
}

func NewQImage3(width int, height int, format QImage__Format) *QImage {
	return NewQImageFromPointer(C.QImage_NewQImage3(C.int(int32(width)), C.int(int32(height)), C.longlong(format)))
}

func NewQImage4(data string, width int, height int, format QImage__Format) *QImage {
	var dataC *C.char
	if data != "" {
		dataC = C.CString(data)
		defer C.free(unsafe.Pointer(dataC))
	}
	return NewQImageFromPointer(C.QImage_NewQImage4(dataC, C.int(int32(width)), C.int(int32(height)), C.longlong(format)))
}

func NewQImage5(data string, width int, height int, format QImage__Format) *QImage {
	var dataC *C.char
	if data != "" {
		dataC = C.CString(data)
		defer C.free(unsafe.Pointer(dataC))
	}
	return NewQImageFromPointer(C.QImage_NewQImage5(dataC, C.int(int32(width)), C.int(int32(height)), C.longlong(format)))
}

func NewQImage6(data string, width int, height int, bytesPerLine int, format QImage__Format) *QImage {
	var dataC *C.char
	if data != "" {
		dataC = C.CString(data)
		defer C.free(unsafe.Pointer(dataC))
	}
	return NewQImageFromPointer(C.QImage_NewQImage6(dataC, C.int(int32(width)), C.int(int32(height)), C.int(int32(bytesPerLine)), C.longlong(format)))
}

func NewQImage7(data string, width int, height int, bytesPerLine int, format QImage__Format) *QImage {
	var dataC *C.char
	if data != "" {
		dataC = C.CString(data)
		defer C.free(unsafe.Pointer(dataC))
	}
	return NewQImageFromPointer(C.QImage_NewQImage7(dataC, C.int(int32(width)), C.int(int32(height)), C.int(int32(bytesPerLine)), C.longlong(format)))
}

func NewQImage9(fileName string, format string) *QImage {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	var formatC *C.char
	if format != "" {
		formatC = C.CString(format)
		defer C.free(unsafe.Pointer(formatC))
	}
	return NewQImageFromPointer(C.QImage_NewQImage9(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, formatC))
}

func NewQImage10(image QImage_ITF) *QImage {
	return NewQImageFromPointer(C.QImage_NewQImage10(PointerFromQImage(image)))
}

func NewQImage11(other QImage_ITF) *QImage {
	return NewQImageFromPointer(C.QImage_NewQImage11(PointerFromQImage(other)))
}

func (ptr *QImage) AllGray() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImage_AllGray(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImage) BitPlaneCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImage_BitPlaneCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImage) Bits() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QImage_Bits(ptr.Pointer()))
	}
	return ""
}

func (ptr *QImage) Bits2() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QImage_Bits2(ptr.Pointer()))
	}
	return ""
}

func (ptr *QImage) BytesPerLine() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImage_BytesPerLine(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImage) CacheKey() int64 {
	if ptr.Pointer() != nil {
		return int64(C.QImage_CacheKey(ptr.Pointer()))
	}
	return 0
}

func (ptr *QImage) Color(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QImage_Color(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QImage) ColorTable() []uint {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []uint {
			out := make([]uint, int(l.len))
			tmpList := NewQImageFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__colorTable_atList(i)
			}
			return out
		}(C.QImage_ColorTable(ptr.Pointer()))
	}
	return make([]uint, 0)
}

func (ptr *QImage) ConstBits() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QImage_ConstBits(ptr.Pointer()))
	}
	return ""
}

func (ptr *QImage) ConstScanLine(i int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QImage_ConstScanLine(ptr.Pointer(), C.int(int32(i))))
	}
	return ""
}

func (ptr *QImage) ConvertTo(format QImage__Format, flags core.Qt__ImageConversionFlag) {
	if ptr.Pointer() != nil {
		C.QImage_ConvertTo(ptr.Pointer(), C.longlong(format), C.longlong(flags))
	}
}

func (ptr *QImage) ConvertToFormat(format QImage__Format, flags core.Qt__ImageConversionFlag) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_ConvertToFormat(ptr.Pointer(), C.longlong(format), C.longlong(flags)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) ConvertToFormat2(format QImage__Format, flags core.Qt__ImageConversionFlag) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_ConvertToFormat2(ptr.Pointer(), C.longlong(format), C.longlong(flags)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) ConvertToFormat3(format QImage__Format, colorTable []uint, flags core.Qt__ImageConversionFlag) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_ConvertToFormat3(ptr.Pointer(), C.longlong(format), func() unsafe.Pointer {
			tmpList := NewQImageFromPointer(NewQImageFromPointer(nil).__convertToFormat_colorTable_newList3())
			for _, v := range colorTable {
				tmpList.__convertToFormat_colorTable_setList3(v)
			}
			return tmpList.Pointer()
		}(), C.longlong(flags)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) Copy(rectangle core.QRect_ITF) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_Copy(ptr.Pointer(), core.PointerFromQRect(rectangle)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) Copy2(x int, y int, width int, height int) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_Copy2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height))))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) CreateAlphaMask(flags core.Qt__ImageConversionFlag) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_CreateAlphaMask(ptr.Pointer(), C.longlong(flags)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) CreateHeuristicMask(clipTight bool) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_CreateHeuristicMask(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(clipTight)))))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) CreateMaskFromColor(color uint, mode core.Qt__MaskMode) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_CreateMaskFromColor(ptr.Pointer(), C.uint(uint32(color)), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) DevicePixelRatio() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QImage_DevicePixelRatio(ptr.Pointer()))
	}
	return 0
}

func (ptr *QImage) DotsPerMeterX() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImage_DotsPerMeterX(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImage) DotsPerMeterY() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImage_DotsPerMeterY(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImage) Fill(pixelValue uint) {
	if ptr.Pointer() != nil {
		C.QImage_Fill(ptr.Pointer(), C.uint(uint32(pixelValue)))
	}
}

func (ptr *QImage) Fill2(color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QImage_Fill2(ptr.Pointer(), PointerFromQColor(color))
	}
}

func (ptr *QImage) Fill3(color core.Qt__GlobalColor) {
	if ptr.Pointer() != nil {
		C.QImage_Fill3(ptr.Pointer(), C.longlong(color))
	}
}

func (ptr *QImage) Format() QImage__Format {
	if ptr.Pointer() != nil {
		return QImage__Format(C.QImage_Format(ptr.Pointer()))
	}
	return 0
}

func QImage_FromData(data []byte, size int, format string) *QImage {
	var dataC *C.char
	if len(data) != 0 {
		dataC = (*C.char)(unsafe.Pointer(&data[0]))
	}
	var formatC *C.char
	if format != "" {
		formatC = C.CString(format)
		defer C.free(unsafe.Pointer(formatC))
	}
	tmpValue := NewQImageFromPointer(C.QImage_QImage_FromData(dataC, C.int(int32(size)), formatC))
	qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
	return tmpValue
}

func (ptr *QImage) FromData(data []byte, size int, format string) *QImage {
	var dataC *C.char
	if len(data) != 0 {
		dataC = (*C.char)(unsafe.Pointer(&data[0]))
	}
	var formatC *C.char
	if format != "" {
		formatC = C.CString(format)
		defer C.free(unsafe.Pointer(formatC))
	}
	tmpValue := NewQImageFromPointer(C.QImage_QImage_FromData(dataC, C.int(int32(size)), formatC))
	qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
	return tmpValue
}

func QImage_FromData2(data core.QByteArray_ITF, format string) *QImage {
	var formatC *C.char
	if format != "" {
		formatC = C.CString(format)
		defer C.free(unsafe.Pointer(formatC))
	}
	tmpValue := NewQImageFromPointer(C.QImage_QImage_FromData2(core.PointerFromQByteArray(data), formatC))
	qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
	return tmpValue
}

func (ptr *QImage) FromData2(data core.QByteArray_ITF, format string) *QImage {
	var formatC *C.char
	if format != "" {
		formatC = C.CString(format)
		defer C.free(unsafe.Pointer(formatC))
	}
	tmpValue := NewQImageFromPointer(C.QImage_QImage_FromData2(core.PointerFromQByteArray(data), formatC))
	qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
	return tmpValue
}

func (ptr *QImage) HasAlphaChannel() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImage_HasAlphaChannel(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImage) InvertPixels(mode QImage__InvertMode) {
	if ptr.Pointer() != nil {
		C.QImage_InvertPixels(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QImage) IsGrayscale() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImage_IsGrayscale(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImage) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImage_IsNull(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImage) Load(fileName string, format string) bool {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QImage_Load(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, formatC)) != 0
	}
	return false
}

func (ptr *QImage) Load2(device core.QIODevice_ITF, format string) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QImage_Load2(ptr.Pointer(), core.PointerFromQIODevice(device), formatC)) != 0
	}
	return false
}

func (ptr *QImage) LoadFromData(data []byte, l int, format string) bool {
	if ptr.Pointer() != nil {
		var dataC *C.char
		if len(data) != 0 {
			dataC = (*C.char)(unsafe.Pointer(&data[0]))
		}
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QImage_LoadFromData(ptr.Pointer(), dataC, C.int(int32(l)), formatC)) != 0
	}
	return false
}

func (ptr *QImage) LoadFromData2(data core.QByteArray_ITF, format string) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QImage_LoadFromData2(ptr.Pointer(), core.PointerFromQByteArray(data), formatC)) != 0
	}
	return false
}

func (ptr *QImage) Mirrored(horizontal bool, vertical bool) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_Mirrored(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(horizontal))), C.char(int8(qt.GoBoolToInt(vertical)))))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) Mirrored2(horizontal bool, vertical bool) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_Mirrored2(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(horizontal))), C.char(int8(qt.GoBoolToInt(vertical)))))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) Offset() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QImage_Offset(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) Pixel(position core.QPoint_ITF) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QImage_Pixel(ptr.Pointer(), core.PointerFromQPoint(position))))
	}
	return 0
}

func (ptr *QImage) Pixel2(x int, y int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QImage_Pixel2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)))))
	}
	return 0
}

func (ptr *QImage) PixelColor(position core.QPoint_ITF) *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QImage_PixelColor(ptr.Pointer(), core.PointerFromQPoint(position)))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) PixelColor2(x int, y int) *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QImage_PixelColor2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y))))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) PixelFormat() *QPixelFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixelFormatFromPointer(C.QImage_PixelFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPixelFormat).DestroyQPixelFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) PixelIndex(position core.QPoint_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImage_PixelIndex(ptr.Pointer(), core.PointerFromQPoint(position))))
	}
	return 0
}

func (ptr *QImage) PixelIndex2(x int, y int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImage_PixelIndex2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)))))
	}
	return 0
}

func (ptr *QImage) Rect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QImage_Rect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) ReinterpretAsFormat(format QImage__Format) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImage_ReinterpretAsFormat(ptr.Pointer(), C.longlong(format))) != 0
	}
	return false
}

func (ptr *QImage) RgbSwapped() *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_RgbSwapped(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) RgbSwapped2() *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_RgbSwapped2(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) Save(fileName string, format string, quality int) bool {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QImage_Save(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, formatC, C.int(int32(quality)))) != 0
	}
	return false
}

func (ptr *QImage) Save2(device core.QIODevice_ITF, format string, quality int) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QImage_Save2(ptr.Pointer(), core.PointerFromQIODevice(device), formatC, C.int(int32(quality)))) != 0
	}
	return false
}

func (ptr *QImage) Scaled(size core.QSize_ITF, aspectRatioMode core.Qt__AspectRatioMode, transformMode core.Qt__TransformationMode) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_Scaled(ptr.Pointer(), core.PointerFromQSize(size), C.longlong(aspectRatioMode), C.longlong(transformMode)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) Scaled2(width int, height int, aspectRatioMode core.Qt__AspectRatioMode, transformMode core.Qt__TransformationMode) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_Scaled2(ptr.Pointer(), C.int(int32(width)), C.int(int32(height)), C.longlong(aspectRatioMode), C.longlong(transformMode)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) ScaledToHeight(height int, mode core.Qt__TransformationMode) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_ScaledToHeight(ptr.Pointer(), C.int(int32(height)), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) ScaledToWidth(width int, mode core.Qt__TransformationMode) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_ScaledToWidth(ptr.Pointer(), C.int(int32(width)), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) ScanLine(i int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QImage_ScanLine(ptr.Pointer(), C.int(int32(i))))
	}
	return ""
}

func (ptr *QImage) ScanLine2(i int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QImage_ScanLine2(ptr.Pointer(), C.int(int32(i))))
	}
	return ""
}

func (ptr *QImage) SetColor(index int, colorValue uint) {
	if ptr.Pointer() != nil {
		C.QImage_SetColor(ptr.Pointer(), C.int(int32(index)), C.uint(uint32(colorValue)))
	}
}

func (ptr *QImage) SetColorCount(colorCount int) {
	if ptr.Pointer() != nil {
		C.QImage_SetColorCount(ptr.Pointer(), C.int(int32(colorCount)))
	}
}

func (ptr *QImage) SetColorTable(colors []uint) {
	if ptr.Pointer() != nil {
		C.QImage_SetColorTable(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQImageFromPointer(NewQImageFromPointer(nil).__setColorTable_colors_newList())
			for _, v := range colors {
				tmpList.__setColorTable_colors_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QImage) SetDevicePixelRatio(scaleFactor float64) {
	if ptr.Pointer() != nil {
		C.QImage_SetDevicePixelRatio(ptr.Pointer(), C.double(scaleFactor))
	}
}

func (ptr *QImage) SetDotsPerMeterX(x int) {
	if ptr.Pointer() != nil {
		C.QImage_SetDotsPerMeterX(ptr.Pointer(), C.int(int32(x)))
	}
}

func (ptr *QImage) SetDotsPerMeterY(y int) {
	if ptr.Pointer() != nil {
		C.QImage_SetDotsPerMeterY(ptr.Pointer(), C.int(int32(y)))
	}
}

func (ptr *QImage) SetOffset(offset core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QImage_SetOffset(ptr.Pointer(), core.PointerFromQPoint(offset))
	}
}

func (ptr *QImage) SetPixel(position core.QPoint_ITF, index_or_rgb uint) {
	if ptr.Pointer() != nil {
		C.QImage_SetPixel(ptr.Pointer(), core.PointerFromQPoint(position), C.uint(uint32(index_or_rgb)))
	}
}

func (ptr *QImage) SetPixel2(x int, y int, index_or_rgb uint) {
	if ptr.Pointer() != nil {
		C.QImage_SetPixel2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.uint(uint32(index_or_rgb)))
	}
}

func (ptr *QImage) SetPixelColor(position core.QPoint_ITF, color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QImage_SetPixelColor(ptr.Pointer(), core.PointerFromQPoint(position), PointerFromQColor(color))
	}
}

func (ptr *QImage) SetPixelColor2(x int, y int, color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QImage_SetPixelColor2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), PointerFromQColor(color))
	}
}

func (ptr *QImage) SetText(key string, text string) {
	if ptr.Pointer() != nil {
		var keyC *C.char
		if key != "" {
			keyC = C.CString(key)
			defer C.free(unsafe.Pointer(keyC))
		}
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QImage_SetText(ptr.Pointer(), C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QImage) Size() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QImage_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) SmoothScaled(w int, h int) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_SmoothScaled(ptr.Pointer(), C.int(int32(w)), C.int(int32(h))))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) Swap(other QImage_ITF) {
	if ptr.Pointer() != nil {
		C.QImage_Swap(ptr.Pointer(), PointerFromQImage(other))
	}
}

func (ptr *QImage) Text(key string) string {
	if ptr.Pointer() != nil {
		var keyC *C.char
		if key != "" {
			keyC = C.CString(key)
			defer C.free(unsafe.Pointer(keyC))
		}
		return cGoUnpackString(C.QImage_Text(ptr.Pointer(), C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}))
	}
	return ""
}

func (ptr *QImage) TextKeys() []string {
	if ptr.Pointer() != nil {
		return unpackStringList(cGoUnpackString(C.QImage_TextKeys(ptr.Pointer())))
	}
	return make([]string, 0)
}

func QImage_ToImageFormat(format QPixelFormat_ITF) QImage__Format {
	return QImage__Format(C.QImage_QImage_ToImageFormat(PointerFromQPixelFormat(format)))
}

func (ptr *QImage) ToImageFormat(format QPixelFormat_ITF) QImage__Format {
	return QImage__Format(C.QImage_QImage_ToImageFormat(PointerFromQPixelFormat(format)))
}

func QImage_ToPixelFormat(format QImage__Format) *QPixelFormat {
	tmpValue := NewQPixelFormatFromPointer(C.QImage_QImage_ToPixelFormat(C.longlong(format)))
	qt.SetFinalizer(tmpValue, (*QPixelFormat).DestroyQPixelFormat)
	return tmpValue
}

func (ptr *QImage) ToPixelFormat(format QImage__Format) *QPixelFormat {
	tmpValue := NewQPixelFormatFromPointer(C.QImage_QImage_ToPixelFormat(C.longlong(format)))
	qt.SetFinalizer(tmpValue, (*QPixelFormat).DestroyQPixelFormat)
	return tmpValue
}

func (ptr *QImage) Transformed(matrix QMatrix_ITF, mode core.Qt__TransformationMode) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_Transformed(ptr.Pointer(), PointerFromQMatrix(matrix), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) Transformed2(matrix QTransform_ITF, mode core.Qt__TransformationMode) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImage_Transformed2(ptr.Pointer(), PointerFromQTransform(matrix), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func QImage_TrueMatrix(matrix QMatrix_ITF, width int, height int) *QMatrix {
	tmpValue := NewQMatrixFromPointer(C.QImage_QImage_TrueMatrix(PointerFromQMatrix(matrix), C.int(int32(width)), C.int(int32(height))))
	qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
	return tmpValue
}

func (ptr *QImage) TrueMatrix(matrix QMatrix_ITF, width int, height int) *QMatrix {
	tmpValue := NewQMatrixFromPointer(C.QImage_QImage_TrueMatrix(PointerFromQMatrix(matrix), C.int(int32(width)), C.int(int32(height))))
	qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
	return tmpValue
}

func QImage_TrueMatrix2(matrix QTransform_ITF, width int, height int) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QImage_QImage_TrueMatrix2(PointerFromQTransform(matrix), C.int(int32(width)), C.int(int32(height))))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func (ptr *QImage) TrueMatrix2(matrix QTransform_ITF, width int, height int) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QImage_QImage_TrueMatrix2(PointerFromQTransform(matrix), C.int(int32(width)), C.int(int32(height))))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func (ptr *QImage) Valid(pos core.QPoint_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImage_Valid(ptr.Pointer(), core.PointerFromQPoint(pos))) != 0
	}
	return false
}

func (ptr *QImage) Valid2(x int, y int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImage_Valid2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)))) != 0
	}
	return false
}

//export callbackQImage_DestroyQImage
func callbackQImage_DestroyQImage(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QImage"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQImageFromPointer(ptr).DestroyQImageDefault()
	}
}

func (ptr *QImage) ConnectDestroyQImage(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QImage"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QImage", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QImage", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImage) DisconnectDestroyQImage() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QImage")
	}
}

func (ptr *QImage) DestroyQImage() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QImage_DestroyQImage(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QImage) DestroyQImageDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QImage_DestroyQImageDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QImage) ToVariant() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QImage_ToVariant(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QImage) __colorTable_atList(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QImage___colorTable_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QImage) __colorTable_setList(i uint) {
	if ptr.Pointer() != nil {
		C.QImage___colorTable_setList(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QImage) __colorTable_newList() unsafe.Pointer {
	return C.QImage___colorTable_newList(ptr.Pointer())
}

func (ptr *QImage) __convertToFormat_colorTable_atList3(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QImage___convertToFormat_colorTable_atList3(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QImage) __convertToFormat_colorTable_setList3(i uint) {
	if ptr.Pointer() != nil {
		C.QImage___convertToFormat_colorTable_setList3(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QImage) __convertToFormat_colorTable_newList3() unsafe.Pointer {
	return C.QImage___convertToFormat_colorTable_newList3(ptr.Pointer())
}

func (ptr *QImage) __setColorTable_colors_atList(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QImage___setColorTable_colors_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QImage) __setColorTable_colors_setList(i uint) {
	if ptr.Pointer() != nil {
		C.QImage___setColorTable_colors_setList(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QImage) __setColorTable_colors_newList() unsafe.Pointer {
	return C.QImage___setColorTable_colors_newList(ptr.Pointer())
}

//export callbackQImage_PaintEngine
func callbackQImage_PaintEngine(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "paintEngine"); signal != nil {
		return PointerFromQPaintEngine((*(*func() *QPaintEngine)(signal))())
	}

	return PointerFromQPaintEngine(NewQImageFromPointer(ptr).PaintEngineDefault())
}

func (ptr *QImage) PaintEngine() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QImage_PaintEngine(ptr.Pointer()))
	}
	return nil
}

func (ptr *QImage) PaintEngineDefault() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QImage_PaintEngineDefault(ptr.Pointer()))
	}
	return nil
}

type QImageIOHandler struct {
	ptr unsafe.Pointer
}

type QImageIOHandler_ITF interface {
	QImageIOHandler_PTR() *QImageIOHandler
}

func (ptr *QImageIOHandler) QImageIOHandler_PTR() *QImageIOHandler {
	return ptr
}

func (ptr *QImageIOHandler) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QImageIOHandler) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQImageIOHandler(ptr QImageIOHandler_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QImageIOHandler_PTR().Pointer()
	}
	return nil
}

func NewQImageIOHandlerFromPointer(ptr unsafe.Pointer) (n *QImageIOHandler) {
	n = new(QImageIOHandler)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QImageIOHandler__ImageOption
//QImageIOHandler::ImageOption
type QImageIOHandler__ImageOption int64

const (
	QImageIOHandler__Size                 QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(0)
	QImageIOHandler__ClipRect             QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(1)
	QImageIOHandler__Description          QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(2)
	QImageIOHandler__ScaledClipRect       QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(3)
	QImageIOHandler__ScaledSize           QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(4)
	QImageIOHandler__CompressionRatio     QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(5)
	QImageIOHandler__Gamma                QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(6)
	QImageIOHandler__Quality              QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(7)
	QImageIOHandler__Name                 QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(8)
	QImageIOHandler__SubType              QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(9)
	QImageIOHandler__IncrementalReading   QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(10)
	QImageIOHandler__Endianness           QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(11)
	QImageIOHandler__Animation            QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(12)
	QImageIOHandler__BackgroundColor      QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(13)
	QImageIOHandler__ImageFormat          QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(14)
	QImageIOHandler__SupportedSubTypes    QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(15)
	QImageIOHandler__OptimizedWrite       QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(16)
	QImageIOHandler__ProgressiveScanWrite QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(17)
	QImageIOHandler__ImageTransformation  QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(18)
	QImageIOHandler__TransformedByDefault QImageIOHandler__ImageOption = QImageIOHandler__ImageOption(19)
)

//go:generate stringer -type=QImageIOHandler__Transformation
//QImageIOHandler::Transformation
type QImageIOHandler__Transformation int64

const (
	QImageIOHandler__TransformationNone              QImageIOHandler__Transformation = QImageIOHandler__Transformation(0)
	QImageIOHandler__TransformationMirror            QImageIOHandler__Transformation = QImageIOHandler__Transformation(1)
	QImageIOHandler__TransformationFlip              QImageIOHandler__Transformation = QImageIOHandler__Transformation(2)
	QImageIOHandler__TransformationRotate180         QImageIOHandler__Transformation = QImageIOHandler__Transformation(QImageIOHandler__TransformationMirror | QImageIOHandler__TransformationFlip)
	QImageIOHandler__TransformationRotate90          QImageIOHandler__Transformation = QImageIOHandler__Transformation(4)
	QImageIOHandler__TransformationMirrorAndRotate90 QImageIOHandler__Transformation = QImageIOHandler__Transformation(QImageIOHandler__TransformationMirror | QImageIOHandler__TransformationRotate90)
	QImageIOHandler__TransformationFlipAndRotate90   QImageIOHandler__Transformation = QImageIOHandler__Transformation(QImageIOHandler__TransformationFlip | QImageIOHandler__TransformationRotate90)
	QImageIOHandler__TransformationRotate270         QImageIOHandler__Transformation = QImageIOHandler__Transformation(QImageIOHandler__TransformationRotate180 | QImageIOHandler__TransformationRotate90)
)

func NewQImageIOHandler() *QImageIOHandler {
	return NewQImageIOHandlerFromPointer(C.QImageIOHandler_NewQImageIOHandler())
}

//export callbackQImageIOHandler_CanRead
func callbackQImageIOHandler_CanRead(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "canRead"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QImageIOHandler) ConnectCanRead(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "canRead"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "canRead", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "canRead", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectCanRead() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "canRead")
	}
}

func (ptr *QImageIOHandler) CanRead() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOHandler_CanRead(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQImageIOHandler_CurrentImageNumber
func callbackQImageIOHandler_CurrentImageNumber(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "currentImageNumber"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(NewQImageIOHandlerFromPointer(ptr).CurrentImageNumberDefault()))
}

func (ptr *QImageIOHandler) ConnectCurrentImageNumber(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "currentImageNumber"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "currentImageNumber", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "currentImageNumber", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectCurrentImageNumber() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "currentImageNumber")
	}
}

func (ptr *QImageIOHandler) CurrentImageNumber() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageIOHandler_CurrentImageNumber(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageIOHandler) CurrentImageNumberDefault() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageIOHandler_CurrentImageNumberDefault(ptr.Pointer())))
	}
	return 0
}

//export callbackQImageIOHandler_CurrentImageRect
func callbackQImageIOHandler_CurrentImageRect(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "currentImageRect"); signal != nil {
		return core.PointerFromQRect((*(*func() *core.QRect)(signal))())
	}

	return core.PointerFromQRect(NewQImageIOHandlerFromPointer(ptr).CurrentImageRectDefault())
}

func (ptr *QImageIOHandler) ConnectCurrentImageRect(f func() *core.QRect) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "currentImageRect"); signal != nil {
			f := func() *core.QRect {
				(*(*func() *core.QRect)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "currentImageRect", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "currentImageRect", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectCurrentImageRect() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "currentImageRect")
	}
}

func (ptr *QImageIOHandler) CurrentImageRect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QImageIOHandler_CurrentImageRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QImageIOHandler) CurrentImageRectDefault() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QImageIOHandler_CurrentImageRectDefault(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QImageIOHandler) Device() *core.QIODevice {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQIODeviceFromPointer(C.QImageIOHandler_Device(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QImageIOHandler) Format() *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageIOHandler_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

//export callbackQImageIOHandler_ImageCount
func callbackQImageIOHandler_ImageCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "imageCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(NewQImageIOHandlerFromPointer(ptr).ImageCountDefault()))
}

func (ptr *QImageIOHandler) ConnectImageCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "imageCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "imageCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "imageCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectImageCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "imageCount")
	}
}

func (ptr *QImageIOHandler) ImageCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageIOHandler_ImageCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageIOHandler) ImageCountDefault() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageIOHandler_ImageCountDefault(ptr.Pointer())))
	}
	return 0
}

//export callbackQImageIOHandler_JumpToImage
func callbackQImageIOHandler_JumpToImage(ptr unsafe.Pointer, imageNumber C.int) C.char {
	if signal := qt.GetSignal(ptr, "jumpToImage"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int) bool)(signal))(int(int32(imageNumber))))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQImageIOHandlerFromPointer(ptr).JumpToImageDefault(int(int32(imageNumber))))))
}

func (ptr *QImageIOHandler) ConnectJumpToImage(f func(imageNumber int) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "jumpToImage"); signal != nil {
			f := func(imageNumber int) bool {
				(*(*func(int) bool)(signal))(imageNumber)
				return f(imageNumber)
			}
			qt.ConnectSignal(ptr.Pointer(), "jumpToImage", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "jumpToImage", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectJumpToImage() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "jumpToImage")
	}
}

func (ptr *QImageIOHandler) JumpToImage(imageNumber int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOHandler_JumpToImage(ptr.Pointer(), C.int(int32(imageNumber)))) != 0
	}
	return false
}

func (ptr *QImageIOHandler) JumpToImageDefault(imageNumber int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOHandler_JumpToImageDefault(ptr.Pointer(), C.int(int32(imageNumber)))) != 0
	}
	return false
}

//export callbackQImageIOHandler_JumpToNextImage
func callbackQImageIOHandler_JumpToNextImage(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "jumpToNextImage"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(NewQImageIOHandlerFromPointer(ptr).JumpToNextImageDefault())))
}

func (ptr *QImageIOHandler) ConnectJumpToNextImage(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "jumpToNextImage"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "jumpToNextImage", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "jumpToNextImage", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectJumpToNextImage() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "jumpToNextImage")
	}
}

func (ptr *QImageIOHandler) JumpToNextImage() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOHandler_JumpToNextImage(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImageIOHandler) JumpToNextImageDefault() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOHandler_JumpToNextImageDefault(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQImageIOHandler_LoopCount
func callbackQImageIOHandler_LoopCount(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "loopCount"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(NewQImageIOHandlerFromPointer(ptr).LoopCountDefault()))
}

func (ptr *QImageIOHandler) ConnectLoopCount(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "loopCount"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "loopCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "loopCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectLoopCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "loopCount")
	}
}

func (ptr *QImageIOHandler) LoopCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageIOHandler_LoopCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageIOHandler) LoopCountDefault() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageIOHandler_LoopCountDefault(ptr.Pointer())))
	}
	return 0
}

//export callbackQImageIOHandler_NextImageDelay
func callbackQImageIOHandler_NextImageDelay(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "nextImageDelay"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(NewQImageIOHandlerFromPointer(ptr).NextImageDelayDefault()))
}

func (ptr *QImageIOHandler) ConnectNextImageDelay(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "nextImageDelay"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "nextImageDelay", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "nextImageDelay", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectNextImageDelay() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "nextImageDelay")
	}
}

func (ptr *QImageIOHandler) NextImageDelay() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageIOHandler_NextImageDelay(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageIOHandler) NextImageDelayDefault() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageIOHandler_NextImageDelayDefault(ptr.Pointer())))
	}
	return 0
}

//export callbackQImageIOHandler_Option
func callbackQImageIOHandler_Option(ptr unsafe.Pointer, option C.longlong) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "option"); signal != nil {
		return core.PointerFromQVariant((*(*func(QImageIOHandler__ImageOption) *core.QVariant)(signal))(QImageIOHandler__ImageOption(option)))
	}

	return core.PointerFromQVariant(NewQImageIOHandlerFromPointer(ptr).OptionDefault(QImageIOHandler__ImageOption(option)))
}

func (ptr *QImageIOHandler) ConnectOption(f func(option QImageIOHandler__ImageOption) *core.QVariant) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "option"); signal != nil {
			f := func(option QImageIOHandler__ImageOption) *core.QVariant {
				(*(*func(QImageIOHandler__ImageOption) *core.QVariant)(signal))(option)
				return f(option)
			}
			qt.ConnectSignal(ptr.Pointer(), "option", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "option", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectOption() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "option")
	}
}

func (ptr *QImageIOHandler) Option(option QImageIOHandler__ImageOption) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QImageIOHandler_Option(ptr.Pointer(), C.longlong(option)))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QImageIOHandler) OptionDefault(option QImageIOHandler__ImageOption) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QImageIOHandler_OptionDefault(ptr.Pointer(), C.longlong(option)))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

//export callbackQImageIOHandler_Read
func callbackQImageIOHandler_Read(ptr unsafe.Pointer, image unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "read"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*QImage) bool)(signal))(NewQImageFromPointer(image)))))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QImageIOHandler) ConnectRead(f func(image *QImage) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "read"); signal != nil {
			f := func(image *QImage) bool {
				(*(*func(*QImage) bool)(signal))(image)
				return f(image)
			}
			qt.ConnectSignal(ptr.Pointer(), "read", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "read", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectRead() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "read")
	}
}

func (ptr *QImageIOHandler) Read(image QImage_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOHandler_Read(ptr.Pointer(), PointerFromQImage(image))) != 0
	}
	return false
}

func (ptr *QImageIOHandler) SetDevice(device core.QIODevice_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOHandler_SetDevice(ptr.Pointer(), core.PointerFromQIODevice(device))
	}
}

func (ptr *QImageIOHandler) SetFormat(format core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOHandler_SetFormat(ptr.Pointer(), core.PointerFromQByteArray(format))
	}
}

func (ptr *QImageIOHandler) SetFormat2(format core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOHandler_SetFormat2(ptr.Pointer(), core.PointerFromQByteArray(format))
	}
}

//export callbackQImageIOHandler_SetOption
func callbackQImageIOHandler_SetOption(ptr unsafe.Pointer, option C.longlong, value unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "setOption"); signal != nil {
		(*(*func(QImageIOHandler__ImageOption, *core.QVariant))(signal))(QImageIOHandler__ImageOption(option), core.NewQVariantFromPointer(value))
	} else {
		NewQImageIOHandlerFromPointer(ptr).SetOptionDefault(QImageIOHandler__ImageOption(option), core.NewQVariantFromPointer(value))
	}
}

func (ptr *QImageIOHandler) ConnectSetOption(f func(option QImageIOHandler__ImageOption, value *core.QVariant)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setOption"); signal != nil {
			f := func(option QImageIOHandler__ImageOption, value *core.QVariant) {
				(*(*func(QImageIOHandler__ImageOption, *core.QVariant))(signal))(option, value)
				f(option, value)
			}
			qt.ConnectSignal(ptr.Pointer(), "setOption", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setOption", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectSetOption() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setOption")
	}
}

func (ptr *QImageIOHandler) SetOption(option QImageIOHandler__ImageOption, value core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOHandler_SetOption(ptr.Pointer(), C.longlong(option), core.PointerFromQVariant(value))
	}
}

func (ptr *QImageIOHandler) SetOptionDefault(option QImageIOHandler__ImageOption, value core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOHandler_SetOptionDefault(ptr.Pointer(), C.longlong(option), core.PointerFromQVariant(value))
	}
}

//export callbackQImageIOHandler_SupportsOption
func callbackQImageIOHandler_SupportsOption(ptr unsafe.Pointer, option C.longlong) C.char {
	if signal := qt.GetSignal(ptr, "supportsOption"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(QImageIOHandler__ImageOption) bool)(signal))(QImageIOHandler__ImageOption(option)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQImageIOHandlerFromPointer(ptr).SupportsOptionDefault(QImageIOHandler__ImageOption(option)))))
}

func (ptr *QImageIOHandler) ConnectSupportsOption(f func(option QImageIOHandler__ImageOption) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "supportsOption"); signal != nil {
			f := func(option QImageIOHandler__ImageOption) bool {
				(*(*func(QImageIOHandler__ImageOption) bool)(signal))(option)
				return f(option)
			}
			qt.ConnectSignal(ptr.Pointer(), "supportsOption", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "supportsOption", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectSupportsOption() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "supportsOption")
	}
}

func (ptr *QImageIOHandler) SupportsOption(option QImageIOHandler__ImageOption) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOHandler_SupportsOption(ptr.Pointer(), C.longlong(option))) != 0
	}
	return false
}

func (ptr *QImageIOHandler) SupportsOptionDefault(option QImageIOHandler__ImageOption) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOHandler_SupportsOptionDefault(ptr.Pointer(), C.longlong(option))) != 0
	}
	return false
}

//export callbackQImageIOHandler_Write
func callbackQImageIOHandler_Write(ptr unsafe.Pointer, image unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "write"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*QImage) bool)(signal))(NewQImageFromPointer(image)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQImageIOHandlerFromPointer(ptr).WriteDefault(NewQImageFromPointer(image)))))
}

func (ptr *QImageIOHandler) ConnectWrite(f func(image *QImage) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "write"); signal != nil {
			f := func(image *QImage) bool {
				(*(*func(*QImage) bool)(signal))(image)
				return f(image)
			}
			qt.ConnectSignal(ptr.Pointer(), "write", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "write", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectWrite() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "write")
	}
}

func (ptr *QImageIOHandler) Write(image QImage_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOHandler_Write(ptr.Pointer(), PointerFromQImage(image))) != 0
	}
	return false
}

func (ptr *QImageIOHandler) WriteDefault(image QImage_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOHandler_WriteDefault(ptr.Pointer(), PointerFromQImage(image))) != 0
	}
	return false
}

//export callbackQImageIOHandler_DestroyQImageIOHandler
func callbackQImageIOHandler_DestroyQImageIOHandler(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QImageIOHandler"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQImageIOHandlerFromPointer(ptr).DestroyQImageIOHandlerDefault()
	}
}

func (ptr *QImageIOHandler) ConnectDestroyQImageIOHandler(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QImageIOHandler"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QImageIOHandler", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QImageIOHandler", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOHandler) DisconnectDestroyQImageIOHandler() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QImageIOHandler")
	}
}

func (ptr *QImageIOHandler) DestroyQImageIOHandler() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QImageIOHandler_DestroyQImageIOHandler(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QImageIOHandler) DestroyQImageIOHandlerDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QImageIOHandler_DestroyQImageIOHandlerDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QImageIOPlugin struct {
	core.QObject
}

type QImageIOPlugin_ITF interface {
	core.QObject_ITF
	QImageIOPlugin_PTR() *QImageIOPlugin
}

func (ptr *QImageIOPlugin) QImageIOPlugin_PTR() *QImageIOPlugin {
	return ptr
}

func (ptr *QImageIOPlugin) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QImageIOPlugin) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQImageIOPlugin(ptr QImageIOPlugin_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QImageIOPlugin_PTR().Pointer()
	}
	return nil
}

func NewQImageIOPluginFromPointer(ptr unsafe.Pointer) (n *QImageIOPlugin) {
	n = new(QImageIOPlugin)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QImageIOPlugin__Capability
//QImageIOPlugin::Capability
type QImageIOPlugin__Capability int64

const (
	QImageIOPlugin__CanRead            QImageIOPlugin__Capability = QImageIOPlugin__Capability(0x1)
	QImageIOPlugin__CanWrite           QImageIOPlugin__Capability = QImageIOPlugin__Capability(0x2)
	QImageIOPlugin__CanReadIncremental QImageIOPlugin__Capability = QImageIOPlugin__Capability(0x4)
)

func NewQImageIOPlugin(parent core.QObject_ITF) *QImageIOPlugin {
	tmpValue := NewQImageIOPluginFromPointer(C.QImageIOPlugin_NewQImageIOPlugin(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQImageIOPlugin_Capabilities
func callbackQImageIOPlugin_Capabilities(ptr unsafe.Pointer, device unsafe.Pointer, format unsafe.Pointer) C.longlong {
	if signal := qt.GetSignal(ptr, "capabilities"); signal != nil {
		return C.longlong((*(*func(*core.QIODevice, *core.QByteArray) QImageIOPlugin__Capability)(signal))(core.NewQIODeviceFromPointer(device), core.NewQByteArrayFromPointer(format)))
	}

	return C.longlong(0)
}

func (ptr *QImageIOPlugin) ConnectCapabilities(f func(device *core.QIODevice, format *core.QByteArray) QImageIOPlugin__Capability) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "capabilities"); signal != nil {
			f := func(device *core.QIODevice, format *core.QByteArray) QImageIOPlugin__Capability {
				(*(*func(*core.QIODevice, *core.QByteArray) QImageIOPlugin__Capability)(signal))(device, format)
				return f(device, format)
			}
			qt.ConnectSignal(ptr.Pointer(), "capabilities", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "capabilities", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOPlugin) DisconnectCapabilities() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "capabilities")
	}
}

func (ptr *QImageIOPlugin) Capabilities(device core.QIODevice_ITF, format core.QByteArray_ITF) QImageIOPlugin__Capability {
	if ptr.Pointer() != nil {
		return QImageIOPlugin__Capability(C.QImageIOPlugin_Capabilities(ptr.Pointer(), core.PointerFromQIODevice(device), core.PointerFromQByteArray(format)))
	}
	return 0
}

//export callbackQImageIOPlugin_Create
func callbackQImageIOPlugin_Create(ptr unsafe.Pointer, device unsafe.Pointer, format unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "create"); signal != nil {
		return PointerFromQImageIOHandler((*(*func(*core.QIODevice, *core.QByteArray) *QImageIOHandler)(signal))(core.NewQIODeviceFromPointer(device), core.NewQByteArrayFromPointer(format)))
	}

	return PointerFromQImageIOHandler(NewQImageIOHandler())
}

func (ptr *QImageIOPlugin) ConnectCreate(f func(device *core.QIODevice, format *core.QByteArray) *QImageIOHandler) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "create"); signal != nil {
			f := func(device *core.QIODevice, format *core.QByteArray) *QImageIOHandler {
				(*(*func(*core.QIODevice, *core.QByteArray) *QImageIOHandler)(signal))(device, format)
				return f(device, format)
			}
			qt.ConnectSignal(ptr.Pointer(), "create", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "create", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOPlugin) DisconnectCreate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "create")
	}
}

func (ptr *QImageIOPlugin) Create(device core.QIODevice_ITF, format core.QByteArray_ITF) *QImageIOHandler {
	if ptr.Pointer() != nil {
		return NewQImageIOHandlerFromPointer(C.QImageIOPlugin_Create(ptr.Pointer(), core.PointerFromQIODevice(device), core.PointerFromQByteArray(format)))
	}
	return nil
}

//export callbackQImageIOPlugin_DestroyQImageIOPlugin
func callbackQImageIOPlugin_DestroyQImageIOPlugin(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QImageIOPlugin"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQImageIOPluginFromPointer(ptr).DestroyQImageIOPluginDefault()
	}
}

func (ptr *QImageIOPlugin) ConnectDestroyQImageIOPlugin(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QImageIOPlugin"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QImageIOPlugin", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QImageIOPlugin", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QImageIOPlugin) DisconnectDestroyQImageIOPlugin() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QImageIOPlugin")
	}
}

func (ptr *QImageIOPlugin) DestroyQImageIOPlugin() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QImageIOPlugin_DestroyQImageIOPlugin(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QImageIOPlugin) DestroyQImageIOPluginDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QImageIOPlugin_DestroyQImageIOPluginDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QImageIOPlugin) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QImageIOPlugin___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QImageIOPlugin) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOPlugin___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QImageIOPlugin) __children_newList() unsafe.Pointer {
	return C.QImageIOPlugin___children_newList(ptr.Pointer())
}

func (ptr *QImageIOPlugin) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageIOPlugin___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageIOPlugin) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOPlugin___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QImageIOPlugin) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QImageIOPlugin___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QImageIOPlugin) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QImageIOPlugin___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QImageIOPlugin) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOPlugin___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QImageIOPlugin) __findChildren_newList() unsafe.Pointer {
	return C.QImageIOPlugin___findChildren_newList(ptr.Pointer())
}

func (ptr *QImageIOPlugin) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QImageIOPlugin___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QImageIOPlugin) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOPlugin___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QImageIOPlugin) __findChildren_newList3() unsafe.Pointer {
	return C.QImageIOPlugin___findChildren_newList3(ptr.Pointer())
}

//export callbackQImageIOPlugin_ChildEvent
func callbackQImageIOPlugin_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQImageIOPluginFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QImageIOPlugin) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOPlugin_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQImageIOPlugin_ConnectNotify
func callbackQImageIOPlugin_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQImageIOPluginFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QImageIOPlugin) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOPlugin_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQImageIOPlugin_CustomEvent
func callbackQImageIOPlugin_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQImageIOPluginFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QImageIOPlugin) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOPlugin_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQImageIOPlugin_DeleteLater
func callbackQImageIOPlugin_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQImageIOPluginFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QImageIOPlugin) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QImageIOPlugin_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQImageIOPlugin_Destroyed
func callbackQImageIOPlugin_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQImageIOPlugin_DisconnectNotify
func callbackQImageIOPlugin_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQImageIOPluginFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QImageIOPlugin) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOPlugin_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQImageIOPlugin_Event
func callbackQImageIOPlugin_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQImageIOPluginFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QImageIOPlugin) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOPlugin_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQImageIOPlugin_EventFilter
func callbackQImageIOPlugin_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQImageIOPluginFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QImageIOPlugin) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageIOPlugin_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQImageIOPlugin_MetaObject
func callbackQImageIOPlugin_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQImageIOPluginFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QImageIOPlugin) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QImageIOPlugin_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQImageIOPlugin_ObjectNameChanged
func callbackQImageIOPlugin_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQImageIOPlugin_TimerEvent
func callbackQImageIOPlugin_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQImageIOPluginFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QImageIOPlugin) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QImageIOPlugin_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QImageReader struct {
	ptr unsafe.Pointer
}

type QImageReader_ITF interface {
	QImageReader_PTR() *QImageReader
}

func (ptr *QImageReader) QImageReader_PTR() *QImageReader {
	return ptr
}

func (ptr *QImageReader) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QImageReader) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQImageReader(ptr QImageReader_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QImageReader_PTR().Pointer()
	}
	return nil
}

func NewQImageReaderFromPointer(ptr unsafe.Pointer) (n *QImageReader) {
	n = new(QImageReader)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QImageReader__ImageReaderError
//QImageReader::ImageReaderError
type QImageReader__ImageReaderError int64

const (
	QImageReader__UnknownError           QImageReader__ImageReaderError = QImageReader__ImageReaderError(0)
	QImageReader__FileNotFoundError      QImageReader__ImageReaderError = QImageReader__ImageReaderError(1)
	QImageReader__DeviceError            QImageReader__ImageReaderError = QImageReader__ImageReaderError(2)
	QImageReader__UnsupportedFormatError QImageReader__ImageReaderError = QImageReader__ImageReaderError(3)
	QImageReader__InvalidDataError       QImageReader__ImageReaderError = QImageReader__ImageReaderError(4)
)

func NewQImageReader() *QImageReader {
	tmpValue := NewQImageReaderFromPointer(C.QImageReader_NewQImageReader())
	qt.SetFinalizer(tmpValue, (*QImageReader).DestroyQImageReader)
	return tmpValue
}

func NewQImageReader2(device core.QIODevice_ITF, format core.QByteArray_ITF) *QImageReader {
	tmpValue := NewQImageReaderFromPointer(C.QImageReader_NewQImageReader2(core.PointerFromQIODevice(device), core.PointerFromQByteArray(format)))
	qt.SetFinalizer(tmpValue, (*QImageReader).DestroyQImageReader)
	return tmpValue
}

func NewQImageReader3(fileName string, format core.QByteArray_ITF) *QImageReader {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	tmpValue := NewQImageReaderFromPointer(C.QImageReader_NewQImageReader3(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, core.PointerFromQByteArray(format)))
	qt.SetFinalizer(tmpValue, (*QImageReader).DestroyQImageReader)
	return tmpValue
}

func (ptr *QImageReader) AutoDetectImageFormat() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageReader_AutoDetectImageFormat(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImageReader) AutoTransform() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageReader_AutoTransform(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImageReader) BackgroundColor() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QImageReader_BackgroundColor(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) CanRead() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageReader_CanRead(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImageReader) ClipRect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QImageReader_ClipRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) CurrentImageNumber() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageReader_CurrentImageNumber(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageReader) CurrentImageRect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QImageReader_CurrentImageRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) DecideFormatFromContent() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageReader_DecideFormatFromContent(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImageReader) Device() *core.QIODevice {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQIODeviceFromPointer(C.QImageReader_Device(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) Error() QImageReader__ImageReaderError {
	if ptr.Pointer() != nil {
		return QImageReader__ImageReaderError(C.QImageReader_Error(ptr.Pointer()))
	}
	return 0
}

func (ptr *QImageReader) ErrorString() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QImageReader_ErrorString(ptr.Pointer()))
	}
	return ""
}

func (ptr *QImageReader) FileName() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QImageReader_FileName(ptr.Pointer()))
	}
	return ""
}

func (ptr *QImageReader) Format() *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageReader_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) Gamma() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QImageReader_Gamma(ptr.Pointer()))
	}
	return 0
}

func (ptr *QImageReader) ImageCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageReader_ImageCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageReader) ImageFormat() QImage__Format {
	if ptr.Pointer() != nil {
		return QImage__Format(C.QImageReader_ImageFormat(ptr.Pointer()))
	}
	return 0
}

func QImageReader_ImageFormat2(fileName string) *core.QByteArray {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	tmpValue := core.NewQByteArrayFromPointer(C.QImageReader_QImageReader_ImageFormat2(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}))
	qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
	return tmpValue
}

func (ptr *QImageReader) ImageFormat2(fileName string) *core.QByteArray {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	tmpValue := core.NewQByteArrayFromPointer(C.QImageReader_QImageReader_ImageFormat2(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}))
	qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
	return tmpValue
}

func QImageReader_ImageFormat3(device core.QIODevice_ITF) *core.QByteArray {
	tmpValue := core.NewQByteArrayFromPointer(C.QImageReader_QImageReader_ImageFormat3(core.PointerFromQIODevice(device)))
	qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
	return tmpValue
}

func (ptr *QImageReader) ImageFormat3(device core.QIODevice_ITF) *core.QByteArray {
	tmpValue := core.NewQByteArrayFromPointer(C.QImageReader_QImageReader_ImageFormat3(core.PointerFromQIODevice(device)))
	qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
	return tmpValue
}

func QImageReader_ImageFormatsForMimeType(mimeType core.QByteArray_ITF) []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageReaderFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__imageFormatsForMimeType_atList(i)
		}
		return out
	}(C.QImageReader_QImageReader_ImageFormatsForMimeType(core.PointerFromQByteArray(mimeType)))
}

func (ptr *QImageReader) ImageFormatsForMimeType(mimeType core.QByteArray_ITF) []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageReaderFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__imageFormatsForMimeType_atList(i)
		}
		return out
	}(C.QImageReader_QImageReader_ImageFormatsForMimeType(core.PointerFromQByteArray(mimeType)))
}

func (ptr *QImageReader) JumpToImage(imageNumber int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageReader_JumpToImage(ptr.Pointer(), C.int(int32(imageNumber)))) != 0
	}
	return false
}

func (ptr *QImageReader) JumpToNextImage() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageReader_JumpToNextImage(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImageReader) LoopCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageReader_LoopCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageReader) NextImageDelay() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageReader_NextImageDelay(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageReader) Quality() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageReader_Quality(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageReader) Read() *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QImageReader_Read(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) Read2(image QImage_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageReader_Read2(ptr.Pointer(), PointerFromQImage(image))) != 0
	}
	return false
}

func (ptr *QImageReader) ScaledClipRect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QImageReader_ScaledClipRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) ScaledSize() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QImageReader_ScaledSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) SetAutoDetectImageFormat(enabled bool) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetAutoDetectImageFormat(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enabled))))
	}
}

func (ptr *QImageReader) SetAutoTransform(enabled bool) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetAutoTransform(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enabled))))
	}
}

func (ptr *QImageReader) SetBackgroundColor(color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetBackgroundColor(ptr.Pointer(), PointerFromQColor(color))
	}
}

func (ptr *QImageReader) SetClipRect(rect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetClipRect(ptr.Pointer(), core.PointerFromQRect(rect))
	}
}

func (ptr *QImageReader) SetDecideFormatFromContent(ignored bool) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetDecideFormatFromContent(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(ignored))))
	}
}

func (ptr *QImageReader) SetDevice(device core.QIODevice_ITF) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetDevice(ptr.Pointer(), core.PointerFromQIODevice(device))
	}
}

func (ptr *QImageReader) SetFileName(fileName string) {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		C.QImageReader_SetFileName(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))})
	}
}

func (ptr *QImageReader) SetFormat(format core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetFormat(ptr.Pointer(), core.PointerFromQByteArray(format))
	}
}

func (ptr *QImageReader) SetGamma(gamma float32) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetGamma(ptr.Pointer(), C.float(gamma))
	}
}

func (ptr *QImageReader) SetQuality(quality int) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetQuality(ptr.Pointer(), C.int(int32(quality)))
	}
}

func (ptr *QImageReader) SetScaledClipRect(rect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetScaledClipRect(ptr.Pointer(), core.PointerFromQRect(rect))
	}
}

func (ptr *QImageReader) SetScaledSize(size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QImageReader_SetScaledSize(ptr.Pointer(), core.PointerFromQSize(size))
	}
}

func (ptr *QImageReader) Size() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QImageReader_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) SubType() *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageReader_SubType(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func QImageReader_SupportedImageFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageReaderFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedImageFormats_atList(i)
		}
		return out
	}(C.QImageReader_QImageReader_SupportedImageFormats())
}

func (ptr *QImageReader) SupportedImageFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageReaderFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedImageFormats_atList(i)
		}
		return out
	}(C.QImageReader_QImageReader_SupportedImageFormats())
}

func QImageReader_SupportedMimeTypes() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageReaderFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedMimeTypes_atList(i)
		}
		return out
	}(C.QImageReader_QImageReader_SupportedMimeTypes())
}

func (ptr *QImageReader) SupportedMimeTypes() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageReaderFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedMimeTypes_atList(i)
		}
		return out
	}(C.QImageReader_QImageReader_SupportedMimeTypes())
}

func (ptr *QImageReader) SupportedSubTypes() []*core.QByteArray {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
			out := make([]*core.QByteArray, int(l.len))
			tmpList := NewQImageReaderFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__supportedSubTypes_atList(i)
			}
			return out
		}(C.QImageReader_SupportedSubTypes(ptr.Pointer()))
	}
	return make([]*core.QByteArray, 0)
}

func (ptr *QImageReader) SupportsAnimation() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageReader_SupportsAnimation(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImageReader) SupportsOption(option QImageIOHandler__ImageOption) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageReader_SupportsOption(ptr.Pointer(), C.longlong(option))) != 0
	}
	return false
}

func (ptr *QImageReader) Text(key string) string {
	if ptr.Pointer() != nil {
		var keyC *C.char
		if key != "" {
			keyC = C.CString(key)
			defer C.free(unsafe.Pointer(keyC))
		}
		return cGoUnpackString(C.QImageReader_Text(ptr.Pointer(), C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}))
	}
	return ""
}

func (ptr *QImageReader) TextKeys() []string {
	if ptr.Pointer() != nil {
		return unpackStringList(cGoUnpackString(C.QImageReader_TextKeys(ptr.Pointer())))
	}
	return make([]string, 0)
}

func (ptr *QImageReader) Transformation() QImageIOHandler__Transformation {
	if ptr.Pointer() != nil {
		return QImageIOHandler__Transformation(C.QImageReader_Transformation(ptr.Pointer()))
	}
	return 0
}

func (ptr *QImageReader) DestroyQImageReader() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QImageReader_DestroyQImageReader(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QImageReader) __imageFormatsForMimeType_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageReader___imageFormatsForMimeType_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) __imageFormatsForMimeType_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageReader___imageFormatsForMimeType_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QImageReader) __imageFormatsForMimeType_newList() unsafe.Pointer {
	return C.QImageReader___imageFormatsForMimeType_newList(ptr.Pointer())
}

func (ptr *QImageReader) __supportedImageFormats_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageReader___supportedImageFormats_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) __supportedImageFormats_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageReader___supportedImageFormats_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QImageReader) __supportedImageFormats_newList() unsafe.Pointer {
	return C.QImageReader___supportedImageFormats_newList(ptr.Pointer())
}

func (ptr *QImageReader) __supportedMimeTypes_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageReader___supportedMimeTypes_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) __supportedMimeTypes_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageReader___supportedMimeTypes_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QImageReader) __supportedMimeTypes_newList() unsafe.Pointer {
	return C.QImageReader___supportedMimeTypes_newList(ptr.Pointer())
}

func (ptr *QImageReader) __supportedSubTypes_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageReader___supportedSubTypes_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageReader) __supportedSubTypes_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageReader___supportedSubTypes_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QImageReader) __supportedSubTypes_newList() unsafe.Pointer {
	return C.QImageReader___supportedSubTypes_newList(ptr.Pointer())
}

type QImageTextKeyLang struct {
	ptr unsafe.Pointer
}

type QImageTextKeyLang_ITF interface {
	QImageTextKeyLang_PTR() *QImageTextKeyLang
}

func (ptr *QImageTextKeyLang) QImageTextKeyLang_PTR() *QImageTextKeyLang {
	return ptr
}

func (ptr *QImageTextKeyLang) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QImageTextKeyLang) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQImageTextKeyLang(ptr QImageTextKeyLang_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QImageTextKeyLang_PTR().Pointer()
	}
	return nil
}

func NewQImageTextKeyLangFromPointer(ptr unsafe.Pointer) (n *QImageTextKeyLang) {
	n = new(QImageTextKeyLang)
	n.SetPointer(ptr)
	return
}
func (ptr *QImageTextKeyLang) DestroyQImageTextKeyLang() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QImageWriter struct {
	ptr unsafe.Pointer
}

type QImageWriter_ITF interface {
	QImageWriter_PTR() *QImageWriter
}

func (ptr *QImageWriter) QImageWriter_PTR() *QImageWriter {
	return ptr
}

func (ptr *QImageWriter) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QImageWriter) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQImageWriter(ptr QImageWriter_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QImageWriter_PTR().Pointer()
	}
	return nil
}

func NewQImageWriterFromPointer(ptr unsafe.Pointer) (n *QImageWriter) {
	n = new(QImageWriter)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QImageWriter__ImageWriterError
//QImageWriter::ImageWriterError
type QImageWriter__ImageWriterError int64

const (
	QImageWriter__UnknownError           QImageWriter__ImageWriterError = QImageWriter__ImageWriterError(0)
	QImageWriter__DeviceError            QImageWriter__ImageWriterError = QImageWriter__ImageWriterError(1)
	QImageWriter__UnsupportedFormatError QImageWriter__ImageWriterError = QImageWriter__ImageWriterError(2)
	QImageWriter__InvalidImageError      QImageWriter__ImageWriterError = QImageWriter__ImageWriterError(3)
)

func NewQImageWriter() *QImageWriter {
	tmpValue := NewQImageWriterFromPointer(C.QImageWriter_NewQImageWriter())
	qt.SetFinalizer(tmpValue, (*QImageWriter).DestroyQImageWriter)
	return tmpValue
}

func NewQImageWriter2(device core.QIODevice_ITF, format core.QByteArray_ITF) *QImageWriter {
	tmpValue := NewQImageWriterFromPointer(C.QImageWriter_NewQImageWriter2(core.PointerFromQIODevice(device), core.PointerFromQByteArray(format)))
	qt.SetFinalizer(tmpValue, (*QImageWriter).DestroyQImageWriter)
	return tmpValue
}

func NewQImageWriter3(fileName string, format core.QByteArray_ITF) *QImageWriter {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	tmpValue := NewQImageWriterFromPointer(C.QImageWriter_NewQImageWriter3(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, core.PointerFromQByteArray(format)))
	qt.SetFinalizer(tmpValue, (*QImageWriter).DestroyQImageWriter)
	return tmpValue
}

func (ptr *QImageWriter) CanWrite() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageWriter_CanWrite(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImageWriter) Compression() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageWriter_Compression(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageWriter) Device() *core.QIODevice {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQIODeviceFromPointer(C.QImageWriter_Device(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QImageWriter) Error() QImageWriter__ImageWriterError {
	if ptr.Pointer() != nil {
		return QImageWriter__ImageWriterError(C.QImageWriter_Error(ptr.Pointer()))
	}
	return 0
}

func (ptr *QImageWriter) ErrorString() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QImageWriter_ErrorString(ptr.Pointer()))
	}
	return ""
}

func (ptr *QImageWriter) FileName() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QImageWriter_FileName(ptr.Pointer()))
	}
	return ""
}

func (ptr *QImageWriter) Format() *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageWriter_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageWriter) Gamma() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QImageWriter_Gamma(ptr.Pointer()))
	}
	return 0
}

func QImageWriter_ImageFormatsForMimeType(mimeType core.QByteArray_ITF) []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageWriterFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__imageFormatsForMimeType_atList(i)
		}
		return out
	}(C.QImageWriter_QImageWriter_ImageFormatsForMimeType(core.PointerFromQByteArray(mimeType)))
}

func (ptr *QImageWriter) ImageFormatsForMimeType(mimeType core.QByteArray_ITF) []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageWriterFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__imageFormatsForMimeType_atList(i)
		}
		return out
	}(C.QImageWriter_QImageWriter_ImageFormatsForMimeType(core.PointerFromQByteArray(mimeType)))
}

func (ptr *QImageWriter) OptimizedWrite() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageWriter_OptimizedWrite(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImageWriter) ProgressiveScanWrite() bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageWriter_ProgressiveScanWrite(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QImageWriter) Quality() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QImageWriter_Quality(ptr.Pointer())))
	}
	return 0
}

func (ptr *QImageWriter) SetCompression(compression int) {
	if ptr.Pointer() != nil {
		C.QImageWriter_SetCompression(ptr.Pointer(), C.int(int32(compression)))
	}
}

func (ptr *QImageWriter) SetDevice(device core.QIODevice_ITF) {
	if ptr.Pointer() != nil {
		C.QImageWriter_SetDevice(ptr.Pointer(), core.PointerFromQIODevice(device))
	}
}

func (ptr *QImageWriter) SetFileName(fileName string) {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		C.QImageWriter_SetFileName(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))})
	}
}

func (ptr *QImageWriter) SetFormat(format core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageWriter_SetFormat(ptr.Pointer(), core.PointerFromQByteArray(format))
	}
}

func (ptr *QImageWriter) SetGamma(gamma float32) {
	if ptr.Pointer() != nil {
		C.QImageWriter_SetGamma(ptr.Pointer(), C.float(gamma))
	}
}

func (ptr *QImageWriter) SetOptimizedWrite(optimize bool) {
	if ptr.Pointer() != nil {
		C.QImageWriter_SetOptimizedWrite(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(optimize))))
	}
}

func (ptr *QImageWriter) SetProgressiveScanWrite(progressive bool) {
	if ptr.Pointer() != nil {
		C.QImageWriter_SetProgressiveScanWrite(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(progressive))))
	}
}

func (ptr *QImageWriter) SetQuality(quality int) {
	if ptr.Pointer() != nil {
		C.QImageWriter_SetQuality(ptr.Pointer(), C.int(int32(quality)))
	}
}

func (ptr *QImageWriter) SetSubType(ty core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageWriter_SetSubType(ptr.Pointer(), core.PointerFromQByteArray(ty))
	}
}

func (ptr *QImageWriter) SetText(key string, text string) {
	if ptr.Pointer() != nil {
		var keyC *C.char
		if key != "" {
			keyC = C.CString(key)
			defer C.free(unsafe.Pointer(keyC))
		}
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QImageWriter_SetText(ptr.Pointer(), C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QImageWriter) SetTransformation(transform QImageIOHandler__Transformation) {
	if ptr.Pointer() != nil {
		C.QImageWriter_SetTransformation(ptr.Pointer(), C.longlong(transform))
	}
}

func (ptr *QImageWriter) SubType() *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageWriter_SubType(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func QImageWriter_SupportedImageFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageWriterFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedImageFormats_atList(i)
		}
		return out
	}(C.QImageWriter_QImageWriter_SupportedImageFormats())
}

func (ptr *QImageWriter) SupportedImageFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageWriterFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedImageFormats_atList(i)
		}
		return out
	}(C.QImageWriter_QImageWriter_SupportedImageFormats())
}

func QImageWriter_SupportedMimeTypes() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageWriterFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedMimeTypes_atList(i)
		}
		return out
	}(C.QImageWriter_QImageWriter_SupportedMimeTypes())
}

func (ptr *QImageWriter) SupportedMimeTypes() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQImageWriterFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedMimeTypes_atList(i)
		}
		return out
	}(C.QImageWriter_QImageWriter_SupportedMimeTypes())
}

func (ptr *QImageWriter) SupportedSubTypes() []*core.QByteArray {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
			out := make([]*core.QByteArray, int(l.len))
			tmpList := NewQImageWriterFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__supportedSubTypes_atList(i)
			}
			return out
		}(C.QImageWriter_SupportedSubTypes(ptr.Pointer()))
	}
	return make([]*core.QByteArray, 0)
}

func (ptr *QImageWriter) SupportsOption(option QImageIOHandler__ImageOption) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageWriter_SupportsOption(ptr.Pointer(), C.longlong(option))) != 0
	}
	return false
}

func (ptr *QImageWriter) Transformation() QImageIOHandler__Transformation {
	if ptr.Pointer() != nil {
		return QImageIOHandler__Transformation(C.QImageWriter_Transformation(ptr.Pointer()))
	}
	return 0
}

func (ptr *QImageWriter) Write(image QImage_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QImageWriter_Write(ptr.Pointer(), PointerFromQImage(image))) != 0
	}
	return false
}

func (ptr *QImageWriter) DestroyQImageWriter() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QImageWriter_DestroyQImageWriter(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QImageWriter) __imageFormatsForMimeType_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageWriter___imageFormatsForMimeType_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageWriter) __imageFormatsForMimeType_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageWriter___imageFormatsForMimeType_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QImageWriter) __imageFormatsForMimeType_newList() unsafe.Pointer {
	return C.QImageWriter___imageFormatsForMimeType_newList(ptr.Pointer())
}

func (ptr *QImageWriter) __supportedImageFormats_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageWriter___supportedImageFormats_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageWriter) __supportedImageFormats_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageWriter___supportedImageFormats_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QImageWriter) __supportedImageFormats_newList() unsafe.Pointer {
	return C.QImageWriter___supportedImageFormats_newList(ptr.Pointer())
}

func (ptr *QImageWriter) __supportedMimeTypes_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageWriter___supportedMimeTypes_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageWriter) __supportedMimeTypes_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageWriter___supportedMimeTypes_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QImageWriter) __supportedMimeTypes_newList() unsafe.Pointer {
	return C.QImageWriter___supportedMimeTypes_newList(ptr.Pointer())
}

func (ptr *QImageWriter) __supportedSubTypes_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QImageWriter___supportedSubTypes_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QImageWriter) __supportedSubTypes_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QImageWriter___supportedSubTypes_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QImageWriter) __supportedSubTypes_newList() unsafe.Pointer {
	return C.QImageWriter___supportedSubTypes_newList(ptr.Pointer())
}

type QInputEvent struct {
	core.QEvent
}

type QInputEvent_ITF interface {
	core.QEvent_ITF
	QInputEvent_PTR() *QInputEvent
}

func (ptr *QInputEvent) QInputEvent_PTR() *QInputEvent {
	return ptr
}

func (ptr *QInputEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QInputEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQInputEvent(ptr QInputEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputEvent_PTR().Pointer()
	}
	return nil
}

func NewQInputEventFromPointer(ptr unsafe.Pointer) (n *QInputEvent) {
	n = new(QInputEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QInputEvent) DestroyQInputEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func (ptr *QInputEvent) Modifiers() core.Qt__KeyboardModifier {
	if ptr.Pointer() != nil {
		return core.Qt__KeyboardModifier(C.QInputEvent_Modifiers(ptr.Pointer()))
	}
	return 0
}

func (ptr *QInputEvent) Timestamp() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QInputEvent_Timestamp(ptr.Pointer())))
	}
	return 0
}

type QInputMethod struct {
	core.QObject
}

type QInputMethod_ITF interface {
	core.QObject_ITF
	QInputMethod_PTR() *QInputMethod
}

func (ptr *QInputMethod) QInputMethod_PTR() *QInputMethod {
	return ptr
}

func (ptr *QInputMethod) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QInputMethod) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQInputMethod(ptr QInputMethod_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputMethod_PTR().Pointer()
	}
	return nil
}

func NewQInputMethodFromPointer(ptr unsafe.Pointer) (n *QInputMethod) {
	n = new(QInputMethod)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QInputMethod__Action
//QInputMethod::Action
type QInputMethod__Action int64

const (
	QInputMethod__Click       QInputMethod__Action = QInputMethod__Action(0)
	QInputMethod__ContextMenu QInputMethod__Action = QInputMethod__Action(1)
)

func (ptr *QInputMethod) AnchorRectangle() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QInputMethod_AnchorRectangle(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

//export callbackQInputMethod_AnchorRectangleChanged
func callbackQInputMethod_AnchorRectangleChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "anchorRectangleChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QInputMethod) ConnectAnchorRectangleChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "anchorRectangleChanged") {
			C.QInputMethod_ConnectAnchorRectangleChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "anchorRectangleChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "anchorRectangleChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "anchorRectangleChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "anchorRectangleChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectAnchorRectangleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_DisconnectAnchorRectangleChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "anchorRectangleChanged")
	}
}

func (ptr *QInputMethod) AnchorRectangleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_AnchorRectangleChanged(ptr.Pointer())
	}
}

//export callbackQInputMethod_AnimatingChanged
func callbackQInputMethod_AnimatingChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "animatingChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QInputMethod) ConnectAnimatingChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "animatingChanged") {
			C.QInputMethod_ConnectAnimatingChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "animatingChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "animatingChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "animatingChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "animatingChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectAnimatingChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_DisconnectAnimatingChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "animatingChanged")
	}
}

func (ptr *QInputMethod) AnimatingChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_AnimatingChanged(ptr.Pointer())
	}
}

//export callbackQInputMethod_Commit
func callbackQInputMethod_Commit(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "commit"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQInputMethodFromPointer(ptr).CommitDefault()
	}
}

func (ptr *QInputMethod) ConnectCommit(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "commit"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "commit", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "commit", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectCommit() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "commit")
	}
}

func (ptr *QInputMethod) Commit() {
	if ptr.Pointer() != nil {
		C.QInputMethod_Commit(ptr.Pointer())
	}
}

func (ptr *QInputMethod) CommitDefault() {
	if ptr.Pointer() != nil {
		C.QInputMethod_CommitDefault(ptr.Pointer())
	}
}

func (ptr *QInputMethod) CursorRectangle() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QInputMethod_CursorRectangle(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

//export callbackQInputMethod_CursorRectangleChanged
func callbackQInputMethod_CursorRectangleChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "cursorRectangleChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QInputMethod) ConnectCursorRectangleChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "cursorRectangleChanged") {
			C.QInputMethod_ConnectCursorRectangleChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "cursorRectangleChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "cursorRectangleChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "cursorRectangleChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "cursorRectangleChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectCursorRectangleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_DisconnectCursorRectangleChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "cursorRectangleChanged")
	}
}

func (ptr *QInputMethod) CursorRectangleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_CursorRectangleChanged(ptr.Pointer())
	}
}

//export callbackQInputMethod_Hide
func callbackQInputMethod_Hide(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "hide"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQInputMethodFromPointer(ptr).HideDefault()
	}
}

func (ptr *QInputMethod) ConnectHide(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "hide"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "hide", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "hide", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectHide() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "hide")
	}
}

func (ptr *QInputMethod) Hide() {
	if ptr.Pointer() != nil {
		C.QInputMethod_Hide(ptr.Pointer())
	}
}

func (ptr *QInputMethod) HideDefault() {
	if ptr.Pointer() != nil {
		C.QInputMethod_HideDefault(ptr.Pointer())
	}
}

func (ptr *QInputMethod) InputDirection() core.Qt__LayoutDirection {
	if ptr.Pointer() != nil {
		return core.Qt__LayoutDirection(C.QInputMethod_InputDirection(ptr.Pointer()))
	}
	return 0
}

//export callbackQInputMethod_InputDirectionChanged
func callbackQInputMethod_InputDirectionChanged(ptr unsafe.Pointer, newDirection C.longlong) {
	if signal := qt.GetSignal(ptr, "inputDirectionChanged"); signal != nil {
		(*(*func(core.Qt__LayoutDirection))(signal))(core.Qt__LayoutDirection(newDirection))
	}

}

func (ptr *QInputMethod) ConnectInputDirectionChanged(f func(newDirection core.Qt__LayoutDirection)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "inputDirectionChanged") {
			C.QInputMethod_ConnectInputDirectionChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "inputDirectionChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "inputDirectionChanged"); signal != nil {
			f := func(newDirection core.Qt__LayoutDirection) {
				(*(*func(core.Qt__LayoutDirection))(signal))(newDirection)
				f(newDirection)
			}
			qt.ConnectSignal(ptr.Pointer(), "inputDirectionChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "inputDirectionChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectInputDirectionChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_DisconnectInputDirectionChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "inputDirectionChanged")
	}
}

func (ptr *QInputMethod) InputDirectionChanged(newDirection core.Qt__LayoutDirection) {
	if ptr.Pointer() != nil {
		C.QInputMethod_InputDirectionChanged(ptr.Pointer(), C.longlong(newDirection))
	}
}

func (ptr *QInputMethod) InputItemClipRectangle() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QInputMethod_InputItemClipRectangle(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

//export callbackQInputMethod_InputItemClipRectangleChanged
func callbackQInputMethod_InputItemClipRectangleChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "inputItemClipRectangleChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QInputMethod) ConnectInputItemClipRectangleChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "inputItemClipRectangleChanged") {
			C.QInputMethod_ConnectInputItemClipRectangleChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "inputItemClipRectangleChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "inputItemClipRectangleChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "inputItemClipRectangleChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "inputItemClipRectangleChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectInputItemClipRectangleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_DisconnectInputItemClipRectangleChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "inputItemClipRectangleChanged")
	}
}

func (ptr *QInputMethod) InputItemClipRectangleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_InputItemClipRectangleChanged(ptr.Pointer())
	}
}

func (ptr *QInputMethod) InputItemRectangle() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QInputMethod_InputItemRectangle(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QInputMethod) InputItemTransform() *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QInputMethod_InputItemTransform(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

//export callbackQInputMethod_InvokeAction
func callbackQInputMethod_InvokeAction(ptr unsafe.Pointer, a C.longlong, cursorPosition C.int) {
	if signal := qt.GetSignal(ptr, "invokeAction"); signal != nil {
		(*(*func(QInputMethod__Action, int))(signal))(QInputMethod__Action(a), int(int32(cursorPosition)))
	} else {
		NewQInputMethodFromPointer(ptr).InvokeActionDefault(QInputMethod__Action(a), int(int32(cursorPosition)))
	}
}

func (ptr *QInputMethod) ConnectInvokeAction(f func(a QInputMethod__Action, cursorPosition int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "invokeAction"); signal != nil {
			f := func(a QInputMethod__Action, cursorPosition int) {
				(*(*func(QInputMethod__Action, int))(signal))(a, cursorPosition)
				f(a, cursorPosition)
			}
			qt.ConnectSignal(ptr.Pointer(), "invokeAction", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "invokeAction", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectInvokeAction() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "invokeAction")
	}
}

func (ptr *QInputMethod) InvokeAction(a QInputMethod__Action, cursorPosition int) {
	if ptr.Pointer() != nil {
		C.QInputMethod_InvokeAction(ptr.Pointer(), C.longlong(a), C.int(int32(cursorPosition)))
	}
}

func (ptr *QInputMethod) InvokeActionDefault(a QInputMethod__Action, cursorPosition int) {
	if ptr.Pointer() != nil {
		C.QInputMethod_InvokeActionDefault(ptr.Pointer(), C.longlong(a), C.int(int32(cursorPosition)))
	}
}

func (ptr *QInputMethod) IsAnimating() bool {
	if ptr.Pointer() != nil {
		return int8(C.QInputMethod_IsAnimating(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QInputMethod) IsVisible() bool {
	if ptr.Pointer() != nil {
		return int8(C.QInputMethod_IsVisible(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QInputMethod) KeyboardRectangle() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QInputMethod_KeyboardRectangle(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

//export callbackQInputMethod_KeyboardRectangleChanged
func callbackQInputMethod_KeyboardRectangleChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "keyboardRectangleChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QInputMethod) ConnectKeyboardRectangleChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "keyboardRectangleChanged") {
			C.QInputMethod_ConnectKeyboardRectangleChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "keyboardRectangleChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "keyboardRectangleChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "keyboardRectangleChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "keyboardRectangleChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectKeyboardRectangleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_DisconnectKeyboardRectangleChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "keyboardRectangleChanged")
	}
}

func (ptr *QInputMethod) KeyboardRectangleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_KeyboardRectangleChanged(ptr.Pointer())
	}
}

func (ptr *QInputMethod) Locale() *core.QLocale {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQLocaleFromPointer(C.QInputMethod_Locale(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QLocale).DestroyQLocale)
		return tmpValue
	}
	return nil
}

//export callbackQInputMethod_LocaleChanged
func callbackQInputMethod_LocaleChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "localeChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QInputMethod) ConnectLocaleChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "localeChanged") {
			C.QInputMethod_ConnectLocaleChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "localeChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "localeChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "localeChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "localeChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectLocaleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_DisconnectLocaleChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "localeChanged")
	}
}

func (ptr *QInputMethod) LocaleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_LocaleChanged(ptr.Pointer())
	}
}

func QInputMethod_QueryFocusObject(query core.Qt__InputMethodQuery, argument core.QVariant_ITF) *core.QVariant {
	tmpValue := core.NewQVariantFromPointer(C.QInputMethod_QInputMethod_QueryFocusObject(C.longlong(query), core.PointerFromQVariant(argument)))
	qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
	return tmpValue
}

func (ptr *QInputMethod) QueryFocusObject(query core.Qt__InputMethodQuery, argument core.QVariant_ITF) *core.QVariant {
	tmpValue := core.NewQVariantFromPointer(C.QInputMethod_QInputMethod_QueryFocusObject(C.longlong(query), core.PointerFromQVariant(argument)))
	qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
	return tmpValue
}

//export callbackQInputMethod_Reset
func callbackQInputMethod_Reset(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "reset"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQInputMethodFromPointer(ptr).ResetDefault()
	}
}

func (ptr *QInputMethod) ConnectReset(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "reset"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "reset", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "reset", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectReset() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "reset")
	}
}

func (ptr *QInputMethod) Reset() {
	if ptr.Pointer() != nil {
		C.QInputMethod_Reset(ptr.Pointer())
	}
}

func (ptr *QInputMethod) ResetDefault() {
	if ptr.Pointer() != nil {
		C.QInputMethod_ResetDefault(ptr.Pointer())
	}
}

func (ptr *QInputMethod) SetInputItemRectangle(rect core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod_SetInputItemRectangle(ptr.Pointer(), core.PointerFromQRectF(rect))
	}
}

func (ptr *QInputMethod) SetInputItemTransform(transform QTransform_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod_SetInputItemTransform(ptr.Pointer(), PointerFromQTransform(transform))
	}
}

func (ptr *QInputMethod) SetVisible(visible bool) {
	if ptr.Pointer() != nil {
		C.QInputMethod_SetVisible(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(visible))))
	}
}

//export callbackQInputMethod_Show
func callbackQInputMethod_Show(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "show"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQInputMethodFromPointer(ptr).ShowDefault()
	}
}

func (ptr *QInputMethod) ConnectShow(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "show"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "show", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "show", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectShow() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "show")
	}
}

func (ptr *QInputMethod) Show() {
	if ptr.Pointer() != nil {
		C.QInputMethod_Show(ptr.Pointer())
	}
}

func (ptr *QInputMethod) ShowDefault() {
	if ptr.Pointer() != nil {
		C.QInputMethod_ShowDefault(ptr.Pointer())
	}
}

//export callbackQInputMethod_Update
func callbackQInputMethod_Update(ptr unsafe.Pointer, queries C.longlong) {
	if signal := qt.GetSignal(ptr, "update"); signal != nil {
		(*(*func(core.Qt__InputMethodQuery))(signal))(core.Qt__InputMethodQuery(queries))
	} else {
		NewQInputMethodFromPointer(ptr).UpdateDefault(core.Qt__InputMethodQuery(queries))
	}
}

func (ptr *QInputMethod) ConnectUpdate(f func(queries core.Qt__InputMethodQuery)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "update"); signal != nil {
			f := func(queries core.Qt__InputMethodQuery) {
				(*(*func(core.Qt__InputMethodQuery))(signal))(queries)
				f(queries)
			}
			qt.ConnectSignal(ptr.Pointer(), "update", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "update", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectUpdate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "update")
	}
}

func (ptr *QInputMethod) Update(queries core.Qt__InputMethodQuery) {
	if ptr.Pointer() != nil {
		C.QInputMethod_Update(ptr.Pointer(), C.longlong(queries))
	}
}

func (ptr *QInputMethod) UpdateDefault(queries core.Qt__InputMethodQuery) {
	if ptr.Pointer() != nil {
		C.QInputMethod_UpdateDefault(ptr.Pointer(), C.longlong(queries))
	}
}

//export callbackQInputMethod_VisibleChanged
func callbackQInputMethod_VisibleChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "visibleChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QInputMethod) ConnectVisibleChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "visibleChanged") {
			C.QInputMethod_ConnectVisibleChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "visibleChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "visibleChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "visibleChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "visibleChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QInputMethod) DisconnectVisibleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_DisconnectVisibleChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "visibleChanged")
	}
}

func (ptr *QInputMethod) VisibleChanged() {
	if ptr.Pointer() != nil {
		C.QInputMethod_VisibleChanged(ptr.Pointer())
	}
}

func (ptr *QInputMethod) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QInputMethod___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QInputMethod) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QInputMethod) __children_newList() unsafe.Pointer {
	return C.QInputMethod___children_newList(ptr.Pointer())
}

func (ptr *QInputMethod) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QInputMethod___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QInputMethod) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QInputMethod) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QInputMethod___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QInputMethod) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QInputMethod___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QInputMethod) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QInputMethod) __findChildren_newList() unsafe.Pointer {
	return C.QInputMethod___findChildren_newList(ptr.Pointer())
}

func (ptr *QInputMethod) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QInputMethod___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QInputMethod) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QInputMethod) __findChildren_newList3() unsafe.Pointer {
	return C.QInputMethod___findChildren_newList3(ptr.Pointer())
}

//export callbackQInputMethod_ChildEvent
func callbackQInputMethod_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQInputMethodFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QInputMethod) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQInputMethod_ConnectNotify
func callbackQInputMethod_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQInputMethodFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QInputMethod) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQInputMethod_CustomEvent
func callbackQInputMethod_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQInputMethodFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QInputMethod) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQInputMethod_DeleteLater
func callbackQInputMethod_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQInputMethodFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QInputMethod) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QInputMethod_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQInputMethod_Destroyed
func callbackQInputMethod_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQInputMethod_DisconnectNotify
func callbackQInputMethod_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQInputMethodFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QInputMethod) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQInputMethod_Event
func callbackQInputMethod_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQInputMethodFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QInputMethod) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QInputMethod_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQInputMethod_EventFilter
func callbackQInputMethod_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQInputMethodFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QInputMethod) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QInputMethod_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQInputMethod_MetaObject
func callbackQInputMethod_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQInputMethodFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QInputMethod) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QInputMethod_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQInputMethod_ObjectNameChanged
func callbackQInputMethod_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQInputMethod_TimerEvent
func callbackQInputMethod_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQInputMethodFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QInputMethod) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethod_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QInputMethodEvent struct {
	core.QEvent
}

type QInputMethodEvent_ITF interface {
	core.QEvent_ITF
	QInputMethodEvent_PTR() *QInputMethodEvent
}

func (ptr *QInputMethodEvent) QInputMethodEvent_PTR() *QInputMethodEvent {
	return ptr
}

func (ptr *QInputMethodEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QInputMethodEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQInputMethodEvent(ptr QInputMethodEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputMethodEvent_PTR().Pointer()
	}
	return nil
}

func NewQInputMethodEventFromPointer(ptr unsafe.Pointer) (n *QInputMethodEvent) {
	n = new(QInputMethodEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QInputMethodEvent) DestroyQInputMethodEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QInputMethodEvent__AttributeType
//QInputMethodEvent::AttributeType
type QInputMethodEvent__AttributeType int64

const (
	QInputMethodEvent__TextFormat QInputMethodEvent__AttributeType = QInputMethodEvent__AttributeType(0)
	QInputMethodEvent__Cursor     QInputMethodEvent__AttributeType = QInputMethodEvent__AttributeType(1)
	QInputMethodEvent__Language   QInputMethodEvent__AttributeType = QInputMethodEvent__AttributeType(2)
	QInputMethodEvent__Ruby       QInputMethodEvent__AttributeType = QInputMethodEvent__AttributeType(3)
	QInputMethodEvent__Selection  QInputMethodEvent__AttributeType = QInputMethodEvent__AttributeType(4)
)

func NewQInputMethodEvent() *QInputMethodEvent {
	tmpValue := NewQInputMethodEventFromPointer(C.QInputMethodEvent_NewQInputMethodEvent())
	qt.SetFinalizer(tmpValue, (*QInputMethodEvent).DestroyQInputMethodEvent)
	return tmpValue
}

func NewQInputMethodEvent3(other QInputMethodEvent_ITF) *QInputMethodEvent {
	tmpValue := NewQInputMethodEventFromPointer(C.QInputMethodEvent_NewQInputMethodEvent3(PointerFromQInputMethodEvent(other)))
	qt.SetFinalizer(tmpValue, (*QInputMethodEvent).DestroyQInputMethodEvent)
	return tmpValue
}

func (ptr *QInputMethodEvent) CommitString() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QInputMethodEvent_CommitString(ptr.Pointer()))
	}
	return ""
}

func (ptr *QInputMethodEvent) PreeditString() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QInputMethodEvent_PreeditString(ptr.Pointer()))
	}
	return ""
}

func (ptr *QInputMethodEvent) ReplacementLength() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QInputMethodEvent_ReplacementLength(ptr.Pointer())))
	}
	return 0
}

func (ptr *QInputMethodEvent) ReplacementStart() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QInputMethodEvent_ReplacementStart(ptr.Pointer())))
	}
	return 0
}

func (ptr *QInputMethodEvent) SetCommitString(commitString string, replaceFrom int, replaceLength int) {
	if ptr.Pointer() != nil {
		var commitStringC *C.char
		if commitString != "" {
			commitStringC = C.CString(commitString)
			defer C.free(unsafe.Pointer(commitStringC))
		}
		C.QInputMethodEvent_SetCommitString(ptr.Pointer(), C.struct_QtGui_PackedString{data: commitStringC, len: C.longlong(len(commitString))}, C.int(int32(replaceFrom)), C.int(int32(replaceLength)))
	}
}

func (ptr *QInputMethodEvent) __attrs_newList() unsafe.Pointer {
	return C.QInputMethodEvent___attrs_newList(ptr.Pointer())
}

func (ptr *QInputMethodEvent) __setAttrs__newList() unsafe.Pointer {
	return C.QInputMethodEvent___setAttrs__newList(ptr.Pointer())
}

type QInputMethodQueryEvent struct {
	core.QEvent
}

type QInputMethodQueryEvent_ITF interface {
	core.QEvent_ITF
	QInputMethodQueryEvent_PTR() *QInputMethodQueryEvent
}

func (ptr *QInputMethodQueryEvent) QInputMethodQueryEvent_PTR() *QInputMethodQueryEvent {
	return ptr
}

func (ptr *QInputMethodQueryEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QInputMethodQueryEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQInputMethodQueryEvent(ptr QInputMethodQueryEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputMethodQueryEvent_PTR().Pointer()
	}
	return nil
}

func NewQInputMethodQueryEventFromPointer(ptr unsafe.Pointer) (n *QInputMethodQueryEvent) {
	n = new(QInputMethodQueryEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QInputMethodQueryEvent) DestroyQInputMethodQueryEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQInputMethodQueryEvent(queries core.Qt__InputMethodQuery) *QInputMethodQueryEvent {
	tmpValue := NewQInputMethodQueryEventFromPointer(C.QInputMethodQueryEvent_NewQInputMethodQueryEvent(C.longlong(queries)))
	qt.SetFinalizer(tmpValue, (*QInputMethodQueryEvent).DestroyQInputMethodQueryEvent)
	return tmpValue
}

func (ptr *QInputMethodQueryEvent) Queries() core.Qt__InputMethodQuery {
	if ptr.Pointer() != nil {
		return core.Qt__InputMethodQuery(C.QInputMethodQueryEvent_Queries(ptr.Pointer()))
	}
	return 0
}

func (ptr *QInputMethodQueryEvent) SetValue(query core.Qt__InputMethodQuery, value core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QInputMethodQueryEvent_SetValue(ptr.Pointer(), C.longlong(query), core.PointerFromQVariant(value))
	}
}

func (ptr *QInputMethodQueryEvent) Value(query core.Qt__InputMethodQuery) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QInputMethodQueryEvent_Value(ptr.Pointer(), C.longlong(query)))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QInputMethodQueryEvent) __m_values_newList() unsafe.Pointer {
	return C.QInputMethodQueryEvent___m_values_newList(ptr.Pointer())
}

func (ptr *QInputMethodQueryEvent) __setM_values__newList() unsafe.Pointer {
	return C.QInputMethodQueryEvent___setM_values__newList(ptr.Pointer())
}

type QIntValidator struct {
	QValidator
}

type QIntValidator_ITF interface {
	QValidator_ITF
	QIntValidator_PTR() *QIntValidator
}

func (ptr *QIntValidator) QIntValidator_PTR() *QIntValidator {
	return ptr
}

func (ptr *QIntValidator) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QValidator_PTR().Pointer()
	}
	return nil
}

func (ptr *QIntValidator) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QValidator_PTR().SetPointer(p)
	}
}

func PointerFromQIntValidator(ptr QIntValidator_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QIntValidator_PTR().Pointer()
	}
	return nil
}

func NewQIntValidatorFromPointer(ptr unsafe.Pointer) (n *QIntValidator) {
	n = new(QIntValidator)
	n.SetPointer(ptr)
	return
}
func NewQIntValidator(parent core.QObject_ITF) *QIntValidator {
	tmpValue := NewQIntValidatorFromPointer(C.QIntValidator_NewQIntValidator(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQIntValidator2(minimum int, maximum int, parent core.QObject_ITF) *QIntValidator {
	tmpValue := NewQIntValidatorFromPointer(C.QIntValidator_NewQIntValidator2(C.int(int32(minimum)), C.int(int32(maximum)), core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QIntValidator) Bottom() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QIntValidator_Bottom(ptr.Pointer())))
	}
	return 0
}

//export callbackQIntValidator_BottomChanged
func callbackQIntValidator_BottomChanged(ptr unsafe.Pointer, bottom C.int) {
	if signal := qt.GetSignal(ptr, "bottomChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(bottom)))
	}

}

func (ptr *QIntValidator) ConnectBottomChanged(f func(bottom int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "bottomChanged") {
			C.QIntValidator_ConnectBottomChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "bottomChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "bottomChanged"); signal != nil {
			f := func(bottom int) {
				(*(*func(int))(signal))(bottom)
				f(bottom)
			}
			qt.ConnectSignal(ptr.Pointer(), "bottomChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "bottomChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIntValidator) DisconnectBottomChanged() {
	if ptr.Pointer() != nil {
		C.QIntValidator_DisconnectBottomChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "bottomChanged")
	}
}

func (ptr *QIntValidator) BottomChanged(bottom int) {
	if ptr.Pointer() != nil {
		C.QIntValidator_BottomChanged(ptr.Pointer(), C.int(int32(bottom)))
	}
}

func (ptr *QIntValidator) SetBottom(vin int) {
	if ptr.Pointer() != nil {
		C.QIntValidator_SetBottom(ptr.Pointer(), C.int(int32(vin)))
	}
}

//export callbackQIntValidator_SetRange
func callbackQIntValidator_SetRange(ptr unsafe.Pointer, bottom C.int, top C.int) {
	if signal := qt.GetSignal(ptr, "setRange"); signal != nil {
		(*(*func(int, int))(signal))(int(int32(bottom)), int(int32(top)))
	} else {
		NewQIntValidatorFromPointer(ptr).SetRangeDefault(int(int32(bottom)), int(int32(top)))
	}
}

func (ptr *QIntValidator) ConnectSetRange(f func(bottom int, top int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setRange"); signal != nil {
			f := func(bottom int, top int) {
				(*(*func(int, int))(signal))(bottom, top)
				f(bottom, top)
			}
			qt.ConnectSignal(ptr.Pointer(), "setRange", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setRange", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIntValidator) DisconnectSetRange() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setRange")
	}
}

func (ptr *QIntValidator) SetRange(bottom int, top int) {
	if ptr.Pointer() != nil {
		C.QIntValidator_SetRange(ptr.Pointer(), C.int(int32(bottom)), C.int(int32(top)))
	}
}

func (ptr *QIntValidator) SetRangeDefault(bottom int, top int) {
	if ptr.Pointer() != nil {
		C.QIntValidator_SetRangeDefault(ptr.Pointer(), C.int(int32(bottom)), C.int(int32(top)))
	}
}

func (ptr *QIntValidator) SetTop(vin int) {
	if ptr.Pointer() != nil {
		C.QIntValidator_SetTop(ptr.Pointer(), C.int(int32(vin)))
	}
}

func (ptr *QIntValidator) Top() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QIntValidator_Top(ptr.Pointer())))
	}
	return 0
}

//export callbackQIntValidator_TopChanged
func callbackQIntValidator_TopChanged(ptr unsafe.Pointer, top C.int) {
	if signal := qt.GetSignal(ptr, "topChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(top)))
	}

}

func (ptr *QIntValidator) ConnectTopChanged(f func(top int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "topChanged") {
			C.QIntValidator_ConnectTopChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "topChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "topChanged"); signal != nil {
			f := func(top int) {
				(*(*func(int))(signal))(top)
				f(top)
			}
			qt.ConnectSignal(ptr.Pointer(), "topChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "topChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIntValidator) DisconnectTopChanged() {
	if ptr.Pointer() != nil {
		C.QIntValidator_DisconnectTopChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "topChanged")
	}
}

func (ptr *QIntValidator) TopChanged(top int) {
	if ptr.Pointer() != nil {
		C.QIntValidator_TopChanged(ptr.Pointer(), C.int(int32(top)))
	}
}

//export callbackQIntValidator_Validate
func callbackQIntValidator_Validate(ptr unsafe.Pointer, input C.struct_QtGui_PackedString, pos C.int) C.longlong {
	if signal := qt.GetSignal(ptr, "validate"); signal != nil {
		return C.longlong((*(*func(string, int) QValidator__State)(signal))(cGoUnpackString(input), int(int32(pos))))
	}

	return C.longlong(NewQIntValidatorFromPointer(ptr).ValidateDefault(cGoUnpackString(input), int(int32(pos))))
}

func (ptr *QIntValidator) ConnectValidate(f func(input string, pos int) QValidator__State) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "validate"); signal != nil {
			f := func(input string, pos int) QValidator__State {
				(*(*func(string, int) QValidator__State)(signal))(input, pos)
				return f(input, pos)
			}
			qt.ConnectSignal(ptr.Pointer(), "validate", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "validate", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIntValidator) DisconnectValidate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "validate")
	}
}

func (ptr *QIntValidator) Validate(input string, pos int) QValidator__State {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		return QValidator__State(C.QIntValidator_Validate(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))}, C.int(int32(pos))))
	}
	return 0
}

func (ptr *QIntValidator) ValidateDefault(input string, pos int) QValidator__State {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		return QValidator__State(C.QIntValidator_ValidateDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))}, C.int(int32(pos))))
	}
	return 0
}

//export callbackQIntValidator_DestroyQIntValidator
func callbackQIntValidator_DestroyQIntValidator(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QIntValidator"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQIntValidatorFromPointer(ptr).DestroyQIntValidatorDefault()
	}
}

func (ptr *QIntValidator) ConnectDestroyQIntValidator(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QIntValidator"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QIntValidator", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QIntValidator", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QIntValidator) DisconnectDestroyQIntValidator() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QIntValidator")
	}
}

func (ptr *QIntValidator) DestroyQIntValidator() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QIntValidator_DestroyQIntValidator(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QIntValidator) DestroyQIntValidatorDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QIntValidator_DestroyQIntValidatorDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QKeyEvent struct {
	QInputEvent
}

type QKeyEvent_ITF interface {
	QInputEvent_ITF
	QKeyEvent_PTR() *QKeyEvent
}

func (ptr *QKeyEvent) QKeyEvent_PTR() *QKeyEvent {
	return ptr
}

func (ptr *QKeyEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QKeyEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QInputEvent_PTR().SetPointer(p)
	}
}

func PointerFromQKeyEvent(ptr QKeyEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QKeyEvent_PTR().Pointer()
	}
	return nil
}

func NewQKeyEventFromPointer(ptr unsafe.Pointer) (n *QKeyEvent) {
	n = new(QKeyEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QKeyEvent) DestroyQKeyEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQKeyEvent(ty core.QEvent__Type, key int, modifiers core.Qt__KeyboardModifier, text string, autorep bool, count uint16) *QKeyEvent {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQKeyEventFromPointer(C.QKeyEvent_NewQKeyEvent(C.longlong(ty), C.int(int32(key)), C.longlong(modifiers), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.char(int8(qt.GoBoolToInt(autorep))), C.ushort(count)))
	qt.SetFinalizer(tmpValue, (*QKeyEvent).DestroyQKeyEvent)
	return tmpValue
}

func NewQKeyEvent2(ty core.QEvent__Type, key int, modifiers core.Qt__KeyboardModifier, nativeScanCode uint, nativeVirtualKey uint, nativeModifiers uint, text string, autorep bool, count uint16) *QKeyEvent {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQKeyEventFromPointer(C.QKeyEvent_NewQKeyEvent2(C.longlong(ty), C.int(int32(key)), C.longlong(modifiers), C.uint(uint32(nativeScanCode)), C.uint(uint32(nativeVirtualKey)), C.uint(uint32(nativeModifiers)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.char(int8(qt.GoBoolToInt(autorep))), C.ushort(count)))
	qt.SetFinalizer(tmpValue, (*QKeyEvent).DestroyQKeyEvent)
	return tmpValue
}

func (ptr *QKeyEvent) Count() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QKeyEvent_Count(ptr.Pointer())))
	}
	return 0
}

func (ptr *QKeyEvent) IsAutoRepeat() bool {
	if ptr.Pointer() != nil {
		return int8(C.QKeyEvent_IsAutoRepeat(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QKeyEvent) Key() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QKeyEvent_Key(ptr.Pointer())))
	}
	return 0
}

func (ptr *QKeyEvent) Matches(key QKeySequence__StandardKey) bool {
	if ptr.Pointer() != nil {
		return int8(C.QKeyEvent_Matches(ptr.Pointer(), C.longlong(key))) != 0
	}
	return false
}

func (ptr *QKeyEvent) NativeModifiers() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QKeyEvent_NativeModifiers(ptr.Pointer())))
	}
	return 0
}

func (ptr *QKeyEvent) NativeScanCode() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QKeyEvent_NativeScanCode(ptr.Pointer())))
	}
	return 0
}

func (ptr *QKeyEvent) NativeVirtualKey() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QKeyEvent_NativeVirtualKey(ptr.Pointer())))
	}
	return 0
}

func (ptr *QKeyEvent) Text() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QKeyEvent_Text(ptr.Pointer()))
	}
	return ""
}

type QKeySequence struct {
	ptr unsafe.Pointer
}

type QKeySequence_ITF interface {
	QKeySequence_PTR() *QKeySequence
}

func (ptr *QKeySequence) QKeySequence_PTR() *QKeySequence {
	return ptr
}

func (ptr *QKeySequence) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QKeySequence) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQKeySequence(ptr QKeySequence_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QKeySequence_PTR().Pointer()
	}
	return nil
}

func NewQKeySequenceFromPointer(ptr unsafe.Pointer) (n *QKeySequence) {
	n = new(QKeySequence)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QKeySequence__StandardKey
//QKeySequence::StandardKey
type QKeySequence__StandardKey int64

const (
	QKeySequence__UnknownKey               QKeySequence__StandardKey = QKeySequence__StandardKey(0)
	QKeySequence__HelpContents             QKeySequence__StandardKey = QKeySequence__StandardKey(1)
	QKeySequence__WhatsThis                QKeySequence__StandardKey = QKeySequence__StandardKey(2)
	QKeySequence__Open                     QKeySequence__StandardKey = QKeySequence__StandardKey(3)
	QKeySequence__Close                    QKeySequence__StandardKey = QKeySequence__StandardKey(4)
	QKeySequence__Save                     QKeySequence__StandardKey = QKeySequence__StandardKey(5)
	QKeySequence__New                      QKeySequence__StandardKey = QKeySequence__StandardKey(6)
	QKeySequence__Delete                   QKeySequence__StandardKey = QKeySequence__StandardKey(7)
	QKeySequence__Cut                      QKeySequence__StandardKey = QKeySequence__StandardKey(8)
	QKeySequence__Copy                     QKeySequence__StandardKey = QKeySequence__StandardKey(9)
	QKeySequence__Paste                    QKeySequence__StandardKey = QKeySequence__StandardKey(10)
	QKeySequence__Undo                     QKeySequence__StandardKey = QKeySequence__StandardKey(11)
	QKeySequence__Redo                     QKeySequence__StandardKey = QKeySequence__StandardKey(12)
	QKeySequence__Back                     QKeySequence__StandardKey = QKeySequence__StandardKey(13)
	QKeySequence__Forward                  QKeySequence__StandardKey = QKeySequence__StandardKey(14)
	QKeySequence__Refresh                  QKeySequence__StandardKey = QKeySequence__StandardKey(15)
	QKeySequence__ZoomIn                   QKeySequence__StandardKey = QKeySequence__StandardKey(16)
	QKeySequence__ZoomOut                  QKeySequence__StandardKey = QKeySequence__StandardKey(17)
	QKeySequence__Print                    QKeySequence__StandardKey = QKeySequence__StandardKey(18)
	QKeySequence__AddTab                   QKeySequence__StandardKey = QKeySequence__StandardKey(19)
	QKeySequence__NextChild                QKeySequence__StandardKey = QKeySequence__StandardKey(20)
	QKeySequence__PreviousChild            QKeySequence__StandardKey = QKeySequence__StandardKey(21)
	QKeySequence__Find                     QKeySequence__StandardKey = QKeySequence__StandardKey(22)
	QKeySequence__FindNext                 QKeySequence__StandardKey = QKeySequence__StandardKey(23)
	QKeySequence__FindPrevious             QKeySequence__StandardKey = QKeySequence__StandardKey(24)
	QKeySequence__Replace                  QKeySequence__StandardKey = QKeySequence__StandardKey(25)
	QKeySequence__SelectAll                QKeySequence__StandardKey = QKeySequence__StandardKey(26)
	QKeySequence__Bold                     QKeySequence__StandardKey = QKeySequence__StandardKey(27)
	QKeySequence__Italic                   QKeySequence__StandardKey = QKeySequence__StandardKey(28)
	QKeySequence__Underline                QKeySequence__StandardKey = QKeySequence__StandardKey(29)
	QKeySequence__MoveToNextChar           QKeySequence__StandardKey = QKeySequence__StandardKey(30)
	QKeySequence__MoveToPreviousChar       QKeySequence__StandardKey = QKeySequence__StandardKey(31)
	QKeySequence__MoveToNextWord           QKeySequence__StandardKey = QKeySequence__StandardKey(32)
	QKeySequence__MoveToPreviousWord       QKeySequence__StandardKey = QKeySequence__StandardKey(33)
	QKeySequence__MoveToNextLine           QKeySequence__StandardKey = QKeySequence__StandardKey(34)
	QKeySequence__MoveToPreviousLine       QKeySequence__StandardKey = QKeySequence__StandardKey(35)
	QKeySequence__MoveToNextPage           QKeySequence__StandardKey = QKeySequence__StandardKey(36)
	QKeySequence__MoveToPreviousPage       QKeySequence__StandardKey = QKeySequence__StandardKey(37)
	QKeySequence__MoveToStartOfLine        QKeySequence__StandardKey = QKeySequence__StandardKey(38)
	QKeySequence__MoveToEndOfLine          QKeySequence__StandardKey = QKeySequence__StandardKey(39)
	QKeySequence__MoveToStartOfBlock       QKeySequence__StandardKey = QKeySequence__StandardKey(40)
	QKeySequence__MoveToEndOfBlock         QKeySequence__StandardKey = QKeySequence__StandardKey(41)
	QKeySequence__MoveToStartOfDocument    QKeySequence__StandardKey = QKeySequence__StandardKey(42)
	QKeySequence__MoveToEndOfDocument      QKeySequence__StandardKey = QKeySequence__StandardKey(43)
	QKeySequence__SelectNextChar           QKeySequence__StandardKey = QKeySequence__StandardKey(44)
	QKeySequence__SelectPreviousChar       QKeySequence__StandardKey = QKeySequence__StandardKey(45)
	QKeySequence__SelectNextWord           QKeySequence__StandardKey = QKeySequence__StandardKey(46)
	QKeySequence__SelectPreviousWord       QKeySequence__StandardKey = QKeySequence__StandardKey(47)
	QKeySequence__SelectNextLine           QKeySequence__StandardKey = QKeySequence__StandardKey(48)
	QKeySequence__SelectPreviousLine       QKeySequence__StandardKey = QKeySequence__StandardKey(49)
	QKeySequence__SelectNextPage           QKeySequence__StandardKey = QKeySequence__StandardKey(50)
	QKeySequence__SelectPreviousPage       QKeySequence__StandardKey = QKeySequence__StandardKey(51)
	QKeySequence__SelectStartOfLine        QKeySequence__StandardKey = QKeySequence__StandardKey(52)
	QKeySequence__SelectEndOfLine          QKeySequence__StandardKey = QKeySequence__StandardKey(53)
	QKeySequence__SelectStartOfBlock       QKeySequence__StandardKey = QKeySequence__StandardKey(54)
	QKeySequence__SelectEndOfBlock         QKeySequence__StandardKey = QKeySequence__StandardKey(55)
	QKeySequence__SelectStartOfDocument    QKeySequence__StandardKey = QKeySequence__StandardKey(56)
	QKeySequence__SelectEndOfDocument      QKeySequence__StandardKey = QKeySequence__StandardKey(57)
	QKeySequence__DeleteStartOfWord        QKeySequence__StandardKey = QKeySequence__StandardKey(58)
	QKeySequence__DeleteEndOfWord          QKeySequence__StandardKey = QKeySequence__StandardKey(59)
	QKeySequence__DeleteEndOfLine          QKeySequence__StandardKey = QKeySequence__StandardKey(60)
	QKeySequence__InsertParagraphSeparator QKeySequence__StandardKey = QKeySequence__StandardKey(61)
	QKeySequence__InsertLineSeparator      QKeySequence__StandardKey = QKeySequence__StandardKey(62)
	QKeySequence__SaveAs                   QKeySequence__StandardKey = QKeySequence__StandardKey(63)
	QKeySequence__Preferences              QKeySequence__StandardKey = QKeySequence__StandardKey(64)
	QKeySequence__Quit                     QKeySequence__StandardKey = QKeySequence__StandardKey(65)
	QKeySequence__FullScreen               QKeySequence__StandardKey = QKeySequence__StandardKey(66)
	QKeySequence__Deselect                 QKeySequence__StandardKey = QKeySequence__StandardKey(67)
	QKeySequence__DeleteCompleteLine       QKeySequence__StandardKey = QKeySequence__StandardKey(68)
	QKeySequence__Backspace                QKeySequence__StandardKey = QKeySequence__StandardKey(69)
	QKeySequence__Cancel                   QKeySequence__StandardKey = QKeySequence__StandardKey(70)
)

//go:generate stringer -type=QKeySequence__SequenceFormat
//QKeySequence::SequenceFormat
type QKeySequence__SequenceFormat int64

const (
	QKeySequence__NativeText   QKeySequence__SequenceFormat = QKeySequence__SequenceFormat(0)
	QKeySequence__PortableText QKeySequence__SequenceFormat = QKeySequence__SequenceFormat(1)
)

//go:generate stringer -type=QKeySequence__SequenceMatch
//QKeySequence::SequenceMatch
type QKeySequence__SequenceMatch int64

const (
	QKeySequence__NoMatch      QKeySequence__SequenceMatch = QKeySequence__SequenceMatch(0)
	QKeySequence__PartialMatch QKeySequence__SequenceMatch = QKeySequence__SequenceMatch(1)
	QKeySequence__ExactMatch   QKeySequence__SequenceMatch = QKeySequence__SequenceMatch(2)
)

func NewQKeySequence() *QKeySequence {
	tmpValue := NewQKeySequenceFromPointer(C.QKeySequence_NewQKeySequence())
	qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
	return tmpValue
}

func NewQKeySequence2(key string, format QKeySequence__SequenceFormat) *QKeySequence {
	var keyC *C.char
	if key != "" {
		keyC = C.CString(key)
		defer C.free(unsafe.Pointer(keyC))
	}
	tmpValue := NewQKeySequenceFromPointer(C.QKeySequence_NewQKeySequence2(C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, C.longlong(format)))
	qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
	return tmpValue
}

func NewQKeySequence3(k1 int, k2 int, k3 int, k4 int) *QKeySequence {
	tmpValue := NewQKeySequenceFromPointer(C.QKeySequence_NewQKeySequence3(C.int(int32(k1)), C.int(int32(k2)), C.int(int32(k3)), C.int(int32(k4))))
	qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
	return tmpValue
}

func NewQKeySequence4(keysequence QKeySequence_ITF) *QKeySequence {
	tmpValue := NewQKeySequenceFromPointer(C.QKeySequence_NewQKeySequence4(PointerFromQKeySequence(keysequence)))
	qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
	return tmpValue
}

func NewQKeySequence5(key QKeySequence__StandardKey) *QKeySequence {
	tmpValue := NewQKeySequenceFromPointer(C.QKeySequence_NewQKeySequence5(C.longlong(key)))
	qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
	return tmpValue
}

func (ptr *QKeySequence) Count() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QKeySequence_Count(ptr.Pointer())))
	}
	return 0
}

func QKeySequence_FromString(str string, format QKeySequence__SequenceFormat) *QKeySequence {
	var strC *C.char
	if str != "" {
		strC = C.CString(str)
		defer C.free(unsafe.Pointer(strC))
	}
	tmpValue := NewQKeySequenceFromPointer(C.QKeySequence_QKeySequence_FromString(C.struct_QtGui_PackedString{data: strC, len: C.longlong(len(str))}, C.longlong(format)))
	qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
	return tmpValue
}

func (ptr *QKeySequence) FromString(str string, format QKeySequence__SequenceFormat) *QKeySequence {
	var strC *C.char
	if str != "" {
		strC = C.CString(str)
		defer C.free(unsafe.Pointer(strC))
	}
	tmpValue := NewQKeySequenceFromPointer(C.QKeySequence_QKeySequence_FromString(C.struct_QtGui_PackedString{data: strC, len: C.longlong(len(str))}, C.longlong(format)))
	qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
	return tmpValue
}

func (ptr *QKeySequence) IsEmpty() bool {
	if ptr.Pointer() != nil {
		return int8(C.QKeySequence_IsEmpty(ptr.Pointer())) != 0
	}
	return false
}

func QKeySequence_KeyBindings(key QKeySequence__StandardKey) []*QKeySequence {
	return func(l C.struct_QtGui_PackedList) []*QKeySequence {
		out := make([]*QKeySequence, int(l.len))
		tmpList := NewQKeySequenceFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__keyBindings_atList(i)
		}
		return out
	}(C.QKeySequence_QKeySequence_KeyBindings(C.longlong(key)))
}

func (ptr *QKeySequence) KeyBindings(key QKeySequence__StandardKey) []*QKeySequence {
	return func(l C.struct_QtGui_PackedList) []*QKeySequence {
		out := make([]*QKeySequence, int(l.len))
		tmpList := NewQKeySequenceFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__keyBindings_atList(i)
		}
		return out
	}(C.QKeySequence_QKeySequence_KeyBindings(C.longlong(key)))
}

func QKeySequence_ListFromString(str string, format QKeySequence__SequenceFormat) []*QKeySequence {
	var strC *C.char
	if str != "" {
		strC = C.CString(str)
		defer C.free(unsafe.Pointer(strC))
	}
	return func(l C.struct_QtGui_PackedList) []*QKeySequence {
		out := make([]*QKeySequence, int(l.len))
		tmpList := NewQKeySequenceFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__listFromString_atList(i)
		}
		return out
	}(C.QKeySequence_QKeySequence_ListFromString(C.struct_QtGui_PackedString{data: strC, len: C.longlong(len(str))}, C.longlong(format)))
}

func (ptr *QKeySequence) ListFromString(str string, format QKeySequence__SequenceFormat) []*QKeySequence {
	var strC *C.char
	if str != "" {
		strC = C.CString(str)
		defer C.free(unsafe.Pointer(strC))
	}
	return func(l C.struct_QtGui_PackedList) []*QKeySequence {
		out := make([]*QKeySequence, int(l.len))
		tmpList := NewQKeySequenceFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__listFromString_atList(i)
		}
		return out
	}(C.QKeySequence_QKeySequence_ListFromString(C.struct_QtGui_PackedString{data: strC, len: C.longlong(len(str))}, C.longlong(format)))
}

func QKeySequence_ListToString(list []*QKeySequence, format QKeySequence__SequenceFormat) string {
	return cGoUnpackString(C.QKeySequence_QKeySequence_ListToString(func() unsafe.Pointer {
		tmpList := NewQKeySequenceFromPointer(NewQKeySequenceFromPointer(nil).__listToString_list_newList())
		for _, v := range list {
			tmpList.__listToString_list_setList(v)
		}
		return tmpList.Pointer()
	}(), C.longlong(format)))
}

func (ptr *QKeySequence) ListToString(list []*QKeySequence, format QKeySequence__SequenceFormat) string {
	return cGoUnpackString(C.QKeySequence_QKeySequence_ListToString(func() unsafe.Pointer {
		tmpList := NewQKeySequenceFromPointer(NewQKeySequenceFromPointer(nil).__listToString_list_newList())
		for _, v := range list {
			tmpList.__listToString_list_setList(v)
		}
		return tmpList.Pointer()
	}(), C.longlong(format)))
}

func (ptr *QKeySequence) Matches(seq QKeySequence_ITF) QKeySequence__SequenceMatch {
	if ptr.Pointer() != nil {
		return QKeySequence__SequenceMatch(C.QKeySequence_Matches(ptr.Pointer(), PointerFromQKeySequence(seq)))
	}
	return 0
}

func QKeySequence_Mnemonic(text string) *QKeySequence {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQKeySequenceFromPointer(C.QKeySequence_QKeySequence_Mnemonic(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
	qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
	return tmpValue
}

func (ptr *QKeySequence) Mnemonic(text string) *QKeySequence {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQKeySequenceFromPointer(C.QKeySequence_QKeySequence_Mnemonic(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
	qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
	return tmpValue
}

func (ptr *QKeySequence) Swap(other QKeySequence_ITF) {
	if ptr.Pointer() != nil {
		C.QKeySequence_Swap(ptr.Pointer(), PointerFromQKeySequence(other))
	}
}

func (ptr *QKeySequence) ToString(format QKeySequence__SequenceFormat) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QKeySequence_ToString(ptr.Pointer(), C.longlong(format)))
	}
	return ""
}

func (ptr *QKeySequence) DestroyQKeySequence() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QKeySequence_DestroyQKeySequence(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QKeySequence) __keyBindings_atList(i int) *QKeySequence {
	if ptr.Pointer() != nil {
		tmpValue := NewQKeySequenceFromPointer(C.QKeySequence___keyBindings_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
		return tmpValue
	}
	return nil
}

func (ptr *QKeySequence) __keyBindings_setList(i QKeySequence_ITF) {
	if ptr.Pointer() != nil {
		C.QKeySequence___keyBindings_setList(ptr.Pointer(), PointerFromQKeySequence(i))
	}
}

func (ptr *QKeySequence) __keyBindings_newList() unsafe.Pointer {
	return C.QKeySequence___keyBindings_newList(ptr.Pointer())
}

func (ptr *QKeySequence) __listFromString_atList(i int) *QKeySequence {
	if ptr.Pointer() != nil {
		tmpValue := NewQKeySequenceFromPointer(C.QKeySequence___listFromString_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
		return tmpValue
	}
	return nil
}

func (ptr *QKeySequence) __listFromString_setList(i QKeySequence_ITF) {
	if ptr.Pointer() != nil {
		C.QKeySequence___listFromString_setList(ptr.Pointer(), PointerFromQKeySequence(i))
	}
}

func (ptr *QKeySequence) __listFromString_newList() unsafe.Pointer {
	return C.QKeySequence___listFromString_newList(ptr.Pointer())
}

func (ptr *QKeySequence) __listToString_list_atList(i int) *QKeySequence {
	if ptr.Pointer() != nil {
		tmpValue := NewQKeySequenceFromPointer(C.QKeySequence___listToString_list_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QKeySequence).DestroyQKeySequence)
		return tmpValue
	}
	return nil
}

func (ptr *QKeySequence) __listToString_list_setList(i QKeySequence_ITF) {
	if ptr.Pointer() != nil {
		C.QKeySequence___listToString_list_setList(ptr.Pointer(), PointerFromQKeySequence(i))
	}
}

func (ptr *QKeySequence) __listToString_list_newList() unsafe.Pointer {
	return C.QKeySequence___listToString_list_newList(ptr.Pointer())
}

type QLinearGradient struct {
	QGradient
}

type QLinearGradient_ITF interface {
	QGradient_ITF
	QLinearGradient_PTR() *QLinearGradient
}

func (ptr *QLinearGradient) QLinearGradient_PTR() *QLinearGradient {
	return ptr
}

func (ptr *QLinearGradient) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QGradient_PTR().Pointer()
	}
	return nil
}

func (ptr *QLinearGradient) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QGradient_PTR().SetPointer(p)
	}
}

func PointerFromQLinearGradient(ptr QLinearGradient_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QLinearGradient_PTR().Pointer()
	}
	return nil
}

func NewQLinearGradientFromPointer(ptr unsafe.Pointer) (n *QLinearGradient) {
	n = new(QLinearGradient)
	n.SetPointer(ptr)
	return
}
func (ptr *QLinearGradient) DestroyQLinearGradient() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQLinearGradient() *QLinearGradient {
	tmpValue := NewQLinearGradientFromPointer(C.QLinearGradient_NewQLinearGradient())
	qt.SetFinalizer(tmpValue, (*QLinearGradient).DestroyQLinearGradient)
	return tmpValue
}

func NewQLinearGradient2(start core.QPointF_ITF, finalStop core.QPointF_ITF) *QLinearGradient {
	tmpValue := NewQLinearGradientFromPointer(C.QLinearGradient_NewQLinearGradient2(core.PointerFromQPointF(start), core.PointerFromQPointF(finalStop)))
	qt.SetFinalizer(tmpValue, (*QLinearGradient).DestroyQLinearGradient)
	return tmpValue
}

func NewQLinearGradient3(x1 float64, y1 float64, x2 float64, y2 float64) *QLinearGradient {
	tmpValue := NewQLinearGradientFromPointer(C.QLinearGradient_NewQLinearGradient3(C.double(x1), C.double(y1), C.double(x2), C.double(y2)))
	qt.SetFinalizer(tmpValue, (*QLinearGradient).DestroyQLinearGradient)
	return tmpValue
}

func (ptr *QLinearGradient) FinalStop() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QLinearGradient_FinalStop(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QLinearGradient) SetFinalStop(stop core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QLinearGradient_SetFinalStop(ptr.Pointer(), core.PointerFromQPointF(stop))
	}
}

func (ptr *QLinearGradient) SetFinalStop2(x float64, y float64) {
	if ptr.Pointer() != nil {
		C.QLinearGradient_SetFinalStop2(ptr.Pointer(), C.double(x), C.double(y))
	}
}

func (ptr *QLinearGradient) SetStart(start core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QLinearGradient_SetStart(ptr.Pointer(), core.PointerFromQPointF(start))
	}
}

func (ptr *QLinearGradient) SetStart2(x float64, y float64) {
	if ptr.Pointer() != nil {
		C.QLinearGradient_SetStart2(ptr.Pointer(), C.double(x), C.double(y))
	}
}

func (ptr *QLinearGradient) Start() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QLinearGradient_Start(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

type QMatrix struct {
	ptr unsafe.Pointer
}

type QMatrix_ITF interface {
	QMatrix_PTR() *QMatrix
}

func (ptr *QMatrix) QMatrix_PTR() *QMatrix {
	return ptr
}

func (ptr *QMatrix) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QMatrix) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQMatrix(ptr QMatrix_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QMatrix_PTR().Pointer()
	}
	return nil
}

func NewQMatrixFromPointer(ptr unsafe.Pointer) (n *QMatrix) {
	n = new(QMatrix)
	n.SetPointer(ptr)
	return
}
func (ptr *QMatrix) DestroyQMatrix() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQMatrix2() *QMatrix {
	tmpValue := NewQMatrixFromPointer(C.QMatrix_NewQMatrix2())
	qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
	return tmpValue
}

func NewQMatrix3(m11 float64, m12 float64, m21 float64, m22 float64, dx float64, dy float64) *QMatrix {
	tmpValue := NewQMatrixFromPointer(C.QMatrix_NewQMatrix3(C.double(m11), C.double(m12), C.double(m21), C.double(m22), C.double(dx), C.double(dy)))
	qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
	return tmpValue
}

func NewQMatrix5(matrix QMatrix_ITF) *QMatrix {
	tmpValue := NewQMatrixFromPointer(C.QMatrix_NewQMatrix5(PointerFromQMatrix(matrix)))
	qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
	return tmpValue
}

func (ptr *QMatrix) Determinant() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QMatrix_Determinant(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix) Dx() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QMatrix_Dx(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix) Dy() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QMatrix_Dy(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix) Inverted(invertible *bool) *QMatrix {
	if ptr.Pointer() != nil {
		var invertibleC C.char
		if invertible != nil {
			invertibleC = C.char(int8(qt.GoBoolToInt(*invertible)))
			defer func() { *invertible = int8(invertibleC) != 0 }()
		}
		tmpValue := NewQMatrixFromPointer(C.QMatrix_Inverted(ptr.Pointer(), &invertibleC))
		qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) IsIdentity() bool {
	if ptr.Pointer() != nil {
		return int8(C.QMatrix_IsIdentity(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QMatrix) IsInvertible() bool {
	if ptr.Pointer() != nil {
		return int8(C.QMatrix_IsInvertible(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QMatrix) M11() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QMatrix_M11(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix) M12() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QMatrix_M12(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix) M21() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QMatrix_M21(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix) M22() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QMatrix_M22(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix) Map(x float64, y float64, tx float64, ty float64) {
	if ptr.Pointer() != nil {
		C.QMatrix_Map(ptr.Pointer(), C.double(x), C.double(y), C.double(tx), C.double(ty))
	}
}

func (ptr *QMatrix) Map2(x int, y int, tx int, ty int) {
	if ptr.Pointer() != nil {
		C.QMatrix_Map2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(tx)), C.int(int32(ty)))
	}
}

func (ptr *QMatrix) Map3(point core.QPoint_ITF) *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QMatrix_Map3(ptr.Pointer(), core.PointerFromQPoint(point)))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) Map4(point core.QPointF_ITF) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QMatrix_Map4(ptr.Pointer(), core.PointerFromQPointF(point)))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) Map5(line core.QLine_ITF) *core.QLine {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQLineFromPointer(C.QMatrix_Map5(ptr.Pointer(), core.PointerFromQLine(line)))
		qt.SetFinalizer(tmpValue, (*core.QLine).DestroyQLine)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) Map6(line core.QLineF_ITF) *core.QLineF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQLineFFromPointer(C.QMatrix_Map6(ptr.Pointer(), core.PointerFromQLineF(line)))
		qt.SetFinalizer(tmpValue, (*core.QLineF).DestroyQLineF)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) Map7(polygon QPolygonF_ITF) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QMatrix_Map7(ptr.Pointer(), PointerFromQPolygonF(polygon)))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) Map8(polygon QPolygon_ITF) *QPolygon {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFromPointer(C.QMatrix_Map8(ptr.Pointer(), PointerFromQPolygon(polygon)))
		qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) Map9(region QRegion_ITF) *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QMatrix_Map9(ptr.Pointer(), PointerFromQRegion(region)))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) Map10(path QPainterPath_ITF) *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QMatrix_Map10(ptr.Pointer(), PointerFromQPainterPath(path)))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) MapRect(rectangle core.QRectF_ITF) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QMatrix_MapRect(ptr.Pointer(), core.PointerFromQRectF(rectangle)))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) MapRect2(rectangle core.QRect_ITF) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QMatrix_MapRect2(ptr.Pointer(), core.PointerFromQRect(rectangle)))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) MapToPolygon(rectangle core.QRect_ITF) *QPolygon {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFromPointer(C.QMatrix_MapToPolygon(ptr.Pointer(), core.PointerFromQRect(rectangle)))
		qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) Reset() {
	if ptr.Pointer() != nil {
		C.QMatrix_Reset(ptr.Pointer())
	}
}

func (ptr *QMatrix) Rotate(degrees float64) *QMatrix {
	if ptr.Pointer() != nil {
		tmpValue := NewQMatrixFromPointer(C.QMatrix_Rotate(ptr.Pointer(), C.double(degrees)))
		qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) Scale(sx float64, sy float64) *QMatrix {
	if ptr.Pointer() != nil {
		tmpValue := NewQMatrixFromPointer(C.QMatrix_Scale(ptr.Pointer(), C.double(sx), C.double(sy)))
		qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) SetMatrix(m11 float64, m12 float64, m21 float64, m22 float64, dx float64, dy float64) {
	if ptr.Pointer() != nil {
		C.QMatrix_SetMatrix(ptr.Pointer(), C.double(m11), C.double(m12), C.double(m21), C.double(m22), C.double(dx), C.double(dy))
	}
}

func (ptr *QMatrix) Shear(sh float64, sv float64) *QMatrix {
	if ptr.Pointer() != nil {
		tmpValue := NewQMatrixFromPointer(C.QMatrix_Shear(ptr.Pointer(), C.double(sh), C.double(sv)))
		qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix) Translate(dx float64, dy float64) *QMatrix {
	if ptr.Pointer() != nil {
		tmpValue := NewQMatrixFromPointer(C.QMatrix_Translate(ptr.Pointer(), C.double(dx), C.double(dy)))
		qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
		return tmpValue
	}
	return nil
}

type QMatrix4x4 struct {
	ptr unsafe.Pointer
}

type QMatrix4x4_ITF interface {
	QMatrix4x4_PTR() *QMatrix4x4
}

func (ptr *QMatrix4x4) QMatrix4x4_PTR() *QMatrix4x4 {
	return ptr
}

func (ptr *QMatrix4x4) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QMatrix4x4) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQMatrix4x4(ptr QMatrix4x4_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QMatrix4x4_PTR().Pointer()
	}
	return nil
}

func NewQMatrix4x4FromPointer(ptr unsafe.Pointer) (n *QMatrix4x4) {
	n = new(QMatrix4x4)
	n.SetPointer(ptr)
	return
}
func (ptr *QMatrix4x4) DestroyQMatrix4x4() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQMatrix4x4() *QMatrix4x4 {
	tmpValue := NewQMatrix4x4FromPointer(C.QMatrix4x4_NewQMatrix4x4())
	qt.SetFinalizer(tmpValue, (*QMatrix4x4).DestroyQMatrix4x4)
	return tmpValue
}

func NewQMatrix4x43(values float32) *QMatrix4x4 {
	tmpValue := NewQMatrix4x4FromPointer(C.QMatrix4x4_NewQMatrix4x43(C.float(values)))
	qt.SetFinalizer(tmpValue, (*QMatrix4x4).DestroyQMatrix4x4)
	return tmpValue
}

func NewQMatrix4x44(m11 float32, m12 float32, m13 float32, m14 float32, m21 float32, m22 float32, m23 float32, m24 float32, m31 float32, m32 float32, m33 float32, m34 float32, m41 float32, m42 float32, m43 float32, m44 float32) *QMatrix4x4 {
	tmpValue := NewQMatrix4x4FromPointer(C.QMatrix4x4_NewQMatrix4x44(C.float(m11), C.float(m12), C.float(m13), C.float(m14), C.float(m21), C.float(m22), C.float(m23), C.float(m24), C.float(m31), C.float(m32), C.float(m33), C.float(m34), C.float(m41), C.float(m42), C.float(m43), C.float(m44)))
	qt.SetFinalizer(tmpValue, (*QMatrix4x4).DestroyQMatrix4x4)
	return tmpValue
}

func NewQMatrix4x46(matrix QMatrix_ITF) *QMatrix4x4 {
	tmpValue := NewQMatrix4x4FromPointer(C.QMatrix4x4_NewQMatrix4x46(PointerFromQMatrix(matrix)))
	qt.SetFinalizer(tmpValue, (*QMatrix4x4).DestroyQMatrix4x4)
	return tmpValue
}

func (ptr *QMatrix4x4) Column(index int) *QVector4D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector4DFromPointer(C.QMatrix4x4_Column(ptr.Pointer(), C.int(int32(index))))
		qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) ConstData() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QMatrix4x4_ConstData(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix4x4) CopyDataTo(values float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_CopyDataTo(ptr.Pointer(), C.float(values))
	}
}

func (ptr *QMatrix4x4) Data() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QMatrix4x4_Data(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix4x4) Data2() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QMatrix4x4_Data2(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix4x4) Determinant() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QMatrix4x4_Determinant(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMatrix4x4) Fill(value float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Fill(ptr.Pointer(), C.float(value))
	}
}

func (ptr *QMatrix4x4) Frustum(left float32, right float32, bottom float32, top float32, nearPlane float32, farPlane float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Frustum(ptr.Pointer(), C.float(left), C.float(right), C.float(bottom), C.float(top), C.float(nearPlane), C.float(farPlane))
	}
}

func (ptr *QMatrix4x4) Inverted(invertible *bool) *QMatrix4x4 {
	if ptr.Pointer() != nil {
		var invertibleC C.char
		if invertible != nil {
			invertibleC = C.char(int8(qt.GoBoolToInt(*invertible)))
			defer func() { *invertible = int8(invertibleC) != 0 }()
		}
		tmpValue := NewQMatrix4x4FromPointer(C.QMatrix4x4_Inverted(ptr.Pointer(), &invertibleC))
		qt.SetFinalizer(tmpValue, (*QMatrix4x4).DestroyQMatrix4x4)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) IsAffine() bool {
	if ptr.Pointer() != nil {
		return int8(C.QMatrix4x4_IsAffine(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QMatrix4x4) IsIdentity() bool {
	if ptr.Pointer() != nil {
		return int8(C.QMatrix4x4_IsIdentity(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QMatrix4x4) LookAt(eye QVector3D_ITF, center QVector3D_ITF, up QVector3D_ITF) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_LookAt(ptr.Pointer(), PointerFromQVector3D(eye), PointerFromQVector3D(center), PointerFromQVector3D(up))
	}
}

func (ptr *QMatrix4x4) Map(point core.QPoint_ITF) *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QMatrix4x4_Map(ptr.Pointer(), core.PointerFromQPoint(point)))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) Map2(point core.QPointF_ITF) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QMatrix4x4_Map2(ptr.Pointer(), core.PointerFromQPointF(point)))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) Map3(point QVector3D_ITF) *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QMatrix4x4_Map3(ptr.Pointer(), PointerFromQVector3D(point)))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) Map4(point QVector4D_ITF) *QVector4D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector4DFromPointer(C.QMatrix4x4_Map4(ptr.Pointer(), PointerFromQVector4D(point)))
		qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) MapRect(rect core.QRect_ITF) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QMatrix4x4_MapRect(ptr.Pointer(), core.PointerFromQRect(rect)))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) MapRect2(rect core.QRectF_ITF) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QMatrix4x4_MapRect2(ptr.Pointer(), core.PointerFromQRectF(rect)))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) MapVector(vector QVector3D_ITF) *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QMatrix4x4_MapVector(ptr.Pointer(), PointerFromQVector3D(vector)))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) Optimize() {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Optimize(ptr.Pointer())
	}
}

func (ptr *QMatrix4x4) Ortho(left float32, right float32, bottom float32, top float32, nearPlane float32, farPlane float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Ortho(ptr.Pointer(), C.float(left), C.float(right), C.float(bottom), C.float(top), C.float(nearPlane), C.float(farPlane))
	}
}

func (ptr *QMatrix4x4) Ortho2(rect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Ortho2(ptr.Pointer(), core.PointerFromQRect(rect))
	}
}

func (ptr *QMatrix4x4) Ortho3(rect core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Ortho3(ptr.Pointer(), core.PointerFromQRectF(rect))
	}
}

func (ptr *QMatrix4x4) Perspective(verticalAngle float32, aspectRatio float32, nearPlane float32, farPlane float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Perspective(ptr.Pointer(), C.float(verticalAngle), C.float(aspectRatio), C.float(nearPlane), C.float(farPlane))
	}
}

func (ptr *QMatrix4x4) Rotate(angle float32, vector QVector3D_ITF) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Rotate(ptr.Pointer(), C.float(angle), PointerFromQVector3D(vector))
	}
}

func (ptr *QMatrix4x4) Rotate2(angle float32, x float32, y float32, z float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Rotate2(ptr.Pointer(), C.float(angle), C.float(x), C.float(y), C.float(z))
	}
}

func (ptr *QMatrix4x4) Rotate3(quaternion QQuaternion_ITF) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Rotate3(ptr.Pointer(), PointerFromQQuaternion(quaternion))
	}
}

func (ptr *QMatrix4x4) Row(index int) *QVector4D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector4DFromPointer(C.QMatrix4x4_Row(ptr.Pointer(), C.int(int32(index))))
		qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) Scale(vector QVector3D_ITF) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Scale(ptr.Pointer(), PointerFromQVector3D(vector))
	}
}

func (ptr *QMatrix4x4) Scale2(x float32, y float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Scale2(ptr.Pointer(), C.float(x), C.float(y))
	}
}

func (ptr *QMatrix4x4) Scale3(x float32, y float32, z float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Scale3(ptr.Pointer(), C.float(x), C.float(y), C.float(z))
	}
}

func (ptr *QMatrix4x4) Scale4(factor float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Scale4(ptr.Pointer(), C.float(factor))
	}
}

func (ptr *QMatrix4x4) SetColumn(index int, value QVector4D_ITF) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_SetColumn(ptr.Pointer(), C.int(int32(index)), PointerFromQVector4D(value))
	}
}

func (ptr *QMatrix4x4) SetRow(index int, value QVector4D_ITF) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_SetRow(ptr.Pointer(), C.int(int32(index)), PointerFromQVector4D(value))
	}
}

func (ptr *QMatrix4x4) SetToIdentity() {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_SetToIdentity(ptr.Pointer())
	}
}

func (ptr *QMatrix4x4) ToAffine() *QMatrix {
	if ptr.Pointer() != nil {
		tmpValue := NewQMatrixFromPointer(C.QMatrix4x4_ToAffine(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) ToTransform() *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QMatrix4x4_ToTransform(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) ToTransform2(distanceToPlane float32) *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QMatrix4x4_ToTransform2(ptr.Pointer(), C.float(distanceToPlane)))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) Translate(vector QVector3D_ITF) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Translate(ptr.Pointer(), PointerFromQVector3D(vector))
	}
}

func (ptr *QMatrix4x4) Translate2(x float32, y float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Translate2(ptr.Pointer(), C.float(x), C.float(y))
	}
}

func (ptr *QMatrix4x4) Translate3(x float32, y float32, z float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Translate3(ptr.Pointer(), C.float(x), C.float(y), C.float(z))
	}
}

func (ptr *QMatrix4x4) Transposed() *QMatrix4x4 {
	if ptr.Pointer() != nil {
		tmpValue := NewQMatrix4x4FromPointer(C.QMatrix4x4_Transposed(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QMatrix4x4).DestroyQMatrix4x4)
		return tmpValue
	}
	return nil
}

func (ptr *QMatrix4x4) Viewport(left float32, bottom float32, width float32, height float32, nearPlane float32, farPlane float32) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Viewport(ptr.Pointer(), C.float(left), C.float(bottom), C.float(width), C.float(height), C.float(nearPlane), C.float(farPlane))
	}
}

func (ptr *QMatrix4x4) Viewport2(rect core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QMatrix4x4_Viewport2(ptr.Pointer(), core.PointerFromQRectF(rect))
	}
}

type QMessageDialogOptions struct {
	ptr unsafe.Pointer
}

type QMessageDialogOptions_ITF interface {
	QMessageDialogOptions_PTR() *QMessageDialogOptions
}

func (ptr *QMessageDialogOptions) QMessageDialogOptions_PTR() *QMessageDialogOptions {
	return ptr
}

func (ptr *QMessageDialogOptions) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QMessageDialogOptions) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQMessageDialogOptions(ptr QMessageDialogOptions_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QMessageDialogOptions_PTR().Pointer()
	}
	return nil
}

func NewQMessageDialogOptionsFromPointer(ptr unsafe.Pointer) (n *QMessageDialogOptions) {
	n = new(QMessageDialogOptions)
	n.SetPointer(ptr)
	return
}
func (ptr *QMessageDialogOptions) DestroyQMessageDialogOptions() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QMouseEvent struct {
	QInputEvent
}

type QMouseEvent_ITF interface {
	QInputEvent_ITF
	QMouseEvent_PTR() *QMouseEvent
}

func (ptr *QMouseEvent) QMouseEvent_PTR() *QMouseEvent {
	return ptr
}

func (ptr *QMouseEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QMouseEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QInputEvent_PTR().SetPointer(p)
	}
}

func PointerFromQMouseEvent(ptr QMouseEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QMouseEvent_PTR().Pointer()
	}
	return nil
}

func NewQMouseEventFromPointer(ptr unsafe.Pointer) (n *QMouseEvent) {
	n = new(QMouseEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QMouseEvent) DestroyQMouseEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQMouseEvent(ty core.QEvent__Type, localPos core.QPointF_ITF, button core.Qt__MouseButton, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier) *QMouseEvent {
	tmpValue := NewQMouseEventFromPointer(C.QMouseEvent_NewQMouseEvent(C.longlong(ty), core.PointerFromQPointF(localPos), C.longlong(button), C.longlong(buttons), C.longlong(modifiers)))
	qt.SetFinalizer(tmpValue, (*QMouseEvent).DestroyQMouseEvent)
	return tmpValue
}

func NewQMouseEvent2(ty core.QEvent__Type, localPos core.QPointF_ITF, screenPos core.QPointF_ITF, button core.Qt__MouseButton, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier) *QMouseEvent {
	tmpValue := NewQMouseEventFromPointer(C.QMouseEvent_NewQMouseEvent2(C.longlong(ty), core.PointerFromQPointF(localPos), core.PointerFromQPointF(screenPos), C.longlong(button), C.longlong(buttons), C.longlong(modifiers)))
	qt.SetFinalizer(tmpValue, (*QMouseEvent).DestroyQMouseEvent)
	return tmpValue
}

func NewQMouseEvent3(ty core.QEvent__Type, localPos core.QPointF_ITF, windowPos core.QPointF_ITF, screenPos core.QPointF_ITF, button core.Qt__MouseButton, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier) *QMouseEvent {
	tmpValue := NewQMouseEventFromPointer(C.QMouseEvent_NewQMouseEvent3(C.longlong(ty), core.PointerFromQPointF(localPos), core.PointerFromQPointF(windowPos), core.PointerFromQPointF(screenPos), C.longlong(button), C.longlong(buttons), C.longlong(modifiers)))
	qt.SetFinalizer(tmpValue, (*QMouseEvent).DestroyQMouseEvent)
	return tmpValue
}

func NewQMouseEvent4(ty core.QEvent__Type, localPos core.QPointF_ITF, windowPos core.QPointF_ITF, screenPos core.QPointF_ITF, button core.Qt__MouseButton, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier, source core.Qt__MouseEventSource) *QMouseEvent {
	tmpValue := NewQMouseEventFromPointer(C.QMouseEvent_NewQMouseEvent4(C.longlong(ty), core.PointerFromQPointF(localPos), core.PointerFromQPointF(windowPos), core.PointerFromQPointF(screenPos), C.longlong(button), C.longlong(buttons), C.longlong(modifiers), C.longlong(source)))
	qt.SetFinalizer(tmpValue, (*QMouseEvent).DestroyQMouseEvent)
	return tmpValue
}

func (ptr *QMouseEvent) Button() core.Qt__MouseButton {
	if ptr.Pointer() != nil {
		return core.Qt__MouseButton(C.QMouseEvent_Button(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMouseEvent) Buttons() core.Qt__MouseButton {
	if ptr.Pointer() != nil {
		return core.Qt__MouseButton(C.QMouseEvent_Buttons(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMouseEvent) Flags() core.Qt__MouseEventFlag {
	if ptr.Pointer() != nil {
		return core.Qt__MouseEventFlag(C.QMouseEvent_Flags(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMouseEvent) GlobalPos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QMouseEvent_GlobalPos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QMouseEvent) GlobalX() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QMouseEvent_GlobalX(ptr.Pointer())))
	}
	return 0
}

func (ptr *QMouseEvent) GlobalY() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QMouseEvent_GlobalY(ptr.Pointer())))
	}
	return 0
}

func (ptr *QMouseEvent) LocalPos() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QMouseEvent_LocalPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QMouseEvent) Pos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QMouseEvent_Pos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QMouseEvent) ScreenPos() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QMouseEvent_ScreenPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QMouseEvent) Source() core.Qt__MouseEventSource {
	if ptr.Pointer() != nil {
		return core.Qt__MouseEventSource(C.QMouseEvent_Source(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMouseEvent) WindowPos() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QMouseEvent_WindowPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QMouseEvent) X() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QMouseEvent_X(ptr.Pointer())))
	}
	return 0
}

func (ptr *QMouseEvent) Y() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QMouseEvent_Y(ptr.Pointer())))
	}
	return 0
}

type QMoveEvent struct {
	core.QEvent
}

type QMoveEvent_ITF interface {
	core.QEvent_ITF
	QMoveEvent_PTR() *QMoveEvent
}

func (ptr *QMoveEvent) QMoveEvent_PTR() *QMoveEvent {
	return ptr
}

func (ptr *QMoveEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QMoveEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQMoveEvent(ptr QMoveEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QMoveEvent_PTR().Pointer()
	}
	return nil
}

func NewQMoveEventFromPointer(ptr unsafe.Pointer) (n *QMoveEvent) {
	n = new(QMoveEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QMoveEvent) DestroyQMoveEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQMoveEvent(pos core.QPoint_ITF, oldPos core.QPoint_ITF) *QMoveEvent {
	tmpValue := NewQMoveEventFromPointer(C.QMoveEvent_NewQMoveEvent(core.PointerFromQPoint(pos), core.PointerFromQPoint(oldPos)))
	qt.SetFinalizer(tmpValue, (*QMoveEvent).DestroyQMoveEvent)
	return tmpValue
}

func (ptr *QMoveEvent) OldPos() *core.QPoint {
	if ptr.Pointer() != nil {
		return core.NewQPointFromPointer(C.QMoveEvent_OldPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QMoveEvent) Pos() *core.QPoint {
	if ptr.Pointer() != nil {
		return core.NewQPointFromPointer(C.QMoveEvent_Pos(ptr.Pointer()))
	}
	return nil
}

type QMovie struct {
	core.QObject
}

type QMovie_ITF interface {
	core.QObject_ITF
	QMovie_PTR() *QMovie
}

func (ptr *QMovie) QMovie_PTR() *QMovie {
	return ptr
}

func (ptr *QMovie) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QMovie) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQMovie(ptr QMovie_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QMovie_PTR().Pointer()
	}
	return nil
}

func NewQMovieFromPointer(ptr unsafe.Pointer) (n *QMovie) {
	n = new(QMovie)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QMovie__MovieState
//QMovie::MovieState
type QMovie__MovieState int64

const (
	QMovie__NotRunning QMovie__MovieState = QMovie__MovieState(0)
	QMovie__Paused     QMovie__MovieState = QMovie__MovieState(1)
	QMovie__Running    QMovie__MovieState = QMovie__MovieState(2)
)

//go:generate stringer -type=QMovie__CacheMode
//QMovie::CacheMode
type QMovie__CacheMode int64

const (
	QMovie__CacheNone QMovie__CacheMode = QMovie__CacheMode(0)
	QMovie__CacheAll  QMovie__CacheMode = QMovie__CacheMode(1)
)

func NewQMovie(parent core.QObject_ITF) *QMovie {
	tmpValue := NewQMovieFromPointer(C.QMovie_NewQMovie(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQMovie2(device core.QIODevice_ITF, format core.QByteArray_ITF, parent core.QObject_ITF) *QMovie {
	tmpValue := NewQMovieFromPointer(C.QMovie_NewQMovie2(core.PointerFromQIODevice(device), core.PointerFromQByteArray(format), core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQMovie3(fileName string, format core.QByteArray_ITF, parent core.QObject_ITF) *QMovie {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	tmpValue := NewQMovieFromPointer(C.QMovie_NewQMovie3(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, core.PointerFromQByteArray(format), core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QMovie) BackgroundColor() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QMovie_BackgroundColor(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QMovie) CacheMode() QMovie__CacheMode {
	if ptr.Pointer() != nil {
		return QMovie__CacheMode(C.QMovie_CacheMode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMovie) CurrentFrameNumber() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QMovie_CurrentFrameNumber(ptr.Pointer())))
	}
	return 0
}

func (ptr *QMovie) CurrentImage() *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QMovie_CurrentImage(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QMovie) CurrentPixmap() *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QMovie_CurrentPixmap(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QMovie) Device() *core.QIODevice {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQIODeviceFromPointer(C.QMovie_Device(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQMovie_Error
func callbackQMovie_Error(ptr unsafe.Pointer, error C.longlong) {
	if signal := qt.GetSignal(ptr, "error"); signal != nil {
		(*(*func(QImageReader__ImageReaderError))(signal))(QImageReader__ImageReaderError(error))
	}

}

func (ptr *QMovie) ConnectError(f func(error QImageReader__ImageReaderError)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "error") {
			C.QMovie_ConnectError(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "error")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "error"); signal != nil {
			f := func(error QImageReader__ImageReaderError) {
				(*(*func(QImageReader__ImageReaderError))(signal))(error)
				f(error)
			}
			qt.ConnectSignal(ptr.Pointer(), "error", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "error", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectError() {
	if ptr.Pointer() != nil {
		C.QMovie_DisconnectError(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "error")
	}
}

func (ptr *QMovie) Error(error QImageReader__ImageReaderError) {
	if ptr.Pointer() != nil {
		C.QMovie_Error(ptr.Pointer(), C.longlong(error))
	}
}

func (ptr *QMovie) FileName() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QMovie_FileName(ptr.Pointer()))
	}
	return ""
}

//export callbackQMovie_Finished
func callbackQMovie_Finished(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "finished"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QMovie) ConnectFinished(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "finished") {
			C.QMovie_ConnectFinished(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "finished")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "finished"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "finished", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "finished", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectFinished() {
	if ptr.Pointer() != nil {
		C.QMovie_DisconnectFinished(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "finished")
	}
}

func (ptr *QMovie) Finished() {
	if ptr.Pointer() != nil {
		C.QMovie_Finished(ptr.Pointer())
	}
}

func (ptr *QMovie) Format() *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QMovie_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

//export callbackQMovie_FrameChanged
func callbackQMovie_FrameChanged(ptr unsafe.Pointer, frameNumber C.int) {
	if signal := qt.GetSignal(ptr, "frameChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(frameNumber)))
	}

}

func (ptr *QMovie) ConnectFrameChanged(f func(frameNumber int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "frameChanged") {
			C.QMovie_ConnectFrameChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "frameChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "frameChanged"); signal != nil {
			f := func(frameNumber int) {
				(*(*func(int))(signal))(frameNumber)
				f(frameNumber)
			}
			qt.ConnectSignal(ptr.Pointer(), "frameChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "frameChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectFrameChanged() {
	if ptr.Pointer() != nil {
		C.QMovie_DisconnectFrameChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "frameChanged")
	}
}

func (ptr *QMovie) FrameChanged(frameNumber int) {
	if ptr.Pointer() != nil {
		C.QMovie_FrameChanged(ptr.Pointer(), C.int(int32(frameNumber)))
	}
}

func (ptr *QMovie) FrameCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QMovie_FrameCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QMovie) FrameRect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QMovie_FrameRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QMovie) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QMovie_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QMovie) JumpToFrame(frameNumber int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QMovie_JumpToFrame(ptr.Pointer(), C.int(int32(frameNumber)))) != 0
	}
	return false
}

//export callbackQMovie_JumpToNextFrame
func callbackQMovie_JumpToNextFrame(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "jumpToNextFrame"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(NewQMovieFromPointer(ptr).JumpToNextFrameDefault())))
}

func (ptr *QMovie) ConnectJumpToNextFrame(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "jumpToNextFrame"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "jumpToNextFrame", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "jumpToNextFrame", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectJumpToNextFrame() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "jumpToNextFrame")
	}
}

func (ptr *QMovie) JumpToNextFrame() bool {
	if ptr.Pointer() != nil {
		return int8(C.QMovie_JumpToNextFrame(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QMovie) JumpToNextFrameDefault() bool {
	if ptr.Pointer() != nil {
		return int8(C.QMovie_JumpToNextFrameDefault(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QMovie) LastError() QImageReader__ImageReaderError {
	if ptr.Pointer() != nil {
		return QImageReader__ImageReaderError(C.QMovie_LastError(ptr.Pointer()))
	}
	return 0
}

func (ptr *QMovie) LastErrorString() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QMovie_LastErrorString(ptr.Pointer()))
	}
	return ""
}

func (ptr *QMovie) LoopCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QMovie_LoopCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QMovie) NextFrameDelay() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QMovie_NextFrameDelay(ptr.Pointer())))
	}
	return 0
}

//export callbackQMovie_Resized
func callbackQMovie_Resized(ptr unsafe.Pointer, size unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "resized"); signal != nil {
		(*(*func(*core.QSize))(signal))(core.NewQSizeFromPointer(size))
	}

}

func (ptr *QMovie) ConnectResized(f func(size *core.QSize)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "resized") {
			C.QMovie_ConnectResized(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "resized")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "resized"); signal != nil {
			f := func(size *core.QSize) {
				(*(*func(*core.QSize))(signal))(size)
				f(size)
			}
			qt.ConnectSignal(ptr.Pointer(), "resized", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "resized", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectResized() {
	if ptr.Pointer() != nil {
		C.QMovie_DisconnectResized(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "resized")
	}
}

func (ptr *QMovie) Resized(size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_Resized(ptr.Pointer(), core.PointerFromQSize(size))
	}
}

func (ptr *QMovie) ScaledSize() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QMovie_ScaledSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QMovie) SetBackgroundColor(color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_SetBackgroundColor(ptr.Pointer(), PointerFromQColor(color))
	}
}

func (ptr *QMovie) SetCacheMode(mode QMovie__CacheMode) {
	if ptr.Pointer() != nil {
		C.QMovie_SetCacheMode(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QMovie) SetDevice(device core.QIODevice_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_SetDevice(ptr.Pointer(), core.PointerFromQIODevice(device))
	}
}

func (ptr *QMovie) SetFileName(fileName string) {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		C.QMovie_SetFileName(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))})
	}
}

func (ptr *QMovie) SetFormat(format core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_SetFormat(ptr.Pointer(), core.PointerFromQByteArray(format))
	}
}

//export callbackQMovie_SetPaused
func callbackQMovie_SetPaused(ptr unsafe.Pointer, paused C.char) {
	if signal := qt.GetSignal(ptr, "setPaused"); signal != nil {
		(*(*func(bool))(signal))(int8(paused) != 0)
	} else {
		NewQMovieFromPointer(ptr).SetPausedDefault(int8(paused) != 0)
	}
}

func (ptr *QMovie) ConnectSetPaused(f func(paused bool)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setPaused"); signal != nil {
			f := func(paused bool) {
				(*(*func(bool))(signal))(paused)
				f(paused)
			}
			qt.ConnectSignal(ptr.Pointer(), "setPaused", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setPaused", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectSetPaused() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setPaused")
	}
}

func (ptr *QMovie) SetPaused(paused bool) {
	if ptr.Pointer() != nil {
		C.QMovie_SetPaused(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(paused))))
	}
}

func (ptr *QMovie) SetPausedDefault(paused bool) {
	if ptr.Pointer() != nil {
		C.QMovie_SetPausedDefault(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(paused))))
	}
}

func (ptr *QMovie) SetScaledSize(size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_SetScaledSize(ptr.Pointer(), core.PointerFromQSize(size))
	}
}

//export callbackQMovie_SetSpeed
func callbackQMovie_SetSpeed(ptr unsafe.Pointer, percentSpeed C.int) {
	if signal := qt.GetSignal(ptr, "setSpeed"); signal != nil {
		(*(*func(int))(signal))(int(int32(percentSpeed)))
	} else {
		NewQMovieFromPointer(ptr).SetSpeedDefault(int(int32(percentSpeed)))
	}
}

func (ptr *QMovie) ConnectSetSpeed(f func(percentSpeed int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setSpeed"); signal != nil {
			f := func(percentSpeed int) {
				(*(*func(int))(signal))(percentSpeed)
				f(percentSpeed)
			}
			qt.ConnectSignal(ptr.Pointer(), "setSpeed", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setSpeed", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectSetSpeed() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setSpeed")
	}
}

func (ptr *QMovie) SetSpeed(percentSpeed int) {
	if ptr.Pointer() != nil {
		C.QMovie_SetSpeed(ptr.Pointer(), C.int(int32(percentSpeed)))
	}
}

func (ptr *QMovie) SetSpeedDefault(percentSpeed int) {
	if ptr.Pointer() != nil {
		C.QMovie_SetSpeedDefault(ptr.Pointer(), C.int(int32(percentSpeed)))
	}
}

func (ptr *QMovie) Speed() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QMovie_Speed(ptr.Pointer())))
	}
	return 0
}

//export callbackQMovie_Start
func callbackQMovie_Start(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "start"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQMovieFromPointer(ptr).StartDefault()
	}
}

func (ptr *QMovie) ConnectStart(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "start"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "start", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "start", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectStart() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "start")
	}
}

func (ptr *QMovie) Start() {
	if ptr.Pointer() != nil {
		C.QMovie_Start(ptr.Pointer())
	}
}

func (ptr *QMovie) StartDefault() {
	if ptr.Pointer() != nil {
		C.QMovie_StartDefault(ptr.Pointer())
	}
}

//export callbackQMovie_Started
func callbackQMovie_Started(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "started"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QMovie) ConnectStarted(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "started") {
			C.QMovie_ConnectStarted(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "started")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "started"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "started", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "started", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectStarted() {
	if ptr.Pointer() != nil {
		C.QMovie_DisconnectStarted(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "started")
	}
}

func (ptr *QMovie) Started() {
	if ptr.Pointer() != nil {
		C.QMovie_Started(ptr.Pointer())
	}
}

func (ptr *QMovie) State() QMovie__MovieState {
	if ptr.Pointer() != nil {
		return QMovie__MovieState(C.QMovie_State(ptr.Pointer()))
	}
	return 0
}

//export callbackQMovie_StateChanged
func callbackQMovie_StateChanged(ptr unsafe.Pointer, state C.longlong) {
	if signal := qt.GetSignal(ptr, "stateChanged"); signal != nil {
		(*(*func(QMovie__MovieState))(signal))(QMovie__MovieState(state))
	}

}

func (ptr *QMovie) ConnectStateChanged(f func(state QMovie__MovieState)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "stateChanged") {
			C.QMovie_ConnectStateChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "stateChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "stateChanged"); signal != nil {
			f := func(state QMovie__MovieState) {
				(*(*func(QMovie__MovieState))(signal))(state)
				f(state)
			}
			qt.ConnectSignal(ptr.Pointer(), "stateChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "stateChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectStateChanged() {
	if ptr.Pointer() != nil {
		C.QMovie_DisconnectStateChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "stateChanged")
	}
}

func (ptr *QMovie) StateChanged(state QMovie__MovieState) {
	if ptr.Pointer() != nil {
		C.QMovie_StateChanged(ptr.Pointer(), C.longlong(state))
	}
}

//export callbackQMovie_Stop
func callbackQMovie_Stop(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "stop"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQMovieFromPointer(ptr).StopDefault()
	}
}

func (ptr *QMovie) ConnectStop(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "stop"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "stop", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "stop", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectStop() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "stop")
	}
}

func (ptr *QMovie) Stop() {
	if ptr.Pointer() != nil {
		C.QMovie_Stop(ptr.Pointer())
	}
}

func (ptr *QMovie) StopDefault() {
	if ptr.Pointer() != nil {
		C.QMovie_StopDefault(ptr.Pointer())
	}
}

func QMovie_SupportedFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQMovieFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedFormats_atList(i)
		}
		return out
	}(C.QMovie_QMovie_SupportedFormats())
}

func (ptr *QMovie) SupportedFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQMovieFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedFormats_atList(i)
		}
		return out
	}(C.QMovie_QMovie_SupportedFormats())
}

//export callbackQMovie_Updated
func callbackQMovie_Updated(ptr unsafe.Pointer, rect unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "updated"); signal != nil {
		(*(*func(*core.QRect))(signal))(core.NewQRectFromPointer(rect))
	}

}

func (ptr *QMovie) ConnectUpdated(f func(rect *core.QRect)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "updated") {
			C.QMovie_ConnectUpdated(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "updated")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "updated"); signal != nil {
			f := func(rect *core.QRect) {
				(*(*func(*core.QRect))(signal))(rect)
				f(rect)
			}
			qt.ConnectSignal(ptr.Pointer(), "updated", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "updated", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectUpdated() {
	if ptr.Pointer() != nil {
		C.QMovie_DisconnectUpdated(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "updated")
	}
}

func (ptr *QMovie) Updated(rect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_Updated(ptr.Pointer(), core.PointerFromQRect(rect))
	}
}

//export callbackQMovie_DestroyQMovie
func callbackQMovie_DestroyQMovie(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QMovie"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQMovieFromPointer(ptr).DestroyQMovieDefault()
	}
}

func (ptr *QMovie) ConnectDestroyQMovie(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QMovie"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QMovie", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QMovie", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QMovie) DisconnectDestroyQMovie() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QMovie")
	}
}

func (ptr *QMovie) DestroyQMovie() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QMovie_DestroyQMovie(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QMovie) DestroyQMovieDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QMovie_DestroyQMovieDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QMovie) __supportedFormats_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QMovie___supportedFormats_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QMovie) __supportedFormats_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie___supportedFormats_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QMovie) __supportedFormats_newList() unsafe.Pointer {
	return C.QMovie___supportedFormats_newList(ptr.Pointer())
}

func (ptr *QMovie) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QMovie___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QMovie) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QMovie) __children_newList() unsafe.Pointer {
	return C.QMovie___children_newList(ptr.Pointer())
}

func (ptr *QMovie) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QMovie___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QMovie) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QMovie) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QMovie___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QMovie) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QMovie___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QMovie) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QMovie) __findChildren_newList() unsafe.Pointer {
	return C.QMovie___findChildren_newList(ptr.Pointer())
}

func (ptr *QMovie) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QMovie___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QMovie) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QMovie) __findChildren_newList3() unsafe.Pointer {
	return C.QMovie___findChildren_newList3(ptr.Pointer())
}

//export callbackQMovie_ChildEvent
func callbackQMovie_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQMovieFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QMovie) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQMovie_ConnectNotify
func callbackQMovie_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQMovieFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QMovie) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQMovie_CustomEvent
func callbackQMovie_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQMovieFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QMovie) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQMovie_DeleteLater
func callbackQMovie_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQMovieFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QMovie) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QMovie_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQMovie_Destroyed
func callbackQMovie_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQMovie_DisconnectNotify
func callbackQMovie_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQMovieFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QMovie) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQMovie_Event
func callbackQMovie_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQMovieFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QMovie) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QMovie_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQMovie_EventFilter
func callbackQMovie_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQMovieFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QMovie) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QMovie_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQMovie_MetaObject
func callbackQMovie_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQMovieFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QMovie) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QMovie_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQMovie_ObjectNameChanged
func callbackQMovie_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQMovie_TimerEvent
func callbackQMovie_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQMovieFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QMovie) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QMovie_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QNativeGestureEvent struct {
	QInputEvent
}

type QNativeGestureEvent_ITF interface {
	QInputEvent_ITF
	QNativeGestureEvent_PTR() *QNativeGestureEvent
}

func (ptr *QNativeGestureEvent) QNativeGestureEvent_PTR() *QNativeGestureEvent {
	return ptr
}

func (ptr *QNativeGestureEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QNativeGestureEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QInputEvent_PTR().SetPointer(p)
	}
}

func PointerFromQNativeGestureEvent(ptr QNativeGestureEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QNativeGestureEvent_PTR().Pointer()
	}
	return nil
}

func NewQNativeGestureEventFromPointer(ptr unsafe.Pointer) (n *QNativeGestureEvent) {
	n = new(QNativeGestureEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QNativeGestureEvent) DestroyQNativeGestureEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQNativeGestureEvent2(ty core.Qt__NativeGestureType, device QTouchDevice_ITF, localPos core.QPointF_ITF, windowPos core.QPointF_ITF, screenPos core.QPointF_ITF, realValue float64, sequenceId uint, intValue uint64) *QNativeGestureEvent {
	tmpValue := NewQNativeGestureEventFromPointer(C.QNativeGestureEvent_NewQNativeGestureEvent2(C.longlong(ty), PointerFromQTouchDevice(device), core.PointerFromQPointF(localPos), core.PointerFromQPointF(windowPos), core.PointerFromQPointF(screenPos), C.double(realValue), C.ulong(uint32(sequenceId)), C.ulonglong(intValue)))
	qt.SetFinalizer(tmpValue, (*QNativeGestureEvent).DestroyQNativeGestureEvent)
	return tmpValue
}

func (ptr *QNativeGestureEvent) Device() *QTouchDevice {
	if ptr.Pointer() != nil {
		return NewQTouchDeviceFromPointer(C.QNativeGestureEvent_Device(ptr.Pointer()))
	}
	return nil
}

func (ptr *QNativeGestureEvent) GestureType() core.Qt__NativeGestureType {
	if ptr.Pointer() != nil {
		return core.Qt__NativeGestureType(C.QNativeGestureEvent_GestureType(ptr.Pointer()))
	}
	return 0
}

func (ptr *QNativeGestureEvent) GlobalPos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QNativeGestureEvent_GlobalPos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QNativeGestureEvent) LocalPos() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QNativeGestureEvent_LocalPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QNativeGestureEvent) Pos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QNativeGestureEvent_Pos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QNativeGestureEvent) ScreenPos() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QNativeGestureEvent_ScreenPos(ptr.Pointer()))
	}
	return nil
}

func (ptr *QNativeGestureEvent) Value() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QNativeGestureEvent_Value(ptr.Pointer()))
	}
	return 0
}

func (ptr *QNativeGestureEvent) WindowPos() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QNativeGestureEvent_WindowPos(ptr.Pointer()))
	}
	return nil
}

type QOffscreenSurface struct {
	core.QObject
	QSurface
}

type QOffscreenSurface_ITF interface {
	core.QObject_ITF
	QSurface_ITF
	QOffscreenSurface_PTR() *QOffscreenSurface
}

func (ptr *QOffscreenSurface) QOffscreenSurface_PTR() *QOffscreenSurface {
	return ptr
}

func (ptr *QOffscreenSurface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QOffscreenSurface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
		ptr.QSurface_PTR().SetPointer(p)
	}
}

func PointerFromQOffscreenSurface(ptr QOffscreenSurface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOffscreenSurface_PTR().Pointer()
	}
	return nil
}

func NewQOffscreenSurfaceFromPointer(ptr unsafe.Pointer) (n *QOffscreenSurface) {
	n = new(QOffscreenSurface)
	n.SetPointer(ptr)
	return
}
func NewQOffscreenSurface(targetScreen QScreen_ITF, parent core.QObject_ITF) *QOffscreenSurface {
	tmpValue := NewQOffscreenSurfaceFromPointer(C.QOffscreenSurface_NewQOffscreenSurface(PointerFromQScreen(targetScreen), core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQOffscreenSurface2(targetScreen QScreen_ITF) *QOffscreenSurface {
	tmpValue := NewQOffscreenSurfaceFromPointer(C.QOffscreenSurface_NewQOffscreenSurface2(PointerFromQScreen(targetScreen)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOffscreenSurface) Create() {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_Create(ptr.Pointer())
	}
}

func (ptr *QOffscreenSurface) Destroy() {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_Destroy(ptr.Pointer())
	}
}

//export callbackQOffscreenSurface_Format
func callbackQOffscreenSurface_Format(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "format"); signal != nil {
		return PointerFromQSurfaceFormat((*(*func() *QSurfaceFormat)(signal))())
	}

	return PointerFromQSurfaceFormat(NewQOffscreenSurfaceFromPointer(ptr).FormatDefault())
}

func (ptr *QOffscreenSurface) ConnectFormat(f func() *QSurfaceFormat) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "format"); signal != nil {
			f := func() *QSurfaceFormat {
				(*(*func() *QSurfaceFormat)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "format", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "format", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOffscreenSurface) DisconnectFormat() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "format")
	}
}

func (ptr *QOffscreenSurface) Format() *QSurfaceFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQSurfaceFormatFromPointer(C.QOffscreenSurface_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QOffscreenSurface) FormatDefault() *QSurfaceFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQSurfaceFormatFromPointer(C.QOffscreenSurface_FormatDefault(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QOffscreenSurface) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOffscreenSurface_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOffscreenSurface) NativeHandle() unsafe.Pointer {
	if ptr.Pointer() != nil {
		return C.QOffscreenSurface_NativeHandle(ptr.Pointer())
	}
	return nil
}

func (ptr *QOffscreenSurface) RequestedFormat() *QSurfaceFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQSurfaceFormatFromPointer(C.QOffscreenSurface_RequestedFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QOffscreenSurface) Screen() *QScreen {
	if ptr.Pointer() != nil {
		tmpValue := NewQScreenFromPointer(C.QOffscreenSurface_Screen(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQOffscreenSurface_ScreenChanged
func callbackQOffscreenSurface_ScreenChanged(ptr unsafe.Pointer, screen unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "screenChanged"); signal != nil {
		(*(*func(*QScreen))(signal))(NewQScreenFromPointer(screen))
	}

}

func (ptr *QOffscreenSurface) ConnectScreenChanged(f func(screen *QScreen)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "screenChanged") {
			C.QOffscreenSurface_ConnectScreenChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "screenChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "screenChanged"); signal != nil {
			f := func(screen *QScreen) {
				(*(*func(*QScreen))(signal))(screen)
				f(screen)
			}
			qt.ConnectSignal(ptr.Pointer(), "screenChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "screenChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOffscreenSurface) DisconnectScreenChanged() {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_DisconnectScreenChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "screenChanged")
	}
}

func (ptr *QOffscreenSurface) ScreenChanged(screen QScreen_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_ScreenChanged(ptr.Pointer(), PointerFromQScreen(screen))
	}
}

func (ptr *QOffscreenSurface) SetFormat(format QSurfaceFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_SetFormat(ptr.Pointer(), PointerFromQSurfaceFormat(format))
	}
}

func (ptr *QOffscreenSurface) SetNativeHandle(handle unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_SetNativeHandle(ptr.Pointer(), handle)
	}
}

func (ptr *QOffscreenSurface) SetScreen(newScreen QScreen_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_SetScreen(ptr.Pointer(), PointerFromQScreen(newScreen))
	}
}

//export callbackQOffscreenSurface_Size
func callbackQOffscreenSurface_Size(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "size"); signal != nil {
		return core.PointerFromQSize((*(*func() *core.QSize)(signal))())
	}

	return core.PointerFromQSize(NewQOffscreenSurfaceFromPointer(ptr).SizeDefault())
}

func (ptr *QOffscreenSurface) ConnectSize(f func() *core.QSize) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "size"); signal != nil {
			f := func() *core.QSize {
				(*(*func() *core.QSize)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "size", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "size", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOffscreenSurface) DisconnectSize() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "size")
	}
}

func (ptr *QOffscreenSurface) Size() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QOffscreenSurface_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QOffscreenSurface) SizeDefault() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QOffscreenSurface_SizeDefault(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

//export callbackQOffscreenSurface_SurfaceType
func callbackQOffscreenSurface_SurfaceType(ptr unsafe.Pointer) C.longlong {
	if signal := qt.GetSignal(ptr, "surfaceType"); signal != nil {
		return C.longlong((*(*func() QSurface__SurfaceType)(signal))())
	}

	return C.longlong(NewQOffscreenSurfaceFromPointer(ptr).SurfaceTypeDefault())
}

func (ptr *QOffscreenSurface) ConnectSurfaceType(f func() QSurface__SurfaceType) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "surfaceType"); signal != nil {
			f := func() QSurface__SurfaceType {
				(*(*func() QSurface__SurfaceType)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "surfaceType", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "surfaceType", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOffscreenSurface) DisconnectSurfaceType() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "surfaceType")
	}
}

func (ptr *QOffscreenSurface) SurfaceType() QSurface__SurfaceType {
	if ptr.Pointer() != nil {
		return QSurface__SurfaceType(C.QOffscreenSurface_SurfaceType(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOffscreenSurface) SurfaceTypeDefault() QSurface__SurfaceType {
	if ptr.Pointer() != nil {
		return QSurface__SurfaceType(C.QOffscreenSurface_SurfaceTypeDefault(ptr.Pointer()))
	}
	return 0
}

//export callbackQOffscreenSurface_DestroyQOffscreenSurface
func callbackQOffscreenSurface_DestroyQOffscreenSurface(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOffscreenSurface"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOffscreenSurfaceFromPointer(ptr).DestroyQOffscreenSurfaceDefault()
	}
}

func (ptr *QOffscreenSurface) ConnectDestroyQOffscreenSurface(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOffscreenSurface"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOffscreenSurface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOffscreenSurface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOffscreenSurface) DisconnectDestroyQOffscreenSurface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOffscreenSurface")
	}
}

func (ptr *QOffscreenSurface) DestroyQOffscreenSurface() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOffscreenSurface_DestroyQOffscreenSurface(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOffscreenSurface) DestroyQOffscreenSurfaceDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOffscreenSurface_DestroyQOffscreenSurfaceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOffscreenSurface) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOffscreenSurface___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOffscreenSurface) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOffscreenSurface) __children_newList() unsafe.Pointer {
	return C.QOffscreenSurface___children_newList(ptr.Pointer())
}

func (ptr *QOffscreenSurface) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QOffscreenSurface___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QOffscreenSurface) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QOffscreenSurface) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QOffscreenSurface___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QOffscreenSurface) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOffscreenSurface___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOffscreenSurface) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOffscreenSurface) __findChildren_newList() unsafe.Pointer {
	return C.QOffscreenSurface___findChildren_newList(ptr.Pointer())
}

func (ptr *QOffscreenSurface) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOffscreenSurface___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOffscreenSurface) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOffscreenSurface) __findChildren_newList3() unsafe.Pointer {
	return C.QOffscreenSurface___findChildren_newList3(ptr.Pointer())
}

//export callbackQOffscreenSurface_ChildEvent
func callbackQOffscreenSurface_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQOffscreenSurfaceFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QOffscreenSurface) ChildEvent(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_ChildEvent(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

func (ptr *QOffscreenSurface) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQOffscreenSurface_ConnectNotify
func callbackQOffscreenSurface_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOffscreenSurfaceFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOffscreenSurface) ConnectNotify(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_ConnectNotify(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

func (ptr *QOffscreenSurface) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOffscreenSurface_CustomEvent
func callbackQOffscreenSurface_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQOffscreenSurfaceFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QOffscreenSurface) CustomEvent(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_CustomEvent(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

func (ptr *QOffscreenSurface) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQOffscreenSurface_DeleteLater
func callbackQOffscreenSurface_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOffscreenSurfaceFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QOffscreenSurface) DeleteLater() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOffscreenSurface_DeleteLater(ptr.Pointer())
	}
}

func (ptr *QOffscreenSurface) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOffscreenSurface_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQOffscreenSurface_Destroyed
func callbackQOffscreenSurface_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQOffscreenSurface_DisconnectNotify
func callbackQOffscreenSurface_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOffscreenSurfaceFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOffscreenSurface) DisconnectNotify(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_DisconnectNotify(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

func (ptr *QOffscreenSurface) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOffscreenSurface_Event
func callbackQOffscreenSurface_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOffscreenSurfaceFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QOffscreenSurface) Event(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOffscreenSurface_Event(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

func (ptr *QOffscreenSurface) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOffscreenSurface_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQOffscreenSurface_EventFilter
func callbackQOffscreenSurface_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOffscreenSurfaceFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QOffscreenSurface) EventFilter(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOffscreenSurface_EventFilter(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

func (ptr *QOffscreenSurface) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOffscreenSurface_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQOffscreenSurface_MetaObject
func callbackQOffscreenSurface_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQOffscreenSurfaceFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QOffscreenSurface) MetaObject() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QOffscreenSurface_MetaObject(ptr.Pointer()))
	}
	return nil
}

func (ptr *QOffscreenSurface) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QOffscreenSurface_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQOffscreenSurface_ObjectNameChanged
func callbackQOffscreenSurface_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQOffscreenSurface_TimerEvent
func callbackQOffscreenSurface_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQOffscreenSurfaceFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QOffscreenSurface) TimerEvent(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_TimerEvent(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

func (ptr *QOffscreenSurface) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOffscreenSurface_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QOpenGLBuffer struct {
	ptr unsafe.Pointer
}

type QOpenGLBuffer_ITF interface {
	QOpenGLBuffer_PTR() *QOpenGLBuffer
}

func (ptr *QOpenGLBuffer) QOpenGLBuffer_PTR() *QOpenGLBuffer {
	return ptr
}

func (ptr *QOpenGLBuffer) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLBuffer) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLBuffer(ptr QOpenGLBuffer_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLBuffer_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLBufferFromPointer(ptr unsafe.Pointer) (n *QOpenGLBuffer) {
	n = new(QOpenGLBuffer)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QOpenGLBuffer__Type
//QOpenGLBuffer::Type
type QOpenGLBuffer__Type int64

const (
	QOpenGLBuffer__VertexBuffer      QOpenGLBuffer__Type = QOpenGLBuffer__Type(0x8892)
	QOpenGLBuffer__IndexBuffer       QOpenGLBuffer__Type = QOpenGLBuffer__Type(0x8893)
	QOpenGLBuffer__PixelPackBuffer   QOpenGLBuffer__Type = QOpenGLBuffer__Type(0x88EB)
	QOpenGLBuffer__PixelUnpackBuffer QOpenGLBuffer__Type = QOpenGLBuffer__Type(0x88EC)
)

//go:generate stringer -type=QOpenGLBuffer__UsagePattern
//QOpenGLBuffer::UsagePattern
type QOpenGLBuffer__UsagePattern int64

const (
	QOpenGLBuffer__StreamDraw  QOpenGLBuffer__UsagePattern = QOpenGLBuffer__UsagePattern(0x88E0)
	QOpenGLBuffer__StreamRead  QOpenGLBuffer__UsagePattern = QOpenGLBuffer__UsagePattern(0x88E1)
	QOpenGLBuffer__StreamCopy  QOpenGLBuffer__UsagePattern = QOpenGLBuffer__UsagePattern(0x88E2)
	QOpenGLBuffer__StaticDraw  QOpenGLBuffer__UsagePattern = QOpenGLBuffer__UsagePattern(0x88E4)
	QOpenGLBuffer__StaticRead  QOpenGLBuffer__UsagePattern = QOpenGLBuffer__UsagePattern(0x88E5)
	QOpenGLBuffer__StaticCopy  QOpenGLBuffer__UsagePattern = QOpenGLBuffer__UsagePattern(0x88E6)
	QOpenGLBuffer__DynamicDraw QOpenGLBuffer__UsagePattern = QOpenGLBuffer__UsagePattern(0x88E8)
	QOpenGLBuffer__DynamicRead QOpenGLBuffer__UsagePattern = QOpenGLBuffer__UsagePattern(0x88E9)
	QOpenGLBuffer__DynamicCopy QOpenGLBuffer__UsagePattern = QOpenGLBuffer__UsagePattern(0x88EA)
)

//go:generate stringer -type=QOpenGLBuffer__Access
//QOpenGLBuffer::Access
type QOpenGLBuffer__Access int64

const (
	QOpenGLBuffer__ReadOnly  QOpenGLBuffer__Access = QOpenGLBuffer__Access(0x88B8)
	QOpenGLBuffer__WriteOnly QOpenGLBuffer__Access = QOpenGLBuffer__Access(0x88B9)
	QOpenGLBuffer__ReadWrite QOpenGLBuffer__Access = QOpenGLBuffer__Access(0x88BA)
)

//go:generate stringer -type=QOpenGLBuffer__RangeAccessFlag
//QOpenGLBuffer::RangeAccessFlag
type QOpenGLBuffer__RangeAccessFlag int64

const (
	QOpenGLBuffer__RangeRead             QOpenGLBuffer__RangeAccessFlag = QOpenGLBuffer__RangeAccessFlag(0x0001)
	QOpenGLBuffer__RangeWrite            QOpenGLBuffer__RangeAccessFlag = QOpenGLBuffer__RangeAccessFlag(0x0002)
	QOpenGLBuffer__RangeInvalidate       QOpenGLBuffer__RangeAccessFlag = QOpenGLBuffer__RangeAccessFlag(0x0004)
	QOpenGLBuffer__RangeInvalidateBuffer QOpenGLBuffer__RangeAccessFlag = QOpenGLBuffer__RangeAccessFlag(0x0008)
	QOpenGLBuffer__RangeFlushExplicit    QOpenGLBuffer__RangeAccessFlag = QOpenGLBuffer__RangeAccessFlag(0x0010)
	QOpenGLBuffer__RangeUnsynchronized   QOpenGLBuffer__RangeAccessFlag = QOpenGLBuffer__RangeAccessFlag(0x0020)
)

func NewQOpenGLBuffer() *QOpenGLBuffer {
	tmpValue := NewQOpenGLBufferFromPointer(C.QOpenGLBuffer_NewQOpenGLBuffer())
	qt.SetFinalizer(tmpValue, (*QOpenGLBuffer).DestroyQOpenGLBuffer)
	return tmpValue
}

func NewQOpenGLBuffer2(ty QOpenGLBuffer__Type) *QOpenGLBuffer {
	tmpValue := NewQOpenGLBufferFromPointer(C.QOpenGLBuffer_NewQOpenGLBuffer2(C.longlong(ty)))
	qt.SetFinalizer(tmpValue, (*QOpenGLBuffer).DestroyQOpenGLBuffer)
	return tmpValue
}

func NewQOpenGLBuffer3(other QOpenGLBuffer_ITF) *QOpenGLBuffer {
	tmpValue := NewQOpenGLBufferFromPointer(C.QOpenGLBuffer_NewQOpenGLBuffer3(PointerFromQOpenGLBuffer(other)))
	qt.SetFinalizer(tmpValue, (*QOpenGLBuffer).DestroyQOpenGLBuffer)
	return tmpValue
}

func (ptr *QOpenGLBuffer) Allocate(data unsafe.Pointer, count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLBuffer_Allocate(ptr.Pointer(), data, C.int(int32(count)))
	}
}

func (ptr *QOpenGLBuffer) Allocate2(count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLBuffer_Allocate2(ptr.Pointer(), C.int(int32(count)))
	}
}

func (ptr *QOpenGLBuffer) Bind() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLBuffer_Bind(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLBuffer) BufferId() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLBuffer_BufferId(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLBuffer) Create() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLBuffer_Create(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLBuffer) Destroy() {
	if ptr.Pointer() != nil {
		C.QOpenGLBuffer_Destroy(ptr.Pointer())
	}
}

func (ptr *QOpenGLBuffer) IsCreated() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLBuffer_IsCreated(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLBuffer) Map(access QOpenGLBuffer__Access) unsafe.Pointer {
	if ptr.Pointer() != nil {
		return C.QOpenGLBuffer_Map(ptr.Pointer(), C.longlong(access))
	}
	return nil
}

func (ptr *QOpenGLBuffer) MapRange(offset int, count int, access QOpenGLBuffer__RangeAccessFlag) unsafe.Pointer {
	if ptr.Pointer() != nil {
		return C.QOpenGLBuffer_MapRange(ptr.Pointer(), C.int(int32(offset)), C.int(int32(count)), C.longlong(access))
	}
	return nil
}

func (ptr *QOpenGLBuffer) Read(offset int, data unsafe.Pointer, count int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLBuffer_Read(ptr.Pointer(), C.int(int32(offset)), data, C.int(int32(count)))) != 0
	}
	return false
}

func (ptr *QOpenGLBuffer) Release() {
	if ptr.Pointer() != nil {
		C.QOpenGLBuffer_Release(ptr.Pointer())
	}
}

func QOpenGLBuffer_Release2(ty QOpenGLBuffer__Type) {
	C.QOpenGLBuffer_QOpenGLBuffer_Release2(C.longlong(ty))
}

func (ptr *QOpenGLBuffer) Release2(ty QOpenGLBuffer__Type) {
	C.QOpenGLBuffer_QOpenGLBuffer_Release2(C.longlong(ty))
}

func (ptr *QOpenGLBuffer) SetUsagePattern(value QOpenGLBuffer__UsagePattern) {
	if ptr.Pointer() != nil {
		C.QOpenGLBuffer_SetUsagePattern(ptr.Pointer(), C.longlong(value))
	}
}

func (ptr *QOpenGLBuffer) Size() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLBuffer_Size(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLBuffer) Type() QOpenGLBuffer__Type {
	if ptr.Pointer() != nil {
		return QOpenGLBuffer__Type(C.QOpenGLBuffer_Type(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLBuffer) Unmap() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLBuffer_Unmap(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLBuffer) UsagePattern() QOpenGLBuffer__UsagePattern {
	if ptr.Pointer() != nil {
		return QOpenGLBuffer__UsagePattern(C.QOpenGLBuffer_UsagePattern(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLBuffer) Write(offset int, data unsafe.Pointer, count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLBuffer_Write(ptr.Pointer(), C.int(int32(offset)), data, C.int(int32(count)))
	}
}

func (ptr *QOpenGLBuffer) DestroyQOpenGLBuffer() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLBuffer_DestroyQOpenGLBuffer(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLContext struct {
	core.QObject
}

type QOpenGLContext_ITF interface {
	core.QObject_ITF
	QOpenGLContext_PTR() *QOpenGLContext
}

func (ptr *QOpenGLContext) QOpenGLContext_PTR() *QOpenGLContext {
	return ptr
}

func (ptr *QOpenGLContext) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLContext) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLContext(ptr QOpenGLContext_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLContext_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLContextFromPointer(ptr unsafe.Pointer) (n *QOpenGLContext) {
	n = new(QOpenGLContext)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QOpenGLContext__OpenGLModuleType
//QOpenGLContext::OpenGLModuleType
type QOpenGLContext__OpenGLModuleType int64

const (
	QOpenGLContext__LibGL   QOpenGLContext__OpenGLModuleType = QOpenGLContext__OpenGLModuleType(0)
	QOpenGLContext__LibGLES QOpenGLContext__OpenGLModuleType = QOpenGLContext__OpenGLModuleType(1)
)

func NewQOpenGLContext(parent core.QObject_ITF) *QOpenGLContext {
	tmpValue := NewQOpenGLContextFromPointer(C.QOpenGLContext_NewQOpenGLContext(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQOpenGLContext_AboutToBeDestroyed
func callbackQOpenGLContext_AboutToBeDestroyed(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "aboutToBeDestroyed"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QOpenGLContext) ConnectAboutToBeDestroyed(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "aboutToBeDestroyed") {
			C.QOpenGLContext_ConnectAboutToBeDestroyed(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "aboutToBeDestroyed")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "aboutToBeDestroyed"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "aboutToBeDestroyed", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "aboutToBeDestroyed", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLContext) DisconnectAboutToBeDestroyed() {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_DisconnectAboutToBeDestroyed(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "aboutToBeDestroyed")
	}
}

func (ptr *QOpenGLContext) AboutToBeDestroyed() {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_AboutToBeDestroyed(ptr.Pointer())
	}
}

func QOpenGLContext_AreSharing(first QOpenGLContext_ITF, second QOpenGLContext_ITF) bool {
	return int8(C.QOpenGLContext_QOpenGLContext_AreSharing(PointerFromQOpenGLContext(first), PointerFromQOpenGLContext(second))) != 0
}

func (ptr *QOpenGLContext) AreSharing(first QOpenGLContext_ITF, second QOpenGLContext_ITF) bool {
	return int8(C.QOpenGLContext_QOpenGLContext_AreSharing(PointerFromQOpenGLContext(first), PointerFromQOpenGLContext(second))) != 0
}

func (ptr *QOpenGLContext) Create() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLContext_Create(ptr.Pointer())) != 0
	}
	return false
}

func QOpenGLContext_CurrentContext() *QOpenGLContext {
	tmpValue := NewQOpenGLContextFromPointer(C.QOpenGLContext_QOpenGLContext_CurrentContext())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLContext) CurrentContext() *QOpenGLContext {
	tmpValue := NewQOpenGLContextFromPointer(C.QOpenGLContext_QOpenGLContext_CurrentContext())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLContext) DefaultFramebufferObject() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLContext_DefaultFramebufferObject(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLContext) DoneCurrent() {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_DoneCurrent(ptr.Pointer())
	}
}

func (ptr *QOpenGLContext) ExtraFunctions() *QOpenGLExtraFunctions {
	if ptr.Pointer() != nil {
		return NewQOpenGLExtraFunctionsFromPointer(C.QOpenGLContext_ExtraFunctions(ptr.Pointer()))
	}
	return nil
}

func (ptr *QOpenGLContext) Format() *QSurfaceFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQSurfaceFormatFromPointer(C.QOpenGLContext_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContext) Functions() *QOpenGLFunctions {
	if ptr.Pointer() != nil {
		return NewQOpenGLFunctionsFromPointer(C.QOpenGLContext_Functions(ptr.Pointer()))
	}
	return nil
}

func (ptr *QOpenGLContext) GetProcAddress(procName core.QByteArray_ITF) unsafe.Pointer {
	if ptr.Pointer() != nil {
		return C.QOpenGLContext_GetProcAddress(ptr.Pointer(), core.PointerFromQByteArray(procName))
	}
	return nil
}

func (ptr *QOpenGLContext) GetProcAddress2(procName string) unsafe.Pointer {
	if ptr.Pointer() != nil {
		var procNameC *C.char
		if procName != "" {
			procNameC = C.CString(procName)
			defer C.free(unsafe.Pointer(procNameC))
		}
		return C.QOpenGLContext_GetProcAddress2(ptr.Pointer(), procNameC)
	}
	return nil
}

func QOpenGLContext_GlobalShareContext() *QOpenGLContext {
	tmpValue := NewQOpenGLContextFromPointer(C.QOpenGLContext_QOpenGLContext_GlobalShareContext())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLContext) GlobalShareContext() *QOpenGLContext {
	tmpValue := NewQOpenGLContextFromPointer(C.QOpenGLContext_QOpenGLContext_GlobalShareContext())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLContext) HasExtension(extensi core.QByteArray_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLContext_HasExtension(ptr.Pointer(), core.PointerFromQByteArray(extensi))) != 0
	}
	return false
}

func (ptr *QOpenGLContext) IsOpenGLES() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLContext_IsOpenGLES(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLContext) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLContext_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLContext) MakeCurrent(surface QSurface_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLContext_MakeCurrent(ptr.Pointer(), PointerFromQSurface(surface))) != 0
	}
	return false
}

func (ptr *QOpenGLContext) NativeHandle() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QOpenGLContext_NativeHandle(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func QOpenGLContext_OpenGLModuleHandle() unsafe.Pointer {
	return C.QOpenGLContext_QOpenGLContext_OpenGLModuleHandle()
}

func (ptr *QOpenGLContext) OpenGLModuleHandle() unsafe.Pointer {
	return C.QOpenGLContext_QOpenGLContext_OpenGLModuleHandle()
}

func QOpenGLContext_OpenGLModuleType() QOpenGLContext__OpenGLModuleType {
	return QOpenGLContext__OpenGLModuleType(C.QOpenGLContext_QOpenGLContext_OpenGLModuleType())
}

func (ptr *QOpenGLContext) OpenGLModuleType() QOpenGLContext__OpenGLModuleType {
	return QOpenGLContext__OpenGLModuleType(C.QOpenGLContext_QOpenGLContext_OpenGLModuleType())
}

func (ptr *QOpenGLContext) Screen() *QScreen {
	if ptr.Pointer() != nil {
		tmpValue := NewQScreenFromPointer(C.QOpenGLContext_Screen(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContext) SetFormat(format QSurfaceFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_SetFormat(ptr.Pointer(), PointerFromQSurfaceFormat(format))
	}
}

func (ptr *QOpenGLContext) SetNativeHandle(handle core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_SetNativeHandle(ptr.Pointer(), core.PointerFromQVariant(handle))
	}
}

func (ptr *QOpenGLContext) SetScreen(screen QScreen_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_SetScreen(ptr.Pointer(), PointerFromQScreen(screen))
	}
}

func (ptr *QOpenGLContext) SetShareContext(shareContext QOpenGLContext_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_SetShareContext(ptr.Pointer(), PointerFromQOpenGLContext(shareContext))
	}
}

func (ptr *QOpenGLContext) ShareContext() *QOpenGLContext {
	if ptr.Pointer() != nil {
		tmpValue := NewQOpenGLContextFromPointer(C.QOpenGLContext_ShareContext(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContext) ShareGroup() *QOpenGLContextGroup {
	if ptr.Pointer() != nil {
		tmpValue := NewQOpenGLContextGroupFromPointer(C.QOpenGLContext_ShareGroup(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func QOpenGLContext_SupportsThreadedOpenGL() bool {
	return int8(C.QOpenGLContext_QOpenGLContext_SupportsThreadedOpenGL()) != 0
}

func (ptr *QOpenGLContext) SupportsThreadedOpenGL() bool {
	return int8(C.QOpenGLContext_QOpenGLContext_SupportsThreadedOpenGL()) != 0
}

func (ptr *QOpenGLContext) Surface() *QSurface {
	if ptr.Pointer() != nil {
		return NewQSurfaceFromPointer(C.QOpenGLContext_Surface(ptr.Pointer()))
	}
	return nil
}

func (ptr *QOpenGLContext) SwapBuffers(surface QSurface_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_SwapBuffers(ptr.Pointer(), PointerFromQSurface(surface))
	}
}

func (ptr *QOpenGLContext) VersionFunctions(versionProfile QOpenGLVersionProfile_ITF) *QAbstractOpenGLFunctions {
	if ptr.Pointer() != nil {
		return NewQAbstractOpenGLFunctionsFromPointer(C.QOpenGLContext_VersionFunctions(ptr.Pointer(), PointerFromQOpenGLVersionProfile(versionProfile)))
	}
	return nil
}

//export callbackQOpenGLContext_DestroyQOpenGLContext
func callbackQOpenGLContext_DestroyQOpenGLContext(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOpenGLContext"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLContextFromPointer(ptr).DestroyQOpenGLContextDefault()
	}
}

func (ptr *QOpenGLContext) ConnectDestroyQOpenGLContext(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOpenGLContext"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLContext", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLContext", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLContext) DisconnectDestroyQOpenGLContext() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOpenGLContext")
	}
}

func (ptr *QOpenGLContext) DestroyQOpenGLContext() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLContext_DestroyQOpenGLContext(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLContext) DestroyQOpenGLContextDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLContext_DestroyQOpenGLContextDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLContext) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLContext___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContext) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLContext) __children_newList() unsafe.Pointer {
	return C.QOpenGLContext___children_newList(ptr.Pointer())
}

func (ptr *QOpenGLContext) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QOpenGLContext___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContext) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QOpenGLContext) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QOpenGLContext___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QOpenGLContext) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLContext___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContext) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLContext) __findChildren_newList() unsafe.Pointer {
	return C.QOpenGLContext___findChildren_newList(ptr.Pointer())
}

func (ptr *QOpenGLContext) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLContext___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContext) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLContext) __findChildren_newList3() unsafe.Pointer {
	return C.QOpenGLContext___findChildren_newList3(ptr.Pointer())
}

//export callbackQOpenGLContext_ChildEvent
func callbackQOpenGLContext_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQOpenGLContextFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QOpenGLContext) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQOpenGLContext_ConnectNotify
func callbackQOpenGLContext_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLContextFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLContext) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLContext_CustomEvent
func callbackQOpenGLContext_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQOpenGLContextFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QOpenGLContext) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQOpenGLContext_DeleteLater
func callbackQOpenGLContext_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLContextFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QOpenGLContext) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLContext_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLContext_Destroyed
func callbackQOpenGLContext_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQOpenGLContext_DisconnectNotify
func callbackQOpenGLContext_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLContextFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLContext) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLContext_Event
func callbackQOpenGLContext_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLContextFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QOpenGLContext) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLContext_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQOpenGLContext_EventFilter
func callbackQOpenGLContext_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLContextFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QOpenGLContext) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLContext_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQOpenGLContext_MetaObject
func callbackQOpenGLContext_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQOpenGLContextFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QOpenGLContext) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QOpenGLContext_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQOpenGLContext_ObjectNameChanged
func callbackQOpenGLContext_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQOpenGLContext_TimerEvent
func callbackQOpenGLContext_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQOpenGLContextFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QOpenGLContext) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContext_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QOpenGLContextGroup struct {
	core.QObject
}

type QOpenGLContextGroup_ITF interface {
	core.QObject_ITF
	QOpenGLContextGroup_PTR() *QOpenGLContextGroup
}

func (ptr *QOpenGLContextGroup) QOpenGLContextGroup_PTR() *QOpenGLContextGroup {
	return ptr
}

func (ptr *QOpenGLContextGroup) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLContextGroup) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLContextGroup(ptr QOpenGLContextGroup_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLContextGroup_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLContextGroupFromPointer(ptr unsafe.Pointer) (n *QOpenGLContextGroup) {
	n = new(QOpenGLContextGroup)
	n.SetPointer(ptr)
	return
}
func QOpenGLContextGroup_CurrentContextGroup() *QOpenGLContextGroup {
	tmpValue := NewQOpenGLContextGroupFromPointer(C.QOpenGLContextGroup_QOpenGLContextGroup_CurrentContextGroup())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLContextGroup) CurrentContextGroup() *QOpenGLContextGroup {
	tmpValue := NewQOpenGLContextGroupFromPointer(C.QOpenGLContextGroup_QOpenGLContextGroup_CurrentContextGroup())
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLContextGroup) Shares() []*QOpenGLContext {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QOpenGLContext {
			out := make([]*QOpenGLContext, int(l.len))
			tmpList := NewQOpenGLContextGroupFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__shares_atList(i)
			}
			return out
		}(C.QOpenGLContextGroup_Shares(ptr.Pointer()))
	}
	return make([]*QOpenGLContext, 0)
}

func (ptr *QOpenGLContextGroup) __shares_atList(i int) *QOpenGLContext {
	if ptr.Pointer() != nil {
		tmpValue := NewQOpenGLContextFromPointer(C.QOpenGLContextGroup___shares_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContextGroup) __shares_setList(i QOpenGLContext_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContextGroup___shares_setList(ptr.Pointer(), PointerFromQOpenGLContext(i))
	}
}

func (ptr *QOpenGLContextGroup) __shares_newList() unsafe.Pointer {
	return C.QOpenGLContextGroup___shares_newList(ptr.Pointer())
}

func (ptr *QOpenGLContextGroup) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLContextGroup___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContextGroup) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContextGroup___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLContextGroup) __children_newList() unsafe.Pointer {
	return C.QOpenGLContextGroup___children_newList(ptr.Pointer())
}

func (ptr *QOpenGLContextGroup) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QOpenGLContextGroup___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContextGroup) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContextGroup___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QOpenGLContextGroup) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QOpenGLContextGroup___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QOpenGLContextGroup) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLContextGroup___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContextGroup) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContextGroup___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLContextGroup) __findChildren_newList() unsafe.Pointer {
	return C.QOpenGLContextGroup___findChildren_newList(ptr.Pointer())
}

func (ptr *QOpenGLContextGroup) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLContextGroup___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLContextGroup) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContextGroup___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLContextGroup) __findChildren_newList3() unsafe.Pointer {
	return C.QOpenGLContextGroup___findChildren_newList3(ptr.Pointer())
}

//export callbackQOpenGLContextGroup_ChildEvent
func callbackQOpenGLContextGroup_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQOpenGLContextGroupFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QOpenGLContextGroup) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContextGroup_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQOpenGLContextGroup_ConnectNotify
func callbackQOpenGLContextGroup_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLContextGroupFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLContextGroup) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContextGroup_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLContextGroup_CustomEvent
func callbackQOpenGLContextGroup_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQOpenGLContextGroupFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QOpenGLContextGroup) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContextGroup_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQOpenGLContextGroup_DeleteLater
func callbackQOpenGLContextGroup_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLContextGroupFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QOpenGLContextGroup) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLContextGroup_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLContextGroup_Destroyed
func callbackQOpenGLContextGroup_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQOpenGLContextGroup_DisconnectNotify
func callbackQOpenGLContextGroup_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLContextGroupFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLContextGroup) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContextGroup_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLContextGroup_Event
func callbackQOpenGLContextGroup_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLContextGroupFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QOpenGLContextGroup) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLContextGroup_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQOpenGLContextGroup_EventFilter
func callbackQOpenGLContextGroup_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLContextGroupFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QOpenGLContextGroup) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLContextGroup_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQOpenGLContextGroup_MetaObject
func callbackQOpenGLContextGroup_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQOpenGLContextGroupFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QOpenGLContextGroup) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QOpenGLContextGroup_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQOpenGLContextGroup_ObjectNameChanged
func callbackQOpenGLContextGroup_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQOpenGLContextGroup_TimerEvent
func callbackQOpenGLContextGroup_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQOpenGLContextGroupFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QOpenGLContextGroup) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLContextGroup_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QOpenGLDebugLogger struct {
	core.QObject
}

type QOpenGLDebugLogger_ITF interface {
	core.QObject_ITF
	QOpenGLDebugLogger_PTR() *QOpenGLDebugLogger
}

func (ptr *QOpenGLDebugLogger) QOpenGLDebugLogger_PTR() *QOpenGLDebugLogger {
	return ptr
}

func (ptr *QOpenGLDebugLogger) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLDebugLogger) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLDebugLogger(ptr QOpenGLDebugLogger_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLDebugLogger_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLDebugLoggerFromPointer(ptr unsafe.Pointer) (n *QOpenGLDebugLogger) {
	n = new(QOpenGLDebugLogger)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QOpenGLDebugLogger__LoggingMode
//QOpenGLDebugLogger::LoggingMode
type QOpenGLDebugLogger__LoggingMode int64

const (
	QOpenGLDebugLogger__AsynchronousLogging QOpenGLDebugLogger__LoggingMode = QOpenGLDebugLogger__LoggingMode(0)
	QOpenGLDebugLogger__SynchronousLogging  QOpenGLDebugLogger__LoggingMode = QOpenGLDebugLogger__LoggingMode(1)
)

func NewQOpenGLDebugLogger(parent core.QObject_ITF) *QOpenGLDebugLogger {
	tmpValue := NewQOpenGLDebugLoggerFromPointer(C.QOpenGLDebugLogger_NewQOpenGLDebugLogger(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLDebugLogger) DisableMessages(sources QOpenGLDebugMessage__Source, types QOpenGLDebugMessage__Type, severities QOpenGLDebugMessage__Severity) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_DisableMessages(ptr.Pointer(), C.longlong(sources), C.longlong(types), C.longlong(severities))
	}
}

func (ptr *QOpenGLDebugLogger) DisableMessages2(ids []uint, sources QOpenGLDebugMessage__Source, types QOpenGLDebugMessage__Type) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_DisableMessages2(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQOpenGLDebugLoggerFromPointer(NewQOpenGLDebugLoggerFromPointer(nil).__disableMessages_ids_newList2())
			for _, v := range ids {
				tmpList.__disableMessages_ids_setList2(v)
			}
			return tmpList.Pointer()
		}(), C.longlong(sources), C.longlong(types))
	}
}

func (ptr *QOpenGLDebugLogger) EnableMessages(sources QOpenGLDebugMessage__Source, types QOpenGLDebugMessage__Type, severities QOpenGLDebugMessage__Severity) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_EnableMessages(ptr.Pointer(), C.longlong(sources), C.longlong(types), C.longlong(severities))
	}
}

func (ptr *QOpenGLDebugLogger) EnableMessages2(ids []uint, sources QOpenGLDebugMessage__Source, types QOpenGLDebugMessage__Type) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_EnableMessages2(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQOpenGLDebugLoggerFromPointer(NewQOpenGLDebugLoggerFromPointer(nil).__enableMessages_ids_newList2())
			for _, v := range ids {
				tmpList.__enableMessages_ids_setList2(v)
			}
			return tmpList.Pointer()
		}(), C.longlong(sources), C.longlong(types))
	}
}

func (ptr *QOpenGLDebugLogger) Initialize() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLDebugLogger_Initialize(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLDebugLogger) IsLogging() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLDebugLogger_IsLogging(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQOpenGLDebugLogger_LogMessage
func callbackQOpenGLDebugLogger_LogMessage(ptr unsafe.Pointer, debugMessage unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "logMessage"); signal != nil {
		(*(*func(*QOpenGLDebugMessage))(signal))(NewQOpenGLDebugMessageFromPointer(debugMessage))
	} else {
		NewQOpenGLDebugLoggerFromPointer(ptr).LogMessageDefault(NewQOpenGLDebugMessageFromPointer(debugMessage))
	}
}

func (ptr *QOpenGLDebugLogger) ConnectLogMessage(f func(debugMessage *QOpenGLDebugMessage)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "logMessage"); signal != nil {
			f := func(debugMessage *QOpenGLDebugMessage) {
				(*(*func(*QOpenGLDebugMessage))(signal))(debugMessage)
				f(debugMessage)
			}
			qt.ConnectSignal(ptr.Pointer(), "logMessage", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "logMessage", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLDebugLogger) DisconnectLogMessage() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "logMessage")
	}
}

func (ptr *QOpenGLDebugLogger) LogMessage(debugMessage QOpenGLDebugMessage_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_LogMessage(ptr.Pointer(), PointerFromQOpenGLDebugMessage(debugMessage))
	}
}

func (ptr *QOpenGLDebugLogger) LogMessageDefault(debugMessage QOpenGLDebugMessage_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_LogMessageDefault(ptr.Pointer(), PointerFromQOpenGLDebugMessage(debugMessage))
	}
}

func (ptr *QOpenGLDebugLogger) LoggedMessages() []*QOpenGLDebugMessage {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QOpenGLDebugMessage {
			out := make([]*QOpenGLDebugMessage, int(l.len))
			tmpList := NewQOpenGLDebugLoggerFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__loggedMessages_atList(i)
			}
			return out
		}(C.QOpenGLDebugLogger_LoggedMessages(ptr.Pointer()))
	}
	return make([]*QOpenGLDebugMessage, 0)
}

func (ptr *QOpenGLDebugLogger) LoggingMode() QOpenGLDebugLogger__LoggingMode {
	if ptr.Pointer() != nil {
		return QOpenGLDebugLogger__LoggingMode(C.QOpenGLDebugLogger_LoggingMode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLDebugLogger) MaximumMessageLength() int64 {
	if ptr.Pointer() != nil {
		return int64(C.QOpenGLDebugLogger_MaximumMessageLength(ptr.Pointer()))
	}
	return 0
}

//export callbackQOpenGLDebugLogger_MessageLogged
func callbackQOpenGLDebugLogger_MessageLogged(ptr unsafe.Pointer, debugMessage unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "messageLogged"); signal != nil {
		(*(*func(*QOpenGLDebugMessage))(signal))(NewQOpenGLDebugMessageFromPointer(debugMessage))
	}

}

func (ptr *QOpenGLDebugLogger) ConnectMessageLogged(f func(debugMessage *QOpenGLDebugMessage)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "messageLogged") {
			C.QOpenGLDebugLogger_ConnectMessageLogged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "messageLogged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "messageLogged"); signal != nil {
			f := func(debugMessage *QOpenGLDebugMessage) {
				(*(*func(*QOpenGLDebugMessage))(signal))(debugMessage)
				f(debugMessage)
			}
			qt.ConnectSignal(ptr.Pointer(), "messageLogged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "messageLogged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLDebugLogger) DisconnectMessageLogged() {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_DisconnectMessageLogged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "messageLogged")
	}
}

func (ptr *QOpenGLDebugLogger) MessageLogged(debugMessage QOpenGLDebugMessage_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_MessageLogged(ptr.Pointer(), PointerFromQOpenGLDebugMessage(debugMessage))
	}
}

func (ptr *QOpenGLDebugLogger) PopGroup() {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_PopGroup(ptr.Pointer())
	}
}

func (ptr *QOpenGLDebugLogger) PushGroup(name string, id uint, source QOpenGLDebugMessage__Source) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLDebugLogger_PushGroup(ptr.Pointer(), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}, C.uint(uint32(id)), C.longlong(source))
	}
}

//export callbackQOpenGLDebugLogger_StartLogging
func callbackQOpenGLDebugLogger_StartLogging(ptr unsafe.Pointer, loggingMode C.longlong) {
	if signal := qt.GetSignal(ptr, "startLogging"); signal != nil {
		(*(*func(QOpenGLDebugLogger__LoggingMode))(signal))(QOpenGLDebugLogger__LoggingMode(loggingMode))
	} else {
		NewQOpenGLDebugLoggerFromPointer(ptr).StartLoggingDefault(QOpenGLDebugLogger__LoggingMode(loggingMode))
	}
}

func (ptr *QOpenGLDebugLogger) ConnectStartLogging(f func(loggingMode QOpenGLDebugLogger__LoggingMode)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "startLogging"); signal != nil {
			f := func(loggingMode QOpenGLDebugLogger__LoggingMode) {
				(*(*func(QOpenGLDebugLogger__LoggingMode))(signal))(loggingMode)
				f(loggingMode)
			}
			qt.ConnectSignal(ptr.Pointer(), "startLogging", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "startLogging", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLDebugLogger) DisconnectStartLogging() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "startLogging")
	}
}

func (ptr *QOpenGLDebugLogger) StartLogging(loggingMode QOpenGLDebugLogger__LoggingMode) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_StartLogging(ptr.Pointer(), C.longlong(loggingMode))
	}
}

func (ptr *QOpenGLDebugLogger) StartLoggingDefault(loggingMode QOpenGLDebugLogger__LoggingMode) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_StartLoggingDefault(ptr.Pointer(), C.longlong(loggingMode))
	}
}

//export callbackQOpenGLDebugLogger_StopLogging
func callbackQOpenGLDebugLogger_StopLogging(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "stopLogging"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLDebugLoggerFromPointer(ptr).StopLoggingDefault()
	}
}

func (ptr *QOpenGLDebugLogger) ConnectStopLogging(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "stopLogging"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "stopLogging", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "stopLogging", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLDebugLogger) DisconnectStopLogging() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "stopLogging")
	}
}

func (ptr *QOpenGLDebugLogger) StopLogging() {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_StopLogging(ptr.Pointer())
	}
}

func (ptr *QOpenGLDebugLogger) StopLoggingDefault() {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_StopLoggingDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLDebugLogger_DestroyQOpenGLDebugLogger
func callbackQOpenGLDebugLogger_DestroyQOpenGLDebugLogger(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOpenGLDebugLogger"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLDebugLoggerFromPointer(ptr).DestroyQOpenGLDebugLoggerDefault()
	}
}

func (ptr *QOpenGLDebugLogger) ConnectDestroyQOpenGLDebugLogger(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOpenGLDebugLogger"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLDebugLogger", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLDebugLogger", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLDebugLogger) DisconnectDestroyQOpenGLDebugLogger() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOpenGLDebugLogger")
	}
}

func (ptr *QOpenGLDebugLogger) DestroyQOpenGLDebugLogger() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLDebugLogger_DestroyQOpenGLDebugLogger(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLDebugLogger) DestroyQOpenGLDebugLoggerDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLDebugLogger_DestroyQOpenGLDebugLoggerDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLDebugLogger) __disableMessages_ids_atList2(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLDebugLogger___disableMessages_ids_atList2(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QOpenGLDebugLogger) __disableMessages_ids_setList2(i uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger___disableMessages_ids_setList2(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QOpenGLDebugLogger) __disableMessages_ids_newList2() unsafe.Pointer {
	return C.QOpenGLDebugLogger___disableMessages_ids_newList2(ptr.Pointer())
}

func (ptr *QOpenGLDebugLogger) __enableMessages_ids_atList2(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLDebugLogger___enableMessages_ids_atList2(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QOpenGLDebugLogger) __enableMessages_ids_setList2(i uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger___enableMessages_ids_setList2(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QOpenGLDebugLogger) __enableMessages_ids_newList2() unsafe.Pointer {
	return C.QOpenGLDebugLogger___enableMessages_ids_newList2(ptr.Pointer())
}

func (ptr *QOpenGLDebugLogger) __loggedMessages_atList(i int) *QOpenGLDebugMessage {
	if ptr.Pointer() != nil {
		tmpValue := NewQOpenGLDebugMessageFromPointer(C.QOpenGLDebugLogger___loggedMessages_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QOpenGLDebugMessage).DestroyQOpenGLDebugMessage)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLDebugLogger) __loggedMessages_setList(i QOpenGLDebugMessage_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger___loggedMessages_setList(ptr.Pointer(), PointerFromQOpenGLDebugMessage(i))
	}
}

func (ptr *QOpenGLDebugLogger) __loggedMessages_newList() unsafe.Pointer {
	return C.QOpenGLDebugLogger___loggedMessages_newList(ptr.Pointer())
}

func (ptr *QOpenGLDebugLogger) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLDebugLogger___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLDebugLogger) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLDebugLogger) __children_newList() unsafe.Pointer {
	return C.QOpenGLDebugLogger___children_newList(ptr.Pointer())
}

func (ptr *QOpenGLDebugLogger) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QOpenGLDebugLogger___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLDebugLogger) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QOpenGLDebugLogger) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QOpenGLDebugLogger___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QOpenGLDebugLogger) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLDebugLogger___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLDebugLogger) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLDebugLogger) __findChildren_newList() unsafe.Pointer {
	return C.QOpenGLDebugLogger___findChildren_newList(ptr.Pointer())
}

func (ptr *QOpenGLDebugLogger) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLDebugLogger___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLDebugLogger) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLDebugLogger) __findChildren_newList3() unsafe.Pointer {
	return C.QOpenGLDebugLogger___findChildren_newList3(ptr.Pointer())
}

//export callbackQOpenGLDebugLogger_ChildEvent
func callbackQOpenGLDebugLogger_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQOpenGLDebugLoggerFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QOpenGLDebugLogger) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQOpenGLDebugLogger_ConnectNotify
func callbackQOpenGLDebugLogger_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLDebugLoggerFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLDebugLogger) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLDebugLogger_CustomEvent
func callbackQOpenGLDebugLogger_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQOpenGLDebugLoggerFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QOpenGLDebugLogger) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQOpenGLDebugLogger_DeleteLater
func callbackQOpenGLDebugLogger_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLDebugLoggerFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QOpenGLDebugLogger) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLDebugLogger_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLDebugLogger_Destroyed
func callbackQOpenGLDebugLogger_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQOpenGLDebugLogger_DisconnectNotify
func callbackQOpenGLDebugLogger_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLDebugLoggerFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLDebugLogger) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLDebugLogger_Event
func callbackQOpenGLDebugLogger_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLDebugLoggerFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QOpenGLDebugLogger) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLDebugLogger_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQOpenGLDebugLogger_EventFilter
func callbackQOpenGLDebugLogger_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLDebugLoggerFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QOpenGLDebugLogger) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLDebugLogger_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQOpenGLDebugLogger_MetaObject
func callbackQOpenGLDebugLogger_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQOpenGLDebugLoggerFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QOpenGLDebugLogger) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QOpenGLDebugLogger_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQOpenGLDebugLogger_ObjectNameChanged
func callbackQOpenGLDebugLogger_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQOpenGLDebugLogger_TimerEvent
func callbackQOpenGLDebugLogger_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQOpenGLDebugLoggerFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QOpenGLDebugLogger) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugLogger_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QOpenGLDebugMessage struct {
	ptr unsafe.Pointer
}

type QOpenGLDebugMessage_ITF interface {
	QOpenGLDebugMessage_PTR() *QOpenGLDebugMessage
}

func (ptr *QOpenGLDebugMessage) QOpenGLDebugMessage_PTR() *QOpenGLDebugMessage {
	return ptr
}

func (ptr *QOpenGLDebugMessage) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLDebugMessage) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLDebugMessage(ptr QOpenGLDebugMessage_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLDebugMessage_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLDebugMessageFromPointer(ptr unsafe.Pointer) (n *QOpenGLDebugMessage) {
	n = new(QOpenGLDebugMessage)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QOpenGLDebugMessage__Source
//QOpenGLDebugMessage::Source
type QOpenGLDebugMessage__Source int64

const (
	QOpenGLDebugMessage__InvalidSource        QOpenGLDebugMessage__Source = QOpenGLDebugMessage__Source(0x00000000)
	QOpenGLDebugMessage__APISource            QOpenGLDebugMessage__Source = QOpenGLDebugMessage__Source(0x00000001)
	QOpenGLDebugMessage__WindowSystemSource   QOpenGLDebugMessage__Source = QOpenGLDebugMessage__Source(0x00000002)
	QOpenGLDebugMessage__ShaderCompilerSource QOpenGLDebugMessage__Source = QOpenGLDebugMessage__Source(0x00000004)
	QOpenGLDebugMessage__ThirdPartySource     QOpenGLDebugMessage__Source = QOpenGLDebugMessage__Source(0x00000008)
	QOpenGLDebugMessage__ApplicationSource    QOpenGLDebugMessage__Source = QOpenGLDebugMessage__Source(0x00000010)
	QOpenGLDebugMessage__OtherSource          QOpenGLDebugMessage__Source = QOpenGLDebugMessage__Source(0x00000020)
	QOpenGLDebugMessage__LastSource           QOpenGLDebugMessage__Source = QOpenGLDebugMessage__Source(QOpenGLDebugMessage__OtherSource)
	QOpenGLDebugMessage__AnySource            QOpenGLDebugMessage__Source = QOpenGLDebugMessage__Source(0xffffffff)
)

//go:generate stringer -type=QOpenGLDebugMessage__Type
//QOpenGLDebugMessage::Type
type QOpenGLDebugMessage__Type int64

const (
	QOpenGLDebugMessage__InvalidType            QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0x00000000)
	QOpenGLDebugMessage__ErrorType              QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0x00000001)
	QOpenGLDebugMessage__DeprecatedBehaviorType QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0x00000002)
	QOpenGLDebugMessage__UndefinedBehaviorType  QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0x00000004)
	QOpenGLDebugMessage__PortabilityType        QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0x00000008)
	QOpenGLDebugMessage__PerformanceType        QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0x00000010)
	QOpenGLDebugMessage__OtherType              QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0x00000020)
	QOpenGLDebugMessage__MarkerType             QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0x00000040)
	QOpenGLDebugMessage__GroupPushType          QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0x00000080)
	QOpenGLDebugMessage__GroupPopType           QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0x00000100)
	QOpenGLDebugMessage__LastType               QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(QOpenGLDebugMessage__GroupPopType)
	QOpenGLDebugMessage__AnyType                QOpenGLDebugMessage__Type = QOpenGLDebugMessage__Type(0xffffffff)
)

//go:generate stringer -type=QOpenGLDebugMessage__Severity
//QOpenGLDebugMessage::Severity
type QOpenGLDebugMessage__Severity int64

const (
	QOpenGLDebugMessage__InvalidSeverity      QOpenGLDebugMessage__Severity = QOpenGLDebugMessage__Severity(0x00000000)
	QOpenGLDebugMessage__HighSeverity         QOpenGLDebugMessage__Severity = QOpenGLDebugMessage__Severity(0x00000001)
	QOpenGLDebugMessage__MediumSeverity       QOpenGLDebugMessage__Severity = QOpenGLDebugMessage__Severity(0x00000002)
	QOpenGLDebugMessage__LowSeverity          QOpenGLDebugMessage__Severity = QOpenGLDebugMessage__Severity(0x00000004)
	QOpenGLDebugMessage__NotificationSeverity QOpenGLDebugMessage__Severity = QOpenGLDebugMessage__Severity(0x00000008)
	QOpenGLDebugMessage__LastSeverity         QOpenGLDebugMessage__Severity = QOpenGLDebugMessage__Severity(QOpenGLDebugMessage__NotificationSeverity)
	QOpenGLDebugMessage__AnySeverity          QOpenGLDebugMessage__Severity = QOpenGLDebugMessage__Severity(0xffffffff)
)

func NewQOpenGLDebugMessage() *QOpenGLDebugMessage {
	tmpValue := NewQOpenGLDebugMessageFromPointer(C.QOpenGLDebugMessage_NewQOpenGLDebugMessage())
	qt.SetFinalizer(tmpValue, (*QOpenGLDebugMessage).DestroyQOpenGLDebugMessage)
	return tmpValue
}

func NewQOpenGLDebugMessage2(debugMessage QOpenGLDebugMessage_ITF) *QOpenGLDebugMessage {
	tmpValue := NewQOpenGLDebugMessageFromPointer(C.QOpenGLDebugMessage_NewQOpenGLDebugMessage2(PointerFromQOpenGLDebugMessage(debugMessage)))
	qt.SetFinalizer(tmpValue, (*QOpenGLDebugMessage).DestroyQOpenGLDebugMessage)
	return tmpValue
}

func QOpenGLDebugMessage_CreateApplicationMessage(text string, id uint, severity QOpenGLDebugMessage__Severity, ty QOpenGLDebugMessage__Type) *QOpenGLDebugMessage {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQOpenGLDebugMessageFromPointer(C.QOpenGLDebugMessage_QOpenGLDebugMessage_CreateApplicationMessage(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.uint(uint32(id)), C.longlong(severity), C.longlong(ty)))
	qt.SetFinalizer(tmpValue, (*QOpenGLDebugMessage).DestroyQOpenGLDebugMessage)
	return tmpValue
}

func (ptr *QOpenGLDebugMessage) CreateApplicationMessage(text string, id uint, severity QOpenGLDebugMessage__Severity, ty QOpenGLDebugMessage__Type) *QOpenGLDebugMessage {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQOpenGLDebugMessageFromPointer(C.QOpenGLDebugMessage_QOpenGLDebugMessage_CreateApplicationMessage(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.uint(uint32(id)), C.longlong(severity), C.longlong(ty)))
	qt.SetFinalizer(tmpValue, (*QOpenGLDebugMessage).DestroyQOpenGLDebugMessage)
	return tmpValue
}

func QOpenGLDebugMessage_CreateThirdPartyMessage(text string, id uint, severity QOpenGLDebugMessage__Severity, ty QOpenGLDebugMessage__Type) *QOpenGLDebugMessage {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQOpenGLDebugMessageFromPointer(C.QOpenGLDebugMessage_QOpenGLDebugMessage_CreateThirdPartyMessage(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.uint(uint32(id)), C.longlong(severity), C.longlong(ty)))
	qt.SetFinalizer(tmpValue, (*QOpenGLDebugMessage).DestroyQOpenGLDebugMessage)
	return tmpValue
}

func (ptr *QOpenGLDebugMessage) CreateThirdPartyMessage(text string, id uint, severity QOpenGLDebugMessage__Severity, ty QOpenGLDebugMessage__Type) *QOpenGLDebugMessage {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQOpenGLDebugMessageFromPointer(C.QOpenGLDebugMessage_QOpenGLDebugMessage_CreateThirdPartyMessage(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.uint(uint32(id)), C.longlong(severity), C.longlong(ty)))
	qt.SetFinalizer(tmpValue, (*QOpenGLDebugMessage).DestroyQOpenGLDebugMessage)
	return tmpValue
}

func (ptr *QOpenGLDebugMessage) Id() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLDebugMessage_Id(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLDebugMessage) Message() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QOpenGLDebugMessage_Message(ptr.Pointer()))
	}
	return ""
}

func (ptr *QOpenGLDebugMessage) Severity() QOpenGLDebugMessage__Severity {
	if ptr.Pointer() != nil {
		return QOpenGLDebugMessage__Severity(C.QOpenGLDebugMessage_Severity(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLDebugMessage) Source() QOpenGLDebugMessage__Source {
	if ptr.Pointer() != nil {
		return QOpenGLDebugMessage__Source(C.QOpenGLDebugMessage_Source(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLDebugMessage) Swap(debugMessage QOpenGLDebugMessage_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLDebugMessage_Swap(ptr.Pointer(), PointerFromQOpenGLDebugMessage(debugMessage))
	}
}

func (ptr *QOpenGLDebugMessage) Type() QOpenGLDebugMessage__Type {
	if ptr.Pointer() != nil {
		return QOpenGLDebugMessage__Type(C.QOpenGLDebugMessage_Type(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLDebugMessage) DestroyQOpenGLDebugMessage() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLDebugMessage_DestroyQOpenGLDebugMessage(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLExtraFunctions struct {
	QOpenGLFunctions
}

type QOpenGLExtraFunctions_ITF interface {
	QOpenGLFunctions_ITF
	QOpenGLExtraFunctions_PTR() *QOpenGLExtraFunctions
}

func (ptr *QOpenGLExtraFunctions) QOpenGLExtraFunctions_PTR() *QOpenGLExtraFunctions {
	return ptr
}

func (ptr *QOpenGLExtraFunctions) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLExtraFunctions) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLExtraFunctions(ptr QOpenGLExtraFunctions_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLExtraFunctions_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLExtraFunctionsFromPointer(ptr unsafe.Pointer) (n *QOpenGLExtraFunctions) {
	n = new(QOpenGLExtraFunctions)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLExtraFunctions) DestroyQOpenGLExtraFunctions() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQOpenGLExtraFunctions() *QOpenGLExtraFunctions {
	tmpValue := NewQOpenGLExtraFunctionsFromPointer(C.QOpenGLExtraFunctions_NewQOpenGLExtraFunctions())
	qt.SetFinalizer(tmpValue, (*QOpenGLExtraFunctions).DestroyQOpenGLExtraFunctions)
	return tmpValue
}

func NewQOpenGLExtraFunctions2(context QOpenGLContext_ITF) *QOpenGLExtraFunctions {
	tmpValue := NewQOpenGLExtraFunctionsFromPointer(C.QOpenGLExtraFunctions_NewQOpenGLExtraFunctions2(PointerFromQOpenGLContext(context)))
	qt.SetFinalizer(tmpValue, (*QOpenGLExtraFunctions).DestroyQOpenGLExtraFunctions)
	return tmpValue
}

func (ptr *QOpenGLExtraFunctions) GlActiveShaderProgram(pipeline uint, program uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlActiveShaderProgram(ptr.Pointer(), C.uint(uint32(pipeline)), C.uint(uint32(program)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBeginQuery(target uint, id uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBeginQuery(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(id)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBeginTransformFeedback(primitiveMode uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBeginTransformFeedback(ptr.Pointer(), C.uint(uint32(primitiveMode)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBindBufferBase(target uint, index uint, buffer uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBindBufferBase(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(index)), C.uint(uint32(buffer)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBindImageTexture(unit uint, texture uint, level int, layered bool, layer int, access uint, format uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBindImageTexture(ptr.Pointer(), C.uint(uint32(unit)), C.uint(uint32(texture)), C.int(int32(level)), C.char(int8(qt.GoBoolToInt(layered))), C.int(int32(layer)), C.uint(uint32(access)), C.uint(uint32(format)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBindProgramPipeline(pipeline uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBindProgramPipeline(ptr.Pointer(), C.uint(uint32(pipeline)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBindSampler(unit uint, sampler uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBindSampler(ptr.Pointer(), C.uint(uint32(unit)), C.uint(uint32(sampler)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBindTransformFeedback(target uint, id uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBindTransformFeedback(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(id)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBindVertexArray(array uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBindVertexArray(ptr.Pointer(), C.uint(uint32(array)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBlendBarrier() {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBlendBarrier(ptr.Pointer())
	}
}

func (ptr *QOpenGLExtraFunctions) GlBlendEquationSeparatei(buf uint, modeRGB uint, modeAlpha uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBlendEquationSeparatei(ptr.Pointer(), C.uint(uint32(buf)), C.uint(uint32(modeRGB)), C.uint(uint32(modeAlpha)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBlendEquationi(buf uint, mode uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBlendEquationi(ptr.Pointer(), C.uint(uint32(buf)), C.uint(uint32(mode)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBlendFuncSeparatei(buf uint, srcRGB uint, dstRGB uint, srcAlpha uint, dstAlpha uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBlendFuncSeparatei(ptr.Pointer(), C.uint(uint32(buf)), C.uint(uint32(srcRGB)), C.uint(uint32(dstRGB)), C.uint(uint32(srcAlpha)), C.uint(uint32(dstAlpha)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBlendFunci(buf uint, src uint, dst uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBlendFunci(ptr.Pointer(), C.uint(uint32(buf)), C.uint(uint32(src)), C.uint(uint32(dst)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlBlitFramebuffer(srcX0 int, srcY0 int, srcX1 int, srcY1 int, dstX0 int, dstY0 int, dstX1 int, dstY1 int, mask uint, filter uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlBlitFramebuffer(ptr.Pointer(), C.int(int32(srcX0)), C.int(int32(srcY0)), C.int(int32(srcX1)), C.int(int32(srcY1)), C.int(int32(dstX0)), C.int(int32(dstY0)), C.int(int32(dstX1)), C.int(int32(dstY1)), C.uint(uint32(mask)), C.uint(uint32(filter)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlClearBufferfi(buffer uint, drawbuffer int, depth float32, stencil int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlClearBufferfi(ptr.Pointer(), C.uint(uint32(buffer)), C.int(int32(drawbuffer)), C.float(depth), C.int(int32(stencil)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlClearBufferfv(buffer uint, drawbuffer int, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlClearBufferfv(ptr.Pointer(), C.uint(uint32(buffer)), C.int(int32(drawbuffer)), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlClearBufferiv(buffer uint, drawbuffer int, value int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlClearBufferiv(ptr.Pointer(), C.uint(uint32(buffer)), C.int(int32(drawbuffer)), C.int(int32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlClearBufferuiv(buffer uint, drawbuffer int, value uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlClearBufferuiv(ptr.Pointer(), C.uint(uint32(buffer)), C.int(int32(drawbuffer)), C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlColorMaski(index uint, r bool, g bool, b bool, a bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlColorMaski(ptr.Pointer(), C.uint(uint32(index)), C.char(int8(qt.GoBoolToInt(r))), C.char(int8(qt.GoBoolToInt(g))), C.char(int8(qt.GoBoolToInt(b))), C.char(int8(qt.GoBoolToInt(a))))
	}
}

func (ptr *QOpenGLExtraFunctions) GlCompressedTexImage3D(target uint, level int, internalformat uint, width int, height int, depth int, border int, imageSize int, data unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlCompressedTexImage3D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.uint(uint32(internalformat)), C.int(int32(width)), C.int(int32(height)), C.int(int32(depth)), C.int(int32(border)), C.int(int32(imageSize)), data)
	}
}

func (ptr *QOpenGLExtraFunctions) GlCompressedTexSubImage3D(target uint, level int, xoffset int, yoffset int, zoffset int, width int, height int, depth int, format uint, imageSize int, data unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlCompressedTexSubImage3D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.int(int32(xoffset)), C.int(int32(yoffset)), C.int(int32(zoffset)), C.int(int32(width)), C.int(int32(height)), C.int(int32(depth)), C.uint(uint32(format)), C.int(int32(imageSize)), data)
	}
}

func (ptr *QOpenGLExtraFunctions) GlCopyImageSubData(srcName uint, srcTarget uint, srcLevel int, srcX int, srcY int, srcZ int, dstName uint, dstTarget uint, dstLevel int, dstX int, dstY int, dstZ int, srcWidth int, srcHeight int, srcDepth int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlCopyImageSubData(ptr.Pointer(), C.uint(uint32(srcName)), C.uint(uint32(srcTarget)), C.int(int32(srcLevel)), C.int(int32(srcX)), C.int(int32(srcY)), C.int(int32(srcZ)), C.uint(uint32(dstName)), C.uint(uint32(dstTarget)), C.int(int32(dstLevel)), C.int(int32(dstX)), C.int(int32(dstY)), C.int(int32(dstZ)), C.int(int32(srcWidth)), C.int(int32(srcHeight)), C.int(int32(srcDepth)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlCopyTexSubImage3D(target uint, level int, xoffset int, yoffset int, zoffset int, x int, y int, width int, height int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlCopyTexSubImage3D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.int(int32(xoffset)), C.int(int32(yoffset)), C.int(int32(zoffset)), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDebugMessageControl(source uint, ty uint, severity uint, count int, ids uint, enabled bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDebugMessageControl(ptr.Pointer(), C.uint(uint32(source)), C.uint(uint32(ty)), C.uint(uint32(severity)), C.int(int32(count)), C.uint(uint32(ids)), C.char(int8(qt.GoBoolToInt(enabled))))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDeleteProgramPipelines(n int, pipelines uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDeleteProgramPipelines(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(pipelines)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDeleteQueries(n int, ids uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDeleteQueries(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(ids)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDeleteSamplers(count int, samplers uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDeleteSamplers(ptr.Pointer(), C.int(int32(count)), C.uint(uint32(samplers)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDeleteTransformFeedbacks(n int, ids uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDeleteTransformFeedbacks(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(ids)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDeleteVertexArrays(n int, arrays uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDeleteVertexArrays(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(arrays)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDisablei(target uint, index uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDisablei(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(index)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDispatchCompute(num_groups_x uint, num_groups_y uint, num_groups_z uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDispatchCompute(ptr.Pointer(), C.uint(uint32(num_groups_x)), C.uint(uint32(num_groups_y)), C.uint(uint32(num_groups_z)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDrawArraysIndirect(mode uint, indirect unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDrawArraysIndirect(ptr.Pointer(), C.uint(uint32(mode)), indirect)
	}
}

func (ptr *QOpenGLExtraFunctions) GlDrawArraysInstanced(mode uint, first int, count int, instancecount int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDrawArraysInstanced(ptr.Pointer(), C.uint(uint32(mode)), C.int(int32(first)), C.int(int32(count)), C.int(int32(instancecount)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDrawBuffers(n int, bufs uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDrawBuffers(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(bufs)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDrawElementsBaseVertex(mode uint, count int, ty uint, indices unsafe.Pointer, basevertex int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDrawElementsBaseVertex(ptr.Pointer(), C.uint(uint32(mode)), C.int(int32(count)), C.uint(uint32(ty)), indices, C.int(int32(basevertex)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDrawElementsIndirect(mode uint, ty uint, indirect unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDrawElementsIndirect(ptr.Pointer(), C.uint(uint32(mode)), C.uint(uint32(ty)), indirect)
	}
}

func (ptr *QOpenGLExtraFunctions) GlDrawElementsInstanced(mode uint, count int, ty uint, indices unsafe.Pointer, instancecount int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDrawElementsInstanced(ptr.Pointer(), C.uint(uint32(mode)), C.int(int32(count)), C.uint(uint32(ty)), indices, C.int(int32(instancecount)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDrawElementsInstancedBaseVertex(mode uint, count int, ty uint, indices unsafe.Pointer, instancecount int, basevertex int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDrawElementsInstancedBaseVertex(ptr.Pointer(), C.uint(uint32(mode)), C.int(int32(count)), C.uint(uint32(ty)), indices, C.int(int32(instancecount)), C.int(int32(basevertex)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlDrawRangeElements(mode uint, start uint, end uint, count int, ty uint, indices unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDrawRangeElements(ptr.Pointer(), C.uint(uint32(mode)), C.uint(uint32(start)), C.uint(uint32(end)), C.int(int32(count)), C.uint(uint32(ty)), indices)
	}
}

func (ptr *QOpenGLExtraFunctions) GlDrawRangeElementsBaseVertex(mode uint, start uint, end uint, count int, ty uint, indices unsafe.Pointer, basevertex int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlDrawRangeElementsBaseVertex(ptr.Pointer(), C.uint(uint32(mode)), C.uint(uint32(start)), C.uint(uint32(end)), C.int(int32(count)), C.uint(uint32(ty)), indices, C.int(int32(basevertex)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlEnablei(target uint, index uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlEnablei(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(index)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlEndQuery(target uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlEndQuery(ptr.Pointer(), C.uint(uint32(target)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlEndTransformFeedback() {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlEndTransformFeedback(ptr.Pointer())
	}
}

func (ptr *QOpenGLExtraFunctions) GlFramebufferParameteri(target uint, pname uint, param int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlFramebufferParameteri(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.int(int32(param)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlFramebufferTexture(target uint, attachment uint, texture uint, level int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlFramebufferTexture(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(attachment)), C.uint(uint32(texture)), C.int(int32(level)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlFramebufferTextureLayer(target uint, attachment uint, texture uint, level int, layer int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlFramebufferTextureLayer(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(attachment)), C.uint(uint32(texture)), C.int(int32(level)), C.int(int32(layer)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGenProgramPipelines(n int, pipelines uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGenProgramPipelines(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(pipelines)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGenQueries(n int, ids uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGenQueries(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(ids)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGenSamplers(count int, samplers uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGenSamplers(ptr.Pointer(), C.int(int32(count)), C.uint(uint32(samplers)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGenTransformFeedbacks(n int, ids uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGenTransformFeedbacks(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(ids)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGenVertexArrays(n int, arrays uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGenVertexArrays(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(arrays)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetActiveUniformBlockiv(program uint, uniformBlockIndex uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetActiveUniformBlockiv(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(uniformBlockIndex)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetActiveUniformsiv(program uint, uniformCount int, uniformIndices uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetActiveUniformsiv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(uniformCount)), C.uint(uint32(uniformIndices)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetBooleani_v(target uint, index uint, data *bool) {
	if ptr.Pointer() != nil {
		var dataC C.char
		if data != nil {
			dataC = C.char(int8(qt.GoBoolToInt(*data)))
			defer func() { *data = int8(dataC) != 0 }()
		}
		C.QOpenGLExtraFunctions_GlGetBooleani_v(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(index)), &dataC)
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetBufferPointerv(target uint, pname uint, params unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetBufferPointerv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), params)
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetFramebufferParameteriv(target uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetFramebufferParameteriv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetGraphicsResetStatus() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLExtraFunctions_GlGetGraphicsResetStatus(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLExtraFunctions) GlGetIntegeri_v(target uint, index uint, data int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetIntegeri_v(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(index)), C.int(int32(data)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetInternalformativ(target uint, internalformat uint, pname uint, bufSize int, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetInternalformativ(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(internalformat)), C.uint(uint32(pname)), C.int(int32(bufSize)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetMultisamplefv(pname uint, index uint, val float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetMultisamplefv(ptr.Pointer(), C.uint(uint32(pname)), C.uint(uint32(index)), C.float(val))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetPointerv(pname uint, params unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetPointerv(ptr.Pointer(), C.uint(uint32(pname)), params)
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetProgramBinary(program uint, bufSize int, length int, binaryFormat uint, binary unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetProgramBinary(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(bufSize)), C.int(int32(length)), C.uint(uint32(binaryFormat)), binary)
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetProgramInterfaceiv(program uint, programInterface uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetProgramInterfaceiv(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(programInterface)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetProgramPipelineiv(pipeline uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetProgramPipelineiv(ptr.Pointer(), C.uint(uint32(pipeline)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetProgramResourceiv(program uint, programInterface uint, index uint, propCount int, props uint, bufSize int, length int, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetProgramResourceiv(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(programInterface)), C.uint(uint32(index)), C.int(int32(propCount)), C.uint(uint32(props)), C.int(int32(bufSize)), C.int(int32(length)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetQueryObjectuiv(id uint, pname uint, params uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetQueryObjectuiv(ptr.Pointer(), C.uint(uint32(id)), C.uint(uint32(pname)), C.uint(uint32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetQueryiv(target uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetQueryiv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetSamplerParameterIiv(sampler uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetSamplerParameterIiv(ptr.Pointer(), C.uint(uint32(sampler)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetSamplerParameterIuiv(sampler uint, pname uint, params uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetSamplerParameterIuiv(ptr.Pointer(), C.uint(uint32(sampler)), C.uint(uint32(pname)), C.uint(uint32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetSamplerParameterfv(sampler uint, pname uint, params float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetSamplerParameterfv(ptr.Pointer(), C.uint(uint32(sampler)), C.uint(uint32(pname)), C.float(params))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetSamplerParameteriv(sampler uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetSamplerParameteriv(ptr.Pointer(), C.uint(uint32(sampler)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetStringi(name uint, index uint) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QOpenGLExtraFunctions_GlGetStringi(ptr.Pointer(), C.uint(uint32(name)), C.uint(uint32(index))))
	}
	return ""
}

func (ptr *QOpenGLExtraFunctions) GlGetTexLevelParameterfv(target uint, level int, pname uint, params float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetTexLevelParameterfv(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.uint(uint32(pname)), C.float(params))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetTexLevelParameteriv(target uint, level int, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetTexLevelParameteriv(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetTexParameterIiv(target uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetTexParameterIiv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetTexParameterIuiv(target uint, pname uint, params uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetTexParameterIuiv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.uint(uint32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetUniformuiv(program uint, location int, params uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetUniformuiv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.uint(uint32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetVertexAttribIiv(index uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetVertexAttribIiv(ptr.Pointer(), C.uint(uint32(index)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetVertexAttribIuiv(index uint, pname uint, params uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetVertexAttribIuiv(ptr.Pointer(), C.uint(uint32(index)), C.uint(uint32(pname)), C.uint(uint32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetnUniformfv(program uint, location int, bufSize int, params float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetnUniformfv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(bufSize)), C.float(params))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetnUniformiv(program uint, location int, bufSize int, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetnUniformiv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(bufSize)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlGetnUniformuiv(program uint, location int, bufSize int, params uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlGetnUniformuiv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(bufSize)), C.uint(uint32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlInvalidateFramebuffer(target uint, numAttachments int, attachments uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlInvalidateFramebuffer(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(numAttachments)), C.uint(uint32(attachments)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlInvalidateSubFramebuffer(target uint, numAttachments int, attachments uint, x int, y int, width int, height int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlInvalidateSubFramebuffer(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(numAttachments)), C.uint(uint32(attachments)), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlIsEnabledi(target uint, index uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLExtraFunctions_GlIsEnabledi(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(index)))) != 0
	}
	return false
}

func (ptr *QOpenGLExtraFunctions) GlIsProgramPipeline(pipeline uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLExtraFunctions_GlIsProgramPipeline(ptr.Pointer(), C.uint(uint32(pipeline)))) != 0
	}
	return false
}

func (ptr *QOpenGLExtraFunctions) GlIsQuery(id uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLExtraFunctions_GlIsQuery(ptr.Pointer(), C.uint(uint32(id)))) != 0
	}
	return false
}

func (ptr *QOpenGLExtraFunctions) GlIsSampler(sampler uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLExtraFunctions_GlIsSampler(ptr.Pointer(), C.uint(uint32(sampler)))) != 0
	}
	return false
}

func (ptr *QOpenGLExtraFunctions) GlIsTransformFeedback(id uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLExtraFunctions_GlIsTransformFeedback(ptr.Pointer(), C.uint(uint32(id)))) != 0
	}
	return false
}

func (ptr *QOpenGLExtraFunctions) GlIsVertexArray(array uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLExtraFunctions_GlIsVertexArray(ptr.Pointer(), C.uint(uint32(array)))) != 0
	}
	return false
}

func (ptr *QOpenGLExtraFunctions) GlMemoryBarrier(barriers uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlMemoryBarrier(ptr.Pointer(), C.uint(uint32(barriers)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlMemoryBarrierByRegion(barriers uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlMemoryBarrierByRegion(ptr.Pointer(), C.uint(uint32(barriers)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlMinSampleShading(value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlMinSampleShading(ptr.Pointer(), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlPatchParameteri(pname uint, value int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlPatchParameteri(ptr.Pointer(), C.uint(uint32(pname)), C.int(int32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlPauseTransformFeedback() {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlPauseTransformFeedback(ptr.Pointer())
	}
}

func (ptr *QOpenGLExtraFunctions) GlPopDebugGroup() {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlPopDebugGroup(ptr.Pointer())
	}
}

func (ptr *QOpenGLExtraFunctions) GlPrimitiveBoundingBox(minX float32, minY float32, minZ float32, minW float32, maxX float32, maxY float32, maxZ float32, maxW float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlPrimitiveBoundingBox(ptr.Pointer(), C.float(minX), C.float(minY), C.float(minZ), C.float(minW), C.float(maxX), C.float(maxY), C.float(maxZ), C.float(maxW))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramBinary(program uint, binaryFormat uint, binary unsafe.Pointer, length int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramBinary(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(binaryFormat)), binary, C.int(int32(length)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramParameteri(program uint, pname uint, value int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramParameteri(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(pname)), C.int(int32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform1f(program uint, location int, v0 float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform1f(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.float(v0))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform1fv(program uint, location int, count int, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform1fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform1i(program uint, location int, v0 int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform1i(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(v0)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform1iv(program uint, location int, count int, value int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform1iv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.int(int32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform1ui(program uint, location int, v0 uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform1ui(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.uint(uint32(v0)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform1uiv(program uint, location int, count int, value uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform1uiv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform2f(program uint, location int, v0 float32, v1 float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform2f(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.float(v0), C.float(v1))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform2fv(program uint, location int, count int, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform2fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform2i(program uint, location int, v0 int, v1 int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform2i(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(v0)), C.int(int32(v1)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform2iv(program uint, location int, count int, value int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform2iv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.int(int32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform2ui(program uint, location int, v0 uint, v1 uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform2ui(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.uint(uint32(v0)), C.uint(uint32(v1)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform2uiv(program uint, location int, count int, value uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform2uiv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform3f(program uint, location int, v0 float32, v1 float32, v2 float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform3f(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.float(v0), C.float(v1), C.float(v2))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform3fv(program uint, location int, count int, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform3fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform3i(program uint, location int, v0 int, v1 int, v2 int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform3i(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(v0)), C.int(int32(v1)), C.int(int32(v2)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform3iv(program uint, location int, count int, value int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform3iv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.int(int32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform3ui(program uint, location int, v0 uint, v1 uint, v2 uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform3ui(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.uint(uint32(v0)), C.uint(uint32(v1)), C.uint(uint32(v2)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform3uiv(program uint, location int, count int, value uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform3uiv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform4f(program uint, location int, v0 float32, v1 float32, v2 float32, v3 float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform4f(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.float(v0), C.float(v1), C.float(v2), C.float(v3))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform4fv(program uint, location int, count int, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform4fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform4i(program uint, location int, v0 int, v1 int, v2 int, v3 int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform4i(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(v0)), C.int(int32(v1)), C.int(int32(v2)), C.int(int32(v3)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform4iv(program uint, location int, count int, value int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform4iv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.int(int32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform4ui(program uint, location int, v0 uint, v1 uint, v2 uint, v3 uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform4ui(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.uint(uint32(v0)), C.uint(uint32(v1)), C.uint(uint32(v2)), C.uint(uint32(v3)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniform4uiv(program uint, location int, count int, value uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniform4uiv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniformMatrix2fv(program uint, location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniformMatrix2fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniformMatrix2x3fv(program uint, location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniformMatrix2x3fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniformMatrix2x4fv(program uint, location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniformMatrix2x4fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniformMatrix3fv(program uint, location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniformMatrix3fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniformMatrix3x2fv(program uint, location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniformMatrix3x2fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniformMatrix3x4fv(program uint, location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniformMatrix3x4fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniformMatrix4fv(program uint, location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniformMatrix4fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniformMatrix4x2fv(program uint, location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniformMatrix4x2fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlProgramUniformMatrix4x3fv(program uint, location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlProgramUniformMatrix4x3fv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlReadBuffer(src uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlReadBuffer(ptr.Pointer(), C.uint(uint32(src)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlReadnPixels(x int, y int, width int, height int, format uint, ty uint, bufSize int, data unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlReadnPixels(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.uint(uint32(format)), C.uint(uint32(ty)), C.int(int32(bufSize)), data)
	}
}

func (ptr *QOpenGLExtraFunctions) GlRenderbufferStorageMultisample(target uint, samples int, internalformat uint, width int, height int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlRenderbufferStorageMultisample(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(samples)), C.uint(uint32(internalformat)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlResumeTransformFeedback() {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlResumeTransformFeedback(ptr.Pointer())
	}
}

func (ptr *QOpenGLExtraFunctions) GlSampleMaski(maskNumber uint, mask uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlSampleMaski(ptr.Pointer(), C.uint(uint32(maskNumber)), C.uint(uint32(mask)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlSamplerParameterIiv(sampler uint, pname uint, param int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlSamplerParameterIiv(ptr.Pointer(), C.uint(uint32(sampler)), C.uint(uint32(pname)), C.int(int32(param)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlSamplerParameterIuiv(sampler uint, pname uint, param uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlSamplerParameterIuiv(ptr.Pointer(), C.uint(uint32(sampler)), C.uint(uint32(pname)), C.uint(uint32(param)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlSamplerParameterf(sampler uint, pname uint, param float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlSamplerParameterf(ptr.Pointer(), C.uint(uint32(sampler)), C.uint(uint32(pname)), C.float(param))
	}
}

func (ptr *QOpenGLExtraFunctions) GlSamplerParameterfv(sampler uint, pname uint, param float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlSamplerParameterfv(ptr.Pointer(), C.uint(uint32(sampler)), C.uint(uint32(pname)), C.float(param))
	}
}

func (ptr *QOpenGLExtraFunctions) GlSamplerParameteri(sampler uint, pname uint, param int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlSamplerParameteri(ptr.Pointer(), C.uint(uint32(sampler)), C.uint(uint32(pname)), C.int(int32(param)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlSamplerParameteriv(sampler uint, pname uint, param int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlSamplerParameteriv(ptr.Pointer(), C.uint(uint32(sampler)), C.uint(uint32(pname)), C.int(int32(param)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlTexBuffer(target uint, internalformat uint, buffer uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlTexBuffer(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(internalformat)), C.uint(uint32(buffer)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlTexImage3D(target uint, level int, internalformat int, width int, height int, depth int, border int, format uint, ty uint, pixels unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlTexImage3D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.int(int32(internalformat)), C.int(int32(width)), C.int(int32(height)), C.int(int32(depth)), C.int(int32(border)), C.uint(uint32(format)), C.uint(uint32(ty)), pixels)
	}
}

func (ptr *QOpenGLExtraFunctions) GlTexParameterIiv(target uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlTexParameterIiv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlTexParameterIuiv(target uint, pname uint, params uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlTexParameterIuiv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.uint(uint32(params)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlTexStorage2D(target uint, levels int, internalformat uint, width int, height int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlTexStorage2D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(levels)), C.uint(uint32(internalformat)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlTexStorage2DMultisample(target uint, samples int, internalformat uint, width int, height int, fixedsamplelocations bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlTexStorage2DMultisample(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(samples)), C.uint(uint32(internalformat)), C.int(int32(width)), C.int(int32(height)), C.char(int8(qt.GoBoolToInt(fixedsamplelocations))))
	}
}

func (ptr *QOpenGLExtraFunctions) GlTexStorage3D(target uint, levels int, internalformat uint, width int, height int, depth int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlTexStorage3D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(levels)), C.uint(uint32(internalformat)), C.int(int32(width)), C.int(int32(height)), C.int(int32(depth)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlTexStorage3DMultisample(target uint, samples int, internalformat uint, width int, height int, depth int, fixedsamplelocations bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlTexStorage3DMultisample(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(samples)), C.uint(uint32(internalformat)), C.int(int32(width)), C.int(int32(height)), C.int(int32(depth)), C.char(int8(qt.GoBoolToInt(fixedsamplelocations))))
	}
}

func (ptr *QOpenGLExtraFunctions) GlTexSubImage3D(target uint, level int, xoffset int, yoffset int, zoffset int, width int, height int, depth int, format uint, ty uint, pixels unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlTexSubImage3D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.int(int32(xoffset)), C.int(int32(yoffset)), C.int(int32(zoffset)), C.int(int32(width)), C.int(int32(height)), C.int(int32(depth)), C.uint(uint32(format)), C.uint(uint32(ty)), pixels)
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniform1ui(location int, v0 uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniform1ui(ptr.Pointer(), C.int(int32(location)), C.uint(uint32(v0)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniform1uiv(location int, count int, value uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniform1uiv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniform2ui(location int, v0 uint, v1 uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniform2ui(ptr.Pointer(), C.int(int32(location)), C.uint(uint32(v0)), C.uint(uint32(v1)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniform2uiv(location int, count int, value uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniform2uiv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniform3ui(location int, v0 uint, v1 uint, v2 uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniform3ui(ptr.Pointer(), C.int(int32(location)), C.uint(uint32(v0)), C.uint(uint32(v1)), C.uint(uint32(v2)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniform3uiv(location int, count int, value uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniform3uiv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniform4ui(location int, v0 uint, v1 uint, v2 uint, v3 uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniform4ui(ptr.Pointer(), C.int(int32(location)), C.uint(uint32(v0)), C.uint(uint32(v1)), C.uint(uint32(v2)), C.uint(uint32(v3)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniform4uiv(location int, count int, value uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniform4uiv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniformBlockBinding(program uint, uniformBlockIndex uint, uniformBlockBinding uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniformBlockBinding(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(uniformBlockIndex)), C.uint(uint32(uniformBlockBinding)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniformMatrix2x3fv(location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniformMatrix2x3fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniformMatrix2x4fv(location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniformMatrix2x4fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniformMatrix3x2fv(location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniformMatrix3x2fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniformMatrix3x4fv(location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniformMatrix3x4fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniformMatrix4x2fv(location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniformMatrix4x2fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUniformMatrix4x3fv(location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUniformMatrix4x3fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLExtraFunctions) GlUnmapBuffer(target uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLExtraFunctions_GlUnmapBuffer(ptr.Pointer(), C.uint(uint32(target)))) != 0
	}
	return false
}

func (ptr *QOpenGLExtraFunctions) GlUseProgramStages(pipeline uint, stages uint, program uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlUseProgramStages(ptr.Pointer(), C.uint(uint32(pipeline)), C.uint(uint32(stages)), C.uint(uint32(program)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlValidateProgramPipeline(pipeline uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlValidateProgramPipeline(ptr.Pointer(), C.uint(uint32(pipeline)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlVertexAttribBinding(attribindex uint, bindingindex uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlVertexAttribBinding(ptr.Pointer(), C.uint(uint32(attribindex)), C.uint(uint32(bindingindex)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlVertexAttribDivisor(index uint, divisor uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlVertexAttribDivisor(ptr.Pointer(), C.uint(uint32(index)), C.uint(uint32(divisor)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlVertexAttribFormat(attribindex uint, size int, ty uint, normalized bool, relativeoffset uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlVertexAttribFormat(ptr.Pointer(), C.uint(uint32(attribindex)), C.int(int32(size)), C.uint(uint32(ty)), C.char(int8(qt.GoBoolToInt(normalized))), C.uint(uint32(relativeoffset)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlVertexAttribI4i(index uint, x int, y int, z int, w int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlVertexAttribI4i(ptr.Pointer(), C.uint(uint32(index)), C.int(int32(x)), C.int(int32(y)), C.int(int32(z)), C.int(int32(w)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlVertexAttribI4iv(index uint, v int) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlVertexAttribI4iv(ptr.Pointer(), C.uint(uint32(index)), C.int(int32(v)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlVertexAttribI4ui(index uint, x uint, y uint, z uint, w uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlVertexAttribI4ui(ptr.Pointer(), C.uint(uint32(index)), C.uint(uint32(x)), C.uint(uint32(y)), C.uint(uint32(z)), C.uint(uint32(w)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlVertexAttribI4uiv(index uint, v uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlVertexAttribI4uiv(ptr.Pointer(), C.uint(uint32(index)), C.uint(uint32(v)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlVertexAttribIFormat(attribindex uint, size int, ty uint, relativeoffset uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlVertexAttribIFormat(ptr.Pointer(), C.uint(uint32(attribindex)), C.int(int32(size)), C.uint(uint32(ty)), C.uint(uint32(relativeoffset)))
	}
}

func (ptr *QOpenGLExtraFunctions) GlVertexAttribIPointer(index uint, size int, ty uint, stride int, pointer unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlVertexAttribIPointer(ptr.Pointer(), C.uint(uint32(index)), C.int(int32(size)), C.uint(uint32(ty)), C.int(int32(stride)), pointer)
	}
}

func (ptr *QOpenGLExtraFunctions) GlVertexBindingDivisor(bindingindex uint, divisor uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLExtraFunctions_GlVertexBindingDivisor(ptr.Pointer(), C.uint(uint32(bindingindex)), C.uint(uint32(divisor)))
	}
}

type QOpenGLFramebufferObject struct {
	ptr unsafe.Pointer
}

type QOpenGLFramebufferObject_ITF interface {
	QOpenGLFramebufferObject_PTR() *QOpenGLFramebufferObject
}

func (ptr *QOpenGLFramebufferObject) QOpenGLFramebufferObject_PTR() *QOpenGLFramebufferObject {
	return ptr
}

func (ptr *QOpenGLFramebufferObject) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLFramebufferObject) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLFramebufferObject(ptr QOpenGLFramebufferObject_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFramebufferObject_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFramebufferObjectFromPointer(ptr unsafe.Pointer) (n *QOpenGLFramebufferObject) {
	n = new(QOpenGLFramebufferObject)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QOpenGLFramebufferObject__Attachment
//QOpenGLFramebufferObject::Attachment
type QOpenGLFramebufferObject__Attachment int64

const (
	QOpenGLFramebufferObject__NoAttachment         QOpenGLFramebufferObject__Attachment = QOpenGLFramebufferObject__Attachment(0)
	QOpenGLFramebufferObject__CombinedDepthStencil QOpenGLFramebufferObject__Attachment = QOpenGLFramebufferObject__Attachment(1)
	QOpenGLFramebufferObject__Depth                QOpenGLFramebufferObject__Attachment = QOpenGLFramebufferObject__Attachment(2)
)

//go:generate stringer -type=QOpenGLFramebufferObject__FramebufferRestorePolicy
//QOpenGLFramebufferObject::FramebufferRestorePolicy
type QOpenGLFramebufferObject__FramebufferRestorePolicy int64

const (
	QOpenGLFramebufferObject__DontRestoreFramebufferBinding      QOpenGLFramebufferObject__FramebufferRestorePolicy = QOpenGLFramebufferObject__FramebufferRestorePolicy(0)
	QOpenGLFramebufferObject__RestoreFramebufferBindingToDefault QOpenGLFramebufferObject__FramebufferRestorePolicy = QOpenGLFramebufferObject__FramebufferRestorePolicy(1)
	QOpenGLFramebufferObject__RestoreFrameBufferBinding          QOpenGLFramebufferObject__FramebufferRestorePolicy = QOpenGLFramebufferObject__FramebufferRestorePolicy(2)
)

func NewQOpenGLFramebufferObject(size core.QSize_ITF, target uint) *QOpenGLFramebufferObject {
	tmpValue := NewQOpenGLFramebufferObjectFromPointer(C.QOpenGLFramebufferObject_NewQOpenGLFramebufferObject(core.PointerFromQSize(size), C.uint(uint32(target))))
	qt.SetFinalizer(tmpValue, (*QOpenGLFramebufferObject).DestroyQOpenGLFramebufferObject)
	return tmpValue
}

func NewQOpenGLFramebufferObject2(width int, height int, target uint) *QOpenGLFramebufferObject {
	tmpValue := NewQOpenGLFramebufferObjectFromPointer(C.QOpenGLFramebufferObject_NewQOpenGLFramebufferObject2(C.int(int32(width)), C.int(int32(height)), C.uint(uint32(target))))
	qt.SetFinalizer(tmpValue, (*QOpenGLFramebufferObject).DestroyQOpenGLFramebufferObject)
	return tmpValue
}

func NewQOpenGLFramebufferObject3(size core.QSize_ITF, attachment QOpenGLFramebufferObject__Attachment, target uint, internalFormat uint) *QOpenGLFramebufferObject {
	tmpValue := NewQOpenGLFramebufferObjectFromPointer(C.QOpenGLFramebufferObject_NewQOpenGLFramebufferObject3(core.PointerFromQSize(size), C.longlong(attachment), C.uint(uint32(target)), C.uint(uint32(internalFormat))))
	qt.SetFinalizer(tmpValue, (*QOpenGLFramebufferObject).DestroyQOpenGLFramebufferObject)
	return tmpValue
}

func NewQOpenGLFramebufferObject4(width int, height int, attachment QOpenGLFramebufferObject__Attachment, target uint, internalFormat uint) *QOpenGLFramebufferObject {
	tmpValue := NewQOpenGLFramebufferObjectFromPointer(C.QOpenGLFramebufferObject_NewQOpenGLFramebufferObject4(C.int(int32(width)), C.int(int32(height)), C.longlong(attachment), C.uint(uint32(target)), C.uint(uint32(internalFormat))))
	qt.SetFinalizer(tmpValue, (*QOpenGLFramebufferObject).DestroyQOpenGLFramebufferObject)
	return tmpValue
}

func NewQOpenGLFramebufferObject5(size core.QSize_ITF, format QOpenGLFramebufferObjectFormat_ITF) *QOpenGLFramebufferObject {
	tmpValue := NewQOpenGLFramebufferObjectFromPointer(C.QOpenGLFramebufferObject_NewQOpenGLFramebufferObject5(core.PointerFromQSize(size), PointerFromQOpenGLFramebufferObjectFormat(format)))
	qt.SetFinalizer(tmpValue, (*QOpenGLFramebufferObject).DestroyQOpenGLFramebufferObject)
	return tmpValue
}

func NewQOpenGLFramebufferObject6(width int, height int, format QOpenGLFramebufferObjectFormat_ITF) *QOpenGLFramebufferObject {
	tmpValue := NewQOpenGLFramebufferObjectFromPointer(C.QOpenGLFramebufferObject_NewQOpenGLFramebufferObject6(C.int(int32(width)), C.int(int32(height)), PointerFromQOpenGLFramebufferObjectFormat(format)))
	qt.SetFinalizer(tmpValue, (*QOpenGLFramebufferObject).DestroyQOpenGLFramebufferObject)
	return tmpValue
}

func (ptr *QOpenGLFramebufferObject) AddColorAttachment(size core.QSize_ITF, internalFormat uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFramebufferObject_AddColorAttachment(ptr.Pointer(), core.PointerFromQSize(size), C.uint(uint32(internalFormat)))
	}
}

func (ptr *QOpenGLFramebufferObject) AddColorAttachment2(width int, height int, internalFormat uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFramebufferObject_AddColorAttachment2(ptr.Pointer(), C.int(int32(width)), C.int(int32(height)), C.uint(uint32(internalFormat)))
	}
}

func (ptr *QOpenGLFramebufferObject) Attachment() QOpenGLFramebufferObject__Attachment {
	if ptr.Pointer() != nil {
		return QOpenGLFramebufferObject__Attachment(C.QOpenGLFramebufferObject_Attachment(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLFramebufferObject) Bind() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFramebufferObject_Bind(ptr.Pointer())) != 0
	}
	return false
}

func QOpenGLFramebufferObject_BindDefault() bool {
	return int8(C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_BindDefault()) != 0
}

func (ptr *QOpenGLFramebufferObject) BindDefault() bool {
	return int8(C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_BindDefault()) != 0
}

func QOpenGLFramebufferObject_BlitFramebuffer(target QOpenGLFramebufferObject_ITF, targetRect core.QRect_ITF, source QOpenGLFramebufferObject_ITF, sourceRect core.QRect_ITF, buffers uint, filter uint, readColorAttachmentIndex int, drawColorAttachmentIndex int, restorePolicy QOpenGLFramebufferObject__FramebufferRestorePolicy) {
	C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_BlitFramebuffer(PointerFromQOpenGLFramebufferObject(target), core.PointerFromQRect(targetRect), PointerFromQOpenGLFramebufferObject(source), core.PointerFromQRect(sourceRect), C.uint(uint32(buffers)), C.uint(uint32(filter)), C.int(int32(readColorAttachmentIndex)), C.int(int32(drawColorAttachmentIndex)), C.longlong(restorePolicy))
}

func (ptr *QOpenGLFramebufferObject) BlitFramebuffer(target QOpenGLFramebufferObject_ITF, targetRect core.QRect_ITF, source QOpenGLFramebufferObject_ITF, sourceRect core.QRect_ITF, buffers uint, filter uint, readColorAttachmentIndex int, drawColorAttachmentIndex int, restorePolicy QOpenGLFramebufferObject__FramebufferRestorePolicy) {
	C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_BlitFramebuffer(PointerFromQOpenGLFramebufferObject(target), core.PointerFromQRect(targetRect), PointerFromQOpenGLFramebufferObject(source), core.PointerFromQRect(sourceRect), C.uint(uint32(buffers)), C.uint(uint32(filter)), C.int(int32(readColorAttachmentIndex)), C.int(int32(drawColorAttachmentIndex)), C.longlong(restorePolicy))
}

func QOpenGLFramebufferObject_BlitFramebuffer2(target QOpenGLFramebufferObject_ITF, targetRect core.QRect_ITF, source QOpenGLFramebufferObject_ITF, sourceRect core.QRect_ITF, buffers uint, filter uint, readColorAttachmentIndex int, drawColorAttachmentIndex int) {
	C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_BlitFramebuffer2(PointerFromQOpenGLFramebufferObject(target), core.PointerFromQRect(targetRect), PointerFromQOpenGLFramebufferObject(source), core.PointerFromQRect(sourceRect), C.uint(uint32(buffers)), C.uint(uint32(filter)), C.int(int32(readColorAttachmentIndex)), C.int(int32(drawColorAttachmentIndex)))
}

func (ptr *QOpenGLFramebufferObject) BlitFramebuffer2(target QOpenGLFramebufferObject_ITF, targetRect core.QRect_ITF, source QOpenGLFramebufferObject_ITF, sourceRect core.QRect_ITF, buffers uint, filter uint, readColorAttachmentIndex int, drawColorAttachmentIndex int) {
	C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_BlitFramebuffer2(PointerFromQOpenGLFramebufferObject(target), core.PointerFromQRect(targetRect), PointerFromQOpenGLFramebufferObject(source), core.PointerFromQRect(sourceRect), C.uint(uint32(buffers)), C.uint(uint32(filter)), C.int(int32(readColorAttachmentIndex)), C.int(int32(drawColorAttachmentIndex)))
}

func QOpenGLFramebufferObject_BlitFramebuffer3(target QOpenGLFramebufferObject_ITF, targetRect core.QRect_ITF, source QOpenGLFramebufferObject_ITF, sourceRect core.QRect_ITF, buffers uint, filter uint) {
	C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_BlitFramebuffer3(PointerFromQOpenGLFramebufferObject(target), core.PointerFromQRect(targetRect), PointerFromQOpenGLFramebufferObject(source), core.PointerFromQRect(sourceRect), C.uint(uint32(buffers)), C.uint(uint32(filter)))
}

func (ptr *QOpenGLFramebufferObject) BlitFramebuffer3(target QOpenGLFramebufferObject_ITF, targetRect core.QRect_ITF, source QOpenGLFramebufferObject_ITF, sourceRect core.QRect_ITF, buffers uint, filter uint) {
	C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_BlitFramebuffer3(PointerFromQOpenGLFramebufferObject(target), core.PointerFromQRect(targetRect), PointerFromQOpenGLFramebufferObject(source), core.PointerFromQRect(sourceRect), C.uint(uint32(buffers)), C.uint(uint32(filter)))
}

func QOpenGLFramebufferObject_BlitFramebuffer4(target QOpenGLFramebufferObject_ITF, source QOpenGLFramebufferObject_ITF, buffers uint, filter uint) {
	C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_BlitFramebuffer4(PointerFromQOpenGLFramebufferObject(target), PointerFromQOpenGLFramebufferObject(source), C.uint(uint32(buffers)), C.uint(uint32(filter)))
}

func (ptr *QOpenGLFramebufferObject) BlitFramebuffer4(target QOpenGLFramebufferObject_ITF, source QOpenGLFramebufferObject_ITF, buffers uint, filter uint) {
	C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_BlitFramebuffer4(PointerFromQOpenGLFramebufferObject(target), PointerFromQOpenGLFramebufferObject(source), C.uint(uint32(buffers)), C.uint(uint32(filter)))
}

func (ptr *QOpenGLFramebufferObject) Format() *QOpenGLFramebufferObjectFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQOpenGLFramebufferObjectFormatFromPointer(C.QOpenGLFramebufferObject_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QOpenGLFramebufferObjectFormat).DestroyQOpenGLFramebufferObjectFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLFramebufferObject) Handle() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFramebufferObject_Handle(ptr.Pointer())))
	}
	return 0
}

func QOpenGLFramebufferObject_HasOpenGLFramebufferBlit() bool {
	return int8(C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_HasOpenGLFramebufferBlit()) != 0
}

func (ptr *QOpenGLFramebufferObject) HasOpenGLFramebufferBlit() bool {
	return int8(C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_HasOpenGLFramebufferBlit()) != 0
}

func QOpenGLFramebufferObject_HasOpenGLFramebufferObjects() bool {
	return int8(C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_HasOpenGLFramebufferObjects()) != 0
}

func (ptr *QOpenGLFramebufferObject) HasOpenGLFramebufferObjects() bool {
	return int8(C.QOpenGLFramebufferObject_QOpenGLFramebufferObject_HasOpenGLFramebufferObjects()) != 0
}

func (ptr *QOpenGLFramebufferObject) Height() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLFramebufferObject_Height(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLFramebufferObject) IsBound() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFramebufferObject_IsBound(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLFramebufferObject) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFramebufferObject_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLFramebufferObject) Release() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFramebufferObject_Release(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLFramebufferObject) SetAttachment(attachment QOpenGLFramebufferObject__Attachment) {
	if ptr.Pointer() != nil {
		C.QOpenGLFramebufferObject_SetAttachment(ptr.Pointer(), C.longlong(attachment))
	}
}

func (ptr *QOpenGLFramebufferObject) Size() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QOpenGLFramebufferObject_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLFramebufferObject) Sizes() []*core.QSize {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*core.QSize {
			out := make([]*core.QSize, int(l.len))
			tmpList := NewQOpenGLFramebufferObjectFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__sizes_atList(i)
			}
			return out
		}(C.QOpenGLFramebufferObject_Sizes(ptr.Pointer()))
	}
	return make([]*core.QSize, 0)
}

func (ptr *QOpenGLFramebufferObject) TakeTexture() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFramebufferObject_TakeTexture(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLFramebufferObject) TakeTexture2(colorAttachmentIndex int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFramebufferObject_TakeTexture2(ptr.Pointer(), C.int(int32(colorAttachmentIndex)))))
	}
	return 0
}

func (ptr *QOpenGLFramebufferObject) Texture() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFramebufferObject_Texture(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLFramebufferObject) Textures() []uint {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []uint {
			out := make([]uint, int(l.len))
			tmpList := NewQOpenGLFramebufferObjectFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__textures_atList(i)
			}
			return out
		}(C.QOpenGLFramebufferObject_Textures(ptr.Pointer()))
	}
	return make([]uint, 0)
}

func (ptr *QOpenGLFramebufferObject) ToImage(flipped bool) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QOpenGLFramebufferObject_ToImage(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(flipped)))))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLFramebufferObject) ToImage2() *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QOpenGLFramebufferObject_ToImage2(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLFramebufferObject) ToImage3(flipped bool, colorAttachmentIndex int) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QOpenGLFramebufferObject_ToImage3(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(flipped))), C.int(int32(colorAttachmentIndex))))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLFramebufferObject) Width() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLFramebufferObject_Width(ptr.Pointer())))
	}
	return 0
}

//export callbackQOpenGLFramebufferObject_DestroyQOpenGLFramebufferObject
func callbackQOpenGLFramebufferObject_DestroyQOpenGLFramebufferObject(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOpenGLFramebufferObject"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLFramebufferObjectFromPointer(ptr).DestroyQOpenGLFramebufferObjectDefault()
	}
}

func (ptr *QOpenGLFramebufferObject) ConnectDestroyQOpenGLFramebufferObject(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOpenGLFramebufferObject"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLFramebufferObject", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLFramebufferObject", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLFramebufferObject) DisconnectDestroyQOpenGLFramebufferObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOpenGLFramebufferObject")
	}
}

func (ptr *QOpenGLFramebufferObject) DestroyQOpenGLFramebufferObject() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLFramebufferObject_DestroyQOpenGLFramebufferObject(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLFramebufferObject) DestroyQOpenGLFramebufferObjectDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLFramebufferObject_DestroyQOpenGLFramebufferObjectDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLFramebufferObject) __sizes_atList(i int) *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QOpenGLFramebufferObject___sizes_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLFramebufferObject) __sizes_setList(i core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLFramebufferObject___sizes_setList(ptr.Pointer(), core.PointerFromQSize(i))
	}
}

func (ptr *QOpenGLFramebufferObject) __sizes_newList() unsafe.Pointer {
	return C.QOpenGLFramebufferObject___sizes_newList(ptr.Pointer())
}

func (ptr *QOpenGLFramebufferObject) __textures_atList(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFramebufferObject___textures_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QOpenGLFramebufferObject) __textures_setList(i uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFramebufferObject___textures_setList(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QOpenGLFramebufferObject) __textures_newList() unsafe.Pointer {
	return C.QOpenGLFramebufferObject___textures_newList(ptr.Pointer())
}

type QOpenGLFramebufferObjectFormat struct {
	ptr unsafe.Pointer
}

type QOpenGLFramebufferObjectFormat_ITF interface {
	QOpenGLFramebufferObjectFormat_PTR() *QOpenGLFramebufferObjectFormat
}

func (ptr *QOpenGLFramebufferObjectFormat) QOpenGLFramebufferObjectFormat_PTR() *QOpenGLFramebufferObjectFormat {
	return ptr
}

func (ptr *QOpenGLFramebufferObjectFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLFramebufferObjectFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLFramebufferObjectFormat(ptr QOpenGLFramebufferObjectFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFramebufferObjectFormat_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFramebufferObjectFormatFromPointer(ptr unsafe.Pointer) (n *QOpenGLFramebufferObjectFormat) {
	n = new(QOpenGLFramebufferObjectFormat)
	n.SetPointer(ptr)
	return
}
func NewQOpenGLFramebufferObjectFormat() *QOpenGLFramebufferObjectFormat {
	tmpValue := NewQOpenGLFramebufferObjectFormatFromPointer(C.QOpenGLFramebufferObjectFormat_NewQOpenGLFramebufferObjectFormat())
	qt.SetFinalizer(tmpValue, (*QOpenGLFramebufferObjectFormat).DestroyQOpenGLFramebufferObjectFormat)
	return tmpValue
}

func NewQOpenGLFramebufferObjectFormat2(other QOpenGLFramebufferObjectFormat_ITF) *QOpenGLFramebufferObjectFormat {
	tmpValue := NewQOpenGLFramebufferObjectFormatFromPointer(C.QOpenGLFramebufferObjectFormat_NewQOpenGLFramebufferObjectFormat2(PointerFromQOpenGLFramebufferObjectFormat(other)))
	qt.SetFinalizer(tmpValue, (*QOpenGLFramebufferObjectFormat).DestroyQOpenGLFramebufferObjectFormat)
	return tmpValue
}

func (ptr *QOpenGLFramebufferObjectFormat) Attachment() QOpenGLFramebufferObject__Attachment {
	if ptr.Pointer() != nil {
		return QOpenGLFramebufferObject__Attachment(C.QOpenGLFramebufferObjectFormat_Attachment(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLFramebufferObjectFormat) InternalTextureFormat() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFramebufferObjectFormat_InternalTextureFormat(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLFramebufferObjectFormat) Mipmap() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFramebufferObjectFormat_Mipmap(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLFramebufferObjectFormat) Samples() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLFramebufferObjectFormat_Samples(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLFramebufferObjectFormat) SetAttachment(attachment QOpenGLFramebufferObject__Attachment) {
	if ptr.Pointer() != nil {
		C.QOpenGLFramebufferObjectFormat_SetAttachment(ptr.Pointer(), C.longlong(attachment))
	}
}

func (ptr *QOpenGLFramebufferObjectFormat) SetInternalTextureFormat(internalTextureFormat uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFramebufferObjectFormat_SetInternalTextureFormat(ptr.Pointer(), C.uint(uint32(internalTextureFormat)))
	}
}

func (ptr *QOpenGLFramebufferObjectFormat) SetMipmap(enabled bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLFramebufferObjectFormat_SetMipmap(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enabled))))
	}
}

func (ptr *QOpenGLFramebufferObjectFormat) SetSamples(samples int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFramebufferObjectFormat_SetSamples(ptr.Pointer(), C.int(int32(samples)))
	}
}

func (ptr *QOpenGLFramebufferObjectFormat) SetTextureTarget(target uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFramebufferObjectFormat_SetTextureTarget(ptr.Pointer(), C.uint(uint32(target)))
	}
}

func (ptr *QOpenGLFramebufferObjectFormat) TextureTarget() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFramebufferObjectFormat_TextureTarget(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLFramebufferObjectFormat) DestroyQOpenGLFramebufferObjectFormat() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLFramebufferObjectFormat_DestroyQOpenGLFramebufferObjectFormat(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions struct {
	ptr unsafe.Pointer
}

type QOpenGLFunctions_ITF interface {
	QOpenGLFunctions_PTR() *QOpenGLFunctions
}

func (ptr *QOpenGLFunctions) QOpenGLFunctions_PTR() *QOpenGLFunctions {
	return ptr
}

func (ptr *QOpenGLFunctions) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLFunctions) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLFunctions(ptr QOpenGLFunctions_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctionsFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions) {
	n = new(QOpenGLFunctions)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QOpenGLFunctions__OpenGLFeature
//QOpenGLFunctions::OpenGLFeature
type QOpenGLFunctions__OpenGLFeature int64

const (
	QOpenGLFunctions__Multitexture          QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0001)
	QOpenGLFunctions__Shaders               QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0002)
	QOpenGLFunctions__Buffers               QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0004)
	QOpenGLFunctions__Framebuffers          QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0008)
	QOpenGLFunctions__BlendColor            QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0010)
	QOpenGLFunctions__BlendEquation         QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0020)
	QOpenGLFunctions__BlendEquationSeparate QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0040)
	QOpenGLFunctions__BlendFuncSeparate     QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0080)
	QOpenGLFunctions__BlendSubtract         QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0100)
	QOpenGLFunctions__CompressedTextures    QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0200)
	QOpenGLFunctions__Multisample           QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0400)
	QOpenGLFunctions__StencilSeparate       QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x0800)
	QOpenGLFunctions__NPOTTextures          QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x1000)
	QOpenGLFunctions__NPOTTextureRepeat     QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x2000)
	QOpenGLFunctions__FixedFunctionPipeline QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x4000)
	QOpenGLFunctions__TextureRGFormats      QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x8000)
	QOpenGLFunctions__MultipleRenderTargets QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x10000)
	QOpenGLFunctions__BlendEquationAdvanced QOpenGLFunctions__OpenGLFeature = QOpenGLFunctions__OpenGLFeature(0x20000)
)

func NewQOpenGLFunctions() *QOpenGLFunctions {
	tmpValue := NewQOpenGLFunctionsFromPointer(C.QOpenGLFunctions_NewQOpenGLFunctions())
	qt.SetFinalizer(tmpValue, (*QOpenGLFunctions).DestroyQOpenGLFunctions)
	return tmpValue
}

func NewQOpenGLFunctions2(context QOpenGLContext_ITF) *QOpenGLFunctions {
	tmpValue := NewQOpenGLFunctionsFromPointer(C.QOpenGLFunctions_NewQOpenGLFunctions2(PointerFromQOpenGLContext(context)))
	qt.SetFinalizer(tmpValue, (*QOpenGLFunctions).DestroyQOpenGLFunctions)
	return tmpValue
}

func (ptr *QOpenGLFunctions) GlActiveTexture(texture uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlActiveTexture(ptr.Pointer(), C.uint(uint32(texture)))
	}
}

func (ptr *QOpenGLFunctions) GlAttachShader(program uint, shader uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlAttachShader(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(shader)))
	}
}

func (ptr *QOpenGLFunctions) GlBindAttribLocation(program uint, index uint, name string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLFunctions_GlBindAttribLocation(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(index)), nameC)
	}
}

func (ptr *QOpenGLFunctions) GlBindBuffer(target uint, buffer uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlBindBuffer(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(buffer)))
	}
}

func (ptr *QOpenGLFunctions) GlBindFramebuffer(target uint, framebuffer uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlBindFramebuffer(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(framebuffer)))
	}
}

func (ptr *QOpenGLFunctions) GlBindRenderbuffer(target uint, renderbuffer uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlBindRenderbuffer(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(renderbuffer)))
	}
}

func (ptr *QOpenGLFunctions) GlBindTexture(target uint, texture uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlBindTexture(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(texture)))
	}
}

func (ptr *QOpenGLFunctions) GlBlendColor(red float32, green float32, blue float32, alpha float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlBlendColor(ptr.Pointer(), C.float(red), C.float(green), C.float(blue), C.float(alpha))
	}
}

func (ptr *QOpenGLFunctions) GlBlendEquation(mode uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlBlendEquation(ptr.Pointer(), C.uint(uint32(mode)))
	}
}

func (ptr *QOpenGLFunctions) GlBlendEquationSeparate(modeRGB uint, modeAlpha uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlBlendEquationSeparate(ptr.Pointer(), C.uint(uint32(modeRGB)), C.uint(uint32(modeAlpha)))
	}
}

func (ptr *QOpenGLFunctions) GlBlendFunc(sfactor uint, dfactor uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlBlendFunc(ptr.Pointer(), C.uint(uint32(sfactor)), C.uint(uint32(dfactor)))
	}
}

func (ptr *QOpenGLFunctions) GlBlendFuncSeparate(srcRGB uint, dstRGB uint, srcAlpha uint, dstAlpha uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlBlendFuncSeparate(ptr.Pointer(), C.uint(uint32(srcRGB)), C.uint(uint32(dstRGB)), C.uint(uint32(srcAlpha)), C.uint(uint32(dstAlpha)))
	}
}

func (ptr *QOpenGLFunctions) GlCheckFramebufferStatus(target uint) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFunctions_GlCheckFramebufferStatus(ptr.Pointer(), C.uint(uint32(target)))))
	}
	return 0
}

func (ptr *QOpenGLFunctions) GlClear(mask uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlClear(ptr.Pointer(), C.uint(uint32(mask)))
	}
}

func (ptr *QOpenGLFunctions) GlClearColor(red float32, green float32, blue float32, alpha float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlClearColor(ptr.Pointer(), C.float(red), C.float(green), C.float(blue), C.float(alpha))
	}
}

func (ptr *QOpenGLFunctions) GlClearDepthf(depth float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlClearDepthf(ptr.Pointer(), C.float(depth))
	}
}

func (ptr *QOpenGLFunctions) GlClearStencil(s int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlClearStencil(ptr.Pointer(), C.int(int32(s)))
	}
}

func (ptr *QOpenGLFunctions) GlColorMask(red bool, green bool, blue bool, alpha bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlColorMask(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(red))), C.char(int8(qt.GoBoolToInt(green))), C.char(int8(qt.GoBoolToInt(blue))), C.char(int8(qt.GoBoolToInt(alpha))))
	}
}

func (ptr *QOpenGLFunctions) GlCompileShader(shader uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlCompileShader(ptr.Pointer(), C.uint(uint32(shader)))
	}
}

func (ptr *QOpenGLFunctions) GlCompressedTexImage2D(target uint, level int, internalformat uint, width int, height int, border int, imageSize int, data unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlCompressedTexImage2D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.uint(uint32(internalformat)), C.int(int32(width)), C.int(int32(height)), C.int(int32(border)), C.int(int32(imageSize)), data)
	}
}

func (ptr *QOpenGLFunctions) GlCompressedTexSubImage2D(target uint, level int, xoffset int, yoffset int, width int, height int, format uint, imageSize int, data unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlCompressedTexSubImage2D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.int(int32(xoffset)), C.int(int32(yoffset)), C.int(int32(width)), C.int(int32(height)), C.uint(uint32(format)), C.int(int32(imageSize)), data)
	}
}

func (ptr *QOpenGLFunctions) GlCopyTexImage2D(target uint, level int, internalformat uint, x int, y int, width int, height int, border int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlCopyTexImage2D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.uint(uint32(internalformat)), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.int(int32(border)))
	}
}

func (ptr *QOpenGLFunctions) GlCopyTexSubImage2D(target uint, level int, xoffset int, yoffset int, x int, y int, width int, height int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlCopyTexSubImage2D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.int(int32(xoffset)), C.int(int32(yoffset)), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QOpenGLFunctions) GlCreateProgram() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFunctions_GlCreateProgram(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLFunctions) GlCreateShader(ty uint) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFunctions_GlCreateShader(ptr.Pointer(), C.uint(uint32(ty)))))
	}
	return 0
}

func (ptr *QOpenGLFunctions) GlCullFace(mode uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlCullFace(ptr.Pointer(), C.uint(uint32(mode)))
	}
}

func (ptr *QOpenGLFunctions) GlDeleteBuffers(n int, buffers uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDeleteBuffers(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(buffers)))
	}
}

func (ptr *QOpenGLFunctions) GlDeleteFramebuffers(n int, framebuffers uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDeleteFramebuffers(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(framebuffers)))
	}
}

func (ptr *QOpenGLFunctions) GlDeleteProgram(program uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDeleteProgram(ptr.Pointer(), C.uint(uint32(program)))
	}
}

func (ptr *QOpenGLFunctions) GlDeleteRenderbuffers(n int, renderbuffers uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDeleteRenderbuffers(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(renderbuffers)))
	}
}

func (ptr *QOpenGLFunctions) GlDeleteShader(shader uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDeleteShader(ptr.Pointer(), C.uint(uint32(shader)))
	}
}

func (ptr *QOpenGLFunctions) GlDeleteTextures(n int, textures uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDeleteTextures(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(textures)))
	}
}

func (ptr *QOpenGLFunctions) GlDepthFunc(fu uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDepthFunc(ptr.Pointer(), C.uint(uint32(fu)))
	}
}

func (ptr *QOpenGLFunctions) GlDepthMask(flag bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDepthMask(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(flag))))
	}
}

func (ptr *QOpenGLFunctions) GlDepthRangef(zNear float32, zFar float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDepthRangef(ptr.Pointer(), C.float(zNear), C.float(zFar))
	}
}

func (ptr *QOpenGLFunctions) GlDetachShader(program uint, shader uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDetachShader(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(shader)))
	}
}

func (ptr *QOpenGLFunctions) GlDisable(cap uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDisable(ptr.Pointer(), C.uint(uint32(cap)))
	}
}

func (ptr *QOpenGLFunctions) GlDisableVertexAttribArray(index uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDisableVertexAttribArray(ptr.Pointer(), C.uint(uint32(index)))
	}
}

func (ptr *QOpenGLFunctions) GlDrawArrays(mode uint, first int, count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDrawArrays(ptr.Pointer(), C.uint(uint32(mode)), C.int(int32(first)), C.int(int32(count)))
	}
}

func (ptr *QOpenGLFunctions) GlDrawElements(mode uint, count int, ty uint, indices unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlDrawElements(ptr.Pointer(), C.uint(uint32(mode)), C.int(int32(count)), C.uint(uint32(ty)), indices)
	}
}

func (ptr *QOpenGLFunctions) GlEnable(cap uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlEnable(ptr.Pointer(), C.uint(uint32(cap)))
	}
}

func (ptr *QOpenGLFunctions) GlEnableVertexAttribArray(index uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlEnableVertexAttribArray(ptr.Pointer(), C.uint(uint32(index)))
	}
}

func (ptr *QOpenGLFunctions) GlFinish() {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlFinish(ptr.Pointer())
	}
}

func (ptr *QOpenGLFunctions) GlFlush() {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlFlush(ptr.Pointer())
	}
}

func (ptr *QOpenGLFunctions) GlFramebufferRenderbuffer(target uint, attachment uint, renderbuffertarget uint, renderbuffer uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlFramebufferRenderbuffer(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(attachment)), C.uint(uint32(renderbuffertarget)), C.uint(uint32(renderbuffer)))
	}
}

func (ptr *QOpenGLFunctions) GlFramebufferTexture2D(target uint, attachment uint, textarget uint, texture uint, level int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlFramebufferTexture2D(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(attachment)), C.uint(uint32(textarget)), C.uint(uint32(texture)), C.int(int32(level)))
	}
}

func (ptr *QOpenGLFunctions) GlFrontFace(mode uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlFrontFace(ptr.Pointer(), C.uint(uint32(mode)))
	}
}

func (ptr *QOpenGLFunctions) GlGenBuffers(n int, buffers uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGenBuffers(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(buffers)))
	}
}

func (ptr *QOpenGLFunctions) GlGenFramebuffers(n int, framebuffers uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGenFramebuffers(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(framebuffers)))
	}
}

func (ptr *QOpenGLFunctions) GlGenRenderbuffers(n int, renderbuffers uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGenRenderbuffers(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(renderbuffers)))
	}
}

func (ptr *QOpenGLFunctions) GlGenTextures(n int, textures uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGenTextures(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(textures)))
	}
}

func (ptr *QOpenGLFunctions) GlGenerateMipmap(target uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGenerateMipmap(ptr.Pointer(), C.uint(uint32(target)))
	}
}

func (ptr *QOpenGLFunctions) GlGetActiveAttrib(program uint, index uint, bufsize int, length int, size int, ty uint, name string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLFunctions_GlGetActiveAttrib(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(index)), C.int(int32(bufsize)), C.int(int32(length)), C.int(int32(size)), C.uint(uint32(ty)), nameC)
	}
}

func (ptr *QOpenGLFunctions) GlGetActiveUniform(program uint, index uint, bufsize int, length int, size int, ty uint, name string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLFunctions_GlGetActiveUniform(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(index)), C.int(int32(bufsize)), C.int(int32(length)), C.int(int32(size)), C.uint(uint32(ty)), nameC)
	}
}

func (ptr *QOpenGLFunctions) GlGetAttachedShaders(program uint, maxcount int, count int, shaders uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetAttachedShaders(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(maxcount)), C.int(int32(count)), C.uint(uint32(shaders)))
	}
}

func (ptr *QOpenGLFunctions) GlGetAttribLocation(program uint, name string) int {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		return int(int32(C.QOpenGLFunctions_GlGetAttribLocation(ptr.Pointer(), C.uint(uint32(program)), nameC)))
	}
	return 0
}

func (ptr *QOpenGLFunctions) GlGetBooleanv(pname uint, params *bool) {
	if ptr.Pointer() != nil {
		var paramsC C.char
		if params != nil {
			paramsC = C.char(int8(qt.GoBoolToInt(*params)))
			defer func() { *params = int8(paramsC) != 0 }()
		}
		C.QOpenGLFunctions_GlGetBooleanv(ptr.Pointer(), C.uint(uint32(pname)), &paramsC)
	}
}

func (ptr *QOpenGLFunctions) GlGetBufferParameteriv(target uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetBufferParameteriv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLFunctions) GlGetError() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLFunctions_GlGetError(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLFunctions) GlGetFloatv(pname uint, params float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetFloatv(ptr.Pointer(), C.uint(uint32(pname)), C.float(params))
	}
}

func (ptr *QOpenGLFunctions) GlGetFramebufferAttachmentParameteriv(target uint, attachment uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetFramebufferAttachmentParameteriv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(attachment)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLFunctions) GlGetIntegerv(pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetIntegerv(ptr.Pointer(), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLFunctions) GlGetProgramInfoLog(program uint, bufsize int, length int, infolog string) {
	if ptr.Pointer() != nil {
		var infologC *C.char
		if infolog != "" {
			infologC = C.CString(infolog)
			defer C.free(unsafe.Pointer(infologC))
		}
		C.QOpenGLFunctions_GlGetProgramInfoLog(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(bufsize)), C.int(int32(length)), infologC)
	}
}

func (ptr *QOpenGLFunctions) GlGetProgramiv(program uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetProgramiv(ptr.Pointer(), C.uint(uint32(program)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLFunctions) GlGetRenderbufferParameteriv(target uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetRenderbufferParameteriv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLFunctions) GlGetShaderInfoLog(shader uint, bufsize int, length int, infolog string) {
	if ptr.Pointer() != nil {
		var infologC *C.char
		if infolog != "" {
			infologC = C.CString(infolog)
			defer C.free(unsafe.Pointer(infologC))
		}
		C.QOpenGLFunctions_GlGetShaderInfoLog(ptr.Pointer(), C.uint(uint32(shader)), C.int(int32(bufsize)), C.int(int32(length)), infologC)
	}
}

func (ptr *QOpenGLFunctions) GlGetShaderPrecisionFormat(shadertype uint, precisiontype uint, ran int, precision int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetShaderPrecisionFormat(ptr.Pointer(), C.uint(uint32(shadertype)), C.uint(uint32(precisiontype)), C.int(int32(ran)), C.int(int32(precision)))
	}
}

func (ptr *QOpenGLFunctions) GlGetShaderSource(shader uint, bufsize int, length int, source string) {
	if ptr.Pointer() != nil {
		var sourceC *C.char
		if source != "" {
			sourceC = C.CString(source)
			defer C.free(unsafe.Pointer(sourceC))
		}
		C.QOpenGLFunctions_GlGetShaderSource(ptr.Pointer(), C.uint(uint32(shader)), C.int(int32(bufsize)), C.int(int32(length)), sourceC)
	}
}

func (ptr *QOpenGLFunctions) GlGetShaderiv(shader uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetShaderiv(ptr.Pointer(), C.uint(uint32(shader)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLFunctions) GlGetString(name uint) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QOpenGLFunctions_GlGetString(ptr.Pointer(), C.uint(uint32(name))))
	}
	return ""
}

func (ptr *QOpenGLFunctions) GlGetTexParameterfv(target uint, pname uint, params float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetTexParameterfv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.float(params))
	}
}

func (ptr *QOpenGLFunctions) GlGetTexParameteriv(target uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetTexParameteriv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLFunctions) GlGetUniformLocation(program uint, name string) int {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		return int(int32(C.QOpenGLFunctions_GlGetUniformLocation(ptr.Pointer(), C.uint(uint32(program)), nameC)))
	}
	return 0
}

func (ptr *QOpenGLFunctions) GlGetUniformfv(program uint, location int, params float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetUniformfv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.float(params))
	}
}

func (ptr *QOpenGLFunctions) GlGetUniformiv(program uint, location int, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetUniformiv(ptr.Pointer(), C.uint(uint32(program)), C.int(int32(location)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLFunctions) GlGetVertexAttribPointerv(index uint, pname uint, pointer unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetVertexAttribPointerv(ptr.Pointer(), C.uint(uint32(index)), C.uint(uint32(pname)), pointer)
	}
}

func (ptr *QOpenGLFunctions) GlGetVertexAttribfv(index uint, pname uint, params float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetVertexAttribfv(ptr.Pointer(), C.uint(uint32(index)), C.uint(uint32(pname)), C.float(params))
	}
}

func (ptr *QOpenGLFunctions) GlGetVertexAttribiv(index uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlGetVertexAttribiv(ptr.Pointer(), C.uint(uint32(index)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLFunctions) GlHint(target uint, mode uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlHint(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(mode)))
	}
}

func (ptr *QOpenGLFunctions) GlIsBuffer(buffer uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFunctions_GlIsBuffer(ptr.Pointer(), C.uint(uint32(buffer)))) != 0
	}
	return false
}

func (ptr *QOpenGLFunctions) GlIsEnabled(cap uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFunctions_GlIsEnabled(ptr.Pointer(), C.uint(uint32(cap)))) != 0
	}
	return false
}

func (ptr *QOpenGLFunctions) GlIsFramebuffer(framebuffer uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFunctions_GlIsFramebuffer(ptr.Pointer(), C.uint(uint32(framebuffer)))) != 0
	}
	return false
}

func (ptr *QOpenGLFunctions) GlIsProgram(program uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFunctions_GlIsProgram(ptr.Pointer(), C.uint(uint32(program)))) != 0
	}
	return false
}

func (ptr *QOpenGLFunctions) GlIsRenderbuffer(renderbuffer uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFunctions_GlIsRenderbuffer(ptr.Pointer(), C.uint(uint32(renderbuffer)))) != 0
	}
	return false
}

func (ptr *QOpenGLFunctions) GlIsShader(shader uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFunctions_GlIsShader(ptr.Pointer(), C.uint(uint32(shader)))) != 0
	}
	return false
}

func (ptr *QOpenGLFunctions) GlIsTexture(texture uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFunctions_GlIsTexture(ptr.Pointer(), C.uint(uint32(texture)))) != 0
	}
	return false
}

func (ptr *QOpenGLFunctions) GlLineWidth(width float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlLineWidth(ptr.Pointer(), C.float(width))
	}
}

func (ptr *QOpenGLFunctions) GlLinkProgram(program uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlLinkProgram(ptr.Pointer(), C.uint(uint32(program)))
	}
}

func (ptr *QOpenGLFunctions) GlPixelStorei(pname uint, param int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlPixelStorei(ptr.Pointer(), C.uint(uint32(pname)), C.int(int32(param)))
	}
}

func (ptr *QOpenGLFunctions) GlPolygonOffset(factor float32, units float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlPolygonOffset(ptr.Pointer(), C.float(factor), C.float(units))
	}
}

func (ptr *QOpenGLFunctions) GlReadPixels(x int, y int, width int, height int, format uint, ty uint, pixels unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlReadPixels(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.uint(uint32(format)), C.uint(uint32(ty)), pixels)
	}
}

func (ptr *QOpenGLFunctions) GlReleaseShaderCompiler() {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlReleaseShaderCompiler(ptr.Pointer())
	}
}

func (ptr *QOpenGLFunctions) GlRenderbufferStorage(target uint, internalformat uint, width int, height int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlRenderbufferStorage(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(internalformat)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QOpenGLFunctions) GlSampleCoverage(value float32, invert bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlSampleCoverage(ptr.Pointer(), C.float(value), C.char(int8(qt.GoBoolToInt(invert))))
	}
}

func (ptr *QOpenGLFunctions) GlScissor(x int, y int, width int, height int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlScissor(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QOpenGLFunctions) GlShaderBinary(n int, shaders uint, binaryformat uint, binary unsafe.Pointer, length int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlShaderBinary(ptr.Pointer(), C.int(int32(n)), C.uint(uint32(shaders)), C.uint(uint32(binaryformat)), binary, C.int(int32(length)))
	}
}

func (ptr *QOpenGLFunctions) GlStencilFunc(fu uint, ref int, mask uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlStencilFunc(ptr.Pointer(), C.uint(uint32(fu)), C.int(int32(ref)), C.uint(uint32(mask)))
	}
}

func (ptr *QOpenGLFunctions) GlStencilFuncSeparate(face uint, fu uint, ref int, mask uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlStencilFuncSeparate(ptr.Pointer(), C.uint(uint32(face)), C.uint(uint32(fu)), C.int(int32(ref)), C.uint(uint32(mask)))
	}
}

func (ptr *QOpenGLFunctions) GlStencilMask(mask uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlStencilMask(ptr.Pointer(), C.uint(uint32(mask)))
	}
}

func (ptr *QOpenGLFunctions) GlStencilMaskSeparate(face uint, mask uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlStencilMaskSeparate(ptr.Pointer(), C.uint(uint32(face)), C.uint(uint32(mask)))
	}
}

func (ptr *QOpenGLFunctions) GlStencilOp(fail uint, zfail uint, zpass uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlStencilOp(ptr.Pointer(), C.uint(uint32(fail)), C.uint(uint32(zfail)), C.uint(uint32(zpass)))
	}
}

func (ptr *QOpenGLFunctions) GlStencilOpSeparate(face uint, fail uint, zfail uint, zpass uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlStencilOpSeparate(ptr.Pointer(), C.uint(uint32(face)), C.uint(uint32(fail)), C.uint(uint32(zfail)), C.uint(uint32(zpass)))
	}
}

func (ptr *QOpenGLFunctions) GlTexImage2D(target uint, level int, internalformat int, width int, height int, border int, format uint, ty uint, pixels unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlTexImage2D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.int(int32(internalformat)), C.int(int32(width)), C.int(int32(height)), C.int(int32(border)), C.uint(uint32(format)), C.uint(uint32(ty)), pixels)
	}
}

func (ptr *QOpenGLFunctions) GlTexParameterf(target uint, pname uint, param float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlTexParameterf(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.float(param))
	}
}

func (ptr *QOpenGLFunctions) GlTexParameterfv(target uint, pname uint, params float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlTexParameterfv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.float(params))
	}
}

func (ptr *QOpenGLFunctions) GlTexParameteri(target uint, pname uint, param int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlTexParameteri(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.int(int32(param)))
	}
}

func (ptr *QOpenGLFunctions) GlTexParameteriv(target uint, pname uint, params int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlTexParameteriv(ptr.Pointer(), C.uint(uint32(target)), C.uint(uint32(pname)), C.int(int32(params)))
	}
}

func (ptr *QOpenGLFunctions) GlTexSubImage2D(target uint, level int, xoffset int, yoffset int, width int, height int, format uint, ty uint, pixels unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlTexSubImage2D(ptr.Pointer(), C.uint(uint32(target)), C.int(int32(level)), C.int(int32(xoffset)), C.int(int32(yoffset)), C.int(int32(width)), C.int(int32(height)), C.uint(uint32(format)), C.uint(uint32(ty)), pixels)
	}
}

func (ptr *QOpenGLFunctions) GlUniform1f(location int, x float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform1f(ptr.Pointer(), C.int(int32(location)), C.float(x))
	}
}

func (ptr *QOpenGLFunctions) GlUniform1fv(location int, count int, v float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform1fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.float(v))
	}
}

func (ptr *QOpenGLFunctions) GlUniform1i(location int, x int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform1i(ptr.Pointer(), C.int(int32(location)), C.int(int32(x)))
	}
}

func (ptr *QOpenGLFunctions) GlUniform1iv(location int, count int, v int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform1iv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.int(int32(v)))
	}
}

func (ptr *QOpenGLFunctions) GlUniform2f(location int, x float32, y float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform2f(ptr.Pointer(), C.int(int32(location)), C.float(x), C.float(y))
	}
}

func (ptr *QOpenGLFunctions) GlUniform2fv(location int, count int, v float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform2fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.float(v))
	}
}

func (ptr *QOpenGLFunctions) GlUniform2i(location int, x int, y int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform2i(ptr.Pointer(), C.int(int32(location)), C.int(int32(x)), C.int(int32(y)))
	}
}

func (ptr *QOpenGLFunctions) GlUniform2iv(location int, count int, v int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform2iv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.int(int32(v)))
	}
}

func (ptr *QOpenGLFunctions) GlUniform3f(location int, x float32, y float32, z float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform3f(ptr.Pointer(), C.int(int32(location)), C.float(x), C.float(y), C.float(z))
	}
}

func (ptr *QOpenGLFunctions) GlUniform3fv(location int, count int, v float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform3fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.float(v))
	}
}

func (ptr *QOpenGLFunctions) GlUniform3i(location int, x int, y int, z int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform3i(ptr.Pointer(), C.int(int32(location)), C.int(int32(x)), C.int(int32(y)), C.int(int32(z)))
	}
}

func (ptr *QOpenGLFunctions) GlUniform3iv(location int, count int, v int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform3iv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.int(int32(v)))
	}
}

func (ptr *QOpenGLFunctions) GlUniform4f(location int, x float32, y float32, z float32, w float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform4f(ptr.Pointer(), C.int(int32(location)), C.float(x), C.float(y), C.float(z), C.float(w))
	}
}

func (ptr *QOpenGLFunctions) GlUniform4fv(location int, count int, v float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform4fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.float(v))
	}
}

func (ptr *QOpenGLFunctions) GlUniform4i(location int, x int, y int, z int, w int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform4i(ptr.Pointer(), C.int(int32(location)), C.int(int32(x)), C.int(int32(y)), C.int(int32(z)), C.int(int32(w)))
	}
}

func (ptr *QOpenGLFunctions) GlUniform4iv(location int, count int, v int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniform4iv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.int(int32(v)))
	}
}

func (ptr *QOpenGLFunctions) GlUniformMatrix2fv(location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniformMatrix2fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLFunctions) GlUniformMatrix3fv(location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniformMatrix3fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLFunctions) GlUniformMatrix4fv(location int, count int, transpose bool, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUniformMatrix4fv(ptr.Pointer(), C.int(int32(location)), C.int(int32(count)), C.char(int8(qt.GoBoolToInt(transpose))), C.float(value))
	}
}

func (ptr *QOpenGLFunctions) GlUseProgram(program uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlUseProgram(ptr.Pointer(), C.uint(uint32(program)))
	}
}

func (ptr *QOpenGLFunctions) GlValidateProgram(program uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlValidateProgram(ptr.Pointer(), C.uint(uint32(program)))
	}
}

func (ptr *QOpenGLFunctions) GlVertexAttrib1f(indx uint, x float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlVertexAttrib1f(ptr.Pointer(), C.uint(uint32(indx)), C.float(x))
	}
}

func (ptr *QOpenGLFunctions) GlVertexAttrib1fv(indx uint, values float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlVertexAttrib1fv(ptr.Pointer(), C.uint(uint32(indx)), C.float(values))
	}
}

func (ptr *QOpenGLFunctions) GlVertexAttrib2f(indx uint, x float32, y float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlVertexAttrib2f(ptr.Pointer(), C.uint(uint32(indx)), C.float(x), C.float(y))
	}
}

func (ptr *QOpenGLFunctions) GlVertexAttrib2fv(indx uint, values float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlVertexAttrib2fv(ptr.Pointer(), C.uint(uint32(indx)), C.float(values))
	}
}

func (ptr *QOpenGLFunctions) GlVertexAttrib3f(indx uint, x float32, y float32, z float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlVertexAttrib3f(ptr.Pointer(), C.uint(uint32(indx)), C.float(x), C.float(y), C.float(z))
	}
}

func (ptr *QOpenGLFunctions) GlVertexAttrib3fv(indx uint, values float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlVertexAttrib3fv(ptr.Pointer(), C.uint(uint32(indx)), C.float(values))
	}
}

func (ptr *QOpenGLFunctions) GlVertexAttrib4f(indx uint, x float32, y float32, z float32, w float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlVertexAttrib4f(ptr.Pointer(), C.uint(uint32(indx)), C.float(x), C.float(y), C.float(z), C.float(w))
	}
}

func (ptr *QOpenGLFunctions) GlVertexAttrib4fv(indx uint, values float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlVertexAttrib4fv(ptr.Pointer(), C.uint(uint32(indx)), C.float(values))
	}
}

func (ptr *QOpenGLFunctions) GlVertexAttribPointer(indx uint, size int, ty uint, normalized bool, stride int, p unsafe.Pointer) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlVertexAttribPointer(ptr.Pointer(), C.uint(uint32(indx)), C.int(int32(size)), C.uint(uint32(ty)), C.char(int8(qt.GoBoolToInt(normalized))), C.int(int32(stride)), p)
	}
}

func (ptr *QOpenGLFunctions) GlViewport(x int, y int, width int, height int) {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_GlViewport(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QOpenGLFunctions) HasOpenGLFeature(feature QOpenGLFunctions__OpenGLFeature) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLFunctions_HasOpenGLFeature(ptr.Pointer(), C.longlong(feature))) != 0
	}
	return false
}

func (ptr *QOpenGLFunctions) InitializeOpenGLFunctions() {
	if ptr.Pointer() != nil {
		C.QOpenGLFunctions_InitializeOpenGLFunctions(ptr.Pointer())
	}
}

func (ptr *QOpenGLFunctions) OpenGLFeatures() QOpenGLFunctions__OpenGLFeature {
	if ptr.Pointer() != nil {
		return QOpenGLFunctions__OpenGLFeature(C.QOpenGLFunctions_OpenGLFeatures(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLFunctions) DestroyQOpenGLFunctions() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLFunctions_DestroyQOpenGLFunctions(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_0 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_1_0_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_1_0_PTR() *QOpenGLFunctions_1_0
}

func (ptr *QOpenGLFunctions_1_0) QOpenGLFunctions_1_0_PTR() *QOpenGLFunctions_1_0 {
	return ptr
}

func (ptr *QOpenGLFunctions_1_0) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_0) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_0(ptr QOpenGLFunctions_1_0_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_0_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_0FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_0) {
	n = new(QOpenGLFunctions_1_0)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_0) DestroyQOpenGLFunctions_1_0() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_0_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_0_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_0_CoreBackend_PTR() *QOpenGLFunctions_1_0_CoreBackend
}

func (ptr *QOpenGLFunctions_1_0_CoreBackend) QOpenGLFunctions_1_0_CoreBackend_PTR() *QOpenGLFunctions_1_0_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_0_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_0_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_0_CoreBackend(ptr QOpenGLFunctions_1_0_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_0_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_0_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_0_CoreBackend) {
	n = new(QOpenGLFunctions_1_0_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_0_CoreBackend) DestroyQOpenGLFunctions_1_0_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_0_DeprecatedBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_0_DeprecatedBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_0_DeprecatedBackend_PTR() *QOpenGLFunctions_1_0_DeprecatedBackend
}

func (ptr *QOpenGLFunctions_1_0_DeprecatedBackend) QOpenGLFunctions_1_0_DeprecatedBackend_PTR() *QOpenGLFunctions_1_0_DeprecatedBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_0_DeprecatedBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_0_DeprecatedBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_0_DeprecatedBackend(ptr QOpenGLFunctions_1_0_DeprecatedBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_0_DeprecatedBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_0_DeprecatedBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_0_DeprecatedBackend) {
	n = new(QOpenGLFunctions_1_0_DeprecatedBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_0_DeprecatedBackend) DestroyQOpenGLFunctions_1_0_DeprecatedBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_1 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_1_1_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_1_1_PTR() *QOpenGLFunctions_1_1
}

func (ptr *QOpenGLFunctions_1_1) QOpenGLFunctions_1_1_PTR() *QOpenGLFunctions_1_1 {
	return ptr
}

func (ptr *QOpenGLFunctions_1_1) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_1) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_1(ptr QOpenGLFunctions_1_1_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_1_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_1FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_1) {
	n = new(QOpenGLFunctions_1_1)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_1) DestroyQOpenGLFunctions_1_1() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_1_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_1_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_1_CoreBackend_PTR() *QOpenGLFunctions_1_1_CoreBackend
}

func (ptr *QOpenGLFunctions_1_1_CoreBackend) QOpenGLFunctions_1_1_CoreBackend_PTR() *QOpenGLFunctions_1_1_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_1_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_1_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_1_CoreBackend(ptr QOpenGLFunctions_1_1_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_1_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_1_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_1_CoreBackend) {
	n = new(QOpenGLFunctions_1_1_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_1_CoreBackend) DestroyQOpenGLFunctions_1_1_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_1_DeprecatedBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_1_DeprecatedBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_1_DeprecatedBackend_PTR() *QOpenGLFunctions_1_1_DeprecatedBackend
}

func (ptr *QOpenGLFunctions_1_1_DeprecatedBackend) QOpenGLFunctions_1_1_DeprecatedBackend_PTR() *QOpenGLFunctions_1_1_DeprecatedBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_1_DeprecatedBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_1_DeprecatedBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_1_DeprecatedBackend(ptr QOpenGLFunctions_1_1_DeprecatedBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_1_DeprecatedBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_1_DeprecatedBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_1_DeprecatedBackend) {
	n = new(QOpenGLFunctions_1_1_DeprecatedBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_1_DeprecatedBackend) DestroyQOpenGLFunctions_1_1_DeprecatedBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_2 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_1_2_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_1_2_PTR() *QOpenGLFunctions_1_2
}

func (ptr *QOpenGLFunctions_1_2) QOpenGLFunctions_1_2_PTR() *QOpenGLFunctions_1_2 {
	return ptr
}

func (ptr *QOpenGLFunctions_1_2) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_2) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_2(ptr QOpenGLFunctions_1_2_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_2_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_2FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_2) {
	n = new(QOpenGLFunctions_1_2)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_2) DestroyQOpenGLFunctions_1_2() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_2_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_2_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_2_CoreBackend_PTR() *QOpenGLFunctions_1_2_CoreBackend
}

func (ptr *QOpenGLFunctions_1_2_CoreBackend) QOpenGLFunctions_1_2_CoreBackend_PTR() *QOpenGLFunctions_1_2_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_2_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_2_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_2_CoreBackend(ptr QOpenGLFunctions_1_2_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_2_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_2_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_2_CoreBackend) {
	n = new(QOpenGLFunctions_1_2_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_2_CoreBackend) DestroyQOpenGLFunctions_1_2_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_2_DeprecatedBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_2_DeprecatedBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_2_DeprecatedBackend_PTR() *QOpenGLFunctions_1_2_DeprecatedBackend
}

func (ptr *QOpenGLFunctions_1_2_DeprecatedBackend) QOpenGLFunctions_1_2_DeprecatedBackend_PTR() *QOpenGLFunctions_1_2_DeprecatedBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_2_DeprecatedBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_2_DeprecatedBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_2_DeprecatedBackend(ptr QOpenGLFunctions_1_2_DeprecatedBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_2_DeprecatedBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_2_DeprecatedBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_2_DeprecatedBackend) {
	n = new(QOpenGLFunctions_1_2_DeprecatedBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_2_DeprecatedBackend) DestroyQOpenGLFunctions_1_2_DeprecatedBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_3 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_1_3_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_1_3_PTR() *QOpenGLFunctions_1_3
}

func (ptr *QOpenGLFunctions_1_3) QOpenGLFunctions_1_3_PTR() *QOpenGLFunctions_1_3 {
	return ptr
}

func (ptr *QOpenGLFunctions_1_3) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_3) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_3(ptr QOpenGLFunctions_1_3_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_3_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_3FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_3) {
	n = new(QOpenGLFunctions_1_3)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_3) DestroyQOpenGLFunctions_1_3() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_3_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_3_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_3_CoreBackend_PTR() *QOpenGLFunctions_1_3_CoreBackend
}

func (ptr *QOpenGLFunctions_1_3_CoreBackend) QOpenGLFunctions_1_3_CoreBackend_PTR() *QOpenGLFunctions_1_3_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_3_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_3_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_3_CoreBackend(ptr QOpenGLFunctions_1_3_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_3_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_3_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_3_CoreBackend) {
	n = new(QOpenGLFunctions_1_3_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_3_CoreBackend) DestroyQOpenGLFunctions_1_3_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_3_DeprecatedBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_3_DeprecatedBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_3_DeprecatedBackend_PTR() *QOpenGLFunctions_1_3_DeprecatedBackend
}

func (ptr *QOpenGLFunctions_1_3_DeprecatedBackend) QOpenGLFunctions_1_3_DeprecatedBackend_PTR() *QOpenGLFunctions_1_3_DeprecatedBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_3_DeprecatedBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_3_DeprecatedBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_3_DeprecatedBackend(ptr QOpenGLFunctions_1_3_DeprecatedBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_3_DeprecatedBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_3_DeprecatedBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_3_DeprecatedBackend) {
	n = new(QOpenGLFunctions_1_3_DeprecatedBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_3_DeprecatedBackend) DestroyQOpenGLFunctions_1_3_DeprecatedBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_4 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_1_4_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_1_4_PTR() *QOpenGLFunctions_1_4
}

func (ptr *QOpenGLFunctions_1_4) QOpenGLFunctions_1_4_PTR() *QOpenGLFunctions_1_4 {
	return ptr
}

func (ptr *QOpenGLFunctions_1_4) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_4) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_4(ptr QOpenGLFunctions_1_4_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_4_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_4FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_4) {
	n = new(QOpenGLFunctions_1_4)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_4) DestroyQOpenGLFunctions_1_4() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_4_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_4_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_4_CoreBackend_PTR() *QOpenGLFunctions_1_4_CoreBackend
}

func (ptr *QOpenGLFunctions_1_4_CoreBackend) QOpenGLFunctions_1_4_CoreBackend_PTR() *QOpenGLFunctions_1_4_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_4_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_4_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_4_CoreBackend(ptr QOpenGLFunctions_1_4_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_4_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_4_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_4_CoreBackend) {
	n = new(QOpenGLFunctions_1_4_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_4_CoreBackend) DestroyQOpenGLFunctions_1_4_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_4_DeprecatedBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_4_DeprecatedBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_4_DeprecatedBackend_PTR() *QOpenGLFunctions_1_4_DeprecatedBackend
}

func (ptr *QOpenGLFunctions_1_4_DeprecatedBackend) QOpenGLFunctions_1_4_DeprecatedBackend_PTR() *QOpenGLFunctions_1_4_DeprecatedBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_4_DeprecatedBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_4_DeprecatedBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_4_DeprecatedBackend(ptr QOpenGLFunctions_1_4_DeprecatedBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_4_DeprecatedBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_4_DeprecatedBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_4_DeprecatedBackend) {
	n = new(QOpenGLFunctions_1_4_DeprecatedBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_4_DeprecatedBackend) DestroyQOpenGLFunctions_1_4_DeprecatedBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_5 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_1_5_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_1_5_PTR() *QOpenGLFunctions_1_5
}

func (ptr *QOpenGLFunctions_1_5) QOpenGLFunctions_1_5_PTR() *QOpenGLFunctions_1_5 {
	return ptr
}

func (ptr *QOpenGLFunctions_1_5) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_5) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_5(ptr QOpenGLFunctions_1_5_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_5_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_5FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_5) {
	n = new(QOpenGLFunctions_1_5)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_5) DestroyQOpenGLFunctions_1_5() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_1_5_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_1_5_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_1_5_CoreBackend_PTR() *QOpenGLFunctions_1_5_CoreBackend
}

func (ptr *QOpenGLFunctions_1_5_CoreBackend) QOpenGLFunctions_1_5_CoreBackend_PTR() *QOpenGLFunctions_1_5_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_1_5_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_1_5_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_1_5_CoreBackend(ptr QOpenGLFunctions_1_5_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_1_5_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_1_5_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_1_5_CoreBackend) {
	n = new(QOpenGLFunctions_1_5_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_1_5_CoreBackend) DestroyQOpenGLFunctions_1_5_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_2_0 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_2_0_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_2_0_PTR() *QOpenGLFunctions_2_0
}

func (ptr *QOpenGLFunctions_2_0) QOpenGLFunctions_2_0_PTR() *QOpenGLFunctions_2_0 {
	return ptr
}

func (ptr *QOpenGLFunctions_2_0) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_2_0) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_2_0(ptr QOpenGLFunctions_2_0_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_2_0_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_2_0FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_2_0) {
	n = new(QOpenGLFunctions_2_0)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_2_0) DestroyQOpenGLFunctions_2_0() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_2_0_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_2_0_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_2_0_CoreBackend_PTR() *QOpenGLFunctions_2_0_CoreBackend
}

func (ptr *QOpenGLFunctions_2_0_CoreBackend) QOpenGLFunctions_2_0_CoreBackend_PTR() *QOpenGLFunctions_2_0_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_2_0_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_2_0_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_2_0_CoreBackend(ptr QOpenGLFunctions_2_0_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_2_0_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_2_0_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_2_0_CoreBackend) {
	n = new(QOpenGLFunctions_2_0_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_2_0_CoreBackend) DestroyQOpenGLFunctions_2_0_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_2_0_DeprecatedBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_2_0_DeprecatedBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_2_0_DeprecatedBackend_PTR() *QOpenGLFunctions_2_0_DeprecatedBackend
}

func (ptr *QOpenGLFunctions_2_0_DeprecatedBackend) QOpenGLFunctions_2_0_DeprecatedBackend_PTR() *QOpenGLFunctions_2_0_DeprecatedBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_2_0_DeprecatedBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_2_0_DeprecatedBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_2_0_DeprecatedBackend(ptr QOpenGLFunctions_2_0_DeprecatedBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_2_0_DeprecatedBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_2_0_DeprecatedBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_2_0_DeprecatedBackend) {
	n = new(QOpenGLFunctions_2_0_DeprecatedBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_2_0_DeprecatedBackend) DestroyQOpenGLFunctions_2_0_DeprecatedBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_2_1 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_2_1_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_2_1_PTR() *QOpenGLFunctions_2_1
}

func (ptr *QOpenGLFunctions_2_1) QOpenGLFunctions_2_1_PTR() *QOpenGLFunctions_2_1 {
	return ptr
}

func (ptr *QOpenGLFunctions_2_1) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_2_1) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_2_1(ptr QOpenGLFunctions_2_1_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_2_1_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_2_1FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_2_1) {
	n = new(QOpenGLFunctions_2_1)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_2_1) DestroyQOpenGLFunctions_2_1() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_2_1_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_2_1_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_2_1_CoreBackend_PTR() *QOpenGLFunctions_2_1_CoreBackend
}

func (ptr *QOpenGLFunctions_2_1_CoreBackend) QOpenGLFunctions_2_1_CoreBackend_PTR() *QOpenGLFunctions_2_1_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_2_1_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_2_1_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_2_1_CoreBackend(ptr QOpenGLFunctions_2_1_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_2_1_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_2_1_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_2_1_CoreBackend) {
	n = new(QOpenGLFunctions_2_1_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_2_1_CoreBackend) DestroyQOpenGLFunctions_2_1_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_0 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_3_0_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_3_0_PTR() *QOpenGLFunctions_3_0
}

func (ptr *QOpenGLFunctions_3_0) QOpenGLFunctions_3_0_PTR() *QOpenGLFunctions_3_0 {
	return ptr
}

func (ptr *QOpenGLFunctions_3_0) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_0) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_0(ptr QOpenGLFunctions_3_0_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_0_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_0FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_0) {
	n = new(QOpenGLFunctions_3_0)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_0) DestroyQOpenGLFunctions_3_0() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_0_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_3_0_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_3_0_CoreBackend_PTR() *QOpenGLFunctions_3_0_CoreBackend
}

func (ptr *QOpenGLFunctions_3_0_CoreBackend) QOpenGLFunctions_3_0_CoreBackend_PTR() *QOpenGLFunctions_3_0_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_3_0_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_0_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_0_CoreBackend(ptr QOpenGLFunctions_3_0_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_0_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_0_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_0_CoreBackend) {
	n = new(QOpenGLFunctions_3_0_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_0_CoreBackend) DestroyQOpenGLFunctions_3_0_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_0_DeprecatedBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_3_0_DeprecatedBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_3_0_DeprecatedBackend_PTR() *QOpenGLFunctions_3_0_DeprecatedBackend
}

func (ptr *QOpenGLFunctions_3_0_DeprecatedBackend) QOpenGLFunctions_3_0_DeprecatedBackend_PTR() *QOpenGLFunctions_3_0_DeprecatedBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_3_0_DeprecatedBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_0_DeprecatedBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_0_DeprecatedBackend(ptr QOpenGLFunctions_3_0_DeprecatedBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_0_DeprecatedBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_0_DeprecatedBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_0_DeprecatedBackend) {
	n = new(QOpenGLFunctions_3_0_DeprecatedBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_0_DeprecatedBackend) DestroyQOpenGLFunctions_3_0_DeprecatedBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_1 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_3_1_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_3_1_PTR() *QOpenGLFunctions_3_1
}

func (ptr *QOpenGLFunctions_3_1) QOpenGLFunctions_3_1_PTR() *QOpenGLFunctions_3_1 {
	return ptr
}

func (ptr *QOpenGLFunctions_3_1) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_1) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_1(ptr QOpenGLFunctions_3_1_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_1_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_1FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_1) {
	n = new(QOpenGLFunctions_3_1)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_1) DestroyQOpenGLFunctions_3_1() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_1_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_3_1_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_3_1_CoreBackend_PTR() *QOpenGLFunctions_3_1_CoreBackend
}

func (ptr *QOpenGLFunctions_3_1_CoreBackend) QOpenGLFunctions_3_1_CoreBackend_PTR() *QOpenGLFunctions_3_1_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_3_1_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_1_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_1_CoreBackend(ptr QOpenGLFunctions_3_1_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_1_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_1_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_1_CoreBackend) {
	n = new(QOpenGLFunctions_3_1_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_1_CoreBackend) DestroyQOpenGLFunctions_3_1_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_2_Compatibility struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_3_2_Compatibility_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_3_2_Compatibility_PTR() *QOpenGLFunctions_3_2_Compatibility
}

func (ptr *QOpenGLFunctions_3_2_Compatibility) QOpenGLFunctions_3_2_Compatibility_PTR() *QOpenGLFunctions_3_2_Compatibility {
	return ptr
}

func (ptr *QOpenGLFunctions_3_2_Compatibility) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_2_Compatibility) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_2_Compatibility(ptr QOpenGLFunctions_3_2_Compatibility_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_2_Compatibility_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_2_CompatibilityFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_2_Compatibility) {
	n = new(QOpenGLFunctions_3_2_Compatibility)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_2_Compatibility) DestroyQOpenGLFunctions_3_2_Compatibility() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_2_Core struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_3_2_Core_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_3_2_Core_PTR() *QOpenGLFunctions_3_2_Core
}

func (ptr *QOpenGLFunctions_3_2_Core) QOpenGLFunctions_3_2_Core_PTR() *QOpenGLFunctions_3_2_Core {
	return ptr
}

func (ptr *QOpenGLFunctions_3_2_Core) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_2_Core) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_2_Core(ptr QOpenGLFunctions_3_2_Core_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_2_Core_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_2_CoreFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_2_Core) {
	n = new(QOpenGLFunctions_3_2_Core)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_2_Core) DestroyQOpenGLFunctions_3_2_Core() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_2_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_3_2_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_3_2_CoreBackend_PTR() *QOpenGLFunctions_3_2_CoreBackend
}

func (ptr *QOpenGLFunctions_3_2_CoreBackend) QOpenGLFunctions_3_2_CoreBackend_PTR() *QOpenGLFunctions_3_2_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_3_2_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_2_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_2_CoreBackend(ptr QOpenGLFunctions_3_2_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_2_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_2_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_2_CoreBackend) {
	n = new(QOpenGLFunctions_3_2_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_2_CoreBackend) DestroyQOpenGLFunctions_3_2_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_3_Compatibility struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_3_3_Compatibility_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_3_3_Compatibility_PTR() *QOpenGLFunctions_3_3_Compatibility
}

func (ptr *QOpenGLFunctions_3_3_Compatibility) QOpenGLFunctions_3_3_Compatibility_PTR() *QOpenGLFunctions_3_3_Compatibility {
	return ptr
}

func (ptr *QOpenGLFunctions_3_3_Compatibility) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_3_Compatibility) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_3_Compatibility(ptr QOpenGLFunctions_3_3_Compatibility_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_3_Compatibility_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_3_CompatibilityFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_3_Compatibility) {
	n = new(QOpenGLFunctions_3_3_Compatibility)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_3_Compatibility) DestroyQOpenGLFunctions_3_3_Compatibility() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_3_Core struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_3_3_Core_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_3_3_Core_PTR() *QOpenGLFunctions_3_3_Core
}

func (ptr *QOpenGLFunctions_3_3_Core) QOpenGLFunctions_3_3_Core_PTR() *QOpenGLFunctions_3_3_Core {
	return ptr
}

func (ptr *QOpenGLFunctions_3_3_Core) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_3_Core) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_3_Core(ptr QOpenGLFunctions_3_3_Core_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_3_Core_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_3_CoreFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_3_Core) {
	n = new(QOpenGLFunctions_3_3_Core)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_3_Core) DestroyQOpenGLFunctions_3_3_Core() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_3_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_3_3_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_3_3_CoreBackend_PTR() *QOpenGLFunctions_3_3_CoreBackend
}

func (ptr *QOpenGLFunctions_3_3_CoreBackend) QOpenGLFunctions_3_3_CoreBackend_PTR() *QOpenGLFunctions_3_3_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_3_3_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_3_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_3_CoreBackend(ptr QOpenGLFunctions_3_3_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_3_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_3_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_3_CoreBackend) {
	n = new(QOpenGLFunctions_3_3_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_3_CoreBackend) DestroyQOpenGLFunctions_3_3_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_3_3_DeprecatedBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_3_3_DeprecatedBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_3_3_DeprecatedBackend_PTR() *QOpenGLFunctions_3_3_DeprecatedBackend
}

func (ptr *QOpenGLFunctions_3_3_DeprecatedBackend) QOpenGLFunctions_3_3_DeprecatedBackend_PTR() *QOpenGLFunctions_3_3_DeprecatedBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_3_3_DeprecatedBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_3_3_DeprecatedBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_3_3_DeprecatedBackend(ptr QOpenGLFunctions_3_3_DeprecatedBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_3_3_DeprecatedBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_3_3_DeprecatedBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_3_3_DeprecatedBackend) {
	n = new(QOpenGLFunctions_3_3_DeprecatedBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_3_3_DeprecatedBackend) DestroyQOpenGLFunctions_3_3_DeprecatedBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_0_Compatibility struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_0_Compatibility_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_0_Compatibility_PTR() *QOpenGLFunctions_4_0_Compatibility
}

func (ptr *QOpenGLFunctions_4_0_Compatibility) QOpenGLFunctions_4_0_Compatibility_PTR() *QOpenGLFunctions_4_0_Compatibility {
	return ptr
}

func (ptr *QOpenGLFunctions_4_0_Compatibility) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_0_Compatibility) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_0_Compatibility(ptr QOpenGLFunctions_4_0_Compatibility_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_0_Compatibility_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_0_CompatibilityFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_0_Compatibility) {
	n = new(QOpenGLFunctions_4_0_Compatibility)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_0_Compatibility) DestroyQOpenGLFunctions_4_0_Compatibility() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_0_Core struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_0_Core_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_0_Core_PTR() *QOpenGLFunctions_4_0_Core
}

func (ptr *QOpenGLFunctions_4_0_Core) QOpenGLFunctions_4_0_Core_PTR() *QOpenGLFunctions_4_0_Core {
	return ptr
}

func (ptr *QOpenGLFunctions_4_0_Core) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_0_Core) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_0_Core(ptr QOpenGLFunctions_4_0_Core_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_0_Core_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_0_CoreFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_0_Core) {
	n = new(QOpenGLFunctions_4_0_Core)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_0_Core) DestroyQOpenGLFunctions_4_0_Core() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_0_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_4_0_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_4_0_CoreBackend_PTR() *QOpenGLFunctions_4_0_CoreBackend
}

func (ptr *QOpenGLFunctions_4_0_CoreBackend) QOpenGLFunctions_4_0_CoreBackend_PTR() *QOpenGLFunctions_4_0_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_4_0_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_0_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_0_CoreBackend(ptr QOpenGLFunctions_4_0_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_0_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_0_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_0_CoreBackend) {
	n = new(QOpenGLFunctions_4_0_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_0_CoreBackend) DestroyQOpenGLFunctions_4_0_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_1_Compatibility struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_1_Compatibility_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_1_Compatibility_PTR() *QOpenGLFunctions_4_1_Compatibility
}

func (ptr *QOpenGLFunctions_4_1_Compatibility) QOpenGLFunctions_4_1_Compatibility_PTR() *QOpenGLFunctions_4_1_Compatibility {
	return ptr
}

func (ptr *QOpenGLFunctions_4_1_Compatibility) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_1_Compatibility) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_1_Compatibility(ptr QOpenGLFunctions_4_1_Compatibility_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_1_Compatibility_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_1_CompatibilityFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_1_Compatibility) {
	n = new(QOpenGLFunctions_4_1_Compatibility)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_1_Compatibility) DestroyQOpenGLFunctions_4_1_Compatibility() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_1_Core struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_1_Core_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_1_Core_PTR() *QOpenGLFunctions_4_1_Core
}

func (ptr *QOpenGLFunctions_4_1_Core) QOpenGLFunctions_4_1_Core_PTR() *QOpenGLFunctions_4_1_Core {
	return ptr
}

func (ptr *QOpenGLFunctions_4_1_Core) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_1_Core) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_1_Core(ptr QOpenGLFunctions_4_1_Core_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_1_Core_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_1_CoreFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_1_Core) {
	n = new(QOpenGLFunctions_4_1_Core)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_1_Core) DestroyQOpenGLFunctions_4_1_Core() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_1_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_4_1_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_4_1_CoreBackend_PTR() *QOpenGLFunctions_4_1_CoreBackend
}

func (ptr *QOpenGLFunctions_4_1_CoreBackend) QOpenGLFunctions_4_1_CoreBackend_PTR() *QOpenGLFunctions_4_1_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_4_1_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_1_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_1_CoreBackend(ptr QOpenGLFunctions_4_1_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_1_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_1_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_1_CoreBackend) {
	n = new(QOpenGLFunctions_4_1_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_1_CoreBackend) DestroyQOpenGLFunctions_4_1_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_2_Compatibility struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_2_Compatibility_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_2_Compatibility_PTR() *QOpenGLFunctions_4_2_Compatibility
}

func (ptr *QOpenGLFunctions_4_2_Compatibility) QOpenGLFunctions_4_2_Compatibility_PTR() *QOpenGLFunctions_4_2_Compatibility {
	return ptr
}

func (ptr *QOpenGLFunctions_4_2_Compatibility) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_2_Compatibility) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_2_Compatibility(ptr QOpenGLFunctions_4_2_Compatibility_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_2_Compatibility_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_2_CompatibilityFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_2_Compatibility) {
	n = new(QOpenGLFunctions_4_2_Compatibility)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_2_Compatibility) DestroyQOpenGLFunctions_4_2_Compatibility() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_2_Core struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_2_Core_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_2_Core_PTR() *QOpenGLFunctions_4_2_Core
}

func (ptr *QOpenGLFunctions_4_2_Core) QOpenGLFunctions_4_2_Core_PTR() *QOpenGLFunctions_4_2_Core {
	return ptr
}

func (ptr *QOpenGLFunctions_4_2_Core) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_2_Core) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_2_Core(ptr QOpenGLFunctions_4_2_Core_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_2_Core_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_2_CoreFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_2_Core) {
	n = new(QOpenGLFunctions_4_2_Core)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_2_Core) DestroyQOpenGLFunctions_4_2_Core() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_2_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_4_2_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_4_2_CoreBackend_PTR() *QOpenGLFunctions_4_2_CoreBackend
}

func (ptr *QOpenGLFunctions_4_2_CoreBackend) QOpenGLFunctions_4_2_CoreBackend_PTR() *QOpenGLFunctions_4_2_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_4_2_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_2_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_2_CoreBackend(ptr QOpenGLFunctions_4_2_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_2_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_2_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_2_CoreBackend) {
	n = new(QOpenGLFunctions_4_2_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_2_CoreBackend) DestroyQOpenGLFunctions_4_2_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_3_Compatibility struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_3_Compatibility_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_3_Compatibility_PTR() *QOpenGLFunctions_4_3_Compatibility
}

func (ptr *QOpenGLFunctions_4_3_Compatibility) QOpenGLFunctions_4_3_Compatibility_PTR() *QOpenGLFunctions_4_3_Compatibility {
	return ptr
}

func (ptr *QOpenGLFunctions_4_3_Compatibility) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_3_Compatibility) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_3_Compatibility(ptr QOpenGLFunctions_4_3_Compatibility_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_3_Compatibility_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_3_CompatibilityFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_3_Compatibility) {
	n = new(QOpenGLFunctions_4_3_Compatibility)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_3_Compatibility) DestroyQOpenGLFunctions_4_3_Compatibility() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_3_Core struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_3_Core_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_3_Core_PTR() *QOpenGLFunctions_4_3_Core
}

func (ptr *QOpenGLFunctions_4_3_Core) QOpenGLFunctions_4_3_Core_PTR() *QOpenGLFunctions_4_3_Core {
	return ptr
}

func (ptr *QOpenGLFunctions_4_3_Core) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_3_Core) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_3_Core(ptr QOpenGLFunctions_4_3_Core_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_3_Core_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_3_CoreFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_3_Core) {
	n = new(QOpenGLFunctions_4_3_Core)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_3_Core) DestroyQOpenGLFunctions_4_3_Core() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_3_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_4_3_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_4_3_CoreBackend_PTR() *QOpenGLFunctions_4_3_CoreBackend
}

func (ptr *QOpenGLFunctions_4_3_CoreBackend) QOpenGLFunctions_4_3_CoreBackend_PTR() *QOpenGLFunctions_4_3_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_4_3_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_3_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_3_CoreBackend(ptr QOpenGLFunctions_4_3_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_3_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_3_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_3_CoreBackend) {
	n = new(QOpenGLFunctions_4_3_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_3_CoreBackend) DestroyQOpenGLFunctions_4_3_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_4_Compatibility struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_4_Compatibility_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_4_Compatibility_PTR() *QOpenGLFunctions_4_4_Compatibility
}

func (ptr *QOpenGLFunctions_4_4_Compatibility) QOpenGLFunctions_4_4_Compatibility_PTR() *QOpenGLFunctions_4_4_Compatibility {
	return ptr
}

func (ptr *QOpenGLFunctions_4_4_Compatibility) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_4_Compatibility) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_4_Compatibility(ptr QOpenGLFunctions_4_4_Compatibility_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_4_Compatibility_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_4_CompatibilityFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_4_Compatibility) {
	n = new(QOpenGLFunctions_4_4_Compatibility)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_4_Compatibility) DestroyQOpenGLFunctions_4_4_Compatibility() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_4_Core struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_4_Core_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_4_Core_PTR() *QOpenGLFunctions_4_4_Core
}

func (ptr *QOpenGLFunctions_4_4_Core) QOpenGLFunctions_4_4_Core_PTR() *QOpenGLFunctions_4_4_Core {
	return ptr
}

func (ptr *QOpenGLFunctions_4_4_Core) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_4_Core) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_4_Core(ptr QOpenGLFunctions_4_4_Core_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_4_Core_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_4_CoreFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_4_Core) {
	n = new(QOpenGLFunctions_4_4_Core)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_4_Core) DestroyQOpenGLFunctions_4_4_Core() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_4_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_4_4_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_4_4_CoreBackend_PTR() *QOpenGLFunctions_4_4_CoreBackend
}

func (ptr *QOpenGLFunctions_4_4_CoreBackend) QOpenGLFunctions_4_4_CoreBackend_PTR() *QOpenGLFunctions_4_4_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_4_4_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_4_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_4_CoreBackend(ptr QOpenGLFunctions_4_4_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_4_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_4_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_4_CoreBackend) {
	n = new(QOpenGLFunctions_4_4_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_4_CoreBackend) DestroyQOpenGLFunctions_4_4_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_5_Compatibility struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_5_Compatibility_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_5_Compatibility_PTR() *QOpenGLFunctions_4_5_Compatibility
}

func (ptr *QOpenGLFunctions_4_5_Compatibility) QOpenGLFunctions_4_5_Compatibility_PTR() *QOpenGLFunctions_4_5_Compatibility {
	return ptr
}

func (ptr *QOpenGLFunctions_4_5_Compatibility) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_5_Compatibility) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_5_Compatibility(ptr QOpenGLFunctions_4_5_Compatibility_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_5_Compatibility_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_5_CompatibilityFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_5_Compatibility) {
	n = new(QOpenGLFunctions_4_5_Compatibility)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_5_Compatibility) DestroyQOpenGLFunctions_4_5_Compatibility() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_5_Core struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_4_5_Core_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_4_5_Core_PTR() *QOpenGLFunctions_4_5_Core
}

func (ptr *QOpenGLFunctions_4_5_Core) QOpenGLFunctions_4_5_Core_PTR() *QOpenGLFunctions_4_5_Core {
	return ptr
}

func (ptr *QOpenGLFunctions_4_5_Core) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_5_Core) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_5_Core(ptr QOpenGLFunctions_4_5_Core_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_5_Core_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_5_CoreFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_5_Core) {
	n = new(QOpenGLFunctions_4_5_Core)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_5_Core) DestroyQOpenGLFunctions_4_5_Core() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_5_CoreBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_4_5_CoreBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_4_5_CoreBackend_PTR() *QOpenGLFunctions_4_5_CoreBackend
}

func (ptr *QOpenGLFunctions_4_5_CoreBackend) QOpenGLFunctions_4_5_CoreBackend_PTR() *QOpenGLFunctions_4_5_CoreBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_4_5_CoreBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_5_CoreBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_5_CoreBackend(ptr QOpenGLFunctions_4_5_CoreBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_5_CoreBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_5_CoreBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_5_CoreBackend) {
	n = new(QOpenGLFunctions_4_5_CoreBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_5_CoreBackend) DestroyQOpenGLFunctions_4_5_CoreBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_4_5_DeprecatedBackend struct {
	QOpenGLVersionFunctionsBackend
}

type QOpenGLFunctions_4_5_DeprecatedBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_ITF
	QOpenGLFunctions_4_5_DeprecatedBackend_PTR() *QOpenGLFunctions_4_5_DeprecatedBackend
}

func (ptr *QOpenGLFunctions_4_5_DeprecatedBackend) QOpenGLFunctions_4_5_DeprecatedBackend_PTR() *QOpenGLFunctions_4_5_DeprecatedBackend {
	return ptr
}

func (ptr *QOpenGLFunctions_4_5_DeprecatedBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_4_5_DeprecatedBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QOpenGLVersionFunctionsBackend_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_4_5_DeprecatedBackend(ptr QOpenGLFunctions_4_5_DeprecatedBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_4_5_DeprecatedBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_4_5_DeprecatedBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_4_5_DeprecatedBackend) {
	n = new(QOpenGLFunctions_4_5_DeprecatedBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_4_5_DeprecatedBackend) DestroyQOpenGLFunctions_4_5_DeprecatedBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLFunctions_ES2 struct {
	QAbstractOpenGLFunctions
}

type QOpenGLFunctions_ES2_ITF interface {
	QAbstractOpenGLFunctions_ITF
	QOpenGLFunctions_ES2_PTR() *QOpenGLFunctions_ES2
}

func (ptr *QOpenGLFunctions_ES2) QOpenGLFunctions_ES2_PTR() *QOpenGLFunctions_ES2 {
	return ptr
}

func (ptr *QOpenGLFunctions_ES2) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractOpenGLFunctions_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLFunctions_ES2) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractOpenGLFunctions_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLFunctions_ES2(ptr QOpenGLFunctions_ES2_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLFunctions_ES2_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLFunctions_ES2FromPointer(ptr unsafe.Pointer) (n *QOpenGLFunctions_ES2) {
	n = new(QOpenGLFunctions_ES2)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLFunctions_ES2) DestroyQOpenGLFunctions_ES2() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLPaintDevice struct {
	QPaintDevice
}

type QOpenGLPaintDevice_ITF interface {
	QPaintDevice_ITF
	QOpenGLPaintDevice_PTR() *QOpenGLPaintDevice
}

func (ptr *QOpenGLPaintDevice) QOpenGLPaintDevice_PTR() *QOpenGLPaintDevice {
	return ptr
}

func (ptr *QOpenGLPaintDevice) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintDevice_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLPaintDevice) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QPaintDevice_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLPaintDevice(ptr QOpenGLPaintDevice_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLPaintDevice_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLPaintDeviceFromPointer(ptr unsafe.Pointer) (n *QOpenGLPaintDevice) {
	n = new(QOpenGLPaintDevice)
	n.SetPointer(ptr)
	return
}
func NewQOpenGLPaintDevice() *QOpenGLPaintDevice {
	return NewQOpenGLPaintDeviceFromPointer(C.QOpenGLPaintDevice_NewQOpenGLPaintDevice())
}

func NewQOpenGLPaintDevice2(size core.QSize_ITF) *QOpenGLPaintDevice {
	return NewQOpenGLPaintDeviceFromPointer(C.QOpenGLPaintDevice_NewQOpenGLPaintDevice2(core.PointerFromQSize(size)))
}

func NewQOpenGLPaintDevice3(width int, height int) *QOpenGLPaintDevice {
	return NewQOpenGLPaintDeviceFromPointer(C.QOpenGLPaintDevice_NewQOpenGLPaintDevice3(C.int(int32(width)), C.int(int32(height))))
}

func (ptr *QOpenGLPaintDevice) Context() *QOpenGLContext {
	if ptr.Pointer() != nil {
		tmpValue := NewQOpenGLContextFromPointer(C.QOpenGLPaintDevice_Context(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLPaintDevice) DotsPerMeterX() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QOpenGLPaintDevice_DotsPerMeterX(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLPaintDevice) DotsPerMeterY() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QOpenGLPaintDevice_DotsPerMeterY(ptr.Pointer()))
	}
	return 0
}

//export callbackQOpenGLPaintDevice_EnsureActiveTarget
func callbackQOpenGLPaintDevice_EnsureActiveTarget(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "ensureActiveTarget"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLPaintDeviceFromPointer(ptr).EnsureActiveTargetDefault()
	}
}

func (ptr *QOpenGLPaintDevice) ConnectEnsureActiveTarget(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "ensureActiveTarget"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "ensureActiveTarget", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "ensureActiveTarget", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLPaintDevice) DisconnectEnsureActiveTarget() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "ensureActiveTarget")
	}
}

func (ptr *QOpenGLPaintDevice) EnsureActiveTarget() {
	if ptr.Pointer() != nil {
		C.QOpenGLPaintDevice_EnsureActiveTarget(ptr.Pointer())
	}
}

func (ptr *QOpenGLPaintDevice) EnsureActiveTargetDefault() {
	if ptr.Pointer() != nil {
		C.QOpenGLPaintDevice_EnsureActiveTargetDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLPaintDevice_PaintEngine
func callbackQOpenGLPaintDevice_PaintEngine(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "paintEngine"); signal != nil {
		return PointerFromQPaintEngine((*(*func() *QPaintEngine)(signal))())
	}

	return PointerFromQPaintEngine(NewQOpenGLPaintDeviceFromPointer(ptr).PaintEngineDefault())
}

func (ptr *QOpenGLPaintDevice) ConnectPaintEngine(f func() *QPaintEngine) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "paintEngine"); signal != nil {
			f := func() *QPaintEngine {
				(*(*func() *QPaintEngine)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "paintEngine", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "paintEngine", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLPaintDevice) DisconnectPaintEngine() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "paintEngine")
	}
}

func (ptr *QOpenGLPaintDevice) PaintEngine() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QOpenGLPaintDevice_PaintEngine(ptr.Pointer()))
	}
	return nil
}

func (ptr *QOpenGLPaintDevice) PaintEngineDefault() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QOpenGLPaintDevice_PaintEngineDefault(ptr.Pointer()))
	}
	return nil
}

func (ptr *QOpenGLPaintDevice) PaintFlipped() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLPaintDevice_PaintFlipped(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLPaintDevice) SetDevicePixelRatio(devicePixelRatio float64) {
	if ptr.Pointer() != nil {
		C.QOpenGLPaintDevice_SetDevicePixelRatio(ptr.Pointer(), C.double(devicePixelRatio))
	}
}

func (ptr *QOpenGLPaintDevice) SetDotsPerMeterX(dpmx float64) {
	if ptr.Pointer() != nil {
		C.QOpenGLPaintDevice_SetDotsPerMeterX(ptr.Pointer(), C.double(dpmx))
	}
}

func (ptr *QOpenGLPaintDevice) SetDotsPerMeterY(dpmy float64) {
	if ptr.Pointer() != nil {
		C.QOpenGLPaintDevice_SetDotsPerMeterY(ptr.Pointer(), C.double(dpmy))
	}
}

func (ptr *QOpenGLPaintDevice) SetPaintFlipped(flipped bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLPaintDevice_SetPaintFlipped(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(flipped))))
	}
}

func (ptr *QOpenGLPaintDevice) SetSize(size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLPaintDevice_SetSize(ptr.Pointer(), core.PointerFromQSize(size))
	}
}

func (ptr *QOpenGLPaintDevice) Size() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QOpenGLPaintDevice_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

//export callbackQOpenGLPaintDevice_DestroyQOpenGLPaintDevice
func callbackQOpenGLPaintDevice_DestroyQOpenGLPaintDevice(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOpenGLPaintDevice"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLPaintDeviceFromPointer(ptr).DestroyQOpenGLPaintDeviceDefault()
	}
}

func (ptr *QOpenGLPaintDevice) ConnectDestroyQOpenGLPaintDevice(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOpenGLPaintDevice"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLPaintDevice", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLPaintDevice", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLPaintDevice) DisconnectDestroyQOpenGLPaintDevice() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOpenGLPaintDevice")
	}
}

func (ptr *QOpenGLPaintDevice) DestroyQOpenGLPaintDevice() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLPaintDevice_DestroyQOpenGLPaintDevice(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLPaintDevice) DestroyQOpenGLPaintDeviceDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLPaintDevice_DestroyQOpenGLPaintDeviceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLPixelTransferOptions struct {
	ptr unsafe.Pointer
}

type QOpenGLPixelTransferOptions_ITF interface {
	QOpenGLPixelTransferOptions_PTR() *QOpenGLPixelTransferOptions
}

func (ptr *QOpenGLPixelTransferOptions) QOpenGLPixelTransferOptions_PTR() *QOpenGLPixelTransferOptions {
	return ptr
}

func (ptr *QOpenGLPixelTransferOptions) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLPixelTransferOptions) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLPixelTransferOptions(ptr QOpenGLPixelTransferOptions_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLPixelTransferOptions_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLPixelTransferOptionsFromPointer(ptr unsafe.Pointer) (n *QOpenGLPixelTransferOptions) {
	n = new(QOpenGLPixelTransferOptions)
	n.SetPointer(ptr)
	return
}
func NewQOpenGLPixelTransferOptions() *QOpenGLPixelTransferOptions {
	tmpValue := NewQOpenGLPixelTransferOptionsFromPointer(C.QOpenGLPixelTransferOptions_NewQOpenGLPixelTransferOptions())
	qt.SetFinalizer(tmpValue, (*QOpenGLPixelTransferOptions).DestroyQOpenGLPixelTransferOptions)
	return tmpValue
}

func (ptr *QOpenGLPixelTransferOptions) Alignment() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLPixelTransferOptions_Alignment(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLPixelTransferOptions) ImageHeight() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLPixelTransferOptions_ImageHeight(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLPixelTransferOptions) IsLeastSignificantBitFirst() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLPixelTransferOptions_IsLeastSignificantBitFirst(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLPixelTransferOptions) IsSwapBytesEnabled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLPixelTransferOptions_IsSwapBytesEnabled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLPixelTransferOptions) RowLength() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLPixelTransferOptions_RowLength(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLPixelTransferOptions) SetAlignment(alignment int) {
	if ptr.Pointer() != nil {
		C.QOpenGLPixelTransferOptions_SetAlignment(ptr.Pointer(), C.int(int32(alignment)))
	}
}

func (ptr *QOpenGLPixelTransferOptions) SetImageHeight(imageHeight int) {
	if ptr.Pointer() != nil {
		C.QOpenGLPixelTransferOptions_SetImageHeight(ptr.Pointer(), C.int(int32(imageHeight)))
	}
}

func (ptr *QOpenGLPixelTransferOptions) SetLeastSignificantByteFirst(lsbFirst bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLPixelTransferOptions_SetLeastSignificantByteFirst(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(lsbFirst))))
	}
}

func (ptr *QOpenGLPixelTransferOptions) SetRowLength(rowLength int) {
	if ptr.Pointer() != nil {
		C.QOpenGLPixelTransferOptions_SetRowLength(ptr.Pointer(), C.int(int32(rowLength)))
	}
}

func (ptr *QOpenGLPixelTransferOptions) SetSkipImages(skipImages int) {
	if ptr.Pointer() != nil {
		C.QOpenGLPixelTransferOptions_SetSkipImages(ptr.Pointer(), C.int(int32(skipImages)))
	}
}

func (ptr *QOpenGLPixelTransferOptions) SetSkipPixels(skipPixels int) {
	if ptr.Pointer() != nil {
		C.QOpenGLPixelTransferOptions_SetSkipPixels(ptr.Pointer(), C.int(int32(skipPixels)))
	}
}

func (ptr *QOpenGLPixelTransferOptions) SetSkipRows(skipRows int) {
	if ptr.Pointer() != nil {
		C.QOpenGLPixelTransferOptions_SetSkipRows(ptr.Pointer(), C.int(int32(skipRows)))
	}
}

func (ptr *QOpenGLPixelTransferOptions) SetSwapBytesEnabled(swapBytes bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLPixelTransferOptions_SetSwapBytesEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(swapBytes))))
	}
}

func (ptr *QOpenGLPixelTransferOptions) SkipImages() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLPixelTransferOptions_SkipImages(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLPixelTransferOptions) SkipPixels() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLPixelTransferOptions_SkipPixels(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLPixelTransferOptions) SkipRows() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLPixelTransferOptions_SkipRows(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLPixelTransferOptions) DestroyQOpenGLPixelTransferOptions() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLPixelTransferOptions_DestroyQOpenGLPixelTransferOptions(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLShader struct {
	core.QObject
}

type QOpenGLShader_ITF interface {
	core.QObject_ITF
	QOpenGLShader_PTR() *QOpenGLShader
}

func (ptr *QOpenGLShader) QOpenGLShader_PTR() *QOpenGLShader {
	return ptr
}

func (ptr *QOpenGLShader) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLShader) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLShader(ptr QOpenGLShader_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLShader_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLShaderFromPointer(ptr unsafe.Pointer) (n *QOpenGLShader) {
	n = new(QOpenGLShader)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QOpenGLShader__ShaderTypeBit
//QOpenGLShader::ShaderTypeBit
type QOpenGLShader__ShaderTypeBit int64

const (
	QOpenGLShader__Vertex                 QOpenGLShader__ShaderTypeBit = QOpenGLShader__ShaderTypeBit(0x0001)
	QOpenGLShader__Fragment               QOpenGLShader__ShaderTypeBit = QOpenGLShader__ShaderTypeBit(0x0002)
	QOpenGLShader__Geometry               QOpenGLShader__ShaderTypeBit = QOpenGLShader__ShaderTypeBit(0x0004)
	QOpenGLShader__TessellationControl    QOpenGLShader__ShaderTypeBit = QOpenGLShader__ShaderTypeBit(0x0008)
	QOpenGLShader__TessellationEvaluation QOpenGLShader__ShaderTypeBit = QOpenGLShader__ShaderTypeBit(0x0010)
	QOpenGLShader__Compute                QOpenGLShader__ShaderTypeBit = QOpenGLShader__ShaderTypeBit(0x0020)
)

func NewQOpenGLShader(ty QOpenGLShader__ShaderTypeBit, parent core.QObject_ITF) *QOpenGLShader {
	tmpValue := NewQOpenGLShaderFromPointer(C.QOpenGLShader_NewQOpenGLShader(C.longlong(ty), core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLShader) CompileSourceCode(source string) bool {
	if ptr.Pointer() != nil {
		var sourceC *C.char
		if source != "" {
			sourceC = C.CString(source)
			defer C.free(unsafe.Pointer(sourceC))
		}
		return int8(C.QOpenGLShader_CompileSourceCode(ptr.Pointer(), sourceC)) != 0
	}
	return false
}

func (ptr *QOpenGLShader) CompileSourceCode2(source core.QByteArray_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShader_CompileSourceCode2(ptr.Pointer(), core.PointerFromQByteArray(source))) != 0
	}
	return false
}

func (ptr *QOpenGLShader) CompileSourceCode3(source string) bool {
	if ptr.Pointer() != nil {
		var sourceC *C.char
		if source != "" {
			sourceC = C.CString(source)
			defer C.free(unsafe.Pointer(sourceC))
		}
		return int8(C.QOpenGLShader_CompileSourceCode3(ptr.Pointer(), C.struct_QtGui_PackedString{data: sourceC, len: C.longlong(len(source))})) != 0
	}
	return false
}

func (ptr *QOpenGLShader) CompileSourceFile(fileName string) bool {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		return int8(C.QOpenGLShader_CompileSourceFile(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))})) != 0
	}
	return false
}

func QOpenGLShader_HasOpenGLShaders(ty QOpenGLShader__ShaderTypeBit, context QOpenGLContext_ITF) bool {
	return int8(C.QOpenGLShader_QOpenGLShader_HasOpenGLShaders(C.longlong(ty), PointerFromQOpenGLContext(context))) != 0
}

func (ptr *QOpenGLShader) HasOpenGLShaders(ty QOpenGLShader__ShaderTypeBit, context QOpenGLContext_ITF) bool {
	return int8(C.QOpenGLShader_QOpenGLShader_HasOpenGLShaders(C.longlong(ty), PointerFromQOpenGLContext(context))) != 0
}

func (ptr *QOpenGLShader) IsCompiled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShader_IsCompiled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLShader) Log() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QOpenGLShader_Log(ptr.Pointer()))
	}
	return ""
}

func (ptr *QOpenGLShader) ShaderId() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLShader_ShaderId(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLShader) ShaderType() QOpenGLShader__ShaderTypeBit {
	if ptr.Pointer() != nil {
		return QOpenGLShader__ShaderTypeBit(C.QOpenGLShader_ShaderType(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLShader) SourceCode() *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QOpenGLShader_SourceCode(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

//export callbackQOpenGLShader_DestroyQOpenGLShader
func callbackQOpenGLShader_DestroyQOpenGLShader(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOpenGLShader"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLShaderFromPointer(ptr).DestroyQOpenGLShaderDefault()
	}
}

func (ptr *QOpenGLShader) ConnectDestroyQOpenGLShader(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOpenGLShader"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLShader", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLShader", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLShader) DisconnectDestroyQOpenGLShader() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOpenGLShader")
	}
}

func (ptr *QOpenGLShader) DestroyQOpenGLShader() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLShader_DestroyQOpenGLShader(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLShader) DestroyQOpenGLShaderDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLShader_DestroyQOpenGLShaderDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLShader) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLShader___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLShader) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShader___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLShader) __children_newList() unsafe.Pointer {
	return C.QOpenGLShader___children_newList(ptr.Pointer())
}

func (ptr *QOpenGLShader) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QOpenGLShader___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLShader) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShader___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QOpenGLShader) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QOpenGLShader___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QOpenGLShader) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLShader___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLShader) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShader___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLShader) __findChildren_newList() unsafe.Pointer {
	return C.QOpenGLShader___findChildren_newList(ptr.Pointer())
}

func (ptr *QOpenGLShader) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLShader___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLShader) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShader___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLShader) __findChildren_newList3() unsafe.Pointer {
	return C.QOpenGLShader___findChildren_newList3(ptr.Pointer())
}

//export callbackQOpenGLShader_ChildEvent
func callbackQOpenGLShader_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQOpenGLShaderFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QOpenGLShader) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShader_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQOpenGLShader_ConnectNotify
func callbackQOpenGLShader_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLShaderFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLShader) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShader_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLShader_CustomEvent
func callbackQOpenGLShader_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQOpenGLShaderFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QOpenGLShader) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShader_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQOpenGLShader_DeleteLater
func callbackQOpenGLShader_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLShaderFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QOpenGLShader) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLShader_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLShader_Destroyed
func callbackQOpenGLShader_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQOpenGLShader_DisconnectNotify
func callbackQOpenGLShader_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLShaderFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLShader) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShader_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLShader_Event
func callbackQOpenGLShader_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLShaderFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QOpenGLShader) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShader_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQOpenGLShader_EventFilter
func callbackQOpenGLShader_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLShaderFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QOpenGLShader) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShader_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQOpenGLShader_MetaObject
func callbackQOpenGLShader_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQOpenGLShaderFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QOpenGLShader) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QOpenGLShader_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQOpenGLShader_ObjectNameChanged
func callbackQOpenGLShader_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQOpenGLShader_TimerEvent
func callbackQOpenGLShader_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQOpenGLShaderFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QOpenGLShader) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShader_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QOpenGLShaderProgram struct {
	core.QObject
}

type QOpenGLShaderProgram_ITF interface {
	core.QObject_ITF
	QOpenGLShaderProgram_PTR() *QOpenGLShaderProgram
}

func (ptr *QOpenGLShaderProgram) QOpenGLShaderProgram_PTR() *QOpenGLShaderProgram {
	return ptr
}

func (ptr *QOpenGLShaderProgram) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLShaderProgram) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLShaderProgram(ptr QOpenGLShaderProgram_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLShaderProgram_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLShaderProgramFromPointer(ptr unsafe.Pointer) (n *QOpenGLShaderProgram) {
	n = new(QOpenGLShaderProgram)
	n.SetPointer(ptr)
	return
}
func NewQOpenGLShaderProgram(parent core.QObject_ITF) *QOpenGLShaderProgram {
	tmpValue := NewQOpenGLShaderProgramFromPointer(C.QOpenGLShaderProgram_NewQOpenGLShaderProgram(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLShaderProgram) AddCacheableShaderFromSourceCode(ty QOpenGLShader__ShaderTypeBit, source string) bool {
	if ptr.Pointer() != nil {
		var sourceC *C.char
		if source != "" {
			sourceC = C.CString(source)
			defer C.free(unsafe.Pointer(sourceC))
		}
		return int8(C.QOpenGLShaderProgram_AddCacheableShaderFromSourceCode(ptr.Pointer(), C.longlong(ty), sourceC)) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) AddCacheableShaderFromSourceCode2(ty QOpenGLShader__ShaderTypeBit, source core.QByteArray_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShaderProgram_AddCacheableShaderFromSourceCode2(ptr.Pointer(), C.longlong(ty), core.PointerFromQByteArray(source))) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) AddCacheableShaderFromSourceCode3(ty QOpenGLShader__ShaderTypeBit, source string) bool {
	if ptr.Pointer() != nil {
		var sourceC *C.char
		if source != "" {
			sourceC = C.CString(source)
			defer C.free(unsafe.Pointer(sourceC))
		}
		return int8(C.QOpenGLShaderProgram_AddCacheableShaderFromSourceCode3(ptr.Pointer(), C.longlong(ty), C.struct_QtGui_PackedString{data: sourceC, len: C.longlong(len(source))})) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) AddCacheableShaderFromSourceFile(ty QOpenGLShader__ShaderTypeBit, fileName string) bool {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		return int8(C.QOpenGLShaderProgram_AddCacheableShaderFromSourceFile(ptr.Pointer(), C.longlong(ty), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))})) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) AddShader(shader QOpenGLShader_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShaderProgram_AddShader(ptr.Pointer(), PointerFromQOpenGLShader(shader))) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) AddShaderFromSourceCode(ty QOpenGLShader__ShaderTypeBit, source string) bool {
	if ptr.Pointer() != nil {
		var sourceC *C.char
		if source != "" {
			sourceC = C.CString(source)
			defer C.free(unsafe.Pointer(sourceC))
		}
		return int8(C.QOpenGLShaderProgram_AddShaderFromSourceCode(ptr.Pointer(), C.longlong(ty), sourceC)) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) AddShaderFromSourceCode2(ty QOpenGLShader__ShaderTypeBit, source core.QByteArray_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShaderProgram_AddShaderFromSourceCode2(ptr.Pointer(), C.longlong(ty), core.PointerFromQByteArray(source))) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) AddShaderFromSourceCode3(ty QOpenGLShader__ShaderTypeBit, source string) bool {
	if ptr.Pointer() != nil {
		var sourceC *C.char
		if source != "" {
			sourceC = C.CString(source)
			defer C.free(unsafe.Pointer(sourceC))
		}
		return int8(C.QOpenGLShaderProgram_AddShaderFromSourceCode3(ptr.Pointer(), C.longlong(ty), C.struct_QtGui_PackedString{data: sourceC, len: C.longlong(len(source))})) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) AddShaderFromSourceFile(ty QOpenGLShader__ShaderTypeBit, fileName string) bool {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		return int8(C.QOpenGLShaderProgram_AddShaderFromSourceFile(ptr.Pointer(), C.longlong(ty), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))})) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) AttributeLocation(name string) int {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		return int(int32(C.QOpenGLShaderProgram_AttributeLocation(ptr.Pointer(), nameC)))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) AttributeLocation2(name core.QByteArray_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLShaderProgram_AttributeLocation2(ptr.Pointer(), core.PointerFromQByteArray(name))))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) AttributeLocation3(name string) int {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		return int(int32(C.QOpenGLShaderProgram_AttributeLocation3(ptr.Pointer(), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) Bind() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShaderProgram_Bind(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) BindAttributeLocation(name string, location int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_BindAttributeLocation(ptr.Pointer(), nameC, C.int(int32(location)))
	}
}

func (ptr *QOpenGLShaderProgram) BindAttributeLocation2(name core.QByteArray_ITF, location int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_BindAttributeLocation2(ptr.Pointer(), core.PointerFromQByteArray(name), C.int(int32(location)))
	}
}

func (ptr *QOpenGLShaderProgram) BindAttributeLocation3(name string, location int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_BindAttributeLocation3(ptr.Pointer(), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}, C.int(int32(location)))
	}
}

func (ptr *QOpenGLShaderProgram) Create() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShaderProgram_Create(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) DefaultInnerTessellationLevels() []float32 {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []float32 {
			out := make([]float32, int(l.len))
			tmpList := NewQOpenGLShaderProgramFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__defaultInnerTessellationLevels_atList(i)
			}
			return out
		}(C.QOpenGLShaderProgram_DefaultInnerTessellationLevels(ptr.Pointer()))
	}
	return make([]float32, 0)
}

func (ptr *QOpenGLShaderProgram) DefaultOuterTessellationLevels() []float32 {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []float32 {
			out := make([]float32, int(l.len))
			tmpList := NewQOpenGLShaderProgramFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__defaultOuterTessellationLevels_atList(i)
			}
			return out
		}(C.QOpenGLShaderProgram_DefaultOuterTessellationLevels(ptr.Pointer()))
	}
	return make([]float32, 0)
}

func (ptr *QOpenGLShaderProgram) DisableAttributeArray(location int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_DisableAttributeArray(ptr.Pointer(), C.int(int32(location)))
	}
}

func (ptr *QOpenGLShaderProgram) DisableAttributeArray2(name string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_DisableAttributeArray2(ptr.Pointer(), nameC)
	}
}

func (ptr *QOpenGLShaderProgram) EnableAttributeArray(location int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_EnableAttributeArray(ptr.Pointer(), C.int(int32(location)))
	}
}

func (ptr *QOpenGLShaderProgram) EnableAttributeArray2(name string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_EnableAttributeArray2(ptr.Pointer(), nameC)
	}
}

func QOpenGLShaderProgram_HasOpenGLShaderPrograms(context QOpenGLContext_ITF) bool {
	return int8(C.QOpenGLShaderProgram_QOpenGLShaderProgram_HasOpenGLShaderPrograms(PointerFromQOpenGLContext(context))) != 0
}

func (ptr *QOpenGLShaderProgram) HasOpenGLShaderPrograms(context QOpenGLContext_ITF) bool {
	return int8(C.QOpenGLShaderProgram_QOpenGLShaderProgram_HasOpenGLShaderPrograms(PointerFromQOpenGLContext(context))) != 0
}

func (ptr *QOpenGLShaderProgram) IsLinked() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShaderProgram_IsLinked(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQOpenGLShaderProgram_Link
func callbackQOpenGLShaderProgram_Link(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "link"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLShaderProgramFromPointer(ptr).LinkDefault())))
}

func (ptr *QOpenGLShaderProgram) ConnectLink(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "link"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "link", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "link", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLShaderProgram) DisconnectLink() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "link")
	}
}

func (ptr *QOpenGLShaderProgram) Link() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShaderProgram_Link(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) LinkDefault() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShaderProgram_LinkDefault(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLShaderProgram) Log() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QOpenGLShaderProgram_Log(ptr.Pointer()))
	}
	return ""
}

func (ptr *QOpenGLShaderProgram) MaxGeometryOutputVertices() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLShaderProgram_MaxGeometryOutputVertices(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) PatchVertexCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLShaderProgram_PatchVertexCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) ProgramId() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLShaderProgram_ProgramId(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) Release() {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_Release(ptr.Pointer())
	}
}

func (ptr *QOpenGLShaderProgram) RemoveAllShaders() {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_RemoveAllShaders(ptr.Pointer())
	}
}

func (ptr *QOpenGLShaderProgram) RemoveShader(shader QOpenGLShader_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_RemoveShader(ptr.Pointer(), PointerFromQOpenGLShader(shader))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeArray(location int, values float32, tupleSize int, stride int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeArray(ptr.Pointer(), C.int(int32(location)), C.float(values), C.int(int32(tupleSize)), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeArray2(location int, values QVector2D_ITF, stride int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeArray2(ptr.Pointer(), C.int(int32(location)), PointerFromQVector2D(values), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeArray3(location int, values QVector3D_ITF, stride int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeArray3(ptr.Pointer(), C.int(int32(location)), PointerFromQVector3D(values), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeArray4(location int, values QVector4D_ITF, stride int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeArray4(ptr.Pointer(), C.int(int32(location)), PointerFromQVector4D(values), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeArray5(location int, ty uint, values unsafe.Pointer, tupleSize int, stride int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeArray5(ptr.Pointer(), C.int(int32(location)), C.uint(uint32(ty)), values, C.int(int32(tupleSize)), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeArray6(name string, values float32, tupleSize int, stride int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeArray6(ptr.Pointer(), nameC, C.float(values), C.int(int32(tupleSize)), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeArray7(name string, values QVector2D_ITF, stride int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeArray7(ptr.Pointer(), nameC, PointerFromQVector2D(values), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeArray8(name string, values QVector3D_ITF, stride int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeArray8(ptr.Pointer(), nameC, PointerFromQVector3D(values), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeArray9(name string, values QVector4D_ITF, stride int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeArray9(ptr.Pointer(), nameC, PointerFromQVector4D(values), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeArray10(name string, ty uint, values unsafe.Pointer, tupleSize int, stride int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeArray10(ptr.Pointer(), nameC, C.uint(uint32(ty)), values, C.int(int32(tupleSize)), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeBuffer(location int, ty uint, offset int, tupleSize int, stride int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeBuffer(ptr.Pointer(), C.int(int32(location)), C.uint(uint32(ty)), C.int(int32(offset)), C.int(int32(tupleSize)), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeBuffer2(name string, ty uint, offset int, tupleSize int, stride int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeBuffer2(ptr.Pointer(), nameC, C.uint(uint32(ty)), C.int(int32(offset)), C.int(int32(tupleSize)), C.int(int32(stride)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue(location int, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeValue(ptr.Pointer(), C.int(int32(location)), C.float(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue2(location int, x float32, y float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeValue2(ptr.Pointer(), C.int(int32(location)), C.float(x), C.float(y))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue3(location int, x float32, y float32, z float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeValue3(ptr.Pointer(), C.int(int32(location)), C.float(x), C.float(y), C.float(z))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue4(location int, x float32, y float32, z float32, w float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeValue4(ptr.Pointer(), C.int(int32(location)), C.float(x), C.float(y), C.float(z), C.float(w))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue5(location int, value QVector2D_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeValue5(ptr.Pointer(), C.int(int32(location)), PointerFromQVector2D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue6(location int, value QVector3D_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeValue6(ptr.Pointer(), C.int(int32(location)), PointerFromQVector3D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue7(location int, value QVector4D_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeValue7(ptr.Pointer(), C.int(int32(location)), PointerFromQVector4D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue8(location int, value QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeValue8(ptr.Pointer(), C.int(int32(location)), PointerFromQColor(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue9(location int, values float32, columns int, rows int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetAttributeValue9(ptr.Pointer(), C.int(int32(location)), C.float(values), C.int(int32(columns)), C.int(int32(rows)))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue10(name string, value float32) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeValue10(ptr.Pointer(), nameC, C.float(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue11(name string, x float32, y float32) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeValue11(ptr.Pointer(), nameC, C.float(x), C.float(y))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue12(name string, x float32, y float32, z float32) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeValue12(ptr.Pointer(), nameC, C.float(x), C.float(y), C.float(z))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue13(name string, x float32, y float32, z float32, w float32) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeValue13(ptr.Pointer(), nameC, C.float(x), C.float(y), C.float(z), C.float(w))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue14(name string, value QVector2D_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeValue14(ptr.Pointer(), nameC, PointerFromQVector2D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue15(name string, value QVector3D_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeValue15(ptr.Pointer(), nameC, PointerFromQVector3D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue16(name string, value QVector4D_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeValue16(ptr.Pointer(), nameC, PointerFromQVector4D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue17(name string, value QColor_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeValue17(ptr.Pointer(), nameC, PointerFromQColor(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetAttributeValue18(name string, values float32, columns int, rows int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetAttributeValue18(ptr.Pointer(), nameC, C.float(values), C.int(int32(columns)), C.int(int32(rows)))
	}
}

func (ptr *QOpenGLShaderProgram) SetDefaultInnerTessellationLevels(levels []float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetDefaultInnerTessellationLevels(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQOpenGLShaderProgramFromPointer(NewQOpenGLShaderProgramFromPointer(nil).__setDefaultInnerTessellationLevels_levels_newList())
			for _, v := range levels {
				tmpList.__setDefaultInnerTessellationLevels_levels_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QOpenGLShaderProgram) SetDefaultOuterTessellationLevels(levels []float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetDefaultOuterTessellationLevels(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQOpenGLShaderProgramFromPointer(NewQOpenGLShaderProgramFromPointer(nil).__setDefaultOuterTessellationLevels_levels_newList())
			for _, v := range levels {
				tmpList.__setDefaultOuterTessellationLevels_levels_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QOpenGLShaderProgram) SetPatchVertexCount(count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetPatchVertexCount(ptr.Pointer(), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue(location int, value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue(ptr.Pointer(), C.int(int32(location)), C.float(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue2(location int, value int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue2(ptr.Pointer(), C.int(int32(location)), C.int(int32(value)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue3(location int, value uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue3(ptr.Pointer(), C.int(int32(location)), C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue4(location int, x float32, y float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue4(ptr.Pointer(), C.int(int32(location)), C.float(x), C.float(y))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue5(location int, x float32, y float32, z float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue5(ptr.Pointer(), C.int(int32(location)), C.float(x), C.float(y), C.float(z))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue6(location int, x float32, y float32, z float32, w float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue6(ptr.Pointer(), C.int(int32(location)), C.float(x), C.float(y), C.float(z), C.float(w))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue7(location int, value QVector2D_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue7(ptr.Pointer(), C.int(int32(location)), PointerFromQVector2D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue8(location int, value QVector3D_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue8(ptr.Pointer(), C.int(int32(location)), PointerFromQVector3D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue9(location int, value QVector4D_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue9(ptr.Pointer(), C.int(int32(location)), PointerFromQVector4D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue10(location int, color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue10(ptr.Pointer(), C.int(int32(location)), PointerFromQColor(color))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue11(location int, point core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue11(ptr.Pointer(), C.int(int32(location)), core.PointerFromQPoint(point))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue12(location int, point core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue12(ptr.Pointer(), C.int(int32(location)), core.PointerFromQPointF(point))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue13(location int, size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue13(ptr.Pointer(), C.int(int32(location)), core.PointerFromQSize(size))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue14(location int, size core.QSizeF_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue14(ptr.Pointer(), C.int(int32(location)), core.PointerFromQSizeF(size))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue23(location int, value QMatrix4x4_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue23(ptr.Pointer(), C.int(int32(location)), PointerFromQMatrix4x4(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue27(location int, value QTransform_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValue27(ptr.Pointer(), C.int(int32(location)), PointerFromQTransform(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue28(name string, value float32) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue28(ptr.Pointer(), nameC, C.float(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue29(name string, value int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue29(ptr.Pointer(), nameC, C.int(int32(value)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue30(name string, value uint) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue30(ptr.Pointer(), nameC, C.uint(uint32(value)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue31(name string, x float32, y float32) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue31(ptr.Pointer(), nameC, C.float(x), C.float(y))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue32(name string, x float32, y float32, z float32) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue32(ptr.Pointer(), nameC, C.float(x), C.float(y), C.float(z))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue33(name string, x float32, y float32, z float32, w float32) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue33(ptr.Pointer(), nameC, C.float(x), C.float(y), C.float(z), C.float(w))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue34(name string, value QVector2D_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue34(ptr.Pointer(), nameC, PointerFromQVector2D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue35(name string, value QVector3D_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue35(ptr.Pointer(), nameC, PointerFromQVector3D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue36(name string, value QVector4D_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue36(ptr.Pointer(), nameC, PointerFromQVector4D(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue37(name string, color QColor_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue37(ptr.Pointer(), nameC, PointerFromQColor(color))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue38(name string, point core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue38(ptr.Pointer(), nameC, core.PointerFromQPoint(point))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue39(name string, point core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue39(ptr.Pointer(), nameC, core.PointerFromQPointF(point))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue40(name string, size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue40(ptr.Pointer(), nameC, core.PointerFromQSize(size))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue41(name string, size core.QSizeF_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue41(ptr.Pointer(), nameC, core.PointerFromQSizeF(size))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue50(name string, value QMatrix4x4_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue50(ptr.Pointer(), nameC, PointerFromQMatrix4x4(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValue54(name string, value QTransform_ITF) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValue54(ptr.Pointer(), nameC, PointerFromQTransform(value))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray(location int, values float32, count int, tupleSize int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValueArray(ptr.Pointer(), C.int(int32(location)), C.float(values), C.int(int32(count)), C.int(int32(tupleSize)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray2(location int, values int, count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValueArray2(ptr.Pointer(), C.int(int32(location)), C.int(int32(values)), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray3(location int, values uint, count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValueArray3(ptr.Pointer(), C.int(int32(location)), C.uint(uint32(values)), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray4(location int, values QVector2D_ITF, count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValueArray4(ptr.Pointer(), C.int(int32(location)), PointerFromQVector2D(values), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray5(location int, values QVector3D_ITF, count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValueArray5(ptr.Pointer(), C.int(int32(location)), PointerFromQVector3D(values), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray6(location int, values QVector4D_ITF, count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValueArray6(ptr.Pointer(), C.int(int32(location)), PointerFromQVector4D(values), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray15(location int, values QMatrix4x4_ITF, count int) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_SetUniformValueArray15(ptr.Pointer(), C.int(int32(location)), PointerFromQMatrix4x4(values), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray16(name string, values float32, count int, tupleSize int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValueArray16(ptr.Pointer(), nameC, C.float(values), C.int(int32(count)), C.int(int32(tupleSize)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray17(name string, values int, count int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValueArray17(ptr.Pointer(), nameC, C.int(int32(values)), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray18(name string, values uint, count int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValueArray18(ptr.Pointer(), nameC, C.uint(uint32(values)), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray19(name string, values QVector2D_ITF, count int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValueArray19(ptr.Pointer(), nameC, PointerFromQVector2D(values), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray20(name string, values QVector3D_ITF, count int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValueArray20(ptr.Pointer(), nameC, PointerFromQVector3D(values), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray21(name string, values QVector4D_ITF, count int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValueArray21(ptr.Pointer(), nameC, PointerFromQVector4D(values), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) SetUniformValueArray30(name string, values QMatrix4x4_ITF, count int) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QOpenGLShaderProgram_SetUniformValueArray30(ptr.Pointer(), nameC, PointerFromQMatrix4x4(values), C.int(int32(count)))
	}
}

func (ptr *QOpenGLShaderProgram) Shaders() []*QOpenGLShader {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QOpenGLShader {
			out := make([]*QOpenGLShader, int(l.len))
			tmpList := NewQOpenGLShaderProgramFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__shaders_atList(i)
			}
			return out
		}(C.QOpenGLShaderProgram_Shaders(ptr.Pointer()))
	}
	return make([]*QOpenGLShader, 0)
}

func (ptr *QOpenGLShaderProgram) UniformLocation(name string) int {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		return int(int32(C.QOpenGLShaderProgram_UniformLocation(ptr.Pointer(), nameC)))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) UniformLocation2(name core.QByteArray_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLShaderProgram_UniformLocation2(ptr.Pointer(), core.PointerFromQByteArray(name))))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) UniformLocation3(name string) int {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		return int(int32(C.QOpenGLShaderProgram_UniformLocation3(ptr.Pointer(), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})))
	}
	return 0
}

//export callbackQOpenGLShaderProgram_DestroyQOpenGLShaderProgram
func callbackQOpenGLShaderProgram_DestroyQOpenGLShaderProgram(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOpenGLShaderProgram"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLShaderProgramFromPointer(ptr).DestroyQOpenGLShaderProgramDefault()
	}
}

func (ptr *QOpenGLShaderProgram) ConnectDestroyQOpenGLShaderProgram(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOpenGLShaderProgram"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLShaderProgram", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLShaderProgram", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLShaderProgram) DisconnectDestroyQOpenGLShaderProgram() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOpenGLShaderProgram")
	}
}

func (ptr *QOpenGLShaderProgram) DestroyQOpenGLShaderProgram() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLShaderProgram_DestroyQOpenGLShaderProgram(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLShaderProgram) DestroyQOpenGLShaderProgramDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLShaderProgram_DestroyQOpenGLShaderProgramDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLShaderProgram) __defaultInnerTessellationLevels_atList(i int) float32 {
	if ptr.Pointer() != nil {
		return float32(C.QOpenGLShaderProgram___defaultInnerTessellationLevels_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) __defaultInnerTessellationLevels_setList(i float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram___defaultInnerTessellationLevels_setList(ptr.Pointer(), C.float(i))
	}
}

func (ptr *QOpenGLShaderProgram) __defaultInnerTessellationLevels_newList() unsafe.Pointer {
	return C.QOpenGLShaderProgram___defaultInnerTessellationLevels_newList(ptr.Pointer())
}

func (ptr *QOpenGLShaderProgram) __defaultOuterTessellationLevels_atList(i int) float32 {
	if ptr.Pointer() != nil {
		return float32(C.QOpenGLShaderProgram___defaultOuterTessellationLevels_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) __defaultOuterTessellationLevels_setList(i float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram___defaultOuterTessellationLevels_setList(ptr.Pointer(), C.float(i))
	}
}

func (ptr *QOpenGLShaderProgram) __defaultOuterTessellationLevels_newList() unsafe.Pointer {
	return C.QOpenGLShaderProgram___defaultOuterTessellationLevels_newList(ptr.Pointer())
}

func (ptr *QOpenGLShaderProgram) __setDefaultInnerTessellationLevels_levels_atList(i int) float32 {
	if ptr.Pointer() != nil {
		return float32(C.QOpenGLShaderProgram___setDefaultInnerTessellationLevels_levels_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) __setDefaultInnerTessellationLevels_levels_setList(i float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram___setDefaultInnerTessellationLevels_levels_setList(ptr.Pointer(), C.float(i))
	}
}

func (ptr *QOpenGLShaderProgram) __setDefaultInnerTessellationLevels_levels_newList() unsafe.Pointer {
	return C.QOpenGLShaderProgram___setDefaultInnerTessellationLevels_levels_newList(ptr.Pointer())
}

func (ptr *QOpenGLShaderProgram) __setDefaultOuterTessellationLevels_levels_atList(i int) float32 {
	if ptr.Pointer() != nil {
		return float32(C.QOpenGLShaderProgram___setDefaultOuterTessellationLevels_levels_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QOpenGLShaderProgram) __setDefaultOuterTessellationLevels_levels_setList(i float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram___setDefaultOuterTessellationLevels_levels_setList(ptr.Pointer(), C.float(i))
	}
}

func (ptr *QOpenGLShaderProgram) __setDefaultOuterTessellationLevels_levels_newList() unsafe.Pointer {
	return C.QOpenGLShaderProgram___setDefaultOuterTessellationLevels_levels_newList(ptr.Pointer())
}

func (ptr *QOpenGLShaderProgram) __shaders_atList(i int) *QOpenGLShader {
	if ptr.Pointer() != nil {
		tmpValue := NewQOpenGLShaderFromPointer(C.QOpenGLShaderProgram___shaders_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLShaderProgram) __shaders_setList(i QOpenGLShader_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram___shaders_setList(ptr.Pointer(), PointerFromQOpenGLShader(i))
	}
}

func (ptr *QOpenGLShaderProgram) __shaders_newList() unsafe.Pointer {
	return C.QOpenGLShaderProgram___shaders_newList(ptr.Pointer())
}

func (ptr *QOpenGLShaderProgram) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLShaderProgram___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLShaderProgram) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLShaderProgram) __children_newList() unsafe.Pointer {
	return C.QOpenGLShaderProgram___children_newList(ptr.Pointer())
}

func (ptr *QOpenGLShaderProgram) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QOpenGLShaderProgram___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLShaderProgram) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QOpenGLShaderProgram) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QOpenGLShaderProgram___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QOpenGLShaderProgram) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLShaderProgram___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLShaderProgram) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLShaderProgram) __findChildren_newList() unsafe.Pointer {
	return C.QOpenGLShaderProgram___findChildren_newList(ptr.Pointer())
}

func (ptr *QOpenGLShaderProgram) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLShaderProgram___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLShaderProgram) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLShaderProgram) __findChildren_newList3() unsafe.Pointer {
	return C.QOpenGLShaderProgram___findChildren_newList3(ptr.Pointer())
}

//export callbackQOpenGLShaderProgram_ChildEvent
func callbackQOpenGLShaderProgram_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQOpenGLShaderProgramFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QOpenGLShaderProgram) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQOpenGLShaderProgram_ConnectNotify
func callbackQOpenGLShaderProgram_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLShaderProgramFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLShaderProgram) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLShaderProgram_CustomEvent
func callbackQOpenGLShaderProgram_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQOpenGLShaderProgramFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QOpenGLShaderProgram) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQOpenGLShaderProgram_DeleteLater
func callbackQOpenGLShaderProgram_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLShaderProgramFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QOpenGLShaderProgram) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLShaderProgram_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLShaderProgram_Destroyed
func callbackQOpenGLShaderProgram_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQOpenGLShaderProgram_DisconnectNotify
func callbackQOpenGLShaderProgram_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLShaderProgramFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLShaderProgram) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLShaderProgram_Event
func callbackQOpenGLShaderProgram_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLShaderProgramFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QOpenGLShaderProgram) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShaderProgram_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQOpenGLShaderProgram_EventFilter
func callbackQOpenGLShaderProgram_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLShaderProgramFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QOpenGLShaderProgram) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLShaderProgram_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQOpenGLShaderProgram_MetaObject
func callbackQOpenGLShaderProgram_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQOpenGLShaderProgramFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QOpenGLShaderProgram) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QOpenGLShaderProgram_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQOpenGLShaderProgram_ObjectNameChanged
func callbackQOpenGLShaderProgram_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQOpenGLShaderProgram_TimerEvent
func callbackQOpenGLShaderProgram_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQOpenGLShaderProgramFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QOpenGLShaderProgram) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLShaderProgram_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QOpenGLTexture struct {
	ptr unsafe.Pointer
}

type QOpenGLTexture_ITF interface {
	QOpenGLTexture_PTR() *QOpenGLTexture
}

func (ptr *QOpenGLTexture) QOpenGLTexture_PTR() *QOpenGLTexture {
	return ptr
}

func (ptr *QOpenGLTexture) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLTexture) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLTexture(ptr QOpenGLTexture_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLTexture_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLTextureFromPointer(ptr unsafe.Pointer) (n *QOpenGLTexture) {
	n = new(QOpenGLTexture)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLTexture) DestroyQOpenGLTexture() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QOpenGLTexture__Target
//QOpenGLTexture::Target
type QOpenGLTexture__Target int64

const (
	QOpenGLTexture__Target1D                 QOpenGLTexture__Target = QOpenGLTexture__Target(0x0DE0)
	QOpenGLTexture__Target1DArray            QOpenGLTexture__Target = QOpenGLTexture__Target(0x8C18)
	QOpenGLTexture__Target2D                 QOpenGLTexture__Target = QOpenGLTexture__Target(0x0DE1)
	QOpenGLTexture__Target2DArray            QOpenGLTexture__Target = QOpenGLTexture__Target(0x8C1A)
	QOpenGLTexture__Target3D                 QOpenGLTexture__Target = QOpenGLTexture__Target(0x806F)
	QOpenGLTexture__TargetCubeMap            QOpenGLTexture__Target = QOpenGLTexture__Target(0x8513)
	QOpenGLTexture__TargetCubeMapArray       QOpenGLTexture__Target = QOpenGLTexture__Target(0x9009)
	QOpenGLTexture__Target2DMultisample      QOpenGLTexture__Target = QOpenGLTexture__Target(0x9100)
	QOpenGLTexture__Target2DMultisampleArray QOpenGLTexture__Target = QOpenGLTexture__Target(0x9102)
	QOpenGLTexture__TargetRectangle          QOpenGLTexture__Target = QOpenGLTexture__Target(0x84F5)
	QOpenGLTexture__TargetBuffer             QOpenGLTexture__Target = QOpenGLTexture__Target(0x8C2A)
)

//go:generate stringer -type=QOpenGLTexture__BindingTarget
//QOpenGLTexture::BindingTarget
type QOpenGLTexture__BindingTarget int64

const (
	QOpenGLTexture__BindingTarget1D                 QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x8068)
	QOpenGLTexture__BindingTarget1DArray            QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x8C1C)
	QOpenGLTexture__BindingTarget2D                 QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x8069)
	QOpenGLTexture__BindingTarget2DArray            QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x8C1D)
	QOpenGLTexture__BindingTarget3D                 QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x806A)
	QOpenGLTexture__BindingTargetCubeMap            QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x8514)
	QOpenGLTexture__BindingTargetCubeMapArray       QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x900A)
	QOpenGLTexture__BindingTarget2DMultisample      QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x9104)
	QOpenGLTexture__BindingTarget2DMultisampleArray QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x9105)
	QOpenGLTexture__BindingTargetRectangle          QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x84F6)
	QOpenGLTexture__BindingTargetBuffer             QOpenGLTexture__BindingTarget = QOpenGLTexture__BindingTarget(0x8C2C)
)

//go:generate stringer -type=QOpenGLTexture__MipMapGeneration
//QOpenGLTexture::MipMapGeneration
type QOpenGLTexture__MipMapGeneration int64

const (
	QOpenGLTexture__GenerateMipMaps     QOpenGLTexture__MipMapGeneration = QOpenGLTexture__MipMapGeneration(0)
	QOpenGLTexture__DontGenerateMipMaps QOpenGLTexture__MipMapGeneration = QOpenGLTexture__MipMapGeneration(1)
)

//go:generate stringer -type=QOpenGLTexture__TextureUnitReset
//QOpenGLTexture::TextureUnitReset
type QOpenGLTexture__TextureUnitReset int64

const (
	QOpenGLTexture__ResetTextureUnit     QOpenGLTexture__TextureUnitReset = QOpenGLTexture__TextureUnitReset(0)
	QOpenGLTexture__DontResetTextureUnit QOpenGLTexture__TextureUnitReset = QOpenGLTexture__TextureUnitReset(1)
)

//go:generate stringer -type=QOpenGLTexture__TextureFormat
//QOpenGLTexture::TextureFormat
type QOpenGLTexture__TextureFormat int64

const (
	QOpenGLTexture__NoFormat                       QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0)
	QOpenGLTexture__R8_UNorm                       QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8229)
	QOpenGLTexture__RG8_UNorm                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x822B)
	QOpenGLTexture__RGB8_UNorm                     QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8051)
	QOpenGLTexture__RGBA8_UNorm                    QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8058)
	QOpenGLTexture__R16_UNorm                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x822A)
	QOpenGLTexture__RG16_UNorm                     QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x822C)
	QOpenGLTexture__RGB16_UNorm                    QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8054)
	QOpenGLTexture__RGBA16_UNorm                   QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x805B)
	QOpenGLTexture__R8_SNorm                       QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8F94)
	QOpenGLTexture__RG8_SNorm                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8F95)
	QOpenGLTexture__RGB8_SNorm                     QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8F96)
	QOpenGLTexture__RGBA8_SNorm                    QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8F97)
	QOpenGLTexture__R16_SNorm                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8F98)
	QOpenGLTexture__RG16_SNorm                     QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8F99)
	QOpenGLTexture__RGB16_SNorm                    QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8F9A)
	QOpenGLTexture__RGBA16_SNorm                   QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8F9B)
	QOpenGLTexture__R8U                            QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8232)
	QOpenGLTexture__RG8U                           QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8238)
	QOpenGLTexture__RGB8U                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D7D)
	QOpenGLTexture__RGBA8U                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D7C)
	QOpenGLTexture__R16U                           QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8234)
	QOpenGLTexture__RG16U                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x823A)
	QOpenGLTexture__RGB16U                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D77)
	QOpenGLTexture__RGBA16U                        QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D76)
	QOpenGLTexture__R32U                           QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8236)
	QOpenGLTexture__RG32U                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x823C)
	QOpenGLTexture__RGB32U                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D71)
	QOpenGLTexture__RGBA32U                        QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D70)
	QOpenGLTexture__R8I                            QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8231)
	QOpenGLTexture__RG8I                           QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8237)
	QOpenGLTexture__RGB8I                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D8F)
	QOpenGLTexture__RGBA8I                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D8E)
	QOpenGLTexture__R16I                           QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8233)
	QOpenGLTexture__RG16I                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8239)
	QOpenGLTexture__RGB16I                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D89)
	QOpenGLTexture__RGBA16I                        QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D88)
	QOpenGLTexture__R32I                           QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8235)
	QOpenGLTexture__RG32I                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x823B)
	QOpenGLTexture__RGB32I                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D83)
	QOpenGLTexture__RGBA32I                        QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D82)
	QOpenGLTexture__R16F                           QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x822D)
	QOpenGLTexture__RG16F                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x822F)
	QOpenGLTexture__RGB16F                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x881B)
	QOpenGLTexture__RGBA16F                        QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x881A)
	QOpenGLTexture__R32F                           QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x822E)
	QOpenGLTexture__RG32F                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8230)
	QOpenGLTexture__RGB32F                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8815)
	QOpenGLTexture__RGBA32F                        QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8814)
	QOpenGLTexture__RGB9E5                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8C3D)
	QOpenGLTexture__RG11B10F                       QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8C3A)
	QOpenGLTexture__RG3B2                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x2A10)
	QOpenGLTexture__R5G6B5                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D62)
	QOpenGLTexture__RGB5A1                         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8057)
	QOpenGLTexture__RGBA4                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8056)
	QOpenGLTexture__RGB10A2                        QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x906F)
	QOpenGLTexture__D16                            QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x81A5)
	QOpenGLTexture__D24                            QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x81A6)
	QOpenGLTexture__D24S8                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x88F0)
	QOpenGLTexture__D32                            QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x81A7)
	QOpenGLTexture__D32F                           QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8CAC)
	QOpenGLTexture__D32FS8X24                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8CAD)
	QOpenGLTexture__S8                             QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D48)
	QOpenGLTexture__RGB_DXT1                       QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x83F0)
	QOpenGLTexture__RGBA_DXT1                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x83F1)
	QOpenGLTexture__RGBA_DXT3                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x83F2)
	QOpenGLTexture__RGBA_DXT5                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x83F3)
	QOpenGLTexture__R_ATI1N_UNorm                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8DBB)
	QOpenGLTexture__R_ATI1N_SNorm                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8DBC)
	QOpenGLTexture__RG_ATI2N_UNorm                 QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8DBD)
	QOpenGLTexture__RG_ATI2N_SNorm                 QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8DBE)
	QOpenGLTexture__RGB_BP_UNSIGNED_FLOAT          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8E8F)
	QOpenGLTexture__RGB_BP_SIGNED_FLOAT            QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8E8E)
	QOpenGLTexture__RGB_BP_UNorm                   QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8E8C)
	QOpenGLTexture__R11_EAC_UNorm                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x9270)
	QOpenGLTexture__R11_EAC_SNorm                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x9271)
	QOpenGLTexture__RG11_EAC_UNorm                 QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x9272)
	QOpenGLTexture__RG11_EAC_SNorm                 QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x9273)
	QOpenGLTexture__RGB8_ETC2                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x9274)
	QOpenGLTexture__SRGB8_ETC2                     QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x9275)
	QOpenGLTexture__RGB8_PunchThrough_Alpha1_ETC2  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x9276)
	QOpenGLTexture__SRGB8_PunchThrough_Alpha1_ETC2 QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x9277)
	QOpenGLTexture__RGBA8_ETC2_EAC                 QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x9278)
	QOpenGLTexture__SRGB8_Alpha8_ETC2_EAC          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x9279)
	QOpenGLTexture__RGB8_ETC1                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8D64)
	QOpenGLTexture__RGBA_ASTC_4x4                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93B0)
	QOpenGLTexture__RGBA_ASTC_5x4                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93B1)
	QOpenGLTexture__RGBA_ASTC_5x5                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93B2)
	QOpenGLTexture__RGBA_ASTC_6x5                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93B3)
	QOpenGLTexture__RGBA_ASTC_6x6                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93B4)
	QOpenGLTexture__RGBA_ASTC_8x5                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93B5)
	QOpenGLTexture__RGBA_ASTC_8x6                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93B6)
	QOpenGLTexture__RGBA_ASTC_8x8                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93B7)
	QOpenGLTexture__RGBA_ASTC_10x5                 QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93B8)
	QOpenGLTexture__RGBA_ASTC_10x6                 QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93B9)
	QOpenGLTexture__RGBA_ASTC_10x8                 QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93BA)
	QOpenGLTexture__RGBA_ASTC_10x10                QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93BB)
	QOpenGLTexture__RGBA_ASTC_12x10                QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93BC)
	QOpenGLTexture__RGBA_ASTC_12x12                QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93BD)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_4x4          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93D0)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_5x4          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93D1)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_5x5          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93D2)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_6x5          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93D3)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_6x6          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93D4)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_8x5          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93D5)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_8x6          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93D6)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_8x8          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93D7)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_10x5         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93D8)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_10x6         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93D9)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_10x8         QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93DA)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_10x10        QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93DB)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_12x10        QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93DC)
	QOpenGLTexture__SRGB8_Alpha8_ASTC_12x12        QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x93DD)
	QOpenGLTexture__SRGB8                          QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8C41)
	QOpenGLTexture__SRGB8_Alpha8                   QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8C43)
	QOpenGLTexture__SRGB_DXT1                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8C4C)
	QOpenGLTexture__SRGB_Alpha_DXT1                QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8C4D)
	QOpenGLTexture__SRGB_Alpha_DXT3                QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8C4E)
	QOpenGLTexture__SRGB_Alpha_DXT5                QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8C4F)
	QOpenGLTexture__SRGB_BP_UNorm                  QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x8E8D)
	QOpenGLTexture__DepthFormat                    QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x1902)
	QOpenGLTexture__AlphaFormat                    QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x1906)
	QOpenGLTexture__RGBFormat                      QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x1907)
	QOpenGLTexture__RGBAFormat                     QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x1908)
	QOpenGLTexture__LuminanceFormat                QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x1909)
	QOpenGLTexture__LuminanceAlphaFormat           QOpenGLTexture__TextureFormat = QOpenGLTexture__TextureFormat(0x190A)
)

//go:generate stringer -type=QOpenGLTexture__CubeMapFace
//QOpenGLTexture::CubeMapFace
type QOpenGLTexture__CubeMapFace int64

const (
	QOpenGLTexture__CubeMapPositiveX QOpenGLTexture__CubeMapFace = QOpenGLTexture__CubeMapFace(0x8515)
	QOpenGLTexture__CubeMapNegativeX QOpenGLTexture__CubeMapFace = QOpenGLTexture__CubeMapFace(0x8516)
	QOpenGLTexture__CubeMapPositiveY QOpenGLTexture__CubeMapFace = QOpenGLTexture__CubeMapFace(0x8517)
	QOpenGLTexture__CubeMapNegativeY QOpenGLTexture__CubeMapFace = QOpenGLTexture__CubeMapFace(0x8518)
	QOpenGLTexture__CubeMapPositiveZ QOpenGLTexture__CubeMapFace = QOpenGLTexture__CubeMapFace(0x8519)
	QOpenGLTexture__CubeMapNegativeZ QOpenGLTexture__CubeMapFace = QOpenGLTexture__CubeMapFace(0x851A)
)

//go:generate stringer -type=QOpenGLTexture__PixelFormat
//QOpenGLTexture::PixelFormat
type QOpenGLTexture__PixelFormat int64

const (
	QOpenGLTexture__NoSourceFormat QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0)
	QOpenGLTexture__Red            QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x1903)
	QOpenGLTexture__RG             QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x8227)
	QOpenGLTexture__RGB            QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x1907)
	QOpenGLTexture__BGR            QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x80E0)
	QOpenGLTexture__RGBA           QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x1908)
	QOpenGLTexture__BGRA           QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x80E1)
	QOpenGLTexture__Red_Integer    QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x8D94)
	QOpenGLTexture__RG_Integer     QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x8228)
	QOpenGLTexture__RGB_Integer    QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x8D98)
	QOpenGLTexture__BGR_Integer    QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x8D9A)
	QOpenGLTexture__RGBA_Integer   QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x8D99)
	QOpenGLTexture__BGRA_Integer   QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x8D9B)
	QOpenGLTexture__Stencil        QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x1901)
	QOpenGLTexture__Depth          QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x1902)
	QOpenGLTexture__DepthStencil   QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x84F9)
	QOpenGLTexture__Alpha          QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x1906)
	QOpenGLTexture__Luminance      QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x1909)
	QOpenGLTexture__LuminanceAlpha QOpenGLTexture__PixelFormat = QOpenGLTexture__PixelFormat(0x190A)
)

//go:generate stringer -type=QOpenGLTexture__PixelType
//QOpenGLTexture::PixelType
type QOpenGLTexture__PixelType int64

const (
	QOpenGLTexture__NoPixelType               QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0)
	QOpenGLTexture__Int8                      QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x1400)
	QOpenGLTexture__UInt8                     QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x1401)
	QOpenGLTexture__Int16                     QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x1402)
	QOpenGLTexture__UInt16                    QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x1403)
	QOpenGLTexture__Int32                     QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x1404)
	QOpenGLTexture__UInt32                    QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x1405)
	QOpenGLTexture__Float16                   QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x140B)
	QOpenGLTexture__Float16OES                QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8D61)
	QOpenGLTexture__Float32                   QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x1406)
	QOpenGLTexture__UInt32_RGB9_E5            QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8C3E)
	QOpenGLTexture__UInt32_RG11B10F           QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8C3B)
	QOpenGLTexture__UInt8_RG3B2               QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8032)
	QOpenGLTexture__UInt8_RG3B2_Rev           QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8362)
	QOpenGLTexture__UInt16_RGB5A1             QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8034)
	QOpenGLTexture__UInt16_RGB5A1_Rev         QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8366)
	QOpenGLTexture__UInt16_R5G6B5             QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8363)
	QOpenGLTexture__UInt16_R5G6B5_Rev         QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8364)
	QOpenGLTexture__UInt16_RGBA4              QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8033)
	QOpenGLTexture__UInt16_RGBA4_Rev          QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8365)
	QOpenGLTexture__UInt32_RGBA8              QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8035)
	QOpenGLTexture__UInt32_RGBA8_Rev          QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8367)
	QOpenGLTexture__UInt32_RGB10A2            QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8036)
	QOpenGLTexture__UInt32_RGB10A2_Rev        QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8368)
	QOpenGLTexture__UInt32_D24S8              QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x84FA)
	QOpenGLTexture__Float32_D32_UInt32_S8_X24 QOpenGLTexture__PixelType = QOpenGLTexture__PixelType(0x8DAD)
)

//go:generate stringer -type=QOpenGLTexture__SwizzleComponent
//QOpenGLTexture::SwizzleComponent
type QOpenGLTexture__SwizzleComponent int64

const (
	QOpenGLTexture__SwizzleRed   QOpenGLTexture__SwizzleComponent = QOpenGLTexture__SwizzleComponent(0x8E42)
	QOpenGLTexture__SwizzleGreen QOpenGLTexture__SwizzleComponent = QOpenGLTexture__SwizzleComponent(0x8E43)
	QOpenGLTexture__SwizzleBlue  QOpenGLTexture__SwizzleComponent = QOpenGLTexture__SwizzleComponent(0x8E44)
	QOpenGLTexture__SwizzleAlpha QOpenGLTexture__SwizzleComponent = QOpenGLTexture__SwizzleComponent(0x8E45)
)

//go:generate stringer -type=QOpenGLTexture__SwizzleValue
//QOpenGLTexture::SwizzleValue
type QOpenGLTexture__SwizzleValue int64

const (
	QOpenGLTexture__RedValue   QOpenGLTexture__SwizzleValue = QOpenGLTexture__SwizzleValue(0x1903)
	QOpenGLTexture__GreenValue QOpenGLTexture__SwizzleValue = QOpenGLTexture__SwizzleValue(0x1904)
	QOpenGLTexture__BlueValue  QOpenGLTexture__SwizzleValue = QOpenGLTexture__SwizzleValue(0x1905)
	QOpenGLTexture__AlphaValue QOpenGLTexture__SwizzleValue = QOpenGLTexture__SwizzleValue(0x1906)
	QOpenGLTexture__ZeroValue  QOpenGLTexture__SwizzleValue = QOpenGLTexture__SwizzleValue(0)
	QOpenGLTexture__OneValue   QOpenGLTexture__SwizzleValue = QOpenGLTexture__SwizzleValue(1)
)

//go:generate stringer -type=QOpenGLTexture__WrapMode
//QOpenGLTexture::WrapMode
type QOpenGLTexture__WrapMode int64

const (
	QOpenGLTexture__Repeat         QOpenGLTexture__WrapMode = QOpenGLTexture__WrapMode(0x2901)
	QOpenGLTexture__MirroredRepeat QOpenGLTexture__WrapMode = QOpenGLTexture__WrapMode(0x8370)
	QOpenGLTexture__ClampToEdge    QOpenGLTexture__WrapMode = QOpenGLTexture__WrapMode(0x812F)
	QOpenGLTexture__ClampToBorder  QOpenGLTexture__WrapMode = QOpenGLTexture__WrapMode(0x812D)
)

//go:generate stringer -type=QOpenGLTexture__CoordinateDirection
//QOpenGLTexture::CoordinateDirection
type QOpenGLTexture__CoordinateDirection int64

const (
	QOpenGLTexture__DirectionS QOpenGLTexture__CoordinateDirection = QOpenGLTexture__CoordinateDirection(0x2802)
	QOpenGLTexture__DirectionT QOpenGLTexture__CoordinateDirection = QOpenGLTexture__CoordinateDirection(0x2803)
	QOpenGLTexture__DirectionR QOpenGLTexture__CoordinateDirection = QOpenGLTexture__CoordinateDirection(0x8072)
)

//go:generate stringer -type=QOpenGLTexture__Feature
//QOpenGLTexture::Feature
type QOpenGLTexture__Feature int64

const (
	QOpenGLTexture__ImmutableStorage            QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000001)
	QOpenGLTexture__ImmutableMultisampleStorage QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000002)
	QOpenGLTexture__TextureRectangle            QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000004)
	QOpenGLTexture__TextureArrays               QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000008)
	QOpenGLTexture__Texture3D                   QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000010)
	QOpenGLTexture__TextureMultisample          QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000020)
	QOpenGLTexture__TextureBuffer               QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000040)
	QOpenGLTexture__TextureCubeMapArrays        QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000080)
	QOpenGLTexture__Swizzle                     QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000100)
	QOpenGLTexture__StencilTexturing            QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000200)
	QOpenGLTexture__AnisotropicFiltering        QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000400)
	QOpenGLTexture__NPOTTextures                QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00000800)
	QOpenGLTexture__NPOTTextureRepeat           QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00001000)
	QOpenGLTexture__Texture1D                   QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00002000)
	QOpenGLTexture__TextureComparisonOperators  QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00004000)
	QOpenGLTexture__TextureMipMapLevel          QOpenGLTexture__Feature = QOpenGLTexture__Feature(0x00008000)
)

//go:generate stringer -type=QOpenGLTexture__DepthStencilMode
//QOpenGLTexture::DepthStencilMode
type QOpenGLTexture__DepthStencilMode int64

const (
	QOpenGLTexture__DepthMode   QOpenGLTexture__DepthStencilMode = QOpenGLTexture__DepthStencilMode(0x1902)
	QOpenGLTexture__StencilMode QOpenGLTexture__DepthStencilMode = QOpenGLTexture__DepthStencilMode(0x1901)
)

//go:generate stringer -type=QOpenGLTexture__ComparisonFunction
//QOpenGLTexture::ComparisonFunction
type QOpenGLTexture__ComparisonFunction int64

const (
	QOpenGLTexture__CompareLessEqual    QOpenGLTexture__ComparisonFunction = QOpenGLTexture__ComparisonFunction(0x0203)
	QOpenGLTexture__CompareGreaterEqual QOpenGLTexture__ComparisonFunction = QOpenGLTexture__ComparisonFunction(0x0206)
	QOpenGLTexture__CompareLess         QOpenGLTexture__ComparisonFunction = QOpenGLTexture__ComparisonFunction(0x0201)
	QOpenGLTexture__CompareGreater      QOpenGLTexture__ComparisonFunction = QOpenGLTexture__ComparisonFunction(0x0204)
	QOpenGLTexture__CompareEqual        QOpenGLTexture__ComparisonFunction = QOpenGLTexture__ComparisonFunction(0x0202)
	QOpenGLTexture__CommpareNotEqual    QOpenGLTexture__ComparisonFunction = QOpenGLTexture__ComparisonFunction(0x0205)
	QOpenGLTexture__CompareAlways       QOpenGLTexture__ComparisonFunction = QOpenGLTexture__ComparisonFunction(0x0207)
	QOpenGLTexture__CompareNever        QOpenGLTexture__ComparisonFunction = QOpenGLTexture__ComparisonFunction(0x0200)
)

//go:generate stringer -type=QOpenGLTexture__ComparisonMode
//QOpenGLTexture::ComparisonMode
type QOpenGLTexture__ComparisonMode int64

const (
	QOpenGLTexture__CompareRefToTexture QOpenGLTexture__ComparisonMode = QOpenGLTexture__ComparisonMode(0x884E)
	QOpenGLTexture__CompareNone         QOpenGLTexture__ComparisonMode = QOpenGLTexture__ComparisonMode(0x0000)
)

//go:generate stringer -type=QOpenGLTexture__Filter
//QOpenGLTexture::Filter
type QOpenGLTexture__Filter int64

const (
	QOpenGLTexture__Nearest              QOpenGLTexture__Filter = QOpenGLTexture__Filter(0x2600)
	QOpenGLTexture__Linear               QOpenGLTexture__Filter = QOpenGLTexture__Filter(0x2601)
	QOpenGLTexture__NearestMipMapNearest QOpenGLTexture__Filter = QOpenGLTexture__Filter(0x2700)
	QOpenGLTexture__NearestMipMapLinear  QOpenGLTexture__Filter = QOpenGLTexture__Filter(0x2702)
	QOpenGLTexture__LinearMipMapNearest  QOpenGLTexture__Filter = QOpenGLTexture__Filter(0x2701)
	QOpenGLTexture__LinearMipMapLinear   QOpenGLTexture__Filter = QOpenGLTexture__Filter(0x2703)
)

func NewQOpenGLTexture(target QOpenGLTexture__Target) *QOpenGLTexture {
	tmpValue := NewQOpenGLTextureFromPointer(C.QOpenGLTexture_NewQOpenGLTexture(C.longlong(target)))
	qt.SetFinalizer(tmpValue, (*QOpenGLTexture).DestroyQOpenGLTexture)
	return tmpValue
}

func NewQOpenGLTexture2(image QImage_ITF, genMipMaps QOpenGLTexture__MipMapGeneration) *QOpenGLTexture {
	tmpValue := NewQOpenGLTextureFromPointer(C.QOpenGLTexture_NewQOpenGLTexture2(PointerFromQImage(image), C.longlong(genMipMaps)))
	qt.SetFinalizer(tmpValue, (*QOpenGLTexture).DestroyQOpenGLTexture)
	return tmpValue
}

func (ptr *QOpenGLTexture) AllocateStorage() {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_AllocateStorage(ptr.Pointer())
	}
}

func (ptr *QOpenGLTexture) AllocateStorage2(pixelFormat QOpenGLTexture__PixelFormat, pixelType QOpenGLTexture__PixelType) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_AllocateStorage2(ptr.Pointer(), C.longlong(pixelFormat), C.longlong(pixelType))
	}
}

func (ptr *QOpenGLTexture) Bind() {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_Bind(ptr.Pointer())
	}
}

func (ptr *QOpenGLTexture) Bind2(unit uint, reset QOpenGLTexture__TextureUnitReset) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_Bind2(ptr.Pointer(), C.uint(uint32(unit)), C.longlong(reset))
	}
}

func (ptr *QOpenGLTexture) BorderColor() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QOpenGLTexture_BorderColor(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLTexture) BorderColor2(border float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_BorderColor2(ptr.Pointer(), C.float(border))
	}
}

func (ptr *QOpenGLTexture) BorderColor3(border int) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_BorderColor3(ptr.Pointer(), C.int(int32(border)))
	}
}

func (ptr *QOpenGLTexture) BorderColor4(border uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_BorderColor4(ptr.Pointer(), C.uint(uint32(border)))
	}
}

func QOpenGLTexture_BoundTextureId(target QOpenGLTexture__BindingTarget) uint {
	return uint(uint32(C.QOpenGLTexture_QOpenGLTexture_BoundTextureId(C.longlong(target))))
}

func (ptr *QOpenGLTexture) BoundTextureId(target QOpenGLTexture__BindingTarget) uint {
	return uint(uint32(C.QOpenGLTexture_QOpenGLTexture_BoundTextureId(C.longlong(target))))
}

func QOpenGLTexture_BoundTextureId2(unit uint, target QOpenGLTexture__BindingTarget) uint {
	return uint(uint32(C.QOpenGLTexture_QOpenGLTexture_BoundTextureId2(C.uint(uint32(unit)), C.longlong(target))))
}

func (ptr *QOpenGLTexture) BoundTextureId2(unit uint, target QOpenGLTexture__BindingTarget) uint {
	return uint(uint32(C.QOpenGLTexture_QOpenGLTexture_BoundTextureId2(C.uint(uint32(unit)), C.longlong(target))))
}

func (ptr *QOpenGLTexture) ComparisonFunction() QOpenGLTexture__ComparisonFunction {
	if ptr.Pointer() != nil {
		return QOpenGLTexture__ComparisonFunction(C.QOpenGLTexture_ComparisonFunction(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) ComparisonMode() QOpenGLTexture__ComparisonMode {
	if ptr.Pointer() != nil {
		return QOpenGLTexture__ComparisonMode(C.QOpenGLTexture_ComparisonMode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) Create() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTexture_Create(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTexture) CreateTextureView(target QOpenGLTexture__Target, viewFormat QOpenGLTexture__TextureFormat, minimumMipmapLevel int, maximumMipmapLevel int, minimumLayer int, maximumLayer int) *QOpenGLTexture {
	if ptr.Pointer() != nil {
		return NewQOpenGLTextureFromPointer(C.QOpenGLTexture_CreateTextureView(ptr.Pointer(), C.longlong(target), C.longlong(viewFormat), C.int(int32(minimumMipmapLevel)), C.int(int32(maximumMipmapLevel)), C.int(int32(minimumLayer)), C.int(int32(maximumLayer))))
	}
	return nil
}

func (ptr *QOpenGLTexture) Depth() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTexture_Depth(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) DepthStencilMode() QOpenGLTexture__DepthStencilMode {
	if ptr.Pointer() != nil {
		return QOpenGLTexture__DepthStencilMode(C.QOpenGLTexture_DepthStencilMode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) Destroy() {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_Destroy(ptr.Pointer())
	}
}

func (ptr *QOpenGLTexture) Faces() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTexture_Faces(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) Format() QOpenGLTexture__TextureFormat {
	if ptr.Pointer() != nil {
		return QOpenGLTexture__TextureFormat(C.QOpenGLTexture_Format(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) GenerateMipMaps() {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_GenerateMipMaps(ptr.Pointer())
	}
}

func (ptr *QOpenGLTexture) GenerateMipMaps2(baseLevel int, resetBaseLevel bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_GenerateMipMaps2(ptr.Pointer(), C.int(int32(baseLevel)), C.char(int8(qt.GoBoolToInt(resetBaseLevel))))
	}
}

func QOpenGLTexture_HasFeature(feature QOpenGLTexture__Feature) bool {
	return int8(C.QOpenGLTexture_QOpenGLTexture_HasFeature(C.longlong(feature))) != 0
}

func (ptr *QOpenGLTexture) HasFeature(feature QOpenGLTexture__Feature) bool {
	return int8(C.QOpenGLTexture_QOpenGLTexture_HasFeature(C.longlong(feature))) != 0
}

func (ptr *QOpenGLTexture) Height() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTexture_Height(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) IsAutoMipMapGenerationEnabled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTexture_IsAutoMipMapGenerationEnabled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTexture) IsBound() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTexture_IsBound(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTexture) IsBound2(unit uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTexture_IsBound2(ptr.Pointer(), C.uint(uint32(unit)))) != 0
	}
	return false
}

func (ptr *QOpenGLTexture) IsCreated() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTexture_IsCreated(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTexture) IsFixedSamplePositions() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTexture_IsFixedSamplePositions(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTexture) IsStorageAllocated() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTexture_IsStorageAllocated(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTexture) IsTextureView() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTexture_IsTextureView(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTexture) Layers() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTexture_Layers(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) LevelofDetailBias() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QOpenGLTexture_LevelofDetailBias(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) MagnificationFilter() QOpenGLTexture__Filter {
	if ptr.Pointer() != nil {
		return QOpenGLTexture__Filter(C.QOpenGLTexture_MagnificationFilter(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) MaximumAnisotropy() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QOpenGLTexture_MaximumAnisotropy(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) MaximumLevelOfDetail() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QOpenGLTexture_MaximumLevelOfDetail(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) MaximumMipLevels() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTexture_MaximumMipLevels(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) MinificationFilter() QOpenGLTexture__Filter {
	if ptr.Pointer() != nil {
		return QOpenGLTexture__Filter(C.QOpenGLTexture_MinificationFilter(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) MinimumLevelOfDetail() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QOpenGLTexture_MinimumLevelOfDetail(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) MipBaseLevel() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTexture_MipBaseLevel(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) MipLevels() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTexture_MipLevels(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) MipMaxLevel() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTexture_MipMaxLevel(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) Release() {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_Release(ptr.Pointer())
	}
}

func (ptr *QOpenGLTexture) Release2(unit uint, reset QOpenGLTexture__TextureUnitReset) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_Release2(ptr.Pointer(), C.uint(uint32(unit)), C.longlong(reset))
	}
}

func (ptr *QOpenGLTexture) Samples() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTexture_Samples(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) SetAutoMipMapGenerationEnabled(enabled bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetAutoMipMapGenerationEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enabled))))
	}
}

func (ptr *QOpenGLTexture) SetBorderColor(color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetBorderColor(ptr.Pointer(), PointerFromQColor(color))
	}
}

func (ptr *QOpenGLTexture) SetBorderColor2(r float32, g float32, b float32, a float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetBorderColor2(ptr.Pointer(), C.float(r), C.float(g), C.float(b), C.float(a))
	}
}

func (ptr *QOpenGLTexture) SetBorderColor3(r int, g int, b int, a int) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetBorderColor3(ptr.Pointer(), C.int(int32(r)), C.int(int32(g)), C.int(int32(b)), C.int(int32(a)))
	}
}

func (ptr *QOpenGLTexture) SetBorderColor4(r uint, g uint, b uint, a uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetBorderColor4(ptr.Pointer(), C.uint(uint32(r)), C.uint(uint32(g)), C.uint(uint32(b)), C.uint(uint32(a)))
	}
}

func (ptr *QOpenGLTexture) SetComparisonFunction(function QOpenGLTexture__ComparisonFunction) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetComparisonFunction(ptr.Pointer(), C.longlong(function))
	}
}

func (ptr *QOpenGLTexture) SetComparisonMode(mode QOpenGLTexture__ComparisonMode) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetComparisonMode(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QOpenGLTexture) SetCompressedData(mipLevel int, layer int, cubeFace QOpenGLTexture__CubeMapFace, dataSize int, data unsafe.Pointer, options QOpenGLPixelTransferOptions_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetCompressedData(ptr.Pointer(), C.int(int32(mipLevel)), C.int(int32(layer)), C.longlong(cubeFace), C.int(int32(dataSize)), data, PointerFromQOpenGLPixelTransferOptions(options))
	}
}

func (ptr *QOpenGLTexture) SetCompressedData6(mipLevel int, layer int, layerCount int, cubeFace QOpenGLTexture__CubeMapFace, dataSize int, data unsafe.Pointer, options QOpenGLPixelTransferOptions_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetCompressedData6(ptr.Pointer(), C.int(int32(mipLevel)), C.int(int32(layer)), C.int(int32(layerCount)), C.longlong(cubeFace), C.int(int32(dataSize)), data, PointerFromQOpenGLPixelTransferOptions(options))
	}
}

func (ptr *QOpenGLTexture) SetCompressedData7(mipLevel int, layer int, dataSize int, data unsafe.Pointer, options QOpenGLPixelTransferOptions_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetCompressedData7(ptr.Pointer(), C.int(int32(mipLevel)), C.int(int32(layer)), C.int(int32(dataSize)), data, PointerFromQOpenGLPixelTransferOptions(options))
	}
}

func (ptr *QOpenGLTexture) SetCompressedData8(mipLevel int, dataSize int, data unsafe.Pointer, options QOpenGLPixelTransferOptions_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetCompressedData8(ptr.Pointer(), C.int(int32(mipLevel)), C.int(int32(dataSize)), data, PointerFromQOpenGLPixelTransferOptions(options))
	}
}

func (ptr *QOpenGLTexture) SetCompressedData9(dataSize int, data unsafe.Pointer, options QOpenGLPixelTransferOptions_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetCompressedData9(ptr.Pointer(), C.int(int32(dataSize)), data, PointerFromQOpenGLPixelTransferOptions(options))
	}
}

func (ptr *QOpenGLTexture) SetData(mipLevel int, layer int, cubeFace QOpenGLTexture__CubeMapFace, sourceFormat QOpenGLTexture__PixelFormat, sourceType QOpenGLTexture__PixelType, data unsafe.Pointer, options QOpenGLPixelTransferOptions_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetData(ptr.Pointer(), C.int(int32(mipLevel)), C.int(int32(layer)), C.longlong(cubeFace), C.longlong(sourceFormat), C.longlong(sourceType), data, PointerFromQOpenGLPixelTransferOptions(options))
	}
}

func (ptr *QOpenGLTexture) SetData6(mipLevel int, layer int, layerCount int, cubeFace QOpenGLTexture__CubeMapFace, sourceFormat QOpenGLTexture__PixelFormat, sourceType QOpenGLTexture__PixelType, data unsafe.Pointer, options QOpenGLPixelTransferOptions_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetData6(ptr.Pointer(), C.int(int32(mipLevel)), C.int(int32(layer)), C.int(int32(layerCount)), C.longlong(cubeFace), C.longlong(sourceFormat), C.longlong(sourceType), data, PointerFromQOpenGLPixelTransferOptions(options))
	}
}

func (ptr *QOpenGLTexture) SetData7(mipLevel int, layer int, sourceFormat QOpenGLTexture__PixelFormat, sourceType QOpenGLTexture__PixelType, data unsafe.Pointer, options QOpenGLPixelTransferOptions_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetData7(ptr.Pointer(), C.int(int32(mipLevel)), C.int(int32(layer)), C.longlong(sourceFormat), C.longlong(sourceType), data, PointerFromQOpenGLPixelTransferOptions(options))
	}
}

func (ptr *QOpenGLTexture) SetData8(mipLevel int, sourceFormat QOpenGLTexture__PixelFormat, sourceType QOpenGLTexture__PixelType, data unsafe.Pointer, options QOpenGLPixelTransferOptions_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetData8(ptr.Pointer(), C.int(int32(mipLevel)), C.longlong(sourceFormat), C.longlong(sourceType), data, PointerFromQOpenGLPixelTransferOptions(options))
	}
}

func (ptr *QOpenGLTexture) SetData9(sourceFormat QOpenGLTexture__PixelFormat, sourceType QOpenGLTexture__PixelType, data unsafe.Pointer, options QOpenGLPixelTransferOptions_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetData9(ptr.Pointer(), C.longlong(sourceFormat), C.longlong(sourceType), data, PointerFromQOpenGLPixelTransferOptions(options))
	}
}

func (ptr *QOpenGLTexture) SetData10(image QImage_ITF, genMipMaps QOpenGLTexture__MipMapGeneration) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetData10(ptr.Pointer(), PointerFromQImage(image), C.longlong(genMipMaps))
	}
}

func (ptr *QOpenGLTexture) SetDepthStencilMode(mode QOpenGLTexture__DepthStencilMode) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetDepthStencilMode(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QOpenGLTexture) SetFixedSamplePositions(fixed bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetFixedSamplePositions(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(fixed))))
	}
}

func (ptr *QOpenGLTexture) SetFormat(format QOpenGLTexture__TextureFormat) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetFormat(ptr.Pointer(), C.longlong(format))
	}
}

func (ptr *QOpenGLTexture) SetLayers(layers int) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetLayers(ptr.Pointer(), C.int(int32(layers)))
	}
}

func (ptr *QOpenGLTexture) SetLevelOfDetailRange(min float32, max float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetLevelOfDetailRange(ptr.Pointer(), C.float(min), C.float(max))
	}
}

func (ptr *QOpenGLTexture) SetLevelofDetailBias(bias float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetLevelofDetailBias(ptr.Pointer(), C.float(bias))
	}
}

func (ptr *QOpenGLTexture) SetMagnificationFilter(filter QOpenGLTexture__Filter) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetMagnificationFilter(ptr.Pointer(), C.longlong(filter))
	}
}

func (ptr *QOpenGLTexture) SetMaximumAnisotropy(anisotropy float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetMaximumAnisotropy(ptr.Pointer(), C.float(anisotropy))
	}
}

func (ptr *QOpenGLTexture) SetMaximumLevelOfDetail(value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetMaximumLevelOfDetail(ptr.Pointer(), C.float(value))
	}
}

func (ptr *QOpenGLTexture) SetMinMagFilters(minificationFilter QOpenGLTexture__Filter, magnificationFilter QOpenGLTexture__Filter) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetMinMagFilters(ptr.Pointer(), C.longlong(minificationFilter), C.longlong(magnificationFilter))
	}
}

func (ptr *QOpenGLTexture) SetMinificationFilter(filter QOpenGLTexture__Filter) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetMinificationFilter(ptr.Pointer(), C.longlong(filter))
	}
}

func (ptr *QOpenGLTexture) SetMinimumLevelOfDetail(value float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetMinimumLevelOfDetail(ptr.Pointer(), C.float(value))
	}
}

func (ptr *QOpenGLTexture) SetMipBaseLevel(baseLevel int) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetMipBaseLevel(ptr.Pointer(), C.int(int32(baseLevel)))
	}
}

func (ptr *QOpenGLTexture) SetMipLevelRange(baseLevel int, maxLevel int) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetMipLevelRange(ptr.Pointer(), C.int(int32(baseLevel)), C.int(int32(maxLevel)))
	}
}

func (ptr *QOpenGLTexture) SetMipLevels(levels int) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetMipLevels(ptr.Pointer(), C.int(int32(levels)))
	}
}

func (ptr *QOpenGLTexture) SetMipMaxLevel(maxLevel int) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetMipMaxLevel(ptr.Pointer(), C.int(int32(maxLevel)))
	}
}

func (ptr *QOpenGLTexture) SetSamples(samples int) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetSamples(ptr.Pointer(), C.int(int32(samples)))
	}
}

func (ptr *QOpenGLTexture) SetSize(width int, height int, depth int) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetSize(ptr.Pointer(), C.int(int32(width)), C.int(int32(height)), C.int(int32(depth)))
	}
}

func (ptr *QOpenGLTexture) SetSwizzleMask(component QOpenGLTexture__SwizzleComponent, value QOpenGLTexture__SwizzleValue) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetSwizzleMask(ptr.Pointer(), C.longlong(component), C.longlong(value))
	}
}

func (ptr *QOpenGLTexture) SetSwizzleMask2(r QOpenGLTexture__SwizzleValue, g QOpenGLTexture__SwizzleValue, b QOpenGLTexture__SwizzleValue, a QOpenGLTexture__SwizzleValue) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetSwizzleMask2(ptr.Pointer(), C.longlong(r), C.longlong(g), C.longlong(b), C.longlong(a))
	}
}

func (ptr *QOpenGLTexture) SetWrapMode(mode QOpenGLTexture__WrapMode) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetWrapMode(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QOpenGLTexture) SetWrapMode2(direction QOpenGLTexture__CoordinateDirection, mode QOpenGLTexture__WrapMode) {
	if ptr.Pointer() != nil {
		C.QOpenGLTexture_SetWrapMode2(ptr.Pointer(), C.longlong(direction), C.longlong(mode))
	}
}

func (ptr *QOpenGLTexture) SwizzleMask(component QOpenGLTexture__SwizzleComponent) QOpenGLTexture__SwizzleValue {
	if ptr.Pointer() != nil {
		return QOpenGLTexture__SwizzleValue(C.QOpenGLTexture_SwizzleMask(ptr.Pointer(), C.longlong(component)))
	}
	return 0
}

func (ptr *QOpenGLTexture) Target() QOpenGLTexture__Target {
	if ptr.Pointer() != nil {
		return QOpenGLTexture__Target(C.QOpenGLTexture_Target(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLTexture) TextureId() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLTexture_TextureId(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) Width() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTexture_Width(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTexture) WrapMode(direction QOpenGLTexture__CoordinateDirection) QOpenGLTexture__WrapMode {
	if ptr.Pointer() != nil {
		return QOpenGLTexture__WrapMode(C.QOpenGLTexture_WrapMode(ptr.Pointer(), C.longlong(direction)))
	}
	return 0
}

type QOpenGLTextureBlitter struct {
	ptr unsafe.Pointer
}

type QOpenGLTextureBlitter_ITF interface {
	QOpenGLTextureBlitter_PTR() *QOpenGLTextureBlitter
}

func (ptr *QOpenGLTextureBlitter) QOpenGLTextureBlitter_PTR() *QOpenGLTextureBlitter {
	return ptr
}

func (ptr *QOpenGLTextureBlitter) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLTextureBlitter) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLTextureBlitter(ptr QOpenGLTextureBlitter_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLTextureBlitter_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLTextureBlitterFromPointer(ptr unsafe.Pointer) (n *QOpenGLTextureBlitter) {
	n = new(QOpenGLTextureBlitter)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QOpenGLTextureBlitter__Origin
//QOpenGLTextureBlitter::Origin
type QOpenGLTextureBlitter__Origin int64

const (
	QOpenGLTextureBlitter__OriginBottomLeft QOpenGLTextureBlitter__Origin = QOpenGLTextureBlitter__Origin(0)
	QOpenGLTextureBlitter__OriginTopLeft    QOpenGLTextureBlitter__Origin = QOpenGLTextureBlitter__Origin(1)
)

func NewQOpenGLTextureBlitter() *QOpenGLTextureBlitter {
	tmpValue := NewQOpenGLTextureBlitterFromPointer(C.QOpenGLTextureBlitter_NewQOpenGLTextureBlitter())
	qt.SetFinalizer(tmpValue, (*QOpenGLTextureBlitter).DestroyQOpenGLTextureBlitter)
	return tmpValue
}

func (ptr *QOpenGLTextureBlitter) Bind(target uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLTextureBlitter_Bind(ptr.Pointer(), C.uint(uint32(target)))
	}
}

func (ptr *QOpenGLTextureBlitter) Blit(texture uint, targetTransform QMatrix4x4_ITF, sourceOrigin QOpenGLTextureBlitter__Origin) {
	if ptr.Pointer() != nil {
		C.QOpenGLTextureBlitter_Blit(ptr.Pointer(), C.uint(uint32(texture)), PointerFromQMatrix4x4(targetTransform), C.longlong(sourceOrigin))
	}
}

func (ptr *QOpenGLTextureBlitter) Create() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTextureBlitter_Create(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTextureBlitter) Destroy() {
	if ptr.Pointer() != nil {
		C.QOpenGLTextureBlitter_Destroy(ptr.Pointer())
	}
}

func (ptr *QOpenGLTextureBlitter) IsCreated() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTextureBlitter_IsCreated(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTextureBlitter) Release() {
	if ptr.Pointer() != nil {
		C.QOpenGLTextureBlitter_Release(ptr.Pointer())
	}
}

func (ptr *QOpenGLTextureBlitter) SetOpacity(opacity float32) {
	if ptr.Pointer() != nil {
		C.QOpenGLTextureBlitter_SetOpacity(ptr.Pointer(), C.float(opacity))
	}
}

func (ptr *QOpenGLTextureBlitter) SetRedBlueSwizzle(swizzle bool) {
	if ptr.Pointer() != nil {
		C.QOpenGLTextureBlitter_SetRedBlueSwizzle(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(swizzle))))
	}
}

func (ptr *QOpenGLTextureBlitter) SupportsExternalOESTarget() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTextureBlitter_SupportsExternalOESTarget(ptr.Pointer())) != 0
	}
	return false
}

func QOpenGLTextureBlitter_TargetTransform(target core.QRectF_ITF, viewport core.QRect_ITF) *QMatrix4x4 {
	tmpValue := NewQMatrix4x4FromPointer(C.QOpenGLTextureBlitter_QOpenGLTextureBlitter_TargetTransform(core.PointerFromQRectF(target), core.PointerFromQRect(viewport)))
	qt.SetFinalizer(tmpValue, (*QMatrix4x4).DestroyQMatrix4x4)
	return tmpValue
}

func (ptr *QOpenGLTextureBlitter) TargetTransform(target core.QRectF_ITF, viewport core.QRect_ITF) *QMatrix4x4 {
	tmpValue := NewQMatrix4x4FromPointer(C.QOpenGLTextureBlitter_QOpenGLTextureBlitter_TargetTransform(core.PointerFromQRectF(target), core.PointerFromQRect(viewport)))
	qt.SetFinalizer(tmpValue, (*QMatrix4x4).DestroyQMatrix4x4)
	return tmpValue
}

func (ptr *QOpenGLTextureBlitter) DestroyQOpenGLTextureBlitter() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLTextureBlitter_DestroyQOpenGLTextureBlitter(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLTimeMonitor struct {
	core.QObject
}

type QOpenGLTimeMonitor_ITF interface {
	core.QObject_ITF
	QOpenGLTimeMonitor_PTR() *QOpenGLTimeMonitor
}

func (ptr *QOpenGLTimeMonitor) QOpenGLTimeMonitor_PTR() *QOpenGLTimeMonitor {
	return ptr
}

func (ptr *QOpenGLTimeMonitor) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLTimeMonitor) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLTimeMonitor(ptr QOpenGLTimeMonitor_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLTimeMonitor_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLTimeMonitorFromPointer(ptr unsafe.Pointer) (n *QOpenGLTimeMonitor) {
	n = new(QOpenGLTimeMonitor)
	n.SetPointer(ptr)
	return
}
func NewQOpenGLTimeMonitor(parent core.QObject_ITF) *QOpenGLTimeMonitor {
	tmpValue := NewQOpenGLTimeMonitorFromPointer(C.QOpenGLTimeMonitor_NewQOpenGLTimeMonitor(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLTimeMonitor) Create() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTimeMonitor_Create(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTimeMonitor) Destroy() {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor_Destroy(ptr.Pointer())
	}
}

func (ptr *QOpenGLTimeMonitor) IsCreated() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTimeMonitor_IsCreated(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTimeMonitor) IsResultAvailable() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTimeMonitor_IsResultAvailable(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTimeMonitor) ObjectIds() []uint {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []uint {
			out := make([]uint, int(l.len))
			tmpList := NewQOpenGLTimeMonitorFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__objectIds_atList(i)
			}
			return out
		}(C.QOpenGLTimeMonitor_ObjectIds(ptr.Pointer()))
	}
	return make([]uint, 0)
}

func (ptr *QOpenGLTimeMonitor) RecordSample() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTimeMonitor_RecordSample(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTimeMonitor) Reset() {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor_Reset(ptr.Pointer())
	}
}

func (ptr *QOpenGLTimeMonitor) SampleCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QOpenGLTimeMonitor_SampleCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTimeMonitor) SetSampleCount(sampleCount int) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor_SetSampleCount(ptr.Pointer(), C.int(int32(sampleCount)))
	}
}

//export callbackQOpenGLTimeMonitor_DestroyQOpenGLTimeMonitor
func callbackQOpenGLTimeMonitor_DestroyQOpenGLTimeMonitor(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOpenGLTimeMonitor"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLTimeMonitorFromPointer(ptr).DestroyQOpenGLTimeMonitorDefault()
	}
}

func (ptr *QOpenGLTimeMonitor) ConnectDestroyQOpenGLTimeMonitor(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOpenGLTimeMonitor"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLTimeMonitor", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLTimeMonitor", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLTimeMonitor) DisconnectDestroyQOpenGLTimeMonitor() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOpenGLTimeMonitor")
	}
}

func (ptr *QOpenGLTimeMonitor) DestroyQOpenGLTimeMonitor() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLTimeMonitor_DestroyQOpenGLTimeMonitor(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLTimeMonitor) DestroyQOpenGLTimeMonitorDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLTimeMonitor_DestroyQOpenGLTimeMonitorDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLTimeMonitor) __objectIds_atList(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLTimeMonitor___objectIds_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QOpenGLTimeMonitor) __objectIds_setList(i uint) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor___objectIds_setList(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QOpenGLTimeMonitor) __objectIds_newList() unsafe.Pointer {
	return C.QOpenGLTimeMonitor___objectIds_newList(ptr.Pointer())
}

func (ptr *QOpenGLTimeMonitor) __waitForIntervals_newList() unsafe.Pointer {
	return C.QOpenGLTimeMonitor___waitForIntervals_newList(ptr.Pointer())
}

func (ptr *QOpenGLTimeMonitor) __waitForSamples_newList() unsafe.Pointer {
	return C.QOpenGLTimeMonitor___waitForSamples_newList(ptr.Pointer())
}

func (ptr *QOpenGLTimeMonitor) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLTimeMonitor___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLTimeMonitor) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLTimeMonitor) __children_newList() unsafe.Pointer {
	return C.QOpenGLTimeMonitor___children_newList(ptr.Pointer())
}

func (ptr *QOpenGLTimeMonitor) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QOpenGLTimeMonitor___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLTimeMonitor) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QOpenGLTimeMonitor) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QOpenGLTimeMonitor___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QOpenGLTimeMonitor) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLTimeMonitor___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLTimeMonitor) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLTimeMonitor) __findChildren_newList() unsafe.Pointer {
	return C.QOpenGLTimeMonitor___findChildren_newList(ptr.Pointer())
}

func (ptr *QOpenGLTimeMonitor) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLTimeMonitor___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLTimeMonitor) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLTimeMonitor) __findChildren_newList3() unsafe.Pointer {
	return C.QOpenGLTimeMonitor___findChildren_newList3(ptr.Pointer())
}

//export callbackQOpenGLTimeMonitor_ChildEvent
func callbackQOpenGLTimeMonitor_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQOpenGLTimeMonitorFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QOpenGLTimeMonitor) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQOpenGLTimeMonitor_ConnectNotify
func callbackQOpenGLTimeMonitor_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLTimeMonitorFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLTimeMonitor) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLTimeMonitor_CustomEvent
func callbackQOpenGLTimeMonitor_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQOpenGLTimeMonitorFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QOpenGLTimeMonitor) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQOpenGLTimeMonitor_DeleteLater
func callbackQOpenGLTimeMonitor_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLTimeMonitorFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QOpenGLTimeMonitor) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLTimeMonitor_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLTimeMonitor_Destroyed
func callbackQOpenGLTimeMonitor_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQOpenGLTimeMonitor_DisconnectNotify
func callbackQOpenGLTimeMonitor_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLTimeMonitorFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLTimeMonitor) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLTimeMonitor_Event
func callbackQOpenGLTimeMonitor_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLTimeMonitorFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QOpenGLTimeMonitor) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTimeMonitor_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQOpenGLTimeMonitor_EventFilter
func callbackQOpenGLTimeMonitor_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLTimeMonitorFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QOpenGLTimeMonitor) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTimeMonitor_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQOpenGLTimeMonitor_MetaObject
func callbackQOpenGLTimeMonitor_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQOpenGLTimeMonitorFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QOpenGLTimeMonitor) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QOpenGLTimeMonitor_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQOpenGLTimeMonitor_ObjectNameChanged
func callbackQOpenGLTimeMonitor_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQOpenGLTimeMonitor_TimerEvent
func callbackQOpenGLTimeMonitor_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQOpenGLTimeMonitorFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QOpenGLTimeMonitor) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimeMonitor_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QOpenGLTimerQuery struct {
	core.QObject
}

type QOpenGLTimerQuery_ITF interface {
	core.QObject_ITF
	QOpenGLTimerQuery_PTR() *QOpenGLTimerQuery
}

func (ptr *QOpenGLTimerQuery) QOpenGLTimerQuery_PTR() *QOpenGLTimerQuery {
	return ptr
}

func (ptr *QOpenGLTimerQuery) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLTimerQuery) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLTimerQuery(ptr QOpenGLTimerQuery_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLTimerQuery_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLTimerQueryFromPointer(ptr unsafe.Pointer) (n *QOpenGLTimerQuery) {
	n = new(QOpenGLTimerQuery)
	n.SetPointer(ptr)
	return
}
func NewQOpenGLTimerQuery(parent core.QObject_ITF) *QOpenGLTimerQuery {
	tmpValue := NewQOpenGLTimerQueryFromPointer(C.QOpenGLTimerQuery_NewQOpenGLTimerQuery(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLTimerQuery) Begin() {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery_Begin(ptr.Pointer())
	}
}

func (ptr *QOpenGLTimerQuery) Create() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTimerQuery_Create(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTimerQuery) Destroy() {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery_Destroy(ptr.Pointer())
	}
}

func (ptr *QOpenGLTimerQuery) End() {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery_End(ptr.Pointer())
	}
}

func (ptr *QOpenGLTimerQuery) IsCreated() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTimerQuery_IsCreated(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTimerQuery) IsResultAvailable() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTimerQuery_IsResultAvailable(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLTimerQuery) ObjectId() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLTimerQuery_ObjectId(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLTimerQuery) RecordTimestamp() {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery_RecordTimestamp(ptr.Pointer())
	}
}

//export callbackQOpenGLTimerQuery_DestroyQOpenGLTimerQuery
func callbackQOpenGLTimerQuery_DestroyQOpenGLTimerQuery(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOpenGLTimerQuery"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLTimerQueryFromPointer(ptr).DestroyQOpenGLTimerQueryDefault()
	}
}

func (ptr *QOpenGLTimerQuery) ConnectDestroyQOpenGLTimerQuery(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOpenGLTimerQuery"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLTimerQuery", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLTimerQuery", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLTimerQuery) DisconnectDestroyQOpenGLTimerQuery() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOpenGLTimerQuery")
	}
}

func (ptr *QOpenGLTimerQuery) DestroyQOpenGLTimerQuery() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLTimerQuery_DestroyQOpenGLTimerQuery(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLTimerQuery) DestroyQOpenGLTimerQueryDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLTimerQuery_DestroyQOpenGLTimerQueryDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLTimerQuery) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLTimerQuery___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLTimerQuery) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLTimerQuery) __children_newList() unsafe.Pointer {
	return C.QOpenGLTimerQuery___children_newList(ptr.Pointer())
}

func (ptr *QOpenGLTimerQuery) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QOpenGLTimerQuery___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLTimerQuery) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QOpenGLTimerQuery) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QOpenGLTimerQuery___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QOpenGLTimerQuery) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLTimerQuery___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLTimerQuery) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLTimerQuery) __findChildren_newList() unsafe.Pointer {
	return C.QOpenGLTimerQuery___findChildren_newList(ptr.Pointer())
}

func (ptr *QOpenGLTimerQuery) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLTimerQuery___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLTimerQuery) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLTimerQuery) __findChildren_newList3() unsafe.Pointer {
	return C.QOpenGLTimerQuery___findChildren_newList3(ptr.Pointer())
}

//export callbackQOpenGLTimerQuery_ChildEvent
func callbackQOpenGLTimerQuery_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQOpenGLTimerQueryFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QOpenGLTimerQuery) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQOpenGLTimerQuery_ConnectNotify
func callbackQOpenGLTimerQuery_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLTimerQueryFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLTimerQuery) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLTimerQuery_CustomEvent
func callbackQOpenGLTimerQuery_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQOpenGLTimerQueryFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QOpenGLTimerQuery) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQOpenGLTimerQuery_DeleteLater
func callbackQOpenGLTimerQuery_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLTimerQueryFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QOpenGLTimerQuery) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLTimerQuery_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLTimerQuery_Destroyed
func callbackQOpenGLTimerQuery_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQOpenGLTimerQuery_DisconnectNotify
func callbackQOpenGLTimerQuery_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLTimerQueryFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLTimerQuery) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLTimerQuery_Event
func callbackQOpenGLTimerQuery_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLTimerQueryFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QOpenGLTimerQuery) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTimerQuery_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQOpenGLTimerQuery_EventFilter
func callbackQOpenGLTimerQuery_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLTimerQueryFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QOpenGLTimerQuery) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLTimerQuery_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQOpenGLTimerQuery_MetaObject
func callbackQOpenGLTimerQuery_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQOpenGLTimerQueryFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QOpenGLTimerQuery) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QOpenGLTimerQuery_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQOpenGLTimerQuery_ObjectNameChanged
func callbackQOpenGLTimerQuery_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQOpenGLTimerQuery_TimerEvent
func callbackQOpenGLTimerQuery_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQOpenGLTimerQueryFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QOpenGLTimerQuery) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLTimerQuery_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QOpenGLVersionFunctionsBackend struct {
	ptr unsafe.Pointer
}

type QOpenGLVersionFunctionsBackend_ITF interface {
	QOpenGLVersionFunctionsBackend_PTR() *QOpenGLVersionFunctionsBackend
}

func (ptr *QOpenGLVersionFunctionsBackend) QOpenGLVersionFunctionsBackend_PTR() *QOpenGLVersionFunctionsBackend {
	return ptr
}

func (ptr *QOpenGLVersionFunctionsBackend) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLVersionFunctionsBackend) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLVersionFunctionsBackend(ptr QOpenGLVersionFunctionsBackend_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsBackend_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLVersionFunctionsBackendFromPointer(ptr unsafe.Pointer) (n *QOpenGLVersionFunctionsBackend) {
	n = new(QOpenGLVersionFunctionsBackend)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLVersionFunctionsBackend) DestroyQOpenGLVersionFunctionsBackend() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLVersionFunctionsStorage struct {
	ptr unsafe.Pointer
}

type QOpenGLVersionFunctionsStorage_ITF interface {
	QOpenGLVersionFunctionsStorage_PTR() *QOpenGLVersionFunctionsStorage
}

func (ptr *QOpenGLVersionFunctionsStorage) QOpenGLVersionFunctionsStorage_PTR() *QOpenGLVersionFunctionsStorage {
	return ptr
}

func (ptr *QOpenGLVersionFunctionsStorage) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLVersionFunctionsStorage) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLVersionFunctionsStorage(ptr QOpenGLVersionFunctionsStorage_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionFunctionsStorage_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLVersionFunctionsStorageFromPointer(ptr unsafe.Pointer) (n *QOpenGLVersionFunctionsStorage) {
	n = new(QOpenGLVersionFunctionsStorage)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLVersionFunctionsStorage) DestroyQOpenGLVersionFunctionsStorage() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLVersionProfile struct {
	ptr unsafe.Pointer
}

type QOpenGLVersionProfile_ITF interface {
	QOpenGLVersionProfile_PTR() *QOpenGLVersionProfile
}

func (ptr *QOpenGLVersionProfile) QOpenGLVersionProfile_PTR() *QOpenGLVersionProfile {
	return ptr
}

func (ptr *QOpenGLVersionProfile) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLVersionProfile) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLVersionProfile(ptr QOpenGLVersionProfile_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionProfile_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLVersionProfileFromPointer(ptr unsafe.Pointer) (n *QOpenGLVersionProfile) {
	n = new(QOpenGLVersionProfile)
	n.SetPointer(ptr)
	return
}
func NewQOpenGLVersionProfile() *QOpenGLVersionProfile {
	tmpValue := NewQOpenGLVersionProfileFromPointer(C.QOpenGLVersionProfile_NewQOpenGLVersionProfile())
	qt.SetFinalizer(tmpValue, (*QOpenGLVersionProfile).DestroyQOpenGLVersionProfile)
	return tmpValue
}

func NewQOpenGLVersionProfile2(format QSurfaceFormat_ITF) *QOpenGLVersionProfile {
	tmpValue := NewQOpenGLVersionProfileFromPointer(C.QOpenGLVersionProfile_NewQOpenGLVersionProfile2(PointerFromQSurfaceFormat(format)))
	qt.SetFinalizer(tmpValue, (*QOpenGLVersionProfile).DestroyQOpenGLVersionProfile)
	return tmpValue
}

func NewQOpenGLVersionProfile3(other QOpenGLVersionProfile_ITF) *QOpenGLVersionProfile {
	tmpValue := NewQOpenGLVersionProfileFromPointer(C.QOpenGLVersionProfile_NewQOpenGLVersionProfile3(PointerFromQOpenGLVersionProfile(other)))
	qt.SetFinalizer(tmpValue, (*QOpenGLVersionProfile).DestroyQOpenGLVersionProfile)
	return tmpValue
}

func (ptr *QOpenGLVersionProfile) HasProfiles() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLVersionProfile_HasProfiles(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLVersionProfile) IsLegacyVersion() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLVersionProfile_IsLegacyVersion(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLVersionProfile) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLVersionProfile_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLVersionProfile) Profile() QSurfaceFormat__OpenGLContextProfile {
	if ptr.Pointer() != nil {
		return QSurfaceFormat__OpenGLContextProfile(C.QOpenGLVersionProfile_Profile(ptr.Pointer()))
	}
	return 0
}

func (ptr *QOpenGLVersionProfile) SetProfile(profile QSurfaceFormat__OpenGLContextProfile) {
	if ptr.Pointer() != nil {
		C.QOpenGLVersionProfile_SetProfile(ptr.Pointer(), C.longlong(profile))
	}
}

func (ptr *QOpenGLVersionProfile) SetVersion(majorVersion int, minorVersion int) {
	if ptr.Pointer() != nil {
		C.QOpenGLVersionProfile_SetVersion(ptr.Pointer(), C.int(int32(majorVersion)), C.int(int32(minorVersion)))
	}
}

func (ptr *QOpenGLVersionProfile) DestroyQOpenGLVersionProfile() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLVersionProfile_DestroyQOpenGLVersionProfile(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLVersionStatus struct {
	ptr unsafe.Pointer
}

type QOpenGLVersionStatus_ITF interface {
	QOpenGLVersionStatus_PTR() *QOpenGLVersionStatus
}

func (ptr *QOpenGLVersionStatus) QOpenGLVersionStatus_PTR() *QOpenGLVersionStatus {
	return ptr
}

func (ptr *QOpenGLVersionStatus) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QOpenGLVersionStatus) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQOpenGLVersionStatus(ptr QOpenGLVersionStatus_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVersionStatus_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLVersionStatusFromPointer(ptr unsafe.Pointer) (n *QOpenGLVersionStatus) {
	n = new(QOpenGLVersionStatus)
	n.SetPointer(ptr)
	return
}
func (ptr *QOpenGLVersionStatus) DestroyQOpenGLVersionStatus() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QOpenGLVertexArrayObject struct {
	core.QObject
}

type QOpenGLVertexArrayObject_ITF interface {
	core.QObject_ITF
	QOpenGLVertexArrayObject_PTR() *QOpenGLVertexArrayObject
}

func (ptr *QOpenGLVertexArrayObject) QOpenGLVertexArrayObject_PTR() *QOpenGLVertexArrayObject {
	return ptr
}

func (ptr *QOpenGLVertexArrayObject) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLVertexArrayObject) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLVertexArrayObject(ptr QOpenGLVertexArrayObject_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLVertexArrayObject_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLVertexArrayObjectFromPointer(ptr unsafe.Pointer) (n *QOpenGLVertexArrayObject) {
	n = new(QOpenGLVertexArrayObject)
	n.SetPointer(ptr)
	return
}
func NewQOpenGLVertexArrayObject(parent core.QObject_ITF) *QOpenGLVertexArrayObject {
	tmpValue := NewQOpenGLVertexArrayObjectFromPointer(C.QOpenGLVertexArrayObject_NewQOpenGLVertexArrayObject(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLVertexArrayObject) Bind() {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject_Bind(ptr.Pointer())
	}
}

func (ptr *QOpenGLVertexArrayObject) Create() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLVertexArrayObject_Create(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLVertexArrayObject) Destroy() {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject_Destroy(ptr.Pointer())
	}
}

func (ptr *QOpenGLVertexArrayObject) IsCreated() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLVertexArrayObject_IsCreated(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLVertexArrayObject) ObjectId() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLVertexArrayObject_ObjectId(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLVertexArrayObject) Release() {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject_Release(ptr.Pointer())
	}
}

//export callbackQOpenGLVertexArrayObject_DestroyQOpenGLVertexArrayObject
func callbackQOpenGLVertexArrayObject_DestroyQOpenGLVertexArrayObject(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOpenGLVertexArrayObject"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLVertexArrayObjectFromPointer(ptr).DestroyQOpenGLVertexArrayObjectDefault()
	}
}

func (ptr *QOpenGLVertexArrayObject) ConnectDestroyQOpenGLVertexArrayObject(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOpenGLVertexArrayObject"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLVertexArrayObject", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLVertexArrayObject", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLVertexArrayObject) DisconnectDestroyQOpenGLVertexArrayObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOpenGLVertexArrayObject")
	}
}

func (ptr *QOpenGLVertexArrayObject) DestroyQOpenGLVertexArrayObject() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLVertexArrayObject_DestroyQOpenGLVertexArrayObject(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLVertexArrayObject) DestroyQOpenGLVertexArrayObjectDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLVertexArrayObject_DestroyQOpenGLVertexArrayObjectDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLVertexArrayObject) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLVertexArrayObject___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLVertexArrayObject) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLVertexArrayObject) __children_newList() unsafe.Pointer {
	return C.QOpenGLVertexArrayObject___children_newList(ptr.Pointer())
}

func (ptr *QOpenGLVertexArrayObject) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QOpenGLVertexArrayObject___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLVertexArrayObject) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QOpenGLVertexArrayObject) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QOpenGLVertexArrayObject___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QOpenGLVertexArrayObject) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLVertexArrayObject___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLVertexArrayObject) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLVertexArrayObject) __findChildren_newList() unsafe.Pointer {
	return C.QOpenGLVertexArrayObject___findChildren_newList(ptr.Pointer())
}

func (ptr *QOpenGLVertexArrayObject) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QOpenGLVertexArrayObject___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLVertexArrayObject) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QOpenGLVertexArrayObject) __findChildren_newList3() unsafe.Pointer {
	return C.QOpenGLVertexArrayObject___findChildren_newList3(ptr.Pointer())
}

//export callbackQOpenGLVertexArrayObject_ChildEvent
func callbackQOpenGLVertexArrayObject_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQOpenGLVertexArrayObjectFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QOpenGLVertexArrayObject) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQOpenGLVertexArrayObject_ConnectNotify
func callbackQOpenGLVertexArrayObject_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLVertexArrayObjectFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLVertexArrayObject) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLVertexArrayObject_CustomEvent
func callbackQOpenGLVertexArrayObject_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQOpenGLVertexArrayObjectFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QOpenGLVertexArrayObject) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQOpenGLVertexArrayObject_DeleteLater
func callbackQOpenGLVertexArrayObject_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLVertexArrayObjectFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QOpenGLVertexArrayObject) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLVertexArrayObject_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLVertexArrayObject_Destroyed
func callbackQOpenGLVertexArrayObject_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQOpenGLVertexArrayObject_DisconnectNotify
func callbackQOpenGLVertexArrayObject_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQOpenGLVertexArrayObjectFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QOpenGLVertexArrayObject) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQOpenGLVertexArrayObject_Event
func callbackQOpenGLVertexArrayObject_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLVertexArrayObjectFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QOpenGLVertexArrayObject) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLVertexArrayObject_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQOpenGLVertexArrayObject_EventFilter
func callbackQOpenGLVertexArrayObject_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQOpenGLVertexArrayObjectFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QOpenGLVertexArrayObject) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLVertexArrayObject_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQOpenGLVertexArrayObject_MetaObject
func callbackQOpenGLVertexArrayObject_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQOpenGLVertexArrayObjectFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QOpenGLVertexArrayObject) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QOpenGLVertexArrayObject_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQOpenGLVertexArrayObject_ObjectNameChanged
func callbackQOpenGLVertexArrayObject_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQOpenGLVertexArrayObject_TimerEvent
func callbackQOpenGLVertexArrayObject_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQOpenGLVertexArrayObjectFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QOpenGLVertexArrayObject) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QOpenGLVertexArrayObject_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QOpenGLWindow struct {
	QPaintDeviceWindow
}

type QOpenGLWindow_ITF interface {
	QPaintDeviceWindow_ITF
	QOpenGLWindow_PTR() *QOpenGLWindow
}

func (ptr *QOpenGLWindow) QOpenGLWindow_PTR() *QOpenGLWindow {
	return ptr
}

func (ptr *QOpenGLWindow) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintDeviceWindow_PTR().Pointer()
	}
	return nil
}

func (ptr *QOpenGLWindow) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QPaintDeviceWindow_PTR().SetPointer(p)
	}
}

func PointerFromQOpenGLWindow(ptr QOpenGLWindow_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QOpenGLWindow_PTR().Pointer()
	}
	return nil
}

func NewQOpenGLWindowFromPointer(ptr unsafe.Pointer) (n *QOpenGLWindow) {
	n = new(QOpenGLWindow)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QOpenGLWindow__UpdateBehavior
//QOpenGLWindow::UpdateBehavior
type QOpenGLWindow__UpdateBehavior int64

const (
	QOpenGLWindow__NoPartialUpdate    QOpenGLWindow__UpdateBehavior = QOpenGLWindow__UpdateBehavior(0)
	QOpenGLWindow__PartialUpdateBlit  QOpenGLWindow__UpdateBehavior = QOpenGLWindow__UpdateBehavior(1)
	QOpenGLWindow__PartialUpdateBlend QOpenGLWindow__UpdateBehavior = QOpenGLWindow__UpdateBehavior(2)
)

func NewQOpenGLWindow(updateBehavior QOpenGLWindow__UpdateBehavior, parent QWindow_ITF) *QOpenGLWindow {
	tmpValue := NewQOpenGLWindowFromPointer(C.QOpenGLWindow_NewQOpenGLWindow(C.longlong(updateBehavior), PointerFromQWindow(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQOpenGLWindow2(shareContext QOpenGLContext_ITF, updateBehavior QOpenGLWindow__UpdateBehavior, parent QWindow_ITF) *QOpenGLWindow {
	tmpValue := NewQOpenGLWindowFromPointer(C.QOpenGLWindow_NewQOpenGLWindow2(PointerFromQOpenGLContext(shareContext), C.longlong(updateBehavior), PointerFromQWindow(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QOpenGLWindow) Context() *QOpenGLContext {
	if ptr.Pointer() != nil {
		tmpValue := NewQOpenGLContextFromPointer(C.QOpenGLWindow_Context(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLWindow) DefaultFramebufferObject() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QOpenGLWindow_DefaultFramebufferObject(ptr.Pointer())))
	}
	return 0
}

func (ptr *QOpenGLWindow) DoneCurrent() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_DoneCurrent(ptr.Pointer())
	}
}

//export callbackQOpenGLWindow_FrameSwapped
func callbackQOpenGLWindow_FrameSwapped(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "frameSwapped"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QOpenGLWindow) ConnectFrameSwapped(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "frameSwapped") {
			C.QOpenGLWindow_ConnectFrameSwapped(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "frameSwapped")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "frameSwapped"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "frameSwapped", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "frameSwapped", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLWindow) DisconnectFrameSwapped() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_DisconnectFrameSwapped(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "frameSwapped")
	}
}

func (ptr *QOpenGLWindow) FrameSwapped() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_FrameSwapped(ptr.Pointer())
	}
}

func (ptr *QOpenGLWindow) GrabFramebuffer() *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QOpenGLWindow_GrabFramebuffer(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

//export callbackQOpenGLWindow_InitializeGL
func callbackQOpenGLWindow_InitializeGL(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "initializeGL"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLWindowFromPointer(ptr).InitializeGLDefault()
	}
}

func (ptr *QOpenGLWindow) ConnectInitializeGL(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "initializeGL"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "initializeGL", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "initializeGL", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLWindow) DisconnectInitializeGL() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "initializeGL")
	}
}

func (ptr *QOpenGLWindow) InitializeGL() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_InitializeGL(ptr.Pointer())
	}
}

func (ptr *QOpenGLWindow) InitializeGLDefault() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_InitializeGLDefault(ptr.Pointer())
	}
}

func (ptr *QOpenGLWindow) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QOpenGLWindow_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QOpenGLWindow) MakeCurrent() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_MakeCurrent(ptr.Pointer())
	}
}

//export callbackQOpenGLWindow_PaintGL
func callbackQOpenGLWindow_PaintGL(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "paintGL"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLWindowFromPointer(ptr).PaintGLDefault()
	}
}

func (ptr *QOpenGLWindow) ConnectPaintGL(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "paintGL"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "paintGL", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "paintGL", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLWindow) DisconnectPaintGL() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "paintGL")
	}
}

func (ptr *QOpenGLWindow) PaintGL() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_PaintGL(ptr.Pointer())
	}
}

func (ptr *QOpenGLWindow) PaintGLDefault() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_PaintGLDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLWindow_PaintOverGL
func callbackQOpenGLWindow_PaintOverGL(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "paintOverGL"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLWindowFromPointer(ptr).PaintOverGLDefault()
	}
}

func (ptr *QOpenGLWindow) ConnectPaintOverGL(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "paintOverGL"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "paintOverGL", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "paintOverGL", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLWindow) DisconnectPaintOverGL() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "paintOverGL")
	}
}

func (ptr *QOpenGLWindow) PaintOverGL() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_PaintOverGL(ptr.Pointer())
	}
}

func (ptr *QOpenGLWindow) PaintOverGLDefault() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_PaintOverGLDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLWindow_PaintUnderGL
func callbackQOpenGLWindow_PaintUnderGL(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "paintUnderGL"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLWindowFromPointer(ptr).PaintUnderGLDefault()
	}
}

func (ptr *QOpenGLWindow) ConnectPaintUnderGL(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "paintUnderGL"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "paintUnderGL", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "paintUnderGL", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLWindow) DisconnectPaintUnderGL() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "paintUnderGL")
	}
}

func (ptr *QOpenGLWindow) PaintUnderGL() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_PaintUnderGL(ptr.Pointer())
	}
}

func (ptr *QOpenGLWindow) PaintUnderGLDefault() {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_PaintUnderGLDefault(ptr.Pointer())
	}
}

//export callbackQOpenGLWindow_ResizeGL
func callbackQOpenGLWindow_ResizeGL(ptr unsafe.Pointer, w C.int, h C.int) {
	if signal := qt.GetSignal(ptr, "resizeGL"); signal != nil {
		(*(*func(int, int))(signal))(int(int32(w)), int(int32(h)))
	} else {
		NewQOpenGLWindowFromPointer(ptr).ResizeGLDefault(int(int32(w)), int(int32(h)))
	}
}

func (ptr *QOpenGLWindow) ConnectResizeGL(f func(w int, h int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "resizeGL"); signal != nil {
			f := func(w int, h int) {
				(*(*func(int, int))(signal))(w, h)
				f(w, h)
			}
			qt.ConnectSignal(ptr.Pointer(), "resizeGL", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "resizeGL", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLWindow) DisconnectResizeGL() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "resizeGL")
	}
}

func (ptr *QOpenGLWindow) ResizeGL(w int, h int) {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_ResizeGL(ptr.Pointer(), C.int(int32(w)), C.int(int32(h)))
	}
}

func (ptr *QOpenGLWindow) ResizeGLDefault(w int, h int) {
	if ptr.Pointer() != nil {
		C.QOpenGLWindow_ResizeGLDefault(ptr.Pointer(), C.int(int32(w)), C.int(int32(h)))
	}
}

func (ptr *QOpenGLWindow) ShareContext() *QOpenGLContext {
	if ptr.Pointer() != nil {
		tmpValue := NewQOpenGLContextFromPointer(C.QOpenGLWindow_ShareContext(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QOpenGLWindow) UpdateBehavior() QOpenGLWindow__UpdateBehavior {
	if ptr.Pointer() != nil {
		return QOpenGLWindow__UpdateBehavior(C.QOpenGLWindow_UpdateBehavior(ptr.Pointer()))
	}
	return 0
}

//export callbackQOpenGLWindow_DestroyQOpenGLWindow
func callbackQOpenGLWindow_DestroyQOpenGLWindow(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QOpenGLWindow"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQOpenGLWindowFromPointer(ptr).DestroyQOpenGLWindowDefault()
	}
}

func (ptr *QOpenGLWindow) ConnectDestroyQOpenGLWindow(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QOpenGLWindow"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLWindow", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QOpenGLWindow", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QOpenGLWindow) DisconnectDestroyQOpenGLWindow() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QOpenGLWindow")
	}
}

func (ptr *QOpenGLWindow) DestroyQOpenGLWindow() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLWindow_DestroyQOpenGLWindow(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QOpenGLWindow) DestroyQOpenGLWindowDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QOpenGLWindow_DestroyQOpenGLWindowDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QPageLayout struct {
	ptr unsafe.Pointer
}

type QPageLayout_ITF interface {
	QPageLayout_PTR() *QPageLayout
}

func (ptr *QPageLayout) QPageLayout_PTR() *QPageLayout {
	return ptr
}

func (ptr *QPageLayout) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPageLayout) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPageLayout(ptr QPageLayout_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPageLayout_PTR().Pointer()
	}
	return nil
}

func NewQPageLayoutFromPointer(ptr unsafe.Pointer) (n *QPageLayout) {
	n = new(QPageLayout)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QPageLayout__Unit
//QPageLayout::Unit
type QPageLayout__Unit int64

const (
	QPageLayout__Millimeter QPageLayout__Unit = QPageLayout__Unit(0)
	QPageLayout__Point      QPageLayout__Unit = QPageLayout__Unit(1)
	QPageLayout__Inch       QPageLayout__Unit = QPageLayout__Unit(2)
	QPageLayout__Pica       QPageLayout__Unit = QPageLayout__Unit(3)
	QPageLayout__Didot      QPageLayout__Unit = QPageLayout__Unit(4)
	QPageLayout__Cicero     QPageLayout__Unit = QPageLayout__Unit(5)
)

//go:generate stringer -type=QPageLayout__Orientation
//QPageLayout::Orientation
type QPageLayout__Orientation int64

const (
	QPageLayout__Portrait  QPageLayout__Orientation = QPageLayout__Orientation(0)
	QPageLayout__Landscape QPageLayout__Orientation = QPageLayout__Orientation(1)
)

//go:generate stringer -type=QPageLayout__Mode
//QPageLayout::Mode
type QPageLayout__Mode int64

const (
	QPageLayout__StandardMode QPageLayout__Mode = QPageLayout__Mode(0)
	QPageLayout__FullPageMode QPageLayout__Mode = QPageLayout__Mode(1)
)

func NewQPageLayout() *QPageLayout {
	tmpValue := NewQPageLayoutFromPointer(C.QPageLayout_NewQPageLayout())
	qt.SetFinalizer(tmpValue, (*QPageLayout).DestroyQPageLayout)
	return tmpValue
}

func NewQPageLayout2(pageSize QPageSize_ITF, orientation QPageLayout__Orientation, margins core.QMarginsF_ITF, units QPageLayout__Unit, minMargins core.QMarginsF_ITF) *QPageLayout {
	tmpValue := NewQPageLayoutFromPointer(C.QPageLayout_NewQPageLayout2(PointerFromQPageSize(pageSize), C.longlong(orientation), core.PointerFromQMarginsF(margins), C.longlong(units), core.PointerFromQMarginsF(minMargins)))
	qt.SetFinalizer(tmpValue, (*QPageLayout).DestroyQPageLayout)
	return tmpValue
}

func NewQPageLayout3(other QPageLayout_ITF) *QPageLayout {
	tmpValue := NewQPageLayoutFromPointer(C.QPageLayout_NewQPageLayout3(PointerFromQPageLayout(other)))
	qt.SetFinalizer(tmpValue, (*QPageLayout).DestroyQPageLayout)
	return tmpValue
}

func (ptr *QPageLayout) FullRect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QPageLayout_FullRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) FullRect2(units QPageLayout__Unit) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QPageLayout_FullRect2(ptr.Pointer(), C.longlong(units)))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) FullRectPixels(resolution int) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPageLayout_FullRectPixels(ptr.Pointer(), C.int(int32(resolution))))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) FullRectPoints() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPageLayout_FullRectPoints(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) IsEquivalentTo(other QPageLayout_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPageLayout_IsEquivalentTo(ptr.Pointer(), PointerFromQPageLayout(other))) != 0
	}
	return false
}

func (ptr *QPageLayout) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPageLayout_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPageLayout) Margins() *core.QMarginsF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMarginsFFromPointer(C.QPageLayout_Margins(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QMarginsF).DestroyQMarginsF)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) Margins2(units QPageLayout__Unit) *core.QMarginsF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMarginsFFromPointer(C.QPageLayout_Margins2(ptr.Pointer(), C.longlong(units)))
		qt.SetFinalizer(tmpValue, (*core.QMarginsF).DestroyQMarginsF)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) MarginsPixels(resolution int) *core.QMargins {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMarginsFromPointer(C.QPageLayout_MarginsPixels(ptr.Pointer(), C.int(int32(resolution))))
		qt.SetFinalizer(tmpValue, (*core.QMargins).DestroyQMargins)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) MarginsPoints() *core.QMargins {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMarginsFromPointer(C.QPageLayout_MarginsPoints(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QMargins).DestroyQMargins)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) MaximumMargins() *core.QMarginsF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMarginsFFromPointer(C.QPageLayout_MaximumMargins(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QMarginsF).DestroyQMarginsF)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) MinimumMargins() *core.QMarginsF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMarginsFFromPointer(C.QPageLayout_MinimumMargins(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QMarginsF).DestroyQMarginsF)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) Mode() QPageLayout__Mode {
	if ptr.Pointer() != nil {
		return QPageLayout__Mode(C.QPageLayout_Mode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPageLayout) Orientation() QPageLayout__Orientation {
	if ptr.Pointer() != nil {
		return QPageLayout__Orientation(C.QPageLayout_Orientation(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPageLayout) PageSize() *QPageSize {
	if ptr.Pointer() != nil {
		tmpValue := NewQPageSizeFromPointer(C.QPageLayout_PageSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPageSize).DestroyQPageSize)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) PaintRect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QPageLayout_PaintRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) PaintRect2(units QPageLayout__Unit) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QPageLayout_PaintRect2(ptr.Pointer(), C.longlong(units)))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) PaintRectPixels(resolution int) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPageLayout_PaintRectPixels(ptr.Pointer(), C.int(int32(resolution))))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) PaintRectPoints() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPageLayout_PaintRectPoints(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPageLayout) SetBottomMargin(bottomMargin float64) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPageLayout_SetBottomMargin(ptr.Pointer(), C.double(bottomMargin))) != 0
	}
	return false
}

func (ptr *QPageLayout) SetLeftMargin(leftMargin float64) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPageLayout_SetLeftMargin(ptr.Pointer(), C.double(leftMargin))) != 0
	}
	return false
}

func (ptr *QPageLayout) SetMargins(margins core.QMarginsF_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPageLayout_SetMargins(ptr.Pointer(), core.PointerFromQMarginsF(margins))) != 0
	}
	return false
}

func (ptr *QPageLayout) SetMinimumMargins(minMargins core.QMarginsF_ITF) {
	if ptr.Pointer() != nil {
		C.QPageLayout_SetMinimumMargins(ptr.Pointer(), core.PointerFromQMarginsF(minMargins))
	}
}

func (ptr *QPageLayout) SetMode(mode QPageLayout__Mode) {
	if ptr.Pointer() != nil {
		C.QPageLayout_SetMode(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QPageLayout) SetOrientation(orientation QPageLayout__Orientation) {
	if ptr.Pointer() != nil {
		C.QPageLayout_SetOrientation(ptr.Pointer(), C.longlong(orientation))
	}
}

func (ptr *QPageLayout) SetPageSize(pageSize QPageSize_ITF, minMargins core.QMarginsF_ITF) {
	if ptr.Pointer() != nil {
		C.QPageLayout_SetPageSize(ptr.Pointer(), PointerFromQPageSize(pageSize), core.PointerFromQMarginsF(minMargins))
	}
}

func (ptr *QPageLayout) SetRightMargin(rightMargin float64) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPageLayout_SetRightMargin(ptr.Pointer(), C.double(rightMargin))) != 0
	}
	return false
}

func (ptr *QPageLayout) SetTopMargin(topMargin float64) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPageLayout_SetTopMargin(ptr.Pointer(), C.double(topMargin))) != 0
	}
	return false
}

func (ptr *QPageLayout) SetUnits(units QPageLayout__Unit) {
	if ptr.Pointer() != nil {
		C.QPageLayout_SetUnits(ptr.Pointer(), C.longlong(units))
	}
}

func (ptr *QPageLayout) Swap(other QPageLayout_ITF) {
	if ptr.Pointer() != nil {
		C.QPageLayout_Swap(ptr.Pointer(), PointerFromQPageLayout(other))
	}
}

func (ptr *QPageLayout) Units() QPageLayout__Unit {
	if ptr.Pointer() != nil {
		return QPageLayout__Unit(C.QPageLayout_Units(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPageLayout) DestroyQPageLayout() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPageLayout_DestroyQPageLayout(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QPageSize struct {
	ptr unsafe.Pointer
}

type QPageSize_ITF interface {
	QPageSize_PTR() *QPageSize
}

func (ptr *QPageSize) QPageSize_PTR() *QPageSize {
	return ptr
}

func (ptr *QPageSize) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPageSize) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPageSize(ptr QPageSize_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPageSize_PTR().Pointer()
	}
	return nil
}

func NewQPageSizeFromPointer(ptr unsafe.Pointer) (n *QPageSize) {
	n = new(QPageSize)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QPageSize__PageSizeId
//QPageSize::PageSizeId
type QPageSize__PageSizeId int64

const (
	QPageSize__A4                 QPageSize__PageSizeId = QPageSize__PageSizeId(0)
	QPageSize__B5                 QPageSize__PageSizeId = QPageSize__PageSizeId(1)
	QPageSize__Letter             QPageSize__PageSizeId = QPageSize__PageSizeId(2)
	QPageSize__Legal              QPageSize__PageSizeId = QPageSize__PageSizeId(3)
	QPageSize__Executive          QPageSize__PageSizeId = QPageSize__PageSizeId(4)
	QPageSize__A0                 QPageSize__PageSizeId = QPageSize__PageSizeId(5)
	QPageSize__A1                 QPageSize__PageSizeId = QPageSize__PageSizeId(6)
	QPageSize__A2                 QPageSize__PageSizeId = QPageSize__PageSizeId(7)
	QPageSize__A3                 QPageSize__PageSizeId = QPageSize__PageSizeId(8)
	QPageSize__A5                 QPageSize__PageSizeId = QPageSize__PageSizeId(9)
	QPageSize__A6                 QPageSize__PageSizeId = QPageSize__PageSizeId(10)
	QPageSize__A7                 QPageSize__PageSizeId = QPageSize__PageSizeId(11)
	QPageSize__A8                 QPageSize__PageSizeId = QPageSize__PageSizeId(12)
	QPageSize__A9                 QPageSize__PageSizeId = QPageSize__PageSizeId(13)
	QPageSize__B0                 QPageSize__PageSizeId = QPageSize__PageSizeId(14)
	QPageSize__B1                 QPageSize__PageSizeId = QPageSize__PageSizeId(15)
	QPageSize__B10                QPageSize__PageSizeId = QPageSize__PageSizeId(16)
	QPageSize__B2                 QPageSize__PageSizeId = QPageSize__PageSizeId(17)
	QPageSize__B3                 QPageSize__PageSizeId = QPageSize__PageSizeId(18)
	QPageSize__B4                 QPageSize__PageSizeId = QPageSize__PageSizeId(19)
	QPageSize__B6                 QPageSize__PageSizeId = QPageSize__PageSizeId(20)
	QPageSize__B7                 QPageSize__PageSizeId = QPageSize__PageSizeId(21)
	QPageSize__B8                 QPageSize__PageSizeId = QPageSize__PageSizeId(22)
	QPageSize__B9                 QPageSize__PageSizeId = QPageSize__PageSizeId(23)
	QPageSize__C5E                QPageSize__PageSizeId = QPageSize__PageSizeId(24)
	QPageSize__Comm10E            QPageSize__PageSizeId = QPageSize__PageSizeId(25)
	QPageSize__DLE                QPageSize__PageSizeId = QPageSize__PageSizeId(26)
	QPageSize__Folio              QPageSize__PageSizeId = QPageSize__PageSizeId(27)
	QPageSize__Ledger             QPageSize__PageSizeId = QPageSize__PageSizeId(28)
	QPageSize__Tabloid            QPageSize__PageSizeId = QPageSize__PageSizeId(29)
	QPageSize__Custom             QPageSize__PageSizeId = QPageSize__PageSizeId(30)
	QPageSize__A10                QPageSize__PageSizeId = QPageSize__PageSizeId(31)
	QPageSize__A3Extra            QPageSize__PageSizeId = QPageSize__PageSizeId(32)
	QPageSize__A4Extra            QPageSize__PageSizeId = QPageSize__PageSizeId(33)
	QPageSize__A4Plus             QPageSize__PageSizeId = QPageSize__PageSizeId(34)
	QPageSize__A4Small            QPageSize__PageSizeId = QPageSize__PageSizeId(35)
	QPageSize__A5Extra            QPageSize__PageSizeId = QPageSize__PageSizeId(36)
	QPageSize__B5Extra            QPageSize__PageSizeId = QPageSize__PageSizeId(37)
	QPageSize__JisB0              QPageSize__PageSizeId = QPageSize__PageSizeId(38)
	QPageSize__JisB1              QPageSize__PageSizeId = QPageSize__PageSizeId(39)
	QPageSize__JisB2              QPageSize__PageSizeId = QPageSize__PageSizeId(40)
	QPageSize__JisB3              QPageSize__PageSizeId = QPageSize__PageSizeId(41)
	QPageSize__JisB4              QPageSize__PageSizeId = QPageSize__PageSizeId(42)
	QPageSize__JisB5              QPageSize__PageSizeId = QPageSize__PageSizeId(43)
	QPageSize__JisB6              QPageSize__PageSizeId = QPageSize__PageSizeId(44)
	QPageSize__JisB7              QPageSize__PageSizeId = QPageSize__PageSizeId(45)
	QPageSize__JisB8              QPageSize__PageSizeId = QPageSize__PageSizeId(46)
	QPageSize__JisB9              QPageSize__PageSizeId = QPageSize__PageSizeId(47)
	QPageSize__JisB10             QPageSize__PageSizeId = QPageSize__PageSizeId(48)
	QPageSize__AnsiC              QPageSize__PageSizeId = QPageSize__PageSizeId(49)
	QPageSize__AnsiD              QPageSize__PageSizeId = QPageSize__PageSizeId(50)
	QPageSize__AnsiE              QPageSize__PageSizeId = QPageSize__PageSizeId(51)
	QPageSize__LegalExtra         QPageSize__PageSizeId = QPageSize__PageSizeId(52)
	QPageSize__LetterExtra        QPageSize__PageSizeId = QPageSize__PageSizeId(53)
	QPageSize__LetterPlus         QPageSize__PageSizeId = QPageSize__PageSizeId(54)
	QPageSize__LetterSmall        QPageSize__PageSizeId = QPageSize__PageSizeId(55)
	QPageSize__TabloidExtra       QPageSize__PageSizeId = QPageSize__PageSizeId(56)
	QPageSize__ArchA              QPageSize__PageSizeId = QPageSize__PageSizeId(57)
	QPageSize__ArchB              QPageSize__PageSizeId = QPageSize__PageSizeId(58)
	QPageSize__ArchC              QPageSize__PageSizeId = QPageSize__PageSizeId(59)
	QPageSize__ArchD              QPageSize__PageSizeId = QPageSize__PageSizeId(60)
	QPageSize__ArchE              QPageSize__PageSizeId = QPageSize__PageSizeId(61)
	QPageSize__Imperial7x9        QPageSize__PageSizeId = QPageSize__PageSizeId(62)
	QPageSize__Imperial8x10       QPageSize__PageSizeId = QPageSize__PageSizeId(63)
	QPageSize__Imperial9x11       QPageSize__PageSizeId = QPageSize__PageSizeId(64)
	QPageSize__Imperial9x12       QPageSize__PageSizeId = QPageSize__PageSizeId(65)
	QPageSize__Imperial10x11      QPageSize__PageSizeId = QPageSize__PageSizeId(66)
	QPageSize__Imperial10x13      QPageSize__PageSizeId = QPageSize__PageSizeId(67)
	QPageSize__Imperial10x14      QPageSize__PageSizeId = QPageSize__PageSizeId(68)
	QPageSize__Imperial12x11      QPageSize__PageSizeId = QPageSize__PageSizeId(69)
	QPageSize__Imperial15x11      QPageSize__PageSizeId = QPageSize__PageSizeId(70)
	QPageSize__ExecutiveStandard  QPageSize__PageSizeId = QPageSize__PageSizeId(71)
	QPageSize__Note               QPageSize__PageSizeId = QPageSize__PageSizeId(72)
	QPageSize__Quarto             QPageSize__PageSizeId = QPageSize__PageSizeId(73)
	QPageSize__Statement          QPageSize__PageSizeId = QPageSize__PageSizeId(74)
	QPageSize__SuperA             QPageSize__PageSizeId = QPageSize__PageSizeId(75)
	QPageSize__SuperB             QPageSize__PageSizeId = QPageSize__PageSizeId(76)
	QPageSize__Postcard           QPageSize__PageSizeId = QPageSize__PageSizeId(77)
	QPageSize__DoublePostcard     QPageSize__PageSizeId = QPageSize__PageSizeId(78)
	QPageSize__Prc16K             QPageSize__PageSizeId = QPageSize__PageSizeId(79)
	QPageSize__Prc32K             QPageSize__PageSizeId = QPageSize__PageSizeId(80)
	QPageSize__Prc32KBig          QPageSize__PageSizeId = QPageSize__PageSizeId(81)
	QPageSize__FanFoldUS          QPageSize__PageSizeId = QPageSize__PageSizeId(82)
	QPageSize__FanFoldGerman      QPageSize__PageSizeId = QPageSize__PageSizeId(83)
	QPageSize__FanFoldGermanLegal QPageSize__PageSizeId = QPageSize__PageSizeId(84)
	QPageSize__EnvelopeB4         QPageSize__PageSizeId = QPageSize__PageSizeId(85)
	QPageSize__EnvelopeB5         QPageSize__PageSizeId = QPageSize__PageSizeId(86)
	QPageSize__EnvelopeB6         QPageSize__PageSizeId = QPageSize__PageSizeId(87)
	QPageSize__EnvelopeC0         QPageSize__PageSizeId = QPageSize__PageSizeId(88)
	QPageSize__EnvelopeC1         QPageSize__PageSizeId = QPageSize__PageSizeId(89)
	QPageSize__EnvelopeC2         QPageSize__PageSizeId = QPageSize__PageSizeId(90)
	QPageSize__EnvelopeC3         QPageSize__PageSizeId = QPageSize__PageSizeId(91)
	QPageSize__EnvelopeC4         QPageSize__PageSizeId = QPageSize__PageSizeId(92)
	QPageSize__EnvelopeC6         QPageSize__PageSizeId = QPageSize__PageSizeId(93)
	QPageSize__EnvelopeC65        QPageSize__PageSizeId = QPageSize__PageSizeId(94)
	QPageSize__EnvelopeC7         QPageSize__PageSizeId = QPageSize__PageSizeId(95)
	QPageSize__Envelope9          QPageSize__PageSizeId = QPageSize__PageSizeId(96)
	QPageSize__Envelope11         QPageSize__PageSizeId = QPageSize__PageSizeId(97)
	QPageSize__Envelope12         QPageSize__PageSizeId = QPageSize__PageSizeId(98)
	QPageSize__Envelope14         QPageSize__PageSizeId = QPageSize__PageSizeId(99)
	QPageSize__EnvelopeMonarch    QPageSize__PageSizeId = QPageSize__PageSizeId(100)
	QPageSize__EnvelopePersonal   QPageSize__PageSizeId = QPageSize__PageSizeId(101)
	QPageSize__EnvelopeChou3      QPageSize__PageSizeId = QPageSize__PageSizeId(102)
	QPageSize__EnvelopeChou4      QPageSize__PageSizeId = QPageSize__PageSizeId(103)
	QPageSize__EnvelopeInvite     QPageSize__PageSizeId = QPageSize__PageSizeId(104)
	QPageSize__EnvelopeItalian    QPageSize__PageSizeId = QPageSize__PageSizeId(105)
	QPageSize__EnvelopeKaku2      QPageSize__PageSizeId = QPageSize__PageSizeId(106)
	QPageSize__EnvelopeKaku3      QPageSize__PageSizeId = QPageSize__PageSizeId(107)
	QPageSize__EnvelopePrc1       QPageSize__PageSizeId = QPageSize__PageSizeId(108)
	QPageSize__EnvelopePrc2       QPageSize__PageSizeId = QPageSize__PageSizeId(109)
	QPageSize__EnvelopePrc3       QPageSize__PageSizeId = QPageSize__PageSizeId(110)
	QPageSize__EnvelopePrc4       QPageSize__PageSizeId = QPageSize__PageSizeId(111)
	QPageSize__EnvelopePrc5       QPageSize__PageSizeId = QPageSize__PageSizeId(112)
	QPageSize__EnvelopePrc6       QPageSize__PageSizeId = QPageSize__PageSizeId(113)
	QPageSize__EnvelopePrc7       QPageSize__PageSizeId = QPageSize__PageSizeId(114)
	QPageSize__EnvelopePrc8       QPageSize__PageSizeId = QPageSize__PageSizeId(115)
	QPageSize__EnvelopePrc9       QPageSize__PageSizeId = QPageSize__PageSizeId(116)
	QPageSize__EnvelopePrc10      QPageSize__PageSizeId = QPageSize__PageSizeId(117)
	QPageSize__EnvelopeYou4       QPageSize__PageSizeId = QPageSize__PageSizeId(118)
	QPageSize__LastPageSize       QPageSize__PageSizeId = QPageSize__PageSizeId(QPageSize__EnvelopeYou4)
	QPageSize__NPageSize          QPageSize__PageSizeId = QPageSize__PageSizeId(QPageSize__LastPageSize)
	QPageSize__NPaperSize         QPageSize__PageSizeId = QPageSize__PageSizeId(QPageSize__LastPageSize)
	QPageSize__AnsiA              QPageSize__PageSizeId = QPageSize__PageSizeId(QPageSize__Letter)
	QPageSize__AnsiB              QPageSize__PageSizeId = QPageSize__PageSizeId(QPageSize__Ledger)
	QPageSize__EnvelopeC5         QPageSize__PageSizeId = QPageSize__PageSizeId(QPageSize__C5E)
	QPageSize__EnvelopeDL         QPageSize__PageSizeId = QPageSize__PageSizeId(QPageSize__DLE)
	QPageSize__Envelope10         QPageSize__PageSizeId = QPageSize__PageSizeId(QPageSize__Comm10E)
)

//go:generate stringer -type=QPageSize__Unit
//QPageSize::Unit
type QPageSize__Unit int64

const (
	QPageSize__Millimeter QPageSize__Unit = QPageSize__Unit(0)
	QPageSize__Point      QPageSize__Unit = QPageSize__Unit(1)
	QPageSize__Inch       QPageSize__Unit = QPageSize__Unit(2)
	QPageSize__Pica       QPageSize__Unit = QPageSize__Unit(3)
	QPageSize__Didot      QPageSize__Unit = QPageSize__Unit(4)
	QPageSize__Cicero     QPageSize__Unit = QPageSize__Unit(5)
)

//go:generate stringer -type=QPageSize__SizeMatchPolicy
//QPageSize::SizeMatchPolicy
type QPageSize__SizeMatchPolicy int64

const (
	QPageSize__FuzzyMatch            QPageSize__SizeMatchPolicy = QPageSize__SizeMatchPolicy(0)
	QPageSize__FuzzyOrientationMatch QPageSize__SizeMatchPolicy = QPageSize__SizeMatchPolicy(1)
	QPageSize__ExactMatch            QPageSize__SizeMatchPolicy = QPageSize__SizeMatchPolicy(2)
)

func NewQPageSize() *QPageSize {
	tmpValue := NewQPageSizeFromPointer(C.QPageSize_NewQPageSize())
	qt.SetFinalizer(tmpValue, (*QPageSize).DestroyQPageSize)
	return tmpValue
}

func NewQPageSize2(pageSize QPageSize__PageSizeId) *QPageSize {
	tmpValue := NewQPageSizeFromPointer(C.QPageSize_NewQPageSize2(C.longlong(pageSize)))
	qt.SetFinalizer(tmpValue, (*QPageSize).DestroyQPageSize)
	return tmpValue
}

func NewQPageSize3(pointSize core.QSize_ITF, name string, matchPolicy QPageSize__SizeMatchPolicy) *QPageSize {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	tmpValue := NewQPageSizeFromPointer(C.QPageSize_NewQPageSize3(core.PointerFromQSize(pointSize), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}, C.longlong(matchPolicy)))
	qt.SetFinalizer(tmpValue, (*QPageSize).DestroyQPageSize)
	return tmpValue
}

func NewQPageSize4(size core.QSizeF_ITF, units QPageSize__Unit, name string, matchPolicy QPageSize__SizeMatchPolicy) *QPageSize {
	var nameC *C.char
	if name != "" {
		nameC = C.CString(name)
		defer C.free(unsafe.Pointer(nameC))
	}
	tmpValue := NewQPageSizeFromPointer(C.QPageSize_NewQPageSize4(core.PointerFromQSizeF(size), C.longlong(units), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}, C.longlong(matchPolicy)))
	qt.SetFinalizer(tmpValue, (*QPageSize).DestroyQPageSize)
	return tmpValue
}

func NewQPageSize5(other QPageSize_ITF) *QPageSize {
	tmpValue := NewQPageSizeFromPointer(C.QPageSize_NewQPageSize5(PointerFromQPageSize(other)))
	qt.SetFinalizer(tmpValue, (*QPageSize).DestroyQPageSize)
	return tmpValue
}

func (ptr *QPageSize) DefinitionSize() *core.QSizeF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFFromPointer(C.QPageSize_DefinitionSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

func QPageSize_DefinitionSize2(pageSizeId QPageSize__PageSizeId) *core.QSizeF {
	tmpValue := core.NewQSizeFFromPointer(C.QPageSize_QPageSize_DefinitionSize2(C.longlong(pageSizeId)))
	qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
	return tmpValue
}

func (ptr *QPageSize) DefinitionSize2(pageSizeId QPageSize__PageSizeId) *core.QSizeF {
	tmpValue := core.NewQSizeFFromPointer(C.QPageSize_QPageSize_DefinitionSize2(C.longlong(pageSizeId)))
	qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
	return tmpValue
}

func (ptr *QPageSize) DefinitionUnits() QPageSize__Unit {
	if ptr.Pointer() != nil {
		return QPageSize__Unit(C.QPageSize_DefinitionUnits(ptr.Pointer()))
	}
	return 0
}

func QPageSize_DefinitionUnits2(pageSizeId QPageSize__PageSizeId) QPageSize__Unit {
	return QPageSize__Unit(C.QPageSize_QPageSize_DefinitionUnits2(C.longlong(pageSizeId)))
}

func (ptr *QPageSize) DefinitionUnits2(pageSizeId QPageSize__PageSizeId) QPageSize__Unit {
	return QPageSize__Unit(C.QPageSize_QPageSize_DefinitionUnits2(C.longlong(pageSizeId)))
}

func (ptr *QPageSize) Id() QPageSize__PageSizeId {
	if ptr.Pointer() != nil {
		return QPageSize__PageSizeId(C.QPageSize_Id(ptr.Pointer()))
	}
	return 0
}

func QPageSize_Id2(pointSize core.QSize_ITF, matchPolicy QPageSize__SizeMatchPolicy) QPageSize__PageSizeId {
	return QPageSize__PageSizeId(C.QPageSize_QPageSize_Id2(core.PointerFromQSize(pointSize), C.longlong(matchPolicy)))
}

func (ptr *QPageSize) Id2(pointSize core.QSize_ITF, matchPolicy QPageSize__SizeMatchPolicy) QPageSize__PageSizeId {
	return QPageSize__PageSizeId(C.QPageSize_QPageSize_Id2(core.PointerFromQSize(pointSize), C.longlong(matchPolicy)))
}

func QPageSize_Id3(size core.QSizeF_ITF, units QPageSize__Unit, matchPolicy QPageSize__SizeMatchPolicy) QPageSize__PageSizeId {
	return QPageSize__PageSizeId(C.QPageSize_QPageSize_Id3(core.PointerFromQSizeF(size), C.longlong(units), C.longlong(matchPolicy)))
}

func (ptr *QPageSize) Id3(size core.QSizeF_ITF, units QPageSize__Unit, matchPolicy QPageSize__SizeMatchPolicy) QPageSize__PageSizeId {
	return QPageSize__PageSizeId(C.QPageSize_QPageSize_Id3(core.PointerFromQSizeF(size), C.longlong(units), C.longlong(matchPolicy)))
}

func QPageSize_Id4(windowsId int) QPageSize__PageSizeId {
	return QPageSize__PageSizeId(C.QPageSize_QPageSize_Id4(C.int(int32(windowsId))))
}

func (ptr *QPageSize) Id4(windowsId int) QPageSize__PageSizeId {
	return QPageSize__PageSizeId(C.QPageSize_QPageSize_Id4(C.int(int32(windowsId))))
}

func (ptr *QPageSize) IsEquivalentTo(other QPageSize_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPageSize_IsEquivalentTo(ptr.Pointer(), PointerFromQPageSize(other))) != 0
	}
	return false
}

func (ptr *QPageSize) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPageSize_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPageSize) Key() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPageSize_Key(ptr.Pointer()))
	}
	return ""
}

func QPageSize_Key2(pageSizeId QPageSize__PageSizeId) string {
	return cGoUnpackString(C.QPageSize_QPageSize_Key2(C.longlong(pageSizeId)))
}

func (ptr *QPageSize) Key2(pageSizeId QPageSize__PageSizeId) string {
	return cGoUnpackString(C.QPageSize_QPageSize_Key2(C.longlong(pageSizeId)))
}

func (ptr *QPageSize) Name() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPageSize_Name(ptr.Pointer()))
	}
	return ""
}

func QPageSize_Name2(pageSizeId QPageSize__PageSizeId) string {
	return cGoUnpackString(C.QPageSize_QPageSize_Name2(C.longlong(pageSizeId)))
}

func (ptr *QPageSize) Name2(pageSizeId QPageSize__PageSizeId) string {
	return cGoUnpackString(C.QPageSize_QPageSize_Name2(C.longlong(pageSizeId)))
}

func (ptr *QPageSize) Rect(units QPageSize__Unit) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QPageSize_Rect(ptr.Pointer(), C.longlong(units)))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPageSize) RectPixels(resolution int) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPageSize_RectPixels(ptr.Pointer(), C.int(int32(resolution))))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPageSize) RectPoints() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPageSize_RectPoints(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPageSize) Size(units QPageSize__Unit) *core.QSizeF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFFromPointer(C.QPageSize_Size(ptr.Pointer(), C.longlong(units)))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

func QPageSize_Size2(pageSizeId QPageSize__PageSizeId, units QPageSize__Unit) *core.QSizeF {
	tmpValue := core.NewQSizeFFromPointer(C.QPageSize_QPageSize_Size2(C.longlong(pageSizeId), C.longlong(units)))
	qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
	return tmpValue
}

func (ptr *QPageSize) Size2(pageSizeId QPageSize__PageSizeId, units QPageSize__Unit) *core.QSizeF {
	tmpValue := core.NewQSizeFFromPointer(C.QPageSize_QPageSize_Size2(C.longlong(pageSizeId), C.longlong(units)))
	qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
	return tmpValue
}

func (ptr *QPageSize) SizePixels(resolution int) *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QPageSize_SizePixels(ptr.Pointer(), C.int(int32(resolution))))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func QPageSize_SizePixels2(pageSizeId QPageSize__PageSizeId, resolution int) *core.QSize {
	tmpValue := core.NewQSizeFromPointer(C.QPageSize_QPageSize_SizePixels2(C.longlong(pageSizeId), C.int(int32(resolution))))
	qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
	return tmpValue
}

func (ptr *QPageSize) SizePixels2(pageSizeId QPageSize__PageSizeId, resolution int) *core.QSize {
	tmpValue := core.NewQSizeFromPointer(C.QPageSize_QPageSize_SizePixels2(C.longlong(pageSizeId), C.int(int32(resolution))))
	qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
	return tmpValue
}

func (ptr *QPageSize) SizePoints() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QPageSize_SizePoints(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func QPageSize_SizePoints2(pageSizeId QPageSize__PageSizeId) *core.QSize {
	tmpValue := core.NewQSizeFromPointer(C.QPageSize_QPageSize_SizePoints2(C.longlong(pageSizeId)))
	qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
	return tmpValue
}

func (ptr *QPageSize) SizePoints2(pageSizeId QPageSize__PageSizeId) *core.QSize {
	tmpValue := core.NewQSizeFromPointer(C.QPageSize_QPageSize_SizePoints2(C.longlong(pageSizeId)))
	qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
	return tmpValue
}

func (ptr *QPageSize) Swap(other QPageSize_ITF) {
	if ptr.Pointer() != nil {
		C.QPageSize_Swap(ptr.Pointer(), PointerFromQPageSize(other))
	}
}

func (ptr *QPageSize) WindowsId() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPageSize_WindowsId(ptr.Pointer())))
	}
	return 0
}

func QPageSize_WindowsId2(pageSizeId QPageSize__PageSizeId) int {
	return int(int32(C.QPageSize_QPageSize_WindowsId2(C.longlong(pageSizeId))))
}

func (ptr *QPageSize) WindowsId2(pageSizeId QPageSize__PageSizeId) int {
	return int(int32(C.QPageSize_QPageSize_WindowsId2(C.longlong(pageSizeId))))
}

func (ptr *QPageSize) DestroyQPageSize() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPageSize_DestroyQPageSize(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QPagedPaintDevice struct {
	QPaintDevice
}

type QPagedPaintDevice_ITF interface {
	QPaintDevice_ITF
	QPagedPaintDevice_PTR() *QPagedPaintDevice
}

func (ptr *QPagedPaintDevice) QPagedPaintDevice_PTR() *QPagedPaintDevice {
	return ptr
}

func (ptr *QPagedPaintDevice) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintDevice_PTR().Pointer()
	}
	return nil
}

func (ptr *QPagedPaintDevice) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QPaintDevice_PTR().SetPointer(p)
	}
}

func PointerFromQPagedPaintDevice(ptr QPagedPaintDevice_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPagedPaintDevice_PTR().Pointer()
	}
	return nil
}

func NewQPagedPaintDeviceFromPointer(ptr unsafe.Pointer) (n *QPagedPaintDevice) {
	n = new(QPagedPaintDevice)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QPagedPaintDevice__PageSize
//QPagedPaintDevice::PageSize
type QPagedPaintDevice__PageSize int64

const (
	QPagedPaintDevice__A4                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(0)
	QPagedPaintDevice__B5                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(1)
	QPagedPaintDevice__Letter             QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(2)
	QPagedPaintDevice__Legal              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(3)
	QPagedPaintDevice__Executive          QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(4)
	QPagedPaintDevice__A0                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(5)
	QPagedPaintDevice__A1                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(6)
	QPagedPaintDevice__A2                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(7)
	QPagedPaintDevice__A3                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(8)
	QPagedPaintDevice__A5                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(9)
	QPagedPaintDevice__A6                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(10)
	QPagedPaintDevice__A7                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(11)
	QPagedPaintDevice__A8                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(12)
	QPagedPaintDevice__A9                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(13)
	QPagedPaintDevice__B0                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(14)
	QPagedPaintDevice__B1                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(15)
	QPagedPaintDevice__B10                QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(16)
	QPagedPaintDevice__B2                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(17)
	QPagedPaintDevice__B3                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(18)
	QPagedPaintDevice__B4                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(19)
	QPagedPaintDevice__B6                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(20)
	QPagedPaintDevice__B7                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(21)
	QPagedPaintDevice__B8                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(22)
	QPagedPaintDevice__B9                 QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(23)
	QPagedPaintDevice__C5E                QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(24)
	QPagedPaintDevice__Comm10E            QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(25)
	QPagedPaintDevice__DLE                QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(26)
	QPagedPaintDevice__Folio              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(27)
	QPagedPaintDevice__Ledger             QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(28)
	QPagedPaintDevice__Tabloid            QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(29)
	QPagedPaintDevice__Custom             QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(30)
	QPagedPaintDevice__A10                QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(31)
	QPagedPaintDevice__A3Extra            QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(32)
	QPagedPaintDevice__A4Extra            QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(33)
	QPagedPaintDevice__A4Plus             QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(34)
	QPagedPaintDevice__A4Small            QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(35)
	QPagedPaintDevice__A5Extra            QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(36)
	QPagedPaintDevice__B5Extra            QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(37)
	QPagedPaintDevice__JisB0              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(38)
	QPagedPaintDevice__JisB1              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(39)
	QPagedPaintDevice__JisB2              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(40)
	QPagedPaintDevice__JisB3              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(41)
	QPagedPaintDevice__JisB4              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(42)
	QPagedPaintDevice__JisB5              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(43)
	QPagedPaintDevice__JisB6              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(44)
	QPagedPaintDevice__JisB7              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(45)
	QPagedPaintDevice__JisB8              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(46)
	QPagedPaintDevice__JisB9              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(47)
	QPagedPaintDevice__JisB10             QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(48)
	QPagedPaintDevice__AnsiC              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(49)
	QPagedPaintDevice__AnsiD              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(50)
	QPagedPaintDevice__AnsiE              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(51)
	QPagedPaintDevice__LegalExtra         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(52)
	QPagedPaintDevice__LetterExtra        QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(53)
	QPagedPaintDevice__LetterPlus         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(54)
	QPagedPaintDevice__LetterSmall        QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(55)
	QPagedPaintDevice__TabloidExtra       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(56)
	QPagedPaintDevice__ArchA              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(57)
	QPagedPaintDevice__ArchB              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(58)
	QPagedPaintDevice__ArchC              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(59)
	QPagedPaintDevice__ArchD              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(60)
	QPagedPaintDevice__ArchE              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(61)
	QPagedPaintDevice__Imperial7x9        QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(62)
	QPagedPaintDevice__Imperial8x10       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(63)
	QPagedPaintDevice__Imperial9x11       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(64)
	QPagedPaintDevice__Imperial9x12       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(65)
	QPagedPaintDevice__Imperial10x11      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(66)
	QPagedPaintDevice__Imperial10x13      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(67)
	QPagedPaintDevice__Imperial10x14      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(68)
	QPagedPaintDevice__Imperial12x11      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(69)
	QPagedPaintDevice__Imperial15x11      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(70)
	QPagedPaintDevice__ExecutiveStandard  QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(71)
	QPagedPaintDevice__Note               QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(72)
	QPagedPaintDevice__Quarto             QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(73)
	QPagedPaintDevice__Statement          QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(74)
	QPagedPaintDevice__SuperA             QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(75)
	QPagedPaintDevice__SuperB             QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(76)
	QPagedPaintDevice__Postcard           QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(77)
	QPagedPaintDevice__DoublePostcard     QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(78)
	QPagedPaintDevice__Prc16K             QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(79)
	QPagedPaintDevice__Prc32K             QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(80)
	QPagedPaintDevice__Prc32KBig          QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(81)
	QPagedPaintDevice__FanFoldUS          QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(82)
	QPagedPaintDevice__FanFoldGerman      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(83)
	QPagedPaintDevice__FanFoldGermanLegal QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(84)
	QPagedPaintDevice__EnvelopeB4         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(85)
	QPagedPaintDevice__EnvelopeB5         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(86)
	QPagedPaintDevice__EnvelopeB6         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(87)
	QPagedPaintDevice__EnvelopeC0         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(88)
	QPagedPaintDevice__EnvelopeC1         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(89)
	QPagedPaintDevice__EnvelopeC2         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(90)
	QPagedPaintDevice__EnvelopeC3         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(91)
	QPagedPaintDevice__EnvelopeC4         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(92)
	QPagedPaintDevice__EnvelopeC6         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(93)
	QPagedPaintDevice__EnvelopeC65        QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(94)
	QPagedPaintDevice__EnvelopeC7         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(95)
	QPagedPaintDevice__Envelope9          QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(96)
	QPagedPaintDevice__Envelope11         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(97)
	QPagedPaintDevice__Envelope12         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(98)
	QPagedPaintDevice__Envelope14         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(99)
	QPagedPaintDevice__EnvelopeMonarch    QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(100)
	QPagedPaintDevice__EnvelopePersonal   QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(101)
	QPagedPaintDevice__EnvelopeChou3      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(102)
	QPagedPaintDevice__EnvelopeChou4      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(103)
	QPagedPaintDevice__EnvelopeInvite     QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(104)
	QPagedPaintDevice__EnvelopeItalian    QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(105)
	QPagedPaintDevice__EnvelopeKaku2      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(106)
	QPagedPaintDevice__EnvelopeKaku3      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(107)
	QPagedPaintDevice__EnvelopePrc1       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(108)
	QPagedPaintDevice__EnvelopePrc2       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(109)
	QPagedPaintDevice__EnvelopePrc3       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(110)
	QPagedPaintDevice__EnvelopePrc4       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(111)
	QPagedPaintDevice__EnvelopePrc5       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(112)
	QPagedPaintDevice__EnvelopePrc6       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(113)
	QPagedPaintDevice__EnvelopePrc7       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(114)
	QPagedPaintDevice__EnvelopePrc8       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(115)
	QPagedPaintDevice__EnvelopePrc9       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(116)
	QPagedPaintDevice__EnvelopePrc10      QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(117)
	QPagedPaintDevice__EnvelopeYou4       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(118)
	QPagedPaintDevice__LastPageSize       QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(QPagedPaintDevice__EnvelopeYou4)
	QPagedPaintDevice__NPageSize          QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(QPagedPaintDevice__LastPageSize)
	QPagedPaintDevice__NPaperSize         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(QPagedPaintDevice__LastPageSize)
	QPagedPaintDevice__AnsiA              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(QPagedPaintDevice__Letter)
	QPagedPaintDevice__AnsiB              QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(QPagedPaintDevice__Ledger)
	QPagedPaintDevice__EnvelopeC5         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(QPagedPaintDevice__C5E)
	QPagedPaintDevice__EnvelopeDL         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(QPagedPaintDevice__DLE)
	QPagedPaintDevice__Envelope10         QPagedPaintDevice__PageSize = QPagedPaintDevice__PageSize(QPagedPaintDevice__Comm10E)
)

//go:generate stringer -type=QPagedPaintDevice__PdfVersion
//QPagedPaintDevice::PdfVersion
type QPagedPaintDevice__PdfVersion int64

const (
	QPagedPaintDevice__PdfVersion_1_4 QPagedPaintDevice__PdfVersion = QPagedPaintDevice__PdfVersion(0)
	QPagedPaintDevice__PdfVersion_A1b QPagedPaintDevice__PdfVersion = QPagedPaintDevice__PdfVersion(1)
	QPagedPaintDevice__PdfVersion_1_6 QPagedPaintDevice__PdfVersion = QPagedPaintDevice__PdfVersion(2)
)

//export callbackQPagedPaintDevice_NewPage
func callbackQPagedPaintDevice_NewPage(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "newPage"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QPagedPaintDevice) ConnectNewPage(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "newPage"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "newPage", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "newPage", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPagedPaintDevice) DisconnectNewPage() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "newPage")
	}
}

func (ptr *QPagedPaintDevice) NewPage() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPagedPaintDevice_NewPage(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPagedPaintDevice) PageLayout() *QPageLayout {
	if ptr.Pointer() != nil {
		tmpValue := NewQPageLayoutFromPointer(C.QPagedPaintDevice_PageLayout(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPageLayout).DestroyQPageLayout)
		return tmpValue
	}
	return nil
}

func (ptr *QPagedPaintDevice) PageSize() QPagedPaintDevice__PageSize {
	if ptr.Pointer() != nil {
		return QPagedPaintDevice__PageSize(C.QPagedPaintDevice_PageSize(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPagedPaintDevice) PageSizeMM() *core.QSizeF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFFromPointer(C.QPagedPaintDevice_PageSizeMM(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

func (ptr *QPagedPaintDevice) SetPageLayout(newPageLayout QPageLayout_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPagedPaintDevice_SetPageLayout(ptr.Pointer(), PointerFromQPageLayout(newPageLayout))) != 0
	}
	return false
}

func (ptr *QPagedPaintDevice) SetPageMargins(margins core.QMarginsF_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPagedPaintDevice_SetPageMargins(ptr.Pointer(), core.PointerFromQMarginsF(margins))) != 0
	}
	return false
}

func (ptr *QPagedPaintDevice) SetPageMargins2(margins core.QMarginsF_ITF, units QPageLayout__Unit) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPagedPaintDevice_SetPageMargins2(ptr.Pointer(), core.PointerFromQMarginsF(margins), C.longlong(units))) != 0
	}
	return false
}

func (ptr *QPagedPaintDevice) SetPageOrientation(orientation QPageLayout__Orientation) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPagedPaintDevice_SetPageOrientation(ptr.Pointer(), C.longlong(orientation))) != 0
	}
	return false
}

func (ptr *QPagedPaintDevice) SetPageSize(pageSize QPageSize_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPagedPaintDevice_SetPageSize(ptr.Pointer(), PointerFromQPageSize(pageSize))) != 0
	}
	return false
}

//export callbackQPagedPaintDevice_SetPageSize2
func callbackQPagedPaintDevice_SetPageSize2(ptr unsafe.Pointer, size C.longlong) {
	if signal := qt.GetSignal(ptr, "setPageSize2"); signal != nil {
		(*(*func(QPagedPaintDevice__PageSize))(signal))(QPagedPaintDevice__PageSize(size))
	} else {
		NewQPagedPaintDeviceFromPointer(ptr).SetPageSize2Default(QPagedPaintDevice__PageSize(size))
	}
}

func (ptr *QPagedPaintDevice) ConnectSetPageSize2(f func(size QPagedPaintDevice__PageSize)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setPageSize2"); signal != nil {
			f := func(size QPagedPaintDevice__PageSize) {
				(*(*func(QPagedPaintDevice__PageSize))(signal))(size)
				f(size)
			}
			qt.ConnectSignal(ptr.Pointer(), "setPageSize2", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setPageSize2", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPagedPaintDevice) DisconnectSetPageSize2() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setPageSize2")
	}
}

func (ptr *QPagedPaintDevice) SetPageSize2(size QPagedPaintDevice__PageSize) {
	if ptr.Pointer() != nil {
		C.QPagedPaintDevice_SetPageSize2(ptr.Pointer(), C.longlong(size))
	}
}

func (ptr *QPagedPaintDevice) SetPageSize2Default(size QPagedPaintDevice__PageSize) {
	if ptr.Pointer() != nil {
		C.QPagedPaintDevice_SetPageSize2Default(ptr.Pointer(), C.longlong(size))
	}
}

//export callbackQPagedPaintDevice_SetPageSizeMM
func callbackQPagedPaintDevice_SetPageSizeMM(ptr unsafe.Pointer, size unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "setPageSizeMM"); signal != nil {
		(*(*func(*core.QSizeF))(signal))(core.NewQSizeFFromPointer(size))
	} else {
		NewQPagedPaintDeviceFromPointer(ptr).SetPageSizeMMDefault(core.NewQSizeFFromPointer(size))
	}
}

func (ptr *QPagedPaintDevice) ConnectSetPageSizeMM(f func(size *core.QSizeF)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setPageSizeMM"); signal != nil {
			f := func(size *core.QSizeF) {
				(*(*func(*core.QSizeF))(signal))(size)
				f(size)
			}
			qt.ConnectSignal(ptr.Pointer(), "setPageSizeMM", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setPageSizeMM", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPagedPaintDevice) DisconnectSetPageSizeMM() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setPageSizeMM")
	}
}

func (ptr *QPagedPaintDevice) SetPageSizeMM(size core.QSizeF_ITF) {
	if ptr.Pointer() != nil {
		C.QPagedPaintDevice_SetPageSizeMM(ptr.Pointer(), core.PointerFromQSizeF(size))
	}
}

func (ptr *QPagedPaintDevice) SetPageSizeMMDefault(size core.QSizeF_ITF) {
	if ptr.Pointer() != nil {
		C.QPagedPaintDevice_SetPageSizeMMDefault(ptr.Pointer(), core.PointerFromQSizeF(size))
	}
}

//export callbackQPagedPaintDevice_DestroyQPagedPaintDevice
func callbackQPagedPaintDevice_DestroyQPagedPaintDevice(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QPagedPaintDevice"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQPagedPaintDeviceFromPointer(ptr).DestroyQPagedPaintDeviceDefault()
	}
}

func (ptr *QPagedPaintDevice) ConnectDestroyQPagedPaintDevice(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QPagedPaintDevice"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QPagedPaintDevice", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QPagedPaintDevice", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPagedPaintDevice) DisconnectDestroyQPagedPaintDevice() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QPagedPaintDevice")
	}
}

func (ptr *QPagedPaintDevice) DestroyQPagedPaintDevice() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPagedPaintDevice_DestroyQPagedPaintDevice(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPagedPaintDevice) DestroyQPagedPaintDeviceDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPagedPaintDevice_DestroyQPagedPaintDeviceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//export callbackQPagedPaintDevice_PaintEngine
func callbackQPagedPaintDevice_PaintEngine(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "paintEngine"); signal != nil {
		return PointerFromQPaintEngine((*(*func() *QPaintEngine)(signal))())
	}

	return PointerFromQPaintEngine(NewQPagedPaintDeviceFromPointer(ptr).PaintEngineDefault())
}

func (ptr *QPagedPaintDevice) PaintEngine() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPagedPaintDevice_PaintEngine(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPagedPaintDevice) PaintEngineDefault() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPagedPaintDevice_PaintEngineDefault(ptr.Pointer()))
	}
	return nil
}

type QPaintDevice struct {
	ptr unsafe.Pointer
}

type QPaintDevice_ITF interface {
	QPaintDevice_PTR() *QPaintDevice
}

func (ptr *QPaintDevice) QPaintDevice_PTR() *QPaintDevice {
	return ptr
}

func (ptr *QPaintDevice) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPaintDevice) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPaintDevice(ptr QPaintDevice_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintDevice_PTR().Pointer()
	}
	return nil
}

func NewQPaintDeviceFromPointer(ptr unsafe.Pointer) (n *QPaintDevice) {
	n = new(QPaintDevice)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QPaintDevice__PaintDeviceMetric
//QPaintDevice::PaintDeviceMetric
type QPaintDevice__PaintDeviceMetric int64

const (
	QPaintDevice__PdmWidth                  QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(1)
	QPaintDevice__PdmHeight                 QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(2)
	QPaintDevice__PdmWidthMM                QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(3)
	QPaintDevice__PdmHeightMM               QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(4)
	QPaintDevice__PdmNumColors              QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(5)
	QPaintDevice__PdmDepth                  QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(6)
	QPaintDevice__PdmDpiX                   QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(7)
	QPaintDevice__PdmDpiY                   QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(8)
	QPaintDevice__PdmPhysicalDpiX           QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(9)
	QPaintDevice__PdmPhysicalDpiY           QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(10)
	QPaintDevice__PdmDevicePixelRatio       QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(11)
	QPaintDevice__PdmDevicePixelRatioScaled QPaintDevice__PaintDeviceMetric = QPaintDevice__PaintDeviceMetric(12)
)

func NewQPaintDevice() *QPaintDevice {
	return NewQPaintDeviceFromPointer(C.QPaintDevice_NewQPaintDevice())
}

func (ptr *QPaintDevice) ColorCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_ColorCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPaintDevice) Depth() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_Depth(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPaintDevice) DevicePixelRatio() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_DevicePixelRatio(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPaintDevice) DevicePixelRatioF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPaintDevice_DevicePixelRatioF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPaintDevice) Height() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_Height(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPaintDevice) HeightMM() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_HeightMM(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPaintDevice) LogicalDpiX() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_LogicalDpiX(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPaintDevice) LogicalDpiY() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_LogicalDpiY(ptr.Pointer())))
	}
	return 0
}

//export callbackQPaintDevice_Metric
func callbackQPaintDevice_Metric(ptr unsafe.Pointer, metric C.longlong) C.int {
	if signal := qt.GetSignal(ptr, "metric"); signal != nil {
		return C.int(int32((*(*func(QPaintDevice__PaintDeviceMetric) int)(signal))(QPaintDevice__PaintDeviceMetric(metric))))
	}

	return C.int(int32(NewQPaintDeviceFromPointer(ptr).MetricDefault(QPaintDevice__PaintDeviceMetric(metric))))
}

func (ptr *QPaintDevice) ConnectMetric(f func(metric QPaintDevice__PaintDeviceMetric) int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "metric"); signal != nil {
			f := func(metric QPaintDevice__PaintDeviceMetric) int {
				(*(*func(QPaintDevice__PaintDeviceMetric) int)(signal))(metric)
				return f(metric)
			}
			qt.ConnectSignal(ptr.Pointer(), "metric", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "metric", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintDevice) DisconnectMetric() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "metric")
	}
}

func (ptr *QPaintDevice) Metric(metric QPaintDevice__PaintDeviceMetric) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_Metric(ptr.Pointer(), C.longlong(metric))))
	}
	return 0
}

func (ptr *QPaintDevice) MetricDefault(metric QPaintDevice__PaintDeviceMetric) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_MetricDefault(ptr.Pointer(), C.longlong(metric))))
	}
	return 0
}

//export callbackQPaintDevice_PaintEngine
func callbackQPaintDevice_PaintEngine(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "paintEngine"); signal != nil {
		return PointerFromQPaintEngine((*(*func() *QPaintEngine)(signal))())
	}

	return PointerFromQPaintEngine(NewQPaintEngine(0))
}

func (ptr *QPaintDevice) ConnectPaintEngine(f func() *QPaintEngine) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "paintEngine"); signal != nil {
			f := func() *QPaintEngine {
				(*(*func() *QPaintEngine)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "paintEngine", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "paintEngine", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintDevice) DisconnectPaintEngine() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "paintEngine")
	}
}

func (ptr *QPaintDevice) PaintEngine() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPaintDevice_PaintEngine(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPaintDevice) PaintingActive() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPaintDevice_PaintingActive(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPaintDevice) PhysicalDpiX() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_PhysicalDpiX(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPaintDevice) PhysicalDpiY() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_PhysicalDpiY(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPaintDevice) Width() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_Width(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPaintDevice) WidthMM() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPaintDevice_WidthMM(ptr.Pointer())))
	}
	return 0
}

//export callbackQPaintDevice_DestroyQPaintDevice
func callbackQPaintDevice_DestroyQPaintDevice(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QPaintDevice"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQPaintDeviceFromPointer(ptr).DestroyQPaintDeviceDefault()
	}
}

func (ptr *QPaintDevice) ConnectDestroyQPaintDevice(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QPaintDevice"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QPaintDevice", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QPaintDevice", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintDevice) DisconnectDestroyQPaintDevice() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QPaintDevice")
	}
}

func (ptr *QPaintDevice) DestroyQPaintDevice() {
	if ptr.Pointer() != nil {
		C.QPaintDevice_DestroyQPaintDevice(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPaintDevice) DestroyQPaintDeviceDefault() {
	if ptr.Pointer() != nil {
		C.QPaintDevice_DestroyQPaintDeviceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QPaintDeviceWindow struct {
	QWindow
	QPaintDevice
}

type QPaintDeviceWindow_ITF interface {
	QWindow_ITF
	QPaintDevice_ITF
	QPaintDeviceWindow_PTR() *QPaintDeviceWindow
}

func (ptr *QPaintDeviceWindow) QPaintDeviceWindow_PTR() *QPaintDeviceWindow {
	return ptr
}

func (ptr *QPaintDeviceWindow) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QWindow_PTR().Pointer()
	}
	return nil
}

func (ptr *QPaintDeviceWindow) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QWindow_PTR().SetPointer(p)
		ptr.QPaintDevice_PTR().SetPointer(p)
	}
}

func PointerFromQPaintDeviceWindow(ptr QPaintDeviceWindow_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintDeviceWindow_PTR().Pointer()
	}
	return nil
}

func NewQPaintDeviceWindowFromPointer(ptr unsafe.Pointer) (n *QPaintDeviceWindow) {
	n = new(QPaintDeviceWindow)
	n.SetPointer(ptr)
	return
}

//export callbackQPaintDeviceWindow_PaintEvent
func callbackQPaintDeviceWindow_PaintEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "paintEvent"); signal != nil {
		(*(*func(*QPaintEvent))(signal))(NewQPaintEventFromPointer(event))
	} else {
		NewQPaintDeviceWindowFromPointer(ptr).PaintEventDefault(NewQPaintEventFromPointer(event))
	}
}

func (ptr *QPaintDeviceWindow) ConnectPaintEvent(f func(event *QPaintEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "paintEvent"); signal != nil {
			f := func(event *QPaintEvent) {
				(*(*func(*QPaintEvent))(signal))(event)
				f(event)
			}
			qt.ConnectSignal(ptr.Pointer(), "paintEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "paintEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintDeviceWindow) DisconnectPaintEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "paintEvent")
	}
}

func (ptr *QPaintDeviceWindow) PaintEvent(event QPaintEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintDeviceWindow_PaintEvent(ptr.Pointer(), PointerFromQPaintEvent(event))
	}
}

func (ptr *QPaintDeviceWindow) PaintEventDefault(event QPaintEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintDeviceWindow_PaintEventDefault(ptr.Pointer(), PointerFromQPaintEvent(event))
	}
}

func (ptr *QPaintDeviceWindow) Update(rect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintDeviceWindow_Update(ptr.Pointer(), core.PointerFromQRect(rect))
	}
}

func (ptr *QPaintDeviceWindow) Update2(region QRegion_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintDeviceWindow_Update2(ptr.Pointer(), PointerFromQRegion(region))
	}
}

//export callbackQPaintDeviceWindow_Update3
func callbackQPaintDeviceWindow_Update3(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "update3"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQPaintDeviceWindowFromPointer(ptr).Update3Default()
	}
}

func (ptr *QPaintDeviceWindow) ConnectUpdate3(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "update3"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "update3", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "update3", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintDeviceWindow) DisconnectUpdate3() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "update3")
	}
}

func (ptr *QPaintDeviceWindow) Update3() {
	if ptr.Pointer() != nil {
		C.QPaintDeviceWindow_Update3(ptr.Pointer())
	}
}

func (ptr *QPaintDeviceWindow) Update3Default() {
	if ptr.Pointer() != nil {
		C.QPaintDeviceWindow_Update3Default(ptr.Pointer())
	}
}

//export callbackQPaintDeviceWindow_PaintEngine
func callbackQPaintDeviceWindow_PaintEngine(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "paintEngine"); signal != nil {
		return PointerFromQPaintEngine((*(*func() *QPaintEngine)(signal))())
	}

	return PointerFromQPaintEngine(NewQPaintDeviceWindowFromPointer(ptr).PaintEngineDefault())
}

func (ptr *QPaintDeviceWindow) PaintEngine() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPaintDeviceWindow_PaintEngine(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPaintDeviceWindow) PaintEngineDefault() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPaintDeviceWindow_PaintEngineDefault(ptr.Pointer()))
	}
	return nil
}

type QPaintEngine struct {
	ptr unsafe.Pointer
}

type QPaintEngine_ITF interface {
	QPaintEngine_PTR() *QPaintEngine
}

func (ptr *QPaintEngine) QPaintEngine_PTR() *QPaintEngine {
	return ptr
}

func (ptr *QPaintEngine) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPaintEngine) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPaintEngine(ptr QPaintEngine_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintEngine_PTR().Pointer()
	}
	return nil
}

func NewQPaintEngineFromPointer(ptr unsafe.Pointer) (n *QPaintEngine) {
	n = new(QPaintEngine)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QPaintEngine__PaintEngineFeature
//QPaintEngine::PaintEngineFeature
type QPaintEngine__PaintEngineFeature int64

const (
	QPaintEngine__PrimitiveTransform          QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000001)
	QPaintEngine__PatternTransform            QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000002)
	QPaintEngine__PixmapTransform             QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000004)
	QPaintEngine__PatternBrush                QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000008)
	QPaintEngine__LinearGradientFill          QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000010)
	QPaintEngine__RadialGradientFill          QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000020)
	QPaintEngine__ConicalGradientFill         QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000040)
	QPaintEngine__AlphaBlend                  QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000080)
	QPaintEngine__PorterDuff                  QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000100)
	QPaintEngine__PainterPaths                QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000200)
	QPaintEngine__Antialiasing                QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000400)
	QPaintEngine__BrushStroke                 QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00000800)
	QPaintEngine__ConstantOpacity             QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00001000)
	QPaintEngine__MaskedBrush                 QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00002000)
	QPaintEngine__PerspectiveTransform        QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00004000)
	QPaintEngine__BlendModes                  QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00008000)
	QPaintEngine__ObjectBoundingModeGradients QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00010000)
	QPaintEngine__RasterOpModes               QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x00020000)
	QPaintEngine__PaintOutsidePaintEvent      QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0x20000000)
	QPaintEngine__AllFeatures                 QPaintEngine__PaintEngineFeature = QPaintEngine__PaintEngineFeature(0xffffffff)
)

//go:generate stringer -type=QPaintEngine__DirtyFlag
//QPaintEngine::DirtyFlag
type QPaintEngine__DirtyFlag int64

const (
	QPaintEngine__DirtyPen             QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0001)
	QPaintEngine__DirtyBrush           QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0002)
	QPaintEngine__DirtyBrushOrigin     QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0004)
	QPaintEngine__DirtyFont            QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0008)
	QPaintEngine__DirtyBackground      QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0010)
	QPaintEngine__DirtyBackgroundMode  QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0020)
	QPaintEngine__DirtyTransform       QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0040)
	QPaintEngine__DirtyClipRegion      QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0080)
	QPaintEngine__DirtyClipPath        QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0100)
	QPaintEngine__DirtyHints           QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0200)
	QPaintEngine__DirtyCompositionMode QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0400)
	QPaintEngine__DirtyClipEnabled     QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x0800)
	QPaintEngine__DirtyOpacity         QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0x1000)
	QPaintEngine__AllDirty             QPaintEngine__DirtyFlag = QPaintEngine__DirtyFlag(0xffff)
)

//go:generate stringer -type=QPaintEngine__PolygonDrawMode
//QPaintEngine::PolygonDrawMode
type QPaintEngine__PolygonDrawMode int64

const (
	QPaintEngine__OddEvenMode  QPaintEngine__PolygonDrawMode = QPaintEngine__PolygonDrawMode(0)
	QPaintEngine__WindingMode  QPaintEngine__PolygonDrawMode = QPaintEngine__PolygonDrawMode(1)
	QPaintEngine__ConvexMode   QPaintEngine__PolygonDrawMode = QPaintEngine__PolygonDrawMode(2)
	QPaintEngine__PolylineMode QPaintEngine__PolygonDrawMode = QPaintEngine__PolygonDrawMode(3)
)

//go:generate stringer -type=QPaintEngine__Type
//QPaintEngine::Type
type QPaintEngine__Type int64

const (
	QPaintEngine__X11           QPaintEngine__Type = QPaintEngine__Type(0)
	QPaintEngine__Windows       QPaintEngine__Type = QPaintEngine__Type(1)
	QPaintEngine__QuickDraw     QPaintEngine__Type = QPaintEngine__Type(2)
	QPaintEngine__CoreGraphics  QPaintEngine__Type = QPaintEngine__Type(3)
	QPaintEngine__MacPrinter    QPaintEngine__Type = QPaintEngine__Type(4)
	QPaintEngine__QWindowSystem QPaintEngine__Type = QPaintEngine__Type(5)
	QPaintEngine__PostScript    QPaintEngine__Type = QPaintEngine__Type(6)
	QPaintEngine__OpenGL        QPaintEngine__Type = QPaintEngine__Type(7)
	QPaintEngine__Picture       QPaintEngine__Type = QPaintEngine__Type(8)
	QPaintEngine__SVG           QPaintEngine__Type = QPaintEngine__Type(9)
	QPaintEngine__Raster        QPaintEngine__Type = QPaintEngine__Type(10)
	QPaintEngine__Direct3D      QPaintEngine__Type = QPaintEngine__Type(11)
	QPaintEngine__Pdf           QPaintEngine__Type = QPaintEngine__Type(12)
	QPaintEngine__OpenVG        QPaintEngine__Type = QPaintEngine__Type(13)
	QPaintEngine__OpenGL2       QPaintEngine__Type = QPaintEngine__Type(14)
	QPaintEngine__PaintBuffer   QPaintEngine__Type = QPaintEngine__Type(15)
	QPaintEngine__Blitter       QPaintEngine__Type = QPaintEngine__Type(16)
	QPaintEngine__Direct2D      QPaintEngine__Type = QPaintEngine__Type(17)
	QPaintEngine__User          QPaintEngine__Type = QPaintEngine__Type(50)
	QPaintEngine__MaxUser       QPaintEngine__Type = QPaintEngine__Type(100)
)

func NewQPaintEngine(caps QPaintEngine__PaintEngineFeature) *QPaintEngine {
	return NewQPaintEngineFromPointer(C.QPaintEngine_NewQPaintEngine(C.longlong(caps)))
}

//export callbackQPaintEngine_Begin
func callbackQPaintEngine_Begin(ptr unsafe.Pointer, pdev unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "begin"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*QPaintDevice) bool)(signal))(NewQPaintDeviceFromPointer(pdev)))))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QPaintEngine) ConnectBegin(f func(pdev *QPaintDevice) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "begin"); signal != nil {
			f := func(pdev *QPaintDevice) bool {
				(*(*func(*QPaintDevice) bool)(signal))(pdev)
				return f(pdev)
			}
			qt.ConnectSignal(ptr.Pointer(), "begin", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "begin", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectBegin() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "begin")
	}
}

func (ptr *QPaintEngine) Begin(pdev QPaintDevice_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPaintEngine_Begin(ptr.Pointer(), PointerFromQPaintDevice(pdev))) != 0
	}
	return false
}

//export callbackQPaintEngine_DrawEllipse
func callbackQPaintEngine_DrawEllipse(ptr unsafe.Pointer, rect unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "drawEllipse"); signal != nil {
		(*(*func(*core.QRectF))(signal))(core.NewQRectFFromPointer(rect))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawEllipseDefault(core.NewQRectFFromPointer(rect))
	}
}

func (ptr *QPaintEngine) ConnectDrawEllipse(f func(rect *core.QRectF)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawEllipse"); signal != nil {
			f := func(rect *core.QRectF) {
				(*(*func(*core.QRectF))(signal))(rect)
				f(rect)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawEllipse", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawEllipse", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawEllipse() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawEllipse")
	}
}

func (ptr *QPaintEngine) DrawEllipse(rect core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawEllipse(ptr.Pointer(), core.PointerFromQRectF(rect))
	}
}

func (ptr *QPaintEngine) DrawEllipseDefault(rect core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawEllipseDefault(ptr.Pointer(), core.PointerFromQRectF(rect))
	}
}

//export callbackQPaintEngine_DrawEllipse2
func callbackQPaintEngine_DrawEllipse2(ptr unsafe.Pointer, rect unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "drawEllipse2"); signal != nil {
		(*(*func(*core.QRect))(signal))(core.NewQRectFromPointer(rect))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawEllipse2Default(core.NewQRectFromPointer(rect))
	}
}

func (ptr *QPaintEngine) ConnectDrawEllipse2(f func(rect *core.QRect)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawEllipse2"); signal != nil {
			f := func(rect *core.QRect) {
				(*(*func(*core.QRect))(signal))(rect)
				f(rect)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawEllipse2", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawEllipse2", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawEllipse2() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawEllipse2")
	}
}

func (ptr *QPaintEngine) DrawEllipse2(rect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawEllipse2(ptr.Pointer(), core.PointerFromQRect(rect))
	}
}

func (ptr *QPaintEngine) DrawEllipse2Default(rect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawEllipse2Default(ptr.Pointer(), core.PointerFromQRect(rect))
	}
}

//export callbackQPaintEngine_DrawImage
func callbackQPaintEngine_DrawImage(ptr unsafe.Pointer, rectangle unsafe.Pointer, image unsafe.Pointer, sr unsafe.Pointer, flags C.longlong) {
	if signal := qt.GetSignal(ptr, "drawImage"); signal != nil {
		(*(*func(*core.QRectF, *QImage, *core.QRectF, core.Qt__ImageConversionFlag))(signal))(core.NewQRectFFromPointer(rectangle), NewQImageFromPointer(image), core.NewQRectFFromPointer(sr), core.Qt__ImageConversionFlag(flags))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawImageDefault(core.NewQRectFFromPointer(rectangle), NewQImageFromPointer(image), core.NewQRectFFromPointer(sr), core.Qt__ImageConversionFlag(flags))
	}
}

func (ptr *QPaintEngine) ConnectDrawImage(f func(rectangle *core.QRectF, image *QImage, sr *core.QRectF, flags core.Qt__ImageConversionFlag)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawImage"); signal != nil {
			f := func(rectangle *core.QRectF, image *QImage, sr *core.QRectF, flags core.Qt__ImageConversionFlag) {
				(*(*func(*core.QRectF, *QImage, *core.QRectF, core.Qt__ImageConversionFlag))(signal))(rectangle, image, sr, flags)
				f(rectangle, image, sr, flags)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawImage", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawImage", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawImage() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawImage")
	}
}

func (ptr *QPaintEngine) DrawImage(rectangle core.QRectF_ITF, image QImage_ITF, sr core.QRectF_ITF, flags core.Qt__ImageConversionFlag) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawImage(ptr.Pointer(), core.PointerFromQRectF(rectangle), PointerFromQImage(image), core.PointerFromQRectF(sr), C.longlong(flags))
	}
}

func (ptr *QPaintEngine) DrawImageDefault(rectangle core.QRectF_ITF, image QImage_ITF, sr core.QRectF_ITF, flags core.Qt__ImageConversionFlag) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawImageDefault(ptr.Pointer(), core.PointerFromQRectF(rectangle), PointerFromQImage(image), core.PointerFromQRectF(sr), C.longlong(flags))
	}
}

//export callbackQPaintEngine_DrawLines
func callbackQPaintEngine_DrawLines(ptr unsafe.Pointer, lines unsafe.Pointer, lineCount C.int) {
	if signal := qt.GetSignal(ptr, "drawLines"); signal != nil {
		(*(*func(*core.QLineF, int))(signal))(core.NewQLineFFromPointer(lines), int(int32(lineCount)))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawLinesDefault(core.NewQLineFFromPointer(lines), int(int32(lineCount)))
	}
}

func (ptr *QPaintEngine) ConnectDrawLines(f func(lines *core.QLineF, lineCount int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawLines"); signal != nil {
			f := func(lines *core.QLineF, lineCount int) {
				(*(*func(*core.QLineF, int))(signal))(lines, lineCount)
				f(lines, lineCount)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawLines", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawLines", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawLines() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawLines")
	}
}

func (ptr *QPaintEngine) DrawLines(lines core.QLineF_ITF, lineCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawLines(ptr.Pointer(), core.PointerFromQLineF(lines), C.int(int32(lineCount)))
	}
}

func (ptr *QPaintEngine) DrawLinesDefault(lines core.QLineF_ITF, lineCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawLinesDefault(ptr.Pointer(), core.PointerFromQLineF(lines), C.int(int32(lineCount)))
	}
}

//export callbackQPaintEngine_DrawLines2
func callbackQPaintEngine_DrawLines2(ptr unsafe.Pointer, lines unsafe.Pointer, lineCount C.int) {
	if signal := qt.GetSignal(ptr, "drawLines2"); signal != nil {
		(*(*func(*core.QLine, int))(signal))(core.NewQLineFromPointer(lines), int(int32(lineCount)))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawLines2Default(core.NewQLineFromPointer(lines), int(int32(lineCount)))
	}
}

func (ptr *QPaintEngine) ConnectDrawLines2(f func(lines *core.QLine, lineCount int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawLines2"); signal != nil {
			f := func(lines *core.QLine, lineCount int) {
				(*(*func(*core.QLine, int))(signal))(lines, lineCount)
				f(lines, lineCount)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawLines2", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawLines2", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawLines2() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawLines2")
	}
}

func (ptr *QPaintEngine) DrawLines2(lines core.QLine_ITF, lineCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawLines2(ptr.Pointer(), core.PointerFromQLine(lines), C.int(int32(lineCount)))
	}
}

func (ptr *QPaintEngine) DrawLines2Default(lines core.QLine_ITF, lineCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawLines2Default(ptr.Pointer(), core.PointerFromQLine(lines), C.int(int32(lineCount)))
	}
}

//export callbackQPaintEngine_DrawPath
func callbackQPaintEngine_DrawPath(ptr unsafe.Pointer, path unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "drawPath"); signal != nil {
		(*(*func(*QPainterPath))(signal))(NewQPainterPathFromPointer(path))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawPathDefault(NewQPainterPathFromPointer(path))
	}
}

func (ptr *QPaintEngine) ConnectDrawPath(f func(path *QPainterPath)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawPath"); signal != nil {
			f := func(path *QPainterPath) {
				(*(*func(*QPainterPath))(signal))(path)
				f(path)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawPath", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawPath", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawPath() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawPath")
	}
}

func (ptr *QPaintEngine) DrawPath(path QPainterPath_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPath(ptr.Pointer(), PointerFromQPainterPath(path))
	}
}

func (ptr *QPaintEngine) DrawPathDefault(path QPainterPath_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPathDefault(ptr.Pointer(), PointerFromQPainterPath(path))
	}
}

//export callbackQPaintEngine_DrawPixmap
func callbackQPaintEngine_DrawPixmap(ptr unsafe.Pointer, r unsafe.Pointer, pm unsafe.Pointer, sr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "drawPixmap"); signal != nil {
		(*(*func(*core.QRectF, *QPixmap, *core.QRectF))(signal))(core.NewQRectFFromPointer(r), NewQPixmapFromPointer(pm), core.NewQRectFFromPointer(sr))
	}

}

func (ptr *QPaintEngine) ConnectDrawPixmap(f func(r *core.QRectF, pm *QPixmap, sr *core.QRectF)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawPixmap"); signal != nil {
			f := func(r *core.QRectF, pm *QPixmap, sr *core.QRectF) {
				(*(*func(*core.QRectF, *QPixmap, *core.QRectF))(signal))(r, pm, sr)
				f(r, pm, sr)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawPixmap", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawPixmap", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawPixmap() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawPixmap")
	}
}

func (ptr *QPaintEngine) DrawPixmap(r core.QRectF_ITF, pm QPixmap_ITF, sr core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPixmap(ptr.Pointer(), core.PointerFromQRectF(r), PointerFromQPixmap(pm), core.PointerFromQRectF(sr))
	}
}

//export callbackQPaintEngine_DrawPoints
func callbackQPaintEngine_DrawPoints(ptr unsafe.Pointer, points unsafe.Pointer, pointCount C.int) {
	if signal := qt.GetSignal(ptr, "drawPoints"); signal != nil {
		(*(*func(*core.QPointF, int))(signal))(core.NewQPointFFromPointer(points), int(int32(pointCount)))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawPointsDefault(core.NewQPointFFromPointer(points), int(int32(pointCount)))
	}
}

func (ptr *QPaintEngine) ConnectDrawPoints(f func(points *core.QPointF, pointCount int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawPoints"); signal != nil {
			f := func(points *core.QPointF, pointCount int) {
				(*(*func(*core.QPointF, int))(signal))(points, pointCount)
				f(points, pointCount)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawPoints", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawPoints", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawPoints() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawPoints")
	}
}

func (ptr *QPaintEngine) DrawPoints(points core.QPointF_ITF, pointCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPoints(ptr.Pointer(), core.PointerFromQPointF(points), C.int(int32(pointCount)))
	}
}

func (ptr *QPaintEngine) DrawPointsDefault(points core.QPointF_ITF, pointCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPointsDefault(ptr.Pointer(), core.PointerFromQPointF(points), C.int(int32(pointCount)))
	}
}

//export callbackQPaintEngine_DrawPoints2
func callbackQPaintEngine_DrawPoints2(ptr unsafe.Pointer, points unsafe.Pointer, pointCount C.int) {
	if signal := qt.GetSignal(ptr, "drawPoints2"); signal != nil {
		(*(*func(*core.QPoint, int))(signal))(core.NewQPointFromPointer(points), int(int32(pointCount)))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawPoints2Default(core.NewQPointFromPointer(points), int(int32(pointCount)))
	}
}

func (ptr *QPaintEngine) ConnectDrawPoints2(f func(points *core.QPoint, pointCount int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawPoints2"); signal != nil {
			f := func(points *core.QPoint, pointCount int) {
				(*(*func(*core.QPoint, int))(signal))(points, pointCount)
				f(points, pointCount)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawPoints2", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawPoints2", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawPoints2() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawPoints2")
	}
}

func (ptr *QPaintEngine) DrawPoints2(points core.QPoint_ITF, pointCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPoints2(ptr.Pointer(), core.PointerFromQPoint(points), C.int(int32(pointCount)))
	}
}

func (ptr *QPaintEngine) DrawPoints2Default(points core.QPoint_ITF, pointCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPoints2Default(ptr.Pointer(), core.PointerFromQPoint(points), C.int(int32(pointCount)))
	}
}

//export callbackQPaintEngine_DrawPolygon
func callbackQPaintEngine_DrawPolygon(ptr unsafe.Pointer, points unsafe.Pointer, pointCount C.int, mode C.longlong) {
	if signal := qt.GetSignal(ptr, "drawPolygon"); signal != nil {
		(*(*func(*core.QPointF, int, QPaintEngine__PolygonDrawMode))(signal))(core.NewQPointFFromPointer(points), int(int32(pointCount)), QPaintEngine__PolygonDrawMode(mode))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawPolygonDefault(core.NewQPointFFromPointer(points), int(int32(pointCount)), QPaintEngine__PolygonDrawMode(mode))
	}
}

func (ptr *QPaintEngine) ConnectDrawPolygon(f func(points *core.QPointF, pointCount int, mode QPaintEngine__PolygonDrawMode)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawPolygon"); signal != nil {
			f := func(points *core.QPointF, pointCount int, mode QPaintEngine__PolygonDrawMode) {
				(*(*func(*core.QPointF, int, QPaintEngine__PolygonDrawMode))(signal))(points, pointCount, mode)
				f(points, pointCount, mode)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawPolygon", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawPolygon", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawPolygon() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawPolygon")
	}
}

func (ptr *QPaintEngine) DrawPolygon(points core.QPointF_ITF, pointCount int, mode QPaintEngine__PolygonDrawMode) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPolygon(ptr.Pointer(), core.PointerFromQPointF(points), C.int(int32(pointCount)), C.longlong(mode))
	}
}

func (ptr *QPaintEngine) DrawPolygonDefault(points core.QPointF_ITF, pointCount int, mode QPaintEngine__PolygonDrawMode) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPolygonDefault(ptr.Pointer(), core.PointerFromQPointF(points), C.int(int32(pointCount)), C.longlong(mode))
	}
}

//export callbackQPaintEngine_DrawPolygon2
func callbackQPaintEngine_DrawPolygon2(ptr unsafe.Pointer, points unsafe.Pointer, pointCount C.int, mode C.longlong) {
	if signal := qt.GetSignal(ptr, "drawPolygon2"); signal != nil {
		(*(*func(*core.QPoint, int, QPaintEngine__PolygonDrawMode))(signal))(core.NewQPointFromPointer(points), int(int32(pointCount)), QPaintEngine__PolygonDrawMode(mode))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawPolygon2Default(core.NewQPointFromPointer(points), int(int32(pointCount)), QPaintEngine__PolygonDrawMode(mode))
	}
}

func (ptr *QPaintEngine) ConnectDrawPolygon2(f func(points *core.QPoint, pointCount int, mode QPaintEngine__PolygonDrawMode)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawPolygon2"); signal != nil {
			f := func(points *core.QPoint, pointCount int, mode QPaintEngine__PolygonDrawMode) {
				(*(*func(*core.QPoint, int, QPaintEngine__PolygonDrawMode))(signal))(points, pointCount, mode)
				f(points, pointCount, mode)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawPolygon2", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawPolygon2", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawPolygon2() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawPolygon2")
	}
}

func (ptr *QPaintEngine) DrawPolygon2(points core.QPoint_ITF, pointCount int, mode QPaintEngine__PolygonDrawMode) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPolygon2(ptr.Pointer(), core.PointerFromQPoint(points), C.int(int32(pointCount)), C.longlong(mode))
	}
}

func (ptr *QPaintEngine) DrawPolygon2Default(points core.QPoint_ITF, pointCount int, mode QPaintEngine__PolygonDrawMode) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawPolygon2Default(ptr.Pointer(), core.PointerFromQPoint(points), C.int(int32(pointCount)), C.longlong(mode))
	}
}

//export callbackQPaintEngine_DrawRects
func callbackQPaintEngine_DrawRects(ptr unsafe.Pointer, rects unsafe.Pointer, rectCount C.int) {
	if signal := qt.GetSignal(ptr, "drawRects"); signal != nil {
		(*(*func(*core.QRectF, int))(signal))(core.NewQRectFFromPointer(rects), int(int32(rectCount)))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawRectsDefault(core.NewQRectFFromPointer(rects), int(int32(rectCount)))
	}
}

func (ptr *QPaintEngine) ConnectDrawRects(f func(rects *core.QRectF, rectCount int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawRects"); signal != nil {
			f := func(rects *core.QRectF, rectCount int) {
				(*(*func(*core.QRectF, int))(signal))(rects, rectCount)
				f(rects, rectCount)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawRects", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawRects", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawRects() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawRects")
	}
}

func (ptr *QPaintEngine) DrawRects(rects core.QRectF_ITF, rectCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawRects(ptr.Pointer(), core.PointerFromQRectF(rects), C.int(int32(rectCount)))
	}
}

func (ptr *QPaintEngine) DrawRectsDefault(rects core.QRectF_ITF, rectCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawRectsDefault(ptr.Pointer(), core.PointerFromQRectF(rects), C.int(int32(rectCount)))
	}
}

//export callbackQPaintEngine_DrawRects2
func callbackQPaintEngine_DrawRects2(ptr unsafe.Pointer, rects unsafe.Pointer, rectCount C.int) {
	if signal := qt.GetSignal(ptr, "drawRects2"); signal != nil {
		(*(*func(*core.QRect, int))(signal))(core.NewQRectFromPointer(rects), int(int32(rectCount)))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawRects2Default(core.NewQRectFromPointer(rects), int(int32(rectCount)))
	}
}

func (ptr *QPaintEngine) ConnectDrawRects2(f func(rects *core.QRect, rectCount int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawRects2"); signal != nil {
			f := func(rects *core.QRect, rectCount int) {
				(*(*func(*core.QRect, int))(signal))(rects, rectCount)
				f(rects, rectCount)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawRects2", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawRects2", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawRects2() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawRects2")
	}
}

func (ptr *QPaintEngine) DrawRects2(rects core.QRect_ITF, rectCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawRects2(ptr.Pointer(), core.PointerFromQRect(rects), C.int(int32(rectCount)))
	}
}

func (ptr *QPaintEngine) DrawRects2Default(rects core.QRect_ITF, rectCount int) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawRects2Default(ptr.Pointer(), core.PointerFromQRect(rects), C.int(int32(rectCount)))
	}
}

//export callbackQPaintEngine_DrawTextItem
func callbackQPaintEngine_DrawTextItem(ptr unsafe.Pointer, p unsafe.Pointer, textItem unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "drawTextItem"); signal != nil {
		(*(*func(*core.QPointF, *QTextItem))(signal))(core.NewQPointFFromPointer(p), NewQTextItemFromPointer(textItem))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawTextItemDefault(core.NewQPointFFromPointer(p), NewQTextItemFromPointer(textItem))
	}
}

func (ptr *QPaintEngine) ConnectDrawTextItem(f func(p *core.QPointF, textItem *QTextItem)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawTextItem"); signal != nil {
			f := func(p *core.QPointF, textItem *QTextItem) {
				(*(*func(*core.QPointF, *QTextItem))(signal))(p, textItem)
				f(p, textItem)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawTextItem", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawTextItem", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawTextItem() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawTextItem")
	}
}

func (ptr *QPaintEngine) DrawTextItem(p core.QPointF_ITF, textItem QTextItem_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawTextItem(ptr.Pointer(), core.PointerFromQPointF(p), PointerFromQTextItem(textItem))
	}
}

func (ptr *QPaintEngine) DrawTextItemDefault(p core.QPointF_ITF, textItem QTextItem_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawTextItemDefault(ptr.Pointer(), core.PointerFromQPointF(p), PointerFromQTextItem(textItem))
	}
}

//export callbackQPaintEngine_DrawTiledPixmap
func callbackQPaintEngine_DrawTiledPixmap(ptr unsafe.Pointer, rect unsafe.Pointer, pixmap unsafe.Pointer, p unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "drawTiledPixmap"); signal != nil {
		(*(*func(*core.QRectF, *QPixmap, *core.QPointF))(signal))(core.NewQRectFFromPointer(rect), NewQPixmapFromPointer(pixmap), core.NewQPointFFromPointer(p))
	} else {
		NewQPaintEngineFromPointer(ptr).DrawTiledPixmapDefault(core.NewQRectFFromPointer(rect), NewQPixmapFromPointer(pixmap), core.NewQPointFFromPointer(p))
	}
}

func (ptr *QPaintEngine) ConnectDrawTiledPixmap(f func(rect *core.QRectF, pixmap *QPixmap, p *core.QPointF)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawTiledPixmap"); signal != nil {
			f := func(rect *core.QRectF, pixmap *QPixmap, p *core.QPointF) {
				(*(*func(*core.QRectF, *QPixmap, *core.QPointF))(signal))(rect, pixmap, p)
				f(rect, pixmap, p)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawTiledPixmap", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawTiledPixmap", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDrawTiledPixmap() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawTiledPixmap")
	}
}

func (ptr *QPaintEngine) DrawTiledPixmap(rect core.QRectF_ITF, pixmap QPixmap_ITF, p core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawTiledPixmap(ptr.Pointer(), core.PointerFromQRectF(rect), PointerFromQPixmap(pixmap), core.PointerFromQPointF(p))
	}
}

func (ptr *QPaintEngine) DrawTiledPixmapDefault(rect core.QRectF_ITF, pixmap QPixmap_ITF, p core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DrawTiledPixmapDefault(ptr.Pointer(), core.PointerFromQRectF(rect), PointerFromQPixmap(pixmap), core.PointerFromQPointF(p))
	}
}

//export callbackQPaintEngine_End
func callbackQPaintEngine_End(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "end"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QPaintEngine) ConnectEnd(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "end"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "end", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "end", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectEnd() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "end")
	}
}

func (ptr *QPaintEngine) End() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPaintEngine_End(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPaintEngine) HasFeature(feature QPaintEngine__PaintEngineFeature) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPaintEngine_HasFeature(ptr.Pointer(), C.longlong(feature))) != 0
	}
	return false
}

func (ptr *QPaintEngine) IsActive() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPaintEngine_IsActive(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPaintEngine) PaintDevice() *QPaintDevice {
	if ptr.Pointer() != nil {
		return NewQPaintDeviceFromPointer(C.QPaintEngine_PaintDevice(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPaintEngine) Painter() *QPainter {
	if ptr.Pointer() != nil {
		return NewQPainterFromPointer(C.QPaintEngine_Painter(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPaintEngine) SetActive(state bool) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_SetActive(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(state))))
	}
}

//export callbackQPaintEngine_Type
func callbackQPaintEngine_Type(ptr unsafe.Pointer) C.longlong {
	if signal := qt.GetSignal(ptr, "type"); signal != nil {
		return C.longlong((*(*func() QPaintEngine__Type)(signal))())
	}

	return C.longlong(0)
}

func (ptr *QPaintEngine) ConnectType(f func() QPaintEngine__Type) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "type"); signal != nil {
			f := func() QPaintEngine__Type {
				(*(*func() QPaintEngine__Type)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "type", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "type", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectType() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "type")
	}
}

func (ptr *QPaintEngine) Type() QPaintEngine__Type {
	if ptr.Pointer() != nil {
		return QPaintEngine__Type(C.QPaintEngine_Type(ptr.Pointer()))
	}
	return 0
}

//export callbackQPaintEngine_UpdateState
func callbackQPaintEngine_UpdateState(ptr unsafe.Pointer, state unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "updateState"); signal != nil {
		(*(*func(*QPaintEngineState))(signal))(NewQPaintEngineStateFromPointer(state))
	}

}

func (ptr *QPaintEngine) ConnectUpdateState(f func(state *QPaintEngineState)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "updateState"); signal != nil {
			f := func(state *QPaintEngineState) {
				(*(*func(*QPaintEngineState))(signal))(state)
				f(state)
			}
			qt.ConnectSignal(ptr.Pointer(), "updateState", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "updateState", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectUpdateState() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "updateState")
	}
}

func (ptr *QPaintEngine) UpdateState(state QPaintEngineState_ITF) {
	if ptr.Pointer() != nil {
		C.QPaintEngine_UpdateState(ptr.Pointer(), PointerFromQPaintEngineState(state))
	}
}

//export callbackQPaintEngine_DestroyQPaintEngine
func callbackQPaintEngine_DestroyQPaintEngine(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QPaintEngine"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQPaintEngineFromPointer(ptr).DestroyQPaintEngineDefault()
	}
}

func (ptr *QPaintEngine) ConnectDestroyQPaintEngine(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QPaintEngine"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QPaintEngine", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QPaintEngine", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPaintEngine) DisconnectDestroyQPaintEngine() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QPaintEngine")
	}
}

func (ptr *QPaintEngine) DestroyQPaintEngine() {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DestroyQPaintEngine(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPaintEngine) DestroyQPaintEngineDefault() {
	if ptr.Pointer() != nil {
		C.QPaintEngine_DestroyQPaintEngineDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QPaintEngineState struct {
	ptr unsafe.Pointer
}

type QPaintEngineState_ITF interface {
	QPaintEngineState_PTR() *QPaintEngineState
}

func (ptr *QPaintEngineState) QPaintEngineState_PTR() *QPaintEngineState {
	return ptr
}

func (ptr *QPaintEngineState) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPaintEngineState) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPaintEngineState(ptr QPaintEngineState_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintEngineState_PTR().Pointer()
	}
	return nil
}

func NewQPaintEngineStateFromPointer(ptr unsafe.Pointer) (n *QPaintEngineState) {
	n = new(QPaintEngineState)
	n.SetPointer(ptr)
	return
}
func (ptr *QPaintEngineState) DestroyQPaintEngineState() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func (ptr *QPaintEngineState) BackgroundBrush() *QBrush {
	if ptr.Pointer() != nil {
		tmpValue := NewQBrushFromPointer(C.QPaintEngineState_BackgroundBrush(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
		return tmpValue
	}
	return nil
}

func (ptr *QPaintEngineState) BackgroundMode() core.Qt__BGMode {
	if ptr.Pointer() != nil {
		return core.Qt__BGMode(C.QPaintEngineState_BackgroundMode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPaintEngineState) Brush() *QBrush {
	if ptr.Pointer() != nil {
		tmpValue := NewQBrushFromPointer(C.QPaintEngineState_Brush(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
		return tmpValue
	}
	return nil
}

func (ptr *QPaintEngineState) BrushNeedsResolving() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPaintEngineState_BrushNeedsResolving(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPaintEngineState) BrushOrigin() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QPaintEngineState_BrushOrigin(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QPaintEngineState) ClipOperation() core.Qt__ClipOperation {
	if ptr.Pointer() != nil {
		return core.Qt__ClipOperation(C.QPaintEngineState_ClipOperation(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPaintEngineState) ClipPath() *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QPaintEngineState_ClipPath(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QPaintEngineState) ClipRegion() *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QPaintEngineState_ClipRegion(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QPaintEngineState) CompositionMode() QPainter__CompositionMode {
	if ptr.Pointer() != nil {
		return QPainter__CompositionMode(C.QPaintEngineState_CompositionMode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPaintEngineState) Font() *QFont {
	if ptr.Pointer() != nil {
		tmpValue := NewQFontFromPointer(C.QPaintEngineState_Font(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
		return tmpValue
	}
	return nil
}

func (ptr *QPaintEngineState) IsClipEnabled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPaintEngineState_IsClipEnabled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPaintEngineState) Opacity() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPaintEngineState_Opacity(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPaintEngineState) Painter() *QPainter {
	if ptr.Pointer() != nil {
		return NewQPainterFromPointer(C.QPaintEngineState_Painter(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPaintEngineState) Pen() *QPen {
	if ptr.Pointer() != nil {
		tmpValue := NewQPenFromPointer(C.QPaintEngineState_Pen(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPen).DestroyQPen)
		return tmpValue
	}
	return nil
}

func (ptr *QPaintEngineState) PenNeedsResolving() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPaintEngineState_PenNeedsResolving(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPaintEngineState) RenderHints() QPainter__RenderHint {
	if ptr.Pointer() != nil {
		return QPainter__RenderHint(C.QPaintEngineState_RenderHints(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPaintEngineState) State() QPaintEngine__DirtyFlag {
	if ptr.Pointer() != nil {
		return QPaintEngine__DirtyFlag(C.QPaintEngineState_State(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPaintEngineState) Transform() *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QPaintEngineState_Transform(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

type QPaintEvent struct {
	core.QEvent
}

type QPaintEvent_ITF interface {
	core.QEvent_ITF
	QPaintEvent_PTR() *QPaintEvent
}

func (ptr *QPaintEvent) QPaintEvent_PTR() *QPaintEvent {
	return ptr
}

func (ptr *QPaintEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QPaintEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQPaintEvent(ptr QPaintEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintEvent_PTR().Pointer()
	}
	return nil
}

func NewQPaintEventFromPointer(ptr unsafe.Pointer) (n *QPaintEvent) {
	n = new(QPaintEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QPaintEvent) DestroyQPaintEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQPaintEvent(paintRegion QRegion_ITF) *QPaintEvent {
	tmpValue := NewQPaintEventFromPointer(C.QPaintEvent_NewQPaintEvent(PointerFromQRegion(paintRegion)))
	qt.SetFinalizer(tmpValue, (*QPaintEvent).DestroyQPaintEvent)
	return tmpValue
}

func NewQPaintEvent2(paintRect core.QRect_ITF) *QPaintEvent {
	tmpValue := NewQPaintEventFromPointer(C.QPaintEvent_NewQPaintEvent2(core.PointerFromQRect(paintRect)))
	qt.SetFinalizer(tmpValue, (*QPaintEvent).DestroyQPaintEvent)
	return tmpValue
}

func (ptr *QPaintEvent) Rect() *core.QRect {
	if ptr.Pointer() != nil {
		return core.NewQRectFromPointer(C.QPaintEvent_Rect(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPaintEvent) Region() *QRegion {
	if ptr.Pointer() != nil {
		return NewQRegionFromPointer(C.QPaintEvent_Region(ptr.Pointer()))
	}
	return nil
}

type QPainter struct {
	ptr unsafe.Pointer
}

type QPainter_ITF interface {
	QPainter_PTR() *QPainter
}

func (ptr *QPainter) QPainter_PTR() *QPainter {
	return ptr
}

func (ptr *QPainter) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPainter) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPainter(ptr QPainter_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPainter_PTR().Pointer()
	}
	return nil
}

func NewQPainterFromPointer(ptr unsafe.Pointer) (n *QPainter) {
	n = new(QPainter)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QPainter__RenderHint
//QPainter::RenderHint
type QPainter__RenderHint int64

const (
	QPainter__Antialiasing            QPainter__RenderHint = QPainter__RenderHint(0x01)
	QPainter__TextAntialiasing        QPainter__RenderHint = QPainter__RenderHint(0x02)
	QPainter__SmoothPixmapTransform   QPainter__RenderHint = QPainter__RenderHint(0x04)
	QPainter__HighQualityAntialiasing QPainter__RenderHint = QPainter__RenderHint(0x08)
	QPainter__NonCosmeticDefaultPen   QPainter__RenderHint = QPainter__RenderHint(0x10)
	QPainter__Qt4CompatiblePainting   QPainter__RenderHint = QPainter__RenderHint(0x20)
	QPainter__LosslessImageRendering  QPainter__RenderHint = QPainter__RenderHint(0x40)
)

//go:generate stringer -type=QPainter__PixmapFragmentHint
//QPainter::PixmapFragmentHint
type QPainter__PixmapFragmentHint int64

const (
	QPainter__OpaqueHint QPainter__PixmapFragmentHint = QPainter__PixmapFragmentHint(0x01)
)

//go:generate stringer -type=QPainter__CompositionMode
//QPainter::CompositionMode
type QPainter__CompositionMode int64

const (
	QPainter__CompositionMode_SourceOver          QPainter__CompositionMode = QPainter__CompositionMode(0)
	QPainter__CompositionMode_DestinationOver     QPainter__CompositionMode = QPainter__CompositionMode(1)
	QPainter__CompositionMode_Clear               QPainter__CompositionMode = QPainter__CompositionMode(2)
	QPainter__CompositionMode_Source              QPainter__CompositionMode = QPainter__CompositionMode(3)
	QPainter__CompositionMode_Destination         QPainter__CompositionMode = QPainter__CompositionMode(4)
	QPainter__CompositionMode_SourceIn            QPainter__CompositionMode = QPainter__CompositionMode(5)
	QPainter__CompositionMode_DestinationIn       QPainter__CompositionMode = QPainter__CompositionMode(6)
	QPainter__CompositionMode_SourceOut           QPainter__CompositionMode = QPainter__CompositionMode(7)
	QPainter__CompositionMode_DestinationOut      QPainter__CompositionMode = QPainter__CompositionMode(8)
	QPainter__CompositionMode_SourceAtop          QPainter__CompositionMode = QPainter__CompositionMode(9)
	QPainter__CompositionMode_DestinationAtop     QPainter__CompositionMode = QPainter__CompositionMode(10)
	QPainter__CompositionMode_Xor                 QPainter__CompositionMode = QPainter__CompositionMode(11)
	QPainter__CompositionMode_Plus                QPainter__CompositionMode = QPainter__CompositionMode(12)
	QPainter__CompositionMode_Multiply            QPainter__CompositionMode = QPainter__CompositionMode(13)
	QPainter__CompositionMode_Screen              QPainter__CompositionMode = QPainter__CompositionMode(14)
	QPainter__CompositionMode_Overlay             QPainter__CompositionMode = QPainter__CompositionMode(15)
	QPainter__CompositionMode_Darken              QPainter__CompositionMode = QPainter__CompositionMode(16)
	QPainter__CompositionMode_Lighten             QPainter__CompositionMode = QPainter__CompositionMode(17)
	QPainter__CompositionMode_ColorDodge          QPainter__CompositionMode = QPainter__CompositionMode(18)
	QPainter__CompositionMode_ColorBurn           QPainter__CompositionMode = QPainter__CompositionMode(19)
	QPainter__CompositionMode_HardLight           QPainter__CompositionMode = QPainter__CompositionMode(20)
	QPainter__CompositionMode_SoftLight           QPainter__CompositionMode = QPainter__CompositionMode(21)
	QPainter__CompositionMode_Difference          QPainter__CompositionMode = QPainter__CompositionMode(22)
	QPainter__CompositionMode_Exclusion           QPainter__CompositionMode = QPainter__CompositionMode(23)
	QPainter__RasterOp_SourceOrDestination        QPainter__CompositionMode = QPainter__CompositionMode(24)
	QPainter__RasterOp_SourceAndDestination       QPainter__CompositionMode = QPainter__CompositionMode(25)
	QPainter__RasterOp_SourceXorDestination       QPainter__CompositionMode = QPainter__CompositionMode(26)
	QPainter__RasterOp_NotSourceAndNotDestination QPainter__CompositionMode = QPainter__CompositionMode(27)
	QPainter__RasterOp_NotSourceOrNotDestination  QPainter__CompositionMode = QPainter__CompositionMode(28)
	QPainter__RasterOp_NotSourceXorDestination    QPainter__CompositionMode = QPainter__CompositionMode(29)
	QPainter__RasterOp_NotSource                  QPainter__CompositionMode = QPainter__CompositionMode(30)
	QPainter__RasterOp_NotSourceAndDestination    QPainter__CompositionMode = QPainter__CompositionMode(31)
	QPainter__RasterOp_SourceAndNotDestination    QPainter__CompositionMode = QPainter__CompositionMode(32)
	QPainter__RasterOp_NotSourceOrDestination     QPainter__CompositionMode = QPainter__CompositionMode(33)
	QPainter__RasterOp_SourceOrNotDestination     QPainter__CompositionMode = QPainter__CompositionMode(34)
	QPainter__RasterOp_ClearDestination           QPainter__CompositionMode = QPainter__CompositionMode(35)
	QPainter__RasterOp_SetDestination             QPainter__CompositionMode = QPainter__CompositionMode(36)
	QPainter__RasterOp_NotDestination             QPainter__CompositionMode = QPainter__CompositionMode(37)
)

func NewQPainter() *QPainter {
	tmpValue := NewQPainterFromPointer(C.QPainter_NewQPainter())
	qt.SetFinalizer(tmpValue, (*QPainter).DestroyQPainter)
	return tmpValue
}

func NewQPainter2(device QPaintDevice_ITF) *QPainter {
	tmpValue := NewQPainterFromPointer(C.QPainter_NewQPainter2(PointerFromQPaintDevice(device)))
	qt.SetFinalizer(tmpValue, (*QPainter).DestroyQPainter)
	return tmpValue
}

func (ptr *QPainter) Background() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPainter_Background(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPainter) BackgroundMode() core.Qt__BGMode {
	if ptr.Pointer() != nil {
		return core.Qt__BGMode(C.QPainter_BackgroundMode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainter) Begin(device QPaintDevice_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainter_Begin(ptr.Pointer(), PointerFromQPaintDevice(device))) != 0
	}
	return false
}

func (ptr *QPainter) BeginNativePainting() {
	if ptr.Pointer() != nil {
		C.QPainter_BeginNativePainting(ptr.Pointer())
	}
}

func (ptr *QPainter) BoundingRect(rectangle core.QRectF_ITF, flags int, text string) *core.QRectF {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFFromPointer(C.QPainter_BoundingRect(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.int(int32(flags)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) BoundingRect2(rectangle core.QRect_ITF, flags int, text string) *core.QRect {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFromPointer(C.QPainter_BoundingRect2(ptr.Pointer(), core.PointerFromQRect(rectangle), C.int(int32(flags)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) BoundingRect3(x int, y int, w int, h int, flags int, text string) *core.QRect {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFromPointer(C.QPainter_BoundingRect3(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(w)), C.int(int32(h)), C.int(int32(flags)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) BoundingRect4(rectangle core.QRectF_ITF, text string, option QTextOption_ITF) *core.QRectF {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		tmpValue := core.NewQRectFFromPointer(C.QPainter_BoundingRect4(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, PointerFromQTextOption(option)))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) Brush() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPainter_Brush(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPainter) BrushOrigin() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QPainter_BrushOrigin(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) ClipBoundingRect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QPainter_ClipBoundingRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) ClipPath() *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QPainter_ClipPath(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) ClipRegion() *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QPainter_ClipRegion(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) CombinedTransform() *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QPainter_CombinedTransform(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) CompositionMode() QPainter__CompositionMode {
	if ptr.Pointer() != nil {
		return QPainter__CompositionMode(C.QPainter_CompositionMode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainter) Device() *QPaintDevice {
	if ptr.Pointer() != nil {
		return NewQPaintDeviceFromPointer(C.QPainter_Device(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPainter) DeviceTransform() *QTransform {
	if ptr.Pointer() != nil {
		return NewQTransformFromPointer(C.QPainter_DeviceTransform(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPainter) DrawArc(rectangle core.QRectF_ITF, startAngle int, spanAngle int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawArc(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.int(int32(startAngle)), C.int(int32(spanAngle)))
	}
}

func (ptr *QPainter) DrawArc2(rectangle core.QRect_ITF, startAngle int, spanAngle int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawArc2(ptr.Pointer(), core.PointerFromQRect(rectangle), C.int(int32(startAngle)), C.int(int32(spanAngle)))
	}
}

func (ptr *QPainter) DrawArc3(x int, y int, width int, height int, startAngle int, spanAngle int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawArc3(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.int(int32(startAngle)), C.int(int32(spanAngle)))
	}
}

func (ptr *QPainter) DrawChord(rectangle core.QRectF_ITF, startAngle int, spanAngle int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawChord(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.int(int32(startAngle)), C.int(int32(spanAngle)))
	}
}

func (ptr *QPainter) DrawChord2(x int, y int, width int, height int, startAngle int, spanAngle int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawChord2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.int(int32(startAngle)), C.int(int32(spanAngle)))
	}
}

func (ptr *QPainter) DrawChord3(rectangle core.QRect_ITF, startAngle int, spanAngle int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawChord3(ptr.Pointer(), core.PointerFromQRect(rectangle), C.int(int32(startAngle)), C.int(int32(spanAngle)))
	}
}

func (ptr *QPainter) DrawConvexPolygon(points core.QPointF_ITF, pointCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawConvexPolygon(ptr.Pointer(), core.PointerFromQPointF(points), C.int(int32(pointCount)))
	}
}

func (ptr *QPainter) DrawConvexPolygon2(polygon QPolygonF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawConvexPolygon2(ptr.Pointer(), PointerFromQPolygonF(polygon))
	}
}

func (ptr *QPainter) DrawConvexPolygon3(points core.QPoint_ITF, pointCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawConvexPolygon3(ptr.Pointer(), core.PointerFromQPoint(points), C.int(int32(pointCount)))
	}
}

func (ptr *QPainter) DrawConvexPolygon4(polygon QPolygon_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawConvexPolygon4(ptr.Pointer(), PointerFromQPolygon(polygon))
	}
}

func (ptr *QPainter) DrawEllipse(rectangle core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawEllipse(ptr.Pointer(), core.PointerFromQRectF(rectangle))
	}
}

func (ptr *QPainter) DrawEllipse2(rectangle core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawEllipse2(ptr.Pointer(), core.PointerFromQRect(rectangle))
	}
}

func (ptr *QPainter) DrawEllipse3(x int, y int, width int, height int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawEllipse3(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QPainter) DrawEllipse4(center core.QPointF_ITF, rx float64, ry float64) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawEllipse4(ptr.Pointer(), core.PointerFromQPointF(center), C.double(rx), C.double(ry))
	}
}

func (ptr *QPainter) DrawEllipse5(center core.QPoint_ITF, rx int, ry int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawEllipse5(ptr.Pointer(), core.PointerFromQPoint(center), C.int(int32(rx)), C.int(int32(ry)))
	}
}

func (ptr *QPainter) DrawGlyphRun(position core.QPointF_ITF, glyphs QGlyphRun_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawGlyphRun(ptr.Pointer(), core.PointerFromQPointF(position), PointerFromQGlyphRun(glyphs))
	}
}

func (ptr *QPainter) DrawImage(target core.QRectF_ITF, image QImage_ITF, source core.QRectF_ITF, flags core.Qt__ImageConversionFlag) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawImage(ptr.Pointer(), core.PointerFromQRectF(target), PointerFromQImage(image), core.PointerFromQRectF(source), C.longlong(flags))
	}
}

func (ptr *QPainter) DrawImage2(target core.QRect_ITF, image QImage_ITF, source core.QRect_ITF, flags core.Qt__ImageConversionFlag) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawImage2(ptr.Pointer(), core.PointerFromQRect(target), PointerFromQImage(image), core.PointerFromQRect(source), C.longlong(flags))
	}
}

func (ptr *QPainter) DrawImage3(point core.QPointF_ITF, image QImage_ITF, source core.QRectF_ITF, flags core.Qt__ImageConversionFlag) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawImage3(ptr.Pointer(), core.PointerFromQPointF(point), PointerFromQImage(image), core.PointerFromQRectF(source), C.longlong(flags))
	}
}

func (ptr *QPainter) DrawImage4(point core.QPoint_ITF, image QImage_ITF, source core.QRect_ITF, flags core.Qt__ImageConversionFlag) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawImage4(ptr.Pointer(), core.PointerFromQPoint(point), PointerFromQImage(image), core.PointerFromQRect(source), C.longlong(flags))
	}
}

func (ptr *QPainter) DrawImage5(rectangle core.QRectF_ITF, image QImage_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawImage5(ptr.Pointer(), core.PointerFromQRectF(rectangle), PointerFromQImage(image))
	}
}

func (ptr *QPainter) DrawImage6(rectangle core.QRect_ITF, image QImage_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawImage6(ptr.Pointer(), core.PointerFromQRect(rectangle), PointerFromQImage(image))
	}
}

func (ptr *QPainter) DrawImage7(point core.QPointF_ITF, image QImage_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawImage7(ptr.Pointer(), core.PointerFromQPointF(point), PointerFromQImage(image))
	}
}

func (ptr *QPainter) DrawImage8(point core.QPoint_ITF, image QImage_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawImage8(ptr.Pointer(), core.PointerFromQPoint(point), PointerFromQImage(image))
	}
}

func (ptr *QPainter) DrawImage9(x int, y int, image QImage_ITF, sx int, sy int, sw int, sh int, flags core.Qt__ImageConversionFlag) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawImage9(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), PointerFromQImage(image), C.int(int32(sx)), C.int(int32(sy)), C.int(int32(sw)), C.int(int32(sh)), C.longlong(flags))
	}
}

func (ptr *QPainter) DrawLine(line core.QLineF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLine(ptr.Pointer(), core.PointerFromQLineF(line))
	}
}

func (ptr *QPainter) DrawLine2(line core.QLine_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLine2(ptr.Pointer(), core.PointerFromQLine(line))
	}
}

func (ptr *QPainter) DrawLine3(x1 int, y1 int, x2 int, y2 int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLine3(ptr.Pointer(), C.int(int32(x1)), C.int(int32(y1)), C.int(int32(x2)), C.int(int32(y2)))
	}
}

func (ptr *QPainter) DrawLine4(p1 core.QPoint_ITF, p2 core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLine4(ptr.Pointer(), core.PointerFromQPoint(p1), core.PointerFromQPoint(p2))
	}
}

func (ptr *QPainter) DrawLine5(p1 core.QPointF_ITF, p2 core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLine5(ptr.Pointer(), core.PointerFromQPointF(p1), core.PointerFromQPointF(p2))
	}
}

func (ptr *QPainter) DrawLines(lines core.QLineF_ITF, lineCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLines(ptr.Pointer(), core.PointerFromQLineF(lines), C.int(int32(lineCount)))
	}
}

func (ptr *QPainter) DrawLines2(lines []*core.QLineF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLines2(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQPainterFromPointer(NewQPainterFromPointer(nil).__drawLines_lines_newList2())
			for _, v := range lines {
				tmpList.__drawLines_lines_setList2(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QPainter) DrawLines3(pointPairs core.QPointF_ITF, lineCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLines3(ptr.Pointer(), core.PointerFromQPointF(pointPairs), C.int(int32(lineCount)))
	}
}

func (ptr *QPainter) DrawLines4(pointPairs []*core.QPointF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLines4(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQPainterFromPointer(NewQPainterFromPointer(nil).__drawLines_pointPairs_newList4())
			for _, v := range pointPairs {
				tmpList.__drawLines_pointPairs_setList4(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QPainter) DrawLines5(lines core.QLine_ITF, lineCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLines5(ptr.Pointer(), core.PointerFromQLine(lines), C.int(int32(lineCount)))
	}
}

func (ptr *QPainter) DrawLines6(lines []*core.QLine) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLines6(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQPainterFromPointer(NewQPainterFromPointer(nil).__drawLines_lines_newList6())
			for _, v := range lines {
				tmpList.__drawLines_lines_setList6(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QPainter) DrawLines7(pointPairs core.QPoint_ITF, lineCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLines7(ptr.Pointer(), core.PointerFromQPoint(pointPairs), C.int(int32(lineCount)))
	}
}

func (ptr *QPainter) DrawLines8(pointPairs []*core.QPoint) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawLines8(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQPainterFromPointer(NewQPainterFromPointer(nil).__drawLines_pointPairs_newList8())
			for _, v := range pointPairs {
				tmpList.__drawLines_pointPairs_setList8(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QPainter) DrawPath(path QPainterPath_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPath(ptr.Pointer(), PointerFromQPainterPath(path))
	}
}

func (ptr *QPainter) DrawPicture(point core.QPointF_ITF, picture QPicture_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPicture(ptr.Pointer(), core.PointerFromQPointF(point), PointerFromQPicture(picture))
	}
}

func (ptr *QPainter) DrawPicture2(x int, y int, picture QPicture_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPicture2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), PointerFromQPicture(picture))
	}
}

func (ptr *QPainter) DrawPicture3(point core.QPoint_ITF, picture QPicture_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPicture3(ptr.Pointer(), core.PointerFromQPoint(point), PointerFromQPicture(picture))
	}
}

func (ptr *QPainter) DrawPie(rectangle core.QRectF_ITF, startAngle int, spanAngle int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPie(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.int(int32(startAngle)), C.int(int32(spanAngle)))
	}
}

func (ptr *QPainter) DrawPie2(x int, y int, width int, height int, startAngle int, spanAngle int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPie2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.int(int32(startAngle)), C.int(int32(spanAngle)))
	}
}

func (ptr *QPainter) DrawPie3(rectangle core.QRect_ITF, startAngle int, spanAngle int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPie3(ptr.Pointer(), core.PointerFromQRect(rectangle), C.int(int32(startAngle)), C.int(int32(spanAngle)))
	}
}

func (ptr *QPainter) DrawPixmap(target core.QRectF_ITF, pixmap QPixmap_ITF, source core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap(ptr.Pointer(), core.PointerFromQRectF(target), PointerFromQPixmap(pixmap), core.PointerFromQRectF(source))
	}
}

func (ptr *QPainter) DrawPixmap2(target core.QRect_ITF, pixmap QPixmap_ITF, source core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap2(ptr.Pointer(), core.PointerFromQRect(target), PointerFromQPixmap(pixmap), core.PointerFromQRect(source))
	}
}

func (ptr *QPainter) DrawPixmap3(x int, y int, w int, h int, pixmap QPixmap_ITF, sx int, sy int, sw int, sh int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap3(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(w)), C.int(int32(h)), PointerFromQPixmap(pixmap), C.int(int32(sx)), C.int(int32(sy)), C.int(int32(sw)), C.int(int32(sh)))
	}
}

func (ptr *QPainter) DrawPixmap4(x int, y int, pixmap QPixmap_ITF, sx int, sy int, sw int, sh int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap4(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), PointerFromQPixmap(pixmap), C.int(int32(sx)), C.int(int32(sy)), C.int(int32(sw)), C.int(int32(sh)))
	}
}

func (ptr *QPainter) DrawPixmap5(point core.QPointF_ITF, pixmap QPixmap_ITF, source core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap5(ptr.Pointer(), core.PointerFromQPointF(point), PointerFromQPixmap(pixmap), core.PointerFromQRectF(source))
	}
}

func (ptr *QPainter) DrawPixmap6(point core.QPoint_ITF, pixmap QPixmap_ITF, source core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap6(ptr.Pointer(), core.PointerFromQPoint(point), PointerFromQPixmap(pixmap), core.PointerFromQRect(source))
	}
}

func (ptr *QPainter) DrawPixmap7(point core.QPointF_ITF, pixmap QPixmap_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap7(ptr.Pointer(), core.PointerFromQPointF(point), PointerFromQPixmap(pixmap))
	}
}

func (ptr *QPainter) DrawPixmap8(point core.QPoint_ITF, pixmap QPixmap_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap8(ptr.Pointer(), core.PointerFromQPoint(point), PointerFromQPixmap(pixmap))
	}
}

func (ptr *QPainter) DrawPixmap9(x int, y int, pixmap QPixmap_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap9(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), PointerFromQPixmap(pixmap))
	}
}

func (ptr *QPainter) DrawPixmap10(rectangle core.QRect_ITF, pixmap QPixmap_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap10(ptr.Pointer(), core.PointerFromQRect(rectangle), PointerFromQPixmap(pixmap))
	}
}

func (ptr *QPainter) DrawPixmap11(x int, y int, width int, height int, pixmap QPixmap_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPixmap11(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), PointerFromQPixmap(pixmap))
	}
}

func (ptr *QPainter) DrawPoint(position core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPoint(ptr.Pointer(), core.PointerFromQPointF(position))
	}
}

func (ptr *QPainter) DrawPoint2(position core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPoint2(ptr.Pointer(), core.PointerFromQPoint(position))
	}
}

func (ptr *QPainter) DrawPoint3(x int, y int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPoint3(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)))
	}
}

func (ptr *QPainter) DrawPoints(points core.QPointF_ITF, pointCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPoints(ptr.Pointer(), core.PointerFromQPointF(points), C.int(int32(pointCount)))
	}
}

func (ptr *QPainter) DrawPoints2(points QPolygonF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPoints2(ptr.Pointer(), PointerFromQPolygonF(points))
	}
}

func (ptr *QPainter) DrawPoints3(points core.QPoint_ITF, pointCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPoints3(ptr.Pointer(), core.PointerFromQPoint(points), C.int(int32(pointCount)))
	}
}

func (ptr *QPainter) DrawPoints4(points QPolygon_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPoints4(ptr.Pointer(), PointerFromQPolygon(points))
	}
}

func (ptr *QPainter) DrawPolygon(points core.QPointF_ITF, pointCount int, fillRule core.Qt__FillRule) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPolygon(ptr.Pointer(), core.PointerFromQPointF(points), C.int(int32(pointCount)), C.longlong(fillRule))
	}
}

func (ptr *QPainter) DrawPolygon2(points QPolygonF_ITF, fillRule core.Qt__FillRule) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPolygon2(ptr.Pointer(), PointerFromQPolygonF(points), C.longlong(fillRule))
	}
}

func (ptr *QPainter) DrawPolygon3(points core.QPoint_ITF, pointCount int, fillRule core.Qt__FillRule) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPolygon3(ptr.Pointer(), core.PointerFromQPoint(points), C.int(int32(pointCount)), C.longlong(fillRule))
	}
}

func (ptr *QPainter) DrawPolygon4(points QPolygon_ITF, fillRule core.Qt__FillRule) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPolygon4(ptr.Pointer(), PointerFromQPolygon(points), C.longlong(fillRule))
	}
}

func (ptr *QPainter) DrawPolyline(points core.QPointF_ITF, pointCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPolyline(ptr.Pointer(), core.PointerFromQPointF(points), C.int(int32(pointCount)))
	}
}

func (ptr *QPainter) DrawPolyline2(points QPolygonF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPolyline2(ptr.Pointer(), PointerFromQPolygonF(points))
	}
}

func (ptr *QPainter) DrawPolyline3(points core.QPoint_ITF, pointCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPolyline3(ptr.Pointer(), core.PointerFromQPoint(points), C.int(int32(pointCount)))
	}
}

func (ptr *QPainter) DrawPolyline4(points QPolygon_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawPolyline4(ptr.Pointer(), PointerFromQPolygon(points))
	}
}

func (ptr *QPainter) DrawRect(rectangle core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawRect(ptr.Pointer(), core.PointerFromQRectF(rectangle))
	}
}

func (ptr *QPainter) DrawRect2(x int, y int, width int, height int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawRect2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QPainter) DrawRect3(rectangle core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawRect3(ptr.Pointer(), core.PointerFromQRect(rectangle))
	}
}

func (ptr *QPainter) DrawRects(rectangles core.QRectF_ITF, rectCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawRects(ptr.Pointer(), core.PointerFromQRectF(rectangles), C.int(int32(rectCount)))
	}
}

func (ptr *QPainter) DrawRects2(rectangles []*core.QRectF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawRects2(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQPainterFromPointer(NewQPainterFromPointer(nil).__drawRects_rectangles_newList2())
			for _, v := range rectangles {
				tmpList.__drawRects_rectangles_setList2(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QPainter) DrawRects3(rectangles core.QRect_ITF, rectCount int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawRects3(ptr.Pointer(), core.PointerFromQRect(rectangles), C.int(int32(rectCount)))
	}
}

func (ptr *QPainter) DrawRects4(rectangles []*core.QRect) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawRects4(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQPainterFromPointer(NewQPainterFromPointer(nil).__drawRects_rectangles_newList4())
			for _, v := range rectangles {
				tmpList.__drawRects_rectangles_setList4(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QPainter) DrawRoundedRect(rect core.QRectF_ITF, xRadius float64, yRadius float64, mode core.Qt__SizeMode) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawRoundedRect(ptr.Pointer(), core.PointerFromQRectF(rect), C.double(xRadius), C.double(yRadius), C.longlong(mode))
	}
}

func (ptr *QPainter) DrawRoundedRect2(x int, y int, w int, h int, xRadius float64, yRadius float64, mode core.Qt__SizeMode) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawRoundedRect2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(w)), C.int(int32(h)), C.double(xRadius), C.double(yRadius), C.longlong(mode))
	}
}

func (ptr *QPainter) DrawRoundedRect3(rect core.QRect_ITF, xRadius float64, yRadius float64, mode core.Qt__SizeMode) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawRoundedRect3(ptr.Pointer(), core.PointerFromQRect(rect), C.double(xRadius), C.double(yRadius), C.longlong(mode))
	}
}

func (ptr *QPainter) DrawStaticText(topLeftPosition core.QPointF_ITF, staticText QStaticText_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawStaticText(ptr.Pointer(), core.PointerFromQPointF(topLeftPosition), PointerFromQStaticText(staticText))
	}
}

func (ptr *QPainter) DrawStaticText2(topLeftPosition core.QPoint_ITF, staticText QStaticText_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawStaticText2(ptr.Pointer(), core.PointerFromQPoint(topLeftPosition), PointerFromQStaticText(staticText))
	}
}

func (ptr *QPainter) DrawStaticText3(left int, top int, staticText QStaticText_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawStaticText3(ptr.Pointer(), C.int(int32(left)), C.int(int32(top)), PointerFromQStaticText(staticText))
	}
}

func (ptr *QPainter) DrawText(position core.QPointF_ITF, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QPainter_DrawText(ptr.Pointer(), core.PointerFromQPointF(position), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QPainter) DrawText2(position core.QPoint_ITF, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QPainter_DrawText2(ptr.Pointer(), core.PointerFromQPoint(position), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QPainter) DrawText3(x int, y int, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QPainter_DrawText3(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QPainter) DrawText4(rectangle core.QRect_ITF, flags int, text string, boundingRect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QPainter_DrawText4(ptr.Pointer(), core.PointerFromQRect(rectangle), C.int(int32(flags)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, core.PointerFromQRect(boundingRect))
	}
}

func (ptr *QPainter) DrawText5(x int, y int, width int, height int, flags int, text string, boundingRect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QPainter_DrawText5(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.int(int32(flags)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, core.PointerFromQRect(boundingRect))
	}
}

func (ptr *QPainter) DrawText6(rectangle core.QRectF_ITF, text string, option QTextOption_ITF) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QPainter_DrawText6(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, PointerFromQTextOption(option))
	}
}

func (ptr *QPainter) DrawTiledPixmap(rectangle core.QRectF_ITF, pixmap QPixmap_ITF, position core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawTiledPixmap(ptr.Pointer(), core.PointerFromQRectF(rectangle), PointerFromQPixmap(pixmap), core.PointerFromQPointF(position))
	}
}

func (ptr *QPainter) DrawTiledPixmap2(x int, y int, width int, height int, pixmap QPixmap_ITF, sx int, sy int) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawTiledPixmap2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), PointerFromQPixmap(pixmap), C.int(int32(sx)), C.int(int32(sy)))
	}
}

func (ptr *QPainter) DrawTiledPixmap3(rectangle core.QRect_ITF, pixmap QPixmap_ITF, position core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_DrawTiledPixmap3(ptr.Pointer(), core.PointerFromQRect(rectangle), PointerFromQPixmap(pixmap), core.PointerFromQPoint(position))
	}
}

func (ptr *QPainter) End() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainter_End(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPainter) EndNativePainting() {
	if ptr.Pointer() != nil {
		C.QPainter_EndNativePainting(ptr.Pointer())
	}
}

func (ptr *QPainter) EraseRect(rectangle core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_EraseRect(ptr.Pointer(), core.PointerFromQRectF(rectangle))
	}
}

func (ptr *QPainter) EraseRect2(x int, y int, width int, height int) {
	if ptr.Pointer() != nil {
		C.QPainter_EraseRect2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QPainter) EraseRect3(rectangle core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_EraseRect3(ptr.Pointer(), core.PointerFromQRect(rectangle))
	}
}

func (ptr *QPainter) FillPath(path QPainterPath_ITF, brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_FillPath(ptr.Pointer(), PointerFromQPainterPath(path), PointerFromQBrush(brush))
	}
}

func (ptr *QPainter) FillRect(rectangle core.QRectF_ITF, brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect(ptr.Pointer(), core.PointerFromQRectF(rectangle), PointerFromQBrush(brush))
	}
}

func (ptr *QPainter) FillRect2(x int, y int, width int, height int, brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), PointerFromQBrush(brush))
	}
}

func (ptr *QPainter) FillRect3(rectangle core.QRect_ITF, brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect3(ptr.Pointer(), core.PointerFromQRect(rectangle), PointerFromQBrush(brush))
	}
}

func (ptr *QPainter) FillRect4(rectangle core.QRectF_ITF, color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect4(ptr.Pointer(), core.PointerFromQRectF(rectangle), PointerFromQColor(color))
	}
}

func (ptr *QPainter) FillRect5(x int, y int, width int, height int, color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect5(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), PointerFromQColor(color))
	}
}

func (ptr *QPainter) FillRect6(rectangle core.QRect_ITF, color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect6(ptr.Pointer(), core.PointerFromQRect(rectangle), PointerFromQColor(color))
	}
}

func (ptr *QPainter) FillRect7(x int, y int, width int, height int, color core.Qt__GlobalColor) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect7(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.longlong(color))
	}
}

func (ptr *QPainter) FillRect8(rectangle core.QRect_ITF, color core.Qt__GlobalColor) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect8(ptr.Pointer(), core.PointerFromQRect(rectangle), C.longlong(color))
	}
}

func (ptr *QPainter) FillRect9(rectangle core.QRectF_ITF, color core.Qt__GlobalColor) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect9(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.longlong(color))
	}
}

func (ptr *QPainter) FillRect10(x int, y int, width int, height int, style core.Qt__BrushStyle) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect10(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.longlong(style))
	}
}

func (ptr *QPainter) FillRect11(rectangle core.QRect_ITF, style core.Qt__BrushStyle) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect11(ptr.Pointer(), core.PointerFromQRect(rectangle), C.longlong(style))
	}
}

func (ptr *QPainter) FillRect12(rectangle core.QRectF_ITF, style core.Qt__BrushStyle) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect12(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.longlong(style))
	}
}

func (ptr *QPainter) FillRect13(x int, y int, width int, height int, preset QGradient__Preset) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect13(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.longlong(preset))
	}
}

func (ptr *QPainter) FillRect14(rectangle core.QRect_ITF, preset QGradient__Preset) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect14(ptr.Pointer(), core.PointerFromQRect(rectangle), C.longlong(preset))
	}
}

func (ptr *QPainter) FillRect15(rectangle core.QRectF_ITF, preset QGradient__Preset) {
	if ptr.Pointer() != nil {
		C.QPainter_FillRect15(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.longlong(preset))
	}
}

func (ptr *QPainter) Font() *QFont {
	if ptr.Pointer() != nil {
		return NewQFontFromPointer(C.QPainter_Font(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPainter) FontInfo() *QFontInfo {
	if ptr.Pointer() != nil {
		tmpValue := NewQFontInfoFromPointer(C.QPainter_FontInfo(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QFontInfo).DestroyQFontInfo)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) FontMetrics() *QFontMetrics {
	if ptr.Pointer() != nil {
		tmpValue := NewQFontMetricsFromPointer(C.QPainter_FontMetrics(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QFontMetrics).DestroyQFontMetrics)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) HasClipping() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainter_HasClipping(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPainter) IsActive() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainter_IsActive(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPainter) LayoutDirection() core.Qt__LayoutDirection {
	if ptr.Pointer() != nil {
		return core.Qt__LayoutDirection(C.QPainter_LayoutDirection(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainter) Opacity() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainter_Opacity(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainter) PaintEngine() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPainter_PaintEngine(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPainter) Pen() *QPen {
	if ptr.Pointer() != nil {
		return NewQPenFromPointer(C.QPainter_Pen(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPainter) RenderHints() QPainter__RenderHint {
	if ptr.Pointer() != nil {
		return QPainter__RenderHint(C.QPainter_RenderHints(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainter) ResetTransform() {
	if ptr.Pointer() != nil {
		C.QPainter_ResetTransform(ptr.Pointer())
	}
}

func (ptr *QPainter) Restore() {
	if ptr.Pointer() != nil {
		C.QPainter_Restore(ptr.Pointer())
	}
}

func (ptr *QPainter) Rotate(angle float64) {
	if ptr.Pointer() != nil {
		C.QPainter_Rotate(ptr.Pointer(), C.double(angle))
	}
}

func (ptr *QPainter) Save() {
	if ptr.Pointer() != nil {
		C.QPainter_Save(ptr.Pointer())
	}
}

func (ptr *QPainter) Scale(sx float64, sy float64) {
	if ptr.Pointer() != nil {
		C.QPainter_Scale(ptr.Pointer(), C.double(sx), C.double(sy))
	}
}

func (ptr *QPainter) SetBackground(brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_SetBackground(ptr.Pointer(), PointerFromQBrush(brush))
	}
}

func (ptr *QPainter) SetBackgroundMode(mode core.Qt__BGMode) {
	if ptr.Pointer() != nil {
		C.QPainter_SetBackgroundMode(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QPainter) SetBrush(brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_SetBrush(ptr.Pointer(), PointerFromQBrush(brush))
	}
}

func (ptr *QPainter) SetBrush2(style core.Qt__BrushStyle) {
	if ptr.Pointer() != nil {
		C.QPainter_SetBrush2(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QPainter) SetBrushOrigin(position core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_SetBrushOrigin(ptr.Pointer(), core.PointerFromQPointF(position))
	}
}

func (ptr *QPainter) SetBrushOrigin2(x int, y int) {
	if ptr.Pointer() != nil {
		C.QPainter_SetBrushOrigin2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)))
	}
}

func (ptr *QPainter) SetBrushOrigin3(position core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_SetBrushOrigin3(ptr.Pointer(), core.PointerFromQPoint(position))
	}
}

func (ptr *QPainter) SetClipPath(path QPainterPath_ITF, operation core.Qt__ClipOperation) {
	if ptr.Pointer() != nil {
		C.QPainter_SetClipPath(ptr.Pointer(), PointerFromQPainterPath(path), C.longlong(operation))
	}
}

func (ptr *QPainter) SetClipRect(rectangle core.QRectF_ITF, operation core.Qt__ClipOperation) {
	if ptr.Pointer() != nil {
		C.QPainter_SetClipRect(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.longlong(operation))
	}
}

func (ptr *QPainter) SetClipRect2(rectangle core.QRect_ITF, operation core.Qt__ClipOperation) {
	if ptr.Pointer() != nil {
		C.QPainter_SetClipRect2(ptr.Pointer(), core.PointerFromQRect(rectangle), C.longlong(operation))
	}
}

func (ptr *QPainter) SetClipRect3(x int, y int, width int, height int, operation core.Qt__ClipOperation) {
	if ptr.Pointer() != nil {
		C.QPainter_SetClipRect3(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), C.longlong(operation))
	}
}

func (ptr *QPainter) SetClipRegion(region QRegion_ITF, operation core.Qt__ClipOperation) {
	if ptr.Pointer() != nil {
		C.QPainter_SetClipRegion(ptr.Pointer(), PointerFromQRegion(region), C.longlong(operation))
	}
}

func (ptr *QPainter) SetClipping(enable bool) {
	if ptr.Pointer() != nil {
		C.QPainter_SetClipping(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QPainter) SetCompositionMode(mode QPainter__CompositionMode) {
	if ptr.Pointer() != nil {
		C.QPainter_SetCompositionMode(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QPainter) SetFont(font QFont_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_SetFont(ptr.Pointer(), PointerFromQFont(font))
	}
}

func (ptr *QPainter) SetLayoutDirection(direction core.Qt__LayoutDirection) {
	if ptr.Pointer() != nil {
		C.QPainter_SetLayoutDirection(ptr.Pointer(), C.longlong(direction))
	}
}

func (ptr *QPainter) SetOpacity(opacity float64) {
	if ptr.Pointer() != nil {
		C.QPainter_SetOpacity(ptr.Pointer(), C.double(opacity))
	}
}

func (ptr *QPainter) SetPen(pen QPen_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_SetPen(ptr.Pointer(), PointerFromQPen(pen))
	}
}

func (ptr *QPainter) SetPen2(color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_SetPen2(ptr.Pointer(), PointerFromQColor(color))
	}
}

func (ptr *QPainter) SetPen3(style core.Qt__PenStyle) {
	if ptr.Pointer() != nil {
		C.QPainter_SetPen3(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QPainter) SetRenderHint(hint QPainter__RenderHint, on bool) {
	if ptr.Pointer() != nil {
		C.QPainter_SetRenderHint(ptr.Pointer(), C.longlong(hint), C.char(int8(qt.GoBoolToInt(on))))
	}
}

func (ptr *QPainter) SetRenderHints(hints QPainter__RenderHint, on bool) {
	if ptr.Pointer() != nil {
		C.QPainter_SetRenderHints(ptr.Pointer(), C.longlong(hints), C.char(int8(qt.GoBoolToInt(on))))
	}
}

func (ptr *QPainter) SetTransform(transform QTransform_ITF, combine bool) {
	if ptr.Pointer() != nil {
		C.QPainter_SetTransform(ptr.Pointer(), PointerFromQTransform(transform), C.char(int8(qt.GoBoolToInt(combine))))
	}
}

func (ptr *QPainter) SetViewTransformEnabled(enable bool) {
	if ptr.Pointer() != nil {
		C.QPainter_SetViewTransformEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QPainter) SetViewport(rectangle core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_SetViewport(ptr.Pointer(), core.PointerFromQRect(rectangle))
	}
}

func (ptr *QPainter) SetViewport2(x int, y int, width int, height int) {
	if ptr.Pointer() != nil {
		C.QPainter_SetViewport2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QPainter) SetWindow(rectangle core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_SetWindow(ptr.Pointer(), core.PointerFromQRect(rectangle))
	}
}

func (ptr *QPainter) SetWindow2(x int, y int, width int, height int) {
	if ptr.Pointer() != nil {
		C.QPainter_SetWindow2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)))
	}
}

func (ptr *QPainter) SetWorldMatrixEnabled(enable bool) {
	if ptr.Pointer() != nil {
		C.QPainter_SetWorldMatrixEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QPainter) SetWorldTransform(matrix QTransform_ITF, combine bool) {
	if ptr.Pointer() != nil {
		C.QPainter_SetWorldTransform(ptr.Pointer(), PointerFromQTransform(matrix), C.char(int8(qt.GoBoolToInt(combine))))
	}
}

func (ptr *QPainter) Shear(sh float64, sv float64) {
	if ptr.Pointer() != nil {
		C.QPainter_Shear(ptr.Pointer(), C.double(sh), C.double(sv))
	}
}

func (ptr *QPainter) StrokePath(path QPainterPath_ITF, pen QPen_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_StrokePath(ptr.Pointer(), PointerFromQPainterPath(path), PointerFromQPen(pen))
	}
}

func (ptr *QPainter) TestRenderHint(hint QPainter__RenderHint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainter_TestRenderHint(ptr.Pointer(), C.longlong(hint))) != 0
	}
	return false
}

func (ptr *QPainter) Transform() *QTransform {
	if ptr.Pointer() != nil {
		return NewQTransformFromPointer(C.QPainter_Transform(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPainter) Translate(offset core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_Translate(ptr.Pointer(), core.PointerFromQPointF(offset))
	}
}

func (ptr *QPainter) Translate2(offset core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter_Translate2(ptr.Pointer(), core.PointerFromQPoint(offset))
	}
}

func (ptr *QPainter) Translate3(dx float64, dy float64) {
	if ptr.Pointer() != nil {
		C.QPainter_Translate3(ptr.Pointer(), C.double(dx), C.double(dy))
	}
}

func (ptr *QPainter) ViewTransformEnabled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainter_ViewTransformEnabled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPainter) Viewport() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPainter_Viewport(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) Window() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPainter_Window(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) WorldMatrixEnabled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainter_WorldMatrixEnabled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPainter) WorldTransform() *QTransform {
	if ptr.Pointer() != nil {
		return NewQTransformFromPointer(C.QPainter_WorldTransform(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPainter) DestroyQPainter() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPainter_DestroyQPainter(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPainter) __drawLines_lines_atList2(i int) *core.QLineF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQLineFFromPointer(C.QPainter___drawLines_lines_atList2(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QLineF).DestroyQLineF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) __drawLines_lines_setList2(i core.QLineF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter___drawLines_lines_setList2(ptr.Pointer(), core.PointerFromQLineF(i))
	}
}

func (ptr *QPainter) __drawLines_lines_newList2() unsafe.Pointer {
	return C.QPainter___drawLines_lines_newList2(ptr.Pointer())
}

func (ptr *QPainter) __drawLines_pointPairs_atList4(i int) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QPainter___drawLines_pointPairs_atList4(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) __drawLines_pointPairs_setList4(i core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter___drawLines_pointPairs_setList4(ptr.Pointer(), core.PointerFromQPointF(i))
	}
}

func (ptr *QPainter) __drawLines_pointPairs_newList4() unsafe.Pointer {
	return C.QPainter___drawLines_pointPairs_newList4(ptr.Pointer())
}

func (ptr *QPainter) __drawLines_lines_atList6(i int) *core.QLine {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQLineFromPointer(C.QPainter___drawLines_lines_atList6(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QLine).DestroyQLine)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) __drawLines_lines_setList6(i core.QLine_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter___drawLines_lines_setList6(ptr.Pointer(), core.PointerFromQLine(i))
	}
}

func (ptr *QPainter) __drawLines_lines_newList6() unsafe.Pointer {
	return C.QPainter___drawLines_lines_newList6(ptr.Pointer())
}

func (ptr *QPainter) __drawLines_pointPairs_atList8(i int) *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QPainter___drawLines_pointPairs_atList8(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) __drawLines_pointPairs_setList8(i core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter___drawLines_pointPairs_setList8(ptr.Pointer(), core.PointerFromQPoint(i))
	}
}

func (ptr *QPainter) __drawLines_pointPairs_newList8() unsafe.Pointer {
	return C.QPainter___drawLines_pointPairs_newList8(ptr.Pointer())
}

func (ptr *QPainter) __drawRects_rectangles_atList2(i int) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QPainter___drawRects_rectangles_atList2(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) __drawRects_rectangles_setList2(i core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter___drawRects_rectangles_setList2(ptr.Pointer(), core.PointerFromQRectF(i))
	}
}

func (ptr *QPainter) __drawRects_rectangles_newList2() unsafe.Pointer {
	return C.QPainter___drawRects_rectangles_newList2(ptr.Pointer())
}

func (ptr *QPainter) __drawRects_rectangles_atList4(i int) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPainter___drawRects_rectangles_atList4(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPainter) __drawRects_rectangles_setList4(i core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPainter___drawRects_rectangles_setList4(ptr.Pointer(), core.PointerFromQRect(i))
	}
}

func (ptr *QPainter) __drawRects_rectangles_newList4() unsafe.Pointer {
	return C.QPainter___drawRects_rectangles_newList4(ptr.Pointer())
}

type QPainterPath struct {
	ptr unsafe.Pointer
}

type QPainterPath_ITF interface {
	QPainterPath_PTR() *QPainterPath
}

func (ptr *QPainterPath) QPainterPath_PTR() *QPainterPath {
	return ptr
}

func (ptr *QPainterPath) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPainterPath) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPainterPath(ptr QPainterPath_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPainterPath_PTR().Pointer()
	}
	return nil
}

func NewQPainterPathFromPointer(ptr unsafe.Pointer) (n *QPainterPath) {
	n = new(QPainterPath)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QPainterPath__ElementType
//QPainterPath::ElementType
type QPainterPath__ElementType int64

const (
	QPainterPath__MoveToElement      QPainterPath__ElementType = QPainterPath__ElementType(0)
	QPainterPath__LineToElement      QPainterPath__ElementType = QPainterPath__ElementType(1)
	QPainterPath__CurveToElement     QPainterPath__ElementType = QPainterPath__ElementType(2)
	QPainterPath__CurveToDataElement QPainterPath__ElementType = QPainterPath__ElementType(3)
)

func NewQPainterPath() *QPainterPath {
	tmpValue := NewQPainterPathFromPointer(C.QPainterPath_NewQPainterPath())
	qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
	return tmpValue
}

func NewQPainterPath2(startPoint core.QPointF_ITF) *QPainterPath {
	tmpValue := NewQPainterPathFromPointer(C.QPainterPath_NewQPainterPath2(core.PointerFromQPointF(startPoint)))
	qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
	return tmpValue
}

func NewQPainterPath3(path QPainterPath_ITF) *QPainterPath {
	tmpValue := NewQPainterPathFromPointer(C.QPainterPath_NewQPainterPath3(PointerFromQPainterPath(path)))
	qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
	return tmpValue
}

func (ptr *QPainterPath) AddEllipse(boundingRectangle core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_AddEllipse(ptr.Pointer(), core.PointerFromQRectF(boundingRectangle))
	}
}

func (ptr *QPainterPath) AddEllipse2(x float64, y float64, width float64, height float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_AddEllipse2(ptr.Pointer(), C.double(x), C.double(y), C.double(width), C.double(height))
	}
}

func (ptr *QPainterPath) AddEllipse3(center core.QPointF_ITF, rx float64, ry float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_AddEllipse3(ptr.Pointer(), core.PointerFromQPointF(center), C.double(rx), C.double(ry))
	}
}

func (ptr *QPainterPath) AddPath(path QPainterPath_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_AddPath(ptr.Pointer(), PointerFromQPainterPath(path))
	}
}

func (ptr *QPainterPath) AddPolygon(polygon QPolygonF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_AddPolygon(ptr.Pointer(), PointerFromQPolygonF(polygon))
	}
}

func (ptr *QPainterPath) AddRect(rectangle core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_AddRect(ptr.Pointer(), core.PointerFromQRectF(rectangle))
	}
}

func (ptr *QPainterPath) AddRect2(x float64, y float64, width float64, height float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_AddRect2(ptr.Pointer(), C.double(x), C.double(y), C.double(width), C.double(height))
	}
}

func (ptr *QPainterPath) AddRegion(region QRegion_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_AddRegion(ptr.Pointer(), PointerFromQRegion(region))
	}
}

func (ptr *QPainterPath) AddRoundedRect(rect core.QRectF_ITF, xRadius float64, yRadius float64, mode core.Qt__SizeMode) {
	if ptr.Pointer() != nil {
		C.QPainterPath_AddRoundedRect(ptr.Pointer(), core.PointerFromQRectF(rect), C.double(xRadius), C.double(yRadius), C.longlong(mode))
	}
}

func (ptr *QPainterPath) AddRoundedRect2(x float64, y float64, w float64, h float64, xRadius float64, yRadius float64, mode core.Qt__SizeMode) {
	if ptr.Pointer() != nil {
		C.QPainterPath_AddRoundedRect2(ptr.Pointer(), C.double(x), C.double(y), C.double(w), C.double(h), C.double(xRadius), C.double(yRadius), C.longlong(mode))
	}
}

func (ptr *QPainterPath) AddText(point core.QPointF_ITF, font QFont_ITF, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QPainterPath_AddText(ptr.Pointer(), core.PointerFromQPointF(point), PointerFromQFont(font), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QPainterPath) AddText2(x float64, y float64, font QFont_ITF, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QPainterPath_AddText2(ptr.Pointer(), C.double(x), C.double(y), PointerFromQFont(font), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QPainterPath) AngleAtPercent(t float64) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainterPath_AngleAtPercent(ptr.Pointer(), C.double(t)))
	}
	return 0
}

func (ptr *QPainterPath) ArcMoveTo(rectangle core.QRectF_ITF, angle float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_ArcMoveTo(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.double(angle))
	}
}

func (ptr *QPainterPath) ArcMoveTo2(x float64, y float64, width float64, height float64, angle float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_ArcMoveTo2(ptr.Pointer(), C.double(x), C.double(y), C.double(width), C.double(height), C.double(angle))
	}
}

func (ptr *QPainterPath) ArcTo(rectangle core.QRectF_ITF, startAngle float64, sweepLength float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_ArcTo(ptr.Pointer(), core.PointerFromQRectF(rectangle), C.double(startAngle), C.double(sweepLength))
	}
}

func (ptr *QPainterPath) ArcTo2(x float64, y float64, width float64, height float64, startAngle float64, sweepLength float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_ArcTo2(ptr.Pointer(), C.double(x), C.double(y), C.double(width), C.double(height), C.double(startAngle), C.double(sweepLength))
	}
}

func (ptr *QPainterPath) BoundingRect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QPainterPath_BoundingRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) Capacity() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPainterPath_Capacity(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPainterPath) Clear() {
	if ptr.Pointer() != nil {
		C.QPainterPath_Clear(ptr.Pointer())
	}
}

func (ptr *QPainterPath) CloseSubpath() {
	if ptr.Pointer() != nil {
		C.QPainterPath_CloseSubpath(ptr.Pointer())
	}
}

func (ptr *QPainterPath) ConnectPath(path QPainterPath_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_ConnectPath(ptr.Pointer(), PointerFromQPainterPath(path))
	}
}

func (ptr *QPainterPath) Contains(point core.QPointF_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainterPath_Contains(ptr.Pointer(), core.PointerFromQPointF(point))) != 0
	}
	return false
}

func (ptr *QPainterPath) Contains2(rectangle core.QRectF_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainterPath_Contains2(ptr.Pointer(), core.PointerFromQRectF(rectangle))) != 0
	}
	return false
}

func (ptr *QPainterPath) Contains3(p QPainterPath_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainterPath_Contains3(ptr.Pointer(), PointerFromQPainterPath(p))) != 0
	}
	return false
}

func (ptr *QPainterPath) ControlPointRect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QPainterPath_ControlPointRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) CubicTo(c1 core.QPointF_ITF, c2 core.QPointF_ITF, endPoint core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_CubicTo(ptr.Pointer(), core.PointerFromQPointF(c1), core.PointerFromQPointF(c2), core.PointerFromQPointF(endPoint))
	}
}

func (ptr *QPainterPath) CubicTo2(c1X float64, c1Y float64, c2X float64, c2Y float64, endPointX float64, endPointY float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_CubicTo2(ptr.Pointer(), C.double(c1X), C.double(c1Y), C.double(c2X), C.double(c2Y), C.double(endPointX), C.double(endPointY))
	}
}

func (ptr *QPainterPath) CurrentPosition() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QPainterPath_CurrentPosition(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) ElementCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPainterPath_ElementCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPainterPath) FillRule() core.Qt__FillRule {
	if ptr.Pointer() != nil {
		return core.Qt__FillRule(C.QPainterPath_FillRule(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainterPath) Intersected(p QPainterPath_ITF) *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QPainterPath_Intersected(ptr.Pointer(), PointerFromQPainterPath(p)))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) Intersects(rectangle core.QRectF_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainterPath_Intersects(ptr.Pointer(), core.PointerFromQRectF(rectangle))) != 0
	}
	return false
}

func (ptr *QPainterPath) Intersects2(p QPainterPath_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainterPath_Intersects2(ptr.Pointer(), PointerFromQPainterPath(p))) != 0
	}
	return false
}

func (ptr *QPainterPath) IsEmpty() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPainterPath_IsEmpty(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPainterPath) Length() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainterPath_Length(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainterPath) LineTo(endPoint core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_LineTo(ptr.Pointer(), core.PointerFromQPointF(endPoint))
	}
}

func (ptr *QPainterPath) LineTo2(x float64, y float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_LineTo2(ptr.Pointer(), C.double(x), C.double(y))
	}
}

func (ptr *QPainterPath) MoveTo(point core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_MoveTo(ptr.Pointer(), core.PointerFromQPointF(point))
	}
}

func (ptr *QPainterPath) MoveTo2(x float64, y float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_MoveTo2(ptr.Pointer(), C.double(x), C.double(y))
	}
}

func (ptr *QPainterPath) PercentAtLength(l float64) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainterPath_PercentAtLength(ptr.Pointer(), C.double(l)))
	}
	return 0
}

func (ptr *QPainterPath) PointAtPercent(t float64) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QPainterPath_PointAtPercent(ptr.Pointer(), C.double(t)))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) QuadTo(c core.QPointF_ITF, endPoint core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_QuadTo(ptr.Pointer(), core.PointerFromQPointF(c), core.PointerFromQPointF(endPoint))
	}
}

func (ptr *QPainterPath) QuadTo2(cx float64, cy float64, endPointX float64, endPointY float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_QuadTo2(ptr.Pointer(), C.double(cx), C.double(cy), C.double(endPointX), C.double(endPointY))
	}
}

func (ptr *QPainterPath) Reserve(size int) {
	if ptr.Pointer() != nil {
		C.QPainterPath_Reserve(ptr.Pointer(), C.int(int32(size)))
	}
}

func (ptr *QPainterPath) SetElementPositionAt(index int, x float64, y float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_SetElementPositionAt(ptr.Pointer(), C.int(int32(index)), C.double(x), C.double(y))
	}
}

func (ptr *QPainterPath) SetFillRule(fillRule core.Qt__FillRule) {
	if ptr.Pointer() != nil {
		C.QPainterPath_SetFillRule(ptr.Pointer(), C.longlong(fillRule))
	}
}

func (ptr *QPainterPath) Simplified() *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QPainterPath_Simplified(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) SlopeAtPercent(t float64) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainterPath_SlopeAtPercent(ptr.Pointer(), C.double(t)))
	}
	return 0
}

func (ptr *QPainterPath) Subtracted(p QPainterPath_ITF) *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QPainterPath_Subtracted(ptr.Pointer(), PointerFromQPainterPath(p)))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) Swap(other QPainterPath_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_Swap(ptr.Pointer(), PointerFromQPainterPath(other))
	}
}

func (ptr *QPainterPath) ToFillPolygon(matrix QTransform_ITF) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPainterPath_ToFillPolygon(ptr.Pointer(), PointerFromQTransform(matrix)))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) ToFillPolygon2(matrix QMatrix_ITF) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPainterPath_ToFillPolygon2(ptr.Pointer(), PointerFromQMatrix(matrix)))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) ToFillPolygons(matrix QTransform_ITF) []*QPolygonF {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QPolygonF {
			out := make([]*QPolygonF, int(l.len))
			tmpList := NewQPainterPathFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__toFillPolygons_atList(i)
			}
			return out
		}(C.QPainterPath_ToFillPolygons(ptr.Pointer(), PointerFromQTransform(matrix)))
	}
	return make([]*QPolygonF, 0)
}

func (ptr *QPainterPath) ToFillPolygons2(matrix QMatrix_ITF) []*QPolygonF {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QPolygonF {
			out := make([]*QPolygonF, int(l.len))
			tmpList := NewQPainterPathFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__toFillPolygons_atList2(i)
			}
			return out
		}(C.QPainterPath_ToFillPolygons2(ptr.Pointer(), PointerFromQMatrix(matrix)))
	}
	return make([]*QPolygonF, 0)
}

func (ptr *QPainterPath) ToReversed() *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QPainterPath_ToReversed(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) ToSubpathPolygons(matrix QTransform_ITF) []*QPolygonF {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QPolygonF {
			out := make([]*QPolygonF, int(l.len))
			tmpList := NewQPainterPathFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__toSubpathPolygons_atList(i)
			}
			return out
		}(C.QPainterPath_ToSubpathPolygons(ptr.Pointer(), PointerFromQTransform(matrix)))
	}
	return make([]*QPolygonF, 0)
}

func (ptr *QPainterPath) ToSubpathPolygons2(matrix QMatrix_ITF) []*QPolygonF {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QPolygonF {
			out := make([]*QPolygonF, int(l.len))
			tmpList := NewQPainterPathFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__toSubpathPolygons_atList2(i)
			}
			return out
		}(C.QPainterPath_ToSubpathPolygons2(ptr.Pointer(), PointerFromQMatrix(matrix)))
	}
	return make([]*QPolygonF, 0)
}

func (ptr *QPainterPath) Translate(dx float64, dy float64) {
	if ptr.Pointer() != nil {
		C.QPainterPath_Translate(ptr.Pointer(), C.double(dx), C.double(dy))
	}
}

func (ptr *QPainterPath) Translate2(offset core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath_Translate2(ptr.Pointer(), core.PointerFromQPointF(offset))
	}
}

func (ptr *QPainterPath) Translated(dx float64, dy float64) *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QPainterPath_Translated(ptr.Pointer(), C.double(dx), C.double(dy)))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) Translated2(offset core.QPointF_ITF) *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QPainterPath_Translated2(ptr.Pointer(), core.PointerFromQPointF(offset)))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) United(p QPainterPath_ITF) *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QPainterPath_United(ptr.Pointer(), PointerFromQPainterPath(p)))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) DestroyQPainterPath() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPainterPath_DestroyQPainterPath(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPainterPath) __toFillPolygons_atList(i int) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPainterPath___toFillPolygons_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) __toFillPolygons_setList(i QPolygonF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath___toFillPolygons_setList(ptr.Pointer(), PointerFromQPolygonF(i))
	}
}

func (ptr *QPainterPath) __toFillPolygons_newList() unsafe.Pointer {
	return C.QPainterPath___toFillPolygons_newList(ptr.Pointer())
}

func (ptr *QPainterPath) __toFillPolygons_atList2(i int) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPainterPath___toFillPolygons_atList2(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) __toFillPolygons_setList2(i QPolygonF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath___toFillPolygons_setList2(ptr.Pointer(), PointerFromQPolygonF(i))
	}
}

func (ptr *QPainterPath) __toFillPolygons_newList2() unsafe.Pointer {
	return C.QPainterPath___toFillPolygons_newList2(ptr.Pointer())
}

func (ptr *QPainterPath) __toSubpathPolygons_atList(i int) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPainterPath___toSubpathPolygons_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) __toSubpathPolygons_setList(i QPolygonF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath___toSubpathPolygons_setList(ptr.Pointer(), PointerFromQPolygonF(i))
	}
}

func (ptr *QPainterPath) __toSubpathPolygons_newList() unsafe.Pointer {
	return C.QPainterPath___toSubpathPolygons_newList(ptr.Pointer())
}

func (ptr *QPainterPath) __toSubpathPolygons_atList2(i int) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPainterPath___toSubpathPolygons_atList2(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPath) __toSubpathPolygons_setList2(i QPolygonF_ITF) {
	if ptr.Pointer() != nil {
		C.QPainterPath___toSubpathPolygons_setList2(ptr.Pointer(), PointerFromQPolygonF(i))
	}
}

func (ptr *QPainterPath) __toSubpathPolygons_newList2() unsafe.Pointer {
	return C.QPainterPath___toSubpathPolygons_newList2(ptr.Pointer())
}

type QPainterPathStroker struct {
	ptr unsafe.Pointer
}

type QPainterPathStroker_ITF interface {
	QPainterPathStroker_PTR() *QPainterPathStroker
}

func (ptr *QPainterPathStroker) QPainterPathStroker_PTR() *QPainterPathStroker {
	return ptr
}

func (ptr *QPainterPathStroker) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPainterPathStroker) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPainterPathStroker(ptr QPainterPathStroker_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPainterPathStroker_PTR().Pointer()
	}
	return nil
}

func NewQPainterPathStrokerFromPointer(ptr unsafe.Pointer) (n *QPainterPathStroker) {
	n = new(QPainterPathStroker)
	n.SetPointer(ptr)
	return
}
func NewQPainterPathStroker() *QPainterPathStroker {
	tmpValue := NewQPainterPathStrokerFromPointer(C.QPainterPathStroker_NewQPainterPathStroker())
	qt.SetFinalizer(tmpValue, (*QPainterPathStroker).DestroyQPainterPathStroker)
	return tmpValue
}

func NewQPainterPathStroker2(pen QPen_ITF) *QPainterPathStroker {
	tmpValue := NewQPainterPathStrokerFromPointer(C.QPainterPathStroker_NewQPainterPathStroker2(PointerFromQPen(pen)))
	qt.SetFinalizer(tmpValue, (*QPainterPathStroker).DestroyQPainterPathStroker)
	return tmpValue
}

func (ptr *QPainterPathStroker) CapStyle() core.Qt__PenCapStyle {
	if ptr.Pointer() != nil {
		return core.Qt__PenCapStyle(C.QPainterPathStroker_CapStyle(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainterPathStroker) CreateStroke(path QPainterPath_ITF) *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QPainterPathStroker_CreateStroke(ptr.Pointer(), PointerFromQPainterPath(path)))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QPainterPathStroker) CurveThreshold() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainterPathStroker_CurveThreshold(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainterPathStroker) DashOffset() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainterPathStroker_DashOffset(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainterPathStroker) DashPattern() []float64 {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []float64 {
			out := make([]float64, int(l.len))
			tmpList := NewQPainterPathStrokerFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__dashPattern_atList(i)
			}
			return out
		}(C.QPainterPathStroker_DashPattern(ptr.Pointer()))
	}
	return make([]float64, 0)
}

func (ptr *QPainterPathStroker) JoinStyle() core.Qt__PenJoinStyle {
	if ptr.Pointer() != nil {
		return core.Qt__PenJoinStyle(C.QPainterPathStroker_JoinStyle(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainterPathStroker) MiterLimit() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainterPathStroker_MiterLimit(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainterPathStroker) SetCapStyle(style core.Qt__PenCapStyle) {
	if ptr.Pointer() != nil {
		C.QPainterPathStroker_SetCapStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QPainterPathStroker) SetCurveThreshold(threshold float64) {
	if ptr.Pointer() != nil {
		C.QPainterPathStroker_SetCurveThreshold(ptr.Pointer(), C.double(threshold))
	}
}

func (ptr *QPainterPathStroker) SetDashOffset(offset float64) {
	if ptr.Pointer() != nil {
		C.QPainterPathStroker_SetDashOffset(ptr.Pointer(), C.double(offset))
	}
}

func (ptr *QPainterPathStroker) SetDashPattern(style core.Qt__PenStyle) {
	if ptr.Pointer() != nil {
		C.QPainterPathStroker_SetDashPattern(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QPainterPathStroker) SetDashPattern2(dashPattern []float64) {
	if ptr.Pointer() != nil {
		C.QPainterPathStroker_SetDashPattern2(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQPainterPathStrokerFromPointer(NewQPainterPathStrokerFromPointer(nil).__setDashPattern_dashPattern_newList2())
			for _, v := range dashPattern {
				tmpList.__setDashPattern_dashPattern_setList2(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QPainterPathStroker) SetJoinStyle(style core.Qt__PenJoinStyle) {
	if ptr.Pointer() != nil {
		C.QPainterPathStroker_SetJoinStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QPainterPathStroker) SetMiterLimit(limit float64) {
	if ptr.Pointer() != nil {
		C.QPainterPathStroker_SetMiterLimit(ptr.Pointer(), C.double(limit))
	}
}

func (ptr *QPainterPathStroker) SetWidth(width float64) {
	if ptr.Pointer() != nil {
		C.QPainterPathStroker_SetWidth(ptr.Pointer(), C.double(width))
	}
}

func (ptr *QPainterPathStroker) Width() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainterPathStroker_Width(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPainterPathStroker) DestroyQPainterPathStroker() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPainterPathStroker_DestroyQPainterPathStroker(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPainterPathStroker) __dashPattern_atList(i int) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainterPathStroker___dashPattern_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QPainterPathStroker) __dashPattern_setList(i float64) {
	if ptr.Pointer() != nil {
		C.QPainterPathStroker___dashPattern_setList(ptr.Pointer(), C.double(i))
	}
}

func (ptr *QPainterPathStroker) __dashPattern_newList() unsafe.Pointer {
	return C.QPainterPathStroker___dashPattern_newList(ptr.Pointer())
}

func (ptr *QPainterPathStroker) __setDashPattern_dashPattern_atList2(i int) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPainterPathStroker___setDashPattern_dashPattern_atList2(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QPainterPathStroker) __setDashPattern_dashPattern_setList2(i float64) {
	if ptr.Pointer() != nil {
		C.QPainterPathStroker___setDashPattern_dashPattern_setList2(ptr.Pointer(), C.double(i))
	}
}

func (ptr *QPainterPathStroker) __setDashPattern_dashPattern_newList2() unsafe.Pointer {
	return C.QPainterPathStroker___setDashPattern_dashPattern_newList2(ptr.Pointer())
}

type QPalette struct {
	ptr unsafe.Pointer
}

type QPalette_ITF interface {
	QPalette_PTR() *QPalette
}

func (ptr *QPalette) QPalette_PTR() *QPalette {
	return ptr
}

func (ptr *QPalette) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPalette) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPalette(ptr QPalette_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPalette_PTR().Pointer()
	}
	return nil
}

func NewQPaletteFromPointer(ptr unsafe.Pointer) (n *QPalette) {
	n = new(QPalette)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QPalette__ColorGroup
//QPalette::ColorGroup
type QPalette__ColorGroup int64

const (
	QPalette__Active       QPalette__ColorGroup = QPalette__ColorGroup(0)
	QPalette__Disabled     QPalette__ColorGroup = QPalette__ColorGroup(1)
	QPalette__Inactive     QPalette__ColorGroup = QPalette__ColorGroup(2)
	QPalette__NColorGroups QPalette__ColorGroup = QPalette__ColorGroup(3)
	QPalette__Current      QPalette__ColorGroup = QPalette__ColorGroup(4)
	QPalette__All          QPalette__ColorGroup = QPalette__ColorGroup(5)
	QPalette__Normal       QPalette__ColorGroup = QPalette__ColorGroup(QPalette__Active)
)

//go:generate stringer -type=QPalette__ColorRole
//QPalette::ColorRole
type QPalette__ColorRole int64

var (
	QPalette__WindowText      QPalette__ColorRole = QPalette__ColorRole(0)
	QPalette__Button          QPalette__ColorRole = QPalette__ColorRole(1)
	QPalette__Light           QPalette__ColorRole = QPalette__ColorRole(2)
	QPalette__Midlight        QPalette__ColorRole = QPalette__ColorRole(3)
	QPalette__Dark            QPalette__ColorRole = QPalette__ColorRole(4)
	QPalette__Mid             QPalette__ColorRole = QPalette__ColorRole(5)
	QPalette__Text            QPalette__ColorRole = QPalette__ColorRole(6)
	QPalette__BrightText      QPalette__ColorRole = QPalette__ColorRole(7)
	QPalette__ButtonText      QPalette__ColorRole = QPalette__ColorRole(8)
	QPalette__Base            QPalette__ColorRole = QPalette__ColorRole(9)
	QPalette__Window          QPalette__ColorRole = QPalette__ColorRole(10)
	QPalette__Shadow          QPalette__ColorRole = QPalette__ColorRole(11)
	QPalette__Highlight       QPalette__ColorRole = QPalette__ColorRole(12)
	QPalette__HighlightedText QPalette__ColorRole = QPalette__ColorRole(13)
	QPalette__Link            QPalette__ColorRole = QPalette__ColorRole(14)
	QPalette__LinkVisited     QPalette__ColorRole = QPalette__ColorRole(15)
	QPalette__AlternateBase   QPalette__ColorRole = QPalette__ColorRole(16)
	QPalette__NoRole          QPalette__ColorRole = QPalette__ColorRole(17)
	QPalette__ToolTipBase     QPalette__ColorRole = QPalette__ColorRole(18)
	QPalette__ToolTipText     QPalette__ColorRole = QPalette__ColorRole(19)
	QPalette__PlaceholderText QPalette__ColorRole = QPalette__ColorRole(20)
	QPalette__NColorRoles     QPalette__ColorRole = QPalette__ColorRole(C.QPalette_NColorRoles_Type())
	QPalette__Foreground      QPalette__ColorRole = QPalette__ColorRole(QPalette__WindowText)
	QPalette__Background      QPalette__ColorRole = QPalette__ColorRole(QPalette__Window)
)

func NewQPalette() *QPalette {
	tmpValue := NewQPaletteFromPointer(C.QPalette_NewQPalette())
	qt.SetFinalizer(tmpValue, (*QPalette).DestroyQPalette)
	return tmpValue
}

func NewQPalette2(button QColor_ITF) *QPalette {
	tmpValue := NewQPaletteFromPointer(C.QPalette_NewQPalette2(PointerFromQColor(button)))
	qt.SetFinalizer(tmpValue, (*QPalette).DestroyQPalette)
	return tmpValue
}

func NewQPalette3(button core.Qt__GlobalColor) *QPalette {
	tmpValue := NewQPaletteFromPointer(C.QPalette_NewQPalette3(C.longlong(button)))
	qt.SetFinalizer(tmpValue, (*QPalette).DestroyQPalette)
	return tmpValue
}

func NewQPalette4(button QColor_ITF, window QColor_ITF) *QPalette {
	tmpValue := NewQPaletteFromPointer(C.QPalette_NewQPalette4(PointerFromQColor(button), PointerFromQColor(window)))
	qt.SetFinalizer(tmpValue, (*QPalette).DestroyQPalette)
	return tmpValue
}

func NewQPalette5(windowText QBrush_ITF, button QBrush_ITF, light QBrush_ITF, dark QBrush_ITF, mid QBrush_ITF, text QBrush_ITF, bright_text QBrush_ITF, base QBrush_ITF, window QBrush_ITF) *QPalette {
	tmpValue := NewQPaletteFromPointer(C.QPalette_NewQPalette5(PointerFromQBrush(windowText), PointerFromQBrush(button), PointerFromQBrush(light), PointerFromQBrush(dark), PointerFromQBrush(mid), PointerFromQBrush(text), PointerFromQBrush(bright_text), PointerFromQBrush(base), PointerFromQBrush(window)))
	qt.SetFinalizer(tmpValue, (*QPalette).DestroyQPalette)
	return tmpValue
}

func NewQPalette7(p QPalette_ITF) *QPalette {
	tmpValue := NewQPaletteFromPointer(C.QPalette_NewQPalette7(PointerFromQPalette(p)))
	qt.SetFinalizer(tmpValue, (*QPalette).DestroyQPalette)
	return tmpValue
}

func NewQPalette8(other QPalette_ITF) *QPalette {
	tmpValue := NewQPaletteFromPointer(C.QPalette_NewQPalette8(PointerFromQPalette(other)))
	qt.SetFinalizer(tmpValue, (*QPalette).DestroyQPalette)
	return tmpValue
}

func (ptr *QPalette) AlternateBase() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_AlternateBase(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) Base() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Base(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) BrightText() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_BrightText(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) Brush(group QPalette__ColorGroup, role QPalette__ColorRole) *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Brush(ptr.Pointer(), C.longlong(group), C.longlong(role)))
	}
	return nil
}

func (ptr *QPalette) Brush2(role QPalette__ColorRole) *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Brush2(ptr.Pointer(), C.longlong(role)))
	}
	return nil
}

func (ptr *QPalette) Button() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Button(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) ButtonText() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_ButtonText(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) CacheKey() int64 {
	if ptr.Pointer() != nil {
		return int64(C.QPalette_CacheKey(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPalette) Color(group QPalette__ColorGroup, role QPalette__ColorRole) *QColor {
	if ptr.Pointer() != nil {
		return NewQColorFromPointer(C.QPalette_Color(ptr.Pointer(), C.longlong(group), C.longlong(role)))
	}
	return nil
}

func (ptr *QPalette) Color2(role QPalette__ColorRole) *QColor {
	if ptr.Pointer() != nil {
		return NewQColorFromPointer(C.QPalette_Color2(ptr.Pointer(), C.longlong(role)))
	}
	return nil
}

func (ptr *QPalette) CurrentColorGroup() QPalette__ColorGroup {
	if ptr.Pointer() != nil {
		return QPalette__ColorGroup(C.QPalette_CurrentColorGroup(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPalette) Dark() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Dark(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) Highlight() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Highlight(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) HighlightedText() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_HighlightedText(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) IsBrushSet(cg QPalette__ColorGroup, cr QPalette__ColorRole) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPalette_IsBrushSet(ptr.Pointer(), C.longlong(cg), C.longlong(cr))) != 0
	}
	return false
}

func (ptr *QPalette) IsCopyOf(p QPalette_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPalette_IsCopyOf(ptr.Pointer(), PointerFromQPalette(p))) != 0
	}
	return false
}

func (ptr *QPalette) IsEqual(cg1 QPalette__ColorGroup, cg2 QPalette__ColorGroup) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPalette_IsEqual(ptr.Pointer(), C.longlong(cg1), C.longlong(cg2))) != 0
	}
	return false
}

func (ptr *QPalette) Light() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Light(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) Link() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Link(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) LinkVisited() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_LinkVisited(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) Mid() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Mid(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) Midlight() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Midlight(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) PlaceholderText() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_PlaceholderText(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) Resolve(other QPalette_ITF) *QPalette {
	if ptr.Pointer() != nil {
		tmpValue := NewQPaletteFromPointer(C.QPalette_Resolve(ptr.Pointer(), PointerFromQPalette(other)))
		qt.SetFinalizer(tmpValue, (*QPalette).DestroyQPalette)
		return tmpValue
	}
	return nil
}

func (ptr *QPalette) SetBrush(role QPalette__ColorRole, brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QPalette_SetBrush(ptr.Pointer(), C.longlong(role), PointerFromQBrush(brush))
	}
}

func (ptr *QPalette) SetBrush2(group QPalette__ColorGroup, role QPalette__ColorRole, brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QPalette_SetBrush2(ptr.Pointer(), C.longlong(group), C.longlong(role), PointerFromQBrush(brush))
	}
}

func (ptr *QPalette) SetColor(group QPalette__ColorGroup, role QPalette__ColorRole, color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QPalette_SetColor(ptr.Pointer(), C.longlong(group), C.longlong(role), PointerFromQColor(color))
	}
}

func (ptr *QPalette) SetColor2(role QPalette__ColorRole, color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QPalette_SetColor2(ptr.Pointer(), C.longlong(role), PointerFromQColor(color))
	}
}

func (ptr *QPalette) SetColorGroup(cg QPalette__ColorGroup, windowText QBrush_ITF, button QBrush_ITF, light QBrush_ITF, dark QBrush_ITF, mid QBrush_ITF, text QBrush_ITF, bright_text QBrush_ITF, base QBrush_ITF, window QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QPalette_SetColorGroup(ptr.Pointer(), C.longlong(cg), PointerFromQBrush(windowText), PointerFromQBrush(button), PointerFromQBrush(light), PointerFromQBrush(dark), PointerFromQBrush(mid), PointerFromQBrush(text), PointerFromQBrush(bright_text), PointerFromQBrush(base), PointerFromQBrush(window))
	}
}

func (ptr *QPalette) SetCurrentColorGroup(cg QPalette__ColorGroup) {
	if ptr.Pointer() != nil {
		C.QPalette_SetCurrentColorGroup(ptr.Pointer(), C.longlong(cg))
	}
}

func (ptr *QPalette) Shadow() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Shadow(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) Swap(other QPalette_ITF) {
	if ptr.Pointer() != nil {
		C.QPalette_Swap(ptr.Pointer(), PointerFromQPalette(other))
	}
}

func (ptr *QPalette) Text() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Text(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) ToolTipBase() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_ToolTipBase(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) ToolTipText() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_ToolTipText(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) Window() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_Window(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) WindowText() *QBrush {
	if ptr.Pointer() != nil {
		return NewQBrushFromPointer(C.QPalette_WindowText(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPalette) DestroyQPalette() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPalette_DestroyQPalette(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QPdfWriter struct {
	core.QObject
	QPagedPaintDevice
}

type QPdfWriter_ITF interface {
	core.QObject_ITF
	QPagedPaintDevice_ITF
	QPdfWriter_PTR() *QPdfWriter
}

func (ptr *QPdfWriter) QPdfWriter_PTR() *QPdfWriter {
	return ptr
}

func (ptr *QPdfWriter) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QPdfWriter) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
		ptr.QPagedPaintDevice_PTR().SetPointer(p)
	}
}

func PointerFromQPdfWriter(ptr QPdfWriter_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPdfWriter_PTR().Pointer()
	}
	return nil
}

func NewQPdfWriterFromPointer(ptr unsafe.Pointer) (n *QPdfWriter) {
	n = new(QPdfWriter)
	n.SetPointer(ptr)
	return
}
func NewQPdfWriter(filename string) *QPdfWriter {
	var filenameC *C.char
	if filename != "" {
		filenameC = C.CString(filename)
		defer C.free(unsafe.Pointer(filenameC))
	}
	tmpValue := NewQPdfWriterFromPointer(C.QPdfWriter_NewQPdfWriter(C.struct_QtGui_PackedString{data: filenameC, len: C.longlong(len(filename))}))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQPdfWriter2(device core.QIODevice_ITF) *QPdfWriter {
	tmpValue := NewQPdfWriterFromPointer(C.QPdfWriter_NewQPdfWriter2(core.PointerFromQIODevice(device)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QPdfWriter) Creator() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPdfWriter_Creator(ptr.Pointer()))
	}
	return ""
}

//export callbackQPdfWriter_NewPage
func callbackQPdfWriter_NewPage(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "newPage"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(NewQPdfWriterFromPointer(ptr).NewPageDefault())))
}

func (ptr *QPdfWriter) ConnectNewPage(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "newPage"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "newPage", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "newPage", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPdfWriter) DisconnectNewPage() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "newPage")
	}
}

func (ptr *QPdfWriter) NewPage() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPdfWriter_NewPage(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPdfWriter) NewPageDefault() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPdfWriter_NewPageDefault(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQPdfWriter_PaintEngine
func callbackQPdfWriter_PaintEngine(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "paintEngine"); signal != nil {
		return PointerFromQPaintEngine((*(*func() *QPaintEngine)(signal))())
	}

	return PointerFromQPaintEngine(NewQPdfWriterFromPointer(ptr).PaintEngineDefault())
}

func (ptr *QPdfWriter) ConnectPaintEngine(f func() *QPaintEngine) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "paintEngine"); signal != nil {
			f := func() *QPaintEngine {
				(*(*func() *QPaintEngine)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "paintEngine", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "paintEngine", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPdfWriter) DisconnectPaintEngine() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "paintEngine")
	}
}

func (ptr *QPdfWriter) PaintEngine() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPdfWriter_PaintEngine(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPdfWriter) PaintEngineDefault() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPdfWriter_PaintEngineDefault(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPdfWriter) PdfVersion() QPagedPaintDevice__PdfVersion {
	if ptr.Pointer() != nil {
		return QPagedPaintDevice__PdfVersion(C.QPdfWriter_PdfVersion(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPdfWriter) Resolution() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPdfWriter_Resolution(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPdfWriter) SetCreator(creator string) {
	if ptr.Pointer() != nil {
		var creatorC *C.char
		if creator != "" {
			creatorC = C.CString(creator)
			defer C.free(unsafe.Pointer(creatorC))
		}
		C.QPdfWriter_SetCreator(ptr.Pointer(), C.struct_QtGui_PackedString{data: creatorC, len: C.longlong(len(creator))})
	}
}

func (ptr *QPdfWriter) SetPdfVersion(version QPagedPaintDevice__PdfVersion) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_SetPdfVersion(ptr.Pointer(), C.longlong(version))
	}
}

func (ptr *QPdfWriter) SetResolution(resolution int) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_SetResolution(ptr.Pointer(), C.int(int32(resolution)))
	}
}

func (ptr *QPdfWriter) SetTitle(title string) {
	if ptr.Pointer() != nil {
		var titleC *C.char
		if title != "" {
			titleC = C.CString(title)
			defer C.free(unsafe.Pointer(titleC))
		}
		C.QPdfWriter_SetTitle(ptr.Pointer(), C.struct_QtGui_PackedString{data: titleC, len: C.longlong(len(title))})
	}
}

func (ptr *QPdfWriter) Title() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPdfWriter_Title(ptr.Pointer()))
	}
	return ""
}

//export callbackQPdfWriter_DestroyQPdfWriter
func callbackQPdfWriter_DestroyQPdfWriter(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QPdfWriter"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQPdfWriterFromPointer(ptr).DestroyQPdfWriterDefault()
	}
}

func (ptr *QPdfWriter) ConnectDestroyQPdfWriter(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QPdfWriter"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QPdfWriter", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QPdfWriter", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPdfWriter) DisconnectDestroyQPdfWriter() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QPdfWriter")
	}
}

func (ptr *QPdfWriter) DestroyQPdfWriter() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPdfWriter_DestroyQPdfWriter(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPdfWriter) DestroyQPdfWriterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPdfWriter_DestroyQPdfWriterDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPdfWriter) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPdfWriter___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPdfWriter) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPdfWriter) __children_newList() unsafe.Pointer {
	return C.QPdfWriter___children_newList(ptr.Pointer())
}

func (ptr *QPdfWriter) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QPdfWriter___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QPdfWriter) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QPdfWriter) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QPdfWriter___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QPdfWriter) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPdfWriter___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPdfWriter) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPdfWriter) __findChildren_newList() unsafe.Pointer {
	return C.QPdfWriter___findChildren_newList(ptr.Pointer())
}

func (ptr *QPdfWriter) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPdfWriter___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPdfWriter) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPdfWriter) __findChildren_newList3() unsafe.Pointer {
	return C.QPdfWriter___findChildren_newList3(ptr.Pointer())
}

//export callbackQPdfWriter_ChildEvent
func callbackQPdfWriter_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQPdfWriterFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QPdfWriter) ChildEvent(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_ChildEvent(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

func (ptr *QPdfWriter) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQPdfWriter_ConnectNotify
func callbackQPdfWriter_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQPdfWriterFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QPdfWriter) ConnectNotify(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_ConnectNotify(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

func (ptr *QPdfWriter) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQPdfWriter_CustomEvent
func callbackQPdfWriter_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQPdfWriterFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QPdfWriter) CustomEvent(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_CustomEvent(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

func (ptr *QPdfWriter) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQPdfWriter_DeleteLater
func callbackQPdfWriter_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQPdfWriterFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QPdfWriter) DeleteLater() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPdfWriter_DeleteLater(ptr.Pointer())
	}
}

func (ptr *QPdfWriter) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPdfWriter_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQPdfWriter_Destroyed
func callbackQPdfWriter_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQPdfWriter_DisconnectNotify
func callbackQPdfWriter_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQPdfWriterFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QPdfWriter) DisconnectNotify(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_DisconnectNotify(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

func (ptr *QPdfWriter) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQPdfWriter_Event
func callbackQPdfWriter_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQPdfWriterFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QPdfWriter) Event(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPdfWriter_Event(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

func (ptr *QPdfWriter) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPdfWriter_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQPdfWriter_EventFilter
func callbackQPdfWriter_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQPdfWriterFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QPdfWriter) EventFilter(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPdfWriter_EventFilter(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

func (ptr *QPdfWriter) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPdfWriter_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQPdfWriter_MetaObject
func callbackQPdfWriter_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQPdfWriterFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QPdfWriter) MetaObject() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QPdfWriter_MetaObject(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPdfWriter) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QPdfWriter_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQPdfWriter_ObjectNameChanged
func callbackQPdfWriter_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQPdfWriter_TimerEvent
func callbackQPdfWriter_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQPdfWriterFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QPdfWriter) TimerEvent(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_TimerEvent(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

func (ptr *QPdfWriter) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPdfWriter_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QPen struct {
	ptr unsafe.Pointer
}

type QPen_ITF interface {
	QPen_PTR() *QPen
}

func (ptr *QPen) QPen_PTR() *QPen {
	return ptr
}

func (ptr *QPen) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPen) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPen(ptr QPen_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPen_PTR().Pointer()
	}
	return nil
}

func NewQPenFromPointer(ptr unsafe.Pointer) (n *QPen) {
	n = new(QPen)
	n.SetPointer(ptr)
	return
}
func NewQPen() *QPen {
	tmpValue := NewQPenFromPointer(C.QPen_NewQPen())
	qt.SetFinalizer(tmpValue, (*QPen).DestroyQPen)
	return tmpValue
}

func NewQPen2(style core.Qt__PenStyle) *QPen {
	tmpValue := NewQPenFromPointer(C.QPen_NewQPen2(C.longlong(style)))
	qt.SetFinalizer(tmpValue, (*QPen).DestroyQPen)
	return tmpValue
}

func NewQPen3(color QColor_ITF) *QPen {
	tmpValue := NewQPenFromPointer(C.QPen_NewQPen3(PointerFromQColor(color)))
	qt.SetFinalizer(tmpValue, (*QPen).DestroyQPen)
	return tmpValue
}

func NewQPen4(brush QBrush_ITF, width float64, style core.Qt__PenStyle, cap core.Qt__PenCapStyle, join core.Qt__PenJoinStyle) *QPen {
	tmpValue := NewQPenFromPointer(C.QPen_NewQPen4(PointerFromQBrush(brush), C.double(width), C.longlong(style), C.longlong(cap), C.longlong(join)))
	qt.SetFinalizer(tmpValue, (*QPen).DestroyQPen)
	return tmpValue
}

func NewQPen5(pen QPen_ITF) *QPen {
	tmpValue := NewQPenFromPointer(C.QPen_NewQPen5(PointerFromQPen(pen)))
	qt.SetFinalizer(tmpValue, (*QPen).DestroyQPen)
	return tmpValue
}

func NewQPen6(pen QPen_ITF) *QPen {
	tmpValue := NewQPenFromPointer(C.QPen_NewQPen6(PointerFromQPen(pen)))
	qt.SetFinalizer(tmpValue, (*QPen).DestroyQPen)
	return tmpValue
}

func (ptr *QPen) Brush() *QBrush {
	if ptr.Pointer() != nil {
		tmpValue := NewQBrushFromPointer(C.QPen_Brush(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
		return tmpValue
	}
	return nil
}

func (ptr *QPen) CapStyle() core.Qt__PenCapStyle {
	if ptr.Pointer() != nil {
		return core.Qt__PenCapStyle(C.QPen_CapStyle(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPen) Color() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QPen_Color(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QPen) DashOffset() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPen_DashOffset(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPen) DashPattern() []float64 {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []float64 {
			out := make([]float64, int(l.len))
			tmpList := NewQPenFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__dashPattern_atList(i)
			}
			return out
		}(C.QPen_DashPattern(ptr.Pointer()))
	}
	return make([]float64, 0)
}

func (ptr *QPen) IsCosmetic() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPen_IsCosmetic(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPen) IsSolid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPen_IsSolid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPen) JoinStyle() core.Qt__PenJoinStyle {
	if ptr.Pointer() != nil {
		return core.Qt__PenJoinStyle(C.QPen_JoinStyle(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPen) MiterLimit() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPen_MiterLimit(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPen) SetBrush(brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QPen_SetBrush(ptr.Pointer(), PointerFromQBrush(brush))
	}
}

func (ptr *QPen) SetCapStyle(style core.Qt__PenCapStyle) {
	if ptr.Pointer() != nil {
		C.QPen_SetCapStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QPen) SetColor(color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QPen_SetColor(ptr.Pointer(), PointerFromQColor(color))
	}
}

func (ptr *QPen) SetCosmetic(cosmetic bool) {
	if ptr.Pointer() != nil {
		C.QPen_SetCosmetic(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(cosmetic))))
	}
}

func (ptr *QPen) SetDashOffset(offset float64) {
	if ptr.Pointer() != nil {
		C.QPen_SetDashOffset(ptr.Pointer(), C.double(offset))
	}
}

func (ptr *QPen) SetDashPattern(pattern []float64) {
	if ptr.Pointer() != nil {
		C.QPen_SetDashPattern(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQPenFromPointer(NewQPenFromPointer(nil).__setDashPattern_pattern_newList())
			for _, v := range pattern {
				tmpList.__setDashPattern_pattern_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QPen) SetJoinStyle(style core.Qt__PenJoinStyle) {
	if ptr.Pointer() != nil {
		C.QPen_SetJoinStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QPen) SetMiterLimit(limit float64) {
	if ptr.Pointer() != nil {
		C.QPen_SetMiterLimit(ptr.Pointer(), C.double(limit))
	}
}

func (ptr *QPen) SetStyle(style core.Qt__PenStyle) {
	if ptr.Pointer() != nil {
		C.QPen_SetStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QPen) SetWidth(width int) {
	if ptr.Pointer() != nil {
		C.QPen_SetWidth(ptr.Pointer(), C.int(int32(width)))
	}
}

func (ptr *QPen) SetWidthF(width float64) {
	if ptr.Pointer() != nil {
		C.QPen_SetWidthF(ptr.Pointer(), C.double(width))
	}
}

func (ptr *QPen) Style() core.Qt__PenStyle {
	if ptr.Pointer() != nil {
		return core.Qt__PenStyle(C.QPen_Style(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPen) Swap(other QPen_ITF) {
	if ptr.Pointer() != nil {
		C.QPen_Swap(ptr.Pointer(), PointerFromQPen(other))
	}
}

func (ptr *QPen) Width() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPen_Width(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPen) WidthF() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPen_WidthF(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPen) DestroyQPen() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPen_DestroyQPen(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPen) __dashPattern_atList(i int) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPen___dashPattern_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QPen) __dashPattern_setList(i float64) {
	if ptr.Pointer() != nil {
		C.QPen___dashPattern_setList(ptr.Pointer(), C.double(i))
	}
}

func (ptr *QPen) __dashPattern_newList() unsafe.Pointer {
	return C.QPen___dashPattern_newList(ptr.Pointer())
}

func (ptr *QPen) __setDashPattern_pattern_atList(i int) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPen___setDashPattern_pattern_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QPen) __setDashPattern_pattern_setList(i float64) {
	if ptr.Pointer() != nil {
		C.QPen___setDashPattern_pattern_setList(ptr.Pointer(), C.double(i))
	}
}

func (ptr *QPen) __setDashPattern_pattern_newList() unsafe.Pointer {
	return C.QPen___setDashPattern_pattern_newList(ptr.Pointer())
}

type QPicture struct {
	QPaintDevice
}

type QPicture_ITF interface {
	QPaintDevice_ITF
	QPicture_PTR() *QPicture
}

func (ptr *QPicture) QPicture_PTR() *QPicture {
	return ptr
}

func (ptr *QPicture) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintDevice_PTR().Pointer()
	}
	return nil
}

func (ptr *QPicture) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QPaintDevice_PTR().SetPointer(p)
	}
}

func PointerFromQPicture(ptr QPicture_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPicture_PTR().Pointer()
	}
	return nil
}

func NewQPictureFromPointer(ptr unsafe.Pointer) (n *QPicture) {
	n = new(QPicture)
	n.SetPointer(ptr)
	return
}
func NewQPicture(formatVersion int) *QPicture {
	return NewQPictureFromPointer(C.QPicture_NewQPicture(C.int(int32(formatVersion))))
}

func NewQPicture2(pic QPicture_ITF) *QPicture {
	return NewQPictureFromPointer(C.QPicture_NewQPicture2(PointerFromQPicture(pic)))
}

func (ptr *QPicture) BoundingRect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPicture_BoundingRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPicture) Data() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPicture_Data(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPicture) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPicture_IsNull(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPicture) Load(fileName string, format string) bool {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QPicture_Load(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, formatC)) != 0
	}
	return false
}

func (ptr *QPicture) Load2(dev core.QIODevice_ITF, format string) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QPicture_Load2(ptr.Pointer(), core.PointerFromQIODevice(dev), formatC)) != 0
	}
	return false
}

func (ptr *QPicture) Play(painter QPainter_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPicture_Play(ptr.Pointer(), PointerFromQPainter(painter))) != 0
	}
	return false
}

func (ptr *QPicture) Save(fileName string, format string) bool {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QPicture_Save(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, formatC)) != 0
	}
	return false
}

func (ptr *QPicture) Save2(dev core.QIODevice_ITF, format string) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QPicture_Save2(ptr.Pointer(), core.PointerFromQIODevice(dev), formatC)) != 0
	}
	return false
}

func (ptr *QPicture) SetBoundingRect(r core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QPicture_SetBoundingRect(ptr.Pointer(), core.PointerFromQRect(r))
	}
}

//export callbackQPicture_SetData
func callbackQPicture_SetData(ptr unsafe.Pointer, data C.struct_QtGui_PackedString, size C.uint) {
	if signal := qt.GetSignal(ptr, "setData"); signal != nil {
		(*(*func([]byte, uint))(signal))(cGoUnpackBytes(data), uint(uint32(size)))
	} else {
		NewQPictureFromPointer(ptr).SetDataDefault(cGoUnpackBytes(data), uint(uint32(size)))
	}
}

func (ptr *QPicture) ConnectSetData(f func(data []byte, size uint)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setData"); signal != nil {
			f := func(data []byte, size uint) {
				(*(*func([]byte, uint))(signal))(data, size)
				f(data, size)
			}
			qt.ConnectSignal(ptr.Pointer(), "setData", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setData", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPicture) DisconnectSetData() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setData")
	}
}

func (ptr *QPicture) SetData(data []byte, size uint) {
	if ptr.Pointer() != nil {
		var dataC *C.char
		if len(data) != 0 {
			dataC = (*C.char)(unsafe.Pointer(&data[0]))
		}
		C.QPicture_SetData(ptr.Pointer(), dataC, C.uint(uint32(size)))
	}
}

func (ptr *QPicture) SetDataDefault(data []byte, size uint) {
	if ptr.Pointer() != nil {
		var dataC *C.char
		if len(data) != 0 {
			dataC = (*C.char)(unsafe.Pointer(&data[0]))
		}
		C.QPicture_SetDataDefault(ptr.Pointer(), dataC, C.uint(uint32(size)))
	}
}

func (ptr *QPicture) Size() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QPicture_Size(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPicture) Swap(other QPicture_ITF) {
	if ptr.Pointer() != nil {
		C.QPicture_Swap(ptr.Pointer(), PointerFromQPicture(other))
	}
}

//export callbackQPicture_DestroyQPicture
func callbackQPicture_DestroyQPicture(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QPicture"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQPictureFromPointer(ptr).DestroyQPictureDefault()
	}
}

func (ptr *QPicture) ConnectDestroyQPicture(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QPicture"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QPicture", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QPicture", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPicture) DisconnectDestroyQPicture() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QPicture")
	}
}

func (ptr *QPicture) DestroyQPicture() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPicture_DestroyQPicture(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPicture) DestroyQPictureDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPicture_DestroyQPictureDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPicture) __inputFormats_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QPicture___inputFormats_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QPicture) __inputFormats_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QPicture___inputFormats_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QPicture) __inputFormats_newList() unsafe.Pointer {
	return C.QPicture___inputFormats_newList(ptr.Pointer())
}

func (ptr *QPicture) __outputFormats_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QPicture___outputFormats_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QPicture) __outputFormats_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QPicture___outputFormats_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QPicture) __outputFormats_newList() unsafe.Pointer {
	return C.QPicture___outputFormats_newList(ptr.Pointer())
}

//export callbackQPicture_PaintEngine
func callbackQPicture_PaintEngine(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "paintEngine"); signal != nil {
		return PointerFromQPaintEngine((*(*func() *QPaintEngine)(signal))())
	}

	return PointerFromQPaintEngine(NewQPictureFromPointer(ptr).PaintEngineDefault())
}

func (ptr *QPicture) PaintEngine() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPicture_PaintEngine(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPicture) PaintEngineDefault() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPicture_PaintEngineDefault(ptr.Pointer()))
	}
	return nil
}

type QPictureFormatPlugin struct {
	core.QObject
}

type QPictureFormatPlugin_ITF interface {
	core.QObject_ITF
	QPictureFormatPlugin_PTR() *QPictureFormatPlugin
}

func (ptr *QPictureFormatPlugin) QPictureFormatPlugin_PTR() *QPictureFormatPlugin {
	return ptr
}

func (ptr *QPictureFormatPlugin) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QPictureFormatPlugin) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQPictureFormatPlugin(ptr QPictureFormatPlugin_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPictureFormatPlugin_PTR().Pointer()
	}
	return nil
}

func NewQPictureFormatPluginFromPointer(ptr unsafe.Pointer) (n *QPictureFormatPlugin) {
	n = new(QPictureFormatPlugin)
	n.SetPointer(ptr)
	return
}
func NewQPictureFormatPlugin(parent core.QObject_ITF) *QPictureFormatPlugin {
	tmpValue := NewQPictureFormatPluginFromPointer(C.QPictureFormatPlugin_NewQPictureFormatPlugin(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQPictureFormatPlugin_InstallIOHandler
func callbackQPictureFormatPlugin_InstallIOHandler(ptr unsafe.Pointer, format C.struct_QtGui_PackedString) C.char {
	if signal := qt.GetSignal(ptr, "installIOHandler"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(string) bool)(signal))(cGoUnpackString(format)))))
	}

	return C.char(int8(qt.GoBoolToInt(false)))
}

func (ptr *QPictureFormatPlugin) ConnectInstallIOHandler(f func(format string) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "installIOHandler"); signal != nil {
			f := func(format string) bool {
				(*(*func(string) bool)(signal))(format)
				return f(format)
			}
			qt.ConnectSignal(ptr.Pointer(), "installIOHandler", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "installIOHandler", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPictureFormatPlugin) DisconnectInstallIOHandler() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "installIOHandler")
	}
}

func (ptr *QPictureFormatPlugin) InstallIOHandler(format string) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QPictureFormatPlugin_InstallIOHandler(ptr.Pointer(), C.struct_QtGui_PackedString{data: formatC, len: C.longlong(len(format))})) != 0
	}
	return false
}

//export callbackQPictureFormatPlugin_LoadPicture
func callbackQPictureFormatPlugin_LoadPicture(ptr unsafe.Pointer, format C.struct_QtGui_PackedString, fileName C.struct_QtGui_PackedString, picture unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "loadPicture"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(string, string, *QPicture) bool)(signal))(cGoUnpackString(format), cGoUnpackString(fileName), NewQPictureFromPointer(picture)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQPictureFormatPluginFromPointer(ptr).LoadPictureDefault(cGoUnpackString(format), cGoUnpackString(fileName), NewQPictureFromPointer(picture)))))
}

func (ptr *QPictureFormatPlugin) ConnectLoadPicture(f func(format string, fileName string, picture *QPicture) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "loadPicture"); signal != nil {
			f := func(format string, fileName string, picture *QPicture) bool {
				(*(*func(string, string, *QPicture) bool)(signal))(format, fileName, picture)
				return f(format, fileName, picture)
			}
			qt.ConnectSignal(ptr.Pointer(), "loadPicture", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "loadPicture", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPictureFormatPlugin) DisconnectLoadPicture() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "loadPicture")
	}
}

func (ptr *QPictureFormatPlugin) LoadPicture(format string, fileName string, picture QPicture_ITF) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		return int8(C.QPictureFormatPlugin_LoadPicture(ptr.Pointer(), C.struct_QtGui_PackedString{data: formatC, len: C.longlong(len(format))}, C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, PointerFromQPicture(picture))) != 0
	}
	return false
}

func (ptr *QPictureFormatPlugin) LoadPictureDefault(format string, fileName string, picture QPicture_ITF) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		return int8(C.QPictureFormatPlugin_LoadPictureDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: formatC, len: C.longlong(len(format))}, C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, PointerFromQPicture(picture))) != 0
	}
	return false
}

//export callbackQPictureFormatPlugin_SavePicture
func callbackQPictureFormatPlugin_SavePicture(ptr unsafe.Pointer, format C.struct_QtGui_PackedString, fileName C.struct_QtGui_PackedString, picture unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "savePicture"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(string, string, *QPicture) bool)(signal))(cGoUnpackString(format), cGoUnpackString(fileName), NewQPictureFromPointer(picture)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQPictureFormatPluginFromPointer(ptr).SavePictureDefault(cGoUnpackString(format), cGoUnpackString(fileName), NewQPictureFromPointer(picture)))))
}

func (ptr *QPictureFormatPlugin) ConnectSavePicture(f func(format string, fileName string, picture *QPicture) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "savePicture"); signal != nil {
			f := func(format string, fileName string, picture *QPicture) bool {
				(*(*func(string, string, *QPicture) bool)(signal))(format, fileName, picture)
				return f(format, fileName, picture)
			}
			qt.ConnectSignal(ptr.Pointer(), "savePicture", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "savePicture", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPictureFormatPlugin) DisconnectSavePicture() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "savePicture")
	}
}

func (ptr *QPictureFormatPlugin) SavePicture(format string, fileName string, picture QPicture_ITF) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		return int8(C.QPictureFormatPlugin_SavePicture(ptr.Pointer(), C.struct_QtGui_PackedString{data: formatC, len: C.longlong(len(format))}, C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, PointerFromQPicture(picture))) != 0
	}
	return false
}

func (ptr *QPictureFormatPlugin) SavePictureDefault(format string, fileName string, picture QPicture_ITF) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		return int8(C.QPictureFormatPlugin_SavePictureDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: formatC, len: C.longlong(len(format))}, C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, PointerFromQPicture(picture))) != 0
	}
	return false
}

//export callbackQPictureFormatPlugin_DestroyQPictureFormatPlugin
func callbackQPictureFormatPlugin_DestroyQPictureFormatPlugin(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QPictureFormatPlugin"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQPictureFormatPluginFromPointer(ptr).DestroyQPictureFormatPluginDefault()
	}
}

func (ptr *QPictureFormatPlugin) ConnectDestroyQPictureFormatPlugin(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QPictureFormatPlugin"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QPictureFormatPlugin", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QPictureFormatPlugin", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPictureFormatPlugin) DisconnectDestroyQPictureFormatPlugin() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QPictureFormatPlugin")
	}
}

func (ptr *QPictureFormatPlugin) DestroyQPictureFormatPlugin() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPictureFormatPlugin_DestroyQPictureFormatPlugin(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPictureFormatPlugin) DestroyQPictureFormatPluginDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPictureFormatPlugin_DestroyQPictureFormatPluginDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPictureFormatPlugin) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPictureFormatPlugin___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPictureFormatPlugin) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureFormatPlugin___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPictureFormatPlugin) __children_newList() unsafe.Pointer {
	return C.QPictureFormatPlugin___children_newList(ptr.Pointer())
}

func (ptr *QPictureFormatPlugin) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QPictureFormatPlugin___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QPictureFormatPlugin) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureFormatPlugin___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QPictureFormatPlugin) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QPictureFormatPlugin___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QPictureFormatPlugin) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPictureFormatPlugin___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPictureFormatPlugin) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureFormatPlugin___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPictureFormatPlugin) __findChildren_newList() unsafe.Pointer {
	return C.QPictureFormatPlugin___findChildren_newList(ptr.Pointer())
}

func (ptr *QPictureFormatPlugin) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPictureFormatPlugin___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPictureFormatPlugin) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureFormatPlugin___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPictureFormatPlugin) __findChildren_newList3() unsafe.Pointer {
	return C.QPictureFormatPlugin___findChildren_newList3(ptr.Pointer())
}

//export callbackQPictureFormatPlugin_ChildEvent
func callbackQPictureFormatPlugin_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQPictureFormatPluginFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QPictureFormatPlugin) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureFormatPlugin_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQPictureFormatPlugin_ConnectNotify
func callbackQPictureFormatPlugin_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQPictureFormatPluginFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QPictureFormatPlugin) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureFormatPlugin_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQPictureFormatPlugin_CustomEvent
func callbackQPictureFormatPlugin_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQPictureFormatPluginFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QPictureFormatPlugin) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureFormatPlugin_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQPictureFormatPlugin_DeleteLater
func callbackQPictureFormatPlugin_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQPictureFormatPluginFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QPictureFormatPlugin) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPictureFormatPlugin_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQPictureFormatPlugin_Destroyed
func callbackQPictureFormatPlugin_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQPictureFormatPlugin_DisconnectNotify
func callbackQPictureFormatPlugin_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQPictureFormatPluginFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QPictureFormatPlugin) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureFormatPlugin_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQPictureFormatPlugin_Event
func callbackQPictureFormatPlugin_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQPictureFormatPluginFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QPictureFormatPlugin) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPictureFormatPlugin_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQPictureFormatPlugin_EventFilter
func callbackQPictureFormatPlugin_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQPictureFormatPluginFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QPictureFormatPlugin) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPictureFormatPlugin_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQPictureFormatPlugin_MetaObject
func callbackQPictureFormatPlugin_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQPictureFormatPluginFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QPictureFormatPlugin) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QPictureFormatPlugin_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQPictureFormatPlugin_ObjectNameChanged
func callbackQPictureFormatPlugin_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQPictureFormatPlugin_TimerEvent
func callbackQPictureFormatPlugin_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQPictureFormatPluginFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QPictureFormatPlugin) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureFormatPlugin_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QPictureIO struct {
	ptr unsafe.Pointer
}

type QPictureIO_ITF interface {
	QPictureIO_PTR() *QPictureIO
}

func (ptr *QPictureIO) QPictureIO_PTR() *QPictureIO {
	return ptr
}

func (ptr *QPictureIO) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPictureIO) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPictureIO(ptr QPictureIO_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPictureIO_PTR().Pointer()
	}
	return nil
}

func NewQPictureIOFromPointer(ptr unsafe.Pointer) (n *QPictureIO) {
	n = new(QPictureIO)
	n.SetPointer(ptr)
	return
}
func NewQPictureIO() *QPictureIO {
	tmpValue := NewQPictureIOFromPointer(C.QPictureIO_NewQPictureIO())
	qt.SetFinalizer(tmpValue, (*QPictureIO).DestroyQPictureIO)
	return tmpValue
}

func NewQPictureIO2(ioDevice core.QIODevice_ITF, format string) *QPictureIO {
	var formatC *C.char
	if format != "" {
		formatC = C.CString(format)
		defer C.free(unsafe.Pointer(formatC))
	}
	tmpValue := NewQPictureIOFromPointer(C.QPictureIO_NewQPictureIO2(core.PointerFromQIODevice(ioDevice), formatC))
	qt.SetFinalizer(tmpValue, (*QPictureIO).DestroyQPictureIO)
	return tmpValue
}

func NewQPictureIO3(fileName string, format string) *QPictureIO {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	var formatC *C.char
	if format != "" {
		formatC = C.CString(format)
		defer C.free(unsafe.Pointer(formatC))
	}
	tmpValue := NewQPictureIOFromPointer(C.QPictureIO_NewQPictureIO3(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, formatC))
	qt.SetFinalizer(tmpValue, (*QPictureIO).DestroyQPictureIO)
	return tmpValue
}

func (ptr *QPictureIO) Description() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPictureIO_Description(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPictureIO) FileName() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPictureIO_FileName(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPictureIO) Format() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPictureIO_Format(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPictureIO) Gamma() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QPictureIO_Gamma(ptr.Pointer()))
	}
	return 0
}

func QPictureIO_InputFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQPictureIOFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__inputFormats_atList(i)
		}
		return out
	}(C.QPictureIO_QPictureIO_InputFormats())
}

func (ptr *QPictureIO) InputFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQPictureIOFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__inputFormats_atList(i)
		}
		return out
	}(C.QPictureIO_QPictureIO_InputFormats())
}

func (ptr *QPictureIO) IoDevice() *core.QIODevice {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQIODeviceFromPointer(C.QPictureIO_IoDevice(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func QPictureIO_OutputFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQPictureIOFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__outputFormats_atList(i)
		}
		return out
	}(C.QPictureIO_QPictureIO_OutputFormats())
}

func (ptr *QPictureIO) OutputFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQPictureIOFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__outputFormats_atList(i)
		}
		return out
	}(C.QPictureIO_QPictureIO_OutputFormats())
}

func (ptr *QPictureIO) Parameters() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPictureIO_Parameters(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPictureIO) Picture() *QPicture {
	if ptr.Pointer() != nil {
		return NewQPictureFromPointer(C.QPictureIO_Picture(ptr.Pointer()))
	}
	return nil
}

func QPictureIO_PictureFormat(fileName string) *core.QByteArray {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	tmpValue := core.NewQByteArrayFromPointer(C.QPictureIO_QPictureIO_PictureFormat(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}))
	qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
	return tmpValue
}

func (ptr *QPictureIO) PictureFormat(fileName string) *core.QByteArray {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	tmpValue := core.NewQByteArrayFromPointer(C.QPictureIO_QPictureIO_PictureFormat(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}))
	qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
	return tmpValue
}

func QPictureIO_PictureFormat2(d core.QIODevice_ITF) *core.QByteArray {
	tmpValue := core.NewQByteArrayFromPointer(C.QPictureIO_QPictureIO_PictureFormat2(core.PointerFromQIODevice(d)))
	qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
	return tmpValue
}

func (ptr *QPictureIO) PictureFormat2(d core.QIODevice_ITF) *core.QByteArray {
	tmpValue := core.NewQByteArrayFromPointer(C.QPictureIO_QPictureIO_PictureFormat2(core.PointerFromQIODevice(d)))
	qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
	return tmpValue
}

func (ptr *QPictureIO) Quality() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPictureIO_Quality(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPictureIO) Read() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPictureIO_Read(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPictureIO) SetDescription(description string) {
	if ptr.Pointer() != nil {
		var descriptionC *C.char
		if description != "" {
			descriptionC = C.CString(description)
			defer C.free(unsafe.Pointer(descriptionC))
		}
		C.QPictureIO_SetDescription(ptr.Pointer(), C.struct_QtGui_PackedString{data: descriptionC, len: C.longlong(len(description))})
	}
}

func (ptr *QPictureIO) SetFileName(fileName string) {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		C.QPictureIO_SetFileName(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))})
	}
}

func (ptr *QPictureIO) SetFormat(format string) {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		C.QPictureIO_SetFormat(ptr.Pointer(), formatC)
	}
}

func (ptr *QPictureIO) SetGamma(gamma float32) {
	if ptr.Pointer() != nil {
		C.QPictureIO_SetGamma(ptr.Pointer(), C.float(gamma))
	}
}

func (ptr *QPictureIO) SetIODevice(ioDevice core.QIODevice_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureIO_SetIODevice(ptr.Pointer(), core.PointerFromQIODevice(ioDevice))
	}
}

func (ptr *QPictureIO) SetParameters(parameters string) {
	if ptr.Pointer() != nil {
		var parametersC *C.char
		if parameters != "" {
			parametersC = C.CString(parameters)
			defer C.free(unsafe.Pointer(parametersC))
		}
		C.QPictureIO_SetParameters(ptr.Pointer(), parametersC)
	}
}

func (ptr *QPictureIO) SetPicture(picture QPicture_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureIO_SetPicture(ptr.Pointer(), PointerFromQPicture(picture))
	}
}

func (ptr *QPictureIO) SetQuality(q int) {
	if ptr.Pointer() != nil {
		C.QPictureIO_SetQuality(ptr.Pointer(), C.int(int32(q)))
	}
}

func (ptr *QPictureIO) SetStatus(status int) {
	if ptr.Pointer() != nil {
		C.QPictureIO_SetStatus(ptr.Pointer(), C.int(int32(status)))
	}
}

func (ptr *QPictureIO) Status() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QPictureIO_Status(ptr.Pointer())))
	}
	return 0
}

func (ptr *QPictureIO) Write() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPictureIO_Write(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPictureIO) DestroyQPictureIO() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPictureIO_DestroyQPictureIO(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPictureIO) __inputFormats_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QPictureIO___inputFormats_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QPictureIO) __inputFormats_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureIO___inputFormats_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QPictureIO) __inputFormats_newList() unsafe.Pointer {
	return C.QPictureIO___inputFormats_newList(ptr.Pointer())
}

func (ptr *QPictureIO) __outputFormats_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QPictureIO___outputFormats_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QPictureIO) __outputFormats_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QPictureIO___outputFormats_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QPictureIO) __outputFormats_newList() unsafe.Pointer {
	return C.QPictureIO___outputFormats_newList(ptr.Pointer())
}

type QPixelFormat struct {
	ptr unsafe.Pointer
}

type QPixelFormat_ITF interface {
	QPixelFormat_PTR() *QPixelFormat
}

func (ptr *QPixelFormat) QPixelFormat_PTR() *QPixelFormat {
	return ptr
}

func (ptr *QPixelFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPixelFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPixelFormat(ptr QPixelFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPixelFormat_PTR().Pointer()
	}
	return nil
}

func NewQPixelFormatFromPointer(ptr unsafe.Pointer) (n *QPixelFormat) {
	n = new(QPixelFormat)
	n.SetPointer(ptr)
	return
}
func (ptr *QPixelFormat) DestroyQPixelFormat() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QPixelFormat__ColorModel
//QPixelFormat::ColorModel
type QPixelFormat__ColorModel int64

const (
	QPixelFormat__RGB       QPixelFormat__ColorModel = QPixelFormat__ColorModel(0)
	QPixelFormat__BGR       QPixelFormat__ColorModel = QPixelFormat__ColorModel(1)
	QPixelFormat__Indexed   QPixelFormat__ColorModel = QPixelFormat__ColorModel(2)
	QPixelFormat__Grayscale QPixelFormat__ColorModel = QPixelFormat__ColorModel(3)
	QPixelFormat__CMYK      QPixelFormat__ColorModel = QPixelFormat__ColorModel(4)
	QPixelFormat__HSL       QPixelFormat__ColorModel = QPixelFormat__ColorModel(5)
	QPixelFormat__HSV       QPixelFormat__ColorModel = QPixelFormat__ColorModel(6)
	QPixelFormat__YUV       QPixelFormat__ColorModel = QPixelFormat__ColorModel(7)
	QPixelFormat__Alpha     QPixelFormat__ColorModel = QPixelFormat__ColorModel(8)
)

//go:generate stringer -type=QPixelFormat__AlphaUsage
//QPixelFormat::AlphaUsage
type QPixelFormat__AlphaUsage int64

const (
	QPixelFormat__UsesAlpha    QPixelFormat__AlphaUsage = QPixelFormat__AlphaUsage(0)
	QPixelFormat__IgnoresAlpha QPixelFormat__AlphaUsage = QPixelFormat__AlphaUsage(1)
)

//go:generate stringer -type=QPixelFormat__AlphaPosition
//QPixelFormat::AlphaPosition
type QPixelFormat__AlphaPosition int64

const (
	QPixelFormat__AtBeginning QPixelFormat__AlphaPosition = QPixelFormat__AlphaPosition(0)
	QPixelFormat__AtEnd       QPixelFormat__AlphaPosition = QPixelFormat__AlphaPosition(1)
)

//go:generate stringer -type=QPixelFormat__AlphaPremultiplied
//QPixelFormat::AlphaPremultiplied
type QPixelFormat__AlphaPremultiplied int64

const (
	QPixelFormat__NotPremultiplied QPixelFormat__AlphaPremultiplied = QPixelFormat__AlphaPremultiplied(0)
	QPixelFormat__Premultiplied    QPixelFormat__AlphaPremultiplied = QPixelFormat__AlphaPremultiplied(1)
)

//go:generate stringer -type=QPixelFormat__TypeInterpretation
//QPixelFormat::TypeInterpretation
type QPixelFormat__TypeInterpretation int64

const (
	QPixelFormat__UnsignedInteger QPixelFormat__TypeInterpretation = QPixelFormat__TypeInterpretation(0)
	QPixelFormat__UnsignedShort   QPixelFormat__TypeInterpretation = QPixelFormat__TypeInterpretation(1)
	QPixelFormat__UnsignedByte    QPixelFormat__TypeInterpretation = QPixelFormat__TypeInterpretation(2)
	QPixelFormat__FloatingPoint   QPixelFormat__TypeInterpretation = QPixelFormat__TypeInterpretation(3)
)

//go:generate stringer -type=QPixelFormat__YUVLayout
//QPixelFormat::YUVLayout
type QPixelFormat__YUVLayout int64

const (
	QPixelFormat__YUV444   QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(0)
	QPixelFormat__YUV422   QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(1)
	QPixelFormat__YUV411   QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(2)
	QPixelFormat__YUV420P  QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(3)
	QPixelFormat__YUV420SP QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(4)
	QPixelFormat__YV12     QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(5)
	QPixelFormat__UYVY     QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(6)
	QPixelFormat__YUYV     QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(7)
	QPixelFormat__NV12     QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(8)
	QPixelFormat__NV21     QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(9)
	QPixelFormat__IMC1     QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(10)
	QPixelFormat__IMC2     QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(11)
	QPixelFormat__IMC3     QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(12)
	QPixelFormat__IMC4     QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(13)
	QPixelFormat__Y8       QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(14)
	QPixelFormat__Y16      QPixelFormat__YUVLayout = QPixelFormat__YUVLayout(15)
)

//go:generate stringer -type=QPixelFormat__ByteOrder
//QPixelFormat::ByteOrder
type QPixelFormat__ByteOrder int64

const (
	QPixelFormat__LittleEndian        QPixelFormat__ByteOrder = QPixelFormat__ByteOrder(0)
	QPixelFormat__BigEndian           QPixelFormat__ByteOrder = QPixelFormat__ByteOrder(1)
	QPixelFormat__CurrentSystemEndian QPixelFormat__ByteOrder = QPixelFormat__ByteOrder(2)
)

func NewQPixelFormat() *QPixelFormat {
	tmpValue := NewQPixelFormatFromPointer(C.QPixelFormat_NewQPixelFormat())
	qt.SetFinalizer(tmpValue, (*QPixelFormat).DestroyQPixelFormat)
	return tmpValue
}

func NewQPixelFormat2(colorModel QPixelFormat__ColorModel, firstSize string, secondSize string, thirdSize string, fourthSize string, fifthSize string, alphaSize string, alphaUsage QPixelFormat__AlphaUsage, alphaPosition QPixelFormat__AlphaPosition, premultiplied QPixelFormat__AlphaPremultiplied, typeInterpretation QPixelFormat__TypeInterpretation, byteOrder QPixelFormat__ByteOrder, subEnum string) *QPixelFormat {
	var firstSizeC *C.char
	if firstSize != "" {
		firstSizeC = C.CString(firstSize)
		defer C.free(unsafe.Pointer(firstSizeC))
	}
	var secondSizeC *C.char
	if secondSize != "" {
		secondSizeC = C.CString(secondSize)
		defer C.free(unsafe.Pointer(secondSizeC))
	}
	var thirdSizeC *C.char
	if thirdSize != "" {
		thirdSizeC = C.CString(thirdSize)
		defer C.free(unsafe.Pointer(thirdSizeC))
	}
	var fourthSizeC *C.char
	if fourthSize != "" {
		fourthSizeC = C.CString(fourthSize)
		defer C.free(unsafe.Pointer(fourthSizeC))
	}
	var fifthSizeC *C.char
	if fifthSize != "" {
		fifthSizeC = C.CString(fifthSize)
		defer C.free(unsafe.Pointer(fifthSizeC))
	}
	var alphaSizeC *C.char
	if alphaSize != "" {
		alphaSizeC = C.CString(alphaSize)
		defer C.free(unsafe.Pointer(alphaSizeC))
	}
	var subEnumC *C.char
	if subEnum != "" {
		subEnumC = C.CString(subEnum)
		defer C.free(unsafe.Pointer(subEnumC))
	}
	tmpValue := NewQPixelFormatFromPointer(C.QPixelFormat_NewQPixelFormat2(C.longlong(colorModel), firstSizeC, secondSizeC, thirdSizeC, fourthSizeC, fifthSizeC, alphaSizeC, C.longlong(alphaUsage), C.longlong(alphaPosition), C.longlong(premultiplied), C.longlong(typeInterpretation), C.longlong(byteOrder), subEnumC))
	qt.SetFinalizer(tmpValue, (*QPixelFormat).DestroyQPixelFormat)
	return tmpValue
}

func (ptr *QPixelFormat) AlphaPosition() QPixelFormat__AlphaPosition {
	if ptr.Pointer() != nil {
		return QPixelFormat__AlphaPosition(C.QPixelFormat_AlphaPosition(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPixelFormat) AlphaSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_AlphaSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) AlphaUsage() QPixelFormat__AlphaUsage {
	if ptr.Pointer() != nil {
		return QPixelFormat__AlphaUsage(C.QPixelFormat_AlphaUsage(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPixelFormat) BitsPerPixel() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_BitsPerPixel(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) BlackSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_BlackSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) BlueSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_BlueSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) BrightnessSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_BrightnessSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) ByteOrder() QPixelFormat__ByteOrder {
	if ptr.Pointer() != nil {
		return QPixelFormat__ByteOrder(C.QPixelFormat_ByteOrder(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPixelFormat) ChannelCount() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_ChannelCount(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) ColorModel() QPixelFormat__ColorModel {
	if ptr.Pointer() != nil {
		return QPixelFormat__ColorModel(C.QPixelFormat_ColorModel(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPixelFormat) CyanSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_CyanSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) GreenSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_GreenSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) HueSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_HueSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) LightnessSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_LightnessSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) MagentaSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_MagentaSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) Premultiplied() QPixelFormat__AlphaPremultiplied {
	if ptr.Pointer() != nil {
		return QPixelFormat__AlphaPremultiplied(C.QPixelFormat_Premultiplied(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPixelFormat) RedSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_RedSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) SaturationSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_SaturationSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) TypeInterpretation() QPixelFormat__TypeInterpretation {
	if ptr.Pointer() != nil {
		return QPixelFormat__TypeInterpretation(C.QPixelFormat_TypeInterpretation(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPixelFormat) YellowSize() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QPixelFormat_YellowSize(ptr.Pointer()))
	}
	return ""
}

func (ptr *QPixelFormat) YuvLayout() QPixelFormat__YUVLayout {
	if ptr.Pointer() != nil {
		return QPixelFormat__YUVLayout(C.QPixelFormat_YuvLayout(ptr.Pointer()))
	}
	return 0
}

type QPixmap struct {
	QPaintDevice
}

type QPixmap_ITF interface {
	QPaintDevice_ITF
	QPixmap_PTR() *QPixmap
}

func (ptr *QPixmap) QPixmap_PTR() *QPixmap {
	return ptr
}

func (ptr *QPixmap) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintDevice_PTR().Pointer()
	}
	return nil
}

func (ptr *QPixmap) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QPaintDevice_PTR().SetPointer(p)
	}
}

func PointerFromQPixmap(ptr QPixmap_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPixmap_PTR().Pointer()
	}
	return nil
}

func NewQPixmapFromPointer(ptr unsafe.Pointer) (n *QPixmap) {
	n = new(QPixmap)
	n.SetPointer(ptr)
	return
}
func NewQPixmap() *QPixmap {
	return NewQPixmapFromPointer(C.QPixmap_NewQPixmap())
}

func NewQPixmap2(size core.QSize_ITF) *QPixmap {
	return NewQPixmapFromPointer(C.QPixmap_NewQPixmap2(core.PointerFromQSize(size)))
}

func NewQPixmap3(fileName string, format string, flags core.Qt__ImageConversionFlag) *QPixmap {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	var formatC *C.char
	if format != "" {
		formatC = C.CString(format)
		defer C.free(unsafe.Pointer(formatC))
	}
	return NewQPixmapFromPointer(C.QPixmap_NewQPixmap3(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, formatC, C.longlong(flags)))
}

func NewQPixmap5(pixmap QPixmap_ITF) *QPixmap {
	return NewQPixmapFromPointer(C.QPixmap_NewQPixmap5(PointerFromQPixmap(pixmap)))
}

func (ptr *QPixmap) CacheKey() int64 {
	if ptr.Pointer() != nil {
		return int64(C.QPixmap_CacheKey(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPixmap) ConvertFromImage(image QImage_ITF, flags core.Qt__ImageConversionFlag) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPixmap_ConvertFromImage(ptr.Pointer(), PointerFromQImage(image), C.longlong(flags))) != 0
	}
	return false
}

func (ptr *QPixmap) Copy(rectangle core.QRect_ITF) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QPixmap_Copy(ptr.Pointer(), core.PointerFromQRect(rectangle)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) Copy2(x int, y int, width int, height int) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QPixmap_Copy2(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height))))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) CreateHeuristicMask(clipTight bool) *QBitmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQBitmapFromPointer(C.QPixmap_CreateHeuristicMask(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(clipTight)))))
		qt.SetFinalizer(tmpValue, (*QBitmap).DestroyQBitmap)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) CreateMaskFromColor(maskColor QColor_ITF, mode core.Qt__MaskMode) *QBitmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQBitmapFromPointer(C.QPixmap_CreateMaskFromColor(ptr.Pointer(), PointerFromQColor(maskColor), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QBitmap).DestroyQBitmap)
		return tmpValue
	}
	return nil
}

func QPixmap_DefaultDepth() int {
	return int(int32(C.QPixmap_QPixmap_DefaultDepth()))
}

func (ptr *QPixmap) DefaultDepth() int {
	return int(int32(C.QPixmap_QPixmap_DefaultDepth()))
}

func (ptr *QPixmap) Detach() {
	if ptr.Pointer() != nil {
		C.QPixmap_Detach(ptr.Pointer())
	}
}

func (ptr *QPixmap) DevicePixelRatio() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QPixmap_DevicePixelRatio(ptr.Pointer()))
	}
	return 0
}

func (ptr *QPixmap) Fill(color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QPixmap_Fill(ptr.Pointer(), PointerFromQColor(color))
	}
}

func QPixmap_FromImage(image QImage_ITF, flags core.Qt__ImageConversionFlag) *QPixmap {
	tmpValue := NewQPixmapFromPointer(C.QPixmap_QPixmap_FromImage(PointerFromQImage(image), C.longlong(flags)))
	qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
	return tmpValue
}

func (ptr *QPixmap) FromImage(image QImage_ITF, flags core.Qt__ImageConversionFlag) *QPixmap {
	tmpValue := NewQPixmapFromPointer(C.QPixmap_QPixmap_FromImage(PointerFromQImage(image), C.longlong(flags)))
	qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
	return tmpValue
}

func QPixmap_FromImage2(image QImage_ITF, flags core.Qt__ImageConversionFlag) *QPixmap {
	tmpValue := NewQPixmapFromPointer(C.QPixmap_QPixmap_FromImage2(PointerFromQImage(image), C.longlong(flags)))
	qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
	return tmpValue
}

func (ptr *QPixmap) FromImage2(image QImage_ITF, flags core.Qt__ImageConversionFlag) *QPixmap {
	tmpValue := NewQPixmapFromPointer(C.QPixmap_QPixmap_FromImage2(PointerFromQImage(image), C.longlong(flags)))
	qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
	return tmpValue
}

func QPixmap_FromImageReader(imageReader QImageReader_ITF, flags core.Qt__ImageConversionFlag) *QPixmap {
	tmpValue := NewQPixmapFromPointer(C.QPixmap_QPixmap_FromImageReader(PointerFromQImageReader(imageReader), C.longlong(flags)))
	qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
	return tmpValue
}

func (ptr *QPixmap) FromImageReader(imageReader QImageReader_ITF, flags core.Qt__ImageConversionFlag) *QPixmap {
	tmpValue := NewQPixmapFromPointer(C.QPixmap_QPixmap_FromImageReader(PointerFromQImageReader(imageReader), C.longlong(flags)))
	qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
	return tmpValue
}

func (ptr *QPixmap) HasAlpha() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPixmap_HasAlpha(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPixmap) HasAlphaChannel() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPixmap_HasAlphaChannel(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPixmap) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPixmap_IsNull(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPixmap) IsQBitmap() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPixmap_IsQBitmap(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPixmap) Load(fileName string, format string, flags core.Qt__ImageConversionFlag) bool {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QPixmap_Load(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, formatC, C.longlong(flags))) != 0
	}
	return false
}

func (ptr *QPixmap) LoadFromData(data []byte, l uint, format string, flags core.Qt__ImageConversionFlag) bool {
	if ptr.Pointer() != nil {
		var dataC *C.char
		if len(data) != 0 {
			dataC = (*C.char)(unsafe.Pointer(&data[0]))
		}
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QPixmap_LoadFromData(ptr.Pointer(), dataC, C.uint(uint32(l)), formatC, C.longlong(flags))) != 0
	}
	return false
}

func (ptr *QPixmap) LoadFromData2(data core.QByteArray_ITF, format string, flags core.Qt__ImageConversionFlag) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QPixmap_LoadFromData2(ptr.Pointer(), core.PointerFromQByteArray(data), formatC, C.longlong(flags))) != 0
	}
	return false
}

func (ptr *QPixmap) Mask() *QBitmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQBitmapFromPointer(C.QPixmap_Mask(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QBitmap).DestroyQBitmap)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) Rect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPixmap_Rect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) Save(fileName string, format string, quality int) bool {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QPixmap_Save(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, formatC, C.int(int32(quality)))) != 0
	}
	return false
}

func (ptr *QPixmap) Save2(device core.QIODevice_ITF, format string, quality int) bool {
	if ptr.Pointer() != nil {
		var formatC *C.char
		if format != "" {
			formatC = C.CString(format)
			defer C.free(unsafe.Pointer(formatC))
		}
		return int8(C.QPixmap_Save2(ptr.Pointer(), core.PointerFromQIODevice(device), formatC, C.int(int32(quality)))) != 0
	}
	return false
}

func (ptr *QPixmap) Scaled(size core.QSize_ITF, aspectRatioMode core.Qt__AspectRatioMode, transformMode core.Qt__TransformationMode) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QPixmap_Scaled(ptr.Pointer(), core.PointerFromQSize(size), C.longlong(aspectRatioMode), C.longlong(transformMode)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) Scaled2(width int, height int, aspectRatioMode core.Qt__AspectRatioMode, transformMode core.Qt__TransformationMode) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QPixmap_Scaled2(ptr.Pointer(), C.int(int32(width)), C.int(int32(height)), C.longlong(aspectRatioMode), C.longlong(transformMode)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) ScaledToHeight(height int, mode core.Qt__TransformationMode) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QPixmap_ScaledToHeight(ptr.Pointer(), C.int(int32(height)), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) ScaledToWidth(width int, mode core.Qt__TransformationMode) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QPixmap_ScaledToWidth(ptr.Pointer(), C.int(int32(width)), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) Scroll(dx int, dy int, x int, y int, width int, height int, exposed QRegion_ITF) {
	if ptr.Pointer() != nil {
		C.QPixmap_Scroll(ptr.Pointer(), C.int(int32(dx)), C.int(int32(dy)), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height)), PointerFromQRegion(exposed))
	}
}

func (ptr *QPixmap) Scroll2(dx int, dy int, rect core.QRect_ITF, exposed QRegion_ITF) {
	if ptr.Pointer() != nil {
		C.QPixmap_Scroll2(ptr.Pointer(), C.int(int32(dx)), C.int(int32(dy)), core.PointerFromQRect(rect), PointerFromQRegion(exposed))
	}
}

func (ptr *QPixmap) SetDevicePixelRatio(scaleFactor float64) {
	if ptr.Pointer() != nil {
		C.QPixmap_SetDevicePixelRatio(ptr.Pointer(), C.double(scaleFactor))
	}
}

func (ptr *QPixmap) SetMask(mask QBitmap_ITF) {
	if ptr.Pointer() != nil {
		C.QPixmap_SetMask(ptr.Pointer(), PointerFromQBitmap(mask))
	}
}

func (ptr *QPixmap) Size() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QPixmap_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) Swap(other QPixmap_ITF) {
	if ptr.Pointer() != nil {
		C.QPixmap_Swap(ptr.Pointer(), PointerFromQPixmap(other))
	}
}

func (ptr *QPixmap) ToImage() *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QPixmap_ToImage(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) Transformed(transform QTransform_ITF, mode core.Qt__TransformationMode) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QPixmap_Transformed(ptr.Pointer(), PointerFromQTransform(transform), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QPixmap) Transformed2(matrix QMatrix_ITF, mode core.Qt__TransformationMode) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QPixmap_Transformed2(ptr.Pointer(), PointerFromQMatrix(matrix), C.longlong(mode)))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func QPixmap_TrueMatrix(matrix QTransform_ITF, width int, height int) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QPixmap_QPixmap_TrueMatrix(PointerFromQTransform(matrix), C.int(int32(width)), C.int(int32(height))))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func (ptr *QPixmap) TrueMatrix(matrix QTransform_ITF, width int, height int) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QPixmap_QPixmap_TrueMatrix(PointerFromQTransform(matrix), C.int(int32(width)), C.int(int32(height))))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func QPixmap_TrueMatrix2(m QMatrix_ITF, w int, h int) *QMatrix {
	tmpValue := NewQMatrixFromPointer(C.QPixmap_QPixmap_TrueMatrix2(PointerFromQMatrix(m), C.int(int32(w)), C.int(int32(h))))
	qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
	return tmpValue
}

func (ptr *QPixmap) TrueMatrix2(m QMatrix_ITF, w int, h int) *QMatrix {
	tmpValue := NewQMatrixFromPointer(C.QPixmap_QPixmap_TrueMatrix2(PointerFromQMatrix(m), C.int(int32(w)), C.int(int32(h))))
	qt.SetFinalizer(tmpValue, (*QMatrix).DestroyQMatrix)
	return tmpValue
}

//export callbackQPixmap_DestroyQPixmap
func callbackQPixmap_DestroyQPixmap(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QPixmap"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQPixmapFromPointer(ptr).DestroyQPixmapDefault()
	}
}

func (ptr *QPixmap) ConnectDestroyQPixmap(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QPixmap"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QPixmap", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QPixmap", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QPixmap) DisconnectDestroyQPixmap() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QPixmap")
	}
}

func (ptr *QPixmap) DestroyQPixmap() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPixmap_DestroyQPixmap(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPixmap) DestroyQPixmapDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPixmap_DestroyQPixmapDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPixmap) ToVariant() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QPixmap_ToVariant(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

//export callbackQPixmap_PaintEngine
func callbackQPixmap_PaintEngine(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "paintEngine"); signal != nil {
		return PointerFromQPaintEngine((*(*func() *QPaintEngine)(signal))())
	}

	return PointerFromQPaintEngine(NewQPixmapFromPointer(ptr).PaintEngineDefault())
}

func (ptr *QPixmap) PaintEngine() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPixmap_PaintEngine(ptr.Pointer()))
	}
	return nil
}

func (ptr *QPixmap) PaintEngineDefault() *QPaintEngine {
	if ptr.Pointer() != nil {
		return NewQPaintEngineFromPointer(C.QPixmap_PaintEngineDefault(ptr.Pointer()))
	}
	return nil
}

type QPixmapCache struct {
	ptr unsafe.Pointer
}

type QPixmapCache_ITF interface {
	QPixmapCache_PTR() *QPixmapCache
}

func (ptr *QPixmapCache) QPixmapCache_PTR() *QPixmapCache {
	return ptr
}

func (ptr *QPixmapCache) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPixmapCache) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPixmapCache(ptr QPixmapCache_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPixmapCache_PTR().Pointer()
	}
	return nil
}

func NewQPixmapCacheFromPointer(ptr unsafe.Pointer) (n *QPixmapCache) {
	n = new(QPixmapCache)
	n.SetPointer(ptr)
	return
}
func (ptr *QPixmapCache) DestroyQPixmapCache() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func QPixmapCache_CacheLimit() int {
	return int(int32(C.QPixmapCache_QPixmapCache_CacheLimit()))
}

func (ptr *QPixmapCache) CacheLimit() int {
	return int(int32(C.QPixmapCache_QPixmapCache_CacheLimit()))
}

func QPixmapCache_Clear() {
	C.QPixmapCache_QPixmapCache_Clear()
}

func (ptr *QPixmapCache) Clear() {
	C.QPixmapCache_QPixmapCache_Clear()
}

func QPixmapCache_Find3(key string, pixmap QPixmap_ITF) bool {
	var keyC *C.char
	if key != "" {
		keyC = C.CString(key)
		defer C.free(unsafe.Pointer(keyC))
	}
	return int8(C.QPixmapCache_QPixmapCache_Find3(C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, PointerFromQPixmap(pixmap))) != 0
}

func (ptr *QPixmapCache) Find3(key string, pixmap QPixmap_ITF) bool {
	var keyC *C.char
	if key != "" {
		keyC = C.CString(key)
		defer C.free(unsafe.Pointer(keyC))
	}
	return int8(C.QPixmapCache_QPixmapCache_Find3(C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, PointerFromQPixmap(pixmap))) != 0
}

func QPixmapCache_Insert(key string, pixmap QPixmap_ITF) bool {
	var keyC *C.char
	if key != "" {
		keyC = C.CString(key)
		defer C.free(unsafe.Pointer(keyC))
	}
	return int8(C.QPixmapCache_QPixmapCache_Insert(C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, PointerFromQPixmap(pixmap))) != 0
}

func (ptr *QPixmapCache) Insert(key string, pixmap QPixmap_ITF) bool {
	var keyC *C.char
	if key != "" {
		keyC = C.CString(key)
		defer C.free(unsafe.Pointer(keyC))
	}
	return int8(C.QPixmapCache_QPixmapCache_Insert(C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))}, PointerFromQPixmap(pixmap))) != 0
}

func QPixmapCache_Remove(key string) {
	var keyC *C.char
	if key != "" {
		keyC = C.CString(key)
		defer C.free(unsafe.Pointer(keyC))
	}
	C.QPixmapCache_QPixmapCache_Remove(C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))})
}

func (ptr *QPixmapCache) Remove(key string) {
	var keyC *C.char
	if key != "" {
		keyC = C.CString(key)
		defer C.free(unsafe.Pointer(keyC))
	}
	C.QPixmapCache_QPixmapCache_Remove(C.struct_QtGui_PackedString{data: keyC, len: C.longlong(len(key))})
}

func QPixmapCache_SetCacheLimit(n int) {
	C.QPixmapCache_QPixmapCache_SetCacheLimit(C.int(int32(n)))
}

func (ptr *QPixmapCache) SetCacheLimit(n int) {
	C.QPixmapCache_QPixmapCache_SetCacheLimit(C.int(int32(n)))
}

type QPlatformDragQtResponse struct {
	QPlatformDropQtResponse
}

type QPlatformDragQtResponse_ITF interface {
	QPlatformDropQtResponse_ITF
	QPlatformDragQtResponse_PTR() *QPlatformDragQtResponse
}

func (ptr *QPlatformDragQtResponse) QPlatformDragQtResponse_PTR() *QPlatformDragQtResponse {
	return ptr
}

func (ptr *QPlatformDragQtResponse) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformDropQtResponse_PTR().Pointer()
	}
	return nil
}

func (ptr *QPlatformDragQtResponse) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QPlatformDropQtResponse_PTR().SetPointer(p)
	}
}

func PointerFromQPlatformDragQtResponse(ptr QPlatformDragQtResponse_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformDragQtResponse_PTR().Pointer()
	}
	return nil
}

func NewQPlatformDragQtResponseFromPointer(ptr unsafe.Pointer) (n *QPlatformDragQtResponse) {
	n = new(QPlatformDragQtResponse)
	n.SetPointer(ptr)
	return
}
func (ptr *QPlatformDragQtResponse) DestroyQPlatformDragQtResponse() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QPlatformDropQtResponse struct {
	ptr unsafe.Pointer
}

type QPlatformDropQtResponse_ITF interface {
	QPlatformDropQtResponse_PTR() *QPlatformDropQtResponse
}

func (ptr *QPlatformDropQtResponse) QPlatformDropQtResponse_PTR() *QPlatformDropQtResponse {
	return ptr
}

func (ptr *QPlatformDropQtResponse) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPlatformDropQtResponse) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPlatformDropQtResponse(ptr QPlatformDropQtResponse_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformDropQtResponse_PTR().Pointer()
	}
	return nil
}

func NewQPlatformDropQtResponseFromPointer(ptr unsafe.Pointer) (n *QPlatformDropQtResponse) {
	n = new(QPlatformDropQtResponse)
	n.SetPointer(ptr)
	return
}
func (ptr *QPlatformDropQtResponse) DestroyQPlatformDropQtResponse() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QPlatformIntegrationPlugin struct {
	core.QObject
}

type QPlatformIntegrationPlugin_ITF interface {
	core.QObject_ITF
	QPlatformIntegrationPlugin_PTR() *QPlatformIntegrationPlugin
}

func (ptr *QPlatformIntegrationPlugin) QPlatformIntegrationPlugin_PTR() *QPlatformIntegrationPlugin {
	return ptr
}

func (ptr *QPlatformIntegrationPlugin) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QPlatformIntegrationPlugin) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQPlatformIntegrationPlugin(ptr QPlatformIntegrationPlugin_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformIntegrationPlugin_PTR().Pointer()
	}
	return nil
}

func NewQPlatformIntegrationPluginFromPointer(ptr unsafe.Pointer) (n *QPlatformIntegrationPlugin) {
	n = new(QPlatformIntegrationPlugin)
	n.SetPointer(ptr)
	return
}

type QPlatformMenu struct {
	core.QObject
}

type QPlatformMenu_ITF interface {
	core.QObject_ITF
	QPlatformMenu_PTR() *QPlatformMenu
}

func (ptr *QPlatformMenu) QPlatformMenu_PTR() *QPlatformMenu {
	return ptr
}

func (ptr *QPlatformMenu) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QPlatformMenu) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQPlatformMenu(ptr QPlatformMenu_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformMenu_PTR().Pointer()
	}
	return nil
}

func NewQPlatformMenuFromPointer(ptr unsafe.Pointer) (n *QPlatformMenu) {
	n = new(QPlatformMenu)
	n.SetPointer(ptr)
	return
}

type QPlatformMenuBar struct {
	core.QObject
}

type QPlatformMenuBar_ITF interface {
	core.QObject_ITF
	QPlatformMenuBar_PTR() *QPlatformMenuBar
}

func (ptr *QPlatformMenuBar) QPlatformMenuBar_PTR() *QPlatformMenuBar {
	return ptr
}

func (ptr *QPlatformMenuBar) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QPlatformMenuBar) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQPlatformMenuBar(ptr QPlatformMenuBar_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformMenuBar_PTR().Pointer()
	}
	return nil
}

func NewQPlatformMenuBarFromPointer(ptr unsafe.Pointer) (n *QPlatformMenuBar) {
	n = new(QPlatformMenuBar)
	n.SetPointer(ptr)
	return
}

type QPlatformMenuItem struct {
	core.QObject
}

type QPlatformMenuItem_ITF interface {
	core.QObject_ITF
	QPlatformMenuItem_PTR() *QPlatformMenuItem
}

func (ptr *QPlatformMenuItem) QPlatformMenuItem_PTR() *QPlatformMenuItem {
	return ptr
}

func (ptr *QPlatformMenuItem) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QPlatformMenuItem) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQPlatformMenuItem(ptr QPlatformMenuItem_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformMenuItem_PTR().Pointer()
	}
	return nil
}

func NewQPlatformMenuItemFromPointer(ptr unsafe.Pointer) (n *QPlatformMenuItem) {
	n = new(QPlatformMenuItem)
	n.SetPointer(ptr)
	return
}

type QPlatformOffscreenSurface struct {
	ptr unsafe.Pointer
}

type QPlatformOffscreenSurface_ITF interface {
	QPlatformOffscreenSurface_PTR() *QPlatformOffscreenSurface
}

func (ptr *QPlatformOffscreenSurface) QPlatformOffscreenSurface_PTR() *QPlatformOffscreenSurface {
	return ptr
}

func (ptr *QPlatformOffscreenSurface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPlatformOffscreenSurface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPlatformOffscreenSurface(ptr QPlatformOffscreenSurface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformOffscreenSurface_PTR().Pointer()
	}
	return nil
}

func NewQPlatformOffscreenSurfaceFromPointer(ptr unsafe.Pointer) (n *QPlatformOffscreenSurface) {
	n = new(QPlatformOffscreenSurface)
	n.SetPointer(ptr)
	return
}
func (ptr *QPlatformOffscreenSurface) DestroyQPlatformOffscreenSurface() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QPlatformSessionManager struct {
	ptr unsafe.Pointer
}

type QPlatformSessionManager_ITF interface {
	QPlatformSessionManager_PTR() *QPlatformSessionManager
}

func (ptr *QPlatformSessionManager) QPlatformSessionManager_PTR() *QPlatformSessionManager {
	return ptr
}

func (ptr *QPlatformSessionManager) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPlatformSessionManager) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPlatformSessionManager(ptr QPlatformSessionManager_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformSessionManager_PTR().Pointer()
	}
	return nil
}

func NewQPlatformSessionManagerFromPointer(ptr unsafe.Pointer) (n *QPlatformSessionManager) {
	n = new(QPlatformSessionManager)
	n.SetPointer(ptr)
	return
}
func (ptr *QPlatformSessionManager) DestroyQPlatformSessionManager() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QPlatformSurfaceEvent struct {
	core.QEvent
}

type QPlatformSurfaceEvent_ITF interface {
	core.QEvent_ITF
	QPlatformSurfaceEvent_PTR() *QPlatformSurfaceEvent
}

func (ptr *QPlatformSurfaceEvent) QPlatformSurfaceEvent_PTR() *QPlatformSurfaceEvent {
	return ptr
}

func (ptr *QPlatformSurfaceEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QPlatformSurfaceEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQPlatformSurfaceEvent(ptr QPlatformSurfaceEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformSurfaceEvent_PTR().Pointer()
	}
	return nil
}

func NewQPlatformSurfaceEventFromPointer(ptr unsafe.Pointer) (n *QPlatformSurfaceEvent) {
	n = new(QPlatformSurfaceEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QPlatformSurfaceEvent) DestroyQPlatformSurfaceEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QPlatformSurfaceEvent__SurfaceEventType
//QPlatformSurfaceEvent::SurfaceEventType
type QPlatformSurfaceEvent__SurfaceEventType int64

const (
	QPlatformSurfaceEvent__SurfaceCreated            QPlatformSurfaceEvent__SurfaceEventType = QPlatformSurfaceEvent__SurfaceEventType(0)
	QPlatformSurfaceEvent__SurfaceAboutToBeDestroyed QPlatformSurfaceEvent__SurfaceEventType = QPlatformSurfaceEvent__SurfaceEventType(1)
)

func NewQPlatformSurfaceEvent(surfaceEventType QPlatformSurfaceEvent__SurfaceEventType) *QPlatformSurfaceEvent {
	tmpValue := NewQPlatformSurfaceEventFromPointer(C.QPlatformSurfaceEvent_NewQPlatformSurfaceEvent(C.longlong(surfaceEventType)))
	qt.SetFinalizer(tmpValue, (*QPlatformSurfaceEvent).DestroyQPlatformSurfaceEvent)
	return tmpValue
}

func (ptr *QPlatformSurfaceEvent) SurfaceEventType() QPlatformSurfaceEvent__SurfaceEventType {
	if ptr.Pointer() != nil {
		return QPlatformSurfaceEvent__SurfaceEventType(C.QPlatformSurfaceEvent_SurfaceEventType(ptr.Pointer()))
	}
	return 0
}

type QPlatformTextureList struct {
	core.QObject
}

type QPlatformTextureList_ITF interface {
	core.QObject_ITF
	QPlatformTextureList_PTR() *QPlatformTextureList
}

func (ptr *QPlatformTextureList) QPlatformTextureList_PTR() *QPlatformTextureList {
	return ptr
}

func (ptr *QPlatformTextureList) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QPlatformTextureList) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQPlatformTextureList(ptr QPlatformTextureList_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPlatformTextureList_PTR().Pointer()
	}
	return nil
}

func NewQPlatformTextureListFromPointer(ptr unsafe.Pointer) (n *QPlatformTextureList) {
	n = new(QPlatformTextureList)
	n.SetPointer(ptr)
	return
}

type QPointingDeviceUniqueId struct {
	ptr unsafe.Pointer
}

type QPointingDeviceUniqueId_ITF interface {
	QPointingDeviceUniqueId_PTR() *QPointingDeviceUniqueId
}

func (ptr *QPointingDeviceUniqueId) QPointingDeviceUniqueId_PTR() *QPointingDeviceUniqueId {
	return ptr
}

func (ptr *QPointingDeviceUniqueId) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QPointingDeviceUniqueId) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQPointingDeviceUniqueId(ptr QPointingDeviceUniqueId_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPointingDeviceUniqueId_PTR().Pointer()
	}
	return nil
}

func NewQPointingDeviceUniqueIdFromPointer(ptr unsafe.Pointer) (n *QPointingDeviceUniqueId) {
	n = new(QPointingDeviceUniqueId)
	n.SetPointer(ptr)
	return
}
func (ptr *QPointingDeviceUniqueId) DestroyQPointingDeviceUniqueId() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQPointingDeviceUniqueId() *QPointingDeviceUniqueId {
	tmpValue := NewQPointingDeviceUniqueIdFromPointer(C.QPointingDeviceUniqueId_NewQPointingDeviceUniqueId())
	qt.SetFinalizer(tmpValue, (*QPointingDeviceUniqueId).DestroyQPointingDeviceUniqueId)
	return tmpValue
}

func QPointingDeviceUniqueId_FromNumericId(id int64) *QPointingDeviceUniqueId {
	tmpValue := NewQPointingDeviceUniqueIdFromPointer(C.QPointingDeviceUniqueId_QPointingDeviceUniqueId_FromNumericId(C.longlong(id)))
	qt.SetFinalizer(tmpValue, (*QPointingDeviceUniqueId).DestroyQPointingDeviceUniqueId)
	return tmpValue
}

func (ptr *QPointingDeviceUniqueId) FromNumericId(id int64) *QPointingDeviceUniqueId {
	tmpValue := NewQPointingDeviceUniqueIdFromPointer(C.QPointingDeviceUniqueId_QPointingDeviceUniqueId_FromNumericId(C.longlong(id)))
	qt.SetFinalizer(tmpValue, (*QPointingDeviceUniqueId).DestroyQPointingDeviceUniqueId)
	return tmpValue
}

func (ptr *QPointingDeviceUniqueId) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPointingDeviceUniqueId_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPointingDeviceUniqueId) NumericId() int64 {
	if ptr.Pointer() != nil {
		return int64(C.QPointingDeviceUniqueId_NumericId(ptr.Pointer()))
	}
	return 0
}

type QPolygon struct {
	core.QVector
}

type QPolygon_ITF interface {
	core.QVector_ITF
	QPolygon_PTR() *QPolygon
}

func (ptr *QPolygon) QPolygon_PTR() *QPolygon {
	return ptr
}

func (ptr *QPolygon) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QVector_PTR().Pointer()
	}
	return nil
}

func (ptr *QPolygon) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QVector_PTR().SetPointer(p)
	}
}

func PointerFromQPolygon(ptr QPolygon_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPolygon_PTR().Pointer()
	}
	return nil
}

func NewQPolygonFromPointer(ptr unsafe.Pointer) (n *QPolygon) {
	n = new(QPolygon)
	n.SetPointer(ptr)
	return
}
func NewQPolygon() *QPolygon {
	tmpValue := NewQPolygonFromPointer(C.QPolygon_NewQPolygon())
	qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
	return tmpValue
}

func NewQPolygon2(size int) *QPolygon {
	tmpValue := NewQPolygonFromPointer(C.QPolygon_NewQPolygon2(C.int(int32(size))))
	qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
	return tmpValue
}

func NewQPolygon3(points []*core.QPoint) *QPolygon {
	tmpValue := NewQPolygonFromPointer(C.QPolygon_NewQPolygon3(func() unsafe.Pointer {
		tmpList := NewQPolygonFromPointer(NewQPolygonFromPointer(nil).__QPolygon_points_newList3())
		for _, v := range points {
			tmpList.__QPolygon_points_setList3(v)
		}
		return tmpList.Pointer()
	}()))
	qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
	return tmpValue
}

func NewQPolygon5(rectangle core.QRect_ITF, closed bool) *QPolygon {
	tmpValue := NewQPolygonFromPointer(C.QPolygon_NewQPolygon5(core.PointerFromQRect(rectangle), C.char(int8(qt.GoBoolToInt(closed)))))
	qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
	return tmpValue
}

func (ptr *QPolygon) BoundingRect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QPolygon_BoundingRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) ContainsPoint(point core.QPoint_ITF, fillRule core.Qt__FillRule) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPolygon_ContainsPoint(ptr.Pointer(), core.PointerFromQPoint(point), C.longlong(fillRule))) != 0
	}
	return false
}

func (ptr *QPolygon) Intersected(r QPolygon_ITF) *QPolygon {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFromPointer(C.QPolygon_Intersected(ptr.Pointer(), PointerFromQPolygon(r)))
		qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) Intersects(p QPolygon_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPolygon_Intersects(ptr.Pointer(), PointerFromQPolygon(p))) != 0
	}
	return false
}

func (ptr *QPolygon) Point(index int, x int, y int) {
	if ptr.Pointer() != nil {
		C.QPolygon_Point(ptr.Pointer(), C.int(int32(index)), C.int(int32(x)), C.int(int32(y)))
	}
}

func (ptr *QPolygon) Point2(index int) *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QPolygon_Point2(ptr.Pointer(), C.int(int32(index))))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) PutPoints(index int, nPoints int, fromPolygon QPolygon_ITF, fromIndex int) {
	if ptr.Pointer() != nil {
		C.QPolygon_PutPoints(ptr.Pointer(), C.int(int32(index)), C.int(int32(nPoints)), PointerFromQPolygon(fromPolygon), C.int(int32(fromIndex)))
	}
}

func (ptr *QPolygon) SetPoint(index int, x int, y int) {
	if ptr.Pointer() != nil {
		C.QPolygon_SetPoint(ptr.Pointer(), C.int(int32(index)), C.int(int32(x)), C.int(int32(y)))
	}
}

func (ptr *QPolygon) SetPoint2(index int, point core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon_SetPoint2(ptr.Pointer(), C.int(int32(index)), core.PointerFromQPoint(point))
	}
}

func (ptr *QPolygon) SetPoints(nPoints int, points int) {
	if ptr.Pointer() != nil {
		C.QPolygon_SetPoints(ptr.Pointer(), C.int(int32(nPoints)), C.int(int32(points)))
	}
}

func (ptr *QPolygon) Subtracted(r QPolygon_ITF) *QPolygon {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFromPointer(C.QPolygon_Subtracted(ptr.Pointer(), PointerFromQPolygon(r)))
		qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) Swap(other QPolygon_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon_Swap(ptr.Pointer(), PointerFromQPolygon(other))
	}
}

func (ptr *QPolygon) Translate(dx int, dy int) {
	if ptr.Pointer() != nil {
		C.QPolygon_Translate(ptr.Pointer(), C.int(int32(dx)), C.int(int32(dy)))
	}
}

func (ptr *QPolygon) Translate2(offset core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon_Translate2(ptr.Pointer(), core.PointerFromQPoint(offset))
	}
}

func (ptr *QPolygon) Translated(dx int, dy int) *QPolygon {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFromPointer(C.QPolygon_Translated(ptr.Pointer(), C.int(int32(dx)), C.int(int32(dy))))
		qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) Translated2(offset core.QPoint_ITF) *QPolygon {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFromPointer(C.QPolygon_Translated2(ptr.Pointer(), core.PointerFromQPoint(offset)))
		qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) United(r QPolygon_ITF) *QPolygon {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFromPointer(C.QPolygon_United(ptr.Pointer(), PointerFromQPolygon(r)))
		qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) DestroyQPolygon() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPolygon_DestroyQPolygon(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPolygon) __QPolygon_points_atList3(i int) *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QPolygon___QPolygon_points_atList3(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __QPolygon_points_setList3(i core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___QPolygon_points_setList3(ptr.Pointer(), core.PointerFromQPoint(i))
	}
}

func (ptr *QPolygon) __QPolygon_points_newList3() unsafe.Pointer {
	return C.QPolygon___QPolygon_points_newList3(ptr.Pointer())
}

func (ptr *QPolygon) __QPolygon_v_atList4(i int) *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QPolygon___QPolygon_v_atList4(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __QPolygon_v_setList4(i core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___QPolygon_v_setList4(ptr.Pointer(), core.PointerFromQPoint(i))
	}
}

func (ptr *QPolygon) __QPolygon_v_newList4() unsafe.Pointer {
	return C.QPolygon___QPolygon_v_newList4(ptr.Pointer())
}

func (ptr *QPolygon) __QVector_other_atList4(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygon___QVector_other_atList4(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __QVector_other_setList4(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___QVector_other_setList4(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygon) __QVector_other_newList4() unsafe.Pointer {
	return C.QPolygon___QVector_other_newList4(ptr.Pointer())
}

func (ptr *QPolygon) __QVector_other_atList5(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygon___QVector_other_atList5(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __QVector_other_setList5(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___QVector_other_setList5(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygon) __QVector_other_newList5() unsafe.Pointer {
	return C.QPolygon___QVector_other_newList5(ptr.Pointer())
}

func (ptr *QPolygon) __append_value_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygon___append_value_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __append_value_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___append_value_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygon) __append_value_newList3() unsafe.Pointer {
	return C.QPolygon___append_value_newList3(ptr.Pointer())
}

func (ptr *QPolygon) __fill_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygon___fill_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __fill_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___fill_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygon) __fill_newList() unsafe.Pointer {
	return C.QPolygon___fill_newList(ptr.Pointer())
}

func (ptr *QPolygon) __fromList_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygon___fromList_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __fromList_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___fromList_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygon) __fromList_newList() unsafe.Pointer {
	return C.QPolygon___fromList_newList(ptr.Pointer())
}

func (ptr *QPolygon) __fromList_list_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygon___fromList_list_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __fromList_list_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___fromList_list_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygon) __fromList_list_newList() unsafe.Pointer {
	return C.QPolygon___fromList_list_newList(ptr.Pointer())
}

func (ptr *QPolygon) __fromStdVector_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygon___fromStdVector_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __fromStdVector_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___fromStdVector_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygon) __fromStdVector_newList() unsafe.Pointer {
	return C.QPolygon___fromStdVector_newList(ptr.Pointer())
}

func (ptr *QPolygon) __mid_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygon___mid_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __mid_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___mid_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygon) __mid_newList() unsafe.Pointer {
	return C.QPolygon___mid_newList(ptr.Pointer())
}

func (ptr *QPolygon) __swap_other_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygon___swap_other_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __swap_other_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___swap_other_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygon) __swap_other_newList() unsafe.Pointer {
	return C.QPolygon___swap_other_newList(ptr.Pointer())
}

func (ptr *QPolygon) __toList_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygon___toList_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygon) __toList_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygon___toList_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygon) __toList_newList() unsafe.Pointer {
	return C.QPolygon___toList_newList(ptr.Pointer())
}

type QPolygonF struct {
	core.QVector
}

type QPolygonF_ITF interface {
	core.QVector_ITF
	QPolygonF_PTR() *QPolygonF
}

func (ptr *QPolygonF) QPolygonF_PTR() *QPolygonF {
	return ptr
}

func (ptr *QPolygonF) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QVector_PTR().Pointer()
	}
	return nil
}

func (ptr *QPolygonF) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QVector_PTR().SetPointer(p)
	}
}

func PointerFromQPolygonF(ptr QPolygonF_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QPolygonF_PTR().Pointer()
	}
	return nil
}

func NewQPolygonFFromPointer(ptr unsafe.Pointer) (n *QPolygonF) {
	n = new(QPolygonF)
	n.SetPointer(ptr)
	return
}
func NewQPolygonF() *QPolygonF {
	tmpValue := NewQPolygonFFromPointer(C.QPolygonF_NewQPolygonF())
	qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
	return tmpValue
}

func NewQPolygonF2(size int) *QPolygonF {
	tmpValue := NewQPolygonFFromPointer(C.QPolygonF_NewQPolygonF2(C.int(int32(size))))
	qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
	return tmpValue
}

func NewQPolygonF3(points []*core.QPointF) *QPolygonF {
	tmpValue := NewQPolygonFFromPointer(C.QPolygonF_NewQPolygonF3(func() unsafe.Pointer {
		tmpList := NewQPolygonFFromPointer(NewQPolygonFFromPointer(nil).__QPolygonF_points_newList3())
		for _, v := range points {
			tmpList.__QPolygonF_points_setList3(v)
		}
		return tmpList.Pointer()
	}()))
	qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
	return tmpValue
}

func NewQPolygonF5(rectangle core.QRectF_ITF) *QPolygonF {
	tmpValue := NewQPolygonFFromPointer(C.QPolygonF_NewQPolygonF5(core.PointerFromQRectF(rectangle)))
	qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
	return tmpValue
}

func NewQPolygonF6(polygon QPolygon_ITF) *QPolygonF {
	tmpValue := NewQPolygonFFromPointer(C.QPolygonF_NewQPolygonF6(PointerFromQPolygon(polygon)))
	qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
	return tmpValue
}

func NewQPolygonF7(polygon QPolygonF_ITF) *QPolygonF {
	tmpValue := NewQPolygonFFromPointer(C.QPolygonF_NewQPolygonF7(PointerFromQPolygonF(polygon)))
	qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
	return tmpValue
}

func (ptr *QPolygonF) BoundingRect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QPolygonF_BoundingRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) ContainsPoint(point core.QPointF_ITF, fillRule core.Qt__FillRule) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPolygonF_ContainsPoint(ptr.Pointer(), core.PointerFromQPointF(point), C.longlong(fillRule))) != 0
	}
	return false
}

func (ptr *QPolygonF) Intersected(r QPolygonF_ITF) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPolygonF_Intersected(ptr.Pointer(), PointerFromQPolygonF(r)))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) Intersects(p QPolygonF_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QPolygonF_Intersects(ptr.Pointer(), PointerFromQPolygonF(p))) != 0
	}
	return false
}

func (ptr *QPolygonF) IsClosed() bool {
	if ptr.Pointer() != nil {
		return int8(C.QPolygonF_IsClosed(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QPolygonF) Subtracted(r QPolygonF_ITF) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPolygonF_Subtracted(ptr.Pointer(), PointerFromQPolygonF(r)))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) Swap(other QPolygonF_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF_Swap(ptr.Pointer(), PointerFromQPolygonF(other))
	}
}

func (ptr *QPolygonF) ToPolygon() *QPolygon {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFromPointer(C.QPolygonF_ToPolygon(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) Translate(offset core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF_Translate(ptr.Pointer(), core.PointerFromQPointF(offset))
	}
}

func (ptr *QPolygonF) Translate2(dx float64, dy float64) {
	if ptr.Pointer() != nil {
		C.QPolygonF_Translate2(ptr.Pointer(), C.double(dx), C.double(dy))
	}
}

func (ptr *QPolygonF) Translated(offset core.QPointF_ITF) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPolygonF_Translated(ptr.Pointer(), core.PointerFromQPointF(offset)))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) Translated2(dx float64, dy float64) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPolygonF_Translated2(ptr.Pointer(), C.double(dx), C.double(dy)))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) United(r QPolygonF_ITF) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QPolygonF_United(ptr.Pointer(), PointerFromQPolygonF(r)))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) DestroyQPolygonF() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QPolygonF_DestroyQPolygonF(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QPolygonF) __QPolygonF_points_atList3(i int) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QPolygonF___QPolygonF_points_atList3(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __QPolygonF_points_setList3(i core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___QPolygonF_points_setList3(ptr.Pointer(), core.PointerFromQPointF(i))
	}
}

func (ptr *QPolygonF) __QPolygonF_points_newList3() unsafe.Pointer {
	return C.QPolygonF___QPolygonF_points_newList3(ptr.Pointer())
}

func (ptr *QPolygonF) __QPolygonF_v_atList4(i int) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QPolygonF___QPolygonF_v_atList4(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __QPolygonF_v_setList4(i core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___QPolygonF_v_setList4(ptr.Pointer(), core.PointerFromQPointF(i))
	}
}

func (ptr *QPolygonF) __QPolygonF_v_newList4() unsafe.Pointer {
	return C.QPolygonF___QPolygonF_v_newList4(ptr.Pointer())
}

func (ptr *QPolygonF) __QVector_other_atList4(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygonF___QVector_other_atList4(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __QVector_other_setList4(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___QVector_other_setList4(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygonF) __QVector_other_newList4() unsafe.Pointer {
	return C.QPolygonF___QVector_other_newList4(ptr.Pointer())
}

func (ptr *QPolygonF) __QVector_other_atList5(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygonF___QVector_other_atList5(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __QVector_other_setList5(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___QVector_other_setList5(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygonF) __QVector_other_newList5() unsafe.Pointer {
	return C.QPolygonF___QVector_other_newList5(ptr.Pointer())
}

func (ptr *QPolygonF) __append_value_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygonF___append_value_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __append_value_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___append_value_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygonF) __append_value_newList3() unsafe.Pointer {
	return C.QPolygonF___append_value_newList3(ptr.Pointer())
}

func (ptr *QPolygonF) __fill_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygonF___fill_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __fill_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___fill_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygonF) __fill_newList() unsafe.Pointer {
	return C.QPolygonF___fill_newList(ptr.Pointer())
}

func (ptr *QPolygonF) __fromList_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygonF___fromList_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __fromList_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___fromList_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygonF) __fromList_newList() unsafe.Pointer {
	return C.QPolygonF___fromList_newList(ptr.Pointer())
}

func (ptr *QPolygonF) __fromList_list_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygonF___fromList_list_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __fromList_list_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___fromList_list_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygonF) __fromList_list_newList() unsafe.Pointer {
	return C.QPolygonF___fromList_list_newList(ptr.Pointer())
}

func (ptr *QPolygonF) __fromStdVector_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygonF___fromStdVector_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __fromStdVector_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___fromStdVector_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygonF) __fromStdVector_newList() unsafe.Pointer {
	return C.QPolygonF___fromStdVector_newList(ptr.Pointer())
}

func (ptr *QPolygonF) __mid_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygonF___mid_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __mid_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___mid_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygonF) __mid_newList() unsafe.Pointer {
	return C.QPolygonF___mid_newList(ptr.Pointer())
}

func (ptr *QPolygonF) __swap_other_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygonF___swap_other_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __swap_other_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___swap_other_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygonF) __swap_other_newList() unsafe.Pointer {
	return C.QPolygonF___swap_other_newList(ptr.Pointer())
}

func (ptr *QPolygonF) __toList_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QPolygonF___toList_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QPolygonF) __toList_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QPolygonF___toList_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QPolygonF) __toList_newList() unsafe.Pointer {
	return C.QPolygonF___toList_newList(ptr.Pointer())
}

type QQuaternion struct {
	ptr unsafe.Pointer
}

type QQuaternion_ITF interface {
	QQuaternion_PTR() *QQuaternion
}

func (ptr *QQuaternion) QQuaternion_PTR() *QQuaternion {
	return ptr
}

func (ptr *QQuaternion) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QQuaternion) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQQuaternion(ptr QQuaternion_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QQuaternion_PTR().Pointer()
	}
	return nil
}

func NewQQuaternionFromPointer(ptr unsafe.Pointer) (n *QQuaternion) {
	n = new(QQuaternion)
	n.SetPointer(ptr)
	return
}
func (ptr *QQuaternion) DestroyQQuaternion() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQQuaternion() *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_NewQQuaternion())
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func NewQQuaternion3(scalar float32, xpos float32, ypos float32, zpos float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_NewQQuaternion3(C.float(scalar), C.float(xpos), C.float(ypos), C.float(zpos)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func NewQQuaternion4(scalar float32, vector QVector3D_ITF) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_NewQQuaternion4(C.float(scalar), PointerFromQVector3D(vector)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func NewQQuaternion5(vector QVector4D_ITF) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_NewQQuaternion5(PointerFromQVector4D(vector)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) Conjugated() *QQuaternion {
	if ptr.Pointer() != nil {
		tmpValue := NewQQuaternionFromPointer(C.QQuaternion_Conjugated(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
		return tmpValue
	}
	return nil
}

func QQuaternion_DotProduct(q1 QQuaternion_ITF, q2 QQuaternion_ITF) float32 {
	return float32(C.QQuaternion_QQuaternion_DotProduct(PointerFromQQuaternion(q1), PointerFromQQuaternion(q2)))
}

func (ptr *QQuaternion) DotProduct(q1 QQuaternion_ITF, q2 QQuaternion_ITF) float32 {
	return float32(C.QQuaternion_QQuaternion_DotProduct(PointerFromQQuaternion(q1), PointerFromQQuaternion(q2)))
}

func QQuaternion_FromAxes(xAxis QVector3D_ITF, yAxis QVector3D_ITF, zAxis QVector3D_ITF) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromAxes(PointerFromQVector3D(xAxis), PointerFromQVector3D(yAxis), PointerFromQVector3D(zAxis)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) FromAxes(xAxis QVector3D_ITF, yAxis QVector3D_ITF, zAxis QVector3D_ITF) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromAxes(PointerFromQVector3D(xAxis), PointerFromQVector3D(yAxis), PointerFromQVector3D(zAxis)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func QQuaternion_FromAxisAndAngle(axis QVector3D_ITF, angle float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromAxisAndAngle(PointerFromQVector3D(axis), C.float(angle)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) FromAxisAndAngle(axis QVector3D_ITF, angle float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromAxisAndAngle(PointerFromQVector3D(axis), C.float(angle)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func QQuaternion_FromAxisAndAngle2(x float32, y float32, z float32, angle float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromAxisAndAngle2(C.float(x), C.float(y), C.float(z), C.float(angle)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) FromAxisAndAngle2(x float32, y float32, z float32, angle float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromAxisAndAngle2(C.float(x), C.float(y), C.float(z), C.float(angle)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func QQuaternion_FromDirection(direction QVector3D_ITF, up QVector3D_ITF) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromDirection(PointerFromQVector3D(direction), PointerFromQVector3D(up)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) FromDirection(direction QVector3D_ITF, up QVector3D_ITF) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromDirection(PointerFromQVector3D(direction), PointerFromQVector3D(up)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func QQuaternion_FromEulerAngles(pitch float32, yaw float32, roll float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromEulerAngles(C.float(pitch), C.float(yaw), C.float(roll)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) FromEulerAngles(pitch float32, yaw float32, roll float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromEulerAngles(C.float(pitch), C.float(yaw), C.float(roll)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func QQuaternion_FromEulerAngles2(eulerAngles QVector3D_ITF) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromEulerAngles2(PointerFromQVector3D(eulerAngles)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) FromEulerAngles2(eulerAngles QVector3D_ITF) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_FromEulerAngles2(PointerFromQVector3D(eulerAngles)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) GetAxes(xAxis QVector3D_ITF, yAxis QVector3D_ITF, zAxis QVector3D_ITF) {
	if ptr.Pointer() != nil {
		C.QQuaternion_GetAxes(ptr.Pointer(), PointerFromQVector3D(xAxis), PointerFromQVector3D(yAxis), PointerFromQVector3D(zAxis))
	}
}

func (ptr *QQuaternion) GetAxisAndAngle(x float32, y float32, z float32, angle float32) {
	if ptr.Pointer() != nil {
		C.QQuaternion_GetAxisAndAngle(ptr.Pointer(), C.float(x), C.float(y), C.float(z), C.float(angle))
	}
}

func (ptr *QQuaternion) GetAxisAndAngle2(axis QVector3D_ITF, angle float32) {
	if ptr.Pointer() != nil {
		C.QQuaternion_GetAxisAndAngle2(ptr.Pointer(), PointerFromQVector3D(axis), C.float(angle))
	}
}

func (ptr *QQuaternion) GetEulerAngles(pitch float32, yaw float32, roll float32) {
	if ptr.Pointer() != nil {
		C.QQuaternion_GetEulerAngles(ptr.Pointer(), C.float(pitch), C.float(yaw), C.float(roll))
	}
}

func (ptr *QQuaternion) Inverted() *QQuaternion {
	if ptr.Pointer() != nil {
		tmpValue := NewQQuaternionFromPointer(C.QQuaternion_Inverted(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
		return tmpValue
	}
	return nil
}

func (ptr *QQuaternion) IsIdentity() bool {
	if ptr.Pointer() != nil {
		return int8(C.QQuaternion_IsIdentity(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QQuaternion) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QQuaternion_IsNull(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QQuaternion) Length() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QQuaternion_Length(ptr.Pointer()))
	}
	return 0
}

func (ptr *QQuaternion) LengthSquared() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QQuaternion_LengthSquared(ptr.Pointer()))
	}
	return 0
}

func QQuaternion_Nlerp(q1 QQuaternion_ITF, q2 QQuaternion_ITF, t float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_Nlerp(PointerFromQQuaternion(q1), PointerFromQQuaternion(q2), C.float(t)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) Nlerp(q1 QQuaternion_ITF, q2 QQuaternion_ITF, t float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_Nlerp(PointerFromQQuaternion(q1), PointerFromQQuaternion(q2), C.float(t)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) Normalize() {
	if ptr.Pointer() != nil {
		C.QQuaternion_Normalize(ptr.Pointer())
	}
}

func (ptr *QQuaternion) Normalized() *QQuaternion {
	if ptr.Pointer() != nil {
		tmpValue := NewQQuaternionFromPointer(C.QQuaternion_Normalized(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
		return tmpValue
	}
	return nil
}

func (ptr *QQuaternion) RotatedVector(vector QVector3D_ITF) *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QQuaternion_RotatedVector(ptr.Pointer(), PointerFromQVector3D(vector)))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func QQuaternion_RotationTo(from QVector3D_ITF, to QVector3D_ITF) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_RotationTo(PointerFromQVector3D(from), PointerFromQVector3D(to)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) RotationTo(from QVector3D_ITF, to QVector3D_ITF) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_RotationTo(PointerFromQVector3D(from), PointerFromQVector3D(to)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) Scalar() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QQuaternion_Scalar(ptr.Pointer()))
	}
	return 0
}

func (ptr *QQuaternion) SetScalar(scalar float32) {
	if ptr.Pointer() != nil {
		C.QQuaternion_SetScalar(ptr.Pointer(), C.float(scalar))
	}
}

func (ptr *QQuaternion) SetVector(vector QVector3D_ITF) {
	if ptr.Pointer() != nil {
		C.QQuaternion_SetVector(ptr.Pointer(), PointerFromQVector3D(vector))
	}
}

func (ptr *QQuaternion) SetVector2(x float32, y float32, z float32) {
	if ptr.Pointer() != nil {
		C.QQuaternion_SetVector2(ptr.Pointer(), C.float(x), C.float(y), C.float(z))
	}
}

func (ptr *QQuaternion) SetX(x float32) {
	if ptr.Pointer() != nil {
		C.QQuaternion_SetX(ptr.Pointer(), C.float(x))
	}
}

func (ptr *QQuaternion) SetY(y float32) {
	if ptr.Pointer() != nil {
		C.QQuaternion_SetY(ptr.Pointer(), C.float(y))
	}
}

func (ptr *QQuaternion) SetZ(z float32) {
	if ptr.Pointer() != nil {
		C.QQuaternion_SetZ(ptr.Pointer(), C.float(z))
	}
}

func QQuaternion_Slerp(q1 QQuaternion_ITF, q2 QQuaternion_ITF, t float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_Slerp(PointerFromQQuaternion(q1), PointerFromQQuaternion(q2), C.float(t)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) Slerp(q1 QQuaternion_ITF, q2 QQuaternion_ITF, t float32) *QQuaternion {
	tmpValue := NewQQuaternionFromPointer(C.QQuaternion_QQuaternion_Slerp(PointerFromQQuaternion(q1), PointerFromQQuaternion(q2), C.float(t)))
	qt.SetFinalizer(tmpValue, (*QQuaternion).DestroyQQuaternion)
	return tmpValue
}

func (ptr *QQuaternion) ToEulerAngles() *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QQuaternion_ToEulerAngles(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func (ptr *QQuaternion) ToVector4D() *QVector4D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector4DFromPointer(C.QQuaternion_ToVector4D(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
		return tmpValue
	}
	return nil
}

func (ptr *QQuaternion) Vector() *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QQuaternion_Vector(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func (ptr *QQuaternion) X() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QQuaternion_X(ptr.Pointer()))
	}
	return 0
}

func (ptr *QQuaternion) Y() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QQuaternion_Y(ptr.Pointer()))
	}
	return 0
}

func (ptr *QQuaternion) Z() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QQuaternion_Z(ptr.Pointer()))
	}
	return 0
}

type QRadialGradient struct {
	QGradient
}

type QRadialGradient_ITF interface {
	QGradient_ITF
	QRadialGradient_PTR() *QRadialGradient
}

func (ptr *QRadialGradient) QRadialGradient_PTR() *QRadialGradient {
	return ptr
}

func (ptr *QRadialGradient) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QGradient_PTR().Pointer()
	}
	return nil
}

func (ptr *QRadialGradient) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QGradient_PTR().SetPointer(p)
	}
}

func PointerFromQRadialGradient(ptr QRadialGradient_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QRadialGradient_PTR().Pointer()
	}
	return nil
}

func NewQRadialGradientFromPointer(ptr unsafe.Pointer) (n *QRadialGradient) {
	n = new(QRadialGradient)
	n.SetPointer(ptr)
	return
}
func (ptr *QRadialGradient) DestroyQRadialGradient() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQRadialGradient() *QRadialGradient {
	tmpValue := NewQRadialGradientFromPointer(C.QRadialGradient_NewQRadialGradient())
	qt.SetFinalizer(tmpValue, (*QRadialGradient).DestroyQRadialGradient)
	return tmpValue
}

func NewQRadialGradient2(center core.QPointF_ITF, radius float64, focalPoint core.QPointF_ITF) *QRadialGradient {
	tmpValue := NewQRadialGradientFromPointer(C.QRadialGradient_NewQRadialGradient2(core.PointerFromQPointF(center), C.double(radius), core.PointerFromQPointF(focalPoint)))
	qt.SetFinalizer(tmpValue, (*QRadialGradient).DestroyQRadialGradient)
	return tmpValue
}

func NewQRadialGradient3(cx float64, cy float64, radius float64, fx float64, fy float64) *QRadialGradient {
	tmpValue := NewQRadialGradientFromPointer(C.QRadialGradient_NewQRadialGradient3(C.double(cx), C.double(cy), C.double(radius), C.double(fx), C.double(fy)))
	qt.SetFinalizer(tmpValue, (*QRadialGradient).DestroyQRadialGradient)
	return tmpValue
}

func NewQRadialGradient4(center core.QPointF_ITF, radius float64) *QRadialGradient {
	tmpValue := NewQRadialGradientFromPointer(C.QRadialGradient_NewQRadialGradient4(core.PointerFromQPointF(center), C.double(radius)))
	qt.SetFinalizer(tmpValue, (*QRadialGradient).DestroyQRadialGradient)
	return tmpValue
}

func NewQRadialGradient5(cx float64, cy float64, radius float64) *QRadialGradient {
	tmpValue := NewQRadialGradientFromPointer(C.QRadialGradient_NewQRadialGradient5(C.double(cx), C.double(cy), C.double(radius)))
	qt.SetFinalizer(tmpValue, (*QRadialGradient).DestroyQRadialGradient)
	return tmpValue
}

func NewQRadialGradient6(center core.QPointF_ITF, centerRadius float64, focalPoint core.QPointF_ITF, focalRadius float64) *QRadialGradient {
	tmpValue := NewQRadialGradientFromPointer(C.QRadialGradient_NewQRadialGradient6(core.PointerFromQPointF(center), C.double(centerRadius), core.PointerFromQPointF(focalPoint), C.double(focalRadius)))
	qt.SetFinalizer(tmpValue, (*QRadialGradient).DestroyQRadialGradient)
	return tmpValue
}

func NewQRadialGradient7(cx float64, cy float64, centerRadius float64, fx float64, fy float64, focalRadius float64) *QRadialGradient {
	tmpValue := NewQRadialGradientFromPointer(C.QRadialGradient_NewQRadialGradient7(C.double(cx), C.double(cy), C.double(centerRadius), C.double(fx), C.double(fy), C.double(focalRadius)))
	qt.SetFinalizer(tmpValue, (*QRadialGradient).DestroyQRadialGradient)
	return tmpValue
}

func (ptr *QRadialGradient) Center() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QRadialGradient_Center(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QRadialGradient) CenterRadius() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRadialGradient_CenterRadius(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRadialGradient) FocalPoint() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QRadialGradient_FocalPoint(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QRadialGradient) FocalRadius() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRadialGradient_FocalRadius(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRadialGradient) Radius() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRadialGradient_Radius(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRadialGradient) SetCenter(center core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QRadialGradient_SetCenter(ptr.Pointer(), core.PointerFromQPointF(center))
	}
}

func (ptr *QRadialGradient) SetCenter2(x float64, y float64) {
	if ptr.Pointer() != nil {
		C.QRadialGradient_SetCenter2(ptr.Pointer(), C.double(x), C.double(y))
	}
}

func (ptr *QRadialGradient) SetCenterRadius(radius float64) {
	if ptr.Pointer() != nil {
		C.QRadialGradient_SetCenterRadius(ptr.Pointer(), C.double(radius))
	}
}

func (ptr *QRadialGradient) SetFocalPoint(focalPoint core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QRadialGradient_SetFocalPoint(ptr.Pointer(), core.PointerFromQPointF(focalPoint))
	}
}

func (ptr *QRadialGradient) SetFocalPoint2(x float64, y float64) {
	if ptr.Pointer() != nil {
		C.QRadialGradient_SetFocalPoint2(ptr.Pointer(), C.double(x), C.double(y))
	}
}

func (ptr *QRadialGradient) SetFocalRadius(radius float64) {
	if ptr.Pointer() != nil {
		C.QRadialGradient_SetFocalRadius(ptr.Pointer(), C.double(radius))
	}
}

func (ptr *QRadialGradient) SetRadius(radius float64) {
	if ptr.Pointer() != nil {
		C.QRadialGradient_SetRadius(ptr.Pointer(), C.double(radius))
	}
}

type QRasterPaintEngine struct {
	QPaintEngine
}

type QRasterPaintEngine_ITF interface {
	QPaintEngine_ITF
	QRasterPaintEngine_PTR() *QRasterPaintEngine
}

func (ptr *QRasterPaintEngine) QRasterPaintEngine_PTR() *QRasterPaintEngine {
	return ptr
}

func (ptr *QRasterPaintEngine) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintEngine_PTR().Pointer()
	}
	return nil
}

func (ptr *QRasterPaintEngine) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QPaintEngine_PTR().SetPointer(p)
	}
}

func PointerFromQRasterPaintEngine(ptr QRasterPaintEngine_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QRasterPaintEngine_PTR().Pointer()
	}
	return nil
}

func NewQRasterPaintEngineFromPointer(ptr unsafe.Pointer) (n *QRasterPaintEngine) {
	n = new(QRasterPaintEngine)
	n.SetPointer(ptr)
	return
}

type QRasterWindow struct {
	QPaintDeviceWindow
}

type QRasterWindow_ITF interface {
	QPaintDeviceWindow_ITF
	QRasterWindow_PTR() *QRasterWindow
}

func (ptr *QRasterWindow) QRasterWindow_PTR() *QRasterWindow {
	return ptr
}

func (ptr *QRasterWindow) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QPaintDeviceWindow_PTR().Pointer()
	}
	return nil
}

func (ptr *QRasterWindow) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QPaintDeviceWindow_PTR().SetPointer(p)
	}
}

func PointerFromQRasterWindow(ptr QRasterWindow_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QRasterWindow_PTR().Pointer()
	}
	return nil
}

func NewQRasterWindowFromPointer(ptr unsafe.Pointer) (n *QRasterWindow) {
	n = new(QRasterWindow)
	n.SetPointer(ptr)
	return
}
func NewQRasterWindow(parent QWindow_ITF) *QRasterWindow {
	tmpValue := NewQRasterWindowFromPointer(C.QRasterWindow_NewQRasterWindow(PointerFromQWindow(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

type QRawFont struct {
	ptr unsafe.Pointer
}

type QRawFont_ITF interface {
	QRawFont_PTR() *QRawFont
}

func (ptr *QRawFont) QRawFont_PTR() *QRawFont {
	return ptr
}

func (ptr *QRawFont) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QRawFont) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQRawFont(ptr QRawFont_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QRawFont_PTR().Pointer()
	}
	return nil
}

func NewQRawFontFromPointer(ptr unsafe.Pointer) (n *QRawFont) {
	n = new(QRawFont)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QRawFont__AntialiasingType
//QRawFont::AntialiasingType
type QRawFont__AntialiasingType int64

const (
	QRawFont__PixelAntialiasing    QRawFont__AntialiasingType = QRawFont__AntialiasingType(0)
	QRawFont__SubPixelAntialiasing QRawFont__AntialiasingType = QRawFont__AntialiasingType(1)
)

//go:generate stringer -type=QRawFont__LayoutFlag
//QRawFont::LayoutFlag
type QRawFont__LayoutFlag int64

const (
	QRawFont__SeparateAdvances QRawFont__LayoutFlag = QRawFont__LayoutFlag(0)
	QRawFont__KernedAdvances   QRawFont__LayoutFlag = QRawFont__LayoutFlag(1)
	QRawFont__UseDesignMetrics QRawFont__LayoutFlag = QRawFont__LayoutFlag(2)
)

func NewQRawFont() *QRawFont {
	tmpValue := NewQRawFontFromPointer(C.QRawFont_NewQRawFont())
	qt.SetFinalizer(tmpValue, (*QRawFont).DestroyQRawFont)
	return tmpValue
}

func NewQRawFont2(fileName string, pixelSize float64, hintingPreference QFont__HintingPreference) *QRawFont {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	tmpValue := NewQRawFontFromPointer(C.QRawFont_NewQRawFont2(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, C.double(pixelSize), C.longlong(hintingPreference)))
	qt.SetFinalizer(tmpValue, (*QRawFont).DestroyQRawFont)
	return tmpValue
}

func NewQRawFont3(fontData core.QByteArray_ITF, pixelSize float64, hintingPreference QFont__HintingPreference) *QRawFont {
	tmpValue := NewQRawFontFromPointer(C.QRawFont_NewQRawFont3(core.PointerFromQByteArray(fontData), C.double(pixelSize), C.longlong(hintingPreference)))
	qt.SetFinalizer(tmpValue, (*QRawFont).DestroyQRawFont)
	return tmpValue
}

func NewQRawFont4(other QRawFont_ITF) *QRawFont {
	tmpValue := NewQRawFontFromPointer(C.QRawFont_NewQRawFont4(PointerFromQRawFont(other)))
	qt.SetFinalizer(tmpValue, (*QRawFont).DestroyQRawFont)
	return tmpValue
}

func (ptr *QRawFont) AdvancesForGlyphIndexes(glyphIndexes []uint, layoutFlags QRawFont__LayoutFlag) []*core.QPointF {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*core.QPointF {
			out := make([]*core.QPointF, int(l.len))
			tmpList := NewQRawFontFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__advancesForGlyphIndexes_atList(i)
			}
			return out
		}(C.QRawFont_AdvancesForGlyphIndexes(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQRawFontFromPointer(NewQRawFontFromPointer(nil).__advancesForGlyphIndexes_glyphIndexes_newList())
			for _, v := range glyphIndexes {
				tmpList.__advancesForGlyphIndexes_glyphIndexes_setList(v)
			}
			return tmpList.Pointer()
		}(), C.longlong(layoutFlags)))
	}
	return make([]*core.QPointF, 0)
}

func (ptr *QRawFont) AdvancesForGlyphIndexes2(glyphIndexes []uint) []*core.QPointF {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*core.QPointF {
			out := make([]*core.QPointF, int(l.len))
			tmpList := NewQRawFontFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__advancesForGlyphIndexes_atList2(i)
			}
			return out
		}(C.QRawFont_AdvancesForGlyphIndexes2(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQRawFontFromPointer(NewQRawFontFromPointer(nil).__advancesForGlyphIndexes_glyphIndexes_newList2())
			for _, v := range glyphIndexes {
				tmpList.__advancesForGlyphIndexes_glyphIndexes_setList2(v)
			}
			return tmpList.Pointer()
		}()))
	}
	return make([]*core.QPointF, 0)
}

func (ptr *QRawFont) AdvancesForGlyphIndexes3(glyphIndexes uint, advances core.QPointF_ITF, numGlyphs int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QRawFont_AdvancesForGlyphIndexes3(ptr.Pointer(), C.uint(uint32(glyphIndexes)), core.PointerFromQPointF(advances), C.int(int32(numGlyphs)))) != 0
	}
	return false
}

func (ptr *QRawFont) AdvancesForGlyphIndexes4(glyphIndexes uint, advances core.QPointF_ITF, numGlyphs int, layoutFlags QRawFont__LayoutFlag) bool {
	if ptr.Pointer() != nil {
		return int8(C.QRawFont_AdvancesForGlyphIndexes4(ptr.Pointer(), C.uint(uint32(glyphIndexes)), core.PointerFromQPointF(advances), C.int(int32(numGlyphs)), C.longlong(layoutFlags))) != 0
	}
	return false
}

func (ptr *QRawFont) AlphaMapForGlyph(glyphIndex uint, antialiasingType QRawFont__AntialiasingType, transform QTransform_ITF) *QImage {
	if ptr.Pointer() != nil {
		tmpValue := NewQImageFromPointer(C.QRawFont_AlphaMapForGlyph(ptr.Pointer(), C.uint(uint32(glyphIndex)), C.longlong(antialiasingType), PointerFromQTransform(transform)))
		qt.SetFinalizer(tmpValue, (*QImage).DestroyQImage)
		return tmpValue
	}
	return nil
}

func (ptr *QRawFont) Ascent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_Ascent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) AverageCharWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_AverageCharWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) BoundingRect(glyphIndex uint) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QRawFont_BoundingRect(ptr.Pointer(), C.uint(uint32(glyphIndex))))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QRawFont) CapHeight() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_CapHeight(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) Descent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_Descent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) FamilyName() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QRawFont_FamilyName(ptr.Pointer()))
	}
	return ""
}

func (ptr *QRawFont) FontTable(tagName string) *core.QByteArray {
	if ptr.Pointer() != nil {
		var tagNameC *C.char
		if tagName != "" {
			tagNameC = C.CString(tagName)
			defer C.free(unsafe.Pointer(tagNameC))
		}
		tmpValue := core.NewQByteArrayFromPointer(C.QRawFont_FontTable(ptr.Pointer(), tagNameC))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func QRawFont_FromFont(font QFont_ITF, writingSystem QFontDatabase__WritingSystem) *QRawFont {
	tmpValue := NewQRawFontFromPointer(C.QRawFont_QRawFont_FromFont(PointerFromQFont(font), C.longlong(writingSystem)))
	qt.SetFinalizer(tmpValue, (*QRawFont).DestroyQRawFont)
	return tmpValue
}

func (ptr *QRawFont) FromFont(font QFont_ITF, writingSystem QFontDatabase__WritingSystem) *QRawFont {
	tmpValue := NewQRawFontFromPointer(C.QRawFont_QRawFont_FromFont(PointerFromQFont(font), C.longlong(writingSystem)))
	qt.SetFinalizer(tmpValue, (*QRawFont).DestroyQRawFont)
	return tmpValue
}

func (ptr *QRawFont) GlyphIndexesForChars(chars core.QChar_ITF, numChars int, glyphIndexes uint, numGlyphs int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QRawFont_GlyphIndexesForChars(ptr.Pointer(), core.PointerFromQChar(chars), C.int(int32(numChars)), C.uint(uint32(glyphIndexes)), C.int(int32(numGlyphs)))) != 0
	}
	return false
}

func (ptr *QRawFont) GlyphIndexesForString(text string) []uint {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		return func(l C.struct_QtGui_PackedList) []uint {
			out := make([]uint, int(l.len))
			tmpList := NewQRawFontFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__glyphIndexesForString_atList(i)
			}
			return out
		}(C.QRawFont_GlyphIndexesForString(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
	}
	return make([]uint, 0)
}

func (ptr *QRawFont) HintingPreference() QFont__HintingPreference {
	if ptr.Pointer() != nil {
		return QFont__HintingPreference(C.QRawFont_HintingPreference(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QRawFont_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QRawFont) Leading() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_Leading(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) LineThickness() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_LineThickness(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) LoadFromData(fontData core.QByteArray_ITF, pixelSize float64, hintingPreference QFont__HintingPreference) {
	if ptr.Pointer() != nil {
		C.QRawFont_LoadFromData(ptr.Pointer(), core.PointerFromQByteArray(fontData), C.double(pixelSize), C.longlong(hintingPreference))
	}
}

func (ptr *QRawFont) LoadFromFile(fileName string, pixelSize float64, hintingPreference QFont__HintingPreference) {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		C.QRawFont_LoadFromFile(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, C.double(pixelSize), C.longlong(hintingPreference))
	}
}

func (ptr *QRawFont) MaxCharWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_MaxCharWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) PathForGlyph(glyphIndex uint) *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QRawFont_PathForGlyph(ptr.Pointer(), C.uint(uint32(glyphIndex))))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QRawFont) PixelSize() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_PixelSize(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) SetPixelSize(pixelSize float64) {
	if ptr.Pointer() != nil {
		C.QRawFont_SetPixelSize(ptr.Pointer(), C.double(pixelSize))
	}
}

func (ptr *QRawFont) Style() QFont__Style {
	if ptr.Pointer() != nil {
		return QFont__Style(C.QRawFont_Style(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) StyleName() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QRawFont_StyleName(ptr.Pointer()))
	}
	return ""
}

func (ptr *QRawFont) SupportedWritingSystems() []QFontDatabase__WritingSystem {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []QFontDatabase__WritingSystem {
			out := make([]QFontDatabase__WritingSystem, int(l.len))
			tmpList := NewQRawFontFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__supportedWritingSystems_atList(i)
			}
			return out
		}(C.QRawFont_SupportedWritingSystems(ptr.Pointer()))
	}
	return make([]QFontDatabase__WritingSystem, 0)
}

func (ptr *QRawFont) SupportsCharacter(character core.QChar_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QRawFont_SupportsCharacter(ptr.Pointer(), core.PointerFromQChar(character))) != 0
	}
	return false
}

func (ptr *QRawFont) SupportsCharacter2(ucs4 uint) bool {
	if ptr.Pointer() != nil {
		return int8(C.QRawFont_SupportsCharacter2(ptr.Pointer(), C.uint(uint32(ucs4)))) != 0
	}
	return false
}

func (ptr *QRawFont) Swap(other QRawFont_ITF) {
	if ptr.Pointer() != nil {
		C.QRawFont_Swap(ptr.Pointer(), PointerFromQRawFont(other))
	}
}

func (ptr *QRawFont) UnderlinePosition() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_UnderlinePosition(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) UnitsPerEm() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_UnitsPerEm(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) Weight() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QRawFont_Weight(ptr.Pointer())))
	}
	return 0
}

func (ptr *QRawFont) XHeight() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QRawFont_XHeight(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRawFont) DestroyQRawFont() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QRawFont_DestroyQRawFont(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QRawFont) __advancesForGlyphIndexes_atList(i int) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QRawFont___advancesForGlyphIndexes_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QRawFont) __advancesForGlyphIndexes_setList(i core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QRawFont___advancesForGlyphIndexes_setList(ptr.Pointer(), core.PointerFromQPointF(i))
	}
}

func (ptr *QRawFont) __advancesForGlyphIndexes_newList() unsafe.Pointer {
	return C.QRawFont___advancesForGlyphIndexes_newList(ptr.Pointer())
}

func (ptr *QRawFont) __advancesForGlyphIndexes_glyphIndexes_atList(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QRawFont___advancesForGlyphIndexes_glyphIndexes_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QRawFont) __advancesForGlyphIndexes_glyphIndexes_setList(i uint) {
	if ptr.Pointer() != nil {
		C.QRawFont___advancesForGlyphIndexes_glyphIndexes_setList(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QRawFont) __advancesForGlyphIndexes_glyphIndexes_newList() unsafe.Pointer {
	return C.QRawFont___advancesForGlyphIndexes_glyphIndexes_newList(ptr.Pointer())
}

func (ptr *QRawFont) __advancesForGlyphIndexes_atList2(i int) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QRawFont___advancesForGlyphIndexes_atList2(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QRawFont) __advancesForGlyphIndexes_setList2(i core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QRawFont___advancesForGlyphIndexes_setList2(ptr.Pointer(), core.PointerFromQPointF(i))
	}
}

func (ptr *QRawFont) __advancesForGlyphIndexes_newList2() unsafe.Pointer {
	return C.QRawFont___advancesForGlyphIndexes_newList2(ptr.Pointer())
}

func (ptr *QRawFont) __advancesForGlyphIndexes_glyphIndexes_atList2(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QRawFont___advancesForGlyphIndexes_glyphIndexes_atList2(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QRawFont) __advancesForGlyphIndexes_glyphIndexes_setList2(i uint) {
	if ptr.Pointer() != nil {
		C.QRawFont___advancesForGlyphIndexes_glyphIndexes_setList2(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QRawFont) __advancesForGlyphIndexes_glyphIndexes_newList2() unsafe.Pointer {
	return C.QRawFont___advancesForGlyphIndexes_glyphIndexes_newList2(ptr.Pointer())
}

func (ptr *QRawFont) __glyphIndexesForString_atList(i int) uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QRawFont___glyphIndexesForString_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QRawFont) __glyphIndexesForString_setList(i uint) {
	if ptr.Pointer() != nil {
		C.QRawFont___glyphIndexesForString_setList(ptr.Pointer(), C.uint(uint32(i)))
	}
}

func (ptr *QRawFont) __glyphIndexesForString_newList() unsafe.Pointer {
	return C.QRawFont___glyphIndexesForString_newList(ptr.Pointer())
}

func (ptr *QRawFont) __supportedWritingSystems_atList(i int) QFontDatabase__WritingSystem {
	if ptr.Pointer() != nil {
		return QFontDatabase__WritingSystem(C.QRawFont___supportedWritingSystems_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QRawFont) __supportedWritingSystems_setList(i QFontDatabase__WritingSystem) {
	if ptr.Pointer() != nil {
		C.QRawFont___supportedWritingSystems_setList(ptr.Pointer(), C.longlong(i))
	}
}

func (ptr *QRawFont) __supportedWritingSystems_newList() unsafe.Pointer {
	return C.QRawFont___supportedWritingSystems_newList(ptr.Pointer())
}

type QRegExpValidator struct {
	QValidator
}

type QRegExpValidator_ITF interface {
	QValidator_ITF
	QRegExpValidator_PTR() *QRegExpValidator
}

func (ptr *QRegExpValidator) QRegExpValidator_PTR() *QRegExpValidator {
	return ptr
}

func (ptr *QRegExpValidator) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QValidator_PTR().Pointer()
	}
	return nil
}

func (ptr *QRegExpValidator) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QValidator_PTR().SetPointer(p)
	}
}

func PointerFromQRegExpValidator(ptr QRegExpValidator_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QRegExpValidator_PTR().Pointer()
	}
	return nil
}

func NewQRegExpValidatorFromPointer(ptr unsafe.Pointer) (n *QRegExpValidator) {
	n = new(QRegExpValidator)
	n.SetPointer(ptr)
	return
}
func NewQRegExpValidator(parent core.QObject_ITF) *QRegExpValidator {
	tmpValue := NewQRegExpValidatorFromPointer(C.QRegExpValidator_NewQRegExpValidator(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQRegExpValidator2(rx core.QRegExp_ITF, parent core.QObject_ITF) *QRegExpValidator {
	tmpValue := NewQRegExpValidatorFromPointer(C.QRegExpValidator_NewQRegExpValidator2(core.PointerFromQRegExp(rx), core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QRegExpValidator) RegExp() *core.QRegExp {
	if ptr.Pointer() != nil {
		return core.NewQRegExpFromPointer(C.QRegExpValidator_RegExp(ptr.Pointer()))
	}
	return nil
}

//export callbackQRegExpValidator_RegExpChanged
func callbackQRegExpValidator_RegExpChanged(ptr unsafe.Pointer, regExp unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "regExpChanged"); signal != nil {
		(*(*func(*core.QRegExp))(signal))(core.NewQRegExpFromPointer(regExp))
	}

}

func (ptr *QRegExpValidator) ConnectRegExpChanged(f func(regExp *core.QRegExp)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "regExpChanged") {
			C.QRegExpValidator_ConnectRegExpChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "regExpChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "regExpChanged"); signal != nil {
			f := func(regExp *core.QRegExp) {
				(*(*func(*core.QRegExp))(signal))(regExp)
				f(regExp)
			}
			qt.ConnectSignal(ptr.Pointer(), "regExpChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "regExpChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QRegExpValidator) DisconnectRegExpChanged() {
	if ptr.Pointer() != nil {
		C.QRegExpValidator_DisconnectRegExpChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "regExpChanged")
	}
}

func (ptr *QRegExpValidator) RegExpChanged(regExp core.QRegExp_ITF) {
	if ptr.Pointer() != nil {
		C.QRegExpValidator_RegExpChanged(ptr.Pointer(), core.PointerFromQRegExp(regExp))
	}
}

func (ptr *QRegExpValidator) SetRegExp(rx core.QRegExp_ITF) {
	if ptr.Pointer() != nil {
		C.QRegExpValidator_SetRegExp(ptr.Pointer(), core.PointerFromQRegExp(rx))
	}
}

//export callbackQRegExpValidator_Validate
func callbackQRegExpValidator_Validate(ptr unsafe.Pointer, input C.struct_QtGui_PackedString, pos C.int) C.longlong {
	if signal := qt.GetSignal(ptr, "validate"); signal != nil {
		return C.longlong((*(*func(string, int) QValidator__State)(signal))(cGoUnpackString(input), int(int32(pos))))
	}

	return C.longlong(NewQRegExpValidatorFromPointer(ptr).ValidateDefault(cGoUnpackString(input), int(int32(pos))))
}

func (ptr *QRegExpValidator) ConnectValidate(f func(input string, pos int) QValidator__State) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "validate"); signal != nil {
			f := func(input string, pos int) QValidator__State {
				(*(*func(string, int) QValidator__State)(signal))(input, pos)
				return f(input, pos)
			}
			qt.ConnectSignal(ptr.Pointer(), "validate", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "validate", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QRegExpValidator) DisconnectValidate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "validate")
	}
}

func (ptr *QRegExpValidator) Validate(input string, pos int) QValidator__State {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		return QValidator__State(C.QRegExpValidator_Validate(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))}, C.int(int32(pos))))
	}
	return 0
}

func (ptr *QRegExpValidator) ValidateDefault(input string, pos int) QValidator__State {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		return QValidator__State(C.QRegExpValidator_ValidateDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))}, C.int(int32(pos))))
	}
	return 0
}

//export callbackQRegExpValidator_DestroyQRegExpValidator
func callbackQRegExpValidator_DestroyQRegExpValidator(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QRegExpValidator"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQRegExpValidatorFromPointer(ptr).DestroyQRegExpValidatorDefault()
	}
}

func (ptr *QRegExpValidator) ConnectDestroyQRegExpValidator(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QRegExpValidator"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QRegExpValidator", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QRegExpValidator", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QRegExpValidator) DisconnectDestroyQRegExpValidator() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QRegExpValidator")
	}
}

func (ptr *QRegExpValidator) DestroyQRegExpValidator() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QRegExpValidator_DestroyQRegExpValidator(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QRegExpValidator) DestroyQRegExpValidatorDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QRegExpValidator_DestroyQRegExpValidatorDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QRegion struct {
	ptr unsafe.Pointer
}

type QRegion_ITF interface {
	QRegion_PTR() *QRegion
}

func (ptr *QRegion) QRegion_PTR() *QRegion {
	return ptr
}

func (ptr *QRegion) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QRegion) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQRegion(ptr QRegion_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QRegion_PTR().Pointer()
	}
	return nil
}

func NewQRegionFromPointer(ptr unsafe.Pointer) (n *QRegion) {
	n = new(QRegion)
	n.SetPointer(ptr)
	return
}
func (ptr *QRegion) DestroyQRegion() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QRegion__RegionType
//QRegion::RegionType
type QRegion__RegionType int64

const (
	QRegion__Rectangle QRegion__RegionType = QRegion__RegionType(0)
	QRegion__Ellipse   QRegion__RegionType = QRegion__RegionType(1)
)

func NewQRegion() *QRegion {
	tmpValue := NewQRegionFromPointer(C.QRegion_NewQRegion())
	qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
	return tmpValue
}

func NewQRegion2(x int, y int, w int, h int, t QRegion__RegionType) *QRegion {
	tmpValue := NewQRegionFromPointer(C.QRegion_NewQRegion2(C.int(int32(x)), C.int(int32(y)), C.int(int32(w)), C.int(int32(h)), C.longlong(t)))
	qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
	return tmpValue
}

func NewQRegion3(r core.QRect_ITF, t QRegion__RegionType) *QRegion {
	tmpValue := NewQRegionFromPointer(C.QRegion_NewQRegion3(core.PointerFromQRect(r), C.longlong(t)))
	qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
	return tmpValue
}

func NewQRegion4(a QPolygon_ITF, fillRule core.Qt__FillRule) *QRegion {
	tmpValue := NewQRegionFromPointer(C.QRegion_NewQRegion4(PointerFromQPolygon(a), C.longlong(fillRule)))
	qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
	return tmpValue
}

func NewQRegion5(r QRegion_ITF) *QRegion {
	tmpValue := NewQRegionFromPointer(C.QRegion_NewQRegion5(PointerFromQRegion(r)))
	qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
	return tmpValue
}

func NewQRegion6(other QRegion_ITF) *QRegion {
	tmpValue := NewQRegionFromPointer(C.QRegion_NewQRegion6(PointerFromQRegion(other)))
	qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
	return tmpValue
}

func NewQRegion7(bm QBitmap_ITF) *QRegion {
	tmpValue := NewQRegionFromPointer(C.QRegion_NewQRegion7(PointerFromQBitmap(bm)))
	qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
	return tmpValue
}

func (ptr *QRegion) BoundingRect() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QRegion_BoundingRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QRegion) Contains(p core.QPoint_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QRegion_Contains(ptr.Pointer(), core.PointerFromQPoint(p))) != 0
	}
	return false
}

func (ptr *QRegion) Contains2(r core.QRect_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QRegion_Contains2(ptr.Pointer(), core.PointerFromQRect(r))) != 0
	}
	return false
}

func (ptr *QRegion) Intersected(r QRegion_ITF) *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QRegion_Intersected(ptr.Pointer(), PointerFromQRegion(r)))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QRegion) Intersected2(rect core.QRect_ITF) *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QRegion_Intersected2(ptr.Pointer(), core.PointerFromQRect(rect)))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QRegion) Intersects(region QRegion_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QRegion_Intersects(ptr.Pointer(), PointerFromQRegion(region))) != 0
	}
	return false
}

func (ptr *QRegion) Intersects2(rect core.QRect_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QRegion_Intersects2(ptr.Pointer(), core.PointerFromQRect(rect))) != 0
	}
	return false
}

func (ptr *QRegion) IsEmpty() bool {
	if ptr.Pointer() != nil {
		return int8(C.QRegion_IsEmpty(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QRegion) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QRegion_IsNull(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QRegion) RectCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QRegion_RectCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QRegion) SetRects(rects core.QRect_ITF, number int) {
	if ptr.Pointer() != nil {
		C.QRegion_SetRects(ptr.Pointer(), core.PointerFromQRect(rects), C.int(int32(number)))
	}
}

func (ptr *QRegion) Subtracted(r QRegion_ITF) *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QRegion_Subtracted(ptr.Pointer(), PointerFromQRegion(r)))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QRegion) Swap(other QRegion_ITF) {
	if ptr.Pointer() != nil {
		C.QRegion_Swap(ptr.Pointer(), PointerFromQRegion(other))
	}
}

func (ptr *QRegion) Translate(dx int, dy int) {
	if ptr.Pointer() != nil {
		C.QRegion_Translate(ptr.Pointer(), C.int(int32(dx)), C.int(int32(dy)))
	}
}

func (ptr *QRegion) Translate2(point core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QRegion_Translate2(ptr.Pointer(), core.PointerFromQPoint(point))
	}
}

func (ptr *QRegion) Translated(dx int, dy int) *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QRegion_Translated(ptr.Pointer(), C.int(int32(dx)), C.int(int32(dy))))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QRegion) Translated2(p core.QPoint_ITF) *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QRegion_Translated2(ptr.Pointer(), core.PointerFromQPoint(p)))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QRegion) United(r QRegion_ITF) *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QRegion_United(ptr.Pointer(), PointerFromQRegion(r)))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QRegion) United2(rect core.QRect_ITF) *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QRegion_United2(ptr.Pointer(), core.PointerFromQRect(rect)))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QRegion) Xored(r QRegion_ITF) *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QRegion_Xored(ptr.Pointer(), PointerFromQRegion(r)))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QRegion) __rects_atList(i int) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QRegion___rects_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QRegion) __rects_setList(i core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QRegion___rects_setList(ptr.Pointer(), core.PointerFromQRect(i))
	}
}

func (ptr *QRegion) __rects_newList() unsafe.Pointer {
	return C.QRegion___rects_newList(ptr.Pointer())
}

type QRegularExpressionValidator struct {
	QValidator
}

type QRegularExpressionValidator_ITF interface {
	QValidator_ITF
	QRegularExpressionValidator_PTR() *QRegularExpressionValidator
}

func (ptr *QRegularExpressionValidator) QRegularExpressionValidator_PTR() *QRegularExpressionValidator {
	return ptr
}

func (ptr *QRegularExpressionValidator) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QValidator_PTR().Pointer()
	}
	return nil
}

func (ptr *QRegularExpressionValidator) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QValidator_PTR().SetPointer(p)
	}
}

func PointerFromQRegularExpressionValidator(ptr QRegularExpressionValidator_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QRegularExpressionValidator_PTR().Pointer()
	}
	return nil
}

func NewQRegularExpressionValidatorFromPointer(ptr unsafe.Pointer) (n *QRegularExpressionValidator) {
	n = new(QRegularExpressionValidator)
	n.SetPointer(ptr)
	return
}
func NewQRegularExpressionValidator(parent core.QObject_ITF) *QRegularExpressionValidator {
	tmpValue := NewQRegularExpressionValidatorFromPointer(C.QRegularExpressionValidator_NewQRegularExpressionValidator(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQRegularExpressionValidator2(re core.QRegularExpression_ITF, parent core.QObject_ITF) *QRegularExpressionValidator {
	tmpValue := NewQRegularExpressionValidatorFromPointer(C.QRegularExpressionValidator_NewQRegularExpressionValidator2(core.PointerFromQRegularExpression(re), core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QRegularExpressionValidator) RegularExpression() *core.QRegularExpression {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRegularExpressionFromPointer(C.QRegularExpressionValidator_RegularExpression(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRegularExpression).DestroyQRegularExpression)
		return tmpValue
	}
	return nil
}

//export callbackQRegularExpressionValidator_RegularExpressionChanged
func callbackQRegularExpressionValidator_RegularExpressionChanged(ptr unsafe.Pointer, re unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "regularExpressionChanged"); signal != nil {
		(*(*func(*core.QRegularExpression))(signal))(core.NewQRegularExpressionFromPointer(re))
	}

}

func (ptr *QRegularExpressionValidator) ConnectRegularExpressionChanged(f func(re *core.QRegularExpression)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "regularExpressionChanged") {
			C.QRegularExpressionValidator_ConnectRegularExpressionChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "regularExpressionChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "regularExpressionChanged"); signal != nil {
			f := func(re *core.QRegularExpression) {
				(*(*func(*core.QRegularExpression))(signal))(re)
				f(re)
			}
			qt.ConnectSignal(ptr.Pointer(), "regularExpressionChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "regularExpressionChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QRegularExpressionValidator) DisconnectRegularExpressionChanged() {
	if ptr.Pointer() != nil {
		C.QRegularExpressionValidator_DisconnectRegularExpressionChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "regularExpressionChanged")
	}
}

func (ptr *QRegularExpressionValidator) RegularExpressionChanged(re core.QRegularExpression_ITF) {
	if ptr.Pointer() != nil {
		C.QRegularExpressionValidator_RegularExpressionChanged(ptr.Pointer(), core.PointerFromQRegularExpression(re))
	}
}

//export callbackQRegularExpressionValidator_SetRegularExpression
func callbackQRegularExpressionValidator_SetRegularExpression(ptr unsafe.Pointer, re unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "setRegularExpression"); signal != nil {
		(*(*func(*core.QRegularExpression))(signal))(core.NewQRegularExpressionFromPointer(re))
	} else {
		NewQRegularExpressionValidatorFromPointer(ptr).SetRegularExpressionDefault(core.NewQRegularExpressionFromPointer(re))
	}
}

func (ptr *QRegularExpressionValidator) ConnectSetRegularExpression(f func(re *core.QRegularExpression)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setRegularExpression"); signal != nil {
			f := func(re *core.QRegularExpression) {
				(*(*func(*core.QRegularExpression))(signal))(re)
				f(re)
			}
			qt.ConnectSignal(ptr.Pointer(), "setRegularExpression", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setRegularExpression", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QRegularExpressionValidator) DisconnectSetRegularExpression() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setRegularExpression")
	}
}

func (ptr *QRegularExpressionValidator) SetRegularExpression(re core.QRegularExpression_ITF) {
	if ptr.Pointer() != nil {
		C.QRegularExpressionValidator_SetRegularExpression(ptr.Pointer(), core.PointerFromQRegularExpression(re))
	}
}

func (ptr *QRegularExpressionValidator) SetRegularExpressionDefault(re core.QRegularExpression_ITF) {
	if ptr.Pointer() != nil {
		C.QRegularExpressionValidator_SetRegularExpressionDefault(ptr.Pointer(), core.PointerFromQRegularExpression(re))
	}
}

//export callbackQRegularExpressionValidator_Validate
func callbackQRegularExpressionValidator_Validate(ptr unsafe.Pointer, input C.struct_QtGui_PackedString, pos C.int) C.longlong {
	if signal := qt.GetSignal(ptr, "validate"); signal != nil {
		return C.longlong((*(*func(string, int) QValidator__State)(signal))(cGoUnpackString(input), int(int32(pos))))
	}

	return C.longlong(NewQRegularExpressionValidatorFromPointer(ptr).ValidateDefault(cGoUnpackString(input), int(int32(pos))))
}

func (ptr *QRegularExpressionValidator) ConnectValidate(f func(input string, pos int) QValidator__State) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "validate"); signal != nil {
			f := func(input string, pos int) QValidator__State {
				(*(*func(string, int) QValidator__State)(signal))(input, pos)
				return f(input, pos)
			}
			qt.ConnectSignal(ptr.Pointer(), "validate", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "validate", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QRegularExpressionValidator) DisconnectValidate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "validate")
	}
}

func (ptr *QRegularExpressionValidator) Validate(input string, pos int) QValidator__State {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		return QValidator__State(C.QRegularExpressionValidator_Validate(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))}, C.int(int32(pos))))
	}
	return 0
}

func (ptr *QRegularExpressionValidator) ValidateDefault(input string, pos int) QValidator__State {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		return QValidator__State(C.QRegularExpressionValidator_ValidateDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))}, C.int(int32(pos))))
	}
	return 0
}

//export callbackQRegularExpressionValidator_DestroyQRegularExpressionValidator
func callbackQRegularExpressionValidator_DestroyQRegularExpressionValidator(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QRegularExpressionValidator"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQRegularExpressionValidatorFromPointer(ptr).DestroyQRegularExpressionValidatorDefault()
	}
}

func (ptr *QRegularExpressionValidator) ConnectDestroyQRegularExpressionValidator(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QRegularExpressionValidator"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QRegularExpressionValidator", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QRegularExpressionValidator", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QRegularExpressionValidator) DisconnectDestroyQRegularExpressionValidator() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QRegularExpressionValidator")
	}
}

func (ptr *QRegularExpressionValidator) DestroyQRegularExpressionValidator() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QRegularExpressionValidator_DestroyQRegularExpressionValidator(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QRegularExpressionValidator) DestroyQRegularExpressionValidatorDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QRegularExpressionValidator_DestroyQRegularExpressionValidatorDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QResizeEvent struct {
	core.QEvent
}

type QResizeEvent_ITF interface {
	core.QEvent_ITF
	QResizeEvent_PTR() *QResizeEvent
}

func (ptr *QResizeEvent) QResizeEvent_PTR() *QResizeEvent {
	return ptr
}

func (ptr *QResizeEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QResizeEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQResizeEvent(ptr QResizeEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QResizeEvent_PTR().Pointer()
	}
	return nil
}

func NewQResizeEventFromPointer(ptr unsafe.Pointer) (n *QResizeEvent) {
	n = new(QResizeEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QResizeEvent) DestroyQResizeEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQResizeEvent(size core.QSize_ITF, oldSize core.QSize_ITF) *QResizeEvent {
	tmpValue := NewQResizeEventFromPointer(C.QResizeEvent_NewQResizeEvent(core.PointerFromQSize(size), core.PointerFromQSize(oldSize)))
	qt.SetFinalizer(tmpValue, (*QResizeEvent).DestroyQResizeEvent)
	return tmpValue
}

func (ptr *QResizeEvent) OldSize() *core.QSize {
	if ptr.Pointer() != nil {
		return core.NewQSizeFromPointer(C.QResizeEvent_OldSize(ptr.Pointer()))
	}
	return nil
}

func (ptr *QResizeEvent) Size() *core.QSize {
	if ptr.Pointer() != nil {
		return core.NewQSizeFromPointer(C.QResizeEvent_Size(ptr.Pointer()))
	}
	return nil
}

type QRgba64 struct {
	ptr unsafe.Pointer
}

type QRgba64_ITF interface {
	QRgba64_PTR() *QRgba64
}

func (ptr *QRgba64) QRgba64_PTR() *QRgba64 {
	return ptr
}

func (ptr *QRgba64) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QRgba64) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQRgba64(ptr QRgba64_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QRgba64_PTR().Pointer()
	}
	return nil
}

func NewQRgba64FromPointer(ptr unsafe.Pointer) (n *QRgba64) {
	n = new(QRgba64)
	n.SetPointer(ptr)
	return
}
func (ptr *QRgba64) DestroyQRgba64() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func (ptr *QRgba64) Alpha() uint16 {
	if ptr.Pointer() != nil {
		return uint16(C.QRgba64_Alpha(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRgba64) Alpha8() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QRgba64_Alpha8(ptr.Pointer()))
	}
	return ""
}

func (ptr *QRgba64) Blue() uint16 {
	if ptr.Pointer() != nil {
		return uint16(C.QRgba64_Blue(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRgba64) Blue8() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QRgba64_Blue8(ptr.Pointer()))
	}
	return ""
}

func (ptr *QRgba64) Green() uint16 {
	if ptr.Pointer() != nil {
		return uint16(C.QRgba64_Green(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRgba64) Green8() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QRgba64_Green8(ptr.Pointer()))
	}
	return ""
}

func (ptr *QRgba64) IsOpaque() bool {
	if ptr.Pointer() != nil {
		return int8(C.QRgba64_IsOpaque(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QRgba64) IsTransparent() bool {
	if ptr.Pointer() != nil {
		return int8(C.QRgba64_IsTransparent(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QRgba64) Red() uint16 {
	if ptr.Pointer() != nil {
		return uint16(C.QRgba64_Red(ptr.Pointer()))
	}
	return 0
}

func (ptr *QRgba64) Red8() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QRgba64_Red8(ptr.Pointer()))
	}
	return ""
}

func (ptr *QRgba64) SetAlpha(alpha uint16) {
	if ptr.Pointer() != nil {
		C.QRgba64_SetAlpha(ptr.Pointer(), C.ushort(alpha))
	}
}

func (ptr *QRgba64) SetBlue(blue uint16) {
	if ptr.Pointer() != nil {
		C.QRgba64_SetBlue(ptr.Pointer(), C.ushort(blue))
	}
}

func (ptr *QRgba64) SetGreen(green uint16) {
	if ptr.Pointer() != nil {
		C.QRgba64_SetGreen(ptr.Pointer(), C.ushort(green))
	}
}

func (ptr *QRgba64) SetRed(red uint16) {
	if ptr.Pointer() != nil {
		C.QRgba64_SetRed(ptr.Pointer(), C.ushort(red))
	}
}

func (ptr *QRgba64) ToArgb32() uint {
	if ptr.Pointer() != nil {
		return uint(uint32(C.QRgba64_ToArgb32(ptr.Pointer())))
	}
	return 0
}

func (ptr *QRgba64) ToRgb16() uint16 {
	if ptr.Pointer() != nil {
		return uint16(C.QRgba64_ToRgb16(ptr.Pointer()))
	}
	return 0
}

type QScreen struct {
	core.QObject
}

type QScreen_ITF interface {
	core.QObject_ITF
	QScreen_PTR() *QScreen
}

func (ptr *QScreen) QScreen_PTR() *QScreen {
	return ptr
}

func (ptr *QScreen) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QScreen) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQScreen(ptr QScreen_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QScreen_PTR().Pointer()
	}
	return nil
}

func NewQScreenFromPointer(ptr unsafe.Pointer) (n *QScreen) {
	n = new(QScreen)
	n.SetPointer(ptr)
	return
}
func (ptr *QScreen) AngleBetween(a core.Qt__ScreenOrientation, b core.Qt__ScreenOrientation) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QScreen_AngleBetween(ptr.Pointer(), C.longlong(a), C.longlong(b))))
	}
	return 0
}

func (ptr *QScreen) AvailableGeometry() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QScreen_AvailableGeometry(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

//export callbackQScreen_AvailableGeometryChanged
func callbackQScreen_AvailableGeometryChanged(ptr unsafe.Pointer, geometry unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "availableGeometryChanged"); signal != nil {
		(*(*func(*core.QRect))(signal))(core.NewQRectFromPointer(geometry))
	}

}

func (ptr *QScreen) ConnectAvailableGeometryChanged(f func(geometry *core.QRect)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "availableGeometryChanged") {
			C.QScreen_ConnectAvailableGeometryChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "availableGeometryChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "availableGeometryChanged"); signal != nil {
			f := func(geometry *core.QRect) {
				(*(*func(*core.QRect))(signal))(geometry)
				f(geometry)
			}
			qt.ConnectSignal(ptr.Pointer(), "availableGeometryChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "availableGeometryChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreen) DisconnectAvailableGeometryChanged() {
	if ptr.Pointer() != nil {
		C.QScreen_DisconnectAvailableGeometryChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "availableGeometryChanged")
	}
}

func (ptr *QScreen) AvailableGeometryChanged(geometry core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen_AvailableGeometryChanged(ptr.Pointer(), core.PointerFromQRect(geometry))
	}
}

func (ptr *QScreen) AvailableSize() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QScreen_AvailableSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) AvailableVirtualGeometry() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QScreen_AvailableVirtualGeometry(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) AvailableVirtualSize() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QScreen_AvailableVirtualSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) Depth() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QScreen_Depth(ptr.Pointer())))
	}
	return 0
}

func (ptr *QScreen) DevicePixelRatio() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QScreen_DevicePixelRatio(ptr.Pointer()))
	}
	return 0
}

func (ptr *QScreen) Geometry() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QScreen_Geometry(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

//export callbackQScreen_GeometryChanged
func callbackQScreen_GeometryChanged(ptr unsafe.Pointer, geometry unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "geometryChanged"); signal != nil {
		(*(*func(*core.QRect))(signal))(core.NewQRectFromPointer(geometry))
	}

}

func (ptr *QScreen) ConnectGeometryChanged(f func(geometry *core.QRect)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "geometryChanged") {
			C.QScreen_ConnectGeometryChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "geometryChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "geometryChanged"); signal != nil {
			f := func(geometry *core.QRect) {
				(*(*func(*core.QRect))(signal))(geometry)
				f(geometry)
			}
			qt.ConnectSignal(ptr.Pointer(), "geometryChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "geometryChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreen) DisconnectGeometryChanged() {
	if ptr.Pointer() != nil {
		C.QScreen_DisconnectGeometryChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "geometryChanged")
	}
}

func (ptr *QScreen) GeometryChanged(geometry core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen_GeometryChanged(ptr.Pointer(), core.PointerFromQRect(geometry))
	}
}

func (ptr *QScreen) GrabWindow(window uintptr, x int, y int, width int, height int) *QPixmap {
	if ptr.Pointer() != nil {
		tmpValue := NewQPixmapFromPointer(C.QScreen_GrabWindow(ptr.Pointer(), C.uintptr_t(window), C.int(int32(x)), C.int(int32(y)), C.int(int32(width)), C.int(int32(height))))
		qt.SetFinalizer(tmpValue, (*QPixmap).DestroyQPixmap)
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) IsLandscape(o core.Qt__ScreenOrientation) bool {
	if ptr.Pointer() != nil {
		return int8(C.QScreen_IsLandscape(ptr.Pointer(), C.longlong(o))) != 0
	}
	return false
}

func (ptr *QScreen) IsPortrait(o core.Qt__ScreenOrientation) bool {
	if ptr.Pointer() != nil {
		return int8(C.QScreen_IsPortrait(ptr.Pointer(), C.longlong(o))) != 0
	}
	return false
}

func (ptr *QScreen) LogicalDotsPerInch() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QScreen_LogicalDotsPerInch(ptr.Pointer()))
	}
	return 0
}

//export callbackQScreen_LogicalDotsPerInchChanged
func callbackQScreen_LogicalDotsPerInchChanged(ptr unsafe.Pointer, dpi C.double) {
	if signal := qt.GetSignal(ptr, "logicalDotsPerInchChanged"); signal != nil {
		(*(*func(float64))(signal))(float64(dpi))
	}

}

func (ptr *QScreen) ConnectLogicalDotsPerInchChanged(f func(dpi float64)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "logicalDotsPerInchChanged") {
			C.QScreen_ConnectLogicalDotsPerInchChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "logicalDotsPerInchChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "logicalDotsPerInchChanged"); signal != nil {
			f := func(dpi float64) {
				(*(*func(float64))(signal))(dpi)
				f(dpi)
			}
			qt.ConnectSignal(ptr.Pointer(), "logicalDotsPerInchChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "logicalDotsPerInchChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreen) DisconnectLogicalDotsPerInchChanged() {
	if ptr.Pointer() != nil {
		C.QScreen_DisconnectLogicalDotsPerInchChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "logicalDotsPerInchChanged")
	}
}

func (ptr *QScreen) LogicalDotsPerInchChanged(dpi float64) {
	if ptr.Pointer() != nil {
		C.QScreen_LogicalDotsPerInchChanged(ptr.Pointer(), C.double(dpi))
	}
}

func (ptr *QScreen) LogicalDotsPerInchX() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QScreen_LogicalDotsPerInchX(ptr.Pointer()))
	}
	return 0
}

func (ptr *QScreen) LogicalDotsPerInchY() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QScreen_LogicalDotsPerInchY(ptr.Pointer()))
	}
	return 0
}

func (ptr *QScreen) Manufacturer() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QScreen_Manufacturer(ptr.Pointer()))
	}
	return ""
}

func (ptr *QScreen) MapBetween(a core.Qt__ScreenOrientation, b core.Qt__ScreenOrientation, rect core.QRect_ITF) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QScreen_MapBetween(ptr.Pointer(), C.longlong(a), C.longlong(b), core.PointerFromQRect(rect)))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) Model() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QScreen_Model(ptr.Pointer()))
	}
	return ""
}

func (ptr *QScreen) Name() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QScreen_Name(ptr.Pointer()))
	}
	return ""
}

func (ptr *QScreen) NativeOrientation() core.Qt__ScreenOrientation {
	if ptr.Pointer() != nil {
		return core.Qt__ScreenOrientation(C.QScreen_NativeOrientation(ptr.Pointer()))
	}
	return 0
}

func (ptr *QScreen) Orientation() core.Qt__ScreenOrientation {
	if ptr.Pointer() != nil {
		return core.Qt__ScreenOrientation(C.QScreen_Orientation(ptr.Pointer()))
	}
	return 0
}

//export callbackQScreen_OrientationChanged
func callbackQScreen_OrientationChanged(ptr unsafe.Pointer, orientation C.longlong) {
	if signal := qt.GetSignal(ptr, "orientationChanged"); signal != nil {
		(*(*func(core.Qt__ScreenOrientation))(signal))(core.Qt__ScreenOrientation(orientation))
	}

}

func (ptr *QScreen) ConnectOrientationChanged(f func(orientation core.Qt__ScreenOrientation)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "orientationChanged") {
			C.QScreen_ConnectOrientationChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "orientationChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "orientationChanged"); signal != nil {
			f := func(orientation core.Qt__ScreenOrientation) {
				(*(*func(core.Qt__ScreenOrientation))(signal))(orientation)
				f(orientation)
			}
			qt.ConnectSignal(ptr.Pointer(), "orientationChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "orientationChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreen) DisconnectOrientationChanged() {
	if ptr.Pointer() != nil {
		C.QScreen_DisconnectOrientationChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "orientationChanged")
	}
}

func (ptr *QScreen) OrientationChanged(orientation core.Qt__ScreenOrientation) {
	if ptr.Pointer() != nil {
		C.QScreen_OrientationChanged(ptr.Pointer(), C.longlong(orientation))
	}
}

func (ptr *QScreen) OrientationUpdateMask() core.Qt__ScreenOrientation {
	if ptr.Pointer() != nil {
		return core.Qt__ScreenOrientation(C.QScreen_OrientationUpdateMask(ptr.Pointer()))
	}
	return 0
}

func (ptr *QScreen) PhysicalDotsPerInch() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QScreen_PhysicalDotsPerInch(ptr.Pointer()))
	}
	return 0
}

//export callbackQScreen_PhysicalDotsPerInchChanged
func callbackQScreen_PhysicalDotsPerInchChanged(ptr unsafe.Pointer, dpi C.double) {
	if signal := qt.GetSignal(ptr, "physicalDotsPerInchChanged"); signal != nil {
		(*(*func(float64))(signal))(float64(dpi))
	}

}

func (ptr *QScreen) ConnectPhysicalDotsPerInchChanged(f func(dpi float64)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "physicalDotsPerInchChanged") {
			C.QScreen_ConnectPhysicalDotsPerInchChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "physicalDotsPerInchChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "physicalDotsPerInchChanged"); signal != nil {
			f := func(dpi float64) {
				(*(*func(float64))(signal))(dpi)
				f(dpi)
			}
			qt.ConnectSignal(ptr.Pointer(), "physicalDotsPerInchChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "physicalDotsPerInchChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreen) DisconnectPhysicalDotsPerInchChanged() {
	if ptr.Pointer() != nil {
		C.QScreen_DisconnectPhysicalDotsPerInchChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "physicalDotsPerInchChanged")
	}
}

func (ptr *QScreen) PhysicalDotsPerInchChanged(dpi float64) {
	if ptr.Pointer() != nil {
		C.QScreen_PhysicalDotsPerInchChanged(ptr.Pointer(), C.double(dpi))
	}
}

func (ptr *QScreen) PhysicalDotsPerInchX() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QScreen_PhysicalDotsPerInchX(ptr.Pointer()))
	}
	return 0
}

func (ptr *QScreen) PhysicalDotsPerInchY() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QScreen_PhysicalDotsPerInchY(ptr.Pointer()))
	}
	return 0
}

func (ptr *QScreen) PhysicalSize() *core.QSizeF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFFromPointer(C.QScreen_PhysicalSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

//export callbackQScreen_PhysicalSizeChanged
func callbackQScreen_PhysicalSizeChanged(ptr unsafe.Pointer, size unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "physicalSizeChanged"); signal != nil {
		(*(*func(*core.QSizeF))(signal))(core.NewQSizeFFromPointer(size))
	}

}

func (ptr *QScreen) ConnectPhysicalSizeChanged(f func(size *core.QSizeF)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "physicalSizeChanged") {
			C.QScreen_ConnectPhysicalSizeChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "physicalSizeChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "physicalSizeChanged"); signal != nil {
			f := func(size *core.QSizeF) {
				(*(*func(*core.QSizeF))(signal))(size)
				f(size)
			}
			qt.ConnectSignal(ptr.Pointer(), "physicalSizeChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "physicalSizeChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreen) DisconnectPhysicalSizeChanged() {
	if ptr.Pointer() != nil {
		C.QScreen_DisconnectPhysicalSizeChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "physicalSizeChanged")
	}
}

func (ptr *QScreen) PhysicalSizeChanged(size core.QSizeF_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen_PhysicalSizeChanged(ptr.Pointer(), core.PointerFromQSizeF(size))
	}
}

func (ptr *QScreen) PrimaryOrientation() core.Qt__ScreenOrientation {
	if ptr.Pointer() != nil {
		return core.Qt__ScreenOrientation(C.QScreen_PrimaryOrientation(ptr.Pointer()))
	}
	return 0
}

//export callbackQScreen_PrimaryOrientationChanged
func callbackQScreen_PrimaryOrientationChanged(ptr unsafe.Pointer, orientation C.longlong) {
	if signal := qt.GetSignal(ptr, "primaryOrientationChanged"); signal != nil {
		(*(*func(core.Qt__ScreenOrientation))(signal))(core.Qt__ScreenOrientation(orientation))
	}

}

func (ptr *QScreen) ConnectPrimaryOrientationChanged(f func(orientation core.Qt__ScreenOrientation)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "primaryOrientationChanged") {
			C.QScreen_ConnectPrimaryOrientationChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "primaryOrientationChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "primaryOrientationChanged"); signal != nil {
			f := func(orientation core.Qt__ScreenOrientation) {
				(*(*func(core.Qt__ScreenOrientation))(signal))(orientation)
				f(orientation)
			}
			qt.ConnectSignal(ptr.Pointer(), "primaryOrientationChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "primaryOrientationChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreen) DisconnectPrimaryOrientationChanged() {
	if ptr.Pointer() != nil {
		C.QScreen_DisconnectPrimaryOrientationChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "primaryOrientationChanged")
	}
}

func (ptr *QScreen) PrimaryOrientationChanged(orientation core.Qt__ScreenOrientation) {
	if ptr.Pointer() != nil {
		C.QScreen_PrimaryOrientationChanged(ptr.Pointer(), C.longlong(orientation))
	}
}

func (ptr *QScreen) RefreshRate() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QScreen_RefreshRate(ptr.Pointer()))
	}
	return 0
}

//export callbackQScreen_RefreshRateChanged
func callbackQScreen_RefreshRateChanged(ptr unsafe.Pointer, refreshRate C.double) {
	if signal := qt.GetSignal(ptr, "refreshRateChanged"); signal != nil {
		(*(*func(float64))(signal))(float64(refreshRate))
	}

}

func (ptr *QScreen) ConnectRefreshRateChanged(f func(refreshRate float64)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "refreshRateChanged") {
			C.QScreen_ConnectRefreshRateChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "refreshRateChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "refreshRateChanged"); signal != nil {
			f := func(refreshRate float64) {
				(*(*func(float64))(signal))(refreshRate)
				f(refreshRate)
			}
			qt.ConnectSignal(ptr.Pointer(), "refreshRateChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "refreshRateChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreen) DisconnectRefreshRateChanged() {
	if ptr.Pointer() != nil {
		C.QScreen_DisconnectRefreshRateChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "refreshRateChanged")
	}
}

func (ptr *QScreen) RefreshRateChanged(refreshRate float64) {
	if ptr.Pointer() != nil {
		C.QScreen_RefreshRateChanged(ptr.Pointer(), C.double(refreshRate))
	}
}

func (ptr *QScreen) SerialNumber() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QScreen_SerialNumber(ptr.Pointer()))
	}
	return ""
}

func (ptr *QScreen) SetOrientationUpdateMask(mask core.Qt__ScreenOrientation) {
	if ptr.Pointer() != nil {
		C.QScreen_SetOrientationUpdateMask(ptr.Pointer(), C.longlong(mask))
	}
}

func (ptr *QScreen) Size() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QScreen_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) TransformBetween(a core.Qt__ScreenOrientation, b core.Qt__ScreenOrientation, target core.QRect_ITF) *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QScreen_TransformBetween(ptr.Pointer(), C.longlong(a), C.longlong(b), core.PointerFromQRect(target)))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) VirtualGeometry() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QScreen_VirtualGeometry(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

//export callbackQScreen_VirtualGeometryChanged
func callbackQScreen_VirtualGeometryChanged(ptr unsafe.Pointer, rect unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "virtualGeometryChanged"); signal != nil {
		(*(*func(*core.QRect))(signal))(core.NewQRectFromPointer(rect))
	}

}

func (ptr *QScreen) ConnectVirtualGeometryChanged(f func(rect *core.QRect)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "virtualGeometryChanged") {
			C.QScreen_ConnectVirtualGeometryChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "virtualGeometryChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "virtualGeometryChanged"); signal != nil {
			f := func(rect *core.QRect) {
				(*(*func(*core.QRect))(signal))(rect)
				f(rect)
			}
			qt.ConnectSignal(ptr.Pointer(), "virtualGeometryChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "virtualGeometryChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreen) DisconnectVirtualGeometryChanged() {
	if ptr.Pointer() != nil {
		C.QScreen_DisconnectVirtualGeometryChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "virtualGeometryChanged")
	}
}

func (ptr *QScreen) VirtualGeometryChanged(rect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen_VirtualGeometryChanged(ptr.Pointer(), core.PointerFromQRect(rect))
	}
}

func (ptr *QScreen) VirtualSiblings() []*QScreen {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QScreen {
			out := make([]*QScreen, int(l.len))
			tmpList := NewQScreenFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__virtualSiblings_atList(i)
			}
			return out
		}(C.QScreen_VirtualSiblings(ptr.Pointer()))
	}
	return make([]*QScreen, 0)
}

func (ptr *QScreen) VirtualSize() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QScreen_VirtualSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

//export callbackQScreen_DestroyQScreen
func callbackQScreen_DestroyQScreen(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QScreen"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQScreenFromPointer(ptr).DestroyQScreenDefault()
	}
}

func (ptr *QScreen) ConnectDestroyQScreen(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QScreen"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QScreen", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QScreen", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreen) DisconnectDestroyQScreen() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QScreen")
	}
}

func (ptr *QScreen) DestroyQScreen() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QScreen_DestroyQScreen(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QScreen) DestroyQScreenDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QScreen_DestroyQScreenDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QScreen) __virtualSiblings_atList(i int) *QScreen {
	if ptr.Pointer() != nil {
		tmpValue := NewQScreenFromPointer(C.QScreen___virtualSiblings_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) __virtualSiblings_setList(i QScreen_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen___virtualSiblings_setList(ptr.Pointer(), PointerFromQScreen(i))
	}
}

func (ptr *QScreen) __virtualSiblings_newList() unsafe.Pointer {
	return C.QScreen___virtualSiblings_newList(ptr.Pointer())
}

func (ptr *QScreen) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QScreen___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QScreen) __children_newList() unsafe.Pointer {
	return C.QScreen___children_newList(ptr.Pointer())
}

func (ptr *QScreen) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QScreen___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QScreen) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QScreen___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QScreen) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QScreen___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QScreen) __findChildren_newList() unsafe.Pointer {
	return C.QScreen___findChildren_newList(ptr.Pointer())
}

func (ptr *QScreen) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QScreen___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QScreen) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QScreen) __findChildren_newList3() unsafe.Pointer {
	return C.QScreen___findChildren_newList3(ptr.Pointer())
}

//export callbackQScreen_ChildEvent
func callbackQScreen_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQScreenFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QScreen) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQScreen_ConnectNotify
func callbackQScreen_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQScreenFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QScreen) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQScreen_CustomEvent
func callbackQScreen_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQScreenFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QScreen) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQScreen_DeleteLater
func callbackQScreen_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQScreenFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QScreen) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QScreen_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQScreen_Destroyed
func callbackQScreen_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQScreen_DisconnectNotify
func callbackQScreen_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQScreenFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QScreen) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQScreen_Event
func callbackQScreen_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQScreenFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QScreen) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QScreen_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQScreen_EventFilter
func callbackQScreen_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQScreenFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QScreen) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QScreen_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQScreen_MetaObject
func callbackQScreen_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQScreenFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QScreen) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QScreen_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQScreen_ObjectNameChanged
func callbackQScreen_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQScreen_TimerEvent
func callbackQScreen_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQScreenFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QScreen) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QScreen_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QScreenOrientationChangeEvent struct {
	core.QEvent
}

type QScreenOrientationChangeEvent_ITF interface {
	core.QEvent_ITF
	QScreenOrientationChangeEvent_PTR() *QScreenOrientationChangeEvent
}

func (ptr *QScreenOrientationChangeEvent) QScreenOrientationChangeEvent_PTR() *QScreenOrientationChangeEvent {
	return ptr
}

func (ptr *QScreenOrientationChangeEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QScreenOrientationChangeEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQScreenOrientationChangeEvent(ptr QScreenOrientationChangeEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QScreenOrientationChangeEvent_PTR().Pointer()
	}
	return nil
}

func NewQScreenOrientationChangeEventFromPointer(ptr unsafe.Pointer) (n *QScreenOrientationChangeEvent) {
	n = new(QScreenOrientationChangeEvent)
	n.SetPointer(ptr)
	return
}
func NewQScreenOrientationChangeEvent(screen QScreen_ITF, screenOrientation core.Qt__ScreenOrientation) *QScreenOrientationChangeEvent {
	tmpValue := NewQScreenOrientationChangeEventFromPointer(C.QScreenOrientationChangeEvent_NewQScreenOrientationChangeEvent(PointerFromQScreen(screen), C.longlong(screenOrientation)))
	qt.SetFinalizer(tmpValue, (*QScreenOrientationChangeEvent).DestroyQScreenOrientationChangeEvent)
	return tmpValue
}

func (ptr *QScreenOrientationChangeEvent) Orientation() core.Qt__ScreenOrientation {
	if ptr.Pointer() != nil {
		return core.Qt__ScreenOrientation(C.QScreenOrientationChangeEvent_Orientation(ptr.Pointer()))
	}
	return 0
}

func (ptr *QScreenOrientationChangeEvent) Screen() *QScreen {
	if ptr.Pointer() != nil {
		tmpValue := NewQScreenFromPointer(C.QScreenOrientationChangeEvent_Screen(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQScreenOrientationChangeEvent_DestroyQScreenOrientationChangeEvent
func callbackQScreenOrientationChangeEvent_DestroyQScreenOrientationChangeEvent(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QScreenOrientationChangeEvent"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQScreenOrientationChangeEventFromPointer(ptr).DestroyQScreenOrientationChangeEventDefault()
	}
}

func (ptr *QScreenOrientationChangeEvent) ConnectDestroyQScreenOrientationChangeEvent(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QScreenOrientationChangeEvent"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QScreenOrientationChangeEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QScreenOrientationChangeEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScreenOrientationChangeEvent) DisconnectDestroyQScreenOrientationChangeEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QScreenOrientationChangeEvent")
	}
}

func (ptr *QScreenOrientationChangeEvent) DestroyQScreenOrientationChangeEvent() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QScreenOrientationChangeEvent_DestroyQScreenOrientationChangeEvent(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QScreenOrientationChangeEvent) DestroyQScreenOrientationChangeEventDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QScreenOrientationChangeEvent_DestroyQScreenOrientationChangeEventDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QScrollEvent struct {
	core.QEvent
}

type QScrollEvent_ITF interface {
	core.QEvent_ITF
	QScrollEvent_PTR() *QScrollEvent
}

func (ptr *QScrollEvent) QScrollEvent_PTR() *QScrollEvent {
	return ptr
}

func (ptr *QScrollEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QScrollEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQScrollEvent(ptr QScrollEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QScrollEvent_PTR().Pointer()
	}
	return nil
}

func NewQScrollEventFromPointer(ptr unsafe.Pointer) (n *QScrollEvent) {
	n = new(QScrollEvent)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QScrollEvent__ScrollState
//QScrollEvent::ScrollState
type QScrollEvent__ScrollState int64

const (
	QScrollEvent__ScrollStarted  QScrollEvent__ScrollState = QScrollEvent__ScrollState(0)
	QScrollEvent__ScrollUpdated  QScrollEvent__ScrollState = QScrollEvent__ScrollState(1)
	QScrollEvent__ScrollFinished QScrollEvent__ScrollState = QScrollEvent__ScrollState(2)
)

func NewQScrollEvent(contentPos core.QPointF_ITF, overshootDistance core.QPointF_ITF, scrollState QScrollEvent__ScrollState) *QScrollEvent {
	tmpValue := NewQScrollEventFromPointer(C.QScrollEvent_NewQScrollEvent(core.PointerFromQPointF(contentPos), core.PointerFromQPointF(overshootDistance), C.longlong(scrollState)))
	qt.SetFinalizer(tmpValue, (*QScrollEvent).DestroyQScrollEvent)
	return tmpValue
}

func (ptr *QScrollEvent) ContentPos() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QScrollEvent_ContentPos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QScrollEvent) OvershootDistance() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QScrollEvent_OvershootDistance(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QScrollEvent) ScrollState() QScrollEvent__ScrollState {
	if ptr.Pointer() != nil {
		return QScrollEvent__ScrollState(C.QScrollEvent_ScrollState(ptr.Pointer()))
	}
	return 0
}

//export callbackQScrollEvent_DestroyQScrollEvent
func callbackQScrollEvent_DestroyQScrollEvent(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QScrollEvent"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQScrollEventFromPointer(ptr).DestroyQScrollEventDefault()
	}
}

func (ptr *QScrollEvent) ConnectDestroyQScrollEvent(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QScrollEvent"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QScrollEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QScrollEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScrollEvent) DisconnectDestroyQScrollEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QScrollEvent")
	}
}

func (ptr *QScrollEvent) DestroyQScrollEvent() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QScrollEvent_DestroyQScrollEvent(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QScrollEvent) DestroyQScrollEventDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QScrollEvent_DestroyQScrollEventDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QScrollPrepareEvent struct {
	core.QEvent
}

type QScrollPrepareEvent_ITF interface {
	core.QEvent_ITF
	QScrollPrepareEvent_PTR() *QScrollPrepareEvent
}

func (ptr *QScrollPrepareEvent) QScrollPrepareEvent_PTR() *QScrollPrepareEvent {
	return ptr
}

func (ptr *QScrollPrepareEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QScrollPrepareEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQScrollPrepareEvent(ptr QScrollPrepareEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QScrollPrepareEvent_PTR().Pointer()
	}
	return nil
}

func NewQScrollPrepareEventFromPointer(ptr unsafe.Pointer) (n *QScrollPrepareEvent) {
	n = new(QScrollPrepareEvent)
	n.SetPointer(ptr)
	return
}
func NewQScrollPrepareEvent(startPos core.QPointF_ITF) *QScrollPrepareEvent {
	tmpValue := NewQScrollPrepareEventFromPointer(C.QScrollPrepareEvent_NewQScrollPrepareEvent(core.PointerFromQPointF(startPos)))
	qt.SetFinalizer(tmpValue, (*QScrollPrepareEvent).DestroyQScrollPrepareEvent)
	return tmpValue
}

func (ptr *QScrollPrepareEvent) ContentPos() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QScrollPrepareEvent_ContentPos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QScrollPrepareEvent) ContentPosRange() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QScrollPrepareEvent_ContentPosRange(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QScrollPrepareEvent) SetContentPos(pos core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QScrollPrepareEvent_SetContentPos(ptr.Pointer(), core.PointerFromQPointF(pos))
	}
}

func (ptr *QScrollPrepareEvent) SetContentPosRange(rect core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QScrollPrepareEvent_SetContentPosRange(ptr.Pointer(), core.PointerFromQRectF(rect))
	}
}

func (ptr *QScrollPrepareEvent) SetViewportSize(size core.QSizeF_ITF) {
	if ptr.Pointer() != nil {
		C.QScrollPrepareEvent_SetViewportSize(ptr.Pointer(), core.PointerFromQSizeF(size))
	}
}

func (ptr *QScrollPrepareEvent) StartPos() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QScrollPrepareEvent_StartPos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QScrollPrepareEvent) ViewportSize() *core.QSizeF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFFromPointer(C.QScrollPrepareEvent_ViewportSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

//export callbackQScrollPrepareEvent_DestroyQScrollPrepareEvent
func callbackQScrollPrepareEvent_DestroyQScrollPrepareEvent(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QScrollPrepareEvent"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQScrollPrepareEventFromPointer(ptr).DestroyQScrollPrepareEventDefault()
	}
}

func (ptr *QScrollPrepareEvent) ConnectDestroyQScrollPrepareEvent(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QScrollPrepareEvent"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QScrollPrepareEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QScrollPrepareEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QScrollPrepareEvent) DisconnectDestroyQScrollPrepareEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QScrollPrepareEvent")
	}
}

func (ptr *QScrollPrepareEvent) DestroyQScrollPrepareEvent() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QScrollPrepareEvent_DestroyQScrollPrepareEvent(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QScrollPrepareEvent) DestroyQScrollPrepareEventDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QScrollPrepareEvent_DestroyQScrollPrepareEventDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QSessionManager struct {
	core.QObject
}

type QSessionManager_ITF interface {
	core.QObject_ITF
	QSessionManager_PTR() *QSessionManager
}

func (ptr *QSessionManager) QSessionManager_PTR() *QSessionManager {
	return ptr
}

func (ptr *QSessionManager) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QSessionManager) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQSessionManager(ptr QSessionManager_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QSessionManager_PTR().Pointer()
	}
	return nil
}

func NewQSessionManagerFromPointer(ptr unsafe.Pointer) (n *QSessionManager) {
	n = new(QSessionManager)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QSessionManager__RestartHint
//QSessionManager::RestartHint
type QSessionManager__RestartHint int64

const (
	QSessionManager__RestartIfRunning   QSessionManager__RestartHint = QSessionManager__RestartHint(0)
	QSessionManager__RestartAnyway      QSessionManager__RestartHint = QSessionManager__RestartHint(1)
	QSessionManager__RestartImmediately QSessionManager__RestartHint = QSessionManager__RestartHint(2)
	QSessionManager__RestartNever       QSessionManager__RestartHint = QSessionManager__RestartHint(3)
)

func (ptr *QSessionManager) AllowsErrorInteraction() bool {
	if ptr.Pointer() != nil {
		return int8(C.QSessionManager_AllowsErrorInteraction(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QSessionManager) AllowsInteraction() bool {
	if ptr.Pointer() != nil {
		return int8(C.QSessionManager_AllowsInteraction(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QSessionManager) Cancel() {
	if ptr.Pointer() != nil {
		C.QSessionManager_Cancel(ptr.Pointer())
	}
}

func (ptr *QSessionManager) DiscardCommand() []string {
	if ptr.Pointer() != nil {
		return unpackStringList(cGoUnpackString(C.QSessionManager_DiscardCommand(ptr.Pointer())))
	}
	return make([]string, 0)
}

func (ptr *QSessionManager) IsPhase2() bool {
	if ptr.Pointer() != nil {
		return int8(C.QSessionManager_IsPhase2(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QSessionManager) Release() {
	if ptr.Pointer() != nil {
		C.QSessionManager_Release(ptr.Pointer())
	}
}

func (ptr *QSessionManager) RequestPhase2() {
	if ptr.Pointer() != nil {
		C.QSessionManager_RequestPhase2(ptr.Pointer())
	}
}

func (ptr *QSessionManager) RestartCommand() []string {
	if ptr.Pointer() != nil {
		return unpackStringList(cGoUnpackString(C.QSessionManager_RestartCommand(ptr.Pointer())))
	}
	return make([]string, 0)
}

func (ptr *QSessionManager) RestartHint() QSessionManager__RestartHint {
	if ptr.Pointer() != nil {
		return QSessionManager__RestartHint(C.QSessionManager_RestartHint(ptr.Pointer()))
	}
	return 0
}

func (ptr *QSessionManager) SessionId() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QSessionManager_SessionId(ptr.Pointer()))
	}
	return ""
}

func (ptr *QSessionManager) SessionKey() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QSessionManager_SessionKey(ptr.Pointer()))
	}
	return ""
}

func (ptr *QSessionManager) SetDiscardCommand(command []string) {
	if ptr.Pointer() != nil {
		commandC := C.CString(strings.Join(command, "¡¦!"))
		defer C.free(unsafe.Pointer(commandC))
		C.QSessionManager_SetDiscardCommand(ptr.Pointer(), C.struct_QtGui_PackedString{data: commandC, len: C.longlong(len(strings.Join(command, "¡¦!")))})
	}
}

func (ptr *QSessionManager) SetManagerProperty(name string, value []string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		valueC := C.CString(strings.Join(value, "¡¦!"))
		defer C.free(unsafe.Pointer(valueC))
		C.QSessionManager_SetManagerProperty(ptr.Pointer(), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}, C.struct_QtGui_PackedString{data: valueC, len: C.longlong(len(strings.Join(value, "¡¦!")))})
	}
}

func (ptr *QSessionManager) SetManagerProperty2(name string, value string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		var valueC *C.char
		if value != "" {
			valueC = C.CString(value)
			defer C.free(unsafe.Pointer(valueC))
		}
		C.QSessionManager_SetManagerProperty2(ptr.Pointer(), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))}, C.struct_QtGui_PackedString{data: valueC, len: C.longlong(len(value))})
	}
}

func (ptr *QSessionManager) SetRestartCommand(command []string) {
	if ptr.Pointer() != nil {
		commandC := C.CString(strings.Join(command, "¡¦!"))
		defer C.free(unsafe.Pointer(commandC))
		C.QSessionManager_SetRestartCommand(ptr.Pointer(), C.struct_QtGui_PackedString{data: commandC, len: C.longlong(len(strings.Join(command, "¡¦!")))})
	}
}

func (ptr *QSessionManager) SetRestartHint(hint QSessionManager__RestartHint) {
	if ptr.Pointer() != nil {
		C.QSessionManager_SetRestartHint(ptr.Pointer(), C.longlong(hint))
	}
}

func (ptr *QSessionManager) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QSessionManager___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QSessionManager) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QSessionManager___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QSessionManager) __children_newList() unsafe.Pointer {
	return C.QSessionManager___children_newList(ptr.Pointer())
}

func (ptr *QSessionManager) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QSessionManager___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QSessionManager) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QSessionManager___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QSessionManager) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QSessionManager___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QSessionManager) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QSessionManager___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QSessionManager) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QSessionManager___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QSessionManager) __findChildren_newList() unsafe.Pointer {
	return C.QSessionManager___findChildren_newList(ptr.Pointer())
}

func (ptr *QSessionManager) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QSessionManager___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QSessionManager) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QSessionManager___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QSessionManager) __findChildren_newList3() unsafe.Pointer {
	return C.QSessionManager___findChildren_newList3(ptr.Pointer())
}

//export callbackQSessionManager_ChildEvent
func callbackQSessionManager_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQSessionManagerFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QSessionManager) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QSessionManager_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQSessionManager_ConnectNotify
func callbackQSessionManager_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQSessionManagerFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QSessionManager) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QSessionManager_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQSessionManager_CustomEvent
func callbackQSessionManager_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQSessionManagerFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QSessionManager) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QSessionManager_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQSessionManager_DeleteLater
func callbackQSessionManager_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQSessionManagerFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QSessionManager) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QSessionManager_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQSessionManager_Destroyed
func callbackQSessionManager_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQSessionManager_DisconnectNotify
func callbackQSessionManager_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQSessionManagerFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QSessionManager) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QSessionManager_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQSessionManager_Event
func callbackQSessionManager_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQSessionManagerFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QSessionManager) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QSessionManager_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQSessionManager_EventFilter
func callbackQSessionManager_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQSessionManagerFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QSessionManager) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QSessionManager_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQSessionManager_MetaObject
func callbackQSessionManager_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQSessionManagerFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QSessionManager) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QSessionManager_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQSessionManager_ObjectNameChanged
func callbackQSessionManager_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQSessionManager_TimerEvent
func callbackQSessionManager_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQSessionManagerFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QSessionManager) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QSessionManager_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QShortcutEvent struct {
	core.QEvent
}

type QShortcutEvent_ITF interface {
	core.QEvent_ITF
	QShortcutEvent_PTR() *QShortcutEvent
}

func (ptr *QShortcutEvent) QShortcutEvent_PTR() *QShortcutEvent {
	return ptr
}

func (ptr *QShortcutEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QShortcutEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQShortcutEvent(ptr QShortcutEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QShortcutEvent_PTR().Pointer()
	}
	return nil
}

func NewQShortcutEventFromPointer(ptr unsafe.Pointer) (n *QShortcutEvent) {
	n = new(QShortcutEvent)
	n.SetPointer(ptr)
	return
}
func NewQShortcutEvent(key QKeySequence_ITF, id int, ambiguous bool) *QShortcutEvent {
	tmpValue := NewQShortcutEventFromPointer(C.QShortcutEvent_NewQShortcutEvent(PointerFromQKeySequence(key), C.int(int32(id)), C.char(int8(qt.GoBoolToInt(ambiguous)))))
	qt.SetFinalizer(tmpValue, (*QShortcutEvent).DestroyQShortcutEvent)
	return tmpValue
}

func (ptr *QShortcutEvent) IsAmbiguous() bool {
	if ptr.Pointer() != nil {
		return int8(C.QShortcutEvent_IsAmbiguous(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QShortcutEvent) Key() *QKeySequence {
	if ptr.Pointer() != nil {
		return NewQKeySequenceFromPointer(C.QShortcutEvent_Key(ptr.Pointer()))
	}
	return nil
}

func (ptr *QShortcutEvent) ShortcutId() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QShortcutEvent_ShortcutId(ptr.Pointer())))
	}
	return 0
}

//export callbackQShortcutEvent_DestroyQShortcutEvent
func callbackQShortcutEvent_DestroyQShortcutEvent(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QShortcutEvent"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQShortcutEventFromPointer(ptr).DestroyQShortcutEventDefault()
	}
}

func (ptr *QShortcutEvent) ConnectDestroyQShortcutEvent(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QShortcutEvent"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QShortcutEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QShortcutEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QShortcutEvent) DisconnectDestroyQShortcutEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QShortcutEvent")
	}
}

func (ptr *QShortcutEvent) DestroyQShortcutEvent() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QShortcutEvent_DestroyQShortcutEvent(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QShortcutEvent) DestroyQShortcutEventDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QShortcutEvent_DestroyQShortcutEventDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QShowEvent struct {
	core.QEvent
}

type QShowEvent_ITF interface {
	core.QEvent_ITF
	QShowEvent_PTR() *QShowEvent
}

func (ptr *QShowEvent) QShowEvent_PTR() *QShowEvent {
	return ptr
}

func (ptr *QShowEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QShowEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQShowEvent(ptr QShowEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QShowEvent_PTR().Pointer()
	}
	return nil
}

func NewQShowEventFromPointer(ptr unsafe.Pointer) (n *QShowEvent) {
	n = new(QShowEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QShowEvent) DestroyQShowEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQShowEvent() *QShowEvent {
	tmpValue := NewQShowEventFromPointer(C.QShowEvent_NewQShowEvent())
	qt.SetFinalizer(tmpValue, (*QShowEvent).DestroyQShowEvent)
	return tmpValue
}

type QStandardItem struct {
	ptr unsafe.Pointer
}

type QStandardItem_ITF interface {
	QStandardItem_PTR() *QStandardItem
}

func (ptr *QStandardItem) QStandardItem_PTR() *QStandardItem {
	return ptr
}

func (ptr *QStandardItem) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QStandardItem) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQStandardItem(ptr QStandardItem_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QStandardItem_PTR().Pointer()
	}
	return nil
}

func NewQStandardItemFromPointer(ptr unsafe.Pointer) (n *QStandardItem) {
	n = new(QStandardItem)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QStandardItem__ItemType
//QStandardItem::ItemType
type QStandardItem__ItemType int64

const (
	QStandardItem__Type     QStandardItem__ItemType = QStandardItem__ItemType(0)
	QStandardItem__UserType QStandardItem__ItemType = QStandardItem__ItemType(1000)
)

func NewQStandardItem() *QStandardItem {
	return NewQStandardItemFromPointer(C.QStandardItem_NewQStandardItem())
}

func NewQStandardItem2(text string) *QStandardItem {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	return NewQStandardItemFromPointer(C.QStandardItem_NewQStandardItem2(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
}

func NewQStandardItem3(icon QIcon_ITF, text string) *QStandardItem {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	return NewQStandardItemFromPointer(C.QStandardItem_NewQStandardItem3(PointerFromQIcon(icon), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
}

func NewQStandardItem4(rows int, columns int) *QStandardItem {
	return NewQStandardItemFromPointer(C.QStandardItem_NewQStandardItem4(C.int(int32(rows)), C.int(int32(columns))))
}

func NewQStandardItem5(other QStandardItem_ITF) *QStandardItem {
	return NewQStandardItemFromPointer(C.QStandardItem_NewQStandardItem5(PointerFromQStandardItem(other)))
}

func (ptr *QStandardItem) AccessibleDescription() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QStandardItem_AccessibleDescription(ptr.Pointer()))
	}
	return ""
}

func (ptr *QStandardItem) AccessibleText() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QStandardItem_AccessibleText(ptr.Pointer()))
	}
	return ""
}

func (ptr *QStandardItem) AppendColumn(items []*QStandardItem) {
	if ptr.Pointer() != nil {
		C.QStandardItem_AppendColumn(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQStandardItemFromPointer(NewQStandardItemFromPointer(nil).__appendColumn_items_newList())
			for _, v := range items {
				tmpList.__appendColumn_items_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QStandardItem) AppendRow(items []*QStandardItem) {
	if ptr.Pointer() != nil {
		C.QStandardItem_AppendRow(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQStandardItemFromPointer(NewQStandardItemFromPointer(nil).__appendRow_items_newList())
			for _, v := range items {
				tmpList.__appendRow_items_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QStandardItem) AppendRow2(item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_AppendRow2(ptr.Pointer(), PointerFromQStandardItem(item))
	}
}

func (ptr *QStandardItem) AppendRows(items []*QStandardItem) {
	if ptr.Pointer() != nil {
		C.QStandardItem_AppendRows(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQStandardItemFromPointer(NewQStandardItemFromPointer(nil).__appendRows_items_newList())
			for _, v := range items {
				tmpList.__appendRows_items_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QStandardItem) Background() *QBrush {
	if ptr.Pointer() != nil {
		tmpValue := NewQBrushFromPointer(C.QStandardItem_Background(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItem) CheckState() core.Qt__CheckState {
	if ptr.Pointer() != nil {
		return core.Qt__CheckState(C.QStandardItem_CheckState(ptr.Pointer()))
	}
	return 0
}

func (ptr *QStandardItem) Child(row int, column int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem_Child(ptr.Pointer(), C.int(int32(row)), C.int(int32(column))))
	}
	return nil
}

func (ptr *QStandardItem) ClearData() {
	if ptr.Pointer() != nil {
		C.QStandardItem_ClearData(ptr.Pointer())
	}
}

//export callbackQStandardItem_Clone
func callbackQStandardItem_Clone(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "clone"); signal != nil {
		return PointerFromQStandardItem((*(*func() *QStandardItem)(signal))())
	}

	return PointerFromQStandardItem(NewQStandardItemFromPointer(ptr).CloneDefault())
}

func (ptr *QStandardItem) ConnectClone(f func() *QStandardItem) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "clone"); signal != nil {
			f := func() *QStandardItem {
				(*(*func() *QStandardItem)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "clone", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "clone", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItem) DisconnectClone() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "clone")
	}
}

func (ptr *QStandardItem) Clone() *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem_Clone(ptr.Pointer()))
	}
	return nil
}

func (ptr *QStandardItem) CloneDefault() *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem_CloneDefault(ptr.Pointer()))
	}
	return nil
}

func (ptr *QStandardItem) Column() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItem_Column(ptr.Pointer())))
	}
	return 0
}

func (ptr *QStandardItem) ColumnCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItem_ColumnCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQStandardItem_Data
func callbackQStandardItem_Data(ptr unsafe.Pointer, role C.int) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "data"); signal != nil {
		return core.PointerFromQVariant((*(*func(int) *core.QVariant)(signal))(int(int32(role))))
	}

	return core.PointerFromQVariant(NewQStandardItemFromPointer(ptr).DataDefault(int(int32(role))))
}

func (ptr *QStandardItem) ConnectData(f func(role int) *core.QVariant) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "data"); signal != nil {
			f := func(role int) *core.QVariant {
				(*(*func(int) *core.QVariant)(signal))(role)
				return f(role)
			}
			qt.ConnectSignal(ptr.Pointer(), "data", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "data", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItem) DisconnectData() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "data")
	}
}

func (ptr *QStandardItem) Data(role int) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QStandardItem_Data(ptr.Pointer(), C.int(int32(role))))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItem) DataDefault(role int) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QStandardItem_DataDefault(ptr.Pointer(), C.int(int32(role))))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItem) EmitDataChanged() {
	if ptr.Pointer() != nil {
		C.QStandardItem_EmitDataChanged(ptr.Pointer())
	}
}

func (ptr *QStandardItem) Flags() core.Qt__ItemFlag {
	if ptr.Pointer() != nil {
		return core.Qt__ItemFlag(C.QStandardItem_Flags(ptr.Pointer()))
	}
	return 0
}

func (ptr *QStandardItem) Font() *QFont {
	if ptr.Pointer() != nil {
		tmpValue := NewQFontFromPointer(C.QStandardItem_Font(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItem) Foreground() *QBrush {
	if ptr.Pointer() != nil {
		tmpValue := NewQBrushFromPointer(C.QStandardItem_Foreground(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItem) HasChildren() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItem_HasChildren(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStandardItem) Icon() *QIcon {
	if ptr.Pointer() != nil {
		tmpValue := NewQIconFromPointer(C.QStandardItem_Icon(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItem) Index() *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItem_Index(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItem) InsertColumn(column int, items []*QStandardItem) {
	if ptr.Pointer() != nil {
		C.QStandardItem_InsertColumn(ptr.Pointer(), C.int(int32(column)), func() unsafe.Pointer {
			tmpList := NewQStandardItemFromPointer(NewQStandardItemFromPointer(nil).__insertColumn_items_newList())
			for _, v := range items {
				tmpList.__insertColumn_items_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QStandardItem) InsertColumns(column int, count int) {
	if ptr.Pointer() != nil {
		C.QStandardItem_InsertColumns(ptr.Pointer(), C.int(int32(column)), C.int(int32(count)))
	}
}

func (ptr *QStandardItem) InsertRow(row int, items []*QStandardItem) {
	if ptr.Pointer() != nil {
		C.QStandardItem_InsertRow(ptr.Pointer(), C.int(int32(row)), func() unsafe.Pointer {
			tmpList := NewQStandardItemFromPointer(NewQStandardItemFromPointer(nil).__insertRow_items_newList())
			for _, v := range items {
				tmpList.__insertRow_items_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QStandardItem) InsertRow2(row int, item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_InsertRow2(ptr.Pointer(), C.int(int32(row)), PointerFromQStandardItem(item))
	}
}

func (ptr *QStandardItem) InsertRows(row int, items []*QStandardItem) {
	if ptr.Pointer() != nil {
		C.QStandardItem_InsertRows(ptr.Pointer(), C.int(int32(row)), func() unsafe.Pointer {
			tmpList := NewQStandardItemFromPointer(NewQStandardItemFromPointer(nil).__insertRows_items_newList())
			for _, v := range items {
				tmpList.__insertRows_items_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QStandardItem) InsertRows2(row int, count int) {
	if ptr.Pointer() != nil {
		C.QStandardItem_InsertRows2(ptr.Pointer(), C.int(int32(row)), C.int(int32(count)))
	}
}

func (ptr *QStandardItem) IsAutoTristate() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItem_IsAutoTristate(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStandardItem) IsCheckable() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItem_IsCheckable(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStandardItem) IsDragEnabled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItem_IsDragEnabled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStandardItem) IsDropEnabled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItem_IsDropEnabled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStandardItem) IsEditable() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItem_IsEditable(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStandardItem) IsEnabled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItem_IsEnabled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStandardItem) IsSelectable() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItem_IsSelectable(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStandardItem) IsUserTristate() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItem_IsUserTristate(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStandardItem) Model() *QStandardItemModel {
	if ptr.Pointer() != nil {
		tmpValue := NewQStandardItemModelFromPointer(C.QStandardItem_Model(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItem) Parent() *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem_Parent(ptr.Pointer()))
	}
	return nil
}

//export callbackQStandardItem_Read
func callbackQStandardItem_Read(ptr unsafe.Pointer, in unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "read"); signal != nil {
		(*(*func(*core.QDataStream))(signal))(core.NewQDataStreamFromPointer(in))
	} else {
		NewQStandardItemFromPointer(ptr).ReadDefault(core.NewQDataStreamFromPointer(in))
	}
}

func (ptr *QStandardItem) ConnectRead(f func(in *core.QDataStream)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "read"); signal != nil {
			f := func(in *core.QDataStream) {
				(*(*func(*core.QDataStream))(signal))(in)
				f(in)
			}
			qt.ConnectSignal(ptr.Pointer(), "read", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "read", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItem) DisconnectRead() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "read")
	}
}

func (ptr *QStandardItem) Read(in core.QDataStream_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_Read(ptr.Pointer(), core.PointerFromQDataStream(in))
	}
}

func (ptr *QStandardItem) ReadDefault(in core.QDataStream_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_ReadDefault(ptr.Pointer(), core.PointerFromQDataStream(in))
	}
}

func (ptr *QStandardItem) RemoveColumn(column int) {
	if ptr.Pointer() != nil {
		C.QStandardItem_RemoveColumn(ptr.Pointer(), C.int(int32(column)))
	}
}

func (ptr *QStandardItem) RemoveColumns(column int, count int) {
	if ptr.Pointer() != nil {
		C.QStandardItem_RemoveColumns(ptr.Pointer(), C.int(int32(column)), C.int(int32(count)))
	}
}

func (ptr *QStandardItem) RemoveRow(row int) {
	if ptr.Pointer() != nil {
		C.QStandardItem_RemoveRow(ptr.Pointer(), C.int(int32(row)))
	}
}

func (ptr *QStandardItem) RemoveRows(row int, count int) {
	if ptr.Pointer() != nil {
		C.QStandardItem_RemoveRows(ptr.Pointer(), C.int(int32(row)), C.int(int32(count)))
	}
}

func (ptr *QStandardItem) Row() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItem_Row(ptr.Pointer())))
	}
	return 0
}

func (ptr *QStandardItem) RowCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItem_RowCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QStandardItem) SetAccessibleDescription(accessibleDescription string) {
	if ptr.Pointer() != nil {
		var accessibleDescriptionC *C.char
		if accessibleDescription != "" {
			accessibleDescriptionC = C.CString(accessibleDescription)
			defer C.free(unsafe.Pointer(accessibleDescriptionC))
		}
		C.QStandardItem_SetAccessibleDescription(ptr.Pointer(), C.struct_QtGui_PackedString{data: accessibleDescriptionC, len: C.longlong(len(accessibleDescription))})
	}
}

func (ptr *QStandardItem) SetAccessibleText(accessibleText string) {
	if ptr.Pointer() != nil {
		var accessibleTextC *C.char
		if accessibleText != "" {
			accessibleTextC = C.CString(accessibleText)
			defer C.free(unsafe.Pointer(accessibleTextC))
		}
		C.QStandardItem_SetAccessibleText(ptr.Pointer(), C.struct_QtGui_PackedString{data: accessibleTextC, len: C.longlong(len(accessibleText))})
	}
}

func (ptr *QStandardItem) SetAutoTristate(tristate bool) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetAutoTristate(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(tristate))))
	}
}

func (ptr *QStandardItem) SetBackground(brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetBackground(ptr.Pointer(), PointerFromQBrush(brush))
	}
}

func (ptr *QStandardItem) SetCheckState(state core.Qt__CheckState) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetCheckState(ptr.Pointer(), C.longlong(state))
	}
}

func (ptr *QStandardItem) SetCheckable(checkable bool) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetCheckable(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(checkable))))
	}
}

func (ptr *QStandardItem) SetChild(row int, column int, item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetChild(ptr.Pointer(), C.int(int32(row)), C.int(int32(column)), PointerFromQStandardItem(item))
	}
}

func (ptr *QStandardItem) SetChild2(row int, item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetChild2(ptr.Pointer(), C.int(int32(row)), PointerFromQStandardItem(item))
	}
}

func (ptr *QStandardItem) SetColumnCount(columns int) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetColumnCount(ptr.Pointer(), C.int(int32(columns)))
	}
}

//export callbackQStandardItem_SetData
func callbackQStandardItem_SetData(ptr unsafe.Pointer, value unsafe.Pointer, role C.int) {
	if signal := qt.GetSignal(ptr, "setData"); signal != nil {
		(*(*func(*core.QVariant, int))(signal))(core.NewQVariantFromPointer(value), int(int32(role)))
	} else {
		NewQStandardItemFromPointer(ptr).SetDataDefault(core.NewQVariantFromPointer(value), int(int32(role)))
	}
}

func (ptr *QStandardItem) ConnectSetData(f func(value *core.QVariant, role int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setData"); signal != nil {
			f := func(value *core.QVariant, role int) {
				(*(*func(*core.QVariant, int))(signal))(value, role)
				f(value, role)
			}
			qt.ConnectSignal(ptr.Pointer(), "setData", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setData", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItem) DisconnectSetData() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setData")
	}
}

func (ptr *QStandardItem) SetData(value core.QVariant_ITF, role int) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetData(ptr.Pointer(), core.PointerFromQVariant(value), C.int(int32(role)))
	}
}

func (ptr *QStandardItem) SetDataDefault(value core.QVariant_ITF, role int) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetDataDefault(ptr.Pointer(), core.PointerFromQVariant(value), C.int(int32(role)))
	}
}

func (ptr *QStandardItem) SetDragEnabled(dragEnabled bool) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetDragEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(dragEnabled))))
	}
}

func (ptr *QStandardItem) SetDropEnabled(dropEnabled bool) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetDropEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(dropEnabled))))
	}
}

func (ptr *QStandardItem) SetEditable(editable bool) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetEditable(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(editable))))
	}
}

func (ptr *QStandardItem) SetEnabled(enabled bool) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enabled))))
	}
}

func (ptr *QStandardItem) SetFlags(flags core.Qt__ItemFlag) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetFlags(ptr.Pointer(), C.longlong(flags))
	}
}

func (ptr *QStandardItem) SetFont(font QFont_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetFont(ptr.Pointer(), PointerFromQFont(font))
	}
}

func (ptr *QStandardItem) SetForeground(brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetForeground(ptr.Pointer(), PointerFromQBrush(brush))
	}
}

func (ptr *QStandardItem) SetIcon(icon QIcon_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetIcon(ptr.Pointer(), PointerFromQIcon(icon))
	}
}

func (ptr *QStandardItem) SetRowCount(rows int) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetRowCount(ptr.Pointer(), C.int(int32(rows)))
	}
}

func (ptr *QStandardItem) SetSelectable(selectable bool) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetSelectable(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(selectable))))
	}
}

func (ptr *QStandardItem) SetSizeHint(size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetSizeHint(ptr.Pointer(), core.PointerFromQSize(size))
	}
}

func (ptr *QStandardItem) SetStatusTip(statusTip string) {
	if ptr.Pointer() != nil {
		var statusTipC *C.char
		if statusTip != "" {
			statusTipC = C.CString(statusTip)
			defer C.free(unsafe.Pointer(statusTipC))
		}
		C.QStandardItem_SetStatusTip(ptr.Pointer(), C.struct_QtGui_PackedString{data: statusTipC, len: C.longlong(len(statusTip))})
	}
}

func (ptr *QStandardItem) SetText(text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QStandardItem_SetText(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QStandardItem) SetTextAlignment(alignment core.Qt__AlignmentFlag) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetTextAlignment(ptr.Pointer(), C.longlong(alignment))
	}
}

func (ptr *QStandardItem) SetToolTip(toolTip string) {
	if ptr.Pointer() != nil {
		var toolTipC *C.char
		if toolTip != "" {
			toolTipC = C.CString(toolTip)
			defer C.free(unsafe.Pointer(toolTipC))
		}
		C.QStandardItem_SetToolTip(ptr.Pointer(), C.struct_QtGui_PackedString{data: toolTipC, len: C.longlong(len(toolTip))})
	}
}

func (ptr *QStandardItem) SetUserTristate(tristate bool) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SetUserTristate(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(tristate))))
	}
}

func (ptr *QStandardItem) SetWhatsThis(whatsThis string) {
	if ptr.Pointer() != nil {
		var whatsThisC *C.char
		if whatsThis != "" {
			whatsThisC = C.CString(whatsThis)
			defer C.free(unsafe.Pointer(whatsThisC))
		}
		C.QStandardItem_SetWhatsThis(ptr.Pointer(), C.struct_QtGui_PackedString{data: whatsThisC, len: C.longlong(len(whatsThis))})
	}
}

func (ptr *QStandardItem) SizeHint() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QStandardItem_SizeHint(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItem) SortChildren(column int, order core.Qt__SortOrder) {
	if ptr.Pointer() != nil {
		C.QStandardItem_SortChildren(ptr.Pointer(), C.int(int32(column)), C.longlong(order))
	}
}

func (ptr *QStandardItem) StatusTip() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QStandardItem_StatusTip(ptr.Pointer()))
	}
	return ""
}

func (ptr *QStandardItem) TakeChild(row int, column int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem_TakeChild(ptr.Pointer(), C.int(int32(row)), C.int(int32(column))))
	}
	return nil
}

func (ptr *QStandardItem) TakeColumn(column int) []*QStandardItem {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QStandardItem {
			out := make([]*QStandardItem, int(l.len))
			tmpList := NewQStandardItemFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__takeColumn_atList(i)
			}
			return out
		}(C.QStandardItem_TakeColumn(ptr.Pointer(), C.int(int32(column))))
	}
	return make([]*QStandardItem, 0)
}

func (ptr *QStandardItem) TakeRow(row int) []*QStandardItem {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QStandardItem {
			out := make([]*QStandardItem, int(l.len))
			tmpList := NewQStandardItemFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__takeRow_atList(i)
			}
			return out
		}(C.QStandardItem_TakeRow(ptr.Pointer(), C.int(int32(row))))
	}
	return make([]*QStandardItem, 0)
}

func (ptr *QStandardItem) Text() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QStandardItem_Text(ptr.Pointer()))
	}
	return ""
}

func (ptr *QStandardItem) TextAlignment() core.Qt__AlignmentFlag {
	if ptr.Pointer() != nil {
		return core.Qt__AlignmentFlag(C.QStandardItem_TextAlignment(ptr.Pointer()))
	}
	return 0
}

func (ptr *QStandardItem) ToolTip() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QStandardItem_ToolTip(ptr.Pointer()))
	}
	return ""
}

//export callbackQStandardItem_Type
func callbackQStandardItem_Type(ptr unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "type"); signal != nil {
		return C.int(int32((*(*func() int)(signal))()))
	}

	return C.int(int32(NewQStandardItemFromPointer(ptr).TypeDefault()))
}

func (ptr *QStandardItem) ConnectType(f func() int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "type"); signal != nil {
			f := func() int {
				(*(*func() int)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "type", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "type", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItem) DisconnectType() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "type")
	}
}

func (ptr *QStandardItem) Type() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItem_Type(ptr.Pointer())))
	}
	return 0
}

func (ptr *QStandardItem) TypeDefault() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItem_TypeDefault(ptr.Pointer())))
	}
	return 0
}

func (ptr *QStandardItem) WhatsThis() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QStandardItem_WhatsThis(ptr.Pointer()))
	}
	return ""
}

//export callbackQStandardItem_Write
func callbackQStandardItem_Write(ptr unsafe.Pointer, out unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "write"); signal != nil {
		(*(*func(*core.QDataStream))(signal))(core.NewQDataStreamFromPointer(out))
	} else {
		NewQStandardItemFromPointer(ptr).WriteDefault(core.NewQDataStreamFromPointer(out))
	}
}

func (ptr *QStandardItem) ConnectWrite(f func(out *core.QDataStream)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "write"); signal != nil {
			f := func(out *core.QDataStream) {
				(*(*func(*core.QDataStream))(signal))(out)
				f(out)
			}
			qt.ConnectSignal(ptr.Pointer(), "write", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "write", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItem) DisconnectWrite() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "write")
	}
}

func (ptr *QStandardItem) Write(out core.QDataStream_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_Write(ptr.Pointer(), core.PointerFromQDataStream(out))
	}
}

func (ptr *QStandardItem) WriteDefault(out core.QDataStream_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem_WriteDefault(ptr.Pointer(), core.PointerFromQDataStream(out))
	}
}

//export callbackQStandardItem_DestroyQStandardItem
func callbackQStandardItem_DestroyQStandardItem(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QStandardItem"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQStandardItemFromPointer(ptr).DestroyQStandardItemDefault()
	}
}

func (ptr *QStandardItem) ConnectDestroyQStandardItem(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QStandardItem"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QStandardItem", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QStandardItem", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItem) DisconnectDestroyQStandardItem() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QStandardItem")
	}
}

func (ptr *QStandardItem) DestroyQStandardItem() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QStandardItem_DestroyQStandardItem(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QStandardItem) DestroyQStandardItemDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QStandardItem_DestroyQStandardItemDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QStandardItem) __appendColumn_items_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem___appendColumn_items_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItem) __appendColumn_items_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem___appendColumn_items_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItem) __appendColumn_items_newList() unsafe.Pointer {
	return C.QStandardItem___appendColumn_items_newList(ptr.Pointer())
}

func (ptr *QStandardItem) __appendRow_items_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem___appendRow_items_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItem) __appendRow_items_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem___appendRow_items_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItem) __appendRow_items_newList() unsafe.Pointer {
	return C.QStandardItem___appendRow_items_newList(ptr.Pointer())
}

func (ptr *QStandardItem) __appendRows_items_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem___appendRows_items_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItem) __appendRows_items_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem___appendRows_items_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItem) __appendRows_items_newList() unsafe.Pointer {
	return C.QStandardItem___appendRows_items_newList(ptr.Pointer())
}

func (ptr *QStandardItem) __insertColumn_items_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem___insertColumn_items_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItem) __insertColumn_items_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem___insertColumn_items_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItem) __insertColumn_items_newList() unsafe.Pointer {
	return C.QStandardItem___insertColumn_items_newList(ptr.Pointer())
}

func (ptr *QStandardItem) __insertRow_items_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem___insertRow_items_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItem) __insertRow_items_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem___insertRow_items_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItem) __insertRow_items_newList() unsafe.Pointer {
	return C.QStandardItem___insertRow_items_newList(ptr.Pointer())
}

func (ptr *QStandardItem) __insertRows_items_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem___insertRows_items_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItem) __insertRows_items_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem___insertRows_items_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItem) __insertRows_items_newList() unsafe.Pointer {
	return C.QStandardItem___insertRows_items_newList(ptr.Pointer())
}

func (ptr *QStandardItem) __takeColumn_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem___takeColumn_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItem) __takeColumn_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem___takeColumn_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItem) __takeColumn_newList() unsafe.Pointer {
	return C.QStandardItem___takeColumn_newList(ptr.Pointer())
}

func (ptr *QStandardItem) __takeRow_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItem___takeRow_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItem) __takeRow_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItem___takeRow_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItem) __takeRow_newList() unsafe.Pointer {
	return C.QStandardItem___takeRow_newList(ptr.Pointer())
}

type QStandardItemModel struct {
	core.QAbstractItemModel
}

type QStandardItemModel_ITF interface {
	core.QAbstractItemModel_ITF
	QStandardItemModel_PTR() *QStandardItemModel
}

func (ptr *QStandardItemModel) QStandardItemModel_PTR() *QStandardItemModel {
	return ptr
}

func (ptr *QStandardItemModel) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QAbstractItemModel_PTR().Pointer()
	}
	return nil
}

func (ptr *QStandardItemModel) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QAbstractItemModel_PTR().SetPointer(p)
	}
}

func PointerFromQStandardItemModel(ptr QStandardItemModel_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QStandardItemModel_PTR().Pointer()
	}
	return nil
}

func NewQStandardItemModelFromPointer(ptr unsafe.Pointer) (n *QStandardItemModel) {
	n = new(QStandardItemModel)
	n.SetPointer(ptr)
	return
}
func NewQStandardItemModel(parent core.QObject_ITF) *QStandardItemModel {
	tmpValue := NewQStandardItemModelFromPointer(C.QStandardItemModel_NewQStandardItemModel(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQStandardItemModel2(rows int, columns int, parent core.QObject_ITF) *QStandardItemModel {
	tmpValue := NewQStandardItemModelFromPointer(C.QStandardItemModel_NewQStandardItemModel2(C.int(int32(rows)), C.int(int32(columns)), core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QStandardItemModel) AppendColumn(items []*QStandardItem) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_AppendColumn(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__appendColumn_items_newList())
			for _, v := range items {
				tmpList.__appendColumn_items_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QStandardItemModel) AppendRow(items []*QStandardItem) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_AppendRow(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__appendRow_items_newList())
			for _, v := range items {
				tmpList.__appendRow_items_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QStandardItemModel) AppendRow2(item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_AppendRow2(ptr.Pointer(), PointerFromQStandardItem(item))
	}
}

func (ptr *QStandardItemModel) Clear() {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_Clear(ptr.Pointer())
	}
}

func (ptr *QStandardItemModel) ClearItemData(index core.QModelIndex_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_ClearItemData(ptr.Pointer(), core.PointerFromQModelIndex(index))) != 0
	}
	return false
}

//export callbackQStandardItemModel_ColumnCount
func callbackQStandardItemModel_ColumnCount(ptr unsafe.Pointer, parent unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "columnCount"); signal != nil {
		return C.int(int32((*(*func(*core.QModelIndex) int)(signal))(core.NewQModelIndexFromPointer(parent))))
	}

	return C.int(int32(NewQStandardItemModelFromPointer(ptr).ColumnCountDefault(core.NewQModelIndexFromPointer(parent))))
}

func (ptr *QStandardItemModel) ConnectColumnCount(f func(parent *core.QModelIndex) int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "columnCount"); signal != nil {
			f := func(parent *core.QModelIndex) int {
				(*(*func(*core.QModelIndex) int)(signal))(parent)
				return f(parent)
			}
			qt.ConnectSignal(ptr.Pointer(), "columnCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "columnCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItemModel) DisconnectColumnCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "columnCount")
	}
}

func (ptr *QStandardItemModel) ColumnCount(parent core.QModelIndex_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_ColumnCount(ptr.Pointer(), core.PointerFromQModelIndex(parent))))
	}
	return 0
}

func (ptr *QStandardItemModel) ColumnCountDefault(parent core.QModelIndex_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_ColumnCountDefault(ptr.Pointer(), core.PointerFromQModelIndex(parent))))
	}
	return 0
}

//export callbackQStandardItemModel_Data
func callbackQStandardItemModel_Data(ptr unsafe.Pointer, index unsafe.Pointer, role C.int) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "data"); signal != nil {
		return core.PointerFromQVariant((*(*func(*core.QModelIndex, int) *core.QVariant)(signal))(core.NewQModelIndexFromPointer(index), int(int32(role))))
	}

	return core.PointerFromQVariant(NewQStandardItemModelFromPointer(ptr).DataDefault(core.NewQModelIndexFromPointer(index), int(int32(role))))
}

func (ptr *QStandardItemModel) ConnectData(f func(index *core.QModelIndex, role int) *core.QVariant) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "data"); signal != nil {
			f := func(index *core.QModelIndex, role int) *core.QVariant {
				(*(*func(*core.QModelIndex, int) *core.QVariant)(signal))(index, role)
				return f(index, role)
			}
			qt.ConnectSignal(ptr.Pointer(), "data", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "data", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItemModel) DisconnectData() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "data")
	}
}

func (ptr *QStandardItemModel) Data(index core.QModelIndex_ITF, role int) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QStandardItemModel_Data(ptr.Pointer(), core.PointerFromQModelIndex(index), C.int(int32(role))))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) DataDefault(index core.QModelIndex_ITF, role int) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QStandardItemModel_DataDefault(ptr.Pointer(), core.PointerFromQModelIndex(index), C.int(int32(role))))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

//export callbackQStandardItemModel_DropMimeData
func callbackQStandardItemModel_DropMimeData(ptr unsafe.Pointer, data unsafe.Pointer, action C.longlong, row C.int, column C.int, parent unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "dropMimeData"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QMimeData, core.Qt__DropAction, int, int, *core.QModelIndex) bool)(signal))(core.NewQMimeDataFromPointer(data), core.Qt__DropAction(action), int(int32(row)), int(int32(column)), core.NewQModelIndexFromPointer(parent)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).DropMimeDataDefault(core.NewQMimeDataFromPointer(data), core.Qt__DropAction(action), int(int32(row)), int(int32(column)), core.NewQModelIndexFromPointer(parent)))))
}

func (ptr *QStandardItemModel) DropMimeDataDefault(data core.QMimeData_ITF, action core.Qt__DropAction, row int, column int, parent core.QModelIndex_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_DropMimeDataDefault(ptr.Pointer(), core.PointerFromQMimeData(data), C.longlong(action), C.int(int32(row)), C.int(int32(column)), core.PointerFromQModelIndex(parent))) != 0
	}
	return false
}

func (ptr *QStandardItemModel) FindItems(text string, flags core.Qt__MatchFlag, column int) []*QStandardItem {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		return func(l C.struct_QtGui_PackedList) []*QStandardItem {
			out := make([]*QStandardItem, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__findItems_atList(i)
			}
			return out
		}(C.QStandardItemModel_FindItems(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, C.longlong(flags), C.int(int32(column))))
	}
	return make([]*QStandardItem, 0)
}

//export callbackQStandardItemModel_Flags
func callbackQStandardItemModel_Flags(ptr unsafe.Pointer, index unsafe.Pointer) C.longlong {
	if signal := qt.GetSignal(ptr, "flags"); signal != nil {
		return C.longlong((*(*func(*core.QModelIndex) core.Qt__ItemFlag)(signal))(core.NewQModelIndexFromPointer(index)))
	}

	return C.longlong(NewQStandardItemModelFromPointer(ptr).FlagsDefault(core.NewQModelIndexFromPointer(index)))
}

func (ptr *QStandardItemModel) FlagsDefault(index core.QModelIndex_ITF) core.Qt__ItemFlag {
	if ptr.Pointer() != nil {
		return core.Qt__ItemFlag(C.QStandardItemModel_FlagsDefault(ptr.Pointer(), core.PointerFromQModelIndex(index)))
	}
	return 0
}

//export callbackQStandardItemModel_HasChildren
func callbackQStandardItemModel_HasChildren(ptr unsafe.Pointer, parent unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "hasChildren"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QModelIndex) bool)(signal))(core.NewQModelIndexFromPointer(parent)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).HasChildrenDefault(core.NewQModelIndexFromPointer(parent)))))
}

func (ptr *QStandardItemModel) HasChildrenDefault(parent core.QModelIndex_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_HasChildrenDefault(ptr.Pointer(), core.PointerFromQModelIndex(parent))) != 0
	}
	return false
}

//export callbackQStandardItemModel_HeaderData
func callbackQStandardItemModel_HeaderData(ptr unsafe.Pointer, section C.int, orientation C.longlong, role C.int) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "headerData"); signal != nil {
		return core.PointerFromQVariant((*(*func(int, core.Qt__Orientation, int) *core.QVariant)(signal))(int(int32(section)), core.Qt__Orientation(orientation), int(int32(role))))
	}

	return core.PointerFromQVariant(NewQStandardItemModelFromPointer(ptr).HeaderDataDefault(int(int32(section)), core.Qt__Orientation(orientation), int(int32(role))))
}

func (ptr *QStandardItemModel) HeaderDataDefault(section int, orientation core.Qt__Orientation, role int) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QStandardItemModel_HeaderDataDefault(ptr.Pointer(), C.int(int32(section)), C.longlong(orientation), C.int(int32(role))))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) HorizontalHeaderItem(column int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel_HorizontalHeaderItem(ptr.Pointer(), C.int(int32(column))))
	}
	return nil
}

//export callbackQStandardItemModel_Index
func callbackQStandardItemModel_Index(ptr unsafe.Pointer, row C.int, column C.int, parent unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "index"); signal != nil {
		return core.PointerFromQModelIndex((*(*func(int, int, *core.QModelIndex) *core.QModelIndex)(signal))(int(int32(row)), int(int32(column)), core.NewQModelIndexFromPointer(parent)))
	}

	return core.PointerFromQModelIndex(NewQStandardItemModelFromPointer(ptr).IndexDefault(int(int32(row)), int(int32(column)), core.NewQModelIndexFromPointer(parent)))
}

func (ptr *QStandardItemModel) ConnectIndex(f func(row int, column int, parent *core.QModelIndex) *core.QModelIndex) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "index"); signal != nil {
			f := func(row int, column int, parent *core.QModelIndex) *core.QModelIndex {
				(*(*func(int, int, *core.QModelIndex) *core.QModelIndex)(signal))(row, column, parent)
				return f(row, column, parent)
			}
			qt.ConnectSignal(ptr.Pointer(), "index", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "index", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItemModel) DisconnectIndex() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "index")
	}
}

func (ptr *QStandardItemModel) Index(row int, column int, parent core.QModelIndex_ITF) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel_Index(ptr.Pointer(), C.int(int32(row)), C.int(int32(column)), core.PointerFromQModelIndex(parent)))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) IndexDefault(row int, column int, parent core.QModelIndex_ITF) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel_IndexDefault(ptr.Pointer(), C.int(int32(row)), C.int(int32(column)), core.PointerFromQModelIndex(parent)))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) IndexFromItem(item QStandardItem_ITF) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel_IndexFromItem(ptr.Pointer(), PointerFromQStandardItem(item)))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) InsertColumn(column int, items []*QStandardItem) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_InsertColumn(ptr.Pointer(), C.int(int32(column)), func() unsafe.Pointer {
			tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__insertColumn_items_newList())
			for _, v := range items {
				tmpList.__insertColumn_items_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

//export callbackQStandardItemModel_InsertColumns
func callbackQStandardItemModel_InsertColumns(ptr unsafe.Pointer, column C.int, count C.int, parent unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "insertColumns"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int, int, *core.QModelIndex) bool)(signal))(int(int32(column)), int(int32(count)), core.NewQModelIndexFromPointer(parent)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).InsertColumnsDefault(int(int32(column)), int(int32(count)), core.NewQModelIndexFromPointer(parent)))))
}

func (ptr *QStandardItemModel) InsertColumnsDefault(column int, count int, parent core.QModelIndex_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_InsertColumnsDefault(ptr.Pointer(), C.int(int32(column)), C.int(int32(count)), core.PointerFromQModelIndex(parent))) != 0
	}
	return false
}

func (ptr *QStandardItemModel) InsertRow(row int, items []*QStandardItem) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_InsertRow(ptr.Pointer(), C.int(int32(row)), func() unsafe.Pointer {
			tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__insertRow_items_newList())
			for _, v := range items {
				tmpList.__insertRow_items_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QStandardItemModel) InsertRow2(row int, item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_InsertRow2(ptr.Pointer(), C.int(int32(row)), PointerFromQStandardItem(item))
	}
}

//export callbackQStandardItemModel_InsertRows
func callbackQStandardItemModel_InsertRows(ptr unsafe.Pointer, row C.int, count C.int, parent unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "insertRows"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int, int, *core.QModelIndex) bool)(signal))(int(int32(row)), int(int32(count)), core.NewQModelIndexFromPointer(parent)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).InsertRowsDefault(int(int32(row)), int(int32(count)), core.NewQModelIndexFromPointer(parent)))))
}

func (ptr *QStandardItemModel) InsertRowsDefault(row int, count int, parent core.QModelIndex_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_InsertRowsDefault(ptr.Pointer(), C.int(int32(row)), C.int(int32(count)), core.PointerFromQModelIndex(parent))) != 0
	}
	return false
}

func (ptr *QStandardItemModel) InvisibleRootItem() *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel_InvisibleRootItem(ptr.Pointer()))
	}
	return nil
}

func (ptr *QStandardItemModel) Item(row int, column int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel_Item(ptr.Pointer(), C.int(int32(row)), C.int(int32(column))))
	}
	return nil
}

//export callbackQStandardItemModel_ItemChanged
func callbackQStandardItemModel_ItemChanged(ptr unsafe.Pointer, item unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "itemChanged"); signal != nil {
		(*(*func(*QStandardItem))(signal))(NewQStandardItemFromPointer(item))
	}

}

func (ptr *QStandardItemModel) ConnectItemChanged(f func(item *QStandardItem)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "itemChanged") {
			C.QStandardItemModel_ConnectItemChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "itemChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "itemChanged"); signal != nil {
			f := func(item *QStandardItem) {
				(*(*func(*QStandardItem))(signal))(item)
				f(item)
			}
			qt.ConnectSignal(ptr.Pointer(), "itemChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "itemChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItemModel) DisconnectItemChanged() {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_DisconnectItemChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "itemChanged")
	}
}

func (ptr *QStandardItemModel) ItemChanged(item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_ItemChanged(ptr.Pointer(), PointerFromQStandardItem(item))
	}
}

//export callbackQStandardItemModel_ItemData
func callbackQStandardItemModel_ItemData(ptr unsafe.Pointer, index unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "itemData"); signal != nil {
		return func() unsafe.Pointer {
			tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__itemData_newList())
			for k, v := range (*(*func(*core.QModelIndex) map[int]*core.QVariant)(signal))(core.NewQModelIndexFromPointer(index)) {
				tmpList.__itemData_setList(k, v)
			}
			return tmpList.Pointer()
		}()
	}

	return func() unsafe.Pointer {
		tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__itemData_newList())
		for k, v := range NewQStandardItemModelFromPointer(ptr).ItemDataDefault(core.NewQModelIndexFromPointer(index)) {
			tmpList.__itemData_setList(k, v)
		}
		return tmpList.Pointer()
	}()
}

func (ptr *QStandardItemModel) ItemDataDefault(index core.QModelIndex_ITF) map[int]*core.QVariant {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) map[int]*core.QVariant {
			out := make(map[int]*core.QVariant, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i, v := range tmpList.__itemData_keyList() {
				out[v] = tmpList.__itemData_atList(v, i)
			}
			return out
		}(C.QStandardItemModel_ItemDataDefault(ptr.Pointer(), core.PointerFromQModelIndex(index)))
	}
	return make(map[int]*core.QVariant, 0)
}

func (ptr *QStandardItemModel) ItemFromIndex(index core.QModelIndex_ITF) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel_ItemFromIndex(ptr.Pointer(), core.PointerFromQModelIndex(index)))
	}
	return nil
}

func (ptr *QStandardItemModel) ItemPrototype() *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel_ItemPrototype(ptr.Pointer()))
	}
	return nil
}

//export callbackQStandardItemModel_MimeData
func callbackQStandardItemModel_MimeData(ptr unsafe.Pointer, indexes C.struct_QtGui_PackedList) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "mimeData"); signal != nil {
		return core.PointerFromQMimeData((*(*func([]*core.QModelIndex) *core.QMimeData)(signal))(func(l C.struct_QtGui_PackedList) []*core.QModelIndex {
			out := make([]*core.QModelIndex, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__mimeData_indexes_atList(i)
			}
			return out
		}(indexes)))
	}

	return core.PointerFromQMimeData(NewQStandardItemModelFromPointer(ptr).MimeDataDefault(func(l C.struct_QtGui_PackedList) []*core.QModelIndex {
		out := make([]*core.QModelIndex, int(l.len))
		tmpList := NewQStandardItemModelFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__mimeData_indexes_atList(i)
		}
		return out
	}(indexes)))
}

func (ptr *QStandardItemModel) MimeDataDefault(indexes []*core.QModelIndex) *core.QMimeData {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMimeDataFromPointer(C.QStandardItemModel_MimeDataDefault(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__mimeData_indexes_newList())
			for _, v := range indexes {
				tmpList.__mimeData_indexes_setList(v)
			}
			return tmpList.Pointer()
		}()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQStandardItemModel_MimeTypes
func callbackQStandardItemModel_MimeTypes(ptr unsafe.Pointer) C.struct_QtGui_PackedString {
	if signal := qt.GetSignal(ptr, "mimeTypes"); signal != nil {
		tempVal := (*(*func() []string)(signal))()
		return C.struct_QtGui_PackedString{data: C.CString(strings.Join(tempVal, "¡¦!")), len: C.longlong(len(strings.Join(tempVal, "¡¦!")))}
	}
	tempVal := NewQStandardItemModelFromPointer(ptr).MimeTypesDefault()
	return C.struct_QtGui_PackedString{data: C.CString(strings.Join(tempVal, "¡¦!")), len: C.longlong(len(strings.Join(tempVal, "¡¦!")))}
}

func (ptr *QStandardItemModel) MimeTypesDefault() []string {
	if ptr.Pointer() != nil {
		return unpackStringList(cGoUnpackString(C.QStandardItemModel_MimeTypesDefault(ptr.Pointer())))
	}
	return make([]string, 0)
}

//export callbackQStandardItemModel_Parent
func callbackQStandardItemModel_Parent(ptr unsafe.Pointer, child unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "parent"); signal != nil {
		return core.PointerFromQModelIndex((*(*func(*core.QModelIndex) *core.QModelIndex)(signal))(core.NewQModelIndexFromPointer(child)))
	}

	return core.PointerFromQModelIndex(NewQStandardItemModelFromPointer(ptr).ParentDefault(core.NewQModelIndexFromPointer(child)))
}

func (ptr *QStandardItemModel) ConnectParent(f func(child *core.QModelIndex) *core.QModelIndex) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "parent"); signal != nil {
			f := func(child *core.QModelIndex) *core.QModelIndex {
				(*(*func(*core.QModelIndex) *core.QModelIndex)(signal))(child)
				return f(child)
			}
			qt.ConnectSignal(ptr.Pointer(), "parent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "parent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItemModel) DisconnectParent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "parent")
	}
}

func (ptr *QStandardItemModel) Parent(child core.QModelIndex_ITF) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel_Parent(ptr.Pointer(), core.PointerFromQModelIndex(child)))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) ParentDefault(child core.QModelIndex_ITF) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel_ParentDefault(ptr.Pointer(), core.PointerFromQModelIndex(child)))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

//export callbackQStandardItemModel_RemoveColumns
func callbackQStandardItemModel_RemoveColumns(ptr unsafe.Pointer, column C.int, count C.int, parent unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "removeColumns"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int, int, *core.QModelIndex) bool)(signal))(int(int32(column)), int(int32(count)), core.NewQModelIndexFromPointer(parent)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).RemoveColumnsDefault(int(int32(column)), int(int32(count)), core.NewQModelIndexFromPointer(parent)))))
}

func (ptr *QStandardItemModel) RemoveColumnsDefault(column int, count int, parent core.QModelIndex_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_RemoveColumnsDefault(ptr.Pointer(), C.int(int32(column)), C.int(int32(count)), core.PointerFromQModelIndex(parent))) != 0
	}
	return false
}

//export callbackQStandardItemModel_RemoveRows
func callbackQStandardItemModel_RemoveRows(ptr unsafe.Pointer, row C.int, count C.int, parent unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "removeRows"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int, int, *core.QModelIndex) bool)(signal))(int(int32(row)), int(int32(count)), core.NewQModelIndexFromPointer(parent)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).RemoveRowsDefault(int(int32(row)), int(int32(count)), core.NewQModelIndexFromPointer(parent)))))
}

func (ptr *QStandardItemModel) RemoveRowsDefault(row int, count int, parent core.QModelIndex_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_RemoveRowsDefault(ptr.Pointer(), C.int(int32(row)), C.int(int32(count)), core.PointerFromQModelIndex(parent))) != 0
	}
	return false
}

//export callbackQStandardItemModel_RowCount
func callbackQStandardItemModel_RowCount(ptr unsafe.Pointer, parent unsafe.Pointer) C.int {
	if signal := qt.GetSignal(ptr, "rowCount"); signal != nil {
		return C.int(int32((*(*func(*core.QModelIndex) int)(signal))(core.NewQModelIndexFromPointer(parent))))
	}

	return C.int(int32(NewQStandardItemModelFromPointer(ptr).RowCountDefault(core.NewQModelIndexFromPointer(parent))))
}

func (ptr *QStandardItemModel) ConnectRowCount(f func(parent *core.QModelIndex) int) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "rowCount"); signal != nil {
			f := func(parent *core.QModelIndex) int {
				(*(*func(*core.QModelIndex) int)(signal))(parent)
				return f(parent)
			}
			qt.ConnectSignal(ptr.Pointer(), "rowCount", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "rowCount", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItemModel) DisconnectRowCount() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "rowCount")
	}
}

func (ptr *QStandardItemModel) RowCount(parent core.QModelIndex_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_RowCount(ptr.Pointer(), core.PointerFromQModelIndex(parent))))
	}
	return 0
}

func (ptr *QStandardItemModel) RowCountDefault(parent core.QModelIndex_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_RowCountDefault(ptr.Pointer(), core.PointerFromQModelIndex(parent))))
	}
	return 0
}

func (ptr *QStandardItemModel) SetColumnCount(columns int) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_SetColumnCount(ptr.Pointer(), C.int(int32(columns)))
	}
}

//export callbackQStandardItemModel_SetData
func callbackQStandardItemModel_SetData(ptr unsafe.Pointer, index unsafe.Pointer, value unsafe.Pointer, role C.int) C.char {
	if signal := qt.GetSignal(ptr, "setData"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QModelIndex, *core.QVariant, int) bool)(signal))(core.NewQModelIndexFromPointer(index), core.NewQVariantFromPointer(value), int(int32(role))))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).SetDataDefault(core.NewQModelIndexFromPointer(index), core.NewQVariantFromPointer(value), int(int32(role))))))
}

func (ptr *QStandardItemModel) SetDataDefault(index core.QModelIndex_ITF, value core.QVariant_ITF, role int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_SetDataDefault(ptr.Pointer(), core.PointerFromQModelIndex(index), core.PointerFromQVariant(value), C.int(int32(role)))) != 0
	}
	return false
}

//export callbackQStandardItemModel_SetHeaderData
func callbackQStandardItemModel_SetHeaderData(ptr unsafe.Pointer, section C.int, orientation C.longlong, value unsafe.Pointer, role C.int) C.char {
	if signal := qt.GetSignal(ptr, "setHeaderData"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(int, core.Qt__Orientation, *core.QVariant, int) bool)(signal))(int(int32(section)), core.Qt__Orientation(orientation), core.NewQVariantFromPointer(value), int(int32(role))))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).SetHeaderDataDefault(int(int32(section)), core.Qt__Orientation(orientation), core.NewQVariantFromPointer(value), int(int32(role))))))
}

func (ptr *QStandardItemModel) SetHeaderDataDefault(section int, orientation core.Qt__Orientation, value core.QVariant_ITF, role int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_SetHeaderDataDefault(ptr.Pointer(), C.int(int32(section)), C.longlong(orientation), core.PointerFromQVariant(value), C.int(int32(role)))) != 0
	}
	return false
}

func (ptr *QStandardItemModel) SetHorizontalHeaderItem(column int, item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_SetHorizontalHeaderItem(ptr.Pointer(), C.int(int32(column)), PointerFromQStandardItem(item))
	}
}

func (ptr *QStandardItemModel) SetHorizontalHeaderLabels(labels []string) {
	if ptr.Pointer() != nil {
		labelsC := C.CString(strings.Join(labels, "¡¦!"))
		defer C.free(unsafe.Pointer(labelsC))
		C.QStandardItemModel_SetHorizontalHeaderLabels(ptr.Pointer(), C.struct_QtGui_PackedString{data: labelsC, len: C.longlong(len(strings.Join(labels, "¡¦!")))})
	}
}

func (ptr *QStandardItemModel) SetItem(row int, column int, item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_SetItem(ptr.Pointer(), C.int(int32(row)), C.int(int32(column)), PointerFromQStandardItem(item))
	}
}

func (ptr *QStandardItemModel) SetItem2(row int, item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_SetItem2(ptr.Pointer(), C.int(int32(row)), PointerFromQStandardItem(item))
	}
}

//export callbackQStandardItemModel_SetItemData
func callbackQStandardItemModel_SetItemData(ptr unsafe.Pointer, index unsafe.Pointer, roles C.struct_QtGui_PackedList) C.char {
	if signal := qt.GetSignal(ptr, "setItemData"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QModelIndex, map[int]*core.QVariant) bool)(signal))(core.NewQModelIndexFromPointer(index), func(l C.struct_QtGui_PackedList) map[int]*core.QVariant {
			out := make(map[int]*core.QVariant, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i, v := range tmpList.__setItemData_roles_keyList() {
				out[v] = tmpList.__setItemData_roles_atList(v, i)
			}
			return out
		}(roles)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).SetItemDataDefault(core.NewQModelIndexFromPointer(index), func(l C.struct_QtGui_PackedList) map[int]*core.QVariant {
		out := make(map[int]*core.QVariant, int(l.len))
		tmpList := NewQStandardItemModelFromPointer(l.data)
		for i, v := range tmpList.__setItemData_roles_keyList() {
			out[v] = tmpList.__setItemData_roles_atList(v, i)
		}
		return out
	}(roles)))))
}

func (ptr *QStandardItemModel) SetItemDataDefault(index core.QModelIndex_ITF, roles map[int]*core.QVariant) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_SetItemDataDefault(ptr.Pointer(), core.PointerFromQModelIndex(index), func() unsafe.Pointer {
			tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__setItemData_roles_newList())
			for k, v := range roles {
				tmpList.__setItemData_roles_setList(k, v)
			}
			return tmpList.Pointer()
		}())) != 0
	}
	return false
}

func (ptr *QStandardItemModel) SetItemPrototype(item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_SetItemPrototype(ptr.Pointer(), PointerFromQStandardItem(item))
	}
}

func (ptr *QStandardItemModel) SetItemRoleNames(roleNames map[int]*core.QByteArray) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_SetItemRoleNames(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__setItemRoleNames_roleNames_newList())
			for k, v := range roleNames {
				tmpList.__setItemRoleNames_roleNames_setList(k, v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QStandardItemModel) SetRowCount(rows int) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_SetRowCount(ptr.Pointer(), C.int(int32(rows)))
	}
}

func (ptr *QStandardItemModel) SetSortRole(role int) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_SetSortRole(ptr.Pointer(), C.int(int32(role)))
	}
}

func (ptr *QStandardItemModel) SetVerticalHeaderItem(row int, item QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_SetVerticalHeaderItem(ptr.Pointer(), C.int(int32(row)), PointerFromQStandardItem(item))
	}
}

func (ptr *QStandardItemModel) SetVerticalHeaderLabels(labels []string) {
	if ptr.Pointer() != nil {
		labelsC := C.CString(strings.Join(labels, "¡¦!"))
		defer C.free(unsafe.Pointer(labelsC))
		C.QStandardItemModel_SetVerticalHeaderLabels(ptr.Pointer(), C.struct_QtGui_PackedString{data: labelsC, len: C.longlong(len(strings.Join(labels, "¡¦!")))})
	}
}

//export callbackQStandardItemModel_Sibling
func callbackQStandardItemModel_Sibling(ptr unsafe.Pointer, row C.int, column C.int, idx unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "sibling"); signal != nil {
		return core.PointerFromQModelIndex((*(*func(int, int, *core.QModelIndex) *core.QModelIndex)(signal))(int(int32(row)), int(int32(column)), core.NewQModelIndexFromPointer(idx)))
	}

	return core.PointerFromQModelIndex(NewQStandardItemModelFromPointer(ptr).SiblingDefault(int(int32(row)), int(int32(column)), core.NewQModelIndexFromPointer(idx)))
}

func (ptr *QStandardItemModel) SiblingDefault(row int, column int, idx core.QModelIndex_ITF) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel_SiblingDefault(ptr.Pointer(), C.int(int32(row)), C.int(int32(column)), core.PointerFromQModelIndex(idx)))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

//export callbackQStandardItemModel_Sort
func callbackQStandardItemModel_Sort(ptr unsafe.Pointer, column C.int, order C.longlong) {
	if signal := qt.GetSignal(ptr, "sort"); signal != nil {
		(*(*func(int, core.Qt__SortOrder))(signal))(int(int32(column)), core.Qt__SortOrder(order))
	} else {
		NewQStandardItemModelFromPointer(ptr).SortDefault(int(int32(column)), core.Qt__SortOrder(order))
	}
}

func (ptr *QStandardItemModel) SortDefault(column int, order core.Qt__SortOrder) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_SortDefault(ptr.Pointer(), C.int(int32(column)), C.longlong(order))
	}
}

func (ptr *QStandardItemModel) SortRole() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_SortRole(ptr.Pointer())))
	}
	return 0
}

//export callbackQStandardItemModel_SupportedDropActions
func callbackQStandardItemModel_SupportedDropActions(ptr unsafe.Pointer) C.longlong {
	if signal := qt.GetSignal(ptr, "supportedDropActions"); signal != nil {
		return C.longlong((*(*func() core.Qt__DropAction)(signal))())
	}

	return C.longlong(NewQStandardItemModelFromPointer(ptr).SupportedDropActionsDefault())
}

func (ptr *QStandardItemModel) SupportedDropActionsDefault() core.Qt__DropAction {
	if ptr.Pointer() != nil {
		return core.Qt__DropAction(C.QStandardItemModel_SupportedDropActionsDefault(ptr.Pointer()))
	}
	return 0
}

func (ptr *QStandardItemModel) TakeColumn(column int) []*QStandardItem {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QStandardItem {
			out := make([]*QStandardItem, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__takeColumn_atList(i)
			}
			return out
		}(C.QStandardItemModel_TakeColumn(ptr.Pointer(), C.int(int32(column))))
	}
	return make([]*QStandardItem, 0)
}

func (ptr *QStandardItemModel) TakeHorizontalHeaderItem(column int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel_TakeHorizontalHeaderItem(ptr.Pointer(), C.int(int32(column))))
	}
	return nil
}

func (ptr *QStandardItemModel) TakeItem(row int, column int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel_TakeItem(ptr.Pointer(), C.int(int32(row)), C.int(int32(column))))
	}
	return nil
}

func (ptr *QStandardItemModel) TakeRow(row int) []*QStandardItem {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QStandardItem {
			out := make([]*QStandardItem, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__takeRow_atList(i)
			}
			return out
		}(C.QStandardItemModel_TakeRow(ptr.Pointer(), C.int(int32(row))))
	}
	return make([]*QStandardItem, 0)
}

func (ptr *QStandardItemModel) TakeVerticalHeaderItem(row int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel_TakeVerticalHeaderItem(ptr.Pointer(), C.int(int32(row))))
	}
	return nil
}

func (ptr *QStandardItemModel) VerticalHeaderItem(row int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel_VerticalHeaderItem(ptr.Pointer(), C.int(int32(row))))
	}
	return nil
}

//export callbackQStandardItemModel_DestroyQStandardItemModel
func callbackQStandardItemModel_DestroyQStandardItemModel(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QStandardItemModel"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQStandardItemModelFromPointer(ptr).DestroyQStandardItemModelDefault()
	}
}

func (ptr *QStandardItemModel) ConnectDestroyQStandardItemModel(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QStandardItemModel"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QStandardItemModel", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QStandardItemModel", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStandardItemModel) DisconnectDestroyQStandardItemModel() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QStandardItemModel")
	}
}

func (ptr *QStandardItemModel) DestroyQStandardItemModel() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QStandardItemModel_DestroyQStandardItemModel(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QStandardItemModel) DestroyQStandardItemModelDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QStandardItemModel_DestroyQStandardItemModelDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QStandardItemModel) __appendColumn_items_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel___appendColumn_items_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItemModel) __appendColumn_items_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___appendColumn_items_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItemModel) __appendColumn_items_newList() unsafe.Pointer {
	return C.QStandardItemModel___appendColumn_items_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __appendRow_items_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel___appendRow_items_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItemModel) __appendRow_items_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___appendRow_items_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItemModel) __appendRow_items_newList() unsafe.Pointer {
	return C.QStandardItemModel___appendRow_items_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __findItems_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel___findItems_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItemModel) __findItems_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___findItems_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItemModel) __findItems_newList() unsafe.Pointer {
	return C.QStandardItemModel___findItems_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __insertColumn_items_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel___insertColumn_items_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItemModel) __insertColumn_items_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___insertColumn_items_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItemModel) __insertColumn_items_newList() unsafe.Pointer {
	return C.QStandardItemModel___insertColumn_items_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __insertRow_items_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel___insertRow_items_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItemModel) __insertRow_items_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___insertRow_items_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItemModel) __insertRow_items_newList() unsafe.Pointer {
	return C.QStandardItemModel___insertRow_items_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __itemData_atList(v int, i int) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QStandardItemModel___itemData_atList(ptr.Pointer(), C.int(int32(v)), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __itemData_setList(key int, i core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___itemData_setList(ptr.Pointer(), C.int(int32(key)), core.PointerFromQVariant(i))
	}
}

func (ptr *QStandardItemModel) __itemData_newList() unsafe.Pointer {
	return C.QStandardItemModel___itemData_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __itemData_keyList() []int {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []int {
			out := make([]int, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.____itemData_keyList_atList(i)
			}
			return out
		}(C.QStandardItemModel___itemData_keyList(ptr.Pointer()))
	}
	return make([]int, 0)
}

func (ptr *QStandardItemModel) __mimeData_indexes_atList(i int) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel___mimeData_indexes_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __mimeData_indexes_setList(i core.QModelIndex_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___mimeData_indexes_setList(ptr.Pointer(), core.PointerFromQModelIndex(i))
	}
}

func (ptr *QStandardItemModel) __mimeData_indexes_newList() unsafe.Pointer {
	return C.QStandardItemModel___mimeData_indexes_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __setItemData_roles_atList(v int, i int) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QStandardItemModel___setItemData_roles_atList(ptr.Pointer(), C.int(int32(v)), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __setItemData_roles_setList(key int, i core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___setItemData_roles_setList(ptr.Pointer(), C.int(int32(key)), core.PointerFromQVariant(i))
	}
}

func (ptr *QStandardItemModel) __setItemData_roles_newList() unsafe.Pointer {
	return C.QStandardItemModel___setItemData_roles_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __setItemData_roles_keyList() []int {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []int {
			out := make([]int, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.____setItemData_roles_keyList_atList(i)
			}
			return out
		}(C.QStandardItemModel___setItemData_roles_keyList(ptr.Pointer()))
	}
	return make([]int, 0)
}

func (ptr *QStandardItemModel) __setItemRoleNames_roleNames_atList(v int, i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QStandardItemModel___setItemRoleNames_roleNames_atList(ptr.Pointer(), C.int(int32(v)), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __setItemRoleNames_roleNames_setList(key int, i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___setItemRoleNames_roleNames_setList(ptr.Pointer(), C.int(int32(key)), core.PointerFromQByteArray(i))
	}
}

func (ptr *QStandardItemModel) __setItemRoleNames_roleNames_newList() unsafe.Pointer {
	return C.QStandardItemModel___setItemRoleNames_roleNames_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __setItemRoleNames_roleNames_keyList() []int {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []int {
			out := make([]int, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.____setItemRoleNames_roleNames_keyList_atList(i)
			}
			return out
		}(C.QStandardItemModel___setItemRoleNames_roleNames_keyList(ptr.Pointer()))
	}
	return make([]int, 0)
}

func (ptr *QStandardItemModel) __takeColumn_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel___takeColumn_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItemModel) __takeColumn_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___takeColumn_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItemModel) __takeColumn_newList() unsafe.Pointer {
	return C.QStandardItemModel___takeColumn_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __takeRow_atList(i int) *QStandardItem {
	if ptr.Pointer() != nil {
		return NewQStandardItemFromPointer(C.QStandardItemModel___takeRow_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QStandardItemModel) __takeRow_setList(i QStandardItem_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___takeRow_setList(ptr.Pointer(), PointerFromQStandardItem(i))
	}
}

func (ptr *QStandardItemModel) __takeRow_newList() unsafe.Pointer {
	return C.QStandardItemModel___takeRow_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) ____itemData_keyList_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_____itemData_keyList_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QStandardItemModel) ____itemData_keyList_setList(i int) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_____itemData_keyList_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QStandardItemModel) ____itemData_keyList_newList() unsafe.Pointer {
	return C.QStandardItemModel_____itemData_keyList_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) ____setItemData_roles_keyList_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_____setItemData_roles_keyList_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QStandardItemModel) ____setItemData_roles_keyList_setList(i int) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_____setItemData_roles_keyList_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QStandardItemModel) ____setItemData_roles_keyList_newList() unsafe.Pointer {
	return C.QStandardItemModel_____setItemData_roles_keyList_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) ____setItemRoleNames_roleNames_keyList_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_____setItemRoleNames_roleNames_keyList_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QStandardItemModel) ____setItemRoleNames_roleNames_keyList_setList(i int) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_____setItemRoleNames_roleNames_keyList_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QStandardItemModel) ____setItemRoleNames_roleNames_keyList_newList() unsafe.Pointer {
	return C.QStandardItemModel_____setItemRoleNames_roleNames_keyList_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __changePersistentIndexList_from_atList(i int) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel___changePersistentIndexList_from_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __changePersistentIndexList_from_setList(i core.QModelIndex_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___changePersistentIndexList_from_setList(ptr.Pointer(), core.PointerFromQModelIndex(i))
	}
}

func (ptr *QStandardItemModel) __changePersistentIndexList_from_newList() unsafe.Pointer {
	return C.QStandardItemModel___changePersistentIndexList_from_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __changePersistentIndexList_to_atList(i int) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel___changePersistentIndexList_to_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __changePersistentIndexList_to_setList(i core.QModelIndex_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___changePersistentIndexList_to_setList(ptr.Pointer(), core.PointerFromQModelIndex(i))
	}
}

func (ptr *QStandardItemModel) __changePersistentIndexList_to_newList() unsafe.Pointer {
	return C.QStandardItemModel___changePersistentIndexList_to_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __dataChanged_roles_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel___dataChanged_roles_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QStandardItemModel) __dataChanged_roles_setList(i int) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___dataChanged_roles_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QStandardItemModel) __dataChanged_roles_newList() unsafe.Pointer {
	return C.QStandardItemModel___dataChanged_roles_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __layoutAboutToBeChanged_parents_atList(i int) *core.QPersistentModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPersistentModelIndexFromPointer(C.QStandardItemModel___layoutAboutToBeChanged_parents_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPersistentModelIndex).DestroyQPersistentModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __layoutAboutToBeChanged_parents_setList(i core.QPersistentModelIndex_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___layoutAboutToBeChanged_parents_setList(ptr.Pointer(), core.PointerFromQPersistentModelIndex(i))
	}
}

func (ptr *QStandardItemModel) __layoutAboutToBeChanged_parents_newList() unsafe.Pointer {
	return C.QStandardItemModel___layoutAboutToBeChanged_parents_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __layoutChanged_parents_atList(i int) *core.QPersistentModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPersistentModelIndexFromPointer(C.QStandardItemModel___layoutChanged_parents_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QPersistentModelIndex).DestroyQPersistentModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __layoutChanged_parents_setList(i core.QPersistentModelIndex_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___layoutChanged_parents_setList(ptr.Pointer(), core.PointerFromQPersistentModelIndex(i))
	}
}

func (ptr *QStandardItemModel) __layoutChanged_parents_newList() unsafe.Pointer {
	return C.QStandardItemModel___layoutChanged_parents_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __match_atList(i int) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel___match_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __match_setList(i core.QModelIndex_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___match_setList(ptr.Pointer(), core.PointerFromQModelIndex(i))
	}
}

func (ptr *QStandardItemModel) __match_newList() unsafe.Pointer {
	return C.QStandardItemModel___match_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __persistentIndexList_atList(i int) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel___persistentIndexList_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __persistentIndexList_setList(i core.QModelIndex_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___persistentIndexList_setList(ptr.Pointer(), core.PointerFromQModelIndex(i))
	}
}

func (ptr *QStandardItemModel) __persistentIndexList_newList() unsafe.Pointer {
	return C.QStandardItemModel___persistentIndexList_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __roleNames_atList(v int, i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QStandardItemModel___roleNames_atList(ptr.Pointer(), C.int(int32(v)), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __roleNames_setList(key int, i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___roleNames_setList(ptr.Pointer(), C.int(int32(key)), core.PointerFromQByteArray(i))
	}
}

func (ptr *QStandardItemModel) __roleNames_newList() unsafe.Pointer {
	return C.QStandardItemModel___roleNames_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __roleNames_keyList() []int {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []int {
			out := make([]int, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.____roleNames_keyList_atList(i)
			}
			return out
		}(C.QStandardItemModel___roleNames_keyList(ptr.Pointer()))
	}
	return make([]int, 0)
}

func (ptr *QStandardItemModel) ____doSetRoleNames_roleNames_keyList_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_____doSetRoleNames_roleNames_keyList_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QStandardItemModel) ____doSetRoleNames_roleNames_keyList_setList(i int) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_____doSetRoleNames_roleNames_keyList_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QStandardItemModel) ____doSetRoleNames_roleNames_keyList_newList() unsafe.Pointer {
	return C.QStandardItemModel_____doSetRoleNames_roleNames_keyList_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) ____roleNames_keyList_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_____roleNames_keyList_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QStandardItemModel) ____roleNames_keyList_setList(i int) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_____roleNames_keyList_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QStandardItemModel) ____roleNames_keyList_newList() unsafe.Pointer {
	return C.QStandardItemModel_____roleNames_keyList_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) ____setRoleNames_roleNames_keyList_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStandardItemModel_____setRoleNames_roleNames_keyList_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QStandardItemModel) ____setRoleNames_roleNames_keyList_setList(i int) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_____setRoleNames_roleNames_keyList_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QStandardItemModel) ____setRoleNames_roleNames_keyList_newList() unsafe.Pointer {
	return C.QStandardItemModel_____setRoleNames_roleNames_keyList_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QStandardItemModel___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QStandardItemModel) __children_newList() unsafe.Pointer {
	return C.QStandardItemModel___children_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QStandardItemModel___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QStandardItemModel) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QStandardItemModel___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QStandardItemModel___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QStandardItemModel) __findChildren_newList() unsafe.Pointer {
	return C.QStandardItemModel___findChildren_newList(ptr.Pointer())
}

func (ptr *QStandardItemModel) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QStandardItemModel___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QStandardItemModel) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QStandardItemModel) __findChildren_newList3() unsafe.Pointer {
	return C.QStandardItemModel___findChildren_newList3(ptr.Pointer())
}

//export callbackQStandardItemModel_Buddy
func callbackQStandardItemModel_Buddy(ptr unsafe.Pointer, index unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "buddy"); signal != nil {
		return core.PointerFromQModelIndex((*(*func(*core.QModelIndex) *core.QModelIndex)(signal))(core.NewQModelIndexFromPointer(index)))
	}

	return core.PointerFromQModelIndex(NewQStandardItemModelFromPointer(ptr).BuddyDefault(core.NewQModelIndexFromPointer(index)))
}

func (ptr *QStandardItemModel) BuddyDefault(index core.QModelIndex_ITF) *core.QModelIndex {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQModelIndexFromPointer(C.QStandardItemModel_BuddyDefault(ptr.Pointer(), core.PointerFromQModelIndex(index)))
		qt.SetFinalizer(tmpValue, (*core.QModelIndex).DestroyQModelIndex)
		return tmpValue
	}
	return nil
}

//export callbackQStandardItemModel_CanDropMimeData
func callbackQStandardItemModel_CanDropMimeData(ptr unsafe.Pointer, data unsafe.Pointer, action C.longlong, row C.int, column C.int, parent unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "canDropMimeData"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QMimeData, core.Qt__DropAction, int, int, *core.QModelIndex) bool)(signal))(core.NewQMimeDataFromPointer(data), core.Qt__DropAction(action), int(int32(row)), int(int32(column)), core.NewQModelIndexFromPointer(parent)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).CanDropMimeDataDefault(core.NewQMimeDataFromPointer(data), core.Qt__DropAction(action), int(int32(row)), int(int32(column)), core.NewQModelIndexFromPointer(parent)))))
}

func (ptr *QStandardItemModel) CanDropMimeDataDefault(data core.QMimeData_ITF, action core.Qt__DropAction, row int, column int, parent core.QModelIndex_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_CanDropMimeDataDefault(ptr.Pointer(), core.PointerFromQMimeData(data), C.longlong(action), C.int(int32(row)), C.int(int32(column)), core.PointerFromQModelIndex(parent))) != 0
	}
	return false
}

//export callbackQStandardItemModel_CanFetchMore
func callbackQStandardItemModel_CanFetchMore(ptr unsafe.Pointer, parent unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "canFetchMore"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QModelIndex) bool)(signal))(core.NewQModelIndexFromPointer(parent)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).CanFetchMoreDefault(core.NewQModelIndexFromPointer(parent)))))
}

func (ptr *QStandardItemModel) CanFetchMoreDefault(parent core.QModelIndex_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_CanFetchMoreDefault(ptr.Pointer(), core.PointerFromQModelIndex(parent))) != 0
	}
	return false
}

//export callbackQStandardItemModel_ColumnsAboutToBeInserted
func callbackQStandardItemModel_ColumnsAboutToBeInserted(ptr unsafe.Pointer, parent unsafe.Pointer, first C.int, last C.int) {
	if signal := qt.GetSignal(ptr, "columnsAboutToBeInserted"); signal != nil {
		(*(*func(*core.QModelIndex, int, int))(signal))(core.NewQModelIndexFromPointer(parent), int(int32(first)), int(int32(last)))
	}

}

//export callbackQStandardItemModel_ColumnsAboutToBeMoved
func callbackQStandardItemModel_ColumnsAboutToBeMoved(ptr unsafe.Pointer, sourceParent unsafe.Pointer, sourceStart C.int, sourceEnd C.int, destinationParent unsafe.Pointer, destinationColumn C.int) {
	if signal := qt.GetSignal(ptr, "columnsAboutToBeMoved"); signal != nil {
		(*(*func(*core.QModelIndex, int, int, *core.QModelIndex, int))(signal))(core.NewQModelIndexFromPointer(sourceParent), int(int32(sourceStart)), int(int32(sourceEnd)), core.NewQModelIndexFromPointer(destinationParent), int(int32(destinationColumn)))
	}

}

//export callbackQStandardItemModel_ColumnsAboutToBeRemoved
func callbackQStandardItemModel_ColumnsAboutToBeRemoved(ptr unsafe.Pointer, parent unsafe.Pointer, first C.int, last C.int) {
	if signal := qt.GetSignal(ptr, "columnsAboutToBeRemoved"); signal != nil {
		(*(*func(*core.QModelIndex, int, int))(signal))(core.NewQModelIndexFromPointer(parent), int(int32(first)), int(int32(last)))
	}

}

//export callbackQStandardItemModel_ColumnsInserted
func callbackQStandardItemModel_ColumnsInserted(ptr unsafe.Pointer, parent unsafe.Pointer, first C.int, last C.int) {
	if signal := qt.GetSignal(ptr, "columnsInserted"); signal != nil {
		(*(*func(*core.QModelIndex, int, int))(signal))(core.NewQModelIndexFromPointer(parent), int(int32(first)), int(int32(last)))
	}

}

//export callbackQStandardItemModel_ColumnsMoved
func callbackQStandardItemModel_ColumnsMoved(ptr unsafe.Pointer, parent unsafe.Pointer, start C.int, end C.int, destination unsafe.Pointer, column C.int) {
	if signal := qt.GetSignal(ptr, "columnsMoved"); signal != nil {
		(*(*func(*core.QModelIndex, int, int, *core.QModelIndex, int))(signal))(core.NewQModelIndexFromPointer(parent), int(int32(start)), int(int32(end)), core.NewQModelIndexFromPointer(destination), int(int32(column)))
	}

}

//export callbackQStandardItemModel_ColumnsRemoved
func callbackQStandardItemModel_ColumnsRemoved(ptr unsafe.Pointer, parent unsafe.Pointer, first C.int, last C.int) {
	if signal := qt.GetSignal(ptr, "columnsRemoved"); signal != nil {
		(*(*func(*core.QModelIndex, int, int))(signal))(core.NewQModelIndexFromPointer(parent), int(int32(first)), int(int32(last)))
	}

}

//export callbackQStandardItemModel_DataChanged
func callbackQStandardItemModel_DataChanged(ptr unsafe.Pointer, topLeft unsafe.Pointer, bottomRight unsafe.Pointer, roles C.struct_QtGui_PackedList) {
	if signal := qt.GetSignal(ptr, "dataChanged"); signal != nil {
		(*(*func(*core.QModelIndex, *core.QModelIndex, []int))(signal))(core.NewQModelIndexFromPointer(topLeft), core.NewQModelIndexFromPointer(bottomRight), func(l C.struct_QtGui_PackedList) []int {
			out := make([]int, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__dataChanged_roles_atList(i)
			}
			return out
		}(roles))
	}

}

//export callbackQStandardItemModel_FetchMore
func callbackQStandardItemModel_FetchMore(ptr unsafe.Pointer, parent unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "fetchMore"); signal != nil {
		(*(*func(*core.QModelIndex))(signal))(core.NewQModelIndexFromPointer(parent))
	} else {
		NewQStandardItemModelFromPointer(ptr).FetchMoreDefault(core.NewQModelIndexFromPointer(parent))
	}
}

func (ptr *QStandardItemModel) FetchMoreDefault(parent core.QModelIndex_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_FetchMoreDefault(ptr.Pointer(), core.PointerFromQModelIndex(parent))
	}
}

//export callbackQStandardItemModel_HeaderDataChanged
func callbackQStandardItemModel_HeaderDataChanged(ptr unsafe.Pointer, orientation C.longlong, first C.int, last C.int) {
	if signal := qt.GetSignal(ptr, "headerDataChanged"); signal != nil {
		(*(*func(core.Qt__Orientation, int, int))(signal))(core.Qt__Orientation(orientation), int(int32(first)), int(int32(last)))
	}

}

//export callbackQStandardItemModel_LayoutAboutToBeChanged
func callbackQStandardItemModel_LayoutAboutToBeChanged(ptr unsafe.Pointer, parents C.struct_QtGui_PackedList, hint C.longlong) {
	if signal := qt.GetSignal(ptr, "layoutAboutToBeChanged"); signal != nil {
		(*(*func([]*core.QPersistentModelIndex, core.QAbstractItemModel__LayoutChangeHint))(signal))(func(l C.struct_QtGui_PackedList) []*core.QPersistentModelIndex {
			out := make([]*core.QPersistentModelIndex, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__layoutAboutToBeChanged_parents_atList(i)
			}
			return out
		}(parents), core.QAbstractItemModel__LayoutChangeHint(hint))
	}

}

//export callbackQStandardItemModel_LayoutChanged
func callbackQStandardItemModel_LayoutChanged(ptr unsafe.Pointer, parents C.struct_QtGui_PackedList, hint C.longlong) {
	if signal := qt.GetSignal(ptr, "layoutChanged"); signal != nil {
		(*(*func([]*core.QPersistentModelIndex, core.QAbstractItemModel__LayoutChangeHint))(signal))(func(l C.struct_QtGui_PackedList) []*core.QPersistentModelIndex {
			out := make([]*core.QPersistentModelIndex, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__layoutChanged_parents_atList(i)
			}
			return out
		}(parents), core.QAbstractItemModel__LayoutChangeHint(hint))
	}

}

//export callbackQStandardItemModel_Match
func callbackQStandardItemModel_Match(ptr unsafe.Pointer, start unsafe.Pointer, role C.int, value unsafe.Pointer, hits C.int, flags C.longlong) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "match"); signal != nil {
		return func() unsafe.Pointer {
			tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__match_newList())
			for _, v := range (*(*func(*core.QModelIndex, int, *core.QVariant, int, core.Qt__MatchFlag) []*core.QModelIndex)(signal))(core.NewQModelIndexFromPointer(start), int(int32(role)), core.NewQVariantFromPointer(value), int(int32(hits)), core.Qt__MatchFlag(flags)) {
				tmpList.__match_setList(v)
			}
			return tmpList.Pointer()
		}()
	}

	return func() unsafe.Pointer {
		tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__match_newList())
		for _, v := range NewQStandardItemModelFromPointer(ptr).MatchDefault(core.NewQModelIndexFromPointer(start), int(int32(role)), core.NewQVariantFromPointer(value), int(int32(hits)), core.Qt__MatchFlag(flags)) {
			tmpList.__match_setList(v)
		}
		return tmpList.Pointer()
	}()
}

func (ptr *QStandardItemModel) MatchDefault(start core.QModelIndex_ITF, role int, value core.QVariant_ITF, hits int, flags core.Qt__MatchFlag) []*core.QModelIndex {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*core.QModelIndex {
			out := make([]*core.QModelIndex, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__match_atList(i)
			}
			return out
		}(C.QStandardItemModel_MatchDefault(ptr.Pointer(), core.PointerFromQModelIndex(start), C.int(int32(role)), core.PointerFromQVariant(value), C.int(int32(hits)), C.longlong(flags)))
	}
	return make([]*core.QModelIndex, 0)
}

//export callbackQStandardItemModel_ModelAboutToBeReset
func callbackQStandardItemModel_ModelAboutToBeReset(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "modelAboutToBeReset"); signal != nil {
		(*(*func())(signal))()
	}

}

//export callbackQStandardItemModel_ModelReset
func callbackQStandardItemModel_ModelReset(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "modelReset"); signal != nil {
		(*(*func())(signal))()
	}

}

//export callbackQStandardItemModel_MoveColumns
func callbackQStandardItemModel_MoveColumns(ptr unsafe.Pointer, sourceParent unsafe.Pointer, sourceColumn C.int, count C.int, destinationParent unsafe.Pointer, destinationChild C.int) C.char {
	if signal := qt.GetSignal(ptr, "moveColumns"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QModelIndex, int, int, *core.QModelIndex, int) bool)(signal))(core.NewQModelIndexFromPointer(sourceParent), int(int32(sourceColumn)), int(int32(count)), core.NewQModelIndexFromPointer(destinationParent), int(int32(destinationChild))))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).MoveColumnsDefault(core.NewQModelIndexFromPointer(sourceParent), int(int32(sourceColumn)), int(int32(count)), core.NewQModelIndexFromPointer(destinationParent), int(int32(destinationChild))))))
}

func (ptr *QStandardItemModel) MoveColumnsDefault(sourceParent core.QModelIndex_ITF, sourceColumn int, count int, destinationParent core.QModelIndex_ITF, destinationChild int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_MoveColumnsDefault(ptr.Pointer(), core.PointerFromQModelIndex(sourceParent), C.int(int32(sourceColumn)), C.int(int32(count)), core.PointerFromQModelIndex(destinationParent), C.int(int32(destinationChild)))) != 0
	}
	return false
}

//export callbackQStandardItemModel_MoveRows
func callbackQStandardItemModel_MoveRows(ptr unsafe.Pointer, sourceParent unsafe.Pointer, sourceRow C.int, count C.int, destinationParent unsafe.Pointer, destinationChild C.int) C.char {
	if signal := qt.GetSignal(ptr, "moveRows"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QModelIndex, int, int, *core.QModelIndex, int) bool)(signal))(core.NewQModelIndexFromPointer(sourceParent), int(int32(sourceRow)), int(int32(count)), core.NewQModelIndexFromPointer(destinationParent), int(int32(destinationChild))))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).MoveRowsDefault(core.NewQModelIndexFromPointer(sourceParent), int(int32(sourceRow)), int(int32(count)), core.NewQModelIndexFromPointer(destinationParent), int(int32(destinationChild))))))
}

func (ptr *QStandardItemModel) MoveRowsDefault(sourceParent core.QModelIndex_ITF, sourceRow int, count int, destinationParent core.QModelIndex_ITF, destinationChild int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_MoveRowsDefault(ptr.Pointer(), core.PointerFromQModelIndex(sourceParent), C.int(int32(sourceRow)), C.int(int32(count)), core.PointerFromQModelIndex(destinationParent), C.int(int32(destinationChild)))) != 0
	}
	return false
}

//export callbackQStandardItemModel_ResetInternalData
func callbackQStandardItemModel_ResetInternalData(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "resetInternalData"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQStandardItemModelFromPointer(ptr).ResetInternalDataDefault()
	}
}

func (ptr *QStandardItemModel) ResetInternalDataDefault() {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_ResetInternalDataDefault(ptr.Pointer())
	}
}

//export callbackQStandardItemModel_Revert
func callbackQStandardItemModel_Revert(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "revert"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQStandardItemModelFromPointer(ptr).RevertDefault()
	}
}

func (ptr *QStandardItemModel) RevertDefault() {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_RevertDefault(ptr.Pointer())
	}
}

//export callbackQStandardItemModel_RoleNames
func callbackQStandardItemModel_RoleNames(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "roleNames"); signal != nil {
		return func() unsafe.Pointer {
			tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__roleNames_newList())
			for k, v := range (*(*func() map[int]*core.QByteArray)(signal))() {
				tmpList.__roleNames_setList(k, v)
			}
			return tmpList.Pointer()
		}()
	}

	return func() unsafe.Pointer {
		tmpList := NewQStandardItemModelFromPointer(NewQStandardItemModelFromPointer(nil).__roleNames_newList())
		for k, v := range NewQStandardItemModelFromPointer(ptr).RoleNamesDefault() {
			tmpList.__roleNames_setList(k, v)
		}
		return tmpList.Pointer()
	}()
}

func (ptr *QStandardItemModel) RoleNamesDefault() map[int]*core.QByteArray {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) map[int]*core.QByteArray {
			out := make(map[int]*core.QByteArray, int(l.len))
			tmpList := NewQStandardItemModelFromPointer(l.data)
			for i, v := range tmpList.__roleNames_keyList() {
				out[v] = tmpList.__roleNames_atList(v, i)
			}
			return out
		}(C.QStandardItemModel_RoleNamesDefault(ptr.Pointer()))
	}
	return make(map[int]*core.QByteArray, 0)
}

//export callbackQStandardItemModel_RowsAboutToBeInserted
func callbackQStandardItemModel_RowsAboutToBeInserted(ptr unsafe.Pointer, parent unsafe.Pointer, start C.int, end C.int) {
	if signal := qt.GetSignal(ptr, "rowsAboutToBeInserted"); signal != nil {
		(*(*func(*core.QModelIndex, int, int))(signal))(core.NewQModelIndexFromPointer(parent), int(int32(start)), int(int32(end)))
	}

}

//export callbackQStandardItemModel_RowsAboutToBeMoved
func callbackQStandardItemModel_RowsAboutToBeMoved(ptr unsafe.Pointer, sourceParent unsafe.Pointer, sourceStart C.int, sourceEnd C.int, destinationParent unsafe.Pointer, destinationRow C.int) {
	if signal := qt.GetSignal(ptr, "rowsAboutToBeMoved"); signal != nil {
		(*(*func(*core.QModelIndex, int, int, *core.QModelIndex, int))(signal))(core.NewQModelIndexFromPointer(sourceParent), int(int32(sourceStart)), int(int32(sourceEnd)), core.NewQModelIndexFromPointer(destinationParent), int(int32(destinationRow)))
	}

}

//export callbackQStandardItemModel_RowsAboutToBeRemoved
func callbackQStandardItemModel_RowsAboutToBeRemoved(ptr unsafe.Pointer, parent unsafe.Pointer, first C.int, last C.int) {
	if signal := qt.GetSignal(ptr, "rowsAboutToBeRemoved"); signal != nil {
		(*(*func(*core.QModelIndex, int, int))(signal))(core.NewQModelIndexFromPointer(parent), int(int32(first)), int(int32(last)))
	}

}

//export callbackQStandardItemModel_RowsInserted
func callbackQStandardItemModel_RowsInserted(ptr unsafe.Pointer, parent unsafe.Pointer, first C.int, last C.int) {
	if signal := qt.GetSignal(ptr, "rowsInserted"); signal != nil {
		(*(*func(*core.QModelIndex, int, int))(signal))(core.NewQModelIndexFromPointer(parent), int(int32(first)), int(int32(last)))
	}

}

//export callbackQStandardItemModel_RowsMoved
func callbackQStandardItemModel_RowsMoved(ptr unsafe.Pointer, parent unsafe.Pointer, start C.int, end C.int, destination unsafe.Pointer, row C.int) {
	if signal := qt.GetSignal(ptr, "rowsMoved"); signal != nil {
		(*(*func(*core.QModelIndex, int, int, *core.QModelIndex, int))(signal))(core.NewQModelIndexFromPointer(parent), int(int32(start)), int(int32(end)), core.NewQModelIndexFromPointer(destination), int(int32(row)))
	}

}

//export callbackQStandardItemModel_RowsRemoved
func callbackQStandardItemModel_RowsRemoved(ptr unsafe.Pointer, parent unsafe.Pointer, first C.int, last C.int) {
	if signal := qt.GetSignal(ptr, "rowsRemoved"); signal != nil {
		(*(*func(*core.QModelIndex, int, int))(signal))(core.NewQModelIndexFromPointer(parent), int(int32(first)), int(int32(last)))
	}

}

//export callbackQStandardItemModel_Span
func callbackQStandardItemModel_Span(ptr unsafe.Pointer, index unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "span"); signal != nil {
		return core.PointerFromQSize((*(*func(*core.QModelIndex) *core.QSize)(signal))(core.NewQModelIndexFromPointer(index)))
	}

	return core.PointerFromQSize(NewQStandardItemModelFromPointer(ptr).SpanDefault(core.NewQModelIndexFromPointer(index)))
}

func (ptr *QStandardItemModel) SpanDefault(index core.QModelIndex_ITF) *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QStandardItemModel_SpanDefault(ptr.Pointer(), core.PointerFromQModelIndex(index)))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

//export callbackQStandardItemModel_Submit
func callbackQStandardItemModel_Submit(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "submit"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).SubmitDefault())))
}

func (ptr *QStandardItemModel) SubmitDefault() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_SubmitDefault(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQStandardItemModel_SupportedDragActions
func callbackQStandardItemModel_SupportedDragActions(ptr unsafe.Pointer) C.longlong {
	if signal := qt.GetSignal(ptr, "supportedDragActions"); signal != nil {
		return C.longlong((*(*func() core.Qt__DropAction)(signal))())
	}

	return C.longlong(NewQStandardItemModelFromPointer(ptr).SupportedDragActionsDefault())
}

func (ptr *QStandardItemModel) SupportedDragActionsDefault() core.Qt__DropAction {
	if ptr.Pointer() != nil {
		return core.Qt__DropAction(C.QStandardItemModel_SupportedDragActionsDefault(ptr.Pointer()))
	}
	return 0
}

//export callbackQStandardItemModel_ChildEvent
func callbackQStandardItemModel_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQStandardItemModelFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QStandardItemModel) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQStandardItemModel_ConnectNotify
func callbackQStandardItemModel_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQStandardItemModelFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QStandardItemModel) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQStandardItemModel_CustomEvent
func callbackQStandardItemModel_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQStandardItemModelFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QStandardItemModel) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQStandardItemModel_DeleteLater
func callbackQStandardItemModel_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQStandardItemModelFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QStandardItemModel) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QStandardItemModel_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQStandardItemModel_Destroyed
func callbackQStandardItemModel_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQStandardItemModel_DisconnectNotify
func callbackQStandardItemModel_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQStandardItemModelFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QStandardItemModel) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQStandardItemModel_Event
func callbackQStandardItemModel_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QStandardItemModel) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQStandardItemModel_EventFilter
func callbackQStandardItemModel_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStandardItemModelFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QStandardItemModel) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStandardItemModel_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQStandardItemModel_MetaObject
func callbackQStandardItemModel_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQStandardItemModelFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QStandardItemModel) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QStandardItemModel_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQStandardItemModel_ObjectNameChanged
func callbackQStandardItemModel_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQStandardItemModel_TimerEvent
func callbackQStandardItemModel_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQStandardItemModelFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QStandardItemModel) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QStandardItemModel_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QStaticText struct {
	ptr unsafe.Pointer
}

type QStaticText_ITF interface {
	QStaticText_PTR() *QStaticText
}

func (ptr *QStaticText) QStaticText_PTR() *QStaticText {
	return ptr
}

func (ptr *QStaticText) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QStaticText) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQStaticText(ptr QStaticText_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QStaticText_PTR().Pointer()
	}
	return nil
}

func NewQStaticTextFromPointer(ptr unsafe.Pointer) (n *QStaticText) {
	n = new(QStaticText)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QStaticText__PerformanceHint
//QStaticText::PerformanceHint
type QStaticText__PerformanceHint int64

const (
	QStaticText__ModerateCaching   QStaticText__PerformanceHint = QStaticText__PerformanceHint(0)
	QStaticText__AggressiveCaching QStaticText__PerformanceHint = QStaticText__PerformanceHint(1)
)

func NewQStaticText() *QStaticText {
	tmpValue := NewQStaticTextFromPointer(C.QStaticText_NewQStaticText())
	qt.SetFinalizer(tmpValue, (*QStaticText).DestroyQStaticText)
	return tmpValue
}

func NewQStaticText2(text string) *QStaticText {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQStaticTextFromPointer(C.QStaticText_NewQStaticText2(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
	qt.SetFinalizer(tmpValue, (*QStaticText).DestroyQStaticText)
	return tmpValue
}

func NewQStaticText3(other QStaticText_ITF) *QStaticText {
	tmpValue := NewQStaticTextFromPointer(C.QStaticText_NewQStaticText3(PointerFromQStaticText(other)))
	qt.SetFinalizer(tmpValue, (*QStaticText).DestroyQStaticText)
	return tmpValue
}

func (ptr *QStaticText) PerformanceHint() QStaticText__PerformanceHint {
	if ptr.Pointer() != nil {
		return QStaticText__PerformanceHint(C.QStaticText_PerformanceHint(ptr.Pointer()))
	}
	return 0
}

func (ptr *QStaticText) Prepare(matrix QTransform_ITF, font QFont_ITF) {
	if ptr.Pointer() != nil {
		C.QStaticText_Prepare(ptr.Pointer(), PointerFromQTransform(matrix), PointerFromQFont(font))
	}
}

func (ptr *QStaticText) SetPerformanceHint(performanceHint QStaticText__PerformanceHint) {
	if ptr.Pointer() != nil {
		C.QStaticText_SetPerformanceHint(ptr.Pointer(), C.longlong(performanceHint))
	}
}

func (ptr *QStaticText) SetText(text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QStaticText_SetText(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QStaticText) SetTextFormat(textFormat core.Qt__TextFormat) {
	if ptr.Pointer() != nil {
		C.QStaticText_SetTextFormat(ptr.Pointer(), C.longlong(textFormat))
	}
}

func (ptr *QStaticText) SetTextOption(textOption QTextOption_ITF) {
	if ptr.Pointer() != nil {
		C.QStaticText_SetTextOption(ptr.Pointer(), PointerFromQTextOption(textOption))
	}
}

func (ptr *QStaticText) SetTextWidth(textWidth float64) {
	if ptr.Pointer() != nil {
		C.QStaticText_SetTextWidth(ptr.Pointer(), C.double(textWidth))
	}
}

func (ptr *QStaticText) Size() *core.QSizeF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFFromPointer(C.QStaticText_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

func (ptr *QStaticText) Swap(other QStaticText_ITF) {
	if ptr.Pointer() != nil {
		C.QStaticText_Swap(ptr.Pointer(), PointerFromQStaticText(other))
	}
}

func (ptr *QStaticText) Text() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QStaticText_Text(ptr.Pointer()))
	}
	return ""
}

func (ptr *QStaticText) TextFormat() core.Qt__TextFormat {
	if ptr.Pointer() != nil {
		return core.Qt__TextFormat(C.QStaticText_TextFormat(ptr.Pointer()))
	}
	return 0
}

func (ptr *QStaticText) TextOption() *QTextOption {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextOptionFromPointer(C.QStaticText_TextOption(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextOption).DestroyQTextOption)
		return tmpValue
	}
	return nil
}

func (ptr *QStaticText) TextWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QStaticText_TextWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QStaticText) DestroyQStaticText() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QStaticText_DestroyQStaticText(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QStatusTipEvent struct {
	core.QEvent
}

type QStatusTipEvent_ITF interface {
	core.QEvent_ITF
	QStatusTipEvent_PTR() *QStatusTipEvent
}

func (ptr *QStatusTipEvent) QStatusTipEvent_PTR() *QStatusTipEvent {
	return ptr
}

func (ptr *QStatusTipEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QStatusTipEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQStatusTipEvent(ptr QStatusTipEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QStatusTipEvent_PTR().Pointer()
	}
	return nil
}

func NewQStatusTipEventFromPointer(ptr unsafe.Pointer) (n *QStatusTipEvent) {
	n = new(QStatusTipEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QStatusTipEvent) DestroyQStatusTipEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQStatusTipEvent(tip string) *QStatusTipEvent {
	var tipC *C.char
	if tip != "" {
		tipC = C.CString(tip)
		defer C.free(unsafe.Pointer(tipC))
	}
	tmpValue := NewQStatusTipEventFromPointer(C.QStatusTipEvent_NewQStatusTipEvent(C.struct_QtGui_PackedString{data: tipC, len: C.longlong(len(tip))}))
	qt.SetFinalizer(tmpValue, (*QStatusTipEvent).DestroyQStatusTipEvent)
	return tmpValue
}

func (ptr *QStatusTipEvent) Tip() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QStatusTipEvent_Tip(ptr.Pointer()))
	}
	return ""
}

type QStyleHints struct {
	core.QObject
}

type QStyleHints_ITF interface {
	core.QObject_ITF
	QStyleHints_PTR() *QStyleHints
}

func (ptr *QStyleHints) QStyleHints_PTR() *QStyleHints {
	return ptr
}

func (ptr *QStyleHints) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QStyleHints) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQStyleHints(ptr QStyleHints_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QStyleHints_PTR().Pointer()
	}
	return nil
}

func NewQStyleHintsFromPointer(ptr unsafe.Pointer) (n *QStyleHints) {
	n = new(QStyleHints)
	n.SetPointer(ptr)
	return
}
func (ptr *QStyleHints) CursorFlashTime() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_CursorFlashTime(ptr.Pointer())))
	}
	return 0
}

//export callbackQStyleHints_CursorFlashTimeChanged
func callbackQStyleHints_CursorFlashTimeChanged(ptr unsafe.Pointer, cursorFlashTime C.int) {
	if signal := qt.GetSignal(ptr, "cursorFlashTimeChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(cursorFlashTime)))
	}

}

func (ptr *QStyleHints) ConnectCursorFlashTimeChanged(f func(cursorFlashTime int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "cursorFlashTimeChanged") {
			C.QStyleHints_ConnectCursorFlashTimeChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "cursorFlashTimeChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "cursorFlashTimeChanged"); signal != nil {
			f := func(cursorFlashTime int) {
				(*(*func(int))(signal))(cursorFlashTime)
				f(cursorFlashTime)
			}
			qt.ConnectSignal(ptr.Pointer(), "cursorFlashTimeChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "cursorFlashTimeChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectCursorFlashTimeChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectCursorFlashTimeChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "cursorFlashTimeChanged")
	}
}

func (ptr *QStyleHints) CursorFlashTimeChanged(cursorFlashTime int) {
	if ptr.Pointer() != nil {
		C.QStyleHints_CursorFlashTimeChanged(ptr.Pointer(), C.int(int32(cursorFlashTime)))
	}
}

func (ptr *QStyleHints) FontSmoothingGamma() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QStyleHints_FontSmoothingGamma(ptr.Pointer()))
	}
	return 0
}

func (ptr *QStyleHints) KeyboardAutoRepeatRate() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_KeyboardAutoRepeatRate(ptr.Pointer())))
	}
	return 0
}

func (ptr *QStyleHints) KeyboardInputInterval() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_KeyboardInputInterval(ptr.Pointer())))
	}
	return 0
}

//export callbackQStyleHints_KeyboardInputIntervalChanged
func callbackQStyleHints_KeyboardInputIntervalChanged(ptr unsafe.Pointer, keyboardInputInterval C.int) {
	if signal := qt.GetSignal(ptr, "keyboardInputIntervalChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(keyboardInputInterval)))
	}

}

func (ptr *QStyleHints) ConnectKeyboardInputIntervalChanged(f func(keyboardInputInterval int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "keyboardInputIntervalChanged") {
			C.QStyleHints_ConnectKeyboardInputIntervalChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "keyboardInputIntervalChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "keyboardInputIntervalChanged"); signal != nil {
			f := func(keyboardInputInterval int) {
				(*(*func(int))(signal))(keyboardInputInterval)
				f(keyboardInputInterval)
			}
			qt.ConnectSignal(ptr.Pointer(), "keyboardInputIntervalChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "keyboardInputIntervalChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectKeyboardInputIntervalChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectKeyboardInputIntervalChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "keyboardInputIntervalChanged")
	}
}

func (ptr *QStyleHints) KeyboardInputIntervalChanged(keyboardInputInterval int) {
	if ptr.Pointer() != nil {
		C.QStyleHints_KeyboardInputIntervalChanged(ptr.Pointer(), C.int(int32(keyboardInputInterval)))
	}
}

func (ptr *QStyleHints) MouseDoubleClickInterval() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_MouseDoubleClickInterval(ptr.Pointer())))
	}
	return 0
}

//export callbackQStyleHints_MouseDoubleClickIntervalChanged
func callbackQStyleHints_MouseDoubleClickIntervalChanged(ptr unsafe.Pointer, mouseDoubleClickInterval C.int) {
	if signal := qt.GetSignal(ptr, "mouseDoubleClickIntervalChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(mouseDoubleClickInterval)))
	}

}

func (ptr *QStyleHints) ConnectMouseDoubleClickIntervalChanged(f func(mouseDoubleClickInterval int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "mouseDoubleClickIntervalChanged") {
			C.QStyleHints_ConnectMouseDoubleClickIntervalChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "mouseDoubleClickIntervalChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "mouseDoubleClickIntervalChanged"); signal != nil {
			f := func(mouseDoubleClickInterval int) {
				(*(*func(int))(signal))(mouseDoubleClickInterval)
				f(mouseDoubleClickInterval)
			}
			qt.ConnectSignal(ptr.Pointer(), "mouseDoubleClickIntervalChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "mouseDoubleClickIntervalChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectMouseDoubleClickIntervalChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectMouseDoubleClickIntervalChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "mouseDoubleClickIntervalChanged")
	}
}

func (ptr *QStyleHints) MouseDoubleClickIntervalChanged(mouseDoubleClickInterval int) {
	if ptr.Pointer() != nil {
		C.QStyleHints_MouseDoubleClickIntervalChanged(ptr.Pointer(), C.int(int32(mouseDoubleClickInterval)))
	}
}

func (ptr *QStyleHints) MousePressAndHoldInterval() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_MousePressAndHoldInterval(ptr.Pointer())))
	}
	return 0
}

//export callbackQStyleHints_MousePressAndHoldIntervalChanged
func callbackQStyleHints_MousePressAndHoldIntervalChanged(ptr unsafe.Pointer, mousePressAndHoldInterval C.int) {
	if signal := qt.GetSignal(ptr, "mousePressAndHoldIntervalChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(mousePressAndHoldInterval)))
	}

}

func (ptr *QStyleHints) ConnectMousePressAndHoldIntervalChanged(f func(mousePressAndHoldInterval int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "mousePressAndHoldIntervalChanged") {
			C.QStyleHints_ConnectMousePressAndHoldIntervalChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "mousePressAndHoldIntervalChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "mousePressAndHoldIntervalChanged"); signal != nil {
			f := func(mousePressAndHoldInterval int) {
				(*(*func(int))(signal))(mousePressAndHoldInterval)
				f(mousePressAndHoldInterval)
			}
			qt.ConnectSignal(ptr.Pointer(), "mousePressAndHoldIntervalChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "mousePressAndHoldIntervalChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectMousePressAndHoldIntervalChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectMousePressAndHoldIntervalChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "mousePressAndHoldIntervalChanged")
	}
}

func (ptr *QStyleHints) MousePressAndHoldIntervalChanged(mousePressAndHoldInterval int) {
	if ptr.Pointer() != nil {
		C.QStyleHints_MousePressAndHoldIntervalChanged(ptr.Pointer(), C.int(int32(mousePressAndHoldInterval)))
	}
}

func (ptr *QStyleHints) MouseQuickSelectionThreshold() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_MouseQuickSelectionThreshold(ptr.Pointer())))
	}
	return 0
}

//export callbackQStyleHints_MouseQuickSelectionThresholdChanged
func callbackQStyleHints_MouseQuickSelectionThresholdChanged(ptr unsafe.Pointer, threshold C.int) {
	if signal := qt.GetSignal(ptr, "mouseQuickSelectionThresholdChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(threshold)))
	}

}

func (ptr *QStyleHints) ConnectMouseQuickSelectionThresholdChanged(f func(threshold int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "mouseQuickSelectionThresholdChanged") {
			C.QStyleHints_ConnectMouseQuickSelectionThresholdChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "mouseQuickSelectionThresholdChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "mouseQuickSelectionThresholdChanged"); signal != nil {
			f := func(threshold int) {
				(*(*func(int))(signal))(threshold)
				f(threshold)
			}
			qt.ConnectSignal(ptr.Pointer(), "mouseQuickSelectionThresholdChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "mouseQuickSelectionThresholdChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectMouseQuickSelectionThresholdChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectMouseQuickSelectionThresholdChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "mouseQuickSelectionThresholdChanged")
	}
}

func (ptr *QStyleHints) MouseQuickSelectionThresholdChanged(threshold int) {
	if ptr.Pointer() != nil {
		C.QStyleHints_MouseQuickSelectionThresholdChanged(ptr.Pointer(), C.int(int32(threshold)))
	}
}

func (ptr *QStyleHints) PasswordMaskCharacter() *core.QChar {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQCharFromPointer(C.QStyleHints_PasswordMaskCharacter(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QChar).DestroyQChar)
		return tmpValue
	}
	return nil
}

func (ptr *QStyleHints) PasswordMaskDelay() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_PasswordMaskDelay(ptr.Pointer())))
	}
	return 0
}

func (ptr *QStyleHints) SetFocusOnTouchRelease() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStyleHints_SetFocusOnTouchRelease(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStyleHints) SetShowShortcutsInContextMenus(showShortcutsInContextMenus bool) {
	if ptr.Pointer() != nil {
		C.QStyleHints_SetShowShortcutsInContextMenus(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(showShortcutsInContextMenus))))
	}
}

func (ptr *QStyleHints) SetUseHoverEffects(useHoverEffects bool) {
	if ptr.Pointer() != nil {
		C.QStyleHints_SetUseHoverEffects(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(useHoverEffects))))
	}
}

func (ptr *QStyleHints) ShowIsFullScreen() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStyleHints_ShowIsFullScreen(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStyleHints) ShowIsMaximized() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStyleHints_ShowIsMaximized(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStyleHints) ShowShortcutsInContextMenus() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStyleHints_ShowShortcutsInContextMenus(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQStyleHints_ShowShortcutsInContextMenusChanged
func callbackQStyleHints_ShowShortcutsInContextMenusChanged(ptr unsafe.Pointer, vbo C.char) {
	if signal := qt.GetSignal(ptr, "showShortcutsInContextMenusChanged"); signal != nil {
		(*(*func(bool))(signal))(int8(vbo) != 0)
	}

}

func (ptr *QStyleHints) ConnectShowShortcutsInContextMenusChanged(f func(vbo bool)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "showShortcutsInContextMenusChanged") {
			C.QStyleHints_ConnectShowShortcutsInContextMenusChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "showShortcutsInContextMenusChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "showShortcutsInContextMenusChanged"); signal != nil {
			f := func(vbo bool) {
				(*(*func(bool))(signal))(vbo)
				f(vbo)
			}
			qt.ConnectSignal(ptr.Pointer(), "showShortcutsInContextMenusChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "showShortcutsInContextMenusChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectShowShortcutsInContextMenusChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectShowShortcutsInContextMenusChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "showShortcutsInContextMenusChanged")
	}
}

func (ptr *QStyleHints) ShowShortcutsInContextMenusChanged(vbo bool) {
	if ptr.Pointer() != nil {
		C.QStyleHints_ShowShortcutsInContextMenusChanged(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(vbo))))
	}
}

func (ptr *QStyleHints) SingleClickActivation() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStyleHints_SingleClickActivation(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStyleHints) StartDragDistance() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_StartDragDistance(ptr.Pointer())))
	}
	return 0
}

//export callbackQStyleHints_StartDragDistanceChanged
func callbackQStyleHints_StartDragDistanceChanged(ptr unsafe.Pointer, startDragDistance C.int) {
	if signal := qt.GetSignal(ptr, "startDragDistanceChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(startDragDistance)))
	}

}

func (ptr *QStyleHints) ConnectStartDragDistanceChanged(f func(startDragDistance int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "startDragDistanceChanged") {
			C.QStyleHints_ConnectStartDragDistanceChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "startDragDistanceChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "startDragDistanceChanged"); signal != nil {
			f := func(startDragDistance int) {
				(*(*func(int))(signal))(startDragDistance)
				f(startDragDistance)
			}
			qt.ConnectSignal(ptr.Pointer(), "startDragDistanceChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "startDragDistanceChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectStartDragDistanceChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectStartDragDistanceChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "startDragDistanceChanged")
	}
}

func (ptr *QStyleHints) StartDragDistanceChanged(startDragDistance int) {
	if ptr.Pointer() != nil {
		C.QStyleHints_StartDragDistanceChanged(ptr.Pointer(), C.int(int32(startDragDistance)))
	}
}

func (ptr *QStyleHints) StartDragTime() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_StartDragTime(ptr.Pointer())))
	}
	return 0
}

//export callbackQStyleHints_StartDragTimeChanged
func callbackQStyleHints_StartDragTimeChanged(ptr unsafe.Pointer, startDragTime C.int) {
	if signal := qt.GetSignal(ptr, "startDragTimeChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(startDragTime)))
	}

}

func (ptr *QStyleHints) ConnectStartDragTimeChanged(f func(startDragTime int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "startDragTimeChanged") {
			C.QStyleHints_ConnectStartDragTimeChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "startDragTimeChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "startDragTimeChanged"); signal != nil {
			f := func(startDragTime int) {
				(*(*func(int))(signal))(startDragTime)
				f(startDragTime)
			}
			qt.ConnectSignal(ptr.Pointer(), "startDragTimeChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "startDragTimeChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectStartDragTimeChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectStartDragTimeChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "startDragTimeChanged")
	}
}

func (ptr *QStyleHints) StartDragTimeChanged(startDragTime int) {
	if ptr.Pointer() != nil {
		C.QStyleHints_StartDragTimeChanged(ptr.Pointer(), C.int(int32(startDragTime)))
	}
}

func (ptr *QStyleHints) StartDragVelocity() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_StartDragVelocity(ptr.Pointer())))
	}
	return 0
}

func (ptr *QStyleHints) TabFocusBehavior() core.Qt__TabFocusBehavior {
	if ptr.Pointer() != nil {
		return core.Qt__TabFocusBehavior(C.QStyleHints_TabFocusBehavior(ptr.Pointer()))
	}
	return 0
}

//export callbackQStyleHints_TabFocusBehaviorChanged
func callbackQStyleHints_TabFocusBehaviorChanged(ptr unsafe.Pointer, tabFocusBehavior C.longlong) {
	if signal := qt.GetSignal(ptr, "tabFocusBehaviorChanged"); signal != nil {
		(*(*func(core.Qt__TabFocusBehavior))(signal))(core.Qt__TabFocusBehavior(tabFocusBehavior))
	}

}

func (ptr *QStyleHints) ConnectTabFocusBehaviorChanged(f func(tabFocusBehavior core.Qt__TabFocusBehavior)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "tabFocusBehaviorChanged") {
			C.QStyleHints_ConnectTabFocusBehaviorChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "tabFocusBehaviorChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "tabFocusBehaviorChanged"); signal != nil {
			f := func(tabFocusBehavior core.Qt__TabFocusBehavior) {
				(*(*func(core.Qt__TabFocusBehavior))(signal))(tabFocusBehavior)
				f(tabFocusBehavior)
			}
			qt.ConnectSignal(ptr.Pointer(), "tabFocusBehaviorChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "tabFocusBehaviorChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectTabFocusBehaviorChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectTabFocusBehaviorChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "tabFocusBehaviorChanged")
	}
}

func (ptr *QStyleHints) TabFocusBehaviorChanged(tabFocusBehavior core.Qt__TabFocusBehavior) {
	if ptr.Pointer() != nil {
		C.QStyleHints_TabFocusBehaviorChanged(ptr.Pointer(), C.longlong(tabFocusBehavior))
	}
}

func (ptr *QStyleHints) UseHoverEffects() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStyleHints_UseHoverEffects(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQStyleHints_UseHoverEffectsChanged
func callbackQStyleHints_UseHoverEffectsChanged(ptr unsafe.Pointer, useHoverEffects C.char) {
	if signal := qt.GetSignal(ptr, "useHoverEffectsChanged"); signal != nil {
		(*(*func(bool))(signal))(int8(useHoverEffects) != 0)
	}

}

func (ptr *QStyleHints) ConnectUseHoverEffectsChanged(f func(useHoverEffects bool)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "useHoverEffectsChanged") {
			C.QStyleHints_ConnectUseHoverEffectsChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "useHoverEffectsChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "useHoverEffectsChanged"); signal != nil {
			f := func(useHoverEffects bool) {
				(*(*func(bool))(signal))(useHoverEffects)
				f(useHoverEffects)
			}
			qt.ConnectSignal(ptr.Pointer(), "useHoverEffectsChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "useHoverEffectsChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectUseHoverEffectsChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectUseHoverEffectsChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "useHoverEffectsChanged")
	}
}

func (ptr *QStyleHints) UseHoverEffectsChanged(useHoverEffects bool) {
	if ptr.Pointer() != nil {
		C.QStyleHints_UseHoverEffectsChanged(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(useHoverEffects))))
	}
}

func (ptr *QStyleHints) UseRtlExtensions() bool {
	if ptr.Pointer() != nil {
		return int8(C.QStyleHints_UseRtlExtensions(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QStyleHints) WheelScrollLines() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QStyleHints_WheelScrollLines(ptr.Pointer())))
	}
	return 0
}

//export callbackQStyleHints_WheelScrollLinesChanged
func callbackQStyleHints_WheelScrollLinesChanged(ptr unsafe.Pointer, scrollLines C.int) {
	if signal := qt.GetSignal(ptr, "wheelScrollLinesChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(scrollLines)))
	}

}

func (ptr *QStyleHints) ConnectWheelScrollLinesChanged(f func(scrollLines int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "wheelScrollLinesChanged") {
			C.QStyleHints_ConnectWheelScrollLinesChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "wheelScrollLinesChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "wheelScrollLinesChanged"); signal != nil {
			f := func(scrollLines int) {
				(*(*func(int))(signal))(scrollLines)
				f(scrollLines)
			}
			qt.ConnectSignal(ptr.Pointer(), "wheelScrollLinesChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "wheelScrollLinesChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QStyleHints) DisconnectWheelScrollLinesChanged() {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectWheelScrollLinesChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "wheelScrollLinesChanged")
	}
}

func (ptr *QStyleHints) WheelScrollLinesChanged(scrollLines int) {
	if ptr.Pointer() != nil {
		C.QStyleHints_WheelScrollLinesChanged(ptr.Pointer(), C.int(int32(scrollLines)))
	}
}

func (ptr *QStyleHints) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QStyleHints___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QStyleHints) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QStyleHints___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QStyleHints) __children_newList() unsafe.Pointer {
	return C.QStyleHints___children_newList(ptr.Pointer())
}

func (ptr *QStyleHints) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QStyleHints___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QStyleHints) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QStyleHints___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QStyleHints) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QStyleHints___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QStyleHints) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QStyleHints___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QStyleHints) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QStyleHints___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QStyleHints) __findChildren_newList() unsafe.Pointer {
	return C.QStyleHints___findChildren_newList(ptr.Pointer())
}

func (ptr *QStyleHints) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QStyleHints___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QStyleHints) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QStyleHints___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QStyleHints) __findChildren_newList3() unsafe.Pointer {
	return C.QStyleHints___findChildren_newList3(ptr.Pointer())
}

//export callbackQStyleHints_ChildEvent
func callbackQStyleHints_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQStyleHintsFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QStyleHints) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QStyleHints_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQStyleHints_ConnectNotify
func callbackQStyleHints_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQStyleHintsFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QStyleHints) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QStyleHints_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQStyleHints_CustomEvent
func callbackQStyleHints_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQStyleHintsFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QStyleHints) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QStyleHints_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQStyleHints_DeleteLater
func callbackQStyleHints_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQStyleHintsFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QStyleHints) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QStyleHints_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQStyleHints_Destroyed
func callbackQStyleHints_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQStyleHints_DisconnectNotify
func callbackQStyleHints_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQStyleHintsFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QStyleHints) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QStyleHints_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQStyleHints_Event
func callbackQStyleHints_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStyleHintsFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QStyleHints) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStyleHints_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQStyleHints_EventFilter
func callbackQStyleHints_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQStyleHintsFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QStyleHints) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QStyleHints_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQStyleHints_MetaObject
func callbackQStyleHints_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQStyleHintsFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QStyleHints) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QStyleHints_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQStyleHints_ObjectNameChanged
func callbackQStyleHints_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQStyleHints_TimerEvent
func callbackQStyleHints_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQStyleHintsFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QStyleHints) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QStyleHints_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QSupportedWritingSystems struct {
	ptr unsafe.Pointer
}

type QSupportedWritingSystems_ITF interface {
	QSupportedWritingSystems_PTR() *QSupportedWritingSystems
}

func (ptr *QSupportedWritingSystems) QSupportedWritingSystems_PTR() *QSupportedWritingSystems {
	return ptr
}

func (ptr *QSupportedWritingSystems) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QSupportedWritingSystems) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQSupportedWritingSystems(ptr QSupportedWritingSystems_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QSupportedWritingSystems_PTR().Pointer()
	}
	return nil
}

func NewQSupportedWritingSystemsFromPointer(ptr unsafe.Pointer) (n *QSupportedWritingSystems) {
	n = new(QSupportedWritingSystems)
	n.SetPointer(ptr)
	return
}

type QSurface struct {
	ptr unsafe.Pointer
}

type QSurface_ITF interface {
	QSurface_PTR() *QSurface
}

func (ptr *QSurface) QSurface_PTR() *QSurface {
	return ptr
}

func (ptr *QSurface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QSurface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQSurface(ptr QSurface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QSurface_PTR().Pointer()
	}
	return nil
}

func NewQSurfaceFromPointer(ptr unsafe.Pointer) (n *QSurface) {
	n = new(QSurface)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QSurface__SurfaceClass
//QSurface::SurfaceClass
type QSurface__SurfaceClass int64

const (
	QSurface__Window    QSurface__SurfaceClass = QSurface__SurfaceClass(0)
	QSurface__Offscreen QSurface__SurfaceClass = QSurface__SurfaceClass(1)
)

//go:generate stringer -type=QSurface__SurfaceType
//QSurface::SurfaceType
type QSurface__SurfaceType int64

const (
	QSurface__RasterSurface   QSurface__SurfaceType = QSurface__SurfaceType(0)
	QSurface__OpenGLSurface   QSurface__SurfaceType = QSurface__SurfaceType(1)
	QSurface__RasterGLSurface QSurface__SurfaceType = QSurface__SurfaceType(2)
	QSurface__OpenVGSurface   QSurface__SurfaceType = QSurface__SurfaceType(3)
	QSurface__VulkanSurface   QSurface__SurfaceType = QSurface__SurfaceType(4)
	QSurface__MetalSurface    QSurface__SurfaceType = QSurface__SurfaceType(5)
)

//export callbackQSurface_Format
func callbackQSurface_Format(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "format"); signal != nil {
		return PointerFromQSurfaceFormat((*(*func() *QSurfaceFormat)(signal))())
	}

	return PointerFromQSurfaceFormat(NewQSurfaceFormat())
}

func (ptr *QSurface) ConnectFormat(f func() *QSurfaceFormat) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "format"); signal != nil {
			f := func() *QSurfaceFormat {
				(*(*func() *QSurfaceFormat)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "format", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "format", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QSurface) DisconnectFormat() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "format")
	}
}

func (ptr *QSurface) Format() *QSurfaceFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQSurfaceFormatFromPointer(C.QSurface_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
		return tmpValue
	}
	return nil
}

//export callbackQSurface_Size
func callbackQSurface_Size(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "size"); signal != nil {
		return core.PointerFromQSize((*(*func() *core.QSize)(signal))())
	}

	return core.PointerFromQSize(core.NewQSize())
}

func (ptr *QSurface) ConnectSize(f func() *core.QSize) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "size"); signal != nil {
			f := func() *core.QSize {
				(*(*func() *core.QSize)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "size", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "size", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QSurface) DisconnectSize() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "size")
	}
}

func (ptr *QSurface) Size() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QSurface_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QSurface) SupportsOpenGL() bool {
	if ptr.Pointer() != nil {
		return int8(C.QSurface_SupportsOpenGL(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QSurface) SurfaceClass() QSurface__SurfaceClass {
	if ptr.Pointer() != nil {
		return QSurface__SurfaceClass(C.QSurface_SurfaceClass(ptr.Pointer()))
	}
	return 0
}

//export callbackQSurface_SurfaceType
func callbackQSurface_SurfaceType(ptr unsafe.Pointer) C.longlong {
	if signal := qt.GetSignal(ptr, "surfaceType"); signal != nil {
		return C.longlong((*(*func() QSurface__SurfaceType)(signal))())
	}

	return C.longlong(0)
}

func (ptr *QSurface) ConnectSurfaceType(f func() QSurface__SurfaceType) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "surfaceType"); signal != nil {
			f := func() QSurface__SurfaceType {
				(*(*func() QSurface__SurfaceType)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "surfaceType", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "surfaceType", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QSurface) DisconnectSurfaceType() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "surfaceType")
	}
}

func (ptr *QSurface) SurfaceType() QSurface__SurfaceType {
	if ptr.Pointer() != nil {
		return QSurface__SurfaceType(C.QSurface_SurfaceType(ptr.Pointer()))
	}
	return 0
}

//export callbackQSurface_DestroyQSurface
func callbackQSurface_DestroyQSurface(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QSurface"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQSurfaceFromPointer(ptr).DestroyQSurfaceDefault()
	}
}

func (ptr *QSurface) ConnectDestroyQSurface(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QSurface"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QSurface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QSurface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QSurface) DisconnectDestroyQSurface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QSurface")
	}
}

func (ptr *QSurface) DestroyQSurface() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QSurface_DestroyQSurface(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QSurface) DestroyQSurfaceDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QSurface_DestroyQSurfaceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QSurfaceFormat struct {
	ptr unsafe.Pointer
}

type QSurfaceFormat_ITF interface {
	QSurfaceFormat_PTR() *QSurfaceFormat
}

func (ptr *QSurfaceFormat) QSurfaceFormat_PTR() *QSurfaceFormat {
	return ptr
}

func (ptr *QSurfaceFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QSurfaceFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQSurfaceFormat(ptr QSurfaceFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QSurfaceFormat_PTR().Pointer()
	}
	return nil
}

func NewQSurfaceFormatFromPointer(ptr unsafe.Pointer) (n *QSurfaceFormat) {
	n = new(QSurfaceFormat)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QSurfaceFormat__FormatOption
//QSurfaceFormat::FormatOption
type QSurfaceFormat__FormatOption int64

const (
	QSurfaceFormat__StereoBuffers       QSurfaceFormat__FormatOption = QSurfaceFormat__FormatOption(0x0001)
	QSurfaceFormat__DebugContext        QSurfaceFormat__FormatOption = QSurfaceFormat__FormatOption(0x0002)
	QSurfaceFormat__DeprecatedFunctions QSurfaceFormat__FormatOption = QSurfaceFormat__FormatOption(0x0004)
	QSurfaceFormat__ResetNotification   QSurfaceFormat__FormatOption = QSurfaceFormat__FormatOption(0x0008)
)

//go:generate stringer -type=QSurfaceFormat__SwapBehavior
//QSurfaceFormat::SwapBehavior
type QSurfaceFormat__SwapBehavior int64

const (
	QSurfaceFormat__DefaultSwapBehavior QSurfaceFormat__SwapBehavior = QSurfaceFormat__SwapBehavior(0)
	QSurfaceFormat__SingleBuffer        QSurfaceFormat__SwapBehavior = QSurfaceFormat__SwapBehavior(1)
	QSurfaceFormat__DoubleBuffer        QSurfaceFormat__SwapBehavior = QSurfaceFormat__SwapBehavior(2)
	QSurfaceFormat__TripleBuffer        QSurfaceFormat__SwapBehavior = QSurfaceFormat__SwapBehavior(3)
)

//go:generate stringer -type=QSurfaceFormat__RenderableType
//QSurfaceFormat::RenderableType
type QSurfaceFormat__RenderableType int64

const (
	QSurfaceFormat__DefaultRenderableType QSurfaceFormat__RenderableType = QSurfaceFormat__RenderableType(0x0)
	QSurfaceFormat__OpenGL                QSurfaceFormat__RenderableType = QSurfaceFormat__RenderableType(0x1)
	QSurfaceFormat__OpenGLES              QSurfaceFormat__RenderableType = QSurfaceFormat__RenderableType(0x2)
	QSurfaceFormat__OpenVG                QSurfaceFormat__RenderableType = QSurfaceFormat__RenderableType(0x4)
)

//go:generate stringer -type=QSurfaceFormat__OpenGLContextProfile
//QSurfaceFormat::OpenGLContextProfile
type QSurfaceFormat__OpenGLContextProfile int64

const (
	QSurfaceFormat__NoProfile            QSurfaceFormat__OpenGLContextProfile = QSurfaceFormat__OpenGLContextProfile(0)
	QSurfaceFormat__CoreProfile          QSurfaceFormat__OpenGLContextProfile = QSurfaceFormat__OpenGLContextProfile(1)
	QSurfaceFormat__CompatibilityProfile QSurfaceFormat__OpenGLContextProfile = QSurfaceFormat__OpenGLContextProfile(2)
)

//go:generate stringer -type=QSurfaceFormat__ColorSpace
//QSurfaceFormat::ColorSpace
type QSurfaceFormat__ColorSpace int64

const (
	QSurfaceFormat__DefaultColorSpace QSurfaceFormat__ColorSpace = QSurfaceFormat__ColorSpace(0)
	QSurfaceFormat__sRGBColorSpace    QSurfaceFormat__ColorSpace = QSurfaceFormat__ColorSpace(1)
)

func NewQSurfaceFormat() *QSurfaceFormat {
	tmpValue := NewQSurfaceFormatFromPointer(C.QSurfaceFormat_NewQSurfaceFormat())
	qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
	return tmpValue
}

func NewQSurfaceFormat2(options QSurfaceFormat__FormatOption) *QSurfaceFormat {
	tmpValue := NewQSurfaceFormatFromPointer(C.QSurfaceFormat_NewQSurfaceFormat2(C.longlong(options)))
	qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
	return tmpValue
}

func NewQSurfaceFormat3(other QSurfaceFormat_ITF) *QSurfaceFormat {
	tmpValue := NewQSurfaceFormatFromPointer(C.QSurfaceFormat_NewQSurfaceFormat3(PointerFromQSurfaceFormat(other)))
	qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
	return tmpValue
}

func (ptr *QSurfaceFormat) AlphaBufferSize() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSurfaceFormat_AlphaBufferSize(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSurfaceFormat) BlueBufferSize() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSurfaceFormat_BlueBufferSize(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSurfaceFormat) ColorSpace() QSurfaceFormat__ColorSpace {
	if ptr.Pointer() != nil {
		return QSurfaceFormat__ColorSpace(C.QSurfaceFormat_ColorSpace(ptr.Pointer()))
	}
	return 0
}

func QSurfaceFormat_DefaultFormat() *QSurfaceFormat {
	tmpValue := NewQSurfaceFormatFromPointer(C.QSurfaceFormat_QSurfaceFormat_DefaultFormat())
	qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
	return tmpValue
}

func (ptr *QSurfaceFormat) DefaultFormat() *QSurfaceFormat {
	tmpValue := NewQSurfaceFormatFromPointer(C.QSurfaceFormat_QSurfaceFormat_DefaultFormat())
	qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
	return tmpValue
}

func (ptr *QSurfaceFormat) DepthBufferSize() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSurfaceFormat_DepthBufferSize(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSurfaceFormat) GreenBufferSize() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSurfaceFormat_GreenBufferSize(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSurfaceFormat) HasAlpha() bool {
	if ptr.Pointer() != nil {
		return int8(C.QSurfaceFormat_HasAlpha(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QSurfaceFormat) MajorVersion() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSurfaceFormat_MajorVersion(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSurfaceFormat) MinorVersion() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSurfaceFormat_MinorVersion(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSurfaceFormat) Options() QSurfaceFormat__FormatOption {
	if ptr.Pointer() != nil {
		return QSurfaceFormat__FormatOption(C.QSurfaceFormat_Options(ptr.Pointer()))
	}
	return 0
}

func (ptr *QSurfaceFormat) Profile() QSurfaceFormat__OpenGLContextProfile {
	if ptr.Pointer() != nil {
		return QSurfaceFormat__OpenGLContextProfile(C.QSurfaceFormat_Profile(ptr.Pointer()))
	}
	return 0
}

func (ptr *QSurfaceFormat) RedBufferSize() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSurfaceFormat_RedBufferSize(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSurfaceFormat) RenderableType() QSurfaceFormat__RenderableType {
	if ptr.Pointer() != nil {
		return QSurfaceFormat__RenderableType(C.QSurfaceFormat_RenderableType(ptr.Pointer()))
	}
	return 0
}

func (ptr *QSurfaceFormat) Samples() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSurfaceFormat_Samples(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSurfaceFormat) SetAlphaBufferSize(size int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetAlphaBufferSize(ptr.Pointer(), C.int(int32(size)))
	}
}

func (ptr *QSurfaceFormat) SetBlueBufferSize(size int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetBlueBufferSize(ptr.Pointer(), C.int(int32(size)))
	}
}

func (ptr *QSurfaceFormat) SetColorSpace(colorSpace QSurfaceFormat__ColorSpace) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetColorSpace(ptr.Pointer(), C.longlong(colorSpace))
	}
}

func QSurfaceFormat_SetDefaultFormat(format QSurfaceFormat_ITF) {
	C.QSurfaceFormat_QSurfaceFormat_SetDefaultFormat(PointerFromQSurfaceFormat(format))
}

func (ptr *QSurfaceFormat) SetDefaultFormat(format QSurfaceFormat_ITF) {
	C.QSurfaceFormat_QSurfaceFormat_SetDefaultFormat(PointerFromQSurfaceFormat(format))
}

func (ptr *QSurfaceFormat) SetDepthBufferSize(size int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetDepthBufferSize(ptr.Pointer(), C.int(int32(size)))
	}
}

func (ptr *QSurfaceFormat) SetGreenBufferSize(size int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetGreenBufferSize(ptr.Pointer(), C.int(int32(size)))
	}
}

func (ptr *QSurfaceFormat) SetMajorVersion(major int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetMajorVersion(ptr.Pointer(), C.int(int32(major)))
	}
}

func (ptr *QSurfaceFormat) SetMinorVersion(minor int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetMinorVersion(ptr.Pointer(), C.int(int32(minor)))
	}
}

func (ptr *QSurfaceFormat) SetOption(option QSurfaceFormat__FormatOption, on bool) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetOption(ptr.Pointer(), C.longlong(option), C.char(int8(qt.GoBoolToInt(on))))
	}
}

func (ptr *QSurfaceFormat) SetOptions(options QSurfaceFormat__FormatOption) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetOptions(ptr.Pointer(), C.longlong(options))
	}
}

func (ptr *QSurfaceFormat) SetProfile(profile QSurfaceFormat__OpenGLContextProfile) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetProfile(ptr.Pointer(), C.longlong(profile))
	}
}

func (ptr *QSurfaceFormat) SetRedBufferSize(size int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetRedBufferSize(ptr.Pointer(), C.int(int32(size)))
	}
}

func (ptr *QSurfaceFormat) SetRenderableType(ty QSurfaceFormat__RenderableType) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetRenderableType(ptr.Pointer(), C.longlong(ty))
	}
}

func (ptr *QSurfaceFormat) SetSamples(numSamples int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetSamples(ptr.Pointer(), C.int(int32(numSamples)))
	}
}

func (ptr *QSurfaceFormat) SetStencilBufferSize(size int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetStencilBufferSize(ptr.Pointer(), C.int(int32(size)))
	}
}

func (ptr *QSurfaceFormat) SetStereo(enable bool) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetStereo(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QSurfaceFormat) SetSwapBehavior(behavior QSurfaceFormat__SwapBehavior) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetSwapBehavior(ptr.Pointer(), C.longlong(behavior))
	}
}

func (ptr *QSurfaceFormat) SetSwapInterval(interval int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetSwapInterval(ptr.Pointer(), C.int(int32(interval)))
	}
}

func (ptr *QSurfaceFormat) SetVersion(major int, minor int) {
	if ptr.Pointer() != nil {
		C.QSurfaceFormat_SetVersion(ptr.Pointer(), C.int(int32(major)), C.int(int32(minor)))
	}
}

func (ptr *QSurfaceFormat) StencilBufferSize() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSurfaceFormat_StencilBufferSize(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSurfaceFormat) Stereo() bool {
	if ptr.Pointer() != nil {
		return int8(C.QSurfaceFormat_Stereo(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QSurfaceFormat) SwapBehavior() QSurfaceFormat__SwapBehavior {
	if ptr.Pointer() != nil {
		return QSurfaceFormat__SwapBehavior(C.QSurfaceFormat_SwapBehavior(ptr.Pointer()))
	}
	return 0
}

func (ptr *QSurfaceFormat) SwapInterval() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSurfaceFormat_SwapInterval(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSurfaceFormat) TestOption(option QSurfaceFormat__FormatOption) bool {
	if ptr.Pointer() != nil {
		return int8(C.QSurfaceFormat_TestOption(ptr.Pointer(), C.longlong(option))) != 0
	}
	return false
}

func (ptr *QSurfaceFormat) DestroyQSurfaceFormat() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QSurfaceFormat_DestroyQSurfaceFormat(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QSyntaxHighlighter struct {
	core.QObject
}

type QSyntaxHighlighter_ITF interface {
	core.QObject_ITF
	QSyntaxHighlighter_PTR() *QSyntaxHighlighter
}

func (ptr *QSyntaxHighlighter) QSyntaxHighlighter_PTR() *QSyntaxHighlighter {
	return ptr
}

func (ptr *QSyntaxHighlighter) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QSyntaxHighlighter) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQSyntaxHighlighter(ptr QSyntaxHighlighter_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QSyntaxHighlighter_PTR().Pointer()
	}
	return nil
}

func NewQSyntaxHighlighterFromPointer(ptr unsafe.Pointer) (n *QSyntaxHighlighter) {
	n = new(QSyntaxHighlighter)
	n.SetPointer(ptr)
	return
}
func NewQSyntaxHighlighter(parent core.QObject_ITF) *QSyntaxHighlighter {
	tmpValue := NewQSyntaxHighlighterFromPointer(C.QSyntaxHighlighter_NewQSyntaxHighlighter(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQSyntaxHighlighter2(parent QTextDocument_ITF) *QSyntaxHighlighter {
	tmpValue := NewQSyntaxHighlighterFromPointer(C.QSyntaxHighlighter_NewQSyntaxHighlighter2(PointerFromQTextDocument(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QSyntaxHighlighter) CurrentBlock() *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QSyntaxHighlighter_CurrentBlock(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QSyntaxHighlighter) CurrentBlockState() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSyntaxHighlighter_CurrentBlockState(ptr.Pointer())))
	}
	return 0
}

func (ptr *QSyntaxHighlighter) CurrentBlockUserData() *QTextBlockUserData {
	if ptr.Pointer() != nil {
		return NewQTextBlockUserDataFromPointer(C.QSyntaxHighlighter_CurrentBlockUserData(ptr.Pointer()))
	}
	return nil
}

func (ptr *QSyntaxHighlighter) Document() *QTextDocument {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextDocumentFromPointer(C.QSyntaxHighlighter_Document(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QSyntaxHighlighter) Format(position int) *QTextCharFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCharFormatFromPointer(C.QSyntaxHighlighter_Format(ptr.Pointer(), C.int(int32(position))))
		qt.SetFinalizer(tmpValue, (*QTextCharFormat).DestroyQTextCharFormat)
		return tmpValue
	}
	return nil
}

//export callbackQSyntaxHighlighter_HighlightBlock
func callbackQSyntaxHighlighter_HighlightBlock(ptr unsafe.Pointer, text C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "highlightBlock"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(text))
	}

}

func (ptr *QSyntaxHighlighter) ConnectHighlightBlock(f func(text string)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "highlightBlock"); signal != nil {
			f := func(text string) {
				(*(*func(string))(signal))(text)
				f(text)
			}
			qt.ConnectSignal(ptr.Pointer(), "highlightBlock", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "highlightBlock", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QSyntaxHighlighter) DisconnectHighlightBlock() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "highlightBlock")
	}
}

func (ptr *QSyntaxHighlighter) HighlightBlock(text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QSyntaxHighlighter_HighlightBlock(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QSyntaxHighlighter) PreviousBlockState() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QSyntaxHighlighter_PreviousBlockState(ptr.Pointer())))
	}
	return 0
}

//export callbackQSyntaxHighlighter_Rehighlight
func callbackQSyntaxHighlighter_Rehighlight(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "rehighlight"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQSyntaxHighlighterFromPointer(ptr).RehighlightDefault()
	}
}

func (ptr *QSyntaxHighlighter) ConnectRehighlight(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "rehighlight"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "rehighlight", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "rehighlight", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QSyntaxHighlighter) DisconnectRehighlight() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "rehighlight")
	}
}

func (ptr *QSyntaxHighlighter) Rehighlight() {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_Rehighlight(ptr.Pointer())
	}
}

func (ptr *QSyntaxHighlighter) RehighlightDefault() {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_RehighlightDefault(ptr.Pointer())
	}
}

//export callbackQSyntaxHighlighter_RehighlightBlock
func callbackQSyntaxHighlighter_RehighlightBlock(ptr unsafe.Pointer, block unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "rehighlightBlock"); signal != nil {
		(*(*func(*QTextBlock))(signal))(NewQTextBlockFromPointer(block))
	} else {
		NewQSyntaxHighlighterFromPointer(ptr).RehighlightBlockDefault(NewQTextBlockFromPointer(block))
	}
}

func (ptr *QSyntaxHighlighter) ConnectRehighlightBlock(f func(block *QTextBlock)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "rehighlightBlock"); signal != nil {
			f := func(block *QTextBlock) {
				(*(*func(*QTextBlock))(signal))(block)
				f(block)
			}
			qt.ConnectSignal(ptr.Pointer(), "rehighlightBlock", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "rehighlightBlock", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QSyntaxHighlighter) DisconnectRehighlightBlock() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "rehighlightBlock")
	}
}

func (ptr *QSyntaxHighlighter) RehighlightBlock(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_RehighlightBlock(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

func (ptr *QSyntaxHighlighter) RehighlightBlockDefault(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_RehighlightBlockDefault(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

func (ptr *QSyntaxHighlighter) SetCurrentBlockState(newState int) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_SetCurrentBlockState(ptr.Pointer(), C.int(int32(newState)))
	}
}

func (ptr *QSyntaxHighlighter) SetCurrentBlockUserData(data QTextBlockUserData_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_SetCurrentBlockUserData(ptr.Pointer(), PointerFromQTextBlockUserData(data))
	}
}

func (ptr *QSyntaxHighlighter) SetDocument(doc QTextDocument_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_SetDocument(ptr.Pointer(), PointerFromQTextDocument(doc))
	}
}

func (ptr *QSyntaxHighlighter) SetFormat(start int, count int, format QTextCharFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_SetFormat(ptr.Pointer(), C.int(int32(start)), C.int(int32(count)), PointerFromQTextCharFormat(format))
	}
}

func (ptr *QSyntaxHighlighter) SetFormat2(start int, count int, color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_SetFormat2(ptr.Pointer(), C.int(int32(start)), C.int(int32(count)), PointerFromQColor(color))
	}
}

func (ptr *QSyntaxHighlighter) SetFormat3(start int, count int, font QFont_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_SetFormat3(ptr.Pointer(), C.int(int32(start)), C.int(int32(count)), PointerFromQFont(font))
	}
}

//export callbackQSyntaxHighlighter_DestroyQSyntaxHighlighter
func callbackQSyntaxHighlighter_DestroyQSyntaxHighlighter(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QSyntaxHighlighter"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQSyntaxHighlighterFromPointer(ptr).DestroyQSyntaxHighlighterDefault()
	}
}

func (ptr *QSyntaxHighlighter) ConnectDestroyQSyntaxHighlighter(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QSyntaxHighlighter"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QSyntaxHighlighter", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QSyntaxHighlighter", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QSyntaxHighlighter) DisconnectDestroyQSyntaxHighlighter() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QSyntaxHighlighter")
	}
}

func (ptr *QSyntaxHighlighter) DestroyQSyntaxHighlighter() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QSyntaxHighlighter_DestroyQSyntaxHighlighter(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QSyntaxHighlighter) DestroyQSyntaxHighlighterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QSyntaxHighlighter_DestroyQSyntaxHighlighterDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QSyntaxHighlighter) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QSyntaxHighlighter___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QSyntaxHighlighter) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QSyntaxHighlighter) __children_newList() unsafe.Pointer {
	return C.QSyntaxHighlighter___children_newList(ptr.Pointer())
}

func (ptr *QSyntaxHighlighter) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QSyntaxHighlighter___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QSyntaxHighlighter) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QSyntaxHighlighter) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QSyntaxHighlighter___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QSyntaxHighlighter) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QSyntaxHighlighter___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QSyntaxHighlighter) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QSyntaxHighlighter) __findChildren_newList() unsafe.Pointer {
	return C.QSyntaxHighlighter___findChildren_newList(ptr.Pointer())
}

func (ptr *QSyntaxHighlighter) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QSyntaxHighlighter___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QSyntaxHighlighter) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QSyntaxHighlighter) __findChildren_newList3() unsafe.Pointer {
	return C.QSyntaxHighlighter___findChildren_newList3(ptr.Pointer())
}

//export callbackQSyntaxHighlighter_ChildEvent
func callbackQSyntaxHighlighter_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQSyntaxHighlighterFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QSyntaxHighlighter) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQSyntaxHighlighter_ConnectNotify
func callbackQSyntaxHighlighter_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQSyntaxHighlighterFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QSyntaxHighlighter) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQSyntaxHighlighter_CustomEvent
func callbackQSyntaxHighlighter_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQSyntaxHighlighterFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QSyntaxHighlighter) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQSyntaxHighlighter_DeleteLater
func callbackQSyntaxHighlighter_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQSyntaxHighlighterFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QSyntaxHighlighter) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QSyntaxHighlighter_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQSyntaxHighlighter_Destroyed
func callbackQSyntaxHighlighter_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQSyntaxHighlighter_DisconnectNotify
func callbackQSyntaxHighlighter_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQSyntaxHighlighterFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QSyntaxHighlighter) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQSyntaxHighlighter_Event
func callbackQSyntaxHighlighter_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQSyntaxHighlighterFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QSyntaxHighlighter) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QSyntaxHighlighter_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQSyntaxHighlighter_EventFilter
func callbackQSyntaxHighlighter_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQSyntaxHighlighterFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QSyntaxHighlighter) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QSyntaxHighlighter_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQSyntaxHighlighter_MetaObject
func callbackQSyntaxHighlighter_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQSyntaxHighlighterFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QSyntaxHighlighter) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QSyntaxHighlighter_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQSyntaxHighlighter_ObjectNameChanged
func callbackQSyntaxHighlighter_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQSyntaxHighlighter_TimerEvent
func callbackQSyntaxHighlighter_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQSyntaxHighlighterFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QSyntaxHighlighter) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QSyntaxHighlighter_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QTabletEvent struct {
	QInputEvent
}

type QTabletEvent_ITF interface {
	QInputEvent_ITF
	QTabletEvent_PTR() *QTabletEvent
}

func (ptr *QTabletEvent) QTabletEvent_PTR() *QTabletEvent {
	return ptr
}

func (ptr *QTabletEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QTabletEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QInputEvent_PTR().SetPointer(p)
	}
}

func PointerFromQTabletEvent(ptr QTabletEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTabletEvent_PTR().Pointer()
	}
	return nil
}

func NewQTabletEventFromPointer(ptr unsafe.Pointer) (n *QTabletEvent) {
	n = new(QTabletEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QTabletEvent) DestroyQTabletEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QTabletEvent__TabletDevice
//QTabletEvent::TabletDevice
type QTabletEvent__TabletDevice int64

const (
	QTabletEvent__NoDevice       QTabletEvent__TabletDevice = QTabletEvent__TabletDevice(0)
	QTabletEvent__Puck           QTabletEvent__TabletDevice = QTabletEvent__TabletDevice(1)
	QTabletEvent__Stylus         QTabletEvent__TabletDevice = QTabletEvent__TabletDevice(2)
	QTabletEvent__Airbrush       QTabletEvent__TabletDevice = QTabletEvent__TabletDevice(3)
	QTabletEvent__FourDMouse     QTabletEvent__TabletDevice = QTabletEvent__TabletDevice(4)
	QTabletEvent__XFreeEraser    QTabletEvent__TabletDevice = QTabletEvent__TabletDevice(5)
	QTabletEvent__RotationStylus QTabletEvent__TabletDevice = QTabletEvent__TabletDevice(6)
)

//go:generate stringer -type=QTabletEvent__PointerType
//QTabletEvent::PointerType
type QTabletEvent__PointerType int64

const (
	QTabletEvent__UnknownPointer QTabletEvent__PointerType = QTabletEvent__PointerType(0)
	QTabletEvent__Pen            QTabletEvent__PointerType = QTabletEvent__PointerType(1)
	QTabletEvent__Cursor         QTabletEvent__PointerType = QTabletEvent__PointerType(2)
	QTabletEvent__Eraser         QTabletEvent__PointerType = QTabletEvent__PointerType(3)
)

func NewQTabletEvent2(ty core.QEvent__Type, pos core.QPointF_ITF, globalPos core.QPointF_ITF, device int, pointerType int, pressure float64, xTilt int, yTilt int, tangentialPressure float64, rotation float64, z int, keyState core.Qt__KeyboardModifier, uniqueID int64, button core.Qt__MouseButton, buttons core.Qt__MouseButton) *QTabletEvent {
	tmpValue := NewQTabletEventFromPointer(C.QTabletEvent_NewQTabletEvent2(C.longlong(ty), core.PointerFromQPointF(pos), core.PointerFromQPointF(globalPos), C.int(int32(device)), C.int(int32(pointerType)), C.double(pressure), C.int(int32(xTilt)), C.int(int32(yTilt)), C.double(tangentialPressure), C.double(rotation), C.int(int32(z)), C.longlong(keyState), C.longlong(uniqueID), C.longlong(button), C.longlong(buttons)))
	qt.SetFinalizer(tmpValue, (*QTabletEvent).DestroyQTabletEvent)
	return tmpValue
}

func (ptr *QTabletEvent) Button() core.Qt__MouseButton {
	if ptr.Pointer() != nil {
		return core.Qt__MouseButton(C.QTabletEvent_Button(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTabletEvent) Buttons() core.Qt__MouseButton {
	if ptr.Pointer() != nil {
		return core.Qt__MouseButton(C.QTabletEvent_Buttons(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTabletEvent) Device() QTabletEvent__TabletDevice {
	if ptr.Pointer() != nil {
		return QTabletEvent__TabletDevice(C.QTabletEvent_Device(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTabletEvent) GlobalPos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QTabletEvent_GlobalPos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QTabletEvent) GlobalPosF() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QTabletEvent_GlobalPosF(ptr.Pointer()))
	}
	return nil
}

func (ptr *QTabletEvent) GlobalX() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTabletEvent_GlobalX(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTabletEvent) GlobalY() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTabletEvent_GlobalY(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTabletEvent) HiResGlobalX() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTabletEvent_HiResGlobalX(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTabletEvent) HiResGlobalY() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTabletEvent_HiResGlobalY(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTabletEvent) PointerType() QTabletEvent__PointerType {
	if ptr.Pointer() != nil {
		return QTabletEvent__PointerType(C.QTabletEvent_PointerType(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTabletEvent) Pos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QTabletEvent_Pos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QTabletEvent) PosF() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QTabletEvent_PosF(ptr.Pointer()))
	}
	return nil
}

func (ptr *QTabletEvent) Pressure() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTabletEvent_Pressure(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTabletEvent) Rotation() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTabletEvent_Rotation(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTabletEvent) TangentialPressure() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTabletEvent_TangentialPressure(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTabletEvent) UniqueId() int64 {
	if ptr.Pointer() != nil {
		return int64(C.QTabletEvent_UniqueId(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTabletEvent) X() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTabletEvent_X(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTabletEvent) XTilt() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTabletEvent_XTilt(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTabletEvent) Y() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTabletEvent_Y(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTabletEvent) YTilt() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTabletEvent_YTilt(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTabletEvent) Z() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTabletEvent_Z(ptr.Pointer())))
	}
	return 0
}

type QTextBlock struct {
	ptr unsafe.Pointer
}

type QTextBlock_ITF interface {
	QTextBlock_PTR() *QTextBlock
}

func (ptr *QTextBlock) QTextBlock_PTR() *QTextBlock {
	return ptr
}

func (ptr *QTextBlock) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextBlock) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextBlock(ptr QTextBlock_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextBlock_PTR().Pointer()
	}
	return nil
}

func NewQTextBlockFromPointer(ptr unsafe.Pointer) (n *QTextBlock) {
	n = new(QTextBlock)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextBlock) DestroyQTextBlock() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQTextBlock3(other QTextBlock_ITF) *QTextBlock {
	tmpValue := NewQTextBlockFromPointer(C.QTextBlock_NewQTextBlock3(PointerFromQTextBlock(other)))
	qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
	return tmpValue
}

func (ptr *QTextBlock) BlockFormat() *QTextBlockFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFormatFromPointer(C.QTextBlock_BlockFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlockFormat).DestroyQTextBlockFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextBlock) BlockFormatIndex() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlock_BlockFormatIndex(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlock) BlockNumber() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlock_BlockNumber(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlock) CharFormat() *QTextCharFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCharFormatFromPointer(C.QTextBlock_CharFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextCharFormat).DestroyQTextCharFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextBlock) CharFormatIndex() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlock_CharFormatIndex(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlock) ClearLayout() {
	if ptr.Pointer() != nil {
		C.QTextBlock_ClearLayout(ptr.Pointer())
	}
}

func (ptr *QTextBlock) Contains(position int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextBlock_Contains(ptr.Pointer(), C.int(int32(position)))) != 0
	}
	return false
}

func (ptr *QTextBlock) Document() *QTextDocument {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextDocumentFromPointer(C.QTextBlock_Document(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextBlock) FirstLineNumber() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlock_FirstLineNumber(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlock) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextBlock_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextBlock) IsVisible() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextBlock_IsVisible(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextBlock) Layout() *QTextLayout {
	if ptr.Pointer() != nil {
		return NewQTextLayoutFromPointer(C.QTextBlock_Layout(ptr.Pointer()))
	}
	return nil
}

func (ptr *QTextBlock) Length() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlock_Length(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlock) LineCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlock_LineCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlock) Next() *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextBlock_Next(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextBlock) Position() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlock_Position(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlock) Previous() *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextBlock_Previous(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextBlock) Revision() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlock_Revision(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlock) SetLineCount(count int) {
	if ptr.Pointer() != nil {
		C.QTextBlock_SetLineCount(ptr.Pointer(), C.int(int32(count)))
	}
}

func (ptr *QTextBlock) SetRevision(rev int) {
	if ptr.Pointer() != nil {
		C.QTextBlock_SetRevision(ptr.Pointer(), C.int(int32(rev)))
	}
}

func (ptr *QTextBlock) SetUserData(data QTextBlockUserData_ITF) {
	if ptr.Pointer() != nil {
		C.QTextBlock_SetUserData(ptr.Pointer(), PointerFromQTextBlockUserData(data))
	}
}

func (ptr *QTextBlock) SetUserState(state int) {
	if ptr.Pointer() != nil {
		C.QTextBlock_SetUserState(ptr.Pointer(), C.int(int32(state)))
	}
}

func (ptr *QTextBlock) SetVisible(visible bool) {
	if ptr.Pointer() != nil {
		C.QTextBlock_SetVisible(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(visible))))
	}
}

func (ptr *QTextBlock) Text() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextBlock_Text(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextBlock) TextDirection() core.Qt__LayoutDirection {
	if ptr.Pointer() != nil {
		return core.Qt__LayoutDirection(C.QTextBlock_TextDirection(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextBlock) UserData() *QTextBlockUserData {
	if ptr.Pointer() != nil {
		return NewQTextBlockUserDataFromPointer(C.QTextBlock_UserData(ptr.Pointer()))
	}
	return nil
}

func (ptr *QTextBlock) UserState() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlock_UserState(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlock) __textFormats_newList() unsafe.Pointer {
	return C.QTextBlock___textFormats_newList(ptr.Pointer())
}

type QTextBlockFormat struct {
	QTextFormat
}

type QTextBlockFormat_ITF interface {
	QTextFormat_ITF
	QTextBlockFormat_PTR() *QTextBlockFormat
}

func (ptr *QTextBlockFormat) QTextBlockFormat_PTR() *QTextBlockFormat {
	return ptr
}

func (ptr *QTextBlockFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFormat_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextBlockFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextFormat_PTR().SetPointer(p)
	}
}

func PointerFromQTextBlockFormat(ptr QTextBlockFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextBlockFormat_PTR().Pointer()
	}
	return nil
}

func NewQTextBlockFormatFromPointer(ptr unsafe.Pointer) (n *QTextBlockFormat) {
	n = new(QTextBlockFormat)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextBlockFormat) DestroyQTextBlockFormat() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QTextBlockFormat__LineHeightTypes
//QTextBlockFormat::LineHeightTypes
type QTextBlockFormat__LineHeightTypes int64

const (
	QTextBlockFormat__SingleHeight       QTextBlockFormat__LineHeightTypes = QTextBlockFormat__LineHeightTypes(0)
	QTextBlockFormat__ProportionalHeight QTextBlockFormat__LineHeightTypes = QTextBlockFormat__LineHeightTypes(1)
	QTextBlockFormat__FixedHeight        QTextBlockFormat__LineHeightTypes = QTextBlockFormat__LineHeightTypes(2)
	QTextBlockFormat__MinimumHeight      QTextBlockFormat__LineHeightTypes = QTextBlockFormat__LineHeightTypes(3)
	QTextBlockFormat__LineDistanceHeight QTextBlockFormat__LineHeightTypes = QTextBlockFormat__LineHeightTypes(4)
)

func NewQTextBlockFormat() *QTextBlockFormat {
	tmpValue := NewQTextBlockFormatFromPointer(C.QTextBlockFormat_NewQTextBlockFormat())
	qt.SetFinalizer(tmpValue, (*QTextBlockFormat).DestroyQTextBlockFormat)
	return tmpValue
}

func (ptr *QTextBlockFormat) Alignment() core.Qt__AlignmentFlag {
	if ptr.Pointer() != nil {
		return core.Qt__AlignmentFlag(C.QTextBlockFormat_Alignment(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextBlockFormat) BottomMargin() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextBlockFormat_BottomMargin(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextBlockFormat) HeadingLevel() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlockFormat_HeadingLevel(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlockFormat) Indent() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlockFormat_Indent(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlockFormat) LeftMargin() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextBlockFormat_LeftMargin(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextBlockFormat) LineHeight(scriptLineHeight float64, scaling float64) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextBlockFormat_LineHeight(ptr.Pointer(), C.double(scriptLineHeight), C.double(scaling)))
	}
	return 0
}

func (ptr *QTextBlockFormat) LineHeight2() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextBlockFormat_LineHeight2(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextBlockFormat) LineHeightType() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextBlockFormat_LineHeightType(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextBlockFormat) NonBreakableLines() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextBlockFormat_NonBreakableLines(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextBlockFormat) PageBreakPolicy() QTextFormat__PageBreakFlag {
	if ptr.Pointer() != nil {
		return QTextFormat__PageBreakFlag(C.QTextBlockFormat_PageBreakPolicy(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextBlockFormat) RightMargin() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextBlockFormat_RightMargin(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextBlockFormat) SetAlignment(alignment core.Qt__AlignmentFlag) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetAlignment(ptr.Pointer(), C.longlong(alignment))
	}
}

func (ptr *QTextBlockFormat) SetBottomMargin(margin float64) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetBottomMargin(ptr.Pointer(), C.double(margin))
	}
}

func (ptr *QTextBlockFormat) SetHeadingLevel(level int) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetHeadingLevel(ptr.Pointer(), C.int(int32(level)))
	}
}

func (ptr *QTextBlockFormat) SetIndent(indentation int) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetIndent(ptr.Pointer(), C.int(int32(indentation)))
	}
}

func (ptr *QTextBlockFormat) SetLeftMargin(margin float64) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetLeftMargin(ptr.Pointer(), C.double(margin))
	}
}

func (ptr *QTextBlockFormat) SetLineHeight(height float64, heightType int) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetLineHeight(ptr.Pointer(), C.double(height), C.int(int32(heightType)))
	}
}

func (ptr *QTextBlockFormat) SetNonBreakableLines(b bool) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetNonBreakableLines(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(b))))
	}
}

func (ptr *QTextBlockFormat) SetPageBreakPolicy(policy QTextFormat__PageBreakFlag) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetPageBreakPolicy(ptr.Pointer(), C.longlong(policy))
	}
}

func (ptr *QTextBlockFormat) SetRightMargin(margin float64) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetRightMargin(ptr.Pointer(), C.double(margin))
	}
}

func (ptr *QTextBlockFormat) SetTextIndent(indent float64) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetTextIndent(ptr.Pointer(), C.double(indent))
	}
}

func (ptr *QTextBlockFormat) SetTopMargin(margin float64) {
	if ptr.Pointer() != nil {
		C.QTextBlockFormat_SetTopMargin(ptr.Pointer(), C.double(margin))
	}
}

func (ptr *QTextBlockFormat) TextIndent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextBlockFormat_TextIndent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextBlockFormat) TopMargin() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextBlockFormat_TopMargin(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextBlockFormat) __setTabPositions_tabs_newList() unsafe.Pointer {
	return C.QTextBlockFormat___setTabPositions_tabs_newList(ptr.Pointer())
}

func (ptr *QTextBlockFormat) __tabPositions_newList() unsafe.Pointer {
	return C.QTextBlockFormat___tabPositions_newList(ptr.Pointer())
}

type QTextBlockGroup struct {
	QTextObject
}

type QTextBlockGroup_ITF interface {
	QTextObject_ITF
	QTextBlockGroup_PTR() *QTextBlockGroup
}

func (ptr *QTextBlockGroup) QTextBlockGroup_PTR() *QTextBlockGroup {
	return ptr
}

func (ptr *QTextBlockGroup) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextBlockGroup) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextObject_PTR().SetPointer(p)
	}
}

func PointerFromQTextBlockGroup(ptr QTextBlockGroup_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextBlockGroup_PTR().Pointer()
	}
	return nil
}

func NewQTextBlockGroupFromPointer(ptr unsafe.Pointer) (n *QTextBlockGroup) {
	n = new(QTextBlockGroup)
	n.SetPointer(ptr)
	return
}
func NewQTextBlockGroup(document QTextDocument_ITF) *QTextBlockGroup {
	tmpValue := NewQTextBlockGroupFromPointer(C.QTextBlockGroup_NewQTextBlockGroup(PointerFromQTextDocument(document)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQTextBlockGroup_BlockFormatChanged
func callbackQTextBlockGroup_BlockFormatChanged(ptr unsafe.Pointer, block unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "blockFormatChanged"); signal != nil {
		(*(*func(*QTextBlock))(signal))(NewQTextBlockFromPointer(block))
	} else {
		NewQTextBlockGroupFromPointer(ptr).BlockFormatChangedDefault(NewQTextBlockFromPointer(block))
	}
}

func (ptr *QTextBlockGroup) ConnectBlockFormatChanged(f func(block *QTextBlock)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "blockFormatChanged"); signal != nil {
			f := func(block *QTextBlock) {
				(*(*func(*QTextBlock))(signal))(block)
				f(block)
			}
			qt.ConnectSignal(ptr.Pointer(), "blockFormatChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "blockFormatChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextBlockGroup) DisconnectBlockFormatChanged() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "blockFormatChanged")
	}
}

func (ptr *QTextBlockGroup) BlockFormatChanged(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QTextBlockGroup_BlockFormatChanged(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

func (ptr *QTextBlockGroup) BlockFormatChangedDefault(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QTextBlockGroup_BlockFormatChangedDefault(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

//export callbackQTextBlockGroup_BlockInserted
func callbackQTextBlockGroup_BlockInserted(ptr unsafe.Pointer, block unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "blockInserted"); signal != nil {
		(*(*func(*QTextBlock))(signal))(NewQTextBlockFromPointer(block))
	} else {
		NewQTextBlockGroupFromPointer(ptr).BlockInsertedDefault(NewQTextBlockFromPointer(block))
	}
}

func (ptr *QTextBlockGroup) ConnectBlockInserted(f func(block *QTextBlock)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "blockInserted"); signal != nil {
			f := func(block *QTextBlock) {
				(*(*func(*QTextBlock))(signal))(block)
				f(block)
			}
			qt.ConnectSignal(ptr.Pointer(), "blockInserted", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "blockInserted", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextBlockGroup) DisconnectBlockInserted() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "blockInserted")
	}
}

func (ptr *QTextBlockGroup) BlockInserted(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QTextBlockGroup_BlockInserted(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

func (ptr *QTextBlockGroup) BlockInsertedDefault(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QTextBlockGroup_BlockInsertedDefault(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

func (ptr *QTextBlockGroup) BlockList() []*QTextBlock {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QTextBlock {
			out := make([]*QTextBlock, int(l.len))
			tmpList := NewQTextBlockGroupFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__blockList_atList(i)
			}
			return out
		}(C.QTextBlockGroup_BlockList(ptr.Pointer()))
	}
	return make([]*QTextBlock, 0)
}

//export callbackQTextBlockGroup_BlockRemoved
func callbackQTextBlockGroup_BlockRemoved(ptr unsafe.Pointer, block unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "blockRemoved"); signal != nil {
		(*(*func(*QTextBlock))(signal))(NewQTextBlockFromPointer(block))
	} else {
		NewQTextBlockGroupFromPointer(ptr).BlockRemovedDefault(NewQTextBlockFromPointer(block))
	}
}

func (ptr *QTextBlockGroup) ConnectBlockRemoved(f func(block *QTextBlock)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "blockRemoved"); signal != nil {
			f := func(block *QTextBlock) {
				(*(*func(*QTextBlock))(signal))(block)
				f(block)
			}
			qt.ConnectSignal(ptr.Pointer(), "blockRemoved", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "blockRemoved", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextBlockGroup) DisconnectBlockRemoved() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "blockRemoved")
	}
}

func (ptr *QTextBlockGroup) BlockRemoved(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QTextBlockGroup_BlockRemoved(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

func (ptr *QTextBlockGroup) BlockRemovedDefault(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QTextBlockGroup_BlockRemovedDefault(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

//export callbackQTextBlockGroup_DestroyQTextBlockGroup
func callbackQTextBlockGroup_DestroyQTextBlockGroup(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QTextBlockGroup"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextBlockGroupFromPointer(ptr).DestroyQTextBlockGroupDefault()
	}
}

func (ptr *QTextBlockGroup) ConnectDestroyQTextBlockGroup(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QTextBlockGroup"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QTextBlockGroup", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QTextBlockGroup", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextBlockGroup) DisconnectDestroyQTextBlockGroup() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QTextBlockGroup")
	}
}

func (ptr *QTextBlockGroup) DestroyQTextBlockGroup() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextBlockGroup_DestroyQTextBlockGroup(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextBlockGroup) DestroyQTextBlockGroupDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextBlockGroup_DestroyQTextBlockGroupDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextBlockGroup) __blockList_atList(i int) *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextBlockGroup___blockList_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextBlockGroup) __blockList_setList(i QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QTextBlockGroup___blockList_setList(ptr.Pointer(), PointerFromQTextBlock(i))
	}
}

func (ptr *QTextBlockGroup) __blockList_newList() unsafe.Pointer {
	return C.QTextBlockGroup___blockList_newList(ptr.Pointer())
}

type QTextBlockUserData struct {
	ptr unsafe.Pointer
}

type QTextBlockUserData_ITF interface {
	QTextBlockUserData_PTR() *QTextBlockUserData
}

func (ptr *QTextBlockUserData) QTextBlockUserData_PTR() *QTextBlockUserData {
	return ptr
}

func (ptr *QTextBlockUserData) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextBlockUserData) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextBlockUserData(ptr QTextBlockUserData_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextBlockUserData_PTR().Pointer()
	}
	return nil
}

func NewQTextBlockUserDataFromPointer(ptr unsafe.Pointer) (n *QTextBlockUserData) {
	n = new(QTextBlockUserData)
	n.SetPointer(ptr)
	return
}

//export callbackQTextBlockUserData_DestroyQTextBlockUserData
func callbackQTextBlockUserData_DestroyQTextBlockUserData(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QTextBlockUserData"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextBlockUserDataFromPointer(ptr).DestroyQTextBlockUserDataDefault()
	}
}

func (ptr *QTextBlockUserData) ConnectDestroyQTextBlockUserData(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QTextBlockUserData"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QTextBlockUserData", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QTextBlockUserData", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextBlockUserData) DisconnectDestroyQTextBlockUserData() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QTextBlockUserData")
	}
}

func (ptr *QTextBlockUserData) DestroyQTextBlockUserData() {
	if ptr.Pointer() != nil {
		C.QTextBlockUserData_DestroyQTextBlockUserData(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextBlockUserData) DestroyQTextBlockUserDataDefault() {
	if ptr.Pointer() != nil {
		C.QTextBlockUserData_DestroyQTextBlockUserDataDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QTextCharFormat struct {
	QTextFormat
}

type QTextCharFormat_ITF interface {
	QTextFormat_ITF
	QTextCharFormat_PTR() *QTextCharFormat
}

func (ptr *QTextCharFormat) QTextCharFormat_PTR() *QTextCharFormat {
	return ptr
}

func (ptr *QTextCharFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFormat_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextCharFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextFormat_PTR().SetPointer(p)
	}
}

func PointerFromQTextCharFormat(ptr QTextCharFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextCharFormat_PTR().Pointer()
	}
	return nil
}

func NewQTextCharFormatFromPointer(ptr unsafe.Pointer) (n *QTextCharFormat) {
	n = new(QTextCharFormat)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextCharFormat) DestroyQTextCharFormat() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QTextCharFormat__VerticalAlignment
//QTextCharFormat::VerticalAlignment
type QTextCharFormat__VerticalAlignment int64

const (
	QTextCharFormat__AlignNormal      QTextCharFormat__VerticalAlignment = QTextCharFormat__VerticalAlignment(0)
	QTextCharFormat__AlignSuperScript QTextCharFormat__VerticalAlignment = QTextCharFormat__VerticalAlignment(1)
	QTextCharFormat__AlignSubScript   QTextCharFormat__VerticalAlignment = QTextCharFormat__VerticalAlignment(2)
	QTextCharFormat__AlignMiddle      QTextCharFormat__VerticalAlignment = QTextCharFormat__VerticalAlignment(3)
	QTextCharFormat__AlignTop         QTextCharFormat__VerticalAlignment = QTextCharFormat__VerticalAlignment(4)
	QTextCharFormat__AlignBottom      QTextCharFormat__VerticalAlignment = QTextCharFormat__VerticalAlignment(5)
	QTextCharFormat__AlignBaseline    QTextCharFormat__VerticalAlignment = QTextCharFormat__VerticalAlignment(6)
)

//go:generate stringer -type=QTextCharFormat__UnderlineStyle
//QTextCharFormat::UnderlineStyle
type QTextCharFormat__UnderlineStyle int64

var (
	QTextCharFormat__NoUnderline         QTextCharFormat__UnderlineStyle = QTextCharFormat__UnderlineStyle(0)
	QTextCharFormat__SingleUnderline     QTextCharFormat__UnderlineStyle = QTextCharFormat__UnderlineStyle(1)
	QTextCharFormat__DashUnderline       QTextCharFormat__UnderlineStyle = QTextCharFormat__UnderlineStyle(2)
	QTextCharFormat__DotLine             QTextCharFormat__UnderlineStyle = QTextCharFormat__UnderlineStyle(3)
	QTextCharFormat__DashDotLine         QTextCharFormat__UnderlineStyle = QTextCharFormat__UnderlineStyle(4)
	QTextCharFormat__DashDotDotLine      QTextCharFormat__UnderlineStyle = QTextCharFormat__UnderlineStyle(5)
	QTextCharFormat__WaveUnderline       QTextCharFormat__UnderlineStyle = QTextCharFormat__UnderlineStyle(6)
	QTextCharFormat__SpellCheckUnderline QTextCharFormat__UnderlineStyle = QTextCharFormat__UnderlineStyle(7)
)

//go:generate stringer -type=QTextCharFormat__FontPropertiesInheritanceBehavior
//QTextCharFormat::FontPropertiesInheritanceBehavior
type QTextCharFormat__FontPropertiesInheritanceBehavior int64

const (
	QTextCharFormat__FontPropertiesSpecifiedOnly QTextCharFormat__FontPropertiesInheritanceBehavior = QTextCharFormat__FontPropertiesInheritanceBehavior(0)
	QTextCharFormat__FontPropertiesAll           QTextCharFormat__FontPropertiesInheritanceBehavior = QTextCharFormat__FontPropertiesInheritanceBehavior(1)
)

func NewQTextCharFormat() *QTextCharFormat {
	tmpValue := NewQTextCharFormatFromPointer(C.QTextCharFormat_NewQTextCharFormat())
	qt.SetFinalizer(tmpValue, (*QTextCharFormat).DestroyQTextCharFormat)
	return tmpValue
}

func (ptr *QTextCharFormat) AnchorHref() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextCharFormat_AnchorHref(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextCharFormat) AnchorNames() []string {
	if ptr.Pointer() != nil {
		return unpackStringList(cGoUnpackString(C.QTextCharFormat_AnchorNames(ptr.Pointer())))
	}
	return make([]string, 0)
}

func (ptr *QTextCharFormat) Font() *QFont {
	if ptr.Pointer() != nil {
		tmpValue := NewQFontFromPointer(C.QTextCharFormat_Font(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
		return tmpValue
	}
	return nil
}

func (ptr *QTextCharFormat) FontCapitalization() QFont__Capitalization {
	if ptr.Pointer() != nil {
		return QFont__Capitalization(C.QTextCharFormat_FontCapitalization(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextCharFormat) FontFamilies() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QTextCharFormat_FontFamilies(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QTextCharFormat) FontFamily() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextCharFormat_FontFamily(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextCharFormat) FontFixedPitch() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCharFormat_FontFixedPitch(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCharFormat) FontHintingPreference() QFont__HintingPreference {
	if ptr.Pointer() != nil {
		return QFont__HintingPreference(C.QTextCharFormat_FontHintingPreference(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextCharFormat) FontItalic() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCharFormat_FontItalic(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCharFormat) FontKerning() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCharFormat_FontKerning(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCharFormat) FontLetterSpacing() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextCharFormat_FontLetterSpacing(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextCharFormat) FontLetterSpacingType() QFont__SpacingType {
	if ptr.Pointer() != nil {
		return QFont__SpacingType(C.QTextCharFormat_FontLetterSpacingType(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextCharFormat) FontOverline() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCharFormat_FontOverline(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCharFormat) FontPointSize() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextCharFormat_FontPointSize(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextCharFormat) FontStretch() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextCharFormat_FontStretch(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextCharFormat) FontStrikeOut() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCharFormat_FontStrikeOut(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCharFormat) FontStyleHint() QFont__StyleHint {
	if ptr.Pointer() != nil {
		return QFont__StyleHint(C.QTextCharFormat_FontStyleHint(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextCharFormat) FontStyleName() *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QTextCharFormat_FontStyleName(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QTextCharFormat) FontStyleStrategy() QFont__StyleStrategy {
	if ptr.Pointer() != nil {
		return QFont__StyleStrategy(C.QTextCharFormat_FontStyleStrategy(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextCharFormat) FontUnderline() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCharFormat_FontUnderline(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCharFormat) FontWeight() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextCharFormat_FontWeight(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextCharFormat) FontWordSpacing() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextCharFormat_FontWordSpacing(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextCharFormat) IsAnchor() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCharFormat_IsAnchor(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCharFormat) SetAnchor(anchor bool) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetAnchor(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(anchor))))
	}
}

func (ptr *QTextCharFormat) SetAnchorHref(value string) {
	if ptr.Pointer() != nil {
		var valueC *C.char
		if value != "" {
			valueC = C.CString(value)
			defer C.free(unsafe.Pointer(valueC))
		}
		C.QTextCharFormat_SetAnchorHref(ptr.Pointer(), C.struct_QtGui_PackedString{data: valueC, len: C.longlong(len(value))})
	}
}

func (ptr *QTextCharFormat) SetAnchorNames(names []string) {
	if ptr.Pointer() != nil {
		namesC := C.CString(strings.Join(names, "¡¦!"))
		defer C.free(unsafe.Pointer(namesC))
		C.QTextCharFormat_SetAnchorNames(ptr.Pointer(), C.struct_QtGui_PackedString{data: namesC, len: C.longlong(len(strings.Join(names, "¡¦!")))})
	}
}

func (ptr *QTextCharFormat) SetFont(font QFont_ITF, behavior QTextCharFormat__FontPropertiesInheritanceBehavior) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFont(ptr.Pointer(), PointerFromQFont(font), C.longlong(behavior))
	}
}

func (ptr *QTextCharFormat) SetFont2(font QFont_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFont2(ptr.Pointer(), PointerFromQFont(font))
	}
}

func (ptr *QTextCharFormat) SetFontCapitalization(capitalization QFont__Capitalization) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontCapitalization(ptr.Pointer(), C.longlong(capitalization))
	}
}

func (ptr *QTextCharFormat) SetFontFamilies(families []string) {
	if ptr.Pointer() != nil {
		familiesC := C.CString(strings.Join(families, "¡¦!"))
		defer C.free(unsafe.Pointer(familiesC))
		C.QTextCharFormat_SetFontFamilies(ptr.Pointer(), C.struct_QtGui_PackedString{data: familiesC, len: C.longlong(len(strings.Join(families, "¡¦!")))})
	}
}

func (ptr *QTextCharFormat) SetFontFamily(family string) {
	if ptr.Pointer() != nil {
		var familyC *C.char
		if family != "" {
			familyC = C.CString(family)
			defer C.free(unsafe.Pointer(familyC))
		}
		C.QTextCharFormat_SetFontFamily(ptr.Pointer(), C.struct_QtGui_PackedString{data: familyC, len: C.longlong(len(family))})
	}
}

func (ptr *QTextCharFormat) SetFontFixedPitch(fixedPitch bool) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontFixedPitch(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(fixedPitch))))
	}
}

func (ptr *QTextCharFormat) SetFontHintingPreference(hintingPreference QFont__HintingPreference) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontHintingPreference(ptr.Pointer(), C.longlong(hintingPreference))
	}
}

func (ptr *QTextCharFormat) SetFontItalic(italic bool) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontItalic(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(italic))))
	}
}

func (ptr *QTextCharFormat) SetFontKerning(enable bool) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontKerning(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QTextCharFormat) SetFontLetterSpacing(spacing float64) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontLetterSpacing(ptr.Pointer(), C.double(spacing))
	}
}

func (ptr *QTextCharFormat) SetFontLetterSpacingType(letterSpacingType QFont__SpacingType) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontLetterSpacingType(ptr.Pointer(), C.longlong(letterSpacingType))
	}
}

func (ptr *QTextCharFormat) SetFontOverline(overline bool) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontOverline(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(overline))))
	}
}

func (ptr *QTextCharFormat) SetFontPointSize(size float64) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontPointSize(ptr.Pointer(), C.double(size))
	}
}

func (ptr *QTextCharFormat) SetFontStretch(factor int) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontStretch(ptr.Pointer(), C.int(int32(factor)))
	}
}

func (ptr *QTextCharFormat) SetFontStrikeOut(strikeOut bool) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontStrikeOut(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(strikeOut))))
	}
}

func (ptr *QTextCharFormat) SetFontStyleHint(hint QFont__StyleHint, strategy QFont__StyleStrategy) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontStyleHint(ptr.Pointer(), C.longlong(hint), C.longlong(strategy))
	}
}

func (ptr *QTextCharFormat) SetFontStyleName(styleName string) {
	if ptr.Pointer() != nil {
		var styleNameC *C.char
		if styleName != "" {
			styleNameC = C.CString(styleName)
			defer C.free(unsafe.Pointer(styleNameC))
		}
		C.QTextCharFormat_SetFontStyleName(ptr.Pointer(), C.struct_QtGui_PackedString{data: styleNameC, len: C.longlong(len(styleName))})
	}
}

func (ptr *QTextCharFormat) SetFontStyleStrategy(strategy QFont__StyleStrategy) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontStyleStrategy(ptr.Pointer(), C.longlong(strategy))
	}
}

func (ptr *QTextCharFormat) SetFontUnderline(underline bool) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontUnderline(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(underline))))
	}
}

func (ptr *QTextCharFormat) SetFontWeight(weight int) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontWeight(ptr.Pointer(), C.int(int32(weight)))
	}
}

func (ptr *QTextCharFormat) SetFontWordSpacing(spacing float64) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetFontWordSpacing(ptr.Pointer(), C.double(spacing))
	}
}

func (ptr *QTextCharFormat) SetTextOutline(pen QPen_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetTextOutline(ptr.Pointer(), PointerFromQPen(pen))
	}
}

func (ptr *QTextCharFormat) SetToolTip(text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QTextCharFormat_SetToolTip(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QTextCharFormat) SetUnderlineColor(color QColor_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetUnderlineColor(ptr.Pointer(), PointerFromQColor(color))
	}
}

func (ptr *QTextCharFormat) SetUnderlineStyle(style QTextCharFormat__UnderlineStyle) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetUnderlineStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QTextCharFormat) SetVerticalAlignment(alignment QTextCharFormat__VerticalAlignment) {
	if ptr.Pointer() != nil {
		C.QTextCharFormat_SetVerticalAlignment(ptr.Pointer(), C.longlong(alignment))
	}
}

func (ptr *QTextCharFormat) TextOutline() *QPen {
	if ptr.Pointer() != nil {
		tmpValue := NewQPenFromPointer(C.QTextCharFormat_TextOutline(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QPen).DestroyQPen)
		return tmpValue
	}
	return nil
}

func (ptr *QTextCharFormat) ToolTip() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextCharFormat_ToolTip(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextCharFormat) UnderlineColor() *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QTextCharFormat_UnderlineColor(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextCharFormat) UnderlineStyle() QTextCharFormat__UnderlineStyle {
	if ptr.Pointer() != nil {
		return QTextCharFormat__UnderlineStyle(C.QTextCharFormat_UnderlineStyle(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextCharFormat) VerticalAlignment() QTextCharFormat__VerticalAlignment {
	if ptr.Pointer() != nil {
		return QTextCharFormat__VerticalAlignment(C.QTextCharFormat_VerticalAlignment(ptr.Pointer()))
	}
	return 0
}

type QTextCursor struct {
	ptr unsafe.Pointer
}

type QTextCursor_ITF interface {
	QTextCursor_PTR() *QTextCursor
}

func (ptr *QTextCursor) QTextCursor_PTR() *QTextCursor {
	return ptr
}

func (ptr *QTextCursor) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextCursor) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextCursor(ptr QTextCursor_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextCursor_PTR().Pointer()
	}
	return nil
}

func NewQTextCursorFromPointer(ptr unsafe.Pointer) (n *QTextCursor) {
	n = new(QTextCursor)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QTextCursor__MoveMode
//QTextCursor::MoveMode
type QTextCursor__MoveMode int64

const (
	QTextCursor__MoveAnchor QTextCursor__MoveMode = QTextCursor__MoveMode(0)
	QTextCursor__KeepAnchor QTextCursor__MoveMode = QTextCursor__MoveMode(1)
)

//go:generate stringer -type=QTextCursor__MoveOperation
//QTextCursor::MoveOperation
type QTextCursor__MoveOperation int64

const (
	QTextCursor__NoMove            QTextCursor__MoveOperation = QTextCursor__MoveOperation(0)
	QTextCursor__Start             QTextCursor__MoveOperation = QTextCursor__MoveOperation(1)
	QTextCursor__Up                QTextCursor__MoveOperation = QTextCursor__MoveOperation(2)
	QTextCursor__StartOfLine       QTextCursor__MoveOperation = QTextCursor__MoveOperation(3)
	QTextCursor__StartOfBlock      QTextCursor__MoveOperation = QTextCursor__MoveOperation(4)
	QTextCursor__StartOfWord       QTextCursor__MoveOperation = QTextCursor__MoveOperation(5)
	QTextCursor__PreviousBlock     QTextCursor__MoveOperation = QTextCursor__MoveOperation(6)
	QTextCursor__PreviousCharacter QTextCursor__MoveOperation = QTextCursor__MoveOperation(7)
	QTextCursor__PreviousWord      QTextCursor__MoveOperation = QTextCursor__MoveOperation(8)
	QTextCursor__Left              QTextCursor__MoveOperation = QTextCursor__MoveOperation(9)
	QTextCursor__WordLeft          QTextCursor__MoveOperation = QTextCursor__MoveOperation(10)
	QTextCursor__End               QTextCursor__MoveOperation = QTextCursor__MoveOperation(11)
	QTextCursor__Down              QTextCursor__MoveOperation = QTextCursor__MoveOperation(12)
	QTextCursor__EndOfLine         QTextCursor__MoveOperation = QTextCursor__MoveOperation(13)
	QTextCursor__EndOfWord         QTextCursor__MoveOperation = QTextCursor__MoveOperation(14)
	QTextCursor__EndOfBlock        QTextCursor__MoveOperation = QTextCursor__MoveOperation(15)
	QTextCursor__NextBlock         QTextCursor__MoveOperation = QTextCursor__MoveOperation(16)
	QTextCursor__NextCharacter     QTextCursor__MoveOperation = QTextCursor__MoveOperation(17)
	QTextCursor__NextWord          QTextCursor__MoveOperation = QTextCursor__MoveOperation(18)
	QTextCursor__Right             QTextCursor__MoveOperation = QTextCursor__MoveOperation(19)
	QTextCursor__WordRight         QTextCursor__MoveOperation = QTextCursor__MoveOperation(20)
	QTextCursor__NextCell          QTextCursor__MoveOperation = QTextCursor__MoveOperation(21)
	QTextCursor__PreviousCell      QTextCursor__MoveOperation = QTextCursor__MoveOperation(22)
	QTextCursor__NextRow           QTextCursor__MoveOperation = QTextCursor__MoveOperation(23)
	QTextCursor__PreviousRow       QTextCursor__MoveOperation = QTextCursor__MoveOperation(24)
)

//go:generate stringer -type=QTextCursor__SelectionType
//QTextCursor::SelectionType
type QTextCursor__SelectionType int64

const (
	QTextCursor__WordUnderCursor  QTextCursor__SelectionType = QTextCursor__SelectionType(0)
	QTextCursor__LineUnderCursor  QTextCursor__SelectionType = QTextCursor__SelectionType(1)
	QTextCursor__BlockUnderCursor QTextCursor__SelectionType = QTextCursor__SelectionType(2)
	QTextCursor__Document         QTextCursor__SelectionType = QTextCursor__SelectionType(3)
)

func NewQTextCursor() *QTextCursor {
	tmpValue := NewQTextCursorFromPointer(C.QTextCursor_NewQTextCursor())
	qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
	return tmpValue
}

func NewQTextCursor2(document QTextDocument_ITF) *QTextCursor {
	tmpValue := NewQTextCursorFromPointer(C.QTextCursor_NewQTextCursor2(PointerFromQTextDocument(document)))
	qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
	return tmpValue
}

func NewQTextCursor3(frame QTextFrame_ITF) *QTextCursor {
	tmpValue := NewQTextCursorFromPointer(C.QTextCursor_NewQTextCursor3(PointerFromQTextFrame(frame)))
	qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
	return tmpValue
}

func NewQTextCursor4(block QTextBlock_ITF) *QTextCursor {
	tmpValue := NewQTextCursorFromPointer(C.QTextCursor_NewQTextCursor4(PointerFromQTextBlock(block)))
	qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
	return tmpValue
}

func NewQTextCursor5(cursor QTextCursor_ITF) *QTextCursor {
	tmpValue := NewQTextCursorFromPointer(C.QTextCursor_NewQTextCursor5(PointerFromQTextCursor(cursor)))
	qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
	return tmpValue
}

func (ptr *QTextCursor) Anchor() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextCursor_Anchor(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextCursor) AtBlockEnd() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_AtBlockEnd(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCursor) AtBlockStart() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_AtBlockStart(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCursor) AtEnd() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_AtEnd(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCursor) AtStart() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_AtStart(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCursor) BeginEditBlock() {
	if ptr.Pointer() != nil {
		C.QTextCursor_BeginEditBlock(ptr.Pointer())
	}
}

func (ptr *QTextCursor) Block() *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextCursor_Block(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) BlockCharFormat() *QTextCharFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCharFormatFromPointer(C.QTextCursor_BlockCharFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextCharFormat).DestroyQTextCharFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) BlockFormat() *QTextBlockFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFormatFromPointer(C.QTextCursor_BlockFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlockFormat).DestroyQTextBlockFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) BlockNumber() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextCursor_BlockNumber(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextCursor) CharFormat() *QTextCharFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCharFormatFromPointer(C.QTextCursor_CharFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextCharFormat).DestroyQTextCharFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) ClearSelection() {
	if ptr.Pointer() != nil {
		C.QTextCursor_ClearSelection(ptr.Pointer())
	}
}

func (ptr *QTextCursor) ColumnNumber() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextCursor_ColumnNumber(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextCursor) CreateList(format QTextListFormat_ITF) *QTextList {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextListFromPointer(C.QTextCursor_CreateList(ptr.Pointer(), PointerFromQTextListFormat(format)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) CreateList2(style QTextListFormat__Style) *QTextList {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextListFromPointer(C.QTextCursor_CreateList2(ptr.Pointer(), C.longlong(style)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) CurrentFrame() *QTextFrame {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFrameFromPointer(C.QTextCursor_CurrentFrame(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) CurrentList() *QTextList {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextListFromPointer(C.QTextCursor_CurrentList(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) CurrentTable() *QTextTable {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextTableFromPointer(C.QTextCursor_CurrentTable(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) DeleteChar() {
	if ptr.Pointer() != nil {
		C.QTextCursor_DeleteChar(ptr.Pointer())
	}
}

func (ptr *QTextCursor) DeletePreviousChar() {
	if ptr.Pointer() != nil {
		C.QTextCursor_DeletePreviousChar(ptr.Pointer())
	}
}

func (ptr *QTextCursor) Document() *QTextDocument {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextDocumentFromPointer(C.QTextCursor_Document(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) EndEditBlock() {
	if ptr.Pointer() != nil {
		C.QTextCursor_EndEditBlock(ptr.Pointer())
	}
}

func (ptr *QTextCursor) HasComplexSelection() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_HasComplexSelection(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCursor) HasSelection() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_HasSelection(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCursor) InsertBlock() {
	if ptr.Pointer() != nil {
		C.QTextCursor_InsertBlock(ptr.Pointer())
	}
}

func (ptr *QTextCursor) InsertBlock2(format QTextBlockFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_InsertBlock2(ptr.Pointer(), PointerFromQTextBlockFormat(format))
	}
}

func (ptr *QTextCursor) InsertBlock3(format QTextBlockFormat_ITF, charFormat QTextCharFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_InsertBlock3(ptr.Pointer(), PointerFromQTextBlockFormat(format), PointerFromQTextCharFormat(charFormat))
	}
}

func (ptr *QTextCursor) InsertFragment(fragment QTextDocumentFragment_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_InsertFragment(ptr.Pointer(), PointerFromQTextDocumentFragment(fragment))
	}
}

func (ptr *QTextCursor) InsertFrame(format QTextFrameFormat_ITF) *QTextFrame {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFrameFromPointer(C.QTextCursor_InsertFrame(ptr.Pointer(), PointerFromQTextFrameFormat(format)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) InsertHtml(html string) {
	if ptr.Pointer() != nil {
		var htmlC *C.char
		if html != "" {
			htmlC = C.CString(html)
			defer C.free(unsafe.Pointer(htmlC))
		}
		C.QTextCursor_InsertHtml(ptr.Pointer(), C.struct_QtGui_PackedString{data: htmlC, len: C.longlong(len(html))})
	}
}

func (ptr *QTextCursor) InsertImage(format QTextImageFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_InsertImage(ptr.Pointer(), PointerFromQTextImageFormat(format))
	}
}

func (ptr *QTextCursor) InsertImage2(format QTextImageFormat_ITF, alignment QTextFrameFormat__Position) {
	if ptr.Pointer() != nil {
		C.QTextCursor_InsertImage2(ptr.Pointer(), PointerFromQTextImageFormat(format), C.longlong(alignment))
	}
}

func (ptr *QTextCursor) InsertImage3(name string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QTextCursor_InsertImage3(ptr.Pointer(), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
	}
}

func (ptr *QTextCursor) InsertImage4(image QImage_ITF, name string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QTextCursor_InsertImage4(ptr.Pointer(), PointerFromQImage(image), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
	}
}

func (ptr *QTextCursor) InsertList(format QTextListFormat_ITF) *QTextList {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextListFromPointer(C.QTextCursor_InsertList(ptr.Pointer(), PointerFromQTextListFormat(format)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) InsertList2(style QTextListFormat__Style) *QTextList {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextListFromPointer(C.QTextCursor_InsertList2(ptr.Pointer(), C.longlong(style)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) InsertTable(rows int, columns int, format QTextTableFormat_ITF) *QTextTable {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextTableFromPointer(C.QTextCursor_InsertTable(ptr.Pointer(), C.int(int32(rows)), C.int(int32(columns)), PointerFromQTextTableFormat(format)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) InsertTable2(rows int, columns int) *QTextTable {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextTableFromPointer(C.QTextCursor_InsertTable2(ptr.Pointer(), C.int(int32(rows)), C.int(int32(columns))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) InsertText(text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QTextCursor_InsertText(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QTextCursor) InsertText2(text string, format QTextCharFormat_ITF) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QTextCursor_InsertText2(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, PointerFromQTextCharFormat(format))
	}
}

func (ptr *QTextCursor) IsCopyOf(other QTextCursor_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_IsCopyOf(ptr.Pointer(), PointerFromQTextCursor(other))) != 0
	}
	return false
}

func (ptr *QTextCursor) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_IsNull(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCursor) JoinPreviousEditBlock() {
	if ptr.Pointer() != nil {
		C.QTextCursor_JoinPreviousEditBlock(ptr.Pointer())
	}
}

func (ptr *QTextCursor) KeepPositionOnInsert() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_KeepPositionOnInsert(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCursor) MergeBlockCharFormat(modifier QTextCharFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_MergeBlockCharFormat(ptr.Pointer(), PointerFromQTextCharFormat(modifier))
	}
}

func (ptr *QTextCursor) MergeBlockFormat(modifier QTextBlockFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_MergeBlockFormat(ptr.Pointer(), PointerFromQTextBlockFormat(modifier))
	}
}

func (ptr *QTextCursor) MergeCharFormat(modifier QTextCharFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_MergeCharFormat(ptr.Pointer(), PointerFromQTextCharFormat(modifier))
	}
}

func (ptr *QTextCursor) MovePosition(operation QTextCursor__MoveOperation, mode QTextCursor__MoveMode, n int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_MovePosition(ptr.Pointer(), C.longlong(operation), C.longlong(mode), C.int(int32(n)))) != 0
	}
	return false
}

func (ptr *QTextCursor) Position() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextCursor_Position(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextCursor) PositionInBlock() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextCursor_PositionInBlock(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextCursor) RemoveSelectedText() {
	if ptr.Pointer() != nil {
		C.QTextCursor_RemoveSelectedText(ptr.Pointer())
	}
}

func (ptr *QTextCursor) Select(selection QTextCursor__SelectionType) {
	if ptr.Pointer() != nil {
		C.QTextCursor_Select(ptr.Pointer(), C.longlong(selection))
	}
}

func (ptr *QTextCursor) SelectedTableCells(firstRow int, numRows int, firstColumn int, numColumns int) {
	if ptr.Pointer() != nil {
		C.QTextCursor_SelectedTableCells(ptr.Pointer(), C.int(int32(firstRow)), C.int(int32(numRows)), C.int(int32(firstColumn)), C.int(int32(numColumns)))
	}
}

func (ptr *QTextCursor) SelectedText() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextCursor_SelectedText(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextCursor) Selection() *QTextDocumentFragment {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextCursor_Selection(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
		return tmpValue
	}
	return nil
}

func (ptr *QTextCursor) SelectionEnd() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextCursor_SelectionEnd(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextCursor) SelectionStart() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextCursor_SelectionStart(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextCursor) SetBlockCharFormat(format QTextCharFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_SetBlockCharFormat(ptr.Pointer(), PointerFromQTextCharFormat(format))
	}
}

func (ptr *QTextCursor) SetBlockFormat(format QTextBlockFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_SetBlockFormat(ptr.Pointer(), PointerFromQTextBlockFormat(format))
	}
}

func (ptr *QTextCursor) SetCharFormat(format QTextCharFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_SetCharFormat(ptr.Pointer(), PointerFromQTextCharFormat(format))
	}
}

func (ptr *QTextCursor) SetKeepPositionOnInsert(b bool) {
	if ptr.Pointer() != nil {
		C.QTextCursor_SetKeepPositionOnInsert(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(b))))
	}
}

func (ptr *QTextCursor) SetPosition(pos int, m QTextCursor__MoveMode) {
	if ptr.Pointer() != nil {
		C.QTextCursor_SetPosition(ptr.Pointer(), C.int(int32(pos)), C.longlong(m))
	}
}

func (ptr *QTextCursor) SetVerticalMovementX(x int) {
	if ptr.Pointer() != nil {
		C.QTextCursor_SetVerticalMovementX(ptr.Pointer(), C.int(int32(x)))
	}
}

func (ptr *QTextCursor) SetVisualNavigation(b bool) {
	if ptr.Pointer() != nil {
		C.QTextCursor_SetVisualNavigation(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(b))))
	}
}

func (ptr *QTextCursor) Swap(other QTextCursor_ITF) {
	if ptr.Pointer() != nil {
		C.QTextCursor_Swap(ptr.Pointer(), PointerFromQTextCursor(other))
	}
}

func (ptr *QTextCursor) VerticalMovementX() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextCursor_VerticalMovementX(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextCursor) VisualNavigation() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextCursor_VisualNavigation(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextCursor) DestroyQTextCursor() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextCursor_DestroyQTextCursor(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QTextDocument struct {
	core.QObject
}

type QTextDocument_ITF interface {
	core.QObject_ITF
	QTextDocument_PTR() *QTextDocument
}

func (ptr *QTextDocument) QTextDocument_PTR() *QTextDocument {
	return ptr
}

func (ptr *QTextDocument) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextDocument) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQTextDocument(ptr QTextDocument_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextDocument_PTR().Pointer()
	}
	return nil
}

func NewQTextDocumentFromPointer(ptr unsafe.Pointer) (n *QTextDocument) {
	n = new(QTextDocument)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QTextDocument__MetaInformation
//QTextDocument::MetaInformation
type QTextDocument__MetaInformation int64

const (
	QTextDocument__DocumentTitle QTextDocument__MetaInformation = QTextDocument__MetaInformation(0)
	QTextDocument__DocumentUrl   QTextDocument__MetaInformation = QTextDocument__MetaInformation(1)
)

//go:generate stringer -type=QTextDocument__FindFlag
//QTextDocument::FindFlag
type QTextDocument__FindFlag int64

const (
	QTextDocument__FindBackward        QTextDocument__FindFlag = QTextDocument__FindFlag(0x00001)
	QTextDocument__FindCaseSensitively QTextDocument__FindFlag = QTextDocument__FindFlag(0x00002)
	QTextDocument__FindWholeWords      QTextDocument__FindFlag = QTextDocument__FindFlag(0x00004)
)

//go:generate stringer -type=QTextDocument__ResourceType
//QTextDocument::ResourceType
type QTextDocument__ResourceType int64

const (
	QTextDocument__HtmlResource       QTextDocument__ResourceType = QTextDocument__ResourceType(1)
	QTextDocument__ImageResource      QTextDocument__ResourceType = QTextDocument__ResourceType(2)
	QTextDocument__StyleSheetResource QTextDocument__ResourceType = QTextDocument__ResourceType(3)
	QTextDocument__UserResource       QTextDocument__ResourceType = QTextDocument__ResourceType(100)
)

//go:generate stringer -type=QTextDocument__Stacks
//QTextDocument::Stacks
type QTextDocument__Stacks int64

const (
	QTextDocument__UndoStack         QTextDocument__Stacks = QTextDocument__Stacks(0x01)
	QTextDocument__RedoStack         QTextDocument__Stacks = QTextDocument__Stacks(0x02)
	QTextDocument__UndoAndRedoStacks QTextDocument__Stacks = QTextDocument__Stacks(QTextDocument__UndoStack | QTextDocument__RedoStack)
)

func NewQTextDocument(parent core.QObject_ITF) *QTextDocument {
	tmpValue := NewQTextDocumentFromPointer(C.QTextDocument_NewQTextDocument(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQTextDocument2(text string, parent core.QObject_ITF) *QTextDocument {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQTextDocumentFromPointer(C.QTextDocument_NewQTextDocument2(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QTextDocument) AddResource(ty int, name core.QUrl_ITF, resource core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_AddResource(ptr.Pointer(), C.int(int32(ty)), core.PointerFromQUrl(name), core.PointerFromQVariant(resource))
	}
}

func (ptr *QTextDocument) AdjustSize() {
	if ptr.Pointer() != nil {
		C.QTextDocument_AdjustSize(ptr.Pointer())
	}
}

func (ptr *QTextDocument) AllFormats() []*QTextFormat {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QTextFormat {
			out := make([]*QTextFormat, int(l.len))
			tmpList := NewQTextDocumentFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__allFormats_atList(i)
			}
			return out
		}(C.QTextDocument_AllFormats(ptr.Pointer()))
	}
	return make([]*QTextFormat, 0)
}

func (ptr *QTextDocument) AvailableRedoSteps() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextDocument_AvailableRedoSteps(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextDocument) AvailableUndoSteps() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextDocument_AvailableUndoSteps(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextDocument) BaseUrl() *core.QUrl {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQUrlFromPointer(C.QTextDocument_BaseUrl(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QUrl).DestroyQUrl)
		return tmpValue
	}
	return nil
}

//export callbackQTextDocument_BaseUrlChanged
func callbackQTextDocument_BaseUrlChanged(ptr unsafe.Pointer, url unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "baseUrlChanged"); signal != nil {
		(*(*func(*core.QUrl))(signal))(core.NewQUrlFromPointer(url))
	}

}

func (ptr *QTextDocument) ConnectBaseUrlChanged(f func(url *core.QUrl)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "baseUrlChanged") {
			C.QTextDocument_ConnectBaseUrlChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "baseUrlChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "baseUrlChanged"); signal != nil {
			f := func(url *core.QUrl) {
				(*(*func(*core.QUrl))(signal))(url)
				f(url)
			}
			qt.ConnectSignal(ptr.Pointer(), "baseUrlChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "baseUrlChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectBaseUrlChanged() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectBaseUrlChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "baseUrlChanged")
	}
}

func (ptr *QTextDocument) BaseUrlChanged(url core.QUrl_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_BaseUrlChanged(ptr.Pointer(), core.PointerFromQUrl(url))
	}
}

func (ptr *QTextDocument) Begin() *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextDocument_Begin(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) BlockCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextDocument_BlockCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQTextDocument_BlockCountChanged
func callbackQTextDocument_BlockCountChanged(ptr unsafe.Pointer, newBlockCount C.int) {
	if signal := qt.GetSignal(ptr, "blockCountChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(newBlockCount)))
	}

}

func (ptr *QTextDocument) ConnectBlockCountChanged(f func(newBlockCount int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "blockCountChanged") {
			C.QTextDocument_ConnectBlockCountChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "blockCountChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "blockCountChanged"); signal != nil {
			f := func(newBlockCount int) {
				(*(*func(int))(signal))(newBlockCount)
				f(newBlockCount)
			}
			qt.ConnectSignal(ptr.Pointer(), "blockCountChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "blockCountChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectBlockCountChanged() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectBlockCountChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "blockCountChanged")
	}
}

func (ptr *QTextDocument) BlockCountChanged(newBlockCount int) {
	if ptr.Pointer() != nil {
		C.QTextDocument_BlockCountChanged(ptr.Pointer(), C.int(int32(newBlockCount)))
	}
}

func (ptr *QTextDocument) CharacterAt(pos int) *core.QChar {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQCharFromPointer(C.QTextDocument_CharacterAt(ptr.Pointer(), C.int(int32(pos))))
		qt.SetFinalizer(tmpValue, (*core.QChar).DestroyQChar)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) CharacterCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextDocument_CharacterCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQTextDocument_Clear
func callbackQTextDocument_Clear(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "clear"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextDocumentFromPointer(ptr).ClearDefault()
	}
}

func (ptr *QTextDocument) ConnectClear(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "clear"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "clear", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "clear", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectClear() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "clear")
	}
}

func (ptr *QTextDocument) Clear() {
	if ptr.Pointer() != nil {
		C.QTextDocument_Clear(ptr.Pointer())
	}
}

func (ptr *QTextDocument) ClearDefault() {
	if ptr.Pointer() != nil {
		C.QTextDocument_ClearDefault(ptr.Pointer())
	}
}

func (ptr *QTextDocument) ClearUndoRedoStacks(stacksToClear QTextDocument__Stacks) {
	if ptr.Pointer() != nil {
		C.QTextDocument_ClearUndoRedoStacks(ptr.Pointer(), C.longlong(stacksToClear))
	}
}

func (ptr *QTextDocument) Clone(parent core.QObject_ITF) *QTextDocument {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextDocumentFromPointer(C.QTextDocument_Clone(ptr.Pointer(), core.PointerFromQObject(parent)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQTextDocument_ContentsChange
func callbackQTextDocument_ContentsChange(ptr unsafe.Pointer, position C.int, charsRemoved C.int, charsAdded C.int) {
	if signal := qt.GetSignal(ptr, "contentsChange"); signal != nil {
		(*(*func(int, int, int))(signal))(int(int32(position)), int(int32(charsRemoved)), int(int32(charsAdded)))
	}

}

func (ptr *QTextDocument) ConnectContentsChange(f func(position int, charsRemoved int, charsAdded int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "contentsChange") {
			C.QTextDocument_ConnectContentsChange(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "contentsChange")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "contentsChange"); signal != nil {
			f := func(position int, charsRemoved int, charsAdded int) {
				(*(*func(int, int, int))(signal))(position, charsRemoved, charsAdded)
				f(position, charsRemoved, charsAdded)
			}
			qt.ConnectSignal(ptr.Pointer(), "contentsChange", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "contentsChange", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectContentsChange() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectContentsChange(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "contentsChange")
	}
}

func (ptr *QTextDocument) ContentsChange(position int, charsRemoved int, charsAdded int) {
	if ptr.Pointer() != nil {
		C.QTextDocument_ContentsChange(ptr.Pointer(), C.int(int32(position)), C.int(int32(charsRemoved)), C.int(int32(charsAdded)))
	}
}

//export callbackQTextDocument_ContentsChanged
func callbackQTextDocument_ContentsChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "contentsChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QTextDocument) ConnectContentsChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "contentsChanged") {
			C.QTextDocument_ConnectContentsChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "contentsChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "contentsChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "contentsChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "contentsChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectContentsChanged() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectContentsChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "contentsChanged")
	}
}

func (ptr *QTextDocument) ContentsChanged() {
	if ptr.Pointer() != nil {
		C.QTextDocument_ContentsChanged(ptr.Pointer())
	}
}

//export callbackQTextDocument_CreateObject
func callbackQTextDocument_CreateObject(ptr unsafe.Pointer, format unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "createObject"); signal != nil {
		return PointerFromQTextObject((*(*func(*QTextFormat) *QTextObject)(signal))(NewQTextFormatFromPointer(format)))
	}

	return PointerFromQTextObject(NewQTextDocumentFromPointer(ptr).CreateObjectDefault(NewQTextFormatFromPointer(format)))
}

func (ptr *QTextDocument) ConnectCreateObject(f func(format *QTextFormat) *QTextObject) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "createObject"); signal != nil {
			f := func(format *QTextFormat) *QTextObject {
				(*(*func(*QTextFormat) *QTextObject)(signal))(format)
				return f(format)
			}
			qt.ConnectSignal(ptr.Pointer(), "createObject", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "createObject", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectCreateObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "createObject")
	}
}

func (ptr *QTextDocument) CreateObject(format QTextFormat_ITF) *QTextObject {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextObjectFromPointer(C.QTextDocument_CreateObject(ptr.Pointer(), PointerFromQTextFormat(format)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) CreateObjectDefault(format QTextFormat_ITF) *QTextObject {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextObjectFromPointer(C.QTextDocument_CreateObjectDefault(ptr.Pointer(), PointerFromQTextFormat(format)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQTextDocument_CursorPositionChanged
func callbackQTextDocument_CursorPositionChanged(ptr unsafe.Pointer, cursor unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "cursorPositionChanged"); signal != nil {
		(*(*func(*QTextCursor))(signal))(NewQTextCursorFromPointer(cursor))
	}

}

func (ptr *QTextDocument) ConnectCursorPositionChanged(f func(cursor *QTextCursor)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "cursorPositionChanged") {
			C.QTextDocument_ConnectCursorPositionChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "cursorPositionChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "cursorPositionChanged"); signal != nil {
			f := func(cursor *QTextCursor) {
				(*(*func(*QTextCursor))(signal))(cursor)
				f(cursor)
			}
			qt.ConnectSignal(ptr.Pointer(), "cursorPositionChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "cursorPositionChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectCursorPositionChanged() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectCursorPositionChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "cursorPositionChanged")
	}
}

func (ptr *QTextDocument) CursorPositionChanged(cursor QTextCursor_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_CursorPositionChanged(ptr.Pointer(), PointerFromQTextCursor(cursor))
	}
}

func (ptr *QTextDocument) DefaultCursorMoveStyle() core.Qt__CursorMoveStyle {
	if ptr.Pointer() != nil {
		return core.Qt__CursorMoveStyle(C.QTextDocument_DefaultCursorMoveStyle(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextDocument) DefaultFont() *QFont {
	if ptr.Pointer() != nil {
		tmpValue := NewQFontFromPointer(C.QTextDocument_DefaultFont(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) DefaultStyleSheet() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextDocument_DefaultStyleSheet(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextDocument) DefaultTextOption() *QTextOption {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextOptionFromPointer(C.QTextDocument_DefaultTextOption(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextOption).DestroyQTextOption)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) DocumentLayout() *QAbstractTextDocumentLayout {
	if ptr.Pointer() != nil {
		tmpValue := NewQAbstractTextDocumentLayoutFromPointer(C.QTextDocument_DocumentLayout(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQTextDocument_DocumentLayoutChanged
func callbackQTextDocument_DocumentLayoutChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "documentLayoutChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QTextDocument) ConnectDocumentLayoutChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "documentLayoutChanged") {
			C.QTextDocument_ConnectDocumentLayoutChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "documentLayoutChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "documentLayoutChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "documentLayoutChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "documentLayoutChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectDocumentLayoutChanged() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectDocumentLayoutChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "documentLayoutChanged")
	}
}

func (ptr *QTextDocument) DocumentLayoutChanged() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DocumentLayoutChanged(ptr.Pointer())
	}
}

func (ptr *QTextDocument) DocumentMargin() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextDocument_DocumentMargin(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextDocument) DrawContents(p QPainter_ITF, rect core.QRectF_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_DrawContents(ptr.Pointer(), PointerFromQPainter(p), core.PointerFromQRectF(rect))
	}
}

func (ptr *QTextDocument) End() *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextDocument_End(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) Find(subString string, cursor QTextCursor_ITF, options QTextDocument__FindFlag) *QTextCursor {
	if ptr.Pointer() != nil {
		var subStringC *C.char
		if subString != "" {
			subStringC = C.CString(subString)
			defer C.free(unsafe.Pointer(subStringC))
		}
		tmpValue := NewQTextCursorFromPointer(C.QTextDocument_Find(ptr.Pointer(), C.struct_QtGui_PackedString{data: subStringC, len: C.longlong(len(subString))}, PointerFromQTextCursor(cursor), C.longlong(options)))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) Find2(subString string, position int, options QTextDocument__FindFlag) *QTextCursor {
	if ptr.Pointer() != nil {
		var subStringC *C.char
		if subString != "" {
			subStringC = C.CString(subString)
			defer C.free(unsafe.Pointer(subStringC))
		}
		tmpValue := NewQTextCursorFromPointer(C.QTextDocument_Find2(ptr.Pointer(), C.struct_QtGui_PackedString{data: subStringC, len: C.longlong(len(subString))}, C.int(int32(position)), C.longlong(options)))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) Find3(expr core.QRegExp_ITF, from int, options QTextDocument__FindFlag) *QTextCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCursorFromPointer(C.QTextDocument_Find3(ptr.Pointer(), core.PointerFromQRegExp(expr), C.int(int32(from)), C.longlong(options)))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) Find4(expr core.QRegExp_ITF, cursor QTextCursor_ITF, options QTextDocument__FindFlag) *QTextCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCursorFromPointer(C.QTextDocument_Find4(ptr.Pointer(), core.PointerFromQRegExp(expr), PointerFromQTextCursor(cursor), C.longlong(options)))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) Find5(expr core.QRegularExpression_ITF, from int, options QTextDocument__FindFlag) *QTextCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCursorFromPointer(C.QTextDocument_Find5(ptr.Pointer(), core.PointerFromQRegularExpression(expr), C.int(int32(from)), C.longlong(options)))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) Find6(expr core.QRegularExpression_ITF, cursor QTextCursor_ITF, options QTextDocument__FindFlag) *QTextCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCursorFromPointer(C.QTextDocument_Find6(ptr.Pointer(), core.PointerFromQRegularExpression(expr), PointerFromQTextCursor(cursor), C.longlong(options)))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) FindBlock(pos int) *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextDocument_FindBlock(ptr.Pointer(), C.int(int32(pos))))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) FindBlockByLineNumber(lineNumber int) *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextDocument_FindBlockByLineNumber(ptr.Pointer(), C.int(int32(lineNumber))))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) FindBlockByNumber(blockNumber int) *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextDocument_FindBlockByNumber(ptr.Pointer(), C.int(int32(blockNumber))))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) FirstBlock() *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextDocument_FirstBlock(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) IdealWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextDocument_IdealWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextDocument) IndentWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextDocument_IndentWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextDocument) IsEmpty() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocument_IsEmpty(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextDocument) IsModified() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocument_IsModified(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextDocument) IsRedoAvailable() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocument_IsRedoAvailable(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextDocument) IsUndoAvailable() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocument_IsUndoAvailable(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextDocument) IsUndoRedoEnabled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocument_IsUndoRedoEnabled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextDocument) LastBlock() *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextDocument_LastBlock(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) LineCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextDocument_LineCount(ptr.Pointer())))
	}
	return 0
}

//export callbackQTextDocument_LoadResource
func callbackQTextDocument_LoadResource(ptr unsafe.Pointer, ty C.int, name unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "loadResource"); signal != nil {
		return core.PointerFromQVariant((*(*func(int, *core.QUrl) *core.QVariant)(signal))(int(int32(ty)), core.NewQUrlFromPointer(name)))
	}

	return core.PointerFromQVariant(NewQTextDocumentFromPointer(ptr).LoadResourceDefault(int(int32(ty)), core.NewQUrlFromPointer(name)))
}

func (ptr *QTextDocument) ConnectLoadResource(f func(ty int, name *core.QUrl) *core.QVariant) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "loadResource"); signal != nil {
			f := func(ty int, name *core.QUrl) *core.QVariant {
				(*(*func(int, *core.QUrl) *core.QVariant)(signal))(ty, name)
				return f(ty, name)
			}
			qt.ConnectSignal(ptr.Pointer(), "loadResource", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "loadResource", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectLoadResource() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "loadResource")
	}
}

func (ptr *QTextDocument) LoadResource(ty int, name core.QUrl_ITF) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QTextDocument_LoadResource(ptr.Pointer(), C.int(int32(ty)), core.PointerFromQUrl(name)))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) LoadResourceDefault(ty int, name core.QUrl_ITF) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QTextDocument_LoadResourceDefault(ptr.Pointer(), C.int(int32(ty)), core.PointerFromQUrl(name)))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) MarkContentsDirty(position int, length int) {
	if ptr.Pointer() != nil {
		C.QTextDocument_MarkContentsDirty(ptr.Pointer(), C.int(int32(position)), C.int(int32(length)))
	}
}

func (ptr *QTextDocument) MaximumBlockCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextDocument_MaximumBlockCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextDocument) MetaInformation(info QTextDocument__MetaInformation) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextDocument_MetaInformation(ptr.Pointer(), C.longlong(info)))
	}
	return ""
}

//export callbackQTextDocument_ModificationChanged
func callbackQTextDocument_ModificationChanged(ptr unsafe.Pointer, changed C.char) {
	if signal := qt.GetSignal(ptr, "modificationChanged"); signal != nil {
		(*(*func(bool))(signal))(int8(changed) != 0)
	}

}

func (ptr *QTextDocument) ConnectModificationChanged(f func(changed bool)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "modificationChanged") {
			C.QTextDocument_ConnectModificationChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "modificationChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "modificationChanged"); signal != nil {
			f := func(changed bool) {
				(*(*func(bool))(signal))(changed)
				f(changed)
			}
			qt.ConnectSignal(ptr.Pointer(), "modificationChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "modificationChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectModificationChanged() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectModificationChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "modificationChanged")
	}
}

func (ptr *QTextDocument) ModificationChanged(changed bool) {
	if ptr.Pointer() != nil {
		C.QTextDocument_ModificationChanged(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(changed))))
	}
}

func (ptr *QTextDocument) Object(objectIndex int) *QTextObject {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextObjectFromPointer(C.QTextDocument_Object(ptr.Pointer(), C.int(int32(objectIndex))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) ObjectForFormat(ff QTextFormat_ITF) *QTextObject {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextObjectFromPointer(C.QTextDocument_ObjectForFormat(ptr.Pointer(), PointerFromQTextFormat(ff)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) PageCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextDocument_PageCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextDocument) PageSize() *core.QSizeF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFFromPointer(C.QTextDocument_PageSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) Print(printer QPagedPaintDevice_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_Print(ptr.Pointer(), PointerFromQPagedPaintDevice(printer))
	}
}

func (ptr *QTextDocument) Redo(cursor QTextCursor_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_Redo(ptr.Pointer(), PointerFromQTextCursor(cursor))
	}
}

//export callbackQTextDocument_Redo2
func callbackQTextDocument_Redo2(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "redo2"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextDocumentFromPointer(ptr).Redo2Default()
	}
}

func (ptr *QTextDocument) ConnectRedo2(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "redo2"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "redo2", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "redo2", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectRedo2() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "redo2")
	}
}

func (ptr *QTextDocument) Redo2() {
	if ptr.Pointer() != nil {
		C.QTextDocument_Redo2(ptr.Pointer())
	}
}

func (ptr *QTextDocument) Redo2Default() {
	if ptr.Pointer() != nil {
		C.QTextDocument_Redo2Default(ptr.Pointer())
	}
}

//export callbackQTextDocument_RedoAvailable
func callbackQTextDocument_RedoAvailable(ptr unsafe.Pointer, available C.char) {
	if signal := qt.GetSignal(ptr, "redoAvailable"); signal != nil {
		(*(*func(bool))(signal))(int8(available) != 0)
	}

}

func (ptr *QTextDocument) ConnectRedoAvailable(f func(available bool)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "redoAvailable") {
			C.QTextDocument_ConnectRedoAvailable(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "redoAvailable")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "redoAvailable"); signal != nil {
			f := func(available bool) {
				(*(*func(bool))(signal))(available)
				f(available)
			}
			qt.ConnectSignal(ptr.Pointer(), "redoAvailable", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "redoAvailable", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectRedoAvailable() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectRedoAvailable(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "redoAvailable")
	}
}

func (ptr *QTextDocument) RedoAvailable(available bool) {
	if ptr.Pointer() != nil {
		C.QTextDocument_RedoAvailable(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(available))))
	}
}

func (ptr *QTextDocument) Resource(ty int, name core.QUrl_ITF) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QTextDocument_Resource(ptr.Pointer(), C.int(int32(ty)), core.PointerFromQUrl(name)))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) Revision() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextDocument_Revision(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextDocument) RootFrame() *QTextFrame {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFrameFromPointer(C.QTextDocument_RootFrame(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) SetBaseUrl(url core.QUrl_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetBaseUrl(ptr.Pointer(), core.PointerFromQUrl(url))
	}
}

func (ptr *QTextDocument) SetDefaultCursorMoveStyle(style core.Qt__CursorMoveStyle) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetDefaultCursorMoveStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QTextDocument) SetDefaultFont(font QFont_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetDefaultFont(ptr.Pointer(), PointerFromQFont(font))
	}
}

func (ptr *QTextDocument) SetDefaultStyleSheet(sheet string) {
	if ptr.Pointer() != nil {
		var sheetC *C.char
		if sheet != "" {
			sheetC = C.CString(sheet)
			defer C.free(unsafe.Pointer(sheetC))
		}
		C.QTextDocument_SetDefaultStyleSheet(ptr.Pointer(), C.struct_QtGui_PackedString{data: sheetC, len: C.longlong(len(sheet))})
	}
}

func (ptr *QTextDocument) SetDefaultTextOption(option QTextOption_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetDefaultTextOption(ptr.Pointer(), PointerFromQTextOption(option))
	}
}

func (ptr *QTextDocument) SetDocumentLayout(layout QAbstractTextDocumentLayout_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetDocumentLayout(ptr.Pointer(), PointerFromQAbstractTextDocumentLayout(layout))
	}
}

func (ptr *QTextDocument) SetDocumentMargin(margin float64) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetDocumentMargin(ptr.Pointer(), C.double(margin))
	}
}

func (ptr *QTextDocument) SetHtml(html string) {
	if ptr.Pointer() != nil {
		var htmlC *C.char
		if html != "" {
			htmlC = C.CString(html)
			defer C.free(unsafe.Pointer(htmlC))
		}
		C.QTextDocument_SetHtml(ptr.Pointer(), C.struct_QtGui_PackedString{data: htmlC, len: C.longlong(len(html))})
	}
}

func (ptr *QTextDocument) SetIndentWidth(width float64) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetIndentWidth(ptr.Pointer(), C.double(width))
	}
}

func (ptr *QTextDocument) SetMaximumBlockCount(maximum int) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetMaximumBlockCount(ptr.Pointer(), C.int(int32(maximum)))
	}
}

func (ptr *QTextDocument) SetMetaInformation(info QTextDocument__MetaInformation, stri string) {
	if ptr.Pointer() != nil {
		var striC *C.char
		if stri != "" {
			striC = C.CString(stri)
			defer C.free(unsafe.Pointer(striC))
		}
		C.QTextDocument_SetMetaInformation(ptr.Pointer(), C.longlong(info), C.struct_QtGui_PackedString{data: striC, len: C.longlong(len(stri))})
	}
}

//export callbackQTextDocument_SetModified
func callbackQTextDocument_SetModified(ptr unsafe.Pointer, m C.char) {
	if signal := qt.GetSignal(ptr, "setModified"); signal != nil {
		(*(*func(bool))(signal))(int8(m) != 0)
	} else {
		NewQTextDocumentFromPointer(ptr).SetModifiedDefault(int8(m) != 0)
	}
}

func (ptr *QTextDocument) ConnectSetModified(f func(m bool)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setModified"); signal != nil {
			f := func(m bool) {
				(*(*func(bool))(signal))(m)
				f(m)
			}
			qt.ConnectSignal(ptr.Pointer(), "setModified", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setModified", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectSetModified() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setModified")
	}
}

func (ptr *QTextDocument) SetModified(m bool) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetModified(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(m))))
	}
}

func (ptr *QTextDocument) SetModifiedDefault(m bool) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetModifiedDefault(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(m))))
	}
}

func (ptr *QTextDocument) SetPageSize(size core.QSizeF_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetPageSize(ptr.Pointer(), core.PointerFromQSizeF(size))
	}
}

func (ptr *QTextDocument) SetPlainText(text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QTextDocument_SetPlainText(ptr.Pointer(), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QTextDocument) SetTextWidth(width float64) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetTextWidth(ptr.Pointer(), C.double(width))
	}
}

func (ptr *QTextDocument) SetUndoRedoEnabled(enable bool) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetUndoRedoEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QTextDocument) SetUseDesignMetrics(b bool) {
	if ptr.Pointer() != nil {
		C.QTextDocument_SetUseDesignMetrics(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(b))))
	}
}

func (ptr *QTextDocument) Size() *core.QSizeF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFFromPointer(C.QTextDocument_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) TextWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextDocument_TextWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextDocument) ToHtml(encoding core.QByteArray_ITF) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextDocument_ToHtml(ptr.Pointer(), core.PointerFromQByteArray(encoding)))
	}
	return ""
}

func (ptr *QTextDocument) ToPlainText() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextDocument_ToPlainText(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextDocument) ToRawText() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextDocument_ToRawText(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextDocument) Undo(cursor QTextCursor_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_Undo(ptr.Pointer(), PointerFromQTextCursor(cursor))
	}
}

//export callbackQTextDocument_Undo2
func callbackQTextDocument_Undo2(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "undo2"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextDocumentFromPointer(ptr).Undo2Default()
	}
}

func (ptr *QTextDocument) ConnectUndo2(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "undo2"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "undo2", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "undo2", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectUndo2() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "undo2")
	}
}

func (ptr *QTextDocument) Undo2() {
	if ptr.Pointer() != nil {
		C.QTextDocument_Undo2(ptr.Pointer())
	}
}

func (ptr *QTextDocument) Undo2Default() {
	if ptr.Pointer() != nil {
		C.QTextDocument_Undo2Default(ptr.Pointer())
	}
}

//export callbackQTextDocument_UndoAvailable
func callbackQTextDocument_UndoAvailable(ptr unsafe.Pointer, available C.char) {
	if signal := qt.GetSignal(ptr, "undoAvailable"); signal != nil {
		(*(*func(bool))(signal))(int8(available) != 0)
	}

}

func (ptr *QTextDocument) ConnectUndoAvailable(f func(available bool)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "undoAvailable") {
			C.QTextDocument_ConnectUndoAvailable(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "undoAvailable")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "undoAvailable"); signal != nil {
			f := func(available bool) {
				(*(*func(bool))(signal))(available)
				f(available)
			}
			qt.ConnectSignal(ptr.Pointer(), "undoAvailable", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "undoAvailable", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectUndoAvailable() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectUndoAvailable(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "undoAvailable")
	}
}

func (ptr *QTextDocument) UndoAvailable(available bool) {
	if ptr.Pointer() != nil {
		C.QTextDocument_UndoAvailable(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(available))))
	}
}

//export callbackQTextDocument_UndoCommandAdded
func callbackQTextDocument_UndoCommandAdded(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "undoCommandAdded"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QTextDocument) ConnectUndoCommandAdded(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "undoCommandAdded") {
			C.QTextDocument_ConnectUndoCommandAdded(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "undoCommandAdded")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "undoCommandAdded"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "undoCommandAdded", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "undoCommandAdded", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectUndoCommandAdded() {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectUndoCommandAdded(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "undoCommandAdded")
	}
}

func (ptr *QTextDocument) UndoCommandAdded() {
	if ptr.Pointer() != nil {
		C.QTextDocument_UndoCommandAdded(ptr.Pointer())
	}
}

func (ptr *QTextDocument) UseDesignMetrics() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocument_UseDesignMetrics(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQTextDocument_DestroyQTextDocument
func callbackQTextDocument_DestroyQTextDocument(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QTextDocument"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextDocumentFromPointer(ptr).DestroyQTextDocumentDefault()
	}
}

func (ptr *QTextDocument) ConnectDestroyQTextDocument(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QTextDocument"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QTextDocument", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QTextDocument", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextDocument) DisconnectDestroyQTextDocument() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QTextDocument")
	}
}

func (ptr *QTextDocument) DestroyQTextDocument() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextDocument_DestroyQTextDocument(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextDocument) DestroyQTextDocumentDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextDocument_DestroyQTextDocumentDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextDocument) __allFormats_atList(i int) *QTextFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFormatFromPointer(C.QTextDocument___allFormats_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QTextFormat).DestroyQTextFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) __allFormats_setList(i QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument___allFormats_setList(ptr.Pointer(), PointerFromQTextFormat(i))
	}
}

func (ptr *QTextDocument) __allFormats_newList() unsafe.Pointer {
	return C.QTextDocument___allFormats_newList(ptr.Pointer())
}

func (ptr *QTextDocument) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QTextDocument___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QTextDocument) __children_newList() unsafe.Pointer {
	return C.QTextDocument___children_newList(ptr.Pointer())
}

func (ptr *QTextDocument) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QTextDocument___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QTextDocument) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QTextDocument___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QTextDocument) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QTextDocument___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QTextDocument) __findChildren_newList() unsafe.Pointer {
	return C.QTextDocument___findChildren_newList(ptr.Pointer())
}

func (ptr *QTextDocument) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QTextDocument___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocument) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QTextDocument) __findChildren_newList3() unsafe.Pointer {
	return C.QTextDocument___findChildren_newList3(ptr.Pointer())
}

//export callbackQTextDocument_ChildEvent
func callbackQTextDocument_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQTextDocumentFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QTextDocument) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQTextDocument_ConnectNotify
func callbackQTextDocument_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQTextDocumentFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QTextDocument) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQTextDocument_CustomEvent
func callbackQTextDocument_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQTextDocumentFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QTextDocument) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQTextDocument_DeleteLater
func callbackQTextDocument_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextDocumentFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QTextDocument) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextDocument_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQTextDocument_Destroyed
func callbackQTextDocument_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQTextDocument_DisconnectNotify
func callbackQTextDocument_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQTextDocumentFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QTextDocument) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQTextDocument_Event
func callbackQTextDocument_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQTextDocumentFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QTextDocument) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocument_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQTextDocument_EventFilter
func callbackQTextDocument_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQTextDocumentFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QTextDocument) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocument_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQTextDocument_MetaObject
func callbackQTextDocument_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQTextDocumentFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QTextDocument) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QTextDocument_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQTextDocument_ObjectNameChanged
func callbackQTextDocument_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQTextDocument_TimerEvent
func callbackQTextDocument_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQTextDocumentFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QTextDocument) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocument_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QTextDocumentFragment struct {
	ptr unsafe.Pointer
}

type QTextDocumentFragment_ITF interface {
	QTextDocumentFragment_PTR() *QTextDocumentFragment
}

func (ptr *QTextDocumentFragment) QTextDocumentFragment_PTR() *QTextDocumentFragment {
	return ptr
}

func (ptr *QTextDocumentFragment) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextDocumentFragment) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextDocumentFragment(ptr QTextDocumentFragment_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextDocumentFragment_PTR().Pointer()
	}
	return nil
}

func NewQTextDocumentFragmentFromPointer(ptr unsafe.Pointer) (n *QTextDocumentFragment) {
	n = new(QTextDocumentFragment)
	n.SetPointer(ptr)
	return
}
func NewQTextDocumentFragment() *QTextDocumentFragment {
	tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextDocumentFragment_NewQTextDocumentFragment())
	qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
	return tmpValue
}

func NewQTextDocumentFragment2(document QTextDocument_ITF) *QTextDocumentFragment {
	tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextDocumentFragment_NewQTextDocumentFragment2(PointerFromQTextDocument(document)))
	qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
	return tmpValue
}

func NewQTextDocumentFragment3(cursor QTextCursor_ITF) *QTextDocumentFragment {
	tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextDocumentFragment_NewQTextDocumentFragment3(PointerFromQTextCursor(cursor)))
	qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
	return tmpValue
}

func NewQTextDocumentFragment4(other QTextDocumentFragment_ITF) *QTextDocumentFragment {
	tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextDocumentFragment_NewQTextDocumentFragment4(PointerFromQTextDocumentFragment(other)))
	qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
	return tmpValue
}

func QTextDocumentFragment_FromHtml(text string) *QTextDocumentFragment {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextDocumentFragment_QTextDocumentFragment_FromHtml(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
	qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
	return tmpValue
}

func (ptr *QTextDocumentFragment) FromHtml(text string) *QTextDocumentFragment {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextDocumentFragment_QTextDocumentFragment_FromHtml(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
	qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
	return tmpValue
}

func QTextDocumentFragment_FromHtml2(text string, resourceProvider QTextDocument_ITF) *QTextDocumentFragment {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextDocumentFragment_QTextDocumentFragment_FromHtml2(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, PointerFromQTextDocument(resourceProvider)))
	qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
	return tmpValue
}

func (ptr *QTextDocumentFragment) FromHtml2(text string, resourceProvider QTextDocument_ITF) *QTextDocumentFragment {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextDocumentFragment_QTextDocumentFragment_FromHtml2(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, PointerFromQTextDocument(resourceProvider)))
	qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
	return tmpValue
}

func QTextDocumentFragment_FromPlainText(plainText string) *QTextDocumentFragment {
	var plainTextC *C.char
	if plainText != "" {
		plainTextC = C.CString(plainText)
		defer C.free(unsafe.Pointer(plainTextC))
	}
	tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextDocumentFragment_QTextDocumentFragment_FromPlainText(C.struct_QtGui_PackedString{data: plainTextC, len: C.longlong(len(plainText))}))
	qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
	return tmpValue
}

func (ptr *QTextDocumentFragment) FromPlainText(plainText string) *QTextDocumentFragment {
	var plainTextC *C.char
	if plainText != "" {
		plainTextC = C.CString(plainText)
		defer C.free(unsafe.Pointer(plainTextC))
	}
	tmpValue := NewQTextDocumentFragmentFromPointer(C.QTextDocumentFragment_QTextDocumentFragment_FromPlainText(C.struct_QtGui_PackedString{data: plainTextC, len: C.longlong(len(plainText))}))
	qt.SetFinalizer(tmpValue, (*QTextDocumentFragment).DestroyQTextDocumentFragment)
	return tmpValue
}

func (ptr *QTextDocumentFragment) IsEmpty() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocumentFragment_IsEmpty(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextDocumentFragment) ToHtml(encoding core.QByteArray_ITF) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextDocumentFragment_ToHtml(ptr.Pointer(), core.PointerFromQByteArray(encoding)))
	}
	return ""
}

func (ptr *QTextDocumentFragment) ToPlainText() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextDocumentFragment_ToPlainText(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextDocumentFragment) DestroyQTextDocumentFragment() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextDocumentFragment_DestroyQTextDocumentFragment(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QTextDocumentWriter struct {
	ptr unsafe.Pointer
}

type QTextDocumentWriter_ITF interface {
	QTextDocumentWriter_PTR() *QTextDocumentWriter
}

func (ptr *QTextDocumentWriter) QTextDocumentWriter_PTR() *QTextDocumentWriter {
	return ptr
}

func (ptr *QTextDocumentWriter) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextDocumentWriter) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextDocumentWriter(ptr QTextDocumentWriter_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextDocumentWriter_PTR().Pointer()
	}
	return nil
}

func NewQTextDocumentWriterFromPointer(ptr unsafe.Pointer) (n *QTextDocumentWriter) {
	n = new(QTextDocumentWriter)
	n.SetPointer(ptr)
	return
}
func NewQTextDocumentWriter() *QTextDocumentWriter {
	tmpValue := NewQTextDocumentWriterFromPointer(C.QTextDocumentWriter_NewQTextDocumentWriter())
	qt.SetFinalizer(tmpValue, (*QTextDocumentWriter).DestroyQTextDocumentWriter)
	return tmpValue
}

func NewQTextDocumentWriter2(device core.QIODevice_ITF, format core.QByteArray_ITF) *QTextDocumentWriter {
	tmpValue := NewQTextDocumentWriterFromPointer(C.QTextDocumentWriter_NewQTextDocumentWriter2(core.PointerFromQIODevice(device), core.PointerFromQByteArray(format)))
	qt.SetFinalizer(tmpValue, (*QTextDocumentWriter).DestroyQTextDocumentWriter)
	return tmpValue
}

func NewQTextDocumentWriter3(fileName string, format core.QByteArray_ITF) *QTextDocumentWriter {
	var fileNameC *C.char
	if fileName != "" {
		fileNameC = C.CString(fileName)
		defer C.free(unsafe.Pointer(fileNameC))
	}
	tmpValue := NewQTextDocumentWriterFromPointer(C.QTextDocumentWriter_NewQTextDocumentWriter3(C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))}, core.PointerFromQByteArray(format)))
	qt.SetFinalizer(tmpValue, (*QTextDocumentWriter).DestroyQTextDocumentWriter)
	return tmpValue
}

func (ptr *QTextDocumentWriter) Codec() *core.QTextCodec {
	if ptr.Pointer() != nil {
		return core.NewQTextCodecFromPointer(C.QTextDocumentWriter_Codec(ptr.Pointer()))
	}
	return nil
}

func (ptr *QTextDocumentWriter) Device() *core.QIODevice {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQIODeviceFromPointer(C.QTextDocumentWriter_Device(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocumentWriter) FileName() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextDocumentWriter_FileName(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextDocumentWriter) Format() *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QTextDocumentWriter_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocumentWriter) SetCodec(codec core.QTextCodec_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocumentWriter_SetCodec(ptr.Pointer(), core.PointerFromQTextCodec(codec))
	}
}

func (ptr *QTextDocumentWriter) SetDevice(device core.QIODevice_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocumentWriter_SetDevice(ptr.Pointer(), core.PointerFromQIODevice(device))
	}
}

func (ptr *QTextDocumentWriter) SetFileName(fileName string) {
	if ptr.Pointer() != nil {
		var fileNameC *C.char
		if fileName != "" {
			fileNameC = C.CString(fileName)
			defer C.free(unsafe.Pointer(fileNameC))
		}
		C.QTextDocumentWriter_SetFileName(ptr.Pointer(), C.struct_QtGui_PackedString{data: fileNameC, len: C.longlong(len(fileName))})
	}
}

func (ptr *QTextDocumentWriter) SetFormat(format core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocumentWriter_SetFormat(ptr.Pointer(), core.PointerFromQByteArray(format))
	}
}

func QTextDocumentWriter_SupportedDocumentFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQTextDocumentWriterFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedDocumentFormats_atList(i)
		}
		return out
	}(C.QTextDocumentWriter_QTextDocumentWriter_SupportedDocumentFormats())
}

func (ptr *QTextDocumentWriter) SupportedDocumentFormats() []*core.QByteArray {
	return func(l C.struct_QtGui_PackedList) []*core.QByteArray {
		out := make([]*core.QByteArray, int(l.len))
		tmpList := NewQTextDocumentWriterFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__supportedDocumentFormats_atList(i)
		}
		return out
	}(C.QTextDocumentWriter_QTextDocumentWriter_SupportedDocumentFormats())
}

func (ptr *QTextDocumentWriter) Write(document QTextDocument_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocumentWriter_Write(ptr.Pointer(), PointerFromQTextDocument(document))) != 0
	}
	return false
}

func (ptr *QTextDocumentWriter) Write2(fragment QTextDocumentFragment_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextDocumentWriter_Write2(ptr.Pointer(), PointerFromQTextDocumentFragment(fragment))) != 0
	}
	return false
}

func (ptr *QTextDocumentWriter) DestroyQTextDocumentWriter() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextDocumentWriter_DestroyQTextDocumentWriter(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextDocumentWriter) __supportedDocumentFormats_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QTextDocumentWriter___supportedDocumentFormats_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QTextDocumentWriter) __supportedDocumentFormats_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QTextDocumentWriter___supportedDocumentFormats_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QTextDocumentWriter) __supportedDocumentFormats_newList() unsafe.Pointer {
	return C.QTextDocumentWriter___supportedDocumentFormats_newList(ptr.Pointer())
}

type QTextFormat struct {
	ptr unsafe.Pointer
}

type QTextFormat_ITF interface {
	QTextFormat_PTR() *QTextFormat
}

func (ptr *QTextFormat) QTextFormat_PTR() *QTextFormat {
	return ptr
}

func (ptr *QTextFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextFormat(ptr QTextFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFormat_PTR().Pointer()
	}
	return nil
}

func NewQTextFormatFromPointer(ptr unsafe.Pointer) (n *QTextFormat) {
	n = new(QTextFormat)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QTextFormat__FormatType
//QTextFormat::FormatType
type QTextFormat__FormatType int64

const (
	QTextFormat__InvalidFormat QTextFormat__FormatType = QTextFormat__FormatType(-1)
	QTextFormat__BlockFormat   QTextFormat__FormatType = QTextFormat__FormatType(1)
	QTextFormat__CharFormat    QTextFormat__FormatType = QTextFormat__FormatType(2)
	QTextFormat__ListFormat    QTextFormat__FormatType = QTextFormat__FormatType(3)
	QTextFormat__TableFormat   QTextFormat__FormatType = QTextFormat__FormatType(4)
	QTextFormat__FrameFormat   QTextFormat__FormatType = QTextFormat__FormatType(5)
	QTextFormat__UserFormat    QTextFormat__FormatType = QTextFormat__FormatType(100)
)

//go:generate stringer -type=QTextFormat__Property
//QTextFormat::Property
type QTextFormat__Property int64

const (
	QTextFormat__ObjectIndex                       QTextFormat__Property = QTextFormat__Property(0x0)
	QTextFormat__CssFloat                          QTextFormat__Property = QTextFormat__Property(0x0800)
	QTextFormat__LayoutDirection                   QTextFormat__Property = QTextFormat__Property(0x0801)
	QTextFormat__OutlinePen                        QTextFormat__Property = QTextFormat__Property(0x810)
	QTextFormat__BackgroundBrush                   QTextFormat__Property = QTextFormat__Property(0x820)
	QTextFormat__ForegroundBrush                   QTextFormat__Property = QTextFormat__Property(0x821)
	QTextFormat__BackgroundImageUrl                QTextFormat__Property = QTextFormat__Property(0x823)
	QTextFormat__BlockAlignment                    QTextFormat__Property = QTextFormat__Property(0x1010)
	QTextFormat__BlockTopMargin                    QTextFormat__Property = QTextFormat__Property(0x1030)
	QTextFormat__BlockBottomMargin                 QTextFormat__Property = QTextFormat__Property(0x1031)
	QTextFormat__BlockLeftMargin                   QTextFormat__Property = QTextFormat__Property(0x1032)
	QTextFormat__BlockRightMargin                  QTextFormat__Property = QTextFormat__Property(0x1033)
	QTextFormat__TextIndent                        QTextFormat__Property = QTextFormat__Property(0x1034)
	QTextFormat__TabPositions                      QTextFormat__Property = QTextFormat__Property(0x1035)
	QTextFormat__BlockIndent                       QTextFormat__Property = QTextFormat__Property(0x1040)
	QTextFormat__LineHeight                        QTextFormat__Property = QTextFormat__Property(0x1048)
	QTextFormat__LineHeightType                    QTextFormat__Property = QTextFormat__Property(0x1049)
	QTextFormat__BlockNonBreakableLines            QTextFormat__Property = QTextFormat__Property(0x1050)
	QTextFormat__BlockTrailingHorizontalRulerWidth QTextFormat__Property = QTextFormat__Property(0x1060)
	QTextFormat__HeadingLevel                      QTextFormat__Property = QTextFormat__Property(0x1070)
	QTextFormat__FirstFontProperty                 QTextFormat__Property = QTextFormat__Property(0x1FE0)
	QTextFormat__FontCapitalization                QTextFormat__Property = QTextFormat__Property(QTextFormat__FirstFontProperty)
	QTextFormat__FontLetterSpacingType             QTextFormat__Property = QTextFormat__Property(0x2033)
	QTextFormat__FontLetterSpacing                 QTextFormat__Property = QTextFormat__Property(0x1FE1)
	QTextFormat__FontWordSpacing                   QTextFormat__Property = QTextFormat__Property(0x1FE2)
	QTextFormat__FontStretch                       QTextFormat__Property = QTextFormat__Property(0x2034)
	QTextFormat__FontStyleHint                     QTextFormat__Property = QTextFormat__Property(0x1FE3)
	QTextFormat__FontStyleStrategy                 QTextFormat__Property = QTextFormat__Property(0x1FE4)
	QTextFormat__FontKerning                       QTextFormat__Property = QTextFormat__Property(0x1FE5)
	QTextFormat__FontHintingPreference             QTextFormat__Property = QTextFormat__Property(0x1FE6)
	QTextFormat__FontFamilies                      QTextFormat__Property = QTextFormat__Property(0x1FE7)
	QTextFormat__FontStyleName                     QTextFormat__Property = QTextFormat__Property(0x1FE8)
	QTextFormat__FontFamily                        QTextFormat__Property = QTextFormat__Property(0x2000)
	QTextFormat__FontPointSize                     QTextFormat__Property = QTextFormat__Property(0x2001)
	QTextFormat__FontSizeAdjustment                QTextFormat__Property = QTextFormat__Property(0x2002)
	QTextFormat__FontSizeIncrement                 QTextFormat__Property = QTextFormat__Property(QTextFormat__FontSizeAdjustment)
	QTextFormat__FontWeight                        QTextFormat__Property = QTextFormat__Property(0x2003)
	QTextFormat__FontItalic                        QTextFormat__Property = QTextFormat__Property(0x2004)
	QTextFormat__FontUnderline                     QTextFormat__Property = QTextFormat__Property(0x2005)
	QTextFormat__FontOverline                      QTextFormat__Property = QTextFormat__Property(0x2006)
	QTextFormat__FontStrikeOut                     QTextFormat__Property = QTextFormat__Property(0x2007)
	QTextFormat__FontFixedPitch                    QTextFormat__Property = QTextFormat__Property(0x2008)
	QTextFormat__FontPixelSize                     QTextFormat__Property = QTextFormat__Property(0x2009)
	QTextFormat__LastFontProperty                  QTextFormat__Property = QTextFormat__Property(QTextFormat__FontPixelSize)
	QTextFormat__TextUnderlineColor                QTextFormat__Property = QTextFormat__Property(0x2010)
	QTextFormat__TextVerticalAlignment             QTextFormat__Property = QTextFormat__Property(0x2021)
	QTextFormat__TextOutline                       QTextFormat__Property = QTextFormat__Property(0x2022)
	QTextFormat__TextUnderlineStyle                QTextFormat__Property = QTextFormat__Property(0x2023)
	QTextFormat__TextToolTip                       QTextFormat__Property = QTextFormat__Property(0x2024)
	QTextFormat__IsAnchor                          QTextFormat__Property = QTextFormat__Property(0x2030)
	QTextFormat__AnchorHref                        QTextFormat__Property = QTextFormat__Property(0x2031)
	QTextFormat__AnchorName                        QTextFormat__Property = QTextFormat__Property(0x2032)
	QTextFormat__ObjectType                        QTextFormat__Property = QTextFormat__Property(0x2f00)
	QTextFormat__ListStyle                         QTextFormat__Property = QTextFormat__Property(0x3000)
	QTextFormat__ListIndent                        QTextFormat__Property = QTextFormat__Property(0x3001)
	QTextFormat__ListNumberPrefix                  QTextFormat__Property = QTextFormat__Property(0x3002)
	QTextFormat__ListNumberSuffix                  QTextFormat__Property = QTextFormat__Property(0x3003)
	QTextFormat__FrameBorder                       QTextFormat__Property = QTextFormat__Property(0x4000)
	QTextFormat__FrameMargin                       QTextFormat__Property = QTextFormat__Property(0x4001)
	QTextFormat__FramePadding                      QTextFormat__Property = QTextFormat__Property(0x4002)
	QTextFormat__FrameWidth                        QTextFormat__Property = QTextFormat__Property(0x4003)
	QTextFormat__FrameHeight                       QTextFormat__Property = QTextFormat__Property(0x4004)
	QTextFormat__FrameTopMargin                    QTextFormat__Property = QTextFormat__Property(0x4005)
	QTextFormat__FrameBottomMargin                 QTextFormat__Property = QTextFormat__Property(0x4006)
	QTextFormat__FrameLeftMargin                   QTextFormat__Property = QTextFormat__Property(0x4007)
	QTextFormat__FrameRightMargin                  QTextFormat__Property = QTextFormat__Property(0x4008)
	QTextFormat__FrameBorderBrush                  QTextFormat__Property = QTextFormat__Property(0x4009)
	QTextFormat__FrameBorderStyle                  QTextFormat__Property = QTextFormat__Property(0x4010)
	QTextFormat__TableColumns                      QTextFormat__Property = QTextFormat__Property(0x4100)
	QTextFormat__TableColumnWidthConstraints       QTextFormat__Property = QTextFormat__Property(0x4101)
	QTextFormat__TableCellSpacing                  QTextFormat__Property = QTextFormat__Property(0x4102)
	QTextFormat__TableCellPadding                  QTextFormat__Property = QTextFormat__Property(0x4103)
	QTextFormat__TableHeaderRowCount               QTextFormat__Property = QTextFormat__Property(0x4104)
	QTextFormat__TableCellRowSpan                  QTextFormat__Property = QTextFormat__Property(0x4810)
	QTextFormat__TableCellColumnSpan               QTextFormat__Property = QTextFormat__Property(0x4811)
	QTextFormat__TableCellTopPadding               QTextFormat__Property = QTextFormat__Property(0x4812)
	QTextFormat__TableCellBottomPadding            QTextFormat__Property = QTextFormat__Property(0x4813)
	QTextFormat__TableCellLeftPadding              QTextFormat__Property = QTextFormat__Property(0x4814)
	QTextFormat__TableCellRightPadding             QTextFormat__Property = QTextFormat__Property(0x4815)
	QTextFormat__ImageName                         QTextFormat__Property = QTextFormat__Property(0x5000)
	QTextFormat__ImageWidth                        QTextFormat__Property = QTextFormat__Property(0x5010)
	QTextFormat__ImageHeight                       QTextFormat__Property = QTextFormat__Property(0x5011)
	QTextFormat__ImageQuality                      QTextFormat__Property = QTextFormat__Property(0x5014)
	QTextFormat__FullWidthSelection                QTextFormat__Property = QTextFormat__Property(0x06000)
	QTextFormat__PageBreakPolicy                   QTextFormat__Property = QTextFormat__Property(0x7000)
	QTextFormat__UserProperty                      QTextFormat__Property = QTextFormat__Property(0x100000)
)

//go:generate stringer -type=QTextFormat__ObjectTypes
//QTextFormat::ObjectTypes
type QTextFormat__ObjectTypes int64

const (
	QTextFormat__NoObject        QTextFormat__ObjectTypes = QTextFormat__ObjectTypes(0)
	QTextFormat__ImageObject     QTextFormat__ObjectTypes = QTextFormat__ObjectTypes(1)
	QTextFormat__TableObject     QTextFormat__ObjectTypes = QTextFormat__ObjectTypes(2)
	QTextFormat__TableCellObject QTextFormat__ObjectTypes = QTextFormat__ObjectTypes(3)
	QTextFormat__UserObject      QTextFormat__ObjectTypes = QTextFormat__ObjectTypes(0x1000)
)

//go:generate stringer -type=QTextFormat__PageBreakFlag
//QTextFormat::PageBreakFlag
type QTextFormat__PageBreakFlag int64

const (
	QTextFormat__PageBreak_Auto         QTextFormat__PageBreakFlag = QTextFormat__PageBreakFlag(0)
	QTextFormat__PageBreak_AlwaysBefore QTextFormat__PageBreakFlag = QTextFormat__PageBreakFlag(0x001)
	QTextFormat__PageBreak_AlwaysAfter  QTextFormat__PageBreakFlag = QTextFormat__PageBreakFlag(0x010)
)

func NewQTextFormat() *QTextFormat {
	tmpValue := NewQTextFormatFromPointer(C.QTextFormat_NewQTextFormat())
	qt.SetFinalizer(tmpValue, (*QTextFormat).DestroyQTextFormat)
	return tmpValue
}

func NewQTextFormat2(ty int) *QTextFormat {
	tmpValue := NewQTextFormatFromPointer(C.QTextFormat_NewQTextFormat2(C.int(int32(ty))))
	qt.SetFinalizer(tmpValue, (*QTextFormat).DestroyQTextFormat)
	return tmpValue
}

func NewQTextFormat3(other QTextFormat_ITF) *QTextFormat {
	tmpValue := NewQTextFormatFromPointer(C.QTextFormat_NewQTextFormat3(PointerFromQTextFormat(other)))
	qt.SetFinalizer(tmpValue, (*QTextFormat).DestroyQTextFormat)
	return tmpValue
}

func (ptr *QTextFormat) Background() *QBrush {
	if ptr.Pointer() != nil {
		tmpValue := NewQBrushFromPointer(C.QTextFormat_Background(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) BoolProperty(propertyId int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_BoolProperty(ptr.Pointer(), C.int(int32(propertyId)))) != 0
	}
	return false
}

func (ptr *QTextFormat) BrushProperty(propertyId int) *QBrush {
	if ptr.Pointer() != nil {
		tmpValue := NewQBrushFromPointer(C.QTextFormat_BrushProperty(ptr.Pointer(), C.int(int32(propertyId))))
		qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) ClearBackground() {
	if ptr.Pointer() != nil {
		C.QTextFormat_ClearBackground(ptr.Pointer())
	}
}

func (ptr *QTextFormat) ClearForeground() {
	if ptr.Pointer() != nil {
		C.QTextFormat_ClearForeground(ptr.Pointer())
	}
}

func (ptr *QTextFormat) ClearProperty(propertyId int) {
	if ptr.Pointer() != nil {
		C.QTextFormat_ClearProperty(ptr.Pointer(), C.int(int32(propertyId)))
	}
}

func (ptr *QTextFormat) ColorProperty(propertyId int) *QColor {
	if ptr.Pointer() != nil {
		tmpValue := NewQColorFromPointer(C.QTextFormat_ColorProperty(ptr.Pointer(), C.int(int32(propertyId))))
		qt.SetFinalizer(tmpValue, (*QColor).DestroyQColor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) DoubleProperty(propertyId int) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextFormat_DoubleProperty(ptr.Pointer(), C.int(int32(propertyId))))
	}
	return 0
}

func (ptr *QTextFormat) Foreground() *QBrush {
	if ptr.Pointer() != nil {
		tmpValue := NewQBrushFromPointer(C.QTextFormat_Foreground(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) HasProperty(propertyId int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_HasProperty(ptr.Pointer(), C.int(int32(propertyId)))) != 0
	}
	return false
}

func (ptr *QTextFormat) IntProperty(propertyId int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFormat_IntProperty(ptr.Pointer(), C.int(int32(propertyId)))))
	}
	return 0
}

func (ptr *QTextFormat) IsBlockFormat() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_IsBlockFormat(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextFormat) IsCharFormat() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_IsCharFormat(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextFormat) IsEmpty() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_IsEmpty(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextFormat) IsFrameFormat() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_IsFrameFormat(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextFormat) IsImageFormat() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_IsImageFormat(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextFormat) IsListFormat() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_IsListFormat(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextFormat) IsTableCellFormat() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_IsTableCellFormat(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextFormat) IsTableFormat() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_IsTableFormat(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextFormat) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFormat_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextFormat) LayoutDirection() core.Qt__LayoutDirection {
	if ptr.Pointer() != nil {
		return core.Qt__LayoutDirection(C.QTextFormat_LayoutDirection(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFormat) LengthProperty(propertyId int) *QTextLength {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextLengthFromPointer(C.QTextFormat_LengthProperty(ptr.Pointer(), C.int(int32(propertyId))))
		qt.SetFinalizer(tmpValue, (*QTextLength).DestroyQTextLength)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) Merge(other QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFormat_Merge(ptr.Pointer(), PointerFromQTextFormat(other))
	}
}

func (ptr *QTextFormat) ObjectIndex() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFormat_ObjectIndex(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextFormat) ObjectType() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFormat_ObjectType(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextFormat) PenProperty(propertyId int) *QPen {
	if ptr.Pointer() != nil {
		tmpValue := NewQPenFromPointer(C.QTextFormat_PenProperty(ptr.Pointer(), C.int(int32(propertyId))))
		qt.SetFinalizer(tmpValue, (*QPen).DestroyQPen)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) Properties() map[int]*core.QVariant {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) map[int]*core.QVariant {
			out := make(map[int]*core.QVariant, int(l.len))
			tmpList := NewQTextFormatFromPointer(l.data)
			for i, v := range tmpList.__properties_keyList() {
				out[v] = tmpList.__properties_atList(v, i)
			}
			return out
		}(C.QTextFormat_Properties(ptr.Pointer()))
	}
	return make(map[int]*core.QVariant, 0)
}

func (ptr *QTextFormat) Property(propertyId int) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QTextFormat_Property(ptr.Pointer(), C.int(int32(propertyId))))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) PropertyCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFormat_PropertyCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextFormat) SetBackground(brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFormat_SetBackground(ptr.Pointer(), PointerFromQBrush(brush))
	}
}

func (ptr *QTextFormat) SetForeground(brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFormat_SetForeground(ptr.Pointer(), PointerFromQBrush(brush))
	}
}

func (ptr *QTextFormat) SetLayoutDirection(direction core.Qt__LayoutDirection) {
	if ptr.Pointer() != nil {
		C.QTextFormat_SetLayoutDirection(ptr.Pointer(), C.longlong(direction))
	}
}

func (ptr *QTextFormat) SetObjectIndex(index int) {
	if ptr.Pointer() != nil {
		C.QTextFormat_SetObjectIndex(ptr.Pointer(), C.int(int32(index)))
	}
}

func (ptr *QTextFormat) SetObjectType(ty int) {
	if ptr.Pointer() != nil {
		C.QTextFormat_SetObjectType(ptr.Pointer(), C.int(int32(ty)))
	}
}

func (ptr *QTextFormat) SetProperty(propertyId int, value core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFormat_SetProperty(ptr.Pointer(), C.int(int32(propertyId)), core.PointerFromQVariant(value))
	}
}

func (ptr *QTextFormat) SetProperty2(propertyId int, value []*QTextLength) {
	if ptr.Pointer() != nil {
		C.QTextFormat_SetProperty2(ptr.Pointer(), C.int(int32(propertyId)), func() unsafe.Pointer {
			tmpList := NewQTextFormatFromPointer(NewQTextFormatFromPointer(nil).__setProperty_value_newList2())
			for _, v := range value {
				tmpList.__setProperty_value_setList2(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QTextFormat) StringProperty(propertyId int) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextFormat_StringProperty(ptr.Pointer(), C.int(int32(propertyId))))
	}
	return ""
}

func (ptr *QTextFormat) Swap(other QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFormat_Swap(ptr.Pointer(), PointerFromQTextFormat(other))
	}
}

func (ptr *QTextFormat) ToBlockFormat() *QTextBlockFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFormatFromPointer(C.QTextFormat_ToBlockFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextBlockFormat).DestroyQTextBlockFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) ToCharFormat() *QTextCharFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCharFormatFromPointer(C.QTextFormat_ToCharFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextCharFormat).DestroyQTextCharFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) ToFrameFormat() *QTextFrameFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFrameFormatFromPointer(C.QTextFormat_ToFrameFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextFrameFormat).DestroyQTextFrameFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) ToImageFormat() *QTextImageFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextImageFormatFromPointer(C.QTextFormat_ToImageFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextImageFormat).DestroyQTextImageFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) ToListFormat() *QTextListFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextListFormatFromPointer(C.QTextFormat_ToListFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextListFormat).DestroyQTextListFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) ToTableCellFormat() *QTextTableCellFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextTableCellFormatFromPointer(C.QTextFormat_ToTableCellFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextTableCellFormat).DestroyQTextTableCellFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) ToTableFormat() *QTextTableFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextTableFormatFromPointer(C.QTextFormat_ToTableFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextTableFormat).DestroyQTextTableFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) Type() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFormat_Type(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextFormat) DestroyQTextFormat() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextFormat_DestroyQTextFormat(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextFormat) __lengthVectorProperty_atList(i int) *QTextLength {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextLengthFromPointer(C.QTextFormat___lengthVectorProperty_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QTextLength).DestroyQTextLength)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) __lengthVectorProperty_setList(i QTextLength_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFormat___lengthVectorProperty_setList(ptr.Pointer(), PointerFromQTextLength(i))
	}
}

func (ptr *QTextFormat) __lengthVectorProperty_newList() unsafe.Pointer {
	return C.QTextFormat___lengthVectorProperty_newList(ptr.Pointer())
}

func (ptr *QTextFormat) __properties_atList(v int, i int) *core.QVariant {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQVariantFromPointer(C.QTextFormat___properties_atList(ptr.Pointer(), C.int(int32(v)), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QVariant).DestroyQVariant)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) __properties_setList(key int, i core.QVariant_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFormat___properties_setList(ptr.Pointer(), C.int(int32(key)), core.PointerFromQVariant(i))
	}
}

func (ptr *QTextFormat) __properties_newList() unsafe.Pointer {
	return C.QTextFormat___properties_newList(ptr.Pointer())
}

func (ptr *QTextFormat) __properties_keyList() []int {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []int {
			out := make([]int, int(l.len))
			tmpList := NewQTextFormatFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.____properties_keyList_atList(i)
			}
			return out
		}(C.QTextFormat___properties_keyList(ptr.Pointer()))
	}
	return make([]int, 0)
}

func (ptr *QTextFormat) __setProperty_value_atList2(i int) *QTextLength {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextLengthFromPointer(C.QTextFormat___setProperty_value_atList2(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QTextLength).DestroyQTextLength)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFormat) __setProperty_value_setList2(i QTextLength_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFormat___setProperty_value_setList2(ptr.Pointer(), PointerFromQTextLength(i))
	}
}

func (ptr *QTextFormat) __setProperty_value_newList2() unsafe.Pointer {
	return C.QTextFormat___setProperty_value_newList2(ptr.Pointer())
}

func (ptr *QTextFormat) ____properties_keyList_atList(i int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFormat_____properties_keyList_atList(ptr.Pointer(), C.int(int32(i)))))
	}
	return 0
}

func (ptr *QTextFormat) ____properties_keyList_setList(i int) {
	if ptr.Pointer() != nil {
		C.QTextFormat_____properties_keyList_setList(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QTextFormat) ____properties_keyList_newList() unsafe.Pointer {
	return C.QTextFormat_____properties_keyList_newList(ptr.Pointer())
}

type QTextFragment struct {
	ptr unsafe.Pointer
}

type QTextFragment_ITF interface {
	QTextFragment_PTR() *QTextFragment
}

func (ptr *QTextFragment) QTextFragment_PTR() *QTextFragment {
	return ptr
}

func (ptr *QTextFragment) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextFragment) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextFragment(ptr QTextFragment_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFragment_PTR().Pointer()
	}
	return nil
}

func NewQTextFragmentFromPointer(ptr unsafe.Pointer) (n *QTextFragment) {
	n = new(QTextFragment)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextFragment) DestroyQTextFragment() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQTextFragment2() *QTextFragment {
	tmpValue := NewQTextFragmentFromPointer(C.QTextFragment_NewQTextFragment2())
	qt.SetFinalizer(tmpValue, (*QTextFragment).DestroyQTextFragment)
	return tmpValue
}

func NewQTextFragment3(other QTextFragment_ITF) *QTextFragment {
	tmpValue := NewQTextFragmentFromPointer(C.QTextFragment_NewQTextFragment3(PointerFromQTextFragment(other)))
	qt.SetFinalizer(tmpValue, (*QTextFragment).DestroyQTextFragment)
	return tmpValue
}

func (ptr *QTextFragment) CharFormat() *QTextCharFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCharFormatFromPointer(C.QTextFragment_CharFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextCharFormat).DestroyQTextCharFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFragment) CharFormatIndex() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFragment_CharFormatIndex(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextFragment) Contains(position int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFragment_Contains(ptr.Pointer(), C.int(int32(position)))) != 0
	}
	return false
}

func (ptr *QTextFragment) GlyphRuns(pos int, l int) []*QGlyphRun {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QGlyphRun {
			out := make([]*QGlyphRun, int(l.len))
			tmpList := NewQTextFragmentFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__glyphRuns_atList(i)
			}
			return out
		}(C.QTextFragment_GlyphRuns(ptr.Pointer(), C.int(int32(pos)), C.int(int32(l))))
	}
	return make([]*QGlyphRun, 0)
}

func (ptr *QTextFragment) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextFragment_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextFragment) Length() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFragment_Length(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextFragment) Position() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFragment_Position(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextFragment) Text() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextFragment_Text(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextFragment) __glyphRuns_atList(i int) *QGlyphRun {
	if ptr.Pointer() != nil {
		tmpValue := NewQGlyphRunFromPointer(C.QTextFragment___glyphRuns_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QGlyphRun).DestroyQGlyphRun)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFragment) __glyphRuns_setList(i QGlyphRun_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFragment___glyphRuns_setList(ptr.Pointer(), PointerFromQGlyphRun(i))
	}
}

func (ptr *QTextFragment) __glyphRuns_newList() unsafe.Pointer {
	return C.QTextFragment___glyphRuns_newList(ptr.Pointer())
}

type QTextFrame struct {
	QTextObject
}

type QTextFrame_ITF interface {
	QTextObject_ITF
	QTextFrame_PTR() *QTextFrame
}

func (ptr *QTextFrame) QTextFrame_PTR() *QTextFrame {
	return ptr
}

func (ptr *QTextFrame) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextFrame) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextObject_PTR().SetPointer(p)
	}
}

func PointerFromQTextFrame(ptr QTextFrame_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFrame_PTR().Pointer()
	}
	return nil
}

func NewQTextFrameFromPointer(ptr unsafe.Pointer) (n *QTextFrame) {
	n = new(QTextFrame)
	n.SetPointer(ptr)
	return
}
func NewQTextFrame(document QTextDocument_ITF) *QTextFrame {
	tmpValue := NewQTextFrameFromPointer(C.QTextFrame_NewQTextFrame(PointerFromQTextDocument(document)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QTextFrame) ChildFrames() []*QTextFrame {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QTextFrame {
			out := make([]*QTextFrame, int(l.len))
			tmpList := NewQTextFrameFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__childFrames_atList(i)
			}
			return out
		}(C.QTextFrame_ChildFrames(ptr.Pointer()))
	}
	return make([]*QTextFrame, 0)
}

func (ptr *QTextFrame) FirstCursorPosition() *QTextCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCursorFromPointer(C.QTextFrame_FirstCursorPosition(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFrame) FirstPosition() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFrame_FirstPosition(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextFrame) FrameFormat() *QTextFrameFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFrameFormatFromPointer(C.QTextFrame_FrameFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextFrameFormat).DestroyQTextFrameFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFrame) LastCursorPosition() *QTextCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCursorFromPointer(C.QTextFrame_LastCursorPosition(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFrame) LastPosition() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextFrame_LastPosition(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextFrame) ParentFrame() *QTextFrame {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFrameFromPointer(C.QTextFrame_ParentFrame(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextFrame) SetFrameFormat(format QTextFrameFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFrame_SetFrameFormat(ptr.Pointer(), PointerFromQTextFrameFormat(format))
	}
}

//export callbackQTextFrame_DestroyQTextFrame
func callbackQTextFrame_DestroyQTextFrame(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QTextFrame"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextFrameFromPointer(ptr).DestroyQTextFrameDefault()
	}
}

func (ptr *QTextFrame) ConnectDestroyQTextFrame(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QTextFrame"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QTextFrame", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QTextFrame", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextFrame) DisconnectDestroyQTextFrame() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QTextFrame")
	}
}

func (ptr *QTextFrame) DestroyQTextFrame() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextFrame_DestroyQTextFrame(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextFrame) DestroyQTextFrameDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextFrame_DestroyQTextFrameDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextFrame) __childFrames_atList(i int) *QTextFrame {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFrameFromPointer(C.QTextFrame___childFrames_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextFrame) __childFrames_setList(i QTextFrame_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFrame___childFrames_setList(ptr.Pointer(), PointerFromQTextFrame(i))
	}
}

func (ptr *QTextFrame) __childFrames_newList() unsafe.Pointer {
	return C.QTextFrame___childFrames_newList(ptr.Pointer())
}

type QTextFrameFormat struct {
	QTextFormat
}

type QTextFrameFormat_ITF interface {
	QTextFormat_ITF
	QTextFrameFormat_PTR() *QTextFrameFormat
}

func (ptr *QTextFrameFormat) QTextFrameFormat_PTR() *QTextFrameFormat {
	return ptr
}

func (ptr *QTextFrameFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFormat_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextFrameFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextFormat_PTR().SetPointer(p)
	}
}

func PointerFromQTextFrameFormat(ptr QTextFrameFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFrameFormat_PTR().Pointer()
	}
	return nil
}

func NewQTextFrameFormatFromPointer(ptr unsafe.Pointer) (n *QTextFrameFormat) {
	n = new(QTextFrameFormat)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextFrameFormat) DestroyQTextFrameFormat() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QTextFrameFormat__Position
//QTextFrameFormat::Position
type QTextFrameFormat__Position int64

const (
	QTextFrameFormat__InFlow     QTextFrameFormat__Position = QTextFrameFormat__Position(0)
	QTextFrameFormat__FloatLeft  QTextFrameFormat__Position = QTextFrameFormat__Position(1)
	QTextFrameFormat__FloatRight QTextFrameFormat__Position = QTextFrameFormat__Position(2)
)

//go:generate stringer -type=QTextFrameFormat__BorderStyle
//QTextFrameFormat::BorderStyle
type QTextFrameFormat__BorderStyle int64

var (
	QTextFrameFormat__BorderStyle_None       QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(0)
	QTextFrameFormat__BorderStyle_Dotted     QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(1)
	QTextFrameFormat__BorderStyle_Dashed     QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(2)
	QTextFrameFormat__BorderStyle_Solid      QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(3)
	QTextFrameFormat__BorderStyle_Double     QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(4)
	QTextFrameFormat__BorderStyle_DotDash    QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(5)
	QTextFrameFormat__BorderStyle_DotDotDash QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(6)
	QTextFrameFormat__BorderStyle_Groove     QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(7)
	QTextFrameFormat__BorderStyle_Ridge      QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(8)
	QTextFrameFormat__BorderStyle_Inset      QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(9)
	QTextFrameFormat__BorderStyle_Outset     QTextFrameFormat__BorderStyle = QTextFrameFormat__BorderStyle(10)
)

func NewQTextFrameFormat() *QTextFrameFormat {
	tmpValue := NewQTextFrameFormatFromPointer(C.QTextFrameFormat_NewQTextFrameFormat())
	qt.SetFinalizer(tmpValue, (*QTextFrameFormat).DestroyQTextFrameFormat)
	return tmpValue
}

func (ptr *QTextFrameFormat) Border() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextFrameFormat_Border(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFrameFormat) BorderBrush() *QBrush {
	if ptr.Pointer() != nil {
		tmpValue := NewQBrushFromPointer(C.QTextFrameFormat_BorderBrush(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QBrush).DestroyQBrush)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFrameFormat) BorderStyle() QTextFrameFormat__BorderStyle {
	if ptr.Pointer() != nil {
		return QTextFrameFormat__BorderStyle(C.QTextFrameFormat_BorderStyle(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFrameFormat) BottomMargin() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextFrameFormat_BottomMargin(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFrameFormat) Height() *QTextLength {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextLengthFromPointer(C.QTextFrameFormat_Height(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextLength).DestroyQTextLength)
		return tmpValue
	}
	return nil
}

func (ptr *QTextFrameFormat) LeftMargin() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextFrameFormat_LeftMargin(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFrameFormat) Margin() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextFrameFormat_Margin(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFrameFormat) Padding() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextFrameFormat_Padding(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFrameFormat) PageBreakPolicy() QTextFormat__PageBreakFlag {
	if ptr.Pointer() != nil {
		return QTextFormat__PageBreakFlag(C.QTextFrameFormat_PageBreakPolicy(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFrameFormat) Position() QTextFrameFormat__Position {
	if ptr.Pointer() != nil {
		return QTextFrameFormat__Position(C.QTextFrameFormat_Position(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFrameFormat) RightMargin() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextFrameFormat_RightMargin(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFrameFormat) SetBorder(width float64) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetBorder(ptr.Pointer(), C.double(width))
	}
}

func (ptr *QTextFrameFormat) SetBorderBrush(brush QBrush_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetBorderBrush(ptr.Pointer(), PointerFromQBrush(brush))
	}
}

func (ptr *QTextFrameFormat) SetBorderStyle(style QTextFrameFormat__BorderStyle) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetBorderStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QTextFrameFormat) SetBottomMargin(margin float64) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetBottomMargin(ptr.Pointer(), C.double(margin))
	}
}

func (ptr *QTextFrameFormat) SetHeight(height QTextLength_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetHeight(ptr.Pointer(), PointerFromQTextLength(height))
	}
}

func (ptr *QTextFrameFormat) SetHeight2(height float64) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetHeight2(ptr.Pointer(), C.double(height))
	}
}

func (ptr *QTextFrameFormat) SetLeftMargin(margin float64) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetLeftMargin(ptr.Pointer(), C.double(margin))
	}
}

func (ptr *QTextFrameFormat) SetMargin(margin float64) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetMargin(ptr.Pointer(), C.double(margin))
	}
}

func (ptr *QTextFrameFormat) SetPadding(width float64) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetPadding(ptr.Pointer(), C.double(width))
	}
}

func (ptr *QTextFrameFormat) SetPageBreakPolicy(policy QTextFormat__PageBreakFlag) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetPageBreakPolicy(ptr.Pointer(), C.longlong(policy))
	}
}

func (ptr *QTextFrameFormat) SetPosition(policy QTextFrameFormat__Position) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetPosition(ptr.Pointer(), C.longlong(policy))
	}
}

func (ptr *QTextFrameFormat) SetRightMargin(margin float64) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetRightMargin(ptr.Pointer(), C.double(margin))
	}
}

func (ptr *QTextFrameFormat) SetTopMargin(margin float64) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetTopMargin(ptr.Pointer(), C.double(margin))
	}
}

func (ptr *QTextFrameFormat) SetWidth(width QTextLength_ITF) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetWidth(ptr.Pointer(), PointerFromQTextLength(width))
	}
}

func (ptr *QTextFrameFormat) SetWidth2(width float64) {
	if ptr.Pointer() != nil {
		C.QTextFrameFormat_SetWidth2(ptr.Pointer(), C.double(width))
	}
}

func (ptr *QTextFrameFormat) TopMargin() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextFrameFormat_TopMargin(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextFrameFormat) Width() *QTextLength {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextLengthFromPointer(C.QTextFrameFormat_Width(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextLength).DestroyQTextLength)
		return tmpValue
	}
	return nil
}

type QTextFrameLayoutData struct {
	ptr unsafe.Pointer
}

type QTextFrameLayoutData_ITF interface {
	QTextFrameLayoutData_PTR() *QTextFrameLayoutData
}

func (ptr *QTextFrameLayoutData) QTextFrameLayoutData_PTR() *QTextFrameLayoutData {
	return ptr
}

func (ptr *QTextFrameLayoutData) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextFrameLayoutData) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextFrameLayoutData(ptr QTextFrameLayoutData_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFrameLayoutData_PTR().Pointer()
	}
	return nil
}

func NewQTextFrameLayoutDataFromPointer(ptr unsafe.Pointer) (n *QTextFrameLayoutData) {
	n = new(QTextFrameLayoutData)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextFrameLayoutData) DestroyQTextFrameLayoutData() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QTextImageFormat struct {
	QTextCharFormat
}

type QTextImageFormat_ITF interface {
	QTextCharFormat_ITF
	QTextImageFormat_PTR() *QTextImageFormat
}

func (ptr *QTextImageFormat) QTextImageFormat_PTR() *QTextImageFormat {
	return ptr
}

func (ptr *QTextImageFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextCharFormat_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextImageFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextCharFormat_PTR().SetPointer(p)
	}
}

func PointerFromQTextImageFormat(ptr QTextImageFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextImageFormat_PTR().Pointer()
	}
	return nil
}

func NewQTextImageFormatFromPointer(ptr unsafe.Pointer) (n *QTextImageFormat) {
	n = new(QTextImageFormat)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextImageFormat) DestroyQTextImageFormat() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQTextImageFormat() *QTextImageFormat {
	tmpValue := NewQTextImageFormatFromPointer(C.QTextImageFormat_NewQTextImageFormat())
	qt.SetFinalizer(tmpValue, (*QTextImageFormat).DestroyQTextImageFormat)
	return tmpValue
}

func (ptr *QTextImageFormat) Height() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextImageFormat_Height(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextImageFormat) Name() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextImageFormat_Name(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextImageFormat) Quality() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextImageFormat_Quality(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextImageFormat) SetHeight(height float64) {
	if ptr.Pointer() != nil {
		C.QTextImageFormat_SetHeight(ptr.Pointer(), C.double(height))
	}
}

func (ptr *QTextImageFormat) SetName(name string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QTextImageFormat_SetName(ptr.Pointer(), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
	}
}

func (ptr *QTextImageFormat) SetQuality(quality int) {
	if ptr.Pointer() != nil {
		C.QTextImageFormat_SetQuality(ptr.Pointer(), C.int(int32(quality)))
	}
}

func (ptr *QTextImageFormat) SetWidth(width float64) {
	if ptr.Pointer() != nil {
		C.QTextImageFormat_SetWidth(ptr.Pointer(), C.double(width))
	}
}

func (ptr *QTextImageFormat) Width() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextImageFormat_Width(ptr.Pointer()))
	}
	return 0
}

type QTextInlineObject struct {
	ptr unsafe.Pointer
}

type QTextInlineObject_ITF interface {
	QTextInlineObject_PTR() *QTextInlineObject
}

func (ptr *QTextInlineObject) QTextInlineObject_PTR() *QTextInlineObject {
	return ptr
}

func (ptr *QTextInlineObject) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextInlineObject) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextInlineObject(ptr QTextInlineObject_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextInlineObject_PTR().Pointer()
	}
	return nil
}

func NewQTextInlineObjectFromPointer(ptr unsafe.Pointer) (n *QTextInlineObject) {
	n = new(QTextInlineObject)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextInlineObject) DestroyQTextInlineObject() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func (ptr *QTextInlineObject) Ascent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextInlineObject_Ascent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextInlineObject) Descent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextInlineObject_Descent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextInlineObject) Format() *QTextFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFormatFromPointer(C.QTextInlineObject_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextFormat).DestroyQTextFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextInlineObject) FormatIndex() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextInlineObject_FormatIndex(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextInlineObject) Height() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextInlineObject_Height(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextInlineObject) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextInlineObject_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextInlineObject) Rect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QTextInlineObject_Rect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QTextInlineObject) SetAscent(a float64) {
	if ptr.Pointer() != nil {
		C.QTextInlineObject_SetAscent(ptr.Pointer(), C.double(a))
	}
}

func (ptr *QTextInlineObject) SetDescent(d float64) {
	if ptr.Pointer() != nil {
		C.QTextInlineObject_SetDescent(ptr.Pointer(), C.double(d))
	}
}

func (ptr *QTextInlineObject) SetWidth(w float64) {
	if ptr.Pointer() != nil {
		C.QTextInlineObject_SetWidth(ptr.Pointer(), C.double(w))
	}
}

func (ptr *QTextInlineObject) TextDirection() core.Qt__LayoutDirection {
	if ptr.Pointer() != nil {
		return core.Qt__LayoutDirection(C.QTextInlineObject_TextDirection(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextInlineObject) TextPosition() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextInlineObject_TextPosition(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextInlineObject) Width() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextInlineObject_Width(ptr.Pointer()))
	}
	return 0
}

type QTextItem struct {
	ptr unsafe.Pointer
}

type QTextItem_ITF interface {
	QTextItem_PTR() *QTextItem
}

func (ptr *QTextItem) QTextItem_PTR() *QTextItem {
	return ptr
}

func (ptr *QTextItem) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextItem) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextItem(ptr QTextItem_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextItem_PTR().Pointer()
	}
	return nil
}

func NewQTextItemFromPointer(ptr unsafe.Pointer) (n *QTextItem) {
	n = new(QTextItem)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextItem) DestroyQTextItem() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QTextItem__RenderFlag
//QTextItem::RenderFlag
type QTextItem__RenderFlag int64

const (
	QTextItem__RightToLeft QTextItem__RenderFlag = QTextItem__RenderFlag(0x1)
	QTextItem__Overline    QTextItem__RenderFlag = QTextItem__RenderFlag(0x10)
	QTextItem__Underline   QTextItem__RenderFlag = QTextItem__RenderFlag(0x20)
	QTextItem__StrikeOut   QTextItem__RenderFlag = QTextItem__RenderFlag(0x40)
	QTextItem__Dummy       QTextItem__RenderFlag = QTextItem__RenderFlag(0xffffffff)
)

func (ptr *QTextItem) Ascent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextItem_Ascent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextItem) Descent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextItem_Descent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextItem) Font() *QFont {
	if ptr.Pointer() != nil {
		tmpValue := NewQFontFromPointer(C.QTextItem_Font(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
		return tmpValue
	}
	return nil
}

func (ptr *QTextItem) RenderFlags() QTextItem__RenderFlag {
	if ptr.Pointer() != nil {
		return QTextItem__RenderFlag(C.QTextItem_RenderFlags(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextItem) Text() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextItem_Text(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextItem) Width() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextItem_Width(ptr.Pointer()))
	}
	return 0
}

type QTextLayout struct {
	ptr unsafe.Pointer
}

type QTextLayout_ITF interface {
	QTextLayout_PTR() *QTextLayout
}

func (ptr *QTextLayout) QTextLayout_PTR() *QTextLayout {
	return ptr
}

func (ptr *QTextLayout) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextLayout) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextLayout(ptr QTextLayout_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextLayout_PTR().Pointer()
	}
	return nil
}

func NewQTextLayoutFromPointer(ptr unsafe.Pointer) (n *QTextLayout) {
	n = new(QTextLayout)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QTextLayout__CursorMode
//QTextLayout::CursorMode
type QTextLayout__CursorMode int64

const (
	QTextLayout__SkipCharacters QTextLayout__CursorMode = QTextLayout__CursorMode(0)
	QTextLayout__SkipWords      QTextLayout__CursorMode = QTextLayout__CursorMode(1)
)

func NewQTextLayout() *QTextLayout {
	tmpValue := NewQTextLayoutFromPointer(C.QTextLayout_NewQTextLayout())
	qt.SetFinalizer(tmpValue, (*QTextLayout).DestroyQTextLayout)
	return tmpValue
}

func NewQTextLayout2(text string) *QTextLayout {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQTextLayoutFromPointer(C.QTextLayout_NewQTextLayout2(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}))
	qt.SetFinalizer(tmpValue, (*QTextLayout).DestroyQTextLayout)
	return tmpValue
}

func NewQTextLayout4(text string, font QFont_ITF, paintdevice QPaintDevice_ITF) *QTextLayout {
	var textC *C.char
	if text != "" {
		textC = C.CString(text)
		defer C.free(unsafe.Pointer(textC))
	}
	tmpValue := NewQTextLayoutFromPointer(C.QTextLayout_NewQTextLayout4(C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))}, PointerFromQFont(font), PointerFromQPaintDevice(paintdevice)))
	qt.SetFinalizer(tmpValue, (*QTextLayout).DestroyQTextLayout)
	return tmpValue
}

func (ptr *QTextLayout) BeginLayout() {
	if ptr.Pointer() != nil {
		C.QTextLayout_BeginLayout(ptr.Pointer())
	}
}

func (ptr *QTextLayout) BoundingRect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QTextLayout_BoundingRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLayout) CacheEnabled() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextLayout_CacheEnabled(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextLayout) ClearFormats() {
	if ptr.Pointer() != nil {
		C.QTextLayout_ClearFormats(ptr.Pointer())
	}
}

func (ptr *QTextLayout) ClearLayout() {
	if ptr.Pointer() != nil {
		C.QTextLayout_ClearLayout(ptr.Pointer())
	}
}

func (ptr *QTextLayout) CreateLine() *QTextLine {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextLineFromPointer(C.QTextLayout_CreateLine(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextLine).DestroyQTextLine)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLayout) CursorMoveStyle() core.Qt__CursorMoveStyle {
	if ptr.Pointer() != nil {
		return core.Qt__CursorMoveStyle(C.QTextLayout_CursorMoveStyle(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLayout) DrawCursor(painter QPainter_ITF, position core.QPointF_ITF, cursorPosition int, width int) {
	if ptr.Pointer() != nil {
		C.QTextLayout_DrawCursor(ptr.Pointer(), PointerFromQPainter(painter), core.PointerFromQPointF(position), C.int(int32(cursorPosition)), C.int(int32(width)))
	}
}

func (ptr *QTextLayout) DrawCursor2(painter QPainter_ITF, position core.QPointF_ITF, cursorPosition int) {
	if ptr.Pointer() != nil {
		C.QTextLayout_DrawCursor2(ptr.Pointer(), PointerFromQPainter(painter), core.PointerFromQPointF(position), C.int(int32(cursorPosition)))
	}
}

func (ptr *QTextLayout) EndLayout() {
	if ptr.Pointer() != nil {
		C.QTextLayout_EndLayout(ptr.Pointer())
	}
}

func (ptr *QTextLayout) Font() *QFont {
	if ptr.Pointer() != nil {
		tmpValue := NewQFontFromPointer(C.QTextLayout_Font(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QFont).DestroyQFont)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLayout) GlyphRuns(from int, length int) []*QGlyphRun {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QGlyphRun {
			out := make([]*QGlyphRun, int(l.len))
			tmpList := NewQTextLayoutFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__glyphRuns_atList(i)
			}
			return out
		}(C.QTextLayout_GlyphRuns(ptr.Pointer(), C.int(int32(from)), C.int(int32(length))))
	}
	return make([]*QGlyphRun, 0)
}

func (ptr *QTextLayout) IsValidCursorPosition(pos int) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextLayout_IsValidCursorPosition(ptr.Pointer(), C.int(int32(pos)))) != 0
	}
	return false
}

func (ptr *QTextLayout) LeftCursorPosition(oldPos int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextLayout_LeftCursorPosition(ptr.Pointer(), C.int(int32(oldPos)))))
	}
	return 0
}

func (ptr *QTextLayout) LineAt(i int) *QTextLine {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextLineFromPointer(C.QTextLayout_LineAt(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QTextLine).DestroyQTextLine)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLayout) LineCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextLayout_LineCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextLayout) LineForTextPosition(pos int) *QTextLine {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextLineFromPointer(C.QTextLayout_LineForTextPosition(ptr.Pointer(), C.int(int32(pos))))
		qt.SetFinalizer(tmpValue, (*QTextLine).DestroyQTextLine)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLayout) MaximumWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLayout_MaximumWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLayout) MinimumWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLayout_MinimumWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLayout) NextCursorPosition(oldPos int, mode QTextLayout__CursorMode) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextLayout_NextCursorPosition(ptr.Pointer(), C.int(int32(oldPos)), C.longlong(mode))))
	}
	return 0
}

func (ptr *QTextLayout) Position() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QTextLayout_Position(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLayout) PreeditAreaPosition() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextLayout_PreeditAreaPosition(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextLayout) PreeditAreaText() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextLayout_PreeditAreaText(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextLayout) PreviousCursorPosition(oldPos int, mode QTextLayout__CursorMode) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextLayout_PreviousCursorPosition(ptr.Pointer(), C.int(int32(oldPos)), C.longlong(mode))))
	}
	return 0
}

func (ptr *QTextLayout) RightCursorPosition(oldPos int) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextLayout_RightCursorPosition(ptr.Pointer(), C.int(int32(oldPos)))))
	}
	return 0
}

func (ptr *QTextLayout) SetCacheEnabled(enable bool) {
	if ptr.Pointer() != nil {
		C.QTextLayout_SetCacheEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QTextLayout) SetCursorMoveStyle(style core.Qt__CursorMoveStyle) {
	if ptr.Pointer() != nil {
		C.QTextLayout_SetCursorMoveStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QTextLayout) SetFont(font QFont_ITF) {
	if ptr.Pointer() != nil {
		C.QTextLayout_SetFont(ptr.Pointer(), PointerFromQFont(font))
	}
}

func (ptr *QTextLayout) SetPosition(p core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QTextLayout_SetPosition(ptr.Pointer(), core.PointerFromQPointF(p))
	}
}

func (ptr *QTextLayout) SetPreeditArea(position int, text string) {
	if ptr.Pointer() != nil {
		var textC *C.char
		if text != "" {
			textC = C.CString(text)
			defer C.free(unsafe.Pointer(textC))
		}
		C.QTextLayout_SetPreeditArea(ptr.Pointer(), C.int(int32(position)), C.struct_QtGui_PackedString{data: textC, len: C.longlong(len(text))})
	}
}

func (ptr *QTextLayout) SetText(stri string) {
	if ptr.Pointer() != nil {
		var striC *C.char
		if stri != "" {
			striC = C.CString(stri)
			defer C.free(unsafe.Pointer(striC))
		}
		C.QTextLayout_SetText(ptr.Pointer(), C.struct_QtGui_PackedString{data: striC, len: C.longlong(len(stri))})
	}
}

func (ptr *QTextLayout) SetTextOption(option QTextOption_ITF) {
	if ptr.Pointer() != nil {
		C.QTextLayout_SetTextOption(ptr.Pointer(), PointerFromQTextOption(option))
	}
}

func (ptr *QTextLayout) Text() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextLayout_Text(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextLayout) TextOption() *QTextOption {
	if ptr.Pointer() != nil {
		return NewQTextOptionFromPointer(C.QTextLayout_TextOption(ptr.Pointer()))
	}
	return nil
}

func (ptr *QTextLayout) DestroyQTextLayout() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextLayout_DestroyQTextLayout(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextLayout) __formats_newList() unsafe.Pointer {
	return C.QTextLayout___formats_newList(ptr.Pointer())
}

func (ptr *QTextLayout) __glyphRuns_atList(i int) *QGlyphRun {
	if ptr.Pointer() != nil {
		tmpValue := NewQGlyphRunFromPointer(C.QTextLayout___glyphRuns_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QGlyphRun).DestroyQGlyphRun)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLayout) __glyphRuns_setList(i QGlyphRun_ITF) {
	if ptr.Pointer() != nil {
		C.QTextLayout___glyphRuns_setList(ptr.Pointer(), PointerFromQGlyphRun(i))
	}
}

func (ptr *QTextLayout) __glyphRuns_newList() unsafe.Pointer {
	return C.QTextLayout___glyphRuns_newList(ptr.Pointer())
}

type QTextLength struct {
	ptr unsafe.Pointer
}

type QTextLength_ITF interface {
	QTextLength_PTR() *QTextLength
}

func (ptr *QTextLength) QTextLength_PTR() *QTextLength {
	return ptr
}

func (ptr *QTextLength) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextLength) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextLength(ptr QTextLength_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextLength_PTR().Pointer()
	}
	return nil
}

func NewQTextLengthFromPointer(ptr unsafe.Pointer) (n *QTextLength) {
	n = new(QTextLength)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextLength) DestroyQTextLength() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QTextLength__Type
//QTextLength::Type
type QTextLength__Type int64

const (
	QTextLength__VariableLength   QTextLength__Type = QTextLength__Type(0)
	QTextLength__FixedLength      QTextLength__Type = QTextLength__Type(1)
	QTextLength__PercentageLength QTextLength__Type = QTextLength__Type(2)
)

func NewQTextLength() *QTextLength {
	tmpValue := NewQTextLengthFromPointer(C.QTextLength_NewQTextLength())
	qt.SetFinalizer(tmpValue, (*QTextLength).DestroyQTextLength)
	return tmpValue
}

func NewQTextLength2(ty QTextLength__Type, value float64) *QTextLength {
	tmpValue := NewQTextLengthFromPointer(C.QTextLength_NewQTextLength2(C.longlong(ty), C.double(value)))
	qt.SetFinalizer(tmpValue, (*QTextLength).DestroyQTextLength)
	return tmpValue
}

func (ptr *QTextLength) RawValue() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLength_RawValue(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLength) Type() QTextLength__Type {
	if ptr.Pointer() != nil {
		return QTextLength__Type(C.QTextLength_Type(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLength) Value(maximumLength float64) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLength_Value(ptr.Pointer(), C.double(maximumLength)))
	}
	return 0
}

type QTextLine struct {
	ptr unsafe.Pointer
}

type QTextLine_ITF interface {
	QTextLine_PTR() *QTextLine
}

func (ptr *QTextLine) QTextLine_PTR() *QTextLine {
	return ptr
}

func (ptr *QTextLine) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextLine) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextLine(ptr QTextLine_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextLine_PTR().Pointer()
	}
	return nil
}

func NewQTextLineFromPointer(ptr unsafe.Pointer) (n *QTextLine) {
	n = new(QTextLine)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextLine) DestroyQTextLine() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QTextLine__Edge
//QTextLine::Edge
type QTextLine__Edge int64

const (
	QTextLine__Leading  QTextLine__Edge = QTextLine__Edge(0)
	QTextLine__Trailing QTextLine__Edge = QTextLine__Edge(1)
)

//go:generate stringer -type=QTextLine__CursorPosition
//QTextLine::CursorPosition
type QTextLine__CursorPosition int64

const (
	QTextLine__CursorBetweenCharacters QTextLine__CursorPosition = QTextLine__CursorPosition(0)
	QTextLine__CursorOnCharacter       QTextLine__CursorPosition = QTextLine__CursorPosition(1)
)

func NewQTextLine() *QTextLine {
	tmpValue := NewQTextLineFromPointer(C.QTextLine_NewQTextLine())
	qt.SetFinalizer(tmpValue, (*QTextLine).DestroyQTextLine)
	return tmpValue
}

func (ptr *QTextLine) Ascent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_Ascent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLine) CursorToX(cursorPos int, edge QTextLine__Edge) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_CursorToX(ptr.Pointer(), C.int(int32(cursorPos)), C.longlong(edge)))
	}
	return 0
}

func (ptr *QTextLine) CursorToX2(cursorPos int, edge QTextLine__Edge) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_CursorToX2(ptr.Pointer(), C.int(int32(cursorPos)), C.longlong(edge)))
	}
	return 0
}

func (ptr *QTextLine) Descent() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_Descent(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLine) GlyphRuns(from int, length int) []*QGlyphRun {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []*QGlyphRun {
			out := make([]*QGlyphRun, int(l.len))
			tmpList := NewQTextLineFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__glyphRuns_atList(i)
			}
			return out
		}(C.QTextLine_GlyphRuns(ptr.Pointer(), C.int(int32(from)), C.int(int32(length))))
	}
	return make([]*QGlyphRun, 0)
}

func (ptr *QTextLine) Height() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_Height(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLine) HorizontalAdvance() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_HorizontalAdvance(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLine) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextLine_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextLine) Leading() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_Leading(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLine) LeadingIncluded() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextLine_LeadingIncluded(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextLine) LineNumber() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextLine_LineNumber(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextLine) NaturalTextRect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QTextLine_NaturalTextRect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLine) NaturalTextWidth() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_NaturalTextWidth(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLine) Position() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QTextLine_Position(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLine) Rect() *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QTextLine_Rect(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLine) SetLeadingIncluded(included bool) {
	if ptr.Pointer() != nil {
		C.QTextLine_SetLeadingIncluded(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(included))))
	}
}

func (ptr *QTextLine) SetLineWidth(width float64) {
	if ptr.Pointer() != nil {
		C.QTextLine_SetLineWidth(ptr.Pointer(), C.double(width))
	}
}

func (ptr *QTextLine) SetNumColumns(numColumns int) {
	if ptr.Pointer() != nil {
		C.QTextLine_SetNumColumns(ptr.Pointer(), C.int(int32(numColumns)))
	}
}

func (ptr *QTextLine) SetNumColumns2(numColumns int, alignmentWidth float64) {
	if ptr.Pointer() != nil {
		C.QTextLine_SetNumColumns2(ptr.Pointer(), C.int(int32(numColumns)), C.double(alignmentWidth))
	}
}

func (ptr *QTextLine) SetPosition(pos core.QPointF_ITF) {
	if ptr.Pointer() != nil {
		C.QTextLine_SetPosition(ptr.Pointer(), core.PointerFromQPointF(pos))
	}
}

func (ptr *QTextLine) TextLength() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextLine_TextLength(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextLine) TextStart() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextLine_TextStart(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextLine) Width() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_Width(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLine) X() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_X(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLine) XToCursor(x float64, cpos QTextLine__CursorPosition) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextLine_XToCursor(ptr.Pointer(), C.double(x), C.longlong(cpos))))
	}
	return 0
}

func (ptr *QTextLine) Y() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextLine_Y(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextLine) __glyphRuns_atList(i int) *QGlyphRun {
	if ptr.Pointer() != nil {
		tmpValue := NewQGlyphRunFromPointer(C.QTextLine___glyphRuns_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QGlyphRun).DestroyQGlyphRun)
		return tmpValue
	}
	return nil
}

func (ptr *QTextLine) __glyphRuns_setList(i QGlyphRun_ITF) {
	if ptr.Pointer() != nil {
		C.QTextLine___glyphRuns_setList(ptr.Pointer(), PointerFromQGlyphRun(i))
	}
}

func (ptr *QTextLine) __glyphRuns_newList() unsafe.Pointer {
	return C.QTextLine___glyphRuns_newList(ptr.Pointer())
}

type QTextList struct {
	QTextBlockGroup
}

type QTextList_ITF interface {
	QTextBlockGroup_ITF
	QTextList_PTR() *QTextList
}

func (ptr *QTextList) QTextList_PTR() *QTextList {
	return ptr
}

func (ptr *QTextList) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextBlockGroup_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextList) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextBlockGroup_PTR().SetPointer(p)
	}
}

func PointerFromQTextList(ptr QTextList_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextList_PTR().Pointer()
	}
	return nil
}

func NewQTextListFromPointer(ptr unsafe.Pointer) (n *QTextList) {
	n = new(QTextList)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextList) Add(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QTextList_Add(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

func (ptr *QTextList) Count() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextList_Count(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextList) Format() *QTextListFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextListFormatFromPointer(C.QTextList_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextListFormat).DestroyQTextListFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextList) Item(i int) *QTextBlock {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextBlockFromPointer(C.QTextList_Item(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QTextBlock).DestroyQTextBlock)
		return tmpValue
	}
	return nil
}

func (ptr *QTextList) ItemNumber(block QTextBlock_ITF) int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextList_ItemNumber(ptr.Pointer(), PointerFromQTextBlock(block))))
	}
	return 0
}

func (ptr *QTextList) ItemText(block QTextBlock_ITF) string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextList_ItemText(ptr.Pointer(), PointerFromQTextBlock(block)))
	}
	return ""
}

func (ptr *QTextList) Remove(block QTextBlock_ITF) {
	if ptr.Pointer() != nil {
		C.QTextList_Remove(ptr.Pointer(), PointerFromQTextBlock(block))
	}
}

func (ptr *QTextList) RemoveItem(i int) {
	if ptr.Pointer() != nil {
		C.QTextList_RemoveItem(ptr.Pointer(), C.int(int32(i)))
	}
}

func (ptr *QTextList) SetFormat(format QTextListFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextList_SetFormat(ptr.Pointer(), PointerFromQTextListFormat(format))
	}
}

type QTextListFormat struct {
	QTextFormat
}

type QTextListFormat_ITF interface {
	QTextFormat_ITF
	QTextListFormat_PTR() *QTextListFormat
}

func (ptr *QTextListFormat) QTextListFormat_PTR() *QTextListFormat {
	return ptr
}

func (ptr *QTextListFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFormat_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextListFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextFormat_PTR().SetPointer(p)
	}
}

func PointerFromQTextListFormat(ptr QTextListFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextListFormat_PTR().Pointer()
	}
	return nil
}

func NewQTextListFormatFromPointer(ptr unsafe.Pointer) (n *QTextListFormat) {
	n = new(QTextListFormat)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextListFormat) DestroyQTextListFormat() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QTextListFormat__Style
//QTextListFormat::Style
type QTextListFormat__Style int64

var (
	QTextListFormat__ListDisc           QTextListFormat__Style = QTextListFormat__Style(-1)
	QTextListFormat__ListCircle         QTextListFormat__Style = QTextListFormat__Style(-2)
	QTextListFormat__ListSquare         QTextListFormat__Style = QTextListFormat__Style(-3)
	QTextListFormat__ListDecimal        QTextListFormat__Style = QTextListFormat__Style(-4)
	QTextListFormat__ListLowerAlpha     QTextListFormat__Style = QTextListFormat__Style(-5)
	QTextListFormat__ListUpperAlpha     QTextListFormat__Style = QTextListFormat__Style(-6)
	QTextListFormat__ListLowerRoman     QTextListFormat__Style = QTextListFormat__Style(-7)
	QTextListFormat__ListUpperRoman     QTextListFormat__Style = QTextListFormat__Style(-8)
	QTextListFormat__ListStyleUndefined QTextListFormat__Style = QTextListFormat__Style(0)
)

func NewQTextListFormat() *QTextListFormat {
	tmpValue := NewQTextListFormatFromPointer(C.QTextListFormat_NewQTextListFormat())
	qt.SetFinalizer(tmpValue, (*QTextListFormat).DestroyQTextListFormat)
	return tmpValue
}

func (ptr *QTextListFormat) Indent() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextListFormat_Indent(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextListFormat) NumberPrefix() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextListFormat_NumberPrefix(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextListFormat) NumberSuffix() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTextListFormat_NumberSuffix(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTextListFormat) SetIndent(indentation int) {
	if ptr.Pointer() != nil {
		C.QTextListFormat_SetIndent(ptr.Pointer(), C.int(int32(indentation)))
	}
}

func (ptr *QTextListFormat) SetNumberPrefix(numberPrefix string) {
	if ptr.Pointer() != nil {
		var numberPrefixC *C.char
		if numberPrefix != "" {
			numberPrefixC = C.CString(numberPrefix)
			defer C.free(unsafe.Pointer(numberPrefixC))
		}
		C.QTextListFormat_SetNumberPrefix(ptr.Pointer(), C.struct_QtGui_PackedString{data: numberPrefixC, len: C.longlong(len(numberPrefix))})
	}
}

func (ptr *QTextListFormat) SetNumberSuffix(numberSuffix string) {
	if ptr.Pointer() != nil {
		var numberSuffixC *C.char
		if numberSuffix != "" {
			numberSuffixC = C.CString(numberSuffix)
			defer C.free(unsafe.Pointer(numberSuffixC))
		}
		C.QTextListFormat_SetNumberSuffix(ptr.Pointer(), C.struct_QtGui_PackedString{data: numberSuffixC, len: C.longlong(len(numberSuffix))})
	}
}

func (ptr *QTextListFormat) SetStyle(style QTextListFormat__Style) {
	if ptr.Pointer() != nil {
		C.QTextListFormat_SetStyle(ptr.Pointer(), C.longlong(style))
	}
}

func (ptr *QTextListFormat) Style() QTextListFormat__Style {
	if ptr.Pointer() != nil {
		return QTextListFormat__Style(C.QTextListFormat_Style(ptr.Pointer()))
	}
	return 0
}

type QTextObject struct {
	core.QObject
}

type QTextObject_ITF interface {
	core.QObject_ITF
	QTextObject_PTR() *QTextObject
}

func (ptr *QTextObject) QTextObject_PTR() *QTextObject {
	return ptr
}

func (ptr *QTextObject) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextObject) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQTextObject(ptr QTextObject_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextObject_PTR().Pointer()
	}
	return nil
}

func NewQTextObjectFromPointer(ptr unsafe.Pointer) (n *QTextObject) {
	n = new(QTextObject)
	n.SetPointer(ptr)
	return
}
func NewQTextObject(document QTextDocument_ITF) *QTextObject {
	tmpValue := NewQTextObjectFromPointer(C.QTextObject_NewQTextObject(PointerFromQTextDocument(document)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QTextObject) Document() *QTextDocument {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextDocumentFromPointer(C.QTextObject_Document(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextObject) Format() *QTextFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextFormatFromPointer(C.QTextObject_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextFormat).DestroyQTextFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextObject) FormatIndex() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextObject_FormatIndex(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextObject) ObjectIndex() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextObject_ObjectIndex(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextObject) SetFormat(format QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObject_SetFormat(ptr.Pointer(), PointerFromQTextFormat(format))
	}
}

//export callbackQTextObject_DestroyQTextObject
func callbackQTextObject_DestroyQTextObject(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QTextObject"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextObjectFromPointer(ptr).DestroyQTextObjectDefault()
	}
}

func (ptr *QTextObject) ConnectDestroyQTextObject(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QTextObject"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QTextObject", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QTextObject", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextObject) DisconnectDestroyQTextObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QTextObject")
	}
}

func (ptr *QTextObject) DestroyQTextObject() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextObject_DestroyQTextObject(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextObject) DestroyQTextObjectDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextObject_DestroyQTextObjectDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextObject) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QTextObject___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextObject) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObject___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QTextObject) __children_newList() unsafe.Pointer {
	return C.QTextObject___children_newList(ptr.Pointer())
}

func (ptr *QTextObject) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QTextObject___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QTextObject) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObject___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QTextObject) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QTextObject___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QTextObject) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QTextObject___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextObject) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObject___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QTextObject) __findChildren_newList() unsafe.Pointer {
	return C.QTextObject___findChildren_newList(ptr.Pointer())
}

func (ptr *QTextObject) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QTextObject___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTextObject) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObject___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QTextObject) __findChildren_newList3() unsafe.Pointer {
	return C.QTextObject___findChildren_newList3(ptr.Pointer())
}

//export callbackQTextObject_ChildEvent
func callbackQTextObject_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQTextObjectFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QTextObject) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObject_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQTextObject_ConnectNotify
func callbackQTextObject_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQTextObjectFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QTextObject) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObject_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQTextObject_CustomEvent
func callbackQTextObject_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQTextObjectFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QTextObject) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObject_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQTextObject_DeleteLater
func callbackQTextObject_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextObjectFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QTextObject) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextObject_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQTextObject_Destroyed
func callbackQTextObject_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQTextObject_DisconnectNotify
func callbackQTextObject_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQTextObjectFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QTextObject) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObject_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQTextObject_Event
func callbackQTextObject_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQTextObjectFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QTextObject) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextObject_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQTextObject_EventFilter
func callbackQTextObject_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQTextObjectFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QTextObject) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextObject_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQTextObject_MetaObject
func callbackQTextObject_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQTextObjectFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QTextObject) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QTextObject_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQTextObject_ObjectNameChanged
func callbackQTextObject_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQTextObject_TimerEvent
func callbackQTextObject_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQTextObjectFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QTextObject) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObject_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QTextObjectInterface struct {
	ptr unsafe.Pointer
}

type QTextObjectInterface_ITF interface {
	QTextObjectInterface_PTR() *QTextObjectInterface
}

func (ptr *QTextObjectInterface) QTextObjectInterface_PTR() *QTextObjectInterface {
	return ptr
}

func (ptr *QTextObjectInterface) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextObjectInterface) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextObjectInterface(ptr QTextObjectInterface_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextObjectInterface_PTR().Pointer()
	}
	return nil
}

func NewQTextObjectInterfaceFromPointer(ptr unsafe.Pointer) (n *QTextObjectInterface) {
	n = new(QTextObjectInterface)
	n.SetPointer(ptr)
	return
}

//export callbackQTextObjectInterface_DrawObject
func callbackQTextObjectInterface_DrawObject(ptr unsafe.Pointer, painter unsafe.Pointer, rect unsafe.Pointer, doc unsafe.Pointer, posInDocument C.int, format unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "drawObject"); signal != nil {
		(*(*func(*QPainter, *core.QRectF, *QTextDocument, int, *QTextFormat))(signal))(NewQPainterFromPointer(painter), core.NewQRectFFromPointer(rect), NewQTextDocumentFromPointer(doc), int(int32(posInDocument)), NewQTextFormatFromPointer(format))
	}

}

func (ptr *QTextObjectInterface) ConnectDrawObject(f func(painter *QPainter, rect *core.QRectF, doc *QTextDocument, posInDocument int, format *QTextFormat)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "drawObject"); signal != nil {
			f := func(painter *QPainter, rect *core.QRectF, doc *QTextDocument, posInDocument int, format *QTextFormat) {
				(*(*func(*QPainter, *core.QRectF, *QTextDocument, int, *QTextFormat))(signal))(painter, rect, doc, posInDocument, format)
				f(painter, rect, doc, posInDocument, format)
			}
			qt.ConnectSignal(ptr.Pointer(), "drawObject", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "drawObject", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextObjectInterface) DisconnectDrawObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "drawObject")
	}
}

func (ptr *QTextObjectInterface) DrawObject(painter QPainter_ITF, rect core.QRectF_ITF, doc QTextDocument_ITF, posInDocument int, format QTextFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextObjectInterface_DrawObject(ptr.Pointer(), PointerFromQPainter(painter), core.PointerFromQRectF(rect), PointerFromQTextDocument(doc), C.int(int32(posInDocument)), PointerFromQTextFormat(format))
	}
}

//export callbackQTextObjectInterface_IntrinsicSize
func callbackQTextObjectInterface_IntrinsicSize(ptr unsafe.Pointer, doc unsafe.Pointer, posInDocument C.int, format unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "intrinsicSize"); signal != nil {
		return core.PointerFromQSizeF((*(*func(*QTextDocument, int, *QTextFormat) *core.QSizeF)(signal))(NewQTextDocumentFromPointer(doc), int(int32(posInDocument)), NewQTextFormatFromPointer(format)))
	}

	return core.PointerFromQSizeF(core.NewQSizeF())
}

func (ptr *QTextObjectInterface) ConnectIntrinsicSize(f func(doc *QTextDocument, posInDocument int, format *QTextFormat) *core.QSizeF) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "intrinsicSize"); signal != nil {
			f := func(doc *QTextDocument, posInDocument int, format *QTextFormat) *core.QSizeF {
				(*(*func(*QTextDocument, int, *QTextFormat) *core.QSizeF)(signal))(doc, posInDocument, format)
				return f(doc, posInDocument, format)
			}
			qt.ConnectSignal(ptr.Pointer(), "intrinsicSize", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "intrinsicSize", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextObjectInterface) DisconnectIntrinsicSize() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "intrinsicSize")
	}
}

func (ptr *QTextObjectInterface) IntrinsicSize(doc QTextDocument_ITF, posInDocument int, format QTextFormat_ITF) *core.QSizeF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFFromPointer(C.QTextObjectInterface_IntrinsicSize(ptr.Pointer(), PointerFromQTextDocument(doc), C.int(int32(posInDocument)), PointerFromQTextFormat(format)))
		qt.SetFinalizer(tmpValue, (*core.QSizeF).DestroyQSizeF)
		return tmpValue
	}
	return nil
}

//export callbackQTextObjectInterface_DestroyQTextObjectInterface
func callbackQTextObjectInterface_DestroyQTextObjectInterface(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QTextObjectInterface"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTextObjectInterfaceFromPointer(ptr).DestroyQTextObjectInterfaceDefault()
	}
}

func (ptr *QTextObjectInterface) ConnectDestroyQTextObjectInterface(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QTextObjectInterface"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QTextObjectInterface", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QTextObjectInterface", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTextObjectInterface) DisconnectDestroyQTextObjectInterface() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QTextObjectInterface")
	}
}

func (ptr *QTextObjectInterface) DestroyQTextObjectInterface() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextObjectInterface_DestroyQTextObjectInterface(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextObjectInterface) DestroyQTextObjectInterfaceDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextObjectInterface_DestroyQTextObjectInterfaceDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QTextOption struct {
	ptr unsafe.Pointer
}

type QTextOption_ITF interface {
	QTextOption_PTR() *QTextOption
}

func (ptr *QTextOption) QTextOption_PTR() *QTextOption {
	return ptr
}

func (ptr *QTextOption) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextOption) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextOption(ptr QTextOption_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextOption_PTR().Pointer()
	}
	return nil
}

func NewQTextOptionFromPointer(ptr unsafe.Pointer) (n *QTextOption) {
	n = new(QTextOption)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QTextOption__TabType
//QTextOption::TabType
type QTextOption__TabType int64

const (
	QTextOption__LeftTab      QTextOption__TabType = QTextOption__TabType(0)
	QTextOption__RightTab     QTextOption__TabType = QTextOption__TabType(1)
	QTextOption__CenterTab    QTextOption__TabType = QTextOption__TabType(2)
	QTextOption__DelimiterTab QTextOption__TabType = QTextOption__TabType(3)
)

//go:generate stringer -type=QTextOption__WrapMode
//QTextOption::WrapMode
type QTextOption__WrapMode int64

const (
	QTextOption__NoWrap                       QTextOption__WrapMode = QTextOption__WrapMode(0)
	QTextOption__WordWrap                     QTextOption__WrapMode = QTextOption__WrapMode(1)
	QTextOption__ManualWrap                   QTextOption__WrapMode = QTextOption__WrapMode(2)
	QTextOption__WrapAnywhere                 QTextOption__WrapMode = QTextOption__WrapMode(3)
	QTextOption__WrapAtWordBoundaryOrAnywhere QTextOption__WrapMode = QTextOption__WrapMode(4)
)

//go:generate stringer -type=QTextOption__Flag
//QTextOption::Flag
type QTextOption__Flag int64

const (
	QTextOption__ShowTabsAndSpaces                     QTextOption__Flag = QTextOption__Flag(0x1)
	QTextOption__ShowLineAndParagraphSeparators        QTextOption__Flag = QTextOption__Flag(0x2)
	QTextOption__AddSpaceForLineAndParagraphSeparators QTextOption__Flag = QTextOption__Flag(0x4)
	QTextOption__SuppressColors                        QTextOption__Flag = QTextOption__Flag(0x8)
	QTextOption__ShowDocumentTerminator                QTextOption__Flag = QTextOption__Flag(0x10)
	QTextOption__IncludeTrailingSpaces                 QTextOption__Flag = QTextOption__Flag(0x80000000)
)

func NewQTextOption() *QTextOption {
	tmpValue := NewQTextOptionFromPointer(C.QTextOption_NewQTextOption())
	qt.SetFinalizer(tmpValue, (*QTextOption).DestroyQTextOption)
	return tmpValue
}

func NewQTextOption2(alignment core.Qt__AlignmentFlag) *QTextOption {
	tmpValue := NewQTextOptionFromPointer(C.QTextOption_NewQTextOption2(C.longlong(alignment)))
	qt.SetFinalizer(tmpValue, (*QTextOption).DestroyQTextOption)
	return tmpValue
}

func NewQTextOption3(other QTextOption_ITF) *QTextOption {
	tmpValue := NewQTextOptionFromPointer(C.QTextOption_NewQTextOption3(PointerFromQTextOption(other)))
	qt.SetFinalizer(tmpValue, (*QTextOption).DestroyQTextOption)
	return tmpValue
}

func (ptr *QTextOption) Alignment() core.Qt__AlignmentFlag {
	if ptr.Pointer() != nil {
		return core.Qt__AlignmentFlag(C.QTextOption_Alignment(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextOption) Flags() QTextOption__Flag {
	if ptr.Pointer() != nil {
		return QTextOption__Flag(C.QTextOption_Flags(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextOption) SetAlignment(alignment core.Qt__AlignmentFlag) {
	if ptr.Pointer() != nil {
		C.QTextOption_SetAlignment(ptr.Pointer(), C.longlong(alignment))
	}
}

func (ptr *QTextOption) SetFlags(flags QTextOption__Flag) {
	if ptr.Pointer() != nil {
		C.QTextOption_SetFlags(ptr.Pointer(), C.longlong(flags))
	}
}

func (ptr *QTextOption) SetTabArray(tabStops []float64) {
	if ptr.Pointer() != nil {
		C.QTextOption_SetTabArray(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQTextOptionFromPointer(NewQTextOptionFromPointer(nil).__setTabArray_tabStops_newList())
			for _, v := range tabStops {
				tmpList.__setTabArray_tabStops_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QTextOption) SetTabStopDistance(tabStopDistance float64) {
	if ptr.Pointer() != nil {
		C.QTextOption_SetTabStopDistance(ptr.Pointer(), C.double(tabStopDistance))
	}
}

func (ptr *QTextOption) SetTextDirection(direction core.Qt__LayoutDirection) {
	if ptr.Pointer() != nil {
		C.QTextOption_SetTextDirection(ptr.Pointer(), C.longlong(direction))
	}
}

func (ptr *QTextOption) SetUseDesignMetrics(enable bool) {
	if ptr.Pointer() != nil {
		C.QTextOption_SetUseDesignMetrics(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(enable))))
	}
}

func (ptr *QTextOption) SetWrapMode(mode QTextOption__WrapMode) {
	if ptr.Pointer() != nil {
		C.QTextOption_SetWrapMode(ptr.Pointer(), C.longlong(mode))
	}
}

func (ptr *QTextOption) TabArray() []float64 {
	if ptr.Pointer() != nil {
		return func(l C.struct_QtGui_PackedList) []float64 {
			out := make([]float64, int(l.len))
			tmpList := NewQTextOptionFromPointer(l.data)
			for i := 0; i < len(out); i++ {
				out[i] = tmpList.__tabArray_atList(i)
			}
			return out
		}(C.QTextOption_TabArray(ptr.Pointer()))
	}
	return make([]float64, 0)
}

func (ptr *QTextOption) TabStopDistance() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextOption_TabStopDistance(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextOption) TextDirection() core.Qt__LayoutDirection {
	if ptr.Pointer() != nil {
		return core.Qt__LayoutDirection(C.QTextOption_TextDirection(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextOption) UseDesignMetrics() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextOption_UseDesignMetrics(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextOption) WrapMode() QTextOption__WrapMode {
	if ptr.Pointer() != nil {
		return QTextOption__WrapMode(C.QTextOption_WrapMode(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextOption) DestroyQTextOption() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextOption_DestroyQTextOption(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTextOption) __setTabArray_tabStops_atList(i int) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextOption___setTabArray_tabStops_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QTextOption) __setTabArray_tabStops_setList(i float64) {
	if ptr.Pointer() != nil {
		C.QTextOption___setTabArray_tabStops_setList(ptr.Pointer(), C.double(i))
	}
}

func (ptr *QTextOption) __setTabArray_tabStops_newList() unsafe.Pointer {
	return C.QTextOption___setTabArray_tabStops_newList(ptr.Pointer())
}

func (ptr *QTextOption) __tabArray_atList(i int) float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextOption___tabArray_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return 0
}

func (ptr *QTextOption) __tabArray_setList(i float64) {
	if ptr.Pointer() != nil {
		C.QTextOption___tabArray_setList(ptr.Pointer(), C.double(i))
	}
}

func (ptr *QTextOption) __tabArray_newList() unsafe.Pointer {
	return C.QTextOption___tabArray_newList(ptr.Pointer())
}

type QTextTable struct {
	QTextFrame
}

type QTextTable_ITF interface {
	QTextFrame_ITF
	QTextTable_PTR() *QTextTable
}

func (ptr *QTextTable) QTextTable_PTR() *QTextTable {
	return ptr
}

func (ptr *QTextTable) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFrame_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextTable) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextFrame_PTR().SetPointer(p)
	}
}

func PointerFromQTextTable(ptr QTextTable_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextTable_PTR().Pointer()
	}
	return nil
}

func NewQTextTableFromPointer(ptr unsafe.Pointer) (n *QTextTable) {
	n = new(QTextTable)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextTable) AppendColumns(count int) {
	if ptr.Pointer() != nil {
		C.QTextTable_AppendColumns(ptr.Pointer(), C.int(int32(count)))
	}
}

func (ptr *QTextTable) AppendRows(count int) {
	if ptr.Pointer() != nil {
		C.QTextTable_AppendRows(ptr.Pointer(), C.int(int32(count)))
	}
}

func (ptr *QTextTable) CellAt(row int, column int) *QTextTableCell {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextTableCellFromPointer(C.QTextTable_CellAt(ptr.Pointer(), C.int(int32(row)), C.int(int32(column))))
		qt.SetFinalizer(tmpValue, (*QTextTableCell).DestroyQTextTableCell)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTable) CellAt2(position int) *QTextTableCell {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextTableCellFromPointer(C.QTextTable_CellAt2(ptr.Pointer(), C.int(int32(position))))
		qt.SetFinalizer(tmpValue, (*QTextTableCell).DestroyQTextTableCell)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTable) CellAt3(cursor QTextCursor_ITF) *QTextTableCell {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextTableCellFromPointer(C.QTextTable_CellAt3(ptr.Pointer(), PointerFromQTextCursor(cursor)))
		qt.SetFinalizer(tmpValue, (*QTextTableCell).DestroyQTextTableCell)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTable) Columns() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextTable_Columns(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextTable) Format() *QTextTableFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextTableFormatFromPointer(C.QTextTable_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextTableFormat).DestroyQTextTableFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTable) InsertColumns(index int, columns int) {
	if ptr.Pointer() != nil {
		C.QTextTable_InsertColumns(ptr.Pointer(), C.int(int32(index)), C.int(int32(columns)))
	}
}

func (ptr *QTextTable) InsertRows(index int, rows int) {
	if ptr.Pointer() != nil {
		C.QTextTable_InsertRows(ptr.Pointer(), C.int(int32(index)), C.int(int32(rows)))
	}
}

func (ptr *QTextTable) MergeCells(row int, column int, numRows int, numCols int) {
	if ptr.Pointer() != nil {
		C.QTextTable_MergeCells(ptr.Pointer(), C.int(int32(row)), C.int(int32(column)), C.int(int32(numRows)), C.int(int32(numCols)))
	}
}

func (ptr *QTextTable) MergeCells2(cursor QTextCursor_ITF) {
	if ptr.Pointer() != nil {
		C.QTextTable_MergeCells2(ptr.Pointer(), PointerFromQTextCursor(cursor))
	}
}

func (ptr *QTextTable) RemoveColumns(index int, columns int) {
	if ptr.Pointer() != nil {
		C.QTextTable_RemoveColumns(ptr.Pointer(), C.int(int32(index)), C.int(int32(columns)))
	}
}

func (ptr *QTextTable) RemoveRows(index int, rows int) {
	if ptr.Pointer() != nil {
		C.QTextTable_RemoveRows(ptr.Pointer(), C.int(int32(index)), C.int(int32(rows)))
	}
}

func (ptr *QTextTable) Resize(rows int, columns int) {
	if ptr.Pointer() != nil {
		C.QTextTable_Resize(ptr.Pointer(), C.int(int32(rows)), C.int(int32(columns)))
	}
}

func (ptr *QTextTable) RowEnd(cursor QTextCursor_ITF) *QTextCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCursorFromPointer(C.QTextTable_RowEnd(ptr.Pointer(), PointerFromQTextCursor(cursor)))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTable) RowStart(cursor QTextCursor_ITF) *QTextCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCursorFromPointer(C.QTextTable_RowStart(ptr.Pointer(), PointerFromQTextCursor(cursor)))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTable) Rows() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextTable_Rows(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextTable) SetFormat(format QTextTableFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextTable_SetFormat(ptr.Pointer(), PointerFromQTextTableFormat(format))
	}
}

func (ptr *QTextTable) SplitCell(row int, column int, numRows int, numCols int) {
	if ptr.Pointer() != nil {
		C.QTextTable_SplitCell(ptr.Pointer(), C.int(int32(row)), C.int(int32(column)), C.int(int32(numRows)), C.int(int32(numCols)))
	}
}

type QTextTableCell struct {
	ptr unsafe.Pointer
}

type QTextTableCell_ITF interface {
	QTextTableCell_PTR() *QTextTableCell
}

func (ptr *QTextTableCell) QTextTableCell_PTR() *QTextTableCell {
	return ptr
}

func (ptr *QTextTableCell) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTextTableCell) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTextTableCell(ptr QTextTableCell_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextTableCell_PTR().Pointer()
	}
	return nil
}

func NewQTextTableCellFromPointer(ptr unsafe.Pointer) (n *QTextTableCell) {
	n = new(QTextTableCell)
	n.SetPointer(ptr)
	return
}
func NewQTextTableCell() *QTextTableCell {
	tmpValue := NewQTextTableCellFromPointer(C.QTextTableCell_NewQTextTableCell())
	qt.SetFinalizer(tmpValue, (*QTextTableCell).DestroyQTextTableCell)
	return tmpValue
}

func NewQTextTableCell2(other QTextTableCell_ITF) *QTextTableCell {
	tmpValue := NewQTextTableCellFromPointer(C.QTextTableCell_NewQTextTableCell2(PointerFromQTextTableCell(other)))
	qt.SetFinalizer(tmpValue, (*QTextTableCell).DestroyQTextTableCell)
	return tmpValue
}

func (ptr *QTextTableCell) Column() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextTableCell_Column(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextTableCell) ColumnSpan() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextTableCell_ColumnSpan(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextTableCell) FirstCursorPosition() *QTextCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCursorFromPointer(C.QTextTableCell_FirstCursorPosition(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTableCell) Format() *QTextCharFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCharFormatFromPointer(C.QTextTableCell_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextCharFormat).DestroyQTextCharFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTableCell) IsValid() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTextTableCell_IsValid(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTextTableCell) LastCursorPosition() *QTextCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextCursorFromPointer(C.QTextTableCell_LastCursorPosition(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTextCursor).DestroyQTextCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTableCell) Row() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextTableCell_Row(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextTableCell) RowSpan() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextTableCell_RowSpan(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextTableCell) SetFormat(format QTextCharFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QTextTableCell_SetFormat(ptr.Pointer(), PointerFromQTextCharFormat(format))
	}
}

func (ptr *QTextTableCell) TableCellFormatIndex() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextTableCell_TableCellFormatIndex(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextTableCell) DestroyQTextTableCell() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTextTableCell_DestroyQTextTableCell(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QTextTableCellFormat struct {
	QTextCharFormat
}

type QTextTableCellFormat_ITF interface {
	QTextCharFormat_ITF
	QTextTableCellFormat_PTR() *QTextTableCellFormat
}

func (ptr *QTextTableCellFormat) QTextTableCellFormat_PTR() *QTextTableCellFormat {
	return ptr
}

func (ptr *QTextTableCellFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextCharFormat_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextTableCellFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextCharFormat_PTR().SetPointer(p)
	}
}

func PointerFromQTextTableCellFormat(ptr QTextTableCellFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextTableCellFormat_PTR().Pointer()
	}
	return nil
}

func NewQTextTableCellFormatFromPointer(ptr unsafe.Pointer) (n *QTextTableCellFormat) {
	n = new(QTextTableCellFormat)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextTableCellFormat) DestroyQTextTableCellFormat() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQTextTableCellFormat() *QTextTableCellFormat {
	tmpValue := NewQTextTableCellFormatFromPointer(C.QTextTableCellFormat_NewQTextTableCellFormat())
	qt.SetFinalizer(tmpValue, (*QTextTableCellFormat).DestroyQTextTableCellFormat)
	return tmpValue
}

func (ptr *QTextTableCellFormat) BottomPadding() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextTableCellFormat_BottomPadding(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextTableCellFormat) LeftPadding() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextTableCellFormat_LeftPadding(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextTableCellFormat) RightPadding() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextTableCellFormat_RightPadding(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextTableCellFormat) SetBottomPadding(padding float64) {
	if ptr.Pointer() != nil {
		C.QTextTableCellFormat_SetBottomPadding(ptr.Pointer(), C.double(padding))
	}
}

func (ptr *QTextTableCellFormat) SetLeftPadding(padding float64) {
	if ptr.Pointer() != nil {
		C.QTextTableCellFormat_SetLeftPadding(ptr.Pointer(), C.double(padding))
	}
}

func (ptr *QTextTableCellFormat) SetPadding(padding float64) {
	if ptr.Pointer() != nil {
		C.QTextTableCellFormat_SetPadding(ptr.Pointer(), C.double(padding))
	}
}

func (ptr *QTextTableCellFormat) SetRightPadding(padding float64) {
	if ptr.Pointer() != nil {
		C.QTextTableCellFormat_SetRightPadding(ptr.Pointer(), C.double(padding))
	}
}

func (ptr *QTextTableCellFormat) SetTopPadding(padding float64) {
	if ptr.Pointer() != nil {
		C.QTextTableCellFormat_SetTopPadding(ptr.Pointer(), C.double(padding))
	}
}

func (ptr *QTextTableCellFormat) TopPadding() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextTableCellFormat_TopPadding(ptr.Pointer()))
	}
	return 0
}

type QTextTableFormat struct {
	QTextFrameFormat
}

type QTextTableFormat_ITF interface {
	QTextFrameFormat_ITF
	QTextTableFormat_PTR() *QTextTableFormat
}

func (ptr *QTextTableFormat) QTextTableFormat_PTR() *QTextTableFormat {
	return ptr
}

func (ptr *QTextTableFormat) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextFrameFormat_PTR().Pointer()
	}
	return nil
}

func (ptr *QTextTableFormat) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QTextFrameFormat_PTR().SetPointer(p)
	}
}

func PointerFromQTextTableFormat(ptr QTextTableFormat_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTextTableFormat_PTR().Pointer()
	}
	return nil
}

func NewQTextTableFormatFromPointer(ptr unsafe.Pointer) (n *QTextTableFormat) {
	n = new(QTextTableFormat)
	n.SetPointer(ptr)
	return
}
func (ptr *QTextTableFormat) DestroyQTextTableFormat() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQTextTableFormat() *QTextTableFormat {
	tmpValue := NewQTextTableFormatFromPointer(C.QTextTableFormat_NewQTextTableFormat())
	qt.SetFinalizer(tmpValue, (*QTextTableFormat).DestroyQTextTableFormat)
	return tmpValue
}

func (ptr *QTextTableFormat) Alignment() core.Qt__AlignmentFlag {
	if ptr.Pointer() != nil {
		return core.Qt__AlignmentFlag(C.QTextTableFormat_Alignment(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextTableFormat) CellPadding() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextTableFormat_CellPadding(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextTableFormat) CellSpacing() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTextTableFormat_CellSpacing(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTextTableFormat) ClearColumnWidthConstraints() {
	if ptr.Pointer() != nil {
		C.QTextTableFormat_ClearColumnWidthConstraints(ptr.Pointer())
	}
}

func (ptr *QTextTableFormat) Columns() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextTableFormat_Columns(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextTableFormat) HeaderRowCount() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTextTableFormat_HeaderRowCount(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTextTableFormat) SetAlignment(alignment core.Qt__AlignmentFlag) {
	if ptr.Pointer() != nil {
		C.QTextTableFormat_SetAlignment(ptr.Pointer(), C.longlong(alignment))
	}
}

func (ptr *QTextTableFormat) SetCellPadding(padding float64) {
	if ptr.Pointer() != nil {
		C.QTextTableFormat_SetCellPadding(ptr.Pointer(), C.double(padding))
	}
}

func (ptr *QTextTableFormat) SetCellSpacing(spacing float64) {
	if ptr.Pointer() != nil {
		C.QTextTableFormat_SetCellSpacing(ptr.Pointer(), C.double(spacing))
	}
}

func (ptr *QTextTableFormat) SetColumnWidthConstraints(constraints []*QTextLength) {
	if ptr.Pointer() != nil {
		C.QTextTableFormat_SetColumnWidthConstraints(ptr.Pointer(), func() unsafe.Pointer {
			tmpList := NewQTextTableFormatFromPointer(NewQTextTableFormatFromPointer(nil).__setColumnWidthConstraints_constraints_newList())
			for _, v := range constraints {
				tmpList.__setColumnWidthConstraints_constraints_setList(v)
			}
			return tmpList.Pointer()
		}())
	}
}

func (ptr *QTextTableFormat) SetHeaderRowCount(count int) {
	if ptr.Pointer() != nil {
		C.QTextTableFormat_SetHeaderRowCount(ptr.Pointer(), C.int(int32(count)))
	}
}

func (ptr *QTextTableFormat) __columnWidthConstraints_atList(i int) *QTextLength {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextLengthFromPointer(C.QTextTableFormat___columnWidthConstraints_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QTextLength).DestroyQTextLength)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTableFormat) __columnWidthConstraints_setList(i QTextLength_ITF) {
	if ptr.Pointer() != nil {
		C.QTextTableFormat___columnWidthConstraints_setList(ptr.Pointer(), PointerFromQTextLength(i))
	}
}

func (ptr *QTextTableFormat) __columnWidthConstraints_newList() unsafe.Pointer {
	return C.QTextTableFormat___columnWidthConstraints_newList(ptr.Pointer())
}

func (ptr *QTextTableFormat) __setColumnWidthConstraints_constraints_atList(i int) *QTextLength {
	if ptr.Pointer() != nil {
		tmpValue := NewQTextLengthFromPointer(C.QTextTableFormat___setColumnWidthConstraints_constraints_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*QTextLength).DestroyQTextLength)
		return tmpValue
	}
	return nil
}

func (ptr *QTextTableFormat) __setColumnWidthConstraints_constraints_setList(i QTextLength_ITF) {
	if ptr.Pointer() != nil {
		C.QTextTableFormat___setColumnWidthConstraints_constraints_setList(ptr.Pointer(), PointerFromQTextLength(i))
	}
}

func (ptr *QTextTableFormat) __setColumnWidthConstraints_constraints_newList() unsafe.Pointer {
	return C.QTextTableFormat___setColumnWidthConstraints_constraints_newList(ptr.Pointer())
}

type QTouchDevice struct {
	ptr unsafe.Pointer
}

type QTouchDevice_ITF interface {
	QTouchDevice_PTR() *QTouchDevice
}

func (ptr *QTouchDevice) QTouchDevice_PTR() *QTouchDevice {
	return ptr
}

func (ptr *QTouchDevice) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTouchDevice) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTouchDevice(ptr QTouchDevice_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTouchDevice_PTR().Pointer()
	}
	return nil
}

func NewQTouchDeviceFromPointer(ptr unsafe.Pointer) (n *QTouchDevice) {
	n = new(QTouchDevice)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QTouchDevice__DeviceType
//QTouchDevice::DeviceType
type QTouchDevice__DeviceType int64

const (
	QTouchDevice__TouchScreen QTouchDevice__DeviceType = QTouchDevice__DeviceType(0)
	QTouchDevice__TouchPad    QTouchDevice__DeviceType = QTouchDevice__DeviceType(1)
)

//go:generate stringer -type=QTouchDevice__CapabilityFlag
//QTouchDevice::CapabilityFlag
type QTouchDevice__CapabilityFlag int64

const (
	QTouchDevice__Position           QTouchDevice__CapabilityFlag = QTouchDevice__CapabilityFlag(0x0001)
	QTouchDevice__Area               QTouchDevice__CapabilityFlag = QTouchDevice__CapabilityFlag(0x0002)
	QTouchDevice__Pressure           QTouchDevice__CapabilityFlag = QTouchDevice__CapabilityFlag(0x0004)
	QTouchDevice__Velocity           QTouchDevice__CapabilityFlag = QTouchDevice__CapabilityFlag(0x0008)
	QTouchDevice__RawPositions       QTouchDevice__CapabilityFlag = QTouchDevice__CapabilityFlag(0x0010)
	QTouchDevice__NormalizedPosition QTouchDevice__CapabilityFlag = QTouchDevice__CapabilityFlag(0x0020)
	QTouchDevice__MouseEmulation     QTouchDevice__CapabilityFlag = QTouchDevice__CapabilityFlag(0x0040)
)

func NewQTouchDevice() *QTouchDevice {
	tmpValue := NewQTouchDeviceFromPointer(C.QTouchDevice_NewQTouchDevice())
	qt.SetFinalizer(tmpValue, (*QTouchDevice).DestroyQTouchDevice)
	return tmpValue
}

func (ptr *QTouchDevice) Capabilities() QTouchDevice__CapabilityFlag {
	if ptr.Pointer() != nil {
		return QTouchDevice__CapabilityFlag(C.QTouchDevice_Capabilities(ptr.Pointer()))
	}
	return 0
}

func QTouchDevice_Devices() []*QTouchDevice {
	return func(l C.struct_QtGui_PackedList) []*QTouchDevice {
		out := make([]*QTouchDevice, int(l.len))
		tmpList := NewQTouchDeviceFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__devices_atList(i)
		}
		return out
	}(C.QTouchDevice_QTouchDevice_Devices())
}

func (ptr *QTouchDevice) Devices() []*QTouchDevice {
	return func(l C.struct_QtGui_PackedList) []*QTouchDevice {
		out := make([]*QTouchDevice, int(l.len))
		tmpList := NewQTouchDeviceFromPointer(l.data)
		for i := 0; i < len(out); i++ {
			out[i] = tmpList.__devices_atList(i)
		}
		return out
	}(C.QTouchDevice_QTouchDevice_Devices())
}

func (ptr *QTouchDevice) MaximumTouchPoints() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QTouchDevice_MaximumTouchPoints(ptr.Pointer())))
	}
	return 0
}

func (ptr *QTouchDevice) Name() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QTouchDevice_Name(ptr.Pointer()))
	}
	return ""
}

func (ptr *QTouchDevice) SetCapabilities(caps QTouchDevice__CapabilityFlag) {
	if ptr.Pointer() != nil {
		C.QTouchDevice_SetCapabilities(ptr.Pointer(), C.longlong(caps))
	}
}

func (ptr *QTouchDevice) SetMaximumTouchPoints(max int) {
	if ptr.Pointer() != nil {
		C.QTouchDevice_SetMaximumTouchPoints(ptr.Pointer(), C.int(int32(max)))
	}
}

func (ptr *QTouchDevice) SetName(name string) {
	if ptr.Pointer() != nil {
		var nameC *C.char
		if name != "" {
			nameC = C.CString(name)
			defer C.free(unsafe.Pointer(nameC))
		}
		C.QTouchDevice_SetName(ptr.Pointer(), C.struct_QtGui_PackedString{data: nameC, len: C.longlong(len(name))})
	}
}

func (ptr *QTouchDevice) SetType(devType QTouchDevice__DeviceType) {
	if ptr.Pointer() != nil {
		C.QTouchDevice_SetType(ptr.Pointer(), C.longlong(devType))
	}
}

func (ptr *QTouchDevice) Type() QTouchDevice__DeviceType {
	if ptr.Pointer() != nil {
		return QTouchDevice__DeviceType(C.QTouchDevice_Type(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTouchDevice) DestroyQTouchDevice() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTouchDevice_DestroyQTouchDevice(ptr.Pointer())
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTouchDevice) __devices_atList(i int) *QTouchDevice {
	if ptr.Pointer() != nil {
		return NewQTouchDeviceFromPointer(C.QTouchDevice___devices_atList(ptr.Pointer(), C.int(int32(i))))
	}
	return nil
}

func (ptr *QTouchDevice) __devices_setList(i QTouchDevice_ITF) {
	if ptr.Pointer() != nil {
		C.QTouchDevice___devices_setList(ptr.Pointer(), PointerFromQTouchDevice(i))
	}
}

func (ptr *QTouchDevice) __devices_newList() unsafe.Pointer {
	return C.QTouchDevice___devices_newList(ptr.Pointer())
}

type QTouchEvent struct {
	QInputEvent
}

type QTouchEvent_ITF interface {
	QInputEvent_ITF
	QTouchEvent_PTR() *QTouchEvent
}

func (ptr *QTouchEvent) QTouchEvent_PTR() *QTouchEvent {
	return ptr
}

func (ptr *QTouchEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QTouchEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QInputEvent_PTR().SetPointer(p)
	}
}

func PointerFromQTouchEvent(ptr QTouchEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTouchEvent_PTR().Pointer()
	}
	return nil
}

func NewQTouchEventFromPointer(ptr unsafe.Pointer) (n *QTouchEvent) {
	n = new(QTouchEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QTouchEvent) Device() *QTouchDevice {
	if ptr.Pointer() != nil {
		return NewQTouchDeviceFromPointer(C.QTouchEvent_Device(ptr.Pointer()))
	}
	return nil
}

func (ptr *QTouchEvent) Target() *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QTouchEvent_Target(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QTouchEvent) TouchPointStates() core.Qt__TouchPointState {
	if ptr.Pointer() != nil {
		return core.Qt__TouchPointState(C.QTouchEvent_TouchPointStates(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTouchEvent) Window() *QWindow {
	if ptr.Pointer() != nil {
		tmpValue := NewQWindowFromPointer(C.QTouchEvent_Window(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQTouchEvent_DestroyQTouchEvent
func callbackQTouchEvent_DestroyQTouchEvent(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QTouchEvent"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQTouchEventFromPointer(ptr).DestroyQTouchEventDefault()
	}
}

func (ptr *QTouchEvent) ConnectDestroyQTouchEvent(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QTouchEvent"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QTouchEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QTouchEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QTouchEvent) DisconnectDestroyQTouchEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QTouchEvent")
	}
}

func (ptr *QTouchEvent) DestroyQTouchEvent() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTouchEvent_DestroyQTouchEvent(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTouchEvent) DestroyQTouchEventDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QTouchEvent_DestroyQTouchEventDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QTouchEvent) __QTouchEvent_touchPoints_newList() unsafe.Pointer {
	return C.QTouchEvent___QTouchEvent_touchPoints_newList(ptr.Pointer())
}

func (ptr *QTouchEvent) __setTouchPoints_atouchPoints_newList() unsafe.Pointer {
	return C.QTouchEvent___setTouchPoints_atouchPoints_newList(ptr.Pointer())
}

func (ptr *QTouchEvent) __touchPoints_newList() unsafe.Pointer {
	return C.QTouchEvent___touchPoints_newList(ptr.Pointer())
}

func (ptr *QTouchEvent) ___touchPoints_newList() unsafe.Pointer {
	return C.QTouchEvent____touchPoints_newList(ptr.Pointer())
}

func (ptr *QTouchEvent) __set_touchPoints__newList() unsafe.Pointer {
	return C.QTouchEvent___set_touchPoints__newList(ptr.Pointer())
}

type QTransform struct {
	ptr unsafe.Pointer
}

type QTransform_ITF interface {
	QTransform_PTR() *QTransform
}

func (ptr *QTransform) QTransform_PTR() *QTransform {
	return ptr
}

func (ptr *QTransform) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QTransform) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQTransform(ptr QTransform_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QTransform_PTR().Pointer()
	}
	return nil
}

func NewQTransformFromPointer(ptr unsafe.Pointer) (n *QTransform) {
	n = new(QTransform)
	n.SetPointer(ptr)
	return
}
func (ptr *QTransform) DestroyQTransform() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

//go:generate stringer -type=QTransform__TransformationType
//QTransform::TransformationType
type QTransform__TransformationType int64

const (
	QTransform__TxNone      QTransform__TransformationType = QTransform__TransformationType(0x00)
	QTransform__TxTranslate QTransform__TransformationType = QTransform__TransformationType(0x01)
	QTransform__TxScale     QTransform__TransformationType = QTransform__TransformationType(0x02)
	QTransform__TxRotate    QTransform__TransformationType = QTransform__TransformationType(0x04)
	QTransform__TxShear     QTransform__TransformationType = QTransform__TransformationType(0x08)
	QTransform__TxProject   QTransform__TransformationType = QTransform__TransformationType(0x10)
)

func NewQTransform2() *QTransform {
	tmpValue := NewQTransformFromPointer(C.QTransform_NewQTransform2())
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func NewQTransform3(m11 float64, m12 float64, m13 float64, m21 float64, m22 float64, m23 float64, m31 float64, m32 float64, m33 float64) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QTransform_NewQTransform3(C.double(m11), C.double(m12), C.double(m13), C.double(m21), C.double(m22), C.double(m23), C.double(m31), C.double(m32), C.double(m33)))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func NewQTransform4(m11 float64, m12 float64, m21 float64, m22 float64, dx float64, dy float64) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QTransform_NewQTransform4(C.double(m11), C.double(m12), C.double(m21), C.double(m22), C.double(dx), C.double(dy)))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func NewQTransform5(matrix QMatrix_ITF) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QTransform_NewQTransform5(PointerFromQMatrix(matrix)))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func (ptr *QTransform) Adjoint() *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QTransform_Adjoint(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Determinant() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_Determinant(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) Dx() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_Dx(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) Dy() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_Dy(ptr.Pointer()))
	}
	return 0
}

func QTransform_FromScale(sx float64, sy float64) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QTransform_QTransform_FromScale(C.double(sx), C.double(sy)))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func (ptr *QTransform) FromScale(sx float64, sy float64) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QTransform_QTransform_FromScale(C.double(sx), C.double(sy)))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func QTransform_FromTranslate(dx float64, dy float64) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QTransform_QTransform_FromTranslate(C.double(dx), C.double(dy)))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func (ptr *QTransform) FromTranslate(dx float64, dy float64) *QTransform {
	tmpValue := NewQTransformFromPointer(C.QTransform_QTransform_FromTranslate(C.double(dx), C.double(dy)))
	qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
	return tmpValue
}

func (ptr *QTransform) Inverted(invertible *bool) *QTransform {
	if ptr.Pointer() != nil {
		var invertibleC C.char
		if invertible != nil {
			invertibleC = C.char(int8(qt.GoBoolToInt(*invertible)))
			defer func() { *invertible = int8(invertibleC) != 0 }()
		}
		tmpValue := NewQTransformFromPointer(C.QTransform_Inverted(ptr.Pointer(), &invertibleC))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) IsAffine() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTransform_IsAffine(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTransform) IsIdentity() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTransform_IsIdentity(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTransform) IsInvertible() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTransform_IsInvertible(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTransform) IsRotating() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTransform_IsRotating(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTransform) IsScaling() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTransform_IsScaling(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTransform) IsTranslating() bool {
	if ptr.Pointer() != nil {
		return int8(C.QTransform_IsTranslating(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QTransform) M11() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_M11(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) M12() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_M12(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) M13() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_M13(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) M21() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_M21(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) M22() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_M22(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) M23() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_M23(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) M31() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_M31(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) M32() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_M32(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) M33() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QTransform_M33(ptr.Pointer()))
	}
	return 0
}

func (ptr *QTransform) Map(x float64, y float64, tx float64, ty float64) {
	if ptr.Pointer() != nil {
		C.QTransform_Map(ptr.Pointer(), C.double(x), C.double(y), C.double(tx), C.double(ty))
	}
}

func (ptr *QTransform) Map2(point core.QPoint_ITF) *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QTransform_Map2(ptr.Pointer(), core.PointerFromQPoint(point)))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Map3(p core.QPointF_ITF) *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QTransform_Map3(ptr.Pointer(), core.PointerFromQPointF(p)))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Map4(l core.QLine_ITF) *core.QLine {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQLineFromPointer(C.QTransform_Map4(ptr.Pointer(), core.PointerFromQLine(l)))
		qt.SetFinalizer(tmpValue, (*core.QLine).DestroyQLine)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Map5(line core.QLineF_ITF) *core.QLineF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQLineFFromPointer(C.QTransform_Map5(ptr.Pointer(), core.PointerFromQLineF(line)))
		qt.SetFinalizer(tmpValue, (*core.QLineF).DestroyQLineF)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Map6(polygon QPolygonF_ITF) *QPolygonF {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFFromPointer(C.QTransform_Map6(ptr.Pointer(), PointerFromQPolygonF(polygon)))
		qt.SetFinalizer(tmpValue, (*QPolygonF).DestroyQPolygonF)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Map7(polygon QPolygon_ITF) *QPolygon {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFromPointer(C.QTransform_Map7(ptr.Pointer(), PointerFromQPolygon(polygon)))
		qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Map8(region QRegion_ITF) *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QTransform_Map8(ptr.Pointer(), PointerFromQRegion(region)))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Map9(path QPainterPath_ITF) *QPainterPath {
	if ptr.Pointer() != nil {
		tmpValue := NewQPainterPathFromPointer(C.QTransform_Map9(ptr.Pointer(), PointerFromQPainterPath(path)))
		qt.SetFinalizer(tmpValue, (*QPainterPath).DestroyQPainterPath)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Map10(x int, y int, tx int, ty int) {
	if ptr.Pointer() != nil {
		C.QTransform_Map10(ptr.Pointer(), C.int(int32(x)), C.int(int32(y)), C.int(int32(tx)), C.int(int32(ty)))
	}
}

func (ptr *QTransform) MapRect(rectangle core.QRectF_ITF) *core.QRectF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFFromPointer(C.QTransform_MapRect(ptr.Pointer(), core.PointerFromQRectF(rectangle)))
		qt.SetFinalizer(tmpValue, (*core.QRectF).DestroyQRectF)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) MapRect2(rectangle core.QRect_ITF) *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QTransform_MapRect2(ptr.Pointer(), core.PointerFromQRect(rectangle)))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) MapToPolygon(rectangle core.QRect_ITF) *QPolygon {
	if ptr.Pointer() != nil {
		tmpValue := NewQPolygonFromPointer(C.QTransform_MapToPolygon(ptr.Pointer(), core.PointerFromQRect(rectangle)))
		qt.SetFinalizer(tmpValue, (*QPolygon).DestroyQPolygon)
		return tmpValue
	}
	return nil
}

func QTransform_QuadToQuad(one QPolygonF_ITF, two QPolygonF_ITF, trans QTransform_ITF) bool {
	return int8(C.QTransform_QTransform_QuadToQuad(PointerFromQPolygonF(one), PointerFromQPolygonF(two), PointerFromQTransform(trans))) != 0
}

func (ptr *QTransform) QuadToQuad(one QPolygonF_ITF, two QPolygonF_ITF, trans QTransform_ITF) bool {
	return int8(C.QTransform_QTransform_QuadToQuad(PointerFromQPolygonF(one), PointerFromQPolygonF(two), PointerFromQTransform(trans))) != 0
}

func QTransform_QuadToSquare(quad QPolygonF_ITF, trans QTransform_ITF) bool {
	return int8(C.QTransform_QTransform_QuadToSquare(PointerFromQPolygonF(quad), PointerFromQTransform(trans))) != 0
}

func (ptr *QTransform) QuadToSquare(quad QPolygonF_ITF, trans QTransform_ITF) bool {
	return int8(C.QTransform_QTransform_QuadToSquare(PointerFromQPolygonF(quad), PointerFromQTransform(trans))) != 0
}

func (ptr *QTransform) Reset() {
	if ptr.Pointer() != nil {
		C.QTransform_Reset(ptr.Pointer())
	}
}

func (ptr *QTransform) Rotate(angle float64, axis core.Qt__Axis) *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QTransform_Rotate(ptr.Pointer(), C.double(angle), C.longlong(axis)))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) RotateRadians(angle float64, axis core.Qt__Axis) *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QTransform_RotateRadians(ptr.Pointer(), C.double(angle), C.longlong(axis)))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Scale(sx float64, sy float64) *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QTransform_Scale(ptr.Pointer(), C.double(sx), C.double(sy)))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) SetMatrix(m11 float64, m12 float64, m13 float64, m21 float64, m22 float64, m23 float64, m31 float64, m32 float64, m33 float64) {
	if ptr.Pointer() != nil {
		C.QTransform_SetMatrix(ptr.Pointer(), C.double(m11), C.double(m12), C.double(m13), C.double(m21), C.double(m22), C.double(m23), C.double(m31), C.double(m32), C.double(m33))
	}
}

func (ptr *QTransform) Shear(sh float64, sv float64) *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QTransform_Shear(ptr.Pointer(), C.double(sh), C.double(sv)))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func QTransform_SquareToQuad(quad QPolygonF_ITF, trans QTransform_ITF) bool {
	return int8(C.QTransform_QTransform_SquareToQuad(PointerFromQPolygonF(quad), PointerFromQTransform(trans))) != 0
}

func (ptr *QTransform) SquareToQuad(quad QPolygonF_ITF, trans QTransform_ITF) bool {
	return int8(C.QTransform_QTransform_SquareToQuad(PointerFromQPolygonF(quad), PointerFromQTransform(trans))) != 0
}

func (ptr *QTransform) ToAffine() *QMatrix {
	if ptr.Pointer() != nil {
		return NewQMatrixFromPointer(C.QTransform_ToAffine(ptr.Pointer()))
	}
	return nil
}

func (ptr *QTransform) Translate(dx float64, dy float64) *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QTransform_Translate(ptr.Pointer(), C.double(dx), C.double(dy)))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Transposed() *QTransform {
	if ptr.Pointer() != nil {
		tmpValue := NewQTransformFromPointer(C.QTransform_Transposed(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QTransform).DestroyQTransform)
		return tmpValue
	}
	return nil
}

func (ptr *QTransform) Type() QTransform__TransformationType {
	if ptr.Pointer() != nil {
		return QTransform__TransformationType(C.QTransform_Type(ptr.Pointer()))
	}
	return 0
}

type QValidator struct {
	core.QObject
}

type QValidator_ITF interface {
	core.QObject_ITF
	QValidator_PTR() *QValidator
}

func (ptr *QValidator) QValidator_PTR() *QValidator {
	return ptr
}

func (ptr *QValidator) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QValidator) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
	}
}

func PointerFromQValidator(ptr QValidator_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QValidator_PTR().Pointer()
	}
	return nil
}

func NewQValidatorFromPointer(ptr unsafe.Pointer) (n *QValidator) {
	n = new(QValidator)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QValidator__State
//QValidator::State
type QValidator__State int64

const (
	QValidator__Invalid      QValidator__State = QValidator__State(0)
	QValidator__Intermediate QValidator__State = QValidator__State(1)
	QValidator__Acceptable   QValidator__State = QValidator__State(2)
)

func NewQValidator(parent core.QObject_ITF) *QValidator {
	tmpValue := NewQValidatorFromPointer(C.QValidator_NewQValidator(core.PointerFromQObject(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQValidator_Changed
func callbackQValidator_Changed(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "changed"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QValidator) ConnectChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "changed") {
			C.QValidator_ConnectChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "changed")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "changed"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "changed", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "changed", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QValidator) DisconnectChanged() {
	if ptr.Pointer() != nil {
		C.QValidator_DisconnectChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "changed")
	}
}

func (ptr *QValidator) Changed() {
	if ptr.Pointer() != nil {
		C.QValidator_Changed(ptr.Pointer())
	}
}

//export callbackQValidator_Fixup
func callbackQValidator_Fixup(ptr unsafe.Pointer, input C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "fixup"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(input))
	} else {
		NewQValidatorFromPointer(ptr).FixupDefault(cGoUnpackString(input))
	}
}

func (ptr *QValidator) ConnectFixup(f func(input string)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "fixup"); signal != nil {
			f := func(input string) {
				(*(*func(string))(signal))(input)
				f(input)
			}
			qt.ConnectSignal(ptr.Pointer(), "fixup", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "fixup", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QValidator) DisconnectFixup() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "fixup")
	}
}

func (ptr *QValidator) Fixup(input string) {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		C.QValidator_Fixup(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))})
	}
}

func (ptr *QValidator) FixupDefault(input string) {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		C.QValidator_FixupDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))})
	}
}

func (ptr *QValidator) Locale() *core.QLocale {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQLocaleFromPointer(C.QValidator_Locale(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QLocale).DestroyQLocale)
		return tmpValue
	}
	return nil
}

func (ptr *QValidator) SetLocale(locale core.QLocale_ITF) {
	if ptr.Pointer() != nil {
		C.QValidator_SetLocale(ptr.Pointer(), core.PointerFromQLocale(locale))
	}
}

//export callbackQValidator_Validate
func callbackQValidator_Validate(ptr unsafe.Pointer, input C.struct_QtGui_PackedString, pos C.int) C.longlong {
	if signal := qt.GetSignal(ptr, "validate"); signal != nil {
		return C.longlong((*(*func(string, int) QValidator__State)(signal))(cGoUnpackString(input), int(int32(pos))))
	}

	return C.longlong(0)
}

func (ptr *QValidator) ConnectValidate(f func(input string, pos int) QValidator__State) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "validate"); signal != nil {
			f := func(input string, pos int) QValidator__State {
				(*(*func(string, int) QValidator__State)(signal))(input, pos)
				return f(input, pos)
			}
			qt.ConnectSignal(ptr.Pointer(), "validate", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "validate", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QValidator) DisconnectValidate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "validate")
	}
}

func (ptr *QValidator) Validate(input string, pos int) QValidator__State {
	if ptr.Pointer() != nil {
		var inputC *C.char
		if input != "" {
			inputC = C.CString(input)
			defer C.free(unsafe.Pointer(inputC))
		}
		return QValidator__State(C.QValidator_Validate(ptr.Pointer(), C.struct_QtGui_PackedString{data: inputC, len: C.longlong(len(input))}, C.int(int32(pos))))
	}
	return 0
}

//export callbackQValidator_DestroyQValidator
func callbackQValidator_DestroyQValidator(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QValidator"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQValidatorFromPointer(ptr).DestroyQValidatorDefault()
	}
}

func (ptr *QValidator) ConnectDestroyQValidator(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QValidator"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QValidator", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QValidator", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QValidator) DisconnectDestroyQValidator() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QValidator")
	}
}

func (ptr *QValidator) DestroyQValidator() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QValidator_DestroyQValidator(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QValidator) DestroyQValidatorDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QValidator_DestroyQValidatorDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QValidator) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QValidator___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QValidator) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QValidator___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QValidator) __children_newList() unsafe.Pointer {
	return C.QValidator___children_newList(ptr.Pointer())
}

func (ptr *QValidator) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QValidator___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QValidator) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QValidator___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QValidator) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QValidator___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QValidator) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QValidator___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QValidator) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QValidator___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QValidator) __findChildren_newList() unsafe.Pointer {
	return C.QValidator___findChildren_newList(ptr.Pointer())
}

func (ptr *QValidator) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QValidator___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QValidator) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QValidator___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QValidator) __findChildren_newList3() unsafe.Pointer {
	return C.QValidator___findChildren_newList3(ptr.Pointer())
}

//export callbackQValidator_ChildEvent
func callbackQValidator_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQValidatorFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QValidator) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QValidator_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQValidator_ConnectNotify
func callbackQValidator_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQValidatorFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QValidator) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QValidator_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQValidator_CustomEvent
func callbackQValidator_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQValidatorFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QValidator) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QValidator_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQValidator_DeleteLater
func callbackQValidator_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQValidatorFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QValidator) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QValidator_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQValidator_Destroyed
func callbackQValidator_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQValidator_DisconnectNotify
func callbackQValidator_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQValidatorFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QValidator) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QValidator_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQValidator_Event
func callbackQValidator_Event(ptr unsafe.Pointer, e unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(e)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQValidatorFromPointer(ptr).EventDefault(core.NewQEventFromPointer(e)))))
}

func (ptr *QValidator) EventDefault(e core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QValidator_EventDefault(ptr.Pointer(), core.PointerFromQEvent(e))) != 0
	}
	return false
}

//export callbackQValidator_EventFilter
func callbackQValidator_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQValidatorFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QValidator) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QValidator_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQValidator_MetaObject
func callbackQValidator_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQValidatorFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QValidator) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QValidator_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQValidator_ObjectNameChanged
func callbackQValidator_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQValidator_TimerEvent
func callbackQValidator_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQValidatorFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QValidator) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QValidator_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QVector2D struct {
	ptr unsafe.Pointer
}

type QVector2D_ITF interface {
	QVector2D_PTR() *QVector2D
}

func (ptr *QVector2D) QVector2D_PTR() *QVector2D {
	return ptr
}

func (ptr *QVector2D) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QVector2D) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQVector2D(ptr QVector2D_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVector2D_PTR().Pointer()
	}
	return nil
}

func NewQVector2DFromPointer(ptr unsafe.Pointer) (n *QVector2D) {
	n = new(QVector2D)
	n.SetPointer(ptr)
	return
}
func (ptr *QVector2D) DestroyQVector2D() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQVector2D() *QVector2D {
	tmpValue := NewQVector2DFromPointer(C.QVector2D_NewQVector2D())
	qt.SetFinalizer(tmpValue, (*QVector2D).DestroyQVector2D)
	return tmpValue
}

func NewQVector2D3(xpos float32, ypos float32) *QVector2D {
	tmpValue := NewQVector2DFromPointer(C.QVector2D_NewQVector2D3(C.float(xpos), C.float(ypos)))
	qt.SetFinalizer(tmpValue, (*QVector2D).DestroyQVector2D)
	return tmpValue
}

func NewQVector2D4(point core.QPoint_ITF) *QVector2D {
	tmpValue := NewQVector2DFromPointer(C.QVector2D_NewQVector2D4(core.PointerFromQPoint(point)))
	qt.SetFinalizer(tmpValue, (*QVector2D).DestroyQVector2D)
	return tmpValue
}

func NewQVector2D5(point core.QPointF_ITF) *QVector2D {
	tmpValue := NewQVector2DFromPointer(C.QVector2D_NewQVector2D5(core.PointerFromQPointF(point)))
	qt.SetFinalizer(tmpValue, (*QVector2D).DestroyQVector2D)
	return tmpValue
}

func NewQVector2D6(vector QVector3D_ITF) *QVector2D {
	tmpValue := NewQVector2DFromPointer(C.QVector2D_NewQVector2D6(PointerFromQVector3D(vector)))
	qt.SetFinalizer(tmpValue, (*QVector2D).DestroyQVector2D)
	return tmpValue
}

func NewQVector2D7(vector QVector4D_ITF) *QVector2D {
	tmpValue := NewQVector2DFromPointer(C.QVector2D_NewQVector2D7(PointerFromQVector4D(vector)))
	qt.SetFinalizer(tmpValue, (*QVector2D).DestroyQVector2D)
	return tmpValue
}

func (ptr *QVector2D) DistanceToLine(point QVector2D_ITF, direction QVector2D_ITF) float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector2D_DistanceToLine(ptr.Pointer(), PointerFromQVector2D(point), PointerFromQVector2D(direction)))
	}
	return 0
}

func (ptr *QVector2D) DistanceToPoint(point QVector2D_ITF) float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector2D_DistanceToPoint(ptr.Pointer(), PointerFromQVector2D(point)))
	}
	return 0
}

func QVector2D_DotProduct(v1 QVector2D_ITF, v2 QVector2D_ITF) float32 {
	return float32(C.QVector2D_QVector2D_DotProduct(PointerFromQVector2D(v1), PointerFromQVector2D(v2)))
}

func (ptr *QVector2D) DotProduct(v1 QVector2D_ITF, v2 QVector2D_ITF) float32 {
	return float32(C.QVector2D_QVector2D_DotProduct(PointerFromQVector2D(v1), PointerFromQVector2D(v2)))
}

func (ptr *QVector2D) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QVector2D_IsNull(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QVector2D) Length() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector2D_Length(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector2D) LengthSquared() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector2D_LengthSquared(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector2D) Normalize() {
	if ptr.Pointer() != nil {
		C.QVector2D_Normalize(ptr.Pointer())
	}
}

func (ptr *QVector2D) Normalized() *QVector2D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector2DFromPointer(C.QVector2D_Normalized(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector2D).DestroyQVector2D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector2D) SetX(x float32) {
	if ptr.Pointer() != nil {
		C.QVector2D_SetX(ptr.Pointer(), C.float(x))
	}
}

func (ptr *QVector2D) SetY(y float32) {
	if ptr.Pointer() != nil {
		C.QVector2D_SetY(ptr.Pointer(), C.float(y))
	}
}

func (ptr *QVector2D) ToPoint() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QVector2D_ToPoint(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QVector2D) ToPointF() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QVector2D_ToPointF(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QVector2D) ToVector3D() *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QVector2D_ToVector3D(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector2D) ToVector4D() *QVector4D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector4DFromPointer(C.QVector2D_ToVector4D(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector2D) X() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector2D_X(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector2D) Y() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector2D_Y(ptr.Pointer()))
	}
	return 0
}

type QVector3D struct {
	ptr unsafe.Pointer
}

type QVector3D_ITF interface {
	QVector3D_PTR() *QVector3D
}

func (ptr *QVector3D) QVector3D_PTR() *QVector3D {
	return ptr
}

func (ptr *QVector3D) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QVector3D) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQVector3D(ptr QVector3D_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVector3D_PTR().Pointer()
	}
	return nil
}

func NewQVector3DFromPointer(ptr unsafe.Pointer) (n *QVector3D) {
	n = new(QVector3D)
	n.SetPointer(ptr)
	return
}
func (ptr *QVector3D) DestroyQVector3D() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQVector3D() *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_NewQVector3D())
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func NewQVector3D3(xpos float32, ypos float32, zpos float32) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_NewQVector3D3(C.float(xpos), C.float(ypos), C.float(zpos)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func NewQVector3D4(point core.QPoint_ITF) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_NewQVector3D4(core.PointerFromQPoint(point)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func NewQVector3D5(point core.QPointF_ITF) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_NewQVector3D5(core.PointerFromQPointF(point)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func NewQVector3D6(vector QVector2D_ITF) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_NewQVector3D6(PointerFromQVector2D(vector)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func NewQVector3D7(vector QVector2D_ITF, zpos float32) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_NewQVector3D7(PointerFromQVector2D(vector), C.float(zpos)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func NewQVector3D8(vector QVector4D_ITF) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_NewQVector3D8(PointerFromQVector4D(vector)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func QVector3D_CrossProduct(v1 QVector3D_ITF, v2 QVector3D_ITF) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_QVector3D_CrossProduct(PointerFromQVector3D(v1), PointerFromQVector3D(v2)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func (ptr *QVector3D) CrossProduct(v1 QVector3D_ITF, v2 QVector3D_ITF) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_QVector3D_CrossProduct(PointerFromQVector3D(v1), PointerFromQVector3D(v2)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func (ptr *QVector3D) DistanceToLine(point QVector3D_ITF, direction QVector3D_ITF) float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector3D_DistanceToLine(ptr.Pointer(), PointerFromQVector3D(point), PointerFromQVector3D(direction)))
	}
	return 0
}

func (ptr *QVector3D) DistanceToPlane(plane QVector3D_ITF, normal QVector3D_ITF) float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector3D_DistanceToPlane(ptr.Pointer(), PointerFromQVector3D(plane), PointerFromQVector3D(normal)))
	}
	return 0
}

func (ptr *QVector3D) DistanceToPlane2(plane1 QVector3D_ITF, plane2 QVector3D_ITF, plane3 QVector3D_ITF) float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector3D_DistanceToPlane2(ptr.Pointer(), PointerFromQVector3D(plane1), PointerFromQVector3D(plane2), PointerFromQVector3D(plane3)))
	}
	return 0
}

func (ptr *QVector3D) DistanceToPoint(point QVector3D_ITF) float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector3D_DistanceToPoint(ptr.Pointer(), PointerFromQVector3D(point)))
	}
	return 0
}

func QVector3D_DotProduct(v1 QVector3D_ITF, v2 QVector3D_ITF) float32 {
	return float32(C.QVector3D_QVector3D_DotProduct(PointerFromQVector3D(v1), PointerFromQVector3D(v2)))
}

func (ptr *QVector3D) DotProduct(v1 QVector3D_ITF, v2 QVector3D_ITF) float32 {
	return float32(C.QVector3D_QVector3D_DotProduct(PointerFromQVector3D(v1), PointerFromQVector3D(v2)))
}

func (ptr *QVector3D) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QVector3D_IsNull(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QVector3D) Length() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector3D_Length(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector3D) LengthSquared() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector3D_LengthSquared(ptr.Pointer()))
	}
	return 0
}

func QVector3D_Normal(v1 QVector3D_ITF, v2 QVector3D_ITF) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_QVector3D_Normal(PointerFromQVector3D(v1), PointerFromQVector3D(v2)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func (ptr *QVector3D) Normal(v1 QVector3D_ITF, v2 QVector3D_ITF) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_QVector3D_Normal(PointerFromQVector3D(v1), PointerFromQVector3D(v2)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func QVector3D_Normal2(v1 QVector3D_ITF, v2 QVector3D_ITF, v3 QVector3D_ITF) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_QVector3D_Normal2(PointerFromQVector3D(v1), PointerFromQVector3D(v2), PointerFromQVector3D(v3)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func (ptr *QVector3D) Normal2(v1 QVector3D_ITF, v2 QVector3D_ITF, v3 QVector3D_ITF) *QVector3D {
	tmpValue := NewQVector3DFromPointer(C.QVector3D_QVector3D_Normal2(PointerFromQVector3D(v1), PointerFromQVector3D(v2), PointerFromQVector3D(v3)))
	qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
	return tmpValue
}

func (ptr *QVector3D) Normalize() {
	if ptr.Pointer() != nil {
		C.QVector3D_Normalize(ptr.Pointer())
	}
}

func (ptr *QVector3D) Normalized() *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QVector3D_Normalized(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector3D) Project(modelView QMatrix4x4_ITF, projection QMatrix4x4_ITF, viewport core.QRect_ITF) *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QVector3D_Project(ptr.Pointer(), PointerFromQMatrix4x4(modelView), PointerFromQMatrix4x4(projection), core.PointerFromQRect(viewport)))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector3D) SetX(x float32) {
	if ptr.Pointer() != nil {
		C.QVector3D_SetX(ptr.Pointer(), C.float(x))
	}
}

func (ptr *QVector3D) SetY(y float32) {
	if ptr.Pointer() != nil {
		C.QVector3D_SetY(ptr.Pointer(), C.float(y))
	}
}

func (ptr *QVector3D) SetZ(z float32) {
	if ptr.Pointer() != nil {
		C.QVector3D_SetZ(ptr.Pointer(), C.float(z))
	}
}

func (ptr *QVector3D) ToPoint() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QVector3D_ToPoint(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QVector3D) ToPointF() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QVector3D_ToPointF(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QVector3D) ToVector2D() *QVector2D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector2DFromPointer(C.QVector3D_ToVector2D(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector2D).DestroyQVector2D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector3D) ToVector4D() *QVector4D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector4DFromPointer(C.QVector3D_ToVector4D(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector3D) Unproject(modelView QMatrix4x4_ITF, projection QMatrix4x4_ITF, viewport core.QRect_ITF) *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QVector3D_Unproject(ptr.Pointer(), PointerFromQMatrix4x4(modelView), PointerFromQMatrix4x4(projection), core.PointerFromQRect(viewport)))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector3D) X() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector3D_X(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector3D) Y() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector3D_Y(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector3D) Z() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector3D_Z(ptr.Pointer()))
	}
	return 0
}

type QVector4D struct {
	ptr unsafe.Pointer
}

type QVector4D_ITF interface {
	QVector4D_PTR() *QVector4D
}

func (ptr *QVector4D) QVector4D_PTR() *QVector4D {
	return ptr
}

func (ptr *QVector4D) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QVector4D) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQVector4D(ptr QVector4D_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVector4D_PTR().Pointer()
	}
	return nil
}

func NewQVector4DFromPointer(ptr unsafe.Pointer) (n *QVector4D) {
	n = new(QVector4D)
	n.SetPointer(ptr)
	return
}
func (ptr *QVector4D) DestroyQVector4D() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQVector4D() *QVector4D {
	tmpValue := NewQVector4DFromPointer(C.QVector4D_NewQVector4D())
	qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
	return tmpValue
}

func NewQVector4D3(xpos float32, ypos float32, zpos float32, wpos float32) *QVector4D {
	tmpValue := NewQVector4DFromPointer(C.QVector4D_NewQVector4D3(C.float(xpos), C.float(ypos), C.float(zpos), C.float(wpos)))
	qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
	return tmpValue
}

func NewQVector4D4(point core.QPoint_ITF) *QVector4D {
	tmpValue := NewQVector4DFromPointer(C.QVector4D_NewQVector4D4(core.PointerFromQPoint(point)))
	qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
	return tmpValue
}

func NewQVector4D5(point core.QPointF_ITF) *QVector4D {
	tmpValue := NewQVector4DFromPointer(C.QVector4D_NewQVector4D5(core.PointerFromQPointF(point)))
	qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
	return tmpValue
}

func NewQVector4D6(vector QVector2D_ITF) *QVector4D {
	tmpValue := NewQVector4DFromPointer(C.QVector4D_NewQVector4D6(PointerFromQVector2D(vector)))
	qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
	return tmpValue
}

func NewQVector4D7(vector QVector2D_ITF, zpos float32, wpos float32) *QVector4D {
	tmpValue := NewQVector4DFromPointer(C.QVector4D_NewQVector4D7(PointerFromQVector2D(vector), C.float(zpos), C.float(wpos)))
	qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
	return tmpValue
}

func NewQVector4D8(vector QVector3D_ITF) *QVector4D {
	tmpValue := NewQVector4DFromPointer(C.QVector4D_NewQVector4D8(PointerFromQVector3D(vector)))
	qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
	return tmpValue
}

func NewQVector4D9(vector QVector3D_ITF, wpos float32) *QVector4D {
	tmpValue := NewQVector4DFromPointer(C.QVector4D_NewQVector4D9(PointerFromQVector3D(vector), C.float(wpos)))
	qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
	return tmpValue
}

func QVector4D_DotProduct(v1 QVector4D_ITF, v2 QVector4D_ITF) float32 {
	return float32(C.QVector4D_QVector4D_DotProduct(PointerFromQVector4D(v1), PointerFromQVector4D(v2)))
}

func (ptr *QVector4D) DotProduct(v1 QVector4D_ITF, v2 QVector4D_ITF) float32 {
	return float32(C.QVector4D_QVector4D_DotProduct(PointerFromQVector4D(v1), PointerFromQVector4D(v2)))
}

func (ptr *QVector4D) IsNull() bool {
	if ptr.Pointer() != nil {
		return int8(C.QVector4D_IsNull(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QVector4D) Length() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector4D_Length(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector4D) LengthSquared() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector4D_LengthSquared(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector4D) Normalize() {
	if ptr.Pointer() != nil {
		C.QVector4D_Normalize(ptr.Pointer())
	}
}

func (ptr *QVector4D) Normalized() *QVector4D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector4DFromPointer(C.QVector4D_Normalized(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector4D).DestroyQVector4D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector4D) SetW(w float32) {
	if ptr.Pointer() != nil {
		C.QVector4D_SetW(ptr.Pointer(), C.float(w))
	}
}

func (ptr *QVector4D) SetX(x float32) {
	if ptr.Pointer() != nil {
		C.QVector4D_SetX(ptr.Pointer(), C.float(x))
	}
}

func (ptr *QVector4D) SetY(y float32) {
	if ptr.Pointer() != nil {
		C.QVector4D_SetY(ptr.Pointer(), C.float(y))
	}
}

func (ptr *QVector4D) SetZ(z float32) {
	if ptr.Pointer() != nil {
		C.QVector4D_SetZ(ptr.Pointer(), C.float(z))
	}
}

func (ptr *QVector4D) ToPoint() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QVector4D_ToPoint(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QVector4D) ToPointF() *core.QPointF {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFFromPointer(C.QVector4D_ToPointF(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPointF).DestroyQPointF)
		return tmpValue
	}
	return nil
}

func (ptr *QVector4D) ToVector2D() *QVector2D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector2DFromPointer(C.QVector4D_ToVector2D(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector2D).DestroyQVector2D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector4D) ToVector2DAffine() *QVector2D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector2DFromPointer(C.QVector4D_ToVector2DAffine(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector2D).DestroyQVector2D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector4D) ToVector3D() *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QVector4D_ToVector3D(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector4D) ToVector3DAffine() *QVector3D {
	if ptr.Pointer() != nil {
		tmpValue := NewQVector3DFromPointer(C.QVector4D_ToVector3DAffine(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QVector3D).DestroyQVector3D)
		return tmpValue
	}
	return nil
}

func (ptr *QVector4D) W() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector4D_W(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector4D) X() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector4D_X(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector4D) Y() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector4D_Y(ptr.Pointer()))
	}
	return 0
}

func (ptr *QVector4D) Z() float32 {
	if ptr.Pointer() != nil {
		return float32(C.QVector4D_Z(ptr.Pointer()))
	}
	return 0
}

type QVulkanDeviceFunctions struct {
	ptr unsafe.Pointer
}

type QVulkanDeviceFunctions_ITF interface {
	QVulkanDeviceFunctions_PTR() *QVulkanDeviceFunctions
}

func (ptr *QVulkanDeviceFunctions) QVulkanDeviceFunctions_PTR() *QVulkanDeviceFunctions {
	return ptr
}

func (ptr *QVulkanDeviceFunctions) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QVulkanDeviceFunctions) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQVulkanDeviceFunctions(ptr QVulkanDeviceFunctions_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVulkanDeviceFunctions_PTR().Pointer()
	}
	return nil
}

func NewQVulkanDeviceFunctionsFromPointer(ptr unsafe.Pointer) (n *QVulkanDeviceFunctions) {
	n = new(QVulkanDeviceFunctions)
	n.SetPointer(ptr)
	return
}
func (ptr *QVulkanDeviceFunctions) DestroyQVulkanDeviceFunctions() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QVulkanExtension struct {
	ptr unsafe.Pointer
}

type QVulkanExtension_ITF interface {
	QVulkanExtension_PTR() *QVulkanExtension
}

func (ptr *QVulkanExtension) QVulkanExtension_PTR() *QVulkanExtension {
	return ptr
}

func (ptr *QVulkanExtension) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QVulkanExtension) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQVulkanExtension(ptr QVulkanExtension_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVulkanExtension_PTR().Pointer()
	}
	return nil
}

func NewQVulkanExtensionFromPointer(ptr unsafe.Pointer) (n *QVulkanExtension) {
	n = new(QVulkanExtension)
	n.SetPointer(ptr)
	return
}
func (ptr *QVulkanExtension) DestroyQVulkanExtension() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QVulkanFunctions struct {
	ptr unsafe.Pointer
}

type QVulkanFunctions_ITF interface {
	QVulkanFunctions_PTR() *QVulkanFunctions
}

func (ptr *QVulkanFunctions) QVulkanFunctions_PTR() *QVulkanFunctions {
	return ptr
}

func (ptr *QVulkanFunctions) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QVulkanFunctions) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQVulkanFunctions(ptr QVulkanFunctions_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVulkanFunctions_PTR().Pointer()
	}
	return nil
}

func NewQVulkanFunctionsFromPointer(ptr unsafe.Pointer) (n *QVulkanFunctions) {
	n = new(QVulkanFunctions)
	n.SetPointer(ptr)
	return
}
func (ptr *QVulkanFunctions) DestroyQVulkanFunctions() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QVulkanInfoVector struct {
	core.QVector
}

type QVulkanInfoVector_ITF interface {
	core.QVector_ITF
	QVulkanInfoVector_PTR() *QVulkanInfoVector
}

func (ptr *QVulkanInfoVector) QVulkanInfoVector_PTR() *QVulkanInfoVector {
	return ptr
}

func (ptr *QVulkanInfoVector) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QVector_PTR().Pointer()
	}
	return nil
}

func (ptr *QVulkanInfoVector) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QVector_PTR().SetPointer(p)
	}
}

func PointerFromQVulkanInfoVector(ptr QVulkanInfoVector_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVulkanInfoVector_PTR().Pointer()
	}
	return nil
}

func NewQVulkanInfoVectorFromPointer(ptr unsafe.Pointer) (n *QVulkanInfoVector) {
	n = new(QVulkanInfoVector)
	n.SetPointer(ptr)
	return
}
func (ptr *QVulkanInfoVector) DestroyQVulkanInfoVector() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QVulkanInstance struct {
	ptr unsafe.Pointer
}

type QVulkanInstance_ITF interface {
	QVulkanInstance_PTR() *QVulkanInstance
}

func (ptr *QVulkanInstance) QVulkanInstance_PTR() *QVulkanInstance {
	return ptr
}

func (ptr *QVulkanInstance) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QVulkanInstance) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQVulkanInstance(ptr QVulkanInstance_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVulkanInstance_PTR().Pointer()
	}
	return nil
}

func NewQVulkanInstanceFromPointer(ptr unsafe.Pointer) (n *QVulkanInstance) {
	n = new(QVulkanInstance)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QVulkanInstance__Flag
//QVulkanInstance::Flag
type QVulkanInstance__Flag int64

const (
	QVulkanInstance__NoDebugOutputRedirect QVulkanInstance__Flag = QVulkanInstance__Flag(0x01)
)

type QVulkanLayer struct {
	ptr unsafe.Pointer
}

type QVulkanLayer_ITF interface {
	QVulkanLayer_PTR() *QVulkanLayer
}

func (ptr *QVulkanLayer) QVulkanLayer_PTR() *QVulkanLayer {
	return ptr
}

func (ptr *QVulkanLayer) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QVulkanLayer) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQVulkanLayer(ptr QVulkanLayer_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVulkanLayer_PTR().Pointer()
	}
	return nil
}

func NewQVulkanLayerFromPointer(ptr unsafe.Pointer) (n *QVulkanLayer) {
	n = new(QVulkanLayer)
	n.SetPointer(ptr)
	return
}
func (ptr *QVulkanLayer) DestroyQVulkanLayer() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

type QVulkanWindow struct {
	QWindow
}

type QVulkanWindow_ITF interface {
	QWindow_ITF
	QVulkanWindow_PTR() *QVulkanWindow
}

func (ptr *QVulkanWindow) QVulkanWindow_PTR() *QVulkanWindow {
	return ptr
}

func (ptr *QVulkanWindow) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QWindow_PTR().Pointer()
	}
	return nil
}

func (ptr *QVulkanWindow) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QWindow_PTR().SetPointer(p)
	}
}

func PointerFromQVulkanWindow(ptr QVulkanWindow_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVulkanWindow_PTR().Pointer()
	}
	return nil
}

func NewQVulkanWindowFromPointer(ptr unsafe.Pointer) (n *QVulkanWindow) {
	n = new(QVulkanWindow)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QVulkanWindow__Flag
//QVulkanWindow::Flag
type QVulkanWindow__Flag int64

const (
	QVulkanWindow__PersistentResources QVulkanWindow__Flag = QVulkanWindow__Flag(0x01)
)

type QVulkanWindowRenderer struct {
	ptr unsafe.Pointer
}

type QVulkanWindowRenderer_ITF interface {
	QVulkanWindowRenderer_PTR() *QVulkanWindowRenderer
}

func (ptr *QVulkanWindowRenderer) QVulkanWindowRenderer_PTR() *QVulkanWindowRenderer {
	return ptr
}

func (ptr *QVulkanWindowRenderer) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.ptr
	}
	return nil
}

func (ptr *QVulkanWindowRenderer) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.ptr = p
	}
}

func PointerFromQVulkanWindowRenderer(ptr QVulkanWindowRenderer_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QVulkanWindowRenderer_PTR().Pointer()
	}
	return nil
}

func NewQVulkanWindowRendererFromPointer(ptr unsafe.Pointer) (n *QVulkanWindowRenderer) {
	n = new(QVulkanWindowRenderer)
	n.SetPointer(ptr)
	return
}

type QWhatsThisClickedEvent struct {
	core.QEvent
}

type QWhatsThisClickedEvent_ITF interface {
	core.QEvent_ITF
	QWhatsThisClickedEvent_PTR() *QWhatsThisClickedEvent
}

func (ptr *QWhatsThisClickedEvent) QWhatsThisClickedEvent_PTR() *QWhatsThisClickedEvent {
	return ptr
}

func (ptr *QWhatsThisClickedEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QWhatsThisClickedEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQWhatsThisClickedEvent(ptr QWhatsThisClickedEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QWhatsThisClickedEvent_PTR().Pointer()
	}
	return nil
}

func NewQWhatsThisClickedEventFromPointer(ptr unsafe.Pointer) (n *QWhatsThisClickedEvent) {
	n = new(QWhatsThisClickedEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QWhatsThisClickedEvent) DestroyQWhatsThisClickedEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQWhatsThisClickedEvent(href string) *QWhatsThisClickedEvent {
	var hrefC *C.char
	if href != "" {
		hrefC = C.CString(href)
		defer C.free(unsafe.Pointer(hrefC))
	}
	tmpValue := NewQWhatsThisClickedEventFromPointer(C.QWhatsThisClickedEvent_NewQWhatsThisClickedEvent(C.struct_QtGui_PackedString{data: hrefC, len: C.longlong(len(href))}))
	qt.SetFinalizer(tmpValue, (*QWhatsThisClickedEvent).DestroyQWhatsThisClickedEvent)
	return tmpValue
}

func (ptr *QWhatsThisClickedEvent) Href() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QWhatsThisClickedEvent_Href(ptr.Pointer()))
	}
	return ""
}

type QWheelEvent struct {
	QInputEvent
}

type QWheelEvent_ITF interface {
	QInputEvent_ITF
	QWheelEvent_PTR() *QWheelEvent
}

func (ptr *QWheelEvent) QWheelEvent_PTR() *QWheelEvent {
	return ptr
}

func (ptr *QWheelEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QInputEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QWheelEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QInputEvent_PTR().SetPointer(p)
	}
}

func PointerFromQWheelEvent(ptr QWheelEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QWheelEvent_PTR().Pointer()
	}
	return nil
}

func NewQWheelEventFromPointer(ptr unsafe.Pointer) (n *QWheelEvent) {
	n = new(QWheelEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QWheelEvent) DestroyQWheelEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func NewQWheelEvent3(pos core.QPointF_ITF, globalPos core.QPointF_ITF, pixelDelta core.QPoint_ITF, angleDelta core.QPoint_ITF, qt4Delta int, qt4Orientation core.Qt__Orientation, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier) *QWheelEvent {
	tmpValue := NewQWheelEventFromPointer(C.QWheelEvent_NewQWheelEvent3(core.PointerFromQPointF(pos), core.PointerFromQPointF(globalPos), core.PointerFromQPoint(pixelDelta), core.PointerFromQPoint(angleDelta), C.int(int32(qt4Delta)), C.longlong(qt4Orientation), C.longlong(buttons), C.longlong(modifiers)))
	qt.SetFinalizer(tmpValue, (*QWheelEvent).DestroyQWheelEvent)
	return tmpValue
}

func NewQWheelEvent4(pos core.QPointF_ITF, globalPos core.QPointF_ITF, pixelDelta core.QPoint_ITF, angleDelta core.QPoint_ITF, qt4Delta int, qt4Orientation core.Qt__Orientation, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier, phase core.Qt__ScrollPhase) *QWheelEvent {
	tmpValue := NewQWheelEventFromPointer(C.QWheelEvent_NewQWheelEvent4(core.PointerFromQPointF(pos), core.PointerFromQPointF(globalPos), core.PointerFromQPoint(pixelDelta), core.PointerFromQPoint(angleDelta), C.int(int32(qt4Delta)), C.longlong(qt4Orientation), C.longlong(buttons), C.longlong(modifiers), C.longlong(phase)))
	qt.SetFinalizer(tmpValue, (*QWheelEvent).DestroyQWheelEvent)
	return tmpValue
}

func NewQWheelEvent5(pos core.QPointF_ITF, globalPos core.QPointF_ITF, pixelDelta core.QPoint_ITF, angleDelta core.QPoint_ITF, qt4Delta int, qt4Orientation core.Qt__Orientation, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier, phase core.Qt__ScrollPhase, source core.Qt__MouseEventSource) *QWheelEvent {
	tmpValue := NewQWheelEventFromPointer(C.QWheelEvent_NewQWheelEvent5(core.PointerFromQPointF(pos), core.PointerFromQPointF(globalPos), core.PointerFromQPoint(pixelDelta), core.PointerFromQPoint(angleDelta), C.int(int32(qt4Delta)), C.longlong(qt4Orientation), C.longlong(buttons), C.longlong(modifiers), C.longlong(phase), C.longlong(source)))
	qt.SetFinalizer(tmpValue, (*QWheelEvent).DestroyQWheelEvent)
	return tmpValue
}

func NewQWheelEvent6(pos core.QPointF_ITF, globalPos core.QPointF_ITF, pixelDelta core.QPoint_ITF, angleDelta core.QPoint_ITF, qt4Delta int, qt4Orientation core.Qt__Orientation, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier, phase core.Qt__ScrollPhase, source core.Qt__MouseEventSource, inverted bool) *QWheelEvent {
	tmpValue := NewQWheelEventFromPointer(C.QWheelEvent_NewQWheelEvent6(core.PointerFromQPointF(pos), core.PointerFromQPointF(globalPos), core.PointerFromQPoint(pixelDelta), core.PointerFromQPoint(angleDelta), C.int(int32(qt4Delta)), C.longlong(qt4Orientation), C.longlong(buttons), C.longlong(modifiers), C.longlong(phase), C.longlong(source), C.char(int8(qt.GoBoolToInt(inverted)))))
	qt.SetFinalizer(tmpValue, (*QWheelEvent).DestroyQWheelEvent)
	return tmpValue
}

func NewQWheelEvent7(pos core.QPointF_ITF, globalPos core.QPointF_ITF, pixelDelta core.QPoint_ITF, angleDelta core.QPoint_ITF, buttons core.Qt__MouseButton, modifiers core.Qt__KeyboardModifier, phase core.Qt__ScrollPhase, inverted bool, source core.Qt__MouseEventSource) *QWheelEvent {
	tmpValue := NewQWheelEventFromPointer(C.QWheelEvent_NewQWheelEvent7(core.PointerFromQPointF(pos), core.PointerFromQPointF(globalPos), core.PointerFromQPoint(pixelDelta), core.PointerFromQPoint(angleDelta), C.longlong(buttons), C.longlong(modifiers), C.longlong(phase), C.char(int8(qt.GoBoolToInt(inverted))), C.longlong(source)))
	qt.SetFinalizer(tmpValue, (*QWheelEvent).DestroyQWheelEvent)
	return tmpValue
}

func (ptr *QWheelEvent) AngleDelta() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QWheelEvent_AngleDelta(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QWheelEvent) Buttons() core.Qt__MouseButton {
	if ptr.Pointer() != nil {
		return core.Qt__MouseButton(C.QWheelEvent_Buttons(ptr.Pointer()))
	}
	return 0
}

func (ptr *QWheelEvent) GlobalPos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QWheelEvent_GlobalPos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QWheelEvent) GlobalPosF() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QWheelEvent_GlobalPosF(ptr.Pointer()))
	}
	return nil
}

func (ptr *QWheelEvent) GlobalX() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWheelEvent_GlobalX(ptr.Pointer())))
	}
	return 0
}

func (ptr *QWheelEvent) GlobalY() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWheelEvent_GlobalY(ptr.Pointer())))
	}
	return 0
}

func (ptr *QWheelEvent) Inverted() bool {
	if ptr.Pointer() != nil {
		return int8(C.QWheelEvent_Inverted(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QWheelEvent) Phase() core.Qt__ScrollPhase {
	if ptr.Pointer() != nil {
		return core.Qt__ScrollPhase(C.QWheelEvent_Phase(ptr.Pointer()))
	}
	return 0
}

func (ptr *QWheelEvent) PixelDelta() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QWheelEvent_PixelDelta(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QWheelEvent) Pos() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QWheelEvent_Pos(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QWheelEvent) PosF() *core.QPointF {
	if ptr.Pointer() != nil {
		return core.NewQPointFFromPointer(C.QWheelEvent_PosF(ptr.Pointer()))
	}
	return nil
}

func (ptr *QWheelEvent) Source() core.Qt__MouseEventSource {
	if ptr.Pointer() != nil {
		return core.Qt__MouseEventSource(C.QWheelEvent_Source(ptr.Pointer()))
	}
	return 0
}

func (ptr *QWheelEvent) X() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWheelEvent_X(ptr.Pointer())))
	}
	return 0
}

func (ptr *QWheelEvent) Y() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWheelEvent_Y(ptr.Pointer())))
	}
	return 0
}

type QWindow struct {
	core.QObject
	QSurface
}

type QWindow_ITF interface {
	core.QObject_ITF
	QSurface_ITF
	QWindow_PTR() *QWindow
}

func (ptr *QWindow) QWindow_PTR() *QWindow {
	return ptr
}

func (ptr *QWindow) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QObject_PTR().Pointer()
	}
	return nil
}

func (ptr *QWindow) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QObject_PTR().SetPointer(p)
		ptr.QSurface_PTR().SetPointer(p)
	}
}

func PointerFromQWindow(ptr QWindow_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QWindow_PTR().Pointer()
	}
	return nil
}

func NewQWindowFromPointer(ptr unsafe.Pointer) (n *QWindow) {
	n = new(QWindow)
	n.SetPointer(ptr)
	return
}

//go:generate stringer -type=QWindow__Visibility
//QWindow::Visibility
type QWindow__Visibility int64

const (
	QWindow__Hidden              QWindow__Visibility = QWindow__Visibility(0)
	QWindow__AutomaticVisibility QWindow__Visibility = QWindow__Visibility(1)
	QWindow__Windowed            QWindow__Visibility = QWindow__Visibility(2)
	QWindow__Minimized           QWindow__Visibility = QWindow__Visibility(3)
	QWindow__Maximized           QWindow__Visibility = QWindow__Visibility(4)
	QWindow__FullScreen          QWindow__Visibility = QWindow__Visibility(5)
)

//go:generate stringer -type=QWindow__AncestorMode
//QWindow::AncestorMode
type QWindow__AncestorMode int64

const (
	QWindow__ExcludeTransients QWindow__AncestorMode = QWindow__AncestorMode(0)
	QWindow__IncludeTransients QWindow__AncestorMode = QWindow__AncestorMode(1)
)

func NewQWindow(targetScreen QScreen_ITF) *QWindow {
	tmpValue := NewQWindowFromPointer(C.QWindow_NewQWindow(PointerFromQScreen(targetScreen)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func NewQWindow2(parent QWindow_ITF) *QWindow {
	tmpValue := NewQWindowFromPointer(C.QWindow_NewQWindow2(PointerFromQWindow(parent)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

//export callbackQWindow_ActiveChanged
func callbackQWindow_ActiveChanged(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "activeChanged"); signal != nil {
		(*(*func())(signal))()
	}

}

func (ptr *QWindow) ConnectActiveChanged(f func()) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "activeChanged") {
			C.QWindow_ConnectActiveChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "activeChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "activeChanged"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "activeChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "activeChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectActiveChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectActiveChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "activeChanged")
	}
}

func (ptr *QWindow) ActiveChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_ActiveChanged(ptr.Pointer())
	}
}

//export callbackQWindow_Alert
func callbackQWindow_Alert(ptr unsafe.Pointer, msec C.int) {
	if signal := qt.GetSignal(ptr, "alert"); signal != nil {
		(*(*func(int))(signal))(int(int32(msec)))
	} else {
		NewQWindowFromPointer(ptr).AlertDefault(int(int32(msec)))
	}
}

func (ptr *QWindow) ConnectAlert(f func(msec int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "alert"); signal != nil {
			f := func(msec int) {
				(*(*func(int))(signal))(msec)
				f(msec)
			}
			qt.ConnectSignal(ptr.Pointer(), "alert", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "alert", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectAlert() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "alert")
	}
}

func (ptr *QWindow) Alert(msec int) {
	if ptr.Pointer() != nil {
		C.QWindow_Alert(ptr.Pointer(), C.int(int32(msec)))
	}
}

func (ptr *QWindow) AlertDefault(msec int) {
	if ptr.Pointer() != nil {
		C.QWindow_AlertDefault(ptr.Pointer(), C.int(int32(msec)))
	}
}

func (ptr *QWindow) BaseSize() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QWindow_BaseSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

//export callbackQWindow_Close
func callbackQWindow_Close(ptr unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "close"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func() bool)(signal))())))
	}

	return C.char(int8(qt.GoBoolToInt(NewQWindowFromPointer(ptr).CloseDefault())))
}

func (ptr *QWindow) ConnectClose(f func() bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "close"); signal != nil {
			f := func() bool {
				(*(*func() bool)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "close", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "close", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectClose() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "close")
	}
}

func (ptr *QWindow) Close() bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_Close(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QWindow) CloseDefault() bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_CloseDefault(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QWindow) ContentOrientation() core.Qt__ScreenOrientation {
	if ptr.Pointer() != nil {
		return core.Qt__ScreenOrientation(C.QWindow_ContentOrientation(ptr.Pointer()))
	}
	return 0
}

//export callbackQWindow_ContentOrientationChanged
func callbackQWindow_ContentOrientationChanged(ptr unsafe.Pointer, orientation C.longlong) {
	if signal := qt.GetSignal(ptr, "contentOrientationChanged"); signal != nil {
		(*(*func(core.Qt__ScreenOrientation))(signal))(core.Qt__ScreenOrientation(orientation))
	}

}

func (ptr *QWindow) ConnectContentOrientationChanged(f func(orientation core.Qt__ScreenOrientation)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "contentOrientationChanged") {
			C.QWindow_ConnectContentOrientationChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "contentOrientationChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "contentOrientationChanged"); signal != nil {
			f := func(orientation core.Qt__ScreenOrientation) {
				(*(*func(core.Qt__ScreenOrientation))(signal))(orientation)
				f(orientation)
			}
			qt.ConnectSignal(ptr.Pointer(), "contentOrientationChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "contentOrientationChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectContentOrientationChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectContentOrientationChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "contentOrientationChanged")
	}
}

func (ptr *QWindow) ContentOrientationChanged(orientation core.Qt__ScreenOrientation) {
	if ptr.Pointer() != nil {
		C.QWindow_ContentOrientationChanged(ptr.Pointer(), C.longlong(orientation))
	}
}

func (ptr *QWindow) Create() {
	if ptr.Pointer() != nil {
		C.QWindow_Create(ptr.Pointer())
	}
}

func (ptr *QWindow) Cursor() *QCursor {
	if ptr.Pointer() != nil {
		tmpValue := NewQCursorFromPointer(C.QWindow_Cursor(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QCursor).DestroyQCursor)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) Destroy() {
	if ptr.Pointer() != nil {
		C.QWindow_Destroy(ptr.Pointer())
	}
}

func (ptr *QWindow) DevicePixelRatio() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QWindow_DevicePixelRatio(ptr.Pointer()))
	}
	return 0
}

//export callbackQWindow_Event
func callbackQWindow_Event(ptr unsafe.Pointer, ev unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "event"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QEvent) bool)(signal))(core.NewQEventFromPointer(ev)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQWindowFromPointer(ptr).EventDefault(core.NewQEventFromPointer(ev)))))
}

func (ptr *QWindow) ConnectEvent(f func(ev *core.QEvent) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "event"); signal != nil {
			f := func(ev *core.QEvent) bool {
				(*(*func(*core.QEvent) bool)(signal))(ev)
				return f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "event", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "event", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "event")
	}
}

func (ptr *QWindow) Event(ev core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_Event(ptr.Pointer(), core.PointerFromQEvent(ev))) != 0
	}
	return false
}

func (ptr *QWindow) EventDefault(ev core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_EventDefault(ptr.Pointer(), core.PointerFromQEvent(ev))) != 0
	}
	return false
}

//export callbackQWindow_ExposeEvent
func callbackQWindow_ExposeEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "exposeEvent"); signal != nil {
		(*(*func(*QExposeEvent))(signal))(NewQExposeEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).ExposeEventDefault(NewQExposeEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectExposeEvent(f func(ev *QExposeEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "exposeEvent"); signal != nil {
			f := func(ev *QExposeEvent) {
				(*(*func(*QExposeEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "exposeEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "exposeEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectExposeEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "exposeEvent")
	}
}

func (ptr *QWindow) ExposeEvent(ev QExposeEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ExposeEvent(ptr.Pointer(), PointerFromQExposeEvent(ev))
	}
}

func (ptr *QWindow) ExposeEventDefault(ev QExposeEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ExposeEventDefault(ptr.Pointer(), PointerFromQExposeEvent(ev))
	}
}

func (ptr *QWindow) FilePath() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QWindow_FilePath(ptr.Pointer()))
	}
	return ""
}

func (ptr *QWindow) Flags() core.Qt__WindowType {
	if ptr.Pointer() != nil {
		return core.Qt__WindowType(C.QWindow_Flags(ptr.Pointer()))
	}
	return 0
}

//export callbackQWindow_FocusInEvent
func callbackQWindow_FocusInEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "focusInEvent"); signal != nil {
		(*(*func(*QFocusEvent))(signal))(NewQFocusEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).FocusInEventDefault(NewQFocusEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectFocusInEvent(f func(ev *QFocusEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "focusInEvent"); signal != nil {
			f := func(ev *QFocusEvent) {
				(*(*func(*QFocusEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "focusInEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "focusInEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectFocusInEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "focusInEvent")
	}
}

func (ptr *QWindow) FocusInEvent(ev QFocusEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_FocusInEvent(ptr.Pointer(), PointerFromQFocusEvent(ev))
	}
}

func (ptr *QWindow) FocusInEventDefault(ev QFocusEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_FocusInEventDefault(ptr.Pointer(), PointerFromQFocusEvent(ev))
	}
}

//export callbackQWindow_FocusObject
func callbackQWindow_FocusObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "focusObject"); signal != nil {
		return core.PointerFromQObject((*(*func() *core.QObject)(signal))())
	}

	return core.PointerFromQObject(NewQWindowFromPointer(ptr).FocusObjectDefault())
}

func (ptr *QWindow) ConnectFocusObject(f func() *core.QObject) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "focusObject"); signal != nil {
			f := func() *core.QObject {
				(*(*func() *core.QObject)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "focusObject", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "focusObject", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectFocusObject() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "focusObject")
	}
}

func (ptr *QWindow) FocusObject() *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QWindow_FocusObject(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) FocusObjectDefault() *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QWindow_FocusObjectDefault(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQWindow_FocusObjectChanged
func callbackQWindow_FocusObjectChanged(ptr unsafe.Pointer, object unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "focusObjectChanged"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(object))
	}

}

func (ptr *QWindow) ConnectFocusObjectChanged(f func(object *core.QObject)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "focusObjectChanged") {
			C.QWindow_ConnectFocusObjectChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "focusObjectChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "focusObjectChanged"); signal != nil {
			f := func(object *core.QObject) {
				(*(*func(*core.QObject))(signal))(object)
				f(object)
			}
			qt.ConnectSignal(ptr.Pointer(), "focusObjectChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "focusObjectChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectFocusObjectChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectFocusObjectChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "focusObjectChanged")
	}
}

func (ptr *QWindow) FocusObjectChanged(object core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_FocusObjectChanged(ptr.Pointer(), core.PointerFromQObject(object))
	}
}

//export callbackQWindow_FocusOutEvent
func callbackQWindow_FocusOutEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "focusOutEvent"); signal != nil {
		(*(*func(*QFocusEvent))(signal))(NewQFocusEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).FocusOutEventDefault(NewQFocusEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectFocusOutEvent(f func(ev *QFocusEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "focusOutEvent"); signal != nil {
			f := func(ev *QFocusEvent) {
				(*(*func(*QFocusEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "focusOutEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "focusOutEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectFocusOutEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "focusOutEvent")
	}
}

func (ptr *QWindow) FocusOutEvent(ev QFocusEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_FocusOutEvent(ptr.Pointer(), PointerFromQFocusEvent(ev))
	}
}

func (ptr *QWindow) FocusOutEventDefault(ev QFocusEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_FocusOutEventDefault(ptr.Pointer(), PointerFromQFocusEvent(ev))
	}
}

//export callbackQWindow_Format
func callbackQWindow_Format(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "format"); signal != nil {
		return PointerFromQSurfaceFormat((*(*func() *QSurfaceFormat)(signal))())
	}

	return PointerFromQSurfaceFormat(NewQWindowFromPointer(ptr).FormatDefault())
}

func (ptr *QWindow) ConnectFormat(f func() *QSurfaceFormat) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "format"); signal != nil {
			f := func() *QSurfaceFormat {
				(*(*func() *QSurfaceFormat)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "format", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "format", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectFormat() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "format")
	}
}

func (ptr *QWindow) Format() *QSurfaceFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQSurfaceFormatFromPointer(C.QWindow_Format(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) FormatDefault() *QSurfaceFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQSurfaceFormatFromPointer(C.QWindow_FormatDefault(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) FrameGeometry() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QWindow_FrameGeometry(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) FrameMargins() *core.QMargins {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQMarginsFromPointer(C.QWindow_FrameMargins(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QMargins).DestroyQMargins)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) FramePosition() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QWindow_FramePosition(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func QWindow_FromWinId(id uintptr) *QWindow {
	tmpValue := NewQWindowFromPointer(C.QWindow_QWindow_FromWinId(C.uintptr_t(id)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QWindow) FromWinId(id uintptr) *QWindow {
	tmpValue := NewQWindowFromPointer(C.QWindow_QWindow_FromWinId(C.uintptr_t(id)))
	if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
		tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
	}
	return tmpValue
}

func (ptr *QWindow) Geometry() *core.QRect {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQRectFromPointer(C.QWindow_Geometry(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QRect).DestroyQRect)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) Height() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWindow_Height(ptr.Pointer())))
	}
	return 0
}

//export callbackQWindow_HeightChanged
func callbackQWindow_HeightChanged(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "heightChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	}

}

func (ptr *QWindow) ConnectHeightChanged(f func(arg int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "heightChanged") {
			C.QWindow_ConnectHeightChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "heightChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "heightChanged"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "heightChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "heightChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectHeightChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectHeightChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "heightChanged")
	}
}

func (ptr *QWindow) HeightChanged(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_HeightChanged(ptr.Pointer(), C.int(int32(arg)))
	}
}

//export callbackQWindow_Hide
func callbackQWindow_Hide(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "hide"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).HideDefault()
	}
}

func (ptr *QWindow) ConnectHide(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "hide"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "hide", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "hide", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectHide() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "hide")
	}
}

func (ptr *QWindow) Hide() {
	if ptr.Pointer() != nil {
		C.QWindow_Hide(ptr.Pointer())
	}
}

func (ptr *QWindow) HideDefault() {
	if ptr.Pointer() != nil {
		C.QWindow_HideDefault(ptr.Pointer())
	}
}

//export callbackQWindow_HideEvent
func callbackQWindow_HideEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "hideEvent"); signal != nil {
		(*(*func(*QHideEvent))(signal))(NewQHideEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).HideEventDefault(NewQHideEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectHideEvent(f func(ev *QHideEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "hideEvent"); signal != nil {
			f := func(ev *QHideEvent) {
				(*(*func(*QHideEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "hideEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "hideEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectHideEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "hideEvent")
	}
}

func (ptr *QWindow) HideEvent(ev QHideEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_HideEvent(ptr.Pointer(), PointerFromQHideEvent(ev))
	}
}

func (ptr *QWindow) HideEventDefault(ev QHideEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_HideEventDefault(ptr.Pointer(), PointerFromQHideEvent(ev))
	}
}

func (ptr *QWindow) Icon() *QIcon {
	if ptr.Pointer() != nil {
		tmpValue := NewQIconFromPointer(C.QWindow_Icon(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QIcon).DestroyQIcon)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) IsActive() bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_IsActive(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QWindow) IsAncestorOf(child QWindow_ITF, mode QWindow__AncestorMode) bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_IsAncestorOf(ptr.Pointer(), PointerFromQWindow(child), C.longlong(mode))) != 0
	}
	return false
}

func (ptr *QWindow) IsExposed() bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_IsExposed(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QWindow) IsModal() bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_IsModal(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QWindow) IsTopLevel() bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_IsTopLevel(ptr.Pointer())) != 0
	}
	return false
}

func (ptr *QWindow) IsVisible() bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_IsVisible(ptr.Pointer())) != 0
	}
	return false
}

//export callbackQWindow_KeyPressEvent
func callbackQWindow_KeyPressEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "keyPressEvent"); signal != nil {
		(*(*func(*QKeyEvent))(signal))(NewQKeyEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).KeyPressEventDefault(NewQKeyEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectKeyPressEvent(f func(ev *QKeyEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "keyPressEvent"); signal != nil {
			f := func(ev *QKeyEvent) {
				(*(*func(*QKeyEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "keyPressEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "keyPressEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectKeyPressEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "keyPressEvent")
	}
}

func (ptr *QWindow) KeyPressEvent(ev QKeyEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_KeyPressEvent(ptr.Pointer(), PointerFromQKeyEvent(ev))
	}
}

func (ptr *QWindow) KeyPressEventDefault(ev QKeyEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_KeyPressEventDefault(ptr.Pointer(), PointerFromQKeyEvent(ev))
	}
}

//export callbackQWindow_KeyReleaseEvent
func callbackQWindow_KeyReleaseEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "keyReleaseEvent"); signal != nil {
		(*(*func(*QKeyEvent))(signal))(NewQKeyEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).KeyReleaseEventDefault(NewQKeyEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectKeyReleaseEvent(f func(ev *QKeyEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "keyReleaseEvent"); signal != nil {
			f := func(ev *QKeyEvent) {
				(*(*func(*QKeyEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "keyReleaseEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "keyReleaseEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectKeyReleaseEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "keyReleaseEvent")
	}
}

func (ptr *QWindow) KeyReleaseEvent(ev QKeyEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_KeyReleaseEvent(ptr.Pointer(), PointerFromQKeyEvent(ev))
	}
}

func (ptr *QWindow) KeyReleaseEventDefault(ev QKeyEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_KeyReleaseEventDefault(ptr.Pointer(), PointerFromQKeyEvent(ev))
	}
}

//export callbackQWindow_Lower
func callbackQWindow_Lower(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "lower"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).LowerDefault()
	}
}

func (ptr *QWindow) ConnectLower(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "lower"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "lower", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "lower", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectLower() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "lower")
	}
}

func (ptr *QWindow) Lower() {
	if ptr.Pointer() != nil {
		C.QWindow_Lower(ptr.Pointer())
	}
}

func (ptr *QWindow) LowerDefault() {
	if ptr.Pointer() != nil {
		C.QWindow_LowerDefault(ptr.Pointer())
	}
}

func (ptr *QWindow) MapFromGlobal(pos core.QPoint_ITF) *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QWindow_MapFromGlobal(ptr.Pointer(), core.PointerFromQPoint(pos)))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) MapToGlobal(pos core.QPoint_ITF) *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QWindow_MapToGlobal(ptr.Pointer(), core.PointerFromQPoint(pos)))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) Mask() *QRegion {
	if ptr.Pointer() != nil {
		tmpValue := NewQRegionFromPointer(C.QWindow_Mask(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QRegion).DestroyQRegion)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) MaximumHeight() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWindow_MaximumHeight(ptr.Pointer())))
	}
	return 0
}

//export callbackQWindow_MaximumHeightChanged
func callbackQWindow_MaximumHeightChanged(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "maximumHeightChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	}

}

func (ptr *QWindow) ConnectMaximumHeightChanged(f func(arg int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "maximumHeightChanged") {
			C.QWindow_ConnectMaximumHeightChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "maximumHeightChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "maximumHeightChanged"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "maximumHeightChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "maximumHeightChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectMaximumHeightChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectMaximumHeightChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "maximumHeightChanged")
	}
}

func (ptr *QWindow) MaximumHeightChanged(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_MaximumHeightChanged(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) MaximumSize() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QWindow_MaximumSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) MaximumWidth() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWindow_MaximumWidth(ptr.Pointer())))
	}
	return 0
}

//export callbackQWindow_MaximumWidthChanged
func callbackQWindow_MaximumWidthChanged(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "maximumWidthChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	}

}

func (ptr *QWindow) ConnectMaximumWidthChanged(f func(arg int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "maximumWidthChanged") {
			C.QWindow_ConnectMaximumWidthChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "maximumWidthChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "maximumWidthChanged"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "maximumWidthChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "maximumWidthChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectMaximumWidthChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectMaximumWidthChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "maximumWidthChanged")
	}
}

func (ptr *QWindow) MaximumWidthChanged(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_MaximumWidthChanged(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) MinimumHeight() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWindow_MinimumHeight(ptr.Pointer())))
	}
	return 0
}

//export callbackQWindow_MinimumHeightChanged
func callbackQWindow_MinimumHeightChanged(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "minimumHeightChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	}

}

func (ptr *QWindow) ConnectMinimumHeightChanged(f func(arg int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "minimumHeightChanged") {
			C.QWindow_ConnectMinimumHeightChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "minimumHeightChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "minimumHeightChanged"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "minimumHeightChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "minimumHeightChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectMinimumHeightChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectMinimumHeightChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "minimumHeightChanged")
	}
}

func (ptr *QWindow) MinimumHeightChanged(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_MinimumHeightChanged(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) MinimumSize() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QWindow_MinimumSize(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) MinimumWidth() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWindow_MinimumWidth(ptr.Pointer())))
	}
	return 0
}

//export callbackQWindow_MinimumWidthChanged
func callbackQWindow_MinimumWidthChanged(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "minimumWidthChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	}

}

func (ptr *QWindow) ConnectMinimumWidthChanged(f func(arg int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "minimumWidthChanged") {
			C.QWindow_ConnectMinimumWidthChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "minimumWidthChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "minimumWidthChanged"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "minimumWidthChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "minimumWidthChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectMinimumWidthChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectMinimumWidthChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "minimumWidthChanged")
	}
}

func (ptr *QWindow) MinimumWidthChanged(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_MinimumWidthChanged(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) Modality() core.Qt__WindowModality {
	if ptr.Pointer() != nil {
		return core.Qt__WindowModality(C.QWindow_Modality(ptr.Pointer()))
	}
	return 0
}

//export callbackQWindow_ModalityChanged
func callbackQWindow_ModalityChanged(ptr unsafe.Pointer, modality C.longlong) {
	if signal := qt.GetSignal(ptr, "modalityChanged"); signal != nil {
		(*(*func(core.Qt__WindowModality))(signal))(core.Qt__WindowModality(modality))
	}

}

func (ptr *QWindow) ConnectModalityChanged(f func(modality core.Qt__WindowModality)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "modalityChanged") {
			C.QWindow_ConnectModalityChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "modalityChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "modalityChanged"); signal != nil {
			f := func(modality core.Qt__WindowModality) {
				(*(*func(core.Qt__WindowModality))(signal))(modality)
				f(modality)
			}
			qt.ConnectSignal(ptr.Pointer(), "modalityChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "modalityChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectModalityChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectModalityChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "modalityChanged")
	}
}

func (ptr *QWindow) ModalityChanged(modality core.Qt__WindowModality) {
	if ptr.Pointer() != nil {
		C.QWindow_ModalityChanged(ptr.Pointer(), C.longlong(modality))
	}
}

//export callbackQWindow_MouseDoubleClickEvent
func callbackQWindow_MouseDoubleClickEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "mouseDoubleClickEvent"); signal != nil {
		(*(*func(*QMouseEvent))(signal))(NewQMouseEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).MouseDoubleClickEventDefault(NewQMouseEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectMouseDoubleClickEvent(f func(ev *QMouseEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "mouseDoubleClickEvent"); signal != nil {
			f := func(ev *QMouseEvent) {
				(*(*func(*QMouseEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "mouseDoubleClickEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "mouseDoubleClickEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectMouseDoubleClickEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "mouseDoubleClickEvent")
	}
}

func (ptr *QWindow) MouseDoubleClickEvent(ev QMouseEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_MouseDoubleClickEvent(ptr.Pointer(), PointerFromQMouseEvent(ev))
	}
}

func (ptr *QWindow) MouseDoubleClickEventDefault(ev QMouseEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_MouseDoubleClickEventDefault(ptr.Pointer(), PointerFromQMouseEvent(ev))
	}
}

//export callbackQWindow_MouseMoveEvent
func callbackQWindow_MouseMoveEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "mouseMoveEvent"); signal != nil {
		(*(*func(*QMouseEvent))(signal))(NewQMouseEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).MouseMoveEventDefault(NewQMouseEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectMouseMoveEvent(f func(ev *QMouseEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "mouseMoveEvent"); signal != nil {
			f := func(ev *QMouseEvent) {
				(*(*func(*QMouseEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "mouseMoveEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "mouseMoveEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectMouseMoveEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "mouseMoveEvent")
	}
}

func (ptr *QWindow) MouseMoveEvent(ev QMouseEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_MouseMoveEvent(ptr.Pointer(), PointerFromQMouseEvent(ev))
	}
}

func (ptr *QWindow) MouseMoveEventDefault(ev QMouseEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_MouseMoveEventDefault(ptr.Pointer(), PointerFromQMouseEvent(ev))
	}
}

//export callbackQWindow_MousePressEvent
func callbackQWindow_MousePressEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "mousePressEvent"); signal != nil {
		(*(*func(*QMouseEvent))(signal))(NewQMouseEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).MousePressEventDefault(NewQMouseEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectMousePressEvent(f func(ev *QMouseEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "mousePressEvent"); signal != nil {
			f := func(ev *QMouseEvent) {
				(*(*func(*QMouseEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "mousePressEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "mousePressEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectMousePressEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "mousePressEvent")
	}
}

func (ptr *QWindow) MousePressEvent(ev QMouseEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_MousePressEvent(ptr.Pointer(), PointerFromQMouseEvent(ev))
	}
}

func (ptr *QWindow) MousePressEventDefault(ev QMouseEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_MousePressEventDefault(ptr.Pointer(), PointerFromQMouseEvent(ev))
	}
}

//export callbackQWindow_MouseReleaseEvent
func callbackQWindow_MouseReleaseEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "mouseReleaseEvent"); signal != nil {
		(*(*func(*QMouseEvent))(signal))(NewQMouseEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).MouseReleaseEventDefault(NewQMouseEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectMouseReleaseEvent(f func(ev *QMouseEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "mouseReleaseEvent"); signal != nil {
			f := func(ev *QMouseEvent) {
				(*(*func(*QMouseEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "mouseReleaseEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "mouseReleaseEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectMouseReleaseEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "mouseReleaseEvent")
	}
}

func (ptr *QWindow) MouseReleaseEvent(ev QMouseEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_MouseReleaseEvent(ptr.Pointer(), PointerFromQMouseEvent(ev))
	}
}

func (ptr *QWindow) MouseReleaseEventDefault(ev QMouseEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_MouseReleaseEventDefault(ptr.Pointer(), PointerFromQMouseEvent(ev))
	}
}

//export callbackQWindow_MoveEvent
func callbackQWindow_MoveEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "moveEvent"); signal != nil {
		(*(*func(*QMoveEvent))(signal))(NewQMoveEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).MoveEventDefault(NewQMoveEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectMoveEvent(f func(ev *QMoveEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "moveEvent"); signal != nil {
			f := func(ev *QMoveEvent) {
				(*(*func(*QMoveEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "moveEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "moveEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectMoveEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "moveEvent")
	}
}

func (ptr *QWindow) MoveEvent(ev QMoveEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_MoveEvent(ptr.Pointer(), PointerFromQMoveEvent(ev))
	}
}

func (ptr *QWindow) MoveEventDefault(ev QMoveEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_MoveEventDefault(ptr.Pointer(), PointerFromQMoveEvent(ev))
	}
}

//export callbackQWindow_NativeEvent
func callbackQWindow_NativeEvent(ptr unsafe.Pointer, eventType unsafe.Pointer, message unsafe.Pointer, result *C.long) C.char {
	var resultR int
	if result != nil {
		resultR = int(int32(*result))
		defer func() { *result = C.long(int32(resultR)) }()
	}
	if signal := qt.GetSignal(ptr, "nativeEvent"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QByteArray, unsafe.Pointer, *int) bool)(signal))(core.NewQByteArrayFromPointer(eventType), message, &resultR))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQWindowFromPointer(ptr).NativeEventDefault(core.NewQByteArrayFromPointer(eventType), message, &resultR))))
}

func (ptr *QWindow) ConnectNativeEvent(f func(eventType *core.QByteArray, message unsafe.Pointer, result *int) bool) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "nativeEvent"); signal != nil {
			f := func(eventType *core.QByteArray, message unsafe.Pointer, result *int) bool {
				(*(*func(*core.QByteArray, unsafe.Pointer, *int) bool)(signal))(eventType, message, result)
				return f(eventType, message, result)
			}
			qt.ConnectSignal(ptr.Pointer(), "nativeEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "nativeEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectNativeEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "nativeEvent")
	}
}

func (ptr *QWindow) NativeEvent(eventType core.QByteArray_ITF, message unsafe.Pointer, result *int) bool {
	if ptr.Pointer() != nil {
		var resultC C.long
		if result != nil {
			resultC = C.long(int32(*result))
			defer func() { *result = int(int32(resultC)) }()
		}
		return int8(C.QWindow_NativeEvent(ptr.Pointer(), core.PointerFromQByteArray(eventType), message, &resultC)) != 0
	}
	return false
}

func (ptr *QWindow) NativeEventDefault(eventType core.QByteArray_ITF, message unsafe.Pointer, result *int) bool {
	if ptr.Pointer() != nil {
		var resultC C.long
		if result != nil {
			resultC = C.long(int32(*result))
			defer func() { *result = int(int32(resultC)) }()
		}
		return int8(C.QWindow_NativeEventDefault(ptr.Pointer(), core.PointerFromQByteArray(eventType), message, &resultC)) != 0
	}
	return false
}

func (ptr *QWindow) Opacity() float64 {
	if ptr.Pointer() != nil {
		return float64(C.QWindow_Opacity(ptr.Pointer()))
	}
	return 0
}

//export callbackQWindow_OpacityChanged
func callbackQWindow_OpacityChanged(ptr unsafe.Pointer, opacity C.double) {
	if signal := qt.GetSignal(ptr, "opacityChanged"); signal != nil {
		(*(*func(float64))(signal))(float64(opacity))
	}

}

func (ptr *QWindow) ConnectOpacityChanged(f func(opacity float64)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "opacityChanged") {
			C.QWindow_ConnectOpacityChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "opacityChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "opacityChanged"); signal != nil {
			f := func(opacity float64) {
				(*(*func(float64))(signal))(opacity)
				f(opacity)
			}
			qt.ConnectSignal(ptr.Pointer(), "opacityChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "opacityChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectOpacityChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectOpacityChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "opacityChanged")
	}
}

func (ptr *QWindow) OpacityChanged(opacity float64) {
	if ptr.Pointer() != nil {
		C.QWindow_OpacityChanged(ptr.Pointer(), C.double(opacity))
	}
}

func (ptr *QWindow) Parent(mode QWindow__AncestorMode) *QWindow {
	if ptr.Pointer() != nil {
		tmpValue := NewQWindowFromPointer(C.QWindow_Parent(ptr.Pointer(), C.longlong(mode)))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) Parent2() *QWindow {
	if ptr.Pointer() != nil {
		tmpValue := NewQWindowFromPointer(C.QWindow_Parent2(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) Position() *core.QPoint {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQPointFromPointer(C.QWindow_Position(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QPoint).DestroyQPoint)
		return tmpValue
	}
	return nil
}

//export callbackQWindow_Raise
func callbackQWindow_Raise(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "raise"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).RaiseDefault()
	}
}

func (ptr *QWindow) ConnectRaise(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "raise"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "raise", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "raise", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectRaise() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "raise")
	}
}

func (ptr *QWindow) Raise() {
	if ptr.Pointer() != nil {
		C.QWindow_Raise(ptr.Pointer())
	}
}

func (ptr *QWindow) RaiseDefault() {
	if ptr.Pointer() != nil {
		C.QWindow_RaiseDefault(ptr.Pointer())
	}
}

func (ptr *QWindow) ReportContentOrientationChange(orientation core.Qt__ScreenOrientation) {
	if ptr.Pointer() != nil {
		C.QWindow_ReportContentOrientationChange(ptr.Pointer(), C.longlong(orientation))
	}
}

//export callbackQWindow_RequestActivate
func callbackQWindow_RequestActivate(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "requestActivate"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).RequestActivateDefault()
	}
}

func (ptr *QWindow) ConnectRequestActivate(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "requestActivate"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "requestActivate", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "requestActivate", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectRequestActivate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "requestActivate")
	}
}

func (ptr *QWindow) RequestActivate() {
	if ptr.Pointer() != nil {
		C.QWindow_RequestActivate(ptr.Pointer())
	}
}

func (ptr *QWindow) RequestActivateDefault() {
	if ptr.Pointer() != nil {
		C.QWindow_RequestActivateDefault(ptr.Pointer())
	}
}

//export callbackQWindow_RequestUpdate
func callbackQWindow_RequestUpdate(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "requestUpdate"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).RequestUpdateDefault()
	}
}

func (ptr *QWindow) ConnectRequestUpdate(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "requestUpdate"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "requestUpdate", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "requestUpdate", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectRequestUpdate() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "requestUpdate")
	}
}

func (ptr *QWindow) RequestUpdate() {
	if ptr.Pointer() != nil {
		C.QWindow_RequestUpdate(ptr.Pointer())
	}
}

func (ptr *QWindow) RequestUpdateDefault() {
	if ptr.Pointer() != nil {
		C.QWindow_RequestUpdateDefault(ptr.Pointer())
	}
}

func (ptr *QWindow) RequestedFormat() *QSurfaceFormat {
	if ptr.Pointer() != nil {
		tmpValue := NewQSurfaceFormatFromPointer(C.QWindow_RequestedFormat(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*QSurfaceFormat).DestroyQSurfaceFormat)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) Resize(newSize core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_Resize(ptr.Pointer(), core.PointerFromQSize(newSize))
	}
}

func (ptr *QWindow) Resize2(w int, h int) {
	if ptr.Pointer() != nil {
		C.QWindow_Resize2(ptr.Pointer(), C.int(int32(w)), C.int(int32(h)))
	}
}

//export callbackQWindow_ResizeEvent
func callbackQWindow_ResizeEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "resizeEvent"); signal != nil {
		(*(*func(*QResizeEvent))(signal))(NewQResizeEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).ResizeEventDefault(NewQResizeEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectResizeEvent(f func(ev *QResizeEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "resizeEvent"); signal != nil {
			f := func(ev *QResizeEvent) {
				(*(*func(*QResizeEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "resizeEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "resizeEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectResizeEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "resizeEvent")
	}
}

func (ptr *QWindow) ResizeEvent(ev QResizeEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ResizeEvent(ptr.Pointer(), PointerFromQResizeEvent(ev))
	}
}

func (ptr *QWindow) ResizeEventDefault(ev QResizeEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ResizeEventDefault(ptr.Pointer(), PointerFromQResizeEvent(ev))
	}
}

func (ptr *QWindow) Screen() *QScreen {
	if ptr.Pointer() != nil {
		tmpValue := NewQScreenFromPointer(C.QWindow_Screen(ptr.Pointer()))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

//export callbackQWindow_ScreenChanged
func callbackQWindow_ScreenChanged(ptr unsafe.Pointer, screen unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "screenChanged"); signal != nil {
		(*(*func(*QScreen))(signal))(NewQScreenFromPointer(screen))
	}

}

func (ptr *QWindow) ConnectScreenChanged(f func(screen *QScreen)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "screenChanged") {
			C.QWindow_ConnectScreenChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "screenChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "screenChanged"); signal != nil {
			f := func(screen *QScreen) {
				(*(*func(*QScreen))(signal))(screen)
				f(screen)
			}
			qt.ConnectSignal(ptr.Pointer(), "screenChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "screenChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectScreenChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectScreenChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "screenChanged")
	}
}

func (ptr *QWindow) ScreenChanged(screen QScreen_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ScreenChanged(ptr.Pointer(), PointerFromQScreen(screen))
	}
}

func (ptr *QWindow) SetBaseSize(size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetBaseSize(ptr.Pointer(), core.PointerFromQSize(size))
	}
}

func (ptr *QWindow) SetCursor(cursor QCursor_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetCursor(ptr.Pointer(), PointerFromQCursor(cursor))
	}
}

func (ptr *QWindow) SetFilePath(filePath string) {
	if ptr.Pointer() != nil {
		var filePathC *C.char
		if filePath != "" {
			filePathC = C.CString(filePath)
			defer C.free(unsafe.Pointer(filePathC))
		}
		C.QWindow_SetFilePath(ptr.Pointer(), C.struct_QtGui_PackedString{data: filePathC, len: C.longlong(len(filePath))})
	}
}

func (ptr *QWindow) SetFlag(flag core.Qt__WindowType, on bool) {
	if ptr.Pointer() != nil {
		C.QWindow_SetFlag(ptr.Pointer(), C.longlong(flag), C.char(int8(qt.GoBoolToInt(on))))
	}
}

func (ptr *QWindow) SetFlags(flags core.Qt__WindowType) {
	if ptr.Pointer() != nil {
		C.QWindow_SetFlags(ptr.Pointer(), C.longlong(flags))
	}
}

func (ptr *QWindow) SetFormat(format QSurfaceFormat_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetFormat(ptr.Pointer(), PointerFromQSurfaceFormat(format))
	}
}

func (ptr *QWindow) SetFramePosition(point core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetFramePosition(ptr.Pointer(), core.PointerFromQPoint(point))
	}
}

//export callbackQWindow_SetGeometry
func callbackQWindow_SetGeometry(ptr unsafe.Pointer, posx C.int, posy C.int, w C.int, h C.int) {
	if signal := qt.GetSignal(ptr, "setGeometry"); signal != nil {
		(*(*func(int, int, int, int))(signal))(int(int32(posx)), int(int32(posy)), int(int32(w)), int(int32(h)))
	} else {
		NewQWindowFromPointer(ptr).SetGeometryDefault(int(int32(posx)), int(int32(posy)), int(int32(w)), int(int32(h)))
	}
}

func (ptr *QWindow) ConnectSetGeometry(f func(posx int, posy int, w int, h int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setGeometry"); signal != nil {
			f := func(posx int, posy int, w int, h int) {
				(*(*func(int, int, int, int))(signal))(posx, posy, w, h)
				f(posx, posy, w, h)
			}
			qt.ConnectSignal(ptr.Pointer(), "setGeometry", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setGeometry", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetGeometry() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setGeometry")
	}
}

func (ptr *QWindow) SetGeometry(posx int, posy int, w int, h int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetGeometry(ptr.Pointer(), C.int(int32(posx)), C.int(int32(posy)), C.int(int32(w)), C.int(int32(h)))
	}
}

func (ptr *QWindow) SetGeometryDefault(posx int, posy int, w int, h int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetGeometryDefault(ptr.Pointer(), C.int(int32(posx)), C.int(int32(posy)), C.int(int32(w)), C.int(int32(h)))
	}
}

//export callbackQWindow_SetGeometry2
func callbackQWindow_SetGeometry2(ptr unsafe.Pointer, rect unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "setGeometry2"); signal != nil {
		(*(*func(*core.QRect))(signal))(core.NewQRectFromPointer(rect))
	} else {
		NewQWindowFromPointer(ptr).SetGeometry2Default(core.NewQRectFromPointer(rect))
	}
}

func (ptr *QWindow) ConnectSetGeometry2(f func(rect *core.QRect)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setGeometry2"); signal != nil {
			f := func(rect *core.QRect) {
				(*(*func(*core.QRect))(signal))(rect)
				f(rect)
			}
			qt.ConnectSignal(ptr.Pointer(), "setGeometry2", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setGeometry2", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetGeometry2() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setGeometry2")
	}
}

func (ptr *QWindow) SetGeometry2(rect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetGeometry2(ptr.Pointer(), core.PointerFromQRect(rect))
	}
}

func (ptr *QWindow) SetGeometry2Default(rect core.QRect_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetGeometry2Default(ptr.Pointer(), core.PointerFromQRect(rect))
	}
}

//export callbackQWindow_SetHeight
func callbackQWindow_SetHeight(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "setHeight"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	} else {
		NewQWindowFromPointer(ptr).SetHeightDefault(int(int32(arg)))
	}
}

func (ptr *QWindow) ConnectSetHeight(f func(arg int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setHeight"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "setHeight", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setHeight", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetHeight() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setHeight")
	}
}

func (ptr *QWindow) SetHeight(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetHeight(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) SetHeightDefault(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetHeightDefault(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) SetIcon(icon QIcon_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetIcon(ptr.Pointer(), PointerFromQIcon(icon))
	}
}

func (ptr *QWindow) SetKeyboardGrabEnabled(grab bool) bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_SetKeyboardGrabEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(grab))))) != 0
	}
	return false
}

func (ptr *QWindow) SetMask(region QRegion_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMask(ptr.Pointer(), PointerFromQRegion(region))
	}
}

//export callbackQWindow_SetMaximumHeight
func callbackQWindow_SetMaximumHeight(ptr unsafe.Pointer, h C.int) {
	if signal := qt.GetSignal(ptr, "setMaximumHeight"); signal != nil {
		(*(*func(int))(signal))(int(int32(h)))
	} else {
		NewQWindowFromPointer(ptr).SetMaximumHeightDefault(int(int32(h)))
	}
}

func (ptr *QWindow) ConnectSetMaximumHeight(f func(h int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setMaximumHeight"); signal != nil {
			f := func(h int) {
				(*(*func(int))(signal))(h)
				f(h)
			}
			qt.ConnectSignal(ptr.Pointer(), "setMaximumHeight", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setMaximumHeight", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetMaximumHeight() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setMaximumHeight")
	}
}

func (ptr *QWindow) SetMaximumHeight(h int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMaximumHeight(ptr.Pointer(), C.int(int32(h)))
	}
}

func (ptr *QWindow) SetMaximumHeightDefault(h int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMaximumHeightDefault(ptr.Pointer(), C.int(int32(h)))
	}
}

func (ptr *QWindow) SetMaximumSize(size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMaximumSize(ptr.Pointer(), core.PointerFromQSize(size))
	}
}

//export callbackQWindow_SetMaximumWidth
func callbackQWindow_SetMaximumWidth(ptr unsafe.Pointer, w C.int) {
	if signal := qt.GetSignal(ptr, "setMaximumWidth"); signal != nil {
		(*(*func(int))(signal))(int(int32(w)))
	} else {
		NewQWindowFromPointer(ptr).SetMaximumWidthDefault(int(int32(w)))
	}
}

func (ptr *QWindow) ConnectSetMaximumWidth(f func(w int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setMaximumWidth"); signal != nil {
			f := func(w int) {
				(*(*func(int))(signal))(w)
				f(w)
			}
			qt.ConnectSignal(ptr.Pointer(), "setMaximumWidth", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setMaximumWidth", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetMaximumWidth() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setMaximumWidth")
	}
}

func (ptr *QWindow) SetMaximumWidth(w int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMaximumWidth(ptr.Pointer(), C.int(int32(w)))
	}
}

func (ptr *QWindow) SetMaximumWidthDefault(w int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMaximumWidthDefault(ptr.Pointer(), C.int(int32(w)))
	}
}

//export callbackQWindow_SetMinimumHeight
func callbackQWindow_SetMinimumHeight(ptr unsafe.Pointer, h C.int) {
	if signal := qt.GetSignal(ptr, "setMinimumHeight"); signal != nil {
		(*(*func(int))(signal))(int(int32(h)))
	} else {
		NewQWindowFromPointer(ptr).SetMinimumHeightDefault(int(int32(h)))
	}
}

func (ptr *QWindow) ConnectSetMinimumHeight(f func(h int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setMinimumHeight"); signal != nil {
			f := func(h int) {
				(*(*func(int))(signal))(h)
				f(h)
			}
			qt.ConnectSignal(ptr.Pointer(), "setMinimumHeight", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setMinimumHeight", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetMinimumHeight() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setMinimumHeight")
	}
}

func (ptr *QWindow) SetMinimumHeight(h int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMinimumHeight(ptr.Pointer(), C.int(int32(h)))
	}
}

func (ptr *QWindow) SetMinimumHeightDefault(h int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMinimumHeightDefault(ptr.Pointer(), C.int(int32(h)))
	}
}

func (ptr *QWindow) SetMinimumSize(size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMinimumSize(ptr.Pointer(), core.PointerFromQSize(size))
	}
}

//export callbackQWindow_SetMinimumWidth
func callbackQWindow_SetMinimumWidth(ptr unsafe.Pointer, w C.int) {
	if signal := qt.GetSignal(ptr, "setMinimumWidth"); signal != nil {
		(*(*func(int))(signal))(int(int32(w)))
	} else {
		NewQWindowFromPointer(ptr).SetMinimumWidthDefault(int(int32(w)))
	}
}

func (ptr *QWindow) ConnectSetMinimumWidth(f func(w int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setMinimumWidth"); signal != nil {
			f := func(w int) {
				(*(*func(int))(signal))(w)
				f(w)
			}
			qt.ConnectSignal(ptr.Pointer(), "setMinimumWidth", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setMinimumWidth", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetMinimumWidth() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setMinimumWidth")
	}
}

func (ptr *QWindow) SetMinimumWidth(w int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMinimumWidth(ptr.Pointer(), C.int(int32(w)))
	}
}

func (ptr *QWindow) SetMinimumWidthDefault(w int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetMinimumWidthDefault(ptr.Pointer(), C.int(int32(w)))
	}
}

func (ptr *QWindow) SetModality(modality core.Qt__WindowModality) {
	if ptr.Pointer() != nil {
		C.QWindow_SetModality(ptr.Pointer(), C.longlong(modality))
	}
}

func (ptr *QWindow) SetMouseGrabEnabled(grab bool) bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_SetMouseGrabEnabled(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(grab))))) != 0
	}
	return false
}

func (ptr *QWindow) SetOpacity(level float64) {
	if ptr.Pointer() != nil {
		C.QWindow_SetOpacity(ptr.Pointer(), C.double(level))
	}
}

func (ptr *QWindow) SetParent(parent QWindow_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetParent(ptr.Pointer(), PointerFromQWindow(parent))
	}
}

func (ptr *QWindow) SetPosition(pt core.QPoint_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetPosition(ptr.Pointer(), core.PointerFromQPoint(pt))
	}
}

func (ptr *QWindow) SetPosition2(posx int, posy int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetPosition2(ptr.Pointer(), C.int(int32(posx)), C.int(int32(posy)))
	}
}

func (ptr *QWindow) SetScreen(newScreen QScreen_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetScreen(ptr.Pointer(), PointerFromQScreen(newScreen))
	}
}

func (ptr *QWindow) SetSizeIncrement(size core.QSize_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_SetSizeIncrement(ptr.Pointer(), core.PointerFromQSize(size))
	}
}

func (ptr *QWindow) SetSurfaceType(surfaceType QSurface__SurfaceType) {
	if ptr.Pointer() != nil {
		C.QWindow_SetSurfaceType(ptr.Pointer(), C.longlong(surfaceType))
	}
}

//export callbackQWindow_SetTitle
func callbackQWindow_SetTitle(ptr unsafe.Pointer, vqs C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "setTitle"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(vqs))
	} else {
		NewQWindowFromPointer(ptr).SetTitleDefault(cGoUnpackString(vqs))
	}
}

func (ptr *QWindow) ConnectSetTitle(f func(vqs string)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setTitle"); signal != nil {
			f := func(vqs string) {
				(*(*func(string))(signal))(vqs)
				f(vqs)
			}
			qt.ConnectSignal(ptr.Pointer(), "setTitle", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setTitle", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetTitle() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setTitle")
	}
}

func (ptr *QWindow) SetTitle(vqs string) {
	if ptr.Pointer() != nil {
		var vqsC *C.char
		if vqs != "" {
			vqsC = C.CString(vqs)
			defer C.free(unsafe.Pointer(vqsC))
		}
		C.QWindow_SetTitle(ptr.Pointer(), C.struct_QtGui_PackedString{data: vqsC, len: C.longlong(len(vqs))})
	}
}

func (ptr *QWindow) SetTitleDefault(vqs string) {
	if ptr.Pointer() != nil {
		var vqsC *C.char
		if vqs != "" {
			vqsC = C.CString(vqs)
			defer C.free(unsafe.Pointer(vqsC))
		}
		C.QWindow_SetTitleDefault(ptr.Pointer(), C.struct_QtGui_PackedString{data: vqsC, len: C.longlong(len(vqs))})
	}
}

func (ptr *QWindow) SetVisibility(v QWindow__Visibility) {
	if ptr.Pointer() != nil {
		C.QWindow_SetVisibility(ptr.Pointer(), C.longlong(v))
	}
}

//export callbackQWindow_SetVisible
func callbackQWindow_SetVisible(ptr unsafe.Pointer, visible C.char) {
	if signal := qt.GetSignal(ptr, "setVisible"); signal != nil {
		(*(*func(bool))(signal))(int8(visible) != 0)
	} else {
		NewQWindowFromPointer(ptr).SetVisibleDefault(int8(visible) != 0)
	}
}

func (ptr *QWindow) ConnectSetVisible(f func(visible bool)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setVisible"); signal != nil {
			f := func(visible bool) {
				(*(*func(bool))(signal))(visible)
				f(visible)
			}
			qt.ConnectSignal(ptr.Pointer(), "setVisible", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setVisible", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetVisible() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setVisible")
	}
}

func (ptr *QWindow) SetVisible(visible bool) {
	if ptr.Pointer() != nil {
		C.QWindow_SetVisible(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(visible))))
	}
}

func (ptr *QWindow) SetVisibleDefault(visible bool) {
	if ptr.Pointer() != nil {
		C.QWindow_SetVisibleDefault(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(visible))))
	}
}

//export callbackQWindow_SetWidth
func callbackQWindow_SetWidth(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "setWidth"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	} else {
		NewQWindowFromPointer(ptr).SetWidthDefault(int(int32(arg)))
	}
}

func (ptr *QWindow) ConnectSetWidth(f func(arg int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setWidth"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "setWidth", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setWidth", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetWidth() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setWidth")
	}
}

func (ptr *QWindow) SetWidth(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetWidth(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) SetWidthDefault(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetWidthDefault(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) SetWindowState(state core.Qt__WindowState) {
	if ptr.Pointer() != nil {
		C.QWindow_SetWindowState(ptr.Pointer(), C.longlong(state))
	}
}

func (ptr *QWindow) SetWindowStates(state core.Qt__WindowState) {
	if ptr.Pointer() != nil {
		C.QWindow_SetWindowStates(ptr.Pointer(), C.longlong(state))
	}
}

//export callbackQWindow_SetX
func callbackQWindow_SetX(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "setX"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	} else {
		NewQWindowFromPointer(ptr).SetXDefault(int(int32(arg)))
	}
}

func (ptr *QWindow) ConnectSetX(f func(arg int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setX"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "setX", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setX", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetX() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setX")
	}
}

func (ptr *QWindow) SetX(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetX(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) SetXDefault(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetXDefault(ptr.Pointer(), C.int(int32(arg)))
	}
}

//export callbackQWindow_SetY
func callbackQWindow_SetY(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "setY"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	} else {
		NewQWindowFromPointer(ptr).SetYDefault(int(int32(arg)))
	}
}

func (ptr *QWindow) ConnectSetY(f func(arg int)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "setY"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "setY", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "setY", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSetY() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "setY")
	}
}

func (ptr *QWindow) SetY(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetY(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) SetYDefault(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_SetYDefault(ptr.Pointer(), C.int(int32(arg)))
	}
}

//export callbackQWindow_Show
func callbackQWindow_Show(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "show"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).ShowDefault()
	}
}

func (ptr *QWindow) ConnectShow(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "show"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "show", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "show", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectShow() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "show")
	}
}

func (ptr *QWindow) Show() {
	if ptr.Pointer() != nil {
		C.QWindow_Show(ptr.Pointer())
	}
}

func (ptr *QWindow) ShowDefault() {
	if ptr.Pointer() != nil {
		C.QWindow_ShowDefault(ptr.Pointer())
	}
}

//export callbackQWindow_ShowEvent
func callbackQWindow_ShowEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "showEvent"); signal != nil {
		(*(*func(*QShowEvent))(signal))(NewQShowEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).ShowEventDefault(NewQShowEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectShowEvent(f func(ev *QShowEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "showEvent"); signal != nil {
			f := func(ev *QShowEvent) {
				(*(*func(*QShowEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "showEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "showEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectShowEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "showEvent")
	}
}

func (ptr *QWindow) ShowEvent(ev QShowEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ShowEvent(ptr.Pointer(), PointerFromQShowEvent(ev))
	}
}

func (ptr *QWindow) ShowEventDefault(ev QShowEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ShowEventDefault(ptr.Pointer(), PointerFromQShowEvent(ev))
	}
}

//export callbackQWindow_ShowFullScreen
func callbackQWindow_ShowFullScreen(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "showFullScreen"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).ShowFullScreenDefault()
	}
}

func (ptr *QWindow) ConnectShowFullScreen(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "showFullScreen"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "showFullScreen", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "showFullScreen", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectShowFullScreen() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "showFullScreen")
	}
}

func (ptr *QWindow) ShowFullScreen() {
	if ptr.Pointer() != nil {
		C.QWindow_ShowFullScreen(ptr.Pointer())
	}
}

func (ptr *QWindow) ShowFullScreenDefault() {
	if ptr.Pointer() != nil {
		C.QWindow_ShowFullScreenDefault(ptr.Pointer())
	}
}

//export callbackQWindow_ShowMaximized
func callbackQWindow_ShowMaximized(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "showMaximized"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).ShowMaximizedDefault()
	}
}

func (ptr *QWindow) ConnectShowMaximized(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "showMaximized"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "showMaximized", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "showMaximized", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectShowMaximized() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "showMaximized")
	}
}

func (ptr *QWindow) ShowMaximized() {
	if ptr.Pointer() != nil {
		C.QWindow_ShowMaximized(ptr.Pointer())
	}
}

func (ptr *QWindow) ShowMaximizedDefault() {
	if ptr.Pointer() != nil {
		C.QWindow_ShowMaximizedDefault(ptr.Pointer())
	}
}

//export callbackQWindow_ShowMinimized
func callbackQWindow_ShowMinimized(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "showMinimized"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).ShowMinimizedDefault()
	}
}

func (ptr *QWindow) ConnectShowMinimized(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "showMinimized"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "showMinimized", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "showMinimized", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectShowMinimized() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "showMinimized")
	}
}

func (ptr *QWindow) ShowMinimized() {
	if ptr.Pointer() != nil {
		C.QWindow_ShowMinimized(ptr.Pointer())
	}
}

func (ptr *QWindow) ShowMinimizedDefault() {
	if ptr.Pointer() != nil {
		C.QWindow_ShowMinimizedDefault(ptr.Pointer())
	}
}

//export callbackQWindow_ShowNormal
func callbackQWindow_ShowNormal(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "showNormal"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).ShowNormalDefault()
	}
}

func (ptr *QWindow) ConnectShowNormal(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "showNormal"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "showNormal", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "showNormal", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectShowNormal() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "showNormal")
	}
}

func (ptr *QWindow) ShowNormal() {
	if ptr.Pointer() != nil {
		C.QWindow_ShowNormal(ptr.Pointer())
	}
}

func (ptr *QWindow) ShowNormalDefault() {
	if ptr.Pointer() != nil {
		C.QWindow_ShowNormalDefault(ptr.Pointer())
	}
}

//export callbackQWindow_Size
func callbackQWindow_Size(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "size"); signal != nil {
		return core.PointerFromQSize((*(*func() *core.QSize)(signal))())
	}

	return core.PointerFromQSize(NewQWindowFromPointer(ptr).SizeDefault())
}

func (ptr *QWindow) ConnectSize(f func() *core.QSize) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "size"); signal != nil {
			f := func() *core.QSize {
				(*(*func() *core.QSize)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "size", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "size", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSize() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "size")
	}
}

func (ptr *QWindow) Size() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QWindow_Size(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) SizeDefault() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QWindow_SizeDefault(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) SizeIncrement() *core.QSize {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQSizeFromPointer(C.QWindow_SizeIncrement(ptr.Pointer()))
		qt.SetFinalizer(tmpValue, (*core.QSize).DestroyQSize)
		return tmpValue
	}
	return nil
}

//export callbackQWindow_SurfaceType
func callbackQWindow_SurfaceType(ptr unsafe.Pointer) C.longlong {
	if signal := qt.GetSignal(ptr, "surfaceType"); signal != nil {
		return C.longlong((*(*func() QSurface__SurfaceType)(signal))())
	}

	return C.longlong(NewQWindowFromPointer(ptr).SurfaceTypeDefault())
}

func (ptr *QWindow) ConnectSurfaceType(f func() QSurface__SurfaceType) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "surfaceType"); signal != nil {
			f := func() QSurface__SurfaceType {
				(*(*func() QSurface__SurfaceType)(signal))()
				return f()
			}
			qt.ConnectSignal(ptr.Pointer(), "surfaceType", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "surfaceType", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectSurfaceType() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "surfaceType")
	}
}

func (ptr *QWindow) SurfaceType() QSurface__SurfaceType {
	if ptr.Pointer() != nil {
		return QSurface__SurfaceType(C.QWindow_SurfaceType(ptr.Pointer()))
	}
	return 0
}

func (ptr *QWindow) SurfaceTypeDefault() QSurface__SurfaceType {
	if ptr.Pointer() != nil {
		return QSurface__SurfaceType(C.QWindow_SurfaceTypeDefault(ptr.Pointer()))
	}
	return 0
}

//export callbackQWindow_TabletEvent
func callbackQWindow_TabletEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "tabletEvent"); signal != nil {
		(*(*func(*QTabletEvent))(signal))(NewQTabletEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).TabletEventDefault(NewQTabletEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectTabletEvent(f func(ev *QTabletEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "tabletEvent"); signal != nil {
			f := func(ev *QTabletEvent) {
				(*(*func(*QTabletEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "tabletEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "tabletEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectTabletEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "tabletEvent")
	}
}

func (ptr *QWindow) TabletEvent(ev QTabletEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_TabletEvent(ptr.Pointer(), PointerFromQTabletEvent(ev))
	}
}

func (ptr *QWindow) TabletEventDefault(ev QTabletEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_TabletEventDefault(ptr.Pointer(), PointerFromQTabletEvent(ev))
	}
}

func (ptr *QWindow) Title() string {
	if ptr.Pointer() != nil {
		return cGoUnpackString(C.QWindow_Title(ptr.Pointer()))
	}
	return ""
}

//export callbackQWindow_TouchEvent
func callbackQWindow_TouchEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "touchEvent"); signal != nil {
		(*(*func(*QTouchEvent))(signal))(NewQTouchEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).TouchEventDefault(NewQTouchEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectTouchEvent(f func(ev *QTouchEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "touchEvent"); signal != nil {
			f := func(ev *QTouchEvent) {
				(*(*func(*QTouchEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "touchEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "touchEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectTouchEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "touchEvent")
	}
}

func (ptr *QWindow) TouchEvent(ev QTouchEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_TouchEvent(ptr.Pointer(), PointerFromQTouchEvent(ev))
	}
}

func (ptr *QWindow) TouchEventDefault(ev QTouchEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_TouchEventDefault(ptr.Pointer(), PointerFromQTouchEvent(ev))
	}
}

func (ptr *QWindow) Type() core.Qt__WindowType {
	if ptr.Pointer() != nil {
		return core.Qt__WindowType(C.QWindow_Type(ptr.Pointer()))
	}
	return 0
}

func (ptr *QWindow) UnsetCursor() {
	if ptr.Pointer() != nil {
		C.QWindow_UnsetCursor(ptr.Pointer())
	}
}

func (ptr *QWindow) Visibility() QWindow__Visibility {
	if ptr.Pointer() != nil {
		return QWindow__Visibility(C.QWindow_Visibility(ptr.Pointer()))
	}
	return 0
}

//export callbackQWindow_VisibilityChanged
func callbackQWindow_VisibilityChanged(ptr unsafe.Pointer, visibility C.longlong) {
	if signal := qt.GetSignal(ptr, "visibilityChanged"); signal != nil {
		(*(*func(QWindow__Visibility))(signal))(QWindow__Visibility(visibility))
	}

}

func (ptr *QWindow) ConnectVisibilityChanged(f func(visibility QWindow__Visibility)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "visibilityChanged") {
			C.QWindow_ConnectVisibilityChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "visibilityChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "visibilityChanged"); signal != nil {
			f := func(visibility QWindow__Visibility) {
				(*(*func(QWindow__Visibility))(signal))(visibility)
				f(visibility)
			}
			qt.ConnectSignal(ptr.Pointer(), "visibilityChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "visibilityChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectVisibilityChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectVisibilityChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "visibilityChanged")
	}
}

func (ptr *QWindow) VisibilityChanged(visibility QWindow__Visibility) {
	if ptr.Pointer() != nil {
		C.QWindow_VisibilityChanged(ptr.Pointer(), C.longlong(visibility))
	}
}

//export callbackQWindow_VisibleChanged
func callbackQWindow_VisibleChanged(ptr unsafe.Pointer, arg C.char) {
	if signal := qt.GetSignal(ptr, "visibleChanged"); signal != nil {
		(*(*func(bool))(signal))(int8(arg) != 0)
	}

}

func (ptr *QWindow) ConnectVisibleChanged(f func(arg bool)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "visibleChanged") {
			C.QWindow_ConnectVisibleChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "visibleChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "visibleChanged"); signal != nil {
			f := func(arg bool) {
				(*(*func(bool))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "visibleChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "visibleChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectVisibleChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectVisibleChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "visibleChanged")
	}
}

func (ptr *QWindow) VisibleChanged(arg bool) {
	if ptr.Pointer() != nil {
		C.QWindow_VisibleChanged(ptr.Pointer(), C.char(int8(qt.GoBoolToInt(arg))))
	}
}

//export callbackQWindow_WheelEvent
func callbackQWindow_WheelEvent(ptr unsafe.Pointer, ev unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "wheelEvent"); signal != nil {
		(*(*func(*QWheelEvent))(signal))(NewQWheelEventFromPointer(ev))
	} else {
		NewQWindowFromPointer(ptr).WheelEventDefault(NewQWheelEventFromPointer(ev))
	}
}

func (ptr *QWindow) ConnectWheelEvent(f func(ev *QWheelEvent)) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "wheelEvent"); signal != nil {
			f := func(ev *QWheelEvent) {
				(*(*func(*QWheelEvent))(signal))(ev)
				f(ev)
			}
			qt.ConnectSignal(ptr.Pointer(), "wheelEvent", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "wheelEvent", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectWheelEvent() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "wheelEvent")
	}
}

func (ptr *QWindow) WheelEvent(ev QWheelEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_WheelEvent(ptr.Pointer(), PointerFromQWheelEvent(ev))
	}
}

func (ptr *QWindow) WheelEventDefault(ev QWheelEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_WheelEventDefault(ptr.Pointer(), PointerFromQWheelEvent(ev))
	}
}

func (ptr *QWindow) Width() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWindow_Width(ptr.Pointer())))
	}
	return 0
}

//export callbackQWindow_WidthChanged
func callbackQWindow_WidthChanged(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "widthChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	}

}

func (ptr *QWindow) ConnectWidthChanged(f func(arg int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "widthChanged") {
			C.QWindow_ConnectWidthChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "widthChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "widthChanged"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "widthChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "widthChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectWidthChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectWidthChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "widthChanged")
	}
}

func (ptr *QWindow) WidthChanged(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_WidthChanged(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) WinId() uintptr {
	if ptr.Pointer() != nil {
		return uintptr(C.QWindow_WinId(ptr.Pointer()))
	}
	return 0
}

func (ptr *QWindow) WindowState() core.Qt__WindowState {
	if ptr.Pointer() != nil {
		return core.Qt__WindowState(C.QWindow_WindowState(ptr.Pointer()))
	}
	return 0
}

//export callbackQWindow_WindowStateChanged
func callbackQWindow_WindowStateChanged(ptr unsafe.Pointer, windowState C.longlong) {
	if signal := qt.GetSignal(ptr, "windowStateChanged"); signal != nil {
		(*(*func(core.Qt__WindowState))(signal))(core.Qt__WindowState(windowState))
	}

}

func (ptr *QWindow) ConnectWindowStateChanged(f func(windowState core.Qt__WindowState)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "windowStateChanged") {
			C.QWindow_ConnectWindowStateChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "windowStateChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "windowStateChanged"); signal != nil {
			f := func(windowState core.Qt__WindowState) {
				(*(*func(core.Qt__WindowState))(signal))(windowState)
				f(windowState)
			}
			qt.ConnectSignal(ptr.Pointer(), "windowStateChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "windowStateChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectWindowStateChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectWindowStateChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "windowStateChanged")
	}
}

func (ptr *QWindow) WindowStateChanged(windowState core.Qt__WindowState) {
	if ptr.Pointer() != nil {
		C.QWindow_WindowStateChanged(ptr.Pointer(), C.longlong(windowState))
	}
}

func (ptr *QWindow) WindowStates() core.Qt__WindowState {
	if ptr.Pointer() != nil {
		return core.Qt__WindowState(C.QWindow_WindowStates(ptr.Pointer()))
	}
	return 0
}

//export callbackQWindow_WindowTitleChanged
func callbackQWindow_WindowTitleChanged(ptr unsafe.Pointer, title C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "windowTitleChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(title))
	}

}

func (ptr *QWindow) ConnectWindowTitleChanged(f func(title string)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "windowTitleChanged") {
			C.QWindow_ConnectWindowTitleChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "windowTitleChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "windowTitleChanged"); signal != nil {
			f := func(title string) {
				(*(*func(string))(signal))(title)
				f(title)
			}
			qt.ConnectSignal(ptr.Pointer(), "windowTitleChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "windowTitleChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectWindowTitleChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectWindowTitleChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "windowTitleChanged")
	}
}

func (ptr *QWindow) WindowTitleChanged(title string) {
	if ptr.Pointer() != nil {
		var titleC *C.char
		if title != "" {
			titleC = C.CString(title)
			defer C.free(unsafe.Pointer(titleC))
		}
		C.QWindow_WindowTitleChanged(ptr.Pointer(), C.struct_QtGui_PackedString{data: titleC, len: C.longlong(len(title))})
	}
}

func (ptr *QWindow) X() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWindow_X(ptr.Pointer())))
	}
	return 0
}

//export callbackQWindow_XChanged
func callbackQWindow_XChanged(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "xChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	}

}

func (ptr *QWindow) ConnectXChanged(f func(arg int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "xChanged") {
			C.QWindow_ConnectXChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "xChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "xChanged"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "xChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "xChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectXChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectXChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "xChanged")
	}
}

func (ptr *QWindow) XChanged(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_XChanged(ptr.Pointer(), C.int(int32(arg)))
	}
}

func (ptr *QWindow) Y() int {
	if ptr.Pointer() != nil {
		return int(int32(C.QWindow_Y(ptr.Pointer())))
	}
	return 0
}

//export callbackQWindow_YChanged
func callbackQWindow_YChanged(ptr unsafe.Pointer, arg C.int) {
	if signal := qt.GetSignal(ptr, "yChanged"); signal != nil {
		(*(*func(int))(signal))(int(int32(arg)))
	}

}

func (ptr *QWindow) ConnectYChanged(f func(arg int)) {
	if ptr.Pointer() != nil {

		if !qt.ExistsSignal(ptr.Pointer(), "yChanged") {
			C.QWindow_ConnectYChanged(ptr.Pointer(), C.longlong(qt.ConnectionType(ptr.Pointer(), "yChanged")))
		}

		if signal := qt.LendSignal(ptr.Pointer(), "yChanged"); signal != nil {
			f := func(arg int) {
				(*(*func(int))(signal))(arg)
				f(arg)
			}
			qt.ConnectSignal(ptr.Pointer(), "yChanged", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "yChanged", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectYChanged() {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectYChanged(ptr.Pointer())
		qt.DisconnectSignal(ptr.Pointer(), "yChanged")
	}
}

func (ptr *QWindow) YChanged(arg int) {
	if ptr.Pointer() != nil {
		C.QWindow_YChanged(ptr.Pointer(), C.int(int32(arg)))
	}
}

//export callbackQWindow_DestroyQWindow
func callbackQWindow_DestroyQWindow(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "~QWindow"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).DestroyQWindowDefault()
	}
}

func (ptr *QWindow) ConnectDestroyQWindow(f func()) {
	if ptr.Pointer() != nil {

		if signal := qt.LendSignal(ptr.Pointer(), "~QWindow"); signal != nil {
			f := func() {
				(*(*func())(signal))()
				f()
			}
			qt.ConnectSignal(ptr.Pointer(), "~QWindow", unsafe.Pointer(&f))
		} else {
			qt.ConnectSignal(ptr.Pointer(), "~QWindow", unsafe.Pointer(&f))
		}
	}
}

func (ptr *QWindow) DisconnectDestroyQWindow() {
	if ptr.Pointer() != nil {

		qt.DisconnectSignal(ptr.Pointer(), "~QWindow")
	}
}

func (ptr *QWindow) DestroyQWindow() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QWindow_DestroyQWindow(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QWindow) DestroyQWindowDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QWindow_DestroyQWindowDefault(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}

func (ptr *QWindow) __children_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QWindow___children_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) __children_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow___children_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QWindow) __children_newList() unsafe.Pointer {
	return C.QWindow___children_newList(ptr.Pointer())
}

func (ptr *QWindow) __dynamicPropertyNames_atList(i int) *core.QByteArray {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQByteArrayFromPointer(C.QWindow___dynamicPropertyNames_atList(ptr.Pointer(), C.int(int32(i))))
		qt.SetFinalizer(tmpValue, (*core.QByteArray).DestroyQByteArray)
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) __dynamicPropertyNames_setList(i core.QByteArray_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow___dynamicPropertyNames_setList(ptr.Pointer(), core.PointerFromQByteArray(i))
	}
}

func (ptr *QWindow) __dynamicPropertyNames_newList() unsafe.Pointer {
	return C.QWindow___dynamicPropertyNames_newList(ptr.Pointer())
}

func (ptr *QWindow) __findChildren_atList(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QWindow___findChildren_atList(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) __findChildren_setList(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow___findChildren_setList(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QWindow) __findChildren_newList() unsafe.Pointer {
	return C.QWindow___findChildren_newList(ptr.Pointer())
}

func (ptr *QWindow) __findChildren_atList3(i int) *core.QObject {
	if ptr.Pointer() != nil {
		tmpValue := core.NewQObjectFromPointer(C.QWindow___findChildren_atList3(ptr.Pointer(), C.int(int32(i))))
		if !qt.ExistsSignal(tmpValue.Pointer(), "destroyed") {
			tmpValue.ConnectDestroyed(func(*core.QObject) { tmpValue.SetPointer(nil) })
		}
		return tmpValue
	}
	return nil
}

func (ptr *QWindow) __findChildren_setList3(i core.QObject_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow___findChildren_setList3(ptr.Pointer(), core.PointerFromQObject(i))
	}
}

func (ptr *QWindow) __findChildren_newList3() unsafe.Pointer {
	return C.QWindow___findChildren_newList3(ptr.Pointer())
}

//export callbackQWindow_ChildEvent
func callbackQWindow_ChildEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "childEvent"); signal != nil {
		(*(*func(*core.QChildEvent))(signal))(core.NewQChildEventFromPointer(event))
	} else {
		NewQWindowFromPointer(ptr).ChildEventDefault(core.NewQChildEventFromPointer(event))
	}
}

func (ptr *QWindow) ChildEvent(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ChildEvent(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

func (ptr *QWindow) ChildEventDefault(event core.QChildEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ChildEventDefault(ptr.Pointer(), core.PointerFromQChildEvent(event))
	}
}

//export callbackQWindow_ConnectNotify
func callbackQWindow_ConnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "connectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQWindowFromPointer(ptr).ConnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QWindow) ConnectNotify(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ConnectNotify(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

func (ptr *QWindow) ConnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_ConnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQWindow_CustomEvent
func callbackQWindow_CustomEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "customEvent"); signal != nil {
		(*(*func(*core.QEvent))(signal))(core.NewQEventFromPointer(event))
	} else {
		NewQWindowFromPointer(ptr).CustomEventDefault(core.NewQEventFromPointer(event))
	}
}

func (ptr *QWindow) CustomEvent(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_CustomEvent(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

func (ptr *QWindow) CustomEventDefault(event core.QEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_CustomEventDefault(ptr.Pointer(), core.PointerFromQEvent(event))
	}
}

//export callbackQWindow_DeleteLater
func callbackQWindow_DeleteLater(ptr unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "deleteLater"); signal != nil {
		(*(*func())(signal))()
	} else {
		NewQWindowFromPointer(ptr).DeleteLaterDefault()
	}
}

func (ptr *QWindow) DeleteLater() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QWindow_DeleteLater(ptr.Pointer())
	}
}

func (ptr *QWindow) DeleteLaterDefault() {
	if ptr.Pointer() != nil {

		qt.SetFinalizer(ptr, nil)
		C.QWindow_DeleteLaterDefault(ptr.Pointer())
	}
}

//export callbackQWindow_Destroyed
func callbackQWindow_Destroyed(ptr unsafe.Pointer, obj unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "destroyed"); signal != nil {
		(*(*func(*core.QObject))(signal))(core.NewQObjectFromPointer(obj))
	}

}

//export callbackQWindow_DisconnectNotify
func callbackQWindow_DisconnectNotify(ptr unsafe.Pointer, sign unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "disconnectNotify"); signal != nil {
		(*(*func(*core.QMetaMethod))(signal))(core.NewQMetaMethodFromPointer(sign))
	} else {
		NewQWindowFromPointer(ptr).DisconnectNotifyDefault(core.NewQMetaMethodFromPointer(sign))
	}
}

func (ptr *QWindow) DisconnectNotify(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectNotify(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

func (ptr *QWindow) DisconnectNotifyDefault(sign core.QMetaMethod_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_DisconnectNotifyDefault(ptr.Pointer(), core.PointerFromQMetaMethod(sign))
	}
}

//export callbackQWindow_EventFilter
func callbackQWindow_EventFilter(ptr unsafe.Pointer, watched unsafe.Pointer, event unsafe.Pointer) C.char {
	if signal := qt.GetSignal(ptr, "eventFilter"); signal != nil {
		return C.char(int8(qt.GoBoolToInt((*(*func(*core.QObject, *core.QEvent) bool)(signal))(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
	}

	return C.char(int8(qt.GoBoolToInt(NewQWindowFromPointer(ptr).EventFilterDefault(core.NewQObjectFromPointer(watched), core.NewQEventFromPointer(event)))))
}

func (ptr *QWindow) EventFilter(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_EventFilter(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

func (ptr *QWindow) EventFilterDefault(watched core.QObject_ITF, event core.QEvent_ITF) bool {
	if ptr.Pointer() != nil {
		return int8(C.QWindow_EventFilterDefault(ptr.Pointer(), core.PointerFromQObject(watched), core.PointerFromQEvent(event))) != 0
	}
	return false
}

//export callbackQWindow_MetaObject
func callbackQWindow_MetaObject(ptr unsafe.Pointer) unsafe.Pointer {
	if signal := qt.GetSignal(ptr, "metaObject"); signal != nil {
		return core.PointerFromQMetaObject((*(*func() *core.QMetaObject)(signal))())
	}

	return core.PointerFromQMetaObject(NewQWindowFromPointer(ptr).MetaObjectDefault())
}

func (ptr *QWindow) MetaObject() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QWindow_MetaObject(ptr.Pointer()))
	}
	return nil
}

func (ptr *QWindow) MetaObjectDefault() *core.QMetaObject {
	if ptr.Pointer() != nil {
		return core.NewQMetaObjectFromPointer(C.QWindow_MetaObjectDefault(ptr.Pointer()))
	}
	return nil
}

//export callbackQWindow_ObjectNameChanged
func callbackQWindow_ObjectNameChanged(ptr unsafe.Pointer, objectName C.struct_QtGui_PackedString) {
	if signal := qt.GetSignal(ptr, "objectNameChanged"); signal != nil {
		(*(*func(string))(signal))(cGoUnpackString(objectName))
	}

}

//export callbackQWindow_TimerEvent
func callbackQWindow_TimerEvent(ptr unsafe.Pointer, event unsafe.Pointer) {
	if signal := qt.GetSignal(ptr, "timerEvent"); signal != nil {
		(*(*func(*core.QTimerEvent))(signal))(core.NewQTimerEventFromPointer(event))
	} else {
		NewQWindowFromPointer(ptr).TimerEventDefault(core.NewQTimerEventFromPointer(event))
	}
}

func (ptr *QWindow) TimerEvent(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_TimerEvent(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

func (ptr *QWindow) TimerEventDefault(event core.QTimerEvent_ITF) {
	if ptr.Pointer() != nil {
		C.QWindow_TimerEventDefault(ptr.Pointer(), core.PointerFromQTimerEvent(event))
	}
}

type QWindowStateChangeEvent struct {
	core.QEvent
}

type QWindowStateChangeEvent_ITF interface {
	core.QEvent_ITF
	QWindowStateChangeEvent_PTR() *QWindowStateChangeEvent
}

func (ptr *QWindowStateChangeEvent) QWindowStateChangeEvent_PTR() *QWindowStateChangeEvent {
	return ptr
}

func (ptr *QWindowStateChangeEvent) Pointer() unsafe.Pointer {
	if ptr != nil {
		return ptr.QEvent_PTR().Pointer()
	}
	return nil
}

func (ptr *QWindowStateChangeEvent) SetPointer(p unsafe.Pointer) {
	if ptr != nil {
		ptr.QEvent_PTR().SetPointer(p)
	}
}

func PointerFromQWindowStateChangeEvent(ptr QWindowStateChangeEvent_ITF) unsafe.Pointer {
	if ptr != nil {
		return ptr.QWindowStateChangeEvent_PTR().Pointer()
	}
	return nil
}

func NewQWindowStateChangeEventFromPointer(ptr unsafe.Pointer) (n *QWindowStateChangeEvent) {
	n = new(QWindowStateChangeEvent)
	n.SetPointer(ptr)
	return
}
func (ptr *QWindowStateChangeEvent) DestroyQWindowStateChangeEvent() {
	if ptr != nil {
		qt.SetFinalizer(ptr, nil)

		qt.DisconnectAllSignals(ptr.Pointer(), "")
		C.free(ptr.Pointer())
		ptr.SetPointer(nil)
	}
}
func (ptr *QWindowStateChangeEvent) OldState() core.Qt__WindowState {
	if ptr.Pointer() != nil {
		return core.Qt__WindowState(C.QWindowStateChangeEvent_OldState(ptr.Pointer()))
	}
	return 0
}

func init() {
	qt.ItfMap["gui.QAbstractOpenGLFunctions_ITF"] = QAbstractOpenGLFunctions{}
	qt.ItfMap["gui.QAbstractTextDocumentLayout_ITF"] = QAbstractTextDocumentLayout{}
	qt.ItfMap["gui.QAbstractUndoItem_ITF"] = QAbstractUndoItem{}
	qt.ItfMap["gui.QAccessible_ITF"] = QAccessible{}
	qt.FuncMap["gui.QAccessible_IsActive"] = QAccessible_IsActive
	qt.FuncMap["gui.QAccessible_QueryAccessibleInterface"] = QAccessible_QueryAccessibleInterface
	qt.FuncMap["gui.QAccessible_SetRootObject"] = QAccessible_SetRootObject
	qt.FuncMap["gui.QAccessible_UpdateAccessibility2"] = QAccessible_UpdateAccessibility2
	qt.EnumMap["gui.QAccessible__SoundPlayed"] = int64(QAccessible__SoundPlayed)
	qt.EnumMap["gui.QAccessible__Alert"] = int64(QAccessible__Alert)
	qt.EnumMap["gui.QAccessible__ForegroundChanged"] = int64(QAccessible__ForegroundChanged)
	qt.EnumMap["gui.QAccessible__MenuStart"] = int64(QAccessible__MenuStart)
	qt.EnumMap["gui.QAccessible__MenuEnd"] = int64(QAccessible__MenuEnd)
	qt.EnumMap["gui.QAccessible__PopupMenuStart"] = int64(QAccessible__PopupMenuStart)
	qt.EnumMap["gui.QAccessible__PopupMenuEnd"] = int64(QAccessible__PopupMenuEnd)
	qt.EnumMap["gui.QAccessible__ContextHelpStart"] = int64(QAccessible__ContextHelpStart)
	qt.EnumMap["gui.QAccessible__ContextHelpEnd"] = int64(QAccessible__ContextHelpEnd)
	qt.EnumMap["gui.QAccessible__DragDropStart"] = int64(QAccessible__DragDropStart)
	qt.EnumMap["gui.QAccessible__DragDropEnd"] = int64(QAccessible__DragDropEnd)
	qt.EnumMap["gui.QAccessible__DialogStart"] = int64(QAccessible__DialogStart)
	qt.EnumMap["gui.QAccessible__DialogEnd"] = int64(QAccessible__DialogEnd)
	qt.EnumMap["gui.QAccessible__ScrollingStart"] = int64(QAccessible__ScrollingStart)
	qt.EnumMap["gui.QAccessible__ScrollingEnd"] = int64(QAccessible__ScrollingEnd)
	qt.EnumMap["gui.QAccessible__MenuCommand"] = int64(QAccessible__MenuCommand)
	qt.EnumMap["gui.QAccessible__ActionChanged"] = int64(QAccessible__ActionChanged)
	qt.EnumMap["gui.QAccessible__ActiveDescendantChanged"] = int64(QAccessible__ActiveDescendantChanged)
	qt.EnumMap["gui.QAccessible__AttributeChanged"] = int64(QAccessible__AttributeChanged)
	qt.EnumMap["gui.QAccessible__DocumentContentChanged"] = int64(QAccessible__DocumentContentChanged)
	qt.EnumMap["gui.QAccessible__DocumentLoadComplete"] = int64(QAccessible__DocumentLoadComplete)
	qt.EnumMap["gui.QAccessible__DocumentLoadStopped"] = int64(QAccessible__DocumentLoadStopped)
	qt.EnumMap["gui.QAccessible__DocumentReload"] = int64(QAccessible__DocumentReload)
	qt.EnumMap["gui.QAccessible__HyperlinkEndIndexChanged"] = int64(QAccessible__HyperlinkEndIndexChanged)
	qt.EnumMap["gui.QAccessible__HyperlinkNumberOfAnchorsChanged"] = int64(QAccessible__HyperlinkNumberOfAnchorsChanged)
	qt.EnumMap["gui.QAccessible__HyperlinkSelectedLinkChanged"] = int64(QAccessible__HyperlinkSelectedLinkChanged)
	qt.EnumMap["gui.QAccessible__HypertextLinkActivated"] = int64(QAccessible__HypertextLinkActivated)
	qt.EnumMap["gui.QAccessible__HypertextLinkSelected"] = int64(QAccessible__HypertextLinkSelected)
	qt.EnumMap["gui.QAccessible__HyperlinkStartIndexChanged"] = int64(QAccessible__HyperlinkStartIndexChanged)
	qt.EnumMap["gui.QAccessible__HypertextChanged"] = int64(QAccessible__HypertextChanged)
	qt.EnumMap["gui.QAccessible__HypertextNLinksChanged"] = int64(QAccessible__HypertextNLinksChanged)
	qt.EnumMap["gui.QAccessible__ObjectAttributeChanged"] = int64(QAccessible__ObjectAttributeChanged)
	qt.EnumMap["gui.QAccessible__PageChanged"] = int64(QAccessible__PageChanged)
	qt.EnumMap["gui.QAccessible__SectionChanged"] = int64(QAccessible__SectionChanged)
	qt.EnumMap["gui.QAccessible__TableCaptionChanged"] = int64(QAccessible__TableCaptionChanged)
	qt.EnumMap["gui.QAccessible__TableColumnDescriptionChanged"] = int64(QAccessible__TableColumnDescriptionChanged)
	qt.EnumMap["gui.QAccessible__TableColumnHeaderChanged"] = int64(QAccessible__TableColumnHeaderChanged)
	qt.EnumMap["gui.QAccessible__TableModelChanged"] = int64(QAccessible__TableModelChanged)
	qt.EnumMap["gui.QAccessible__TableRowDescriptionChanged"] = int64(QAccessible__TableRowDescriptionChanged)
	qt.EnumMap["gui.QAccessible__TableRowHeaderChanged"] = int64(QAccessible__TableRowHeaderChanged)
	qt.EnumMap["gui.QAccessible__TableSummaryChanged"] = int64(QAccessible__TableSummaryChanged)
	qt.EnumMap["gui.QAccessible__TextAttributeChanged"] = int64(QAccessible__TextAttributeChanged)
	qt.EnumMap["gui.QAccessible__TextCaretMoved"] = int64(QAccessible__TextCaretMoved)
	qt.EnumMap["gui.QAccessible__TextColumnChanged"] = int64(QAccessible__TextColumnChanged)
	qt.EnumMap["gui.QAccessible__TextInserted"] = int64(QAccessible__TextInserted)
	qt.EnumMap["gui.QAccessible__TextRemoved"] = int64(QAccessible__TextRemoved)
	qt.EnumMap["gui.QAccessible__TextUpdated"] = int64(QAccessible__TextUpdated)
	qt.EnumMap["gui.QAccessible__TextSelectionChanged"] = int64(QAccessible__TextSelectionChanged)
	qt.EnumMap["gui.QAccessible__VisibleDataChanged"] = int64(QAccessible__VisibleDataChanged)
	qt.EnumMap["gui.QAccessible__ObjectCreated"] = int64(QAccessible__ObjectCreated)
	qt.EnumMap["gui.QAccessible__ObjectDestroyed"] = int64(QAccessible__ObjectDestroyed)
	qt.EnumMap["gui.QAccessible__ObjectShow"] = int64(QAccessible__ObjectShow)
	qt.EnumMap["gui.QAccessible__ObjectHide"] = int64(QAccessible__ObjectHide)
	qt.EnumMap["gui.QAccessible__ObjectReorder"] = int64(QAccessible__ObjectReorder)
	qt.EnumMap["gui.QAccessible__Focus"] = int64(QAccessible__Focus)
	qt.EnumMap["gui.QAccessible__Selection"] = int64(QAccessible__Selection)
	qt.EnumMap["gui.QAccessible__SelectionAdd"] = int64(QAccessible__SelectionAdd)
	qt.EnumMap["gui.QAccessible__SelectionRemove"] = int64(QAccessible__SelectionRemove)
	qt.EnumMap["gui.QAccessible__SelectionWithin"] = int64(QAccessible__SelectionWithin)
	qt.EnumMap["gui.QAccessible__StateChanged"] = int64(QAccessible__StateChanged)
	qt.EnumMap["gui.QAccessible__LocationChanged"] = int64(QAccessible__LocationChanged)
	qt.EnumMap["gui.QAccessible__NameChanged"] = int64(QAccessible__NameChanged)
	qt.EnumMap["gui.QAccessible__DescriptionChanged"] = int64(QAccessible__DescriptionChanged)
	qt.EnumMap["gui.QAccessible__ValueChanged"] = int64(QAccessible__ValueChanged)
	qt.EnumMap["gui.QAccessible__ParentChanged"] = int64(QAccessible__ParentChanged)
	qt.EnumMap["gui.QAccessible__HelpChanged"] = int64(QAccessible__HelpChanged)
	qt.EnumMap["gui.QAccessible__DefaultActionChanged"] = int64(QAccessible__DefaultActionChanged)
	qt.EnumMap["gui.QAccessible__AcceleratorChanged"] = int64(QAccessible__AcceleratorChanged)
	qt.EnumMap["gui.QAccessible__InvalidEvent"] = int64(QAccessible__InvalidEvent)
	qt.EnumMap["gui.QAccessible__NoRole"] = int64(QAccessible__NoRole)
	qt.EnumMap["gui.QAccessible__TitleBar"] = int64(QAccessible__TitleBar)
	qt.EnumMap["gui.QAccessible__MenuBar"] = int64(QAccessible__MenuBar)
	qt.EnumMap["gui.QAccessible__ScrollBar"] = int64(QAccessible__ScrollBar)
	qt.EnumMap["gui.QAccessible__Grip"] = int64(QAccessible__Grip)
	qt.EnumMap["gui.QAccessible__Sound"] = int64(QAccessible__Sound)
	qt.EnumMap["gui.QAccessible__Cursor"] = int64(QAccessible__Cursor)
	qt.EnumMap["gui.QAccessible__Caret"] = int64(QAccessible__Caret)
	qt.EnumMap["gui.QAccessible__AlertMessage"] = int64(QAccessible__AlertMessage)
	qt.EnumMap["gui.QAccessible__Window"] = int64(QAccessible__Window)
	qt.EnumMap["gui.QAccessible__Client"] = int64(QAccessible__Client)
	qt.EnumMap["gui.QAccessible__PopupMenu"] = int64(QAccessible__PopupMenu)
	qt.EnumMap["gui.QAccessible__MenuItem"] = int64(QAccessible__MenuItem)
	qt.EnumMap["gui.QAccessible__ToolTip"] = int64(QAccessible__ToolTip)
	qt.EnumMap["gui.QAccessible__Application"] = int64(QAccessible__Application)
	qt.EnumMap["gui.QAccessible__Document"] = int64(QAccessible__Document)
	qt.EnumMap["gui.QAccessible__Pane"] = int64(QAccessible__Pane)
	qt.EnumMap["gui.QAccessible__Chart"] = int64(QAccessible__Chart)
	qt.EnumMap["gui.QAccessible__Dialog"] = int64(QAccessible__Dialog)
	qt.EnumMap["gui.QAccessible__Border"] = int64(QAccessible__Border)
	qt.EnumMap["gui.QAccessible__Grouping"] = int64(QAccessible__Grouping)
	qt.EnumMap["gui.QAccessible__Separator"] = int64(QAccessible__Separator)
	qt.EnumMap["gui.QAccessible__ToolBar"] = int64(QAccessible__ToolBar)
	qt.EnumMap["gui.QAccessible__StatusBar"] = int64(QAccessible__StatusBar)
	qt.EnumMap["gui.QAccessible__Table"] = int64(QAccessible__Table)
	qt.EnumMap["gui.QAccessible__ColumnHeader"] = int64(QAccessible__ColumnHeader)
	qt.EnumMap["gui.QAccessible__RowHeader"] = int64(QAccessible__RowHeader)
	qt.EnumMap["gui.QAccessible__Column"] = int64(QAccessible__Column)
	qt.EnumMap["gui.QAccessible__Row"] = int64(QAccessible__Row)
	qt.EnumMap["gui.QAccessible__Cell"] = int64(QAccessible__Cell)
	qt.EnumMap["gui.QAccessible__Link"] = int64(QAccessible__Link)
	qt.EnumMap["gui.QAccessible__HelpBalloon"] = int64(QAccessible__HelpBalloon)
	qt.EnumMap["gui.QAccessible__Assistant"] = int64(QAccessible__Assistant)
	qt.EnumMap["gui.QAccessible__List"] = int64(QAccessible__List)
	qt.EnumMap["gui.QAccessible__ListItem"] = int64(QAccessible__ListItem)
	qt.EnumMap["gui.QAccessible__Tree"] = int64(QAccessible__Tree)
	qt.EnumMap["gui.QAccessible__TreeItem"] = int64(QAccessible__TreeItem)
	qt.EnumMap["gui.QAccessible__PageTab"] = int64(QAccessible__PageTab)
	qt.EnumMap["gui.QAccessible__PropertyPage"] = int64(QAccessible__PropertyPage)
	qt.EnumMap["gui.QAccessible__Indicator"] = int64(QAccessible__Indicator)
	qt.EnumMap["gui.QAccessible__Graphic"] = int64(QAccessible__Graphic)
	qt.EnumMap["gui.QAccessible__StaticText"] = int64(QAccessible__StaticText)
	qt.EnumMap["gui.QAccessible__EditableText"] = int64(QAccessible__EditableText)
	qt.EnumMap["gui.QAccessible__Button"] = int64(QAccessible__Button)
	qt.EnumMap["gui.QAccessible__CheckBox"] = int64(QAccessible__CheckBox)
	qt.EnumMap["gui.QAccessible__RadioButton"] = int64(QAccessible__RadioButton)
	qt.EnumMap["gui.QAccessible__ComboBox"] = int64(QAccessible__ComboBox)
	qt.EnumMap["gui.QAccessible__ProgressBar"] = int64(QAccessible__ProgressBar)
	qt.EnumMap["gui.QAccessible__Dial"] = int64(QAccessible__Dial)
	qt.EnumMap["gui.QAccessible__HotkeyField"] = int64(QAccessible__HotkeyField)
	qt.EnumMap["gui.QAccessible__Slider"] = int64(QAccessible__Slider)
	qt.EnumMap["gui.QAccessible__SpinBox"] = int64(QAccessible__SpinBox)
	qt.EnumMap["gui.QAccessible__Canvas"] = int64(QAccessible__Canvas)
	qt.EnumMap["gui.QAccessible__Animation"] = int64(QAccessible__Animation)
	qt.EnumMap["gui.QAccessible__Equation"] = int64(QAccessible__Equation)
	qt.EnumMap["gui.QAccessible__ButtonDropDown"] = int64(QAccessible__ButtonDropDown)
	qt.EnumMap["gui.QAccessible__ButtonMenu"] = int64(QAccessible__ButtonMenu)
	qt.EnumMap["gui.QAccessible__ButtonDropGrid"] = int64(QAccessible__ButtonDropGrid)
	qt.EnumMap["gui.QAccessible__Whitespace"] = int64(QAccessible__Whitespace)
	qt.EnumMap["gui.QAccessible__PageTabList"] = int64(QAccessible__PageTabList)
	qt.EnumMap["gui.QAccessible__Clock"] = int64(QAccessible__Clock)
	qt.EnumMap["gui.QAccessible__Splitter"] = int64(QAccessible__Splitter)
	qt.EnumMap["gui.QAccessible__LayeredPane"] = int64(QAccessible__LayeredPane)
	qt.EnumMap["gui.QAccessible__Terminal"] = int64(QAccessible__Terminal)
	qt.EnumMap["gui.QAccessible__Desktop"] = int64(QAccessible__Desktop)
	qt.EnumMap["gui.QAccessible__Paragraph"] = int64(QAccessible__Paragraph)
	qt.EnumMap["gui.QAccessible__WebDocument"] = int64(QAccessible__WebDocument)
	qt.EnumMap["gui.QAccessible__Section"] = int64(QAccessible__Section)
	qt.EnumMap["gui.QAccessible__ColorChooser"] = int64(QAccessible__ColorChooser)
	qt.EnumMap["gui.QAccessible__Footer"] = int64(QAccessible__Footer)
	qt.EnumMap["gui.QAccessible__Form"] = int64(QAccessible__Form)
	qt.EnumMap["gui.QAccessible__Heading"] = int64(QAccessible__Heading)
	qt.EnumMap["gui.QAccessible__Note"] = int64(QAccessible__Note)
	qt.EnumMap["gui.QAccessible__ComplementaryContent"] = int64(QAccessible__ComplementaryContent)
	qt.EnumMap["gui.QAccessible__UserRole"] = int64(QAccessible__UserRole)
	qt.EnumMap["gui.QAccessible__Name"] = int64(QAccessible__Name)
	qt.EnumMap["gui.QAccessible__Description"] = int64(QAccessible__Description)
	qt.EnumMap["gui.QAccessible__Value"] = int64(QAccessible__Value)
	qt.EnumMap["gui.QAccessible__Help"] = int64(QAccessible__Help)
	qt.EnumMap["gui.QAccessible__Accelerator"] = int64(QAccessible__Accelerator)
	qt.EnumMap["gui.QAccessible__DebugDescription"] = int64(QAccessible__DebugDescription)
	qt.EnumMap["gui.QAccessible__UserText"] = int64(QAccessible__UserText)
	qt.EnumMap["gui.QAccessible__Label"] = int64(QAccessible__Label)
	qt.EnumMap["gui.QAccessible__Labelled"] = int64(QAccessible__Labelled)
	qt.EnumMap["gui.QAccessible__Controller"] = int64(QAccessible__Controller)
	qt.EnumMap["gui.QAccessible__Controlled"] = int64(QAccessible__Controlled)
	qt.EnumMap["gui.QAccessible__AllRelations"] = int64(QAccessible__AllRelations)
	qt.EnumMap["gui.QAccessible__TextInterface"] = int64(QAccessible__TextInterface)
	qt.EnumMap["gui.QAccessible__EditableTextInterface"] = int64(QAccessible__EditableTextInterface)
	qt.EnumMap["gui.QAccessible__ValueInterface"] = int64(QAccessible__ValueInterface)
	qt.EnumMap["gui.QAccessible__ActionInterface"] = int64(QAccessible__ActionInterface)
	qt.EnumMap["gui.QAccessible__ImageInterface"] = int64(QAccessible__ImageInterface)
	qt.EnumMap["gui.QAccessible__TableInterface"] = int64(QAccessible__TableInterface)
	qt.EnumMap["gui.QAccessible__TableCellInterface"] = int64(QAccessible__TableCellInterface)
	qt.EnumMap["gui.QAccessible__CharBoundary"] = int64(QAccessible__CharBoundary)
	qt.EnumMap["gui.QAccessible__WordBoundary"] = int64(QAccessible__WordBoundary)
	qt.EnumMap["gui.QAccessible__SentenceBoundary"] = int64(QAccessible__SentenceBoundary)
	qt.EnumMap["gui.QAccessible__ParagraphBoundary"] = int64(QAccessible__ParagraphBoundary)
	qt.EnumMap["gui.QAccessible__LineBoundary"] = int64(QAccessible__LineBoundary)
	qt.EnumMap["gui.QAccessible__NoBoundary"] = int64(QAccessible__NoBoundary)
	qt.ItfMap["gui.QAccessibleActionInterface_ITF"] = QAccessibleActionInterface{}
	qt.FuncMap["gui.QAccessibleActionInterface_DecreaseAction"] = QAccessibleActionInterface_DecreaseAction
	qt.FuncMap["gui.QAccessibleActionInterface_IncreaseAction"] = QAccessibleActionInterface_IncreaseAction
	qt.FuncMap["gui.QAccessibleActionInterface_NextPageAction"] = QAccessibleActionInterface_NextPageAction
	qt.FuncMap["gui.QAccessibleActionInterface_PressAction"] = QAccessibleActionInterface_PressAction
	qt.FuncMap["gui.QAccessibleActionInterface_PreviousPageAction"] = QAccessibleActionInterface_PreviousPageAction
	qt.FuncMap["gui.QAccessibleActionInterface_ScrollDownAction"] = QAccessibleActionInterface_ScrollDownAction
	qt.FuncMap["gui.QAccessibleActionInterface_ScrollLeftAction"] = QAccessibleActionInterface_ScrollLeftAction
	qt.FuncMap["gui.QAccessibleActionInterface_ScrollRightAction"] = QAccessibleActionInterface_ScrollRightAction
	qt.FuncMap["gui.QAccessibleActionInterface_ScrollUpAction"] = QAccessibleActionInterface_ScrollUpAction
	qt.FuncMap["gui.QAccessibleActionInterface_SetFocusAction"] = QAccessibleActionInterface_SetFocusAction
	qt.FuncMap["gui.QAccessibleActionInterface_ShowMenuAction"] = QAccessibleActionInterface_ShowMenuAction
	qt.FuncMap["gui.QAccessibleActionInterface_ToggleAction"] = QAccessibleActionInterface_ToggleAction
	qt.ItfMap["gui.QAccessibleEditableTextInterface_ITF"] = QAccessibleEditableTextInterface{}
	qt.ItfMap["gui.QAccessibleEvent_ITF"] = QAccessibleEvent{}
	qt.FuncMap["gui.NewQAccessibleEvent2"] = NewQAccessibleEvent2
	qt.FuncMap["gui.NewQAccessibleEvent3"] = NewQAccessibleEvent3
	qt.ItfMap["gui.QAccessibleInterface_ITF"] = QAccessibleInterface{}
	qt.ItfMap["gui.QAccessibleObject_ITF"] = QAccessibleObject{}
	qt.ItfMap["gui.QAccessiblePlugin_ITF"] = QAccessiblePlugin{}
	qt.FuncMap["gui.NewQAccessiblePlugin"] = NewQAccessiblePlugin
	qt.ItfMap["gui.QAccessibleStateChangeEvent_ITF"] = QAccessibleStateChangeEvent{}
	qt.ItfMap["gui.QAccessibleTableCellInterface_ITF"] = QAccessibleTableCellInterface{}
	qt.ItfMap["gui.QAccessibleTableInterface_ITF"] = QAccessibleTableInterface{}
	qt.ItfMap["gui.QAccessibleTableModelChangeEvent_ITF"] = QAccessibleTableModelChangeEvent{}
	qt.FuncMap["gui.NewQAccessibleTableModelChangeEvent"] = NewQAccessibleTableModelChangeEvent
	qt.FuncMap["gui.NewQAccessibleTableModelChangeEvent2"] = NewQAccessibleTableModelChangeEvent2
	qt.EnumMap["gui.QAccessibleTableModelChangeEvent__ModelReset"] = int64(QAccessibleTableModelChangeEvent__ModelReset)
	qt.EnumMap["gui.QAccessibleTableModelChangeEvent__DataChanged"] = int64(QAccessibleTableModelChangeEvent__DataChanged)
	qt.EnumMap["gui.QAccessibleTableModelChangeEvent__RowsInserted"] = int64(QAccessibleTableModelChangeEvent__RowsInserted)
	qt.EnumMap["gui.QAccessibleTableModelChangeEvent__ColumnsInserted"] = int64(QAccessibleTableModelChangeEvent__ColumnsInserted)
	qt.EnumMap["gui.QAccessibleTableModelChangeEvent__RowsRemoved"] = int64(QAccessibleTableModelChangeEvent__RowsRemoved)
	qt.EnumMap["gui.QAccessibleTableModelChangeEvent__ColumnsRemoved"] = int64(QAccessibleTableModelChangeEvent__ColumnsRemoved)
	qt.ItfMap["gui.QAccessibleTextCursorEvent_ITF"] = QAccessibleTextCursorEvent{}
	qt.FuncMap["gui.NewQAccessibleTextCursorEvent"] = NewQAccessibleTextCursorEvent
	qt.FuncMap["gui.NewQAccessibleTextCursorEvent2"] = NewQAccessibleTextCursorEvent2
	qt.ItfMap["gui.QAccessibleTextInsertEvent_ITF"] = QAccessibleTextInsertEvent{}
	qt.FuncMap["gui.NewQAccessibleTextInsertEvent"] = NewQAccessibleTextInsertEvent
	qt.FuncMap["gui.NewQAccessibleTextInsertEvent2"] = NewQAccessibleTextInsertEvent2
	qt.ItfMap["gui.QAccessibleTextInterface_ITF"] = QAccessibleTextInterface{}
	qt.ItfMap["gui.QAccessibleTextRemoveEvent_ITF"] = QAccessibleTextRemoveEvent{}
	qt.FuncMap["gui.NewQAccessibleTextRemoveEvent"] = NewQAccessibleTextRemoveEvent
	qt.FuncMap["gui.NewQAccessibleTextRemoveEvent2"] = NewQAccessibleTextRemoveEvent2
	qt.ItfMap["gui.QAccessibleTextSelectionEvent_ITF"] = QAccessibleTextSelectionEvent{}
	qt.FuncMap["gui.NewQAccessibleTextSelectionEvent"] = NewQAccessibleTextSelectionEvent
	qt.FuncMap["gui.NewQAccessibleTextSelectionEvent2"] = NewQAccessibleTextSelectionEvent2
	qt.ItfMap["gui.QAccessibleTextUpdateEvent_ITF"] = QAccessibleTextUpdateEvent{}
	qt.FuncMap["gui.NewQAccessibleTextUpdateEvent"] = NewQAccessibleTextUpdateEvent
	qt.FuncMap["gui.NewQAccessibleTextUpdateEvent2"] = NewQAccessibleTextUpdateEvent2
	qt.ItfMap["gui.QAccessibleValueChangeEvent_ITF"] = QAccessibleValueChangeEvent{}
	qt.FuncMap["gui.NewQAccessibleValueChangeEvent"] = NewQAccessibleValueChangeEvent
	qt.FuncMap["gui.NewQAccessibleValueChangeEvent2"] = NewQAccessibleValueChangeEvent2
	qt.ItfMap["gui.QAccessibleValueInterface_ITF"] = QAccessibleValueInterface{}
	qt.ItfMap["gui.QActionEvent_ITF"] = QActionEvent{}
	qt.FuncMap["gui.NewQActionEvent"] = NewQActionEvent
	qt.ItfMap["gui.QApplicationStateChangeEvent_ITF"] = QApplicationStateChangeEvent{}
	qt.FuncMap["gui.NewQApplicationStateChangeEvent"] = NewQApplicationStateChangeEvent
	qt.ItfMap["gui.QBackingStore_ITF"] = QBackingStore{}
	qt.FuncMap["gui.NewQBackingStore"] = NewQBackingStore
	qt.ItfMap["gui.QBitmap_ITF"] = QBitmap{}
	qt.FuncMap["gui.NewQBitmap"] = NewQBitmap
	qt.FuncMap["gui.NewQBitmap2"] = NewQBitmap2
	qt.FuncMap["gui.NewQBitmap3"] = NewQBitmap3
	qt.FuncMap["gui.NewQBitmap4"] = NewQBitmap4
	qt.FuncMap["gui.NewQBitmap5"] = NewQBitmap5
	qt.FuncMap["gui.QBitmap_FromData"] = QBitmap_FromData
	qt.FuncMap["gui.QBitmap_FromImage"] = QBitmap_FromImage
	qt.FuncMap["gui.QBitmap_FromImage2"] = QBitmap_FromImage2
	qt.ItfMap["gui.QBrush_ITF"] = QBrush{}
	qt.FuncMap["gui.NewQBrush"] = NewQBrush
	qt.FuncMap["gui.NewQBrush2"] = NewQBrush2
	qt.FuncMap["gui.NewQBrush3"] = NewQBrush3
	qt.FuncMap["gui.NewQBrush4"] = NewQBrush4
	qt.FuncMap["gui.NewQBrush5"] = NewQBrush5
	qt.FuncMap["gui.NewQBrush6"] = NewQBrush6
	qt.FuncMap["gui.NewQBrush7"] = NewQBrush7
	qt.FuncMap["gui.NewQBrush8"] = NewQBrush8
	qt.FuncMap["gui.NewQBrush9"] = NewQBrush9
	qt.FuncMap["gui.NewQBrush10"] = NewQBrush10
	qt.ItfMap["gui.QClipboard_ITF"] = QClipboard{}
	qt.EnumMap["gui.QClipboard__Clipboard"] = int64(QClipboard__Clipboard)
	qt.EnumMap["gui.QClipboard__Selection"] = int64(QClipboard__Selection)
	qt.EnumMap["gui.QClipboard__FindBuffer"] = int64(QClipboard__FindBuffer)
	qt.EnumMap["gui.QClipboard__LastMode"] = int64(QClipboard__LastMode)
	qt.ItfMap["gui.QCloseEvent_ITF"] = QCloseEvent{}
	qt.FuncMap["gui.NewQCloseEvent"] = NewQCloseEvent
	qt.ItfMap["gui.QColor_ITF"] = QColor{}
	qt.FuncMap["gui.NewQColor"] = NewQColor
	qt.FuncMap["gui.NewQColor2"] = NewQColor2
	qt.FuncMap["gui.NewQColor3"] = NewQColor3
	qt.FuncMap["gui.NewQColor4"] = NewQColor4
	qt.FuncMap["gui.NewQColor5"] = NewQColor5
	qt.FuncMap["gui.NewQColor6"] = NewQColor6
	qt.FuncMap["gui.NewQColor8"] = NewQColor8
	qt.FuncMap["gui.NewQColor9"] = NewQColor9
	qt.FuncMap["gui.QColor_ColorNames"] = QColor_ColorNames
	qt.FuncMap["gui.QColor_FromCmyk"] = QColor_FromCmyk
	qt.FuncMap["gui.QColor_FromCmykF"] = QColor_FromCmykF
	qt.FuncMap["gui.QColor_FromHsl"] = QColor_FromHsl
	qt.FuncMap["gui.QColor_FromHslF"] = QColor_FromHslF
	qt.FuncMap["gui.QColor_FromHsv"] = QColor_FromHsv
	qt.FuncMap["gui.QColor_FromHsvF"] = QColor_FromHsvF
	qt.FuncMap["gui.QColor_FromRgb"] = QColor_FromRgb
	qt.FuncMap["gui.QColor_FromRgb2"] = QColor_FromRgb2
	qt.FuncMap["gui.QColor_FromRgbF"] = QColor_FromRgbF
	qt.FuncMap["gui.QColor_FromRgba"] = QColor_FromRgba
	qt.FuncMap["gui.QColor_FromRgba64"] = QColor_FromRgba64
	qt.FuncMap["gui.QColor_FromRgba642"] = QColor_FromRgba642
	qt.FuncMap["gui.QColor_IsValidColor"] = QColor_IsValidColor
	qt.FuncMap["gui.QColor_IsValidColor2"] = QColor_IsValidColor2
	qt.FuncMap["gui.QColor_IsValidColor3"] = QColor_IsValidColor3
	qt.EnumMap["gui.QColor__Invalid"] = int64(QColor__Invalid)
	qt.EnumMap["gui.QColor__Rgb"] = int64(QColor__Rgb)
	qt.EnumMap["gui.QColor__Hsv"] = int64(QColor__Hsv)
	qt.EnumMap["gui.QColor__Cmyk"] = int64(QColor__Cmyk)
	qt.EnumMap["gui.QColor__Hsl"] = int64(QColor__Hsl)
	qt.EnumMap["gui.QColor__HexRgb"] = int64(QColor__HexRgb)
	qt.EnumMap["gui.QColor__HexArgb"] = int64(QColor__HexArgb)
	qt.ItfMap["gui.QColorDialogOptions_ITF"] = QColorDialogOptions{}
	qt.ItfMap["gui.QConicalGradient_ITF"] = QConicalGradient{}
	qt.FuncMap["gui.NewQConicalGradient"] = NewQConicalGradient
	qt.FuncMap["gui.NewQConicalGradient2"] = NewQConicalGradient2
	qt.FuncMap["gui.NewQConicalGradient3"] = NewQConicalGradient3
	qt.ItfMap["gui.QContextMenuEvent_ITF"] = QContextMenuEvent{}
	qt.FuncMap["gui.NewQContextMenuEvent"] = NewQContextMenuEvent
	qt.FuncMap["gui.NewQContextMenuEvent2"] = NewQContextMenuEvent2
	qt.FuncMap["gui.NewQContextMenuEvent3"] = NewQContextMenuEvent3
	qt.EnumMap["gui.QContextMenuEvent__Mouse"] = int64(QContextMenuEvent__Mouse)
	qt.EnumMap["gui.QContextMenuEvent__Keyboard"] = int64(QContextMenuEvent__Keyboard)
	qt.EnumMap["gui.QContextMenuEvent__Other"] = int64(QContextMenuEvent__Other)
	qt.ItfMap["gui.QCursor_ITF"] = QCursor{}
	qt.FuncMap["gui.NewQCursor"] = NewQCursor
	qt.FuncMap["gui.NewQCursor2"] = NewQCursor2
	qt.FuncMap["gui.NewQCursor3"] = NewQCursor3
	qt.FuncMap["gui.NewQCursor4"] = NewQCursor4
	qt.FuncMap["gui.NewQCursor5"] = NewQCursor5
	qt.FuncMap["gui.NewQCursor6"] = NewQCursor6
	qt.FuncMap["gui.QCursor_Pos"] = QCursor_Pos
	qt.FuncMap["gui.QCursor_Pos2"] = QCursor_Pos2
	qt.FuncMap["gui.QCursor_SetPos"] = QCursor_SetPos
	qt.FuncMap["gui.QCursor_SetPos2"] = QCursor_SetPos2
	qt.FuncMap["gui.QCursor_SetPos3"] = QCursor_SetPos3
	qt.FuncMap["gui.QCursor_SetPos4"] = QCursor_SetPos4
	qt.ItfMap["gui.QDesktopServices_ITF"] = QDesktopServices{}
	qt.FuncMap["gui.QDesktopServices_OpenUrl"] = QDesktopServices_OpenUrl
	qt.FuncMap["gui.QDesktopServices_SetUrlHandler"] = QDesktopServices_SetUrlHandler
	qt.FuncMap["gui.QDesktopServices_UnsetUrlHandler"] = QDesktopServices_UnsetUrlHandler
	qt.ItfMap["gui.QDoubleValidator_ITF"] = QDoubleValidator{}
	qt.FuncMap["gui.NewQDoubleValidator"] = NewQDoubleValidator
	qt.FuncMap["gui.NewQDoubleValidator2"] = NewQDoubleValidator2
	qt.EnumMap["gui.QDoubleValidator__StandardNotation"] = int64(QDoubleValidator__StandardNotation)
	qt.EnumMap["gui.QDoubleValidator__ScientificNotation"] = int64(QDoubleValidator__ScientificNotation)
	qt.ItfMap["gui.QDrag_ITF"] = QDrag{}
	qt.FuncMap["gui.NewQDrag"] = NewQDrag
	qt.FuncMap["gui.QDrag_Cancel"] = QDrag_Cancel
	qt.ItfMap["gui.QDragEnterEvent_ITF"] = QDragEnterEvent{}
	qt.FuncMap["gui.NewQDragEnterEvent"] = NewQDragEnterEvent
	qt.ItfMap["gui.QDragLeaveEvent_ITF"] = QDragLeaveEvent{}
	qt.FuncMap["gui.NewQDragLeaveEvent"] = NewQDragLeaveEvent
	qt.ItfMap["gui.QDragMoveEvent_ITF"] = QDragMoveEvent{}
	qt.FuncMap["gui.NewQDragMoveEvent"] = NewQDragMoveEvent
	qt.ItfMap["gui.QDropEvent_ITF"] = QDropEvent{}
	qt.FuncMap["gui.NewQDropEvent"] = NewQDropEvent
	qt.ItfMap["gui.QEnterEvent_ITF"] = QEnterEvent{}
	qt.FuncMap["gui.NewQEnterEvent"] = NewQEnterEvent
	qt.ItfMap["gui.QExposeEvent_ITF"] = QExposeEvent{}
	qt.FuncMap["gui.NewQExposeEvent"] = NewQExposeEvent
	qt.ItfMap["gui.QFileDialogOptions_ITF"] = QFileDialogOptions{}
	qt.ItfMap["gui.QFileOpenEvent_ITF"] = QFileOpenEvent{}
	qt.ItfMap["gui.QFocusEvent_ITF"] = QFocusEvent{}
	qt.FuncMap["gui.NewQFocusEvent"] = NewQFocusEvent
	qt.ItfMap["gui.QFont_ITF"] = QFont{}
	qt.FuncMap["gui.NewQFont"] = NewQFont
	qt.FuncMap["gui.NewQFont2"] = NewQFont2
	qt.FuncMap["gui.NewQFont4"] = NewQFont4
	qt.FuncMap["gui.NewQFont5"] = NewQFont5
	qt.FuncMap["gui.QFont_InsertSubstitution"] = QFont_InsertSubstitution
	qt.FuncMap["gui.QFont_InsertSubstitutions"] = QFont_InsertSubstitutions
	qt.FuncMap["gui.QFont_RemoveSubstitutions"] = QFont_RemoveSubstitutions
	qt.FuncMap["gui.QFont_Substitute"] = QFont_Substitute
	qt.FuncMap["gui.QFont_Substitutes"] = QFont_Substitutes
	qt.FuncMap["gui.QFont_Substitutions"] = QFont_Substitutions
	qt.EnumMap["gui.QFont__Helvetica"] = int64(QFont__Helvetica)
	qt.EnumMap["gui.QFont__SansSerif"] = int64(QFont__SansSerif)
	qt.EnumMap["gui.QFont__Times"] = int64(QFont__Times)
	qt.EnumMap["gui.QFont__Serif"] = int64(QFont__Serif)
	qt.EnumMap["gui.QFont__Courier"] = int64(QFont__Courier)
	qt.EnumMap["gui.QFont__TypeWriter"] = int64(QFont__TypeWriter)
	qt.EnumMap["gui.QFont__OldEnglish"] = int64(QFont__OldEnglish)
	qt.EnumMap["gui.QFont__Decorative"] = int64(QFont__Decorative)
	qt.EnumMap["gui.QFont__System"] = int64(QFont__System)
	qt.EnumMap["gui.QFont__AnyStyle"] = int64(QFont__AnyStyle)
	qt.EnumMap["gui.QFont__Cursive"] = int64(QFont__Cursive)
	qt.EnumMap["gui.QFont__Monospace"] = int64(QFont__Monospace)
	qt.EnumMap["gui.QFont__Fantasy"] = int64(QFont__Fantasy)
	qt.EnumMap["gui.QFont__PreferDefault"] = int64(QFont__PreferDefault)
	qt.EnumMap["gui.QFont__PreferBitmap"] = int64(QFont__PreferBitmap)
	qt.EnumMap["gui.QFont__PreferDevice"] = int64(QFont__PreferDevice)
	qt.EnumMap["gui.QFont__PreferOutline"] = int64(QFont__PreferOutline)
	qt.EnumMap["gui.QFont__ForceOutline"] = int64(QFont__ForceOutline)
	qt.EnumMap["gui.QFont__PreferMatch"] = int64(QFont__PreferMatch)
	qt.EnumMap["gui.QFont__PreferQuality"] = int64(QFont__PreferQuality)
	qt.EnumMap["gui.QFont__PreferAntialias"] = int64(QFont__PreferAntialias)
	qt.EnumMap["gui.QFont__NoAntialias"] = int64(QFont__NoAntialias)
	qt.EnumMap["gui.QFont__OpenGLCompatible"] = int64(QFont__OpenGLCompatible)
	qt.EnumMap["gui.QFont__ForceIntegerMetrics"] = int64(QFont__ForceIntegerMetrics)
	qt.EnumMap["gui.QFont__NoSubpixelAntialias"] = int64(QFont__NoSubpixelAntialias)
	qt.EnumMap["gui.QFont__PreferNoShaping"] = int64(QFont__PreferNoShaping)
	qt.EnumMap["gui.QFont__NoFontMerging"] = int64(QFont__NoFontMerging)
	qt.EnumMap["gui.QFont__PreferDefaultHinting"] = int64(QFont__PreferDefaultHinting)
	qt.EnumMap["gui.QFont__PreferNoHinting"] = int64(QFont__PreferNoHinting)
	qt.EnumMap["gui.QFont__PreferVerticalHinting"] = int64(QFont__PreferVerticalHinting)
	qt.EnumMap["gui.QFont__PreferFullHinting"] = int64(QFont__PreferFullHinting)
	qt.EnumMap["gui.QFont__Thin"] = int64(QFont__Thin)
	qt.EnumMap["gui.QFont__ExtraLight"] = int64(QFont__ExtraLight)
	qt.EnumMap["gui.QFont__Light"] = int64(QFont__Light)
	qt.EnumMap["gui.QFont__Normal"] = int64(QFont__Normal)
	qt.EnumMap["gui.QFont__Medium"] = int64(QFont__Medium)
	qt.EnumMap["gui.QFont__DemiBold"] = int64(QFont__DemiBold)
	qt.EnumMap["gui.QFont__Bold"] = int64(QFont__Bold)
	qt.EnumMap["gui.QFont__ExtraBold"] = int64(QFont__ExtraBold)
	qt.EnumMap["gui.QFont__Black"] = int64(QFont__Black)
	qt.EnumMap["gui.QFont__StyleNormal"] = int64(QFont__StyleNormal)
	qt.EnumMap["gui.QFont__StyleItalic"] = int64(QFont__StyleItalic)
	qt.EnumMap["gui.QFont__StyleOblique"] = int64(QFont__StyleOblique)
	qt.EnumMap["gui.QFont__AnyStretch"] = int64(QFont__AnyStretch)
	qt.EnumMap["gui.QFont__UltraCondensed"] = int64(QFont__UltraCondensed)
	qt.EnumMap["gui.QFont__ExtraCondensed"] = int64(QFont__ExtraCondensed)
	qt.EnumMap["gui.QFont__Condensed"] = int64(QFont__Condensed)
	qt.EnumMap["gui.QFont__SemiCondensed"] = int64(QFont__SemiCondensed)
	qt.EnumMap["gui.QFont__Unstretched"] = int64(QFont__Unstretched)
	qt.EnumMap["gui.QFont__SemiExpanded"] = int64(QFont__SemiExpanded)
	qt.EnumMap["gui.QFont__Expanded"] = int64(QFont__Expanded)
	qt.EnumMap["gui.QFont__ExtraExpanded"] = int64(QFont__ExtraExpanded)
	qt.EnumMap["gui.QFont__UltraExpanded"] = int64(QFont__UltraExpanded)
	qt.EnumMap["gui.QFont__MixedCase"] = int64(QFont__MixedCase)
	qt.EnumMap["gui.QFont__AllUppercase"] = int64(QFont__AllUppercase)
	qt.EnumMap["gui.QFont__AllLowercase"] = int64(QFont__AllLowercase)
	qt.EnumMap["gui.QFont__SmallCaps"] = int64(QFont__SmallCaps)
	qt.EnumMap["gui.QFont__Capitalize"] = int64(QFont__Capitalize)
	qt.EnumMap["gui.QFont__PercentageSpacing"] = int64(QFont__PercentageSpacing)
	qt.EnumMap["gui.QFont__AbsoluteSpacing"] = int64(QFont__AbsoluteSpacing)
	qt.ItfMap["gui.QFontDatabase_ITF"] = QFontDatabase{}
	qt.FuncMap["gui.NewQFontDatabase"] = NewQFontDatabase
	qt.FuncMap["gui.QFontDatabase_AddApplicationFont"] = QFontDatabase_AddApplicationFont
	qt.FuncMap["gui.QFontDatabase_AddApplicationFontFromData"] = QFontDatabase_AddApplicationFontFromData
	qt.FuncMap["gui.QFontDatabase_ApplicationFontFamilies"] = QFontDatabase_ApplicationFontFamilies
	qt.FuncMap["gui.QFontDatabase_RemoveAllApplicationFonts"] = QFontDatabase_RemoveAllApplicationFonts
	qt.FuncMap["gui.QFontDatabase_RemoveApplicationFont"] = QFontDatabase_RemoveApplicationFont
	qt.FuncMap["gui.QFontDatabase_StandardSizes"] = QFontDatabase_StandardSizes
	qt.FuncMap["gui.QFontDatabase_SystemFont"] = QFontDatabase_SystemFont
	qt.EnumMap["gui.QFontDatabase__Any"] = int64(QFontDatabase__Any)
	qt.EnumMap["gui.QFontDatabase__Latin"] = int64(QFontDatabase__Latin)
	qt.EnumMap["gui.QFontDatabase__Greek"] = int64(QFontDatabase__Greek)
	qt.EnumMap["gui.QFontDatabase__Cyrillic"] = int64(QFontDatabase__Cyrillic)
	qt.EnumMap["gui.QFontDatabase__Armenian"] = int64(QFontDatabase__Armenian)
	qt.EnumMap["gui.QFontDatabase__Hebrew"] = int64(QFontDatabase__Hebrew)
	qt.EnumMap["gui.QFontDatabase__Arabic"] = int64(QFontDatabase__Arabic)
	qt.EnumMap["gui.QFontDatabase__Syriac"] = int64(QFontDatabase__Syriac)
	qt.EnumMap["gui.QFontDatabase__Thaana"] = int64(QFontDatabase__Thaana)
	qt.EnumMap["gui.QFontDatabase__Devanagari"] = int64(QFontDatabase__Devanagari)
	qt.EnumMap["gui.QFontDatabase__Bengali"] = int64(QFontDatabase__Bengali)
	qt.EnumMap["gui.QFontDatabase__Gurmukhi"] = int64(QFontDatabase__Gurmukhi)
	qt.EnumMap["gui.QFontDatabase__Gujarati"] = int64(QFontDatabase__Gujarati)
	qt.EnumMap["gui.QFontDatabase__Oriya"] = int64(QFontDatabase__Oriya)
	qt.EnumMap["gui.QFontDatabase__Tamil"] = int64(QFontDatabase__Tamil)
	qt.EnumMap["gui.QFontDatabase__Telugu"] = int64(QFontDatabase__Telugu)
	qt.EnumMap["gui.QFontDatabase__Kannada"] = int64(QFontDatabase__Kannada)
	qt.EnumMap["gui.QFontDatabase__Malayalam"] = int64(QFontDatabase__Malayalam)
	qt.EnumMap["gui.QFontDatabase__Sinhala"] = int64(QFontDatabase__Sinhala)
	qt.EnumMap["gui.QFontDatabase__Thai"] = int64(QFontDatabase__Thai)
	qt.EnumMap["gui.QFontDatabase__Lao"] = int64(QFontDatabase__Lao)
	qt.EnumMap["gui.QFontDatabase__Tibetan"] = int64(QFontDatabase__Tibetan)
	qt.EnumMap["gui.QFontDatabase__Myanmar"] = int64(QFontDatabase__Myanmar)
	qt.EnumMap["gui.QFontDatabase__Georgian"] = int64(QFontDatabase__Georgian)
	qt.EnumMap["gui.QFontDatabase__Khmer"] = int64(QFontDatabase__Khmer)
	qt.EnumMap["gui.QFontDatabase__SimplifiedChinese"] = int64(QFontDatabase__SimplifiedChinese)
	qt.EnumMap["gui.QFontDatabase__TraditionalChinese"] = int64(QFontDatabase__TraditionalChinese)
	qt.EnumMap["gui.QFontDatabase__Japanese"] = int64(QFontDatabase__Japanese)
	qt.EnumMap["gui.QFontDatabase__Korean"] = int64(QFontDatabase__Korean)
	qt.EnumMap["gui.QFontDatabase__Vietnamese"] = int64(QFontDatabase__Vietnamese)
	qt.EnumMap["gui.QFontDatabase__Symbol"] = int64(QFontDatabase__Symbol)
	qt.EnumMap["gui.QFontDatabase__Other"] = int64(QFontDatabase__Other)
	qt.EnumMap["gui.QFontDatabase__Ogham"] = int64(QFontDatabase__Ogham)
	qt.EnumMap["gui.QFontDatabase__Runic"] = int64(QFontDatabase__Runic)
	qt.EnumMap["gui.QFontDatabase__Nko"] = int64(QFontDatabase__Nko)
	qt.EnumMap["gui.QFontDatabase__WritingSystemsCount"] = int64(QFontDatabase__WritingSystemsCount)
	qt.EnumMap["gui.QFontDatabase__GeneralFont"] = int64(QFontDatabase__GeneralFont)
	qt.EnumMap["gui.QFontDatabase__FixedFont"] = int64(QFontDatabase__FixedFont)
	qt.EnumMap["gui.QFontDatabase__TitleFont"] = int64(QFontDatabase__TitleFont)
	qt.EnumMap["gui.QFontDatabase__SmallestReadableFont"] = int64(QFontDatabase__SmallestReadableFont)
	qt.ItfMap["gui.QFontDialogOptions_ITF"] = QFontDialogOptions{}
	qt.ItfMap["gui.QFontInfo_ITF"] = QFontInfo{}
	qt.FuncMap["gui.NewQFontInfo"] = NewQFontInfo
	qt.FuncMap["gui.NewQFontInfo2"] = NewQFontInfo2
	qt.ItfMap["gui.QFontMetrics_ITF"] = QFontMetrics{}
	qt.FuncMap["gui.NewQFontMetrics"] = NewQFontMetrics
	qt.FuncMap["gui.NewQFontMetrics3"] = NewQFontMetrics3
	qt.FuncMap["gui.NewQFontMetrics4"] = NewQFontMetrics4
	qt.ItfMap["gui.QFontMetricsF_ITF"] = QFontMetricsF{}
	qt.FuncMap["gui.NewQFontMetricsF"] = NewQFontMetricsF
	qt.FuncMap["gui.NewQFontMetricsF3"] = NewQFontMetricsF3
	qt.FuncMap["gui.NewQFontMetricsF4"] = NewQFontMetricsF4
	qt.FuncMap["gui.NewQFontMetricsF5"] = NewQFontMetricsF5
	qt.ItfMap["gui.QGenericMatrix_ITF"] = QGenericMatrix{}
	qt.ItfMap["gui.QGenericPlugin_ITF"] = QGenericPlugin{}
	qt.FuncMap["gui.NewQGenericPlugin"] = NewQGenericPlugin
	qt.ItfMap["gui.QGenericPluginFactory_ITF"] = QGenericPluginFactory{}
	qt.FuncMap["gui.QGenericPluginFactory_Create"] = QGenericPluginFactory_Create
	qt.FuncMap["gui.QGenericPluginFactory_Keys"] = QGenericPluginFactory_Keys
	qt.ItfMap["gui.QGlyphRun_ITF"] = QGlyphRun{}
	qt.FuncMap["gui.NewQGlyphRun"] = NewQGlyphRun
	qt.FuncMap["gui.NewQGlyphRun2"] = NewQGlyphRun2
	qt.EnumMap["gui.QGlyphRun__Overline"] = int64(QGlyphRun__Overline)
	qt.EnumMap["gui.QGlyphRun__Underline"] = int64(QGlyphRun__Underline)
	qt.EnumMap["gui.QGlyphRun__StrikeOut"] = int64(QGlyphRun__StrikeOut)
	qt.EnumMap["gui.QGlyphRun__RightToLeft"] = int64(QGlyphRun__RightToLeft)
	qt.EnumMap["gui.QGlyphRun__SplitLigature"] = int64(QGlyphRun__SplitLigature)
	qt.ItfMap["gui.QGradient_ITF"] = QGradient{}
	qt.FuncMap["gui.NewQGradient2"] = NewQGradient2
	qt.EnumMap["gui.QGradient__LinearGradient"] = int64(QGradient__LinearGradient)
	qt.EnumMap["gui.QGradient__RadialGradient"] = int64(QGradient__RadialGradient)
	qt.EnumMap["gui.QGradient__ConicalGradient"] = int64(QGradient__ConicalGradient)
	qt.EnumMap["gui.QGradient__NoGradient"] = int64(QGradient__NoGradient)
	qt.EnumMap["gui.QGradient__PadSpread"] = int64(QGradient__PadSpread)
	qt.EnumMap["gui.QGradient__ReflectSpread"] = int64(QGradient__ReflectSpread)
	qt.EnumMap["gui.QGradient__RepeatSpread"] = int64(QGradient__RepeatSpread)
	qt.EnumMap["gui.QGradient__LogicalMode"] = int64(QGradient__LogicalMode)
	qt.EnumMap["gui.QGradient__StretchToDeviceMode"] = int64(QGradient__StretchToDeviceMode)
	qt.EnumMap["gui.QGradient__ObjectBoundingMode"] = int64(QGradient__ObjectBoundingMode)
	qt.EnumMap["gui.QGradient__ObjectMode"] = int64(QGradient__ObjectMode)
	qt.EnumMap["gui.QGradient__WarmFlame"] = int64(QGradient__WarmFlame)
	qt.EnumMap["gui.QGradient__NightFade"] = int64(QGradient__NightFade)
	qt.EnumMap["gui.QGradient__SpringWarmth"] = int64(QGradient__SpringWarmth)
	qt.EnumMap["gui.QGradient__JuicyPeach"] = int64(QGradient__JuicyPeach)
	qt.EnumMap["gui.QGradient__YoungPassion"] = int64(QGradient__YoungPassion)
	qt.EnumMap["gui.QGradient__LadyLips"] = int64(QGradient__LadyLips)
	qt.EnumMap["gui.QGradient__SunnyMorning"] = int64(QGradient__SunnyMorning)
	qt.EnumMap["gui.QGradient__RainyAshville"] = int64(QGradient__RainyAshville)
	qt.EnumMap["gui.QGradient__FrozenDreams"] = int64(QGradient__FrozenDreams)
	qt.EnumMap["gui.QGradient__WinterNeva"] = int64(QGradient__WinterNeva)
	qt.EnumMap["gui.QGradient__DustyGrass"] = int64(QGradient__DustyGrass)
	qt.EnumMap["gui.QGradient__TemptingAzure"] = int64(QGradient__TemptingAzure)
	qt.EnumMap["gui.QGradient__HeavyRain"] = int64(QGradient__HeavyRain)
	qt.EnumMap["gui.QGradient__AmyCrisp"] = int64(QGradient__AmyCrisp)
	qt.EnumMap["gui.QGradient__MeanFruit"] = int64(QGradient__MeanFruit)
	qt.EnumMap["gui.QGradient__DeepBlue"] = int64(QGradient__DeepBlue)
	qt.EnumMap["gui.QGradient__RipeMalinka"] = int64(QGradient__RipeMalinka)
	qt.EnumMap["gui.QGradient__CloudyKnoxville"] = int64(QGradient__CloudyKnoxville)
	qt.EnumMap["gui.QGradient__MalibuBeach"] = int64(QGradient__MalibuBeach)
	qt.EnumMap["gui.QGradient__NewLife"] = int64(QGradient__NewLife)
	qt.EnumMap["gui.QGradient__TrueSunset"] = int64(QGradient__TrueSunset)
	qt.EnumMap["gui.QGradient__MorpheusDen"] = int64(QGradient__MorpheusDen)
	qt.EnumMap["gui.QGradient__RareWind"] = int64(QGradient__RareWind)
	qt.EnumMap["gui.QGradient__NearMoon"] = int64(QGradient__NearMoon)
	qt.EnumMap["gui.QGradient__WildApple"] = int64(QGradient__WildApple)
	qt.EnumMap["gui.QGradient__SaintPetersburg"] = int64(QGradient__SaintPetersburg)
	qt.EnumMap["gui.QGradient__PlumPlate"] = int64(QGradient__PlumPlate)
	qt.EnumMap["gui.QGradient__EverlastingSky"] = int64(QGradient__EverlastingSky)
	qt.EnumMap["gui.QGradient__HappyFisher"] = int64(QGradient__HappyFisher)
	qt.EnumMap["gui.QGradient__Blessing"] = int64(QGradient__Blessing)
	qt.EnumMap["gui.QGradient__SharpeyeEagle"] = int64(QGradient__SharpeyeEagle)
	qt.EnumMap["gui.QGradient__LadogaBottom"] = int64(QGradient__LadogaBottom)
	qt.EnumMap["gui.QGradient__LemonGate"] = int64(QGradient__LemonGate)
	qt.EnumMap["gui.QGradient__ItmeoBranding"] = int64(QGradient__ItmeoBranding)
	qt.EnumMap["gui.QGradient__ZeusMiracle"] = int64(QGradient__ZeusMiracle)
	qt.EnumMap["gui.QGradient__OldHat"] = int64(QGradient__OldHat)
	qt.EnumMap["gui.QGradient__StarWine"] = int64(QGradient__StarWine)
	qt.EnumMap["gui.QGradient__HappyAcid"] = int64(QGradient__HappyAcid)
	qt.EnumMap["gui.QGradient__AwesomePine"] = int64(QGradient__AwesomePine)
	qt.EnumMap["gui.QGradient__NewYork"] = int64(QGradient__NewYork)
	qt.EnumMap["gui.QGradient__ShyRainbow"] = int64(QGradient__ShyRainbow)
	qt.EnumMap["gui.QGradient__MixedHopes"] = int64(QGradient__MixedHopes)
	qt.EnumMap["gui.QGradient__FlyHigh"] = int64(QGradient__FlyHigh)
	qt.EnumMap["gui.QGradient__StrongBliss"] = int64(QGradient__StrongBliss)
	qt.EnumMap["gui.QGradient__FreshMilk"] = int64(QGradient__FreshMilk)
	qt.EnumMap["gui.QGradient__SnowAgain"] = int64(QGradient__SnowAgain)
	qt.EnumMap["gui.QGradient__FebruaryInk"] = int64(QGradient__FebruaryInk)
	qt.EnumMap["gui.QGradient__KindSteel"] = int64(QGradient__KindSteel)
	qt.EnumMap["gui.QGradient__SoftGrass"] = int64(QGradient__SoftGrass)
	qt.EnumMap["gui.QGradient__GrownEarly"] = int64(QGradient__GrownEarly)
	qt.EnumMap["gui.QGradient__SharpBlues"] = int64(QGradient__SharpBlues)
	qt.EnumMap["gui.QGradient__ShadyWater"] = int64(QGradient__ShadyWater)
	qt.EnumMap["gui.QGradient__DirtyBeauty"] = int64(QGradient__DirtyBeauty)
	qt.EnumMap["gui.QGradient__GreatWhale"] = int64(QGradient__GreatWhale)
	qt.EnumMap["gui.QGradient__TeenNotebook"] = int64(QGradient__TeenNotebook)
	qt.EnumMap["gui.QGradient__PoliteRumors"] = int64(QGradient__PoliteRumors)
	qt.EnumMap["gui.QGradient__SweetPeriod"] = int64(QGradient__SweetPeriod)
	qt.EnumMap["gui.QGradient__WideMatrix"] = int64(QGradient__WideMatrix)
	qt.EnumMap["gui.QGradient__SoftCherish"] = int64(QGradient__SoftCherish)
	qt.EnumMap["gui.QGradient__RedSalvation"] = int64(QGradient__RedSalvation)
	qt.EnumMap["gui.QGradient__BurningSpring"] = int64(QGradient__BurningSpring)
	qt.EnumMap["gui.QGradient__NightParty"] = int64(QGradient__NightParty)
	qt.EnumMap["gui.QGradient__SkyGlider"] = int64(QGradient__SkyGlider)
	qt.EnumMap["gui.QGradient__HeavenPeach"] = int64(QGradient__HeavenPeach)
	qt.EnumMap["gui.QGradient__PurpleDivision"] = int64(QGradient__PurpleDivision)
	qt.EnumMap["gui.QGradient__AquaSplash"] = int64(QGradient__AquaSplash)
	qt.EnumMap["gui.QGradient__SpikyNaga"] = int64(QGradient__SpikyNaga)
	qt.EnumMap["gui.QGradient__LoveKiss"] = int64(QGradient__LoveKiss)
	qt.EnumMap["gui.QGradient__CleanMirror"] = int64(QGradient__CleanMirror)
	qt.EnumMap["gui.QGradient__PremiumDark"] = int64(QGradient__PremiumDark)
	qt.EnumMap["gui.QGradient__ColdEvening"] = int64(QGradient__ColdEvening)
	qt.EnumMap["gui.QGradient__CochitiLake"] = int64(QGradient__CochitiLake)
	qt.EnumMap["gui.QGradient__SummerGames"] = int64(QGradient__SummerGames)
	qt.EnumMap["gui.QGradient__PassionateBed"] = int64(QGradient__PassionateBed)
	qt.EnumMap["gui.QGradient__MountainRock"] = int64(QGradient__MountainRock)
	qt.EnumMap["gui.QGradient__DesertHump"] = int64(QGradient__DesertHump)
	qt.EnumMap["gui.QGradient__JungleDay"] = int64(QGradient__JungleDay)
	qt.EnumMap["gui.QGradient__PhoenixStart"] = int64(QGradient__PhoenixStart)
	qt.EnumMap["gui.QGradient__OctoberSilence"] = int64(QGradient__OctoberSilence)
	qt.EnumMap["gui.QGradient__FarawayRiver"] = int64(QGradient__FarawayRiver)
	qt.EnumMap["gui.QGradient__AlchemistLab"] = int64(QGradient__AlchemistLab)
	qt.EnumMap["gui.QGradient__OverSun"] = int64(QGradient__OverSun)
	qt.EnumMap["gui.QGradient__PremiumWhite"] = int64(QGradient__PremiumWhite)
	qt.EnumMap["gui.QGradient__MarsParty"] = int64(QGradient__MarsParty)
	qt.EnumMap["gui.QGradient__EternalConstance"] = int64(QGradient__EternalConstance)
	qt.EnumMap["gui.QGradient__JapanBlush"] = int64(QGradient__JapanBlush)
	qt.EnumMap["gui.QGradient__SmilingRain"] = int64(QGradient__SmilingRain)
	qt.EnumMap["gui.QGradient__CloudyApple"] = int64(QGradient__CloudyApple)
	qt.EnumMap["gui.QGradient__BigMango"] = int64(QGradient__BigMango)
	qt.EnumMap["gui.QGradient__HealthyWater"] = int64(QGradient__HealthyWater)
	qt.EnumMap["gui.QGradient__AmourAmour"] = int64(QGradient__AmourAmour)
	qt.EnumMap["gui.QGradient__RiskyConcrete"] = int64(QGradient__RiskyConcrete)
	qt.EnumMap["gui.QGradient__StrongStick"] = int64(QGradient__StrongStick)
	qt.EnumMap["gui.QGradient__ViciousStance"] = int64(QGradient__ViciousStance)
	qt.EnumMap["gui.QGradient__PaloAlto"] = int64(QGradient__PaloAlto)
	qt.EnumMap["gui.QGradient__HappyMemories"] = int64(QGradient__HappyMemories)
	qt.EnumMap["gui.QGradient__MidnightBloom"] = int64(QGradient__MidnightBloom)
	qt.EnumMap["gui.QGradient__Crystalline"] = int64(QGradient__Crystalline)
	qt.EnumMap["gui.QGradient__PartyBliss"] = int64(QGradient__PartyBliss)
	qt.EnumMap["gui.QGradient__ConfidentCloud"] = int64(QGradient__ConfidentCloud)
	qt.EnumMap["gui.QGradient__LeCocktail"] = int64(QGradient__LeCocktail)
	qt.EnumMap["gui.QGradient__RiverCity"] = int64(QGradient__RiverCity)
	qt.EnumMap["gui.QGradient__FrozenBerry"] = int64(QGradient__FrozenBerry)
	qt.EnumMap["gui.QGradient__ChildCare"] = int64(QGradient__ChildCare)
	qt.EnumMap["gui.QGradient__FlyingLemon"] = int64(QGradient__FlyingLemon)
	qt.EnumMap["gui.QGradient__NewRetrowave"] = int64(QGradient__NewRetrowave)
	qt.EnumMap["gui.QGradient__HiddenJaguar"] = int64(QGradient__HiddenJaguar)
	qt.EnumMap["gui.QGradient__AboveTheSky"] = int64(QGradient__AboveTheSky)
	qt.EnumMap["gui.QGradient__Nega"] = int64(QGradient__Nega)
	qt.EnumMap["gui.QGradient__DenseWater"] = int64(QGradient__DenseWater)
	qt.EnumMap["gui.QGradient__Seashore"] = int64(QGradient__Seashore)
	qt.EnumMap["gui.QGradient__MarbleWall"] = int64(QGradient__MarbleWall)
	qt.EnumMap["gui.QGradient__CheerfulCaramel"] = int64(QGradient__CheerfulCaramel)
	qt.EnumMap["gui.QGradient__NightSky"] = int64(QGradient__NightSky)
	qt.EnumMap["gui.QGradient__MagicLake"] = int64(QGradient__MagicLake)
	qt.EnumMap["gui.QGradient__YoungGrass"] = int64(QGradient__YoungGrass)
	qt.EnumMap["gui.QGradient__ColorfulPeach"] = int64(QGradient__ColorfulPeach)
	qt.EnumMap["gui.QGradient__GentleCare"] = int64(QGradient__GentleCare)
	qt.EnumMap["gui.QGradient__PlumBath"] = int64(QGradient__PlumBath)
	qt.EnumMap["gui.QGradient__HappyUnicorn"] = int64(QGradient__HappyUnicorn)
	qt.EnumMap["gui.QGradient__AfricanField"] = int64(QGradient__AfricanField)
	qt.EnumMap["gui.QGradient__SolidStone"] = int64(QGradient__SolidStone)
	qt.EnumMap["gui.QGradient__OrangeJuice"] = int64(QGradient__OrangeJuice)
	qt.EnumMap["gui.QGradient__GlassWater"] = int64(QGradient__GlassWater)
	qt.EnumMap["gui.QGradient__NorthMiracle"] = int64(QGradient__NorthMiracle)
	qt.EnumMap["gui.QGradient__FruitBlend"] = int64(QGradient__FruitBlend)
	qt.EnumMap["gui.QGradient__MillenniumPine"] = int64(QGradient__MillenniumPine)
	qt.EnumMap["gui.QGradient__HighFlight"] = int64(QGradient__HighFlight)
	qt.EnumMap["gui.QGradient__MoleHall"] = int64(QGradient__MoleHall)
	qt.EnumMap["gui.QGradient__SpaceShift"] = int64(QGradient__SpaceShift)
	qt.EnumMap["gui.QGradient__ForestInei"] = int64(QGradient__ForestInei)
	qt.EnumMap["gui.QGradient__RoyalGarden"] = int64(QGradient__RoyalGarden)
	qt.EnumMap["gui.QGradient__RichMetal"] = int64(QGradient__RichMetal)
	qt.EnumMap["gui.QGradient__JuicyCake"] = int64(QGradient__JuicyCake)
	qt.EnumMap["gui.QGradient__SmartIndigo"] = int64(QGradient__SmartIndigo)
	qt.EnumMap["gui.QGradient__SandStrike"] = int64(QGradient__SandStrike)
	qt.EnumMap["gui.QGradient__NorseBeauty"] = int64(QGradient__NorseBeauty)
	qt.EnumMap["gui.QGradient__AquaGuidance"] = int64(QGradient__AquaGuidance)
	qt.EnumMap["gui.QGradient__SunVeggie"] = int64(QGradient__SunVeggie)
	qt.EnumMap["gui.QGradient__SeaLord"] = int64(QGradient__SeaLord)
	qt.EnumMap["gui.QGradient__BlackSea"] = int64(QGradient__BlackSea)
	qt.EnumMap["gui.QGradient__GrassShampoo"] = int64(QGradient__GrassShampoo)
	qt.EnumMap["gui.QGradient__LandingAircraft"] = int64(QGradient__LandingAircraft)
	qt.EnumMap["gui.QGradient__WitchDance"] = int64(QGradient__WitchDance)
	qt.EnumMap["gui.QGradient__SleeplessNight"] = int64(QGradient__SleeplessNight)
	qt.EnumMap["gui.QGradient__AngelCare"] = int64(QGradient__AngelCare)
	qt.EnumMap["gui.QGradient__CrystalRiver"] = int64(QGradient__CrystalRiver)
	qt.EnumMap["gui.QGradient__SoftLipstick"] = int64(QGradient__SoftLipstick)
	qt.EnumMap["gui.QGradient__SaltMountain"] = int64(QGradient__SaltMountain)
	qt.EnumMap["gui.QGradient__PerfectWhite"] = int64(QGradient__PerfectWhite)
	qt.EnumMap["gui.QGradient__FreshOasis"] = int64(QGradient__FreshOasis)
	qt.EnumMap["gui.QGradient__StrictNovember"] = int64(QGradient__StrictNovember)
	qt.EnumMap["gui.QGradient__MorningSalad"] = int64(QGradient__MorningSalad)
	qt.EnumMap["gui.QGradient__DeepRelief"] = int64(QGradient__DeepRelief)
	qt.EnumMap["gui.QGradient__SeaStrike"] = int64(QGradient__SeaStrike)
	qt.EnumMap["gui.QGradient__NightCall"] = int64(QGradient__NightCall)
	qt.EnumMap["gui.QGradient__SupremeSky"] = int64(QGradient__SupremeSky)
	qt.EnumMap["gui.QGradient__LightBlue"] = int64(QGradient__LightBlue)
	qt.EnumMap["gui.QGradient__MindCrawl"] = int64(QGradient__MindCrawl)
	qt.EnumMap["gui.QGradient__LilyMeadow"] = int64(QGradient__LilyMeadow)
	qt.EnumMap["gui.QGradient__SugarLollipop"] = int64(QGradient__SugarLollipop)
	qt.EnumMap["gui.QGradient__SweetDessert"] = int64(QGradient__SweetDessert)
	qt.EnumMap["gui.QGradient__MagicRay"] = int64(QGradient__MagicRay)
	qt.EnumMap["gui.QGradient__TeenParty"] = int64(QGradient__TeenParty)
	qt.EnumMap["gui.QGradient__FrozenHeat"] = int64(QGradient__FrozenHeat)
	qt.EnumMap["gui.QGradient__GagarinView"] = int64(QGradient__GagarinView)
	qt.EnumMap["gui.QGradient__FabledSunset"] = int64(QGradient__FabledSunset)
	qt.EnumMap["gui.QGradient__PerfectBlue"] = int64(QGradient__PerfectBlue)
	qt.ItfMap["gui.QGuiApplication_ITF"] = QGuiApplication{}
	qt.FuncMap["gui.NewQGuiApplication"] = NewQGuiApplication
	qt.FuncMap["gui.QGuiApplication_ApplicationDisplayName"] = QGuiApplication_ApplicationDisplayName
	qt.FuncMap["gui.QGuiApplication_ApplicationState"] = QGuiApplication_ApplicationState
	qt.FuncMap["gui.QGuiApplication_ChangeOverrideCursor"] = QGuiApplication_ChangeOverrideCursor
	qt.FuncMap["gui.QGuiApplication_Clipboard"] = QGuiApplication_Clipboard
	qt.FuncMap["gui.QGuiApplication_DesktopFileName"] = QGuiApplication_DesktopFileName
	qt.FuncMap["gui.QGuiApplication_DesktopSettingsAware"] = QGuiApplication_DesktopSettingsAware
	qt.FuncMap["gui.QGuiApplication_Exec"] = QGuiApplication_Exec
	qt.FuncMap["gui.QGuiApplication_FocusObject"] = QGuiApplication_FocusObject
	qt.FuncMap["gui.QGuiApplication_FocusWindow"] = QGuiApplication_FocusWindow
	qt.FuncMap["gui.QGuiApplication_Font"] = QGuiApplication_Font
	qt.FuncMap["gui.QGuiApplication_InputMethod"] = QGuiApplication_InputMethod
	qt.FuncMap["gui.QGuiApplication_IsFallbackSessionManagementEnabled"] = QGuiApplication_IsFallbackSessionManagementEnabled
	qt.FuncMap["gui.QGuiApplication_IsLeftToRight"] = QGuiApplication_IsLeftToRight
	qt.FuncMap["gui.QGuiApplication_IsRightToLeft"] = QGuiApplication_IsRightToLeft
	qt.FuncMap["gui.QGuiApplication_KeyboardModifiers"] = QGuiApplication_KeyboardModifiers
	qt.FuncMap["gui.QGuiApplication_LayoutDirection"] = QGuiApplication_LayoutDirection
	qt.FuncMap["gui.QGuiApplication_ModalWindow"] = QGuiApplication_ModalWindow
	qt.FuncMap["gui.QGuiApplication_MouseButtons"] = QGuiApplication_MouseButtons
	qt.FuncMap["gui.QGuiApplication_OverrideCursor"] = QGuiApplication_OverrideCursor
	qt.FuncMap["gui.QGuiApplication_Palette"] = QGuiApplication_Palette
	qt.FuncMap["gui.QGuiApplication_PlatformFunction"] = QGuiApplication_PlatformFunction
	qt.FuncMap["gui.QGuiApplication_PlatformName"] = QGuiApplication_PlatformName
	qt.FuncMap["gui.QGuiApplication_PrimaryScreen"] = QGuiApplication_PrimaryScreen
	qt.FuncMap["gui.QGuiApplication_QueryKeyboardModifiers"] = QGuiApplication_QueryKeyboardModifiers
	qt.FuncMap["gui.QGuiApplication_QuitOnLastWindowClosed"] = QGuiApplication_QuitOnLastWindowClosed
	qt.FuncMap["gui.QGuiApplication_RestoreOverrideCursor"] = QGuiApplication_RestoreOverrideCursor
	qt.FuncMap["gui.QGuiApplication_ScreenAt"] = QGuiApplication_ScreenAt
	qt.FuncMap["gui.QGuiApplication_Screens"] = QGuiApplication_Screens
	qt.FuncMap["gui.QGuiApplication_SetApplicationDisplayName"] = QGuiApplication_SetApplicationDisplayName
	qt.FuncMap["gui.QGuiApplication_SetDesktopFileName"] = QGuiApplication_SetDesktopFileName
	qt.FuncMap["gui.QGuiApplication_SetDesktopSettingsAware"] = QGuiApplication_SetDesktopSettingsAware
	qt.FuncMap["gui.QGuiApplication_SetFallbackSessionManagementEnabled"] = QGuiApplication_SetFallbackSessionManagementEnabled
	qt.FuncMap["gui.QGuiApplication_SetFont"] = QGuiApplication_SetFont
	qt.FuncMap["gui.QGuiApplication_SetLayoutDirection"] = QGuiApplication_SetLayoutDirection
	qt.FuncMap["gui.QGuiApplication_SetOverrideCursor"] = QGuiApplication_SetOverrideCursor
	qt.FuncMap["gui.QGuiApplication_SetPalette"] = QGuiApplication_SetPalette
	qt.FuncMap["gui.QGuiApplication_SetQuitOnLastWindowClosed"] = QGuiApplication_SetQuitOnLastWindowClosed
	qt.FuncMap["gui.QGuiApplication_SetWindowIcon"] = QGuiApplication_SetWindowIcon
	qt.FuncMap["gui.QGuiApplication_StyleHints"] = QGuiApplication_StyleHints
	qt.FuncMap["gui.QGuiApplication_Sync"] = QGuiApplication_Sync
	qt.FuncMap["gui.QGuiApplication_TopLevelAt"] = QGuiApplication_TopLevelAt
	qt.FuncMap["gui.QGuiApplication_WindowIcon"] = QGuiApplication_WindowIcon
	qt.ItfMap["gui.QHelpEvent_ITF"] = QHelpEvent{}
	qt.FuncMap["gui.NewQHelpEvent"] = NewQHelpEvent
	qt.ItfMap["gui.QHideEvent_ITF"] = QHideEvent{}
	qt.FuncMap["gui.NewQHideEvent"] = NewQHideEvent
	qt.ItfMap["gui.QHoverEvent_ITF"] = QHoverEvent{}
	qt.FuncMap["gui.NewQHoverEvent"] = NewQHoverEvent
	qt.ItfMap["gui.QIcon_ITF"] = QIcon{}
	qt.FuncMap["gui.NewQIcon"] = NewQIcon
	qt.FuncMap["gui.NewQIcon2"] = NewQIcon2
	qt.FuncMap["gui.NewQIcon3"] = NewQIcon3
	qt.FuncMap["gui.NewQIcon4"] = NewQIcon4
	qt.FuncMap["gui.NewQIcon5"] = NewQIcon5
	qt.FuncMap["gui.NewQIcon6"] = NewQIcon6
	qt.FuncMap["gui.QIcon_FallbackSearchPaths"] = QIcon_FallbackSearchPaths
	qt.FuncMap["gui.QIcon_FallbackThemeName"] = QIcon_FallbackThemeName
	qt.FuncMap["gui.QIcon_FromTheme"] = QIcon_FromTheme
	qt.FuncMap["gui.QIcon_FromTheme2"] = QIcon_FromTheme2
	qt.FuncMap["gui.QIcon_HasThemeIcon"] = QIcon_HasThemeIcon
	qt.FuncMap["gui.QIcon_SetFallbackSearchPaths"] = QIcon_SetFallbackSearchPaths
	qt.FuncMap["gui.QIcon_SetFallbackThemeName"] = QIcon_SetFallbackThemeName
	qt.FuncMap["gui.QIcon_SetThemeName"] = QIcon_SetThemeName
	qt.FuncMap["gui.QIcon_SetThemeSearchPaths"] = QIcon_SetThemeSearchPaths
	qt.FuncMap["gui.QIcon_ThemeName"] = QIcon_ThemeName
	qt.FuncMap["gui.QIcon_ThemeSearchPaths"] = QIcon_ThemeSearchPaths
	qt.EnumMap["gui.QIcon__Normal"] = int64(QIcon__Normal)
	qt.EnumMap["gui.QIcon__Disabled"] = int64(QIcon__Disabled)
	qt.EnumMap["gui.QIcon__Active"] = int64(QIcon__Active)
	qt.EnumMap["gui.QIcon__Selected"] = int64(QIcon__Selected)
	qt.EnumMap["gui.QIcon__On"] = int64(QIcon__On)
	qt.EnumMap["gui.QIcon__Off"] = int64(QIcon__Off)
	qt.ItfMap["gui.QIconDragEvent_ITF"] = QIconDragEvent{}
	qt.FuncMap["gui.NewQIconDragEvent"] = NewQIconDragEvent
	qt.ItfMap["gui.QIconEngine_ITF"] = QIconEngine{}
	qt.FuncMap["gui.NewQIconEngine"] = NewQIconEngine
	qt.EnumMap["gui.QIconEngine__AvailableSizesHook"] = int64(QIconEngine__AvailableSizesHook)
	qt.EnumMap["gui.QIconEngine__IconNameHook"] = int64(QIconEngine__IconNameHook)
	qt.EnumMap["gui.QIconEngine__IsNullHook"] = int64(QIconEngine__IsNullHook)
	qt.EnumMap["gui.QIconEngine__ScaledPixmapHook"] = int64(QIconEngine__ScaledPixmapHook)
	qt.ItfMap["gui.QIconEnginePlugin_ITF"] = QIconEnginePlugin{}
	qt.FuncMap["gui.NewQIconEnginePlugin"] = NewQIconEnginePlugin
	qt.ItfMap["gui.QImage_ITF"] = QImage{}
	qt.FuncMap["gui.NewQImage"] = NewQImage
	qt.FuncMap["gui.NewQImage2"] = NewQImage2
	qt.FuncMap["gui.NewQImage3"] = NewQImage3
	qt.FuncMap["gui.NewQImage4"] = NewQImage4
	qt.FuncMap["gui.NewQImage5"] = NewQImage5
	qt.FuncMap["gui.NewQImage6"] = NewQImage6
	qt.FuncMap["gui.NewQImage7"] = NewQImage7
	qt.FuncMap["gui.NewQImage9"] = NewQImage9
	qt.FuncMap["gui.NewQImage10"] = NewQImage10
	qt.FuncMap["gui.NewQImage11"] = NewQImage11
	qt.FuncMap["gui.QImage_FromData"] = QImage_FromData
	qt.FuncMap["gui.QImage_FromData2"] = QImage_FromData2
	qt.FuncMap["gui.QImage_ToImageFormat"] = QImage_ToImageFormat
	qt.FuncMap["gui.QImage_ToPixelFormat"] = QImage_ToPixelFormat
	qt.FuncMap["gui.QImage_TrueMatrix"] = QImage_TrueMatrix
	qt.FuncMap["gui.QImage_TrueMatrix2"] = QImage_TrueMatrix2
	qt.EnumMap["gui.QImage__InvertRgb"] = int64(QImage__InvertRgb)
	qt.EnumMap["gui.QImage__InvertRgba"] = int64(QImage__InvertRgba)
	qt.EnumMap["gui.QImage__Format_Invalid"] = int64(QImage__Format_Invalid)
	qt.EnumMap["gui.QImage__Format_Mono"] = int64(QImage__Format_Mono)
	qt.EnumMap["gui.QImage__Format_MonoLSB"] = int64(QImage__Format_MonoLSB)
	qt.EnumMap["gui.QImage__Format_Indexed8"] = int64(QImage__Format_Indexed8)
	qt.EnumMap["gui.QImage__Format_RGB32"] = int64(QImage__Format_RGB32)
	qt.EnumMap["gui.QImage__Format_ARGB32"] = int64(QImage__Format_ARGB32)
	qt.EnumMap["gui.QImage__Format_ARGB32_Premultiplied"] = int64(QImage__Format_ARGB32_Premultiplied)
	qt.EnumMap["gui.QImage__Format_RGB16"] = int64(QImage__Format_RGB16)
	qt.EnumMap["gui.QImage__Format_ARGB8565_Premultiplied"] = int64(QImage__Format_ARGB8565_Premultiplied)
	qt.EnumMap["gui.QImage__Format_RGB666"] = int64(QImage__Format_RGB666)
	qt.EnumMap["gui.QImage__Format_ARGB6666_Premultiplied"] = int64(QImage__Format_ARGB6666_Premultiplied)
	qt.EnumMap["gui.QImage__Format_RGB555"] = int64(QImage__Format_RGB555)
	qt.EnumMap["gui.QImage__Format_ARGB8555_Premultiplied"] = int64(QImage__Format_ARGB8555_Premultiplied)
	qt.EnumMap["gui.QImage__Format_RGB888"] = int64(QImage__Format_RGB888)
	qt.EnumMap["gui.QImage__Format_RGB444"] = int64(QImage__Format_RGB444)
	qt.EnumMap["gui.QImage__Format_ARGB4444_Premultiplied"] = int64(QImage__Format_ARGB4444_Premultiplied)
	qt.EnumMap["gui.QImage__Format_RGBX8888"] = int64(QImage__Format_RGBX8888)
	qt.EnumMap["gui.QImage__Format_RGBA8888"] = int64(QImage__Format_RGBA8888)
	qt.EnumMap["gui.QImage__Format_RGBA8888_Premultiplied"] = int64(QImage__Format_RGBA8888_Premultiplied)
	qt.EnumMap["gui.QImage__Format_BGR30"] = int64(QImage__Format_BGR30)
	qt.EnumMap["gui.QImage__Format_A2BGR30_Premultiplied"] = int64(QImage__Format_A2BGR30_Premultiplied)
	qt.EnumMap["gui.QImage__Format_RGB30"] = int64(QImage__Format_RGB30)
	qt.EnumMap["gui.QImage__Format_A2RGB30_Premultiplied"] = int64(QImage__Format_A2RGB30_Premultiplied)
	qt.EnumMap["gui.QImage__Format_Alpha8"] = int64(QImage__Format_Alpha8)
	qt.EnumMap["gui.QImage__Format_Grayscale8"] = int64(QImage__Format_Grayscale8)
	qt.EnumMap["gui.QImage__Format_RGBX64"] = int64(QImage__Format_RGBX64)
	qt.EnumMap["gui.QImage__Format_RGBA64"] = int64(QImage__Format_RGBA64)
	qt.EnumMap["gui.QImage__Format_RGBA64_Premultiplied"] = int64(QImage__Format_RGBA64_Premultiplied)
	qt.EnumMap["gui.QImage__Format_Grayscale16"] = int64(QImage__Format_Grayscale16)
	qt.ItfMap["gui.QImageIOHandler_ITF"] = QImageIOHandler{}
	qt.FuncMap["gui.NewQImageIOHandler"] = NewQImageIOHandler
	qt.EnumMap["gui.QImageIOHandler__Size"] = int64(QImageIOHandler__Size)
	qt.EnumMap["gui.QImageIOHandler__ClipRect"] = int64(QImageIOHandler__ClipRect)
	qt.EnumMap["gui.QImageIOHandler__Description"] = int64(QImageIOHandler__Description)
	qt.EnumMap["gui.QImageIOHandler__ScaledClipRect"] = int64(QImageIOHandler__ScaledClipRect)
	qt.EnumMap["gui.QImageIOHandler__ScaledSize"] = int64(QImageIOHandler__ScaledSize)
	qt.EnumMap["gui.QImageIOHandler__CompressionRatio"] = int64(QImageIOHandler__CompressionRatio)
	qt.EnumMap["gui.QImageIOHandler__Gamma"] = int64(QImageIOHandler__Gamma)
	qt.EnumMap["gui.QImageIOHandler__Quality"] = int64(QImageIOHandler__Quality)
	qt.EnumMap["gui.QImageIOHandler__Name"] = int64(QImageIOHandler__Name)
	qt.EnumMap["gui.QImageIOHandler__SubType"] = int64(QImageIOHandler__SubType)
	qt.EnumMap["gui.QImageIOHandler__IncrementalReading"] = int64(QImageIOHandler__IncrementalReading)
	qt.EnumMap["gui.QImageIOHandler__Endianness"] = int64(QImageIOHandler__Endianness)
	qt.EnumMap["gui.QImageIOHandler__Animation"] = int64(QImageIOHandler__Animation)
	qt.EnumMap["gui.QImageIOHandler__BackgroundColor"] = int64(QImageIOHandler__BackgroundColor)
	qt.EnumMap["gui.QImageIOHandler__ImageFormat"] = int64(QImageIOHandler__ImageFormat)
	qt.EnumMap["gui.QImageIOHandler__SupportedSubTypes"] = int64(QImageIOHandler__SupportedSubTypes)
	qt.EnumMap["gui.QImageIOHandler__OptimizedWrite"] = int64(QImageIOHandler__OptimizedWrite)
	qt.EnumMap["gui.QImageIOHandler__ProgressiveScanWrite"] = int64(QImageIOHandler__ProgressiveScanWrite)
	qt.EnumMap["gui.QImageIOHandler__ImageTransformation"] = int64(QImageIOHandler__ImageTransformation)
	qt.EnumMap["gui.QImageIOHandler__TransformedByDefault"] = int64(QImageIOHandler__TransformedByDefault)
	qt.EnumMap["gui.QImageIOHandler__TransformationNone"] = int64(QImageIOHandler__TransformationNone)
	qt.EnumMap["gui.QImageIOHandler__TransformationMirror"] = int64(QImageIOHandler__TransformationMirror)
	qt.EnumMap["gui.QImageIOHandler__TransformationFlip"] = int64(QImageIOHandler__TransformationFlip)
	qt.EnumMap["gui.QImageIOHandler__TransformationRotate180"] = int64(QImageIOHandler__TransformationRotate180)
	qt.EnumMap["gui.QImageIOHandler__TransformationRotate90"] = int64(QImageIOHandler__TransformationRotate90)
	qt.EnumMap["gui.QImageIOHandler__TransformationMirrorAndRotate90"] = int64(QImageIOHandler__TransformationMirrorAndRotate90)
	qt.EnumMap["gui.QImageIOHandler__TransformationFlipAndRotate90"] = int64(QImageIOHandler__TransformationFlipAndRotate90)
	qt.EnumMap["gui.QImageIOHandler__TransformationRotate270"] = int64(QImageIOHandler__TransformationRotate270)
	qt.ItfMap["gui.QImageIOPlugin_ITF"] = QImageIOPlugin{}
	qt.FuncMap["gui.NewQImageIOPlugin"] = NewQImageIOPlugin
	qt.EnumMap["gui.QImageIOPlugin__CanRead"] = int64(QImageIOPlugin__CanRead)
	qt.EnumMap["gui.QImageIOPlugin__CanWrite"] = int64(QImageIOPlugin__CanWrite)
	qt.EnumMap["gui.QImageIOPlugin__CanReadIncremental"] = int64(QImageIOPlugin__CanReadIncremental)
	qt.ItfMap["gui.QImageReader_ITF"] = QImageReader{}
	qt.FuncMap["gui.NewQImageReader"] = NewQImageReader
	qt.FuncMap["gui.NewQImageReader2"] = NewQImageReader2
	qt.FuncMap["gui.NewQImageReader3"] = NewQImageReader3
	qt.FuncMap["gui.QImageReader_ImageFormat2"] = QImageReader_ImageFormat2
	qt.FuncMap["gui.QImageReader_ImageFormat3"] = QImageReader_ImageFormat3
	qt.FuncMap["gui.QImageReader_ImageFormatsForMimeType"] = QImageReader_ImageFormatsForMimeType
	qt.FuncMap["gui.QImageReader_SupportedImageFormats"] = QImageReader_SupportedImageFormats
	qt.FuncMap["gui.QImageReader_SupportedMimeTypes"] = QImageReader_SupportedMimeTypes
	qt.EnumMap["gui.QImageReader__UnknownError"] = int64(QImageReader__UnknownError)
	qt.EnumMap["gui.QImageReader__FileNotFoundError"] = int64(QImageReader__FileNotFoundError)
	qt.EnumMap["gui.QImageReader__DeviceError"] = int64(QImageReader__DeviceError)
	qt.EnumMap["gui.QImageReader__UnsupportedFormatError"] = int64(QImageReader__UnsupportedFormatError)
	qt.EnumMap["gui.QImageReader__InvalidDataError"] = int64(QImageReader__InvalidDataError)
	qt.ItfMap["gui.QImageTextKeyLang_ITF"] = QImageTextKeyLang{}
	qt.ItfMap["gui.QImageWriter_ITF"] = QImageWriter{}
	qt.FuncMap["gui.NewQImageWriter"] = NewQImageWriter
	qt.FuncMap["gui.NewQImageWriter2"] = NewQImageWriter2
	qt.FuncMap["gui.NewQImageWriter3"] = NewQImageWriter3
	qt.FuncMap["gui.QImageWriter_ImageFormatsForMimeType"] = QImageWriter_ImageFormatsForMimeType
	qt.FuncMap["gui.QImageWriter_SupportedImageFormats"] = QImageWriter_SupportedImageFormats
	qt.FuncMap["gui.QImageWriter_SupportedMimeTypes"] = QImageWriter_SupportedMimeTypes
	qt.EnumMap["gui.QImageWriter__UnknownError"] = int64(QImageWriter__UnknownError)
	qt.EnumMap["gui.QImageWriter__DeviceError"] = int64(QImageWriter__DeviceError)
	qt.EnumMap["gui.QImageWriter__UnsupportedFormatError"] = int64(QImageWriter__UnsupportedFormatError)
	qt.EnumMap["gui.QImageWriter__InvalidImageError"] = int64(QImageWriter__InvalidImageError)
	qt.ItfMap["gui.QInputEvent_ITF"] = QInputEvent{}
	qt.ItfMap["gui.QInputMethod_ITF"] = QInputMethod{}
	qt.FuncMap["gui.QInputMethod_QueryFocusObject"] = QInputMethod_QueryFocusObject
	qt.EnumMap["gui.QInputMethod__Click"] = int64(QInputMethod__Click)
	qt.EnumMap["gui.QInputMethod__ContextMenu"] = int64(QInputMethod__ContextMenu)
	qt.ItfMap["gui.QInputMethodEvent_ITF"] = QInputMethodEvent{}
	qt.FuncMap["gui.NewQInputMethodEvent"] = NewQInputMethodEvent
	qt.FuncMap["gui.NewQInputMethodEvent3"] = NewQInputMethodEvent3
	qt.EnumMap["gui.QInputMethodEvent__TextFormat"] = int64(QInputMethodEvent__TextFormat)
	qt.EnumMap["gui.QInputMethodEvent__Cursor"] = int64(QInputMethodEvent__Cursor)
	qt.EnumMap["gui.QInputMethodEvent__Language"] = int64(QInputMethodEvent__Language)
	qt.EnumMap["gui.QInputMethodEvent__Ruby"] = int64(QInputMethodEvent__Ruby)
	qt.EnumMap["gui.QInputMethodEvent__Selection"] = int64(QInputMethodEvent__Selection)
	qt.ItfMap["gui.QInputMethodQueryEvent_ITF"] = QInputMethodQueryEvent{}
	qt.FuncMap["gui.NewQInputMethodQueryEvent"] = NewQInputMethodQueryEvent
	qt.ItfMap["gui.QIntValidator_ITF"] = QIntValidator{}
	qt.FuncMap["gui.NewQIntValidator"] = NewQIntValidator
	qt.FuncMap["gui.NewQIntValidator2"] = NewQIntValidator2
	qt.ItfMap["gui.QKeyEvent_ITF"] = QKeyEvent{}
	qt.FuncMap["gui.NewQKeyEvent"] = NewQKeyEvent
	qt.FuncMap["gui.NewQKeyEvent2"] = NewQKeyEvent2
	qt.ItfMap["gui.QKeySequence_ITF"] = QKeySequence{}
	qt.FuncMap["gui.NewQKeySequence"] = NewQKeySequence
	qt.FuncMap["gui.NewQKeySequence2"] = NewQKeySequence2
	qt.FuncMap["gui.NewQKeySequence3"] = NewQKeySequence3
	qt.FuncMap["gui.NewQKeySequence4"] = NewQKeySequence4
	qt.FuncMap["gui.NewQKeySequence5"] = NewQKeySequence5
	qt.FuncMap["gui.QKeySequence_FromString"] = QKeySequence_FromString
	qt.FuncMap["gui.QKeySequence_KeyBindings"] = QKeySequence_KeyBindings
	qt.FuncMap["gui.QKeySequence_ListFromString"] = QKeySequence_ListFromString
	qt.FuncMap["gui.QKeySequence_ListToString"] = QKeySequence_ListToString
	qt.FuncMap["gui.QKeySequence_Mnemonic"] = QKeySequence_Mnemonic
	qt.EnumMap["gui.QKeySequence__UnknownKey"] = int64(QKeySequence__UnknownKey)
	qt.EnumMap["gui.QKeySequence__HelpContents"] = int64(QKeySequence__HelpContents)
	qt.EnumMap["gui.QKeySequence__WhatsThis"] = int64(QKeySequence__WhatsThis)
	qt.EnumMap["gui.QKeySequence__Open"] = int64(QKeySequence__Open)
	qt.EnumMap["gui.QKeySequence__Close"] = int64(QKeySequence__Close)
	qt.EnumMap["gui.QKeySequence__Save"] = int64(QKeySequence__Save)
	qt.EnumMap["gui.QKeySequence__New"] = int64(QKeySequence__New)
	qt.EnumMap["gui.QKeySequence__Delete"] = int64(QKeySequence__Delete)
	qt.EnumMap["gui.QKeySequence__Cut"] = int64(QKeySequence__Cut)
	qt.EnumMap["gui.QKeySequence__Copy"] = int64(QKeySequence__Copy)
	qt.EnumMap["gui.QKeySequence__Paste"] = int64(QKeySequence__Paste)
	qt.EnumMap["gui.QKeySequence__Undo"] = int64(QKeySequence__Undo)
	qt.EnumMap["gui.QKeySequence__Redo"] = int64(QKeySequence__Redo)
	qt.EnumMap["gui.QKeySequence__Back"] = int64(QKeySequence__Back)
	qt.EnumMap["gui.QKeySequence__Forward"] = int64(QKeySequence__Forward)
	qt.EnumMap["gui.QKeySequence__Refresh"] = int64(QKeySequence__Refresh)
	qt.EnumMap["gui.QKeySequence__ZoomIn"] = int64(QKeySequence__ZoomIn)
	qt.EnumMap["gui.QKeySequence__ZoomOut"] = int64(QKeySequence__ZoomOut)
	qt.EnumMap["gui.QKeySequence__Print"] = int64(QKeySequence__Print)
	qt.EnumMap["gui.QKeySequence__AddTab"] = int64(QKeySequence__AddTab)
	qt.EnumMap["gui.QKeySequence__NextChild"] = int64(QKeySequence__NextChild)
	qt.EnumMap["gui.QKeySequence__PreviousChild"] = int64(QKeySequence__PreviousChild)
	qt.EnumMap["gui.QKeySequence__Find"] = int64(QKeySequence__Find)
	qt.EnumMap["gui.QKeySequence__FindNext"] = int64(QKeySequence__FindNext)
	qt.EnumMap["gui.QKeySequence__FindPrevious"] = int64(QKeySequence__FindPrevious)
	qt.EnumMap["gui.QKeySequence__Replace"] = int64(QKeySequence__Replace)
	qt.EnumMap["gui.QKeySequence__SelectAll"] = int64(QKeySequence__SelectAll)
	qt.EnumMap["gui.QKeySequence__Bold"] = int64(QKeySequence__Bold)
	qt.EnumMap["gui.QKeySequence__Italic"] = int64(QKeySequence__Italic)
	qt.EnumMap["gui.QKeySequence__Underline"] = int64(QKeySequence__Underline)
	qt.EnumMap["gui.QKeySequence__MoveToNextChar"] = int64(QKeySequence__MoveToNextChar)
	qt.EnumMap["gui.QKeySequence__MoveToPreviousChar"] = int64(QKeySequence__MoveToPreviousChar)
	qt.EnumMap["gui.QKeySequence__MoveToNextWord"] = int64(QKeySequence__MoveToNextWord)
	qt.EnumMap["gui.QKeySequence__MoveToPreviousWord"] = int64(QKeySequence__MoveToPreviousWord)
	qt.EnumMap["gui.QKeySequence__MoveToNextLine"] = int64(QKeySequence__MoveToNextLine)
	qt.EnumMap["gui.QKeySequence__MoveToPreviousLine"] = int64(QKeySequence__MoveToPreviousLine)
	qt.EnumMap["gui.QKeySequence__MoveToNextPage"] = int64(QKeySequence__MoveToNextPage)
	qt.EnumMap["gui.QKeySequence__MoveToPreviousPage"] = int64(QKeySequence__MoveToPreviousPage)
	qt.EnumMap["gui.QKeySequence__MoveToStartOfLine"] = int64(QKeySequence__MoveToStartOfLine)
	qt.EnumMap["gui.QKeySequence__MoveToEndOfLine"] = int64(QKeySequence__MoveToEndOfLine)
	qt.EnumMap["gui.QKeySequence__MoveToStartOfBlock"] = int64(QKeySequence__MoveToStartOfBlock)
	qt.EnumMap["gui.QKeySequence__MoveToEndOfBlock"] = int64(QKeySequence__MoveToEndOfBlock)
	qt.EnumMap["gui.QKeySequence__MoveToStartOfDocument"] = int64(QKeySequence__MoveToStartOfDocument)
	qt.EnumMap["gui.QKeySequence__MoveToEndOfDocument"] = int64(QKeySequence__MoveToEndOfDocument)
	qt.EnumMap["gui.QKeySequence__SelectNextChar"] = int64(QKeySequence__SelectNextChar)
	qt.EnumMap["gui.QKeySequence__SelectPreviousChar"] = int64(QKeySequence__SelectPreviousChar)
	qt.EnumMap["gui.QKeySequence__SelectNextWord"] = int64(QKeySequence__SelectNextWord)
	qt.EnumMap["gui.QKeySequence__SelectPreviousWord"] = int64(QKeySequence__SelectPreviousWord)
	qt.EnumMap["gui.QKeySequence__SelectNextLine"] = int64(QKeySequence__SelectNextLine)
	qt.EnumMap["gui.QKeySequence__SelectPreviousLine"] = int64(QKeySequence__SelectPreviousLine)
	qt.EnumMap["gui.QKeySequence__SelectNextPage"] = int64(QKeySequence__SelectNextPage)
	qt.EnumMap["gui.QKeySequence__SelectPreviousPage"] = int64(QKeySequence__SelectPreviousPage)
	qt.EnumMap["gui.QKeySequence__SelectStartOfLine"] = int64(QKeySequence__SelectStartOfLine)
	qt.EnumMap["gui.QKeySequence__SelectEndOfLine"] = int64(QKeySequence__SelectEndOfLine)
	qt.EnumMap["gui.QKeySequence__SelectStartOfBlock"] = int64(QKeySequence__SelectStartOfBlock)
	qt.EnumMap["gui.QKeySequence__SelectEndOfBlock"] = int64(QKeySequence__SelectEndOfBlock)
	qt.EnumMap["gui.QKeySequence__SelectStartOfDocument"] = int64(QKeySequence__SelectStartOfDocument)
	qt.EnumMap["gui.QKeySequence__SelectEndOfDocument"] = int64(QKeySequence__SelectEndOfDocument)
	qt.EnumMap["gui.QKeySequence__DeleteStartOfWord"] = int64(QKeySequence__DeleteStartOfWord)
	qt.EnumMap["gui.QKeySequence__DeleteEndOfWord"] = int64(QKeySequence__DeleteEndOfWord)
	qt.EnumMap["gui.QKeySequence__DeleteEndOfLine"] = int64(QKeySequence__DeleteEndOfLine)
	qt.EnumMap["gui.QKeySequence__InsertParagraphSeparator"] = int64(QKeySequence__InsertParagraphSeparator)
	qt.EnumMap["gui.QKeySequence__InsertLineSeparator"] = int64(QKeySequence__InsertLineSeparator)
	qt.EnumMap["gui.QKeySequence__SaveAs"] = int64(QKeySequence__SaveAs)
	qt.EnumMap["gui.QKeySequence__Preferences"] = int64(QKeySequence__Preferences)
	qt.EnumMap["gui.QKeySequence__Quit"] = int64(QKeySequence__Quit)
	qt.EnumMap["gui.QKeySequence__FullScreen"] = int64(QKeySequence__FullScreen)
	qt.EnumMap["gui.QKeySequence__Deselect"] = int64(QKeySequence__Deselect)
	qt.EnumMap["gui.QKeySequence__DeleteCompleteLine"] = int64(QKeySequence__DeleteCompleteLine)
	qt.EnumMap["gui.QKeySequence__Backspace"] = int64(QKeySequence__Backspace)
	qt.EnumMap["gui.QKeySequence__Cancel"] = int64(QKeySequence__Cancel)
	qt.EnumMap["gui.QKeySequence__NativeText"] = int64(QKeySequence__NativeText)
	qt.EnumMap["gui.QKeySequence__PortableText"] = int64(QKeySequence__PortableText)
	qt.EnumMap["gui.QKeySequence__NoMatch"] = int64(QKeySequence__NoMatch)
	qt.EnumMap["gui.QKeySequence__PartialMatch"] = int64(QKeySequence__PartialMatch)
	qt.EnumMap["gui.QKeySequence__ExactMatch"] = int64(QKeySequence__ExactMatch)
	qt.ItfMap["gui.QLinearGradient_ITF"] = QLinearGradient{}
	qt.FuncMap["gui.NewQLinearGradient"] = NewQLinearGradient
	qt.FuncMap["gui.NewQLinearGradient2"] = NewQLinearGradient2
	qt.FuncMap["gui.NewQLinearGradient3"] = NewQLinearGradient3
	qt.ItfMap["gui.QMatrix_ITF"] = QMatrix{}
	qt.FuncMap["gui.NewQMatrix2"] = NewQMatrix2
	qt.FuncMap["gui.NewQMatrix3"] = NewQMatrix3
	qt.FuncMap["gui.NewQMatrix5"] = NewQMatrix5
	qt.ItfMap["gui.QMatrix4x4_ITF"] = QMatrix4x4{}
	qt.FuncMap["gui.NewQMatrix4x4"] = NewQMatrix4x4
	qt.FuncMap["gui.NewQMatrix4x43"] = NewQMatrix4x43
	qt.FuncMap["gui.NewQMatrix4x44"] = NewQMatrix4x44
	qt.FuncMap["gui.NewQMatrix4x46"] = NewQMatrix4x46
	qt.ItfMap["gui.QMessageDialogOptions_ITF"] = QMessageDialogOptions{}
	qt.ItfMap["gui.QMouseEvent_ITF"] = QMouseEvent{}
	qt.FuncMap["gui.NewQMouseEvent"] = NewQMouseEvent
	qt.FuncMap["gui.NewQMouseEvent2"] = NewQMouseEvent2
	qt.FuncMap["gui.NewQMouseEvent3"] = NewQMouseEvent3
	qt.FuncMap["gui.NewQMouseEvent4"] = NewQMouseEvent4
	qt.ItfMap["gui.QMoveEvent_ITF"] = QMoveEvent{}
	qt.FuncMap["gui.NewQMoveEvent"] = NewQMoveEvent
	qt.ItfMap["gui.QMovie_ITF"] = QMovie{}
	qt.FuncMap["gui.NewQMovie"] = NewQMovie
	qt.FuncMap["gui.NewQMovie2"] = NewQMovie2
	qt.FuncMap["gui.NewQMovie3"] = NewQMovie3
	qt.FuncMap["gui.QMovie_SupportedFormats"] = QMovie_SupportedFormats
	qt.EnumMap["gui.QMovie__NotRunning"] = int64(QMovie__NotRunning)
	qt.EnumMap["gui.QMovie__Paused"] = int64(QMovie__Paused)
	qt.EnumMap["gui.QMovie__Running"] = int64(QMovie__Running)
	qt.EnumMap["gui.QMovie__CacheNone"] = int64(QMovie__CacheNone)
	qt.EnumMap["gui.QMovie__CacheAll"] = int64(QMovie__CacheAll)
	qt.ItfMap["gui.QNativeGestureEvent_ITF"] = QNativeGestureEvent{}
	qt.FuncMap["gui.NewQNativeGestureEvent2"] = NewQNativeGestureEvent2
	qt.ItfMap["gui.QOffscreenSurface_ITF"] = QOffscreenSurface{}
	qt.FuncMap["gui.NewQOffscreenSurface"] = NewQOffscreenSurface
	qt.FuncMap["gui.NewQOffscreenSurface2"] = NewQOffscreenSurface2
	qt.ItfMap["gui.QOpenGLBuffer_ITF"] = QOpenGLBuffer{}
	qt.FuncMap["gui.NewQOpenGLBuffer"] = NewQOpenGLBuffer
	qt.FuncMap["gui.NewQOpenGLBuffer2"] = NewQOpenGLBuffer2
	qt.FuncMap["gui.NewQOpenGLBuffer3"] = NewQOpenGLBuffer3
	qt.FuncMap["gui.QOpenGLBuffer_Release2"] = QOpenGLBuffer_Release2
	qt.EnumMap["gui.QOpenGLBuffer__VertexBuffer"] = int64(QOpenGLBuffer__VertexBuffer)
	qt.EnumMap["gui.QOpenGLBuffer__IndexBuffer"] = int64(QOpenGLBuffer__IndexBuffer)
	qt.EnumMap["gui.QOpenGLBuffer__PixelPackBuffer"] = int64(QOpenGLBuffer__PixelPackBuffer)
	qt.EnumMap["gui.QOpenGLBuffer__PixelUnpackBuffer"] = int64(QOpenGLBuffer__PixelUnpackBuffer)
	qt.EnumMap["gui.QOpenGLBuffer__StreamDraw"] = int64(QOpenGLBuffer__StreamDraw)
	qt.EnumMap["gui.QOpenGLBuffer__StreamRead"] = int64(QOpenGLBuffer__StreamRead)
	qt.EnumMap["gui.QOpenGLBuffer__StreamCopy"] = int64(QOpenGLBuffer__StreamCopy)
	qt.EnumMap["gui.QOpenGLBuffer__StaticDraw"] = int64(QOpenGLBuffer__StaticDraw)
	qt.EnumMap["gui.QOpenGLBuffer__StaticRead"] = int64(QOpenGLBuffer__StaticRead)
	qt.EnumMap["gui.QOpenGLBuffer__StaticCopy"] = int64(QOpenGLBuffer__StaticCopy)
	qt.EnumMap["gui.QOpenGLBuffer__DynamicDraw"] = int64(QOpenGLBuffer__DynamicDraw)
	qt.EnumMap["gui.QOpenGLBuffer__DynamicRead"] = int64(QOpenGLBuffer__DynamicRead)
	qt.EnumMap["gui.QOpenGLBuffer__DynamicCopy"] = int64(QOpenGLBuffer__DynamicCopy)
	qt.EnumMap["gui.QOpenGLBuffer__ReadOnly"] = int64(QOpenGLBuffer__ReadOnly)
	qt.EnumMap["gui.QOpenGLBuffer__WriteOnly"] = int64(QOpenGLBuffer__WriteOnly)
	qt.EnumMap["gui.QOpenGLBuffer__ReadWrite"] = int64(QOpenGLBuffer__ReadWrite)
	qt.EnumMap["gui.QOpenGLBuffer__RangeRead"] = int64(QOpenGLBuffer__RangeRead)
	qt.EnumMap["gui.QOpenGLBuffer__RangeWrite"] = int64(QOpenGLBuffer__RangeWrite)
	qt.EnumMap["gui.QOpenGLBuffer__RangeInvalidate"] = int64(QOpenGLBuffer__RangeInvalidate)
	qt.EnumMap["gui.QOpenGLBuffer__RangeInvalidateBuffer"] = int64(QOpenGLBuffer__RangeInvalidateBuffer)
	qt.EnumMap["gui.QOpenGLBuffer__RangeFlushExplicit"] = int64(QOpenGLBuffer__RangeFlushExplicit)
	qt.EnumMap["gui.QOpenGLBuffer__RangeUnsynchronized"] = int64(QOpenGLBuffer__RangeUnsynchronized)
	qt.ItfMap["gui.QOpenGLContext_ITF"] = QOpenGLContext{}
	qt.FuncMap["gui.NewQOpenGLContext"] = NewQOpenGLContext
	qt.FuncMap["gui.QOpenGLContext_AreSharing"] = QOpenGLContext_AreSharing
	qt.FuncMap["gui.QOpenGLContext_CurrentContext"] = QOpenGLContext_CurrentContext
	qt.FuncMap["gui.QOpenGLContext_GlobalShareContext"] = QOpenGLContext_GlobalShareContext
	qt.FuncMap["gui.QOpenGLContext_OpenGLModuleHandle"] = QOpenGLContext_OpenGLModuleHandle
	qt.FuncMap["gui.QOpenGLContext_OpenGLModuleType"] = QOpenGLContext_OpenGLModuleType
	qt.FuncMap["gui.QOpenGLContext_SupportsThreadedOpenGL"] = QOpenGLContext_SupportsThreadedOpenGL
	qt.EnumMap["gui.QOpenGLContext__LibGL"] = int64(QOpenGLContext__LibGL)
	qt.EnumMap["gui.QOpenGLContext__LibGLES"] = int64(QOpenGLContext__LibGLES)
	qt.ItfMap["gui.QOpenGLContextGroup_ITF"] = QOpenGLContextGroup{}
	qt.FuncMap["gui.QOpenGLContextGroup_CurrentContextGroup"] = QOpenGLContextGroup_CurrentContextGroup
	qt.ItfMap["gui.QOpenGLDebugLogger_ITF"] = QOpenGLDebugLogger{}
	qt.FuncMap["gui.NewQOpenGLDebugLogger"] = NewQOpenGLDebugLogger
	qt.EnumMap["gui.QOpenGLDebugLogger__AsynchronousLogging"] = int64(QOpenGLDebugLogger__AsynchronousLogging)
	qt.EnumMap["gui.QOpenGLDebugLogger__SynchronousLogging"] = int64(QOpenGLDebugLogger__SynchronousLogging)
	qt.ItfMap["gui.QOpenGLDebugMessage_ITF"] = QOpenGLDebugMessage{}
	qt.FuncMap["gui.NewQOpenGLDebugMessage"] = NewQOpenGLDebugMessage
	qt.FuncMap["gui.NewQOpenGLDebugMessage2"] = NewQOpenGLDebugMessage2
	qt.FuncMap["gui.QOpenGLDebugMessage_CreateApplicationMessage"] = QOpenGLDebugMessage_CreateApplicationMessage
	qt.FuncMap["gui.QOpenGLDebugMessage_CreateThirdPartyMessage"] = QOpenGLDebugMessage_CreateThirdPartyMessage
	qt.EnumMap["gui.QOpenGLDebugMessage__InvalidSource"] = int64(QOpenGLDebugMessage__InvalidSource)
	qt.EnumMap["gui.QOpenGLDebugMessage__APISource"] = int64(QOpenGLDebugMessage__APISource)
	qt.EnumMap["gui.QOpenGLDebugMessage__WindowSystemSource"] = int64(QOpenGLDebugMessage__WindowSystemSource)
	qt.EnumMap["gui.QOpenGLDebugMessage__ShaderCompilerSource"] = int64(QOpenGLDebugMessage__ShaderCompilerSource)
	qt.EnumMap["gui.QOpenGLDebugMessage__ThirdPartySource"] = int64(QOpenGLDebugMessage__ThirdPartySource)
	qt.EnumMap["gui.QOpenGLDebugMessage__ApplicationSource"] = int64(QOpenGLDebugMessage__ApplicationSource)
	qt.EnumMap["gui.QOpenGLDebugMessage__OtherSource"] = int64(QOpenGLDebugMessage__OtherSource)
	qt.EnumMap["gui.QOpenGLDebugMessage__LastSource"] = int64(QOpenGLDebugMessage__LastSource)
	qt.EnumMap["gui.QOpenGLDebugMessage__AnySource"] = int64(QOpenGLDebugMessage__AnySource)
	qt.EnumMap["gui.QOpenGLDebugMessage__InvalidType"] = int64(QOpenGLDebugMessage__InvalidType)
	qt.EnumMap["gui.QOpenGLDebugMessage__ErrorType"] = int64(QOpenGLDebugMessage__ErrorType)
	qt.EnumMap["gui.QOpenGLDebugMessage__DeprecatedBehaviorType"] = int64(QOpenGLDebugMessage__DeprecatedBehaviorType)
	qt.EnumMap["gui.QOpenGLDebugMessage__UndefinedBehaviorType"] = int64(QOpenGLDebugMessage__UndefinedBehaviorType)
	qt.EnumMap["gui.QOpenGLDebugMessage__PortabilityType"] = int64(QOpenGLDebugMessage__PortabilityType)
	qt.EnumMap["gui.QOpenGLDebugMessage__PerformanceType"] = int64(QOpenGLDebugMessage__PerformanceType)
	qt.EnumMap["gui.QOpenGLDebugMessage__OtherType"] = int64(QOpenGLDebugMessage__OtherType)
	qt.EnumMap["gui.QOpenGLDebugMessage__MarkerType"] = int64(QOpenGLDebugMessage__MarkerType)
	qt.EnumMap["gui.QOpenGLDebugMessage__GroupPushType"] = int64(QOpenGLDebugMessage__GroupPushType)
	qt.EnumMap["gui.QOpenGLDebugMessage__GroupPopType"] = int64(QOpenGLDebugMessage__GroupPopType)
	qt.EnumMap["gui.QOpenGLDebugMessage__LastType"] = int64(QOpenGLDebugMessage__LastType)
	qt.EnumMap["gui.QOpenGLDebugMessage__AnyType"] = int64(QOpenGLDebugMessage__AnyType)
	qt.EnumMap["gui.QOpenGLDebugMessage__InvalidSeverity"] = int64(QOpenGLDebugMessage__InvalidSeverity)
	qt.EnumMap["gui.QOpenGLDebugMessage__HighSeverity"] = int64(QOpenGLDebugMessage__HighSeverity)
	qt.EnumMap["gui.QOpenGLDebugMessage__MediumSeverity"] = int64(QOpenGLDebugMessage__MediumSeverity)
	qt.EnumMap["gui.QOpenGLDebugMessage__LowSeverity"] = int64(QOpenGLDebugMessage__LowSeverity)
	qt.EnumMap["gui.QOpenGLDebugMessage__NotificationSeverity"] = int64(QOpenGLDebugMessage__NotificationSeverity)
	qt.EnumMap["gui.QOpenGLDebugMessage__LastSeverity"] = int64(QOpenGLDebugMessage__LastSeverity)
	qt.EnumMap["gui.QOpenGLDebugMessage__AnySeverity"] = int64(QOpenGLDebugMessage__AnySeverity)
	qt.ItfMap["gui.QOpenGLExtraFunctions_ITF"] = QOpenGLExtraFunctions{}
	qt.FuncMap["gui.NewQOpenGLExtraFunctions"] = NewQOpenGLExtraFunctions
	qt.FuncMap["gui.NewQOpenGLExtraFunctions2"] = NewQOpenGLExtraFunctions2
	qt.ItfMap["gui.QOpenGLFramebufferObject_ITF"] = QOpenGLFramebufferObject{}
	qt.FuncMap["gui.NewQOpenGLFramebufferObject"] = NewQOpenGLFramebufferObject
	qt.FuncMap["gui.NewQOpenGLFramebufferObject2"] = NewQOpenGLFramebufferObject2
	qt.FuncMap["gui.NewQOpenGLFramebufferObject3"] = NewQOpenGLFramebufferObject3
	qt.FuncMap["gui.NewQOpenGLFramebufferObject4"] = NewQOpenGLFramebufferObject4
	qt.FuncMap["gui.NewQOpenGLFramebufferObject5"] = NewQOpenGLFramebufferObject5
	qt.FuncMap["gui.NewQOpenGLFramebufferObject6"] = NewQOpenGLFramebufferObject6
	qt.FuncMap["gui.QOpenGLFramebufferObject_BindDefault"] = QOpenGLFramebufferObject_BindDefault
	qt.FuncMap["gui.QOpenGLFramebufferObject_BlitFramebuffer"] = QOpenGLFramebufferObject_BlitFramebuffer
	qt.FuncMap["gui.QOpenGLFramebufferObject_BlitFramebuffer2"] = QOpenGLFramebufferObject_BlitFramebuffer2
	qt.FuncMap["gui.QOpenGLFramebufferObject_BlitFramebuffer3"] = QOpenGLFramebufferObject_BlitFramebuffer3
	qt.FuncMap["gui.QOpenGLFramebufferObject_BlitFramebuffer4"] = QOpenGLFramebufferObject_BlitFramebuffer4
	qt.FuncMap["gui.QOpenGLFramebufferObject_HasOpenGLFramebufferBlit"] = QOpenGLFramebufferObject_HasOpenGLFramebufferBlit
	qt.FuncMap["gui.QOpenGLFramebufferObject_HasOpenGLFramebufferObjects"] = QOpenGLFramebufferObject_HasOpenGLFramebufferObjects
	qt.EnumMap["gui.QOpenGLFramebufferObject__NoAttachment"] = int64(QOpenGLFramebufferObject__NoAttachment)
	qt.EnumMap["gui.QOpenGLFramebufferObject__CombinedDepthStencil"] = int64(QOpenGLFramebufferObject__CombinedDepthStencil)
	qt.EnumMap["gui.QOpenGLFramebufferObject__Depth"] = int64(QOpenGLFramebufferObject__Depth)
	qt.EnumMap["gui.QOpenGLFramebufferObject__DontRestoreFramebufferBinding"] = int64(QOpenGLFramebufferObject__DontRestoreFramebufferBinding)
	qt.EnumMap["gui.QOpenGLFramebufferObject__RestoreFramebufferBindingToDefault"] = int64(QOpenGLFramebufferObject__RestoreFramebufferBindingToDefault)
	qt.EnumMap["gui.QOpenGLFramebufferObject__RestoreFrameBufferBinding"] = int64(QOpenGLFramebufferObject__RestoreFrameBufferBinding)
	qt.ItfMap["gui.QOpenGLFramebufferObjectFormat_ITF"] = QOpenGLFramebufferObjectFormat{}
	qt.FuncMap["gui.NewQOpenGLFramebufferObjectFormat"] = NewQOpenGLFramebufferObjectFormat
	qt.FuncMap["gui.NewQOpenGLFramebufferObjectFormat2"] = NewQOpenGLFramebufferObjectFormat2
	qt.ItfMap["gui.QOpenGLFunctions_ITF"] = QOpenGLFunctions{}
	qt.FuncMap["gui.NewQOpenGLFunctions"] = NewQOpenGLFunctions
	qt.FuncMap["gui.NewQOpenGLFunctions2"] = NewQOpenGLFunctions2
	qt.EnumMap["gui.QOpenGLFunctions__Multitexture"] = int64(QOpenGLFunctions__Multitexture)
	qt.EnumMap["gui.QOpenGLFunctions__Shaders"] = int64(QOpenGLFunctions__Shaders)
	qt.EnumMap["gui.QOpenGLFunctions__Buffers"] = int64(QOpenGLFunctions__Buffers)
	qt.EnumMap["gui.QOpenGLFunctions__Framebuffers"] = int64(QOpenGLFunctions__Framebuffers)
	qt.EnumMap["gui.QOpenGLFunctions__BlendColor"] = int64(QOpenGLFunctions__BlendColor)
	qt.EnumMap["gui.QOpenGLFunctions__BlendEquation"] = int64(QOpenGLFunctions__BlendEquation)
	qt.EnumMap["gui.QOpenGLFunctions__BlendEquationSeparate"] = int64(QOpenGLFunctions__BlendEquationSeparate)
	qt.EnumMap["gui.QOpenGLFunctions__BlendFuncSeparate"] = int64(QOpenGLFunctions__BlendFuncSeparate)
	qt.EnumMap["gui.QOpenGLFunctions__BlendSubtract"] = int64(QOpenGLFunctions__BlendSubtract)
	qt.EnumMap["gui.QOpenGLFunctions__CompressedTextures"] = int64(QOpenGLFunctions__CompressedTextures)
	qt.EnumMap["gui.QOpenGLFunctions__Multisample"] = int64(QOpenGLFunctions__Multisample)
	qt.EnumMap["gui.QOpenGLFunctions__StencilSeparate"] = int64(QOpenGLFunctions__StencilSeparate)
	qt.EnumMap["gui.QOpenGLFunctions__NPOTTextures"] = int64(QOpenGLFunctions__NPOTTextures)
	qt.EnumMap["gui.QOpenGLFunctions__NPOTTextureRepeat"] = int64(QOpenGLFunctions__NPOTTextureRepeat)
	qt.EnumMap["gui.QOpenGLFunctions__FixedFunctionPipeline"] = int64(QOpenGLFunctions__FixedFunctionPipeline)
	qt.EnumMap["gui.QOpenGLFunctions__TextureRGFormats"] = int64(QOpenGLFunctions__TextureRGFormats)
	qt.EnumMap["gui.QOpenGLFunctions__MultipleRenderTargets"] = int64(QOpenGLFunctions__MultipleRenderTargets)
	qt.EnumMap["gui.QOpenGLFunctions__BlendEquationAdvanced"] = int64(QOpenGLFunctions__BlendEquationAdvanced)
	qt.ItfMap["gui.QOpenGLFunctions_1_0_ITF"] = QOpenGLFunctions_1_0{}
	qt.ItfMap["gui.QOpenGLFunctions_1_0_CoreBackend_ITF"] = QOpenGLFunctions_1_0_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_1_0_DeprecatedBackend_ITF"] = QOpenGLFunctions_1_0_DeprecatedBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_1_1_ITF"] = QOpenGLFunctions_1_1{}
	qt.ItfMap["gui.QOpenGLFunctions_1_1_CoreBackend_ITF"] = QOpenGLFunctions_1_1_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_1_1_DeprecatedBackend_ITF"] = QOpenGLFunctions_1_1_DeprecatedBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_1_2_ITF"] = QOpenGLFunctions_1_2{}
	qt.ItfMap["gui.QOpenGLFunctions_1_2_CoreBackend_ITF"] = QOpenGLFunctions_1_2_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_1_2_DeprecatedBackend_ITF"] = QOpenGLFunctions_1_2_DeprecatedBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_1_3_ITF"] = QOpenGLFunctions_1_3{}
	qt.ItfMap["gui.QOpenGLFunctions_1_3_CoreBackend_ITF"] = QOpenGLFunctions_1_3_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_1_3_DeprecatedBackend_ITF"] = QOpenGLFunctions_1_3_DeprecatedBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_1_4_ITF"] = QOpenGLFunctions_1_4{}
	qt.ItfMap["gui.QOpenGLFunctions_1_4_CoreBackend_ITF"] = QOpenGLFunctions_1_4_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_1_4_DeprecatedBackend_ITF"] = QOpenGLFunctions_1_4_DeprecatedBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_1_5_ITF"] = QOpenGLFunctions_1_5{}
	qt.ItfMap["gui.QOpenGLFunctions_1_5_CoreBackend_ITF"] = QOpenGLFunctions_1_5_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_2_0_ITF"] = QOpenGLFunctions_2_0{}
	qt.ItfMap["gui.QOpenGLFunctions_2_0_CoreBackend_ITF"] = QOpenGLFunctions_2_0_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_2_0_DeprecatedBackend_ITF"] = QOpenGLFunctions_2_0_DeprecatedBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_2_1_ITF"] = QOpenGLFunctions_2_1{}
	qt.ItfMap["gui.QOpenGLFunctions_2_1_CoreBackend_ITF"] = QOpenGLFunctions_2_1_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_3_0_ITF"] = QOpenGLFunctions_3_0{}
	qt.ItfMap["gui.QOpenGLFunctions_3_0_CoreBackend_ITF"] = QOpenGLFunctions_3_0_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_3_0_DeprecatedBackend_ITF"] = QOpenGLFunctions_3_0_DeprecatedBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_3_1_ITF"] = QOpenGLFunctions_3_1{}
	qt.ItfMap["gui.QOpenGLFunctions_3_1_CoreBackend_ITF"] = QOpenGLFunctions_3_1_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_3_2_Compatibility_ITF"] = QOpenGLFunctions_3_2_Compatibility{}
	qt.ItfMap["gui.QOpenGLFunctions_3_2_Core_ITF"] = QOpenGLFunctions_3_2_Core{}
	qt.ItfMap["gui.QOpenGLFunctions_3_2_CoreBackend_ITF"] = QOpenGLFunctions_3_2_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_3_3_Compatibility_ITF"] = QOpenGLFunctions_3_3_Compatibility{}
	qt.ItfMap["gui.QOpenGLFunctions_3_3_Core_ITF"] = QOpenGLFunctions_3_3_Core{}
	qt.ItfMap["gui.QOpenGLFunctions_3_3_CoreBackend_ITF"] = QOpenGLFunctions_3_3_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_3_3_DeprecatedBackend_ITF"] = QOpenGLFunctions_3_3_DeprecatedBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_4_0_Compatibility_ITF"] = QOpenGLFunctions_4_0_Compatibility{}
	qt.ItfMap["gui.QOpenGLFunctions_4_0_Core_ITF"] = QOpenGLFunctions_4_0_Core{}
	qt.ItfMap["gui.QOpenGLFunctions_4_0_CoreBackend_ITF"] = QOpenGLFunctions_4_0_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_4_1_Compatibility_ITF"] = QOpenGLFunctions_4_1_Compatibility{}
	qt.ItfMap["gui.QOpenGLFunctions_4_1_Core_ITF"] = QOpenGLFunctions_4_1_Core{}
	qt.ItfMap["gui.QOpenGLFunctions_4_1_CoreBackend_ITF"] = QOpenGLFunctions_4_1_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_4_2_Compatibility_ITF"] = QOpenGLFunctions_4_2_Compatibility{}
	qt.ItfMap["gui.QOpenGLFunctions_4_2_Core_ITF"] = QOpenGLFunctions_4_2_Core{}
	qt.ItfMap["gui.QOpenGLFunctions_4_2_CoreBackend_ITF"] = QOpenGLFunctions_4_2_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_4_3_Compatibility_ITF"] = QOpenGLFunctions_4_3_Compatibility{}
	qt.ItfMap["gui.QOpenGLFunctions_4_3_Core_ITF"] = QOpenGLFunctions_4_3_Core{}
	qt.ItfMap["gui.QOpenGLFunctions_4_3_CoreBackend_ITF"] = QOpenGLFunctions_4_3_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_4_4_Compatibility_ITF"] = QOpenGLFunctions_4_4_Compatibility{}
	qt.ItfMap["gui.QOpenGLFunctions_4_4_Core_ITF"] = QOpenGLFunctions_4_4_Core{}
	qt.ItfMap["gui.QOpenGLFunctions_4_4_CoreBackend_ITF"] = QOpenGLFunctions_4_4_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_4_5_Compatibility_ITF"] = QOpenGLFunctions_4_5_Compatibility{}
	qt.ItfMap["gui.QOpenGLFunctions_4_5_Core_ITF"] = QOpenGLFunctions_4_5_Core{}
	qt.ItfMap["gui.QOpenGLFunctions_4_5_CoreBackend_ITF"] = QOpenGLFunctions_4_5_CoreBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_4_5_DeprecatedBackend_ITF"] = QOpenGLFunctions_4_5_DeprecatedBackend{}
	qt.ItfMap["gui.QOpenGLFunctions_ES2_ITF"] = QOpenGLFunctions_ES2{}
	qt.ItfMap["gui.QOpenGLPaintDevice_ITF"] = QOpenGLPaintDevice{}
	qt.FuncMap["gui.NewQOpenGLPaintDevice"] = NewQOpenGLPaintDevice
	qt.FuncMap["gui.NewQOpenGLPaintDevice2"] = NewQOpenGLPaintDevice2
	qt.FuncMap["gui.NewQOpenGLPaintDevice3"] = NewQOpenGLPaintDevice3
	qt.ItfMap["gui.QOpenGLPixelTransferOptions_ITF"] = QOpenGLPixelTransferOptions{}
	qt.FuncMap["gui.NewQOpenGLPixelTransferOptions"] = NewQOpenGLPixelTransferOptions
	qt.ItfMap["gui.QOpenGLShader_ITF"] = QOpenGLShader{}
	qt.FuncMap["gui.NewQOpenGLShader"] = NewQOpenGLShader
	qt.FuncMap["gui.QOpenGLShader_HasOpenGLShaders"] = QOpenGLShader_HasOpenGLShaders
	qt.EnumMap["gui.QOpenGLShader__Vertex"] = int64(QOpenGLShader__Vertex)
	qt.EnumMap["gui.QOpenGLShader__Fragment"] = int64(QOpenGLShader__Fragment)
	qt.EnumMap["gui.QOpenGLShader__Geometry"] = int64(QOpenGLShader__Geometry)
	qt.EnumMap["gui.QOpenGLShader__TessellationControl"] = int64(QOpenGLShader__TessellationControl)
	qt.EnumMap["gui.QOpenGLShader__TessellationEvaluation"] = int64(QOpenGLShader__TessellationEvaluation)
	qt.EnumMap["gui.QOpenGLShader__Compute"] = int64(QOpenGLShader__Compute)
	qt.ItfMap["gui.QOpenGLShaderProgram_ITF"] = QOpenGLShaderProgram{}
	qt.FuncMap["gui.NewQOpenGLShaderProgram"] = NewQOpenGLShaderProgram
	qt.FuncMap["gui.QOpenGLShaderProgram_HasOpenGLShaderPrograms"] = QOpenGLShaderProgram_HasOpenGLShaderPrograms
	qt.ItfMap["gui.QOpenGLTexture_ITF"] = QOpenGLTexture{}
	qt.FuncMap["gui.NewQOpenGLTexture"] = NewQOpenGLTexture
	qt.FuncMap["gui.NewQOpenGLTexture2"] = NewQOpenGLTexture2
	qt.FuncMap["gui.QOpenGLTexture_BoundTextureId"] = QOpenGLTexture_BoundTextureId
	qt.FuncMap["gui.QOpenGLTexture_BoundTextureId2"] = QOpenGLTexture_BoundTextureId2
	qt.FuncMap["gui.QOpenGLTexture_HasFeature"] = QOpenGLTexture_HasFeature
	qt.EnumMap["gui.QOpenGLTexture__Target1D"] = int64(QOpenGLTexture__Target1D)
	qt.EnumMap["gui.QOpenGLTexture__Target1DArray"] = int64(QOpenGLTexture__Target1DArray)
	qt.EnumMap["gui.QOpenGLTexture__Target2D"] = int64(QOpenGLTexture__Target2D)
	qt.EnumMap["gui.QOpenGLTexture__Target2DArray"] = int64(QOpenGLTexture__Target2DArray)
	qt.EnumMap["gui.QOpenGLTexture__Target3D"] = int64(QOpenGLTexture__Target3D)
	qt.EnumMap["gui.QOpenGLTexture__TargetCubeMap"] = int64(QOpenGLTexture__TargetCubeMap)
	qt.EnumMap["gui.QOpenGLTexture__TargetCubeMapArray"] = int64(QOpenGLTexture__TargetCubeMapArray)
	qt.EnumMap["gui.QOpenGLTexture__Target2DMultisample"] = int64(QOpenGLTexture__Target2DMultisample)
	qt.EnumMap["gui.QOpenGLTexture__Target2DMultisampleArray"] = int64(QOpenGLTexture__Target2DMultisampleArray)
	qt.EnumMap["gui.QOpenGLTexture__TargetRectangle"] = int64(QOpenGLTexture__TargetRectangle)
	qt.EnumMap["gui.QOpenGLTexture__TargetBuffer"] = int64(QOpenGLTexture__TargetBuffer)
	qt.EnumMap["gui.QOpenGLTexture__BindingTarget1D"] = int64(QOpenGLTexture__BindingTarget1D)
	qt.EnumMap["gui.QOpenGLTexture__BindingTarget1DArray"] = int64(QOpenGLTexture__BindingTarget1DArray)
	qt.EnumMap["gui.QOpenGLTexture__BindingTarget2D"] = int64(QOpenGLTexture__BindingTarget2D)
	qt.EnumMap["gui.QOpenGLTexture__BindingTarget2DArray"] = int64(QOpenGLTexture__BindingTarget2DArray)
	qt.EnumMap["gui.QOpenGLTexture__BindingTarget3D"] = int64(QOpenGLTexture__BindingTarget3D)
	qt.EnumMap["gui.QOpenGLTexture__BindingTargetCubeMap"] = int64(QOpenGLTexture__BindingTargetCubeMap)
	qt.EnumMap["gui.QOpenGLTexture__BindingTargetCubeMapArray"] = int64(QOpenGLTexture__BindingTargetCubeMapArray)
	qt.EnumMap["gui.QOpenGLTexture__BindingTarget2DMultisample"] = int64(QOpenGLTexture__BindingTarget2DMultisample)
	qt.EnumMap["gui.QOpenGLTexture__BindingTarget2DMultisampleArray"] = int64(QOpenGLTexture__BindingTarget2DMultisampleArray)
	qt.EnumMap["gui.QOpenGLTexture__BindingTargetRectangle"] = int64(QOpenGLTexture__BindingTargetRectangle)
	qt.EnumMap["gui.QOpenGLTexture__BindingTargetBuffer"] = int64(QOpenGLTexture__BindingTargetBuffer)
	qt.EnumMap["gui.QOpenGLTexture__GenerateMipMaps"] = int64(QOpenGLTexture__GenerateMipMaps)
	qt.EnumMap["gui.QOpenGLTexture__DontGenerateMipMaps"] = int64(QOpenGLTexture__DontGenerateMipMaps)
	qt.EnumMap["gui.QOpenGLTexture__ResetTextureUnit"] = int64(QOpenGLTexture__ResetTextureUnit)
	qt.EnumMap["gui.QOpenGLTexture__DontResetTextureUnit"] = int64(QOpenGLTexture__DontResetTextureUnit)
	qt.EnumMap["gui.QOpenGLTexture__NoFormat"] = int64(QOpenGLTexture__NoFormat)
	qt.EnumMap["gui.QOpenGLTexture__R8_UNorm"] = int64(QOpenGLTexture__R8_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__RG8_UNorm"] = int64(QOpenGLTexture__RG8_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__RGB8_UNorm"] = int64(QOpenGLTexture__RGB8_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__RGBA8_UNorm"] = int64(QOpenGLTexture__RGBA8_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__R16_UNorm"] = int64(QOpenGLTexture__R16_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__RG16_UNorm"] = int64(QOpenGLTexture__RG16_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__RGB16_UNorm"] = int64(QOpenGLTexture__RGB16_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__RGBA16_UNorm"] = int64(QOpenGLTexture__RGBA16_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__R8_SNorm"] = int64(QOpenGLTexture__R8_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__RG8_SNorm"] = int64(QOpenGLTexture__RG8_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__RGB8_SNorm"] = int64(QOpenGLTexture__RGB8_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__RGBA8_SNorm"] = int64(QOpenGLTexture__RGBA8_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__R16_SNorm"] = int64(QOpenGLTexture__R16_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__RG16_SNorm"] = int64(QOpenGLTexture__RG16_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__RGB16_SNorm"] = int64(QOpenGLTexture__RGB16_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__RGBA16_SNorm"] = int64(QOpenGLTexture__RGBA16_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__R8U"] = int64(QOpenGLTexture__R8U)
	qt.EnumMap["gui.QOpenGLTexture__RG8U"] = int64(QOpenGLTexture__RG8U)
	qt.EnumMap["gui.QOpenGLTexture__RGB8U"] = int64(QOpenGLTexture__RGB8U)
	qt.EnumMap["gui.QOpenGLTexture__RGBA8U"] = int64(QOpenGLTexture__RGBA8U)
	qt.EnumMap["gui.QOpenGLTexture__R16U"] = int64(QOpenGLTexture__R16U)
	qt.EnumMap["gui.QOpenGLTexture__RG16U"] = int64(QOpenGLTexture__RG16U)
	qt.EnumMap["gui.QOpenGLTexture__RGB16U"] = int64(QOpenGLTexture__RGB16U)
	qt.EnumMap["gui.QOpenGLTexture__RGBA16U"] = int64(QOpenGLTexture__RGBA16U)
	qt.EnumMap["gui.QOpenGLTexture__R32U"] = int64(QOpenGLTexture__R32U)
	qt.EnumMap["gui.QOpenGLTexture__RG32U"] = int64(QOpenGLTexture__RG32U)
	qt.EnumMap["gui.QOpenGLTexture__RGB32U"] = int64(QOpenGLTexture__RGB32U)
	qt.EnumMap["gui.QOpenGLTexture__RGBA32U"] = int64(QOpenGLTexture__RGBA32U)
	qt.EnumMap["gui.QOpenGLTexture__R8I"] = int64(QOpenGLTexture__R8I)
	qt.EnumMap["gui.QOpenGLTexture__RG8I"] = int64(QOpenGLTexture__RG8I)
	qt.EnumMap["gui.QOpenGLTexture__RGB8I"] = int64(QOpenGLTexture__RGB8I)
	qt.EnumMap["gui.QOpenGLTexture__RGBA8I"] = int64(QOpenGLTexture__RGBA8I)
	qt.EnumMap["gui.QOpenGLTexture__R16I"] = int64(QOpenGLTexture__R16I)
	qt.EnumMap["gui.QOpenGLTexture__RG16I"] = int64(QOpenGLTexture__RG16I)
	qt.EnumMap["gui.QOpenGLTexture__RGB16I"] = int64(QOpenGLTexture__RGB16I)
	qt.EnumMap["gui.QOpenGLTexture__RGBA16I"] = int64(QOpenGLTexture__RGBA16I)
	qt.EnumMap["gui.QOpenGLTexture__R32I"] = int64(QOpenGLTexture__R32I)
	qt.EnumMap["gui.QOpenGLTexture__RG32I"] = int64(QOpenGLTexture__RG32I)
	qt.EnumMap["gui.QOpenGLTexture__RGB32I"] = int64(QOpenGLTexture__RGB32I)
	qt.EnumMap["gui.QOpenGLTexture__RGBA32I"] = int64(QOpenGLTexture__RGBA32I)
	qt.EnumMap["gui.QOpenGLTexture__R16F"] = int64(QOpenGLTexture__R16F)
	qt.EnumMap["gui.QOpenGLTexture__RG16F"] = int64(QOpenGLTexture__RG16F)
	qt.EnumMap["gui.QOpenGLTexture__RGB16F"] = int64(QOpenGLTexture__RGB16F)
	qt.EnumMap["gui.QOpenGLTexture__RGBA16F"] = int64(QOpenGLTexture__RGBA16F)
	qt.EnumMap["gui.QOpenGLTexture__R32F"] = int64(QOpenGLTexture__R32F)
	qt.EnumMap["gui.QOpenGLTexture__RG32F"] = int64(QOpenGLTexture__RG32F)
	qt.EnumMap["gui.QOpenGLTexture__RGB32F"] = int64(QOpenGLTexture__RGB32F)
	qt.EnumMap["gui.QOpenGLTexture__RGBA32F"] = int64(QOpenGLTexture__RGBA32F)
	qt.EnumMap["gui.QOpenGLTexture__RGB9E5"] = int64(QOpenGLTexture__RGB9E5)
	qt.EnumMap["gui.QOpenGLTexture__RG11B10F"] = int64(QOpenGLTexture__RG11B10F)
	qt.EnumMap["gui.QOpenGLTexture__RG3B2"] = int64(QOpenGLTexture__RG3B2)
	qt.EnumMap["gui.QOpenGLTexture__R5G6B5"] = int64(QOpenGLTexture__R5G6B5)
	qt.EnumMap["gui.QOpenGLTexture__RGB5A1"] = int64(QOpenGLTexture__RGB5A1)
	qt.EnumMap["gui.QOpenGLTexture__RGBA4"] = int64(QOpenGLTexture__RGBA4)
	qt.EnumMap["gui.QOpenGLTexture__RGB10A2"] = int64(QOpenGLTexture__RGB10A2)
	qt.EnumMap["gui.QOpenGLTexture__D16"] = int64(QOpenGLTexture__D16)
	qt.EnumMap["gui.QOpenGLTexture__D24"] = int64(QOpenGLTexture__D24)
	qt.EnumMap["gui.QOpenGLTexture__D24S8"] = int64(QOpenGLTexture__D24S8)
	qt.EnumMap["gui.QOpenGLTexture__D32"] = int64(QOpenGLTexture__D32)
	qt.EnumMap["gui.QOpenGLTexture__D32F"] = int64(QOpenGLTexture__D32F)
	qt.EnumMap["gui.QOpenGLTexture__D32FS8X24"] = int64(QOpenGLTexture__D32FS8X24)
	qt.EnumMap["gui.QOpenGLTexture__S8"] = int64(QOpenGLTexture__S8)
	qt.EnumMap["gui.QOpenGLTexture__RGB_DXT1"] = int64(QOpenGLTexture__RGB_DXT1)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_DXT1"] = int64(QOpenGLTexture__RGBA_DXT1)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_DXT3"] = int64(QOpenGLTexture__RGBA_DXT3)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_DXT5"] = int64(QOpenGLTexture__RGBA_DXT5)
	qt.EnumMap["gui.QOpenGLTexture__R_ATI1N_UNorm"] = int64(QOpenGLTexture__R_ATI1N_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__R_ATI1N_SNorm"] = int64(QOpenGLTexture__R_ATI1N_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__RG_ATI2N_UNorm"] = int64(QOpenGLTexture__RG_ATI2N_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__RG_ATI2N_SNorm"] = int64(QOpenGLTexture__RG_ATI2N_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__RGB_BP_UNSIGNED_FLOAT"] = int64(QOpenGLTexture__RGB_BP_UNSIGNED_FLOAT)
	qt.EnumMap["gui.QOpenGLTexture__RGB_BP_SIGNED_FLOAT"] = int64(QOpenGLTexture__RGB_BP_SIGNED_FLOAT)
	qt.EnumMap["gui.QOpenGLTexture__RGB_BP_UNorm"] = int64(QOpenGLTexture__RGB_BP_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__R11_EAC_UNorm"] = int64(QOpenGLTexture__R11_EAC_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__R11_EAC_SNorm"] = int64(QOpenGLTexture__R11_EAC_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__RG11_EAC_UNorm"] = int64(QOpenGLTexture__RG11_EAC_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__RG11_EAC_SNorm"] = int64(QOpenGLTexture__RG11_EAC_SNorm)
	qt.EnumMap["gui.QOpenGLTexture__RGB8_ETC2"] = int64(QOpenGLTexture__RGB8_ETC2)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_ETC2"] = int64(QOpenGLTexture__SRGB8_ETC2)
	qt.EnumMap["gui.QOpenGLTexture__RGB8_PunchThrough_Alpha1_ETC2"] = int64(QOpenGLTexture__RGB8_PunchThrough_Alpha1_ETC2)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_PunchThrough_Alpha1_ETC2"] = int64(QOpenGLTexture__SRGB8_PunchThrough_Alpha1_ETC2)
	qt.EnumMap["gui.QOpenGLTexture__RGBA8_ETC2_EAC"] = int64(QOpenGLTexture__RGBA8_ETC2_EAC)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ETC2_EAC"] = int64(QOpenGLTexture__SRGB8_Alpha8_ETC2_EAC)
	qt.EnumMap["gui.QOpenGLTexture__RGB8_ETC1"] = int64(QOpenGLTexture__RGB8_ETC1)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_4x4"] = int64(QOpenGLTexture__RGBA_ASTC_4x4)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_5x4"] = int64(QOpenGLTexture__RGBA_ASTC_5x4)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_5x5"] = int64(QOpenGLTexture__RGBA_ASTC_5x5)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_6x5"] = int64(QOpenGLTexture__RGBA_ASTC_6x5)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_6x6"] = int64(QOpenGLTexture__RGBA_ASTC_6x6)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_8x5"] = int64(QOpenGLTexture__RGBA_ASTC_8x5)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_8x6"] = int64(QOpenGLTexture__RGBA_ASTC_8x6)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_8x8"] = int64(QOpenGLTexture__RGBA_ASTC_8x8)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_10x5"] = int64(QOpenGLTexture__RGBA_ASTC_10x5)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_10x6"] = int64(QOpenGLTexture__RGBA_ASTC_10x6)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_10x8"] = int64(QOpenGLTexture__RGBA_ASTC_10x8)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_10x10"] = int64(QOpenGLTexture__RGBA_ASTC_10x10)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_12x10"] = int64(QOpenGLTexture__RGBA_ASTC_12x10)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_ASTC_12x12"] = int64(QOpenGLTexture__RGBA_ASTC_12x12)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_4x4"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_4x4)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_5x4"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_5x4)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_5x5"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_5x5)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_6x5"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_6x5)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_6x6"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_6x6)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_8x5"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_8x5)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_8x6"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_8x6)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_8x8"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_8x8)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_10x5"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_10x5)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_10x6"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_10x6)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_10x8"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_10x8)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_10x10"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_10x10)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_12x10"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_12x10)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8_ASTC_12x12"] = int64(QOpenGLTexture__SRGB8_Alpha8_ASTC_12x12)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8"] = int64(QOpenGLTexture__SRGB8)
	qt.EnumMap["gui.QOpenGLTexture__SRGB8_Alpha8"] = int64(QOpenGLTexture__SRGB8_Alpha8)
	qt.EnumMap["gui.QOpenGLTexture__SRGB_DXT1"] = int64(QOpenGLTexture__SRGB_DXT1)
	qt.EnumMap["gui.QOpenGLTexture__SRGB_Alpha_DXT1"] = int64(QOpenGLTexture__SRGB_Alpha_DXT1)
	qt.EnumMap["gui.QOpenGLTexture__SRGB_Alpha_DXT3"] = int64(QOpenGLTexture__SRGB_Alpha_DXT3)
	qt.EnumMap["gui.QOpenGLTexture__SRGB_Alpha_DXT5"] = int64(QOpenGLTexture__SRGB_Alpha_DXT5)
	qt.EnumMap["gui.QOpenGLTexture__SRGB_BP_UNorm"] = int64(QOpenGLTexture__SRGB_BP_UNorm)
	qt.EnumMap["gui.QOpenGLTexture__DepthFormat"] = int64(QOpenGLTexture__DepthFormat)
	qt.EnumMap["gui.QOpenGLTexture__AlphaFormat"] = int64(QOpenGLTexture__AlphaFormat)
	qt.EnumMap["gui.QOpenGLTexture__RGBFormat"] = int64(QOpenGLTexture__RGBFormat)
	qt.EnumMap["gui.QOpenGLTexture__RGBAFormat"] = int64(QOpenGLTexture__RGBAFormat)
	qt.EnumMap["gui.QOpenGLTexture__LuminanceFormat"] = int64(QOpenGLTexture__LuminanceFormat)
	qt.EnumMap["gui.QOpenGLTexture__LuminanceAlphaFormat"] = int64(QOpenGLTexture__LuminanceAlphaFormat)
	qt.EnumMap["gui.QOpenGLTexture__CubeMapPositiveX"] = int64(QOpenGLTexture__CubeMapPositiveX)
	qt.EnumMap["gui.QOpenGLTexture__CubeMapNegativeX"] = int64(QOpenGLTexture__CubeMapNegativeX)
	qt.EnumMap["gui.QOpenGLTexture__CubeMapPositiveY"] = int64(QOpenGLTexture__CubeMapPositiveY)
	qt.EnumMap["gui.QOpenGLTexture__CubeMapNegativeY"] = int64(QOpenGLTexture__CubeMapNegativeY)
	qt.EnumMap["gui.QOpenGLTexture__CubeMapPositiveZ"] = int64(QOpenGLTexture__CubeMapPositiveZ)
	qt.EnumMap["gui.QOpenGLTexture__CubeMapNegativeZ"] = int64(QOpenGLTexture__CubeMapNegativeZ)
	qt.EnumMap["gui.QOpenGLTexture__NoSourceFormat"] = int64(QOpenGLTexture__NoSourceFormat)
	qt.EnumMap["gui.QOpenGLTexture__Red"] = int64(QOpenGLTexture__Red)
	qt.EnumMap["gui.QOpenGLTexture__RG"] = int64(QOpenGLTexture__RG)
	qt.EnumMap["gui.QOpenGLTexture__RGB"] = int64(QOpenGLTexture__RGB)
	qt.EnumMap["gui.QOpenGLTexture__BGR"] = int64(QOpenGLTexture__BGR)
	qt.EnumMap["gui.QOpenGLTexture__RGBA"] = int64(QOpenGLTexture__RGBA)
	qt.EnumMap["gui.QOpenGLTexture__BGRA"] = int64(QOpenGLTexture__BGRA)
	qt.EnumMap["gui.QOpenGLTexture__Red_Integer"] = int64(QOpenGLTexture__Red_Integer)
	qt.EnumMap["gui.QOpenGLTexture__RG_Integer"] = int64(QOpenGLTexture__RG_Integer)
	qt.EnumMap["gui.QOpenGLTexture__RGB_Integer"] = int64(QOpenGLTexture__RGB_Integer)
	qt.EnumMap["gui.QOpenGLTexture__BGR_Integer"] = int64(QOpenGLTexture__BGR_Integer)
	qt.EnumMap["gui.QOpenGLTexture__RGBA_Integer"] = int64(QOpenGLTexture__RGBA_Integer)
	qt.EnumMap["gui.QOpenGLTexture__BGRA_Integer"] = int64(QOpenGLTexture__BGRA_Integer)
	qt.EnumMap["gui.QOpenGLTexture__Stencil"] = int64(QOpenGLTexture__Stencil)
	qt.EnumMap["gui.QOpenGLTexture__Depth"] = int64(QOpenGLTexture__Depth)
	qt.EnumMap["gui.QOpenGLTexture__DepthStencil"] = int64(QOpenGLTexture__DepthStencil)
	qt.EnumMap["gui.QOpenGLTexture__Alpha"] = int64(QOpenGLTexture__Alpha)
	qt.EnumMap["gui.QOpenGLTexture__Luminance"] = int64(QOpenGLTexture__Luminance)
	qt.EnumMap["gui.QOpenGLTexture__LuminanceAlpha"] = int64(QOpenGLTexture__LuminanceAlpha)
	qt.EnumMap["gui.QOpenGLTexture__NoPixelType"] = int64(QOpenGLTexture__NoPixelType)
	qt.EnumMap["gui.QOpenGLTexture__Int8"] = int64(QOpenGLTexture__Int8)
	qt.EnumMap["gui.QOpenGLTexture__UInt8"] = int64(QOpenGLTexture__UInt8)
	qt.EnumMap["gui.QOpenGLTexture__Int16"] = int64(QOpenGLTexture__Int16)
	qt.EnumMap["gui.QOpenGLTexture__UInt16"] = int64(QOpenGLTexture__UInt16)
	qt.EnumMap["gui.QOpenGLTexture__Int32"] = int64(QOpenGLTexture__Int32)
	qt.EnumMap["gui.QOpenGLTexture__UInt32"] = int64(QOpenGLTexture__UInt32)
	qt.EnumMap["gui.QOpenGLTexture__Float16"] = int64(QOpenGLTexture__Float16)
	qt.EnumMap["gui.QOpenGLTexture__Float16OES"] = int64(QOpenGLTexture__Float16OES)
	qt.EnumMap["gui.QOpenGLTexture__Float32"] = int64(QOpenGLTexture__Float32)
	qt.EnumMap["gui.QOpenGLTexture__UInt32_RGB9_E5"] = int64(QOpenGLTexture__UInt32_RGB9_E5)
	qt.EnumMap["gui.QOpenGLTexture__UInt32_RG11B10F"] = int64(QOpenGLTexture__UInt32_RG11B10F)
	qt.EnumMap["gui.QOpenGLTexture__UInt8_RG3B2"] = int64(QOpenGLTexture__UInt8_RG3B2)
	qt.EnumMap["gui.QOpenGLTexture__UInt8_RG3B2_Rev"] = int64(QOpenGLTexture__UInt8_RG3B2_Rev)
	qt.EnumMap["gui.QOpenGLTexture__UInt16_RGB5A1"] = int64(QOpenGLTexture__UInt16_RGB5A1)
	qt.EnumMap["gui.QOpenGLTexture__UInt16_RGB5A1_Rev"] = int64(QOpenGLTexture__UInt16_RGB5A1_Rev)
	qt.EnumMap["gui.QOpenGLTexture__UInt16_R5G6B5"] = int64(QOpenGLTexture__UInt16_R5G6B5)
	qt.EnumMap["gui.QOpenGLTexture__UInt16_R5G6B5_Rev"] = int64(QOpenGLTexture__UInt16_R5G6B5_Rev)
	qt.EnumMap["gui.QOpenGLTexture__UInt16_RGBA4"] = int64(QOpenGLTexture__UInt16_RGBA4)
	qt.EnumMap["gui.QOpenGLTexture__UInt16_RGBA4_Rev"] = int64(QOpenGLTexture__UInt16_RGBA4_Rev)
	qt.EnumMap["gui.QOpenGLTexture__UInt32_RGBA8"] = int64(QOpenGLTexture__UInt32_RGBA8)
	qt.EnumMap["gui.QOpenGLTexture__UInt32_RGBA8_Rev"] = int64(QOpenGLTexture__UInt32_RGBA8_Rev)
	qt.EnumMap["gui.QOpenGLTexture__UInt32_RGB10A2"] = int64(QOpenGLTexture__UInt32_RGB10A2)
	qt.EnumMap["gui.QOpenGLTexture__UInt32_RGB10A2_Rev"] = int64(QOpenGLTexture__UInt32_RGB10A2_Rev)
	qt.EnumMap["gui.QOpenGLTexture__UInt32_D24S8"] = int64(QOpenGLTexture__UInt32_D24S8)
	qt.EnumMap["gui.QOpenGLTexture__Float32_D32_UInt32_S8_X24"] = int64(QOpenGLTexture__Float32_D32_UInt32_S8_X24)
	qt.EnumMap["gui.QOpenGLTexture__SwizzleRed"] = int64(QOpenGLTexture__SwizzleRed)
	qt.EnumMap["gui.QOpenGLTexture__SwizzleGreen"] = int64(QOpenGLTexture__SwizzleGreen)
	qt.EnumMap["gui.QOpenGLTexture__SwizzleBlue"] = int64(QOpenGLTexture__SwizzleBlue)
	qt.EnumMap["gui.QOpenGLTexture__SwizzleAlpha"] = int64(QOpenGLTexture__SwizzleAlpha)
	qt.EnumMap["gui.QOpenGLTexture__RedValue"] = int64(QOpenGLTexture__RedValue)
	qt.EnumMap["gui.QOpenGLTexture__GreenValue"] = int64(QOpenGLTexture__GreenValue)
	qt.EnumMap["gui.QOpenGLTexture__BlueValue"] = int64(QOpenGLTexture__BlueValue)
	qt.EnumMap["gui.QOpenGLTexture__AlphaValue"] = int64(QOpenGLTexture__AlphaValue)
	qt.EnumMap["gui.QOpenGLTexture__ZeroValue"] = int64(QOpenGLTexture__ZeroValue)
	qt.EnumMap["gui.QOpenGLTexture__OneValue"] = int64(QOpenGLTexture__OneValue)
	qt.EnumMap["gui.QOpenGLTexture__Repeat"] = int64(QOpenGLTexture__Repeat)
	qt.EnumMap["gui.QOpenGLTexture__MirroredRepeat"] = int64(QOpenGLTexture__MirroredRepeat)
	qt.EnumMap["gui.QOpenGLTexture__ClampToEdge"] = int64(QOpenGLTexture__ClampToEdge)
	qt.EnumMap["gui.QOpenGLTexture__ClampToBorder"] = int64(QOpenGLTexture__ClampToBorder)
	qt.EnumMap["gui.QOpenGLTexture__DirectionS"] = int64(QOpenGLTexture__DirectionS)
	qt.EnumMap["gui.QOpenGLTexture__DirectionT"] = int64(QOpenGLTexture__DirectionT)
	qt.EnumMap["gui.QOpenGLTexture__DirectionR"] = int64(QOpenGLTexture__DirectionR)
	qt.EnumMap["gui.QOpenGLTexture__ImmutableStorage"] = int64(QOpenGLTexture__ImmutableStorage)
	qt.EnumMap["gui.QOpenGLTexture__ImmutableMultisampleStorage"] = int64(QOpenGLTexture__ImmutableMultisampleStorage)
	qt.EnumMap["gui.QOpenGLTexture__TextureRectangle"] = int64(QOpenGLTexture__TextureRectangle)
	qt.EnumMap["gui.QOpenGLTexture__TextureArrays"] = int64(QOpenGLTexture__TextureArrays)
	qt.EnumMap["gui.QOpenGLTexture__Texture3D"] = int64(QOpenGLTexture__Texture3D)
	qt.EnumMap["gui.QOpenGLTexture__TextureMultisample"] = int64(QOpenGLTexture__TextureMultisample)
	qt.EnumMap["gui.QOpenGLTexture__TextureBuffer"] = int64(QOpenGLTexture__TextureBuffer)
	qt.EnumMap["gui.QOpenGLTexture__TextureCubeMapArrays"] = int64(QOpenGLTexture__TextureCubeMapArrays)
	qt.EnumMap["gui.QOpenGLTexture__Swizzle"] = int64(QOpenGLTexture__Swizzle)
	qt.EnumMap["gui.QOpenGLTexture__StencilTexturing"] = int64(QOpenGLTexture__StencilTexturing)
	qt.EnumMap["gui.QOpenGLTexture__AnisotropicFiltering"] = int64(QOpenGLTexture__AnisotropicFiltering)
	qt.EnumMap["gui.QOpenGLTexture__NPOTTextures"] = int64(QOpenGLTexture__NPOTTextures)
	qt.EnumMap["gui.QOpenGLTexture__NPOTTextureRepeat"] = int64(QOpenGLTexture__NPOTTextureRepeat)
	qt.EnumMap["gui.QOpenGLTexture__Texture1D"] = int64(QOpenGLTexture__Texture1D)
	qt.EnumMap["gui.QOpenGLTexture__TextureComparisonOperators"] = int64(QOpenGLTexture__TextureComparisonOperators)
	qt.EnumMap["gui.QOpenGLTexture__TextureMipMapLevel"] = int64(QOpenGLTexture__TextureMipMapLevel)
	qt.EnumMap["gui.QOpenGLTexture__DepthMode"] = int64(QOpenGLTexture__DepthMode)
	qt.EnumMap["gui.QOpenGLTexture__StencilMode"] = int64(QOpenGLTexture__StencilMode)
	qt.EnumMap["gui.QOpenGLTexture__CompareLessEqual"] = int64(QOpenGLTexture__CompareLessEqual)
	qt.EnumMap["gui.QOpenGLTexture__CompareGreaterEqual"] = int64(QOpenGLTexture__CompareGreaterEqual)
	qt.EnumMap["gui.QOpenGLTexture__CompareLess"] = int64(QOpenGLTexture__CompareLess)
	qt.EnumMap["gui.QOpenGLTexture__CompareGreater"] = int64(QOpenGLTexture__CompareGreater)
	qt.EnumMap["gui.QOpenGLTexture__CompareEqual"] = int64(QOpenGLTexture__CompareEqual)
	qt.EnumMap["gui.QOpenGLTexture__CommpareNotEqual"] = int64(QOpenGLTexture__CommpareNotEqual)
	qt.EnumMap["gui.QOpenGLTexture__CompareAlways"] = int64(QOpenGLTexture__CompareAlways)
	qt.EnumMap["gui.QOpenGLTexture__CompareNever"] = int64(QOpenGLTexture__CompareNever)
	qt.EnumMap["gui.QOpenGLTexture__CompareRefToTexture"] = int64(QOpenGLTexture__CompareRefToTexture)
	qt.EnumMap["gui.QOpenGLTexture__CompareNone"] = int64(QOpenGLTexture__CompareNone)
	qt.EnumMap["gui.QOpenGLTexture__Nearest"] = int64(QOpenGLTexture__Nearest)
	qt.EnumMap["gui.QOpenGLTexture__Linear"] = int64(QOpenGLTexture__Linear)
	qt.EnumMap["gui.QOpenGLTexture__NearestMipMapNearest"] = int64(QOpenGLTexture__NearestMipMapNearest)
	qt.EnumMap["gui.QOpenGLTexture__NearestMipMapLinear"] = int64(QOpenGLTexture__NearestMipMapLinear)
	qt.EnumMap["gui.QOpenGLTexture__LinearMipMapNearest"] = int64(QOpenGLTexture__LinearMipMapNearest)
	qt.EnumMap["gui.QOpenGLTexture__LinearMipMapLinear"] = int64(QOpenGLTexture__LinearMipMapLinear)
	qt.ItfMap["gui.QOpenGLTextureBlitter_ITF"] = QOpenGLTextureBlitter{}
	qt.FuncMap["gui.NewQOpenGLTextureBlitter"] = NewQOpenGLTextureBlitter
	qt.FuncMap["gui.QOpenGLTextureBlitter_TargetTransform"] = QOpenGLTextureBlitter_TargetTransform
	qt.EnumMap["gui.QOpenGLTextureBlitter__OriginBottomLeft"] = int64(QOpenGLTextureBlitter__OriginBottomLeft)
	qt.EnumMap["gui.QOpenGLTextureBlitter__OriginTopLeft"] = int64(QOpenGLTextureBlitter__OriginTopLeft)
	qt.ItfMap["gui.QOpenGLTimeMonitor_ITF"] = QOpenGLTimeMonitor{}
	qt.FuncMap["gui.NewQOpenGLTimeMonitor"] = NewQOpenGLTimeMonitor
	qt.ItfMap["gui.QOpenGLTimerQuery_ITF"] = QOpenGLTimerQuery{}
	qt.FuncMap["gui.NewQOpenGLTimerQuery"] = NewQOpenGLTimerQuery
	qt.ItfMap["gui.QOpenGLVersionFunctionsBackend_ITF"] = QOpenGLVersionFunctionsBackend{}
	qt.ItfMap["gui.QOpenGLVersionFunctionsStorage_ITF"] = QOpenGLVersionFunctionsStorage{}
	qt.ItfMap["gui.QOpenGLVersionProfile_ITF"] = QOpenGLVersionProfile{}
	qt.FuncMap["gui.NewQOpenGLVersionProfile"] = NewQOpenGLVersionProfile
	qt.FuncMap["gui.NewQOpenGLVersionProfile2"] = NewQOpenGLVersionProfile2
	qt.FuncMap["gui.NewQOpenGLVersionProfile3"] = NewQOpenGLVersionProfile3
	qt.ItfMap["gui.QOpenGLVersionStatus_ITF"] = QOpenGLVersionStatus{}
	qt.ItfMap["gui.QOpenGLVertexArrayObject_ITF"] = QOpenGLVertexArrayObject{}
	qt.FuncMap["gui.NewQOpenGLVertexArrayObject"] = NewQOpenGLVertexArrayObject
	qt.ItfMap["gui.QOpenGLWindow_ITF"] = QOpenGLWindow{}
	qt.FuncMap["gui.NewQOpenGLWindow"] = NewQOpenGLWindow
	qt.FuncMap["gui.NewQOpenGLWindow2"] = NewQOpenGLWindow2
	qt.EnumMap["gui.QOpenGLWindow__NoPartialUpdate"] = int64(QOpenGLWindow__NoPartialUpdate)
	qt.EnumMap["gui.QOpenGLWindow__PartialUpdateBlit"] = int64(QOpenGLWindow__PartialUpdateBlit)
	qt.EnumMap["gui.QOpenGLWindow__PartialUpdateBlend"] = int64(QOpenGLWindow__PartialUpdateBlend)
	qt.ItfMap["gui.QPageLayout_ITF"] = QPageLayout{}
	qt.FuncMap["gui.NewQPageLayout"] = NewQPageLayout
	qt.FuncMap["gui.NewQPageLayout2"] = NewQPageLayout2
	qt.FuncMap["gui.NewQPageLayout3"] = NewQPageLayout3
	qt.EnumMap["gui.QPageLayout__Millimeter"] = int64(QPageLayout__Millimeter)
	qt.EnumMap["gui.QPageLayout__Point"] = int64(QPageLayout__Point)
	qt.EnumMap["gui.QPageLayout__Inch"] = int64(QPageLayout__Inch)
	qt.EnumMap["gui.QPageLayout__Pica"] = int64(QPageLayout__Pica)
	qt.EnumMap["gui.QPageLayout__Didot"] = int64(QPageLayout__Didot)
	qt.EnumMap["gui.QPageLayout__Cicero"] = int64(QPageLayout__Cicero)
	qt.EnumMap["gui.QPageLayout__Portrait"] = int64(QPageLayout__Portrait)
	qt.EnumMap["gui.QPageLayout__Landscape"] = int64(QPageLayout__Landscape)
	qt.EnumMap["gui.QPageLayout__StandardMode"] = int64(QPageLayout__StandardMode)
	qt.EnumMap["gui.QPageLayout__FullPageMode"] = int64(QPageLayout__FullPageMode)
	qt.ItfMap["gui.QPageSize_ITF"] = QPageSize{}
	qt.FuncMap["gui.NewQPageSize"] = NewQPageSize
	qt.FuncMap["gui.NewQPageSize2"] = NewQPageSize2
	qt.FuncMap["gui.NewQPageSize3"] = NewQPageSize3
	qt.FuncMap["gui.NewQPageSize4"] = NewQPageSize4
	qt.FuncMap["gui.NewQPageSize5"] = NewQPageSize5
	qt.FuncMap["gui.QPageSize_DefinitionSize2"] = QPageSize_DefinitionSize2
	qt.FuncMap["gui.QPageSize_DefinitionUnits2"] = QPageSize_DefinitionUnits2
	qt.FuncMap["gui.QPageSize_Id2"] = QPageSize_Id2
	qt.FuncMap["gui.QPageSize_Id3"] = QPageSize_Id3
	qt.FuncMap["gui.QPageSize_Id4"] = QPageSize_Id4
	qt.FuncMap["gui.QPageSize_Key2"] = QPageSize_Key2
	qt.FuncMap["gui.QPageSize_Name2"] = QPageSize_Name2
	qt.FuncMap["gui.QPageSize_Size2"] = QPageSize_Size2
	qt.FuncMap["gui.QPageSize_SizePixels2"] = QPageSize_SizePixels2
	qt.FuncMap["gui.QPageSize_SizePoints2"] = QPageSize_SizePoints2
	qt.FuncMap["gui.QPageSize_WindowsId2"] = QPageSize_WindowsId2
	qt.EnumMap["gui.QPageSize__A4"] = int64(QPageSize__A4)
	qt.EnumMap["gui.QPageSize__B5"] = int64(QPageSize__B5)
	qt.EnumMap["gui.QPageSize__Letter"] = int64(QPageSize__Letter)
	qt.EnumMap["gui.QPageSize__Legal"] = int64(QPageSize__Legal)
	qt.EnumMap["gui.QPageSize__Executive"] = int64(QPageSize__Executive)
	qt.EnumMap["gui.QPageSize__A0"] = int64(QPageSize__A0)
	qt.EnumMap["gui.QPageSize__A1"] = int64(QPageSize__A1)
	qt.EnumMap["gui.QPageSize__A2"] = int64(QPageSize__A2)
	qt.EnumMap["gui.QPageSize__A3"] = int64(QPageSize__A3)
	qt.EnumMap["gui.QPageSize__A5"] = int64(QPageSize__A5)
	qt.EnumMap["gui.QPageSize__A6"] = int64(QPageSize__A6)
	qt.EnumMap["gui.QPageSize__A7"] = int64(QPageSize__A7)
	qt.EnumMap["gui.QPageSize__A8"] = int64(QPageSize__A8)
	qt.EnumMap["gui.QPageSize__A9"] = int64(QPageSize__A9)
	qt.EnumMap["gui.QPageSize__B0"] = int64(QPageSize__B0)
	qt.EnumMap["gui.QPageSize__B1"] = int64(QPageSize__B1)
	qt.EnumMap["gui.QPageSize__B10"] = int64(QPageSize__B10)
	qt.EnumMap["gui.QPageSize__B2"] = int64(QPageSize__B2)
	qt.EnumMap["gui.QPageSize__B3"] = int64(QPageSize__B3)
	qt.EnumMap["gui.QPageSize__B4"] = int64(QPageSize__B4)
	qt.EnumMap["gui.QPageSize__B6"] = int64(QPageSize__B6)
	qt.EnumMap["gui.QPageSize__B7"] = int64(QPageSize__B7)
	qt.EnumMap["gui.QPageSize__B8"] = int64(QPageSize__B8)
	qt.EnumMap["gui.QPageSize__B9"] = int64(QPageSize__B9)
	qt.EnumMap["gui.QPageSize__C5E"] = int64(QPageSize__C5E)
	qt.EnumMap["gui.QPageSize__Comm10E"] = int64(QPageSize__Comm10E)
	qt.EnumMap["gui.QPageSize__DLE"] = int64(QPageSize__DLE)
	qt.EnumMap["gui.QPageSize__Folio"] = int64(QPageSize__Folio)
	qt.EnumMap["gui.QPageSize__Ledger"] = int64(QPageSize__Ledger)
	qt.EnumMap["gui.QPageSize__Tabloid"] = int64(QPageSize__Tabloid)
	qt.EnumMap["gui.QPageSize__Custom"] = int64(QPageSize__Custom)
	qt.EnumMap["gui.QPageSize__A10"] = int64(QPageSize__A10)
	qt.EnumMap["gui.QPageSize__A3Extra"] = int64(QPageSize__A3Extra)
	qt.EnumMap["gui.QPageSize__A4Extra"] = int64(QPageSize__A4Extra)
	qt.EnumMap["gui.QPageSize__A4Plus"] = int64(QPageSize__A4Plus)
	qt.EnumMap["gui.QPageSize__A4Small"] = int64(QPageSize__A4Small)
	qt.EnumMap["gui.QPageSize__A5Extra"] = int64(QPageSize__A5Extra)
	qt.EnumMap["gui.QPageSize__B5Extra"] = int64(QPageSize__B5Extra)
	qt.EnumMap["gui.QPageSize__JisB0"] = int64(QPageSize__JisB0)
	qt.EnumMap["gui.QPageSize__JisB1"] = int64(QPageSize__JisB1)
	qt.EnumMap["gui.QPageSize__JisB2"] = int64(QPageSize__JisB2)
	qt.EnumMap["gui.QPageSize__JisB3"] = int64(QPageSize__JisB3)
	qt.EnumMap["gui.QPageSize__JisB4"] = int64(QPageSize__JisB4)
	qt.EnumMap["gui.QPageSize__JisB5"] = int64(QPageSize__JisB5)
	qt.EnumMap["gui.QPageSize__JisB6"] = int64(QPageSize__JisB6)
	qt.EnumMap["gui.QPageSize__JisB7"] = int64(QPageSize__JisB7)
	qt.EnumMap["gui.QPageSize__JisB8"] = int64(QPageSize__JisB8)
	qt.EnumMap["gui.QPageSize__JisB9"] = int64(QPageSize__JisB9)
	qt.EnumMap["gui.QPageSize__JisB10"] = int64(QPageSize__JisB10)
	qt.EnumMap["gui.QPageSize__AnsiC"] = int64(QPageSize__AnsiC)
	qt.EnumMap["gui.QPageSize__AnsiD"] = int64(QPageSize__AnsiD)
	qt.EnumMap["gui.QPageSize__AnsiE"] = int64(QPageSize__AnsiE)
	qt.EnumMap["gui.QPageSize__LegalExtra"] = int64(QPageSize__LegalExtra)
	qt.EnumMap["gui.QPageSize__LetterExtra"] = int64(QPageSize__LetterExtra)
	qt.EnumMap["gui.QPageSize__LetterPlus"] = int64(QPageSize__LetterPlus)
	qt.EnumMap["gui.QPageSize__LetterSmall"] = int64(QPageSize__LetterSmall)
	qt.EnumMap["gui.QPageSize__TabloidExtra"] = int64(QPageSize__TabloidExtra)
	qt.EnumMap["gui.QPageSize__ArchA"] = int64(QPageSize__ArchA)
	qt.EnumMap["gui.QPageSize__ArchB"] = int64(QPageSize__ArchB)
	qt.EnumMap["gui.QPageSize__ArchC"] = int64(QPageSize__ArchC)
	qt.EnumMap["gui.QPageSize__ArchD"] = int64(QPageSize__ArchD)
	qt.EnumMap["gui.QPageSize__ArchE"] = int64(QPageSize__ArchE)
	qt.EnumMap["gui.QPageSize__Imperial7x9"] = int64(QPageSize__Imperial7x9)
	qt.EnumMap["gui.QPageSize__Imperial8x10"] = int64(QPageSize__Imperial8x10)
	qt.EnumMap["gui.QPageSize__Imperial9x11"] = int64(QPageSize__Imperial9x11)
	qt.EnumMap["gui.QPageSize__Imperial9x12"] = int64(QPageSize__Imperial9x12)
	qt.EnumMap["gui.QPageSize__Imperial10x11"] = int64(QPageSize__Imperial10x11)
	qt.EnumMap["gui.QPageSize__Imperial10x13"] = int64(QPageSize__Imperial10x13)
	qt.EnumMap["gui.QPageSize__Imperial10x14"] = int64(QPageSize__Imperial10x14)
	qt.EnumMap["gui.QPageSize__Imperial12x11"] = int64(QPageSize__Imperial12x11)
	qt.EnumMap["gui.QPageSize__Imperial15x11"] = int64(QPageSize__Imperial15x11)
	qt.EnumMap["gui.QPageSize__ExecutiveStandard"] = int64(QPageSize__ExecutiveStandard)
	qt.EnumMap["gui.QPageSize__Note"] = int64(QPageSize__Note)
	qt.EnumMap["gui.QPageSize__Quarto"] = int64(QPageSize__Quarto)
	qt.EnumMap["gui.QPageSize__Statement"] = int64(QPageSize__Statement)
	qt.EnumMap["gui.QPageSize__SuperA"] = int64(QPageSize__SuperA)
	qt.EnumMap["gui.QPageSize__SuperB"] = int64(QPageSize__SuperB)
	qt.EnumMap["gui.QPageSize__Postcard"] = int64(QPageSize__Postcard)
	qt.EnumMap["gui.QPageSize__DoublePostcard"] = int64(QPageSize__DoublePostcard)
	qt.EnumMap["gui.QPageSize__Prc16K"] = int64(QPageSize__Prc16K)
	qt.EnumMap["gui.QPageSize__Prc32K"] = int64(QPageSize__Prc32K)
	qt.EnumMap["gui.QPageSize__Prc32KBig"] = int64(QPageSize__Prc32KBig)
	qt.EnumMap["gui.QPageSize__FanFoldUS"] = int64(QPageSize__FanFoldUS)
	qt.EnumMap["gui.QPageSize__FanFoldGerman"] = int64(QPageSize__FanFoldGerman)
	qt.EnumMap["gui.QPageSize__FanFoldGermanLegal"] = int64(QPageSize__FanFoldGermanLegal)
	qt.EnumMap["gui.QPageSize__EnvelopeB4"] = int64(QPageSize__EnvelopeB4)
	qt.EnumMap["gui.QPageSize__EnvelopeB5"] = int64(QPageSize__EnvelopeB5)
	qt.EnumMap["gui.QPageSize__EnvelopeB6"] = int64(QPageSize__EnvelopeB6)
	qt.EnumMap["gui.QPageSize__EnvelopeC0"] = int64(QPageSize__EnvelopeC0)
	qt.EnumMap["gui.QPageSize__EnvelopeC1"] = int64(QPageSize__EnvelopeC1)
	qt.EnumMap["gui.QPageSize__EnvelopeC2"] = int64(QPageSize__EnvelopeC2)
	qt.EnumMap["gui.QPageSize__EnvelopeC3"] = int64(QPageSize__EnvelopeC3)
	qt.EnumMap["gui.QPageSize__EnvelopeC4"] = int64(QPageSize__EnvelopeC4)
	qt.EnumMap["gui.QPageSize__EnvelopeC6"] = int64(QPageSize__EnvelopeC6)
	qt.EnumMap["gui.QPageSize__EnvelopeC65"] = int64(QPageSize__EnvelopeC65)
	qt.EnumMap["gui.QPageSize__EnvelopeC7"] = int64(QPageSize__EnvelopeC7)
	qt.EnumMap["gui.QPageSize__Envelope9"] = int64(QPageSize__Envelope9)
	qt.EnumMap["gui.QPageSize__Envelope11"] = int64(QPageSize__Envelope11)
	qt.EnumMap["gui.QPageSize__Envelope12"] = int64(QPageSize__Envelope12)
	qt.EnumMap["gui.QPageSize__Envelope14"] = int64(QPageSize__Envelope14)
	qt.EnumMap["gui.QPageSize__EnvelopeMonarch"] = int64(QPageSize__EnvelopeMonarch)
	qt.EnumMap["gui.QPageSize__EnvelopePersonal"] = int64(QPageSize__EnvelopePersonal)
	qt.EnumMap["gui.QPageSize__EnvelopeChou3"] = int64(QPageSize__EnvelopeChou3)
	qt.EnumMap["gui.QPageSize__EnvelopeChou4"] = int64(QPageSize__EnvelopeChou4)
	qt.EnumMap["gui.QPageSize__EnvelopeInvite"] = int64(QPageSize__EnvelopeInvite)
	qt.EnumMap["gui.QPageSize__EnvelopeItalian"] = int64(QPageSize__EnvelopeItalian)
	qt.EnumMap["gui.QPageSize__EnvelopeKaku2"] = int64(QPageSize__EnvelopeKaku2)
	qt.EnumMap["gui.QPageSize__EnvelopeKaku3"] = int64(QPageSize__EnvelopeKaku3)
	qt.EnumMap["gui.QPageSize__EnvelopePrc1"] = int64(QPageSize__EnvelopePrc1)
	qt.EnumMap["gui.QPageSize__EnvelopePrc2"] = int64(QPageSize__EnvelopePrc2)
	qt.EnumMap["gui.QPageSize__EnvelopePrc3"] = int64(QPageSize__EnvelopePrc3)
	qt.EnumMap["gui.QPageSize__EnvelopePrc4"] = int64(QPageSize__EnvelopePrc4)
	qt.EnumMap["gui.QPageSize__EnvelopePrc5"] = int64(QPageSize__EnvelopePrc5)
	qt.EnumMap["gui.QPageSize__EnvelopePrc6"] = int64(QPageSize__EnvelopePrc6)
	qt.EnumMap["gui.QPageSize__EnvelopePrc7"] = int64(QPageSize__EnvelopePrc7)
	qt.EnumMap["gui.QPageSize__EnvelopePrc8"] = int64(QPageSize__EnvelopePrc8)
	qt.EnumMap["gui.QPageSize__EnvelopePrc9"] = int64(QPageSize__EnvelopePrc9)
	qt.EnumMap["gui.QPageSize__EnvelopePrc10"] = int64(QPageSize__EnvelopePrc10)
	qt.EnumMap["gui.QPageSize__EnvelopeYou4"] = int64(QPageSize__EnvelopeYou4)
	qt.EnumMap["gui.QPageSize__LastPageSize"] = int64(QPageSize__LastPageSize)
	qt.EnumMap["gui.QPageSize__NPageSize"] = int64(QPageSize__NPageSize)
	qt.EnumMap["gui.QPageSize__NPaperSize"] = int64(QPageSize__NPaperSize)
	qt.EnumMap["gui.QPageSize__AnsiA"] = int64(QPageSize__AnsiA)
	qt.EnumMap["gui.QPageSize__AnsiB"] = int64(QPageSize__AnsiB)
	qt.EnumMap["gui.QPageSize__EnvelopeC5"] = int64(QPageSize__EnvelopeC5)
	qt.EnumMap["gui.QPageSize__EnvelopeDL"] = int64(QPageSize__EnvelopeDL)
	qt.EnumMap["gui.QPageSize__Envelope10"] = int64(QPageSize__Envelope10)
	qt.EnumMap["gui.QPageSize__Millimeter"] = int64(QPageSize__Millimeter)
	qt.EnumMap["gui.QPageSize__Point"] = int64(QPageSize__Point)
	qt.EnumMap["gui.QPageSize__Inch"] = int64(QPageSize__Inch)
	qt.EnumMap["gui.QPageSize__Pica"] = int64(QPageSize__Pica)
	qt.EnumMap["gui.QPageSize__Didot"] = int64(QPageSize__Didot)
	qt.EnumMap["gui.QPageSize__Cicero"] = int64(QPageSize__Cicero)
	qt.EnumMap["gui.QPageSize__FuzzyMatch"] = int64(QPageSize__FuzzyMatch)
	qt.EnumMap["gui.QPageSize__FuzzyOrientationMatch"] = int64(QPageSize__FuzzyOrientationMatch)
	qt.EnumMap["gui.QPageSize__ExactMatch"] = int64(QPageSize__ExactMatch)
	qt.ItfMap["gui.QPagedPaintDevice_ITF"] = QPagedPaintDevice{}
	qt.EnumMap["gui.QPagedPaintDevice__A4"] = int64(QPagedPaintDevice__A4)
	qt.EnumMap["gui.QPagedPaintDevice__B5"] = int64(QPagedPaintDevice__B5)
	qt.EnumMap["gui.QPagedPaintDevice__Letter"] = int64(QPagedPaintDevice__Letter)
	qt.EnumMap["gui.QPagedPaintDevice__Legal"] = int64(QPagedPaintDevice__Legal)
	qt.EnumMap["gui.QPagedPaintDevice__Executive"] = int64(QPagedPaintDevice__Executive)
	qt.EnumMap["gui.QPagedPaintDevice__A0"] = int64(QPagedPaintDevice__A0)
	qt.EnumMap["gui.QPagedPaintDevice__A1"] = int64(QPagedPaintDevice__A1)
	qt.EnumMap["gui.QPagedPaintDevice__A2"] = int64(QPagedPaintDevice__A2)
	qt.EnumMap["gui.QPagedPaintDevice__A3"] = int64(QPagedPaintDevice__A3)
	qt.EnumMap["gui.QPagedPaintDevice__A5"] = int64(QPagedPaintDevice__A5)
	qt.EnumMap["gui.QPagedPaintDevice__A6"] = int64(QPagedPaintDevice__A6)
	qt.EnumMap["gui.QPagedPaintDevice__A7"] = int64(QPagedPaintDevice__A7)
	qt.EnumMap["gui.QPagedPaintDevice__A8"] = int64(QPagedPaintDevice__A8)
	qt.EnumMap["gui.QPagedPaintDevice__A9"] = int64(QPagedPaintDevice__A9)
	qt.EnumMap["gui.QPagedPaintDevice__B0"] = int64(QPagedPaintDevice__B0)
	qt.EnumMap["gui.QPagedPaintDevice__B1"] = int64(QPagedPaintDevice__B1)
	qt.EnumMap["gui.QPagedPaintDevice__B10"] = int64(QPagedPaintDevice__B10)
	qt.EnumMap["gui.QPagedPaintDevice__B2"] = int64(QPagedPaintDevice__B2)
	qt.EnumMap["gui.QPagedPaintDevice__B3"] = int64(QPagedPaintDevice__B3)
	qt.EnumMap["gui.QPagedPaintDevice__B4"] = int64(QPagedPaintDevice__B4)
	qt.EnumMap["gui.QPagedPaintDevice__B6"] = int64(QPagedPaintDevice__B6)
	qt.EnumMap["gui.QPagedPaintDevice__B7"] = int64(QPagedPaintDevice__B7)
	qt.EnumMap["gui.QPagedPaintDevice__B8"] = int64(QPagedPaintDevice__B8)
	qt.EnumMap["gui.QPagedPaintDevice__B9"] = int64(QPagedPaintDevice__B9)
	qt.EnumMap["gui.QPagedPaintDevice__C5E"] = int64(QPagedPaintDevice__C5E)
	qt.EnumMap["gui.QPagedPaintDevice__Comm10E"] = int64(QPagedPaintDevice__Comm10E)
	qt.EnumMap["gui.QPagedPaintDevice__DLE"] = int64(QPagedPaintDevice__DLE)
	qt.EnumMap["gui.QPagedPaintDevice__Folio"] = int64(QPagedPaintDevice__Folio)
	qt.EnumMap["gui.QPagedPaintDevice__Ledger"] = int64(QPagedPaintDevice__Ledger)
	qt.EnumMap["gui.QPagedPaintDevice__Tabloid"] = int64(QPagedPaintDevice__Tabloid)
	qt.EnumMap["gui.QPagedPaintDevice__Custom"] = int64(QPagedPaintDevice__Custom)
	qt.EnumMap["gui.QPagedPaintDevice__A10"] = int64(QPagedPaintDevice__A10)
	qt.EnumMap["gui.QPagedPaintDevice__A3Extra"] = int64(QPagedPaintDevice__A3Extra)
	qt.EnumMap["gui.QPagedPaintDevice__A4Extra"] = int64(QPagedPaintDevice__A4Extra)
	qt.EnumMap["gui.QPagedPaintDevice__A4Plus"] = int64(QPagedPaintDevice__A4Plus)
	qt.EnumMap["gui.QPagedPaintDevice__A4Small"] = int64(QPagedPaintDevice__A4Small)
	qt.EnumMap["gui.QPagedPaintDevice__A5Extra"] = int64(QPagedPaintDevice__A5Extra)
	qt.EnumMap["gui.QPagedPaintDevice__B5Extra"] = int64(QPagedPaintDevice__B5Extra)
	qt.EnumMap["gui.QPagedPaintDevice__JisB0"] = int64(QPagedPaintDevice__JisB0)
	qt.EnumMap["gui.QPagedPaintDevice__JisB1"] = int64(QPagedPaintDevice__JisB1)
	qt.EnumMap["gui.QPagedPaintDevice__JisB2"] = int64(QPagedPaintDevice__JisB2)
	qt.EnumMap["gui.QPagedPaintDevice__JisB3"] = int64(QPagedPaintDevice__JisB3)
	qt.EnumMap["gui.QPagedPaintDevice__JisB4"] = int64(QPagedPaintDevice__JisB4)
	qt.EnumMap["gui.QPagedPaintDevice__JisB5"] = int64(QPagedPaintDevice__JisB5)
	qt.EnumMap["gui.QPagedPaintDevice__JisB6"] = int64(QPagedPaintDevice__JisB6)
	qt.EnumMap["gui.QPagedPaintDevice__JisB7"] = int64(QPagedPaintDevice__JisB7)
	qt.EnumMap["gui.QPagedPaintDevice__JisB8"] = int64(QPagedPaintDevice__JisB8)
	qt.EnumMap["gui.QPagedPaintDevice__JisB9"] = int64(QPagedPaintDevice__JisB9)
	qt.EnumMap["gui.QPagedPaintDevice__JisB10"] = int64(QPagedPaintDevice__JisB10)
	qt.EnumMap["gui.QPagedPaintDevice__AnsiC"] = int64(QPagedPaintDevice__AnsiC)
	qt.EnumMap["gui.QPagedPaintDevice__AnsiD"] = int64(QPagedPaintDevice__AnsiD)
	qt.EnumMap["gui.QPagedPaintDevice__AnsiE"] = int64(QPagedPaintDevice__AnsiE)
	qt.EnumMap["gui.QPagedPaintDevice__LegalExtra"] = int64(QPagedPaintDevice__LegalExtra)
	qt.EnumMap["gui.QPagedPaintDevice__LetterExtra"] = int64(QPagedPaintDevice__LetterExtra)
	qt.EnumMap["gui.QPagedPaintDevice__LetterPlus"] = int64(QPagedPaintDevice__LetterPlus)
	qt.EnumMap["gui.QPagedPaintDevice__LetterSmall"] = int64(QPagedPaintDevice__LetterSmall)
	qt.EnumMap["gui.QPagedPaintDevice__TabloidExtra"] = int64(QPagedPaintDevice__TabloidExtra)
	qt.EnumMap["gui.QPagedPaintDevice__ArchA"] = int64(QPagedPaintDevice__ArchA)
	qt.EnumMap["gui.QPagedPaintDevice__ArchB"] = int64(QPagedPaintDevice__ArchB)
	qt.EnumMap["gui.QPagedPaintDevice__ArchC"] = int64(QPagedPaintDevice__ArchC)
	qt.EnumMap["gui.QPagedPaintDevice__ArchD"] = int64(QPagedPaintDevice__ArchD)
	qt.EnumMap["gui.QPagedPaintDevice__ArchE"] = int64(QPagedPaintDevice__ArchE)
	qt.EnumMap["gui.QPagedPaintDevice__Imperial7x9"] = int64(QPagedPaintDevice__Imperial7x9)
	qt.EnumMap["gui.QPagedPaintDevice__Imperial8x10"] = int64(QPagedPaintDevice__Imperial8x10)
	qt.EnumMap["gui.QPagedPaintDevice__Imperial9x11"] = int64(QPagedPaintDevice__Imperial9x11)
	qt.EnumMap["gui.QPagedPaintDevice__Imperial9x12"] = int64(QPagedPaintDevice__Imperial9x12)
	qt.EnumMap["gui.QPagedPaintDevice__Imperial10x11"] = int64(QPagedPaintDevice__Imperial10x11)
	qt.EnumMap["gui.QPagedPaintDevice__Imperial10x13"] = int64(QPagedPaintDevice__Imperial10x13)
	qt.EnumMap["gui.QPagedPaintDevice__Imperial10x14"] = int64(QPagedPaintDevice__Imperial10x14)
	qt.EnumMap["gui.QPagedPaintDevice__Imperial12x11"] = int64(QPagedPaintDevice__Imperial12x11)
	qt.EnumMap["gui.QPagedPaintDevice__Imperial15x11"] = int64(QPagedPaintDevice__Imperial15x11)
	qt.EnumMap["gui.QPagedPaintDevice__ExecutiveStandard"] = int64(QPagedPaintDevice__ExecutiveStandard)
	qt.EnumMap["gui.QPagedPaintDevice__Note"] = int64(QPagedPaintDevice__Note)
	qt.EnumMap["gui.QPagedPaintDevice__Quarto"] = int64(QPagedPaintDevice__Quarto)
	qt.EnumMap["gui.QPagedPaintDevice__Statement"] = int64(QPagedPaintDevice__Statement)
	qt.EnumMap["gui.QPagedPaintDevice__SuperA"] = int64(QPagedPaintDevice__SuperA)
	qt.EnumMap["gui.QPagedPaintDevice__SuperB"] = int64(QPagedPaintDevice__SuperB)
	qt.EnumMap["gui.QPagedPaintDevice__Postcard"] = int64(QPagedPaintDevice__Postcard)
	qt.EnumMap["gui.QPagedPaintDevice__DoublePostcard"] = int64(QPagedPaintDevice__DoublePostcard)
	qt.EnumMap["gui.QPagedPaintDevice__Prc16K"] = int64(QPagedPaintDevice__Prc16K)
	qt.EnumMap["gui.QPagedPaintDevice__Prc32K"] = int64(QPagedPaintDevice__Prc32K)
	qt.EnumMap["gui.QPagedPaintDevice__Prc32KBig"] = int64(QPagedPaintDevice__Prc32KBig)
	qt.EnumMap["gui.QPagedPaintDevice__FanFoldUS"] = int64(QPagedPaintDevice__FanFoldUS)
	qt.EnumMap["gui.QPagedPaintDevice__FanFoldGerman"] = int64(QPagedPaintDevice__FanFoldGerman)
	qt.EnumMap["gui.QPagedPaintDevice__FanFoldGermanLegal"] = int64(QPagedPaintDevice__FanFoldGermanLegal)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeB4"] = int64(QPagedPaintDevice__EnvelopeB4)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeB5"] = int64(QPagedPaintDevice__EnvelopeB5)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeB6"] = int64(QPagedPaintDevice__EnvelopeB6)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeC0"] = int64(QPagedPaintDevice__EnvelopeC0)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeC1"] = int64(QPagedPaintDevice__EnvelopeC1)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeC2"] = int64(QPagedPaintDevice__EnvelopeC2)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeC3"] = int64(QPagedPaintDevice__EnvelopeC3)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeC4"] = int64(QPagedPaintDevice__EnvelopeC4)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeC6"] = int64(QPagedPaintDevice__EnvelopeC6)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeC65"] = int64(QPagedPaintDevice__EnvelopeC65)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeC7"] = int64(QPagedPaintDevice__EnvelopeC7)
	qt.EnumMap["gui.QPagedPaintDevice__Envelope9"] = int64(QPagedPaintDevice__Envelope9)
	qt.EnumMap["gui.QPagedPaintDevice__Envelope11"] = int64(QPagedPaintDevice__Envelope11)
	qt.EnumMap["gui.QPagedPaintDevice__Envelope12"] = int64(QPagedPaintDevice__Envelope12)
	qt.EnumMap["gui.QPagedPaintDevice__Envelope14"] = int64(QPagedPaintDevice__Envelope14)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeMonarch"] = int64(QPagedPaintDevice__EnvelopeMonarch)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePersonal"] = int64(QPagedPaintDevice__EnvelopePersonal)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeChou3"] = int64(QPagedPaintDevice__EnvelopeChou3)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeChou4"] = int64(QPagedPaintDevice__EnvelopeChou4)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeInvite"] = int64(QPagedPaintDevice__EnvelopeInvite)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeItalian"] = int64(QPagedPaintDevice__EnvelopeItalian)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeKaku2"] = int64(QPagedPaintDevice__EnvelopeKaku2)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeKaku3"] = int64(QPagedPaintDevice__EnvelopeKaku3)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePrc1"] = int64(QPagedPaintDevice__EnvelopePrc1)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePrc2"] = int64(QPagedPaintDevice__EnvelopePrc2)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePrc3"] = int64(QPagedPaintDevice__EnvelopePrc3)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePrc4"] = int64(QPagedPaintDevice__EnvelopePrc4)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePrc5"] = int64(QPagedPaintDevice__EnvelopePrc5)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePrc6"] = int64(QPagedPaintDevice__EnvelopePrc6)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePrc7"] = int64(QPagedPaintDevice__EnvelopePrc7)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePrc8"] = int64(QPagedPaintDevice__EnvelopePrc8)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePrc9"] = int64(QPagedPaintDevice__EnvelopePrc9)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopePrc10"] = int64(QPagedPaintDevice__EnvelopePrc10)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeYou4"] = int64(QPagedPaintDevice__EnvelopeYou4)
	qt.EnumMap["gui.QPagedPaintDevice__LastPageSize"] = int64(QPagedPaintDevice__LastPageSize)
	qt.EnumMap["gui.QPagedPaintDevice__NPageSize"] = int64(QPagedPaintDevice__NPageSize)
	qt.EnumMap["gui.QPagedPaintDevice__NPaperSize"] = int64(QPagedPaintDevice__NPaperSize)
	qt.EnumMap["gui.QPagedPaintDevice__AnsiA"] = int64(QPagedPaintDevice__AnsiA)
	qt.EnumMap["gui.QPagedPaintDevice__AnsiB"] = int64(QPagedPaintDevice__AnsiB)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeC5"] = int64(QPagedPaintDevice__EnvelopeC5)
	qt.EnumMap["gui.QPagedPaintDevice__EnvelopeDL"] = int64(QPagedPaintDevice__EnvelopeDL)
	qt.EnumMap["gui.QPagedPaintDevice__Envelope10"] = int64(QPagedPaintDevice__Envelope10)
	qt.EnumMap["gui.QPagedPaintDevice__PdfVersion_1_4"] = int64(QPagedPaintDevice__PdfVersion_1_4)
	qt.EnumMap["gui.QPagedPaintDevice__PdfVersion_A1b"] = int64(QPagedPaintDevice__PdfVersion_A1b)
	qt.EnumMap["gui.QPagedPaintDevice__PdfVersion_1_6"] = int64(QPagedPaintDevice__PdfVersion_1_6)
	qt.ItfMap["gui.QPaintDevice_ITF"] = QPaintDevice{}
	qt.FuncMap["gui.NewQPaintDevice"] = NewQPaintDevice
	qt.EnumMap["gui.QPaintDevice__PdmWidth"] = int64(QPaintDevice__PdmWidth)
	qt.EnumMap["gui.QPaintDevice__PdmHeight"] = int64(QPaintDevice__PdmHeight)
	qt.EnumMap["gui.QPaintDevice__PdmWidthMM"] = int64(QPaintDevice__PdmWidthMM)
	qt.EnumMap["gui.QPaintDevice__PdmHeightMM"] = int64(QPaintDevice__PdmHeightMM)
	qt.EnumMap["gui.QPaintDevice__PdmNumColors"] = int64(QPaintDevice__PdmNumColors)
	qt.EnumMap["gui.QPaintDevice__PdmDepth"] = int64(QPaintDevice__PdmDepth)
	qt.EnumMap["gui.QPaintDevice__PdmDpiX"] = int64(QPaintDevice__PdmDpiX)
	qt.EnumMap["gui.QPaintDevice__PdmDpiY"] = int64(QPaintDevice__PdmDpiY)
	qt.EnumMap["gui.QPaintDevice__PdmPhysicalDpiX"] = int64(QPaintDevice__PdmPhysicalDpiX)
	qt.EnumMap["gui.QPaintDevice__PdmPhysicalDpiY"] = int64(QPaintDevice__PdmPhysicalDpiY)
	qt.EnumMap["gui.QPaintDevice__PdmDevicePixelRatio"] = int64(QPaintDevice__PdmDevicePixelRatio)
	qt.EnumMap["gui.QPaintDevice__PdmDevicePixelRatioScaled"] = int64(QPaintDevice__PdmDevicePixelRatioScaled)
	qt.ItfMap["gui.QPaintDeviceWindow_ITF"] = QPaintDeviceWindow{}
	qt.ItfMap["gui.QPaintEngine_ITF"] = QPaintEngine{}
	qt.FuncMap["gui.NewQPaintEngine"] = NewQPaintEngine
	qt.EnumMap["gui.QPaintEngine__PrimitiveTransform"] = int64(QPaintEngine__PrimitiveTransform)
	qt.EnumMap["gui.QPaintEngine__PatternTransform"] = int64(QPaintEngine__PatternTransform)
	qt.EnumMap["gui.QPaintEngine__PixmapTransform"] = int64(QPaintEngine__PixmapTransform)
	qt.EnumMap["gui.QPaintEngine__PatternBrush"] = int64(QPaintEngine__PatternBrush)
	qt.EnumMap["gui.QPaintEngine__LinearGradientFill"] = int64(QPaintEngine__LinearGradientFill)
	qt.EnumMap["gui.QPaintEngine__RadialGradientFill"] = int64(QPaintEngine__RadialGradientFill)
	qt.EnumMap["gui.QPaintEngine__ConicalGradientFill"] = int64(QPaintEngine__ConicalGradientFill)
	qt.EnumMap["gui.QPaintEngine__AlphaBlend"] = int64(QPaintEngine__AlphaBlend)
	qt.EnumMap["gui.QPaintEngine__PorterDuff"] = int64(QPaintEngine__PorterDuff)
	qt.EnumMap["gui.QPaintEngine__PainterPaths"] = int64(QPaintEngine__PainterPaths)
	qt.EnumMap["gui.QPaintEngine__Antialiasing"] = int64(QPaintEngine__Antialiasing)
	qt.EnumMap["gui.QPaintEngine__BrushStroke"] = int64(QPaintEngine__BrushStroke)
	qt.EnumMap["gui.QPaintEngine__ConstantOpacity"] = int64(QPaintEngine__ConstantOpacity)
	qt.EnumMap["gui.QPaintEngine__MaskedBrush"] = int64(QPaintEngine__MaskedBrush)
	qt.EnumMap["gui.QPaintEngine__PerspectiveTransform"] = int64(QPaintEngine__PerspectiveTransform)
	qt.EnumMap["gui.QPaintEngine__BlendModes"] = int64(QPaintEngine__BlendModes)
	qt.EnumMap["gui.QPaintEngine__ObjectBoundingModeGradients"] = int64(QPaintEngine__ObjectBoundingModeGradients)
	qt.EnumMap["gui.QPaintEngine__RasterOpModes"] = int64(QPaintEngine__RasterOpModes)
	qt.EnumMap["gui.QPaintEngine__PaintOutsidePaintEvent"] = int64(QPaintEngine__PaintOutsidePaintEvent)
	qt.EnumMap["gui.QPaintEngine__AllFeatures"] = int64(QPaintEngine__AllFeatures)
	qt.EnumMap["gui.QPaintEngine__DirtyPen"] = int64(QPaintEngine__DirtyPen)
	qt.EnumMap["gui.QPaintEngine__DirtyBrush"] = int64(QPaintEngine__DirtyBrush)
	qt.EnumMap["gui.QPaintEngine__DirtyBrushOrigin"] = int64(QPaintEngine__DirtyBrushOrigin)
	qt.EnumMap["gui.QPaintEngine__DirtyFont"] = int64(QPaintEngine__DirtyFont)
	qt.EnumMap["gui.QPaintEngine__DirtyBackground"] = int64(QPaintEngine__DirtyBackground)
	qt.EnumMap["gui.QPaintEngine__DirtyBackgroundMode"] = int64(QPaintEngine__DirtyBackgroundMode)
	qt.EnumMap["gui.QPaintEngine__DirtyTransform"] = int64(QPaintEngine__DirtyTransform)
	qt.EnumMap["gui.QPaintEngine__DirtyClipRegion"] = int64(QPaintEngine__DirtyClipRegion)
	qt.EnumMap["gui.QPaintEngine__DirtyClipPath"] = int64(QPaintEngine__DirtyClipPath)
	qt.EnumMap["gui.QPaintEngine__DirtyHints"] = int64(QPaintEngine__DirtyHints)
	qt.EnumMap["gui.QPaintEngine__DirtyCompositionMode"] = int64(QPaintEngine__DirtyCompositionMode)
	qt.EnumMap["gui.QPaintEngine__DirtyClipEnabled"] = int64(QPaintEngine__DirtyClipEnabled)
	qt.EnumMap["gui.QPaintEngine__DirtyOpacity"] = int64(QPaintEngine__DirtyOpacity)
	qt.EnumMap["gui.QPaintEngine__AllDirty"] = int64(QPaintEngine__AllDirty)
	qt.EnumMap["gui.QPaintEngine__OddEvenMode"] = int64(QPaintEngine__OddEvenMode)
	qt.EnumMap["gui.QPaintEngine__WindingMode"] = int64(QPaintEngine__WindingMode)
	qt.EnumMap["gui.QPaintEngine__ConvexMode"] = int64(QPaintEngine__ConvexMode)
	qt.EnumMap["gui.QPaintEngine__PolylineMode"] = int64(QPaintEngine__PolylineMode)
	qt.EnumMap["gui.QPaintEngine__X11"] = int64(QPaintEngine__X11)
	qt.EnumMap["gui.QPaintEngine__Windows"] = int64(QPaintEngine__Windows)
	qt.EnumMap["gui.QPaintEngine__QuickDraw"] = int64(QPaintEngine__QuickDraw)
	qt.EnumMap["gui.QPaintEngine__CoreGraphics"] = int64(QPaintEngine__CoreGraphics)
	qt.EnumMap["gui.QPaintEngine__MacPrinter"] = int64(QPaintEngine__MacPrinter)
	qt.EnumMap["gui.QPaintEngine__QWindowSystem"] = int64(QPaintEngine__QWindowSystem)
	qt.EnumMap["gui.QPaintEngine__PostScript"] = int64(QPaintEngine__PostScript)
	qt.EnumMap["gui.QPaintEngine__OpenGL"] = int64(QPaintEngine__OpenGL)
	qt.EnumMap["gui.QPaintEngine__Picture"] = int64(QPaintEngine__Picture)
	qt.EnumMap["gui.QPaintEngine__SVG"] = int64(QPaintEngine__SVG)
	qt.EnumMap["gui.QPaintEngine__Raster"] = int64(QPaintEngine__Raster)
	qt.EnumMap["gui.QPaintEngine__Direct3D"] = int64(QPaintEngine__Direct3D)
	qt.EnumMap["gui.QPaintEngine__Pdf"] = int64(QPaintEngine__Pdf)
	qt.EnumMap["gui.QPaintEngine__OpenVG"] = int64(QPaintEngine__OpenVG)
	qt.EnumMap["gui.QPaintEngine__OpenGL2"] = int64(QPaintEngine__OpenGL2)
	qt.EnumMap["gui.QPaintEngine__PaintBuffer"] = int64(QPaintEngine__PaintBuffer)
	qt.EnumMap["gui.QPaintEngine__Blitter"] = int64(QPaintEngine__Blitter)
	qt.EnumMap["gui.QPaintEngine__Direct2D"] = int64(QPaintEngine__Direct2D)
	qt.EnumMap["gui.QPaintEngine__User"] = int64(QPaintEngine__User)
	qt.EnumMap["gui.QPaintEngine__MaxUser"] = int64(QPaintEngine__MaxUser)
	qt.ItfMap["gui.QPaintEngineState_ITF"] = QPaintEngineState{}
	qt.ItfMap["gui.QPaintEvent_ITF"] = QPaintEvent{}
	qt.FuncMap["gui.NewQPaintEvent"] = NewQPaintEvent
	qt.FuncMap["gui.NewQPaintEvent2"] = NewQPaintEvent2
	qt.ItfMap["gui.QPainter_ITF"] = QPainter{}
	qt.FuncMap["gui.NewQPainter"] = NewQPainter
	qt.FuncMap["gui.NewQPainter2"] = NewQPainter2
	qt.EnumMap["gui.QPainter__Antialiasing"] = int64(QPainter__Antialiasing)
	qt.EnumMap["gui.QPainter__TextAntialiasing"] = int64(QPainter__TextAntialiasing)
	qt.EnumMap["gui.QPainter__SmoothPixmapTransform"] = int64(QPainter__SmoothPixmapTransform)
	qt.EnumMap["gui.QPainter__HighQualityAntialiasing"] = int64(QPainter__HighQualityAntialiasing)
	qt.EnumMap["gui.QPainter__NonCosmeticDefaultPen"] = int64(QPainter__NonCosmeticDefaultPen)
	qt.EnumMap["gui.QPainter__Qt4CompatiblePainting"] = int64(QPainter__Qt4CompatiblePainting)
	qt.EnumMap["gui.QPainter__LosslessImageRendering"] = int64(QPainter__LosslessImageRendering)
	qt.EnumMap["gui.QPainter__OpaqueHint"] = int64(QPainter__OpaqueHint)
	qt.EnumMap["gui.QPainter__CompositionMode_SourceOver"] = int64(QPainter__CompositionMode_SourceOver)
	qt.EnumMap["gui.QPainter__CompositionMode_DestinationOver"] = int64(QPainter__CompositionMode_DestinationOver)
	qt.EnumMap["gui.QPainter__CompositionMode_Clear"] = int64(QPainter__CompositionMode_Clear)
	qt.EnumMap["gui.QPainter__CompositionMode_Source"] = int64(QPainter__CompositionMode_Source)
	qt.EnumMap["gui.QPainter__CompositionMode_Destination"] = int64(QPainter__CompositionMode_Destination)
	qt.EnumMap["gui.QPainter__CompositionMode_SourceIn"] = int64(QPainter__CompositionMode_SourceIn)
	qt.EnumMap["gui.QPainter__CompositionMode_DestinationIn"] = int64(QPainter__CompositionMode_DestinationIn)
	qt.EnumMap["gui.QPainter__CompositionMode_SourceOut"] = int64(QPainter__CompositionMode_SourceOut)
	qt.EnumMap["gui.QPainter__CompositionMode_DestinationOut"] = int64(QPainter__CompositionMode_DestinationOut)
	qt.EnumMap["gui.QPainter__CompositionMode_SourceAtop"] = int64(QPainter__CompositionMode_SourceAtop)
	qt.EnumMap["gui.QPainter__CompositionMode_DestinationAtop"] = int64(QPainter__CompositionMode_DestinationAtop)
	qt.EnumMap["gui.QPainter__CompositionMode_Xor"] = int64(QPainter__CompositionMode_Xor)
	qt.EnumMap["gui.QPainter__CompositionMode_Plus"] = int64(QPainter__CompositionMode_Plus)
	qt.EnumMap["gui.QPainter__CompositionMode_Multiply"] = int64(QPainter__CompositionMode_Multiply)
	qt.EnumMap["gui.QPainter__CompositionMode_Screen"] = int64(QPainter__CompositionMode_Screen)
	qt.EnumMap["gui.QPainter__CompositionMode_Overlay"] = int64(QPainter__CompositionMode_Overlay)
	qt.EnumMap["gui.QPainter__CompositionMode_Darken"] = int64(QPainter__CompositionMode_Darken)
	qt.EnumMap["gui.QPainter__CompositionMode_Lighten"] = int64(QPainter__CompositionMode_Lighten)
	qt.EnumMap["gui.QPainter__CompositionMode_ColorDodge"] = int64(QPainter__CompositionMode_ColorDodge)
	qt.EnumMap["gui.QPainter__CompositionMode_ColorBurn"] = int64(QPainter__CompositionMode_ColorBurn)
	qt.EnumMap["gui.QPainter__CompositionMode_HardLight"] = int64(QPainter__CompositionMode_HardLight)
	qt.EnumMap["gui.QPainter__CompositionMode_SoftLight"] = int64(QPainter__CompositionMode_SoftLight)
	qt.EnumMap["gui.QPainter__CompositionMode_Difference"] = int64(QPainter__CompositionMode_Difference)
	qt.EnumMap["gui.QPainter__CompositionMode_Exclusion"] = int64(QPainter__CompositionMode_Exclusion)
	qt.EnumMap["gui.QPainter__RasterOp_SourceOrDestination"] = int64(QPainter__RasterOp_SourceOrDestination)
	qt.EnumMap["gui.QPainter__RasterOp_SourceAndDestination"] = int64(QPainter__RasterOp_SourceAndDestination)
	qt.EnumMap["gui.QPainter__RasterOp_SourceXorDestination"] = int64(QPainter__RasterOp_SourceXorDestination)
	qt.EnumMap["gui.QPainter__RasterOp_NotSourceAndNotDestination"] = int64(QPainter__RasterOp_NotSourceAndNotDestination)
	qt.EnumMap["gui.QPainter__RasterOp_NotSourceOrNotDestination"] = int64(QPainter__RasterOp_NotSourceOrNotDestination)
	qt.EnumMap["gui.QPainter__RasterOp_NotSourceXorDestination"] = int64(QPainter__RasterOp_NotSourceXorDestination)
	qt.EnumMap["gui.QPainter__RasterOp_NotSource"] = int64(QPainter__RasterOp_NotSource)
	qt.EnumMap["gui.QPainter__RasterOp_NotSourceAndDestination"] = int64(QPainter__RasterOp_NotSourceAndDestination)
	qt.EnumMap["gui.QPainter__RasterOp_SourceAndNotDestination"] = int64(QPainter__RasterOp_SourceAndNotDestination)
	qt.EnumMap["gui.QPainter__RasterOp_NotSourceOrDestination"] = int64(QPainter__RasterOp_NotSourceOrDestination)
	qt.EnumMap["gui.QPainter__RasterOp_SourceOrNotDestination"] = int64(QPainter__RasterOp_SourceOrNotDestination)
	qt.EnumMap["gui.QPainter__RasterOp_ClearDestination"] = int64(QPainter__RasterOp_ClearDestination)
	qt.EnumMap["gui.QPainter__RasterOp_SetDestination"] = int64(QPainter__RasterOp_SetDestination)
	qt.EnumMap["gui.QPainter__RasterOp_NotDestination"] = int64(QPainter__RasterOp_NotDestination)
	qt.ItfMap["gui.QPainterPath_ITF"] = QPainterPath{}
	qt.FuncMap["gui.NewQPainterPath"] = NewQPainterPath
	qt.FuncMap["gui.NewQPainterPath2"] = NewQPainterPath2
	qt.FuncMap["gui.NewQPainterPath3"] = NewQPainterPath3
	qt.EnumMap["gui.QPainterPath__MoveToElement"] = int64(QPainterPath__MoveToElement)
	qt.EnumMap["gui.QPainterPath__LineToElement"] = int64(QPainterPath__LineToElement)
	qt.EnumMap["gui.QPainterPath__CurveToElement"] = int64(QPainterPath__CurveToElement)
	qt.EnumMap["gui.QPainterPath__CurveToDataElement"] = int64(QPainterPath__CurveToDataElement)
	qt.ItfMap["gui.QPainterPathStroker_ITF"] = QPainterPathStroker{}
	qt.FuncMap["gui.NewQPainterPathStroker"] = NewQPainterPathStroker
	qt.FuncMap["gui.NewQPainterPathStroker2"] = NewQPainterPathStroker2
	qt.ItfMap["gui.QPalette_ITF"] = QPalette{}
	qt.FuncMap["gui.NewQPalette"] = NewQPalette
	qt.FuncMap["gui.NewQPalette2"] = NewQPalette2
	qt.FuncMap["gui.NewQPalette3"] = NewQPalette3
	qt.FuncMap["gui.NewQPalette4"] = NewQPalette4
	qt.FuncMap["gui.NewQPalette5"] = NewQPalette5
	qt.FuncMap["gui.NewQPalette7"] = NewQPalette7
	qt.FuncMap["gui.NewQPalette8"] = NewQPalette8
	qt.EnumMap["gui.QPalette__Active"] = int64(QPalette__Active)
	qt.EnumMap["gui.QPalette__Disabled"] = int64(QPalette__Disabled)
	qt.EnumMap["gui.QPalette__Inactive"] = int64(QPalette__Inactive)
	qt.EnumMap["gui.QPalette__NColorGroups"] = int64(QPalette__NColorGroups)
	qt.EnumMap["gui.QPalette__Current"] = int64(QPalette__Current)
	qt.EnumMap["gui.QPalette__All"] = int64(QPalette__All)
	qt.EnumMap["gui.QPalette__Normal"] = int64(QPalette__Normal)
	qt.EnumMap["gui.QPalette__WindowText"] = int64(QPalette__WindowText)
	qt.EnumMap["gui.QPalette__Button"] = int64(QPalette__Button)
	qt.EnumMap["gui.QPalette__Light"] = int64(QPalette__Light)
	qt.EnumMap["gui.QPalette__Midlight"] = int64(QPalette__Midlight)
	qt.EnumMap["gui.QPalette__Dark"] = int64(QPalette__Dark)
	qt.EnumMap["gui.QPalette__Mid"] = int64(QPalette__Mid)
	qt.EnumMap["gui.QPalette__Text"] = int64(QPalette__Text)
	qt.EnumMap["gui.QPalette__BrightText"] = int64(QPalette__BrightText)
	qt.EnumMap["gui.QPalette__ButtonText"] = int64(QPalette__ButtonText)
	qt.EnumMap["gui.QPalette__Base"] = int64(QPalette__Base)
	qt.EnumMap["gui.QPalette__Window"] = int64(QPalette__Window)
	qt.EnumMap["gui.QPalette__Shadow"] = int64(QPalette__Shadow)
	qt.EnumMap["gui.QPalette__Highlight"] = int64(QPalette__Highlight)
	qt.EnumMap["gui.QPalette__HighlightedText"] = int64(QPalette__HighlightedText)
	qt.EnumMap["gui.QPalette__Link"] = int64(QPalette__Link)
	qt.EnumMap["gui.QPalette__LinkVisited"] = int64(QPalette__LinkVisited)
	qt.EnumMap["gui.QPalette__AlternateBase"] = int64(QPalette__AlternateBase)
	qt.EnumMap["gui.QPalette__NoRole"] = int64(QPalette__NoRole)
	qt.EnumMap["gui.QPalette__ToolTipBase"] = int64(QPalette__ToolTipBase)
	qt.EnumMap["gui.QPalette__ToolTipText"] = int64(QPalette__ToolTipText)
	qt.EnumMap["gui.QPalette__PlaceholderText"] = int64(QPalette__PlaceholderText)
	qt.EnumMap["gui.QPalette__NColorRoles"] = int64(QPalette__NColorRoles)
	qt.EnumMap["gui.QPalette__Foreground"] = int64(QPalette__Foreground)
	qt.EnumMap["gui.QPalette__Background"] = int64(QPalette__Background)
	qt.ItfMap["gui.QPdfWriter_ITF"] = QPdfWriter{}
	qt.FuncMap["gui.NewQPdfWriter"] = NewQPdfWriter
	qt.FuncMap["gui.NewQPdfWriter2"] = NewQPdfWriter2
	qt.ItfMap["gui.QPen_ITF"] = QPen{}
	qt.FuncMap["gui.NewQPen"] = NewQPen
	qt.FuncMap["gui.NewQPen2"] = NewQPen2
	qt.FuncMap["gui.NewQPen3"] = NewQPen3
	qt.FuncMap["gui.NewQPen4"] = NewQPen4
	qt.FuncMap["gui.NewQPen5"] = NewQPen5
	qt.FuncMap["gui.NewQPen6"] = NewQPen6
	qt.ItfMap["gui.QPicture_ITF"] = QPicture{}
	qt.FuncMap["gui.NewQPicture"] = NewQPicture
	qt.FuncMap["gui.NewQPicture2"] = NewQPicture2
	qt.ItfMap["gui.QPictureFormatPlugin_ITF"] = QPictureFormatPlugin{}
	qt.FuncMap["gui.NewQPictureFormatPlugin"] = NewQPictureFormatPlugin
	qt.ItfMap["gui.QPictureIO_ITF"] = QPictureIO{}
	qt.FuncMap["gui.NewQPictureIO"] = NewQPictureIO
	qt.FuncMap["gui.NewQPictureIO2"] = NewQPictureIO2
	qt.FuncMap["gui.NewQPictureIO3"] = NewQPictureIO3
	qt.FuncMap["gui.QPictureIO_InputFormats"] = QPictureIO_InputFormats
	qt.FuncMap["gui.QPictureIO_OutputFormats"] = QPictureIO_OutputFormats
	qt.FuncMap["gui.QPictureIO_PictureFormat"] = QPictureIO_PictureFormat
	qt.FuncMap["gui.QPictureIO_PictureFormat2"] = QPictureIO_PictureFormat2
	qt.ItfMap["gui.QPixelFormat_ITF"] = QPixelFormat{}
	qt.FuncMap["gui.NewQPixelFormat"] = NewQPixelFormat
	qt.FuncMap["gui.NewQPixelFormat2"] = NewQPixelFormat2
	qt.EnumMap["gui.QPixelFormat__RGB"] = int64(QPixelFormat__RGB)
	qt.EnumMap["gui.QPixelFormat__BGR"] = int64(QPixelFormat__BGR)
	qt.EnumMap["gui.QPixelFormat__Indexed"] = int64(QPixelFormat__Indexed)
	qt.EnumMap["gui.QPixelFormat__Grayscale"] = int64(QPixelFormat__Grayscale)
	qt.EnumMap["gui.QPixelFormat__CMYK"] = int64(QPixelFormat__CMYK)
	qt.EnumMap["gui.QPixelFormat__HSL"] = int64(QPixelFormat__HSL)
	qt.EnumMap["gui.QPixelFormat__HSV"] = int64(QPixelFormat__HSV)
	qt.EnumMap["gui.QPixelFormat__YUV"] = int64(QPixelFormat__YUV)
	qt.EnumMap["gui.QPixelFormat__Alpha"] = int64(QPixelFormat__Alpha)
	qt.EnumMap["gui.QPixelFormat__UsesAlpha"] = int64(QPixelFormat__UsesAlpha)
	qt.EnumMap["gui.QPixelFormat__IgnoresAlpha"] = int64(QPixelFormat__IgnoresAlpha)
	qt.EnumMap["gui.QPixelFormat__AtBeginning"] = int64(QPixelFormat__AtBeginning)
	qt.EnumMap["gui.QPixelFormat__AtEnd"] = int64(QPixelFormat__AtEnd)
	qt.EnumMap["gui.QPixelFormat__NotPremultiplied"] = int64(QPixelFormat__NotPremultiplied)
	qt.EnumMap["gui.QPixelFormat__Premultiplied"] = int64(QPixelFormat__Premultiplied)
	qt.EnumMap["gui.QPixelFormat__UnsignedInteger"] = int64(QPixelFormat__UnsignedInteger)
	qt.EnumMap["gui.QPixelFormat__UnsignedShort"] = int64(QPixelFormat__UnsignedShort)
	qt.EnumMap["gui.QPixelFormat__UnsignedByte"] = int64(QPixelFormat__UnsignedByte)
	qt.EnumMap["gui.QPixelFormat__FloatingPoint"] = int64(QPixelFormat__FloatingPoint)
	qt.EnumMap["gui.QPixelFormat__YUV444"] = int64(QPixelFormat__YUV444)
	qt.EnumMap["gui.QPixelFormat__YUV422"] = int64(QPixelFormat__YUV422)
	qt.EnumMap["gui.QPixelFormat__YUV411"] = int64(QPixelFormat__YUV411)
	qt.EnumMap["gui.QPixelFormat__YUV420P"] = int64(QPixelFormat__YUV420P)
	qt.EnumMap["gui.QPixelFormat__YUV420SP"] = int64(QPixelFormat__YUV420SP)
	qt.EnumMap["gui.QPixelFormat__YV12"] = int64(QPixelFormat__YV12)
	qt.EnumMap["gui.QPixelFormat__UYVY"] = int64(QPixelFormat__UYVY)
	qt.EnumMap["gui.QPixelFormat__YUYV"] = int64(QPixelFormat__YUYV)
	qt.EnumMap["gui.QPixelFormat__NV12"] = int64(QPixelFormat__NV12)
	qt.EnumMap["gui.QPixelFormat__NV21"] = int64(QPixelFormat__NV21)
	qt.EnumMap["gui.QPixelFormat__IMC1"] = int64(QPixelFormat__IMC1)
	qt.EnumMap["gui.QPixelFormat__IMC2"] = int64(QPixelFormat__IMC2)
	qt.EnumMap["gui.QPixelFormat__IMC3"] = int64(QPixelFormat__IMC3)
	qt.EnumMap["gui.QPixelFormat__IMC4"] = int64(QPixelFormat__IMC4)
	qt.EnumMap["gui.QPixelFormat__Y8"] = int64(QPixelFormat__Y8)
	qt.EnumMap["gui.QPixelFormat__Y16"] = int64(QPixelFormat__Y16)
	qt.EnumMap["gui.QPixelFormat__LittleEndian"] = int64(QPixelFormat__LittleEndian)
	qt.EnumMap["gui.QPixelFormat__BigEndian"] = int64(QPixelFormat__BigEndian)
	qt.EnumMap["gui.QPixelFormat__CurrentSystemEndian"] = int64(QPixelFormat__CurrentSystemEndian)
	qt.ItfMap["gui.QPixmap_ITF"] = QPixmap{}
	qt.FuncMap["gui.NewQPixmap"] = NewQPixmap
	qt.FuncMap["gui.NewQPixmap2"] = NewQPixmap2
	qt.FuncMap["gui.NewQPixmap3"] = NewQPixmap3
	qt.FuncMap["gui.NewQPixmap5"] = NewQPixmap5
	qt.FuncMap["gui.QPixmap_DefaultDepth"] = QPixmap_DefaultDepth
	qt.FuncMap["gui.QPixmap_FromImage"] = QPixmap_FromImage
	qt.FuncMap["gui.QPixmap_FromImage2"] = QPixmap_FromImage2
	qt.FuncMap["gui.QPixmap_FromImageReader"] = QPixmap_FromImageReader
	qt.FuncMap["gui.QPixmap_TrueMatrix"] = QPixmap_TrueMatrix
	qt.FuncMap["gui.QPixmap_TrueMatrix2"] = QPixmap_TrueMatrix2
	qt.ItfMap["gui.QPixmapCache_ITF"] = QPixmapCache{}
	qt.FuncMap["gui.QPixmapCache_CacheLimit"] = QPixmapCache_CacheLimit
	qt.FuncMap["gui.QPixmapCache_Clear"] = QPixmapCache_Clear
	qt.FuncMap["gui.QPixmapCache_Find3"] = QPixmapCache_Find3
	qt.FuncMap["gui.QPixmapCache_Insert"] = QPixmapCache_Insert
	qt.FuncMap["gui.QPixmapCache_Remove"] = QPixmapCache_Remove
	qt.FuncMap["gui.QPixmapCache_SetCacheLimit"] = QPixmapCache_SetCacheLimit
	qt.ItfMap["gui.QPlatformDragQtResponse_ITF"] = QPlatformDragQtResponse{}
	qt.ItfMap["gui.QPlatformDropQtResponse_ITF"] = QPlatformDropQtResponse{}
	qt.ItfMap["gui.QPlatformIntegrationPlugin_ITF"] = QPlatformIntegrationPlugin{}
	qt.ItfMap["gui.QPlatformMenu_ITF"] = QPlatformMenu{}
	qt.ItfMap["gui.QPlatformMenuBar_ITF"] = QPlatformMenuBar{}
	qt.ItfMap["gui.QPlatformMenuItem_ITF"] = QPlatformMenuItem{}
	qt.ItfMap["gui.QPlatformOffscreenSurface_ITF"] = QPlatformOffscreenSurface{}
	qt.ItfMap["gui.QPlatformSessionManager_ITF"] = QPlatformSessionManager{}
	qt.ItfMap["gui.QPlatformSurfaceEvent_ITF"] = QPlatformSurfaceEvent{}
	qt.FuncMap["gui.NewQPlatformSurfaceEvent"] = NewQPlatformSurfaceEvent
	qt.EnumMap["gui.QPlatformSurfaceEvent__SurfaceCreated"] = int64(QPlatformSurfaceEvent__SurfaceCreated)
	qt.EnumMap["gui.QPlatformSurfaceEvent__SurfaceAboutToBeDestroyed"] = int64(QPlatformSurfaceEvent__SurfaceAboutToBeDestroyed)
	qt.ItfMap["gui.QPlatformTextureList_ITF"] = QPlatformTextureList{}
	qt.ItfMap["gui.QPointingDeviceUniqueId_ITF"] = QPointingDeviceUniqueId{}
	qt.FuncMap["gui.NewQPointingDeviceUniqueId"] = NewQPointingDeviceUniqueId
	qt.FuncMap["gui.QPointingDeviceUniqueId_FromNumericId"] = QPointingDeviceUniqueId_FromNumericId
	qt.ItfMap["gui.QPolygon_ITF"] = QPolygon{}
	qt.FuncMap["gui.NewQPolygon"] = NewQPolygon
	qt.FuncMap["gui.NewQPolygon2"] = NewQPolygon2
	qt.FuncMap["gui.NewQPolygon3"] = NewQPolygon3
	qt.FuncMap["gui.NewQPolygon5"] = NewQPolygon5
	qt.ItfMap["gui.QPolygonF_ITF"] = QPolygonF{}
	qt.FuncMap["gui.NewQPolygonF"] = NewQPolygonF
	qt.FuncMap["gui.NewQPolygonF2"] = NewQPolygonF2
	qt.FuncMap["gui.NewQPolygonF3"] = NewQPolygonF3
	qt.FuncMap["gui.NewQPolygonF5"] = NewQPolygonF5
	qt.FuncMap["gui.NewQPolygonF6"] = NewQPolygonF6
	qt.FuncMap["gui.NewQPolygonF7"] = NewQPolygonF7
	qt.ItfMap["gui.QQuaternion_ITF"] = QQuaternion{}
	qt.FuncMap["gui.NewQQuaternion"] = NewQQuaternion
	qt.FuncMap["gui.NewQQuaternion3"] = NewQQuaternion3
	qt.FuncMap["gui.NewQQuaternion4"] = NewQQuaternion4
	qt.FuncMap["gui.NewQQuaternion5"] = NewQQuaternion5
	qt.FuncMap["gui.QQuaternion_DotProduct"] = QQuaternion_DotProduct
	qt.FuncMap["gui.QQuaternion_FromAxes"] = QQuaternion_FromAxes
	qt.FuncMap["gui.QQuaternion_FromAxisAndAngle"] = QQuaternion_FromAxisAndAngle
	qt.FuncMap["gui.QQuaternion_FromAxisAndAngle2"] = QQuaternion_FromAxisAndAngle2
	qt.FuncMap["gui.QQuaternion_FromDirection"] = QQuaternion_FromDirection
	qt.FuncMap["gui.QQuaternion_FromEulerAngles"] = QQuaternion_FromEulerAngles
	qt.FuncMap["gui.QQuaternion_FromEulerAngles2"] = QQuaternion_FromEulerAngles2
	qt.FuncMap["gui.QQuaternion_Nlerp"] = QQuaternion_Nlerp
	qt.FuncMap["gui.QQuaternion_RotationTo"] = QQuaternion_RotationTo
	qt.FuncMap["gui.QQuaternion_Slerp"] = QQuaternion_Slerp
	qt.ItfMap["gui.QRadialGradient_ITF"] = QRadialGradient{}
	qt.FuncMap["gui.NewQRadialGradient"] = NewQRadialGradient
	qt.FuncMap["gui.NewQRadialGradient2"] = NewQRadialGradient2
	qt.FuncMap["gui.NewQRadialGradient3"] = NewQRadialGradient3
	qt.FuncMap["gui.NewQRadialGradient4"] = NewQRadialGradient4
	qt.FuncMap["gui.NewQRadialGradient5"] = NewQRadialGradient5
	qt.FuncMap["gui.NewQRadialGradient6"] = NewQRadialGradient6
	qt.FuncMap["gui.NewQRadialGradient7"] = NewQRadialGradient7
	qt.ItfMap["gui.QRasterPaintEngine_ITF"] = QRasterPaintEngine{}
	qt.ItfMap["gui.QRasterWindow_ITF"] = QRasterWindow{}
	qt.FuncMap["gui.NewQRasterWindow"] = NewQRasterWindow
	qt.ItfMap["gui.QRawFont_ITF"] = QRawFont{}
	qt.FuncMap["gui.NewQRawFont"] = NewQRawFont
	qt.FuncMap["gui.NewQRawFont2"] = NewQRawFont2
	qt.FuncMap["gui.NewQRawFont3"] = NewQRawFont3
	qt.FuncMap["gui.NewQRawFont4"] = NewQRawFont4
	qt.FuncMap["gui.QRawFont_FromFont"] = QRawFont_FromFont
	qt.EnumMap["gui.QRawFont__PixelAntialiasing"] = int64(QRawFont__PixelAntialiasing)
	qt.EnumMap["gui.QRawFont__SubPixelAntialiasing"] = int64(QRawFont__SubPixelAntialiasing)
	qt.EnumMap["gui.QRawFont__SeparateAdvances"] = int64(QRawFont__SeparateAdvances)
	qt.EnumMap["gui.QRawFont__KernedAdvances"] = int64(QRawFont__KernedAdvances)
	qt.EnumMap["gui.QRawFont__UseDesignMetrics"] = int64(QRawFont__UseDesignMetrics)
	qt.ItfMap["gui.QRegExpValidator_ITF"] = QRegExpValidator{}
	qt.FuncMap["gui.NewQRegExpValidator"] = NewQRegExpValidator
	qt.FuncMap["gui.NewQRegExpValidator2"] = NewQRegExpValidator2
	qt.ItfMap["gui.QRegion_ITF"] = QRegion{}
	qt.FuncMap["gui.NewQRegion"] = NewQRegion
	qt.FuncMap["gui.NewQRegion2"] = NewQRegion2
	qt.FuncMap["gui.NewQRegion3"] = NewQRegion3
	qt.FuncMap["gui.NewQRegion4"] = NewQRegion4
	qt.FuncMap["gui.NewQRegion5"] = NewQRegion5
	qt.FuncMap["gui.NewQRegion6"] = NewQRegion6
	qt.FuncMap["gui.NewQRegion7"] = NewQRegion7
	qt.EnumMap["gui.QRegion__Rectangle"] = int64(QRegion__Rectangle)
	qt.EnumMap["gui.QRegion__Ellipse"] = int64(QRegion__Ellipse)
	qt.ItfMap["gui.QRegularExpressionValidator_ITF"] = QRegularExpressionValidator{}
	qt.FuncMap["gui.NewQRegularExpressionValidator"] = NewQRegularExpressionValidator
	qt.FuncMap["gui.NewQRegularExpressionValidator2"] = NewQRegularExpressionValidator2
	qt.ItfMap["gui.QResizeEvent_ITF"] = QResizeEvent{}
	qt.FuncMap["gui.NewQResizeEvent"] = NewQResizeEvent
	qt.ItfMap["gui.QRgba64_ITF"] = QRgba64{}
	qt.ItfMap["gui.QScreen_ITF"] = QScreen{}
	qt.ItfMap["gui.QScreenOrientationChangeEvent_ITF"] = QScreenOrientationChangeEvent{}
	qt.FuncMap["gui.NewQScreenOrientationChangeEvent"] = NewQScreenOrientationChangeEvent
	qt.ItfMap["gui.QScrollEvent_ITF"] = QScrollEvent{}
	qt.FuncMap["gui.NewQScrollEvent"] = NewQScrollEvent
	qt.EnumMap["gui.QScrollEvent__ScrollStarted"] = int64(QScrollEvent__ScrollStarted)
	qt.EnumMap["gui.QScrollEvent__ScrollUpdated"] = int64(QScrollEvent__ScrollUpdated)
	qt.EnumMap["gui.QScrollEvent__ScrollFinished"] = int64(QScrollEvent__ScrollFinished)
	qt.ItfMap["gui.QScrollPrepareEvent_ITF"] = QScrollPrepareEvent{}
	qt.FuncMap["gui.NewQScrollPrepareEvent"] = NewQScrollPrepareEvent
	qt.ItfMap["gui.QSessionManager_ITF"] = QSessionManager{}
	qt.EnumMap["gui.QSessionManager__RestartIfRunning"] = int64(QSessionManager__RestartIfRunning)
	qt.EnumMap["gui.QSessionManager__RestartAnyway"] = int64(QSessionManager__RestartAnyway)
	qt.EnumMap["gui.QSessionManager__RestartImmediately"] = int64(QSessionManager__RestartImmediately)
	qt.EnumMap["gui.QSessionManager__RestartNever"] = int64(QSessionManager__RestartNever)
	qt.ItfMap["gui.QShortcutEvent_ITF"] = QShortcutEvent{}
	qt.FuncMap["gui.NewQShortcutEvent"] = NewQShortcutEvent
	qt.ItfMap["gui.QShowEvent_ITF"] = QShowEvent{}
	qt.FuncMap["gui.NewQShowEvent"] = NewQShowEvent
	qt.ItfMap["gui.QStandardItem_ITF"] = QStandardItem{}
	qt.FuncMap["gui.NewQStandardItem"] = NewQStandardItem
	qt.FuncMap["gui.NewQStandardItem2"] = NewQStandardItem2
	qt.FuncMap["gui.NewQStandardItem3"] = NewQStandardItem3
	qt.FuncMap["gui.NewQStandardItem4"] = NewQStandardItem4
	qt.FuncMap["gui.NewQStandardItem5"] = NewQStandardItem5
	qt.EnumMap["gui.QStandardItem__Type"] = int64(QStandardItem__Type)
	qt.EnumMap["gui.QStandardItem__UserType"] = int64(QStandardItem__UserType)
	qt.ItfMap["gui.QStandardItemModel_ITF"] = QStandardItemModel{}
	qt.FuncMap["gui.NewQStandardItemModel"] = NewQStandardItemModel
	qt.FuncMap["gui.NewQStandardItemModel2"] = NewQStandardItemModel2
	qt.ItfMap["gui.QStaticText_ITF"] = QStaticText{}
	qt.FuncMap["gui.NewQStaticText"] = NewQStaticText
	qt.FuncMap["gui.NewQStaticText2"] = NewQStaticText2
	qt.FuncMap["gui.NewQStaticText3"] = NewQStaticText3
	qt.EnumMap["gui.QStaticText__ModerateCaching"] = int64(QStaticText__ModerateCaching)
	qt.EnumMap["gui.QStaticText__AggressiveCaching"] = int64(QStaticText__AggressiveCaching)
	qt.ItfMap["gui.QStatusTipEvent_ITF"] = QStatusTipEvent{}
	qt.FuncMap["gui.NewQStatusTipEvent"] = NewQStatusTipEvent
	qt.ItfMap["gui.QStyleHints_ITF"] = QStyleHints{}
	qt.ItfMap["gui.QSupportedWritingSystems_ITF"] = QSupportedWritingSystems{}
	qt.ItfMap["gui.QSurface_ITF"] = QSurface{}
	qt.EnumMap["gui.QSurface__Window"] = int64(QSurface__Window)
	qt.EnumMap["gui.QSurface__Offscreen"] = int64(QSurface__Offscreen)
	qt.EnumMap["gui.QSurface__RasterSurface"] = int64(QSurface__RasterSurface)
	qt.EnumMap["gui.QSurface__OpenGLSurface"] = int64(QSurface__OpenGLSurface)
	qt.EnumMap["gui.QSurface__RasterGLSurface"] = int64(QSurface__RasterGLSurface)
	qt.EnumMap["gui.QSurface__OpenVGSurface"] = int64(QSurface__OpenVGSurface)
	qt.EnumMap["gui.QSurface__VulkanSurface"] = int64(QSurface__VulkanSurface)
	qt.EnumMap["gui.QSurface__MetalSurface"] = int64(QSurface__MetalSurface)
	qt.ItfMap["gui.QSurfaceFormat_ITF"] = QSurfaceFormat{}
	qt.FuncMap["gui.NewQSurfaceFormat"] = NewQSurfaceFormat
	qt.FuncMap["gui.NewQSurfaceFormat2"] = NewQSurfaceFormat2
	qt.FuncMap["gui.NewQSurfaceFormat3"] = NewQSurfaceFormat3
	qt.FuncMap["gui.QSurfaceFormat_DefaultFormat"] = QSurfaceFormat_DefaultFormat
	qt.FuncMap["gui.QSurfaceFormat_SetDefaultFormat"] = QSurfaceFormat_SetDefaultFormat
	qt.EnumMap["gui.QSurfaceFormat__StereoBuffers"] = int64(QSurfaceFormat__StereoBuffers)
	qt.EnumMap["gui.QSurfaceFormat__DebugContext"] = int64(QSurfaceFormat__DebugContext)
	qt.EnumMap["gui.QSurfaceFormat__DeprecatedFunctions"] = int64(QSurfaceFormat__DeprecatedFunctions)
	qt.EnumMap["gui.QSurfaceFormat__ResetNotification"] = int64(QSurfaceFormat__ResetNotification)
	qt.EnumMap["gui.QSurfaceFormat__DefaultSwapBehavior"] = int64(QSurfaceFormat__DefaultSwapBehavior)
	qt.EnumMap["gui.QSurfaceFormat__SingleBuffer"] = int64(QSurfaceFormat__SingleBuffer)
	qt.EnumMap["gui.QSurfaceFormat__DoubleBuffer"] = int64(QSurfaceFormat__DoubleBuffer)
	qt.EnumMap["gui.QSurfaceFormat__TripleBuffer"] = int64(QSurfaceFormat__TripleBuffer)
	qt.EnumMap["gui.QSurfaceFormat__DefaultRenderableType"] = int64(QSurfaceFormat__DefaultRenderableType)
	qt.EnumMap["gui.QSurfaceFormat__OpenGL"] = int64(QSurfaceFormat__OpenGL)
	qt.EnumMap["gui.QSurfaceFormat__OpenGLES"] = int64(QSurfaceFormat__OpenGLES)
	qt.EnumMap["gui.QSurfaceFormat__OpenVG"] = int64(QSurfaceFormat__OpenVG)
	qt.EnumMap["gui.QSurfaceFormat__NoProfile"] = int64(QSurfaceFormat__NoProfile)
	qt.EnumMap["gui.QSurfaceFormat__CoreProfile"] = int64(QSurfaceFormat__CoreProfile)
	qt.EnumMap["gui.QSurfaceFormat__CompatibilityProfile"] = int64(QSurfaceFormat__CompatibilityProfile)
	qt.EnumMap["gui.QSurfaceFormat__DefaultColorSpace"] = int64(QSurfaceFormat__DefaultColorSpace)
	qt.EnumMap["gui.QSurfaceFormat__sRGBColorSpace"] = int64(QSurfaceFormat__sRGBColorSpace)
	qt.ItfMap["gui.QSyntaxHighlighter_ITF"] = QSyntaxHighlighter{}
	qt.FuncMap["gui.NewQSyntaxHighlighter"] = NewQSyntaxHighlighter
	qt.FuncMap["gui.NewQSyntaxHighlighter2"] = NewQSyntaxHighlighter2
	qt.ItfMap["gui.QTabletEvent_ITF"] = QTabletEvent{}
	qt.FuncMap["gui.NewQTabletEvent2"] = NewQTabletEvent2
	qt.EnumMap["gui.QTabletEvent__NoDevice"] = int64(QTabletEvent__NoDevice)
	qt.EnumMap["gui.QTabletEvent__Puck"] = int64(QTabletEvent__Puck)
	qt.EnumMap["gui.QTabletEvent__Stylus"] = int64(QTabletEvent__Stylus)
	qt.EnumMap["gui.QTabletEvent__Airbrush"] = int64(QTabletEvent__Airbrush)
	qt.EnumMap["gui.QTabletEvent__FourDMouse"] = int64(QTabletEvent__FourDMouse)
	qt.EnumMap["gui.QTabletEvent__XFreeEraser"] = int64(QTabletEvent__XFreeEraser)
	qt.EnumMap["gui.QTabletEvent__RotationStylus"] = int64(QTabletEvent__RotationStylus)
	qt.EnumMap["gui.QTabletEvent__UnknownPointer"] = int64(QTabletEvent__UnknownPointer)
	qt.EnumMap["gui.QTabletEvent__Pen"] = int64(QTabletEvent__Pen)
	qt.EnumMap["gui.QTabletEvent__Cursor"] = int64(QTabletEvent__Cursor)
	qt.EnumMap["gui.QTabletEvent__Eraser"] = int64(QTabletEvent__Eraser)
	qt.ItfMap["gui.QTextBlock_ITF"] = QTextBlock{}
	qt.FuncMap["gui.NewQTextBlock3"] = NewQTextBlock3
	qt.ItfMap["gui.QTextBlockFormat_ITF"] = QTextBlockFormat{}
	qt.FuncMap["gui.NewQTextBlockFormat"] = NewQTextBlockFormat
	qt.EnumMap["gui.QTextBlockFormat__SingleHeight"] = int64(QTextBlockFormat__SingleHeight)
	qt.EnumMap["gui.QTextBlockFormat__ProportionalHeight"] = int64(QTextBlockFormat__ProportionalHeight)
	qt.EnumMap["gui.QTextBlockFormat__FixedHeight"] = int64(QTextBlockFormat__FixedHeight)
	qt.EnumMap["gui.QTextBlockFormat__MinimumHeight"] = int64(QTextBlockFormat__MinimumHeight)
	qt.EnumMap["gui.QTextBlockFormat__LineDistanceHeight"] = int64(QTextBlockFormat__LineDistanceHeight)
	qt.ItfMap["gui.QTextBlockGroup_ITF"] = QTextBlockGroup{}
	qt.FuncMap["gui.NewQTextBlockGroup"] = NewQTextBlockGroup
	qt.ItfMap["gui.QTextBlockUserData_ITF"] = QTextBlockUserData{}
	qt.ItfMap["gui.QTextCharFormat_ITF"] = QTextCharFormat{}
	qt.FuncMap["gui.NewQTextCharFormat"] = NewQTextCharFormat
	qt.EnumMap["gui.QTextCharFormat__AlignNormal"] = int64(QTextCharFormat__AlignNormal)
	qt.EnumMap["gui.QTextCharFormat__AlignSuperScript"] = int64(QTextCharFormat__AlignSuperScript)
	qt.EnumMap["gui.QTextCharFormat__AlignSubScript"] = int64(QTextCharFormat__AlignSubScript)
	qt.EnumMap["gui.QTextCharFormat__AlignMiddle"] = int64(QTextCharFormat__AlignMiddle)
	qt.EnumMap["gui.QTextCharFormat__AlignTop"] = int64(QTextCharFormat__AlignTop)
	qt.EnumMap["gui.QTextCharFormat__AlignBottom"] = int64(QTextCharFormat__AlignBottom)
	qt.EnumMap["gui.QTextCharFormat__AlignBaseline"] = int64(QTextCharFormat__AlignBaseline)
	qt.EnumMap["gui.QTextCharFormat__NoUnderline"] = int64(QTextCharFormat__NoUnderline)
	qt.EnumMap["gui.QTextCharFormat__SingleUnderline"] = int64(QTextCharFormat__SingleUnderline)
	qt.EnumMap["gui.QTextCharFormat__DashUnderline"] = int64(QTextCharFormat__DashUnderline)
	qt.EnumMap["gui.QTextCharFormat__DotLine"] = int64(QTextCharFormat__DotLine)
	qt.EnumMap["gui.QTextCharFormat__DashDotLine"] = int64(QTextCharFormat__DashDotLine)
	qt.EnumMap["gui.QTextCharFormat__DashDotDotLine"] = int64(QTextCharFormat__DashDotDotLine)
	qt.EnumMap["gui.QTextCharFormat__WaveUnderline"] = int64(QTextCharFormat__WaveUnderline)
	qt.EnumMap["gui.QTextCharFormat__SpellCheckUnderline"] = int64(QTextCharFormat__SpellCheckUnderline)
	qt.EnumMap["gui.QTextCharFormat__FontPropertiesSpecifiedOnly"] = int64(QTextCharFormat__FontPropertiesSpecifiedOnly)
	qt.EnumMap["gui.QTextCharFormat__FontPropertiesAll"] = int64(QTextCharFormat__FontPropertiesAll)
	qt.ItfMap["gui.QTextCursor_ITF"] = QTextCursor{}
	qt.FuncMap["gui.NewQTextCursor"] = NewQTextCursor
	qt.FuncMap["gui.NewQTextCursor2"] = NewQTextCursor2
	qt.FuncMap["gui.NewQTextCursor3"] = NewQTextCursor3
	qt.FuncMap["gui.NewQTextCursor4"] = NewQTextCursor4
	qt.FuncMap["gui.NewQTextCursor5"] = NewQTextCursor5
	qt.EnumMap["gui.QTextCursor__MoveAnchor"] = int64(QTextCursor__MoveAnchor)
	qt.EnumMap["gui.QTextCursor__KeepAnchor"] = int64(QTextCursor__KeepAnchor)
	qt.EnumMap["gui.QTextCursor__NoMove"] = int64(QTextCursor__NoMove)
	qt.EnumMap["gui.QTextCursor__Start"] = int64(QTextCursor__Start)
	qt.EnumMap["gui.QTextCursor__Up"] = int64(QTextCursor__Up)
	qt.EnumMap["gui.QTextCursor__StartOfLine"] = int64(QTextCursor__StartOfLine)
	qt.EnumMap["gui.QTextCursor__StartOfBlock"] = int64(QTextCursor__StartOfBlock)
	qt.EnumMap["gui.QTextCursor__StartOfWord"] = int64(QTextCursor__StartOfWord)
	qt.EnumMap["gui.QTextCursor__PreviousBlock"] = int64(QTextCursor__PreviousBlock)
	qt.EnumMap["gui.QTextCursor__PreviousCharacter"] = int64(QTextCursor__PreviousCharacter)
	qt.EnumMap["gui.QTextCursor__PreviousWord"] = int64(QTextCursor__PreviousWord)
	qt.EnumMap["gui.QTextCursor__Left"] = int64(QTextCursor__Left)
	qt.EnumMap["gui.QTextCursor__WordLeft"] = int64(QTextCursor__WordLeft)
	qt.EnumMap["gui.QTextCursor__End"] = int64(QTextCursor__End)
	qt.EnumMap["gui.QTextCursor__Down"] = int64(QTextCursor__Down)
	qt.EnumMap["gui.QTextCursor__EndOfLine"] = int64(QTextCursor__EndOfLine)
	qt.EnumMap["gui.QTextCursor__EndOfWord"] = int64(QTextCursor__EndOfWord)
	qt.EnumMap["gui.QTextCursor__EndOfBlock"] = int64(QTextCursor__EndOfBlock)
	qt.EnumMap["gui.QTextCursor__NextBlock"] = int64(QTextCursor__NextBlock)
	qt.EnumMap["gui.QTextCursor__NextCharacter"] = int64(QTextCursor__NextCharacter)
	qt.EnumMap["gui.QTextCursor__NextWord"] = int64(QTextCursor__NextWord)
	qt.EnumMap["gui.QTextCursor__Right"] = int64(QTextCursor__Right)
	qt.EnumMap["gui.QTextCursor__WordRight"] = int64(QTextCursor__WordRight)
	qt.EnumMap["gui.QTextCursor__NextCell"] = int64(QTextCursor__NextCell)
	qt.EnumMap["gui.QTextCursor__PreviousCell"] = int64(QTextCursor__PreviousCell)
	qt.EnumMap["gui.QTextCursor__NextRow"] = int64(QTextCursor__NextRow)
	qt.EnumMap["gui.QTextCursor__PreviousRow"] = int64(QTextCursor__PreviousRow)
	qt.EnumMap["gui.QTextCursor__WordUnderCursor"] = int64(QTextCursor__WordUnderCursor)
	qt.EnumMap["gui.QTextCursor__LineUnderCursor"] = int64(QTextCursor__LineUnderCursor)
	qt.EnumMap["gui.QTextCursor__BlockUnderCursor"] = int64(QTextCursor__BlockUnderCursor)
	qt.EnumMap["gui.QTextCursor__Document"] = int64(QTextCursor__Document)
	qt.ItfMap["gui.QTextDocument_ITF"] = QTextDocument{}
	qt.FuncMap["gui.NewQTextDocument"] = NewQTextDocument
	qt.FuncMap["gui.NewQTextDocument2"] = NewQTextDocument2
	qt.EnumMap["gui.QTextDocument__DocumentTitle"] = int64(QTextDocument__DocumentTitle)
	qt.EnumMap["gui.QTextDocument__DocumentUrl"] = int64(QTextDocument__DocumentUrl)
	qt.EnumMap["gui.QTextDocument__FindBackward"] = int64(QTextDocument__FindBackward)
	qt.EnumMap["gui.QTextDocument__FindCaseSensitively"] = int64(QTextDocument__FindCaseSensitively)
	qt.EnumMap["gui.QTextDocument__FindWholeWords"] = int64(QTextDocument__FindWholeWords)
	qt.EnumMap["gui.QTextDocument__HtmlResource"] = int64(QTextDocument__HtmlResource)
	qt.EnumMap["gui.QTextDocument__ImageResource"] = int64(QTextDocument__ImageResource)
	qt.EnumMap["gui.QTextDocument__StyleSheetResource"] = int64(QTextDocument__StyleSheetResource)
	qt.EnumMap["gui.QTextDocument__UserResource"] = int64(QTextDocument__UserResource)
	qt.EnumMap["gui.QTextDocument__UndoStack"] = int64(QTextDocument__UndoStack)
	qt.EnumMap["gui.QTextDocument__RedoStack"] = int64(QTextDocument__RedoStack)
	qt.EnumMap["gui.QTextDocument__UndoAndRedoStacks"] = int64(QTextDocument__UndoAndRedoStacks)
	qt.ItfMap["gui.QTextDocumentFragment_ITF"] = QTextDocumentFragment{}
	qt.FuncMap["gui.NewQTextDocumentFragment"] = NewQTextDocumentFragment
	qt.FuncMap["gui.NewQTextDocumentFragment2"] = NewQTextDocumentFragment2
	qt.FuncMap["gui.NewQTextDocumentFragment3"] = NewQTextDocumentFragment3
	qt.FuncMap["gui.NewQTextDocumentFragment4"] = NewQTextDocumentFragment4
	qt.FuncMap["gui.QTextDocumentFragment_FromHtml"] = QTextDocumentFragment_FromHtml
	qt.FuncMap["gui.QTextDocumentFragment_FromHtml2"] = QTextDocumentFragment_FromHtml2
	qt.FuncMap["gui.QTextDocumentFragment_FromPlainText"] = QTextDocumentFragment_FromPlainText
	qt.ItfMap["gui.QTextDocumentWriter_ITF"] = QTextDocumentWriter{}
	qt.FuncMap["gui.NewQTextDocumentWriter"] = NewQTextDocumentWriter
	qt.FuncMap["gui.NewQTextDocumentWriter2"] = NewQTextDocumentWriter2
	qt.FuncMap["gui.NewQTextDocumentWriter3"] = NewQTextDocumentWriter3
	qt.FuncMap["gui.QTextDocumentWriter_SupportedDocumentFormats"] = QTextDocumentWriter_SupportedDocumentFormats
	qt.ItfMap["gui.QTextFormat_ITF"] = QTextFormat{}
	qt.FuncMap["gui.NewQTextFormat"] = NewQTextFormat
	qt.FuncMap["gui.NewQTextFormat2"] = NewQTextFormat2
	qt.FuncMap["gui.NewQTextFormat3"] = NewQTextFormat3
	qt.EnumMap["gui.QTextFormat__InvalidFormat"] = int64(QTextFormat__InvalidFormat)
	qt.EnumMap["gui.QTextFormat__BlockFormat"] = int64(QTextFormat__BlockFormat)
	qt.EnumMap["gui.QTextFormat__CharFormat"] = int64(QTextFormat__CharFormat)
	qt.EnumMap["gui.QTextFormat__ListFormat"] = int64(QTextFormat__ListFormat)
	qt.EnumMap["gui.QTextFormat__TableFormat"] = int64(QTextFormat__TableFormat)
	qt.EnumMap["gui.QTextFormat__FrameFormat"] = int64(QTextFormat__FrameFormat)
	qt.EnumMap["gui.QTextFormat__UserFormat"] = int64(QTextFormat__UserFormat)
	qt.EnumMap["gui.QTextFormat__ObjectIndex"] = int64(QTextFormat__ObjectIndex)
	qt.EnumMap["gui.QTextFormat__CssFloat"] = int64(QTextFormat__CssFloat)
	qt.EnumMap["gui.QTextFormat__LayoutDirection"] = int64(QTextFormat__LayoutDirection)
	qt.EnumMap["gui.QTextFormat__OutlinePen"] = int64(QTextFormat__OutlinePen)
	qt.EnumMap["gui.QTextFormat__BackgroundBrush"] = int64(QTextFormat__BackgroundBrush)
	qt.EnumMap["gui.QTextFormat__ForegroundBrush"] = int64(QTextFormat__ForegroundBrush)
	qt.EnumMap["gui.QTextFormat__BackgroundImageUrl"] = int64(QTextFormat__BackgroundImageUrl)
	qt.EnumMap["gui.QTextFormat__BlockAlignment"] = int64(QTextFormat__BlockAlignment)
	qt.EnumMap["gui.QTextFormat__BlockTopMargin"] = int64(QTextFormat__BlockTopMargin)
	qt.EnumMap["gui.QTextFormat__BlockBottomMargin"] = int64(QTextFormat__BlockBottomMargin)
	qt.EnumMap["gui.QTextFormat__BlockLeftMargin"] = int64(QTextFormat__BlockLeftMargin)
	qt.EnumMap["gui.QTextFormat__BlockRightMargin"] = int64(QTextFormat__BlockRightMargin)
	qt.EnumMap["gui.QTextFormat__TextIndent"] = int64(QTextFormat__TextIndent)
	qt.EnumMap["gui.QTextFormat__TabPositions"] = int64(QTextFormat__TabPositions)
	qt.EnumMap["gui.QTextFormat__BlockIndent"] = int64(QTextFormat__BlockIndent)
	qt.EnumMap["gui.QTextFormat__LineHeight"] = int64(QTextFormat__LineHeight)
	qt.EnumMap["gui.QTextFormat__LineHeightType"] = int64(QTextFormat__LineHeightType)
	qt.EnumMap["gui.QTextFormat__BlockNonBreakableLines"] = int64(QTextFormat__BlockNonBreakableLines)
	qt.EnumMap["gui.QTextFormat__BlockTrailingHorizontalRulerWidth"] = int64(QTextFormat__BlockTrailingHorizontalRulerWidth)
	qt.EnumMap["gui.QTextFormat__HeadingLevel"] = int64(QTextFormat__HeadingLevel)
	qt.EnumMap["gui.QTextFormat__FirstFontProperty"] = int64(QTextFormat__FirstFontProperty)
	qt.EnumMap["gui.QTextFormat__FontCapitalization"] = int64(QTextFormat__FontCapitalization)
	qt.EnumMap["gui.QTextFormat__FontLetterSpacingType"] = int64(QTextFormat__FontLetterSpacingType)
	qt.EnumMap["gui.QTextFormat__FontLetterSpacing"] = int64(QTextFormat__FontLetterSpacing)
	qt.EnumMap["gui.QTextFormat__FontWordSpacing"] = int64(QTextFormat__FontWordSpacing)
	qt.EnumMap["gui.QTextFormat__FontStretch"] = int64(QTextFormat__FontStretch)
	qt.EnumMap["gui.QTextFormat__FontStyleHint"] = int64(QTextFormat__FontStyleHint)
	qt.EnumMap["gui.QTextFormat__FontStyleStrategy"] = int64(QTextFormat__FontStyleStrategy)
	qt.EnumMap["gui.QTextFormat__FontKerning"] = int64(QTextFormat__FontKerning)
	qt.EnumMap["gui.QTextFormat__FontHintingPreference"] = int64(QTextFormat__FontHintingPreference)
	qt.EnumMap["gui.QTextFormat__FontFamilies"] = int64(QTextFormat__FontFamilies)
	qt.EnumMap["gui.QTextFormat__FontStyleName"] = int64(QTextFormat__FontStyleName)
	qt.EnumMap["gui.QTextFormat__FontFamily"] = int64(QTextFormat__FontFamily)
	qt.EnumMap["gui.QTextFormat__FontPointSize"] = int64(QTextFormat__FontPointSize)
	qt.EnumMap["gui.QTextFormat__FontSizeAdjustment"] = int64(QTextFormat__FontSizeAdjustment)
	qt.EnumMap["gui.QTextFormat__FontSizeIncrement"] = int64(QTextFormat__FontSizeIncrement)
	qt.EnumMap["gui.QTextFormat__FontWeight"] = int64(QTextFormat__FontWeight)
	qt.EnumMap["gui.QTextFormat__FontItalic"] = int64(QTextFormat__FontItalic)
	qt.EnumMap["gui.QTextFormat__FontUnderline"] = int64(QTextFormat__FontUnderline)
	qt.EnumMap["gui.QTextFormat__FontOverline"] = int64(QTextFormat__FontOverline)
	qt.EnumMap["gui.QTextFormat__FontStrikeOut"] = int64(QTextFormat__FontStrikeOut)
	qt.EnumMap["gui.QTextFormat__FontFixedPitch"] = int64(QTextFormat__FontFixedPitch)
	qt.EnumMap["gui.QTextFormat__FontPixelSize"] = int64(QTextFormat__FontPixelSize)
	qt.EnumMap["gui.QTextFormat__LastFontProperty"] = int64(QTextFormat__LastFontProperty)
	qt.EnumMap["gui.QTextFormat__TextUnderlineColor"] = int64(QTextFormat__TextUnderlineColor)
	qt.EnumMap["gui.QTextFormat__TextVerticalAlignment"] = int64(QTextFormat__TextVerticalAlignment)
	qt.EnumMap["gui.QTextFormat__TextOutline"] = int64(QTextFormat__TextOutline)
	qt.EnumMap["gui.QTextFormat__TextUnderlineStyle"] = int64(QTextFormat__TextUnderlineStyle)
	qt.EnumMap["gui.QTextFormat__TextToolTip"] = int64(QTextFormat__TextToolTip)
	qt.EnumMap["gui.QTextFormat__IsAnchor"] = int64(QTextFormat__IsAnchor)
	qt.EnumMap["gui.QTextFormat__AnchorHref"] = int64(QTextFormat__AnchorHref)
	qt.EnumMap["gui.QTextFormat__AnchorName"] = int64(QTextFormat__AnchorName)
	qt.EnumMap["gui.QTextFormat__ObjectType"] = int64(QTextFormat__ObjectType)
	qt.EnumMap["gui.QTextFormat__ListStyle"] = int64(QTextFormat__ListStyle)
	qt.EnumMap["gui.QTextFormat__ListIndent"] = int64(QTextFormat__ListIndent)
	qt.EnumMap["gui.QTextFormat__ListNumberPrefix"] = int64(QTextFormat__ListNumberPrefix)
	qt.EnumMap["gui.QTextFormat__ListNumberSuffix"] = int64(QTextFormat__ListNumberSuffix)
	qt.EnumMap["gui.QTextFormat__FrameBorder"] = int64(QTextFormat__FrameBorder)
	qt.EnumMap["gui.QTextFormat__FrameMargin"] = int64(QTextFormat__FrameMargin)
	qt.EnumMap["gui.QTextFormat__FramePadding"] = int64(QTextFormat__FramePadding)
	qt.EnumMap["gui.QTextFormat__FrameWidth"] = int64(QTextFormat__FrameWidth)
	qt.EnumMap["gui.QTextFormat__FrameHeight"] = int64(QTextFormat__FrameHeight)
	qt.EnumMap["gui.QTextFormat__FrameTopMargin"] = int64(QTextFormat__FrameTopMargin)
	qt.EnumMap["gui.QTextFormat__FrameBottomMargin"] = int64(QTextFormat__FrameBottomMargin)
	qt.EnumMap["gui.QTextFormat__FrameLeftMargin"] = int64(QTextFormat__FrameLeftMargin)
	qt.EnumMap["gui.QTextFormat__FrameRightMargin"] = int64(QTextFormat__FrameRightMargin)
	qt.EnumMap["gui.QTextFormat__FrameBorderBrush"] = int64(QTextFormat__FrameBorderBrush)
	qt.EnumMap["gui.QTextFormat__FrameBorderStyle"] = int64(QTextFormat__FrameBorderStyle)
	qt.EnumMap["gui.QTextFormat__TableColumns"] = int64(QTextFormat__TableColumns)
	qt.EnumMap["gui.QTextFormat__TableColumnWidthConstraints"] = int64(QTextFormat__TableColumnWidthConstraints)
	qt.EnumMap["gui.QTextFormat__TableCellSpacing"] = int64(QTextFormat__TableCellSpacing)
	qt.EnumMap["gui.QTextFormat__TableCellPadding"] = int64(QTextFormat__TableCellPadding)
	qt.EnumMap["gui.QTextFormat__TableHeaderRowCount"] = int64(QTextFormat__TableHeaderRowCount)
	qt.EnumMap["gui.QTextFormat__TableCellRowSpan"] = int64(QTextFormat__TableCellRowSpan)
	qt.EnumMap["gui.QTextFormat__TableCellColumnSpan"] = int64(QTextFormat__TableCellColumnSpan)
	qt.EnumMap["gui.QTextFormat__TableCellTopPadding"] = int64(QTextFormat__TableCellTopPadding)
	qt.EnumMap["gui.QTextFormat__TableCellBottomPadding"] = int64(QTextFormat__TableCellBottomPadding)
	qt.EnumMap["gui.QTextFormat__TableCellLeftPadding"] = int64(QTextFormat__TableCellLeftPadding)
	qt.EnumMap["gui.QTextFormat__TableCellRightPadding"] = int64(QTextFormat__TableCellRightPadding)
	qt.EnumMap["gui.QTextFormat__ImageName"] = int64(QTextFormat__ImageName)
	qt.EnumMap["gui.QTextFormat__ImageWidth"] = int64(QTextFormat__ImageWidth)
	qt.EnumMap["gui.QTextFormat__ImageHeight"] = int64(QTextFormat__ImageHeight)
	qt.EnumMap["gui.QTextFormat__ImageQuality"] = int64(QTextFormat__ImageQuality)
	qt.EnumMap["gui.QTextFormat__FullWidthSelection"] = int64(QTextFormat__FullWidthSelection)
	qt.EnumMap["gui.QTextFormat__PageBreakPolicy"] = int64(QTextFormat__PageBreakPolicy)
	qt.EnumMap["gui.QTextFormat__UserProperty"] = int64(QTextFormat__UserProperty)
	qt.EnumMap["gui.QTextFormat__NoObject"] = int64(QTextFormat__NoObject)
	qt.EnumMap["gui.QTextFormat__ImageObject"] = int64(QTextFormat__ImageObject)
	qt.EnumMap["gui.QTextFormat__TableObject"] = int64(QTextFormat__TableObject)
	qt.EnumMap["gui.QTextFormat__TableCellObject"] = int64(QTextFormat__TableCellObject)
	qt.EnumMap["gui.QTextFormat__UserObject"] = int64(QTextFormat__UserObject)
	qt.EnumMap["gui.QTextFormat__PageBreak_Auto"] = int64(QTextFormat__PageBreak_Auto)
	qt.EnumMap["gui.QTextFormat__PageBreak_AlwaysBefore"] = int64(QTextFormat__PageBreak_AlwaysBefore)
	qt.EnumMap["gui.QTextFormat__PageBreak_AlwaysAfter"] = int64(QTextFormat__PageBreak_AlwaysAfter)
	qt.ItfMap["gui.QTextFragment_ITF"] = QTextFragment{}
	qt.FuncMap["gui.NewQTextFragment2"] = NewQTextFragment2
	qt.FuncMap["gui.NewQTextFragment3"] = NewQTextFragment3
	qt.ItfMap["gui.QTextFrame_ITF"] = QTextFrame{}
	qt.FuncMap["gui.NewQTextFrame"] = NewQTextFrame
	qt.ItfMap["gui.QTextFrameFormat_ITF"] = QTextFrameFormat{}
	qt.FuncMap["gui.NewQTextFrameFormat"] = NewQTextFrameFormat
	qt.EnumMap["gui.QTextFrameFormat__InFlow"] = int64(QTextFrameFormat__InFlow)
	qt.EnumMap["gui.QTextFrameFormat__FloatLeft"] = int64(QTextFrameFormat__FloatLeft)
	qt.EnumMap["gui.QTextFrameFormat__FloatRight"] = int64(QTextFrameFormat__FloatRight)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_None"] = int64(QTextFrameFormat__BorderStyle_None)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_Dotted"] = int64(QTextFrameFormat__BorderStyle_Dotted)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_Dashed"] = int64(QTextFrameFormat__BorderStyle_Dashed)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_Solid"] = int64(QTextFrameFormat__BorderStyle_Solid)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_Double"] = int64(QTextFrameFormat__BorderStyle_Double)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_DotDash"] = int64(QTextFrameFormat__BorderStyle_DotDash)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_DotDotDash"] = int64(QTextFrameFormat__BorderStyle_DotDotDash)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_Groove"] = int64(QTextFrameFormat__BorderStyle_Groove)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_Ridge"] = int64(QTextFrameFormat__BorderStyle_Ridge)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_Inset"] = int64(QTextFrameFormat__BorderStyle_Inset)
	qt.EnumMap["gui.QTextFrameFormat__BorderStyle_Outset"] = int64(QTextFrameFormat__BorderStyle_Outset)
	qt.ItfMap["gui.QTextFrameLayoutData_ITF"] = QTextFrameLayoutData{}
	qt.ItfMap["gui.QTextImageFormat_ITF"] = QTextImageFormat{}
	qt.FuncMap["gui.NewQTextImageFormat"] = NewQTextImageFormat
	qt.ItfMap["gui.QTextInlineObject_ITF"] = QTextInlineObject{}
	qt.ItfMap["gui.QTextItem_ITF"] = QTextItem{}
	qt.EnumMap["gui.QTextItem__RightToLeft"] = int64(QTextItem__RightToLeft)
	qt.EnumMap["gui.QTextItem__Overline"] = int64(QTextItem__Overline)
	qt.EnumMap["gui.QTextItem__Underline"] = int64(QTextItem__Underline)
	qt.EnumMap["gui.QTextItem__StrikeOut"] = int64(QTextItem__StrikeOut)
	qt.EnumMap["gui.QTextItem__Dummy"] = int64(QTextItem__Dummy)
	qt.ItfMap["gui.QTextLayout_ITF"] = QTextLayout{}
	qt.FuncMap["gui.NewQTextLayout"] = NewQTextLayout
	qt.FuncMap["gui.NewQTextLayout2"] = NewQTextLayout2
	qt.FuncMap["gui.NewQTextLayout4"] = NewQTextLayout4
	qt.EnumMap["gui.QTextLayout__SkipCharacters"] = int64(QTextLayout__SkipCharacters)
	qt.EnumMap["gui.QTextLayout__SkipWords"] = int64(QTextLayout__SkipWords)
	qt.ItfMap["gui.QTextLength_ITF"] = QTextLength{}
	qt.FuncMap["gui.NewQTextLength"] = NewQTextLength
	qt.FuncMap["gui.NewQTextLength2"] = NewQTextLength2
	qt.EnumMap["gui.QTextLength__VariableLength"] = int64(QTextLength__VariableLength)
	qt.EnumMap["gui.QTextLength__FixedLength"] = int64(QTextLength__FixedLength)
	qt.EnumMap["gui.QTextLength__PercentageLength"] = int64(QTextLength__PercentageLength)
	qt.ItfMap["gui.QTextLine_ITF"] = QTextLine{}
	qt.FuncMap["gui.NewQTextLine"] = NewQTextLine
	qt.EnumMap["gui.QTextLine__Leading"] = int64(QTextLine__Leading)
	qt.EnumMap["gui.QTextLine__Trailing"] = int64(QTextLine__Trailing)
	qt.EnumMap["gui.QTextLine__CursorBetweenCharacters"] = int64(QTextLine__CursorBetweenCharacters)
	qt.EnumMap["gui.QTextLine__CursorOnCharacter"] = int64(QTextLine__CursorOnCharacter)
	qt.ItfMap["gui.QTextList_ITF"] = QTextList{}
	qt.ItfMap["gui.QTextListFormat_ITF"] = QTextListFormat{}
	qt.FuncMap["gui.NewQTextListFormat"] = NewQTextListFormat
	qt.EnumMap["gui.QTextListFormat__ListDisc"] = int64(QTextListFormat__ListDisc)
	qt.EnumMap["gui.QTextListFormat__ListCircle"] = int64(QTextListFormat__ListCircle)
	qt.EnumMap["gui.QTextListFormat__ListSquare"] = int64(QTextListFormat__ListSquare)
	qt.EnumMap["gui.QTextListFormat__ListDecimal"] = int64(QTextListFormat__ListDecimal)
	qt.EnumMap["gui.QTextListFormat__ListLowerAlpha"] = int64(QTextListFormat__ListLowerAlpha)
	qt.EnumMap["gui.QTextListFormat__ListUpperAlpha"] = int64(QTextListFormat__ListUpperAlpha)
	qt.EnumMap["gui.QTextListFormat__ListLowerRoman"] = int64(QTextListFormat__ListLowerRoman)
	qt.EnumMap["gui.QTextListFormat__ListUpperRoman"] = int64(QTextListFormat__ListUpperRoman)
	qt.EnumMap["gui.QTextListFormat__ListStyleUndefined"] = int64(QTextListFormat__ListStyleUndefined)
	qt.ItfMap["gui.QTextObject_ITF"] = QTextObject{}
	qt.FuncMap["gui.NewQTextObject"] = NewQTextObject
	qt.ItfMap["gui.QTextObjectInterface_ITF"] = QTextObjectInterface{}
	qt.ItfMap["gui.QTextOption_ITF"] = QTextOption{}
	qt.FuncMap["gui.NewQTextOption"] = NewQTextOption
	qt.FuncMap["gui.NewQTextOption2"] = NewQTextOption2
	qt.FuncMap["gui.NewQTextOption3"] = NewQTextOption3
	qt.EnumMap["gui.QTextOption__LeftTab"] = int64(QTextOption__LeftTab)
	qt.EnumMap["gui.QTextOption__RightTab"] = int64(QTextOption__RightTab)
	qt.EnumMap["gui.QTextOption__CenterTab"] = int64(QTextOption__CenterTab)
	qt.EnumMap["gui.QTextOption__DelimiterTab"] = int64(QTextOption__DelimiterTab)
	qt.EnumMap["gui.QTextOption__NoWrap"] = int64(QTextOption__NoWrap)
	qt.EnumMap["gui.QTextOption__WordWrap"] = int64(QTextOption__WordWrap)
	qt.EnumMap["gui.QTextOption__ManualWrap"] = int64(QTextOption__ManualWrap)
	qt.EnumMap["gui.QTextOption__WrapAnywhere"] = int64(QTextOption__WrapAnywhere)
	qt.EnumMap["gui.QTextOption__WrapAtWordBoundaryOrAnywhere"] = int64(QTextOption__WrapAtWordBoundaryOrAnywhere)
	qt.EnumMap["gui.QTextOption__ShowTabsAndSpaces"] = int64(QTextOption__ShowTabsAndSpaces)
	qt.EnumMap["gui.QTextOption__ShowLineAndParagraphSeparators"] = int64(QTextOption__ShowLineAndParagraphSeparators)
	qt.EnumMap["gui.QTextOption__AddSpaceForLineAndParagraphSeparators"] = int64(QTextOption__AddSpaceForLineAndParagraphSeparators)
	qt.EnumMap["gui.QTextOption__SuppressColors"] = int64(QTextOption__SuppressColors)
	qt.EnumMap["gui.QTextOption__ShowDocumentTerminator"] = int64(QTextOption__ShowDocumentTerminator)
	qt.EnumMap["gui.QTextOption__IncludeTrailingSpaces"] = int64(QTextOption__IncludeTrailingSpaces)
	qt.ItfMap["gui.QTextTable_ITF"] = QTextTable{}
	qt.ItfMap["gui.QTextTableCell_ITF"] = QTextTableCell{}
	qt.FuncMap["gui.NewQTextTableCell"] = NewQTextTableCell
	qt.FuncMap["gui.NewQTextTableCell2"] = NewQTextTableCell2
	qt.ItfMap["gui.QTextTableCellFormat_ITF"] = QTextTableCellFormat{}
	qt.FuncMap["gui.NewQTextTableCellFormat"] = NewQTextTableCellFormat
	qt.ItfMap["gui.QTextTableFormat_ITF"] = QTextTableFormat{}
	qt.FuncMap["gui.NewQTextTableFormat"] = NewQTextTableFormat
	qt.ItfMap["gui.QTouchDevice_ITF"] = QTouchDevice{}
	qt.FuncMap["gui.NewQTouchDevice"] = NewQTouchDevice
	qt.FuncMap["gui.QTouchDevice_Devices"] = QTouchDevice_Devices
	qt.EnumMap["gui.QTouchDevice__TouchScreen"] = int64(QTouchDevice__TouchScreen)
	qt.EnumMap["gui.QTouchDevice__TouchPad"] = int64(QTouchDevice__TouchPad)
	qt.EnumMap["gui.QTouchDevice__Position"] = int64(QTouchDevice__Position)
	qt.EnumMap["gui.QTouchDevice__Area"] = int64(QTouchDevice__Area)
	qt.EnumMap["gui.QTouchDevice__Pressure"] = int64(QTouchDevice__Pressure)
	qt.EnumMap["gui.QTouchDevice__Velocity"] = int64(QTouchDevice__Velocity)
	qt.EnumMap["gui.QTouchDevice__RawPositions"] = int64(QTouchDevice__RawPositions)
	qt.EnumMap["gui.QTouchDevice__NormalizedPosition"] = int64(QTouchDevice__NormalizedPosition)
	qt.EnumMap["gui.QTouchDevice__MouseEmulation"] = int64(QTouchDevice__MouseEmulation)
	qt.ItfMap["gui.QTouchEvent_ITF"] = QTouchEvent{}
	qt.ItfMap["gui.QTransform_ITF"] = QTransform{}
	qt.FuncMap["gui.NewQTransform2"] = NewQTransform2
	qt.FuncMap["gui.NewQTransform3"] = NewQTransform3
	qt.FuncMap["gui.NewQTransform4"] = NewQTransform4
	qt.FuncMap["gui.NewQTransform5"] = NewQTransform5
	qt.FuncMap["gui.QTransform_FromScale"] = QTransform_FromScale
	qt.FuncMap["gui.QTransform_FromTranslate"] = QTransform_FromTranslate
	qt.FuncMap["gui.QTransform_QuadToQuad"] = QTransform_QuadToQuad
	qt.FuncMap["gui.QTransform_QuadToSquare"] = QTransform_QuadToSquare
	qt.FuncMap["gui.QTransform_SquareToQuad"] = QTransform_SquareToQuad
	qt.EnumMap["gui.QTransform__TxNone"] = int64(QTransform__TxNone)
	qt.EnumMap["gui.QTransform__TxTranslate"] = int64(QTransform__TxTranslate)
	qt.EnumMap["gui.QTransform__TxScale"] = int64(QTransform__TxScale)
	qt.EnumMap["gui.QTransform__TxRotate"] = int64(QTransform__TxRotate)
	qt.EnumMap["gui.QTransform__TxShear"] = int64(QTransform__TxShear)
	qt.EnumMap["gui.QTransform__TxProject"] = int64(QTransform__TxProject)
	qt.ItfMap["gui.QValidator_ITF"] = QValidator{}
	qt.FuncMap["gui.NewQValidator"] = NewQValidator
	qt.EnumMap["gui.QValidator__Invalid"] = int64(QValidator__Invalid)
	qt.EnumMap["gui.QValidator__Intermediate"] = int64(QValidator__Intermediate)
	qt.EnumMap["gui.QValidator__Acceptable"] = int64(QValidator__Acceptable)
	qt.ItfMap["gui.QVector2D_ITF"] = QVector2D{}
	qt.FuncMap["gui.NewQVector2D"] = NewQVector2D
	qt.FuncMap["gui.NewQVector2D3"] = NewQVector2D3
	qt.FuncMap["gui.NewQVector2D4"] = NewQVector2D4
	qt.FuncMap["gui.NewQVector2D5"] = NewQVector2D5
	qt.FuncMap["gui.NewQVector2D6"] = NewQVector2D6
	qt.FuncMap["gui.NewQVector2D7"] = NewQVector2D7
	qt.FuncMap["gui.QVector2D_DotProduct"] = QVector2D_DotProduct
	qt.ItfMap["gui.QVector3D_ITF"] = QVector3D{}
	qt.FuncMap["gui.NewQVector3D"] = NewQVector3D
	qt.FuncMap["gui.NewQVector3D3"] = NewQVector3D3
	qt.FuncMap["gui.NewQVector3D4"] = NewQVector3D4
	qt.FuncMap["gui.NewQVector3D5"] = NewQVector3D5
	qt.FuncMap["gui.NewQVector3D6"] = NewQVector3D6
	qt.FuncMap["gui.NewQVector3D7"] = NewQVector3D7
	qt.FuncMap["gui.NewQVector3D8"] = NewQVector3D8
	qt.FuncMap["gui.QVector3D_CrossProduct"] = QVector3D_CrossProduct
	qt.FuncMap["gui.QVector3D_DotProduct"] = QVector3D_DotProduct
	qt.FuncMap["gui.QVector3D_Normal"] = QVector3D_Normal
	qt.FuncMap["gui.QVector3D_Normal2"] = QVector3D_Normal2
	qt.ItfMap["gui.QVector4D_ITF"] = QVector4D{}
	qt.FuncMap["gui.NewQVector4D"] = NewQVector4D
	qt.FuncMap["gui.NewQVector4D3"] = NewQVector4D3
	qt.FuncMap["gui.NewQVector4D4"] = NewQVector4D4
	qt.FuncMap["gui.NewQVector4D5"] = NewQVector4D5
	qt.FuncMap["gui.NewQVector4D6"] = NewQVector4D6
	qt.FuncMap["gui.NewQVector4D7"] = NewQVector4D7
	qt.FuncMap["gui.NewQVector4D8"] = NewQVector4D8
	qt.FuncMap["gui.NewQVector4D9"] = NewQVector4D9
	qt.FuncMap["gui.QVector4D_DotProduct"] = QVector4D_DotProduct
	qt.ItfMap["gui.QVulkanDeviceFunctions_ITF"] = QVulkanDeviceFunctions{}
	qt.ItfMap["gui.QVulkanExtension_ITF"] = QVulkanExtension{}
	qt.ItfMap["gui.QVulkanFunctions_ITF"] = QVulkanFunctions{}
	qt.ItfMap["gui.QVulkanInfoVector_ITF"] = QVulkanInfoVector{}
	qt.ItfMap["gui.QVulkanInstance_ITF"] = QVulkanInstance{}
	qt.EnumMap["gui.QVulkanInstance__NoDebugOutputRedirect"] = int64(QVulkanInstance__NoDebugOutputRedirect)
	qt.ItfMap["gui.QVulkanLayer_ITF"] = QVulkanLayer{}
	qt.ItfMap["gui.QVulkanWindow_ITF"] = QVulkanWindow{}
	qt.EnumMap["gui.QVulkanWindow__PersistentResources"] = int64(QVulkanWindow__PersistentResources)
	qt.ItfMap["gui.QVulkanWindowRenderer_ITF"] = QVulkanWindowRenderer{}
	qt.ItfMap["gui.QWhatsThisClickedEvent_ITF"] = QWhatsThisClickedEvent{}
	qt.FuncMap["gui.NewQWhatsThisClickedEvent"] = NewQWhatsThisClickedEvent
	qt.ItfMap["gui.QWheelEvent_ITF"] = QWheelEvent{}
	qt.FuncMap["gui.NewQWheelEvent3"] = NewQWheelEvent3
	qt.FuncMap["gui.NewQWheelEvent4"] = NewQWheelEvent4
	qt.FuncMap["gui.NewQWheelEvent5"] = NewQWheelEvent5
	qt.FuncMap["gui.NewQWheelEvent6"] = NewQWheelEvent6
	qt.FuncMap["gui.NewQWheelEvent7"] = NewQWheelEvent7
	qt.ItfMap["gui.QWindow_ITF"] = QWindow{}
	qt.FuncMap["gui.NewQWindow"] = NewQWindow
	qt.FuncMap["gui.NewQWindow2"] = NewQWindow2
	qt.FuncMap["gui.QWindow_FromWinId"] = QWindow_FromWinId
	qt.EnumMap["gui.QWindow__Hidden"] = int64(QWindow__Hidden)
	qt.EnumMap["gui.QWindow__AutomaticVisibility"] = int64(QWindow__AutomaticVisibility)
	qt.EnumMap["gui.QWindow__Windowed"] = int64(QWindow__Windowed)
	qt.EnumMap["gui.QWindow__Minimized"] = int64(QWindow__Minimized)
	qt.EnumMap["gui.QWindow__Maximized"] = int64(QWindow__Maximized)
	qt.EnumMap["gui.QWindow__FullScreen"] = int64(QWindow__FullScreen)
	qt.EnumMap["gui.QWindow__ExcludeTransients"] = int64(QWindow__ExcludeTransients)
	qt.EnumMap["gui.QWindow__IncludeTransients"] = int64(QWindow__IncludeTransients)
	qt.ItfMap["gui.QWindowStateChangeEvent_ITF"] = QWindowStateChangeEvent{}
}
